﻿define("epi-find/widget/CmsContentSelector", [
    "dojo/_base/lang",
    "dojo/_base/declare",
    "dojo/when",
    "dojo/dom-class",

    "dijit/layout/ContentPane",

    "epi/shell/widget/dialog/Dialog",

    "epi-cms/widget/SearchBox",
    "epi-cms/widget/ContentSelector",
    "epi-cms/widget/ContentSelectorDialog",
    "./ContentStoreQueryLanguageHandler",

    "./_ContentStatusIndicatorMixin",

    "dojo/NodeList-manipulate" // workaround for the ContentSelectors query usage, bug #111507

],
function(
         lang,
         declare,
         when,
         domClass,
         ContentPane,
         Dialog,
         SearchBox,
         ContentSelector,
         ContentSelectorDialog,
         ContentStoreQueryLanguageHandler,
         _ContentStatusIndicatorMixin
    ) {


    return declare([ContentSelector, _ContentStatusIndicatorMixin], {
        model: null,
        title: "",
        targetContent: null,
        contentStatus: null,
        searchArea: null,
        selectButtonText: null,
        selectedLang: null,

        _setTargetContentAttr: function (value) {
            this.set("value", value);
            this._set("targetContent", value);

            if (!value && this.disabled) {
                this.set("state", ""); // reset invalid state
            } else {
                var content = this._getContentData(value);
                if (content) {
                    this._set("contentStatus", content.status);
                } else {
                    this._set("contentStatus", "");
                }
            }
        },

        rootPage: null,
        _setRootPageAttr: function (value) {
            this.rootPage = value;
            this.roots = [this.rootPage];
        },

        _setDisabledAttr: function (value) {
            this.inherited(arguments);
            domClass.toggle(this.displayNode, "dijitTextBoxDisabled", value); // grayout text node
            domClass.toggle(this.clearButton, "epi-hidden", value); // hide clear button
            this.set("state", ""); // reset invalid state

        },

        constructor: function() {
            this.showButtons = true;
            this.allowedTypes = [];
        },

        postMixInProperties: function() {
            try {
                this.inherited(arguments);
            }
            catch(e) { // if the base widget attempt to resolve roots results in exception
                this.roots = this.roots || [];
            }
        },

        startup: function() {
            this.inherited(arguments);
            this._started = true; // workaround for the bug #111543
        },

        onChange: function(value) {
            this.set("targetContent", value);
        },

        _getDialog: function () { // have to override this method in order to provide to the contentSelectorDialog
            // summary:
            //		Create page tree dialog
            // tags:
            //    protected

            // Verifies that the dialog instance and its dom node exists or not
            if (this.dialog && this.dialog.domNode) {
                return this.dialog;
            }
            var self = this;

            var model = new ContentStoreQueryLanguageHandler({roots: this.roots, lang: this.selectedLang});

            this.contentContainer = new ContentPane({
                "class": "epi-no-overflow-x"
            });

            this.searchBox = new SearchBox({
                baseClass: "epi-search-box",
                onItemAction: function(item) {
                    if (item && item.metadata) {
                        self.contentSelectorDialog.set("value", item.metadata.id);
                    }
                }
            });
            this.searchBox.set("area", this.searchArea);

            this.contentSelectorDialog = new ContentSelectorDialog({
                model: model,
                canSelectOwnerContent: this.canSelectOwnerContent,
                showButtons: false,
                roots: this.roots,
                allowedTypes: this.allowedTypes,
                showAllLanguages: this.showAllLanguages,
                layout: function() {} // prevent layoting
            });

            this.contentContainer.addChild(this.searchBox);
            this.contentContainer.addChild(this.contentSelectorDialog);

            this.dialog = new Dialog({
                title: this.title,
                dialogClass: "epi-dialog-portrait",
                content: this.contentContainer,
                destroyOnHide: true
            });
            this.dialog.confirmActionText = this.selectButtonText || this.dialog.confirmActionText;

            this.dialog.own(this.contentContainer);
            this.dialog.own(this.searchBox);
            this.dialog.own(this.contentSelectorDialog);
            this.dialog.own(model);

            this.connect(this.contentSelectorDialog, "onChange", "_setDialogButtonState");
            this.connect(this.dialog, "onExecute", "_onDialogExecute");
            this.connect(this.dialog, "onHide", "_onDialogHide");
            return this.dialog;
        },

        _setDialogButtonState: function(contentLink) {
            this.inherited(arguments);

            var self = this;
            when(this._store.get(contentLink), function (content) {
                self.dialog.definitionConsumer.setItemProperty(self.dialog._okButtonName, "disabled", !content.isVisibleOnSite);
            });
        },

      _getContentData: function (contentLink) {
            //summary:
            //    Loads the content data from the store
            // tags:
            //    private

            if (!contentLink) {
                return null;
            }
            var queryString = "?showAllLanguages=" + this.showAllLanguages;

            if(this.selectedLang)
            {
              queryString += "&lang=" + this.selectedLang;
            }
            return this._store.get(contentLink + queryString);
        },


        reset: function() {
            this.inherited(arguments);
            this.set("state",""); // reset validation state
        }
    });
});
