define("epi-find/widget/ConnectorGrid", [
        "dojo/_base/declare",
        "dojo/_base/lang",
        "dojo/dom-construct",
        "dojo/dom-class",
        "dojo/when",
        "dojo/Deferred",
        "dojo/on",
        "dojo/date/locale",

        "dojox/html/entities",

        "dijit/form/Button",
        "dijit/MenuItem",

        "dgrid/Grid",
        "dgrid/extensions/DijitRegistry",

        "./Schedule",

        "epi/shell/dgrid/Formatter",
        "epi/datetime",
        "epi-cms/dgrid/formatters",

        "epi-saas-base/widgets/_NotificationMixin",
        "epi-saas-base/widgets/_GridContextMenuMixin",
        "epi-saas-base/widgets/_GridMoreMixin",
        "epi-saas-base/widgets/_AutoLoadScrollMixin",

        "dojo/i18n!./nls/Connectors"
    ],
    function (declare,
            lang,
            domConstruct,
            domClass,
            when,
            Deferred,
            on,
            locale,

            entities,

            Button,
            MenuItem,

            Grid,
            DijitRegistry,

            Schedule,

            Formatter,
            datetime,
            formatters,

            _NotificationMixin,
            _GridContextMenuMixin,
            _GridMoreMixin,
            _AutoLoadScrollMixin,

            i18n
        ) {

        // module:
        //      epi-find/widget/ConnectorGrid

        return declare([Grid, _GridMoreMixin, _AutoLoadScrollMixin, Formatter, _GridContextMenuMixin, DijitRegistry, _NotificationMixin], {

            noDataMessage: i18n.noConnectors,
            className: "dgrid-autoheight",

            // model: epi-find/configure/ConnectorsModel
            //      Model for the connectors view
            model: null,

            selectionMode: "single",

            i18n: i18n,

            // editMenuItem: dijit/MenuItem
            //      Context menu item for Edit action
            editMenuItem: null,

            // deleteMenuItem: dijit/MenuItem
            //      Context menu item for Delete action
            deleteMenuItem: null,

            constructor: function(options) {
                declare.safeMixin(this, options);

                // Sets the loader (#loadNextPage, available in _GridMoreMixn) to be used by the _AutoLoadScrollMixin
                this.loader = lang.hitch(this, this.loadNextPage);

                var self = this;
                this.store = this.model.store;
                this.columns = {
                    name: {
                        label: i18n.fields_name,
                        sortable: false
                    },
                    type: {
                        label: i18n.fields_type,
                        sortable: false,
                        formatter: function(value) {
                            return i18n["types_"+value] || value;
                        }
                    },
                    schedule: {
                        label: i18n.fields_schedule,
                        sortable: false,
                        formatter: function(value) {
                            return self.model.scheduleModel.getTextDescription(value);
                        }
                    },
                    status: {
                        sortable: false,
                        renderHeaderCell: function(node) {
                            var grid = this.grid,
                                refreshAnchor = new Button({
                                    "class": "epi-chromeless epi-visibleLink",
                                    iconClass: "epi-iconReload",
                                    label: i18n.fields_status
                                });
                            domConstruct.place(refreshAnchor.domNode, node);
                            self.own(refreshAnchor,
                                on(refreshAnchor, "click", function(e) {
                                    e.preventDefault();
                                    e.stopPropagation();
                                    grid.refresh();
                                }));
                        },
                        renderCell: function(object, value, node, options) {
                            if (object) {
                                when(self.model.getJob(object), function(job) {
                                    var status = job ? job.mode : "";
                                    if (status) {
                                        var statusDetails = {
                                            stopTime: locale.format(new Date(job.stop_time), {formatLength: "short"})
                                        };
                                        var statusNode = domConstruct.create("span", {innerHTML:  lang.replace(i18n["statuses_" + status] || status, statusDetails)});
                                        if (status === "stopped") {
                                            domClass.add(statusNode, "epi-lozenge epi-background--success");
                                        }
                                        domConstruct.place(statusNode, node);

                                    }
                                });
                            }
                        }
                    },
                    date: {
                        sortable: false,
                        label: i18n.fields_date,
                        renderCell: function(object, value, node, options) {
                            if (object) {
                                when(self.model.getJob(object), function(job) {
                                    var status = job ? job.mode : "";
                                    if (status === "stopped") {
                                        node.innerHTML = datetime.toUserFriendlyString(new Date(job.stop_time));
                                    }
                                });
                            }
                        }
                    },

                    documents: {
                        sortable: false,
                        label: i18n.fields_documents,
                        className: "epi-grid-column--centered",
                        renderCell: function(object, value, node, options) {
                            if (object) {
                                when(self.model.getJob(object), function(job) {
                                    if (job && job.details ) {
                                        node.innerHTML = job.details.indexed_document_count || 0;
                                    }
                                });
                            }
                        }

                    },

                    actions: {
                        sortable: false,
                        label: "",

                        renderCell: function(object, value, node, options) {
                            var grid = this.grid, clickAction = null, anchor = new Button({
                                    "class": "epi-chromeless epi-visibleLink",
                                    disabled: self.model.isActionDisabled(object)
                                });
                            domConstruct.place(anchor.domNode, node);
                            self.own( // own is available because epi-cms/dgrid/WithContextMenu is based on Destroyable
                                anchor,
                                on(anchor, "click", function(e) {
                                    e.preventDefault();
                                    e.stopPropagation();
                                    anchor.set("disabled", true);// disable anchor until the cell is refreshed
                                    if (clickAction === null) {
                                        return;
                                    }
                                    when(clickAction(object), function() {
                                            anchor.set("disabled", self.model.isActionDisabled(object));
                                        },
                                        function(error) {
                                            self.showError(error);
                                        }
                                    );
                                })
                            );
                            when(self.model.getJob(object), function(job) {
                                var status = job ? job.mode : "";
                                if (status === "running" || status === "starting" || status === "stopping") {
                                    anchor.set("iconClass", "epi-gridIconStop");
                                    anchor.set("label", i18n.actions_stop);
                                    clickAction = function(connector) {
                                        return self.model.stopJob(connector);
                                    };
                                }
                                else {
                                    anchor.set("iconClass", "epi-gridIconStart");
                                    anchor.set("label", i18n.actions_start);
                                    clickAction = function(connector) {
                                        return self.model.startJob(connector);
                                    };
                                }

                            });
                        }
                    },
                    context: {
                        sortable: false,
                        label: " ",
                        className: "epi-grid--10",
                        formatter: function() {
                            return formatters.menu({title: i18n.contextMenuTitle});
                        }
                    }
                };
            },

            postCreate: function() {
                var self = this,
                    getCurrentItem = function(context) {
                        var row = self.getRow(context);
                        return encodeURIComponent(row.id);
                    };
                this.inherited(arguments);

                this.editMenuItem = new MenuItem({
                    label: i18n.actions_edit,
                    onClick: function () {
                        self.onEdit(getCurrentItem(this));
                    }
                });
                this.deleteMenuItem = new MenuItem({
                    label: i18n.actions_delete,
                    onClick: function () {
                        var id = getCurrentItem(this);
                        self.showConfirmation({
                            title: i18n.deleteConnector,
                            message: i18n.areYouSure,
                            confirmActionText: i18n.buttons_confirmDelete,
                            cancelActionText: i18n.buttons_cancelDelete,
                            action: function() {
                                when(self.store.remove(id),
                                    function() {
                                        self.showSuccess(i18n.deletedMessage);
                                    },
                                    function(error) {
                                        self.showError(error);
                                        self.refresh();
                                    }
                                );
                            }
                        });
                    }
                });

                this.contextMenu.addChild(this.editMenuItem);
                this.contextMenu.addChild(this.deleteMenuItem);

                this.contextMenu.onOpen = function() {
                    var data, job, mode, row = self.row(this.currentTarget);
                    if (!row) {
                        return;
                    }
                    data = row.data;
                    job = self.model.getJob(data);
                    mode = job && job.mode ? job.mode : null;
                    self.editMenuItem.set("disabled", (mode && mode !== "stopped"));
                };
                this.own(this.editMenuItem, this.editMenuItem,
                    this.on("dgrid-error", function(event) {
                        self.showErrorInGrid(event.error);
                    }),
                    this.model.on("statusChanged", function (e) {
                        var status = e && e.status ? e.status : null;
                        var message = status ? i18n[status + "StatusMessage"] : null;
                        if (!message) {
                            return;
                        }
                        var name = e.connector && e.connector.name ? "\"" + entities.encode(e.connector.name) + "\"" : "";
                        message = lang.replace(message, [name]);
                        self.showInfo(message);
                    })
                );
            },

            refresh: function() {
                // we need to refresh cached connector jobs store before refreshing the grid
                var baseRefresh = lang.hitch(this, this.getInherited(arguments)), self = this;
                return when(this.model.refreshJobCache(), function() {
                        return baseRefresh();
                    },
                    function (error) {
                        self.showErrorInGrid(error);
                    }
                );
            },


            onEdit: function(id) {
                // summary:
                //      A callback for edit context menu

            }
        });
    });
