define("epi-find/widget/Dashboard", [
        "dojo/_base/declare",
        "dojo/_base/lang",
        "dojo/_base/config",
        "dojo/_base/array",
        "dojo/dom-class",
        "dojo/dom-style",
        "dojo/on",
        "dojo/date/stamp",
        "dojo/number",
        "dojo/topic",
        "dojo/when",
        "dijit/layout/ContentPane",
        "dijit/_Contained",
        "dijit/form/Button",
        "dijit/Viewport",
        "dijit/_WidgetBase",
        "dijit/_Container",
        "dojox/charting/Chart",
        "dojox/charting/action2d/Tooltip",
        "dojox/charting/action2d/Magnify",
        "dojox/charting/StoreSeries",
        "dojox/mvc/at",
        "put-selector/put",
        "epi-saas-base/Sticker",
        "epi-saas-base/form/RadioButtonGroupList",
        "../AccessModel",
        "../themes/charting/Dashboard",
        "./_ActionableMixin",
        "./_DisabledViewMixin",
        "./ContentWrapperWithSidebar",
        "./PhraseDashboardDialog",
        "./HelpText",
        "./CustomRanges",
        "./DashboardRanges",
        "./DashboardGrid",
        "dojo/i18n!./nls/Dashboard",
        // Chart plugins (not used as arguments)
        "dojox/charting/plot2d/StackedAreas",
        "dojox/charting/plot2d/Markers",
        "dojox/charting/axis2d/Default"
    ],
    function(
            // dojo
            declare,
            lang,
            config,
            array,
            domClass,
            domStyle,
            on,
            stamp,
            number,
            topic,
            when,
            // dijit
            ContentPane,
            _Contained,
            Button,
            Viewport,
            _WidgetBase,
            _Container,
            // dojox
            Chart,
            Tooltip,
            Magnify,
            StoreSeries,
            at,
            put,
            // epi-saas-base
            Sticker,
            RadioButtonGroupList,
            // .
            AccessModel,
            theme,
            _ActionableMixin,
            _DisabledViewMixin,
            ContentWrapperWithSidebar,
            PhraseDashboardDialog,
            HelpText,
            CustomRanges,
            DashboardRanges,
            DashboardGrid,

            i18n) {

        // module:
        //      epi-find/widget/Dashboard

        var Views = {
            // summary:
            //     These are the views avaiable in the Dashboard widget
            mostFrequent: {
                // name: string
                //      String identifier for most frequent searches
                name: "mostFrequent",
                // queryType: string
                //      Query parameter "type" added to endpoint queries
                queryType: "top"
            },
            withoutHits: {
                // name: string
                //      String identifier for searches without hits
                name: "withoutHits",
                // queryType: string
                //      Query parameter "type" added to endpoint queries
                queryType: "null"
            },
            withoutRelevantHits: {
                // name: string
                //      String identifier for searches without relevant hits
                name: "withoutRelevantHits",
                // queryType: string
                //      Query parameter "type" added to endpoint queries
                queryType: "nullclick"
            }
        };

        var labels = {
            view: {
                mostFrequent: i18n.radioButtons_view_mostFrequent,
                withoutHits: i18n.radioButtons_view_withoutHits,
                withoutRelevantHits: i18n.radioButtons_view_withoutRelevantHits
            },
            span: {
                last24hours: i18n.radioButtons_span_last24hours,
                lastWeek: i18n.radioButtons_span_lastWeek,
                lastMonth: i18n.radioButtons_span_lastMonth,
                lastYear: i18n.radioButtons_span_lastYear,
                dateRange: i18n.radioButtons_span_dateRange
            }
        };

        return declare([ContentPane, _ActionableMixin, _DisabledViewMixin], {
            // summary:
            //      Dashboard widget

            // model: epi-find/manage/ManageModel
            //      A stateful object containing current ranges, queries and stores
            model: null,
            // chart: dojox/charting/Chart
            //      The Chart instance
            chart: null,
            // chartPane: dijit/layout/ContentPane
            //      The ContentPane containing the Chart instance
            chartPane: null,
            // grid: dgrid/OnDemandGrid
            //      The dgrid/Grid instance
            grid: null,
            // chartPane: dijit/layout/ContentPane
            //      The ContentPane containing the Grid instance
            gridPane: null,

            _createBestBetsButton: null,
            _createRelatedQueriesButton: null,
            _createSynonymsButton: null,
            _downloadRawButton: null,

            i18n: i18n,

            constructor: function(model) {
                this.model = model;
            },

            postMixInProperties: function() {
                this.inherited(arguments);
                var model = this.model,
                    _getTS = function( /*Date*/ date) {
                        return stamp.toISOString(date, {
                            zulu: true
                        });
                    };

                if (!model.get("view")) {
                    model.set("view", Views.mostFrequent.name);
                }

                if (!model.get("span")) {
                    model.set("span", DashboardRanges.last24hours.name);
                }

                if (!model.get("range")) {
                    model.set("range", DashboardRanges[model.get("span")].getRange());
                }

                // Init query
                if (!model.get("query")) {
                    model.set("query", {
                        from: _getTS(model.get("range").from),
                        to: _getTS(model.get("range").to),
                        interval: model.get("range").interval
                    });
                }

                this.own(model.watch(function(name, oldValue, newValue) {

                    if (oldValue === newValue) {
                        return;
                    }
                    var currentQuery = lang.mixin({}, model.get("query"));

                    switch (name) {
                        case "span":
                            if (oldValue !== newValue) {
                                model.set("range", DashboardRanges[newValue].getRange());

                                if (newValue !== DashboardRanges.dateRange.name) {
                                    CustomRanges.setDisabled(true);
                                } else {
                                    // Enable datepicker to allow customer to pick a range they want
                                    CustomRanges.setDisabled(false);
                                }
                            }
                            break;
                        case "range":
                            model.set("query", lang.mixin({}, currentQuery, {
                                "to": _getTS(newValue.to),
                                "from": _getTS(newValue.from),
                                "interval": newValue.interval
                            }));
                            break;
                        case "rowQuery":
                            model.set("query", lang.mixin({}, currentQuery, {
                                "query": newValue
                            }));
                            break;
                        case "view":
                            var typeQuery = { "type": Views[newValue].queryType },
                                extendedQuery = Views[newValue].queryType === Views.withoutRelevantHits.queryType ? { extended: true } : {};
                            delete currentQuery.type;
                            delete currentQuery.extended;
                            model.set("query", lang.mixin({}, currentQuery, typeQuery, extendedQuery ));
                            break;
                        default:
                            break;
                    }
                }));

                this.own(CustomRanges.startDate.watch(function(name, oldValue, newValue) {
                    if (oldValue === newValue || name !== "value") {
                        return;
                    }
                    if (!CustomRanges.disabledStatus) {
                        model.set("range", DashboardRanges.dateRange.getRange({from: newValue}));
                    }
                }));
                this.own(CustomRanges.endDate.watch(function(name, oldValue, newValue) {
                    if (oldValue === newValue || name !== "value") {
                        return;
                    }
                    if (!CustomRanges.disabledStatus) {
                        model.set("range", DashboardRanges.dateRange.getRange({to: newValue}));
                    }
                }))
            },
            makeButton: function(grid, label, iconClass, view) {
                return new Button({
                    label: label,
                    disabled: true,
                    iconClass: "owl-function-icon--24x24 " + iconClass,
                    onClick: function() {
                        var phrases = [];
                        for (var key in grid.selection) {
                            phrases.push(key);
                        }
                        topic.publish(view, {
                            phrases: phrases.join()
                        });
                    }
                });
            },
            buildRendering: function() {
                this.inherited(arguments);

                var self = this;
                // Navbar
                var model = this.model,
                    ContainedContentPane = declare([ContentPane, _Contained]),
                    viewFormList = new RadioButtonGroupList({
                        "class": "epi-pane__content__nav  clearfix",
                        name: "view",
                        options: [
                            Views.mostFrequent.name,
                            Views.withoutHits.name,
                            Views.withoutRelevantHits.name
                        ],
                        labels: labels,
                        value: at(model, "view")
                    }),
                    // Chart
                    chartPane = this.chartPane = new ContainedContentPane({
                        "class": "epi-vertical-spacer"
                    }),
                    ChartWrapper = declare([_WidgetBase, _Container], {
                        hide: function() {
                            domClass.add(chartWrapper.domNode, "epi-hidden");
                        },
                        show: function() {
                            domClass.remove(chartWrapper.domNode, "epi-hidden");
                        },
                        isVisible: function() {
                            return !domClass.contains(chartWrapper.domNode, "epi-hidden");
                        }
                    }),
                    chartWrapper = this.chartWrapper = new ChartWrapper({}),
                    // Grid
                    gridPane = this.gridPane = new ContentPane({
                        "class": "epi-vertical-spacer--double"
                    }),
                    grid = this.grid = new DashboardGrid({
                        "class": "epi-plain-grid  epi-plain-grid--no-border dgrid-autoheight",
                        query: this.model.get("query"),
                        store: this.model.topQueryStore,
                        loadingMessage: "<div class='dgrid-loading-icon'></div><div class='dgrid-loading-message'>" + i18n.dataLoading + "</div>",
                        showLoadingMessage: true,
                        shouldObserveStore: false
                    }),
                    // "Create"-buttons
                    createSynonymsButton = this._createSynonymsButton = this.makeButton(grid, i18n.createSynonyms, "epi-iconSynonyms", config.intents.create.synonym),
                    createRelatedQueriesButton = this._createRelatedQueriesButton = this.makeButton(grid, i18n.createRelatedQueries, "epi-iconRelatedQueries", config.intents.create.relatedqueries),
                    createBestBetsButton = this._createBestBetsButton = this.makeButton(grid, i18n.createBestBets, "epi-iconBestBets", config.intents.create.bestbet),
                    downloadRawButton = this._downloadRawButton = new Button({
                        label: i18n.downloadRawData,
                        disabled: true,
                        iconClass: "owl-function-icon--24x24 " + "epi-iconDownloadData",
                        onClick: function() {
                            if (grid._total > 0) {
                                // Download top_query
                                when(self.model.topQueryStore.query(Object.assign({}, grid.query, {size: grid._total})),
                                    function(results) {
                                        if (results) {
                                            var csvTitle = "data:text/csv;charset=utf-8," + i18n.fields_query + "," + i18n.fields_numberOfSearches,
                                                csvFileName = "statistic_most_frequent_searches.csv";

                                            if (grid.query.type == "nullclick") {
                                                csvTitle += "," + i18n.fields_clickRate;
                                                csvFileName = "statistic_searches_without_relevant_hits.csv";
                                            } else if (grid.query.type == "null") {
                                                csvFileName = "statistic_searches_without_hits.csv";
                                            }

                                            var data = array.map(results, function(item) {
                                                if (grid.query.type == "nullclick") {
                                                    // Searches without relevant hits
                                                    var total = 0,
                                                        percentage = 0;

                                                    if (item.extended && item.extended.total > 0) {
                                                        total = item.extended.total;
                                                        percentage = 100 - Math.round(item.count / total * 100);
                                                    }

                                                    return item.query + "," + total + "," + percentage + "%";
                                                }
                                                return item.query + "," + item.count;
                                            }).join("\n");
                                            var csvContent = csvTitle + "\n" + data;
                                            var encodedUri = encodeURI(csvContent);

                                            dojo.create("a", { href: encodedUri, download: csvFileName, id: "linkDownloadRawData" }, dojo.body());
                                            dojo.byId('linkDownloadRawData').click();
                                            dojo.destroy('linkDownloadRawData');
                                        }
                                    },
                                    function(error) {
                                        alert("Can't download data! Error: " + error);
                                    }
                                );

                                if (grid.query.type != "nullclick") {
                                    // Download count_query
                                    when(self.model.queryStore.query(grid.query),
                                        function(results) {
                                            if (results) {
                                                var csvTitle = "data:text/csv;charset=utf-8," + i18n.fields_timestamp + "," + i18n.fields_numberOfSearches;
                                                var csvFileName = "statistic_most_frequent_searches_count.csv";
                                                if (grid.query.type == "null") {
                                                    csvFileName = "statistic_searches_without_hits_count.csv";
                                                }

                                                var data = array.map(results, function(item) {
                                                    var timestamp = "";
                                                    var timestampArr = item.timestamp.split('T');
                                                    if (timestampArr.length < 2) {
                                                        return "";
                                                    }
                                                    timestamp += timestampArr[0];

                                                    if (grid.query.interval === "hour") {
                                                        var timestampHour = timestampArr[1].split(":");
                                                        if (timestampHour.length < 2) {
                                                            return "";
                                                        }
                                                        timestamp += " " + timestampHour[0] + ":" + timestampHour[1];
                                                    }

                                                    return timestamp + "," + item.count;
                                                }).join("\n");
                                                var csvContent = csvTitle + "\n" + data;
                                                var encodedUri = encodeURI(csvContent);

                                                dojo.create("a", { href: encodedUri, download: csvFileName, id: "linkDownloadFacetData" }, dojo.body());
                                                dojo.byId('linkDownloadFacetData').click();
                                                dojo.destroy('linkDownloadFacetData');
                                            }
                                        },
                                        function(error) {
                                            alert("Can't download data! Error: " + error);
                                        }
                                    );
                                }
                            }
                        }
                    }),
                    phraseDashboardDialog = new PhraseDashboardDialog({
                        model: this.model,
                        destroyOnHide: false
                    });

                this.own(phraseDashboardDialog);

                // Span radio buttons
                this.spanFormList = new RadioButtonGroupList({
                    "class": "epi-filter-list",
                    name: "span",
                    options: [
                        DashboardRanges.last24hours.name,
                        DashboardRanges.lastWeek.name,
                        DashboardRanges.lastMonth.name,
                        DashboardRanges.lastYear.name,
                        DashboardRanges.dateRange.name
                    ],
                    labels: labels,
                    value: at(model, "span")
                });

                // View form list (navigation)
                this.own(viewFormList);
                this.addChild(viewFormList);

                this.set("notificationPosition", 1);

                // Content wrapper for chart and span selector
                var contentWrapper = this.contentWrapper = new ContentWrapperWithSidebar();
                this.own(contentWrapper);
                this.addChild(contentWrapper);

                this.set("disabledNode", contentWrapper.domNode);

                // Chart - has to be wrapped with help text for proper hide/show
                chartWrapper.addChild(new HelpText({
                    help: i18n.help_chart
                }));
                this.own(chartPane);
                this.own(chartWrapper);
                chartWrapper.addChild(chartPane);
                contentWrapper.addChild(chartWrapper);
                domStyle.set(chartPane.containerNode, "height", "300px"); // Needed for chart to claim its space properly
                domStyle.set(chartPane.containerNode, "overflow", "visible");


                // Span form list (sidebar)
                this.own(this.spanFormList);
                this.rightSideBar = new ContentPane();
                this.rightSideBar.addChild(this.spanFormList);
                this.rightSideBar.addChild(CustomRanges.dateRange);
                contentWrapper.addSidebar(this.rightSideBar);

                // Grid
                contentWrapper.addChild(new HelpText({
                    help: i18n.help_grid
                }));
                gridPane.own(grid);
                gridPane.addChild(grid);

                this.own(gridPane);
                contentWrapper.addChild(gridPane);

                var self = this;

                // Grid-based button behavior
                this.own(
                    // Toggle disabled-state on buttons depending on selection in grid
                    on(grid, "dgrid-select", function(event) {
                        self._enableButtons(event.grid);
                    }),
                    // Toggle disabled-state on buttons depending on selection in grid
                    on(grid, "dgrid-deselect", function(event) {
                        self._enableButtons(event.grid);
                    }),
                    on(grid, "dgrid-refresh-complete", function(event) {
                        self._downloadRawButton.set("disabled", grid._total === 0);
                    }),

                    on(grid, "phrase-click", function(e) {
                        phraseDashboardDialog.set("showMostClicked", (model.get("view") === "mostFrequent"));
                        phraseDashboardDialog.set("query", e.phrase.query);
                        phraseDashboardDialog.show();
                    })
                );

                // Buttons
                contentWrapper.addChild(new HelpText({
                    help: i18n.help_buttons
                }));
                this.own(createSynonymsButton, createBestBetsButton, createRelatedQueriesButton, downloadRawButton);
                this.buttonWrapper = new ContentPane({
                    "class": "epi-pane__content__button-wrapper"
                });

                this.buttonWrapper.addChild(createBestBetsButton);
                this.buttonWrapper.addChild(createRelatedQueriesButton);
                if (AccessModel.isEnabled(config.intents.create.synonym)) {
                    this.buttonWrapper.addChild(createSynonymsButton);
                }
                this.buttonWrapper.addChild(downloadRawButton);
                contentWrapper.addChild(this.buttonWrapper);

                grid.own(
                    this.model.watch("query", function(name, oldValue, newValue) {
                        if (oldValue.to !== newValue.to ||
                            oldValue.from !== newValue.from ||
                            oldValue.query !== newValue.query ||
                            oldValue.type !== newValue.type) {
                            grid.set("query", newValue);
                        }
                    })
                );
            },

            takeAction: function() {
                this.refresh();
                return this;
            },

            _enableButtons: function(grid) {
                var count = 0;
                for (var key in grid.selection) {
                    count += grid.selection.hasOwnProperty(key) ? 1 : 0;
                }
                this._createSynonymsButton.set("disabled", count === 0);
                this._createBestBetsButton.set("disabled", count === 0);
                this._createRelatedQueriesButton.set("disabled", count !== 1);
            },

            refresh: function() {
                if (!this._started) {
                    return;
                }
                when(this.grid.refresh(), lang.hitch(this, function () {
                    this._updateStickers();
                }));
                this._enableButtons(this.grid);

                var ss = new StoreSeries(this.model.queryStore, {
                    query: this.model.query
                }, "count");
                this.chart.updateSeries("y", ss, true);
            },

            destroyRecursive: function() {
                // summary:
                //      This widget will destroy its children and other related non-widgets
                this.inherited(arguments);

                if (this.chart) {
                    this.chart.destroy();
                    delete this.chart;
                }
                if (this.grid) {
                    this.grid.destroy();
                    delete this.grid;
                }
            },

            resize: function() {
                this.inherited(arguments);
                if (this.chart && this.chartWrapper.isVisible()) {
                    this.chart.resize();
                }
                if (this.grid) {
                    this.grid.resize();
                }
                this._updateStickers();
            },

            startup: function() {
                this.inherited(arguments);
                this.grid.startup();
                var model = this.model,
                    chart = this.chart = this._createChart(this.chartPane.containerNode),
                    chartWrapper = this.chartWrapper,
                    grid = this.grid,
                    tooltip = new Tooltip(chart, "default"),
                    magnify = new Magnify(chart, "default"),
                    ss = new StoreSeries(model.queryStore, {
                        query: model.get("query")
                    }, "count");

                chart.addSeries("y", ss).render();
                this.own(model.watch("query", function(name, oldValue, newValue) {

                    if(newValue.type && newValue.type === Views.withoutRelevantHits.queryType) {
                        grid.toggleColumnHiddenState("number", true);
                        grid.toggleColumnHiddenState("total", false);
                        grid.toggleColumnHiddenState("percentage", false);
                        chartWrapper.hide();
                    } else {
                        grid.toggleColumnHiddenState("number", false);
                        grid.toggleColumnHiddenState("total", true);
                        grid.toggleColumnHiddenState("percentage", true);
                        chartWrapper.show();
                        chart.resize();
                    }

                    var ss = new StoreSeries(model.queryStore, {
                        query: newValue
                    }, "count");
                    if (oldValue.type !== newValue.type) {
                        chart.addSeries("y", ss,
                            newValue.type && newValue.type === Views.withoutHits.queryType ? {
                                stroke: {
                                    color: "#ef9e03"
                                },
                                fill: "#fdc860"
                            } :
                            null).render();
                    } else {
                        chart.updateSeries("y", ss, true).render();
                    }
                }));

                // Chart has static size and needs to get a #resize call when viewport resizes
                var resizeTimeOut = null;
                this.own(Viewport.on("resize", lang.hitch(this, function() {
                    if (this.hasChildren()) { // Only resize when there are
                        window.clearTimeout(resizeTimeOut);
                        resizeTimeOut = window.setTimeout(function() {
                            if (chartWrapper.isVisible()) {
                                chart.resize();
                            }
                        }, 10); // Use timeout to prevent mass-resizing
                    }
                })));

                this._stickers = [
                    new Sticker({ domNode: this.buttonWrapper.domNode, offset: {bottom: 0}, scrollNode: this.containerNode}),
                    new Sticker({ domNode: grid.headerNode, offset: {top: 0}, scrollNode: this.containerNode, limitNode: grid.domNode}),
                    new Sticker({ domNode: this.spanFormList.domNode, offset: {top: 0}, scrollNode: this.containerNode})
                ];

                array.forEach(this._stickers, function (sticker) {
                    this.own(sticker);
                }, this);
            },

            _updateStickers: function () {
                if (this._stickers) {
                    array.forEach(this._stickers, function (sticker) {
                        sticker.update(true);
                    });
                }
            },

            _createChart: function( /*DomNode*/ chartNode) {
                // summary:
                //      Creates a chart instance
                // description:
                //      Creates and returns the chart that will be used in dashboard view.
                // chartNode: DomNode|null
                //      The DomNode where to place the grid. Required!
                // tags:
                //      private
                var model = this.model,
                    chart = new Chart(chartNode);

                chart.setTheme(theme);
                chart.addPlot("default", {
                    type: "StackedAreas",
                    markers: true,
                    precision: 0
                }).addAxis("x", {
                    labelFunc: function(i) {
                        return DashboardRanges[model.get("span")].labelFormat(
                            stamp.fromISOString(chart.series[0].source.objects[number.parse(i) - 1].timestamp)
                        );
                    },
                    majorTicks: true,
                    minorTicks: false,
                    microTicks: false,
                    majorTickStep: 1
                }).addAxis("y", {
                    vertical: true,
                    fixLower: "major",
                    fixUpper: "major",
                    min: 0
                });

                return chart;
            }
        });
    });
