﻿define("epi-find/widget/InvertedValidationGrid", [
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/aspect",
    "dojo/dom",
    "dojo/keys",
    "dojo/store/Memory",

    "dijit/_WidgetBase",
    "dijit/_Container",
    "dijit/focus",
    "dijit/MenuItem",

    "./InvertedValidationTextbox",

    "dgrid/OnDemandGrid",
    "dgrid/Keyboard",

    "epi/shell/dgrid/Formatter",
    "epi-cms/dgrid/formatters",

    "dojo/i18n!./nls/Connectors",

    "epi-saas-base/widgets/_GridContextMenuMixin"
],

function (
    array,
    declare,
    lang,
    aspect,
    dom,
    keys,
    Memory,
    _WidgetBase,
    _Container,
    focusUtil,
    MenuItem,
    InvertedValidationTextBox,
    Grid,
    Keyboard,
    Formatter,
    formatters,
    i18n,
    _GridContextMenuMixin
    ) {

    return declare([_WidgetBase, _Container], {

        textBox: null,
        grid: null,
        store: null,
        value: null,
        validationPattern: null,
        label: null,
        editMode: false,

        _setValueAttr: function(value) {
            this._set("value", value);
            var a = array.map(value, function(item) {
                return { value: item, id: Math.random() };
            });
            this.store.setData(a);
            this.grid.refresh();
        },

        postMixInProperties: function() {
            this.inherited(arguments);
            this.store = new Memory({ data: this.value });
        },

        buildRendering: function() {
            this.inherited(arguments);

            this.textBox = new InvertedValidationTextBox({
                required:false,
                pattern: this.validationPattern,
                intermediateChanges: false
            });

            this.addChild(this.textBox);
            this.own(
                this.textBox,
                this.textBox.watch("value", lang.hitch(this, function(name, oldValue, newValue) {
                    if (!this.editMode && newValue !== "" && this.textBox.isValid()) {
                        this.updateOnEvent();
                    }
                    this.editMode = false;
                })),
                this.textBox.on("keydown", lang.hitch(this, function(event) {
                    if (event.keyCode === keys.ENTER) {
                        if(this.textBox.isValid()) {
                            this.updateOnEvent();
                            focusUtil.focus(this.textBox.focusNode);
                        } else {
                            focusUtil.curNode.blur();
                        }
                        event.preventDefault();
                    } else if (event.keyCode === keys.ESCAPE) {
                        this.editMode = false;
                        this.editId = null;
                        this.textBox.set("value","");
                    }
                }))
            );

            var self = this;

            this.grid = new declare([Grid, Formatter, Keyboard, _GridContextMenuMixin])({
                "class": "dgrid dgrid-grid ui-widget epi-plain-grid epi-grid-height--auto",
                showHeader: false,
                store: this.store,
                selectionMode: "single",
                noDataMessage: i18n.noPatternsYet,
                sort: {
                    name: "id", descending: false
                },
                columns: [
                    {
                        autoSave: true,
                        label: this.label,
                        field: "value",
                        sortable: false
                    },
                    {
                        sortable: false,
                        label: " ",
                        className: "epi-grid--10",
                        formatter: function() {
                            return formatters.menu({title: ""});
                        }
                    }
                ]
            });

            this.grid.editMenuItem = new MenuItem({
                label: i18n.actions_edit,
                onClick: function (e) {
                    self.editMode = true;
                    self.editId = self.grid.getRow(this).id;
                    self.textBox.set("value", self.grid.getRow(this).data.value);
                    focusUtil.focus(self.textBox.domNode);
                }
            });
            this.grid.deleteMenuItem = new MenuItem({
                label: i18n.actions_delete,
                onClick: function (e) {
                    self.store.remove(self.grid.getRow(this).id);
                    self.grid.refresh();
                    self.grid.resize();
                    self.textBox.set("value","");
                    e.preventDefault();
                    e.stopPropagation();
                }
            });

            this.grid.contextMenu.addChild(this.grid.editMenuItem);
            this.grid.contextMenu.addChild(this.grid.deleteMenuItem);

            this.addChild(this.grid);
            this.own(this.grid);
        },

        updateOnEvent: function() {
            if (this.editId) {
                if (this.textBox.get("value") !== "") {
                    this.store.put({ value: this.textBox.get("value"), id: this.editId});
                } else {
                    this.store.remove(this.editId);
                }
                this.editId = null;
            } else {
                this.store.add({ value: this.textBox.get("value") });
            }
            this.textBox.set("value", "");
            this.grid.refresh();
            this.grid.resize();
        },

        postCreate: function() {
            this.inherited(arguments);
            aspect.after(this.store, "put", lang.hitch(this, function(response){
                this._set("value", array.filter(array.map(this.store.query(), function(item) {
                    return item.value;
                }), function(value) {
                    return value !== "";
                }));
                this.grid.refresh();
            }));
            aspect.after(this.store, "remove", lang.hitch(this, function(response){
                this._set("value", array.filter(array.map(this.store.query(), function(item) {
                    return item.value;
                }), function(value) {
                    return value !== "";
                }));
                this.grid.refresh();
            }));
        },

        startup: function() {
            this.inherited(arguments);
            this.grid.startup();
            this.set("value", []);
            this.own(this.grid.on("dgrid-refresh-complete", lang.hitch(this, function () {
                this.grid.resize();
            })));
        }
    });
});
