define("epi-find/widget/PhraseValidationTextBox", [
    "dojo/_base/array",
    "dojo/_base/declare",

    "dijit/form/ValidationTextBox",
    "dojo/i18n!./nls/PhraseValidation"
], function(
    array,
    declare,
    ValidationTextBox,
    i18n
){

    // module:
    //      epi-find/widget/PhraseValidationTextBox

    return declare([ValidationTextBox], {
        // summary:
        //      A validation text box adapted to properly validate phrases.
        validationMessage: "",
        phraseDelimiter: "",

        validator: function(/*anything*/ value){
            // summary:
            //      Overridden function to add validation regarding double spaces within phrases and
            //      empty phrases or duplicates on multiple comma separated phrases
            // tags:
            //      protected
            this.validationMessage = "";
            if(this.inherited(arguments) && value && value.split) {
                // Split value on
                var values = this.phraseDelimiter !== "" ? array.map(value.split(this.phraseDelimiter), function(item) { return item.trim();}) : [value.trim()];
                return !array.some(values, function(item, i){
                    // Empty phrases or double spaces (within phrases) or duplicates are not allowed
                    if(item.length === 0) {
                        this.validationMessage = i18n.emptyPhrases;
                        return true;
                    }
                    if(item.match(/\s{2}/)) {
                        this.validationMessage = i18n.doubleSpaces;
                        return true;
                    }
                    if (values.indexOf(item.toLowerCase(), i+1) !== -1) {
                        this.validationMessage = i18n.duplicatePhrases;
                        return true;
                    }
                }, this);
            }
            return false;
        },

        getErrorMessage: function(/*Boolean*/ /*===== isFocused =====*/){
            // summary:
            //        Return an error message to show if appropriate
            // tags:
            //        protected
            return this.validationMessage || this.inherited(arguments);
        }
    });
});
