define("epi-find/widget/RelatedQueriesGrid", [
    "dojo/_base/declare",
    "dojo/_base/array",
    "dojo/_base/lang",
    "dojo/when",
    "dojo/promise/all",

    "dijit/MenuItem",
    "dijit/MenuSeparator",

    "dgrid/Grid",
    "dgrid/extensions/DijitRegistry",

    "epi/shell/widget/ContextMenu",
    "epi/shell/dgrid/Formatter",
    "epi-cms/dgrid/formatters",

    "epi-saas-base/widgets/_GridMoreMixin",
    "epi-saas-base/widgets/_AutoLoadScrollMixin",
    "epi-saas-base/widgets/_NotificationMixin",
    "epi-saas-base/widgets/_GridContextMenuMixin",

    "dojo/i18n!./nls/RelatedQueries"
],
    function(declare,
             array,
             lang,
             when,
             all,
             MenuItem,
             MenuSeparator,
             Grid,
             DijitRegistry,
             ContextMenu,
             Formatter,
             formatters,
             _GridMoreMixin,
             _AutoLoadScrollMixin,
             _NotificationMixin,
             _GridContextMenuMixin,
             i18n
        ) {
        // module:
        //      epi-find/widget/RelatedQueriesGrid
        return declare([Grid, _GridMoreMixin, _AutoLoadScrollMixin, Formatter, _GridContextMenuMixin, _NotificationMixin], {
            // summary:
            //     Related Queries grid.
            constructor: function() {
                // Sets the loader (#loadNextPage, available in _GridMoreMixn) to be used by the _AutoLoadScrollMixin
                this.loader = lang.hitch(this, this.loadNextPage);
            },
            i18n: i18n,
            margin: 140,
            noDataMessage: i18n.noData,
            keepScrollPosition: true,
            columns: {
                query: {
                    label: i18n.fields_phrase,
                    sortable: true,
                    className:"epi-grid--30",
                    renderCell: function(object, value, node) {
                        // render "query" value only for the first row, so that suggestions
                        // that belong to the same query will look somewhat grouped
                        node.innerHTML = this.grid.model.canMoveUp(object) ? "" : value;
                    }
                },
                suggestion: {
                    label: i18n.fields_suggestions,
                    sortable: false
                },

                context: {
                    label: " ",
                    sortable: false,
                    className: "epi-grid--10  epi-inner-related-queries-grid",
                    formatter: function() {
                        return formatters.menu({title: i18n.contextMenuTitle});
                    }
                }
            },

            postCreate: function() {
                var self = this, menuSeparator1 = new MenuSeparator(), menuSeparator2 = new MenuSeparator(),
                    getCurrentItem = function(context) {
                        var row = self.getRow(context);
                        return encodeURIComponent(row.id);
                    },
                    // This sorting should be in sync with the backend other_suggestions sorting.
                    defaultSort = [
                        {attribute: "priority", descending: true},
                        {attribute: "timestamp", descending: true}
                    ];

                this.on("dgrid-sort", function(event) {
                    event.preventDefault();
                    event.stopPropagation();
                    self.set("sort", event.sort.concat(defaultSort));
                });
                this._sort = [{attribute:"query", descending: false}].concat(defaultSort);

                this.inherited(arguments);

                this.editMenuItem = new MenuItem({
                    label: i18n.actions_edit,
                    onClick: function () {
                        var id = getCurrentItem(this);
                        self.onEdit({grid: self, id: encodeURIComponent(id), suggestion: self.row(id).data.suggestion});
                    }
                });
                this.moveUpMenuItem = new MenuItem({
                    label: i18n.actions_moveUp,
                    onClick: function () {
                        var id = getCurrentItem(this);
                        when(self.model.moveUp(self.row(id).data), function() {
                                self.showSuccess(i18n.savedMessage);
                                self.refresh();
                            },
                            function(error) {
                                self.showError(error);
                                self.refresh();
                            }
                        );
                    }
                });

                this.moveDownMenuItem = new MenuItem({
                    label: i18n.actions_moveDown,
                    onClick: function () {
                        var id = getCurrentItem(this);
                        when(self.model.moveDown(self.row(id).data), function() {
                                self.showSuccess(i18n.savedMessage);
                                self.refresh();
                            },
                            function(error) {
                                self.showError(error);
                                self.refresh();
                            }
                        );
                    }
                });

                this.deleteMenuItem = new MenuItem({
                    label: i18n.actions_deleteSuggestion,
                    onClick: function () {
                        var id = getCurrentItem(this);
                        self.showConfirmation({
                            title: i18n.deleteSuggestionTitle,
                            message: i18n.areYouSureSuggestion,
                            confirmActionText: i18n.buttons_confirmDelete,
                            cancelActionText: i18n.buttons_cancelDelete,
                            action: function() {
                                when(self.model.remove(id), function(){
                                        self.showSuccess(i18n.suggestionDeletedMessage);
                                        self.refresh();
                                    },
                                    function(error){
                                        self.showError(error);
                                        self.refresh();
                                    }
                                );
                            }
                        });
                    }
                });

                this.deleteAllMenuItem = new MenuItem({
                    label: i18n.actions_deleteAll,
                    onClick: function () {
                        var id = getCurrentItem(this);
                        self.showConfirmation({
                            title: i18n.deleteTitle,
                            message: i18n.areYouSurePhrase,
                            confirmActionText: i18n.buttons_confirmDelete,
                            cancelActionText: i18n.buttons_cancelDelete,
                            action: function() {
                                var removals = array.map(self.row(id).data.other_suggestions, function(item) {
                                    return self.model.remove(item);
                                });
                                all(removals).then(function(){
                                        self.showSuccess(i18n.phraseDeletedMessage);
                                        self.refresh();
                                    },
                                    function(error) {
                                        self.showError(error);
                                        self.refresh();
                                    }
                                );
                            }
                        });
                    }
                });

                this.contextMenu = new ContextMenu({
                    onOpen: function() {
                        var data = self.row(this.currentTarget).data;
                        self.moveUpMenuItem.set("disabled", !self.model.canMoveUp(data));
                        self.moveDownMenuItem.set("disabled", !self.model.canMoveDown(data));

                        // Show deleteAllMenuItem only for the first suggestion
                        // and if there are other suggestions
                        if (!self.model.canMoveUp(data) && data.other_suggestions && data.other_suggestions.length > 1) {
                            if (self.contextMenu.getIndexOfChild(self.deleteAllMenuItem) === -1) {
                                // place deleteAllMenuItem before deleteMenuItem
                                self.contextMenu.addChild(self.deleteAllMenuItem,
                                    self.contextMenu.getIndexOfChild(self.deleteMenuItem));
                            }
                        }
                        else {
                            self.contextMenu.removeChild(self.deleteAllMenuItem);
                        }
                    }
                });

                // Creating menu structure
                this.contextMenu.addChild(this.editMenuItem);
                this.contextMenu.addChild(menuSeparator1);
                this.contextMenu.addChild(this.moveUpMenuItem);
                this.contextMenu.addChild(this.moveDownMenuItem);
                this.contextMenu.addChild(menuSeparator2);
                this.contextMenu.addChild(this.deleteMenuItem);

                this.own( this.editMenuItem,
                    menuSeparator1,
                    this.moveUpMenuItem,
                    this.moveDownMenuItem,
                    menuSeparator2,
                    this.deleteMenuItem,
                    this.deleteAllMenuItem,
                    this.on("dgrid-error", function(event) {
                        self.showErrorInGrid(event.error);
                    })
                );

            },

            onEdit: function(args) {
                // summary:
                //      Callback for edit action
                // args: Object
                //      Callback arguments: args.grid - reference to the grid, args.id - selected item ID
            }
        });
    });