﻿define("epi-find/widget/SiteMenuModel", [
    "dojo/_base/declare",
    "dojo/_base/array",
    "dojo/_base/config",

    "dojo/Stateful",
    "dijit/Destroyable",

    "dojox/lang/functional",
    "dojox/mvc/sync",

    "epi-cms/ApplicationSettings",
    "epi/dependency",

    "../store/LangMixin",
    "../store/SiteMixin",
    "../ConfigModel",
    "../ApplicationState",

    "dojo/i18n!epi-saas-base/nls/Languages",
    "dojo/i18n!./nls/SiteMenu"
],
    function(declare,
             array,
             config,
             Stateful,
             Destroyable,
             functional,
             sync,
             ApplicationSettings,
             dependency,
             LangMixin,
             SiteMixin,
             ConfigModel,
             ApplicationState,
             i18nLanguages,
             i18n) {

        // module:
        //      epi-find/widget/SiteMenuModel

        return declare([Stateful, Destroyable], {
            // summary:
            //     Model for the site menu for site and language.

            showHelp: false,

            sites: null,

            selectedSite: null,
            _selectedSiteSetter: function (site) {
                if (site) {
                    ApplicationSettings.startPage = site.startPage;
                    ApplicationSettings.rootPage = site.rootPage;
                    ApplicationSettings.wastebasketPage = site.wasteBasket || "2"; // fallback for backward compatibility

                    functional.forEach(config.dependencies, function (dependency) {
                        // TODO: Consider avoid changes in stores state like this since stores are often used
                        // with 'delegate', which might hinder changing the store state.
                        // Here we have to use a workaround with the 'unwrapped' property in order to reach
                        // the original store object.
                        if (dependency.isInstanceOf(SiteMixin)) {
                            if (dependency.unwrapped) {
                                dependency.unwrapped.setSite(site.id);
                            }
                            else {
                                dependency.setSite(site.id);
                            }
                        }
                    });
                }
                this.selectedSite = site;
                ApplicationState.set("site", site);
            },
            sitesDisabled: false,

            languages: null,
            selectedLanguage: null,
            _selectedLanguageSetter: function (language) {
                if (language) {
                    functional.forEach(config.dependencies, function (dependency) {
                        // TODO: Consider avoid changes in stores state like this since stores are often used
                        // with 'delegate', which might hinder changing the store state.
                        // Here we have to use a workaround with the 'unwrapped' property in order to reach
                        // the original store object.
                        if (dependency.isInstanceOf(LangMixin)) {
                            if (dependency.unwrapped) {
                                dependency.unwrapped.setLanguage(language.id);
                            }
                            else {
                                dependency.setLanguage(language.id);
                            }
                        }
                    });
                }
                this.selectedLanguage = language;
                ApplicationState.set("language", language);


                if (!dependency.exists("epi.shell.Profile")) {
                    dependency.register("epi.shell.Profile", {});
                }
                var profile = dependency.resolve("epi.shell.Profile");
                profile.contentLanguage =  this.selectedLanguage.id;
            },
            languagesDisabled: false,

            // config: object
            //      Configuration parameters
            config: null,
            _configSetter: function(value) {
                if (!value) {
                    return;
                }
                var languages = value.languages, languageItems, sites = value.sites, rootPage;

                languageItems = array.map(languages, function(item) {
                    return {id: item, name: i18nLanguages[item] || item };
                });
                languageItems.unshift({ id: null, name: i18n.allLanguages });
                this.set("languages", languageItems);
                this.set("selectedLanguage", languageItems[0]);


                if (sites.length > 0) {
                    rootPage = sites[0].rootPage;
                }
                sites.unshift({ id: null, name: i18n.allSites, rootPage: rootPage, startPage: rootPage });

                this.set("sites", sites);
                this.set("selectedSite", sites[0]);

                this.config = value;
            },

            constructor: function(options) {
                declare.safeMixin(this, options);
                var self = this;

                // read current configuration
                this.set("config", ConfigModel.config);
                // and watch for configuration changes
                this.own(ConfigModel.watch("config", function(name, oldValue, value) {
                    self.set("config", value);
                }));
            }
        });
    });