define("epi-find/widget/SwoshContainer", [
    "dojo/_base/declare",
    "dojo/dom-class",
    "dojo/dom-style",
    "dojo/Stateful",
    "dojo/topic",

    "dijit/layout/ContentPane",
    "dijit/focus"
],
function(declare, domClass, domStyle, Stateful, topic,
    ContentPane, focusUtil
) {

    // module:
    //      epi-find/widget/Swosh

    return declare([ContentPane], {
        // summary:
        //     A base widget for swosh-containers.

        // swoshPanes: { intent: string,  widget: epi-find/widget/_SwoshPaneBase[] }
        //      A list of content panes to switch amongst.
        swoshPanes: null,

        // model: dojo/Stateful
        //      Used to store the property "activePaneIndex"
        model: null,

        activeClassName: "is-active",

        hiddenClassName: "is-hidden",

        postMixInProperties: function() {
            this.inherited(arguments);

            if (! this.model) {
                this.model = new Stateful();
            }

            if (!this.model.get("activePaneIndex")) {
                this.model.set("activePaneIndex", 0);
            }

            if( !this.swoshPanes) {
                this.swoshPanes = [];
            }
        },

        postCreate: function() {
            this.inherited(arguments);

            for (var i in this.swoshPanes) {
                var widget = this.swoshPanes[i].widget;
                this.addChild(widget);
                this.own(widget);

                domClass.add(widget.domNode, this.hiddenClassName);
            }
            //TODO: MAKE A CLASS!
            domStyle.set(this.domNode, "height", "100%");
            domStyle.set(this.domNode, "position", "relative");
            domStyle.set(this.domNode, "overflow", "hidden");
            domClass.replace(this.getActiveWidget().domNode, this.activeClassName, this.hiddenClassName);

            var self = this;
            this.own(
                // The widget will express the intent of switching views on focus change
                focusUtil.watch("activeStack", function(name, oldFocusStack, newFocusStack) {
                    for(var i in self.swoshPanes) {
                        var pane = self.swoshPanes[i],
                            hadFocus = oldFocusStack.indexOf(pane.widget.id) > -1,
                            hasFocus = newFocusStack.indexOf(pane.widget.id) > -1,
                            isActivePane = self.model.activePaneIndex === i;
                        if (! hadFocus && hasFocus && ! isActivePane) {
                            topic.publish(pane.intent, {});
                            return;
                        }
                    }
                }),

                // The intent will hopefully bubble up and end up in a call to setActiveWidget will will trigger
                // the fancy switching of panes.
                this.model.watch("activePaneIndex", function() {
                    var activePane = self._getActiveWidgetWrapper();

                    if (self.swoshPanes.indexOf(activePane) === -1) {
                        return;
                    }

                    for (var i in self.swoshPanes) {
                        var widget = self.swoshPanes[i].widget;

                        if (widget === activePane.widget) {
                            domClass.replace(widget.domNode, self.activeClassName, self.hiddenClassName);

                            var hasFocus = focusUtil.activeStack.indexOf(widget.id) > -1;
                            if (! hasFocus) {
                                focusUtil.focus(widget.domNode);
                            }

                            widget.showChildren();
                            widget.resize();    // So that widget resizes child elements if needed
                        } else {
                            domClass.replace(widget.domNode, self.hiddenClassName, self.activeClassName);
                            widget.hideChildren();
                        }
                    }
                })
            );
        },

        getActiveWidget: function() {
            return this._getActiveWidgetWrapper().widget;
        },

        _getActiveWidgetWrapper: function() {
            return this.swoshPanes[this.model.get("activePaneIndex")];
        },

        setActiveWidget: function(index) {
            this.model.set("activePaneIndex", index);
        },

        startup: function() {
            this.inherited(arguments);

            // Trigger initial swosh
            this.model.set("activePaneIndex", this.model.get("activePaneIndex"));
        },

        destroy: function() {
            this.inherited(arguments);
        }
    });
});