﻿/*
 Client-side API for Find
 Method starting with underscore are considered private.
 */
 function FindApi() {
    this._applicationUrl = "/";
    this._serviceApiBaseUrl = "";
    this._trackId = "";
    this._trackParam = "_t_";
    this._dontTrackQueryParam = "_t_dtq";
    this._allowTrackingCookieName = null;
    this._bufferTrackRequest = true;

    this.setApplicationUrl = function (applicationUrl) {
        this._applicationUrl = applicationUrl;
    };

    this.setServiceApiBaseUrl = function (serviceApiBaseUrl) {
        this._serviceApiBaseUrl = serviceApiBaseUrl;
    };

    this.setAllowTrackingCookieName = function (cookieName) {
        this._allowTrackingCookieName = cookieName;
    };

    this.setTrackParam = function (trackParam) {
        this._trackParam = trackParam;
    };

    this.setDontTrackQueryParam = function (dontTrackQueryParam) {
        this._dontTrackQueryParam = dontTrackQueryParam;
    };

    this.bindWindowEvents = function () {
        var self = this;
        if (window.history) {
            window.onbeforeunload = function (event) {
                var href = document.location.href;
                if (href.indexOf("q=") > 0 && href.indexOf(self._dontTrackQueryParam + "=") == -1) { // don't track q if a user goes back in history
                    window.history.replaceState(window.history.state, window.document.title, href + (href.indexOf("?") > 0 ? "&" : "?") + self._dontTrackQueryParam + "=true");
                }
            };
        }

        window.addEventListener("load", this.bindClickEvents(), false);
    };

    this.bindClickEvents = function () {
        var self = this;
        var links = self._toArray(document.getElementsByTagName("A"));
        var localUrlParser = document.createElement("A");
        localUrlParser.href = document.location.href;
        var addClickEvents = function (link) {
            var beTracked = false;
            var href = link.href;
            var params = self._getTrackQueryParamsFromUrl(href);
            for (var j = 0; j < params.trackQueryParams.length; j++) {
                if (params.trackQueryParams[j].indexOf("id") == 0) {
                    beTracked = true;
                    break;
                }
            }

            if (params.trackQueryParams.length > 0 && beTracked) {
                var hitUrlParser = document.createElement("A");
                hitUrlParser.href = href;
                if ((self._applicationUrl != "/" && (localUrlParser.hostname + self._applicationUrl) != (hitUrlParser.hostname + "/" + hitUrlParser.pathname.split("/")[1] + "/")) ||
                    (self._applicationUrl === "/")) { // allow click tracking local products

                    // Old Find client renders a link against the _click endpoint, ignore these
                    if (href.split("?")[0].indexOf(self._serviceApiBaseUrl + "_click") != -1) {
                        return;
                    }

                    link.href = self._getUriWithoutTrackParams(href, params);

                    // Catch click, middleclick and ctrl/shift/cmd + click
                    link.addEventListener("mousedown", function handleMousedown(e1) {
                        this.addEventListener("mouseup", function handleMouseup(e2) {
                            if (e1.target == e2.target && e1.which == e2.which && (e2.which == 1 || e2.which == 2)) {
                                // Disable these events to not register clicks for this link more than once
                                this.removeEventListener("mousedown", handleMousedown);
                                this.removeEventListener("mouseup", handleMouseup);
                                var trackUrl = self._constructTrackUrl(params.trackQueryParams);
                                self._sendEvent(trackUrl, true);
                            }
                        });
                    });
                }
            }
        };

        for (var i = 0; i < links.length; i++) {
            addClickEvents(links[i]);
        }
    };

    this.bindAClickEvent = function (api) {
        /*
                var self = this;
                var links = self._toArray(document.getElementsByTagName("A"));
                for (var i = 0; i < links.length; i++) {
                    var scope = function () {
                        var href = links[i].href;
                        if(href.indexOf(self._serviceApiBaseUrl + "_click?") != -1) {
                            return;
                        }

                        var params = self._getTrackQueryParamsFromUrl(href);

                        if (params.trackQueryParams.length > 0) {
                            links[i].onclick = function (event) {
                                if (params.trackQueryParams.length > 0) {
                                    self._bufferEvent(self._constructTrackUrl(params.trackQueryParams));
                                }

                                event = event || window.event;
                                var link = event.currentTarget || event.srcElement;
                                if (link) {
                                    link.href = self._getUriWithoutTrackParams(href, params);
                                }
                            };
                        }
                    };
                    scope();
                }
        */
    };

    this._getUriWithoutTrackParams = function (href, params) {
        if (params.trackQueryParams.length > 0 && params.nonTrackQueryParams.length > 0) {
            return href.split("?")[0] + "?" + params.nonTrackQueryParams.join("&");
        } else if (params.trackQueryParams.length > 0) {
            return href.split("?")[0];
        }
    };

    this._getTrackQueryParamsFromUrl = function (href) {
        var query = href.split("?")[1] || "";
        var queryParams = query.split("&");
        var trackQueryParams = [];
        var nonTrackQueryParams = [];
        this._bufferTrackRequest = true;
        for (var j = 0; j < queryParams.length; j++) {
            var queryParamPair = queryParams[j].split("=");
            if (decodeURIComponent(queryParamPair[0]) == this._dontTrackQueryParam && queryParamPair[1] == "true") {
                this._bufferTrackRequest = false; //if _t_dtq=true is present, do not buffer a _track request later
            }
            if (decodeURIComponent(queryParamPair[0]).slice(0, this._trackParam.length) == this._trackParam) {
                var trackParam = decodeURIComponent(queryParamPair[0]).slice(this._trackParam.length);
                trackQueryParams.push(encodeURIComponent(trackParam) + "=" + queryParamPair[1]);
            }
            else {
                nonTrackQueryParams.push(queryParams[j]);
            }
        }

        return { trackQueryParams: trackQueryParams, nonTrackQueryParams: nonTrackQueryParams };
    };

    this._constructTrackUrl = function (trackQueryParams) {
        return this._serviceApiBaseUrl + "_track?" + trackQueryParams.join("&");
    };


    this.processEventFromCurrentUri = function () {
        var uri = document.location.href;
        var params = this._getTrackQueryParamsFromUrl(uri);
        if (params.trackQueryParams.length > 0) {
            if (this._bufferTrackRequest) {
                this._bufferEvent(this._constructTrackUrl(params.trackQueryParams));
            }

            if (window.history) {
                window.history.replaceState(window.history.state, window.document.title, this._getUriWithoutTrackParams(uri, params));
            }
        }
    };

    this._toArray = function (collection) {
        var result = [];
        if (!collection || !collection.length) {
            return result;
        }
        for (var i = 0; i < collection.length; i++) {
            if (i in collection) {
                result.push(collection[i]);
            }
        }
        return result;
    };

    this._getCookies = function (nameFilter, maxCount) {
        var i, name, value;
        var results = [];
        var cookies = document.cookie.split(";");

        for (i = 0; i < cookies.length; i++) {
            name = cookies[i].substr(0, cookies[i].indexOf("="));
            name = name.replace(/^\s+|\s+$/g, "");

            if (!nameFilter || nameFilter(name)) {
                value = cookies[i].substr(cookies[i].indexOf("=") + 1);
                results.push({ name: name, value: decodeURIComponent(value) });

                if (maxCount && --maxCount === 0) {
                    break;
                }
            }
        }

        return results;
    };

    this._getCookie = function (name) {
        var cookies = this._getCookies(function (cookieName) { return name === cookieName; }, 1);

        if (cookies.length) {
            return cookies[0].value;
        }
    };

    this._setCookie = function (name, value, exdays) {
        var exdate = new Date();
        exdate.setDate(exdate.getDate() + exdays);
        value = encodeURIComponent(value) + "; Path=/" + ((exdays === null) ? "" : "; expires=" + exdate.toUTCString());
        document.cookie = name + "=" + value;
    };

    this._deleteCookie = function (name) {
        document.cookie = name + "=; Path=/; expires=Thu, 01 Jan 1970 00:00:01 GMT";
    };

    this._debug = function (msg) {
        if (window.console) {
            console.log(msg);
        }
    };

    this._getBufferedEvents = function () {
        return this._getCookies(function (cookieName) { return cookieName.indexOf("_findEvent") === 0; });
    };

    this._bufferEvent = function (event) {
        this._setCookie("_findEvent" + this._createUUID(), event, 365 * 10);
    };

    this._deleteBufferedEvent = function (name) {
        this._deleteCookie(name);
    };

    this.sendBufferedEvents = function () {
        var events = this._getBufferedEvents();

        for (var i = 0; i < events.length; i++) {
            this._sendEvent(events[i].value, false);
            this._deleteBufferedEvent(events[i].name);
        }
    };

    this._sendEvent = function (url, checkKeepAlive) {
        const browserSupportsKeepalive = 'keepalive' in new Request('');

        if (checkKeepAlive && !browserSupportsKeepalive) {
            this._bufferEvent(url);
            return;
        }

        if (window.fetch) {
            fetch(url, {
                method: "GET",
                keepalive: true,
                credentials: "same-origin",
                headers: { "Accept": "application/json" }
            })
                .then(function (response) { return response.json(); })
                .then(function (data) { return data; })
                .catch(function (error) { return error; });
            return;
        }

        if (window.XMLHttpRequest) {
            var request = new XMLHttpRequest();
            if ("withCredentials" in request) {
                request.open("GET", url, false);
                request.send();
                return;
            }
        }
        if (window.XDomainRequest) {
            var xdr = new XDomainRequest();
            xdr.open("GET", url);
            xdr.send();
            return;
        }
        var img = new Image(1, 1);
        img.src = url;
    };

    //UUID implementation inspired by http://frugalcoder.us/post/2012/01/13/javascript-guid-uuid-generator.aspx
    this._createUUID = function (data) {
        var ret = "";
        var rnd = this._PRNG();
        for (; ret.length < 32; ret += Math.floor(rnd() * 0xffffffff).toString(16));
        return [ret.substr(0, 8), ret.substr(8, 4), "4" + ret.substr(12, 3), "89AB"[Math.floor(Math.random() * 4)] + ret.substr(16, 3), ret.substr(20, 12)].join("-");
    };

    //Based on Kybos+Mash implementation from http://baagoe.org/en/wiki/Kybos + http://baagoe.org/en/wiki/Mash
    this._PRNG = function () {
        return (function (args) {
            var s0 = 0;
            var s1 = 0;
            var s2 = 0;
            var c = 1;
            var s = [];
            var k = 0;
            var n = 0xefc8249d;

            var mash = function (data) {
                data = data.toString();
                for (var i = 0; i < data.length; i++) {
                    n += data.charCodeAt(i);
                    var h = 0.02519603282416938 * n;
                    n = h >>> 0;
                    h -= n;
                    h *= n;
                    n = h >>> 0;
                    h -= n;
                    n += h * 0x100000000; // 2^32
                }
                return (n >>> 0) * 2.3283064365386963e-10; // 2^-32
            };

            s0 = mash(" ");
            s1 = mash(" ");
            s2 = mash(" ");
            for (var j = 0; j < 8; j++) {
                s[j] = mash(" ");
            }

            if (args.length === 0) {
                args = [+new Date()];
            }
            for (var i = 0; i < args.length; i++) {
                s0 -= mash(args[i]);
                if (s0 < 0) {
                    s0 += 1;
                }
                s1 -= mash(args[i]);
                if (s1 < 0) {
                    s1 += 1;
                }
                s2 -= mash(args[i]);
                if (s2 < 0) {
                    s2 += 1;
                }
                for (j = 0; j < 8; j++) {
                    s[j] -= mash(args[i]);
                    if (s[j] < 0) {
                        s[j] += 1;
                    }
                }
            }

            var random = function () {
                var a = 2091639;
                k = s[k] * 8 | 0;
                var r = s[k];
                var t = a * s0 + c * 2.3283064365386963e-10; // 2^-32
                s0 = s1;
                s1 = s2;
                s2 = t - (c = t | 0);
                s[k] -= s2;
                if (s[k] < 0) {
                    s[k] += 1;
                }
                return r;
            };
            random.uint32 = function () {
                return random() * 0x100000000; // 2^32
            };
            random.fract53 = function () {
                return random() +
                    (random() * 0x200000 | 0) * 1.1102230246251565e-16; // 2^-53
            };
            random.addNoise = function () {
                for (var i = arguments.length - 1; i >= 0; i--) {
                    for (j = 0; j < 8; j++) {
                        s[j] -= mash(arguments[i]);
                        if (s[j] < 0) {
                            s[j] += 1;
                        }
                    }
                }
            };
            random.version = "Kybos 0.9";
            random.args = args;
            return random;

        }(Array.prototype.slice.call(arguments)));
    };
}