require({cache:{
'epi-find/Configuration':function(){
﻿define("epi-find/Configuration", [
    "dojo/_base/config",
    "epi-find/Intents!"
],
    function (config) {
        // module:
        //		epi-find/Configuration
        return {
            // summary:
            //      A plugin that initializes application configuration.
            load: function (id, require, callback) {
                if (!config.find) {
                    config.find = {};
                }

                // The URL where user can order upgraded version of the product to enable features
                config.find.orderFindUrl = "http://www.episerver.com/orderfind";
                config.find.defaultSearchProviders = [
                    { searchArea: "CMS/pages", id:"EPiServer.Find.Cms.SearchProviders.EnterprisePageSearchProvider"},
                    { searchArea: "CMS/files", id:"EPiServer.Find.Cms.SearchProviders.EnterpriseMediaSearchProvider"}
                ];

                config.find.defaultDescriptors = {
                        pages: {roots: ["1"]}
                    };
                callback();
            }
        };
    });
},
'epi-find/Intents':function(){
﻿define("epi-find/Intents", [
    "dojo/_base/config"
],
    function (config) {
        // module:
        //      epi-find/Intents
        return {
            // summary:
            //      A plugin that configures all application intents.
            load: function (id, require, callback) {
                config.intents = {
                    view: {
                        dashboard: "dashboard",
                        synonyms: "optimize/synonyms",
                        bestBets: "optimize/bestbets",
                        relatedQueries: "optimize/relatedqueries",
                        autocomplete: "optimize/autocomplete",
                        connectors: "configure/connectors",
                        boosting: "configure/boosting",
                        index: "configure/index",
                        overview: "content/overview",
                        explore: "content/explore",
                        servicehealth: "servicehealth"
                    },

                    create: {
                        synonym: "optimize/synonyms",
                        bestbet: "optimize/bestbets",
                        relatedqueries: "optimize/relatedqueries"
                    }
                };
                callback();
            }
        };
    });
},
'epi-find/Application':function(){
﻿define("epi-find/Application", [
    "dojo/_base/config",
    "dojo/_base/declare",
    "dojo/query",
    "dojo/dom-attr",
    "dojo/dom-class",
    "dojo/dom-construct",
    "dojo/router",
    "dojo/hash",
    "dojo/topic",
    "dojo/Deferred",
    "dojox/mvc/at",

    "./store/StoreFactory",
    "./widget/SiteMenu",
    "./widget/SiteMenuModel",

    "./ConfigModel",
    "./IndexStateModel",

    "epi-saas-base/NotificationHandler",
    "epi-saas-base/_PropagatingDestroyableMixin",

    "./CommonModule",

    "./manage/DashboardController",
    "./optimize/OptimizationController",
    "./configure/ConfigureController",
    "./servicehealth/ServiceHealthController",
    "./content/ContentController"
],
function(
    config,
    declare,
    query,
    domAttr,
    domClass,
    domConstruct,
    router,
    hash,
    topic,
    Deferred,
    at,
    StoreFactory,
    SiteMenu,
    SiteMenuModel,
    ConfigModel,
    IndexStateModel,
    NotificationHandler,
    _PropagatingDestroyableMixin,

    CommonModule,
    DashboardController,
    OptimizationController,
    ConfigureController,
    ServiceHealthController,
    ContentController
    ) {

    var Application = declare([_PropagatingDestroyableMixin], {
        // summary:
        //          Application code for Find.

        views: null,

        // rootNode: DomNode
        //      This is the root node for the application passed from the bootstrapper
        rootNode: null,

        // siteMenu: /widget/SiteMenu
        //      the menu where the user can change Site and Language
        siteMenu: null,

        // siteMenuModel: epi-find/widget/SiteMenuModel
        //      Model for the site menu
        siteMenuModel: null,

        // defaultView: String
        //      View that should be activated by default
        defaultView: config.intents.view.dashboard,

        startup: function () {
            var self = this;
            var initDeferred = new Deferred();
            this.rootNode = config.rootNode || document.getElementById("applicationContainer");

            config.dependencies = {};

            var controllerContainer = domConstruct.create("div", {});

            this.siteMenuModel = new SiteMenuModel();
            this.own(this.siteMenuModel);

            this.siteMenu = new SiteMenu({
                helpContainer: controllerContainer,
                showHelp: at(this.siteMenuModel, "showHelp"),

                sites: at(this.siteMenuModel, "sites"),
                selectedSite: at(this.siteMenuModel, "selectedSite"),
                sitesDisabled: at(this.siteMenuModel, "sitesDisabled"),

                languages: at(this.siteMenuModel, "languages"),
                selectedLanguage: at(this.siteMenuModel, "selectedLanguage"),
                languagesDisabled: at(this.siteMenuModel, "languagesDisabled")
            });
            this.siteMenu.placeAt(this.rootNode);
            this.siteMenu.startup();
            this.own(this.siteMenu);

            IndexStateModel.init();
            StoreFactory.authToken = config.find.token;
            // start configuration load and when config is loaded, init CommonModule and indicate initialization complete
            ConfigModel.target =  config.find.siteConfigUrl || config.find.siteServiceApiBaseUrl + "config/";

            ConfigModel.refresh().then(function(configuration) {
                StoreFactory.createStores();
                CommonModule.initialize();
                IndexStateModel.refreshDocumentsInIndex();
                initDeferred.resolve();
            });

            // Initializing generic notifications
            var notificationHandler = new NotificationHandler({
                rootNode: this.rootNode
            });
            notificationHandler.startup();
            this.own(notificationHandler);

            domClass.add(controllerContainer, "epi-controllerContainer");
            domConstruct.place(controllerContainer, this.rootNode);

            this.views = {
                dashboard: {
                    name: "dashboard",
                    controller: new DashboardController(this, controllerContainer)
                },
                optimize: {
                    name: "optimize",
                    controller: new OptimizationController(this, controllerContainer)
                },
                configure: {
                    name: "configure",
                    controller: new ConfigureController(this, controllerContainer)
                },
                servicehealth: {
                    name: "servicehealth",
                    controller: new ServiceHealthController(this, controllerContainer)
                },
                content: {
                    name: "content",
                    controller: new ContentController(this, controllerContainer)
                }
            };

            this.own(this.views.dashboard.controller);
            this.own(this.views.optimize.controller);
            this.own(this.views.configure.controller);
            this.own(this.views.servicehealth.controller);
            this.own(this.views.content.controller);

            config.dependencies["epi-find.DashboardController"] = this.views.dashboard.controller;
            config.dependencies["epi-find.OptimizationController"] = this.views.optimize.controller;
            config.dependencies["epi-find.ConfigureController"] = this.views.configure.controller;
            config.dependencies["epi-find.ServiceHealthController"] = this.views.servicehealth.controller;
            config.dependencies["epi-find.ContentController"] = this.views.content.controller;

            this.own(router.register(".*", function(event) {
                self._onHashUpdate(event);
            }));

            this._registerIntents(config.intents);
            initDeferred.then(function(){
                router.startup();
                self.own(
                    self.siteMenuModel.watch("selectedLanguage", function(name, oldValue, newValue) {
                        if (newValue) {
                            self._refreshView();
                        }
                    }),
                    self.siteMenuModel.watch("selectedSite", function(name, oldValue, newValue) {
                        if (newValue) {
                            self._refreshView();
                        }
                    }));
            });

            // Redirect to statistics view if hash is not set
            hash(location.hash || this.defaultView, true);
        },

        _registerIntents: function(intents) {
            for(var i in intents) {
                var intent = intents[i];
                if (intent instanceof Object) {
                    this._registerIntents(intent);
                    continue;
                }
                topic.subscribe(intent, this._newIntentTrigger(intent));
            }
        },

        _newIntentTrigger: function(intent) {
            var self = this;
            return function(params) {
                var queryString = self._paramsToQuery(params);
                router.go(intent + (queryString.length > 0 ? "?" + queryString : ""));
            };
        },

        _paramsToQuery: function(object) {
            var result = [];
            for (var key in object) {
                result.push(key + "=" + encodeURIComponent(object[key]));
            }
            return result.join("&");
        },

        _onHashUpdate: function(event) {
            var path = event.newPath;

            var action = this.getActionFromPath(path);
            var context = action.actions.shift();

            var activated = this._activateView(context, action);

            if (!activated)
            {
                // TODO: Show a proper 404-page
                console.error("Error: Unmapped route!");
                console.error(event);
                console.error(action);

                // Revert the hash to previous value or go to a default view as a workaround until we have 404 pages
                if (event.oldPath) {
                    hash(event.oldPath);
                }
                else {
                    router.go(this.defaultView);
                }
            }
        },

        _toggleEnabledOptions: function(actionString) {
            var sitesDisabled = actionString.indexOf(config.intents.view.synonyms) !== -1 ||
                actionString.indexOf(config.intents.view.connectors) !== -1 ||
                actionString.indexOf(config.intents.view.boosting) !== -1 ||
                actionString.indexOf(config.intents.view.index) !== -1 ||
                actionString.indexOf(config.intents.view.explore) !== -1 ||
                actionString.indexOf(config.intents.view.servicehealth) !== -1;
            this.siteMenuModel.set("sitesDisabled", sitesDisabled);

            var languagesDisabled = actionString.indexOf(config.intents.view.connectors) !== -1 ||
                actionString.indexOf(config.intents.view.boosting) !== -1 ||
                actionString.indexOf(config.intents.view.index) !== -1 ||
                actionString.indexOf(config.intents.view.overview) !== -1 ||
                actionString.indexOf(config.intents.view.explore) !== -1 ||
                actionString.indexOf(config.intents.view.servicehealth) !== -1;
            this.siteMenuModel.set("languagesDisabled", languagesDisabled);
        },

        getActionFromPath: function(hash) {
            var result = {};

            var questionMarkPosition = hash.indexOf("?") === -1 ? hash.length : hash.indexOf("?"),
                rawQuery = hash.substring(questionMarkPosition + 1, hash.length);

            result.actions = hash.substring(0, questionMarkPosition).split("/");
            result.params = this._queryToParams(rawQuery);

            return result;
        },

        _queryToParams: function(rawQuery) {
            var result = {};
            if (rawQuery.length === 0) {
                return result;
            }
            var pairs = rawQuery.split("&");
            for(var i in pairs) {
                var keyvalue = pairs[i].split("=");
                result[keyvalue[0]] = keyvalue[1];
            }
            return result;
        },

        _activateView: function (context, action) {
            var view;
            for(var controllerName in this.views) {
                if (controllerName === context) {
                    view = this.views[controllerName];
                    break;
                }
            }
            if (!view) {
                return false;
            }

            var activeView = this.activeView;
            if (view !== activeView) {
                this._deactivateView(activeView);
            }
            this.activeView = view;

            if (view.name) {
                this._setActiveMenuItem(view.name);
            }

            if (view.controller) {
                view.controller.startup();
                view.controller.takeAction(action.actions, action.params);
                view.controller.show();

                // toggle enabled options using resolved current actions
                if (view.controller.currentAction) {
                    this._toggleEnabledOptions(view.name + "/" + view.controller.currentAction);
                }
                else {
                    // trying to toggle enabled options using original action values if current action is unknown
                    var actions = action && action.actions && action.actions.length > 0 ?
                        "/" + action.actions.join("/") :
                        "";
                    this._toggleEnabledOptions(view.name + actions);
                }
            }
            this.siteMenu.refresh();

            return true;
        },

        _refreshView: function() {
            if (this.activeView) {
                this.activeView.controller.refresh();
            }
        },

        _deactivateView: function (view) {
            this._setActiveMenuItem(null);
        },

        _setActiveMenuItem: function (viewName) {
            // summary: highlights active menu item in the global menu since the menu itself does not support hash tag based navigation
            var subMenu = query("#global_find_sub");
            var menuItems = query("li > a", subMenu[0]);
            menuItems.forEach(function (node) {
                var currentItemUrl = domAttr.get(node, "href");
                if (viewName && currentItemUrl.indexOf("#" + viewName) !== -1) {
                    domClass.add(node.parentNode, "epi-navigation-selected");
                } else {
                    domClass.remove(node.parentNode, "epi-navigation-selected");
                }
            });
        }
    });

    var application = new Application();
    application.startup();
    return application;
});

},
'dojo/router':function(){
define("dojo/router", [
	"./router/RouterBase"
], function(RouterBase){

	// module:
	//		dojo/router

/*=====
return {
	// summary:
	//		A singleton-style instance of dojo/router/RouterBase. See that
	//		module for specifics.
	// example:
	//	|	router.register("/widgets/:id", function(evt){
	//	|		// If "/widgets/3" was matched,
	//	|		// evt.params.id === "3"
	//	|		xhr.get({
	//	|			url: "/some/path/" + evt.params.id,
	//	|			load: function(data){
	//	|				// ...
	//	|			}
	//	|		});
	//	|	});
};
=====*/

	return new RouterBase({});
});

},
'dojo/router/RouterBase':function(){
define("dojo/router/RouterBase", [
	"dojo/_base/declare",
	"dojo/hash",
	"dojo/topic"
], function(declare, hash, topic){

	// module:
	//		dojo/router/RouterBase

	// Creating a basic trim to avoid needing the full dojo/string module
	// similarly to dojo/_base/lang's trim
	var trim;
	if(String.prototype.trim){
		trim = function(str){ return str.trim(); };
	}else{
		trim = function(str){ return str.replace(/^\s\s*/, '').replace(/\s\s*$/, ''); };
	}

	// Firing of routes on the route object is always the same,
	// no clean way to expose this on the prototype since it's for the
	// internal router objects.
	function fireRoute(params, currentPath, newPath){
		var queue, isStopped, isPrevented, eventObj, i, l;

		queue = this.callbackQueue;
		isStopped = false;
		isPrevented = false;
		eventObj = {
			stopImmediatePropagation: function(){ isStopped = true; },
			preventDefault: function(){ isPrevented = true; },
			oldPath: currentPath,
			newPath: newPath,
			params: params
		};

		for(i=0, l=queue.length; i<l; ++i){
			if(!isStopped){
				queue[i](eventObj);
			}
		}

		return !isPrevented;
	}

	// Our actual class-like object
	var RouterBase = declare(null, {
		// summary:
		//		A module that allows one to easily map hash-based structures into
		//		callbacks. The router module is a singleton, offering one central
		//		point for all registrations of this type.
		// example:
		//	|	var router = new RouterBase({});
		//	|	router.register("/widgets/:id", function(evt){
		//	|		// If "/widgets/3" was matched,
		//	|		// evt.params.id === "3"
		//	|		xhr.get({
		//	|			url: "/some/path/" + evt.params.id,
		//	|			load: function(data){
		//	|				// ...
		//	|			}
		//	|		});
		//	|	});

		_routes: null,
		_routeIndex: null,
		_started: false,
		_currentPath: "",

		idMatch: /:(\w[\w\d]*)/g,
		idReplacement: "([^\\/]+)",
		globMatch: /\*(\w[\w\d]*)/,
		globReplacement: "(.+)",

		constructor: function(kwArgs){
			// A couple of safety initializations
			this._routes = [];
			this._routeIndex = {};

			// Simple constructor-style "Decorate myself all over" for now
			for(var i in kwArgs){
				if(kwArgs.hasOwnProperty(i)){
					this[i] = kwArgs[i];
				}
			}
		},

		register: function(/*String|RegExp*/ route, /*Function*/ callback){
			// summary:
			//		Registers a route to a handling callback
			// description:
			//		Given either a string or a regular expression, the router
			//		will monitor the page's hash and respond to changes that
			//		match the string or regex as provided.
			//
			//		When provided a regex for the route:
			//
			//		- Matching is performed, and the resulting capture groups
			//		are passed through to the callback as an array.
			//
			//		When provided a string for the route:
			//
			//		- The string is parsed as a URL-like structure, like
			//		"/foo/bar"
			//		- If any portions of that URL are prefixed with a colon
			//		(:), they will be parsed out and provided to the callback
			//		as properties of an object.
			//		- If the last piece of the URL-like structure is prefixed
			//		with a star (*) instead of a colon, it will be replaced in
			//		the resulting regex with a greedy (.+) match and
			//		anything remaining on the hash will be provided as a
			//		property on the object passed into the callback. Think of
			//		it like a basic means of globbing the end of a route.
			// example:
			//	|	router.register("/foo/:bar/*baz", function(object){
			//	|		// If the hash was "/foo/abc/def/ghi",
			//	|		// object.bar === "abc"
			//	|		// object.baz === "def/ghi"
			//	|	});
			// returns: Object
			//		A plain JavaScript object to be used as a handle for
			//		either removing this specific callback's registration, as
			//		well as to add new callbacks with the same route initially
			//		used.
			// route: String|RegExp
			//		A string or regular expression which will be used when
			//		monitoring hash changes.
			// callback: Function
			//		When the hash matches a pattern as described in the route,
			//		this callback will be executed. It will receive an event
			//		object that will have several properties:
			//
			//		- params: Either an array or object of properties pulled
			//		from the new hash
			//		- oldPath: The hash in its state before the change
			//		- newPath: The new hash being shifted to
			//		- preventDefault: A method that will stop hash changes
			//		from being actually applied to the active hash. This only
			//		works if the hash change was initiated using `router.go`,
			//		as changes initiated more directly to the location.hash
			//		property will already be in place
			//		- stopImmediatePropagation: When called, will stop any
			//		further bound callbacks on this particular route from
			//		being executed. If two distinct routes are bound that are
			//		different, but both happen to match the current hash in
			//		some way, this will *not* keep other routes from receiving
			//		notice of the change.

			return this._registerRoute(route, callback);
		},

		registerBefore: function(/*String|RegExp*/ route, /*Function*/ callback){
			// summary:
			//		Registers a route to a handling callback, except before
			//		any previously registered callbacks
			// description:
			//		Much like the `register` method, `registerBefore` allows
			//		us to register route callbacks to happen before any
			//		previously registered callbacks. See the documentation for
			//		`register` for more details and examples.

			return this._registerRoute(route, callback, true);
		},

		go: function(path, replace){
			// summary:
			//		A simple pass-through to make changing the hash easy,
			//		without having to require dojo/hash directly. It also
			//		synchronously fires off any routes that match.
			// example:
			//	|	router.go("/foo/bar");

			var applyChange;

			path = trim(path);
			applyChange = this._handlePathChange(path);

			if(applyChange){
				hash(path, replace);
			}

			return applyChange;
		},

		startup: function(){
			// summary:
			//		This method must be called to activate the router. Until
			//		startup is called, no hash changes will trigger route
			//		callbacks.

			if(this._started){ return; }

			var self = this;

			this._started = true;
			this._handlePathChange(hash());
			topic.subscribe("/dojo/hashchange", function(){
				// No need to load all of lang for just this
				self._handlePathChange.apply(self, arguments);
			});
		},

		_handlePathChange: function(newPath){
			var i, j, li, lj, routeObj, result,
				allowChange, parameterNames, params,
				routes = this._routes,
				currentPath = this._currentPath;

			if(!this._started || newPath === currentPath){ return allowChange; }

			allowChange = true;

			for(i=0, li=routes.length; i<li; ++i){
				routeObj = routes[i];
				result = routeObj.route.exec(newPath);

				if(result){
					if(routeObj.parameterNames){
						parameterNames = routeObj.parameterNames;
						params = {};

						for(j=0, lj=parameterNames.length; j<lj; ++j){
							params[parameterNames[j]] = result[j+1];
						}
					}else{
						params = result.slice(1);
					}
					allowChange = routeObj.fire(params, currentPath, newPath);
				}
			}

			if(allowChange){
				this._currentPath = newPath;
			}

			return allowChange;
		},

		_convertRouteToRegExp: function(route){
			// Sub in based on IDs and globs
			route = route.replace(this.idMatch, this.idReplacement);
			route = route.replace(this.globMatch, this.globReplacement);
			// Make sure it's an exact match
			route = "^" + route + "$";

			return new RegExp(route);
		},

		_getParameterNames: function(route){
			var idMatch = this.idMatch,
				globMatch = this.globMatch,
				parameterNames = [], match;

			idMatch.lastIndex = 0;

			while((match = idMatch.exec(route)) !== null){
				parameterNames.push(match[1]);
			}
			if((match = globMatch.exec(route)) !== null){
				parameterNames.push(match[1]);
			}

			return parameterNames.length > 0 ? parameterNames : null;
		},

		_indexRoutes: function(){
			var i, l, route, routeIndex, routes = this._routes;

			// Start a new route index
			routeIndex = this._routeIndex = {};

			// Set it up again
			for(i=0, l=routes.length; i<l; ++i){
				route = routes[i];
				routeIndex[route.route] = i;
			}
		},

		_registerRoute: function(/*String|RegExp*/route, /*Function*/callback, /*Boolean?*/isBefore){
			var index, exists, routeObj, callbackQueue, removed,
				self = this, routes = this._routes,
				routeIndex = this._routeIndex;

			// Try to fetch the route if it already exists.
			// This works thanks to stringifying of regex
			index = this._routeIndex[route];
			exists = typeof index !== "undefined";
			if(exists){
				routeObj = routes[index];
			}

			// If we didn't get one, make a default start point
			if(!routeObj){
				routeObj = {
					route: route,
					callbackQueue: [],
					fire: fireRoute
				};
			}

			callbackQueue = routeObj.callbackQueue;

			if(typeof route == "string"){
				routeObj.parameterNames = this._getParameterNames(route);
				routeObj.route = this._convertRouteToRegExp(route);
			}

			if(isBefore){
				callbackQueue.unshift(callback);
			}else{
				callbackQueue.push(callback);
			}

			if(!exists){
				index = routes.length;
				routeIndex[route] = index;
				routes.push(routeObj);
			}

			// Useful in a moment to keep from re-removing routes
			removed = false;

			return { // Object
				remove: function(){
					var i, l;

					if(removed){ return; }

					for(i=0, l=callbackQueue.length; i<l; ++i){
						if(callbackQueue[i] === callback){
							callbackQueue.splice(i, 1);
						}
					}


					if(callbackQueue.length === 0){
						routes.splice(index, 1);
						self._indexRoutes();
					}

					removed = true;
				},
				register: function(callback, isBefore){
					return self.register(route, callback, isBefore);
				}
			};
		}
	});

	return RouterBase;
});

},
'dojo/hash':function(){
define("dojo/hash", ["./_base/kernel", "require", "./_base/config", "./_base/connect", "./_base/lang", "./ready", "./sniff"],
	function(dojo, require, config, connect, lang, ready, has){

	// module:
	//		dojo/hash

	dojo.hash = function(/* String? */ hash, /* Boolean? */ replace){
		// summary:
		//		Gets or sets the hash string in the browser URL.
		// description:
		//		Handles getting and setting of location.hash.
		//
		//		 - If no arguments are passed, acts as a getter.
		//		 - If a string is passed, acts as a setter.
		// hash:
		//		the hash is set - #string.
		// replace:
		//		If true, updates the hash value in the current history
		//		state instead of creating a new history state.
		// returns:
		//		when used as a getter, returns the current hash string.
		//		when used as a setter, returns the new hash string.
		// example:
		//	|	topic.subscribe("/dojo/hashchange", context, callback);
		//	|
		//	|	function callback (hashValue){
		//	|		// do something based on the hash value.
		//	|	}

		// getter
		if(!arguments.length){
			return _getHash();
		}
		// setter
		if(hash.charAt(0) == "#"){
			hash = hash.substring(1);
		}
		if(replace){
			_replace(hash);
		}else{
			location.href = "#" + hash;
		}
		return hash; // String
	};

	// Global vars
	var _recentHash, _ieUriMonitor, _connect,
		_pollFrequency = config.hashPollFrequency || 100;

	//Internal functions
	function _getSegment(str, delimiter){
		var i = str.indexOf(delimiter);
		return (i >= 0) ? str.substring(i+1) : "";
	}

	function _getHash(){
		return _getSegment(location.href, "#");
	}

	function _dispatchEvent(){
		connect.publish("/dojo/hashchange", [_getHash()]);
	}

	function _pollLocation(){
		if(_getHash() === _recentHash){
			return;
		}
		_recentHash = _getHash();
		_dispatchEvent();
	}

	function _replace(hash){
		if(_ieUriMonitor){
			if(_ieUriMonitor.isTransitioning()){
				setTimeout(lang.hitch(null,_replace,hash), _pollFrequency);
				return;
			}
			var href = _ieUriMonitor.iframe.location.href;
			var index = href.indexOf('?');
			// main frame will detect and update itself
			_ieUriMonitor.iframe.location.replace(href.substring(0, index) + "?" + hash);
			return;
		}
		location.replace("#"+hash);
		!_connect && _pollLocation();
	}

	function IEUriMonitor(){
		// summary:
		//		Determine if the browser's URI has changed or if the user has pressed the
		//		back or forward button. If so, call _dispatchEvent.
		//
		// description:
		//		IE doesn't add changes to the URI's hash into the history unless the hash
		//		value corresponds to an actual named anchor in the document. To get around
		//		this IE difference, we use a background IFrame to maintain a back-forward
		//		history, by updating the IFrame's query string to correspond to the
		//		value of the main browser location's hash value.
		//
		//		E.g. if the value of the browser window's location changes to
		//
		//		#action=someAction
		//
		//		... then we'd update the IFrame's source to:
		//
		//		?action=someAction
		//
		//		This design leads to a somewhat complex state machine, which is
		//		described below:
		//
		//		####s1
		//
		//		Stable state - neither the window's location has changed nor
		//		has the IFrame's location. Note that this is the 99.9% case, so
		//		we optimize for it.
		//
		//		Transitions: s1, s2, s3
		//
		//		####s2
		//
		//		Window's location changed - when a user clicks a hyperlink or
		//		code programmatically changes the window's URI.
		//
		//		Transitions: s4
		//
		//		####s3
		//
		//		Iframe's location changed as a result of user pressing back or
		//		forward - when the user presses back or forward, the location of
		//		the background's iframe changes to the previous or next value in
		//		its history.
		//
		//		Transitions: s1
		//
		//		####s4
		//
		//		IEUriMonitor has programmatically changed the location of the
		//		background iframe, but it's location hasn't yet changed. In this
		//		case we do nothing because we need to wait for the iframe's
		//		location to reflect its actual state.
		//
		//		Transitions: s4, s5
		//
		//		####s5
		//
		//		IEUriMonitor has programmatically changed the location of the
		//		background iframe, and the iframe's location has caught up with
		//		reality. In this case we need to transition to s1.
		//
		//		Transitions: s1
		//
		//		The hashchange event is always dispatched on the transition back to s1.


		// create and append iframe
		var ifr = document.createElement("iframe"),
			IFRAME_ID = "dojo-hash-iframe",
			ifrSrc = config.dojoBlankHtmlUrl || require.toUrl("./resources/blank.html");

		if(config.useXDomain && !config.dojoBlankHtmlUrl){
			console.warn("dojo.hash: When using cross-domain Dojo builds,"
				+ " please save dojo/resources/blank.html to your domain and set djConfig.dojoBlankHtmlUrl"
				+ " to the path on your domain to blank.html");
		}

		ifr.id = IFRAME_ID;
		ifr.src = ifrSrc + "?" + _getHash();
		ifr.style.display = "none";
		document.body.appendChild(ifr);

		this.iframe = dojo.global[IFRAME_ID];
		var recentIframeQuery, transitioning, expectedIFrameQuery, docTitle, ifrOffline,
			iframeLoc = this.iframe.location;

		function resetState(){
			_recentHash = _getHash();
			recentIframeQuery = ifrOffline ? _recentHash : _getSegment(iframeLoc.href, "?");
			transitioning = false;
			expectedIFrameQuery = null;
		}

		this.isTransitioning = function(){
			return transitioning;
		};

		this.pollLocation = function(){
			if(!ifrOffline){
				try{
					//see if we can access the iframe's location without a permission denied error
					var iframeSearch = _getSegment(iframeLoc.href, "?");
					//good, the iframe is same origin (no thrown exception)
					if(document.title != docTitle){ //sync title of main window with title of iframe.
						docTitle = this.iframe.document.title = document.title;
					}
				}catch(e){
					//permission denied - server cannot be reached.
					ifrOffline = true;
					console.error("dojo.hash: Error adding history entry. Server unreachable.");
				}
			}
			var hash = _getHash();
			if(transitioning && _recentHash === hash){
				// we're in an iframe transition (s4 or s5)
				if(ifrOffline || iframeSearch === expectedIFrameQuery){
					// s5 (iframe caught up to main window or iframe offline), transition back to s1
					resetState();
					_dispatchEvent();
				}else{
					// s4 (waiting for iframe to catch up to main window)
					setTimeout(lang.hitch(this,this.pollLocation),0);
					return;
				}
			}else if(_recentHash === hash && (ifrOffline || recentIframeQuery === iframeSearch)){
				// we're in stable state (s1, iframe query == main window hash), do nothing
			}else{
				// the user has initiated a URL change somehow.
				// sync iframe query <-> main window hash
				if(_recentHash !== hash){
					// s2 (main window location changed), set iframe url and transition to s4
					_recentHash = hash;
					transitioning = true;
					expectedIFrameQuery = hash;
					ifr.src = ifrSrc + "?" + expectedIFrameQuery;
					ifrOffline = false; //we're updating the iframe src - set offline to false so we can check again on next poll.
					setTimeout(lang.hitch(this,this.pollLocation),0); //yielded transition to s4 while iframe reloads.
					return;
				}else if(!ifrOffline){
					// s3 (iframe location changed via back/forward button), set main window url and transition to s1.
					location.href = "#" + iframeLoc.search.substring(1);
					resetState();
					_dispatchEvent();
				}
			}
			setTimeout(lang.hitch(this,this.pollLocation), _pollFrequency);
		};
		resetState(); // initialize state (transition to s1)
		setTimeout(lang.hitch(this,this.pollLocation), _pollFrequency);
	}
	ready(function(){
		if("onhashchange" in dojo.global && (!has("ie") || (has("ie") >= 8 && document.compatMode != "BackCompat"))){	//need this IE browser test because "onhashchange" exists in IE8 in IE7 mode
			_connect = connect.connect(dojo.global,"onhashchange",_dispatchEvent);
		}else{
			if(document.addEventListener){ // Non-IE
				_recentHash = _getHash();
				setInterval(_pollLocation, _pollFrequency); //Poll the window location for changes
			}else if(document.attachEvent){ // IE7-
				//Use hidden iframe in versions of IE that don't have onhashchange event
				_ieUriMonitor = new IEUriMonitor();
			}
			// else non-supported browser, do nothing.
		}
	});

	return dojo.hash;

});

},
'dojox/mvc/at':function(){
define("dojox/mvc/at", [
	"dojo/_base/kernel",
	"dojo/_base/lang",
	"./sync",
	"./_atBindingExtension"
], function(kernel, lang, sync){

	kernel.experimental("dojox.mvc");

	var at = function(/*dojo/Stateful|String*/ target, /*String*/ targetProp){
		// summary:
		//		Returns a pointer to data binding target (a dojo/Stateful property), called at handle, which is used for start synchronization with data binding source (another dojo/Stateful property).
		// description:
		//		Typically used in data-dojo-props so that a widget can synchronize its attribute with another dojo/Stateful, like shown in the example.
		// target: dojo/Stateful|String
		//		dojo/Stateful to be synchronized.
		// targetProp: String
		//		The property name in target to be synchronized.
		// returns:
		//		A pointer to data binding target (a dojo/Stateful property), called at handle, which is used for start synchronization with data binding source (another dojo/Stateful property).
		// example:
		//		Two seconds later, the text box changes from "Foo" to "Bar" as the "value" property in model changes.
		// |		<html>
		// |			<head>
		// |				<script src="/path/to/dojo-toolkit/dojo/dojo.js" type="text/javascript" data-dojo-config="parseOnLoad: 0"></script>
		// |				<script type="text/javascript">
		// |					require([
		// |						"dojo/parser", "dojo/Stateful", "dijit/form/TextBox", "dojo/domReady!"
		// |					], function(parser, Stateful){
		// |						model = new Stateful({value: "Foo"});
		// |						setTimeout(function(){ model.set("value", "Bar"); }, 2000);
		// |						parser.parse();
		// |					});
		// |				</script>
		// |			</head>
		// |			<body>
		// |				<script type="dojo/require">at: "dojox/mvc/at"</script>
		// |				<input type="text" data-dojo-type="dijit/form/TextBox" data-dojo-props="value: at(model, 'value')">
		// |			</body>
		// |		</html>
		// example:
		//		Edit in text box is reflected to the text next to it.
		// |		<html>
		// |			<head>
		// |				<script src="/path/to/dojo-toolkit/dojo/dojo.js" type="text/javascript" data-dojo-config="parseOnLoad: 0"></script>
		// |				<script type="text/javascript">
		// |					require([
		// |						"dojo/parser", "dojo/Stateful", "dojo/domReady!"
		// |					], function(parser, Stateful){
		// |						model = new Stateful({value: "Foo"});
		// |						parser.parse();
		// |					});
		// |				</script>
		// |			</head>
		// |			<body>
		// |				<script type="dojo/require">at: "dojox/mvc/at"</script>
		// |				<input type="text" data-dojo-type="dijit/form/TextBox" data-dojo-props="value: at(model, 'value')">
		// |				<span data-dojo-type="dijit/_WidgetBase" data-dojo-props="_setValueAttr: {node: 'domNode', type: 'innerText'}, value: at(model, 'value')"></span>
		// |			</body>
		// |		</html>

		return { // dojox/mvc/at.handle
			atsignature: "dojox.mvc.at",
			target: target,
			targetProp: targetProp,
			bindDirection: sync.both,
			direction: function(/*Number*/ bindDirection){
				this.bindDirection = bindDirection;
				return this;
			},
			transform: function(/*dojox/mvc/sync.converter*/ converter){
				this.converter = converter;
				return this;
			}
		};
	};

	/*=====
	at.handle = {
		// summary:
		//		A handle of data binding target (a dojo/Stateful property), which is used for start synchronization with data binding source (another dojo/Stateful property).

		// target: dojo/Stateful|String
		//		The data binding literal or dojo/Stateful to be synchronized.
		target: new dojo/Stateful(),

		// targetProp: String
		//		The property name in target to be synchronized.
		targetProp: "",

		// bindDirection: Number
		//		The data binding bindDirection, choose from: dojox/mvc/sync.from, dojox/mvc/sync.to or dojox/mvc/sync.both.
		bindDirection: dojox/mvc/sync.both,

		// converter: dojox/mvc/sync.converter
		//		Class/object containing the converter functions used when the data goes between data binding target (e.g. data model or controller) to data binding origin (e.g. widget).
		converter: null,

		direction: function(bindDirection){
			// summary:
			//		Sets data binding bindDirection.
			// bindDirection: Number
			//		The data binding bindDirection, choose from: dojox/mvc/sync.from, dojox/mvc/sync.to or dojox/mvc/sync.both.
		},

		transform: function(converter){
			// summary:
			//		Attach a data converter.
			// converter: dojox/mvc/sync.converter
			//		Class/object containing the converter functions used when the data goes between data binding target (e.g. data model or controller) to data binding origin (e.g. widget).
		}
	};
	=====*/

	// Data binding bindDirections
	at.from = sync.from;
	at.to = sync.to;
	at.both = sync.both;

	// lang.setObject() thing is for back-compat, remove it in 2.0
	return lang.setObject("dojox.mvc.at", at);
});

},
'dojox/mvc/sync':function(){
define("dojox/mvc/sync", [
	"dojo/_base/lang",
	"dojo/_base/config",
	"dojo/_base/array",
	"dojo/has"
], function(lang, config, array, has){
	var mvc = lang.getObject("dojox.mvc", true);
	/*=====
	mvc = {};
	=====*/
	
	/*=====
	dojox.mvc.sync.converter = {
		// summary:
		//		Class/object containing the converter functions used when the data goes between data binding source (e.g. data model or controller) to data binding origin (e.g. widget).

		format: function(value, constraints){
			// summary:
			//		The converter function used when the data comes from data binding source (e.g. data model or controller) to data binding origin (e.g. widget).
			// value: Anything
			//		The data.
			// constraints: Object
			//		The options for data conversion, which is: mixin({}, dataBindingTarget.constraints, dataBindingOrigin.constraints).
		},

		parse: function(value, constraints){
			// summary:
			//		The converter function used when the data comes from data binding origin (e.g. widget) to data binding source (e.g. data model or controller).
			// value: Anything
			//		The data.
			// constraints: Object
			//		The options for data conversion, which is: mixin({}, dataBindingTarget.constraints, dataBindingOrigin.constraints).
		}
	};

	dojox.mvc.sync.options = {
		// summary:
		//		Data binding options.

		// bindDirection: Number
		//		The data binding bindDirection, choose from: dojox.mvc.Bind.from, dojox.mvc.Bind.to or dojox.mvc.Bind.both.
		bindDirection: dojox/mvc.both,

		// converter: dojox/mvc/sync.converter
		//		Class/object containing the converter functions used when the data goes between data binding source (e.g. data model or controller) to data binding origin (e.g. widget).
		converter: null
	};
	=====*/

	has.add("mvc-bindings-log-api", (config["mvc"] || {}).debugBindings);

	var sync;

	if(has("mvc-bindings-log-api")){
		function getLogContent(/*dojo/Stateful*/ source, /*String*/ sourceProp, /*dojo/Stateful*/ target, /*String*/ targetProp){
			return [
				[target._setIdAttr || !target.declaredClass ? target : target.declaredClass, targetProp].join(":"),
				[source._setIdAttr || !source.declaredClass ? source : source.declaredClass, sourceProp].join(":")
			];
		}
	}

	function equals(/*Anything*/ dst, /*Anything*/ src){
		// summary:
		//		Returns if the given two values are equal.

		return dst === src
		 || typeof dst == "number" && isNaN(dst) && typeof src == "number" && isNaN(src)
		 || lang.isFunction((dst || {}).getTime) && lang.isFunction((src || {}).getTime) && dst.getTime() == src.getTime()
		 || (lang.isFunction((dst || {}).equals) ? dst.equals(src) : lang.isFunction((src || {}).equals) ? src.equals(dst) : false);
	}

	function copy(/*Function*/ convertFunc, /*Object?*/ constraints, /*dojo/Stateful*/ source, /*String*/ sourceProp, /*dojo/Stateful*/ target, /*String*/ targetProp, /*Anything*/ old, /*Anything*/ current, /*Object?*/ excludes){
		// summary:
		//		Watch for change in property in dojo/Stateful object.
		// description:
		//		Called when targetProp property in target is changed. (This is mainly used as a callback function of dojo/Stateful.watch())
		//		When older value and newer value are different, copies the newer value to sourceProp property in source.
		// convertFunc: Function
		//		The data converter function.
		// constraints: Object?
		//		The data converter options.
		// source: dojo/Stateful
		//		The dojo/Stateful of copy source.
		// sourceProp: String
		//		The property of copy source, specified in data binding. May be wildcarded.
		// target: dojo/Stateful
		//		The dojo/Stateful of copy target.
		// targetProp: String
		//		The property of copy target, being changed. For wildcard-based data binding, this is used as the property to be copied.
		// old: Anything
		//		The older property value.
		// current: Anything
		//		The newer property value.
		// excludes: Object?
		//		The list of properties that should be excluded from wildcarded data binding.

		// Bail if there is no change in value,
		// or property name is wildcarded and the property to be copied is not in source property list (and source property list is defined),
		// or property name is wildcarded and the property to be copied is in explicit "excludes" list
		if(sync.equals(current, old)
		 || sourceProp == "*" && array.indexOf(source.get("properties") || [targetProp], targetProp) < 0
		 || sourceProp == "*" && targetProp in (excludes || {})){ return; }

		var prop = sourceProp == "*" ? targetProp : sourceProp;
		if(has("mvc-bindings-log-api")){
			var logContent = getLogContent(source, prop, target, targetProp);
		}

		try{
			current = convertFunc ? convertFunc(current, constraints) : current;
		}catch(e){
			if(has("mvc-bindings-log-api")){
				console.log("Copy from" + logContent.join(" to ") + " was not done as an error is thrown in the converter.");
			}
			return;
		}

		if(has("mvc-bindings-log-api")){
			console.log(logContent.reverse().join(" is being copied from: ") + " (Value: " + current + " from " + old + ")");
		}

		// Copy the new value to source
		lang.isFunction(source.set) ? source.set(prop, current) : (source[prop] = current);
	}

	var directions = {
		// from: Number
		//		Data binding goes from the source to the target
		from: 1,

		// to: Number
		//		Data binding goes from the target to the source
		to: 2,

		// both: Number
		//		Data binding goes in both directions (dojox/mvc/Bind.from | dojox/mvc/Bind.to)
		both: 3
	}, undef;

	sync = function(/*dojo/Stateful*/ source, /*String*/ sourceProp, /*dojo/Stateful*/ target, /*String*/ targetProp, /*dojox/mvc/sync.options*/ options){
		// summary:
		//		Synchronize two dojo/Stateful properties.
		// description:
		//		Synchronize two dojo/Stateful properties.
		// source: dojo/Stateful
		//		Source dojo/Stateful to be synchronized.
		// sourceProp: String
		//		The property name in source to be synchronized.
		// target: dojo/Stateful
		//		Target dojo/Stateful to be synchronized.
		// targetProp: String
		//		The property name in target to be synchronized.
		// options: dojox/mvc/sync.options
		//		Data binding options.
		// returns:
		//		The handle of data binding synchronization.

		var converter = (options || {}).converter, converterInstance, formatFunc, parseFunc;
		if(converter){
			converterInstance = {source: source, target: target};
			formatFunc = converter.format && lang.hitch(converterInstance, converter.format);
			parseFunc = converter.parse && lang.hitch(converterInstance, converter.parse);
		}

		var _watchHandles = [],
		 excludes = [],
		 list,
		 constraints = lang.mixin({}, source.constraints, target.constraints),
		 bindDirection = (options || {}).bindDirection || mvc.both;

		if(has("mvc-bindings-log-api")){
			var logContent = getLogContent(source, sourceProp, target, targetProp);
		}

		if(targetProp == "*"){
			if(sourceProp != "*"){ throw new Error("Unmatched wildcard is specified between source and target."); }
			list = target.get("properties");
			if(!list){
				list = [];
				for(var s in target){ if(target.hasOwnProperty(s) && s != "_watchCallbacks"){ list.push(s); } }
			}
			excludes = target.get("excludes");
		}else{
			list = [sourceProp];
		}

		if(bindDirection & mvc.from){
			// Start synchronization from source to target (e.g. from model to widget). For wildcard mode (sourceProp == targetProp == "*"), the 1st argument of watch() is omitted
			if(lang.isFunction(source.set) && lang.isFunction(source.watch)){
				_watchHandles.push(source.watch.apply(source, ((sourceProp != "*") ? [sourceProp] : []).concat([function(name, old, current){
					copy(formatFunc, constraints, target, targetProp, source, name, old, current, excludes);
				}])));
			}else if(has("mvc-bindings-log-api")){
				console.log(logContent.reverse().join(" is not a stateful property. Its change is not reflected to ") + ".");
			}

			// Initial copy from source to target (e.g. from model to widget)
			array.forEach(list, function(prop){
				// In "all properties synchronization" case, copy is not done for properties in "exclude" list
				if(targetProp != "*" || !(prop in (excludes || {}))){
					var value = lang.isFunction(source.get) ? source.get(prop) : source[prop];
					copy(formatFunc, constraints, target, targetProp == "*" ? prop : targetProp, source, prop, undef, value);
				}
			});
		}

		if(bindDirection & mvc.to){
			if(!(bindDirection & mvc.from)){
				// Initial copy from source to target (e.g. from model to widget)
				array.forEach(list, function(prop){
					// In "all properties synchronization" case, copy is not done for properties in "exclude" list
					if(targetProp != "*" || !(prop in (excludes || {}))){
						// Initial copy from target to source (e.g. from widget to model), only done for one-way binding from widget to model
						var value = lang.isFunction(target.get) ? target.get(targetProp) : target[targetProp];
						copy(parseFunc, constraints, source, prop, target, targetProp == "*" ? prop : targetProp, undef, value);
					}
				});
			}

			// Start synchronization from target to source (e.g. from widget to model). For wildcard mode (sourceProp == targetProp == "*"), the 1st argument of watch() is omitted
			if(lang.isFunction(target.set) && lang.isFunction(target.watch)){
				_watchHandles.push(target.watch.apply(target, ((targetProp != "*") ? [targetProp] : []).concat([function(name, old, current){
					copy(parseFunc, constraints, source, sourceProp, target, name, old, current, excludes);
				}])));
			}else if(has("mvc-bindings-log-api")){
				console.log(logContent.join(" is not a stateful property. Its change is not reflected to ") + ".");
			}
		}

		if(has("mvc-bindings-log-api")){
			console.log(logContent.join(" is bound to: "));
		}

		var handle = {};
		handle.unwatch = handle.remove = function(){
			for(var h = null; h = _watchHandles.pop();){
				h.unwatch();
				if(has("mvc-bindings-log-api")){
					console.log(logContent.join(" is unbound from: "));
				}
			}
		};
		return handle; // dojo/handle
	};

	lang.mixin(mvc, directions);

	// lang.setObject() thing is for back-compat, remove it in 2.0
	return lang.setObject("dojox.mvc.sync", lang.mixin(sync, {equals: equals}, directions));
});

},
'dojox/mvc/_atBindingExtension':function(){
define("dojox/mvc/_atBindingExtension", [
	"dojo/aspect",
	"dojo/_base/lang",
	"dijit/_WidgetBase",
	"./_atBindingMixin",
	"dijit/registry"
], function(aspect, lang, _WidgetBase, _atBindingMixin){
	// Apply the at binding mixin to all dijits, see mixin class description for details.
	// Hiding this from the doc viewer since it's too much to display for every single widget.
	lang.extend(_WidgetBase, /*===== {} || =====*/ _atBindingMixin.prototype);

	// Monkey patch dijit._WidgetBase.postscript to get the list of dojox/mvc/at handles before startup
	aspect.before(_WidgetBase.prototype, "postscript", function(/*Object?*/ params, /*DomNode|String*/ srcNodeRef){
		this._dbpostscript(params, srcNodeRef);
	});

	// Monkey patch dijit._WidgetBase.startup to get data binds set up
	aspect.before(_WidgetBase.prototype, "startup", function(){
		this._startAtWatchHandles();
	});

	// Monkey patch dijit._WidgetBase.destroy to remove watches setup in _DataBindingMixin
	aspect.before(_WidgetBase.prototype, "destroy", function(){
		this._stopAtWatchHandles();
	});

	// Monkey patch dijit._WidgetBase.set to establish data binding if a dojox/mvc/at handle comes
	aspect.around(_WidgetBase.prototype, "set", function(oldWidgetBaseSet){
		return function(/*String*/ name, /*Anything*/ value){
			if(name == _atBindingMixin.prototype.dataBindAttr){
				return this._setBind(value);
			}else if((value || {}).atsignature == "dojox.mvc.at"){
				return this._setAtWatchHandle(name, value);
			}
			return oldWidgetBaseSet.apply(this, lang._toArray(arguments));
		};
	});
});

},
'dijit/_WidgetBase':function(){
define("dijit/_WidgetBase", [
	"require",			// require.toUrl
	"dojo/_base/array", // array.forEach array.map
	"dojo/aspect",
	"dojo/_base/config", // config.blankGif
	"dojo/_base/connect", // connect.connect
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.byId
	"dojo/dom-attr", // domAttr.set domAttr.remove
	"dojo/dom-class", // domClass.add domClass.replace
	"dojo/dom-construct", // domConstruct.destroy domConstruct.place
	"dojo/dom-geometry",	// isBodyLtr
	"dojo/dom-style", // domStyle.set, domStyle.get
	"dojo/has",
	"dojo/_base/kernel",
	"dojo/_base/lang", // mixin(), isArray(), etc.
	"dojo/on",
	"dojo/ready",
	"dojo/Stateful", // Stateful
	"dojo/topic",
	"dojo/_base/window", // win.doc, win.body()
	"./Destroyable",
	"./registry"	// registry.getUniqueId(), registry.findWidgets()
], function(require, array, aspect, config, connect, declare,
			dom, domAttr, domClass, domConstruct, domGeometry, domStyle, has, kernel,
			lang, on, ready, Stateful, topic, win, Destroyable, registry){

// module:
//		dijit/_WidgetBase

// Flag to make dijit load modules the app didn't explicitly request, for backwards compatibility
has.add("dijit-legacy-requires", !kernel.isAsync);

// For back-compat, remove in 2.0.
if(has("dijit-legacy-requires")){
	ready(0, function(){
		var requires = ["dijit/_base/manager"];
		require(requires);	// use indirection so modules not rolled into a build
	});
}

// Nested hash listing attributes for each tag, all strings in lowercase.
// ex: {"div": {"style": true, "tabindex" true}, "form": { ...
var tagAttrs = {};
function getAttrs(obj){
	var ret = {};
	for(var attr in obj){
		ret[attr.toLowerCase()] = true;
	}
	return ret;
}

function nonEmptyAttrToDom(attr){
	// summary:
	//		Returns a setter function that copies the attribute to this.domNode,
	//		or removes the attribute from this.domNode, depending on whether the
	//		value is defined or not.
	return function(val){
		domAttr[val ? "set" : "remove"](this.domNode, attr, val);
		this._set(attr, val);
	};
}

function isEqual(a, b){
	   // summary:
	   //		Function that determines whether two values are identical,
	   //		taking into account that NaN is not normally equal to itself
	   //		in JS.

	   return a === b || (/* a is NaN */ a !== a && /* b is NaN */ b !== b);
}

return declare("dijit._WidgetBase", [Stateful, Destroyable], {
	// summary:
	//		Future base class for all Dijit widgets.
	// description:
	//		Future base class for all Dijit widgets.
	//		_Widget extends this class adding support for various features needed by desktop.
	//
	//		Provides stubs for widget lifecycle methods for subclasses to extend, like postMixInProperties(), buildRendering(),
	//		postCreate(), startup(), and destroy(), and also public API methods like set(), get(), and watch().
	//
	//		Widgets can provide custom setters/getters for widget attributes, which are called automatically by set(name, value).
	//		For an attribute XXX, define methods _setXXXAttr() and/or _getXXXAttr().
	//
	//		_setXXXAttr can also be a string/hash/array mapping from a widget attribute XXX to the widget's DOMNodes:
	//
	//		- DOM node attribute
	// |		_setFocusAttr: {node: "focusNode", type: "attribute"}
	// |		_setFocusAttr: "focusNode"	(shorthand)
	// |		_setFocusAttr: ""		(shorthand, maps to this.domNode)
	//		Maps this.focus to this.focusNode.focus, or (last example) this.domNode.focus
	//
	//		- DOM node innerHTML
	//	|		_setTitleAttr: { node: "titleNode", type: "innerHTML" }
	//		Maps this.title to this.titleNode.innerHTML
	//
	//		- DOM node innerText
	//	|		_setTitleAttr: { node: "titleNode", type: "innerText" }
	//		Maps this.title to this.titleNode.innerText
	//
	//		- DOM node CSS class
	// |		_setMyClassAttr: { node: "domNode", type: "class" }
	//		Maps this.myClass to this.domNode.className
	//
	//		If the value of _setXXXAttr is an array, then each element in the array matches one of the
	//		formats of the above list.
	//
	//		If the custom setter is null, no action is performed other than saving the new value
	//		in the widget (in this).
	//
	//		If no custom setter is defined for an attribute, then it will be copied
	//		to this.focusNode (if the widget defines a focusNode), or this.domNode otherwise.
	//		That's only done though for attributes that match DOMNode attributes (title,
	//		alt, aria-labelledby, etc.)

	// id: [const] String
	//		A unique, opaque ID string that can be assigned by users or by the
	//		system. If the developer passes an ID which is known not to be
	//		unique, the specified ID is ignored and the system-generated ID is
	//		used instead.
	id: "",
	_setIdAttr: "domNode",	// to copy to this.domNode even for auto-generated id's

	// lang: [const] String
	//		Rarely used.  Overrides the default Dojo locale used to render this widget,
	//		as defined by the [HTML LANG](http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang) attribute.
	//		Value must be among the list of locales specified during by the Dojo bootstrap,
	//		formatted according to [RFC 3066](http://www.ietf.org/rfc/rfc3066.txt) (like en-us).
	lang: "",
	// set on domNode even when there's a focus node.	but don't set lang="", since that's invalid.
	_setLangAttr: nonEmptyAttrToDom("lang"),

	// dir: [const] String
	//		Bi-directional support, as defined by the [HTML DIR](http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir)
	//		attribute. Either left-to-right "ltr" or right-to-left "rtl".  If undefined, widgets renders in page's
	//		default direction.
	dir: "",
	// set on domNode even when there's a focus node.	but don't set dir="", since that's invalid.
	_setDirAttr: nonEmptyAttrToDom("dir"),	// to set on domNode even when there's a focus node

	// textDir: String
	//		Bi-directional support,	the main variable which is responsible for the direction of the text.
	//		The text direction can be different than the GUI direction by using this parameter in creation
	//		of a widget.
	//
	//		Allowed values:
	//
	//		1. "ltr"
	//		2. "rtl"
	//		3. "auto" - contextual the direction of a text defined by first strong letter.
	//
	//		By default is as the page direction.
	textDir: "",

	// class: String
	//		HTML class attribute
	"class": "",
	_setClassAttr: { node: "domNode", type: "class" },

	// style: String||Object
	//		HTML style attributes as cssText string or name/value hash
	style: "",

	// title: String
	//		HTML title attribute.
	//
	//		For form widgets this specifies a tooltip to display when hovering over
	//		the widget (just like the native HTML title attribute).
	//
	//		For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,
	//		etc., it's used to specify the tab label, accordion pane title, etc.
	title: "",

	// tooltip: String
	//		When this widget's title attribute is used to for a tab label, accordion pane title, etc.,
	//		this specifies the tooltip to appear when the mouse is hovered over that text.
	tooltip: "",

	// baseClass: [protected] String
	//		Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate
	//		widget state.
	baseClass: "",

	// srcNodeRef: [readonly] DomNode
	//		pointer to original DOM node
	srcNodeRef: null,

	// domNode: [readonly] DomNode
	//		This is our visible representation of the widget! Other DOM
	//		Nodes may by assigned to other properties, usually through the
	//		template system's data-dojo-attach-point syntax, but the domNode
	//		property is the canonical "top level" node in widget UI.
	domNode: null,

	// containerNode: [readonly] DomNode
	//		Designates where children of the source DOM node will be placed.
	//		"Children" in this case refers to both DOM nodes and widgets.
	//		For example, for myWidget:
	//
	//		|	<div data-dojo-type=myWidget>
	//		|		<b> here's a plain DOM node
	//		|		<span data-dojo-type=subWidget>and a widget</span>
	//		|		<i> and another plain DOM node </i>
	//		|	</div>
	//
	//		containerNode would point to:
	//
	//		|		<b> here's a plain DOM node
	//		|		<span data-dojo-type=subWidget>and a widget</span>
	//		|		<i> and another plain DOM node </i>
	//
	//		In templated widgets, "containerNode" is set via a
	//		data-dojo-attach-point assignment.
	//
	//		containerNode must be defined for any widget that accepts innerHTML
	//		(like ContentPane or BorderContainer or even Button), and conversely
	//		is null for widgets that don't, like TextBox.
	containerNode: null,

	// ownerDocument: [const] Document?
	//		The document this widget belongs to.  If not specified to constructor, will default to
	//		srcNodeRef.ownerDocument, or if no sourceRef specified, then to dojo/_base/window::doc
	ownerDocument: null,
	_setOwnerDocumentAttr: function(val){
		// this setter is merely to avoid automatically trying to set this.domNode.ownerDocument
		this._set("ownerDocument", val);
	},

/*=====
	// _started: [readonly] Boolean
	//		startup() has completed.
	_started: false,
=====*/

	// attributeMap: [protected] Object
	//		Deprecated.	Instead of attributeMap, widget should have a _setXXXAttr attribute
	//		for each XXX attribute to be mapped to the DOM.
	//
	//		attributeMap sets up a "binding" between attributes (aka properties)
	//		of the widget and the widget's DOM.
	//		Changes to widget attributes listed in attributeMap will be
	//		reflected into the DOM.
	//
	//		For example, calling set('title', 'hello')
	//		on a TitlePane will automatically cause the TitlePane's DOM to update
	//		with the new title.
	//
	//		attributeMap is a hash where the key is an attribute of the widget,
	//		and the value reflects a binding to a:
	//
	//		- DOM node attribute
	// |		focus: {node: "focusNode", type: "attribute"}
	//		Maps this.focus to this.focusNode.focus
	//
	//		- DOM node innerHTML
	//	|		title: { node: "titleNode", type: "innerHTML" }
	//		Maps this.title to this.titleNode.innerHTML
	//
	//		- DOM node innerText
	//	|		title: { node: "titleNode", type: "innerText" }
	//		Maps this.title to this.titleNode.innerText
	//
	//		- DOM node CSS class
	// |		myClass: { node: "domNode", type: "class" }
	//		Maps this.myClass to this.domNode.className
	//
	//		If the value is an array, then each element in the array matches one of the
	//		formats of the above list.
	//
	//		There are also some shorthands for backwards compatibility:
	//
	//		- string --> { node: string, type: "attribute" }, for example:
	//
	//	|	"focusNode" ---> { node: "focusNode", type: "attribute" }
	//
	//		- "" --> { node: "domNode", type: "attribute" }
	attributeMap: {},

	// _blankGif: [protected] String
	//		Path to a blank 1x1 image.
	//		Used by `<img>` nodes in templates that really get their image via CSS background-image.
	_blankGif: config.blankGif || require.toUrl("dojo/resources/blank.gif"),

	//////////// INITIALIZATION METHODS ///////////////////////////////////////

	/*=====
	constructor: function(params, srcNodeRef){
		// summary:
		//		Create the widget.
		// params: Object|null
		//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
		//		and functions, typically callbacks like onClick.
	 	//		The hash can contain any of the widget's properties, excluding read-only properties.
	 	// srcNodeRef: DOMNode|String?
		//		If a srcNodeRef (DOM node) is specified:
		//
		//		- use srcNodeRef.innerHTML as my contents
		//		- if this is a behavioral widget then apply behavior to that srcNodeRef
		//		- otherwise, replace srcNodeRef with my generated DOM tree
	 },
	=====*/

	postscript: function(/*Object?*/params, /*DomNode|String*/srcNodeRef){
		// summary:
		//		Kicks off widget instantiation.  See create() for details.
		// tags:
		//		private
		this.create(params, srcNodeRef);
	},

	create: function(params, srcNodeRef){
		// summary:
		//		Kick off the life-cycle of a widget
		// description:
		//		Create calls a number of widget methods (postMixInProperties, buildRendering, postCreate,
		//		etc.), some of which of you'll want to override. See http://dojotoolkit.org/reference-guide/dijit/_WidgetBase.html
		//		for a discussion of the widget creation lifecycle.
		//
		//		Of course, adventurous developers could override create entirely, but this should
		//		only be done as a last resort.
		// params: Object|null
		//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
		//		and functions, typically callbacks like onClick.
		//		The hash can contain any of the widget's properties, excluding read-only properties.
		// srcNodeRef: DOMNode|String?
		//		If a srcNodeRef (DOM node) is specified:
		//
		//		- use srcNodeRef.innerHTML as my contents
		//		- if this is a behavioral widget then apply behavior to that srcNodeRef
		//		- otherwise, replace srcNodeRef with my generated DOM tree
		// tags:
		//		private

		// store pointer to original DOM tree
		this.srcNodeRef = dom.byId(srcNodeRef);

		// No longer used, remove for 2.0.
		this._connects = [];
		this._supportingWidgets = [];

		// this is here for back-compat, remove in 2.0 (but check NodeList-instantiate.html test)
		if(this.srcNodeRef && (typeof this.srcNodeRef.id == "string")){ this.id = this.srcNodeRef.id; }

		// mix in our passed parameters
		if(params){
			this.params = params;
			lang.mixin(this, params);
		}
		this.postMixInProperties();

		// Generate an id for the widget if one wasn't specified, or it was specified as id: undefined.
		// Do this before buildRendering() because it might expect the id to be there.
		if(!this.id){
			this.id = registry.getUniqueId(this.declaredClass.replace(/\./g,"_"));
			if(this.params){
				// if params contains {id: undefined}, prevent _applyAttributes() from processing it
				delete this.params.id;
			}
		}

		// The document and <body> node this widget is associated with
		this.ownerDocument = this.ownerDocument || (this.srcNodeRef ? this.srcNodeRef.ownerDocument : win.doc);
		this.ownerDocumentBody = win.body(this.ownerDocument);

		registry.add(this);

		this.buildRendering();

		var deleteSrcNodeRef;

		if(this.domNode){
			// Copy attributes listed in attributeMap into the [newly created] DOM for the widget.
			// Also calls custom setters for all attributes with custom setters.
			this._applyAttributes();

			// If srcNodeRef was specified, then swap out original srcNode for this widget's DOM tree.
			// For 2.0, move this after postCreate().  postCreate() shouldn't depend on the
			// widget being attached to the DOM since it isn't when a widget is created programmatically like
			// new MyWidget({}).	See #11635.
			var source = this.srcNodeRef;
			if(source && source.parentNode && this.domNode !== source){
				source.parentNode.replaceChild(this.domNode, source);
				deleteSrcNodeRef = true;
			}

			// Note: for 2.0 may want to rename widgetId to dojo._scopeName + "_widgetId",
			// assuming that dojo._scopeName even exists in 2.0
			this.domNode.setAttribute("widgetId", this.id);
		}
		this.postCreate();

		// If srcNodeRef has been processed and removed from the DOM (e.g. TemplatedWidget) then delete it to allow GC.
		// I think for back-compatibility it isn't deleting srcNodeRef until after postCreate() has run.
		if(deleteSrcNodeRef){
			delete this.srcNodeRef;
		}

		this._created = true;
	},

	_applyAttributes: function(){
		// summary:
		//		Step during widget creation to copy  widget attributes to the
		//		DOM according to attributeMap and _setXXXAttr objects, and also to call
		//		custom _setXXXAttr() methods.
		//
		//		Skips over blank/false attribute values, unless they were explicitly specified
		//		as parameters to the widget, since those are the default anyway,
		//		and setting tabIndex="" is different than not setting tabIndex at all.
		//
		//		For backwards-compatibility reasons attributeMap overrides _setXXXAttr when
		//		_setXXXAttr is a hash/string/array, but _setXXXAttr as a functions override attributeMap.
		// tags:
		//		private

		// Get list of attributes where this.set(name, value) will do something beyond
		// setting this[name] = value.  Specifically, attributes that have:
		//		- associated _setXXXAttr() method/hash/string/array
		//		- entries in attributeMap (remove this for 2.0);
		var ctor = this.constructor,
			list = ctor._setterAttrs;
		if(!list){
			list = (ctor._setterAttrs = []);
			for(var attr in this.attributeMap){
				list.push(attr);
			}

			var proto = ctor.prototype;
			for(var fxName in proto){
				if(fxName in this.attributeMap){ continue; }
				var setterName = "_set" + fxName.replace(/^[a-z]|-[a-zA-Z]/g, function(c){ return c.charAt(c.length-1).toUpperCase(); }) + "Attr";
				if(setterName in proto){
					list.push(fxName);
				}
			}
		}

		// Call this.set() for each property that was either specified as parameter to constructor,
		// or is in the list found above.	For correlated properties like value and displayedValue, the one
		// specified as a parameter should take precedence.
		// Particularly important for new DateTextBox({displayedValue: ...}) since DateTextBox's default value is
		// NaN and thus is not ignored like a default value of "".

		// Step 1: Save the current values of the widget properties that were specified as parameters to the constructor.
		// Generally this.foo == this.params.foo, except if postMixInProperties() changed the value of this.foo.
		var params = {};
		for(var key in this.params || {}){
			params[key] = this[key];
		}

		// Step 2: Call set() for each property that wasn't passed as a parameter to the constructor
		array.forEach(list, function(attr){
			if(attr in params){
				// skip this one, do it below
			}else if(this[attr]){
				this.set(attr, this[attr]);
			}
		}, this);

		// Step 3: Call set() for each property that was specified as parameter to constructor.
		// Use params hash created above to ignore side effects from step #2 above.
		for(key in params){
			this.set(key, params[key]);
		}
	},

	postMixInProperties: function(){
		// summary:
		//		Called after the parameters to the widget have been read-in,
		//		but before the widget template is instantiated. Especially
		//		useful to set properties that are referenced in the widget
		//		template.
		// tags:
		//		protected
	},

	buildRendering: function(){
		// summary:
		//		Construct the UI for this widget, setting this.domNode.
		//		Most widgets will mixin `dijit._TemplatedMixin`, which implements this method.
		// tags:
		//		protected

		if(!this.domNode){
			// Create root node if it wasn't created by _Templated
			this.domNode = this.srcNodeRef || this.ownerDocument.createElement("div");
		}

		// baseClass is a single class name or occasionally a space-separated list of names.
		// Add those classes to the DOMNode.  If RTL mode then also add with Rtl suffix.
		// TODO: make baseClass custom setter
		if(this.baseClass){
			var classes = this.baseClass.split(" ");
			if(!this.isLeftToRight()){
				classes = classes.concat( array.map(classes, function(name){ return name+"Rtl"; }));
			}
			domClass.add(this.domNode, classes);
		}
	},

	postCreate: function(){
		// summary:
		//		Processing after the DOM fragment is created
		// description:
		//		Called after the DOM fragment has been created, but not necessarily
		//		added to the document.  Do not include any operations which rely on
		//		node dimensions or placement.
		// tags:
		//		protected
	},

	startup: function(){
		// summary:
		//		Processing after the DOM fragment is added to the document
		// description:
		//		Called after a widget and its children have been created and added to the page,
		//		and all related widgets have finished their create() cycle, up through postCreate().
		//		This is useful for composite widgets that need to control or layout sub-widgets.
		//		Many layout widgets can use this as a wiring phase.
		if(this._started){ return; }
		this._started = true;
		array.forEach(this.getChildren(), function(obj){
			if(!obj._started && !obj._destroyed && lang.isFunction(obj.startup)){
				obj.startup();
				obj._started = true;
			}
		});
	},

	//////////// DESTROY FUNCTIONS ////////////////////////////////

	destroyRecursive: function(/*Boolean?*/ preserveDom){
		// summary:
		//		Destroy this widget and its descendants
		// description:
		//		This is the generic "destructor" function that all widget users
		//		should call to cleanly discard with a widget. Once a widget is
		//		destroyed, it is removed from the manager object.
		// preserveDom:
		//		If true, this method will leave the original DOM structure
		//		alone of descendant Widgets. Note: This will NOT work with
		//		dijit._Templated widgets.

		this._beingDestroyed = true;
		this.destroyDescendants(preserveDom);
		this.destroy(preserveDom);
	},

	destroy: function(/*Boolean*/ preserveDom){
		// summary:
		//		Destroy this widget, but not its descendants.
		//		This method will, however, destroy internal widgets such as those used within a template.
		// preserveDom: Boolean
		//		If true, this method will leave the original DOM structure alone.
		//		Note: This will not yet work with _Templated widgets

		this._beingDestroyed = true;
		this.uninitialize();

		function destroy(w){
			if(w.destroyRecursive){
				w.destroyRecursive(preserveDom);
			}else if(w.destroy){
				w.destroy(preserveDom);
			}
		}

		// Back-compat, remove for 2.0
		array.forEach(this._connects, lang.hitch(this, "disconnect"));
		array.forEach(this._supportingWidgets, destroy);

		// Destroy supporting widgets, but not child widgets under this.containerNode (for 2.0, destroy child widgets
		// here too).   if() statement is to guard against exception if destroy() called multiple times (see #15815).
		if(this.domNode){
			array.forEach(registry.findWidgets(this.domNode, this.containerNode), destroy);
		}

		this.destroyRendering(preserveDom);
		registry.remove(this.id);
		this._destroyed = true;
	},

	destroyRendering: function(/*Boolean?*/ preserveDom){
		// summary:
		//		Destroys the DOM nodes associated with this widget
		// preserveDom:
		//		If true, this method will leave the original DOM structure alone
		//		during tear-down. Note: this will not work with _Templated
		//		widgets yet.
		// tags:
		//		protected

		if(this.bgIframe){
			this.bgIframe.destroy(preserveDom);
			delete this.bgIframe;
		}

		if(this.domNode){
			if(preserveDom){
				domAttr.remove(this.domNode, "widgetId");
			}else{
				domConstruct.destroy(this.domNode);
			}
			delete this.domNode;
		}

		if(this.srcNodeRef){
			if(!preserveDom){
				domConstruct.destroy(this.srcNodeRef);
			}
			delete this.srcNodeRef;
		}
	},

	destroyDescendants: function(/*Boolean?*/ preserveDom){
		// summary:
		//		Recursively destroy the children of this widget and their
		//		descendants.
		// preserveDom:
		//		If true, the preserveDom attribute is passed to all descendant
		//		widget's .destroy() method. Not for use with _Templated
		//		widgets.

		// get all direct descendants and destroy them recursively
		array.forEach(this.getChildren(), function(widget){
			if(widget.destroyRecursive){
				widget.destroyRecursive(preserveDom);
			}
		});
	},

	uninitialize: function(){
		// summary:
		//		Deprecated. Override destroy() instead to implement custom widget tear-down
		//		behavior.
		// tags:
		//		protected
		return false;
	},

	////////////////// GET/SET, CUSTOM SETTERS, ETC. ///////////////////

	_setStyleAttr: function(/*String||Object*/ value){
		// summary:
		//		Sets the style attribute of the widget according to value,
		//		which is either a hash like {height: "5px", width: "3px"}
		//		or a plain string
		// description:
		//		Determines which node to set the style on based on style setting
		//		in attributeMap.
		// tags:
		//		protected

		var mapNode = this.domNode;

		// Note: technically we should revert any style setting made in a previous call
		// to his method, but that's difficult to keep track of.

		if(lang.isObject(value)){
			domStyle.set(mapNode, value);
		}else{
			if(mapNode.style.cssText){
				mapNode.style.cssText += "; " + value;
			}else{
				mapNode.style.cssText = value;
			}
		}

		this._set("style", value);
	},

	_attrToDom: function(/*String*/ attr, /*String*/ value, /*Object?*/ commands){
		// summary:
		//		Reflect a widget attribute (title, tabIndex, duration etc.) to
		//		the widget DOM, as specified by commands parameter.
		//		If commands isn't specified then it's looked up from attributeMap.
		//		Note some attributes like "type"
		//		cannot be processed this way as they are not mutable.
		// attr:
		//		Name of member variable (ex: "focusNode" maps to this.focusNode) pointing
		//		to DOMNode inside the widget, or alternately pointing to a subwidget
		// tags:
		//		private

		commands = arguments.length >= 3 ? commands : this.attributeMap[attr];

		array.forEach(lang.isArray(commands) ? commands : [commands], function(command){

			// Get target node and what we are doing to that node
			var mapNode = this[command.node || command || "domNode"];	// DOM node
			var type = command.type || "attribute";	// class, innerHTML, innerText, or attribute

			switch(type){
				case "attribute":
					if(lang.isFunction(value)){ // functions execute in the context of the widget
						value = lang.hitch(this, value);
					}

					// Get the name of the DOM node attribute; usually it's the same
					// as the name of the attribute in the widget (attr), but can be overridden.
					// Also maps handler names to lowercase, like onSubmit --> onsubmit
					var attrName = command.attribute ? command.attribute :
						(/^on[A-Z][a-zA-Z]*$/.test(attr) ? attr.toLowerCase() : attr);

					if(mapNode.tagName){
						// Normal case, mapping to a DOMNode.  Note that modern browsers will have a mapNode.set()
						// method, but for consistency we still call domAttr
						domAttr.set(mapNode, attrName, value);
					}else{
						// mapping to a sub-widget
						mapNode.set(attrName, value);
					}
					break;
				case "innerText":
					mapNode.innerHTML = "";
					mapNode.appendChild(this.ownerDocument.createTextNode(value));
					break;
				case "innerHTML":
					mapNode.innerHTML = value;
					break;
				case "class":
					domClass.replace(mapNode, value, this[attr]);
					break;
			}
		}, this);
	},

	get: function(name){
		// summary:
		//		Get a property from a widget.
		// name:
		//		The property to get.
		// description:
		//		Get a named property from a widget. The property may
		//		potentially be retrieved via a getter method. If no getter is defined, this
		//		just retrieves the object's property.
		//
		//		For example, if the widget has properties `foo` and `bar`
		//		and a method named `_getFooAttr()`, calling:
		//		`myWidget.get("foo")` would be equivalent to calling
		//		`widget._getFooAttr()` and `myWidget.get("bar")`
		//		would be equivalent to the expression
		//		`widget.bar2`
		var names = this._getAttrNames(name);
		return this[names.g] ? this[names.g]() : this[name];
	},

	set: function(name, value){
		// summary:
		//		Set a property on a widget
		// name:
		//		The property to set.
		// value:
		//		The value to set in the property.
		// description:
		//		Sets named properties on a widget which may potentially be handled by a
		//		setter in the widget.
		//
		//		For example, if the widget has properties `foo` and `bar`
		//		and a method named `_setFooAttr()`, calling
		//		`myWidget.set("foo", "Howdy!")` would be equivalent to calling
		//		`widget._setFooAttr("Howdy!")` and `myWidget.set("bar", 3)`
		//		would be equivalent to the statement `widget.bar = 3;`
		//
		//		set() may also be called with a hash of name/value pairs, ex:
		//
		//	|	myWidget.set({
		//	|		foo: "Howdy",
		//	|		bar: 3
		//	|	});
		//
		//	This is equivalent to calling `set(foo, "Howdy")` and `set(bar, 3)`

		if(typeof name === "object"){
			for(var x in name){
				this.set(x, name[x]);
			}
			return this;
		}
		var names = this._getAttrNames(name),
			setter = this[names.s];
		if(lang.isFunction(setter)){
			// use the explicit setter
			var result = setter.apply(this, Array.prototype.slice.call(arguments, 1));
		}else{
			// Mapping from widget attribute to DOMNode/subwidget attribute/value/etc.
			// Map according to:
			//		1. attributeMap setting, if one exists (TODO: attributeMap deprecated, remove in 2.0)
			//		2. _setFooAttr: {...} type attribute in the widget (if one exists)
			//		3. apply to focusNode or domNode if standard attribute name, excluding funcs like onClick.
			// Checks if an attribute is a "standard attribute" by whether the DOMNode JS object has a similar
			// attribute name (ex: accept-charset attribute matches jsObject.acceptCharset).
			// Note also that Tree.focusNode() is a function not a DOMNode, so test for that.
			var defaultNode = this.focusNode && !lang.isFunction(this.focusNode) ? "focusNode" : "domNode",
				tag = this[defaultNode].tagName,
				attrsForTag = tagAttrs[tag] || (tagAttrs[tag] = getAttrs(this[defaultNode])),
				map =	name in this.attributeMap ? this.attributeMap[name] :
						names.s in this ? this[names.s] :
						((names.l in attrsForTag && typeof value != "function") ||
							/^aria-|^data-|^role$/.test(name)) ? defaultNode : null;
			if(map != null){
				this._attrToDom(name, value, map);
			}
			this._set(name, value);
		}
		return result || this;
	},

	_attrPairNames: {},		// shared between all widgets
	_getAttrNames: function(name){
		// summary:
		//		Helper function for get() and set().
		//		Caches attribute name values so we don't do the string ops every time.
		// tags:
		//		private

		var apn = this._attrPairNames;
		if(apn[name]){ return apn[name]; }
		var uc = name.replace(/^[a-z]|-[a-zA-Z]/g, function(c){ return c.charAt(c.length-1).toUpperCase(); });
		return (apn[name] = {
			n: name+"Node",
			s: "_set"+uc+"Attr",	// converts dashes to camel case, ex: accept-charset --> _setAcceptCharsetAttr
			g: "_get"+uc+"Attr",
			l: uc.toLowerCase()		// lowercase name w/out dashes, ex: acceptcharset
		});
	},

	_set: function(/*String*/ name, /*anything*/ value){
		// summary:
		//		Helper function to set new value for specified attribute, and call handlers
		//		registered with watch() if the value has changed.
		var oldValue = this[name];
		this[name] = value;
		if(this._created && !isEqual(value, oldValue)){
			if(this._watchCallbacks){
				this._watchCallbacks(name, oldValue, value);
			}
			this.emit("attrmodified-" + name, {
				detail: {
					prevValue: oldValue,
					newValue: value
				}
			});
		}
	},

	emit: function(/*String*/ type, /*Object?*/ eventObj, /*Array?*/ callbackArgs){
		// summary:
		//		Used by widgets to signal that a synthetic event occurred, ex:
		//	|	myWidget.emit("attrmodified-selectedChildWidget", {}).
		//
		//		Emits an event on this.domNode named type.toLowerCase(), based on eventObj.
		//		Also calls onType() method, if present, and returns value from that method.
		//		By default passes eventObj to callback, but will pass callbackArgs instead, if specified.
		//		Modifies eventObj by adding missing parameters (bubbles, cancelable, widget).
		// tags:
		//		protected

		// Specify fallback values for bubbles, cancelable in case they are not set in eventObj.
		// Also set pointer to widget, although since we can't add a pointer to the widget for native events
		// (see #14729), maybe we shouldn't do it here?
		eventObj = eventObj || {};
		if(eventObj.bubbles === undefined){ eventObj.bubbles = true; }
		if(eventObj.cancelable === undefined){ eventObj.cancelable = true; }
		if(!eventObj.detail){ eventObj.detail = {}; }
		eventObj.detail.widget = this;

		var ret, callback = this["on"+type];
		if(callback){
			ret = callback.apply(this, callbackArgs ? callbackArgs : [eventObj]);
		}

		// Emit event, but avoid spurious emit()'s as parent sets properties on child during startup/destroy
		if(this._started && !this._beingDestroyed){
			on.emit(this.domNode, type.toLowerCase(), eventObj);
		}

		return ret;
	},

	on: function(/*String|Function*/ type, /*Function*/ func){
		// summary:
		//		Call specified function when event occurs, ex: myWidget.on("click", function(){ ... }).
		// type:
		//		Name of event (ex: "click") or extension event like touch.press.
		// description:
		//		Call specified function when event `type` occurs, ex: `myWidget.on("click", function(){ ... })`.
		//		Note that the function is not run in any particular scope, so if (for example) you want it to run in the
		//		widget's scope you must do `myWidget.on("click", lang.hitch(myWidget, func))`.

		// For backwards compatibility, if there's an onType() method in the widget then connect to that.
		// Remove in 2.0.
		var widgetMethod = this._onMap(type);
		if(widgetMethod){
			return aspect.after(this, widgetMethod, func, true);
		}

		// Otherwise, just listen for the event on this.domNode.
		return this.own(on(this.domNode, type, func))[0];
	},

	_onMap: function(/*String|Function*/ type){
		// summary:
		//		Maps on() type parameter (ex: "mousemove") to method name (ex: "onMouseMove").
		//		If type is a synthetic event like touch.press then returns undefined.
		var ctor = this.constructor, map = ctor._onMap;
		if(!map){
			map = (ctor._onMap = {});
			for(var attr in ctor.prototype){
				if(/^on/.test(attr)){
					map[attr.replace(/^on/, "").toLowerCase()] = attr;
				}
			}
		}
		return map[typeof type == "string" && type.toLowerCase()];	// String
	},

	toString: function(){
		// summary:
		//		Returns a string that represents the widget
		// description:
		//		When a widget is cast to a string, this method will be used to generate the
		//		output. Currently, it does not implement any sort of reversible
		//		serialization.
		return '[Widget ' + this.declaredClass + ', ' + (this.id || 'NO ID') + ']'; // String
	},

	getChildren: function(){
		// summary:
		//		Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.
		//		Does not return nested widgets, nor widgets that are part of this widget's template.
		return this.containerNode ? registry.findWidgets(this.containerNode) : []; // dijit/_WidgetBase[]
	},

	getParent: function(){
		// summary:
		//		Returns the parent widget of this widget
		return registry.getEnclosingWidget(this.domNode.parentNode);
	},

	connect: function(
			/*Object|null*/ obj,
			/*String|Function*/ event,
			/*String|Function*/ method){
		// summary:
		//		Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.
		//
		//		Connects specified obj/event to specified method of this object
		//		and registers for disconnect() on widget destroy.
		//
		//		Provide widget-specific analog to dojo.connect, except with the
		//		implicit use of this widget as the target object.
		//		Events connected with `this.connect` are disconnected upon
		//		destruction.
		// returns:
		//		A handle that can be passed to `disconnect` in order to disconnect before
		//		the widget is destroyed.
		// example:
		//	|	var btn = new Button();
		//	|	// when foo.bar() is called, call the listener we're going to
		//	|	// provide in the scope of btn
		//	|	btn.connect(foo, "bar", function(){
		//	|		console.debug(this.toString());
		//	|	});
		// tags:
		//		protected

		return this.own(connect.connect(obj, event, this, method))[0];	// handle
	},

	disconnect: function(handle){
		// summary:
		//		Deprecated, will be removed in 2.0, use handle.remove() instead.
		//
		//		Disconnects handle created by `connect`.
		// tags:
		//		protected

		handle.remove();
	},

	subscribe: function(t, method){
		// summary:
		//		Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.
		//
		//		Subscribes to the specified topic and calls the specified method
		//		of this object and registers for unsubscribe() on widget destroy.
		//
		//		Provide widget-specific analog to dojo.subscribe, except with the
		//		implicit use of this widget as the target object.
		// t: String
		//		The topic
		// method: Function
		//		The callback
		// example:
		//	|	var btn = new Button();
		//	|	// when /my/topic is published, this button changes its label to
		//	|	// be the parameter of the topic.
		//	|	btn.subscribe("/my/topic", function(v){
		//	|		this.set("label", v);
		//	|	});
		// tags:
		//		protected
		return this.own(topic.subscribe(t, lang.hitch(this, method)))[0];	// handle
	},

	unsubscribe: function(/*Object*/ handle){
		// summary:
		//		Deprecated, will be removed in 2.0, use handle.remove() instead.
		//
		//		Unsubscribes handle created by this.subscribe.
		//		Also removes handle from this widget's list of subscriptions
		// tags:
		//		protected

		handle.remove();
	},

	isLeftToRight: function(){
		// summary:
		//		Return this widget's explicit or implicit orientation (true for LTR, false for RTL)
		// tags:
		//		protected
		return this.dir ? (this.dir == "ltr") : domGeometry.isBodyLtr(this.ownerDocument); //Boolean
	},

	isFocusable: function(){
		// summary:
		//		Return true if this widget can currently be focused
		//		and false if not
		return this.focus && (domStyle.get(this.domNode, "display") != "none");
	},

	placeAt: function(/* String|DomNode|_Widget */ reference, /* String|Int? */ position){
		// summary:
		//		Place this widget somewhere in the DOM based
		//		on standard domConstruct.place() conventions.
		// description:
		//		A convenience function provided in all _Widgets, providing a simple
		//		shorthand mechanism to put an existing (or newly created) Widget
		//		somewhere in the dom, and allow chaining.
		// reference:
		//		Widget, DOMNode, or id of widget or DOMNode
		// position:
		//		If reference is a widget (or id of widget), and that widget has an ".addChild" method,
		//		it will be called passing this widget instance into that method, supplying the optional
		//		position index passed.  In this case position (if specified) should be an integer.
		//
		//		If reference is a DOMNode (or id matching a DOMNode but not a widget),
		//		the position argument can be a numeric index or a string
		//		"first", "last", "before", or "after", same as dojo/dom-construct::place().
		// returns: dijit/_WidgetBase
		//		Provides a useful return of the newly created dijit._Widget instance so you
		//		can "chain" this function by instantiating, placing, then saving the return value
		//		to a variable.
		// example:
		//	|	// create a Button with no srcNodeRef, and place it in the body:
		//	|	var button = new Button({ label:"click" }).placeAt(win.body());
		//	|	// now, 'button' is still the widget reference to the newly created button
		//	|	button.on("click", function(e){ console.log('click'); }));
		// example:
		//	|	// create a button out of a node with id="src" and append it to id="wrapper":
		//	|	var button = new Button({},"src").placeAt("wrapper");
		// example:
		//	|	// place a new button as the first element of some div
		//	|	var button = new Button({ label:"click" }).placeAt("wrapper","first");
		// example:
		//	|	// create a contentpane and add it to a TabContainer
		//	|	var tc = dijit.byId("myTabs");
		//	|	new ContentPane({ href:"foo.html", title:"Wow!" }).placeAt(tc)

		var refWidget = !reference.tagName && registry.byId(reference);
		if(refWidget && refWidget.addChild && (!position || typeof position === "number")){
			// Adding this to refWidget and can use refWidget.addChild() to handle everything.
			refWidget.addChild(this, position);
		}else{
			// "reference" is a plain DOMNode, or we can't use refWidget.addChild().   Use domConstruct.place() and
			// target refWidget.containerNode for nested placement (position==number, "first", "last", "only"), and
			// refWidget.domNode otherwise ("after"/"before"/"replace").  (But not supported officially, see #14946.)
			var ref = refWidget ?
				(refWidget.containerNode && !/after|before|replace/.test(position||"") ?
					refWidget.containerNode : refWidget.domNode) : dom.byId(reference, this.ownerDocument);
			domConstruct.place(this.domNode, ref, position);

			// Start this iff it has a parent widget that's already started.
			if(!this._started && (this.getParent() || {})._started){
				this.startup();
			}
		}
		return this;
	},

	getTextDir: function(/*String*/ text,/*String*/ originalDir){
		// summary:
		//		Return direction of the text.
		//		The function overridden in the _BidiSupport module,
		//		its main purpose is to calculate the direction of the
		//		text, if was defined by the programmer through textDir.
		// tags:
		//		protected.
		return originalDir;
	},

	applyTextDir: function(/*===== element, text =====*/){
		// summary:
		//		The function overridden in the _BidiSupport module,
		//		originally used for setting element.dir according to this.textDir.
		//		In this case does nothing.
		// element: DOMNode
		// text: String
		// tags:
		//		protected.
	},

	defer: function(fcn, delay){ 
		// summary:
		//		Wrapper to setTimeout to avoid deferred functions executing
		//		after the originating widget has been destroyed.
		//		Returns an object handle with a remove method (that returns null) (replaces clearTimeout).
		// fcn: function reference
		// delay: Optional number (defaults to 0)
		// tags:
		//		protected.
		var timer = setTimeout(lang.hitch(this, 
			function(){ 
				if(!timer){ return; }
				timer = null;
				if(!this._destroyed){ 
					lang.hitch(this, fcn)(); 
				} 
			}),
			delay || 0
		);
		return {
			remove:	function(){
					if(timer){
						clearTimeout(timer);
						timer = null;
					}
					return null; // so this works well: handle = handle.remove();
				}
		};
	}
});

});

},
'dojo/Stateful':function(){
define("dojo/Stateful", ["./_base/declare", "./_base/lang", "./_base/array", "./when"], function(declare, lang, array, when){
	// module:
	//		dojo/Stateful

return declare("dojo.Stateful", null, {
	// summary:
	//		Base class for objects that provide named properties with optional getter/setter
	//		control and the ability to watch for property changes
	//
	//		The class also provides the functionality to auto-magically manage getters
	//		and setters for object attributes/properties.
	//
	//		Getters and Setters should follow the format of _xxxGetter or _xxxSetter where
	//		the xxx is a name of the attribute to handle.  So an attribute of "foo"
	//		would have a custom getter of _fooGetter and a custom setter of _fooSetter.
	//
	// example:
	//	|	var obj = new dojo.Stateful();
	//	|	obj.watch("foo", function(){
	//	|		console.log("foo changed to " + this.get("foo"));
	//	|	});
	//	|	obj.set("foo","bar");

	// _attrPairNames: Hash
	//		Used across all instances a hash to cache attribute names and their getter
	//		and setter names.
	_attrPairNames: {},

	_getAttrNames: function(name){
		// summary:
		//		Helper function for get() and set().
		//		Caches attribute name values so we don't do the string ops every time.
		// tags:
		//		private

		var apn = this._attrPairNames;
		if(apn[name]){ return apn[name]; }
		return (apn[name] = {
			s: "_" + name + "Setter",
			g: "_" + name + "Getter"
		});
	},

	postscript: function(/*Object?*/ params){
		// Automatic setting of params during construction
		if (params){ this.set(params); }
	},

	_get: function(name, names){
		// summary:
		//		Private function that does a get based off a hash of names
		// names:
		//		Hash of names of custom attributes
		return typeof this[names.g] === "function" ? this[names.g]() : this[name];
	},
	get: function(/*String*/name){
		// summary:
		//		Get a property on a Stateful instance.
		// name:
		//		The property to get.
		// returns:
		//		The property value on this Stateful instance.
		// description:
		//		Get a named property on a Stateful object. The property may
		//		potentially be retrieved via a getter method in subclasses. In the base class
		//		this just retrieves the object's property.
		//		For example:
		//	|	stateful = new dojo.Stateful({foo: 3});
		//	|	stateful.get("foo") // returns 3
		//	|	stateful.foo // returns 3

		return this._get(name, this._getAttrNames(name)); //Any
	},
	set: function(/*String*/name, /*Object*/value){
		// summary:
		//		Set a property on a Stateful instance
		// name:
		//		The property to set.
		// value:
		//		The value to set in the property.
		// returns:
		//		The function returns this dojo.Stateful instance.
		// description:
		//		Sets named properties on a stateful object and notifies any watchers of
		//		the property. A programmatic setter may be defined in subclasses.
		//		For example:
		//	|	stateful = new dojo.Stateful();
		//	|	stateful.watch(function(name, oldValue, value){
		//	|		// this will be called on the set below
		//	|	}
		//	|	stateful.set(foo, 5);
		//
		//	set() may also be called with a hash of name/value pairs, ex:
		//	|	myObj.set({
		//	|		foo: "Howdy",
		//	|		bar: 3
		//	|	})
		//	This is equivalent to calling set(foo, "Howdy") and set(bar, 3)

		// If an object is used, iterate through object
		if(typeof name === "object"){
			for(var x in name){
				if(name.hasOwnProperty(x) && x !="_watchCallbacks"){
					this.set(x, name[x]);
				}
			}
			return this;
		}

		var names = this._getAttrNames(name),
			oldValue = this._get(name, names),
			setter = this[names.s],
			result;
		if(typeof setter === "function"){
			// use the explicit setter
			result = setter.apply(this, Array.prototype.slice.call(arguments, 1));
		}else{
			// no setter so set attribute directly
			this[name] = value;
		}
		if(this._watchCallbacks){
			var self = this;
			// If setter returned a promise, wait for it to complete, otherwise call watches immediatly
			when(result, function(){
				self._watchCallbacks(name, oldValue, value);
			});
		}
		return this; // dojo/Stateful
	},
	_changeAttrValue: function(name, value){
		// summary:
		//		Internal helper for directly changing an attribute value.
		//
		// name: String
		//		The property to set.
		// value: Mixed
		//		The value to set in the property.
		//
		// description:
		//		Directly change the value of an attribute on an object, bypassing any
		//		accessor setter.  Also handles the calling of watch and emitting events.
		//		It is designed to be used by descendent class when there are two values
		//		of attributes that are linked, but calling .set() is not appropriate.

		var oldValue = this.get(name);
		this[name] = value;
		if(this._watchCallbacks){
			this._watchCallbacks(name, oldValue, value);
		}
		return this; // dojo/Stateful
	},
	watch: function(/*String?*/name, /*Function*/callback){
		// summary:
		//		Watches a property for changes
		// name:
		//		Indicates the property to watch. This is optional (the callback may be the
		//		only parameter), and if omitted, all the properties will be watched
		// returns:
		//		An object handle for the watch. The unwatch method of this object
		//		can be used to discontinue watching this property:
		//		|	var watchHandle = obj.watch("foo", callback);
		//		|	watchHandle.unwatch(); // callback won't be called now
		// callback:
		//		The function to execute when the property changes. This will be called after
		//		the property has been changed. The callback will be called with the |this|
		//		set to the instance, the first argument as the name of the property, the
		//		second argument as the old value and the third argument as the new value.

		var callbacks = this._watchCallbacks;
		if(!callbacks){
			var self = this;
			callbacks = this._watchCallbacks = function(name, oldValue, value, ignoreCatchall){
				var notify = function(propertyCallbacks){
					if(propertyCallbacks){
						propertyCallbacks = propertyCallbacks.slice();
						for(var i = 0, l = propertyCallbacks.length; i < l; i++){
							propertyCallbacks[i].call(self, name, oldValue, value);
						}
					}
				};
				notify(callbacks['_' + name]);
				if(!ignoreCatchall){
					notify(callbacks["*"]); // the catch-all
				}
			}; // we use a function instead of an object so it will be ignored by JSON conversion
		}
		if(!callback && typeof name === "function"){
			callback = name;
			name = "*";
		}else{
			// prepend with dash to prevent name conflicts with function (like "name" property)
			name = '_' + name;
		}
		var propertyCallbacks = callbacks[name];
		if(typeof propertyCallbacks !== "object"){
			propertyCallbacks = callbacks[name] = [];
		}
		propertyCallbacks.push(callback);

		// TODO: Remove unwatch in 2.0
		var handle = {};
		handle.unwatch = handle.remove = function(){
			var index = array.indexOf(propertyCallbacks, callback);
			if(index > -1){
				propertyCallbacks.splice(index, 1);
			}
			propertyCallbacks = callback = null;
		};
		return handle; //Object
	}

});

});

},
'dijit/Destroyable':function(){
define("dijit/Destroyable", [
	"dojo/_base/array", // array.forEach array.map
	"dojo/aspect",
	"dojo/_base/declare"
], function(array, aspect, declare){

// module:
//		dijit/Destroyable

return declare("dijit.Destroyable", null, {
	// summary:
	//		Mixin to track handles and release them when instance is destroyed.
	// description:
	//		Call this.own(...) on list of handles (returned from dojo/aspect, dojo/on,
	//		dojo/Stateful::watch, or any class (including widgets) with a destroyRecursive() or destroy() method.
	//		Then call destroy() later to destroy this instance and release the resources.

	destroy: function(/*Boolean*/ preserveDom){
		// summary:
		//		Destroy this class, releasing any resources registered via own().
		this._destroyed = true;
	},

	own: function(){
		// summary:
		//		Track specified handles and remove/destroy them when this instance is destroyed, unless they were
		//		already removed/destroyed manually.
		// tags:
		//		protected
		// returns:
		//		The array of specified handles, so you can do for example:
		//	|		var handle = this.own(on(...))[0];

		array.forEach(arguments, function(handle){
			var destroyMethodName =
				"destroyRecursive" in handle ? "destroyRecursive" :	// remove "destroyRecursive" for 2.0
				"destroy" in handle ? "destroy" :
				"remove";

			// When this.destroy() is called, destroy handle.  Since I'm using aspect.before(),
			// the handle will be destroyed before a subclass's destroy() method starts running, before it calls
			// this.inherited() or even if it doesn't call this.inherited() at all.  If that's an issue, make an
			// onDestroy() method and connect to that instead.
			var odh = aspect.before(this, "destroy", function(preserveDom){
				handle[destroyMethodName](preserveDom);
			});

			// If handle is destroyed manually before this.destroy() is called, remove the listener set directly above.
			var hdh = aspect.after(handle, destroyMethodName, function(){
				odh.remove();
				hdh.remove();
			}, true);
		}, this);

		return arguments;		// handle
	}
});

});

},
'dijit/registry':function(){
define("dijit/registry", [
	"dojo/_base/array", // array.forEach array.map
	"dojo/sniff", // has("ie")
	"dojo/_base/unload", // unload.addOnWindowUnload
	"dojo/_base/window", // win.body
	"./main"	// dijit._scopeName
], function(array, has, unload, win, dijit){

	// module:
	//		dijit/registry

	var _widgetTypeCtr = {}, hash = {};

	var registry =  {
		// summary:
		//		Registry of existing widget on page, plus some utility methods.

		// length: Number
		//		Number of registered widgets
		length: 0,

		add: function(widget){
			// summary:
			//		Add a widget to the registry. If a duplicate ID is detected, a error is thrown.
			// widget: dijit/_WidgetBase
			//		Any dijit/_WidgetBase subclass.
			if(hash[widget.id]){
				throw new Error("Tried to register widget with id==" + widget.id + " but that id is already registered");
			}
			hash[widget.id] = widget;
			this.length++;
		},

		remove: function(/*String*/ id){
			// summary:
			//		Remove a widget from the registry. Does not destroy the widget; simply
			//		removes the reference.
			if(hash[id]){
				delete hash[id];
				this.length--;
			}
		},

		byId: function(/*String|Widget*/ id){
			// summary:
			//		Find a widget by it's id.
			//		If passed a widget then just returns the widget.
			return typeof id == "string" ? hash[id] : id;	// dijit/_WidgetBase
		},

		byNode: function(/*DOMNode*/ node){
			// summary:
			//		Returns the widget corresponding to the given DOMNode
			return hash[node.getAttribute("widgetId")]; // dijit/_WidgetBase
		},

		toArray: function(){
			// summary:
			//		Convert registry into a true Array
			//
			// example:
			//		Work with the widget .domNodes in a real Array
			//		|	array.map(registry.toArray(), function(w){ return w.domNode; });

			var ar = [];
			for(var id in hash){
				ar.push(hash[id]);
			}
			return ar;	// dijit/_WidgetBase[]
		},

		getUniqueId: function(/*String*/widgetType){
			// summary:
			//		Generates a unique id for a given widgetType

			var id;
			do{
				id = widgetType + "_" +
					(widgetType in _widgetTypeCtr ?
						++_widgetTypeCtr[widgetType] : _widgetTypeCtr[widgetType] = 0);
			}while(hash[id]);
			return dijit._scopeName == "dijit" ? id : dijit._scopeName + "_" + id; // String
		},

		findWidgets: function(root, skipNode){
			// summary:
			//		Search subtree under root returning widgets found.
			//		Doesn't search for nested widgets (ie, widgets inside other widgets).
			// root: DOMNode
			//		Node to search under.
			// skipNode: DOMNode
			//		If specified, don't search beneath this node (usually containerNode).

			var outAry = [];

			function getChildrenHelper(root){
				for(var node = root.firstChild; node; node = node.nextSibling){
					if(node.nodeType == 1){
						var widgetId = node.getAttribute("widgetId");
						if(widgetId){
							var widget = hash[widgetId];
							if(widget){	// may be null on page w/multiple dojo's loaded
								outAry.push(widget);
							}
						}else if(node !== skipNode){
							getChildrenHelper(node);
						}
					}
				}
			}

			getChildrenHelper(root);
			return outAry;
		},

		_destroyAll: function(){
			// summary:
			//		Code to destroy all widgets and do other cleanup on page unload

			// Clean up focus manager lingering references to widgets and nodes
			dijit._curFocus = null;
			dijit._prevFocus = null;
			dijit._activeStack = [];

			// Destroy all the widgets, top down
			array.forEach(registry.findWidgets(win.body()), function(widget){
				// Avoid double destroy of widgets like Menu that are attached to <body>
				// even though they are logically children of other widgets.
				if(!widget._destroyed){
					if(widget.destroyRecursive){
						widget.destroyRecursive();
					}else if(widget.destroy){
						widget.destroy();
					}
				}
			});
		},

		getEnclosingWidget: function(/*DOMNode*/ node){
			// summary:
			//		Returns the widget whose DOM tree contains the specified DOMNode, or null if
			//		the node is not contained within the DOM tree of any widget
			while(node){
				var id = node.nodeType == 1 && node.getAttribute("widgetId");
				if(id){
					return hash[id];
				}
				node = node.parentNode;
			}
			return null;
		},

		// In case someone needs to access hash.
		// Actually, this is accessed from WidgetSet back-compatibility code
		_hash: hash
	};

	dijit.registry = registry;

	return registry;
});

},
'dijit/main':function(){
define("dijit/main", [
	"dojo/_base/kernel"
], function(dojo){
	// module:
	//		dijit/main

/*=====
return {
	// summary:
	//		The dijit package main module.
	//		Deprecated.   Users should access individual modules (ex: dijit/registry) directly.
};
=====*/

	return dojo.dijit;
});

},
'dojox/mvc/_atBindingMixin':function(){
define("dojox/mvc/_atBindingMixin", [
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/_base/declare",
	"dojo/has",
	"./resolve",
	"./sync"
], function(array, lang, declare, has, resolve, sync){
	if(has("mvc-bindings-log-api")){
		function getLogContent(/*dojo/Stateful*/ target, /*String*/ targetProp){
			return [target._setIdAttr || !target.declaredClass ? target : target.declaredClass, targetProp].join(":");
		}

		function logResolveFailure(target, targetProp){
			console.warn(targetProp + " could not be resolved" + (typeof target == "string" ? (" with " + target) : "") + ".");
		}
	}

	function getParent(/*dijit/_WidgetBase*/ w){
		// summary:
		//		Returns parent widget having data binding target for relative data binding.
		// w: dijit/_WidgetBase
		//		The widget.

		// Usage of dijit/registry module is optional. Return null if it's not already loaded.
		var registry;
		try{
			registry = require("dijit/registry");
		}catch(e){
			return;
		}
		var pn = w.domNode && w.domNode.parentNode, pw, pb;
		while(pn){
			pw = registry.getEnclosingWidget(pn);
			if(pw){
				var relTargetProp = pw._relTargetProp || "target", pt = lang.isFunction(pw.get) ? pw.get(relTargetProp) : pw[relTargetProp];
				if(pt || relTargetProp in pw.constructor.prototype){
					return pw; // dijit/_WidgetBase
				}
			}
			pn = pw && pw.domNode.parentNode;
		}
	}

	function bind(/*dojo/Stateful|String*/ source, /*String*/ sourceProp, /*dijit/_WidgetBase*/ target, /*String*/ targetProp, /*dojox/mvc/sync.options*/ options){
		// summary:
		//		Resolves the data binding literal, and starts data binding.
		// source: dojo/Stateful|String
		//		Source data binding literal or dojo/Stateful to be synchronized.
		// sourceProp: String
		//		The property name in source to be synchronized.
		// target: dijit/_WidgetBase
		//		Target dojo/Stateful to be synchronized.
		// targetProp: String
		//		The property name in target to be synchronized.
		// options: dojox/mvc/sync.options
		//		Data binding options.

		var _handles = {}, parent = getParent(target), relTargetProp = parent && parent._relTargetProp || "target";

		function resolveAndBind(){
			_handles["Two"] && _handles["Two"].unwatch();
			delete _handles["Two"];

			var relTarget = parent && (lang.isFunction(parent.get) ? parent.get(relTargetProp) : parent[relTargetProp]),
			 resolvedSource = resolve(source, relTarget),
			 resolvedTarget = resolve(target, relTarget);

			if(has("mvc-bindings-log-api") && (!resolvedSource || /^rel:/.test(source) && !parent)){ logResolveFailure(source, sourceProp); }
			if(has("mvc-bindings-log-api") && (!resolvedTarget || /^rel:/.test(target) && !parent)){ logResolveFailure(target, targetProp); }
			if(!resolvedSource || !resolvedTarget || (/^rel:/.test(source) || /^rel:/.test(target)) && !parent){ return; }
			if((!resolvedSource.set || !resolvedSource.watch) && sourceProp == "*"){
				if(has("mvc-bindings-log-api")){ logResolveFailure(source, sourceProp); }
				return;
			}

			if(sourceProp == null){
				// If source property is not specified, it means this handle is just for resolving data binding target.
				// (For dojox/mvc/Group and dojox/mvc/Repeat)
				// Do not perform data binding synchronization in such case.
				lang.isFunction(resolvedTarget.set) ? resolvedTarget.set(targetProp, resolvedSource) : (resolvedTarget[targetProp] = resolvedSource);
				if(has("mvc-bindings-log-api")){
					console.log("dojox/mvc/_atBindingMixin set " + resolvedSource + " to: " + getLogContent(resolvedTarget, targetProp));
				}
			}else{
				// Start data binding
				_handles["Two"] = sync(resolvedSource, sourceProp, resolvedTarget, targetProp, options); // dojox/mvc/sync.handle
			}
		}

		resolveAndBind();
		if(parent && /^rel:/.test(source) || /^rel:/.test(target) && lang.isFunction(parent.set) && lang.isFunction(parent.watch)){
			_handles["rel"] = parent.watch(relTargetProp, function(name, old, current){
				if(old !== current){
					if(has("mvc-bindings-log-api")){ console.log("Change in relative data binding target: " + parent); }
					resolveAndBind();
				}
			});
		}
		var h = {};
		h.unwatch = h.remove = function(){
			for(var s in _handles){
				_handles[s] && _handles[s].unwatch();
				delete _handles[s];
			}
		};
		return h;
	}

	// TODO: Like _DataBindingMixin, this should probably just be a plain Object rather than a Class
	var _atBindingMixin = declare("dojox/mvc/_atBindingMixin", null, {
		// summary:
		//		The mixin for dijit/_WidgetBase to support data binding.

		// dataBindAttr: String
		//		The attribute name for data binding.
		dataBindAttr: "data-mvc-bindings",

		_dbpostscript: function(/*Object?*/ params, /*DomNode|String*/ srcNodeRef){
			// summary:
			//		See if any parameters for this widget are dojox/mvc/at handles.
			//		If so, move them under this._refs to prevent widget implementations from referring them.

			var refs = this._refs = (params || {}).refs || {};
			for(var prop in params){
				if((params[prop] || {}).atsignature == "dojox.mvc.at"){
					var h = params[prop];
					delete params[prop];
					refs[prop] = h;
				}
			}
		},

		_startAtWatchHandles: function(){
			// summary:
			//		Establish data bindings based on dojox/mvc/at handles.

			var refs = this._refs;
			if(refs){
				var atWatchHandles = this._atWatchHandles = this._atWatchHandles || {};

				// Clear the cache of properties that data binding is established with
				this._excludes = null;

				// First, establish non-wildcard data bindings
				for(var prop in refs){
					if(!refs[prop] || prop == "*"){ continue; }
					atWatchHandles[prop] = bind(refs[prop].target, refs[prop].targetProp, this, prop, {bindDirection: refs[prop].bindDirection, converter: refs[prop].converter});
				}

				// Then establish wildcard data bindings
				if((refs["*"] || {}).atsignature == "dojox.mvc.at"){
					atWatchHandles["*"] = bind(refs["*"].target, refs["*"].targetProp, this, "*", {bindDirection: refs["*"].bindDirection, converter: refs["*"].converter});
				}
			}
		},

		_stopAtWatchHandles: function(){
			// summary:
			//		Stops data binding synchronization handles as widget is destroyed.

			for(var s in this._atWatchHandles){
				this._atWatchHandles[s].unwatch();
				delete this._atWatchHandles[s];
			}
		},

		_setAtWatchHandle: function(/*String*/ name, /*Anything*/ value){
			// summary:
			//		Called if the value is a dojox/mvc/at handle.
			//		If this widget has started, start data binding with the new dojox/mvc/at handle.
			//		Otherwise, queue it up to this._refs so that _dbstartup() can pick it up.

			if(name == "ref"){
				throw new Error(this + ": 1.7 ref syntax used in conjuction with 1.8 dojox/mvc/at syntax, which is not supported.");
			}

			// Claen up older data binding
			var atWatchHandles = this._atWatchHandles = this._atWatchHandles || {};
			if(atWatchHandles[name]){
				atWatchHandles[name].unwatch();
				delete atWatchHandles[name];
			}

			// Claar the value
			this[name] = null;

			// Clear the cache of properties that data binding is established with
			this._excludes = null;

			if(this._started){
				// If this widget has been started already, establish data binding immediately.
				atWatchHandles[name] = bind(value.target, value.targetProp, this, name, {bindDirection: value.bindDirection, converter: value.converter});
			}else{
				// Otherwise, queue it up to this._refs so that _dbstartup() can pick it up.
				this._refs[name] = value;
			}
		},

		_setBind: function(/*Object*/ value){
			// summary:
			//		Sets data binding described in data-mvc-bindings.

			var list = eval("({" + value + "})");
			for(var prop in list){
				var h = list[prop];
				if((h || {}).atsignature != "dojox.mvc.at"){
					console.warn(prop + " in " + dataBindAttr + " is not a data binding handle.");
				}else{
					this._setAtWatchHandle(prop, h);
				}
			}
		},

		_getExcludesAttr: function(){
			// summary:
			//		Returns list of all properties that data binding is established with.

			if(this._excludes){ 
				return this._excludes;  // String[] 
			}
			var list = [];
			for(var s in this._atWatchHandles){
				if(s != "*"){ list.push(s); }
			}
			return list; // String[]
		},

		_getPropertiesAttr: function(){
			// summary:
			//		Returns list of all properties in this widget, except "id".
			// returns: String[]
			//		 The list of all properties in this widget, except "id"..

			if(this.constructor._attribs){
				return this.constructor._attribs; // String[]
			}
			var list = ["onClick"].concat(this.constructor._setterAttrs);
			array.forEach(["id", "excludes", "properties", "ref", "binding"], function(s){
				var index = array.indexOf(list, s);
				if(index >= 0){ list.splice(index, 1); }
			});
			return this.constructor._attribs = list; // String[]
		}
	});

	_atBindingMixin.prototype[_atBindingMixin.prototype.dataBindAttr] = ""; // Let parser treat the attribute as string
	return _atBindingMixin;
});

},
'dojox/mvc/resolve':function(){
define("dojox/mvc/resolve", [
	"dojo/_base/lang",
	"dijit/registry",
	"dojo/Stateful"
], function(lang, registry){
	var resolve = function(/*dojo/Stateful|String*/ target, /*dojo/Stateful?*/ parent){
		// summary:
		//		Find a dojo/Stateful for the target.
		// description:
		//		If target is not a string, return target itself.
		//		If target is "widget:widgetid", returns the widget whose ID is widgetid.
		//		If target is "rel:object.path", or target is other string, returns an object under parent (if specified) or under global scope.
		// target: dojo/Stateful|String
		//		The data binding to resolve.
		// parent: dojo/Stateful?
		//		The parent data binding. Used when the data binding is defined inside repeat.

		if(typeof target == "string"){
			var tokens = target.match(/^(expr|rel|widget):(.*)$/) || [];
			try{
				if(tokens[1] == "rel"){
					target = lang.getObject(tokens[2] || "", false, parent);
				}else if(tokens[1] == "widget"){
					target = registry.byId(tokens[2]);
				}else{
					target = lang.getObject(tokens[2] || target, false, parent);
				}
			}catch(e){}
		}

		return target; // dojo/Stateful
	};

	// lang.setObject() thing is for back-compat, remove it in 2.0
	return lang.setObject("dojox.mvc.resolve", resolve);
});

},
'epi-find/store/StoreFactory':function(){
define([
    "dojo/_base/config",
    "dojo/_base/declare",
    "dojo/_base/lang",

    "dojo/store/Memory",

    "epi/shell/store/Throttle",

    "./IdToNameResolvingMixin",
    "./ResourceQueryEngine",
    "./Elasticsearch",
    "./ResourceStore",
    "./BucketMixin",
    "./Statistics",
    "./Optimization",
    "./BestBets",
    "./SearchWithWeights",
    "./ServiceHealthStore",

    "./LangMixin",
    "./SiteMixin",

    "../AccessModel"
    ],
    function(
        config,
        declare,
        lang,

        Memory,

        Throttle,

        IdToNameResolvingMixin,
        ResourceQueryEngine,
        Elasticsearch,
        ResourceStore,
        BucketMixin,
        Statistics,
        Optimization,
        BestBets,
        SearchWithWeights,
        ServiceHealthStore,
        LangMixin,
        SiteMixin,

        AccessModel
        ) {

        var LangSiteMixin = declare([LangMixin, SiteMixin], {
            languageParameter: "tags",
            languagePrefix: "language:",

            siteParameter: "tags",
            sitePrefix: "siteid:",

            addToObject: true,
            addEmptyToQuery: false
        });

        var LangSiteMixinEmptyFilter = declare([LangSiteMixin], {
            emptyLanguage: "7D2DA0A9FC754533B091FA6886A51C0D",

            emptySite: "84BFAF5C52A349A0BC61A9FFB6983A66"
        });

        var ResourceStoreConfigured = declare([ResourceStore], {
            listProperty: "hits",
            itemProperty: "item"
        });

        var SynonymLangMixin = declare([LangMixin], {
            languageParameter: "tags",
            languagePrefix: "language:",
            emptyLanguage: "7D2DA0A9FC754533B091FA6886A51C0D"
        });

        // Add support for language and sites to some stores
        var StatisticsLangSiteStore = declare([Statistics, LangSiteMixin]),
            QueryLangSiteStore = declare([ResourceStoreConfigured, BucketMixin, LangSiteMixin]),
            TopHitStore = declare([StatisticsLangSiteStore, IdToNameResolvingMixin]),
            SynonymLangStore = declare([ResourceStoreConfigured, SynonymLangMixin]),
            BestBetStore = declare([BestBets, LangSiteMixinEmptyFilter]),
            OptimizationLangSiteStore = declare([Optimization, LangSiteMixinEmptyFilter]);


        var StoreFactory = declare(null, {
            authToken: null,

            createElasticSearchStore: function () {
                var esStore = new Elasticsearch({
                    target: config.find.serviceApiBaseUrl + "_search"
                });

                var throttledStore = Throttle(esStore, "query");
                throttledStore.unwrapped = esStore;
                return throttledStore;
            },

            createQueryStore: function () {
                return new QueryLangSiteStore({
                    target: config.find.serviceApiBaseUrl + "_stats/query"
                });
            },

            createTopQueryStore: function () {
                return new StatisticsLangSiteStore({
                    target: config.find.serviceApiBaseUrl + "_stats/query/top"
                });
            },

            createTopHitStore: function () {
                return new TopHitStore({
                    baseTarget: config.find.serviceApiBaseUrl,
                    target: config.find.serviceApiBaseUrl + "_stats/hit/top"
                });
            },

            createSynonymStore: function () {
                var synonymStore = AccessModel.access.view.synonyms.isEnabled() ?
                    new SynonymLangStore({
                        target: config.find.serviceApiBaseUrl + "_admin/synonym/",
                        idProperty: "id",
                        addToObject: true,
                        addEmptyToQuery: false
                    }) : new Memory({data: []});

                var throttledStore = Throttle(synonymStore, "query");
                throttledStore.unwrapped = synonymStore;
                return throttledStore;
            },

            createBestBetStore: function () {
                var bestBetStore = new BestBetStore({
                    target: config.find.siteServiceApiBaseUrl + "bestbets/",
                    idProperty: "id",
                    returnUpdatedItemFromPut: true,
                    queryEngine: ResourceQueryEngine,
                    handlerOptions: { authToken: this.authToken, epiAuthHeader: true}
                });
                // Throttle prevents issuing several queries simultaneously,
                // which might cause UI glitches, like empty grid.
                var throttledStore = Throttle(bestBetStore, "query");
                throttledStore.unwrapped = bestBetStore;
                return throttledStore;
            },

            createRelatedQueriesStore: function () {
                var relatedQueriesStore = new OptimizationLangSiteStore({
                    target: config.find.serviceApiBaseUrl + "_didyoumean/",
                    queryEngine: ResourceQueryEngine
                });

                var throttledStore = Throttle(relatedQueriesStore, "query");
                throttledStore.unwrapped = relatedQueriesStore;
                return throttledStore;
            },

            createAutocompleteStore: function () {
                var autocompleteStore = new OptimizationLangSiteStore({
                    target: config.find.serviceApiBaseUrl + "_autocomplete/",
                    queryEngine: ResourceQueryEngine
                });

                var throttledStore = Throttle(autocompleteStore, "query");
                throttledStore.unwrapped = autocompleteStore;
                return throttledStore;

            },

            createConnectorStore: function () {
                return AccessModel.access.view.connectors.isEnabled() ?
                    new ResourceStoreConfigured({
                        target: config.find.serviceApiBaseUrl + "_admin/connector/",
                        idProperty: "id",
                        // queryEngine is required to workaround an issue with EditStoreRefController that calls
                        // store.put instead of store.add when a new item is created,
                        // which prevents Observable notifications.
                        queryEngine: ResourceQueryEngine
                    }) : new Memory({data: []});
            },

            createConnectorJobStore: function () {
                return AccessModel.access.view.connectors.isEnabled() ?
                    new ResourceStoreConfigured({
                        target: config.find.serviceApiBaseUrl + "_admin/connector_job/status/",
                        commandTarget: config.find.serviceApiBaseUrl + "_admin/connector_job/",
                        idProperty: "id",
                        _runCommand: function (id, command) {
                            var headers = lang.mixin({ Accept: this.accepts }, this.headers);
                            return this.xhrHandler.xhr("GET", {
                                url: this.commandTarget + command + "/" + id,
                                handleAs: "json",
                                headers: headers
                            });
                        },
                        start: function (id) {
                            return this._runCommand(id, "start");
                        },
                        stop: function (id) {
                            return this._runCommand(id, "stop");
                        }
                    }) : new Memory({data: []});
            },

            createBoostingStore: function () {
                return AccessModel.access.view.boosting.isEnabled() ?
                    new ResourceStoreConfigured({
                        target: config.find.serviceApiBaseUrl + "_admin/unifiedweights/",
                        idProperty: "id",
                        // queryEngine is required to workaround an issue with EditStoreRefController that calls
                        // store.put instead of store.add when a new item is created,
                        // which prevents Observable notifications.
                        queryEngine: ResourceQueryEngine
                    }) : new Memory({data: []});
            },

            createBoostingPreviewStore: function () {
                return AccessModel.access.view.boosting.isEnabled() ?
                    new SearchWithWeights({
                        target: config.find.siteServiceApiBaseUrl + "editorialboosting/searchwithweights/",
                        handlerOptions: { authToken: this.authToken, epiAuthHeader: true }
                    }) :
                    new Memory({
                        data: [
                            {
                                "Title": "SEO Guide | EPiServer",
                                "Excerpt": "A guide to start optimizing content for search.",
                                "Url": "http://www.episerver.com/knowledge-base/eBooks/Why-SEO-Belongs-to-Content-Marketers/"
                            },
                            {
                                "Title": "SiteAttention for EPiServer Delivers On Improved Organic Search Results for Businesses  - EPiServer",
                                "Excerpt": "Search engine optimization (SEO) add-on uses Google Analytics to streamline creation of optimized content",
                                "Url": "http://www.episerver.com/about-us/pressroom/press-release-archive-2012/SiteAttention-for-EPiServer-Delivers-On-Improved-Organic-Search-Results-for-Businesses-/"
                            },
                            {
                                "Title": "Improve customer experience with effective onsite search - EPiServer",
                                "Excerpt": "When we talk about search, most marketers focus their energy and investment in optimizing content keywords and search engine ranking positions (SER...",
                                "Url": "http://www.episerver.com/about-us/our-blog/maria-wasing/How-to-improve-customer-experience-with-effective-onsite-search/"
                            },
                            {
                                "Title": "EPiServer Acquires On-Site Search Provider, Euroling",
                                "Excerpt": "EPiServer, a global software provider for innovative ecommerce and digital marketing solutions, today announced its acquisition of Swedish search p...",
                                "Url": "http://www.episerver.com/about-us/pressroom/pressreleases/EPiServer-Acquires-On-Site-Search-Provider-Euroling-/"
                            }
                        ],
                        idProperty: "Url",
                        query: function () {
                            return this.data;
                        }
                });
            },
            createServiceHealthStore: function () {
                return new ServiceHealthStore({
                    target: config.find.siteServiceApiBaseUrl + "ServiceHealth/GetInformation",
                    handlerOptions: { authToken: this.authToken, epiAuthHeader: true }
                })
            },
            createStores: function () {
                // summary:
                //      Instantiate all stores
                config.dependencies["epi-find.ElasticsearchStore"] = this.createElasticSearchStore();
                config.dependencies["epi-find.QueryStore"] = this.createQueryStore();
                config.dependencies["epi-find.TopQueryStore"] = this.createTopQueryStore();
                config.dependencies["epi-find.TopHitStore"] = this.createTopHitStore();
                config.dependencies["epi-find.SynonymStore"] = this.createSynonymStore() ;
                config.dependencies["epi-find.BestBetStore"] = this.createBestBetStore();
                config.dependencies["epi-find.RelatedQueries"] = this.createRelatedQueriesStore();
                config.dependencies["epi-find.Autocomplete"] = this.createAutocompleteStore();
                config.dependencies["epi-find.ConnectorStore"] =this.createConnectorStore();
                config.dependencies["epi-find.ConnectorJobStore"] = this.createConnectorJobStore();
                config.dependencies["epi-find.BoostingStore"] = this.createBoostingStore();
                config.dependencies["epi-find.BoostingPreviewStore"] = this.createBoostingPreviewStore();
                config.dependencies["epi-find.ServiceHealthStore"] = this.createServiceHealthStore();
            }
        });

        return new StoreFactory();
    }
);

},
'dojo/store/Memory':function(){
define("dojo/store/Memory", ["../_base/declare", "./util/QueryResults", "./util/SimpleQueryEngine" /*=====, "./api/Store" =====*/],
function(declare, QueryResults, SimpleQueryEngine /*=====, Store =====*/){

// module:
//		dojo/store/Memory

// No base class, but for purposes of documentation, the base class is dojo/store/api/Store
var base = null;
/*===== base = Store; =====*/

return declare("dojo.store.Memory", base, {
	// summary:
	//		This is a basic in-memory object store. It implements dojo/store/api/Store.
	constructor: function(options){
		// summary:
		//		Creates a memory object store.
		// options: dojo/store/Memory
		//		This provides any configuration information that will be mixed into the store.
		//		This should generally include the data property to provide the starting set of data.
		for(var i in options){
			this[i] = options[i];
		}
		this.setData(this.data || []);
	},
	// data: Array
	//		The array of all the objects in the memory store
	data:null,

	// idProperty: String
	//		Indicates the property to use as the identity property. The values of this
	//		property should be unique.
	idProperty: "id",

	// index: Object
	//		An index of data indices into the data array by id
	index:null,

	// queryEngine: Function
	//		Defines the query engine to use for querying the data store
	queryEngine: SimpleQueryEngine,
	get: function(id){
		// summary:
		//		Retrieves an object by its identity
		// id: Number
		//		The identity to use to lookup the object
		// returns: Object
		//		The object in the store that matches the given id.
		return this.data[this.index[id]];
	},
	getIdentity: function(object){
		// summary:
		//		Returns an object's identity
		// object: Object
		//		The object to get the identity from
		// returns: Number
		return object[this.idProperty];
	},
	put: function(object, options){
		// summary:
		//		Stores an object
		// object: Object
		//		The object to store.
		// options: dojo/store/api/Store.PutDirectives?
		//		Additional metadata for storing the data.  Includes an "id"
		//		property if a specific id is to be used.
		// returns: Number
		var data = this.data,
			index = this.index,
			idProperty = this.idProperty;
		var id = object[idProperty] = (options && "id" in options) ? options.id : idProperty in object ? object[idProperty] : Math.random();
		if(id in index){
			// object exists
			if(options && options.overwrite === false){
				throw new Error("Object already exists");
			}
			// replace the entry in data
			data[index[id]] = object;
		}else{
			// add the new object
			index[id] = data.push(object) - 1;
		}
		return id;
	},
	add: function(object, options){
		// summary:
		//		Creates an object, throws an error if the object already exists
		// object: Object
		//		The object to store.
		// options: dojo/store/api/Store.PutDirectives?
		//		Additional metadata for storing the data.  Includes an "id"
		//		property if a specific id is to be used.
		// returns: Number
		(options = options || {}).overwrite = false;
		// call put with overwrite being false
		return this.put(object, options);
	},
	remove: function(id){
		// summary:
		//		Deletes an object by its identity
		// id: Number
		//		The identity to use to delete the object
		// returns: Boolean
		//		Returns true if an object was removed, falsy (undefined) if no object matched the id
		var index = this.index;
		var data = this.data;
		if(id in index){
			data.splice(index[id], 1);
			// now we have to reindex
			this.setData(data);
			return true;
		}
	},
	query: function(query, options){
		// summary:
		//		Queries the store for objects.
		// query: Object
		//		The query to use for retrieving objects from the store.
		// options: dojo/store/api/Store.QueryOptions?
		//		The optional arguments to apply to the resultset.
		// returns: dojo/store/api/Store.QueryResults
		//		The results of the query, extended with iterative methods.
		//
		// example:
		//		Given the following store:
		//
		// 	|	var store = new Memory({
		// 	|		data: [
		// 	|			{id: 1, name: "one", prime: false },
		//	|			{id: 2, name: "two", even: true, prime: true},
		//	|			{id: 3, name: "three", prime: true},
		//	|			{id: 4, name: "four", even: true, prime: false},
		//	|			{id: 5, name: "five", prime: true}
		//	|		]
		//	|	});
		//
		//	...find all items where "prime" is true:
		//
		//	|	var results = store.query({ prime: true });
		//
		//	...or find all items where "even" is true:
		//
		//	|	var results = store.query({ even: true });
		return QueryResults(this.queryEngine(query, options)(this.data));
	},
	setData: function(data){
		// summary:
		//		Sets the given data as the source for this store, and indexes it
		// data: Object[]
		//		An array of objects to use as the source of data.
		if(data.items){
			// just for convenience with the data format IFRS expects
			this.idProperty = data.identifier || this.idProperty;
			data = this.data = data.items;
		}else{
			this.data = data;
		}
		this.index = {};
		for(var i = 0, l = data.length; i < l; i++){
			this.index[data[i][this.idProperty]] = i;
		}
	}
});

});

},
'dojo/store/util/QueryResults':function(){
define("dojo/store/util/QueryResults", ["../../_base/array", "../../_base/lang", "../../_base/Deferred"
], function(array, lang, Deferred){

// module:
//		dojo/store/util/QueryResults

var QueryResults = function(results){
	// summary:
	//		A function that wraps the results of a store query with additional
	//		methods.
	// description:
	//		QueryResults is a basic wrapper that allows for array-like iteration
	//		over any kind of returned data from a query.  While the simplest store
	//		will return a plain array of data, other stores may return deferreds or
	//		promises; this wrapper makes sure that *all* results can be treated
	//		the same.
	//
	//		Additional methods include `forEach`, `filter` and `map`.
	// results: Array|dojo/promise/Promise
	//		The result set as an array, or a promise for an array.
	// returns:
	//		An array-like object that can be used for iterating over.
	// example:
	//		Query a store and iterate over the results.
	//
	//	|	store.query({ prime: true }).forEach(function(item){
	//	|		//	do something
	//	|	});

	if(!results){
		return results;
	}

	var isPromise = !!results.then;
	// if it is a promise it may be frozen
	if(isPromise){
		results = lang.delegate(results);
	}
	function addIterativeMethod(method){
		// Always add the iterative methods so a QueryResults is
		// returned whether the environment is ES3 or ES5
		results[method] = function(){
			var args = arguments;
			var result = Deferred.when(results, function(results){
				Array.prototype.unshift.call(args, results);
				return QueryResults(array[method].apply(array, args));
			});
			// forEach should only return the result of Deferred.when()
			// when we're wrapping a promise
			if(method !== "forEach" || isPromise){
				return result;
			}
		};
	}

	addIterativeMethod("forEach");
	addIterativeMethod("filter");
	addIterativeMethod("map");
	if(results.total == null){
		results.total = Deferred.when(results, function(results){
			return results.length;
		});
	}
	return results; // Object
};

lang.setObject("dojo.store.util.QueryResults", QueryResults);

return QueryResults;

});

},
'dojo/store/util/SimpleQueryEngine':function(){
define("dojo/store/util/SimpleQueryEngine", ["../../_base/array" /*=====, "../api/Store" =====*/], function(arrayUtil /*=====, Store =====*/){

// module:
//		dojo/store/util/SimpleQueryEngine

return function(query, options){
	// summary:
	//		Simple query engine that matches using filter functions, named filter
	//		functions or objects by name-value on a query object hash
	//
	// description:
	//		The SimpleQueryEngine provides a way of getting a QueryResults through
	//		the use of a simple object hash as a filter.  The hash will be used to
	//		match properties on data objects with the corresponding value given. In
	//		other words, only exact matches will be returned.
	//
	//		This function can be used as a template for more complex query engines;
	//		for example, an engine can be created that accepts an object hash that
	//		contains filtering functions, or a string that gets evaluated, etc.
	//
	//		When creating a new dojo.store, simply set the store's queryEngine
	//		field as a reference to this function.
	//
	// query: Object
	//		An object hash with fields that may match fields of items in the store.
	//		Values in the hash will be compared by normal == operator, but regular expressions
	//		or any object that provides a test() method are also supported and can be
	//		used to match strings by more complex expressions
	//		(and then the regex's or object's test() method will be used to match values).
	//
	// options: dojo/store/api/Store.QueryOptions?
	//		An object that contains optional information such as sort, start, and count.
	//
	// returns: Function
	//		A function that caches the passed query under the field "matches".  See any
	//		of the "query" methods on dojo.stores.
	//
	// example:
	//		Define a store with a reference to this engine, and set up a query method.
	//
	//	|	var myStore = function(options){
	//	|		//	...more properties here
	//	|		this.queryEngine = SimpleQueryEngine;
	//	|		//	define our query method
	//	|		this.query = function(query, options){
	//	|			return QueryResults(this.queryEngine(query, options)(this.data));
	//	|		};
	//	|	};

	// create our matching query function
	switch(typeof query){
		default:
			throw new Error("Can not query with a " + typeof query);
		case "object": case "undefined":
			var queryObject = query;
			query = function(object){
				for(var key in queryObject){
					var required = queryObject[key];
					if(required && required.test){
						// an object can provide a test method, which makes it work with regex
						if(!required.test(object[key], object)){
							return false;
						}
					}else if(required != object[key]){
						return false;
					}
				}
				return true;
			};
			break;
		case "string":
			// named query
			if(!this[query]){
				throw new Error("No filter function " + query + " was found in store");
			}
			query = this[query];
			// fall through
		case "function":
			// fall through
	}
	function execute(array){
		// execute the whole query, first we filter
		var results = arrayUtil.filter(array, query);
		// next we sort
		var sortSet = options && options.sort;
		if(sortSet){
			results.sort(typeof sortSet == "function" ? sortSet : function(a, b){
				for(var sort, i=0; sort = sortSet[i]; i++){
					var aValue = a[sort.attribute];
					var bValue = b[sort.attribute];
					// valueOf enables proper comparison of dates
					aValue = aValue != null ? aValue.valueOf() : aValue;
					bValue = bValue != null ? bValue.valueOf() : bValue;
					if (aValue != bValue){
						return !!sort.descending == (aValue == null || aValue > bValue) ? -1 : 1;
					}
				}
				return 0;
			});
		}
		// now we paginate
		if(options && (options.start || options.count)){
			var total = results.length;
			results = results.slice(options.start || 0, (options.start || 0) + (options.count || Infinity));
			results.total = total;
		}
		return results;
	}
	execute.matches = query;
	return execute;
};

});

},
'epi/shell/store/Throttle':function(){
﻿define("epi/shell/store/Throttle", [
    "dojo/_base/array",
    "dojo/_base/lang",
    "dojo/aspect",
    "dojo/when",
    "epi/epi"
], function (array, lang, aspect, when, epi) {

    return function (throttledStore, method) {
        // summary:
        //	    The store throttling wrapper takes a store implementing the dojo/store/api/Store and
        //      adds request throttling functionality to the get and query methods as well as the
        //      epi refresh extension. Meaning that only the first call to any of these methods
        //      with the same arguments will be sent to the server, any subsequent calls done
        //      while a request is ongoing will be resolved when the first request is returned.
        //
        // throttledStore:
        //	    This is the backing store which sends requests to the server.
        // method:
        //      The method to throttle, e.g "get" or "query"
        //
        // tags: private

        var activeRequests = [];
        var removeActiveRequest = function (deferred) {
            // summary:
            //  Removes an object from the _activeRequests array. The removed object is returned.

            for (var i = 0; i < activeRequests.length; i++) {
                if (activeRequests[i].deferred === deferred) {
                    return activeRequests.splice(i, 1);
                }
            }
            return null;
        };

        var getActiveRequestByArguments = function (method, args) {
            // summary:
            //  Find and return any active requests by request arguments and method

            for (var i = 0; i < activeRequests.length; i++) {
                var request = activeRequests[i];
                if (request.method === method && epi.areEqual(request.args, args)) {
                    return request;
                }
            }
            return null;
        };

        var execute = function (method, args) {
            // summary:
            //  Executes a method with the provided args on the backing store, or returns the promise
            //  if a request for the same data is pending.

            // Convert the arguments "array" to a real array to please phantomjs
            var argsArray = array.map(args, function (item) {
                return item;
            });

            // Try finding any ongoing requests by its serialized arguments, cloning the
            var activeRequest = getActiveRequestByArguments(method, argsArray);
            if (activeRequest) {
                // Found one, so let everyone get the same deferred.
                return activeRequest.deferred;
            }

            // No ongoing request.
            // Query the backing store and hold on to the promise so everyone wanting the same data
            // while the request is pending can "piggy-back" on the first request.
            var deferred = throttledStore[method].apply(throttledStore, argsArray);

            activeRequests.push({
                method: method,
                args: argsArray,
                deferred: deferred
            });

            // Have to remove it from the array when it's resolved
            when(deferred,
                function () {
                    removeActiveRequest(deferred);
                },
                function () {
                    removeActiveRequest(deferred);
                });

            return deferred;
        };

        var delegator = {};
        delegator[method] = function () {
            return execute(method, arguments);
        };

        return lang.delegate(throttledStore, delegator);
    };
});

},
'epi/epi':function(){
﻿define("epi/epi", [
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/aspect",
    ".",
    "./dependency",
    "./routes",
    "./clientResourcesLoader",
    "./_Module",
    "./ModuleManager",
    "./shell/Bootstrapper",
    "./shell/Stateful",
    "./shell/messaging" // Used for event forwarding between dojo and jQuery. Required for dashboard/mvc gadget compatibility
], function (lang, array, aspect, epi) {


    epi.compareOptions = {
        treatFalsyAsEquals: 1,
        skipEmptyProperties: 2
    };

    /* global CSSImportRule */

    // tags:
    //      public

    epi.areEqual = function (first, second, compareOptions) {
        // summary:
        //		are the first and second objects/values deeply
        //		equivalent?

        // Compare undefined always with three equal signs, because undefined==null
        // is true, but undefined===null is false.
        if ((first === undefined) && (second === undefined)) {
            return true;
        }

        if (first === null && second === null) {
            return true;
        }

        if (compareOptions && (compareOptions & epi.compareOptions.treatFalsyAsEquals)) {
            if (!first && !second) {
                return true;
            }
        }

        // If only one is null, they aren't equal.
        if (first === null || second === null) {
            return false;
        }

        //Are they of the same type?
        if (typeof first !== typeof second) {
            return false;
        }

        if ((first === second) || (first == second) || (typeof first == "number" && typeof second == "number" && isNaN(first) && isNaN(second))) { // eslint-disable-line eqeqeq
            return true;
        }

        //is it an array
        if (lang.isArray(first) && lang.isArray(second)) {
            //Return fals if the length are different
            if (first.length !== second.length) {
                return false;
            }

            //Check each value in the array
            for (var x = 0; x < first.length; x++) {
                if (!epi.areEqual(first[x], second[x], compareOptions)) {
                    return false;
                }
            }

            return true;
        }

        //Is it a date
        if ((first instanceof Date) && (second instanceof Date)) {
            //compare number of milliseconds since 1970.
            return first.getTime() === second.getTime();
        }

        //Is it a function
        if ((typeof first  === "function") && (typeof second  === "function")) {
            return first.toString() === second.toString();
        }

        //Is it an object
        if (((typeof first == "object") && ((typeof second == "object")))) {
            var prop;
            // Make sure ALL THE SAME properties are in both objects!
            for (prop in second) {
                if (!(prop in first)) {
                    if (compareOptions && (compareOptions & epi.compareOptions.skipEmptyProperties)) {
                        //second object has empty property and first object hasn't this property at all
                        if (!second[prop]) {
                            continue;
                        }
                    }

                    return false;
                }
            }

            //Check that the values of the properties are equal
            for (prop in first) {
                if (compareOptions && (compareOptions & epi.compareOptions.skipEmptyProperties)) {
                    // first object has empty property and second object hasn't this property at all
                    if (!first[prop] && !second.hasOwnProperty(prop)) {
                        continue;
                    }
                }

                if (!epi.areEqual(first[prop], second[prop], compareOptions)) {
                    return false;
                }
            }

            return true;
        }

        return false;
    };

    epi.isEmpty = function (obj) {
        // summary:
        //      Utilities function to check whether an object is empty or not
        //
        // obj: Object
        //
        // tags:
        //      public

        for (var prop in obj) {
            if (obj.hasOwnProperty(prop)) {
                return false;
            }
        }
        return true;
    };

    epi.delegate = function (obj, props, aspectedMethods) {
        // summary:
        //      Create a delegate from the given object and make sure all the "after" aspect advices still work.
        //
        // description:
        //      This function does not ensure the aspect call order between the original object's advices and the delegate object's advice.
        //      Note that all the aspect advices will receive original arguments and will not be able to change the method result.
        //
        // obj: Object
        //      The original object.
        //
        // props: Object
        //      The override properties
        //
        // aspectedMethods: Array
        //      The methods that might have aspect connect that we want to keep.
        //
        // tags:
        //      public

        // Use dojo delagate to create a delegate object
        var delegate = lang.delegate(obj, props);

        // Flag indicate that the aspect chain is triggered by the aspectBridge advice we added to the delegate object
        var triggerFromDelegate = false;

        // Aspect chain execution helper
        var executeAfterAspect = function (dispatcher) {
            var after = dispatcher.after;
            while (after) {
                after.advice.apply(this, arguments);
                after = after.next;
            }
        };

        // Create aspect bridge handles array, to be able to remove later on.
        delegate._aspectBridges = [];

        // Loop through all the methods
        array.forEach(aspectedMethods, function (propName) {
            if (!obj.hasOwnProperty(propName) || typeof obj[propName] !== "function") {
                return;
            }

            // Add an after advice to the delegate object to trigger the after advices on the original object when the method is called on the delegate object
            delegate._aspectBridges.push(aspect.after(delegate, propName, function () {
                if (triggerFromDelegate) {
                    return;
                }

                var dispatcher = obj[propName];
                executeAfterAspect(dispatcher);
            }, true));

            // Add an after advice to the original object to trigger the after advices on the delegate object when the method is called on the delegate object
            delegate._aspectBridges.push(aspect.after(obj, propName, function () {
                triggerFromDelegate = true;

                var dispatcher = delegate[propName];
                executeAfterAspect(dispatcher);

                triggerFromDelegate = false;
            }, true));
        });

        return delegate;
    };

    epi.removeDelegateAspects = function (delegate) {
        // summary:
        //      Remove the aspect bridges between the delegate object and the original object created when using epi.delegate.
        // delegate: Object
        //      The delegate object.

        delegate._aspectBridges && delegate._aspectBridges.forEach(function (handle) {
            handle.remove();
        });
    };

    epi._countCSSRules = function (asString) {
        // summary:
        //     Counts selectors and rules in all included stylesheets
        // asString:
        //     Boolean
        // tags:
        //     private

        var results = [];

        function countSheet(sheet) {
            var count = 0,
                rule;

            if (sheet && sheet.cssRules) {
                for (var j = 0, l = sheet.cssRules.length; j < l; j++) {
                    rule = sheet.cssRules[j];

                    if (rule instanceof CSSImportRule) {
                        countSheet(rule.styleSheet);
                        continue;
                    }

                    if (rule.selectorText) {
                        count += rule.selectorText.split(",").length;
                    } else if (rule.cssRules) {
                        for (var m = 0, n = rule.cssRules.length; m < n; m++) {
                            if (rule.cssRules[m].selectorText) {
                                count += rule.cssRules[m].selectorText.split(",").length;
                            }
                        }
                    }
                }

                results.push({
                    file: sheet.href ? sheet.href : "inline",
                    rules: sheet.cssRules.length,
                    selectors: count
                });
            }
        }

        if (!document.styleSheets) {
            return;
        }
        for (var i = 0; i < document.styleSheets.length; i++) {
            countSheet(document.styleSheets[i]);
        }

        return asString ? JSON.stringify(results, null, "    ") : results;
    };

    return epi;
});

},
'epi/main':function(){
// Currently only a place holder for the core epi namespace
// Needed to get the amd loading properly defined, but should contain core epi parts

define("epi/main", ["dojo", "dojo/i18n!epi/nls/episerver.shared"], function (dojo, sharedResources) {

    var epi = dojo.getObject("epi", true);
    epi.resources = dojo.mixin({}, sharedResources);
    return epi;
});

},
'dojo/i18n':function(){
define("dojo/i18n", ["./_base/kernel", "require", "./has", "./_base/array", "./_base/config", "./_base/lang", "./_base/xhr", "./json", "module"],
	function(dojo, require, has, array, config, lang, xhr, json, module){

	// module:
	//		dojo/i18n

	has.add("dojo-preload-i18n-Api",
		// if true, define the preload localizations machinery
		1
	);

	 1 || has.add("dojo-v1x-i18n-Api",
		// if true, define the v1.x i18n functions
		1
	);

	var
		thisModule = dojo.i18n =
			{
				// summary:
				//		This module implements the dojo/i18n! plugin and the v1.6- i18n API
				// description:
				//		We choose to include our own plugin to leverage functionality already contained in dojo
				//		and thereby reduce the size of the plugin compared to various loader implementations. Also, this
				//		allows foreign AMD loaders to be used without their plugins.
			},

		nlsRe =
			// regexp for reconstructing the master bundle name from parts of the regexp match
			// nlsRe.exec("foo/bar/baz/nls/en-ca/foo") gives:
			// ["foo/bar/baz/nls/en-ca/foo", "foo/bar/baz/nls/", "/", "/", "en-ca", "foo"]
			// nlsRe.exec("foo/bar/baz/nls/foo") gives:
			// ["foo/bar/baz/nls/foo", "foo/bar/baz/nls/", "/", "/", "foo", ""]
			// so, if match[5] is blank, it means this is the top bundle definition.
			// courtesy of http://requirejs.org
			/(^.*(^|\/)nls)(\/|$)([^\/]*)\/?([^\/]*)/,

		getAvailableLocales = function(
			root,
			locale,
			bundlePath,
			bundleName
		){
			// summary:
			//		return a vector of module ids containing all available locales with respect to the target locale
			//		For example, assuming:
			//
			//		- the root bundle indicates specific bundles for "fr" and "fr-ca",
			//		-  bundlePath is "myPackage/nls"
			//		- bundleName is "myBundle"
			//
			//		Then a locale argument of "fr-ca" would return
			//
			//			["myPackage/nls/myBundle", "myPackage/nls/fr/myBundle", "myPackage/nls/fr-ca/myBundle"]
			//
			//		Notice that bundles are returned least-specific to most-specific, starting with the root.
			//
			//		If root===false indicates we're working with a pre-AMD i18n bundle that doesn't tell about the available locales;
			//		therefore, assume everything is available and get 404 errors that indicate a particular localization is not available

			for(var result = [bundlePath + bundleName], localeParts = locale.split("-"), current = "", i = 0; i<localeParts.length; i++){
				current += (current ? "-" : "") + localeParts[i];
				if(!root || root[current]){
					result.push(bundlePath + current + "/" + bundleName);
				}
			}
			return result;
		},

		cache = {},

		getBundleName = function(moduleName, bundleName, locale){
			locale = locale ? locale.toLowerCase() : dojo.locale;
			moduleName = moduleName.replace(/\./g, "/");
			bundleName = bundleName.replace(/\./g, "/");
			return (/root/i.test(locale)) ?
				(moduleName + "/nls/" + bundleName) :
				(moduleName + "/nls/" + locale + "/" + bundleName);
		},

		getL10nName = dojo.getL10nName = function(moduleName, bundleName, locale){
			return moduleName = module.id + "!" + getBundleName(moduleName, bundleName, locale);
		},

		doLoad = function(require, bundlePathAndName, bundlePath, bundleName, locale, load){
			// summary:
			//		get the root bundle which instructs which other bundles are required to construct the localized bundle
			require([bundlePathAndName], function(root){
				var current = lang.clone(root.root || root.ROOT),// 1.6 built bundle defined ROOT
					availableLocales = getAvailableLocales(!root._v1x && root, locale, bundlePath, bundleName);
				require(availableLocales, function(){
					for (var i = 1; i<availableLocales.length; i++){
						current = lang.mixin(lang.clone(current), arguments[i]);
					}
					// target may not have been resolve (e.g., maybe only "fr" exists when "fr-ca" was requested)
					var target = bundlePathAndName + "/" + locale;
					cache[target] = current;
					load();
				});
			});
		},

		normalize = function(id, toAbsMid){
			// summary:
			//		id may be relative.
			//		preload has form `*preload*<path>/nls/<module>*<flattened locales>` and
			//		therefore never looks like a relative
			return /^\./.test(id) ? toAbsMid(id) : id;
		},

		getLocalesToLoad = function(targetLocale){
			var list = config.extraLocale || [];
			list = lang.isArray(list) ? list : [list];
			list.push(targetLocale);
			return list;
		},

		load = function(id, require, load){
			// summary:
			//		id is in one of the following formats
			//
			//		1. <path>/nls/<bundle>
			//			=> load the bundle, localized to config.locale; load all bundles localized to
			//			config.extraLocale (if any); return the loaded bundle localized to config.locale.
			//
			//		2. <path>/nls/<locale>/<bundle>
			//			=> load then return the bundle localized to <locale>
			//
			//		3. *preload*<path>/nls/<module>*<JSON array of available locales>
			//			=> for config.locale and all config.extraLocale, load all bundles found
			//			in the best-matching bundle rollup. A value of 1 is returned, which
			//			is meaningless other than to say the plugin is executing the requested
			//			preloads
			//
			//		In cases 1 and 2, <path> is always normalized to an absolute module id upon entry; see
			//		normalize. In case 3, it <path> is assumed to be absolute; this is arranged by the builder.
			//
			//		To load a bundle means to insert the bundle into the plugin's cache and publish the bundle
			//		value to the loader. Given <path>, <bundle>, and a particular <locale>, the cache key
			//
			//			<path>/nls/<bundle>/<locale>
			//
			//		will hold the value. Similarly, then plugin will publish this value to the loader by
			//
			//			define("<path>/nls/<bundle>/<locale>", <bundle-value>);
			//
			//		Given this algorithm, other machinery can provide fast load paths be preplacing
			//		values in the plugin's cache, which is public. When a load is demanded the
			//		cache is inspected before starting any loading. Explicitly placing values in the plugin
			//		cache is an advanced/experimental feature that should not be needed; use at your own risk.
			//
			//		For the normal AMD algorithm, the root bundle is loaded first, which instructs the
			//		plugin what additional localized bundles are required for a particular locale. These
			//		additional locales are loaded and a mix of the root and each progressively-specific
			//		locale is returned. For example:
			//
			//		1. The client demands "dojo/i18n!some/path/nls/someBundle
			//
			//		2. The loader demands load(some/path/nls/someBundle)
			//
			//		3. This plugin require's "some/path/nls/someBundle", which is the root bundle.
			//
			//		4. Assuming config.locale is "ab-cd-ef" and the root bundle indicates that localizations
			//		are available for "ab" and "ab-cd-ef" (note the missing "ab-cd", then the plugin
			//		requires "some/path/nls/ab/someBundle" and "some/path/nls/ab-cd-ef/someBundle"
			//
			//		5. Upon receiving all required bundles, the plugin constructs the value of the bundle
			//		ab-cd-ef as...
			//
			//				mixin(mixin(mixin({}, require("some/path/nls/someBundle"),
			//		  			require("some/path/nls/ab/someBundle")),
			//					require("some/path/nls/ab-cd-ef/someBundle"));
			//
			//		This value is inserted into the cache and published to the loader at the
			//		key/module-id some/path/nls/someBundle/ab-cd-ef.
			//
			//		The special preload signature (case 3) instructs the plugin to stop servicing all normal requests
			//		(further preload requests will be serviced) until all ongoing preloading has completed.
			//
			//		The preload signature instructs the plugin that a special rollup module is available that contains
			//		one or more flattened, localized bundles. The JSON array of available locales indicates which locales
			//		are available. Here is an example:
			//
			//			*preload*some/path/nls/someModule*["root", "ab", "ab-cd-ef"]
			//
			//		This indicates the following rollup modules are available:
			//
			//			some/path/nls/someModule_ROOT
			//			some/path/nls/someModule_ab
			//			some/path/nls/someModule_ab-cd-ef
			//
			//		Each of these modules is a normal AMD module that contains one or more flattened bundles in a hash.
			//		For example, assume someModule contained the bundles some/bundle/path/someBundle and
			//		some/bundle/path/someOtherBundle, then some/path/nls/someModule_ab would be expressed as follows:
			//
			//			define({
			//				some/bundle/path/someBundle:<value of someBundle, flattened with respect to locale ab>,
			//				some/bundle/path/someOtherBundle:<value of someOtherBundle, flattened with respect to locale ab>,
			//			});
			//
			//		E.g., given this design, preloading for locale=="ab" can execute the following algorithm:
			//
			//			require(["some/path/nls/someModule_ab"], function(rollup){
			//				for(var p in rollup){
			//					var id = p + "/ab",
			//					cache[id] = rollup[p];
			//					define(id, rollup[p]);
			//				}
			//			});
			//
			//		Similarly, if "ab-cd" is requested, the algorithm can determine that "ab" is the best available and
			//		load accordingly.
			//
			//		The builder will write such rollups for every layer if a non-empty localeList  profile property is
			//		provided. Further, the builder will include the following cache entry in the cache associated with
			//		any layer.
			//
			//			"*now":function(r){r(['dojo/i18n!*preload*<path>/nls/<module>*<JSON array of available locales>']);}
			//
			//		The *now special cache module instructs the loader to apply the provided function to context-require
			//		with respect to the particular layer being defined. This causes the plugin to hold all normal service
			//		requests until all preloading is complete.
			//
			//		Notice that this algorithm is rarely better than the standard AMD load algorithm. Consider the normal case
			//		where the target locale has a single segment and a layer depends on a single bundle:
			//
			//		Without Preloads:
			//
			//		1. Layer loads root bundle.
			//		2. bundle is demanded; plugin loads single localized bundle.
			//
			//		With Preloads:
			//
			//		1. Layer causes preloading of target bundle.
			//		2. bundle is demanded; service is delayed until preloading complete; bundle is returned.
			//
			//		In each case a single transaction is required to load the target bundle. In cases where multiple bundles
			//		are required and/or the locale has multiple segments, preloads still requires a single transaction whereas
			//		the normal path requires an additional transaction for each additional bundle/locale-segment. However all
			//		of these additional transactions can be done concurrently. Owing to this analysis, the entire preloading
			//		algorithm can be discard during a build by setting the has feature dojo-preload-i18n-Api to false.

			if(has("dojo-preload-i18n-Api")){
				var split = id.split("*"),
					preloadDemand = split[1] == "preload";
				if(preloadDemand){
					if(!cache[id]){
						// use cache[id] to prevent multiple preloads of the same preload; this shouldn't happen, but
						// who knows what over-aggressive human optimizers may attempt
						cache[id] = 1;
						preloadL10n(split[2], json.parse(split[3]), 1, require);
					}
					// don't stall the loader!
					load(1);
				}
				if(preloadDemand || waitForPreloads(id, require, load)){
					return;
				}
			}

			var match = nlsRe.exec(id),
				bundlePath = match[1] + "/",
				bundleName = match[5] || match[4],
				bundlePathAndName = bundlePath + bundleName,
				localeSpecified = (match[5] && match[4]),
				targetLocale =	localeSpecified || dojo.locale,
				loadTarget = bundlePathAndName + "/" + targetLocale,
				loadList = localeSpecified ? [targetLocale] : getLocalesToLoad(targetLocale),
				remaining = loadList.length,
				finish = function(){
					if(!--remaining){
						load(lang.delegate(cache[loadTarget]));
					}
				};
			array.forEach(loadList, function(locale){
				var target = bundlePathAndName + "/" + locale;
				if(has("dojo-preload-i18n-Api")){
					checkForLegacyModules(target);
				}
				if(!cache[target]){
					doLoad(require, bundlePathAndName, bundlePath, bundleName, locale, finish);
				}else{
					finish();
				}
			});
		};

	if(has("dojo-unit-tests")){
		var unitTests = thisModule.unitTests = [];
	}

	if(has("dojo-preload-i18n-Api") ||  1 ){
		var normalizeLocale = thisModule.normalizeLocale = function(locale){
				var result = locale ? locale.toLowerCase() : dojo.locale;
				return result == "root" ? "ROOT" : result;
			},

			isXd = function(mid, contextRequire){
				return ( 1  &&  1 ) ?
					contextRequire.isXdUrl(require.toUrl(mid + ".js")) :
					true;
			},

			preloading = 0,

			preloadWaitQueue = [],

			preloadL10n = thisModule._preloadLocalizations = function(/*String*/bundlePrefix, /*Array*/localesGenerated, /*boolean?*/ guaranteedAmdFormat, /*function?*/ contextRequire){
				// summary:
				//		Load available flattened resource bundles associated with a particular module for dojo/locale and all dojo/config.extraLocale (if any)
				// description:
				//		Only called by built layer files. The entire locale hierarchy is loaded. For example,
				//		if locale=="ab-cd", then ROOT, "ab", and "ab-cd" are loaded. This is different than v1.6-
				//		in that the v1.6- would only load ab-cd...which was *always* flattened.
				//
				//		If guaranteedAmdFormat is true, then the module can be loaded with require thereby circumventing the detection algorithm
				//		and the extra possible extra transaction.

				// If this function is called from legacy code, then guaranteedAmdFormat and contextRequire will be undefined. Since the function
				// needs a require in order to resolve module ids, fall back to the context-require associated with this dojo/i18n module, which
				// itself may have been mapped.
				contextRequire = contextRequire || require;

				function doRequire(mid, callback){
					if(isXd(mid, contextRequire) || guaranteedAmdFormat){
						contextRequire([mid], callback);
					}else{
						syncRequire([mid], callback, contextRequire);
					}
				}

				function forEachLocale(locale, func){
					// given locale= "ab-cd-ef", calls func on "ab-cd-ef", "ab-cd", "ab", "ROOT"; stops calling the first time func returns truthy
					var parts = locale.split("-");
					while(parts.length){
						if(func(parts.join("-"))){
							return;
						}
						parts.pop();
					}
					func("ROOT");
				}

					function preloadingAddLock(){
						preloading++;
					}

					function preloadingRelLock(){
						--preloading;
						while(!preloading && preloadWaitQueue.length){
							load.apply(null, preloadWaitQueue.shift());
						}
					}

					function cacheId(path, name, loc, require){
						// path is assumed to have a trailing "/"
						return require.toAbsMid(path + name + "/" + loc)
					}

					function preload(locale){
						locale = normalizeLocale(locale);
						forEachLocale(locale, function(loc){
							if(array.indexOf(localesGenerated, loc) >= 0){
								var mid = bundlePrefix.replace(/\./g, "/") + "_" + loc;
								preloadingAddLock();
								doRequire(mid, function(rollup){
									for(var p in rollup){
										var bundle = rollup[p],
											match = p.match(/(.+)\/([^\/]+)$/),
											bundleName, bundlePath;
											
											// If there is no match, the bundle is not a regular bundle from an AMD layer.
											if (!match){continue;}

											bundleName = match[2];
											bundlePath = match[1] + "/";

										// backcompat
										bundle._localized = bundle._localized || {};

										var localized;
										if(loc === "ROOT"){
											var root = localized = bundle._localized;
											delete bundle._localized;
											root.root = bundle;
											cache[require.toAbsMid(p)] = root;
										}else{
											localized = bundle._localized;
											cache[cacheId(bundlePath, bundleName, loc, require)] = bundle;
										}

										if(loc !== locale){
											// capture some locale variables
											function improveBundle(bundlePath, bundleName, bundle, localized){
												// locale was not flattened and we've fallen back to a less-specific locale that was flattened
												// for example, we had a flattened 'fr', a 'fr-ca' is available for at least this bundle, and
												// locale==='fr-ca'; therefore, we must improve the bundle as retrieved from the rollup by
												// manually loading the fr-ca version of the bundle and mixing this into the already-retrieved 'fr'
												// version of the bundle.
												//
												// Remember, different bundles may have different sets of locales available.
												//
												// we are really falling back on the regular algorithm here, but--hopefully--starting with most
												// of the required bundles already on board as given by the rollup and we need to "manually" load
												// only one locale from a few bundles...or even better...we won't find anything better to load.
												// This algorithm ensures there is nothing better to load even when we can only load a less-specific rollup.
												//
												// note: this feature is only available in async mode

												// inspect the loaded bundle that came from the rollup to see if something better is available
												// for any bundle in a rollup, more-specific available locales are given at localized.
												var requiredBundles = [],
													cacheIds = [];
												forEachLocale(locale, function(loc){
													if(localized[loc]){
														requiredBundles.push(require.toAbsMid(bundlePath + loc + "/" + bundleName));
														cacheIds.push(cacheId(bundlePath, bundleName, loc, require));
													}
												});

												if(requiredBundles.length){
													preloadingAddLock();
													contextRequire(requiredBundles, function(){
														for(var i = 0; i < requiredBundles.length; i++){
															bundle = lang.mixin(lang.clone(bundle), arguments[i]);
															cache[cacheIds[i]] = bundle;
														}
														// this is the best possible (maybe a perfect match, maybe not), accept it
														cache[cacheId(bundlePath, bundleName, locale, require)] = lang.clone(bundle);
														preloadingRelLock();
													});
												}else{
													// this is the best possible (definitely not a perfect match), accept it
													cache[cacheId(bundlePath, bundleName, locale, require)] = bundle;
												}
											}
											improveBundle(bundlePath, bundleName, bundle, localized);
										}
									}
									preloadingRelLock();
								});
								return true;
							}
							return false;
						});
					}

				preload();
				array.forEach(dojo.config.extraLocale, preload);
			},

			waitForPreloads = function(id, require, load){
				if(preloading){
					preloadWaitQueue.push([id, require, load]);
				}
				return preloading;
			},

			checkForLegacyModules = function()
				{};
	}

	if( 1 ){
		// this code path assumes the dojo loader and won't work with a standard AMD loader
		var amdValue = {},
			evalBundle =
				// use the function ctor to keep the minifiers away (also come close to global scope, but this is secondary)
				new Function(
					"__bundle",				   // the bundle to evalutate
					"__checkForLegacyModules", // a function that checks if __bundle defined __mid in the global space
					"__mid",				   // the mid that __bundle is intended to define
					"__amdValue",

					// returns one of:
					//		1 => the bundle was an AMD bundle
					//		a legacy bundle object that is the value of __mid
					//		instance of Error => could not figure out how to evaluate bundle

					  // used to detect when __bundle calls define
					  "var define = function(mid, factory){define.called = 1; __amdValue.result = factory || mid;},"
					+ "	   require = function(){define.called = 1;};"

					+ "try{"
					+		"define.called = 0;"
					+		"eval(__bundle);"
					+		"if(define.called==1)"
								// bundle called define; therefore signal it's an AMD bundle
					+			"return __amdValue;"

					+		"if((__checkForLegacyModules = __checkForLegacyModules(__mid)))"
								// bundle was probably a v1.6- built NLS flattened NLS bundle that defined __mid in the global space
					+			"return __checkForLegacyModules;"

					+ "}catch(e){}"
					// evaulating the bundle was *neither* an AMD *nor* a legacy flattened bundle
					// either way, re-eval *after* surrounding with parentheses

					+ "try{"
					+		"return eval('('+__bundle+')');"
					+ "}catch(e){"
					+		"return e;"
					+ "}"
				),

			syncRequire = function(deps, callback, require){
				var results = [];
				array.forEach(deps, function(mid){
					var url = require.toUrl(mid + ".js");

					function load(text){
						var result = evalBundle(text, checkForLegacyModules, mid, amdValue);
						if(result===amdValue){
							// the bundle was an AMD module; re-inject it through the normal AMD path
							// we gotta do this since it could be an anonymous module and simply evaluating
							// the text here won't provide the loader with the context to know what
							// module is being defined()'d. With browser caching, this should be free; further
							// this entire code path can be circumvented by using the AMD format to begin with
							results.push(cache[url] = amdValue.result);
						}else{
							if(result instanceof Error){
								console.error("failed to evaluate i18n bundle; url=" + url, result);
								result = {};
							}
							// nls/<locale>/<bundle-name> indicates not the root.
							results.push(cache[url] = (/nls\/[^\/]+\/[^\/]+$/.test(url) ? result : {root:result, _v1x:1}));
						}
					}

					if(cache[url]){
						results.push(cache[url]);
					}else{
						var bundle = require.syncLoadNls(mid);
						// don't need to check for legacy since syncLoadNls returns a module if the module
						// (1) was already loaded, or (2) was in the cache. In case 1, if syncRequire is called
						// from getLocalization --> load, then load will have called checkForLegacyModules() before
						// calling syncRequire; if syncRequire is called from preloadLocalizations, then we
						// don't care about checkForLegacyModules() because that will be done when a particular
						// bundle is actually demanded. In case 2, checkForLegacyModules() is never relevant
						// because cached modules are always v1.7+ built modules.
						if(bundle){
							results.push(bundle);
						}else{
							if(!xhr){
								try{
									require.getText(url, true, load);
								}catch(e){
									results.push(cache[url] = {});
								}
							}else{
								xhr.get({
									url:url,
									sync:true,
									load:load,
									error:function(){
										results.push(cache[url] = {});
									}
								});
							}
						}
					}
				});
				callback && callback.apply(null, results);
			};

		checkForLegacyModules = function(target){
			// legacy code may have already loaded [e.g] the raw bundle x/y/z at x.y.z; when true, push into the cache
			for(var result, names = target.split("/"), object = dojo.global[names[0]], i = 1; object && i<names.length-1; object = object[names[i++]]){}
			if(object){
				result = object[names[i]];
				if(!result){
					// fallback for incorrect bundle build of 1.6
					result = object[names[i].replace(/-/g,"_")];
				}
				if(result){
					cache[target] = result;
				}
			}
			return result;
		};

		thisModule.getLocalization = function(moduleName, bundleName, locale){
			var result,
				l10nName = getBundleName(moduleName, bundleName, locale);
			load(
				l10nName,

				// isXd() and syncRequire() need a context-require in order to resolve the mid with respect to a reference module.
				// Since this legacy function does not have the concept of a reference module, resolve with respect to this
				// dojo/i18n module, which, itself may have been mapped.
				(!isXd(l10nName, require) ? function(deps, callback){ syncRequire(deps, callback, require); } : require),

				function(result_){ result = result_; }
			);
			return result;
		};

		if(has("dojo-unit-tests")){
			unitTests.push(function(doh){
				doh.register("tests.i18n.unit", function(t){
					var check;

					check = evalBundle("{prop:1}", checkForLegacyModules, "nonsense", amdValue);
					t.is({prop:1}, check); t.is(undefined, check[1]);

					check = evalBundle("({prop:1})", checkForLegacyModules, "nonsense", amdValue);
					t.is({prop:1}, check); t.is(undefined, check[1]);

					check = evalBundle("{'prop-x':1}", checkForLegacyModules, "nonsense", amdValue);
					t.is({'prop-x':1}, check); t.is(undefined, check[1]);

					check = evalBundle("({'prop-x':1})", checkForLegacyModules, "nonsense", amdValue);
					t.is({'prop-x':1}, check); t.is(undefined, check[1]);

					check = evalBundle("define({'prop-x':1})", checkForLegacyModules, "nonsense", amdValue);
					t.is(amdValue, check); t.is({'prop-x':1}, amdValue.result);

					check = evalBundle("define('some/module', {'prop-x':1})", checkForLegacyModules, "nonsense", amdValue);
					t.is(amdValue, check); t.is({'prop-x':1}, amdValue.result);

					check = evalBundle("this is total nonsense and should throw an error", checkForLegacyModules, "nonsense", amdValue);
					t.is(check instanceof Error, true);
				});
			});
		}
	}

	return lang.mixin(thisModule, {
		dynamic:true,
		normalize:normalize,
		load:load,
		cache:cache
	});
});

},
'epi/dependency':function(){
﻿define("epi/dependency", ["dojo/_base/declare", "epi"], function (declare, epi) {

    var Dependency = declare([], {
        // tags:
        //      public

        _dependencies: null,

        constructor: function () {
            this._dependencies = {};
        },

        register: function (/*String*/identifier, /*Object|Function*/value, /*Function?*/unregisterFunction) {
            // summary:
            //      Register a dependency
            //
            // description:
            //		Register an object
            //
            // identifier:
            //      The name for the registered dependency
            //
            // value:
            //		The value to store.
            //		If the value is an object, the value will be returned on resolve.
            //		If value is a class constructor, a new instance will be created and returned on first resolve
            //      If value is a function, the result of that function will be returned on first resolve
            //
            // unregisterFunction:
            //		Function to run when dependency is unregistered
            //
            // example:
            //		Register dependency with an object instance
            //	|	require(["epi/dependency", "MyClass"], function (dependency, MyClass) {
            //	|	    dependency.register("MyClass", new MyClass());
            //	|	});
            //
            // example:
            //		Register dependency with a class constructor. An instance of epi/shell/widget/SuperWidgetFactory will be
            //		created upon first resolve and that instance will be returned for all resolves thereafter
            //	|	require(["epi/dependency", "epi/SuperWidgetFactory"], function (dependency, SuperWidgetFactory) {
            //	|	    dependency.register("epi.shell.widget.WidgetFactory", SuperWidgetFactory);
            //	|	});
            //
            // example:
            //		Register dependency with a function.
            //		The function will be called on each resolve.
            //		In this case a new Date object will be returned.
            //	|	dependency.register("epi.shell.widget.WidgetFactory", function() {
            //	|		return new Date();
            //	|	});
            //
            // example:
            //		Register dependency with an unregisterFunction. The function will be called when dependency is unregistered.
            //	|	dependency.register("ComponentController", new ComponentController(), function(controller, initValue) {
            //	|		// do any clean up here
            //	|		controller.destroy();
            //	|	});
            //
            // tags:
            //		public
            //

            var valueIsFunction = (typeof value == "function");
            var valueIsConstructor = (valueIsFunction && "superclass" in value);
            var initialized = (!valueIsFunction && !valueIsConstructor);
            var initFunction;

            if (valueIsConstructor) {
                // create a lazy initFunction when value is a constructor
                var cls = value;
                initFunction = function () {
                    return new cls();
                };
            } else if (valueIsFunction) {
                initFunction = value;
            }

            // unregister any previous value
            this.unregister(identifier);

            // bind value to identifier
            this._dependencies[identifier] = {
                value: initialized ? value : undefined,
                initFunction: initFunction,
                initialized: initialized,
                unregisterFunction: unregisterFunction
            };
        },

        unregister: function (/*String*/identifier) {
            // summary:
            //      Removes a registered dependency. Calls the unregister function if one was register for this value
            //
            // identifier: String
            //      The name for the registered dependency
            //
            // tags:
            //		public
            //

            if (!(identifier in this._dependencies)) {
                return;
            }

            var dep = this._dependencies[identifier];
            if (dep.initialized && typeof dep.unregisterFunction == "function") {
                dep.unregisterFunction(dep.value);
            }

            delete this._dependencies[identifier];
        },

        resolve: function (/*String*/identifier) {
            // summary:
            //      Resolves a dependency
            //
            // identifier: String
            //      The name for the registered dependency
            //
            // example:
            //		Resolve a dependency
            //	|	dojo.require("MyClass");
            //  |   var x = new MyClass();
            //	|	dependency.register("MyClass", x);
            //  |   dependency.resolve("MyClass") == x // => true
            //
            // tags:
            //		public
            //
            // returns:
            //		The registered value. Throws an error if no value is registered for this identifier
            //

            if (!(identifier in this._dependencies)) {
                throw new Error("Could not resolve dependency \"" + identifier + "\" (epi.dependency)");
            }

            var dep = this._dependencies[identifier];

            // resolve any lazy dependency
            if (!dep.initialized) {
                // set value to result of value;
                dep.value = dep.initFunction();
                dep.initialized = true;
            }

            return dep.value; // Object
        },

        exists: function (identifier) {
            // summary:
            //      Checks if an identifier exists in the dependency registry
            // identifier: String
            //      The name for the registered dependency
            return (identifier in this._dependencies);
        },

        list: function () {
            // summary:
            //      List registered dependencies.
            //
            // tags:
            //		public
            //
            // returns:
            //		An array with the registered identifiers
            //

            var identifiers = [];
            for (var identifier in this._dependencies) {
                identifiers.push(identifier);
            }
            return identifiers; // String[]
        },

        clear: function () {
            // summary:
            //      Reset dependency.
            //
            // description:
            //      For tests purpose only.
            //
            // tags:
            //		internal
            //

            for (var identifier in this._dependencies) {
                this.unregister(identifier);
            }

            this._dependencies = {};
        }
    });

    return epi.dependency = new Dependency();
});

},
'epi/routes':function(){
﻿(function () {
    /* jshint jquery: true */

    var routes = {
        // tags:
        //      public


        registeredRoutes: {},
        defaultModuleArea: null,

        init: function (defaultModuleArea) {
            // summary:
            //     Sets up default module area for route resolving
            // defaultModuleArea:
            //     The default moduleArea routed to when moduleArea isn't specified
            this.defaultModuleArea = defaultModuleArea;
        },

        registerRoute: function (routeFormat, parameters) {
            // summary:
            //     Register a route and default parameters
            // routeFormat:
            //     A name/value collection of route parameters
            // parameters:
            //     Default paramters when resolving the route
            this.registeredRoutes[parameters.moduleArea.toLowerCase()] = { url: routeFormat, defaults: parameters };
        },

        getActionPath: function (parameters, customPath) {
            // summary:
            //     Generates a route path for shell controller actions
            // parameters:
            //     A name/value collection of route parameters
            // customPath:
            //     A custom path to use when resolving the route

            var routeInfo = null;
            if (typeof customPath == "string") {
                routeInfo = { url: customPath, defaults: {} };
            } else if (parameters == null || typeof parameters["moduleArea"] != "string") {
                routeInfo = this.registeredRoutes[this.defaultModuleArea.toLowerCase()];
            } else {
                routeInfo = this.registeredRoutes[parameters["moduleArea"].toLowerCase()];
            }

            if (!routeInfo) {
                throw new Error("Failed to resolve a route from parameters or default settings");
            }

            var replace = function (url, key, value) {
                var formatKey = "{" + key + "}";
                var keyIndex = url.indexOf(formatKey, 0);
                if (keyIndex >= 0) {
                    return url.substring(0, keyIndex) + value + url.substring(keyIndex + formatKey.length);
                }

                var queryIndex = url.indexOf("?");
                return url + (queryIndex >= 0 ? "&" : "?") + key + "=" + value;
            };

            // Merge default context parameters and and the route parameters into the mergeParameter object
            // route parameters will override context parameters
            var merge = (typeof dojo === "object" ? dojo.mixin : $.extend);
            var mergedParameters = merge({}, routeInfo.defaults, parameters);
            var path = routeInfo.url;

            for (var key in mergedParameters) {
                path = replace(path, key, mergedParameters[key]);
            }
            var removeExtraSlashRegex = /\/+[?]/;
            //Remove any '/' characters located before the query part (?)
            path = path.replace(removeExtraSlashRegex, '?');
            return path;
        },

        getRestPath: function (/*Object*/parameters, /*String*/customPath) {
            // summary:
            //      Generates a path to the rest store.
            // tags:
            //      public

            var storeName = parameters.storeName;
            delete parameters.storeName;
            return this.getActionPath(dojo.mixin(parameters, { action: storeName, controller: "Stores" }), customPath);
        },

        getRouteParams: function (routeUrl, url) {
            // summary:
            //     Parses an url to extract route parameters from it. Returns a named collection of the parameters found
            // routeUrl:
            //     The route template to containing parameters to find. "/routes/{moduleArea}/{controller}/{action}/{id}"
            // url:
            //     The url containing the route parameters to extract.

            var splitRegEx = /[\/|?]/g;
            // Create url object from string
            var urlParts = this.parseUrl(url);
            // Split path on / and ?
            var pathSections = urlParts.path.split(splitRegEx);
            // Split route-url on / and ?
            var routeSections = routeUrl.split(splitRegEx);
            var routeParams = {};

            // Loop over the route url sections and match with the corrensponding section on the path
            for (var i = 0; i < routeSections.length; i++) {
                // Check if the section is supposed to be replaceble
                if (routeSections[i].charAt(0) === '{') {
                    // Cut off { and } from the name
                    var name = routeSections[i].substring(1, routeSections[i].length - 1);
                    // Set the value to the section of the url or an empty string of not existing (probably default action)
                    var value = pathSections.length > i ? pathSections[i] : "";
                    routeParams[name] = value;
                }
            }

            // Add the parameters from the query string collection
            return $.extend(routeParams, urlParts.queryParams);
        },

        parseUrl: function (urlToParse) {
            // summary:
            //     Creates an object with named properties of url parts
            //     Based on parseUri by Steven Levithan http://blog.stevenlevithan.com/archives/parseuri
            // urlToParse:
            //     a url to parse into parts
            var url = {};
            var matches = /^(?:([^:\/?#]+):)?(?:\/\/((?:(([^:@]*)(?::([^:@]*))?)?@)?([^:\/?#]*)(?::(\d*))?))?((((?:[^?#\/]*\/)*)([^?#]*))(?:\?([^#]*))?(?:#(.*))?)/.exec(urlToParse);
            var keys = ["source", "protocol", "authority", "userInfo", "user", "password", "host", "port", "relative", "path", "directory", "file", "query", "anchor"];
            var i = keys.length;

            while (i--) {
                url[keys[i]] = matches[i] || "";
            }

            url.queryParams = {};
            url.query.replace(/(?:^|&)([^&=]*)=?([^&]*)/g, function ($0, $1, $2) {
                if ($1) {
                    url.queryParams[$1] = $2;
                }
            });

            return url;
        }
    };

    if (typeof define != "undefined") {
        define("epi/routes", ["epi"], function (epi) {
            return epi.routes = routes;
        });
    } else {
        // When not using define, i.e. not in dojo-land, epi is globally available.
        // But if it hasn't been defined yet, we have to define it.
        var e = window.epi || {};
        e.routes = routes;
    }
})();

},
'epi/clientResourcesLoader':function(){
﻿define("epi/clientResourcesLoader", [
    "dojo/_base/array",
    "dojo/_base/lang",
    "dojo/_base/window",
    "dojo/dom-construct",
    "dojo/Deferred",
    "dojo/query",
    "epi"
], function (array, lang, win, dom, Deferred, query, epi) {
    "use strict";
    var loadedStyles = {},
        loadedScripts = {};

    return epi.clientResourcesLoader = {
        // tags:
        //      internal

        _loadStyles: function (stylePaths) {
            if (!lang.isArray(stylePaths)) {
                return;
            }

            array.forEach(stylePaths, function (stylePath) {

                // use lower case path as key
                var stylePathLower = stylePath.toLowerCase();

                if (!(stylePathLower in loadedStyles)) {
                    var head = query("head")[0];
                    dom.create("link", { rel: "stylesheet", type: "text/css", href: stylePath }, head);
                    loadedStyles[stylePathLower] = 1;
                } else {
                    // increase count
                    loadedStyles[stylePathLower] += 1;
                }
            });
        },

        _loadScripts: function (scriptPaths) {
            var deferred = new Deferred();

            var loadedInDOM = function (scriptPath) {
                var scriptNodeId = scriptPath.toLowerCase().replace(/[^A-z0-9$_]/g, "_");
                return !!win.doc.getElementById(scriptNodeId);
            };

            var getScriptDependencies = function (scriptsToLoad) {
                // Called recursively to load the scripts in fake sync mode, resolving the deferred when done.

                // No more scripts to load; resolve and return
                if (scriptsToLoad.length === 0) {
                    deferred.resolve();
                    return;
                }

                // Shift out the first element from the array.
                var scriptPath = scriptsToLoad.shift();

                // Add it to the dictionary of scripts that has been loaded
                require([scriptPath], function () {
                    loadedScripts[scriptPath] = true;
                    // Call recursively for the remaining scripts in the array
                    getScriptDependencies(scriptsToLoad);
                });
            };

            // Add all scripts that aren't already loaded to an array
            var scriptsToLoad = [];
            if (lang.isArray(scriptPaths)) {
                scriptsToLoad = array.filter(scriptPaths, function (scriptPath) {
                    return !((scriptPath in loadedScripts) || loadedInDOM(scriptPath));
                });
            }

            getScriptDependencies(scriptsToLoad);

            return deferred;
        },

        loadResources: function (/* String[]? */stylePaths, /* String[]? */scriptPaths) {
            // summary:
            //      Loads style and script resources
            //
            // stylePaths: String[]?
            //      Paths to stylesheet files
            //
            // scriptPaths: String[]?
            //      Paths to script files
            //
            // returns:
            //  A promise which is resolved when all scripts has been loaded
            //
            // example:
            //      Load a style file
            //  |   clientResourcesLoader.loadResources(["/css/styles.css"]);
            //
            // example:
            //      Load a script file
            //  |   clientResourcesLoader.loadResources(undefined, ["/scripts/script.js"]);
            //
            // example:
            //      Load a multiple files
            //  |   clientResourcesLoader.loadResources(["/css/styles.css", "/css/form.css"], ["/scripts/script.js"]);
            //
            // tags:
            //		public

            // Start with styles, they can be loaded in parallel
            this._loadStyles(stylePaths);

            // Return the deferred from script loading, since we need to know when all scripts has been loaded
            return this._loadScripts(scriptPaths);
        },

        reset: function () {
            // summary:
            //      Reset the loaded script information to its initial state.
            //      Primarily for unit test purposes.
            // tags:
            //      public

            loadedScripts = {};
            loadedStyles = {};
        }
    };
});

},
'epi/_Module':function(){
﻿define("epi/_Module", [
    "dojo/_base/declare",
    "epi/dependency"
], function (
    declare,
    dependency
) {

    return declare(null, {
        // summary:
        //		Base class for Modules.
        // tags:
        //      public

        //  _initialized: [protected] Boolean
        //      initialize() has been completed
        _initialized: false,

        //  _settings: [protected] Object
        //      Settings object
        _settings: null,

        //  _registeredDependencies: [private] String[]
        //      Array of registered dependencies for a module instance
        _registeredDependencies: null,

        constructor: function (settings) {
            this._settings = settings;
            this._registeredDependencies = [];
            this._initialized = false;
        },

        initialize: function () {
            // summary:
            //		Initialize module
            //
            this._initialized = true;
        },

        uninitialize: function () {
            // summary:
            //		Uninitalize module
            //
            this.unregisterDependencies();
        },

        registerDependency: function (/*String*/name, /*Object|Function*/value, /*Function?*/initFunction, /*Function?*/unregisterFunction) {
            // summary:
            //      Register a dependency
            //
            // identifier:
            //      The name for the registered dependency
            //
            // value:
            //		The value to store.
            //		If the value is an object, the value will be returned on resolve.
            //		If value is a string and initFunction is not defined, value is assumed to be the name of a class and will be registered as a lazy dependency which
            //		will be resolved first time.
            //		If the value is a function it will be called on each resolve
            //
            // initFunction:
            //		Function to run first time dependency is resolved.
            //
            // unregisterFunction:
            //		Function to run when dependency is unregistered
            //
            // example:
            //		Register dependency with an object instance
            //	|	var module = new MyModule(); // must be sub class of epi._Module
            //	|	require(["MyClass"], function (MyClass) {
            //	|	    module.registerDependency("MyClass", new MyClass());
            //	|	});
            //
            // example:
            //		Register dependency with a class name. An instance of epi/shell/widget/SuperWidgetFactory will be required
            //		and created upon first resolve.
            //	|	var module = new MyModule(); // must be sub class of epi._Module
            //	|	module.registerDependency("epi.shell.widget.WidgetFactory", "epi/shell/widget/SuperWidgetFactory");
            //
            // example:
            //		Register dependency with a function.
            //		The function will be called on each resolve.
            //		In this case a new Date object will be returned.
            //	|	var module = new MyModule(); // must be sub class of epi._Module
            //	|	module.registerDependency("epi.shell.widget.WidgetFactory", function() {
            //	|		return new Date();
            //	|	});
            //
            // example:
            //		Register dependency with an initFunction. The function will only run once
            //	|	var module = new MyModule(); // must be sub class of epi._Module
            //	|	module.registerDependency("MyClass", "a string", function(initValue) {
            //	|		// do any require
            //	|		dojo.require("MyClass");
            //	|		// return an instance, initValue will be "a string" in this example
            //	|		return new MyClass(initValue);
            //	|	});
            //
            // example:
            //		Register dependency with an unregisterFunction. The function will be called when dependency is unregistered.
            //	|	var module = new MyModule(); // must be sub class of epi._Module
            //	|	module.registerDependency("ComponentController", new ComponentController(), undefined, function(value, initValue) {
            //	|		// do any clean up here
            //	|		value.destroy();
            //	|	});
            //
            // tags:
            //		public
            //

            // add to list of my dependencies
            this._registeredDependencies.push(name);
            dependency.register(name, value, initFunction, unregisterFunction);
        },

        unregisterDependencies: function () {
            // summary:
            //		Unregisters all dependencies for this module
            //
            this._registeredDependencies.forEach(function (entry, index) {
                dependency.unregister(entry);
            }, this);
            this._registeredDependencies = [];
            this._initialized = false;
        },

        resolveDependency: function (identifier) {
            // summary:
            //  Get a dependency by its identifier
            //
            // identifier: String
            //  The unique name of the dependency to get.

            return dependency.resolve(identifier);
        },

        unregisterDependency: function (identifier) {
            // summary:
            //		Unregisters a specific dependency
            //
            // identifier: String
            //  The unique name of the dependency to get.

            dependency.unregister(identifier);
            this._registeredDependencies = this._registeredDependencies.filter(function (item) {
                return item !== identifier;
            });
        }
    });
});

},
'epi/ModuleManager':function(){
﻿define("epi/ModuleManager", [
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/Deferred",
    "dojo/promise/all",
    "dojo/when",

    "epi/clientResourcesLoader",
    "epi/routes"
], function (
    array,
    declare,
    lang,
    Deferred,
    promiseAll,
    when,

    clientResourceLoader,
    routes
) {

    // Module dependency type enum
    var moduleDependencyType = {
        require: 1,
        runAfter: 2
    };

    var ModuleManager = declare(null, {
        // summary:
        //      Manager for modules
        //
        // example:
        //      Pass the module settings in the constructor
        //  |   var modules = [
        //  |       {
        //  |           name: "Shell",
        //  |           initializer: "epi/shell/ShellModule",
        //  |           moduleDependencies: [],
        //  |           cssResources: [],
        //  |           scriptResources: [],
        //  |           routeBasePath: "/EPiServer/{moduleArea}/{controller}/{action}/{id}",
        //  |           routeDefaults: {
        //  |               moduleArea: "Shell",
        //  |               controller: "Dashboard",
        //  |               action: "Index",
        //  |               id: ""
        //  |           }
        //  |       },
        //  |       {
        //  |           name: "CMS",
        //  |           initializer: "epi-cms/CMSModule",
        //  |           moduleDependencies: [{
        //  |               moduleName: "Shell",
        //  |               dependencyType: "require"
        //  |           }],
        //  |           cssResources: [],
        //  |           scriptResources: [],
        //  |           routeBasePath: "/EPiServer/{moduleArea}/{controller}/{action}/{id}",
        //  |           routeDefaults: {
        //  |               id: "",
        //  |               moduleArea: "CMS"
        //  |           }
        //  |       }
        //  |   ]
        //  |   var moduleManager = new ModuleManager(modules);
        //

        // _allModules: [private] Object
        //      Module settings
        _allModules: null,

        // _moduleInstances: [private] Object
        //      Dictionary of modules by name
        _moduleInstances: null,

        // _modulesConfigured: [private] Boolean
        //      A flag indicating if module configuration has been added to the application
        _modulesConfigured: false,

        constructor: function (modules) {
            // modules: Array
            //      An array with module settings.
            // tags:
            //      public
            this._allModules = {};
            this._moduleInstances = {};

            array.forEach(modules, function (module) {
                this._allModules[module.name] = module;
            }, this);
        },

        getModule: function (moduleName) {
            // summary:
            //      Get a module by name
            // tags:
            //      public
            return this._allModules[moduleName];
        },

        configureModules: function () {
            // summary:
            //      Adds module configuration to the application for all modules
            //
            // tags:
            //      public

            if (this._modulesConfigured) {
                return;
            }

            this._modulesConfigured = true;
            for (var moduleName in this._allModules) {
                this._configureModule(this._allModules[moduleName]);
            }
        },

        startModules: function (/*String*/moduleName) {
            // summary:
            //  Start the modules required for the application to run.
            //
            // modulesName:
            //  Name of a module to start. If undefined; all modules are started.
            //
            // returns:
            //  A promise resolved when required modules have been started.

            // Make sure modules settings have been added
            this.configureModules();

            if (moduleName) {
                return this._startModule(this._allModules[moduleName]);
            } else {
                var deferreds = [];
                for (var name in this._allModules) {
                    deferreds.push(this._startModule(this._allModules[name]));
                }
                return promiseAll(deferreds);
            }
        },

        _startModule: function (/*Object*/module) {
            // summary:
            //      Starts a module and all its dependencies
            // module:
            //      The module to start
            // returns:
            //      A promise resolved when the module has been started.

            if (module._started) {
                // The module has started or is starting so return the _started deferred.
                return module._started.promise;
            }

            // Assign a deferred to _started so that we only start this module once.
            module._started = new Deferred();

            var self = this,
                runAfter = [],
                checkDependencies = function (dependency) {
                    return dependency.moduleName === module.name && (dependency.dependencyType & moduleDependencyType.runAfter);
                };

            for (var m in this._allModules) {
                var dependant = this._allModules[m],
                    hasDependency = array.some(dependant.moduleDependencies, checkDependencies);

                if (hasDependency) {
                    runAfter.push(dependant);
                }
            }

            // Start dependant modules that are marked as runAfter when this module is started.
            module._started.then(function () {
                array.forEach(runAfter, function (dependantModule) {
                    self._startModule(dependantModule);
                });
            });

            console.info("Starting module: ", module.name);

            when(this._startRequiredModules(module), function () {
                var resourcesLoaded = clientResourceLoader.loadResources(module.cssResources, module.scriptResources);

                // When our scripts are loaded and the modules we depend on have been started the module initializer can be run.
                when(resourcesLoaded, function () {
                    self._runModuleInitializer(module).then(module._started.resolve, module._started.reject);
                });
            });

            return module._started.promise; // Promise
        },

        _startRequiredModules: function (module) {
            // summary:
            //      Start all "require" and "requireAndRunAfter" dependencies for a module.
            // tags:
            //      private
            var dependencies = array.filter(module.moduleDependencies, function (dependency) {
                return dependency.dependencyType & moduleDependencyType.require;
            });

            console.debug(module.name, dependencies);

            var deferreds = array.map(dependencies, function (dependency) {
                var requiredModule = this._allModules[dependency.moduleName];
                if (!requiredModule) {
                    throw new Error("Module '" + module.name + "' has a dependency named '" + dependency.moduleName + "' which doesn't exist.");
                }
                return this._startModule(requiredModule);
            }, this);

            return promiseAll(deferreds); // Promise
        },

        _configureModule: function (/* Object */moduleSettings) {
            // summary:
            //      Add module configuration settings to the application
            // moduleSettings:
            //      The module settings
            // tags:
            //      private

            // register routes
            if (moduleSettings.routes) {
                array.forEach(moduleSettings.routes, function (route) {
                    routes.registerRoute(route.routeBasePath, route.routeDefaults);
                });
            }
        },

        _runModuleInitializer: function (/*Object*/moduleSettings) {
            // summary:
            //  Runs the module initializer, if one exists, for a module.
            //
            // moduleSettings:
            //
            // returns:
            //  A promise resolved when the module initaliser has been executed.

            var dfd = new Deferred();
            if (moduleSettings.initializer) {
                var initializer = moduleSettings.initializer.replace(/\./g, "/");
                require([initializer], lang.hitch(this, function (moduleClass) {
                    try {
                        // Instatiate the initializer associated with the module.
                        console.debug("Running: ", initializer);

                        var instance = new moduleClass(moduleSettings);
                        when(instance.initialize(), lang.hitch(this, function () {
                            this._moduleInstances[moduleSettings.name] = instance;
                            dfd.resolve();
                        }), dfd.reject);

                    } catch (e) {
                        console.error(e.stack || e);
                        dfd.reject(e);
                    }
                }));
            } else {
                dfd.resolve();
            }
            return dfd.promise;
        },

        uninitializeModule: function (moduleName) {
            // summary:
            //      Uninitializes the module with the given name.
            // moduleName: String
            //      The name of the module to uninitialize
            // tags:
            //      public
            var modules = this._moduleInstances;
            if (modules[moduleName]) {
                modules[moduleName].uninitialize();
                delete modules[moduleName];
            }
        }
    });

    // moduleDependencyType: [public] Object
    //      Static module dependency type enum.
    ModuleManager.moduleDependencyType = moduleDependencyType;

    return ModuleManager;
});

},
'dojo/promise/all':function(){
define("dojo/promise/all", [
	"../_base/array",
	"../Deferred",
	"../when"
], function(array, Deferred, when){
	"use strict";

	// module:
	//		dojo/promise/all

	var some = array.some;

	return function all(objectOrArray){
		// summary:
		//		Takes multiple promises and returns a new promise that is fulfilled
		//		when all promises have been fulfilled.
		// description:
		//		Takes multiple promises and returns a new promise that is fulfilled
		//		when all promises have been fulfilled. If one of the promises is rejected,
		//		the returned promise is also rejected. Canceling the returned promise will
		//		*not* cancel any passed promises.
		// objectOrArray: Object|Array?
		//		The promise will be fulfilled with a list of results if invoked with an
		//		array, or an object of results when passed an object (using the same
		//		keys). If passed neither an object or array it is resolved with an
		//		undefined value.
		// returns: dojo/promise/Promise

		var object, array;
		if(objectOrArray instanceof Array){
			array = objectOrArray;
		}else if(objectOrArray && typeof objectOrArray === "object"){
			object = objectOrArray;
		}

		var results;
		var keyLookup = [];
		if(object){
			array = [];
			for(var key in object){
				if(Object.hasOwnProperty.call(object, key)){
					keyLookup.push(key);
					array.push(object[key]);
				}
			}
			results = {};
		}else if(array){
			results = [];
		}

		if(!array || !array.length){
			return new Deferred().resolve(results);
		}

		var deferred = new Deferred();
		deferred.promise.always(function(){
			results = keyLookup = null;
		});
		var waiting = array.length;
		some(array, function(valueOrPromise, index){
			if(!object){
				keyLookup.push(index);
			}
			when(valueOrPromise, function(value){
				if(!deferred.isFulfilled()){
					results[keyLookup[index]] = value;
					if(--waiting === 0){
						deferred.resolve(results);
					}
				}
			}, deferred.reject);
			return deferred.isFulfilled();
		});
		return deferred.promise;	// dojo/promise/Promise
	};
});

},
'epi/shell/Bootstrapper':function(){
define("epi/shell/Bootstrapper", [
    "dojo/_base/config",
    "dojo/_base/declare",
    "dojo/_base/Deferred",
    "dojo/topic",
    "dojo/when",
    "epi",
    "epi/datetime",
    "epi/dependency",
    "epi/ModuleManager",
    "epi/shell/ViewSettings"],

function (
    config,
    declare,
    Deferred,
    topic,
    when,
    epi,
    epiDate,
    dependency,
    epiModuleManager,
    viewSettings
) {

    return declare(null, {
        // summary:
        //		Shell bootstrapper.
        //
        // tags:
        //      internal

        _moduleManager: null,

        constructor: function (/* Object */settings) {
            // summary:
            //      Creates and configures epi.ModuleManager
            //
            // settings:
            //      Module settings, see `epi.ModuleManager`
            //

            this._moduleManager = new epiModuleManager(settings);
            dependency.register("epi.ModuleManager", this._moduleManager);
            this._setServerTimeDelta();
            this._moduleManager.configureModules();
        },

        initializeApplication: function (/* String */viewName,  /*String*/startupModule, /*Object*/settings) {
            // summary:
            //      Starts modules and loads the view
            //
            // viewName:
            //      Name of the view to load
            // applicationDomNodes:
            //      Array of dom nodes or ids of dom nodes to inject view into
            //
            // startupModule:
            //      Name of the module to start
            //
            // settings:
            //      A dictionary with settings for the current view.

            viewSettings.viewName = viewName;
            viewSettings.settings = settings;

            var started = this._moduleManager.startModules(startupModule);

            when(started, function () {
                topic.publish("/epi/shell/application/initialized");
            });

            return started;
        },

        _setServerTimeDelta: function () {
            // summary:
            //      Calculate difference time between client and server at initialization time
            // tags:
            //      Private

            var clientTime = new Date();
            var serverTime = new Date(config.serverTime);
            epiDate.serverTimeDelta = serverTime.getTime() - clientTime.getTime();
        }
    });
});

},
'epi/datetime':function(){
﻿define("epi/datetime", [
    "epi",
    "dojo/date",
    "dojo/date/stamp",
    "dojo/date/locale",
    "dojo/_base/lang",
    "dojo/i18n"],

function (epi, dojoDate, dateStamp, locale, lang, i18n) {

    function _friendlyTimeDiff(fromDate, toDate) {
        var dayStr = epi.resources.text.day;
        var daysStr = epi.resources.text.days;

        var hourStr = epi.resources.text.hour;
        var hoursStr = epi.resources.text.hours;

        var minuteStr = epi.resources.text.minute;
        var minutesStr = epi.resources.text.minutes;

        var secondStr = epi.resources.text.second;
        var secondsStr = epi.resources.text.seconds;

        var result = "";

        var totalDiff = toDate - fromDate;

        // If the date is in the future the totalDiff will be negative, return 1 second if that is the case
        if (totalDiff < 0) {
            return " 1 " + secondStr;
        }

        var days = Math.floor(totalDiff / 1000 / 60 / 60 / 24);
        var dateDiff = new Date(totalDiff);
        var hours = dateDiff.getUTCHours();
        var minutes = dateDiff.getMinutes();
        var seconds = dateDiff.getSeconds();

        if (days !== 0) {
            result = result + " " + days + " " + (days > 1 ? daysStr : dayStr);
        }

        if (hours !== 0) {
            result = result + " " + hours + " " + (hours > 1 ? hoursStr : hourStr);
        }

        if (minutes !== 0) {
            result = result + " " + minutes + " " + (minutes > 1 ? minutesStr : minuteStr);
        }

        if (result.length > 0) {
            return result;
        }

        return " " + seconds + " " + (seconds > 1 ? secondsStr : secondStr);
    }

    function _hasProperties(/*Object*/obj) {
        // summary:
        //    Helper function that determine if the object is a simple or complex object
        //
        // obj:
        //    Object to be tested.
        //
        // tags:
        //    internal
        if (lang.isObject(obj)) {
            for (var item in obj) {
                return true;
            }
        }
        return false;
    }

    function _transformDate(/*Object*/obj, /*boolean*/forceSimpleProperty) {
        // summary:
        //    Search property that are instance of Date
        //    in a object recursively.
        //
        // obj:
        //    The object the contain the data to be
        //    transformed.
        //
        // tags:
        //    internal
        if (forceSimpleProperty) {
            if (obj instanceof Date) {
                return dateStamp.toISOString(obj, { zulu: true });
            } else {
                return obj;
            }
        }

        var newObject = lang.clone(obj);

        for (var item in obj) {
            if (obj[item] instanceof Date) {
                newObject[item] = dateStamp.toISOString(obj[item], { zulu: true });
            } else {
                if (_hasProperties(obj[item])) {
                    newObject[item] = _transformDate(obj[item]);
                }
            }
        }
        return newObject;
    }

    function escapeFormat(format) {
        // summary:
        //      Escapes the given string so that it will be ignored by the date formatter.
        // tags:
        //      private

        return "'" + format + "'";
    }

    return epi.datetime = {
        // tags:
        //      public

        // serverTimeDelta: Integer
        //      Difference time between client and server
        serverTimeDelta: 0,

        transformDate: function (/*Object*/value) {
            // summary:
            //    Remove the padding server offset from the date and
            //    convert it to ISO String.
            //
            // value:
            //    The object to be converted.
            //
            // tags:
            //    public
            var isSimpleProperty = (_hasProperties(value) && !(value instanceof Date));
            var transformedValue = (isSimpleProperty ? _transformDate(value) : _transformDate(value, true));
            return transformedValue;
        },

        isDateValid: function (/*Object*/obj) {
            // summary:
            //    Check if the date object is valid.
            //
            // obj:
            //    Object to the tested.
            //
            // tags:
            //    private
            return ((obj && obj instanceof Date) && (!isNaN(obj.getTime())));
        },

        toUserFriendlyString: function (date, localeName, hideToday, excludeTime) {
            // summary:
            //      Converts a date object or a date formatted string to a user friendly string.
            // date: Date|String
            //      The date to convert.
            // localeName: String?
            //      The locale in which the date will be formatted.
            // hideToday: Boolean?
            //      Excludes the date portion from the returned date string when true and the given
            //      date is today.
            // excludeTime: Boolean?
            //      Excludes the time portion from the returned date string when true. This option
            //      is ignored in the case hideToday is true and the given date is today.
            // tags:
            //      public

            if (!date) {
                return "";
            }

            // "date" can be passed as a string, but needed as a Date object.
            if (typeof date === "string") {
                var parsedDate = new Date(date);

                // An invalid date is an "Invalid Date" object, which is best checked with isNaN.
                // TODO: Throw exception instead once we release CMS UI 10.0.
                if (!isNaN(parsedDate)) {
                    date = parsedDate;
                }
            }

            if (!(date instanceof Date)) {
                date = this.deserialize(date);
            }

            //Get the gregorian settings for the current locale that specifies some specific locale formats.
            var bundle = i18n.getLocalization("dojo.cldr", "gregorian", localeName);

            var today = new Date(),
                comparisonDate = new Date(date),
                selector = excludeTime ? "date" : "",
                datePattern = "";

            today.setHours(0, 0, 0, 0);
            comparisonDate.setHours(0, 0, 0, 0);

            if (dojoDate.difference(today, date, "year") === 0) {
                // If the date is this year format the date with only month and day.
                datePattern = bundle["dateFormatItem-MMMd"];

                // Store the comparison date in a new variable since we need the original and
                // compare the difference in days.
                var difference = dojoDate.difference(today, comparisonDate, "day");

                switch (difference) {
                    case 0:
                        // Do an early exit with only the formatted time if hideToday is true.
                        if (hideToday) {
                            return locale.format(date, { selector: "time", locale: localeName });
                        }
                        datePattern = escapeFormat(bundle["field-day-relative+0"]);
                        break;
                    case 1:
                        datePattern = escapeFormat(bundle["field-day-relative+1"]);
                        break;
                    case -1:
                        datePattern = escapeFormat(bundle["field-day-relative+-1"]);
                        break;
                }
            }

            // Format the date based on the selector, date pattern and given locale.
            return locale.format(date, { selector: selector, datePattern: datePattern, locale: localeName });
        },

        toUserFriendlyHtml: function (date, localeName, hideToday, excludeTime) {
            // summary:
            //      Converts a date object or a date formatted string to a user friendly string
            //      contained within HTML, e.g. <span class="epi-timestamp">Today, 7:30 AM</span>.
            // date: Date|String
            //      The date to convert.
            // localeName: String?
            //      The locale in which the date will be formatted.
            // hideToday: Boolean?
            //      Excludes the date portion from the returned date string when true and the given
            //      date is today.
            // excludeTime: Boolean?
            //      Excludes the time portion from the returned date string when true. This option
            //      is ignored in the case hideToday is true and the given date is today.
            // tags:
            //      public

            var formattedDate = this.toUserFriendlyString(date, localeName, hideToday, excludeTime);
            if (formattedDate === "") {
                return formattedDate;
            }
            return "<span class='epi-timestamp'>" + formattedDate + "</span>";
        },

        timePassed: function (from, to) {
            // summary:
            //      Get the time passed between two dates
            //
            // from: Date
            //      Start date
            // to: Null|Date
            //      End date
            // tags:
            //      public

            to = to || new Date();
            return _friendlyTimeDiff(from, to);
        },

        timeToGo: function (/*Date*/date) {
            // summary:
            //      Get the time to go to the given time
            //
            // tags:
            //      public

            return _friendlyTimeDiff(this.serverTime(), date);
        },

        serverTime: function () {
            // summary:
            //      Get the current datetime from server
            //
            // tags:
            //      public

            return new Date(new Date().getTime() + this.serverTimeDelta);
        }
    };
});

},
'dojo/date':function(){
define("dojo/date", ["./has", "./_base/lang"], function(has, lang){
// module:
//		dojo/date

var date = {
	// summary:
	//		Date manipulation utilities
};

date.getDaysInMonth = function(/*Date*/dateObject){
	// summary:
	//		Returns the number of days in the month used by dateObject
	var month = dateObject.getMonth();
	var days = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
	if(month == 1 && date.isLeapYear(dateObject)){ return 29; } // Number
	return days[month]; // Number
};

date.isLeapYear = function(/*Date*/dateObject){
	// summary:
	//		Determines if the year of the dateObject is a leap year
	// description:
	//		Leap years are years with an additional day YYYY-02-29, where the
	//		year number is a multiple of four with the following exception: If
	//		a year is a multiple of 100, then it is only a leap year if it is
	//		also a multiple of 400. For example, 1900 was not a leap year, but
	//		2000 is one.

	var year = dateObject.getFullYear();
	return !(year%400) || (!(year%4) && !!(year%100)); // Boolean
};

// FIXME: This is not localized
date.getTimezoneName = function(/*Date*/dateObject){
	// summary:
	//		Get the user's time zone as provided by the browser
	// dateObject:
	//		Needed because the timezone may vary with time (daylight savings)
	// description:
	//		Try to get time zone info from toString or toLocaleString method of
	//		the Date object -- UTC offset is not a time zone.  See
	//		http://www.twinsun.com/tz/tz-link.htm Note: results may be
	//		inconsistent across browsers.

	var str = dateObject.toString(); // Start looking in toString
	var tz = ''; // The result -- return empty string if nothing found
	var match;

	// First look for something in parentheses -- fast lookup, no regex
	var pos = str.indexOf('(');
	if(pos > -1){
		tz = str.substring(++pos, str.indexOf(')'));
	}else{
		// If at first you don't succeed ...
		// If IE knows about the TZ, it appears before the year
		// Capital letters or slash before a 4-digit year
		// at the end of string
		var pat = /([A-Z\/]+) \d{4}$/;
		if((match = str.match(pat))){
			tz = match[1];
		}else{
		// Some browsers (e.g. Safari) glue the TZ on the end
		// of toLocaleString instead of putting it in toString
			str = dateObject.toLocaleString();
			// Capital letters or slash -- end of string,
			// after space
			pat = / ([A-Z\/]+)$/;
			if((match = str.match(pat))){
				tz = match[1];
			}
		}
	}

	// Make sure it doesn't somehow end up return AM or PM
	return (tz == 'AM' || tz == 'PM') ? '' : tz; // String
};

// Utility methods to do arithmetic calculations with Dates

date.compare = function(/*Date*/date1, /*Date?*/date2, /*String?*/portion){
	// summary:
	//		Compare two date objects by date, time, or both.
	// description:
	//		Returns 0 if equal, positive if a > b, else negative.
	// date1:
	//		Date object
	// date2:
	//		Date object.  If not specified, the current Date is used.
	// portion:
	//		A string indicating the "date" or "time" portion of a Date object.
	//		Compares both "date" and "time" by default.  One of the following:
	//		"date", "time", "datetime"

	// Extra step required in copy for IE - see #3112
	date1 = new Date(+date1);
	date2 = new Date(+(date2 || new Date()));

	if(portion == "date"){
		// Ignore times and compare dates.
		date1.setHours(0, 0, 0, 0);
		date2.setHours(0, 0, 0, 0);
	}else if(portion == "time"){
		// Ignore dates and compare times.
		date1.setFullYear(0, 0, 0);
		date2.setFullYear(0, 0, 0);
	}

	if(date1 > date2){ return 1; } // int
	if(date1 < date2){ return -1; } // int
	return 0; // int
};

date.add = function(/*Date*/date, /*String*/interval, /*int*/amount){
	// summary:
	//		Add to a Date in intervals of different size, from milliseconds to years
	// date: Date
	//		Date object to start with
	// interval:
	//		A string representing the interval.  One of the following:
	//		"year", "month", "day", "hour", "minute", "second",
	//		"millisecond", "quarter", "week", "weekday"
	// amount:
	//		How much to add to the date.

	var sum = new Date(+date); // convert to Number before copying to accomodate IE (#3112)
	var fixOvershoot = false;
	var property = "Date";

	switch(interval){
		case "day":
			break;
		case "weekday":
			//i18n FIXME: assumes Saturday/Sunday weekend, but this is not always true.  see dojo/cldr/supplemental

			// Divide the increment time span into weekspans plus leftover days
			// e.g., 8 days is one 5-day weekspan / and two leftover days
			// Can't have zero leftover days, so numbers divisible by 5 get
			// a days value of 5, and the remaining days make up the number of weeks
			var days, weeks;
			var mod = amount % 5;
			if(!mod){
				days = (amount > 0) ? 5 : -5;
				weeks = (amount > 0) ? ((amount-5)/5) : ((amount+5)/5);
			}else{
				days = mod;
				weeks = parseInt(amount/5);
			}
			// Get weekday value for orig date param
			var strt = date.getDay();
			// Orig date is Sat / positive incrementer
			// Jump over Sun
			var adj = 0;
			if(strt == 6 && amount > 0){
				adj = 1;
			}else if(strt == 0 && amount < 0){
			// Orig date is Sun / negative incrementer
			// Jump back over Sat
				adj = -1;
			}
			// Get weekday val for the new date
			var trgt = strt + days;
			// New date is on Sat or Sun
			if(trgt == 0 || trgt == 6){
				adj = (amount > 0) ? 2 : -2;
			}
			// Increment by number of weeks plus leftover days plus
			// weekend adjustments
			amount = (7 * weeks) + days + adj;
			break;
		case "year":
			property = "FullYear";
			// Keep increment/decrement from 2/29 out of March
			fixOvershoot = true;
			break;
		case "week":
			amount *= 7;
			break;
		case "quarter":
			// Naive quarter is just three months
			amount *= 3;
			// fallthrough...
		case "month":
			// Reset to last day of month if you overshoot
			fixOvershoot = true;
			property = "Month";
			break;
//		case "hour":
//		case "minute":
//		case "second":
//		case "millisecond":
		default:
			property = "UTC"+interval.charAt(0).toUpperCase() + interval.substring(1) + "s";
	}

	if(property){
		sum["set"+property](sum["get"+property]()+amount);
	}

	if(fixOvershoot && (sum.getDate() < date.getDate())){
		sum.setDate(0);
	}

	return sum; // Date
};

date.difference = function(/*Date*/date1, /*Date?*/date2, /*String?*/interval){
	// summary:
	//		Get the difference in a specific unit of time (e.g., number of
	//		months, weeks, days, etc.) between two dates, rounded to the
	//		nearest integer.
	// date1:
	//		Date object
	// date2:
	//		Date object.  If not specified, the current Date is used.
	// interval:
	//		A string representing the interval.  One of the following:
	//		"year", "month", "day", "hour", "minute", "second",
	//		"millisecond", "quarter", "week", "weekday"
	//
	//		Defaults to "day".

	date2 = date2 || new Date();
	interval = interval || "day";
	var yearDiff = date2.getFullYear() - date1.getFullYear();
	var delta = 1; // Integer return value

	switch(interval){
		case "quarter":
			var m1 = date1.getMonth();
			var m2 = date2.getMonth();
			// Figure out which quarter the months are in
			var q1 = Math.floor(m1/3) + 1;
			var q2 = Math.floor(m2/3) + 1;
			// Add quarters for any year difference between the dates
			q2 += (yearDiff * 4);
			delta = q2 - q1;
			break;
		case "weekday":
			var days = Math.round(date.difference(date1, date2, "day"));
			var weeks = parseInt(date.difference(date1, date2, "week"));
			var mod = days % 7;

			// Even number of weeks
			if(mod == 0){
				days = weeks*5;
			}else{
				// Weeks plus spare change (< 7 days)
				var adj = 0;
				var aDay = date1.getDay();
				var bDay = date2.getDay();

				weeks = parseInt(days/7);
				mod = days % 7;
				// Mark the date advanced by the number of
				// round weeks (may be zero)
				var dtMark = new Date(date1);
				dtMark.setDate(dtMark.getDate()+(weeks*7));
				var dayMark = dtMark.getDay();

				// Spare change days -- 6 or less
				if(days > 0){
					switch(true){
						// Range starts on Sat
						case aDay == 6:
							adj = -1;
							break;
						// Range starts on Sun
						case aDay == 0:
							adj = 0;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = -1;
							break;
						// Range ends on Sun
						case bDay == 0:
							adj = -2;
							break;
						// Range contains weekend
						case (dayMark + mod) > 5:
							adj = -2;
					}
				}else if(days < 0){
					switch(true){
						// Range starts on Sat
						case aDay == 6:
							adj = 0;
							break;
						// Range starts on Sun
						case aDay == 0:
							adj = 1;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = 2;
							break;
						// Range ends on Sun
						case bDay == 0:
							adj = 1;
							break;
						// Range contains weekend
						case (dayMark + mod) < 0:
							adj = 2;
					}
				}
				days += adj;
				days -= (weeks*2);
			}
			delta = days;
			break;
		case "year":
			delta = yearDiff;
			break;
		case "month":
			delta = (date2.getMonth() - date1.getMonth()) + (yearDiff * 12);
			break;
		case "week":
			// Truncate instead of rounding
			// Don't use Math.floor -- value may be negative
			delta = parseInt(date.difference(date1, date2, "day")/7);
			break;
		case "day":
			delta /= 24;
			// fallthrough
		case "hour":
			delta /= 60;
			// fallthrough
		case "minute":
			delta /= 60;
			// fallthrough
		case "second":
			delta /= 1000;
			// fallthrough
		case "millisecond":
			delta *= date2.getTime() - date1.getTime();
	}

	// Round for fractional values and DST leaps
	return Math.round(delta); // Number (integer)
};

// Don't use setObject() because it may overwrite dojo/date/stamp (if that has already been loaded)
 1  && lang.mixin(lang.getObject("dojo.date", true), date);

return date;
});

},
'dojo/date/stamp':function(){
define("dojo/date/stamp", ["../_base/lang", "../_base/array"], function(lang, array){

// module:
//		dojo/date/stamp

var stamp = {
	// summary:
	//		TODOC
};
lang.setObject("dojo.date.stamp", stamp);

// Methods to convert dates to or from a wire (string) format using well-known conventions

stamp.fromISOString = function(/*String*/ formattedString, /*Number?*/ defaultTime){
	// summary:
	//		Returns a Date object given a string formatted according to a subset of the ISO-8601 standard.
	//
	// description:
	//		Accepts a string formatted according to a profile of ISO8601 as defined by
	//		[RFC3339](http://www.ietf.org/rfc/rfc3339.txt), except that partial input is allowed.
	//		Can also process dates as specified [by the W3C](http://www.w3.org/TR/NOTE-datetime)
	//		The following combinations are valid:
	//
	//		- dates only
	//			- yyyy
	//			- yyyy-MM
	//			- yyyy-MM-dd
	//		- times only, with an optional time zone appended
	//			- THH:mm
	//			- THH:mm:ss
	//			- THH:mm:ss.SSS
	//		- and "datetimes" which could be any combination of the above
	//
	//		timezones may be specified as Z (for UTC) or +/- followed by a time expression HH:mm
	//		Assumes the local time zone if not specified.  Does not validate.  Improperly formatted
	//		input may return null.  Arguments which are out of bounds will be handled
	//		by the Date constructor (e.g. January 32nd typically gets resolved to February 1st)
	//		Only years between 100 and 9999 are supported.
  	// formattedString:
	//		A string such as 2005-06-30T08:05:00-07:00 or 2005-06-30 or T08:05:00
	// defaultTime:
	//		Used for defaults for fields omitted in the formattedString.
	//		Uses 1970-01-01T00:00:00.0Z by default.

	if(!stamp._isoRegExp){
		stamp._isoRegExp =
//TODO: could be more restrictive and check for 00-59, etc.
			/^(?:(\d{4})(?:-(\d{2})(?:-(\d{2}))?)?)?(?:T(\d{2}):(\d{2})(?::(\d{2})(.\d+)?)?((?:[+-](\d{2}):(\d{2}))|Z)?)?$/;
	}

	var match = stamp._isoRegExp.exec(formattedString),
		result = null;

	if(match){
		match.shift();
		if(match[1]){match[1]--;} // Javascript Date months are 0-based
		if(match[6]){match[6] *= 1000;} // Javascript Date expects fractional seconds as milliseconds

		if(defaultTime){
			// mix in defaultTime.  Relatively expensive, so use || operators for the fast path of defaultTime === 0
			defaultTime = new Date(defaultTime);
			array.forEach(array.map(["FullYear", "Month", "Date", "Hours", "Minutes", "Seconds", "Milliseconds"], function(prop){
				return defaultTime["get" + prop]();
			}), function(value, index){
				match[index] = match[index] || value;
			});
		}
		result = new Date(match[0]||1970, match[1]||0, match[2]||1, match[3]||0, match[4]||0, match[5]||0, match[6]||0); //TODO: UTC defaults
		if(match[0] < 100){
			result.setFullYear(match[0] || 1970);
		}

		var offset = 0,
			zoneSign = match[7] && match[7].charAt(0);
		if(zoneSign != 'Z'){
			offset = ((match[8] || 0) * 60) + (Number(match[9]) || 0);
			if(zoneSign != '-'){ offset *= -1; }
		}
		if(zoneSign){
			offset -= result.getTimezoneOffset();
		}
		if(offset){
			result.setTime(result.getTime() + offset * 60000);
		}
	}

	return result; // Date or null
};

/*=====
var __Options = {
	// selector: String
	//		"date" or "time" for partial formatting of the Date object.
	//		Both date and time will be formatted by default.
	// zulu: Boolean
	//		if true, UTC/GMT is used for a timezone
	// milliseconds: Boolean
	//		if true, output milliseconds
};
=====*/

stamp.toISOString = function(/*Date*/ dateObject, /*__Options?*/ options){
	// summary:
	//		Format a Date object as a string according a subset of the ISO-8601 standard
	//
	// description:
	//		When options.selector is omitted, output follows [RFC3339](http://www.ietf.org/rfc/rfc3339.txt)
	//		The local time zone is included as an offset from GMT, except when selector=='time' (time without a date)
	//		Does not check bounds.  Only years between 100 and 9999 are supported.
	//
	// dateObject:
	//		A Date object

	var _ = function(n){ return (n < 10) ? "0" + n : n; };
	options = options || {};
	var formattedDate = [],
		getter = options.zulu ? "getUTC" : "get",
		date = "";
	if(options.selector != "time"){
		var year = dateObject[getter+"FullYear"]();
		date = ["0000".substr((year+"").length)+year, _(dateObject[getter+"Month"]()+1), _(dateObject[getter+"Date"]())].join('-');
	}
	formattedDate.push(date);
	if(options.selector != "date"){
		var time = [_(dateObject[getter+"Hours"]()), _(dateObject[getter+"Minutes"]()), _(dateObject[getter+"Seconds"]())].join(':');
		var millis = dateObject[getter+"Milliseconds"]();
		if(options.milliseconds){
			time += "."+ (millis < 100 ? "0" : "") + _(millis);
		}
		if(options.zulu){
			time += "Z";
		}else if(options.selector != "time"){
			var timezoneOffset = dateObject.getTimezoneOffset();
			var absOffset = Math.abs(timezoneOffset);
			time += (timezoneOffset > 0 ? "-" : "+") +
				_(Math.floor(absOffset/60)) + ":" + _(absOffset%60);
		}
		formattedDate.push(time);
	}
	return formattedDate.join('T'); // String
};

return stamp;
});

},
'dojo/date/locale':function(){
define("dojo/date/locale", [
	"../_base/lang",
	"../_base/array",
	"../date",
	/*===== "../_base/declare", =====*/
	"../cldr/supplemental",
	"../i18n",
	"../regexp",
	"../string",
	"../i18n!../cldr/nls/gregorian",
	"module"
], function(lang, array, date, /*===== declare, =====*/ supplemental, i18n, regexp, string, gregorian, module){

// module:
//		dojo/date/locale

var exports = {
	// summary:
	//		This modules defines dojo/date/locale, localization methods for Date.
};
lang.setObject(module.id.replace(/\//g, "."), exports);

// Localization methods for Date.   Honor local customs using locale-dependent dojo.cldr data.

// Load the bundles containing localization information for
// names and formats

//NOTE: Everything in this module assumes Gregorian calendars.
// Other calendars will be implemented in separate modules.

	// Format a pattern without literals
	function formatPattern(dateObject, bundle, options, pattern){
		return pattern.replace(/([a-z])\1*/ig, function(match){
			var s, pad,
				c = match.charAt(0),
				l = match.length,
				widthList = ["abbr", "wide", "narrow"];
			switch(c){
				case 'G':
					s = bundle[(l < 4) ? "eraAbbr" : "eraNames"][dateObject.getFullYear() < 0 ? 0 : 1];
					break;
				case 'y':
					s = dateObject.getFullYear();
					switch(l){
						case 1:
							break;
						case 2:
							if(!options.fullYear){
								s = String(s); s = s.substr(s.length - 2);
								break;
							}
							// fallthrough
						default:
							pad = true;
					}
					break;
				case 'Q':
				case 'q':
					s = Math.ceil((dateObject.getMonth()+1)/3);
//					switch(l){
//						case 1: case 2:
							pad = true;
//							break;
//						case 3: case 4: // unimplemented
//					}
					break;
				case 'M':
				case 'L':
					var m = dateObject.getMonth();
					if(l<3){
						s = m+1; pad = true;
					}else{
						var propM = [
							"months",
							c == 'L' ? "standAlone" : "format",
							widthList[l-3]
						].join("-");
						s = bundle[propM][m];
					}
					break;
				case 'w':
					var firstDay = 0;
					s = exports._getWeekOfYear(dateObject, firstDay); pad = true;
					break;
				case 'd':
					s = dateObject.getDate(); pad = true;
					break;
				case 'D':
					s = exports._getDayOfYear(dateObject); pad = true;
					break;
				case 'e':
				case 'c':
					var d = dateObject.getDay();
					if(l<2){
						s = (d - supplemental.getFirstDayOfWeek(options.locale) + 8) % 7
						break;
					}
					// fallthrough
				case 'E':
					d = dateObject.getDay();
					if(l<3){
						s = d+1; pad = true;
					}else{
						var propD = [
							"days",
							c == 'c' ? "standAlone" : "format",
							widthList[l-3]
						].join("-");
						s = bundle[propD][d];
					}
					break;
				case 'a':
					var timePeriod = dateObject.getHours() < 12 ? 'am' : 'pm';
					s = options[timePeriod] || bundle['dayPeriods-format-wide-' + timePeriod];
					break;
				case 'h':
				case 'H':
				case 'K':
				case 'k':
					var h = dateObject.getHours();
					// strange choices in the date format make it impossible to write this succinctly
					switch (c){
						case 'h': // 1-12
							s = (h % 12) || 12;
							break;
						case 'H': // 0-23
							s = h;
							break;
						case 'K': // 0-11
							s = (h % 12);
							break;
						case 'k': // 1-24
							s = h || 24;
							break;
					}
					pad = true;
					break;
				case 'm':
					s = dateObject.getMinutes(); pad = true;
					break;
				case 's':
					s = dateObject.getSeconds(); pad = true;
					break;
				case 'S':
					s = Math.round(dateObject.getMilliseconds() * Math.pow(10, l-3)); pad = true;
					break;
				case 'v': // FIXME: don't know what this is. seems to be same as z?
				case 'z':
					// We only have one timezone to offer; the one from the browser
					s = exports._getZone(dateObject, true, options);
					if(s){break;}
					l=4;
					// fallthrough... use GMT if tz not available
				case 'Z':
					var offset = exports._getZone(dateObject, false, options);
					var tz = [
						(offset<=0 ? "+" : "-"),
						string.pad(Math.floor(Math.abs(offset)/60), 2),
						string.pad(Math.abs(offset)% 60, 2)
					];
					if(l==4){
						tz.splice(0, 0, "GMT");
						tz.splice(3, 0, ":");
					}
					s = tz.join("");
					break;
//				case 'Y': case 'u': case 'W': case 'F': case 'g': case 'A':
//					console.log(match+" modifier unimplemented");
				default:
					throw new Error("dojo.date.locale.format: invalid pattern char: "+pattern);
			}
			if(pad){ s = string.pad(s, l); }
			return s;
		});
	}

/*=====
var __FormatOptions = exports.__FormatOptions = declare(null, {
	// selector: String
	//		choice of 'time','date' (default: date and time)
	// formatLength: String
	//		choice of long, short, medium or full (plus any custom additions).  Defaults to 'short'
	// datePattern:String
	//		override pattern with this string
	// timePattern:String
	//		override pattern with this string
	// am: String
	//		override strings for am in times
	// pm: String
	//		override strings for pm in times
	// locale: String
	//		override the locale used to determine formatting rules
	// fullYear: Boolean
	//		(format only) use 4 digit years whenever 2 digit years are called for
	// strict: Boolean
	//		(parse only) strict parsing, off by default
});
=====*/

exports._getZone = function(/*Date*/ dateObject, /*boolean*/ getName, /*__FormatOptions?*/ options){
	// summary:
	//		Returns the zone (or offset) for the given date and options.  This
	//		is broken out into a separate function so that it can be overridden
	//		by timezone-aware code.
	//
	// dateObject:
	//		the date and/or time being formatted.
	//
	// getName:
	//		Whether to return the timezone string (if true), or the offset (if false)
	//
	// options:
	//		The options being used for formatting
	if(getName){
		return date.getTimezoneName(dateObject);
	}else{
		return dateObject.getTimezoneOffset();
	}
};


exports.format = function(/*Date*/ dateObject, /*__FormatOptions?*/ options){
	// summary:
	//		Format a Date object as a String, using locale-specific settings.
	//
	// description:
	//		Create a string from a Date object using a known localized pattern.
	//		By default, this method formats both date and time from dateObject.
	//		Formatting patterns are chosen appropriate to the locale.  Different
	//		formatting lengths may be chosen, with "full" used by default.
	//		Custom patterns may be used or registered with translations using
	//		the dojo/date/locale.addCustomFormats() method.
	//		Formatting patterns are implemented using [the syntax described at
	//		unicode.org](http://www.unicode.org/reports/tr35/tr35-4.html#Date_Format_Patterns)
	//
	// dateObject:
	//		the date and/or time to be formatted.  If a time only is formatted,
	//		the values in the year, month, and day fields are irrelevant.  The
	//		opposite is true when formatting only dates.

	options = options || {};

	var locale = i18n.normalizeLocale(options.locale),
		formatLength = options.formatLength || 'short',
		bundle = exports._getGregorianBundle(locale),
		str = [],
		sauce = lang.hitch(this, formatPattern, dateObject, bundle, options);
	if(options.selector == "year"){
		return _processPattern(bundle["dateFormatItem-yyyy"] || "yyyy", sauce);
	}
	var pattern;
	if(options.selector != "date"){
		pattern = options.timePattern || bundle["timeFormat-"+formatLength];
		if(pattern){str.push(_processPattern(pattern, sauce));}
	}
	if(options.selector != "time"){
		pattern = options.datePattern || bundle["dateFormat-"+formatLength];
		if(pattern){str.push(_processPattern(pattern, sauce));}
	}

	return str.length == 1 ? str[0] : bundle["dateTimeFormat-"+formatLength].replace(/\'/g,'').replace(/\{(\d+)\}/g,
		function(match, key){ return str[key]; }); // String
};

exports.regexp = function(/*__FormatOptions?*/ options){
	// summary:
	//		Builds the regular needed to parse a localized date

	return exports._parseInfo(options).regexp; // String
};

exports._parseInfo = function(/*__FormatOptions?*/ options){
	options = options || {};
	var locale = i18n.normalizeLocale(options.locale),
		bundle = exports._getGregorianBundle(locale),
		formatLength = options.formatLength || 'short',
		datePattern = options.datePattern || bundle["dateFormat-" + formatLength],
		timePattern = options.timePattern || bundle["timeFormat-" + formatLength],
		pattern;
	if(options.selector == 'date'){
		pattern = datePattern;
	}else if(options.selector == 'time'){
		pattern = timePattern;
	}else{
		pattern = bundle["dateTimeFormat-"+formatLength].replace(/\{(\d+)\}/g,
			function(match, key){ return [timePattern, datePattern][key]; });
	}

	var tokens = [],
		re = _processPattern(pattern, lang.hitch(this, _buildDateTimeRE, tokens, bundle, options));
	return {regexp: re, tokens: tokens, bundle: bundle};
};

exports.parse = function(/*String*/ value, /*__FormatOptions?*/ options){
	// summary:
	//		Convert a properly formatted string to a primitive Date object,
	//		using locale-specific settings.
	//
	// description:
	//		Create a Date object from a string using a known localized pattern.
	//		By default, this method parses looking for both date and time in the string.
	//		Formatting patterns are chosen appropriate to the locale.  Different
	//		formatting lengths may be chosen, with "full" used by default.
	//		Custom patterns may be used or registered with translations using
	//		the dojo/date/locale.addCustomFormats() method.
	//
	//		Formatting patterns are implemented using [the syntax described at
	//		unicode.org](http://www.unicode.org/reports/tr35/tr35-4.html#Date_Format_Patterns)
	//		When two digit years are used, a century is chosen according to a sliding
	//		window of 80 years before and 20 years after present year, for both `yy` and `yyyy` patterns.
	//		year < 100CE requires strict mode.
	//
	// value:
	//		A string representation of a date

	// remove non-printing bidi control chars from input and pattern
	var controlChars = /[\u200E\u200F\u202A\u202E]/g,
		info = exports._parseInfo(options),
		tokens = info.tokens, bundle = info.bundle,
		re = new RegExp("^" + info.regexp.replace(controlChars, "") + "$",
			info.strict ? "" : "i"),
		match = re.exec(value && value.replace(controlChars, ""));

	if(!match){ return null; } // null

	var widthList = ['abbr', 'wide', 'narrow'],
		result = [1970,0,1,0,0,0,0], // will get converted to a Date at the end
		amPm = "",
		valid = array.every(match, function(v, i){
		if(!i){return true;}
		var token = tokens[i-1],
			l = token.length,
			c = token.charAt(0);
		switch(c){
			case 'y':
				if(l != 2 && options.strict){
					//interpret year literally, so '5' would be 5 A.D.
					result[0] = v;
				}else{
					if(v<100){
						v = Number(v);
						//choose century to apply, according to a sliding window
						//of 80 years before and 20 years after present year
						var year = '' + new Date().getFullYear(),
							century = year.substring(0, 2) * 100,
							cutoff = Math.min(Number(year.substring(2, 4)) + 20, 99);
						result[0] = (v < cutoff) ? century + v : century - 100 + v;
					}else{
						//we expected 2 digits and got more...
						if(options.strict){
							return false;
						}
						//interpret literally, so '150' would be 150 A.D.
						//also tolerate '1950', if 'yyyy' input passed to 'yy' format
						result[0] = v;
					}
				}
				break;
			case 'M':
			case 'L':
				if(l>2){
					var months = bundle['months-' +
							    (c == 'L' ? 'standAlone' : 'format') +
							    '-' + widthList[l-3]].concat();
					if(!options.strict){
						//Tolerate abbreviating period in month part
						//Case-insensitive comparison
						v = v.replace(".","").toLowerCase();
						months = array.map(months, function(s){ return s.replace(".","").toLowerCase(); } );
					}
					v = array.indexOf(months, v);
					if(v == -1){
//						console.log("dojo/date/locale.parse: Could not parse month name: '" + v + "'.");
						return false;
					}
				}else{
					v--;
				}
				result[1] = v;
				break;
			case 'E':
			case 'e':
			case 'c':
				var days = bundle['days-' +
						  (c == 'c' ? 'standAlone' : 'format') +
						  '-' + widthList[l-3]].concat();
				if(!options.strict){
					//Case-insensitive comparison
					v = v.toLowerCase();
					days = array.map(days, function(d){return d.toLowerCase();});
				}
				v = array.indexOf(days, v);
				if(v == -1){
//					console.log("dojo/date/locale.parse: Could not parse weekday name: '" + v + "'.");
					return false;
				}

				//TODO: not sure what to actually do with this input,
				//in terms of setting something on the Date obj...?
				//without more context, can't affect the actual date
				//TODO: just validate?
				break;
			case 'D':
				result[1] = 0;
				// fallthrough...
			case 'd':
				result[2] = v;
				break;
			case 'a': //am/pm
				var am = options.am || bundle['dayPeriods-format-wide-am'],
					pm = options.pm || bundle['dayPeriods-format-wide-pm'];
				if(!options.strict){
					var period = /\./g;
					v = v.replace(period,'').toLowerCase();
					am = am.replace(period,'').toLowerCase();
					pm = pm.replace(period,'').toLowerCase();
				}
				if(options.strict && v != am && v != pm){
//					console.log("dojo/date/locale.parse: Could not parse am/pm part.");
					return false;
				}

				// we might not have seen the hours field yet, so store the state and apply hour change later
				amPm = (v == pm) ? 'p' : (v == am) ? 'a' : '';
				break;
			case 'K': //hour (1-24)
				if(v == 24){ v = 0; }
				// fallthrough...
			case 'h': //hour (1-12)
			case 'H': //hour (0-23)
			case 'k': //hour (0-11)
				//TODO: strict bounds checking, padding
				if(v > 23){
//					console.log("dojo/date/locale.parse: Illegal hours value");
					return false;
				}

				//in the 12-hour case, adjusting for am/pm requires the 'a' part
				//which could come before or after the hour, so we will adjust later
				result[3] = v;
				break;
			case 'm': //minutes
				result[4] = v;
				break;
			case 's': //seconds
				result[5] = v;
				break;
			case 'S': //milliseconds
				result[6] = v;
//				break;
//			case 'w':
//TODO				var firstDay = 0;
//			default:
//TODO: throw?
//				console.log("dojo/date/locale.parse: unsupported pattern char=" + token.charAt(0));
		}
		return true;
	});

	var hours = +result[3];
	if(amPm === 'p' && hours < 12){
		result[3] = hours + 12; //e.g., 3pm -> 15
	}else if(amPm === 'a' && hours == 12){
		result[3] = 0; //12am -> 0
	}

	//TODO: implement a getWeekday() method in order to test
	//validity of input strings containing 'EEE' or 'EEEE'...

	var dateObject = new Date(result[0], result[1], result[2], result[3], result[4], result[5], result[6]); // Date
	if(options.strict){
		dateObject.setFullYear(result[0]);
	}

	// Check for overflow.  The Date() constructor normalizes things like April 32nd...
	//TODO: why isn't this done for times as well?
	var allTokens = tokens.join(""),
		dateToken = allTokens.indexOf('d') != -1,
		monthToken = allTokens.indexOf('M') != -1;

	if(!valid ||
		(monthToken && dateObject.getMonth() > result[1]) ||
		(dateToken && dateObject.getDate() > result[2])){
		return null;
	}

	// Check for underflow, due to DST shifts.  See #9366
	// This assumes a 1 hour dst shift correction at midnight
	// We could compare the timezone offset after the shift and add the difference instead.
	if((monthToken && dateObject.getMonth() < result[1]) ||
		(dateToken && dateObject.getDate() < result[2])){
		dateObject = date.add(dateObject, "hour", 1);
	}

	return dateObject; // Date
};

function _processPattern(pattern, applyPattern, applyLiteral, applyAll){
	//summary: Process a pattern with literals in it

	// Break up on single quotes, treat every other one as a literal, except '' which becomes '
	var identity = function(x){return x;};
	applyPattern = applyPattern || identity;
	applyLiteral = applyLiteral || identity;
	applyAll = applyAll || identity;

	//split on single quotes (which escape literals in date format strings)
	//but preserve escaped single quotes (e.g., o''clock)
	var chunks = pattern.match(/(''|[^'])+/g),
		literal = pattern.charAt(0) == "'";

	array.forEach(chunks, function(chunk, i){
		if(!chunk){
			chunks[i]='';
		}else{
			chunks[i]=(literal ? applyLiteral : applyPattern)(chunk.replace(/''/g, "'"));
			literal = !literal;
		}
	});
	return applyAll(chunks.join(''));
}

function _buildDateTimeRE(tokens, bundle, options, pattern){
	pattern = regexp.escapeString(pattern);
	if(!options.strict){ pattern = pattern.replace(" a", " ?a"); } // kludge to tolerate no space before am/pm
	return pattern.replace(/([a-z])\1*/ig, function(match){
		// Build a simple regexp.  Avoid captures, which would ruin the tokens list
		var s,
			c = match.charAt(0),
			l = match.length,
			p2 = '', p3 = '';
		if(options.strict){
			if(l > 1){ p2 = '0' + '{'+(l-1)+'}'; }
			if(l > 2){ p3 = '0' + '{'+(l-2)+'}'; }
		}else{
			p2 = '0?'; p3 = '0{0,2}';
		}
		switch(c){
			case 'y':
				s = '\\d{2,4}';
				break;
			case 'M':
			case 'L':
				s = (l>2) ? '\\S+?' : '1[0-2]|'+p2+'[1-9]';
				break;
			case 'D':
				s = '[12][0-9][0-9]|3[0-5][0-9]|36[0-6]|'+p2+'[1-9][0-9]|'+p3+'[1-9]';
				break;
			case 'd':
				s = '3[01]|[12]\\d|'+p2+'[1-9]';
				break;
			case 'w':
				s = '[1-4][0-9]|5[0-3]|'+p2+'[1-9]';
				break;
			case 'E':
			case 'e':
			case 'c':
				s = '\\S+';
				break;
			case 'h': //hour (1-12)
				s = '1[0-2]|'+p2+'[1-9]';
				break;
			case 'k': //hour (0-11)
				s = '1[01]|'+p2+'\\d';
				break;
			case 'H': //hour (0-23)
				s = '1\\d|2[0-3]|'+p2+'\\d';
				break;
			case 'K': //hour (1-24)
				s = '1\\d|2[0-4]|'+p2+'[1-9]';
				break;
			case 'm':
			case 's':
				s = '[0-5]\\d';
				break;
			case 'S':
				s = '\\d{'+l+'}';
				break;
			case 'a':
				var am = options.am || bundle['dayPeriods-format-wide-am'],
					pm = options.pm || bundle['dayPeriods-format-wide-pm'];
					s = am + '|' + pm;
				if(!options.strict){
					if(am != am.toLowerCase()){ s += '|' + am.toLowerCase(); }
					if(pm != pm.toLowerCase()){ s += '|' + pm.toLowerCase(); }
					if(s.indexOf('.') != -1){ s += '|' + s.replace(/\./g, ""); }
				}
				s = s.replace(/\./g, "\\.");
				break;
			default:
			// case 'v':
			// case 'z':
			// case 'Z':
				s = ".*";
//				console.log("parse of date format, pattern=" + pattern);
		}

		if(tokens){ tokens.push(match); }

		return "(" + s + ")"; // add capture
	}).replace(/[\xa0 ]/g, "[\\s\\xa0]"); // normalize whitespace.  Need explicit handling of \xa0 for IE.
}

var _customFormats = [];
exports.addCustomFormats = function(/*String*/ packageName, /*String*/ bundleName){
	// summary:
	//		Add a reference to a bundle containing localized custom formats to be
	//		used by date/time formatting and parsing routines.
	//
	// description:
	//		The user may add custom localized formats where the bundle has properties following the
	//		same naming convention used by dojo.cldr: `dateFormat-xxxx` / `timeFormat-xxxx`
	//		The pattern string should match the format used by the CLDR.
	//		See dojo/date/locale.format() for details.
	//		The resources must be loaded by dojo.requireLocalization() prior to use

	_customFormats.push({pkg:packageName,name:bundleName});
};

exports._getGregorianBundle = function(/*String*/ locale){
	var gregorian = {};
	array.forEach(_customFormats, function(desc){
		var bundle = i18n.getLocalization(desc.pkg, desc.name, locale);
		gregorian = lang.mixin(gregorian, bundle);
	}, this);
	return gregorian; /*Object*/
};

exports.addCustomFormats(module.id.replace(/\/date\/locale$/, ".cldr"),"gregorian");

exports.getNames = function(/*String*/ item, /*String*/ type, /*String?*/ context, /*String?*/ locale){
	// summary:
	//		Used to get localized strings from dojo.cldr for day or month names.
	//
	// item:
	//	'months' || 'days'
	// type:
	//	'wide' || 'abbr' || 'narrow' (e.g. "Monday", "Mon", or "M" respectively, in English)
	// context:
	//	'standAlone' || 'format' (default)
	// locale:
	//	override locale used to find the names

	var label,
		lookup = exports._getGregorianBundle(locale),
		props = [item, context, type];
	if(context == 'standAlone'){
		var key = props.join('-');
		label = lookup[key];
		// Fall back to 'format' flavor of name
		if(label[0] == 1){ label = undefined; } // kludge, in the absence of real aliasing support in dojo.cldr
	}
	props[1] = 'format';

	// return by copy so changes won't be made accidentally to the in-memory model
	return (label || lookup[props.join('-')]).concat(); /*Array*/
};

exports.isWeekend = function(/*Date?*/ dateObject, /*String?*/ locale){
	// summary:
	//	Determines if the date falls on a weekend, according to local custom.

	var weekend = supplemental.getWeekend(locale),
		day = (dateObject || new Date()).getDay();
	if(weekend.end < weekend.start){
		weekend.end += 7;
		if(day < weekend.start){ day += 7; }
	}
	return day >= weekend.start && day <= weekend.end; // Boolean
};

// These are used only by format and strftime.  Do they need to be public?  Which module should they go in?

exports._getDayOfYear = function(/*Date*/ dateObject){
	// summary:
	//		gets the day of the year as represented by dateObject
	return date.difference(new Date(dateObject.getFullYear(), 0, 1, dateObject.getHours()), dateObject) + 1; // Number
};

exports._getWeekOfYear = function(/*Date*/ dateObject, /*Number*/ firstDayOfWeek){
	if(arguments.length == 1){ firstDayOfWeek = 0; } // Sunday

	var firstDayOfYear = new Date(dateObject.getFullYear(), 0, 1).getDay(),
		adj = (firstDayOfYear - firstDayOfWeek + 7) % 7,
		week = Math.floor((exports._getDayOfYear(dateObject) + adj - 1) / 7);

	// if year starts on the specified day, start counting weeks at 1
	if(firstDayOfYear == firstDayOfWeek){ week++; }

	return week; // Number
};

return exports;
});

},
'dojo/cldr/supplemental':function(){
define("dojo/cldr/supplemental", ["../_base/lang", "../i18n"], function(lang, i18n){

// module:
//		dojo/cldr/supplemental


var supplemental = {
	// summary:
	//		TODOC
};
lang.setObject("dojo.cldr.supplemental", supplemental);

supplemental.getFirstDayOfWeek = function(/*String?*/locale){
	// summary:
	//		Returns a zero-based index for first day of the week
	// description:
	//		Returns a zero-based index for first day of the week, as used by the local (Gregorian) calendar.
	//		e.g. Sunday (returns 0), or Monday (returns 1)

	// from http://www.unicode.org/cldr/data/common/supplemental/supplementalData.xml:supplementalData/weekData/firstDay
	var firstDay = {/*default is 1=Monday*/
		bd:5,mv:5,
		ae:6,af:6,bh:6,dj:6,dz:6,eg:6,iq:6,ir:6,jo:6,kw:6,
		ly:6,ma:6,om:6,qa:6,sa:6,sd:6,sy:6,ye:6,
		ag:0,ar:0,as:0,au:0,br:0,bs:0,bt:0,bw:0,by:0,bz:0,ca:0,cn:0,
		co:0,dm:0,'do':0,et:0,gt:0,gu:0,hk:0,hn:0,id:0,ie:0,il:0,'in':0,
		jm:0,jp:0,ke:0,kh:0,kr:0,la:0,mh:0,mm:0,mo:0,mt:0,mx:0,mz:0,
		ni:0,np:0,nz:0,pa:0,pe:0,ph:0,pk:0,pr:0,py:0,sg:0,sv:0,th:0,
		tn:0,tt:0,tw:0,um:0,us:0,ve:0,vi:0,ws:0,za:0,zw:0
	};

	var country = supplemental._region(locale);
	var dow = firstDay[country];
	return (dow === undefined) ? 1 : dow; /*Number*/
};

supplemental._region = function(/*String?*/locale){
	locale = i18n.normalizeLocale(locale);
	var tags = locale.split('-');
	var region = tags[1];
	if(!region){
		// IE often gives language only (#2269)
		// Arbitrary mappings of language-only locales to a country:
		region = {
			aa:"et", ab:"ge", af:"za", ak:"gh", am:"et", ar:"eg", as:"in", av:"ru", ay:"bo", az:"az", ba:"ru",
			be:"by", bg:"bg", bi:"vu", bm:"ml", bn:"bd", bo:"cn", br:"fr", bs:"ba", ca:"es", ce:"ru", ch:"gu",
			co:"fr", cr:"ca", cs:"cz", cv:"ru", cy:"gb", da:"dk", de:"de", dv:"mv", dz:"bt", ee:"gh", el:"gr",
			en:"us", es:"es", et:"ee", eu:"es", fa:"ir", ff:"sn", fi:"fi", fj:"fj", fo:"fo", fr:"fr", fy:"nl",
			ga:"ie", gd:"gb", gl:"es", gn:"py", gu:"in", gv:"gb", ha:"ng", he:"il", hi:"in", ho:"pg", hr:"hr",
			ht:"ht", hu:"hu", hy:"am", ia:"fr", id:"id", ig:"ng", ii:"cn", ik:"us", "in":"id", is:"is", it:"it",
			iu:"ca", iw:"il", ja:"jp", ji:"ua", jv:"id", jw:"id", ka:"ge", kg:"cd", ki:"ke", kj:"na", kk:"kz",
			kl:"gl", km:"kh", kn:"in", ko:"kr", ks:"in", ku:"tr", kv:"ru", kw:"gb", ky:"kg", la:"va", lb:"lu",
			lg:"ug", li:"nl", ln:"cd", lo:"la", lt:"lt", lu:"cd", lv:"lv", mg:"mg", mh:"mh", mi:"nz", mk:"mk",
			ml:"in", mn:"mn", mo:"ro", mr:"in", ms:"my", mt:"mt", my:"mm", na:"nr", nb:"no", nd:"zw", ne:"np",
			ng:"na", nl:"nl", nn:"no", no:"no", nr:"za", nv:"us", ny:"mw", oc:"fr", om:"et", or:"in", os:"ge",
			pa:"in", pl:"pl", ps:"af", pt:"br", qu:"pe", rm:"ch", rn:"bi", ro:"ro", ru:"ru", rw:"rw", sa:"in",
			sd:"in", se:"no", sg:"cf", si:"lk", sk:"sk", sl:"si", sm:"ws", sn:"zw", so:"so", sq:"al", sr:"rs",
			ss:"za", st:"za", su:"id", sv:"se", sw:"tz", ta:"in", te:"in", tg:"tj", th:"th", ti:"et", tk:"tm",
			tl:"ph", tn:"za", to:"to", tr:"tr", ts:"za", tt:"ru", ty:"pf", ug:"cn", uk:"ua", ur:"pk", uz:"uz",
			ve:"za", vi:"vn", wa:"be", wo:"sn", xh:"za", yi:"il", yo:"ng", za:"cn", zh:"cn", zu:"za",
			ace:"id", ady:"ru", agq:"cm", alt:"ru", amo:"ng", asa:"tz", ast:"es", awa:"in", bal:"pk",
			ban:"id", bas:"cm", bax:"cm", bbc:"id", bem:"zm", bez:"tz", bfq:"in", bft:"pk", bfy:"in",
			bhb:"in", bho:"in", bik:"ph", bin:"ng", bjj:"in", bku:"ph", bqv:"ci", bra:"in", brx:"in",
			bss:"cm", btv:"pk", bua:"ru", buc:"yt", bug:"id", bya:"id", byn:"er", cch:"ng", ccp:"in",
			ceb:"ph", cgg:"ug", chk:"fm", chm:"ru", chp:"ca", chr:"us", cja:"kh", cjm:"vn", ckb:"iq",
			crk:"ca", csb:"pl", dar:"ru", dav:"ke", den:"ca", dgr:"ca", dje:"ne", doi:"in", dsb:"de",
			dua:"cm", dyo:"sn", dyu:"bf", ebu:"ke", efi:"ng", ewo:"cm", fan:"gq", fil:"ph", fon:"bj",
			fur:"it", gaa:"gh", gag:"md", gbm:"in", gcr:"gf", gez:"et", gil:"ki", gon:"in", gor:"id",
			grt:"in", gsw:"ch", guz:"ke", gwi:"ca", haw:"us", hil:"ph", hne:"in", hnn:"ph", hoc:"in",
			hoj:"in", ibb:"ng", ilo:"ph", inh:"ru", jgo:"cm", jmc:"tz", kaa:"uz", kab:"dz", kaj:"ng",
			kam:"ke", kbd:"ru", kcg:"ng", kde:"tz", kdt:"th", kea:"cv", ken:"cm", kfo:"ci", kfr:"in",
			kha:"in", khb:"cn", khq:"ml", kht:"in", kkj:"cm", kln:"ke", kmb:"ao", koi:"ru", kok:"in",
			kos:"fm", kpe:"lr", krc:"ru", kri:"sl", krl:"ru", kru:"in", ksb:"tz", ksf:"cm", ksh:"de",
			kum:"ru", lag:"tz", lah:"pk", lbe:"ru", lcp:"cn", lep:"in", lez:"ru", lif:"np", lis:"cn",
			lki:"ir", lmn:"in", lol:"cd", lua:"cd", luo:"ke", luy:"ke", lwl:"th", mad:"id", mag:"in",
			mai:"in", mak:"id", man:"gn", mas:"ke", mdf:"ru", mdh:"ph", mdr:"id", men:"sl", mer:"ke",
			mfe:"mu", mgh:"mz", mgo:"cm", min:"id", mni:"in", mnk:"gm", mnw:"mm", mos:"bf", mua:"cm",
			mwr:"in", myv:"ru", nap:"it", naq:"na", nds:"de", "new":"np", niu:"nu", nmg:"cm", nnh:"cm",
			nod:"th", nso:"za", nus:"sd", nym:"tz", nyn:"ug", pag:"ph", pam:"ph", pap:"bq", pau:"pw",
			pon:"fm", prd:"ir", raj:"in", rcf:"re", rej:"id", rjs:"np", rkt:"in", rof:"tz", rwk:"tz",
			saf:"gh", sah:"ru", saq:"ke", sas:"id", sat:"in", saz:"in", sbp:"tz", scn:"it", sco:"gb",
			sdh:"ir", seh:"mz", ses:"ml", shi:"ma", shn:"mm", sid:"et", sma:"se", smj:"se", smn:"fi",
			sms:"fi", snk:"ml", srn:"sr", srr:"sn", ssy:"er", suk:"tz", sus:"gn", swb:"yt", swc:"cd",
			syl:"bd", syr:"sy", tbw:"ph", tcy:"in", tdd:"cn", tem:"sl", teo:"ug", tet:"tl", tig:"er",
			tiv:"ng", tkl:"tk", tmh:"ne", tpi:"pg", trv:"tw", tsg:"ph", tts:"th", tum:"mw", tvl:"tv",
			twq:"ne", tyv:"ru", tzm:"ma", udm:"ru", uli:"fm", umb:"ao", unr:"in", unx:"in", vai:"lr",
			vun:"tz", wae:"ch", wal:"et", war:"ph", xog:"ug", xsr:"np", yao:"mz", yap:"fm", yav:"cm", zza:"tr"
		}[tags[0]];
	}else if(region.length == 4){
		// The ISO 3166 country code is usually in the second position, unless a
		// 4-letter script is given. See http://www.ietf.org/rfc/rfc4646.txt
		region = tags[2];
	}
	return region;
};

supplemental.getWeekend = function(/*String?*/locale){
	// summary:
	//		Returns a hash containing the start and end days of the weekend
	// description:
	//		Returns a hash containing the start and end days of the weekend according to local custom using locale,
	//		or by default in the user's locale.
	//		e.g. {start:6, end:0}

	// from http://www.unicode.org/cldr/data/common/supplemental/supplementalData.xml:supplementalData/weekData/weekend{Start,End}
	var weekendStart = {/*default is 6=Saturday*/
			'in':0,
			af:4,dz:4,ir:4,om:4,sa:4,ye:4,
			ae:5,bh:5,eg:5,il:5,iq:5,jo:5,kw:5,ly:5,ma:5,qa:5,sd:5,sy:5,tn:5
		},

		weekendEnd = {/*default is 0=Sunday*/
			af:5,dz:5,ir:5,om:5,sa:5,ye:5,
			ae:6,bh:5,eg:6,il:6,iq:6,jo:6,kw:6,ly:6,ma:6,qa:6,sd:6,sy:6,tn:6
		},

		country = supplemental._region(locale),
		start = weekendStart[country],
		end = weekendEnd[country];

	if(start === undefined){start=6;}
	if(end === undefined){end=0;}
	return {start:start, end:end}; /*Object {start,end}*/
};

return supplemental;
});

},
'dojo/regexp':function(){
define("dojo/regexp", ["./_base/kernel", "./_base/lang"], function(dojo, lang){

// module:
//		dojo/regexp

var regexp = {
	// summary:
	//		Regular expressions and Builder resources
};
lang.setObject("dojo.regexp", regexp);

regexp.escapeString = function(/*String*/str, /*String?*/except){
	// summary:
	//		Adds escape sequences for special characters in regular expressions
	// except:
	//		a String with special characters to be left unescaped

	return str.replace(/([\.$?*|{}\(\)\[\]\\\/\+^])/g, function(ch){
		if(except && except.indexOf(ch) != -1){
			return ch;
		}
		return "\\" + ch;
	}); // String
};

regexp.buildGroupRE = function(/*Object|Array*/arr, /*Function*/re, /*Boolean?*/nonCapture){
	// summary:
	//		Builds a regular expression that groups subexpressions
	// description:
	//		A utility function used by some of the RE generators. The
	//		subexpressions are constructed by the function, re, in the second
	//		parameter.  re builds one subexpression for each elem in the array
	//		a, in the first parameter. Returns a string for a regular
	//		expression that groups all the subexpressions.
	// arr:
	//		A single value or an array of values.
	// re:
	//		A function. Takes one parameter and converts it to a regular
	//		expression.
	// nonCapture:
	//		If true, uses non-capturing match, otherwise matches are retained
	//		by regular expression. Defaults to false

	// case 1: a is a single value.
	if(!(arr instanceof Array)){
		return re(arr); // String
	}

	// case 2: a is an array
	var b = [];
	for(var i = 0; i < arr.length; i++){
		// convert each elem to a RE
		b.push(re(arr[i]));
	}

	 // join the REs as alternatives in a RE group.
	return regexp.group(b.join("|"), nonCapture); // String
};

regexp.group = function(/*String*/expression, /*Boolean?*/nonCapture){
	// summary:
	//		adds group match to expression
	// nonCapture:
	//		If true, uses non-capturing match, otherwise matches are retained
	//		by regular expression.
	return "(" + (nonCapture ? "?:":"") + expression + ")"; // String
};

return regexp;
});

},
'dojo/string':function(){
define("dojo/string", [
	"./_base/kernel",	// kernel.global
	"./_base/lang"
], function(kernel, lang){

// module:
//		dojo/string

var string = {
	// summary:
	//		String utilities for Dojo
};
lang.setObject("dojo.string", string);

string.rep = function(/*String*/str, /*Integer*/num){
	// summary:
	//		Efficiently replicate a string `n` times.
	// str:
	//		the string to replicate
	// num:
	//		number of times to replicate the string

	if(num <= 0 || !str){ return ""; }

	var buf = [];
	for(;;){
		if(num & 1){
			buf.push(str);
		}
		if(!(num >>= 1)){ break; }
		str += str;
	}
	return buf.join("");	// String
};

string.pad = function(/*String*/text, /*Integer*/size, /*String?*/ch, /*Boolean?*/end){
	// summary:
	//		Pad a string to guarantee that it is at least `size` length by
	//		filling with the character `ch` at either the start or end of the
	//		string. Pads at the start, by default.
	// text:
	//		the string to pad
	// size:
	//		length to provide padding
	// ch:
	//		character to pad, defaults to '0'
	// end:
	//		adds padding at the end if true, otherwise pads at start
	// example:
	//	|	// Fill the string to length 10 with "+" characters on the right.  Yields "Dojo++++++".
	//	|	string.pad("Dojo", 10, "+", true);

	if(!ch){
		ch = '0';
	}
	var out = String(text),
		pad = string.rep(ch, Math.ceil((size - out.length) / ch.length));
	return end ? out + pad : pad + out;	// String
};

string.substitute = function(	/*String*/		template,
									/*Object|Array*/map,
									/*Function?*/	transform,
									/*Object?*/		thisObject){
	// summary:
	//		Performs parameterized substitutions on a string. Throws an
	//		exception if any parameter is unmatched.
	// template:
	//		a string with expressions in the form `${key}` to be replaced or
	//		`${key:format}` which specifies a format function. keys are case-sensitive.
	// map:
	//		hash to search for substitutions
	// transform:
	//		a function to process all parameters before substitution takes
	//		place, e.g. mylib.encodeXML
	// thisObject:
	//		where to look for optional format function; default to the global
	//		namespace
	// example:
	//		Substitutes two expressions in a string from an Array or Object
	//	|	// returns "File 'foo.html' is not found in directory '/temp'."
	//	|	// by providing substitution data in an Array
	//	|	string.substitute(
	//	|		"File '${0}' is not found in directory '${1}'.",
	//	|		["foo.html","/temp"]
	//	|	);
	//	|
	//	|	// also returns "File 'foo.html' is not found in directory '/temp'."
	//	|	// but provides substitution data in an Object structure.  Dotted
	//	|	// notation may be used to traverse the structure.
	//	|	string.substitute(
	//	|		"File '${name}' is not found in directory '${info.dir}'.",
	//	|		{ name: "foo.html", info: { dir: "/temp" } }
	//	|	);
	// example:
	//		Use a transform function to modify the values:
	//	|	// returns "file 'foo.html' is not found in directory '/temp'."
	//	|	string.substitute(
	//	|		"${0} is not found in ${1}.",
	//	|		["foo.html","/temp"],
	//	|		function(str){
	//	|			// try to figure out the type
	//	|			var prefix = (str.charAt(0) == "/") ? "directory": "file";
	//	|			return prefix + " '" + str + "'";
	//	|		}
	//	|	);
	// example:
	//		Use a formatter
	//	|	// returns "thinger -- howdy"
	//	|	string.substitute(
	//	|		"${0:postfix}", ["thinger"], null, {
	//	|			postfix: function(value, key){
	//	|				return value + " -- howdy";
	//	|			}
	//	|		}
	//	|	);

	thisObject = thisObject || kernel.global;
	transform = transform ?
		lang.hitch(thisObject, transform) : function(v){ return v; };

	return template.replace(/\$\{([^\s\:\}]+)(?:\:([^\s\:\}]+))?\}/g,
		function(match, key, format){
			var value = lang.getObject(key, false, map);
			if(format){
				value = lang.getObject(format, false, thisObject).call(thisObject, value, key);
			}
			return transform(value, key).toString();
		}); // String
};

string.trim = String.prototype.trim ?
	lang.trim : // aliasing to the native function
	function(str){
		str = str.replace(/^\s+/, '');
		for(var i = str.length - 1; i >= 0; i--){
			if(/\S/.test(str.charAt(i))){
				str = str.substring(0, i + 1);
				break;
			}
		}
		return str;
	};

/*=====
 string.trim = function(str){
	 // summary:
	 //		Trims whitespace from both sides of the string
	 // str: String
	 //		String to be trimmed
	 // returns: String
	 //		Returns the trimmed string
	 // description:
	 //		This version of trim() was taken from [Steven Levithan's blog](http://blog.stevenlevithan.com/archives/faster-trim-javascript).
	 //		The short yet performant version of this function is dojo.trim(),
	 //		which is part of Dojo base.  Uses String.prototype.trim instead, if available.
	 return "";	// String
 };
 =====*/

	return string;
});

},
'epi/shell/ViewSettings':function(){
﻿define("epi/shell/ViewSettings", [],

    function () {

        return {
        //  summary:
        //      Contains settings for the current view.
        //
        // tags:
        //      public

        // viewName: String
        //  Name of current view
            viewName: null,

            // settings: Object
            //  Contains settings for the current view.
            settings: null
        };
    });


},
'epi/shell/Stateful':function(){
﻿define("epi/shell/Stateful", [
    "dojo",
    "epi",
    "dojo/Stateful"],

function (dojo, epi, Stateful) {

    return dojo.declare(Stateful, {
        // summary:
        //		Base class for objects that provide named properties with optional getter/setter
        //		control and the ability to watch for property changes
        // example:
        //	|	var obj = new dojo/Stateful();
        //	|	obj.watch("foo", function(){
        //	|		console.log("foo changed to " + this.get("foo"));
        //	|	});
        //	|	obj.set("foo","bar");
        //
        // tags:
        //      public

        //        _prefix: "epi-",
        _prefix: "epi-",

        postscript: function () {
            if (arguments.length > 0) {
                throw new Error("Stateful should not be instantiated with initial params. Use set method to set values.");
            }

            this.inherited(arguments);
        },

        get: function (/*String*/name) {
            // summary:
            //		Get a property on a Stateful instance.
            // name:
            //		The property to get.
            // description:
            //		Get a named property on a Stateful object. The property may
            //		potentially be retrieved from a child object if a dot-delimited
            //		string is passed through, such as "A.B.C".
            return dojo.getObject(this._prefix + name, false, this);
        },

        set: function (/*String*/name, /*Object*/value) {
            // summary:
            //		Set a property on a Stateful instance
            // name:
            //		The property to set.
            // value:
            //		The value to set in the property.
            // description:
            //		Sets named properties on a stateful object and notifies any watchers of
            //		the property. The property may potentially be set on a child object if
            //		a dot-delimited string is passed through, such as "A.B.C".
            //		For example:
            //	|	stateful = new dojo/Stateful();
            //	|	stateful.watch(function(name, oldValue, value){
            //	|		// this will be called on the set below
            //	|	}
            //	|	stateful.set(foo, 5);
            //
            //		set() may also be called with a hash of name/value pairs, ex:
            //	|	myObj.set({
            //	|		foo: "Howdy",
            //	|		bar: 3
            //	|	})
            //		This is equivalent to calling set(foo, "Howdy") and set(bar, 3)

            if (typeof name === "object") {
                for (var x in name) {
                    this.set(x, name[x]);
                }
                return this;
            }
            var oldValue = dojo.getObject(this._prefix + name, false, this);

            var newValue = value;

            dojo.setObject(this._prefix + name, newValue, this);
            if (this._watchCallbacks) {
                this._watchCallbacks(name, oldValue, newValue);
                this._callbackParentObjects(name, oldValue, newValue);
                this._callbackChildObjects(name, oldValue, newValue);
            }
            return this;
        },

        watch: function (/*String?*/name, /*Function*/callback) {
            // summary:
            //		Watches a property for changes
            //	name:
            //		Indicates the property to watch. This is optional (the callback may be the
            //		only parameter), and if omitted, all the properties will be watched
            // returns:
            //		An object handle for the watch. The unwatch method of this object
            //		can be used to discontinue watching this property:
            //		|	var watchHandle = obj.watch("foo", callback);
            //		|	watchHandle.unwatch(); // callback won't be called now
            //	callback:
            //		The function to execute when the property changes. This will be called after
            //		the property has been changed. The callback will be called with the |this|
            //		set to the instance, the first argument as the name of the property, the
            //		second argument as the old value and the third argument as the new value.
            var callbacks = this._watchCallbacks;

            if (!callbacks) {
                var self = this;
                callbacks = this._watchCallbacks = function (name, oldValue, value, ignoreCatchall) {
                    var notify = function (propertyCallbacks) {
                        if (propertyCallbacks) {
                            propertyCallbacks = propertyCallbacks.slice();
                            for (var i = 0, l = propertyCallbacks.length; i < l; i++) {
                                try {
                                    propertyCallbacks[i].call(self, name, oldValue, value);
                                } catch (e) {
                                    console.error(e);
                                }
                            }
                        }
                    };
                    notify(callbacks["_" + name]);
                    if (!ignoreCatchall) {
                        notify(callbacks["*"]); // the catch-all
                    }
                }; // we use a function instead of an object so it will be ignored by JSON conversion
            }
            if (!callback && typeof name === "function") {
                callback = name;
                name = "*";
            } else {
                // prepend with dash to prevent name conflicts with function (like "name" property)
                name = "_" + name;
            }
            var propertyCallbacks = callbacks[name];
            if (typeof propertyCallbacks !== "object") {
                propertyCallbacks = callbacks[name] = [];
            }
            propertyCallbacks.push(callback);

            var handle = {

            };
            handle.unwatch = handle.remove = function () {
                propertyCallbacks.splice(dojo.indexOf(propertyCallbacks, callback), 1);
            };
            return handle;
        },

        _callbackChildObjects: function (/*String*/name, /*Object*/oldValue, /*Object*/value) {
            // summary:
            //		Will recursively call any watch callbacks on the child properties of value; if value is an object.
            // name:
            //		The name of the property associated with value.
            // oldValue:
            //		The old value of the property.
            // value:
            //		The value of the property.
            // tags:
            //		private

            var item;

            if (dojo.isObject(value)) {
                for (item in value) {
                    if (!dojo.isFunction(value[item])) { //prevent infinite loop on function's prototype
                        this._watchCallbacks(name + "." + item, oldValue ? oldValue[item] : null, value[item]);
                        this._callbackChildObjects(name + "." + item, oldValue ? oldValue[item] : null, value[item]);
                    }
                }
            }
        },

        _callbackParentObjects: function (/*String*/name, /*Object*/oldValue, /*Object*/value) {
            // summary:
            //		Will recursively call any watch callbacks on the parent properties of value; if name is a dot-delimited string.
            // name:
            //		The name of the property associated with value.
            // oldValue:
            //		The old value of the property.
            // value:
            //		The value of the property.
            // tags:
            //		private

            var fragments = name.split(".");
            if (fragments.length === 1) {
                return;
            }

            // Get the last item from the fragments as the key for the new object.
            var key = fragments.pop();

            // Join the remaining fragments to create the new property name.
            name = fragments.join(".");

            var parent = this.get(name);
            oldValue = this._getUpdatedObject(parent, key, oldValue);
            value = parent;

            this._watchCallbacks(name, oldValue, value);
            this._callbackParentObjects(name, oldValue, value);
        },

        _getUpdatedObject: function (/*Object*/parent, /*String*/key, /*Object*/value) {
            // summary:
            //		Will create a new object and add the key value pair.
            // parent:
            //		The value of parent property
            // key:
            //		The key of the property associated with value.
            // value:
            //		The value of the property.
            // tags:
            //		private

            var newObject = dojo.clone(parent);
            newObject[key] = value;
            return newObject;
        }
    });
});

},
'epi/shell/messaging':function(){
﻿define("epi/shell/messaging", ["dojo/_base/lang", "dojo/topic", "epi"], function (lang, topic, epi) {

    var _epiJQuery = lang.getObject("epiJQuery");

    var messaging = function () {
        // summary:
        //      Responsible for connecting dojo publish/subscribe with messaging capabilities of external libraries, such as jQuery.
        //      Automatically connects to dojo/topic messaging as well as jQuery.trigger if the global epiJQuery object is available.
        //
        // tags:
        //      internal


        //Object that holds the registered libraries to pass messages to
        var libraries = {};

        return {
            subscribe: function (subject, context, method) {
                // summary:
                //      Sets up a subscription
                // subject: String
                //      The signature to listen for
                // context: Object
                //      Scope in which method will be invoked, or null for default scope
                // method: String|Function
                //      The name of a function in context, or a function reference. This is the function that is invoked when topic is published

                return topic.subscribe(topic, lang.hitch(context, method));
            },

            unsubscribe: function (handle) {
                // summary:
                //      Remove a subscription
                // handle: Handle
                //      The handle returned from a call to subscribe.

                if (handle) {
                    handle.remove();
                }
            },

            publish: function (topic, args /*, string libName */) {
                // summary:
                //      Invoke all listener method subscribed to the topic.
                // topic: String
                //      The name of the topic to publish.
                // args: Array
                //      An array of arguments. The arguments will be applied to each topic subscriber.

                var libName = arguments[2] || null;
                for (var i in libraries) {
                    var lib = libraries[i];
                    if (lib.name !== libName) {
                        lib.context[lib.publishMethod]({ data: topic, __epiPublished: true }, args);
                    }
                }
            },

            registerLibrary: function (name, context, publishMethod, publishFilter) {
                // summary:
                //      Registers JavaScript library to enable sharing of messages.
                // name: String
                //      Name of the library
                // context: Object
                //      Namespace object for the publish method in the library (dojo, jQuery.event etc)
                // publishMethod: String
                //      Name of the publishing method on the context object (publish, trigger etc)
                // publishFilter: function
                //      Function that enables of filtering of which messages to pass on from the library.
                //      Can be good to use if the message bus is chatty.

                libraries[name] = { name: name, publishMethod: publishMethod };
                libraries[name].context = context;
                libraries[name].originalPublish = context[publishMethod];

                context[publishMethod] = function (topic, args) {
                    var newArguments = Array.prototype.slice.call(arguments);

                    if (!topic.__epiPublished) {
                        if (!publishFilter || (publishFilter.apply(this, arguments))) {
                            messaging.publish(topic, args, name);
                        }
                    } else {
                        newArguments[0] = topic.data;
                    }
                    libraries[name].originalPublish.apply(this, newArguments);
                };
            },

            unRegisterLibrary: function (name) {
                // summary:
                //      Removes the JavaScript library from message sharing.
                // name: String
                //      Name of the library

                var lib = libraries[name];
                lib.context[lib.publishMethod] = lib.originalPublish;
                delete libraries[name];
            }

        };
    } ();


    //Registers jQuery for message sharing
    if (_epiJQuery !== undefined) {
        messaging.registerLibrary("jquery", _epiJQuery.event, "trigger",
            function (event, data, elem) {
                if (typeof event == "string" && !elem) {
                    return true;
                }
                return false;
            });
    } else {
        console.warn("epi/shell/messaging module was loaded but could not register epiJQuery for message sharing");
    }

    //Registers dojo for message sharing
    messaging.registerLibrary("dojo", topic, "publish");

    lang.setObject("epi.shell.messaging", messaging);

    return messaging;
});

},
'epi-find/store/IdToNameResolvingMixin':function(){
define([
        "dojo/_base/declare",
        "dojo/_base/config",
        "dojo/json",
		"dojox/html/entities"
],
function (declare, config, JSON) {
    return declare([], {
        // summary:
        //      Store mixin that finds out page names from the page IDs. Should be mixed to a ResourceStore based store.
        nameField: "SearchTitle$$string",
        nameFieldInBase64: "TitleInBase64$$string",

        query: function (query, options) {
            var self = this;
            function b64DecodeUnicode(str) {
                // Going backwards: from bytestream, to percent-encoding, to original string.
                return decodeURIComponent(atob(str).split("").map(function(c) {
                    return "%" + ("00" + c.charCodeAt(0).toString(16)).slice(-2);
                }).join(""));
            }
			
            return this.inherited(arguments).then(function(results) {
                if (results.length === 0) {
                    return results;
                }

                // Let's resolve some names
                var mquery = "";
                for(var i in results) {
                    var split = results[i].id.split("/");
                    var body = {
                        fields: [self.nameField, self.nameFieldInBase64],
                        query: {
                            ids: {
                                type: split[0],
                                values: [split[1]]
                            }
                        }
                    };

                    mquery += "{}\n";
                    mquery += JSON.stringify(body) + "\n";
                }

                var nameField = self.nameField;
                var nameFieldInBase64 = self.nameFieldInBase64;

                return self.xhrHandler.xhr("POST", {
                    url: config.find.serviceApiBaseUrl+"_msearch",
                    handleAs: "json",
                    postData: mquery,
                    xsrfProtection: true
                }).then(function (nameResult) {
                    nameResult.responses.forEach(function(response, n) {
                        var hits = response.hits.hits;
                        // should only have one hit / msearch
                        if (hits && hits.length == 1) {
                            if(hits[0].fields.hasOwnProperty(nameField))
                            {
                                results[n].name = dojox.html.entities.encode(hits[0].fields[nameField]);
                            }
                            else if (hits[0].fields.hasOwnProperty(nameFieldInBase64))
                            {
                                results[n].name = dojox.html.entities.encode(b64DecodeUnicode(hits[0].fields[nameFieldInBase64]));
                            }
                            var index = hits[0]._index.split("-");
                            index.length === 2 ? results[n].language = index[1] : null;
                        }
                    });

                    return results;
                });
            });
        }
    });

});
},
'dojox/html/entities':function(){
define("dojox/html/entities", ["dojo/_base/lang"], function(lang) {
	// dojox.html.entities.html [public] Array
	//		Entity characters for HTML, represented as an array of
	//		character code, entity name (minus & and ; wrapping.
	//		The function wrapper is to fix global leking with the build tools.
	var dhe = lang.getObject("dojox.html.entities",true);	
	
	var _applyEncodingMap = function(str, map){
		// summary:
		//		Private internal function for performing encoding of entity characters.
		// tags:
		//		private
	
		// Check to see if we have genned and cached a regexp for this map yet
		// If we have, use it, if not, gen it, cache, then use.
		var mapper, regexp;
		if(map._encCache &&
			map._encCache.regexp &&
			map._encCache.mapper &&
			map.length == map._encCache.length){
			mapper = map._encCache.mapper;
			regexp = map._encCache.regexp;
		}else{
			mapper = {};
			regexp = ["["];
			var i;
			for(i = 0; i < map.length; i++){
				mapper[map[i][0]] = "&" + map[i][1] + ";";
				regexp.push(map[i][0]);
			}
			regexp.push("]");
			regexp = new RegExp(regexp.join(""), "g");
			map._encCache = {
				mapper: mapper,
				regexp: regexp,
				length: map.length
			};
		}
		str = str.replace(regexp, function(c){
			return mapper[c];
		});
		return str;
	};
	
	var _applyDecodingMap = function(str, map){
		// summary:
		//		Private internal function for performing decoding of entity characters.
		// tags:
		//		private
		var mapper, regexp;
		if(map._decCache &&
			map._decCache.regexp &&
			map._decCache.mapper &&
			map.length == map._decCache.length){
			mapper = map._decCache.mapper;
			regexp = map._decCache.regexp;
		}else{
			mapper = {};
			regexp = ["("];
			var i;
			for(i = 0; i < map.length; i++){
				var e = "&" + map[i][1] + ";";
				if(i){regexp.push("|");}
				mapper[e] = map[i][0];
				regexp.push(e);
			}
			regexp.push(")");
			regexp = new RegExp(regexp.join(""), "g");
			map._decCache = {
				mapper: mapper,
				regexp: regexp,
				length: map.length
			};
		}
		str = str.replace(regexp, function(c){
			return mapper[c];
		});
		return str;
	};

	dhe.html = [
		["\u0026","amp"], ["\u0022","quot"],["\u003C","lt"], ["\u003E","gt"],
		["\u00A0","nbsp"]
	];
	
	// dojox.html.entities.latin [public] Array
	//		Entity characters for Latin characters and similar, represented as an array of
	//		character code, entity name (minus & and ; wrapping.
	dhe.latin = [
		["\u00A1","iexcl"],["\u00A2","cent"],["\u00A3","pound"],["\u20AC","euro"],
		["\u00A4","curren"],["\u00A5","yen"],["\u00A6","brvbar"],["\u00A7","sect"],
		["\u00A8","uml"],["\u00A9","copy"],["\u00AA","ordf"],["\u00AB","laquo"],
		["\u00AC","not"],["\u00AD","shy"],["\u00AE","reg"],["\u00AF","macr"],
		["\u00B0","deg"],["\u00B1","plusmn"],["\u00B2","sup2"],["\u00B3","sup3"],
		["\u00B4","acute"],["\u00B5","micro"],["\u00B6","para"],["\u00B7","middot"],
		["\u00B8","cedil"],["\u00B9","sup1"],["\u00BA","ordm"],["\u00BB","raquo"],
		["\u00BC","frac14"],["\u00BD","frac12"],["\u00BE","frac34"],["\u00BF","iquest"],
		["\u00C0","Agrave"],["\u00C1","Aacute"],["\u00C2","Acirc"],["\u00C3","Atilde"],
		["\u00C4","Auml"],["\u00C5","Aring"],["\u00C6","AElig"],["\u00C7","Ccedil"],
		["\u00C8","Egrave"],["\u00C9","Eacute"],["\u00CA","Ecirc"],["\u00CB","Euml"],
		["\u00CC","Igrave"],["\u00CD","Iacute"],["\u00CE","Icirc"],["\u00CF","Iuml"],
		["\u00D0","ETH"],["\u00D1","Ntilde"],["\u00D2","Ograve"],["\u00D3","Oacute"],
		["\u00D4","Ocirc"],["\u00D5","Otilde"],["\u00D6","Ouml"],["\u00D7","times"],
		["\u00D8","Oslash"],["\u00D9","Ugrave"],["\u00DA","Uacute"],["\u00DB","Ucirc"],
		["\u00DC","Uuml"],["\u00DD","Yacute"],["\u00DE","THORN"],["\u00DF","szlig"],
		["\u00E0","agrave"],["\u00E1","aacute"],["\u00E2","acirc"],["\u00E3","atilde"],
		["\u00E4","auml"],["\u00E5","aring"],["\u00E6","aelig"],["\u00E7","ccedil"],
		["\u00E8","egrave"],["\u00E9","eacute"],["\u00EA","ecirc"],["\u00EB","euml"],
		["\u00EC","igrave"],["\u00ED","iacute"],["\u00EE","icirc"],["\u00EF","iuml"],
		["\u00F0","eth"],["\u00F1","ntilde"],["\u00F2","ograve"],["\u00F3","oacute"],
		["\u00F4","ocirc"],["\u00F5","otilde"],["\u00F6","ouml"],["\u00F7","divide"],
		["\u00F8","oslash"],["\u00F9","ugrave"],["\u00FA","uacute"],["\u00FB","ucirc"],
		["\u00FC","uuml"],["\u00FD","yacute"],["\u00FE","thorn"],["\u00FF","yuml"],
		["\u0192","fnof"],["\u0391","Alpha"],["\u0392","Beta"],["\u0393","Gamma"],
		["\u0394","Delta"],["\u0395","Epsilon"],["\u0396","Zeta"],["\u0397","Eta"],
		["\u0398","Theta"], ["\u0399","Iota"],["\u039A","Kappa"],["\u039B","Lambda"],
		["\u039C","Mu"],["\u039D","Nu"],["\u039E","Xi"],["\u039F","Omicron"],
		["\u03A0","Pi"],["\u03A1","Rho"],["\u03A3","Sigma"],["\u03A4","Tau"],
		["\u03A5","Upsilon"],["\u03A6","Phi"],["\u03A7","Chi"],["\u03A8","Psi"],
		["\u03A9","Omega"],["\u03B1","alpha"],["\u03B2","beta"],["\u03B3","gamma"],
		["\u03B4","delta"],["\u03B5","epsilon"],["\u03B6","zeta"],["\u03B7","eta"],
		["\u03B8","theta"],["\u03B9","iota"],["\u03BA","kappa"],["\u03BB","lambda"],
		["\u03BC","mu"],["\u03BD","nu"],["\u03BE","xi"],["\u03BF","omicron"],
		["\u03C0","pi"],["\u03C1","rho"],["\u03C2","sigmaf"],["\u03C3","sigma"],
		["\u03C4","tau"],["\u03C5","upsilon"],["\u03C6","phi"],["\u03C7","chi"],
		["\u03C8","psi"],["\u03C9","omega"],["\u03D1","thetasym"],["\u03D2","upsih"],
		["\u03D6","piv"],["\u2022","bull"],["\u2026","hellip"],["\u2032","prime"],
		["\u2033","Prime"],["\u203E","oline"],["\u2044","frasl"],["\u2118","weierp"],
		["\u2111","image"],["\u211C","real"],["\u2122","trade"],["\u2135","alefsym"],
		["\u2190","larr"],["\u2191","uarr"],["\u2192","rarr"],["\u2193","darr"],
		["\u2194","harr"],["\u21B5","crarr"],["\u21D0","lArr"],["\u21D1","uArr"],
		["\u21D2","rArr"],["\u21D3","dArr"],["\u21D4","hArr"],["\u2200","forall"],
		["\u2202","part"],["\u2203","exist"],["\u2205","empty"],["\u2207","nabla"],
		["\u2208","isin"],["\u2209","notin"],["\u220B","ni"],["\u220F","prod"],
		["\u2211","sum"],["\u2212","minus"],["\u2217","lowast"],["\u221A","radic"],
		["\u221D","prop"],["\u221E","infin"],["\u2220","ang"],["\u2227","and"],
		["\u2228","or"],["\u2229","cap"],["\u222A","cup"],["\u222B","int"],
		["\u2234","there4"],["\u223C","sim"],["\u2245","cong"],["\u2248","asymp"],
		["\u2260","ne"],["\u2261","equiv"],["\u2264","le"],["\u2265","ge"],
		["\u2282","sub"],["\u2283","sup"],["\u2284","nsub"],["\u2286","sube"],
		["\u2287","supe"],["\u2295","oplus"],["\u2297","otimes"],["\u22A5","perp"],
		["\u22C5","sdot"],["\u2308","lceil"],["\u2309","rceil"],["\u230A","lfloor"],
		["\u230B","rfloor"],["\u2329","lang"],["\u232A","rang"],["\u25CA","loz"],
		["\u2660","spades"],["\u2663","clubs"],["\u2665","hearts"],["\u2666","diams"],
		["\u0152","OElig"],["\u0153","oelig"],["\u0160","Scaron"],["\u0161","scaron"],
		["\u0178","Yuml"],["\u02C6","circ"],["\u02DC","tilde"],["\u2002","ensp"],
		["\u2003","emsp"],["\u2009","thinsp"],["\u200C","zwnj"],["\u200D","zwj"],
		["\u200E","lrm"],["\u200F","rlm"],["\u2013","ndash"],["\u2014","mdash"],
		["\u2018","lsquo"],["\u2019","rsquo"],["\u201A","sbquo"],["\u201C","ldquo"],
		["\u201D","rdquo"],["\u201E","bdquo"],["\u2020","dagger"],["\u2021","Dagger"],
		["\u2030","permil"],["\u2039","lsaquo"],["\u203A","rsaquo"]
	];
	
	dhe.encode = function(str/*string*/, m /*array?*/){
		// summary:
		//		Function to obtain an entity encoding for a specified character
		// str:
		//		The string to process for possible entity encoding.
		// m:
		//		An optional list of character to entity name mappings (array of
		//		arrays).  If not provided, it uses the and Latin entities as the
		//		set to map and escape.
		// tags:
		//		public
		if(str){
			if(!m){
				// Apply the basic mappings.  HTML should always come first when decoding
				// as well.
				str = _applyEncodingMap(str, dhe.html);
				str = _applyEncodingMap(str, dhe.latin);
	
			}else{
				str = _applyEncodingMap(str, m);
			}
		}
		return str;
	};
	
	dhe.decode = function(str/*string*/, m /*array?*/){
		// summary:
		//		Function to obtain an entity encoding for a specified character
		// str:
		//		The string to process for possible entity encoding to decode.
		// m:
		//		An optional list of character to entity name mappings (array of
		//		arrays).  If not provided, it uses the HTML and Latin entities as the
		//		set to map and decode.
		// tags:
		//		public
		if(str){
			if(!m){
				// Apply the basic mappings.  HTML should always come first when decoding
				// as well.
				str = _applyDecodingMap(str, dhe.html);
				str = _applyDecodingMap(str, dhe.latin);
	
			}else{
				str = _applyDecodingMap(str, m);
			}
		}
		return str;
	};
	return dhe;
});


},
'epi-find/store/ResourceQueryEngine':function(){
define([
    "dojo/_base/lang",
    "dojo/store/util/SimpleQueryEngine"
],
    function(lang, SimpleQueryEngine) {
        return function(query, options){
            // Extension for the SimpleQueryEngine that does queries cleanup

            // Remove paging information from the query
            var cleanQuery = lang.clone(query);
            delete cleanQuery.size;
            delete cleanQuery.from;

            return SimpleQueryEngine(cleanQuery, options);
        };
    });
},
'epi-find/store/Elasticsearch':function(){
define([
        "dojo/_base/lang",
        "dojo/Deferred",
        "dojo/json",
        "dojo/_base/declare",
        "dojo/store/util/QueryResults",
        "epi/shell/store/JsonRest",
        "epi-saas-base/RetryableXhrWrapper"
],
function (lang, Deferred, JSON, declare, QueryResults, JsonRest, RetryableXhrWrapper) {
    return declare(JsonRest, {
        // summary:
        //		This is a basic store for RESTful communicating with Elasticsearch.

        constructor: function (options) {
            this.xhrHandler = new RetryableXhrWrapper();
            declare.safeMixin(this, options);
        },

        // idProperty: String
        //		Indicates the property to use as the identity property. The values of this
        //		property should be unique.
        idProperty: "_id",

        query: function (query, options) {
            options = options || {};
            query = query || {};

            if (options.count) {
                query.size = options.count;
            }
            if (options.start) {
                query.from = options.start;
            }
            var results;
            if (query.q) {
                results = this.inherited(arguments);
            } else {
                var headers = lang.mixin({ Accept: this.accepts }, this.headers, options.headers);
                results = this.xhrHandler.xhr("POST", {
                    url: this.target,
                    postData: JSON.stringify(query),
                    handleAs: "json",
                    headers: headers,
                    xsrfProtection: true
                });
            }

            var queryResults = new Deferred();
            results.then(function(data) {
                queryResults.resolve(data.hits.hits);
            }, function(error) {
                queryResults.reject(error);
            });

            queryResults.total = results.then(function (data) {
                return data.hits.total;
            });

            queryResults.facets = results.then(function (data) {
                return data.facets;
            });

            queryResults.query = query;

            return QueryResults(queryResults);
        }
    });

});
},
'epi/shell/store/JsonRest':function(){
﻿define("epi/shell/store/JsonRest", [
    "dojo/_base/declare",
    "dojo/_base/json",
    "dojo/_base/lang",
    "dojo/io-query",
    "dojo/store/JsonRest",
    "dojo/store/util/QueryResults",
    "epi/shell/XhrWrapper"
],

    function (
        declare,
        json,
        lang,
        ioQuery,
        JsonRest,
        QueryResults,
        XhrWrapper
    ) {

        return declare([JsonRest], {
            // summary:
            //      Base implementation of a custom rest store with support for XSRF validation.
            //
            // tags:
            //      public

            // xhrHandler: [public] Object
            //      The xhr implementation to use when requesting data. Defaults to epi/shell/XhrWrapper
            xhrHandler: null,

            // idAttribute: [public] String
            //      Default value is set to "id".
            idAttribute: "id",

            // preventCache: [protected] boolean
            //      Default value is set to false
            preventCache: false,

            // defaultRequestParams: [public] Object
            //      Default options added to the xhr request parameters
            defaultRequestParams: null,

            constructor: function (options) {
                // summary:
                //      Creates a new Json Rest Store.
                // options: Object?
                //      Configuration options that will be mixed into the store.

                this.defaultRequestParams = options && options.defaultRequestParams;

                // The xhr wrapper sets store specific information, e.g. CSRF tokens, so it must be instantiated for each store.
                if (!this.xhrHandler) {
                    this.xhrHandler = new XhrWrapper({ preventLocalizationHeader: options && options.preventLocalizationHeader });
                }
            },

            get: function (id, options) {
                // summary:
                //      Retrieves an object by its identity. This will trigger a GET request to the server using
                //      the url `this.target + id`.
                // id: Number
                //      The identity to use to lookup the object.
                // options: Object?
                //      The optional options for the request header.
                // returns:
                //      The object in the store that matches the given id.
                // tags:
                //      public

                var headers = options || {};
                headers.Accept = "application/javascript, application/json";

                var hasId = typeof id != "undefined";

                var params = {
                    url: hasId ? this.target + id : this.target,
                    handleAs: "json",
                    headers: headers,
                    preventCache: this.preventCache
                };

                return this.xhrHandler.xhrGet(this._mixinDefaultParams(params));
            },

            put: function (object, options) {
                // summary:
                //      Stores an object. This will trigger a PUT request to the server
                //      if the object has an id, otherwise it will trigger a POST request.
                // object: Object
                //      The object to store.
                // options: dojo/store/api/Store/PutDirectives?
                //      Additional metadata for storing the data. Includes an "id"
                //      property if a specific id is to be used.
                // returns: dojo/store/util/QueryResults
                //      The results of the query, extended with iterative methods.
                // tags:
                //      public

                options = options || {};

                var paramHeaders = {
                    "Content-Type": "application/json",
                    "If-Match": options.overwrite === true ? "*" : null,
                    "If-None-Match": options.overwrite === false ? "*" : null,
                    Accept: "application/javascript, application/json"
                };

                var id = ("id" in options) ? options.id : this.getIdentity(object);
                var hasId = typeof id != "undefined";

                if (hasId && !options.incremental) {
                    paramHeaders["X-Http-Method-Override"] = "PUT";
                }

                var params = {
                    url: hasId ? this.target + id : this.target,
                    postData: json.toJson(object),
                    handleAs: "json",
                    headers: paramHeaders
                };

                return this.xhrHandler.xhr("POST", this._mixinDefaultParams(params));
            },

            patch: function (object, options) {
                // summary:
                //      Updates an object.
                // object: Object
                //      The object to update.
                // options: Object?
                //      The optional options for the request header.
                // returns: dojo/store/util/QueryResults
                //      The results of the query, extended with iterative methods.
                // tags:
                //      public

                var headers = options || {};
                headers.Accept = "application/javascript, application/json";
                headers["X-Http-Method-Override"] = "PATCH";
                headers["Content-Type"] = "application/json";

                var id = ("id" in options) ? options.id : this.getIdentity(object);
                var hasId = typeof id != "undefined";

                var params = {
                    url: hasId ? this.target + id : this.target,
                    postData: json.toJson(object),
                    handleAs: "json",
                    headers: headers
                };

                return this.xhrHandler.xhr("POST", this._mixinDefaultParams(params));
            },

            remove: function (id, options) {
                // summary:
                //      Deletes an object by its identity. This will trigger a DELETE request to the server.
                // id: Number
                //      The identity to use to delete the object.
                // options: Object
                //      The optional postData options.
                // returns: dojo/store/util/QueryResults
                //      The results of the query, extended with iterative methods.
                // tags:
                //      public

                var headers = {};
                headers.Accept = "application/javascript, application/json";
                headers["Content-Type"] = "application/json";
                headers["X-Http-Method-Override"] = "DELETE";

                options = options || {};

                return this.xhrHandler.xhr("POST", this._mixinDefaultParams({
                    url: this.target + id,
                    headers: headers,
                    postData: json.toJson(options),
                    handleAs: "json"
                }));
            },

            query: function (query, options) {
                // summary:
                //      Queries the store for objects. This will trigger a GET request to the server, with the
                //      query added as a query string.
                // query: Object
                //      The query to use for retrieving objects from the store.
                // options: dojo/store/api/Store/QueryOptions?
                //      The optional arguments to apply to the resultset.
                // returns: dojo/store/util/QueryResults
                //      The results of the query, extended with iterative methods.
                // tags:
                //      public

                var headers = { Accept: "application/javascript, application/json" };
                var idUrlPart;
                options = options || {};

                if (options.start >= 0 || options.count >= 0) {
                    headers.Range = "items=" + (options.start || "0") + "-" +
                        (("count" in options && options.count !== Infinity) ?
                            (options.count + (options.start || 0) - 1) : "");
                }
                if (lang.isObject(query)) {
                    query = lang.mixin({}, query); // This is a shallow copy. Needed since clone breaks RegExp.
                    if (query[this.idAttribute]) {
                        idUrlPart = query[this.idAttribute];
                        query[this.idAttribute] = null;
                    }
                    query = ioQuery.objectToQuery(query);
                    query = query ? "?" + query : "";
                }
                if (options && options.sort) {
                    query += (query ? "&" : "?") + "sort(";
                    for (var i = 0; i < options.sort.length; i++) {
                        var sort = options.sort[i];
                        query += (i > 0 ? "," : "") + (sort.descending ? "-" : "+") + encodeURIComponent(sort.attribute);
                    }
                    query += ")";
                }
                var results = this.xhrHandler.xhrGet(this._mixinDefaultParams({
                    url: this.target + (idUrlPart || "") + (query || ""),
                    handleAs: "json",
                    headers: headers,
                    preventCache: this.preventCache
                }));

                results.total = results.then(function () {
                    var range = results.ioArgs.xhr.getResponseHeader("Content-Range");
                    return range && (range = range.match(/\/(.*)/)) && +range[1];
                });
                return QueryResults(results);
            },

            move: function (id, params, options) {
                // summary:
                //      Move an object to a specified target.
                // id: Number
                //      The id of the object that is moving.
                // params: Object|Number
                //      The additional params to send in the post data. Can be the id of the target that
                //      the object will be moved to.
                // options: Object?
                //      The optional headers object.
                // returns: dojo/store/util/QueryResults
                //      The results of the query, extended with iterative methods.
                // tags:
                //      public

                return this._send("MOVE", id, params, options);
            },

            copy: function (id, params, options) {
                // summary:
                //      Copy an object to a specified target.
                // id: Number
                //      The id of the object that is copying.
                // params: Object|Number
                //      The additional params to send in the post data. Can be the id of the target that
                //      the object will be copied to.
                // options: Object?
                //      The optional headers object.
                // returns: dojo/store/util/QueryResults
                //      The results of the query, extended with iterative methods.
                // tags:
                //      public

                return this._send("COPY", id, params, options);
            },

            executeMethod: function (method, id, data, options) {
                // summary:
                //      Post a request to the server with X-Http-Method-Override equals method input.
                // method: String
                //      The method name.
                // id: Number
                //      The id of the object which is operated.
                // data: Object?
                //      The data to post
                // options: Object?
                //      The optional headers object.
                // returns: dojo/store/util/QueryResults
                //      The results of the query, extended with iterative methods.
                // tags:
                //      internal

                var hasId = id !== null && typeof id !== "undefined",
                    url = this.target + (hasId ? id : ""),
                    headers = options || {};

                headers.Accept = "application/javascript, application/json";
                headers["Content-Type"] = "application/json";
                headers["X-Http-Method-Override"] = method;

                data = data || {};

                return this.xhrHandler.xhr("POST", this._mixinDefaultParams({
                    url: url,
                    headers: headers,
                    postData: json.toJson(data),
                    handleAs: "json"
                }));
            },

            getAllResponsesWithError: function (response) {
                // summary:
                //      Get the errors from a call that handled multiple operations. I.e. multiple
                //      selection actions on blocks or media, where one item was successfully moved but
                //      the others failed for different reasons.
                // response: Object
                //      The response object to parse.
                // returns: Array
                //      The errors for any failed operations.
                // tags:
                //      private

                var results = [];
                if (response.statusCode !== 200 && !(response.extraInformation instanceof Array)) {
                    results.push(response);
                }

                if (response.extraInformation && typeof response.extraInformation !== "string") {
                    Object.keys(response.extraInformation).forEach(function (prop) {
                        this.getAllResponsesWithError(response.extraInformation[prop]).forEach(function (res) {
                            results.push(res);
                        });
                    }, this);
                }

                return results;
            },

            _send: function (method, id, params, options) {
                // summary:
                //      Post a request to the server with X-Http-Method-Override equals method input.
                // method: String
                //      The method name.
                // id: Number
                //      The id of the object which is operated.
                // params: Object|Number
                //      The additional params to send in the post data.
                // options: Object?
                //      The optional headers object.
                // returns: dojo/store/util/QueryResults
                //      The results of the query, extended with iterative methods.
                // tags:
                //      private

                var headers = options || {};
                headers.Accept = "application/javascript, application/json";
                headers["X-Http-Method-Override"] = method;

                var postContent = lang.isObject(params) ? params : { targetId: params };

                return this.xhrHandler.xhr("POST", this._mixinDefaultParams({
                    url: this.target + id,
                    headers: headers,
                    content: postContent,
                    handleAs: "json"
                }));
            },

            _mixinDefaultParams: function (params) {
                // summary:
                //      Mix the params and the defaultRequestParams into a new object.
                // params: Object
                //      The params to mix with defaultRequestParams.
                // returns: Object
                //      The new combined params.
                // tags:
                //      private

                return lang.mixin({}, this.defaultRequestParams, params);
            }
        });
    });

},
'dojo/store/JsonRest':function(){
define("dojo/store/JsonRest", ["../_base/xhr", "../_base/lang", "../json", "../_base/declare", "./util/QueryResults" /*=====, "./api/Store" =====*/
], function(xhr, lang, JSON, declare, QueryResults /*=====, Store =====*/){

// No base class, but for purposes of documentation, the base class is dojo/store/api/Store
var base = null;
/*===== base = Store; =====*/

/*=====
var __HeaderOptions = {
		// headers: Object?
		//		Additional headers to send along with the request.
	},
	__PutDirectives = declare(Store.PutDirectives, __HeaderOptions),
	__QueryOptions = declare(Store.QueryOptions, __HeaderOptions);
=====*/

return declare("dojo.store.JsonRest", base, {
	// summary:
	//		This is a basic store for RESTful communicating with a server through JSON
	//		formatted data. It implements dojo/store/api/Store.

	constructor: function(options){
		// summary:
		//		This is a basic store for RESTful communicating with a server through JSON
		//		formatted data.
		// options: dojo/store/JsonRest
		//		This provides any configuration information that will be mixed into the store
		this.headers = {};
		declare.safeMixin(this, options);
	},

	// headers: Object
	//		Additional headers to pass in all requests to the server. These can be overridden
	//		by passing additional headers to calls to the store.
	headers: {},

	// target: String
	//		The target base URL to use for all requests to the server. This string will be
	//		prepended to the id to generate the URL (relative or absolute) for requests
	//		sent to the server
	target: "",

	// idProperty: String
	//		Indicates the property to use as the identity property. The values of this
	//		property should be unique.
	idProperty: "id",

	// sortParam: String
	//		The query parameter to used for holding sort information. If this is omitted, than
	//		the sort information is included in a functional query token to avoid colliding
	//		with the set of name/value pairs.

	get: function(id, options){
		// summary:
		//		Retrieves an object by its identity. This will trigger a GET request to the server using
		//		the url `this.target + id`.
		// id: Number
		//		The identity to use to lookup the object
		// options: Object?
		//		HTTP headers. For consistency with other methods, if a `headers` key exists on this object, it will be
		//		used to provide HTTP headers instead.
		// returns: Object
		//		The object in the store that matches the given id.
		options = options || {};
		var headers = lang.mixin({ Accept: this.accepts }, this.headers, options.headers || options);
		return xhr("GET", {
			url: this.target + id,
			handleAs: "json",
			headers: headers
		});
	},

	// accepts: String
	//		Defines the Accept header to use on HTTP requests
	accepts: "application/javascript, application/json",

	getIdentity: function(object){
		// summary:
		//		Returns an object's identity
		// object: Object
		//		The object to get the identity from
		// returns: Number
		return object[this.idProperty];
	},

	put: function(object, options){
		// summary:
		//		Stores an object. This will trigger a PUT request to the server
		//		if the object has an id, otherwise it will trigger a POST request.
		// object: Object
		//		The object to store.
		// options: __PutDirectives?
		//		Additional metadata for storing the data.  Includes an "id"
		//		property if a specific id is to be used.
		// returns: dojo/_base/Deferred
		options = options || {};
		var id = ("id" in options) ? options.id : this.getIdentity(object);
		var hasId = typeof id != "undefined";
		return xhr(hasId && !options.incremental ? "PUT" : "POST", {
				url: hasId ? this.target + id : this.target,
				postData: JSON.stringify(object),
				handleAs: "json",
				headers: lang.mixin({
					"Content-Type": "application/json",
					Accept: this.accepts,
					"If-Match": options.overwrite === true ? "*" : null,
					"If-None-Match": options.overwrite === false ? "*" : null
				}, this.headers, options.headers)
			});
	},

	add: function(object, options){
		// summary:
		//		Adds an object. This will trigger a PUT request to the server
		//		if the object has an id, otherwise it will trigger a POST request.
		// object: Object
		//		The object to store.
		// options: __PutDirectives?
		//		Additional metadata for storing the data.  Includes an "id"
		//		property if a specific id is to be used.
		options = options || {};
		options.overwrite = false;
		return this.put(object, options);
	},

	remove: function(id, options){
		// summary:
		//		Deletes an object by its identity. This will trigger a DELETE request to the server.
		// id: Number
		//		The identity to use to delete the object
		// options: __HeaderOptions?
		//		HTTP headers.
		options = options || {};
		return xhr("DELETE", {
			url: this.target + id,
			headers: lang.mixin({}, this.headers, options.headers)
		});
	},

	query: function(query, options){
		// summary:
		//		Queries the store for objects. This will trigger a GET request to the server, with the
		//		query added as a query string.
		// query: Object
		//		The query to use for retrieving objects from the store.
		// options: __QueryOptions?
		//		The optional arguments to apply to the resultset.
		// returns: dojo/store/api/Store.QueryResults
		//		The results of the query, extended with iterative methods.
		options = options || {};

		var headers = lang.mixin({ Accept: this.accepts }, this.headers, options.headers);

		if(options.start >= 0 || options.count >= 0){
			headers.Range = headers["X-Range"] //set X-Range for Opera since it blocks "Range" header
				 = "items=" + (options.start || '0') + '-' +
				(("count" in options && options.count != Infinity) ?
					(options.count + (options.start || 0) - 1) : '');
		}
		var hasQuestionMark = this.target.indexOf("?") > -1;
		if(query && typeof query == "object"){
			query = xhr.objectToQuery(query);
			query = query ? (hasQuestionMark ? "&" : "?") + query: "";
		}
		if(options && options.sort){
			var sortParam = this.sortParam;
			query += (query || hasQuestionMark ? "&" : "?") + (sortParam ? sortParam + '=' : "sort(");
			for(var i = 0; i<options.sort.length; i++){
				var sort = options.sort[i];
				query += (i > 0 ? "," : "") + (sort.descending ? '-' : '+') + encodeURIComponent(sort.attribute);
			}
			if(!sortParam){
				query += ")";
			}
		}
		var results = xhr("GET", {
			url: this.target + (query || ""),
			handleAs: "json",
			headers: headers
		});
		results.total = results.then(function(){
			var range = results.ioArgs.xhr.getResponseHeader("Content-Range");
			return range && (range = range.match(/\/(.*)/)) && +range[1];
		});
		return QueryResults(results);
	}
});

});
},
'epi/shell/XhrWrapper':function(){
﻿define("epi/shell/XhrWrapper", [
    "dojo/_base/array",
    "dojo/_base/connect",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/Deferred",
    "dojo/when",
    "epi",
    "epi/shell/request/xhr",
    "epi/shell/widget/dialog/Dialog",
    "epi/shell/widget/LoginForm"
], function (
    array,
    connect,
    declare,
    lang,
    Deferred,
    when,
    epi,
    xhr,
    Dialog,
    LoginForm
) {

    // Shared variables between all instances of this class
    var loginDeferred;

    var cls = declare(null, {
        // summary:
        //    Wrapper of xhr requests with support for XSRF validation and logout error handling
        //
        // tags:
        //    internal

        // xhrHandler: [public] Object
        //  The xhr implementation to use when requesting data. Defaults to dojo.xhr
        xhrHandler: null,

        _headerKeys: {
            // validation: [const private] String
            //    The name of the field in the request headers that contains the validation
            //    token. It needs to have the prefix "X-" according to the RFC specifications.
            //    See section 5 on http://www.ietf.org/rfc/rfc2047.txt
            validation: "X-EpiRestAntiForgeryToken",

            // loginScreen: [const private] String
            //    The name of the field in the request headers that indicates that the result
            //    is a login screen rather than the expected json data.
            loginScreen: "X-EPiLogOnScreen",
            postUrl: "X-EPiLogOnScreen-PostUrl",
            readOnlyMode: "X-EPiReadOnlyMode",
            readOnlyModeRedirectUrl: "X-EPiReadOnlyMode-RedirectUrl"
        },

        constructor: function (options) {
            // summary:
            //  Creates a new XhrWrapper instance
            //
            // options: Object
            //  Configuration options that will be mixed into the handler.

            declare.safeMixin(this, options);

            if (!this.xhrHandler) {
                // Default the xhr handler to the dojo one.
                this.xhrHandler = xhr;
            }

        },

        xhr: function (/*String*/method, /*dojo.__XhrArgs*/args, /*Boolean?*/hasBody) {
            //	summary:
            //		Sends an HTTP request with the given method.
            //	description:
            //		Sends an HTTP request with the given method.
            //		See also dojo.xhrGet(), xhrPost(), xhrPut() and dojo.xhrDelete() for shortcuts
            //		for those HTTP methods. There are also methods for "raw" PUT and POST methods
            //		via dojo.rawXhrPut() and dojo.rawXhrPost() respectively.
            //	method:
            //		HTTP method to be used, such as GET, POST, PUT, DELETE.  Should be uppercase.
            //  args:
            //      xhr arguments passed to dojo's native xhr API. This method optionally supports
            //      xsrfProtection which will resolve an xsrf token which is posted with the request.
            //	hasBody:
            //		If the request has an HTTP body, then pass true for hasBody.

            // Calling with apply since dojo.xhr checks arguments count
            var xhrArguments = arguments;

            args.headers = args.headers || {};

            if (args.xsrfProtection) {
                // Retrieve xsrf header and then do the work passing along validation token.
                var deferred = new Deferred();
                when(this._getXsrfHeader(args.url), lang.hitch(this, function (validationToken) {
                    // Got headers, now do the work
                    args.headers = lang.mixin({}, args.headers, validationToken);

                    when(this._xhr.apply(this, xhrArguments), deferred.resolve, deferred.reject);
                }), deferred.reject);
                return deferred;
            } else {
                // just do the work
                return this._xhr.apply(this, xhrArguments);
            }
        },
        xhrDelete: function (/*dojo.__XhrArgs*/args) {
            //	summary:
            //		Sends an HTTP DELETE request to the server.

            return this.xhr("DELETE", args);
        },
        xhrGet: function (/*dojo.__XhrArgs*/args) {
            //	summary:
            //		Sends an HTTP GET request to the server.

            return this.xhr("GET", args);
        },
        xhrPost: function (/*dojo.__XhrArgs*/args) {
            //	summary:
            //		Sends an HTTP POST request to the server. In addtion to the properties
            //		listed for the dojo.__XhrArgs type, the following property is allowed:
            //	postData:
            //		String. Send raw data in the body of the POST request.

            return this.xhr("POST", args, true);
        },
        xhrPut: function (/*dojo.__XhrArgs*/args) {
            //	summary:
            //		Sends an HTTP PUT request to the server. In addtion to the properties
            //		listed for the dojo.__XhrArgs type, the following property is allowed:
            //	putData:
            //		String. Send raw data in the body of the PUT request.

            return this.xhr("PUT", args, true);
        },

        _getXsrfHeader: function (target) {
            // summary:
            //    Resturns the XSRF header property if it's been set,
            //    otherwise a get request is made and a deferred is returned.
            //
            // returns:
            //    A object with the validation token or a deferred.
            //
            // tags:
            //   private

            if (cls.sharedXsrfHeader) {
                return cls.sharedXsrfHeader;
            }

            var params = {
                url: target,
                handleAs: "json",
                headers: {}
            };

            // Adding the XSRF header with and a special keyword tells the server not to execute the get but to return a token in the header instead.
            params.headers[this._headerKeys.validation] = "_CreateToken_";

            var deferred = new Deferred();

            this.xhrGet(params).then(lang.hitch(this, function () {
                deferred.resolve(cls.sharedXsrfHeader);
            }), deferred.reject);

            return deferred;
        },

        _xhr: function (/*String*/method, /*dojo.__XhrArgs*/args, /*Boolean?*/hasBody) {
            // summary:
            //    Wraps the xhr result to add handling for xsrf header update.
            //
            // returns:
            //    A deferred that is resolved to the xhr result data.
            //
            // tags:
            //   private

            var xhrArguments = arguments;

            // Modify arguments to match the dojo/request api
            var options = lang.delegate({
                method: method,
                data: args.content || args.postData
            }, args);

            var xhrResult = this.xhrHandler(options.url, options, true);

            var deferred = new Deferred();
            deferred.ioArgs = xhrResult.ioArgs;

            when(xhrResult, lang.hitch(this, function (data) {
                var xsrfData = xhrResult.ioArgs.xhr.getResponseHeader(this._headerKeys.validation);
                if (xsrfData) {
                    // Always set the xsrf header.
                    // If the user jumps between different sites (ports or virtual paths) on the same domain
                    // the cookie, and the corresponding header, will change.

                    if (!cls.sharedXsrfHeader) {
                        cls.sharedXsrfHeader = {};
                    }

                    cls.sharedXsrfHeader[this._headerKeys.validation] = xsrfData;
                }

                // The query method checks this
                deferred.ioArgs = xhrResult.ioArgs;
                deferred.resolve(data);

            }), lang.hitch(this, function (err) {
                // Error, is it caused by a logout?
                var loginScreen = xhrResult.ioArgs.xhr.getResponseHeader(this._headerKeys.loginScreen);
                if (loginScreen === "true" || xhrResult.ioArgs.xhr.status === 401) {
                    // The login screen was displayed, ask the user to and retry
                    var settings = {
                        postUrl: xhrResult.ioArgs.xhr.getResponseHeader(this._headerKeys.postUrl)
                    };

                    // reset xsrf header as the user's logged out, we need to get a new one
                    cls.sharedXsrfHeader = undefined;

                    var loginResult = this._showLogin(settings);
                    when(loginResult, lang.hitch(this, function () {
                        // Try the xhr request again after successful login
                        when(this.xhr.apply(this, xhrArguments), deferred.resolve, deferred.reject);
                    }), function () {
                        // User cancelled log in, pass the error back to the caller
                        deferred.reject(err);
                    });
                } else if (xhrResult.ioArgs.xhr.status === 503) {
                    // If the server returns a 503 (Service Unavailable)
                    // determine if the system is in read only mode, if so redirect to the provided read only url
                    // otherwise reject the promise
                    var readOnlyMode = xhrResult.ioArgs.xhr.getResponseHeader(this._headerKeys.readOnlyMode);
                    var readOnlyModeRedirectUrl = xhrResult.ioArgs.xhr.getResponseHeader(this._headerKeys.readOnlyModeRedirectUrl);
                    if (readOnlyMode && readOnlyModeRedirectUrl) {
                        this._redirect(readOnlyModeRedirectUrl);
                    } else {
                        deferred.reject(err);
                    }
                } else {
                    // Some unknown error, pass the error back to the caller
                    deferred.reject(err);
                }
            }));
            return deferred;
        },
        _redirect: function (url) {
            window.top.location = url;
        },

        _showLogin: function (settings) {
            //	summary:
            //		Displays a login dialog and interacts with the server to authenticate the user

            if (!loginDeferred) {
                loginDeferred = new Deferred();

                var form = new LoginForm();

                var dialog = new Dialog({
                    title: epi.resources.action.login,
                    confirmActionText: epi.resources.action.login,
                    content: form,
                    dialogClass: "",
                    _onAfterShow: function () {} // We wish to get focus on the input fields, not on the actions.
                });

                var connections = [];
                var cleanupAndNotify = function (wasSuccessful) {
                    // Hides the dialog, disconnects events, destroys the login widgets and notifies via the deferred object
                    array.forEach(connections, connect.disconnect);

                    form.destroyRecursive();
                    //destroyOnHide is set to true so this will destroy the dialog after hide.
                    dialog.hide();

                    if (wasSuccessful) {
                        loginDeferred.resolve();
                    } else {
                        loginDeferred.reject();
                    }
                    loginDeferred = null;
                };

                // validate user credentials by posting them to the server
                dialog.addValidator(lang.hitch(this, function () {
                    // User submitted login form
                    var validationDeferred = new Deferred();

                    var content = form.serializeForm();

                    var loginResult = this.xhr("POST", {
                        url: settings.postUrl,
                        content: content,
                        xsrfProtection: false,
                        handleAs: "json"
                    }, true);

                    when(loginResult, function (data) {
                        // User submitted the login form, was the login successful?
                        if (data.authenticationSuccessful) {
                            cleanupAndNotify(true);
                            validationDeferred.resolve();
                        } else {
                            form.authenticationFailed();
                            validationDeferred.reject();
                        }
                    }, function () {
                        // Unknown error logging in, break out and let the caller handle the original error
                        cleanupAndNotify(false);
                        validationDeferred.reject();
                    });
                    return validationDeferred;
                }));

                dialog.show();

                connections.push(connect.connect(dialog, "onCancel", function () {
                    // User cancelled
                    cleanupAndNotify(false);
                }));
            }
            return loginDeferred;
        }
    });

    // create a shared variable
    cls.sharedXsrfHeader = undefined;

    return cls;
});

},
'epi/shell/request/xhr':function(){
define("epi/shell/request/xhr", [
    "dojo/_base/xhr",
    "dojo/request/util",
    "./Extender",
    "./mutators"
], function (request, util, Extender, mutators) {
    // TODO: Change to use dojo/request as backing request handler and make it public

    var extender = new Extender(request, mutators);

    var xhr = function (url, options) {
        // summary:
        //		Sends a request using XMLHttpRequest with the given URL and options.
        //     Before the request is sent all registered mutators in epi/shell/request/mutators
        // url: String
        //		URL to request
        // options: dojo/request/xhr.__Options?
        //		Options for the request.
        // returns: dojo/promise/Promise
        // tags: internal

        return extender.xhr.apply(extender, arguments);
    };

    util.addCommonMethods(xhr);

    return xhr;
});

},
'epi/shell/request/Extender':function(){
define("epi/shell/request/Extender", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/Deferred"
], function (declare, lang, Deferred) {

    return declare([], {
        // summary:
        //      Xhr request wrapper calling a chain of optional request mutators before sending the request to a
        //      backing request handler.
        // tags: internal

        _mutators: null,
        _request: null,

        constructor: function (request, mutators) {
            // summary:
            //
            // request: dojo/_base/xhr
            //      The backing request handler.
            // mutators: function
            //      A function returning a list of request mutators

            this._request = request;
            this._mutators = mutators;
        },

        xhr: function (url, options, returnDeferred) {
            // summary:
            //		Sends a request using XMLHttpRequest with the given URL and options.
            // url: String
            //		URL to request
            // options: dojo/request/xhr.__Options?
            //		Options for the request.
            // returnDeferred: Boolean
            //      When true; the inner deferred with the original response property is returned.
            // returns: dojo/promise/Promise

            var completed = new Deferred(),
                mutators = this._mutators;

            options = options || {};
            options.headers = options.headers || {};

            mutators.execute("beforeSend", { url: url, options: options })
                .then(lang.hitch(this, function (params) {

                    // The dojo request sets a response property containing headers
                    // and the xhr object on the deferred. We forward it.
                    var deferred = this._sendRequest(params);
                    completed.response = deferred.response || this._response(deferred.ioArgs);
                    completed.ioArgs = deferred.ioArgs;
                    return deferred.promise;
                }))
                .then(function (data) {
                    return mutators.execute("afterReceive", {response: completed.response, data: data});
                })
                .then(function (result) {
                    completed.resolve(result.data);
                })
                .otherwise(completed.reject);

            return returnDeferred ? completed : completed.promise;
        },

        _sendRequest: function (params) {
            // summary:
            //      Send the request to the backing xhr handler
            // tags: private

            var options = params.options;
            options.url = params.url;
            return this._request(options.method, options);
        },

        _response: function (ioArgs) {
            // summary:
            //      Wraps the ioArgs returned from dojo/_base/xhr to a response object matching the one returned
            //      from returned from dojo/request/xhr
            // tags: private

            return {
                getHeader: function getHeader(name) {
                    return this.xhr.getResponseHeader(name);
                },
                options: ioArgs.args,
                url: ioArgs.url,
                xhr: ioArgs.xhr
            };
        }

    });
});

},
'epi/shell/request/mutators':function(){
define("epi/shell/request/mutators", [
    "dojo/when"
], function (
    when
) {

    var _mutators = [];

    var mutators = function () {
        // summary:
        //      Returns an array of configured mutators.
        //      The collection may change during runtime, so don't hold on to the returned array
        // returns: An array with request mutators

        return _mutators.concat();
    };

    mutators.add = function (mutator) {
        // summary:
        //      Register a new request mutator

        _mutators.push(mutator);
    };

    mutators.remove = function (mutator) {
        // summary:
        //      Remove a registered request mutator.
        // returns:
        //      The removed mutator; or null if no matching mutator was found.

        var index = _mutators.indexOf(mutator);
        if (index !== -1) {
            return _mutators.splice(index, 1)[0];
        }
        return null;
    };

    mutators.execute = function (methodName, data) {
        // summary:
        //      Execute the specified method on all mutators in the mutator chain.
        // methodName: String
        //      Name of the method to execute on each mutator
        // data: object
        //      A custom data object passed to the executed method.
        // returns:
        //      A Deferred resolved with the data from the last mutator in the chain.

        var previous = when(data);

        _mutators.forEach(function (mutator) {
            var method = mutator[methodName];
            previous = method ? previous.then(method.bind(mutator)) : previous;
        });

        return previous;
    };

    return mutators;
});

},
'epi/shell/widget/dialog/Dialog':function(){
﻿define("epi/shell/widget/dialog/Dialog", [
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-class",
    "dojo/dom-style",
    "dojo/promise/all",

    "epi",
    "epi/shell/widget/dialog/_DialogBase",
    "epi/shell/widget/_ActionConsumerWidget",
    "epi/shell/widget/Toolbar",
    "epi/shell/widget/ToolbarSet"
], function (
    array,
    declare,
    lang,
    domClass,
    domStyle,
    promiseAll,

    epi,
    _DialogBase,
    _ActionConsumerWidget,
    Toolbar,
    ToolbarSet
) {

    return declare([_DialogBase, _ActionConsumerWidget], {
        // summary:
        //		A modal dialog widget.
        //
        // tags:
        //      public

        // confirmActionText: [public] String
        //		Label to be displayed for the confirm (positive) action.
        confirmActionText: epi.resources.action.ok,

        // cancelActionText: [public] String
        //		Label to be displayed for the cancel (negative) action.
        cancelActionText: epi.resources.action.cancel,

        // contentClass: [protected] String
        //		Class to apply to the container DOMNode of the dialog.
        contentClass: "",

        // dialogClass: [protected] String
        //		Class to apply to the root DOMNode of the dialog. Defaults to "epi-dialog-landscape".
        dialogClass: "epi-dialog-landscape",

        // defaultActionsVisible: [public] Boolean
        //		Flag which indicates whether the default confirm and cancel
        //		actions should be visible. This can only be set in the constructor.
        defaultActionsVisible: true,

        // setFocusOnConfirmButton: [public] Boolean
        //		Flag which indicates whether the Ok button need to be set as focused
        setFocusOnConfirmButton: true,

        _preferredHeight: 0,

        _validators: null,

        _okButtonName: "ok",
        _cancelButtonName: "cancel",

        postMixInProperties: function () {
            // summary:
            //		Initiates collections.
            // tags:
            //		protected
            this.inherited(arguments);

            this._validators = [];

            if (this.content && this.content.onSubmit) {
                this.connect(this.content, "onSubmit", this._onSubmit);
            }
        },

        postCreate: function () {
            // summary:
            //    Post widget creation.
            //
            // tags:
            //    public

            this.inherited(arguments);
            this.set("definitionConsumer", new ToolbarSet({ baseClass: "dijitDialogPaneActionBar", layoutContainerClass: Toolbar }, this.actionContainerNode));
        },

        _size: function () {
            var preferredHeight = this.get("_preferredHeight");

            if (preferredHeight) {
                // Rig the containerNode style to get the sizing algorithm to take css-class set height into account
                domStyle.set(this.containerNode, {
                    minHeight: preferredHeight + "px",
                    maxHeight: preferredHeight + "px"
                });
            }

            this.inherited(arguments);

            // Remove the rigging to get the resulting sizing to apply
            domStyle.set(this.containerNode, { minHeight: "", maxHeight: "" });

            // The dialog implementation explicitly sets width and height to auto which overrides
            // any explicit width and height set in css classes.
            array.forEach(["height", "width"], function (prop) {
                if (this[prop] === "auto") {
                    this[prop] = "";
                }
            }, this.containerNode.style);

        },

        addValidator: function (/*function */validator) {
            // summary:
            //      Adds a validator to the validators collection.
            //      Each validator should be a function that returns a Deferred object
            this._validators.push(validator);
        },

        getActions: function () {
            // summary:
            //      Overridden from _ActionConsumer mixin to assemble the action collection
            // returns:
            //		A collection of action definitions that can be added to the action pane.
            // tags:
            //      protected

            var buttons = this.inherited(arguments);

            if (this.defaultActionsVisible) {

                var okButton = {
                        name: this._okButtonName,
                        label: this.confirmActionText,
                        title: null,
                        settings: { type: "submit" }
                    },

                    cancelButton = {
                        name: this._cancelButtonName,
                        label: this.cancelActionText,
                        title: null,
                        action: lang.hitch(this, this._onCancel)
                    };

                if (this.setFocusOnConfirmButton) {
                    okButton.settings["class"] = "Salt";
                } else {
                    cancelButton.settings = { "class": "Salt", firstFocusable: true };
                }

                buttons.push(okButton, cancelButton);
            }

            return buttons;
        },

        _onCancel: function () {
            // summary:
            //		Called when user has triggered the dialog's cancel action.
            // type:
            //		callback

            this.onCancel();
        },

        _onSubmit: function () {
            // summary:
            //		Called when user has triggered the dialog's submit action.
            // type:
            //      callback

            // regular validation first
            if (!this.validate()) {
                return;
            }

            // if no custom validators - continue with submit
            if (this._validators.length === 0) {
                this.inherited(arguments);
                return;
            }

            // getting deferreds for each validator
            var validationDeferreds = array.map(this._validators, function (item) {
                return item();
            });

            var submitArgs = arguments;
            var deferred = new promiseAll(validationDeferreds);
            deferred.then(lang.hitch(this, function (results) {
                // if any of the validators fail - do not continue
                if (array.some(results, function (result) {
                    return !result[0];
                })) {
                    return;
                }
                // proceed with submit
                this.inherited(submitArgs);
            }));

        },

        _setContentClassAttr: function (value) {
            domClass.remove(this.containerNode, this.contentClass);
            this._set("contentClass", value);
            domClass.add(this.containerNode, this.contentClass);
        },

        _get_preferredHeightAttr: function () {
            if (!!this.dialogClass && this._preferredHeight < 25) {
                this._preferredHeight = domStyle.get(this.containerNode, "height");
            }

            return this._preferredHeight;
        }
    });
});

},
'epi/shell/widget/dialog/_DialogBase':function(){
require({cache:{
'url:epi/shell/widget/dialog/templates/Dialog.html':"﻿<div class=\"dijitDialog\" role=\"dialog\" aria-labelledby=\"${id}_title\">\n    <div data-dojo-attach-point=\"titleBar\" class=\"dijitDialogTitleBar\">\n        <span data-dojo-attach-point=\"titleNode\" class=\"dijitDialogTitle\" id=\"${id}_title\"></span>\n        <span data-dojo-attach-point=\"closeButtonNode\" class=\"dijitDialogCloseIcon\" data-dojo-attach-event=\"ondijitclick: onCancel\" title=\"${resources.close}\" role=\"button\" tabIndex=\"-1\">\n            <span data-dojo-attach-point=\"closeText\" class=\"closeText\" title=\"${resources.cancel}\">x</span>\n        </span>\n    </div>\n    <div data-dojo-attach-point=\"contentContainerNode\" class=\"epi-dialogPaneContent\">\n        <span data-dojo-attach-point=\"iconNode\" class=\"dijitIcon dijitInline epi-dialogIcon\"></span>\n        <div data-dojo-attach-point=\"headingNode\" class=\"epi-dialogDescriptionHeader\"></div>\n        <div data-dojo-attach-point=\"descriptionNode\" class=\"epi-dialogDescriptionSummary\"></div>\n        <div data-dojo-attach-point=\"containerNode\" class=\"dijitDialogPaneContentArea\"></div>\n        <div data-dojo-attach-point=\"actionContainerNode\" class=\"dijitDialogPaneActionBar\"></div>\n    </div>\n</div>"}});
﻿define("epi/shell/widget/dialog/_DialogBase", [
    "epi",
    "dojo",
    "dojo/_base/declare",
    "dojo/_base/array",
    "dojo/dom-class",
    "dojo/dom-style",
    "dojo/query",
    "dojo/text!./templates/Dialog.html",
    "dojo/when",
    "dijit/a11y",
    "dijit/focus",
    "dijit/Dialog",
    "epi/shell/widget/dialog/_DialogMixin",
    "epi/shell/widget/_FocusableMixin"
], function (epi, dojo, declare, array, domClass, domStyle, query, template, when, a11y, focus, Dialog, _DialogMixin, _FocusableMixin) {

    var module = declare([Dialog, _DialogMixin, _FocusableMixin], {
        // summary:
        //		A modal dialog widget
        //
        // tags:
        //      internal

        // templateString: [protected] String
        //		A string that represents the widget template.
        templateString: template,

        // heading: [protected] String
        //		A string heading to be displayed in the dialog.
        heading: "",

        // description: [protected] String
        //		A string description to be displayed in the dialog.
        description: "",

        // dialogClass: [protected] String
        //		Class to apply to the root DOMNode of the dialog.
        dialogClass: "",

        // iconClass: [protected] String
        //		Class to apply to DOMNode to make it display an icon.
        iconClass: "dijitNoIcon",

        // destroyOnHide: [public] Boolean
        //		Flag which indicates whether the dialog will be destroyed after it is hidden.
        destroyOnHide: true,

        // focusActionsOnLoad: [public] Boolean
        //      A flag indicating whether the actions container should be the first focus target on dialog load.
        focusActionsOnLoad: false,

        postMixInProperties: function () {
            // summary:
            //		Mixin language resources to the class.
            // tags:
            //		protected

            this.inherited(arguments);
            this.resources = epi.resources.action;
        },

        buildRendering: function () {
            // summary:
            //		Construct the UI for this widget from a template, adding dialogClass to this.domNode.
            // tags:
            //		protected

            this.inherited(arguments);
            if (this.dialogClass) {
                domClass.add(this.domNode, this.dialogClass);
            }
            this.set("closeIconVisible", this.closeIconVisible);
            this.set("iconClass", this.iconClass);
            this.set("heading", this.heading);
            this.set("description", this.description);
        },

        postCreate: function () {
            this.inherited(arguments);

            // Check if content is _DialogContentMixin then wire up the provided methods to dialog events.
            if (this.content && this.content.executeDialog) {
                this.connect(this.content, "executeDialog", "onExecute");
            }
            if (this.content && this.content.cancelDialog) {
                this.connect(this.content, "cancelDialog", "onCancel");
            }
        },

        hide: function () {
            // summary:
            //		Hides the dialog and destroy, if it is required.
            // returns: dojo/Deferred
            //		Deferred object that resolves when the hide animation is complete or dialog is destroyed

            var onHideComplete = dojo.hitch(this, function () {
                if (this.destroyOnHide) {
                    this.destroyRecursive();
                }
            });
            return when(this.inherited(arguments), onHideComplete);
        },

        _getFocusItems: function () {
            // summary:
            //		Sets this._firstFocusItem and this._lastFocusItem to the dialog content
            //		if it exists or else to items in the action panel.
            // tags:
            //		protected
            var node = this.focusActionsOnLoad ? this.actionContainerNode : this.contentContainerNode,
                container = a11y._getTabNavigable(node);

            // Set focusActionsOnLoad to false since we don't want to limit the focus area after load.
            this.focusActionsOnLoad = false;

            this._firstFocusItem = container.lowest || container.first || this.closeButtonNode || this.domNode;
            this._lastFocusItem = container.last || container.highest || this._firstFocusItem;
        },

        _setDialogContentNode: function (targetNode, value) {
            // summary:
            //      Sets the inner html of content and adds the class epi-firstVisible to the fist visible content child div.
            // tags:
            //      private

            targetNode.innerHTML = value;
            domStyle.set(targetNode, "display", value ? "" : "none");

            var children = query("div", this.contentContainerNode);
            children.removeClass("epi-firstVisible");

            array.some(children, function (targetNode) {
                if (domStyle.get(targetNode, "display") !== "none") {
                    domClass.add(targetNode, "epi-firstVisible");
                    return true;
                }
            });
        },

        _setHeadingAttr: function (value) {
            this._setDialogContentNode(this.headingNode, value);
        },

        _setDescriptionAttr: function (value) {
            this._setDialogContentNode(this.descriptionNode, value);
        },

        _setIconClassAttr: function (value) {
            domClass.remove(this.iconNode, this.iconClass);
            this._set("iconClass", value);
            domClass.add(this.iconNode, this.iconClass);
        }
    });

    module.getDialogZindex = function () {
        // summary:
        //		Return the latest z-index value of the created dialogs or the default z-index value
        // tags:
        //		protected

        var obj = Dialog._dialogStack ? Dialog._dialogStack[Dialog._dialogStack.length - 1] : null;
        if (obj && obj.zIndex) {
            //When there are dialogs we return the latest zIndex of the object in the stack +1,
            //because the dialog steps +2 everytime so its safe to add +1. Or return the default zIndex from the dijit dialog.
            return obj.zIndex + 1;
        }
        return Dialog._DialogLevelManager._beginZIndex;
    };

    return module;
});

},
'dojo/text':function(){
define("dojo/text", ["./_base/kernel", "require", "./has", "./_base/xhr"], function(dojo, require, has, xhr){
	// module:
	//		dojo/text

	var getText;
	if( 1 ){
		getText= function(url, sync, load){
			xhr("GET", {url: url, sync:!!sync, load: load, headers: dojo.config.textPluginHeaders || {}});
		};
	}else{
		// TODOC: only works for dojo AMD loader
		if(require.getText){
			getText= require.getText;
		}else{
			console.error("dojo/text plugin failed to load because loader does not support getText");
		}
	}

	var
		theCache = {},

		strip= function(text){
			//Strips <?xml ...?> declarations so that external SVG and XML
			//documents can be added to a document without worry. Also, if the string
			//is an HTML document, only the part inside the body tag is returned.
			if(text){
				text= text.replace(/^\s*<\?xml(\s)+version=[\'\"](\d)*.(\d)*[\'\"](\s)*\?>/im, "");
				var matches= text.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
				if(matches){
					text= matches[1];
				}
			}else{
				text = "";
			}
			return text;
		},

		notFound = {},

		pending = {};

	dojo.cache = function(/*String||Object*/module, /*String*/url, /*String||Object?*/value){
		// summary:
		//		A getter and setter for storing the string content associated with the
		//		module and url arguments.
		// description:
		//		If module is a string that contains slashes, then it is interpretted as a fully
		//		resolved path (typically a result returned by require.toUrl), and url should not be
		//		provided. This is the preferred signature. If module is a string that does not
		//		contain slashes, then url must also be provided and module and url are used to
		//		call `dojo.moduleUrl()` to generate a module URL. This signature is deprecated.
		//		If value is specified, the cache value for the moduleUrl will be set to
		//		that value. Otherwise, dojo.cache will fetch the moduleUrl and store it
		//		in its internal cache and return that cached value for the URL. To clear
		//		a cache value pass null for value. Since XMLHttpRequest (XHR) is used to fetch the
		//		the URL contents, only modules on the same domain of the page can use this capability.
		//		The build system can inline the cache values though, to allow for xdomain hosting.
		// module: String||Object
		//		If a String with slashes, a fully resolved path; if a String without slashes, the
		//		module name to use for the base part of the URL, similar to module argument
		//		to `dojo.moduleUrl`. If an Object, something that has a .toString() method that
		//		generates a valid path for the cache item. For example, a dojo._Url object.
		// url: String
		//		The rest of the path to append to the path derived from the module argument. If
		//		module is an object, then this second argument should be the "value" argument instead.
		// value: String||Object?
		//		If a String, the value to use in the cache for the module/url combination.
		//		If an Object, it can have two properties: value and sanitize. The value property
		//		should be the value to use in the cache, and sanitize can be set to true or false,
		//		to indicate if XML declarations should be removed from the value and if the HTML
		//		inside a body tag in the value should be extracted as the real value. The value argument
		//		or the value property on the value argument are usually only used by the build system
		//		as it inlines cache content.
		// example:
		//		To ask dojo.cache to fetch content and store it in the cache (the dojo["cache"] style
		//		of call is used to avoid an issue with the build system erroneously trying to intern
		//		this example. To get the build system to intern your dojo.cache calls, use the
		//		"dojo.cache" style of call):
		//		| //If template.html contains "<h1>Hello</h1>" that will be
		//		| //the value for the text variable.
		//		| var text = dojo["cache"]("my.module", "template.html");
		// example:
		//		To ask dojo.cache to fetch content and store it in the cache, and sanitize the input
		//		 (the dojo["cache"] style of call is used to avoid an issue with the build system
		//		erroneously trying to intern this example. To get the build system to intern your
		//		dojo.cache calls, use the "dojo.cache" style of call):
		//		| //If template.html contains "<html><body><h1>Hello</h1></body></html>", the
		//		| //text variable will contain just "<h1>Hello</h1>".
		//		| var text = dojo["cache"]("my.module", "template.html", {sanitize: true});
		// example:
		//		Same example as previous, but demonstrates how an object can be passed in as
		//		the first argument, then the value argument can then be the second argument.
		//		| //If template.html contains "<html><body><h1>Hello</h1></body></html>", the
		//		| //text variable will contain just "<h1>Hello</h1>".
		//		| var text = dojo["cache"](new dojo._Url("my/module/template.html"), {sanitize: true});

		//	 * (string string [value]) => (module, url, value)
		//	 * (object [value])        => (module, value), url defaults to ""
		//
		//	 * if module is an object, then it must be convertable to a string
		//	 * (module, url) module + (url ? ("/" + url) : "") must be a legal argument to require.toUrl
		//	 * value may be a string or an object; if an object then may have the properties "value" and/or "sanitize"
		var key;
		if(typeof module=="string"){
			if(/\//.test(module)){
				// module is a version 1.7+ resolved path
				key = module;
				value = url;
			}else{
				// module is a version 1.6- argument to dojo.moduleUrl
				key = require.toUrl(module.replace(/\./g, "/") + (url ? ("/" + url) : ""));
			}
		}else{
			key = module + "";
			value = url;
		}
		var
			val = (value != undefined && typeof value != "string") ? value.value : value,
			sanitize = value && value.sanitize;

		if(typeof val == "string"){
			//We have a string, set cache value
			theCache[key] = val;
			return sanitize ? strip(val) : val;
		}else if(val === null){
			//Remove cached value
			delete theCache[key];
			return null;
		}else{
			//Allow cache values to be empty strings. If key property does
			//not exist, fetch it.
			if(!(key in theCache)){
				getText(key, true, function(text){
					theCache[key]= text;
				});
			}
			return sanitize ? strip(theCache[key]) : theCache[key];
		}
	};

	return {
		// summary:
		//		This module implements the dojo/text! plugin and the dojo.cache API.
		// description:
		//		We choose to include our own plugin to leverage functionality already contained in dojo
		//		and thereby reduce the size of the plugin compared to various foreign loader implementations.
		//		Also, this allows foreign AMD loaders to be used without their plugins.
		//
		//		CAUTION: this module is designed to optionally function synchronously to support the dojo v1.x synchronous
		//		loader. This feature is outside the scope of the CommonJS plugins specification.

		// the dojo/text caches it's own resources because of dojo.cache
		dynamic: true,

		normalize: function(id, toAbsMid){
			// id is something like (path may be relative):
			//
			//	 "path/to/text.html"
			//	 "path/to/text.html!strip"
			var parts= id.split("!"),
				url= parts[0];
			return (/^\./.test(url) ? toAbsMid(url) : url) + (parts[1] ? "!" + parts[1] : "");
		},

		load: function(id, require, load){
			// id: String
			//		Path to the resource.
			// require: Function
			//		Object that include the function toUrl with given id returns a valid URL from which to load the text.
			// load: Function
			//		Callback function which will be called, when the loading finished.

			// id is something like (path is always absolute):
			//
			//	 "path/to/text.html"
			//	 "path/to/text.html!strip"
			var
				parts= id.split("!"),
				stripFlag= parts.length>1,
				absMid= parts[0],
				url = require.toUrl(parts[0]),
				requireCacheUrl = "url:" + url,
				text = notFound,
				finish = function(text){
					load(stripFlag ? strip(text) : text);
				};
			if(absMid in theCache){
				text = theCache[absMid];
			}else if(requireCacheUrl in require.cache){
				text = require.cache[requireCacheUrl];
			}else if(url in theCache){
				text = theCache[url];
			}
			if(text===notFound){
				if(pending[url]){
					pending[url].push(finish);
				}else{
					var pendingList = pending[url] = [finish];
					getText(url, !require.async, function(text){
						theCache[absMid]= theCache[url]= text;
						for(var i = 0; i<pendingList.length;){
							pendingList[i++](text);
						}
						delete pending[url];
					});
				}
			}else{
				finish(text);
			}
		}
	};

});


},
'url:epi/shell/widget/dialog/templates/Dialog.html':"﻿<div class=\"dijitDialog\" role=\"dialog\" aria-labelledby=\"${id}_title\">\n    <div data-dojo-attach-point=\"titleBar\" class=\"dijitDialogTitleBar\">\n        <span data-dojo-attach-point=\"titleNode\" class=\"dijitDialogTitle\" id=\"${id}_title\"></span>\n        <span data-dojo-attach-point=\"closeButtonNode\" class=\"dijitDialogCloseIcon\" data-dojo-attach-event=\"ondijitclick: onCancel\" title=\"${resources.close}\" role=\"button\" tabIndex=\"-1\">\n            <span data-dojo-attach-point=\"closeText\" class=\"closeText\" title=\"${resources.cancel}\">x</span>\n        </span>\n    </div>\n    <div data-dojo-attach-point=\"contentContainerNode\" class=\"epi-dialogPaneContent\">\n        <span data-dojo-attach-point=\"iconNode\" class=\"dijitIcon dijitInline epi-dialogIcon\"></span>\n        <div data-dojo-attach-point=\"headingNode\" class=\"epi-dialogDescriptionHeader\"></div>\n        <div data-dojo-attach-point=\"descriptionNode\" class=\"epi-dialogDescriptionSummary\"></div>\n        <div data-dojo-attach-point=\"containerNode\" class=\"dijitDialogPaneContentArea\"></div>\n        <div data-dojo-attach-point=\"actionContainerNode\" class=\"dijitDialogPaneActionBar\"></div>\n    </div>\n</div>",
'dijit/a11y':function(){
define("dijit/a11y", [
	"dojo/_base/array", // array.forEach array.map
	"dojo/dom",			// dom.byId
	"dojo/dom-attr", // domAttr.attr domAttr.has
	"dojo/dom-style", // domStyle.style
	"dojo/_base/lang", // lang.mixin()
	"dojo/sniff", // has("ie")  1 
	"./main"	// for exporting methods to dijit namespace
], function(array, dom, domAttr, domStyle, lang, has, dijit){

	// module:
	//		dijit/a11y

	var undefined;

	var a11y = {
		// summary:
		//		Accessibility utility functions (keyboard, tab stops, etc.)

		_isElementShown: function(/*Element*/ elem){
			var s = domStyle.get(elem);
			return (s.visibility != "hidden")
				&& (s.visibility != "collapsed")
				&& (s.display != "none")
				&& (domAttr.get(elem, "type") != "hidden");
		},

		hasDefaultTabStop: function(/*Element*/ elem){
			// summary:
			//		Tests if element is tab-navigable even without an explicit tabIndex setting

			// No explicit tabIndex setting, need to investigate node type
			switch(elem.nodeName.toLowerCase()){
				case "a":
					// An <a> w/out a tabindex is only navigable if it has an href
					return domAttr.has(elem, "href");
				case "area":
				case "button":
				case "input":
				case "object":
				case "select":
				case "textarea":
					// These are navigable by default
					return true;
				case "iframe":
					// If it's an editor <iframe> then it's tab navigable.
					var body;
					try{
						// non-IE
						var contentDocument = elem.contentDocument;
						if("designMode" in contentDocument && contentDocument.designMode == "on"){
							return true;
						}
						body = contentDocument.body;
					}catch(e1){
						// contentWindow.document isn't accessible within IE7/8
						// if the iframe.src points to a foreign url and this
						// page contains an element, that could get focus
						try{
							body = elem.contentWindow.document.body;
						}catch(e2){
							return false;
						}
					}
					return body && (body.contentEditable == 'true' ||
						(body.firstChild && body.firstChild.contentEditable == 'true'));
				default:
					return elem.contentEditable == 'true';
			}
		},

		effectiveTabIndex: function(/*Element*/ elem){
			// summary:
			//		Returns effective tabIndex of an element, either a number, or undefined if element isn't focusable.

			if(domAttr.get(elem, "disabled")){
				return undefined;
			}else if(domAttr.has(elem, "tabIndex")){
				// Explicit tab index setting
				return +domAttr.get(elem, "tabIndex");// + to convert string --> number
			}else{
				// No explicit tabIndex setting, so depends on node type
				return a11y.hasDefaultTabStop(elem) ? 0 : undefined;
			}
		},

		isTabNavigable: function(/*Element*/ elem){
			// summary:
			//		Tests if an element is tab-navigable

			return a11y.effectiveTabIndex(elem) >= 0;
		},

		isFocusable: function(/*Element*/ elem){
			// summary:
			//		Tests if an element is focusable by tabbing to it, or clicking it with the mouse.

			return a11y.effectiveTabIndex(elem) >= -1;
		},

		_getTabNavigable: function(/*DOMNode*/ root){
			// summary:
			//		Finds descendants of the specified root node.
			// description:
			//		Finds the following descendants of the specified root node:
			//
			//		- the first tab-navigable element in document order
			//		  without a tabIndex or with tabIndex="0"
			//		- the last tab-navigable element in document order
			//		  without a tabIndex or with tabIndex="0"
			//		- the first element in document order with the lowest
			//		  positive tabIndex value
			//		- the last element in document order with the highest
			//		  positive tabIndex value
			var first, last, lowest, lowestTabindex, highest, highestTabindex, radioSelected = {};

			function radioName(node){
				// If this element is part of a radio button group, return the name for that group.
				return node && node.tagName.toLowerCase() == "input" &&
					node.type && node.type.toLowerCase() == "radio" &&
					node.name && node.name.toLowerCase();
			}

			var shown = a11y._isElementShown, effectiveTabIndex = a11y.effectiveTabIndex;
			var walkTree = function(/*DOMNode*/ parent){
				for(var child = parent.firstChild; child; child = child.nextSibling){
					// Skip text elements, hidden elements, and also non-HTML elements (those in custom namespaces) in IE,
					// since show() invokes getAttribute("type"), which crash on VML nodes in IE.
					if(child.nodeType != 1 || (has("ie") <= 9 && child.scopeName !== "HTML") || !shown(child)){
						continue;
					}

					var tabindex = effectiveTabIndex(child);
					if(tabindex >= 0){
						if(tabindex == 0){
							if(!first){
								first = child;
							}
							last = child;
						}else if(tabindex > 0){
							if(!lowest || tabindex < lowestTabindex){
								lowestTabindex = tabindex;
								lowest = child;
							}
							if(!highest || tabindex >= highestTabindex){
								highestTabindex = tabindex;
								highest = child;
							}
						}
						var rn = radioName(child);
						if(domAttr.get(child, "checked") && rn){
							radioSelected[rn] = child;
						}
					}
					if(child.nodeName.toUpperCase() != 'SELECT'){
						walkTree(child);
					}
				}
			};
			if(shown(root)){
				walkTree(root);
			}
			function rs(node){
				// substitute checked radio button for unchecked one, if there is a checked one with the same name.
				return radioSelected[radioName(node)] || node;
			}

			return { first: rs(first), last: rs(last), lowest: rs(lowest), highest: rs(highest) };
		},

		getFirstInTabbingOrder: function(/*String|DOMNode*/ root, /*Document?*/ doc){
			// summary:
			//		Finds the descendant of the specified root node
			//		that is first in the tabbing order
			var elems = a11y._getTabNavigable(dom.byId(root, doc));
			return elems.lowest ? elems.lowest : elems.first; // DomNode
		},

		getLastInTabbingOrder: function(/*String|DOMNode*/ root, /*Document?*/ doc){
			// summary:
			//		Finds the descendant of the specified root node
			//		that is last in the tabbing order
			var elems = a11y._getTabNavigable(dom.byId(root, doc));
			return elems.last ? elems.last : elems.highest; // DomNode
		}
	};

	 1  && lang.mixin(dijit, a11y);

	return a11y;
});

},
'dijit/focus':function(){
define("dijit/focus", [
	"dojo/aspect",
	"dojo/_base/declare", // declare
	"dojo/dom", // domAttr.get dom.isDescendant
	"dojo/dom-attr", // domAttr.get dom.isDescendant
	"dojo/dom-construct", // connect to domConstruct.empty, domConstruct.destroy
	"dojo/Evented",
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"dojo/ready",
	"dojo/sniff", // has("ie")
	"dojo/Stateful",
	"dojo/_base/unload", // unload.addOnWindowUnload
	"dojo/_base/window", // win.body
	"dojo/window", // winUtils.get
	"./a11y",	// a11y.isTabNavigable
	"./registry",	// registry.byId
	"./main"		// to set dijit.focus
], function(aspect, declare, dom, domAttr, domConstruct, Evented, lang, on, ready, has, Stateful, unload, win, winUtils,
			a11y, registry, dijit){

	// module:
	//		dijit/focus

	var lastFocusin;

	var FocusManager = declare([Stateful, Evented], {
		// summary:
		//		Tracks the currently focused node, and which widgets are currently "active".
		//		Access via require(["dijit/focus"], function(focus){ ... }).
		//
		//		A widget is considered active if it or a descendant widget has focus,
		//		or if a non-focusable node of this widget or a descendant was recently clicked.
		//
		//		Call focus.watch("curNode", callback) to track the current focused DOMNode,
		//		or focus.watch("activeStack", callback) to track the currently focused stack of widgets.
		//
		//		Call focus.on("widget-blur", func) or focus.on("widget-focus", ...) to monitor when
		//		when widgets become active/inactive
		//
		//		Finally, focus(node) will focus a node, suppressing errors if the node doesn't exist.

		// curNode: DomNode
		//		Currently focused item on screen
		curNode: null,

		// activeStack: dijit/_WidgetBase[]
		//		List of currently active widgets (focused widget and it's ancestors)
		activeStack: [],

		constructor: function(){
			// Don't leave curNode/prevNode pointing to bogus elements
			var check = lang.hitch(this, function(node){
				if(dom.isDescendant(this.curNode, node)){
					this.set("curNode", null);
				}
				if(dom.isDescendant(this.prevNode, node)){
					this.set("prevNode", null);
				}
			});
			aspect.before(domConstruct, "empty", check);
			aspect.before(domConstruct, "destroy", check);
		},

		registerIframe: function(/*DomNode*/ iframe){
			// summary:
			//		Registers listeners on the specified iframe so that any click
			//		or focus event on that iframe (or anything in it) is reported
			//		as a focus/click event on the `<iframe>` itself.
			// description:
			//		Currently only used by editor.
			// returns:
			//		Handle with remove() method to deregister.
			return this.registerWin(iframe.contentWindow, iframe);
		},

		registerWin: function(/*Window?*/targetWindow, /*DomNode?*/ effectiveNode){
			// summary:
			//		Registers listeners on the specified window (either the main
			//		window or an iframe's window) to detect when the user has clicked somewhere
			//		or focused somewhere.
			// description:
			//		Users should call registerIframe() instead of this method.
			// targetWindow:
			//		If specified this is the window associated with the iframe,
			//		i.e. iframe.contentWindow.
			// effectiveNode:
			//		If specified, report any focus events inside targetWindow as
			//		an event on effectiveNode, rather than on evt.target.
			// returns:
			//		Handle with remove() method to deregister.

			// TODO: make this function private in 2.0; Editor/users should call registerIframe(),

			// Listen for blur and focus events on targetWindow's document.
			var _this = this,
				body = targetWindow.document && targetWindow.document.body;

			if(body){
				var mdh = on(body, 'mousedown', function(evt){
					_this._justMouseDowned = true;
					// Use a 13 ms timeout to work-around Chrome resolving too fast and focusout
					// events not seeing that a mousedown just happened when a popup closes.
					// See https://bugs.dojotoolkit.org/ticket/17668
					setTimeout(function(){ _this._justMouseDowned = false; }, 13);

					// workaround weird IE bug where the click is on an orphaned node
					// (first time clicking a Select/DropDownButton inside a TooltipDialog).
					// actually, strangely this is happening on latest chrome too.
					if(evt && evt.target && evt.target.parentNode == null){
						return;
					}

					_this._onTouchNode(effectiveNode || evt.target, "mouse");
				});

				var fih = on(body, 'focusin', function(evt){

					lastFocusin = (new Date()).getTime();

					// When you refocus the browser window, IE gives an event with an empty srcElement
					if(!evt.target.tagName) { return; }

					// IE reports that nodes like <body> have gotten focus, even though they have tabIndex=-1,
					// ignore those events
					var tag = evt.target.tagName.toLowerCase();
					if(tag == "#document" || tag == "body"){ return; }

					if(a11y.isFocusable(evt.target)){
						_this._onFocusNode(effectiveNode || evt.target);
					}else{
						// Previous code called _onTouchNode() for any activate event on a non-focusable node.   Can
						// probably just ignore such an event as it will be handled by onmousedown handler above, but
						// leaving the code for now.
						_this._onTouchNode(effectiveNode || evt.target);
					}
				});

				var foh = on(body, 'focusout', function(evt){
					// IE9+ has a problem where focusout events come after the corresponding focusin event.  At least
					// when moving focus from the Editor's <iframe> to a normal DOMNode.
					if((new Date()).getTime() < lastFocusin + 100){
						return;
					}

					_this._onBlurNode(effectiveNode || evt.target);
				});

				return {
					remove: function(){
						mdh.remove();
						fih.remove();
						foh.remove();
						mdh = fih = foh = null;
						body = null;	// prevent memory leak (apparent circular reference via closure)
					}
				};
			}
		},

		_onBlurNode: function(/*DomNode*/ node){
			// summary:
			//		Called when focus leaves a node.
			//		Usually ignored, _unless_ it *isn't* followed by touching another node,
			//		which indicates that we tabbed off the last field on the page,
			//		in which case every widget is marked inactive

			// If the blur event isn't followed by a focus event, it means the user clicked on something unfocusable,
			// so clear focus.
			if(this._clearFocusTimer){
				clearTimeout(this._clearFocusTimer);
			}
			this._clearFocusTimer = setTimeout(lang.hitch(this, function(){
				this.set("prevNode", this.curNode);
				this.set("curNode", null);
			}), 0);

			if(this._justMouseDowned){
				// the mouse down caused a new widget to be marked as active; this blur event
				// is coming late, so ignore it.
				return;
			}

			// If the blur event isn't followed by a focus or touch event then mark all widgets as inactive.
			if(this._clearActiveWidgetsTimer){
				clearTimeout(this._clearActiveWidgetsTimer);
			}
			this._clearActiveWidgetsTimer = setTimeout(lang.hitch(this, function(){
				delete this._clearActiveWidgetsTimer;
				this._setStack([]);
			}), 0);
		},

		_onTouchNode: function(/*DomNode*/ node, /*String*/ by){
			// summary:
			//		Callback when node is focused or mouse-downed
			// node:
			//		The node that was touched.
			// by:
			//		"mouse" if the focus/touch was caused by a mouse down event

			// ignore the recent blurNode event
			if(this._clearActiveWidgetsTimer){
				clearTimeout(this._clearActiveWidgetsTimer);
				delete this._clearActiveWidgetsTimer;
			}

			// compute stack of active widgets (ex: ComboButton --> Menu --> MenuItem)
			var newStack=[];
			try{
				while(node){
					var popupParent = domAttr.get(node, "dijitPopupParent");
					if(popupParent){
						node=registry.byId(popupParent).domNode;
					}else if(node.tagName && node.tagName.toLowerCase() == "body"){
						// is this the root of the document or just the root of an iframe?
						if(node === win.body()){
							// node is the root of the main document
							break;
						}
						// otherwise, find the iframe this node refers to (can't access it via parentNode,
						// need to do this trick instead). window.frameElement is supported in IE/FF/Webkit
						node=winUtils.get(node.ownerDocument).frameElement;
					}else{
						// if this node is the root node of a widget, then add widget id to stack,
						// except ignore clicks on disabled widgets (actually focusing a disabled widget still works,
						// to support MenuItem)
						var id = node.getAttribute && node.getAttribute("widgetId"),
							widget = id && registry.byId(id);
						if(widget && !(by == "mouse" && widget.get("disabled"))){
							newStack.unshift(id);
						}
						node=node.parentNode;
					}
				}
			}catch(e){ /* squelch */ }

			this._setStack(newStack, by);
		},

		_onFocusNode: function(/*DomNode*/ node){
			// summary:
			//		Callback when node is focused

			if(!node){
				return;
			}

			if(node.nodeType == 9){
				// Ignore focus events on the document itself.  This is here so that
				// (for example) clicking the up/down arrows of a spinner
				// (which don't get focus) won't cause that widget to blur. (FF issue)
				return;
			}

			// There was probably a blur event right before this event, but since we have a new focus, don't
			// do anything with the blur
			if(this._clearFocusTimer){
				clearTimeout(this._clearFocusTimer);
				delete this._clearFocusTimer;
			}

			this._onTouchNode(node);

			if(node == this.curNode){ return; }
			this.set("prevNode", this.curNode);
			this.set("curNode", node);
		},

		_setStack: function(/*String[]*/ newStack, /*String*/ by){
			// summary:
			//		The stack of active widgets has changed.  Send out appropriate events and records new stack.
			// newStack:
			//		array of widget id's, starting from the top (outermost) widget
			// by:
			//		"mouse" if the focus/touch was caused by a mouse down event

			var oldStack = this.activeStack;
			this.set("activeStack", newStack);

			// compare old stack to new stack to see how many elements they have in common
			for(var nCommon=0; nCommon<Math.min(oldStack.length, newStack.length); nCommon++){
				if(oldStack[nCommon] != newStack[nCommon]){
					break;
				}
			}

			var widget;
			// for all elements that have gone out of focus, set focused=false
			for(var i=oldStack.length-1; i>=nCommon; i--){
				widget = registry.byId(oldStack[i]);
				if(widget){
					widget._hasBeenBlurred = true;		// TODO: used by form widgets, should be moved there
					widget.set("focused", false);
					if(widget._focusManager == this){
						widget._onBlur(by);
					}
					this.emit("widget-blur", widget, by);
				}
			}

			// for all element that have come into focus, set focused=true
			for(i=nCommon; i<newStack.length; i++){
				widget = registry.byId(newStack[i]);
				if(widget){
					widget.set("focused", true);
					if(widget._focusManager == this){
						widget._onFocus(by);
					}
					this.emit("widget-focus", widget, by);
				}
			}
		},

		focus: function(node){
			// summary:
			//		Focus the specified node, suppressing errors if they occur
			if(node){
				try{ node.focus(); }catch(e){/*quiet*/}
			}
		}
	});

	var singleton = new FocusManager();

	// register top window and all the iframes it contains
	ready(function(){
		var handle = singleton.registerWin(winUtils.get(win.doc));
		if(has("ie")){
			unload.addOnWindowUnload(function(){
				if(handle){	// because this gets called twice when doh.robot is running
					handle.remove();
					handle = null;
				}
			});
		}
	});

	// Setup dijit.focus as a pointer to the singleton but also (for backwards compatibility)
	// as a function to set focus.   Remove for 2.0.
	dijit.focus = function(node){
		singleton.focus(node);	// indirection here allows dijit/_base/focus.js to override behavior
	};
	for(var attr in singleton){
		if(!/^_/.test(attr)){
			dijit.focus[attr] = typeof singleton[attr] == "function" ? lang.hitch(singleton, attr) : singleton[attr];
		}
	}
	singleton.watch(function(attr, oldVal, newVal){
		dijit.focus[attr] = newVal;
	});

	return singleton;
});

},
'dojo/window':function(){
define("dojo/window", ["./_base/lang", "./sniff", "./_base/window", "./dom", "./dom-geometry", "./dom-style", "./dom-construct"],
	function(lang, has, baseWindow, dom, geom, style, domConstruct){

	// feature detection
	/* not needed but included here for future reference
	has.add("rtl-innerVerticalScrollBar-on-left", function(win, doc){
		var	body = baseWindow.body(doc),
			scrollable = domConstruct.create('div', {
				style: {overflow:'scroll', overflowX:'hidden', direction:'rtl', visibility:'hidden', position:'absolute', left:'0', width:'64px', height:'64px'}
			}, body, "last"),
			center = domConstruct.create('center', {
				style: {overflow:'hidden', direction:'ltr'}
			}, scrollable, "last"),
			inner = domConstruct.create('div', {
				style: {overflow:'visible', display:'inline' }
			}, center, "last");
		inner.innerHTML="&nbsp;";
		var midPoint = Math.max(inner.offsetLeft, geom.position(inner).x);
		var ret = midPoint >= 32;
		center.removeChild(inner);
		scrollable.removeChild(center);
		body.removeChild(scrollable);
		return ret;
	});
	*/
	has.add("rtl-adjust-position-for-verticalScrollBar", function(win, doc){
		var	body = baseWindow.body(doc),
			scrollable = domConstruct.create('div', {
				style: {overflow:'scroll', overflowX:'visible', direction:'rtl', visibility:'hidden', position:'absolute', left:'0', top:'0', width:'64px', height:'64px'}
			}, body, "last"),
			div = domConstruct.create('div', {
				style: {overflow:'hidden', direction:'ltr'}
			}, scrollable, "last"),
			ret = geom.position(div).x != 0;
		scrollable.removeChild(div);
		body.removeChild(scrollable);
		return ret;
	});

	has.add("position-fixed-support", function(win, doc){
		// IE6, IE7+quirks, and some older mobile browsers don't support position:fixed
		var	body = baseWindow.body(doc),
			outer = domConstruct.create('span', {
				style: {visibility:'hidden', position:'fixed', left:'1px', top:'1px'}
			}, body, "last"),
			inner = domConstruct.create('span', {
				style: {position:'fixed', left:'0', top:'0'}
			}, outer, "last"),
			ret = geom.position(inner).x != geom.position(outer).x;
		outer.removeChild(inner);
		body.removeChild(outer);
		return ret;
	});

	// module:
	//		dojo/window

	var window = {
		// summary:
		//		TODOC

		getBox: function(/*Document?*/ doc){
			// summary:
			//		Returns the dimensions and scroll position of the viewable area of a browser window

			doc = doc || baseWindow.doc;

			var
				scrollRoot = (doc.compatMode == 'BackCompat') ? baseWindow.body(doc) : doc.documentElement,
				// get scroll position
				scroll = geom.docScroll(doc), // scrollRoot.scrollTop/Left should work
				w, h;

			if(has("touch")){ // if(scrollbars not supported)
				var uiWindow = window.get(doc);   // use UI window, not dojo.global window
				// on mobile, scrollRoot.clientHeight <= uiWindow.innerHeight <= scrollRoot.offsetHeight, return uiWindow.innerHeight
				w = uiWindow.innerWidth || scrollRoot.clientWidth; // || scrollRoot.clientXXX probably never evaluated
				h = uiWindow.innerHeight || scrollRoot.clientHeight;
			}else{
				// on desktops, scrollRoot.clientHeight <= scrollRoot.offsetHeight <= uiWindow.innerHeight, return scrollRoot.clientHeight
				// uiWindow.innerWidth/Height includes the scrollbar and cannot be used
				w = scrollRoot.clientWidth;
				h = scrollRoot.clientHeight;
			}
			return {
				l: scroll.x,
				t: scroll.y,
				w: w,
				h: h
			};
		},

		get: function(/*Document*/ doc){
			// summary:
			//		Get window object associated with document doc.
			// doc:
			//		The document to get the associated window for.

			// In some IE versions (at least 6.0), document.parentWindow does not return a
			// reference to the real window object (maybe a copy), so we must fix it as well
			// We use IE specific execScript to attach the real window reference to
			// document._parentWindow for later use
			if(has("ie") && window !== document.parentWindow){
				/*
				In IE 6, only the variable "window" can be used to connect events (others
				may be only copies).
				*/
				doc.parentWindow.execScript("document._parentWindow = window;", "Javascript");
				//to prevent memory leak, unset it after use
				//another possibility is to add an onUnload handler which seems overkill to me (liucougar)
				var win = doc._parentWindow;
				doc._parentWindow = null;
				return win;	//	Window
			}

			return doc.parentWindow || doc.defaultView;	//	Window
		},

		scrollIntoView: function(/*DomNode*/ node, /*Object?*/ pos){
			// summary:
			//		Scroll the passed node into view using minimal movement, if it is not already.

			// Don't rely on node.scrollIntoView working just because the function is there since
			// it forces the node to the page's bottom or top (and left or right in IE) without consideration for the minimal movement.
			// WebKit's node.scrollIntoViewIfNeeded doesn't work either for inner scrollbars in right-to-left mode
			// and when there's a fixed position scrollable element

			try{ // catch unexpected/unrecreatable errors (#7808) since we can recover using a semi-acceptable native method
				node = dom.byId(node);
				var	doc = node.ownerDocument || baseWindow.doc,	// TODO: why baseWindow.doc?  Isn't node.ownerDocument always defined?
					body = baseWindow.body(doc),
					html = doc.documentElement || body.parentNode,
					isIE = has("ie"),
					isWK = has("webkit");
				// if an untested browser, then use the native method
				if(node == body || node == html){ return; }
				if(!(has("mozilla") || isIE || isWK || has("opera") || has("trident")) && ("scrollIntoView" in node)){
					node.scrollIntoView(false); // short-circuit to native if possible
					return;
				}
				var	backCompat = doc.compatMode == 'BackCompat',
					rootWidth = Math.min(body.clientWidth || html.clientWidth, html.clientWidth || body.clientWidth),
					rootHeight = Math.min(body.clientHeight || html.clientHeight, html.clientHeight || body.clientHeight),
					scrollRoot = (isWK || backCompat) ? body : html,
					nodePos = pos || geom.position(node),
					el = node.parentNode,
					isFixed = function(el){
						return (isIE <= 6 || (isIE == 7 && backCompat))
							? false
							: (has("position-fixed-support") && (style.get(el, 'position').toLowerCase() == "fixed"));
					},
					self = this,
					scrollElementBy = function(el, x, y){
						if(el.tagName == "BODY" || el.tagName == "HTML"){
							self.get(el.ownerDocument).scrollBy(x, y);
						}else{
							x && (el.scrollLeft += x);
							y && (el.scrollTop += y);
						}
					};
				if(isFixed(node)){ return; } // nothing to do
				while(el){
					if(el == body){ el = scrollRoot; }
					var	elPos = geom.position(el),
						fixedPos = isFixed(el),
						rtl = style.getComputedStyle(el).direction.toLowerCase() == "rtl";

					if(el == scrollRoot){
						elPos.w = rootWidth; elPos.h = rootHeight;
						if(scrollRoot == html && (isIE || has("trident")) && rtl){ elPos.x += scrollRoot.offsetWidth-elPos.w; } // IE workaround where scrollbar causes negative x
						if(elPos.x < 0 || !isIE || isIE >= 9 || has("trident")){ elPos.x = 0; } // older IE can have values > 0
						if(elPos.y < 0 || !isIE || isIE >= 9 || has("trident")){ elPos.y = 0; }
					}else{
						var pb = geom.getPadBorderExtents(el);
						elPos.w -= pb.w; elPos.h -= pb.h; elPos.x += pb.l; elPos.y += pb.t;
						var clientSize = el.clientWidth,
							scrollBarSize = elPos.w - clientSize;
						if(clientSize > 0 && scrollBarSize > 0){
							if(rtl && has("rtl-adjust-position-for-verticalScrollBar")){
								elPos.x += scrollBarSize;
							}
							elPos.w = clientSize;
						}
						clientSize = el.clientHeight;
						scrollBarSize = elPos.h - clientSize;
						if(clientSize > 0 && scrollBarSize > 0){
							elPos.h = clientSize;
						}
					}
					if(fixedPos){ // bounded by viewport, not parents
						if(elPos.y < 0){
							elPos.h += elPos.y; elPos.y = 0;
						}
						if(elPos.x < 0){
							elPos.w += elPos.x; elPos.x = 0;
						}
						if(elPos.y + elPos.h > rootHeight){
							elPos.h = rootHeight - elPos.y;
						}
						if(elPos.x + elPos.w > rootWidth){
							elPos.w = rootWidth - elPos.x;
						}
					}
					// calculate overflow in all 4 directions
					var	l = nodePos.x - elPos.x, // beyond left: < 0
//						t = nodePos.y - Math.max(elPos.y, 0), // beyond top: < 0
						t = nodePos.y - elPos.y, // beyond top: < 0
						r = l + nodePos.w - elPos.w, // beyond right: > 0
						bot = t + nodePos.h - elPos.h; // beyond bottom: > 0
					var s, old;
					if(r * l > 0 && (!!el.scrollLeft || el == scrollRoot || el.scrollWidth > el.offsetHeight)){
						s = Math[l < 0? "max" : "min"](l, r);
						if(rtl && ((isIE == 8 && !backCompat) || isIE >= 9 || has("trident"))){ s = -s; }
						old = el.scrollLeft;
						scrollElementBy(el, s, 0);
						s = el.scrollLeft - old;
						nodePos.x -= s;
					}
					if(bot * t > 0 && (!!el.scrollTop || el == scrollRoot || el.scrollHeight > el.offsetHeight)){
						s = Math.ceil(Math[t < 0? "max" : "min"](t, bot));
						old = el.scrollTop;
						scrollElementBy(el, 0, s);
						s = el.scrollTop - old;
						nodePos.y -= s;
					}
					el = (el != scrollRoot) && !fixedPos && el.parentNode;
				}
			}catch(error){
				console.error('scrollIntoView: ' + error);
				node.scrollIntoView(false);
			}
		}
	};

	 1  && lang.setObject("dojo.window", window);

	return window;
});

},
'dijit/Dialog':function(){
require({cache:{
'url:dijit/templates/Dialog.html':"<div class=\"dijitDialog\" role=\"dialog\" aria-labelledby=\"${id}_title\">\n\t<div data-dojo-attach-point=\"titleBar\" class=\"dijitDialogTitleBar\">\n\t\t<span data-dojo-attach-point=\"titleNode\" class=\"dijitDialogTitle\" id=\"${id}_title\"\n\t\t\t\trole=\"heading\" level=\"1\"></span>\n\t\t<span data-dojo-attach-point=\"closeButtonNode\" class=\"dijitDialogCloseIcon\" data-dojo-attach-event=\"ondijitclick: onCancel\" title=\"${buttonCancel}\" role=\"button\" tabindex=\"0\">\n\t\t\t<span data-dojo-attach-point=\"closeText\" class=\"closeText\" title=\"${buttonCancel}\">x</span>\n\t\t</span>\n\t</div>\n\t<div data-dojo-attach-point=\"containerNode\" class=\"dijitDialogPaneContent\"></div>\n</div>\n"}});
define("dijit/Dialog", [
	"require",
	"dojo/_base/array", // array.forEach array.indexOf array.map
	"dojo/_base/connect", // connect._keypress
	"dojo/_base/declare", // declare
	"dojo/_base/Deferred", // Deferred
	"dojo/dom", // dom.isDescendant
	"dojo/dom-class", // domClass.add domClass.contains
	"dojo/dom-geometry", // domGeometry.position
	"dojo/dom-style", // domStyle.set
	"dojo/_base/event", // event.stop
	"dojo/_base/fx", // fx.fadeIn fx.fadeOut
	"dojo/i18n", // i18n.getLocalization
	"dojo/keys",
	"dojo/_base/lang", // lang.mixin lang.hitch
	"dojo/on",
	"dojo/ready",
	"dojo/sniff", // has("ie") has("opera") has("dijit-legacy-requires")
	"dojo/window", // winUtils.getBox, winUtils.get
	"dojo/dnd/Moveable", // Moveable
	"dojo/dnd/TimedMoveable", // TimedMoveable
	"./focus",
	"./_base/manager",	// manager.defaultDuration
	"./_Widget",
	"./_TemplatedMixin",
	"./_CssStateMixin",
	"./form/_FormMixin",
	"./_DialogMixin",
	"./DialogUnderlay",
	"./layout/ContentPane",
	"dojo/text!./templates/Dialog.html",
	"./main",			// for back-compat, exporting dijit._underlay (remove in 2.0)
	"dojo/i18n!./nls/common"
], function(require, array, connect, declare, Deferred,
			dom, domClass, domGeometry, domStyle, event, fx, i18n, keys, lang, on, ready, has, winUtils,
			Moveable, TimedMoveable, focus, manager, _Widget, _TemplatedMixin, _CssStateMixin, _FormMixin, _DialogMixin,
			DialogUnderlay, ContentPane, template, dijit){

	// module:
	//		dijit/Dialog

	/*=====
	dijit._underlay = function(kwArgs){
		// summary:
		//		A shared instance of a `dijit.DialogUnderlay`
		//
		// description:
		//		A shared instance of a `dijit.DialogUnderlay` created and
		//		used by `dijit.Dialog`, though never created until some Dialog
		//		or subclass thereof is shown.
	};
	=====*/

	var _DialogBase = declare("dijit._DialogBase", [_TemplatedMixin, _FormMixin, _DialogMixin, _CssStateMixin], {
		templateString: template,

		baseClass: "dijitDialog",

		cssStateNodes: {
			closeButtonNode: "dijitDialogCloseIcon"
		},

		// Map widget attributes to DOMNode attributes.
		_setTitleAttr: [
			{ node: "titleNode", type: "innerHTML" },
			{ node: "titleBar", type: "attribute" }
		],

		// open: [readonly] Boolean
		//		True if Dialog is currently displayed on screen.
		open: false,

		// duration: Integer
		//		The time in milliseconds it takes the dialog to fade in and out
		duration: manager.defaultDuration,

		// refocus: Boolean
		//		A Toggle to modify the default focus behavior of a Dialog, which
		//		is to re-focus the element which had focus before being opened.
		//		False will disable refocusing. Default: true
		refocus: true,

		// autofocus: Boolean
		//		A Toggle to modify the default focus behavior of a Dialog, which
		//		is to focus on the first dialog element after opening the dialog.
		//		False will disable autofocusing. Default: true
		autofocus: true,

		// _firstFocusItem: [private readonly] DomNode
		//		The pointer to the first focusable node in the dialog.
		//		Set by `dijit/_DialogMixin._getFocusItems()`.
		_firstFocusItem: null,

		// _lastFocusItem: [private readonly] DomNode
		//		The pointer to which node has focus prior to our dialog.
		//		Set by `dijit/_DialogMixin._getFocusItems()`.
		_lastFocusItem: null,

		// doLayout: [protected] Boolean
		//		Don't change this parameter from the default value.
		//		This ContentPane parameter doesn't make sense for Dialog, since Dialog
		//		is never a child of a layout container, nor can you specify the size of
		//		Dialog in order to control the size of an inner widget.
		doLayout: false,

		// draggable: Boolean
		//		Toggles the moveable aspect of the Dialog. If true, Dialog
		//		can be dragged by it's title. If false it will remain centered
		//		in the viewport.
		draggable: true,

		_setDraggableAttr: function(/*Boolean*/ val){
			// Avoid _WidgetBase behavior of copying draggable attribute to this.domNode,
			// as that prevents text select on modern browsers (#14452)
			this._set("draggable", val);
		},

		// aria-describedby: String
		//		Allows the user to add an aria-describedby attribute onto the dialog. The value should
		//		be the id of the container element of text that describes the dialog purpose (usually
		//		the first text in the dialog).
		//	|	<div data-dojo-type="dijit/Dialog" aria-describedby="intro" .....>
		//	|		<div id="intro">Introductory text</div>
		//	|		<div>rest of dialog contents</div>
		//	|	</div>
		"aria-describedby": "",

		// maxRatio: Number
		//		Maximum size to allow the dialog to expand to, relative to viewport size
		maxRatio: 0.9,

		postMixInProperties: function(){
			var _nlsResources = i18n.getLocalization("dijit", "common");
			lang.mixin(this, _nlsResources);
			this.inherited(arguments);
		},

		postCreate: function(){
			domStyle.set(this.domNode, {
				display: "none",
				position:"absolute"
			});
			this.ownerDocumentBody.appendChild(this.domNode);

			this.inherited(arguments);

			this.connect(this, "onExecute", "hide");
			this.connect(this, "onCancel", "hide");
			this._modalconnects = [];
		},

		onLoad: function(){
			// summary:
			//		Called when data has been loaded from an href.
			//		Unlike most other callbacks, this function can be connected to (via `dojo.connect`)
			//		but should *not* be overridden.
			// tags:
			//		callback

			// when href is specified we need to reposition the dialog after the data is loaded
			// and find the focusable elements
			this._position();
			if(this.autofocus && DialogLevelManager.isTop(this)){
				this._getFocusItems(this.domNode);
				focus.focus(this._firstFocusItem);
			}
			this.inherited(arguments);
		},

		_endDrag: function(){
			// summary:
			//		Called after dragging the Dialog. Saves the position of the dialog in the viewport,
			//		and also adjust position to be fully within the viewport, so user doesn't lose access to handle
			var nodePosition = domGeometry.position(this.domNode),
				viewport = winUtils.getBox(this.ownerDocument);
			nodePosition.y = Math.min(Math.max(nodePosition.y, 0), (viewport.h - nodePosition.h));
			nodePosition.x = Math.min(Math.max(nodePosition.x, 0), (viewport.w - nodePosition.w));
			this._relativePosition = nodePosition;
			this._position();
		},

		_setup: function(){
			// summary:
			//		Stuff we need to do before showing the Dialog for the first
			//		time (but we defer it until right beforehand, for
			//		performance reasons).
			// tags:
			//		private

			var node = this.domNode;

			if(this.titleBar && this.draggable){
				this._moveable = new ((has("ie") == 6) ? TimedMoveable // prevent overload, see #5285
					: Moveable)(node, { handle: this.titleBar });
				this.connect(this._moveable, "onMoveStop", "_endDrag");
			}else{
				domClass.add(node,"dijitDialogFixed");
			}

			this.underlayAttrs = {
				dialogId: this.id,
				"class": array.map(this["class"].split(/\s/), function(s){ return s+"_underlay"; }).join(" "),
				ownerDocument: this.ownerDocument
			};
		},

		_size: function(){
			// summary:
			//		If necessary, shrink dialog contents so dialog fits in viewport
			// tags:
			//		private

			this._checkIfSingleChild();

			// If we resized the dialog contents earlier, reset them back to original size, so
			// that if the user later increases the viewport size, the dialog can display w/out a scrollbar.
			// Need to do this before the domGeometry.position(this.domNode) call below.
			if(this._singleChild){
				if(typeof this._singleChildOriginalStyle != "undefined"){
					this._singleChild.domNode.style.cssText = this._singleChildOriginalStyle;
					delete this._singleChildOriginalStyle;
				}
			}else{
				domStyle.set(this.containerNode, {
					width:"auto",
					height:"auto"
				});
			}

			var bb = domGeometry.position(this.domNode);

			// Get viewport size but then reduce it by a bit; Dialog should always have some space around it
			// to indicate that it's a popup.  This will also compensate for possible scrollbars on viewport.
			var viewport = winUtils.getBox(this.ownerDocument);
			viewport.w *= this.maxRatio;
			viewport.h *= this.maxRatio;

			if(bb.w >= viewport.w || bb.h >= viewport.h){
				// Reduce size of dialog contents so that dialog fits in viewport

				var containerSize = domGeometry.position(this.containerNode),
					w = Math.min(bb.w, viewport.w) - (bb.w - containerSize.w),
					h = Math.min(bb.h, viewport.h) - (bb.h - containerSize.h);

				if(this._singleChild && this._singleChild.resize){
					if(typeof this._singleChildOriginalStyle == "undefined"){
						this._singleChildOriginalStyle = this._singleChild.domNode.style.cssText;
					}
					this._singleChild.resize({w: w, h: h});
				}else{
					domStyle.set(this.containerNode, {
						width: w + "px",
						height: h + "px",
						overflow: "auto",
						position: "relative"	// workaround IE bug moving scrollbar or dragging dialog
					});
				}
			}else{
				if(this._singleChild && this._singleChild.resize){
					this._singleChild.resize();
				}
			}
		},

		_position: function(){
			// summary:
			//		Position modal dialog in the viewport. If no relative offset
			//		in the viewport has been determined (by dragging, for instance),
			//		center the node. Otherwise, use the Dialog's stored relative offset,
			//		and position the node to top: left: values based on the viewport.
			if(!domClass.contains(this.ownerDocumentBody, "dojoMove")){	// don't do anything if called during auto-scroll
				var node = this.domNode,
					viewport = winUtils.getBox(this.ownerDocument),
					p = this._relativePosition,
					bb = p ? null : domGeometry.position(node),
					l = Math.floor(viewport.l + (p ? p.x : (viewport.w - bb.w) / 2)),
					t = Math.floor(viewport.t + (p ? p.y : (viewport.h - bb.h) / 2))
				;
				domStyle.set(node,{
					left: l + "px",
					top: t + "px"
				});
			}
		},

		_onKey: function(/*Event*/ evt){
			// summary:
			//		Handles the keyboard events for accessibility reasons
			// tags:
			//		private

			if(evt.charOrCode){
				var node = evt.target;
				if(evt.charOrCode === keys.TAB){
					this._getFocusItems(this.domNode);
				}
				var singleFocusItem = (this._firstFocusItem == this._lastFocusItem);
				// see if we are shift-tabbing from first focusable item on dialog
				if(node == this._firstFocusItem && evt.shiftKey && evt.charOrCode === keys.TAB){
					if(!singleFocusItem){
						focus.focus(this._lastFocusItem); // send focus to last item in dialog
					}
					event.stop(evt);
				}else if(node == this._lastFocusItem && evt.charOrCode === keys.TAB && !evt.shiftKey){
					if(!singleFocusItem){
						focus.focus(this._firstFocusItem); // send focus to first item in dialog
					}
					event.stop(evt);
				}else{
					// see if the key is for the dialog
					while(node){
						if(node == this.domNode || domClass.contains(node, "dijitPopup")){
							if(evt.charOrCode == keys.ESCAPE){
								this.onCancel();
							}else{
								return; // just let it go
							}
						}
						node = node.parentNode;
					}
					// this key is for the disabled document window
					if(evt.charOrCode !== keys.TAB){ // allow tabbing into the dialog for a11y
						event.stop(evt);
					// opera won't tab to a div
					}else if(!has("opera")){
						try{
							this._firstFocusItem.focus();
						}catch(e){ /*squelch*/ }
					}
				}
			}
		},

		show: function(){
			// summary:
			//		Display the dialog
			// returns: dojo/_base/Deferred
			//		Deferred object that resolves when the display animation is complete

			if(this.open){ return; }

			if(!this._started){
				this.startup();
			}

			// first time we show the dialog, there's some initialization stuff to do
			if(!this._alreadyInitialized){
				this._setup();
				this._alreadyInitialized=true;
			}

			if(this._fadeOutDeferred){
				// There's a hide() operation in progress, so cancel it, but still call DialogLevelManager.hide()
				// as though the hide() completed, in preparation for the DialogLevelManager.show() call below.
				this._fadeOutDeferred.cancel();
				DialogLevelManager.hide(this);
			}

			// Recenter Dialog if user scrolls browser.  Connecting to document doesn't work on IE, need to use window.
			var win = winUtils.get(this.ownerDocument);
			this._modalconnects.push(on(win, "scroll", lang.hitch(this, "resize")));

			this._modalconnects.push(on(this.domNode, connect._keypress, lang.hitch(this, "_onKey")));

			domStyle.set(this.domNode, {
				opacity:0,
				display:""
			});

			this._set("open", true);
			this._onShow(); // lazy load trigger

			this._size();
			this._position();

			// fade-in Animation object, setup below
			var fadeIn;

			this._fadeInDeferred = new Deferred(lang.hitch(this, function(){
				fadeIn.stop();
				delete this._fadeInDeferred;
			}));

			fadeIn = fx.fadeIn({
				node: this.domNode,
				duration: this.duration,
				beforeBegin: lang.hitch(this, function(){
					DialogLevelManager.show(this, this.underlayAttrs);
				}),
				onEnd: lang.hitch(this, function(){
					if(this.autofocus && DialogLevelManager.isTop(this)){
						// find focusable items each time dialog is shown since if dialog contains a widget the
						// first focusable items can change
						this._getFocusItems(this.domNode);
						focus.focus(this._firstFocusItem);
					}
					this._fadeInDeferred.resolve(true);
					delete this._fadeInDeferred;
				})
			}).play();

			return this._fadeInDeferred;
		},

		hide: function(){
			// summary:
			//		Hide the dialog
			// returns: dojo/_base/Deferred
			//		Deferred object that resolves when the hide animation is complete

			// If we haven't been initialized yet then we aren't showing and we can just return.
			// Likewise if we are already hidden, or are currently fading out.
			if(!this._alreadyInitialized || !this.open){
				return;
			}
			if(this._fadeInDeferred){
				this._fadeInDeferred.cancel();
			}

			// fade-in Animation object, setup below
			var fadeOut;

			this._fadeOutDeferred = new Deferred(lang.hitch(this, function(){
				fadeOut.stop();
				delete this._fadeOutDeferred;
			}));
			// fire onHide when the promise resolves.
			this._fadeOutDeferred.then(lang.hitch(this, 'onHide'));

			fadeOut = fx.fadeOut({
				node: this.domNode,
				duration: this.duration,
				onEnd: lang.hitch(this, function(){
					this.domNode.style.display = "none";
					DialogLevelManager.hide(this);
					this._fadeOutDeferred.resolve(true);
					delete this._fadeOutDeferred;
				})
			 }).play();

			if(this._scrollConnected){
				this._scrollConnected = false;
			}
			var h;
			while(h = this._modalconnects.pop()){
				h.remove();
			}

			if(this._relativePosition){
				delete this._relativePosition;
			}
			this._set("open", false);

			return this._fadeOutDeferred;
		},

		resize: function(){
			// summary:
			//		Called when viewport scrolled or size changed.  Position the Dialog and the underlay.
			// tags:
			//		private
			if(this.domNode.style.display != "none"){
				if(DialogUnderlay._singleton){	// avoid race condition during show()
					DialogUnderlay._singleton.layout();
				}
				if(!has("touch")){
					// If the user has scrolled the display then reposition the Dialog.  But don't do it for touch
					// devices, because it will counteract when a keyboard pops up and then the browser auto-scrolls
					// the focused node into view.
					this._position();
				}
				this._size();
			}
		},

		destroy: function(){
			if(this._fadeInDeferred){
				this._fadeInDeferred.cancel();
			}
			if(this._fadeOutDeferred){
				this._fadeOutDeferred.cancel();
			}
			if(this._moveable){
				this._moveable.destroy();
			}
			var h;
			while(h = this._modalconnects.pop()){
				h.remove();
			}

			DialogLevelManager.hide(this);

			this.inherited(arguments);
		}
	});

	var Dialog = declare("dijit.Dialog", [ContentPane, _DialogBase], {
		// summary:
		//		A modal dialog Widget.
		// description:
		//		Pops up a modal dialog window, blocking access to the screen
		//		and also graying out the screen Dialog is extended from
		//		ContentPane so it supports all the same parameters (href, etc.).
		// example:
		// |	<div data-dojo-type="dijit/Dialog" data-dojo-props="href: 'test.html'"></div>
		// example:
		// |	var foo = new Dialog({ title: "test dialog", content: "test content" };
		// |	foo.placeAt(win.body());
		// |	foo.startup();
	});
	Dialog._DialogBase = _DialogBase;	// for monkey patching and dojox/widget/DialogSimple

	var DialogLevelManager = Dialog._DialogLevelManager = {
		// summary:
		//		Controls the various active "levels" on the page, starting with the
		//		stuff initially visible on the page (at z-index 0), and then having an entry for
		//		each Dialog shown.

		_beginZIndex: 950,

		show: function(/*dijit/_WidgetBase*/ dialog, /*Object*/ underlayAttrs){
			// summary:
			//		Call right before fade-in animation for new dialog.
			//		Saves current focus, displays/adjusts underlay for new dialog,
			//		and sets the z-index of the dialog itself.
			//
			//		New dialog will be displayed on top of all currently displayed dialogs.
			//
			//		Caller is responsible for setting focus in new dialog after the fade-in
			//		animation completes.

			// Save current focus
			ds[ds.length-1].focus = focus.curNode;

			// Display the underlay, or if already displayed then adjust for this new dialog
			// TODO: one underlay per document (based on dialog.ownerDocument)
			var underlay = DialogUnderlay._singleton;
			if(!underlay || underlay._destroyed){
				underlay = dijit._underlay = DialogUnderlay._singleton = new DialogUnderlay(underlayAttrs);
			}else{
				underlay.set(dialog.underlayAttrs);
			}

			// Set z-index a bit above previous dialog
			var zIndex = ds[ds.length-1].dialog ? ds[ds.length-1].zIndex + 2 : Dialog._DialogLevelManager._beginZIndex;
			if(ds.length == 1){	// first dialog
				underlay.show();
			}
			domStyle.set(DialogUnderlay._singleton.domNode, 'zIndex', zIndex - 1);

			// Dialog
			domStyle.set(dialog.domNode, 'zIndex', zIndex);

			ds.push({dialog: dialog, underlayAttrs: underlayAttrs, zIndex: zIndex});
		},

		hide: function(/*dijit/_WidgetBase*/ dialog){
			// summary:
			//		Called when the specified dialog is hidden/destroyed, after the fade-out
			//		animation ends, in order to reset page focus, fix the underlay, etc.
			//		If the specified dialog isn't open then does nothing.
			//
			//		Caller is responsible for either setting display:none on the dialog domNode,
			//		or calling dijit/popup.hide(), or removing it from the page DOM.

			if(ds[ds.length-1].dialog == dialog){
				// Removing the top (or only) dialog in the stack, return focus
				// to previous dialog

				ds.pop();

				var pd = ds[ds.length-1];	// the new active dialog (or the base page itself)

				// Adjust underlay, unless the underlay widget has already been destroyed
				// because we are being called during page unload (when all widgets are destroyed)
				if(!DialogUnderlay._singleton._destroyed){
					if(ds.length == 1){
						// Returning to original page.  Hide the underlay.
						DialogUnderlay._singleton.hide();
					}else{
						// Popping back to previous dialog, adjust underlay.
						domStyle.set(DialogUnderlay._singleton.domNode, 'zIndex', pd.zIndex - 1);
						DialogUnderlay._singleton.set(pd.underlayAttrs);
					}
				}

				// Adjust focus
				if(dialog.refocus){
					// If we are returning control to a previous dialog but for some reason
					// that dialog didn't have a focused field, set focus to first focusable item.
					// This situation could happen if two dialogs appeared at nearly the same time,
					// since a dialog doesn't set it's focus until the fade-in is finished.
					var focus = pd.focus;
					if(pd.dialog && (!focus || !dom.isDescendant(focus, pd.dialog.domNode))){
						pd.dialog._getFocusItems(pd.dialog.domNode);
						focus = pd.dialog._firstFocusItem;
					}

					if(focus){
						// Refocus the button that spawned the Dialog.   This will fail in corner cases including
						// page unload on IE, because the dijit/form/Button that launched the Dialog may get destroyed
						// before this code runs.  (#15058)
						try{
							focus.focus();
						}catch(e){}
					}
				}
			}else{
				// Removing a dialog out of order (#9944, #10705).
				// Don't need to mess with underlay or z-index or anything.
				var idx = array.indexOf(array.map(ds, function(elem){return elem.dialog}), dialog);
				if(idx != -1){
					ds.splice(idx, 1);
				}
			}
		},

		isTop: function(/*dijit/_WidgetBase*/ dialog){
			// summary:
			//		Returns true if specified Dialog is the top in the task
			return ds[ds.length-1].dialog == dialog;
		}
	};

	// Stack representing the various active "levels" on the page, starting with the
	// stuff initially visible on the page (at z-index 0), and then having an entry for
	// each Dialog shown.
	// Each element in stack has form {
	//		dialog: dialogWidget,
	//		focus: returnFromGetFocus(),
	//		underlayAttrs: attributes to set on underlay (when this widget is active)
	// }
	var ds = Dialog._dialogStack = [
		{dialog: null, focus: null, underlayAttrs: null}	// entry for stuff at z-index: 0
	];

	// Back compat w/1.6, remove for 2.0
	if(has("dijit-legacy-requires")){
		ready(0, function(){
			var requires = ["dijit/TooltipDialog"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	return Dialog;
});

},
'dojo/dnd/Moveable':function(){
define("dojo/dnd/Moveable", [
	"../_base/array", "../_base/declare", "../_base/event", "../_base/lang",
	"../dom", "../dom-class", "../Evented", "../on", "../topic", "../touch", "./common", "./Mover", "../_base/window"
], function(array, declare, event, lang, dom, domClass, Evented, on, topic, touch, dnd, Mover, win){

// module:
//		dojo/dnd/Moveable


var Moveable = declare("dojo.dnd.Moveable", [Evented], {
	// summary:
	//		an object, which makes a node movable

	// object attributes (for markup)
	handle: "",
	delay: 0,
	skip: false,

	constructor: function(node, params){
		// node: Node
		//		a node (or node's id) to be moved
		// params: Moveable.__MoveableArgs?
		//		optional parameters
		this.node = dom.byId(node);
		if(!params){ params = {}; }
		this.handle = params.handle ? dom.byId(params.handle) : null;
		if(!this.handle){ this.handle = this.node; }
		this.delay = params.delay > 0 ? params.delay : 0;
		this.skip  = params.skip;
		this.mover = params.mover ? params.mover : Mover;
		this.events = [
			on(this.handle, touch.press, lang.hitch(this, "onMouseDown")),
			// cancel text selection and text dragging
			on(this.handle, "dragstart",   lang.hitch(this, "onSelectStart")),
			on(this.handle, "selectstart",   lang.hitch(this, "onSelectStart"))
		];
	},

	// markup methods
	markupFactory: function(params, node, Ctor){
		return new Ctor(node, params);
	},

	// methods
	destroy: function(){
		// summary:
		//		stops watching for possible move, deletes all references, so the object can be garbage-collected
		array.forEach(this.events, function(handle){ handle.remove(); });
		this.events = this.node = this.handle = null;
	},

	// mouse event processors
	onMouseDown: function(e){
		// summary:
		//		event processor for onmousedown/ontouchstart, creates a Mover for the node
		// e: Event
		//		mouse/touch event
		if(this.skip && dnd.isFormElement(e)){ return; }
		if(this.delay){
			this.events.push(
				on(this.handle, touch.move, lang.hitch(this, "onMouseMove")),
				on(this.handle, touch.release, lang.hitch(this, "onMouseUp"))
			);
			this._lastX = e.pageX;
			this._lastY = e.pageY;
		}else{
			this.onDragDetected(e);
		}
		event.stop(e);
	},
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove/ontouchmove, used only for delayed drags
		// e: Event
		//		mouse/touch event
		if(Math.abs(e.pageX - this._lastX) > this.delay || Math.abs(e.pageY - this._lastY) > this.delay){
			this.onMouseUp(e);
			this.onDragDetected(e);
		}
		event.stop(e);
	},
	onMouseUp: function(e){
		// summary:
		//		event processor for onmouseup, used only for delayed drags
		// e: Event
		//		mouse event
		for(var i = 0; i < 2; ++i){
			this.events.pop().remove();
		}
		event.stop(e);
	},
	onSelectStart: function(e){
		// summary:
		//		event processor for onselectevent and ondragevent
		// e: Event
		//		mouse event
		if(!this.skip || !dnd.isFormElement(e)){
			event.stop(e);
		}
	},

	// local events
	onDragDetected: function(/*Event*/ e){
		// summary:
		//		called when the drag is detected;
		//		responsible for creation of the mover
		new this.mover(this.node, e, this);
	},
	onMoveStart: function(/*Mover*/ mover){
		// summary:
		//		called before every move operation
		topic.publish("/dnd/move/start", mover);
		domClass.add(win.body(), "dojoMove");
		domClass.add(this.node, "dojoMoveItem");
	},
	onMoveStop: function(/*Mover*/ mover){
		// summary:
		//		called after every move operation
		topic.publish("/dnd/move/stop", mover);
		domClass.remove(win.body(), "dojoMove");
		domClass.remove(this.node, "dojoMoveItem");
	},
	onFirstMove: function(/*===== mover, e =====*/){
		// summary:
		//		called during the very first move notification;
		//		can be used to initialize coordinates, can be overwritten.
		// mover: Mover
		// e: Event

		// default implementation does nothing
	},
	onMove: function(mover, leftTop /*=====, e =====*/){
		// summary:
		//		called during every move notification;
		//		should actually move the node; can be overwritten.
		// mover: Mover
		// leftTop: Object
		// e: Event
		this.onMoving(mover, leftTop);
		var s = mover.node.style;
		s.left = leftTop.l + "px";
		s.top  = leftTop.t + "px";
		this.onMoved(mover, leftTop);
	},
	onMoving: function(/*===== mover, leftTop =====*/){
		// summary:
		//		called before every incremental move; can be overwritten.
		// mover: Mover
		// leftTop: Object

		// default implementation does nothing
	},
	onMoved: function(/*===== mover, leftTop =====*/){
		// summary:
		//		called after every incremental move; can be overwritten.
		// mover: Mover
		// leftTop: Object

		// default implementation does nothing
	}
});

/*=====
Moveable.__MoveableArgs = declare([], {
	// handle: Node||String
	//		A node (or node's id), which is used as a mouse handle.
	//		If omitted, the node itself is used as a handle.
	handle: null,

	// delay: Number
	//		delay move by this number of pixels
	delay: 0,

	// skip: Boolean
	//		skip move of form elements
	skip: false,

	// mover: Object
	//		a constructor of custom Mover
	mover: dnd.Mover
});
=====*/

return Moveable;
});

},
'dojo/touch':function(){
define("dojo/touch", ["./_base/kernel", "./aspect", "./dom", "./on", "./has", "./mouse", "./domReady", "./_base/window"],
function(dojo, aspect, dom, on, has, mouse, domReady, win){

	// module:
	//		dojo/touch

	var hasTouch = has("touch");

	// TODO: get iOS version from dojo/sniff after #15827 is fixed
	var ios4 = false;
	if(has("ios")){
		var ua = navigator.userAgent;
		var v = ua.match(/OS ([\d_]+)/) ? RegExp.$1 : "1";
		var os = parseFloat(v.replace(/_/, '.').replace(/_/g, ''));
		ios4 = os < 5;
	}

	// Time of most recent touchstart or touchmove event
	var lastTouch;

	function dualEvent(mouseType, touchType){
		// Returns synthetic event that listens for both the specified mouse event and specified touch event.
		// But ignore fake mouse events that were generated due to the user touching the screen.
		if(hasTouch){
			return function(node, listener){
				var handle1 = on(node, touchType, listener),
					handle2 = on(node, mouseType, function(evt){
						if(!lastTouch || (new Date()).getTime() > lastTouch + 1000){
							listener.call(this, evt);
						}
					});
				return {
					remove: function(){
						handle1.remove();
						handle2.remove();
					}
				};
			};
		}else{
			// Avoid creating listeners for touch events on performance sensitive older browsers like IE6
			return function(node, listener){
				return on(node, mouseType, listener);
			}
		}
	}

	var touchmove, hoveredNode;

	if(hasTouch){
		domReady(function(){
			// Keep track of currently hovered node
			hoveredNode = win.body();	// currently hovered node

			win.doc.addEventListener("touchstart", function(evt){
				lastTouch = (new Date()).getTime();

				// Precede touchstart event with touch.over event.  DnD depends on this.
				// Use addEventListener(cb, true) to run cb before any touchstart handlers on node run,
				// and to ensure this code runs even if the listener on the node does event.stop().
				var oldNode = hoveredNode;
				hoveredNode = evt.target;
				on.emit(oldNode, "dojotouchout", {
					target: oldNode,
					relatedTarget: hoveredNode,
					bubbles: true
				});
				on.emit(hoveredNode, "dojotouchover", {
					target: hoveredNode,
					relatedTarget: oldNode,
					bubbles: true
				});
			}, true);

			// Fire synthetic touchover and touchout events on nodes since the browser won't do it natively.
			on(win.doc, "touchmove", function(evt){
				lastTouch = (new Date()).getTime();

				var newNode = win.doc.elementFromPoint(
					evt.pageX - (ios4 ? 0 : win.global.pageXOffset), // iOS 4 expects page coords
					evt.pageY - (ios4 ? 0 : win.global.pageYOffset)
				);
				if(newNode && hoveredNode !== newNode){
					// touch out on the old node
					on.emit(hoveredNode, "dojotouchout", {
						target: hoveredNode,
						relatedTarget: newNode,
						bubbles: true
					});

					// touchover on the new node
					on.emit(newNode, "dojotouchover", {
						target: newNode,
						relatedTarget: hoveredNode,
						bubbles: true
					});

					hoveredNode = newNode;
				}
			});
		});

		// Define synthetic touch.move event that unlike the native touchmove, fires for the node the finger is
		// currently dragging over rather than the node where the touch started.
		touchmove = function(node, listener){
			return on(win.doc, "touchmove", function(evt){
				if(node === win.doc || dom.isDescendant(hoveredNode, node)){
					evt.target = hoveredNode;
					listener.call(this, evt);
				}
			});
		};
	}


	//device neutral events - touch.press|move|release|cancel/over/out
	var touch = {
		press: dualEvent("mousedown", "touchstart"),
		move: dualEvent("mousemove", touchmove),
		release: dualEvent("mouseup", "touchend"),
		cancel: dualEvent(mouse.leave, "touchcancel"),
		over: dualEvent("mouseover", "dojotouchover"),
		out: dualEvent("mouseout", "dojotouchout"),
		enter: mouse._eventHandler(dualEvent("mouseover","dojotouchover")),
		leave: mouse._eventHandler(dualEvent("mouseout", "dojotouchout"))
	};

	/*=====
	touch = {
		// summary:
		//		This module provides unified touch event handlers by exporting
		//		press, move, release and cancel which can also run well on desktop.
		//		Based on http://dvcs.w3.org/hg/webevents/raw-file/tip/touchevents.html
		//
		// example:
		//		Used with dojo.on
		//		|	define(["dojo/on", "dojo/touch"], function(on, touch){
		//		|		on(node, touch.press, function(e){});
		//		|		on(node, touch.move, function(e){});
		//		|		on(node, touch.release, function(e){});
		//		|		on(node, touch.cancel, function(e){});
		// example:
		//		Used with touch.* directly
		//		|	touch.press(node, function(e){});
		//		|	touch.move(node, function(e){});
		//		|	touch.release(node, function(e){});
		//		|	touch.cancel(node, function(e){});

		press: function(node, listener){
			// summary:
			//		Register a listener to 'touchstart'|'mousedown' for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		move: function(node, listener){
			// summary:
			//		Register a listener to 'touchmove'|'mousemove' for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		release: function(node, listener){
			// summary:
			//		Register a listener to 'touchend'|'mouseup' for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		cancel: function(node, listener){
			// summary:
			//		Register a listener to 'touchcancel'|'mouseleave' for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		over: function(node, listener){
			// summary:
			//		Register a listener to 'mouseover' or touch equivalent for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		out: function(node, listener){
			// summary:
			//		Register a listener to 'mouseout' or touch equivalent for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		enter: function(node, listener){
			// summary:
			//		Register a listener to mouse.enter or touch equivalent for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		leave: function(node, listener){
			// summary:
			//		Register a listener to mouse.leave or touch equivalent for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		}
	};
	=====*/

	 1  && (dojo.touch = touch);

	return touch;
});

},
'dojo/dnd/common':function(){
define("dojo/dnd/common", ["../_base/connect", "../_base/kernel", "../_base/lang", "../dom"],
	function(connect, kernel, lang, dom){

// module:
//		dojo/dnd/common

var exports = lang.getObject("dojo.dnd", true);
/*=====
// TODO: for 2.0, replace line above with this code.
var exports = {
	// summary:
	//		TODOC
};
=====*/

exports.getCopyKeyState = connect.isCopyKey;

exports._uniqueId = 0;
exports.getUniqueId = function(){
	// summary:
	//		returns a unique string for use with any DOM element
	var id;
	do{
		id = kernel._scopeName + "Unique" + (++exports._uniqueId);
	}while(dom.byId(id));
	return id;
};

exports._empty = {};

exports.isFormElement = function(/*Event*/ e){
	// summary:
	//		returns true if user clicked on a form element
	var t = e.target;
	if(t.nodeType == 3 /*TEXT_NODE*/){
		t = t.parentNode;
	}
	return " button textarea input select option ".indexOf(" " + t.tagName.toLowerCase() + " ") >= 0;	// Boolean
};

return exports;
});

},
'dojo/dnd/Mover':function(){
define("dojo/dnd/Mover", [
	"../_base/array", "../_base/declare", "../_base/event", "../_base/lang", "../sniff", "../_base/window",
	"../dom", "../dom-geometry", "../dom-style", "../Evented", "../on", "../touch", "./common", "./autoscroll"
], function(array, declare, event, lang, has, win, dom, domGeom, domStyle, Evented, on, touch, dnd, autoscroll){

// module:
//		dojo/dnd/Mover

return declare("dojo.dnd.Mover", [Evented], {
	// summary:
	//		an object which makes a node follow the mouse, or touch-drag on touch devices.
	//		Used as a default mover, and as a base class for custom movers.

	constructor: function(node, e, host){
		// node: Node
		//		a node (or node's id) to be moved
		// e: Event
		//		a mouse event, which started the move;
		//		only pageX and pageY properties are used
		// host: Object?
		//		object which implements the functionality of the move,
		//	 	and defines proper events (onMoveStart and onMoveStop)
		this.node = dom.byId(node);
		this.marginBox = {l: e.pageX, t: e.pageY};
		this.mouseButton = e.button;
		var h = (this.host = host), d = node.ownerDocument;
		this.events = [
			// At the start of a drag, onFirstMove is called, and then the following
			// listener is disconnected.
			on(d, touch.move, lang.hitch(this, "onFirstMove")),

			// These are called continually during the drag
			on(d, touch.move, lang.hitch(this, "onMouseMove")),

			// And these are called at the end of the drag
			on(d, touch.release,  lang.hitch(this, "onMouseUp")),

			// cancel text selection and text dragging
			on(d, "dragstart",   event.stop),
			on(d.body, "selectstart", event.stop)
		];

		// Tell autoscroll that a drag is starting
		autoscroll.autoScrollStart(d);

		// notify that the move has started
		if(h && h.onMoveStart){
			h.onMoveStart(this);
		}
	},
	// mouse event processors
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove/ontouchmove
		// e: Event
		//		mouse/touch event
		autoscroll.autoScroll(e);
		var m = this.marginBox;
		this.host.onMove(this, {l: m.l + e.pageX, t: m.t + e.pageY}, e);
		event.stop(e);
	},
	onMouseUp: function(e){
		if(has("webkit") && has("mac") && this.mouseButton == 2 ?
				e.button == 0 : this.mouseButton == e.button){ // TODO Should condition be met for touch devices, too?
			this.destroy();
		}
		event.stop(e);
	},
	// utilities
	onFirstMove: function(e){
		// summary:
		//		makes the node absolute; it is meant to be called only once.
		//		relative and absolutely positioned nodes are assumed to use pixel units
		var s = this.node.style, l, t, h = this.host;
		switch(s.position){
			case "relative":
			case "absolute":
				// assume that left and top values are in pixels already
				l = Math.round(parseFloat(s.left)) || 0;
				t = Math.round(parseFloat(s.top)) || 0;
				break;
			default:
				s.position = "absolute";	// enforcing the absolute mode
				var m = domGeom.getMarginBox(this.node);
				// event.pageX/pageY (which we used to generate the initial
				// margin box) includes padding and margin set on the body.
				// However, setting the node's position to absolute and then
				// doing domGeom.marginBox on it *doesn't* take that additional
				// space into account - so we need to subtract the combined
				// padding and margin.  We use getComputedStyle and
				// _getMarginBox/_getContentBox to avoid the extra lookup of
				// the computed style.
				var b = win.doc.body;
				var bs = domStyle.getComputedStyle(b);
				var bm = domGeom.getMarginBox(b, bs);
				var bc = domGeom.getContentBox(b, bs);
				l = m.l - (bc.l - bm.l);
				t = m.t - (bc.t - bm.t);
				break;
		}
		this.marginBox.l = l - this.marginBox.l;
		this.marginBox.t = t - this.marginBox.t;
		if(h && h.onFirstMove){
			h.onFirstMove(this, e);
		}

		// Disconnect touch.move that call this function
		this.events.shift().remove();
	},
	destroy: function(){
		// summary:
		//		stops the move, deletes all references, so the object can be garbage-collected
		array.forEach(this.events, function(handle){ handle.remove(); });
		// undo global settings
		var h = this.host;
		if(h && h.onMoveStop){
			h.onMoveStop(this);
		}
		// destroy objects
		this.events = this.node = this.host = null;
	}
});

});

},
'dojo/dnd/autoscroll':function(){
define("dojo/dnd/autoscroll", ["../_base/lang", "../sniff", "../_base/window", "../dom-geometry", "../dom-style", "../window"],
	function(lang, has, win, domGeom, domStyle, winUtils){

// module:
//		dojo/dnd/autoscroll

var exports = {
	// summary:
	//		Used by dojo/dnd/Manager to scroll document or internal node when the user
	//		drags near the edge of the viewport or a scrollable node
};
lang.setObject("dojo.dnd.autoscroll", exports);

exports.getViewport = winUtils.getBox;

exports.V_TRIGGER_AUTOSCROLL = 32;
exports.H_TRIGGER_AUTOSCROLL = 32;

exports.V_AUTOSCROLL_VALUE = 16;
exports.H_AUTOSCROLL_VALUE = 16;

// These are set by autoScrollStart().
// Set to default values in case autoScrollStart() isn't called. (back-compat, remove for 2.0)
var viewport,
	doc = win.doc,
	maxScrollTop = Infinity,
	maxScrollLeft = Infinity;

exports.autoScrollStart = function(d){
	// summary:
	//		Called at the start of a drag.
	// d: Document
	//		The document of the node being dragged.

	doc = d;
	viewport = winUtils.getBox(doc);

	// Save height/width of document at start of drag, before it gets distorted by a user dragging an avatar past
	// the document's edge
	var html = win.body(doc).parentNode;
	maxScrollTop = Math.max(html.scrollHeight - viewport.h, 0);
	maxScrollLeft = Math.max(html.scrollWidth - viewport.w, 0);	// usually 0
};

exports.autoScroll = function(e){
	// summary:
	//		a handler for mousemove and touchmove events, which scrolls the window, if
	//		necessary
	// e: Event
	//		mousemove/touchmove event

	// FIXME: needs more docs!
	var v = viewport || winUtils.getBox(doc), // getBox() call for back-compat, in case autoScrollStart() wasn't called
		html = win.body(doc).parentNode,
		dx = 0, dy = 0;
	if(e.clientX < exports.H_TRIGGER_AUTOSCROLL){
		dx = -exports.H_AUTOSCROLL_VALUE;
	}else if(e.clientX > v.w - exports.H_TRIGGER_AUTOSCROLL){
		dx = Math.min(exports.H_AUTOSCROLL_VALUE, maxScrollLeft - html.scrollLeft);	// don't scroll past edge of doc
	}
	if(e.clientY < exports.V_TRIGGER_AUTOSCROLL){
		dy = -exports.V_AUTOSCROLL_VALUE;
	}else if(e.clientY > v.h - exports.V_TRIGGER_AUTOSCROLL){
		dy = Math.min(exports.V_AUTOSCROLL_VALUE, maxScrollTop - html.scrollTop);	// don't scroll past edge of doc
	}
	window.scrollBy(dx, dy);
};

exports._validNodes = {"div": 1, "p": 1, "td": 1};
exports._validOverflow = {"auto": 1, "scroll": 1};

exports.autoScrollNodes = function(e){
	// summary:
	//		a handler for mousemove and touchmove events, which scrolls the first available
	//		Dom element, it falls back to exports.autoScroll()
	// e: Event
	//		mousemove/touchmove event

	// FIXME: needs more docs!

	var b, t, w, h, rx, ry, dx = 0, dy = 0, oldLeft, oldTop;

	for(var n = e.target; n;){
		if(n.nodeType == 1 && (n.tagName.toLowerCase() in exports._validNodes)){
			var s = domStyle.getComputedStyle(n),
				overflow = (s.overflow.toLowerCase() in exports._validOverflow),
				overflowX = (s.overflowX.toLowerCase() in exports._validOverflow),
				overflowY = (s.overflowY.toLowerCase() in exports._validOverflow);
			if(overflow || overflowX || overflowY){
				b = domGeom.getContentBox(n, s);
				t = domGeom.position(n, true);
			}
			// overflow-x
			if(overflow || overflowX){
				w = Math.min(exports.H_TRIGGER_AUTOSCROLL, b.w / 2);
				rx = e.pageX - t.x;
				if(has("webkit") || has("opera")){
					// FIXME: this code should not be here, it should be taken into account
					// either by the event fixing code, or the domGeom.position()
					// FIXME: this code doesn't work on Opera 9.5 Beta
					rx += win.body().scrollLeft;
				}
				dx = 0;
				if(rx > 0 && rx < b.w){
					if(rx < w){
						dx = -w;
					}else if(rx > b.w - w){
						dx = w;
					}
					oldLeft = n.scrollLeft;
					n.scrollLeft = n.scrollLeft + dx;
				}
			}
			// overflow-y
			if(overflow || overflowY){
				//console.log(b.l, b.t, t.x, t.y, n.scrollLeft, n.scrollTop);
				h = Math.min(exports.V_TRIGGER_AUTOSCROLL, b.h / 2);
				ry = e.pageY - t.y;
				if(has("webkit") || has("opera")){
					// FIXME: this code should not be here, it should be taken into account
					// either by the event fixing code, or the domGeom.position()
					// FIXME: this code doesn't work on Opera 9.5 Beta
					ry += win.body().scrollTop;
				}
				dy = 0;
				if(ry > 0 && ry < b.h){
					if(ry < h){
						dy = -h;
					}else if(ry > b.h - h){
						dy = h;
					}
					oldTop = n.scrollTop;
					n.scrollTop  = n.scrollTop  + dy;
				}
			}
			if(dx || dy){ return; }
		}
		try{
			n = n.parentNode;
		}catch(x){
			n = null;
		}
	}
	exports.autoScroll(e);
};

return exports;

});

},
'dojo/dnd/TimedMoveable':function(){
define("dojo/dnd/TimedMoveable", ["../_base/declare", "./Moveable" /*=====, "./Mover" =====*/], function(declare, Moveable /*=====, Mover =====*/){
	// module:
	//		dojo/dnd/TimedMoveable

	/*=====
	var __TimedMoveableArgs = declare([Moveable.__MoveableArgs], {
		// timeout: Number
		//		delay move by this number of ms,
		//		accumulating position changes during the timeout
		timeout: 0
	});
	=====*/

	// precalculate long expressions
	var oldOnMove = Moveable.prototype.onMove;

	return declare("dojo.dnd.TimedMoveable", Moveable, {
		// summary:
		//		A specialized version of Moveable to support an FPS throttling.
		//		This class puts an upper restriction on FPS, which may reduce
		//		the CPU load. The additional parameter "timeout" regulates
		//		the delay before actually moving the moveable object.

		// object attributes (for markup)
		timeout: 40,	// in ms, 40ms corresponds to 25 fps

		constructor: function(node, params){
			// summary:
			//		an object that makes a node moveable with a timer
			// node: Node||String
			//		a node (or node's id) to be moved
			// params: __TimedMoveableArgs
			//		object with additional parameters.

			// sanitize parameters
			if(!params){ params = {}; }
			if(params.timeout && typeof params.timeout == "number" && params.timeout >= 0){
				this.timeout = params.timeout;
			}
		},

		onMoveStop: function(/*Mover*/ mover){
			if(mover._timer){
				// stop timer
				clearTimeout(mover._timer);
				// reflect the last received position
				oldOnMove.call(this, mover, mover._leftTop);
			}
			Moveable.prototype.onMoveStop.apply(this, arguments);
		},
		onMove: function(/*Mover*/ mover, /*Object*/ leftTop){
			mover._leftTop = leftTop;
			if(!mover._timer){
				var _t = this;	// to avoid using dojo.hitch()
				mover._timer = setTimeout(function(){
					// we don't have any pending requests
					mover._timer = null;
					// reflect the last received position
					oldOnMove.call(_t, mover, mover._leftTop);
				}, this.timeout);
			}
		}
	});
});

},
'dijit/_base/manager':function(){
define("dijit/_base/manager", [
	"dojo/_base/array",
	"dojo/_base/config", // defaultDuration
	"dojo/_base/lang",
	"../registry",
	"../main"	// for setting exports to dijit namespace
], function(array, config, lang, registry, dijit){

	// module:
	//		dijit/_base/manager

	var exports = {
		// summary:
		//		Deprecated.  Shim to methods on registry, plus a few other declarations.
		//		New code should access dijit/registry directly when possible.
	};

	array.forEach(["byId", "getUniqueId", "findWidgets", "_destroyAll", "byNode", "getEnclosingWidget"], function(name){
		exports[name] = registry[name];
	});

	 lang.mixin(exports, {
		 // defaultDuration: Integer
		 //		The default fx.animation speed (in ms) to use for all Dijit
		 //		transitional fx.animations, unless otherwise specified
		 //		on a per-instance basis. Defaults to 200, overrided by
		 //		`djConfig.defaultDuration`
		 defaultDuration: config["defaultDuration"] || 200
	 });

	lang.mixin(dijit, exports);

	/*===== return exports; =====*/
	return dijit;	// for back compat :-(
});

},
'dijit/_Widget':function(){
define("dijit/_Widget", [
	"dojo/aspect",	// aspect.around
	"dojo/_base/config",	// config.isDebug
	"dojo/_base/connect",	// connect.connect
	"dojo/_base/declare", // declare
	"dojo/has",
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.hitch
	"dojo/query",
	"dojo/ready",
	"./registry",	// registry.byNode
	"./_WidgetBase",
	"./_OnDijitClickMixin",
	"./_FocusMixin",
	"dojo/uacss",		// browser sniffing (included for back-compat; subclasses may be using)
	"./hccss"		// high contrast mode sniffing (included to set CSS classes on <body>, module ret value unused)
], function(aspect, config, connect, declare, has, kernel, lang, query, ready,
			registry, _WidgetBase, _OnDijitClickMixin, _FocusMixin){


// module:
//		dijit/_Widget


function connectToDomNode(){
	// summary:
	//		If user connects to a widget method === this function, then they will
	//		instead actually be connecting the equivalent event on this.domNode
}

// Trap dojo.connect() calls to connectToDomNode methods, and redirect to _Widget.on()
function aroundAdvice(originalConnect){
	return function(obj, event, scope, method){
		if(obj && typeof event == "string" && obj[event] == connectToDomNode){
			return obj.on(event.substring(2).toLowerCase(), lang.hitch(scope, method));
		}
		return originalConnect.apply(connect, arguments);
	};
}
aspect.around(connect, "connect", aroundAdvice);
if(kernel.connect){
	aspect.around(kernel, "connect", aroundAdvice);
}

var _Widget = declare("dijit._Widget", [_WidgetBase, _OnDijitClickMixin, _FocusMixin], {
	// summary:
	//		Old base class for widgets.   New widgets should extend `dijit/_WidgetBase` instead
	// description:
	//		Old Base class for Dijit widgets.
	//
	//		Extends _WidgetBase, adding support for:
	//
	//		- declaratively/programatically specifying widget initialization parameters like
	//			onMouseMove="foo" that call foo when this.domNode gets a mousemove event
	//		- ondijitclick:
	//			Support new data-dojo-attach-event="ondijitclick: ..." that is triggered by a mouse click or a SPACE/ENTER keypress
	//		- focus related functions:
	//			In particular, the onFocus()/onBlur() callbacks.   Driven internally by
	//			dijit/_base/focus.js.
	//		- deprecated methods
	//		- onShow(), onHide(), onClose()
	//
	//		Also, by loading code in dijit/_base, turns on:
	//
	//		- browser sniffing (putting browser class like `dj_ie` on `<html>` node)
	//		- high contrast mode sniffing (add `dijit_a11y` class to `<body>` if machine is in high contrast mode)


	////////////////// DEFERRED CONNECTS ///////////////////

	onClick: connectToDomNode,
	/*=====
	onClick: function(event){
		// summary:
		//		Connect to this function to receive notifications of mouse click events.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onDblClick: connectToDomNode,
	/*=====
	onDblClick: function(event){
		// summary:
		//		Connect to this function to receive notifications of mouse double click events.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onKeyDown: connectToDomNode,
	/*=====
	onKeyDown: function(event){
		// summary:
		//		Connect to this function to receive notifications of keys being pressed down.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onKeyPress: connectToDomNode,
	/*=====
	onKeyPress: function(event){
		// summary:
		//		Connect to this function to receive notifications of printable keys being typed.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onKeyUp: connectToDomNode,
	/*=====
	onKeyUp: function(event){
		// summary:
		//		Connect to this function to receive notifications of keys being released.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onMouseDown: connectToDomNode,
	/*=====
	onMouseDown: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse button is pressed down.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseMove: connectToDomNode,
	/*=====
	onMouseMove: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves over nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseOut: connectToDomNode,
	/*=====
	onMouseOut: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves off of nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseOver: connectToDomNode,
	/*=====
	onMouseOver: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves onto nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseLeave: connectToDomNode,
	/*=====
	onMouseLeave: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves off of this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseEnter: connectToDomNode,
	/*=====
	onMouseEnter: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves onto this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseUp: connectToDomNode,
	/*=====
	onMouseUp: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse button is released.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/

	constructor: function(params /*===== ,srcNodeRef =====*/){
		// summary:
		//		Create the widget.
		// params: Object|null
		//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
		//		and functions, typically callbacks like onClick.
		//		The hash can contain any of the widget's properties, excluding read-only properties.
		// srcNodeRef: DOMNode|String?
		//		If a srcNodeRef (DOM node) is specified:
		//
		//		- use srcNodeRef.innerHTML as my contents
		//		- if this is a behavioral widget then apply behavior to that srcNodeRef
		//		- otherwise, replace srcNodeRef with my generated DOM tree

		// extract parameters like onMouseMove that should connect directly to this.domNode
		this._toConnect = {};
		for(var name in params){
			if(this[name] === connectToDomNode){
				this._toConnect[name.replace(/^on/, "").toLowerCase()] = params[name];
				delete params[name];
			}
		}
	},

	postCreate: function(){
		this.inherited(arguments);

		// perform connection from this.domNode to user specified handlers (ex: onMouseMove)
		for(var name in this._toConnect){
			this.on(name, this._toConnect[name]);
		}
		delete this._toConnect;
	},

	on: function(/*String|Function*/ type, /*Function*/ func){
		if(this[this._onMap(type)] === connectToDomNode){
			// Use connect.connect() rather than on() to get handling for "onmouseenter" on non-IE,
			// normalization of onkeypress/onkeydown to behave like firefox, etc.
			// Also, need to specify context as "this" rather than the default context of the DOMNode
			// Remove in 2.0.
			return connect.connect(this.domNode, type.toLowerCase(), this, func);
		}
		return this.inherited(arguments);
	},

	_setFocusedAttr: function(val){
		// Remove this method in 2.0 (or sooner), just here to set _focused == focused, for back compat
		// (but since it's a private variable we aren't required to keep supporting it).
		this._focused = val;
		this._set("focused", val);
	},

	////////////////// DEPRECATED METHODS ///////////////////

	setAttribute: function(/*String*/ attr, /*anything*/ value){
		// summary:
		//		Deprecated.  Use set() instead.
		// tags:
		//		deprecated
		kernel.deprecated(this.declaredClass+"::setAttribute(attr, value) is deprecated. Use set() instead.", "", "2.0");
		this.set(attr, value);
	},

	attr: function(/*String|Object*/name, /*Object?*/value){
		// summary:
		//		Set or get properties on a widget instance.
		// name:
		//		The property to get or set. If an object is passed here and not
		//		a string, its keys are used as names of attributes to be set
		//		and the value of the object as values to set in the widget.
		// value:
		//		Optional. If provided, attr() operates as a setter. If omitted,
		//		the current value of the named property is returned.
		// description:
		//		This method is deprecated, use get() or set() directly.

		// Print deprecation warning but only once per calling function
		if(config.isDebug){
			var alreadyCalledHash = arguments.callee._ach || (arguments.callee._ach = {}),
				caller = (arguments.callee.caller || "unknown caller").toString();
			if(!alreadyCalledHash[caller]){
				kernel.deprecated(this.declaredClass + "::attr() is deprecated. Use get() or set() instead, called from " +
				caller, "", "2.0");
				alreadyCalledHash[caller] = true;
			}
		}

		var args = arguments.length;
		if(args >= 2 || typeof name === "object"){ // setter
			return this.set.apply(this, arguments);
		}else{ // getter
			return this.get(name);
		}
	},

	getDescendants: function(){
		// summary:
		//		Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.
		//		This method should generally be avoided as it returns widgets declared in templates, which are
		//		supposed to be internal/hidden, but it's left here for back-compat reasons.

		kernel.deprecated(this.declaredClass+"::getDescendants() is deprecated. Use getChildren() instead.", "", "2.0");
		return this.containerNode ? query('[widgetId]', this.containerNode).map(registry.byNode) : []; // dijit/_WidgetBase[]
	},

	////////////////// MISCELLANEOUS METHODS ///////////////////

	_onShow: function(){
		// summary:
		//		Internal method called when this widget is made visible.
		//		See `onShow` for details.
		this.onShow();
	},

	onShow: function(){
		// summary:
		//		Called when this widget becomes the selected pane in a
		//		`dijit/layout/TabContainer`, `dijit/layout/StackContainer`,
		//		`dijit/layout/AccordionContainer`, etc.
		//
		//		Also called to indicate display of a `dijit.Dialog`, `dijit.TooltipDialog`, or `dijit.TitlePane`.
		// tags:
		//		callback
	},

	onHide: function(){
		// summary:
		//		Called when another widget becomes the selected pane in a
		//		`dijit/layout/TabContainer`, `dijit/layout/StackContainer`,
		//		`dijit/layout/AccordionContainer`, etc.
		//
		//		Also called to indicate hide of a `dijit.Dialog`, `dijit.TooltipDialog`, or `dijit.TitlePane`.
		// tags:
		//		callback
	},

	onClose: function(){
		// summary:
		//		Called when this widget is being displayed as a popup (ex: a Calendar popped
		//		up from a DateTextBox), and it is hidden.
		//		This is called from the dijit.popup code, and should not be called directly.
		//
		//		Also used as a parameter for children of `dijit/layout/StackContainer` or subclasses.
		//		Callback if a user tries to close the child.   Child will be closed if this function returns true.
		// tags:
		//		extension

		return true;		// Boolean
	}
});

// For back-compat, remove in 2.0.
if(has("dijit-legacy-requires")){
	ready(0, function(){
		var requires = ["dijit/_base"];
		require(requires);	// use indirection so modules not rolled into a build
	});
}
return _Widget;
});

},
'dijit/_OnDijitClickMixin':function(){
define("dijit/_OnDijitClickMixin", [
	"dojo/on",
	"dojo/_base/array", // array.forEach
	"dojo/keys", // keys.ENTER keys.SPACE
	"dojo/_base/declare", // declare
	"dojo/has", // has("dom-addeventlistener")
	"dojo/_base/unload", // unload.addOnWindowUnload
	"dojo/_base/window", // win.doc.addEventListener win.doc.attachEvent win.doc.detachEvent
	"./a11yclick"
], function(on, array, keys, declare, has, unload, win, a11yclick){

	// module:
	//		dijit/_OnDijitClickMixin

	var ret = declare("dijit._OnDijitClickMixin", null, {
		connect: function(
				/*Object|null*/ obj,
				/*String|Function*/ event,
				/*String|Function*/ method){
			// summary:
			//		Connects specified obj/event to specified method of this object
			//		and registers for disconnect() on widget destroy.
			// description:
			//		Provide widget-specific analog to connect.connect, except with the
			//		implicit use of this widget as the target object.
			//		This version of connect also provides a special "ondijitclick"
			//		event which triggers on a click or space or enter keyup.
			//		Events connected with `this.connect` are disconnected upon
			//		destruction.
			// returns:
			//		A handle that can be passed to `disconnect` in order to disconnect before
			//		the widget is destroyed.
			// example:
			//	|	var btn = new Button();
			//	|	// when foo.bar() is called, call the listener we're going to
			//	|	// provide in the scope of btn
			//	|	btn.connect(foo, "bar", function(){
			//	|		console.debug(this.toString());
			//	|	});
			// tags:
			//		protected

			return this.inherited(arguments, [obj, event == "ondijitclick" ? a11yclick : event, method]);
		}
	});

	ret.a11yclick = a11yclick;	// back compat

	return ret;
});

},
'dijit/a11yclick':function(){
define("dijit/a11yclick", [
	"dojo/on",
	"dojo/_base/array", // array.forEach
	"dojo/keys", // keys.ENTER keys.SPACE
	"dojo/_base/declare", // declare
	"dojo/has", // has("dom-addeventlistener")
	"dojo/_base/unload", // unload.addOnWindowUnload
	"dojo/_base/window" // win.doc.addEventListener win.doc.attachEvent win.doc.detachEvent
], function(on, array, keys, declare, has, unload, win){

	// module:
	//		dijit/a11yclick

	// Keep track of where the last keydown event was, to help avoid generating
	// spurious ondijitclick events when:
	// 1. focus is on a <button> or <a>
	// 2. user presses then releases the ENTER key
	// 3. onclick handler fires and shifts focus to another node, with an ondijitclick handler
	// 4. onkeyup event fires, causing the ondijitclick handler to fire
	var lastKeyDownNode = null;
	if(has("dom-addeventlistener")){
		win.doc.addEventListener('keydown', function(evt){
			lastKeyDownNode = evt.target;
		}, true);
	}else{
		// Fallback path for IE6-8
		(function(){
			var keydownCallback = function(evt){
				lastKeyDownNode = evt.srcElement;
			};
			win.doc.attachEvent('onkeydown', keydownCallback);
			unload.addOnWindowUnload(function(){
				win.doc.detachEvent('onkeydown', keydownCallback);
			});
		})();
	}

	function clickKey(/*Event*/ e){
		return (e.keyCode === keys.ENTER || e.keyCode === keys.SPACE) &&
			!e.ctrlKey && !e.shiftKey && !e.altKey && !e.metaKey;
	}

	return function(node, listener){
		// summary:
		//		Custom a11yclick (a.k.a. ondijitclick) event
		//		which triggers on a mouse click, touch, or space/enter keyup.

		if(/input|button/i.test(node.nodeName)){
			// pass through, the browser already generates click event on SPACE/ENTER key
			return on(node, "click", listener);
		}else{
			// Don't fire the click event unless both the keydown and keyup occur on this node.
			// Avoids problems where focus shifted to this node or away from the node on keydown,
			// either causing this node to process a stray keyup event, or causing another node
			// to get a stray keyup event.

			var handles = [
				on(node, "keydown", function(e){
					//console.log(this.id + ": onkeydown, e.target = ", e.target, ", lastKeyDownNode was ", lastKeyDownNode, ", equality is ", (e.target === lastKeyDownNode));
					if(clickKey(e)){
						// needed on IE for when focus changes between keydown and keyup - otherwise dropdown menus do not work
						lastKeyDownNode = e.target;

						// Prevent viewport scrolling on space key in IE<9.
						// (Reproducible on test_Button.html on any of the first dijit/form/Button examples)
						e.preventDefault();
					}
				}),

				on(node, "keyup", function(e){
					//console.log(this.id + ": onkeyup, e.target = ", e.target, ", lastKeyDownNode was ", lastKeyDownNode, ", equality is ", (e.target === lastKeyDownNode));
					if(clickKey(e) && e.target == lastKeyDownNode){	// === breaks greasemonkey
						//need reset here or have problems in FF when focus returns to trigger element after closing popup/alert
						lastKeyDownNode = null;
						on.emit(e.target, "click", {
							cancelable: true,
							bubbles: true
						});
					}
				}),

				on(node, "click", function(e){
					// catch mouse clicks, plus the on.emit() calls from above and below
					listener.call(this, e);
				})
			];

			if(has("touch")){
				// touchstart-->touchend will automatically generate a click event, but there are problems
				// on iOS after focus has been programatically shifted (#14604, #14918), so setup a failsafe
				// if click doesn't fire naturally.

				var clickTimer;
				handles.push(
					on(node, "touchend", function(e){
						var target = e.target;
						clickTimer = setTimeout(function(){
							clickTimer = null;
							on.emit(target, "click", {
								cancelable: true,
								bubbles: true
							});
						}, 600);
					}),
					on(node, "click", function(e){
						// If browser generates a click naturally, clear the timer to fire a synthetic click event
						if(clickTimer){
							clearTimeout(clickTimer);
						}
					})
					// TODO: if the touchstart and touchend were <100ms apart, and then there's another touchstart
					// event <300ms after the touchend event, then clear the synthetic click timer, because user
					// is doing a zoom.   Alternately monitor screen.deviceXDPI (or something similar) to see if
					// zoom level has changed.
				);
			}

			return {
				remove: function(){
					array.forEach(handles, function(h){ h.remove(); });
					if(clickTimer){
						clearTimeout(clickTimer);
						clickTimer = null;
					}
				}
			};
		}
	};

	return ret;
});

},
'dijit/_FocusMixin':function(){
define("dijit/_FocusMixin", [
	"./focus",
	"./_WidgetBase",
	"dojo/_base/declare", // declare
	"dojo/_base/lang" // lang.extend
], function(focus, _WidgetBase, declare, lang){

	// module:
	//		dijit/_FocusMixin

	// We don't know where _FocusMixin will occur in the inheritance chain, but we need the _onFocus()/_onBlur() below
	// to be last in the inheritance chain, so mixin to _WidgetBase.
	lang.extend(_WidgetBase, {
		// focused: [readonly] Boolean
		//		This widget or a widget it contains has focus, or is "active" because
		//		it was recently clicked.
		focused: false,

		onFocus: function(){
			// summary:
			//		Called when the widget becomes "active" because
			//		it or a widget inside of it either has focus, or has recently
			//		been clicked.
			// tags:
			//		callback
		},

		onBlur: function(){
			// summary:
			//		Called when the widget stops being "active" because
			//		focus moved to something outside of it, or the user
			//		clicked somewhere outside of it, or the widget was
			//		hidden.
			// tags:
			//		callback
		},

		_onFocus: function(){
			// summary:
			//		This is where widgets do processing for when they are active,
			//		such as changing CSS classes.  See onFocus() for more details.
			// tags:
			//		protected
			this.onFocus();
		},

		_onBlur: function(){
			// summary:
			//		This is where widgets do processing for when they stop being active,
			//		such as changing CSS classes.  See onBlur() for more details.
			// tags:
			//		protected
			this.onBlur();
		}
	});

	return declare("dijit._FocusMixin", null, {
		// summary:
		//		Mixin to widget to provide _onFocus() and _onBlur() methods that
		//		fire when a widget or its descendants get/lose focus

		// flag that I want _onFocus()/_onBlur() notifications from focus manager
		_focusManager: focus
	});

});

},
'dojo/uacss':function(){
define("dojo/uacss", ["./dom-geometry", "./_base/lang", "./ready", "./sniff", "./_base/window"],
	function(geometry, lang, ready, has, baseWindow){

	// module:
	//		dojo/uacss

	/*=====
	return {
		// summary:
		//		Applies pre-set CSS classes to the top-level HTML node, based on:
		//
		//		- browser (ex: dj_ie)
		//		- browser version (ex: dj_ie6)
		//		- box model (ex: dj_contentBox)
		//		- text direction (ex: dijitRtl)
		//
		//		In addition, browser, browser version, and box model are
		//		combined with an RTL flag when browser text is RTL. ex: dj_ie-rtl.
		//
		//		Returns the has() method.
	};
	=====*/

	var
		html = baseWindow.doc.documentElement,
		ie = has("ie"),
		opera = has("opera"),
		maj = Math.floor,
		ff = has("ff"),
		boxModel = geometry.boxModel.replace(/-/,''),

		classes = {
			"dj_quirks": has("quirks"),

			// NOTE: Opera not supported by dijit
			"dj_opera": opera,

			"dj_khtml": has("khtml"),

			"dj_webkit": has("webkit"),
			"dj_safari": has("safari"),
			"dj_chrome": has("chrome"),

			"dj_gecko": has("mozilla")
		}; // no dojo unsupported browsers

	if(ie){
		classes["dj_ie"] = true;
		classes["dj_ie" + maj(ie)] = true;
		classes["dj_iequirks"] = has("quirks");
	}
	if(ff){
		classes["dj_ff" + maj(ff)] = true;
	}

	classes["dj_" + boxModel] = true;

	// apply browser, browser version, and box model class names
	var classStr = "";
	for(var clz in classes){
		if(classes[clz]){
			classStr += clz + " ";
		}
	}
	html.className = lang.trim(html.className + " " + classStr);

	// If RTL mode, then add dj_rtl flag plus repeat existing classes with -rtl extension.
	// We can't run the code below until the <body> tag has loaded (so we can check for dir=rtl).
	// priority is 90 to run ahead of parser priority of 100
	ready(90, function(){
		if(!geometry.isBodyLtr()){
			var rtlClassStr = "dj_rtl dijitRtl " + classStr.replace(/ /g, "-rtl ");
			html.className = lang.trim(html.className + " " + rtlClassStr + "dj_rtl dijitRtl " + classStr.replace(/ /g, "-rtl "));
		}
	});
	return has;
});

},
'dijit/hccss':function(){
define("dijit/hccss", ["dojo/dom-class", "dojo/hccss", "dojo/ready", "dojo/_base/window"], function(domClass, has, ready, win){

	// module:
	//		dijit/hccss

	/*=====
	return function(){
		// summary:
		//		Test if computer is in high contrast mode, and sets `dijit_a11y` flag on `<body>` if it is.
		//		Deprecated, use ``dojo/hccss`` instead.
	};
	=====*/

	// Priority is 90 to run ahead of parser priority of 100.   For 2.0, remove the ready() call and instead
	// change this module to depend on dojo/domReady!
	ready(90, function(){
		if(has("highcontrast")){
			domClass.add(win.body(), "dijit_a11y");
		}
	});

	return has;
});

},
'dojo/hccss':function(){
define("dojo/hccss", [
	"require",			// require.toUrl
	"./_base/config", // config.blankGif
	"./dom-class", // domClass.add
	"./dom-style", // domStyle.getComputedStyle
	"./has",
	"./ready", // ready
	"./_base/window" // win.body
], function(require, config, domClass, domStyle, has, ready, win){

	// module:
	//		dojo/hccss

	/*=====
	return function(){
		// summary:
		//		Test if computer is in high contrast mode (i.e. if browser is not displaying background images).
		//		Defines `has("highcontrast")` and sets `dj_a11y` CSS class on `<body>` if machine is in high contrast mode.
		//		Returns `has()` method;
	};
	=====*/

	// Has() test for when background images aren't displayed.  Don't call has("highcontrast") before dojo/domReady!.
	has.add("highcontrast", function(){
		// note: if multiple documents, doesn't matter which one we use
		var div = win.doc.createElement("div");
		div.style.cssText = "border: 1px solid; border-color:red green; position: absolute; height: 5px; top: -999px;" +
			"background-image: url(" + (config.blankGif || require.toUrl("./resources/blank.gif")) + ");";
		win.body().appendChild(div);

		var cs = domStyle.getComputedStyle(div),
			bkImg = cs.backgroundImage,
			hc = (cs.borderTopColor == cs.borderRightColor) ||
				(bkImg && (bkImg == "none" || bkImg == "url(invalid-url:)" ));

		if(has("ie") <= 8){
			div.outerHTML = "";		// prevent mixed-content warning, see http://support.microsoft.com/kb/925014
		}else{
			win.body().removeChild(div);
		}

		return hc;
	});

	// Priority is 90 to run ahead of parser priority of 100.   For 2.0, remove the ready() call and instead
	// change this module to depend on dojo/domReady!
	ready(90, function(){
		if(has("highcontrast")){
			domClass.add(win.body(), "dj_a11y");
		}
	});

	return has;
});

},
'dijit/_TemplatedMixin':function(){
define("dijit/_TemplatedMixin", [
	"dojo/_base/lang", // lang.getObject
	"dojo/touch",
	"./_WidgetBase",
	"dojo/string", // string.substitute string.trim
	"dojo/cache",	// dojo.cache
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-construct", // domConstruct.destroy, domConstruct.toDom
	"dojo/sniff", // has("ie")
	"dojo/_base/unload" // unload.addOnWindowUnload
], function(lang, touch, _WidgetBase, string, cache, array, declare, domConstruct, has, unload) {

	// module:
	//		dijit/_TemplatedMixin

	var _TemplatedMixin = declare("dijit._TemplatedMixin", null, {
		// summary:
		//		Mixin for widgets that are instantiated from a template

		// templateString: [protected] String
		//		A string that represents the widget template.
		//		Use in conjunction with dojo.cache() to load from a file.
		templateString: null,

		// templatePath: [protected deprecated] String
		//		Path to template (HTML file) for this widget relative to dojo.baseUrl.
		//		Deprecated: use templateString with require([... "dojo/text!..."], ...) instead
		templatePath: null,

		// skipNodeCache: [protected] Boolean
		//		If using a cached widget template nodes poses issues for a
		//		particular widget class, it can set this property to ensure
		//		that its template is always re-built from a string
		_skipNodeCache: false,

		// _earlyTemplatedStartup: Boolean
		//		A fallback to preserve the 1.0 - 1.3 behavior of children in
		//		templates having their startup called before the parent widget
		//		fires postCreate. Defaults to 'false', causing child widgets to
		//		have their .startup() called immediately before a parent widget
		//		.startup(), but always after the parent .postCreate(). Set to
		//		'true' to re-enable to previous, arguably broken, behavior.
		_earlyTemplatedStartup: false,

/*=====
		// _attachPoints: [private] String[]
		//		List of widget attribute names associated with data-dojo-attach-point=... in the
		//		template, ex: ["containerNode", "labelNode"]
		_attachPoints: [],

		// _attachEvents: [private] Handle[]
		//		List of connections associated with data-dojo-attach-event=... in the
		//		template
		_attachEvents: [],
 =====*/

		constructor: function(/*===== params, srcNodeRef =====*/){
			// summary:
			//		Create the widget.
			// params: Object|null
			//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
			//		and functions, typically callbacks like onClick.
			//		The hash can contain any of the widget's properties, excluding read-only properties.
			// srcNodeRef: DOMNode|String?
			//		If a srcNodeRef (DOM node) is specified, replace srcNodeRef with my generated DOM tree.

			this._attachPoints = [];
			this._attachEvents = [];
		},

		_stringRepl: function(tmpl){
			// summary:
			//		Does substitution of ${foo} type properties in template string
			// tags:
			//		private
			var className = this.declaredClass, _this = this;
			// Cache contains a string because we need to do property replacement
			// do the property replacement
			return string.substitute(tmpl, this, function(value, key){
				if(key.charAt(0) == '!'){ value = lang.getObject(key.substr(1), false, _this); }
				if(typeof value == "undefined"){ throw new Error(className+" template:"+key); } // a debugging aide
				if(value == null){ return ""; }

				// Substitution keys beginning with ! will skip the transform step,
				// in case a user wishes to insert unescaped markup, e.g. ${!foo}
				return key.charAt(0) == "!" ? value :
					// Safer substitution, see heading "Attribute values" in
					// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
					value.toString().replace(/"/g,"&quot;"); //TODO: add &amp? use encodeXML method?
			}, this);
		},

		buildRendering: function(){
			// summary:
			//		Construct the UI for this widget from a template, setting this.domNode.
			// tags:
			//		protected

			if(!this.templateString){
				this.templateString = cache(this.templatePath, {sanitize: true});
			}

			// Lookup cached version of template, and download to cache if it
			// isn't there already.  Returns either a DomNode or a string, depending on
			// whether or not the template contains ${foo} replacement parameters.
			var cached = _TemplatedMixin.getCachedTemplate(this.templateString, this._skipNodeCache, this.ownerDocument);

			var node;
			if(lang.isString(cached)){
				node = domConstruct.toDom(this._stringRepl(cached), this.ownerDocument);
				if(node.nodeType != 1){
					// Flag common problems such as templates with multiple top level nodes (nodeType == 11)
					throw new Error("Invalid template: " + cached);
				}
			}else{
				// if it's a node, all we have to do is clone it
				node = cached.cloneNode(true);
			}

			this.domNode = node;

			// Call down to _Widget.buildRendering() to get base classes assigned
			// TODO: change the baseClass assignment to _setBaseClassAttr
			this.inherited(arguments);

			// recurse through the node, looking for, and attaching to, our
			// attachment points and events, which should be defined on the template node.
			this._attachTemplateNodes(node, function(n,p){ return n.getAttribute(p); });

			this._beforeFillContent();		// hook for _WidgetsInTemplateMixin

			this._fillContent(this.srcNodeRef);
		},

		_beforeFillContent: function(){
		},

		_fillContent: function(/*DomNode*/ source){
			// summary:
			//		Relocate source contents to templated container node.
			//		this.containerNode must be able to receive children, or exceptions will be thrown.
			// tags:
			//		protected
			var dest = this.containerNode;
			if(source && dest){
				while(source.hasChildNodes()){
					dest.appendChild(source.firstChild);
				}
			}
		},

		_attachTemplateNodes: function(rootNode, getAttrFunc){
			// summary:
			//		Iterate through the template and attach functions and nodes accordingly.
			//		Alternately, if rootNode is an array of widgets, then will process data-dojo-attach-point
			//		etc. for those widgets.
			// description:
			//		Map widget properties and functions to the handlers specified in
			//		the dom node and it's descendants. This function iterates over all
			//		nodes and looks for these properties:
			//
			//		- dojoAttachPoint/data-dojo-attach-point
			//		- dojoAttachEvent/data-dojo-attach-event
			// rootNode: DomNode|Widget[]
			//		the node to search for properties. All children will be searched.
			// getAttrFunc: Function
			//		a function which will be used to obtain property for a given
			//		DomNode/Widget
			// tags:
			//		private

			var nodes = lang.isArray(rootNode) ? rootNode : (rootNode.all || rootNode.getElementsByTagName("*"));
			var x = lang.isArray(rootNode) ? 0 : -1;
			for(; x < 0 || nodes[x]; x++){	// don't access nodes.length on IE, see #14346
				var baseNode = (x == -1) ? rootNode : nodes[x];
				if(this.widgetsInTemplate && (getAttrFunc(baseNode, "dojoType") || getAttrFunc(baseNode, "data-dojo-type"))){
					continue;
				}
				// Process data-dojo-attach-point
				var attachPoint = getAttrFunc(baseNode, "dojoAttachPoint") || getAttrFunc(baseNode, "data-dojo-attach-point");
				if(attachPoint){
					var point, points = attachPoint.split(/\s*,\s*/);
					while((point = points.shift())){
						if(lang.isArray(this[point])){
							this[point].push(baseNode);
						}else{
							this[point]=baseNode;
						}
						this._attachPoints.push(point);
					}
				}

				// Process data-dojo-attach-event
				var attachEvent = getAttrFunc(baseNode, "dojoAttachEvent") || getAttrFunc(baseNode, "data-dojo-attach-event");
				if(attachEvent){
					// NOTE: we want to support attributes that have the form
					// "domEvent: nativeEvent; ..."
					var event, events = attachEvent.split(/\s*,\s*/);
					var trim = lang.trim;
					while((event = events.shift())){
						if(event){
							var thisFunc = null;
							if(event.indexOf(":") != -1){
								// oh, if only JS had tuple assignment
								var funcNameArr = event.split(":");
								event = trim(funcNameArr[0]);
								thisFunc = trim(funcNameArr[1]);
							}else{
								event = trim(event);
							}
							if(!thisFunc){
								thisFunc = event;
							}
							// Map "press", "move" and "release" to keys.touch, keys.move, keys.release
							this._attachEvents.push(this.connect(baseNode, touch[event] || event, thisFunc));
						}
					}
				}
			}
		},

		destroyRendering: function(){
			// Delete all attach points to prevent IE6 memory leaks.
			array.forEach(this._attachPoints, function(point){
				delete this[point];
			}, this);
			this._attachPoints = [];

			// And same for event handlers
			array.forEach(this._attachEvents, this.disconnect, this);
			this._attachEvents = [];

			this.inherited(arguments);
		}
	});

	// key is templateString; object is either string or DOM tree
	_TemplatedMixin._templateCache = {};

	_TemplatedMixin.getCachedTemplate = function(templateString, alwaysUseString, doc){
		// summary:
		//		Static method to get a template based on the templatePath or
		//		templateString key
		// templateString: String
		//		The template
		// alwaysUseString: Boolean
		//		Don't cache the DOM tree for this template, even if it doesn't have any variables
		// doc: Document?
		//		The target document.   Defaults to document global if unspecified.
		// returns: Mixed
		//		Either string (if there are ${} variables that need to be replaced) or just
		//		a DOM tree (if the node can be cloned directly)

		// is it already cached?
		var tmplts = _TemplatedMixin._templateCache;
		var key = templateString;
		var cached = tmplts[key];
		if(cached){
			try{
				// if the cached value is an innerHTML string (no ownerDocument) or a DOM tree created within the
				// current document, then use the current cached value
				if(!cached.ownerDocument || cached.ownerDocument == (doc || document)){
					// string or node of the same document
					return cached;
				}
			}catch(e){ /* squelch */ } // IE can throw an exception if cached.ownerDocument was reloaded
			domConstruct.destroy(cached);
		}

		templateString = string.trim(templateString);

		if(alwaysUseString || templateString.match(/\$\{([^\}]+)\}/g)){
			// there are variables in the template so all we can do is cache the string
			return (tmplts[key] = templateString); //String
		}else{
			// there are no variables in the template so we can cache the DOM tree
			var node = domConstruct.toDom(templateString, doc);
			if(node.nodeType != 1){
				throw new Error("Invalid template: " + templateString);
			}
			return (tmplts[key] = node); //Node
		}
	};

	if(has("ie")){
		unload.addOnWindowUnload(function(){
			var cache = _TemplatedMixin._templateCache;
			for(var key in cache){
				var value = cache[key];
				if(typeof value == "object"){ // value is either a string or a DOM node template
					domConstruct.destroy(value);
				}
				delete cache[key];
			}
		});
	}

	// These arguments can be specified for widgets which are used in templates.
	// Since any widget can be specified as sub widgets in template, mix it
	// into the base widget class.  (This is a hack, but it's effective.).
	// Remove for 2.0.   Also, hide from API doc parser.
	lang.extend(_WidgetBase, /*===== {} || =====*/ {
		dojoAttachEvent: "",
		dojoAttachPoint: ""
	});

	return _TemplatedMixin;
});

},
'dojo/cache':function(){
define("dojo/cache", ["./_base/kernel", "./text"], function(dojo){
	// module:
	//		dojo/cache

	// dojo.cache is defined in dojo/text
	return dojo.cache;
});

},
'dijit/_CssStateMixin':function(){
define("dijit/_CssStateMixin", [
	"dojo/_base/array", // array.forEach array.map
	"dojo/_base/declare",	// declare
	"dojo/dom",			// dom.isDescendant()
	"dojo/dom-class", // domClass.toggle
	"dojo/has",
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"dojo/ready",
	"dojo/_base/window", // win.body
	"./registry"
], function(array, declare, dom, domClass, has, lang, on, ready, win, registry){

// module:
//		dijit/_CssStateMixin

var CssStateMixin = declare("dijit._CssStateMixin", [], {
	// summary:
	//		Mixin for widgets to set CSS classes on the widget DOM nodes depending on hover/mouse press/focus
	//		state changes, and also higher-level state changes such becoming disabled or selected.
	//
	// description:
	//		By mixing this class into your widget, and setting the this.baseClass attribute, it will automatically
	//		maintain CSS classes on the widget root node (this.domNode) depending on hover,
	//		active, focus, etc. state.   Ex: with a baseClass of dijitButton, it will apply the classes
	//		dijitButtonHovered and dijitButtonActive, as the user moves the mouse over the widget and clicks it.
	//
	//		It also sets CSS like dijitButtonDisabled based on widget semantic state.
	//
	//		By setting the cssStateNodes attribute, a widget can also track events on subnodes (like buttons
	//		within the widget).

	// cssStateNodes: [protected] Object
	//		List of sub-nodes within the widget that need CSS classes applied on mouse hover/press and focus
	//
	//		Each entry in the hash is a an attachpoint names (like "upArrowButton") mapped to a CSS class names
	//		(like "dijitUpArrowButton"). Example:
	//	|		{
	//	|			"upArrowButton": "dijitUpArrowButton",
	//	|			"downArrowButton": "dijitDownArrowButton"
	//	|		}
	//		The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it
	//		is hovered, etc.
	cssStateNodes: {},

	// hovering: [readonly] Boolean
	//		True if cursor is over this widget
	hovering: false,

	// active: [readonly] Boolean
	//		True if mouse was pressed while over this widget, and hasn't been released yet
	active: false,

	_applyAttributes: function(){
		// This code would typically be in postCreate(), but putting in _applyAttributes() for
		// performance: so the class changes happen before DOM is inserted into the document.
		// Change back to postCreate() in 2.0.  See #11635.

		this.inherited(arguments);

		// Monitoring changes to disabled, readonly, etc. state, and update CSS class of root node
		array.forEach(["disabled", "readOnly", "checked", "selected", "focused", "state", "hovering", "active", "_opened"], function(attr){
			this.watch(attr, lang.hitch(this, "_setStateClass"));
		}, this);

		// Track hover and active mouse events on widget root node, plus possibly on subnodes
		for(var ap in this.cssStateNodes){
			this._trackMouseState(this[ap], this.cssStateNodes[ap]);
		}
		this._trackMouseState(this.domNode, this.baseClass);

		// Set state initially; there's probably no hover/active/focus state but widget might be
		// disabled/readonly/checked/selected so we want to set CSS classes for those conditions.
		this._setStateClass();
	},

	_cssMouseEvent: function(/*Event*/ event){
		// summary:
		//		Handler for CSS event on this.domNode. Sets hovering and active properties depending on mouse state,
		//		which triggers _setStateClass() to set appropriate CSS classes for this.domNode.

		if(!this.disabled){
			switch(event.type){
				case "mouseover":
					this._set("hovering", true);
					this._set("active", this._mouseDown);
					break;
				case "mouseout":
					this._set("hovering", false);
					this._set("active", false);
					break;
				case "mousedown":
				case "touchstart":
					this._set("active", true);
					break;
				case "mouseup":
				case "touchend":
					this._set("active", false);
					break;
			}
		}
	},

	_setStateClass: function(){
		// summary:
		//		Update the visual state of the widget by setting the css classes on this.domNode
		//		(or this.stateNode if defined) by combining this.baseClass with
		//		various suffixes that represent the current widget state(s).
		//
		// description:
		//		In the case where a widget has multiple
		//		states, it sets the class based on all possible
		//		combinations.  For example, an invalid form widget that is being hovered
		//		will be "dijitInput dijitInputInvalid dijitInputHover dijitInputInvalidHover".
		//
		//		The widget may have one or more of the following states, determined
		//		by this.state, this.checked, this.valid, and this.selected:
		//
		//		- Error - ValidationTextBox sets this.state to "Error" if the current input value is invalid
		//		- Incomplete - ValidationTextBox sets this.state to "Incomplete" if the current input value is not finished yet
		//		- Checked - ex: a checkmark or a ToggleButton in a checked state, will have this.checked==true
		//		- Selected - ex: currently selected tab will have this.selected==true
		//
		//		In addition, it may have one or more of the following states,
		//		based on this.disabled and flags set in _onMouse (this.active, this.hovering) and from focus manager (this.focused):
		//
		//		- Disabled	- if the widget is disabled
		//		- Active		- if the mouse (or space/enter key?) is being pressed down
		//		- Focused		- if the widget has focus
		//		- Hover		- if the mouse is over the widget

		// Compute new set of classes
		var newStateClasses = this.baseClass.split(" ");

		function multiply(modifier){
			newStateClasses = newStateClasses.concat(array.map(newStateClasses, function(c){ return c+modifier; }), "dijit"+modifier);
		}

		if(!this.isLeftToRight()){
			// For RTL mode we need to set an addition class like dijitTextBoxRtl.
			multiply("Rtl");
		}

		var checkedState = this.checked == "mixed" ? "Mixed" : (this.checked ? "Checked" : "");
		if(this.checked){
			multiply(checkedState);
		}
		if(this.state){
			multiply(this.state);
		}
		if(this.selected){
			multiply("Selected");
		}
		if(this._opened){
			multiply("Opened");
		}

		if(this.disabled){
			multiply("Disabled");
		}else if(this.readOnly){
			multiply("ReadOnly");
		}else{
			if(this.active){
				multiply("Active");
			}else if(this.hovering){
				multiply("Hover");
			}
		}

		if(this.focused){
			multiply("Focused");
		}

		// Remove old state classes and add new ones.
		// For performance concerns we only write into domNode.className once.
		var tn = this.stateNode || this.domNode,
			classHash = {};	// set of all classes (state and otherwise) for node

		array.forEach(tn.className.split(" "), function(c){ classHash[c] = true; });

		if("_stateClasses" in this){
			array.forEach(this._stateClasses, function(c){ delete classHash[c]; });
		}

		array.forEach(newStateClasses, function(c){ classHash[c] = true; });

		var newClasses = [];
		for(var c in classHash){
			newClasses.push(c);
		}
		tn.className = newClasses.join(" ");

		this._stateClasses = newStateClasses;
	},

	_subnodeCssMouseEvent: function(node, clazz, evt){
		// summary:
		//		Handler for hover/active mouse event on widget's subnode
		if(this.disabled || this.readOnly){
			return;
		}
		function hover(isHovering){
			domClass.toggle(node, clazz+"Hover", isHovering);
		}
		function active(isActive){
			domClass.toggle(node, clazz+"Active", isActive);
		}
		function focused(isFocused){
			domClass.toggle(node, clazz+"Focused", isFocused);
		}
		switch(evt.type){
			case "mouseover":
				hover(true);
				break;
			case "mouseout":
				hover(false);
				active(false);
				break;
			case "mousedown":
			case "touchstart":
				active(true);
				break;
			case "mouseup":
			case "touchend":
				active(false);
				break;
			case "focus":
			case "focusin":
				focused(true);
				break;
			case "blur":
			case "focusout":
				focused(false);
				break;
		}
	},

	_trackMouseState: function(/*DomNode*/ node, /*String*/ clazz){
		// summary:
		//		Track mouse/focus events on specified node and set CSS class on that node to indicate
		//		current state.   Usually not called directly, but via cssStateNodes attribute.
		// description:
		//		Given class=foo, will set the following CSS class on the node
		//
		//		- fooActive: if the user is currently pressing down the mouse button while over the node
		//		- fooHover: if the user is hovering the mouse over the node, but not pressing down a button
		//		- fooFocus: if the node is focused
		//
		//		Note that it won't set any classes if the widget is disabled.
		// node: DomNode
		//		Should be a sub-node of the widget, not the top node (this.domNode), since the top node
		//		is handled specially and automatically just by mixing in this class.
		// clazz: String
		//		CSS class name (ex: dijitSliderUpArrow)

		// Flag for listener code below to call this._cssMouseEvent() or this._subnodeCssMouseEvent()
		// when node is hovered/active
		node._cssState = clazz;
	}
});

ready(function(){
	// Document level listener to catch hover etc. events on widget root nodes and subnodes.
	// Note that when the mouse is moved quickly, a single onmouseenter event could signal that multiple widgets
	// have been hovered or unhovered (try test_Accordion.html)
	function handler(evt){
		// Poor man's event propagation.  Don't propagate event to ancestors of evt.relatedTarget,
		// to avoid processing mouseout events moving from a widget's domNode to a descendant node;
		// such events shouldn't be interpreted as a mouseleave on the widget.
		if(!dom.isDescendant(evt.relatedTarget, evt.target)){
			for(var node = evt.target; node && node != evt.relatedTarget; node = node.parentNode){
				// Process any nodes with _cssState property.   They are generally widget root nodes,
				// but could also be sub-nodes within a widget
				if(node._cssState){
					var widget = registry.getEnclosingWidget(node);
					if(widget){
						if(node == widget.domNode){
							// event on the widget's root node
							widget._cssMouseEvent(evt);
						}else{
							// event on widget's sub-node
							widget._subnodeCssMouseEvent(node, node._cssState, evt);
						}
					}
				}
			}
		}
	}
	function ieHandler(evt){
		evt.target = evt.srcElement;
		handler(evt);
	}

	// Use addEventListener() (and attachEvent() on IE) to catch the relevant events even if other handlers
	// (on individual nodes) call evt.stopPropagation() or event.stopEvent().
	// Currently typematic.js is doing that, not sure why.
	// Don't monitor mouseover/mouseout on mobile because iOS generates "phantom" mouseover/mouseout events when
	// drag-scrolling, at the point in the viewport where the drag originated.   Test the Tree in api viewer.
	var body = win.body(),
		types = (has("touch") ? [] : ["mouseover", "mouseout"]).concat(["mousedown", "touchstart", "mouseup", "touchend"]);
	array.forEach(types, function(type){
		if(body.addEventListener){
			body.addEventListener(type, handler, true);	// W3C
		}else{
			body.attachEvent("on"+type, ieHandler);	// IE
		}
	});

	// Track focus events on widget sub-nodes that have been registered via _trackMouseState().
	// However, don't track focus events on the widget root nodes, because focus is tracked via the
	// focus manager (and it's not really tracking focus, but rather tracking that focus is on one of the widget's
	// nodes or a subwidget's node or a popup node, etc.)
	// Remove for 2.0 (if focus CSS needed, just use :focus pseudo-selector).
	on(body, "focusin, focusout", function(evt){
		var node = evt.target;
		if(node._cssState && !node.getAttribute("widgetId")){
			var widget = registry.getEnclosingWidget(node);
			widget._subnodeCssMouseEvent(node, node._cssState, evt);
		}
	});
});

return CssStateMixin;
});

},
'dijit/form/_FormMixin':function(){
define("dijit/form/_FormMixin", [
	"dojo/_base/array", // array.every array.filter array.forEach array.indexOf array.map
	"dojo/_base/declare", // declare
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.hitch lang.isArray
	"dojo/on",
	"dojo/window" // winUtils.scrollIntoView
], function(array, declare, kernel, lang, on, winUtils){

	// module:
	//		dijit/form/_FormMixin

	return declare("dijit.form._FormMixin", null, {
		// summary:
		//		Mixin for containers of form widgets (i.e. widgets that represent a single value
		//		and can be children of a `<form>` node or `dijit/form/Form` widget)
		// description:
		//		Can extract all the form widgets
		//		values and combine them into a single javascript object, or alternately
		//		take such an object and set the values for all the contained
		//		form widgets

	/*=====
		// value: Object
		//		Name/value hash for each child widget with a name and value.
		//		Child widgets without names are not part of the hash.
		//
		//		If there are multiple child widgets w/the same name, value is an array,
		//		unless they are radio buttons in which case value is a scalar (since only
		//		one radio button can be checked at a time).
		//
		//		If a child widget's name is a dot separated list (like a.b.c.d), it's a nested structure.
		//
		//		Example:
		//	|	{ name: "John Smith", interests: ["sports", "movies"] }
	=====*/

		// state: [readonly] String
		//		Will be "Error" if one or more of the child widgets has an invalid value,
		//		"Incomplete" if not all of the required child widgets are filled in.  Otherwise, "",
		//		which indicates that the form is ready to be submitted.
		state: "",

		// TODO:
		//	* Repeater
		//	* better handling for arrays.  Often form elements have names with [] like
		//	* people[3].sex (for a list of people [{name: Bill, sex: M}, ...])


		_getDescendantFormWidgets: function(/*dijit/_WidgetBase[]?*/ children){
			// summary:
			//		Returns all form widget descendants, searching through non-form child widgets like BorderContainer
			var res = [];
			array.forEach(children || this.getChildren(), function(child){
				if("value" in child){
					res.push(child);
				}else{
					res = res.concat(this._getDescendantFormWidgets(child.getChildren()));
				}
			}, this);
			return res;
		},

		reset: function(){
			array.forEach(this._getDescendantFormWidgets(), function(widget){
				if(widget.reset){
					widget.reset();
				}
			});
		},

		validate: function(){
			// summary:
			//		returns if the form is valid - same as isValid - but
			//		provides a few additional (ui-specific) features:
			//
			//		1. it will highlight any sub-widgets that are not valid
			//		2. it will call focus() on the first invalid sub-widget
			var didFocus = false;
			return array.every(array.map(this._getDescendantFormWidgets(), function(widget){
				// Need to set this so that "required" widgets get their
				// state set.
				widget._hasBeenBlurred = true;
				var valid = widget.disabled || !widget.validate || widget.validate();
				if(!valid && !didFocus){
					// Set focus of the first non-valid widget
					winUtils.scrollIntoView(widget.containerNode || widget.domNode);
					widget.focus();
					didFocus = true;
				}
				return valid;
			}), function(item){ return item; });
		},

		setValues: function(val){
			kernel.deprecated(this.declaredClass+"::setValues() is deprecated. Use set('value', val) instead.", "", "2.0");
			return this.set('value', val);
		},
		_setValueAttr: function(/*Object*/ obj){
			// summary:
			//		Fill in form values from according to an Object (in the format returned by get('value'))

			// generate map from name --> [list of widgets with that name]
			var map = { };
			array.forEach(this._getDescendantFormWidgets(), function(widget){
				if(!widget.name){ return; }
				var entry = map[widget.name] || (map[widget.name] = [] );
				entry.push(widget);
			});

			for(var name in map){
				if(!map.hasOwnProperty(name)){
					continue;
				}
				var widgets = map[name],						// array of widgets w/this name
					values = lang.getObject(name, false, obj);	// list of values for those widgets

				if(values === undefined){
					continue;
				}
				if(!lang.isArray(values)){
					values = [ values ];
				}
				if(typeof widgets[0].checked == 'boolean'){
					// for checkbox/radio, values is a list of which widgets should be checked
					array.forEach(widgets, function(w){
						w.set('value', array.indexOf(values, w.value) != -1);
					});
				}else if(widgets[0].multiple){
					// it takes an array (e.g. multi-select)
					widgets[0].set('value', values);
				}else{
					// otherwise, values is a list of values to be assigned sequentially to each widget
					array.forEach(widgets, function(w, i){
						w.set('value', values[i]);
					});
				}
			}

			/***
			 *	TODO: code for plain input boxes (this shouldn't run for inputs that are part of widgets)

			array.forEach(this.containerNode.elements, function(element){
				if(element.name == ''){return};	// like "continue"
				var namePath = element.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j){
					var p=namePath[j - 1];
					// repeater support block
					var nameA=p.split("[");
					if(nameA.length > 1){
						if(typeof(myObj[nameA[0]]) == "undefined"){
							myObj[nameA[0]]=[ ];
						} // if

						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined"){
							myObj[nameA[0]][nameIndex] = { };
						}
						myObj=myObj[nameA[0]][nameIndex];
						continue;
					} // repeater support ends

					if(typeof(myObj[p]) == "undefined"){
						myObj=undefined;
						break;
					};
					myObj=myObj[p];
				}

				if(typeof(myObj) == "undefined"){
					return;		// like "continue"
				}
				if(typeof(myObj[name]) == "undefined" && this.ignoreNullValues){
					return;		// like "continue"
				}

				// TODO: widget values (just call set('value', ...) on the widget)

				// TODO: maybe should call dojo.getNodeProp() instead
				switch(element.type){
					case "checkbox":
						element.checked = (name in myObj) &&
							array.some(myObj[name], function(val){ return val == element.value; });
						break;
					case "radio":
						element.checked = (name in myObj) && myObj[name] == element.value;
						break;
					case "select-multiple":
						element.selectedIndex=-1;
						array.forEach(element.options, function(option){
							option.selected = array.some(myObj[name], function(val){ return option.value == val; });
						});
						break;
					case "select-one":
						element.selectedIndex="0";
						array.forEach(element.options, function(option){
							option.selected = option.value == myObj[name];
						});
						break;
					case "hidden":
					case "text":
					case "textarea":
					case "password":
						element.value = myObj[name] || "";
						break;
				}
			});
			*/

			// Note: no need to call this._set("value", ...) as the child updates will trigger onChange events
			// which I am monitoring.
		},

		getValues: function(){
			kernel.deprecated(this.declaredClass+"::getValues() is deprecated. Use get('value') instead.", "", "2.0");
			return this.get('value');
		},
		_getValueAttr: function(){
			// summary:
			//		Returns Object representing form values.   See description of `value` for details.
			// description:

			// The value is updated into this.value every time a child has an onChange event,
			// so in the common case this function could just return this.value.   However,
			// that wouldn't work when:
			//
			// 1. User presses return key to submit a form.  That doesn't fire an onchange event,
			// and even if it did it would come too late due to the defer(...) in _handleOnChange()
			//
			// 2. app for some reason calls this.get("value") while the user is typing into a
			// form field.   Not sure if that case needs to be supported or not.

			// get widget values
			var obj = { };
			array.forEach(this._getDescendantFormWidgets(), function(widget){
				var name = widget.name;
				if(!name || widget.disabled){ return; }

				// Single value widget (checkbox, radio, or plain <input> type widget)
				var value = widget.get('value');

				// Store widget's value(s) as a scalar, except for checkboxes which are automatically arrays
				if(typeof widget.checked == 'boolean'){
					if(/Radio/.test(widget.declaredClass)){
						// radio button
						if(value !== false){
							lang.setObject(name, value, obj);
						}else{
							// give radio widgets a default of null
							value = lang.getObject(name, false, obj);
							if(value === undefined){
								lang.setObject(name, null, obj);
							}
						}
					}else{
						// checkbox/toggle button
						var ary=lang.getObject(name, false, obj);
						if(!ary){
							ary=[];
							lang.setObject(name, ary, obj);
						}
						if(value !== false){
							ary.push(value);
						}
					}
				}else{
					var prev=lang.getObject(name, false, obj);
					if(typeof prev != "undefined"){
						if(lang.isArray(prev)){
							prev.push(value);
						}else{
							lang.setObject(name, [prev, value], obj);
						}
					}else{
						// unique name
						lang.setObject(name, value, obj);
					}
				}
			});

			/***
			 * code for plain input boxes (see also domForm.formToObject, can we use that instead of this code?
			 * but it doesn't understand [] notation, presumably)
			var obj = { };
			array.forEach(this.containerNode.elements, function(elm){
				if(!elm.name)	{
					return;		// like "continue"
				}
				var namePath = elm.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j){
					var nameIndex = null;
					var p=namePath[j - 1];
					var nameA=p.split("[");
					if(nameA.length > 1){
						if(typeof(myObj[nameA[0]]) == "undefined"){
							myObj[nameA[0]]=[ ];
						} // if
						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined"){
							myObj[nameA[0]][nameIndex] = { };
						}
					}else if(typeof(myObj[nameA[0]]) == "undefined"){
						myObj[nameA[0]] = { }
					} // if

					if(nameA.length == 1){
						myObj=myObj[nameA[0]];
					}else{
						myObj=myObj[nameA[0]][nameIndex];
					} // if
				} // for

				if((elm.type != "select-multiple" && elm.type != "checkbox" && elm.type != "radio") || (elm.type == "radio" && elm.checked)){
					if(name == name.split("[")[0]){
						myObj[name]=elm.value;
					}else{
						// can not set value when there is no name
					}
				}else if(elm.type == "checkbox" && elm.checked){
					if(typeof(myObj[name]) == 'undefined'){
						myObj[name]=[ ];
					}
					myObj[name].push(elm.value);
				}else if(elm.type == "select-multiple"){
					if(typeof(myObj[name]) == 'undefined'){
						myObj[name]=[ ];
					}
					for(var jdx=0,len3=elm.options.length; jdx<len3; ++jdx){
						if(elm.options[jdx].selected){
							myObj[name].push(elm.options[jdx].value);
						}
					}
				} // if
				name=undefined;
			}); // forEach
			***/
			return obj;
		},

		isValid: function(){
			// summary:
			//		Returns true if all of the widgets are valid.
			//		Deprecated, will be removed in 2.0.  Use get("state") instead.

			return this.state == "";
		},

		onValidStateChange: function(/*Boolean*/ /*===== isValid =====*/){
			// summary:
			//		Stub function to connect to if you want to do something
			//		(like disable/enable a submit button) when the valid
			//		state changes on the form as a whole.
			//
			//		Deprecated.  Will be removed in 2.0.  Use watch("state", ...) instead.
		},

		_getState: function(){
			// summary:
			//		Compute what this.state should be based on state of children
			var states = array.map(this._descendants, function(w){
				return w.get("state") || "";
			});

			return array.indexOf(states, "Error") >= 0 ? "Error" :
				array.indexOf(states, "Incomplete") >= 0 ? "Incomplete" : "";
		},

		disconnectChildren: function(){
			// summary:
			//		Deprecated method.   Applications no longer need to call this.   Remove for 2.0.
		},

		connectChildren: function(/*Boolean*/ inStartup){
			// summary:
			//		You can call this function directly, ex. in the event that you
			//		programmatically add a widget to the form *after* the form has been
			//		initialized.

			// TODO: rename for 2.0

			this._descendants = this._getDescendantFormWidgets();

			// To get notifications from children they need to be started.   Children didn't used to need to be started,
			// so for back-compat, start them here
			array.forEach(this._descendants, function(child){
				if(!child._started){ child.startup(); }
			});

			if(!inStartup){
				this._onChildChange();
			}
		},

		_onChildChange: function(/*String*/ attr){
			// summary:
			//		Called when child's value or disabled state changes

			// The unit tests expect state update to be synchronous, so update it immediately.
			if(!attr || attr == "state" || attr == "disabled"){
				this._set("state", this._getState());
			}

			// Use defer() to collapse value changes in multiple children into a single
			// update to my value.   Multiple updates will occur on:
			//	1. Form.set()
			//	2. Form.reset()
			//	3. user selecting a radio button (which will de-select another radio button,
			//		 causing two onChange events)
			if(!attr || attr == "value" || attr == "disabled" || attr == "checked"){
				if(this._onChangeDelayTimer){
					this._onChangeDelayTimer.remove();
				}
				this._onChangeDelayTimer = this.defer(function(){
					delete this._onChangeDelayTimer;
					this._set("value", this.get("value"));
				}, 10);
			}
		},

		startup: function(){
			this.inherited(arguments);

			// Set initial this.value and this.state.   Don't emit watch() notifications.
			this._descendants = this._getDescendantFormWidgets();
			this.value = this.get("value");
			this.state = this._getState();

			// Initialize value and valid/invalid state tracking.
			var self = this;
			this.own(
				on(
					this.containerNode,
					"attrmodified-state, attrmodified-disabled, attrmodified-value, attrmodified-checked",
					function(evt){
						if(evt.target == self.domNode){
							return;	// ignore events that I fire on myself because my children changed
						}
						self._onChildChange(evt.type.replace("attrmodified-", ""));
					}
				)
			);

			// Make state change call onValidStateChange(), will be removed in 2.0
			this.watch("state", function(attr, oldVal, newVal){ this.onValidStateChange(newVal == ""); });
		},

		destroy: function(){
			this.inherited(arguments);
		}

	});
});

},
'dijit/_DialogMixin':function(){
define("dijit/_DialogMixin", [
	"dojo/_base/declare", // declare
	"./a11y"	// _getTabNavigable
], function(declare, a11y){

	// module:
	//		dijit/_DialogMixin

	return declare("dijit._DialogMixin", null, {
		// summary:
		//		This provides functions useful to Dialog and TooltipDialog

		execute: function(/*Object*/ /*===== formContents =====*/){
			// summary:
			//		Callback when the user hits the submit button.
			//		Override this method to handle Dialog execution.
			// description:
			//		After the user has pressed the submit button, the Dialog
			//		first calls onExecute() to notify the container to hide the
			//		dialog and restore focus to wherever it used to be.
			//
			//		*Then* this method is called.
			// type:
			//		callback
		},

		onCancel: function(){
			// summary:
			//		Called when user has pressed the Dialog's cancel button, to notify container.
			// description:
			//		Developer shouldn't override or connect to this method;
			//		it's a private communication device between the TooltipDialog
			//		and the thing that opened it (ex: `dijit/form/DropDownButton`)
			// type:
			//		protected
		},

		onExecute: function(){
			// summary:
			//		Called when user has pressed the dialog's OK button, to notify container.
			// description:
			//		Developer shouldn't override or connect to this method;
			//		it's a private communication device between the TooltipDialog
			//		and the thing that opened it (ex: `dijit/form/DropDownButton`)
			// type:
			//		protected
		},

		_onSubmit: function(){
			// summary:
			//		Callback when user hits submit button
			// type:
			//		protected
			this.onExecute();	// notify container that we are about to execute
			this.execute(this.get('value'));
		},

		_getFocusItems: function(){
			// summary:
			//		Finds focusable items in dialog,
			//		and sets this._firstFocusItem and this._lastFocusItem
			// tags:
			//		protected

			var elems = a11y._getTabNavigable(this.containerNode);
			this._firstFocusItem = elems.lowest || elems.first || this.closeButtonNode || this.domNode;
			this._lastFocusItem = elems.last || elems.highest || this._firstFocusItem;
		}
	});
});

},
'dijit/DialogUnderlay':function(){
define("dijit/DialogUnderlay", [
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/window", // winUtils.getBox
	"./_Widget",
	"./_TemplatedMixin",
	"./BackgroundIframe"
], function(declare, domAttr, winUtils, _Widget, _TemplatedMixin, BackgroundIframe){

	// module:
	//		dijit/DialogUnderlay

	return declare("dijit.DialogUnderlay", [_Widget, _TemplatedMixin], {
		// summary:
		//		The component that blocks the screen behind a `dijit.Dialog`
		//
		// description:
		//		A component used to block input behind a `dijit.Dialog`. Only a single
		//		instance of this widget is created by `dijit.Dialog`, and saved as
		//		a reference to be shared between all Dialogs as `dijit._underlay`
		//
		//		The underlay itself can be styled based on and id:
		//	|	#myDialog_underlay { background-color:red; }
		//
		//		In the case of `dijit.Dialog`, this id is based on the id of the Dialog,
		//		suffixed with _underlay.

		// Template has two divs; outer div is used for fade-in/fade-out, and also to hold background iframe.
		// Inner div has opacity specified in CSS file.
		templateString: "<div class='dijitDialogUnderlayWrapper'><div class='dijitDialogUnderlay' data-dojo-attach-point='node'></div></div>",

		// Parameters on creation or updatable later

		// dialogId: String
		//		Id of the dialog.... DialogUnderlay's id is based on this id
		dialogId: "",

		// class: String
		//		This class name is used on the DialogUnderlay node, in addition to dijitDialogUnderlay
		"class": "",

		_setDialogIdAttr: function(id){
			domAttr.set(this.node, "id", id + "_underlay");
			this._set("dialogId", id);
		},

		_setClassAttr: function(clazz){
			this.node.className = "dijitDialogUnderlay " + clazz;
			this._set("class", clazz);
		},

		postCreate: function(){
			// summary:
			//		Append the underlay to the body
			this.ownerDocumentBody.appendChild(this.domNode);
		},

		layout: function(){
			// summary:
			//		Sets the background to the size of the viewport
			//
			// description:
			//		Sets the background to the size of the viewport (rather than the size
			//		of the document) since we need to cover the whole browser window, even
			//		if the document is only a few lines long.
			// tags:
			//		private

			var is = this.node.style,
				os = this.domNode.style;

			// hide the background temporarily, so that the background itself isn't
			// causing scrollbars to appear (might happen when user shrinks browser
			// window and then we are called to resize)
			os.display = "none";

			// then resize and show
			var viewport = winUtils.getBox(this.ownerDocument);
			os.top = viewport.t + "px";
			os.left = viewport.l + "px";
			is.width = viewport.w + "px";
			is.height = viewport.h + "px";
			os.display = "block";
		},

		show: function(){
			// summary:
			//		Show the dialog underlay
			this.domNode.style.display = "block";
			this.layout();
			this.bgIframe = new BackgroundIframe(this.domNode);
		},

		hide: function(){
			// summary:
			//		Hides the dialog underlay
			this.bgIframe.destroy();
			delete this.bgIframe;
			this.domNode.style.display = "none";
		}
	});
});

},
'dijit/BackgroundIframe':function(){
define("dijit/BackgroundIframe", [
	"require",			// require.toUrl
	"./main",	// to export dijit.BackgroundIframe
	"dojo/_base/config",
	"dojo/dom-construct", // domConstruct.create
	"dojo/dom-style", // domStyle.set
	"dojo/_base/lang", // lang.extend lang.hitch
	"dojo/on",
	"dojo/sniff", // has("ie"), has("mozilla"), has("quirks")
	"dojo/_base/window" // win.doc.createElement
], function(require, dijit, config, domConstruct, domStyle, lang, on, has, win){

	// module:
	//		dijit/BackgroundIFrame

	// Flag for whether to create background iframe behind popups like Menus and Dialog.
	// A background iframe is useful to prevent problems with popups appearing behind applets/pdf files,
	// and is also useful on older versions of IE (IE6 and IE7) to prevent the "bleed through select" problem.
	// TODO: For 2.0, make this false by default.  Also, possibly move definition to has.js so that this module can be
	// conditionally required via  dojo/has!bgIfame?dijit/BackgroundIframe
	has.add("bgIframe", has("ie") || has("mozilla"));

	// TODO: remove _frames, it isn't being used much, since popups never release their
	// iframes (see [22236])
	var _frames = new function(){
		// summary:
		//		cache of iframes

		var queue = [];

		this.pop = function(){
			var iframe;
			if(queue.length){
				iframe = queue.pop();
				iframe.style.display="";
			}else{
				if(has("ie") < 9){
					var burl = config["dojoBlankHtmlUrl"] || require.toUrl("dojo/resources/blank.html") || "javascript:\"\"";
					var html="<iframe src='" + burl + "' role='presentation'"
						+ " style='position: absolute; left: 0px; top: 0px;"
						+ "z-index: -1; filter:Alpha(Opacity=\"0\");'>";
					iframe = win.doc.createElement(html);
				}else{
					iframe = domConstruct.create("iframe");
					iframe.src = 'javascript:""';
					iframe.className = "dijitBackgroundIframe";
					iframe.setAttribute("role", "presentation");
					domStyle.set(iframe, "opacity", 0.1);
				}
				iframe.tabIndex = -1; // Magic to prevent iframe from getting focus on tab keypress - as style didn't work.
			}
			return iframe;
		};

		this.push = function(iframe){
			iframe.style.display="none";
			queue.push(iframe);
		}
	}();


	dijit.BackgroundIframe = function(/*DomNode*/ node){
		// summary:
		//		For IE/FF z-index schenanigans. id attribute is required.
		//
		// description:
		//		new dijit.BackgroundIframe(node).
		//
		//		Makes a background iframe as a child of node, that fills
		//		area (and position) of node

		if(!node.id){ throw new Error("no id"); }
		if(has("bgIframe")){
			var iframe = (this.iframe = _frames.pop());
			node.appendChild(iframe);
			if(has("ie")<7 || has("quirks")){
				this.resize(node);
				this._conn = on(node, 'resize', lang.hitch(this, function(){
					this.resize(node);
				}));
			}else{
				domStyle.set(iframe, {
					width: '100%',
					height: '100%'
				});
			}
		}
	};

	lang.extend(dijit.BackgroundIframe, {
		resize: function(node){
			// summary:
			//		Resize the iframe so it's the same size as node.
			//		Needed on IE6 and IE/quirks because height:100% doesn't work right.
			if(this.iframe){
				domStyle.set(this.iframe, {
					width: node.offsetWidth + 'px',
					height: node.offsetHeight + 'px'
				});
			}
		},
		destroy: function(){
			// summary:
			//		destroy the iframe
			if(this._conn){
				this._conn.remove();
				this._conn = null;
			}
			if(this.iframe){
				_frames.push(this.iframe);
				delete this.iframe;
			}
		}
	});

	return dijit.BackgroundIframe;
});

},
'dijit/layout/ContentPane':function(){
define("dijit/layout/ContentPane", [
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.mixin lang.delegate lang.hitch lang.isFunction lang.isObject
	"../_Widget",
	"../_Container",
	"./_ContentPaneResizeMixin",
	"dojo/string", // string.substitute
	"dojo/html", // html._ContentSetter
	"dojo/i18n!../nls/loading",
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/_base/Deferred", // Deferred
	"dojo/dom", // dom.byId
	"dojo/dom-attr", // domAttr.attr
	"dojo/dom-construct", // empty()
	"dojo/_base/xhr", // xhr.get
	"dojo/i18n", // i18n.getLocalization
	"dojo/when"
], function(kernel, lang, _Widget, _Container, _ContentPaneResizeMixin, string, html, nlsLoading,
	array, declare, Deferred, dom, domAttr, domConstruct, xhr, i18n, when){

// module:
//		dijit/layout/ContentPane


return declare("dijit.layout.ContentPane", [_Widget, _Container, _ContentPaneResizeMixin], {
	// summary:
	//		A widget containing an HTML fragment, specified inline
	//		or by uri.  Fragment may include widgets.
	//
	// description:
	//		This widget embeds a document fragment in the page, specified
	//		either by uri, javascript generated markup or DOM reference.
	//		Any widgets within this content are instantiated and managed,
	//		but laid out according to the HTML structure.  Unlike IFRAME,
	//		ContentPane embeds a document fragment as would be found
	//		inside the BODY tag of a full HTML document.  It should not
	//		contain the HTML, HEAD, or BODY tags.
	//		For more advanced functionality with scripts and
	//		stylesheets, see dojox/layout/ContentPane.  This widget may be
	//		used stand alone or as a base class for other widgets.
	//		ContentPane is useful as a child of other layout containers
	//		such as BorderContainer or TabContainer, but note that those
	//		widgets can contain any widget as a child.
	//
	// example:
	//		Some quick samples:
	//		To change the innerHTML:
	// |		cp.set('content', '<b>new content</b>')`
	//		Or you can send it a NodeList:
	// |		cp.set('content', dojo.query('div [class=selected]', userSelection))
	//		To do an ajax update:
	// |		cp.set('href', url)

	// href: String
	//		The href of the content that displays now.
	//		Set this at construction if you want to load data externally when the
	//		pane is shown.  (Set preload=true to load it immediately.)
	//		Changing href after creation doesn't have any effect; Use set('href', ...);
	href: "",

	// content: String|DomNode|NodeList|dijit/_Widget
	//		The innerHTML of the ContentPane.
	//		Note that the initialization parameter / argument to set("content", ...)
	//		can be a String, DomNode, Nodelist, or _Widget.
	content: "",

	// extractContent: Boolean
	//		Extract visible content from inside of `<body> .... </body>`.
	//		I.e., strip `<html>` and `<head>` (and it's contents) from the href
	extractContent: false,

	// parseOnLoad: Boolean
	//		Parse content and create the widgets, if any.
	parseOnLoad: true,

	// parserScope: String
	//		Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,
	//		will search for data-dojo-type (or dojoType).  For backwards compatibility
	//		reasons defaults to dojo._scopeName (which is "dojo" except when
	//		multi-version support is used, when it will be something like dojo16, dojo20, etc.)
	parserScope: kernel._scopeName,

	// preventCache: Boolean
	//		Prevent caching of data from href's by appending a timestamp to the href.
	preventCache: false,

	// preload: Boolean
	//		Force load of data on initialization even if pane is hidden.
	preload: false,

	// refreshOnShow: Boolean
	//		Refresh (re-download) content when pane goes from hidden to shown
	refreshOnShow: false,

	// loadingMessage: String
	//		Message that shows while downloading
	loadingMessage: "<span class='dijitContentPaneLoading'><span class='dijitInline dijitIconLoading'></span>${loadingState}</span>",

	// errorMessage: String
	//		Message that shows if an error occurs
	errorMessage: "<span class='dijitContentPaneError'><span class='dijitInline dijitIconError'></span>${errorState}</span>",

	// isLoaded: [readonly] Boolean
	//		True if the ContentPane has data in it, either specified
	//		during initialization (via href or inline content), or set
	//		via set('content', ...) / set('href', ...)
	//
	//		False if it doesn't have any content, or if ContentPane is
	//		still in the process of downloading href.
	isLoaded: false,

	baseClass: "dijitContentPane",

	/*======
	// ioMethod: dojo/_base/xhr.get|dojo._base/xhr.post
	//		Function that should grab the content specified via href.
	ioMethod: dojo.xhrGet,
	======*/

	// ioArgs: Object
	//		Parameters to pass to xhrGet() request, for example:
	// |	<div data-dojo-type="dijit/layout/ContentPane" data-dojo-props="href: './bar', ioArgs: {timeout: 500}">
	ioArgs: {},

	// onLoadDeferred: [readonly] dojo.Deferred
	//		This is the `dojo.Deferred` returned by set('href', ...) and refresh().
	//		Calling onLoadDeferred.then() registers your
	//		callback to be called only once, when the prior set('href', ...) call or
	//		the initial href parameter to the constructor finishes loading.
	//
	//		This is different than an onLoad() handler which gets called any time any href
	//		or content is loaded.
	onLoadDeferred: null,

	// Cancel _WidgetBase's _setTitleAttr because we don't want the title attribute (used to specify
	// tab labels) to be copied to ContentPane.domNode... otherwise a tooltip shows up over the
	// entire pane.
	_setTitleAttr: null,

	// Flag to parser that I'll parse my contents, so it shouldn't.
	stopParser: true,

	// template: [private] Boolean
	//		Flag from the parser that this ContentPane is inside a template
	//		so the contents are pre-parsed.
	// TODO: this declaration can be commented out in 2.0
	template: false,

	create: function(params, srcNodeRef){
		// Convert a srcNodeRef argument into a content parameter, so that the original contents are
		// processed in the same way as contents set via set("content", ...), calling the parser etc.
		// Avoid modifying original params object since that breaks NodeList instantiation, see #11906.
		if((!params || !params.template) && srcNodeRef && !("href" in params) && !("content" in params)){
			srcNodeRef = dom.byId(srcNodeRef);
			var df = srcNodeRef.ownerDocument.createDocumentFragment();
			while(srcNodeRef.firstChild){
				df.appendChild(srcNodeRef.firstChild);
			}
			params = lang.delegate(params, {content: df});
		}
		this.inherited(arguments, [params, srcNodeRef]);
	},

	postMixInProperties: function(){
		this.inherited(arguments);
		var messages = i18n.getLocalization("dijit", "loading", this.lang);
		this.loadingMessage = string.substitute(this.loadingMessage, messages);
		this.errorMessage = string.substitute(this.errorMessage, messages);
	},

	buildRendering: function(){
		this.inherited(arguments);

		// Since we have no template we need to set this.containerNode ourselves, to make getChildren() work.
		// For subclasses of ContentPane that do have a template, does nothing.
		if(!this.containerNode){
			this.containerNode = this.domNode;
		}

		// remove the title attribute so it doesn't show up when hovering
		// over a node  (TODO: remove in 2.0, no longer needed after #11490)
		this.domNode.removeAttribute("title");
	},

	startup: function(){
		// summary:
		//		Call startup() on all children including non _Widget ones like dojo/dnd/Source objects

		// This starts all the widgets
		this.inherited(arguments);

		// And this catches stuff like dojo/dnd/Source
		if(this._contentSetter){
			array.forEach(this._contentSetter.parseResults, function(obj){
				if(!obj._started && !obj._destroyed && lang.isFunction(obj.startup)){
					obj.startup();
					obj._started = true;
				}
			}, this);
		}
	},

	_startChildren: function(){
		// summary:
		//		Called when content is loaded.   Calls startup on each child widget.   Similar to ContentPane.startup()
		//		itself, but avoids marking the ContentPane itself as "restarted" (see #15581).

		// This starts all the widgets
		array.forEach(this.getChildren(), function(obj){
			if(!obj._started && !obj._destroyed && lang.isFunction(obj.startup)){
				obj.startup();
				obj._started = true;
			}
		});

		// And this catches stuff like dojo/dnd/Source
		if(this._contentSetter){
			array.forEach(this._contentSetter.parseResults, function(obj){
				if(!obj._started && !obj._destroyed && lang.isFunction(obj.startup)){
					obj.startup();
					obj._started = true;
				}
			}, this);
		}
	},

	setHref: function(/*String|Uri*/ href){
		// summary:
		//		Deprecated.   Use set('href', ...) instead.
		kernel.deprecated("dijit.layout.ContentPane.setHref() is deprecated. Use set('href', ...) instead.", "", "2.0");
		return this.set("href", href);
	},
	_setHrefAttr: function(/*String|Uri*/ href){
		// summary:
		//		Hook so set("href", ...) works.
		// description:
		//		Reset the (external defined) content of this pane and replace with new url
		//		Note: It delays the download until widget is shown if preload is false.
		// href:
		//		url to the page you want to get, must be within the same domain as your mainpage

		// Cancel any in-flight requests (a set('href', ...) will cancel any in-flight set('href', ...))
		this.cancel();

		this.onLoadDeferred = new Deferred(lang.hitch(this, "cancel"));
		this.onLoadDeferred.then(lang.hitch(this, "onLoad"));

		this._set("href", href);

		// _setHrefAttr() is called during creation and by the user, after creation.
		// Assuming preload == false, only in the second case do we actually load the URL;
		// otherwise it's done in startup(), and only if this widget is shown.
		if(this.preload || (this._created && this._isShown())){
			this._load();
		}else{
			// Set flag to indicate that href needs to be loaded the next time the
			// ContentPane is made visible
			this._hrefChanged = true;
		}

		return this.onLoadDeferred;		// Deferred
	},

	setContent: function(/*String|DomNode|Nodelist*/data){
		// summary:
		//		Deprecated.   Use set('content', ...) instead.
		kernel.deprecated("dijit.layout.ContentPane.setContent() is deprecated.  Use set('content', ...) instead.", "", "2.0");
		this.set("content", data);
	},
	_setContentAttr: function(/*String|DomNode|Nodelist*/data){
		// summary:
		//		Hook to make set("content", ...) work.
		//		Replaces old content with data content, include style classes from old content
		// data:
		//		the new Content may be String, DomNode or NodeList
		//
		//		if data is a NodeList (or an array of nodes) nodes are copied
		//		so you can import nodes from another document implicitly

		// clear href so we can't run refresh and clear content
		// refresh should only work if we downloaded the content
		this._set("href", "");

		// Cancel any in-flight requests (a set('content', ...) will cancel any in-flight set('href', ...))
		this.cancel();

		// Even though user is just setting content directly, still need to define an onLoadDeferred
		// because the _onLoadHandler() handler is still getting called from setContent()
		this.onLoadDeferred = new Deferred(lang.hitch(this, "cancel"));
		if(this._created){
			// For back-compat reasons, call onLoad() for set('content', ...)
			// calls but not for content specified in srcNodeRef (ie: <div data-dojo-type=ContentPane>...</div>)
			// or as initialization parameter (ie: new ContentPane({content: ...})
			this.onLoadDeferred.then(lang.hitch(this, "onLoad"));
		}

		this._setContent(data || "");

		this._isDownloaded = false; // mark that content is from a set('content') not a set('href')

		return this.onLoadDeferred;	// Deferred
	},
	_getContentAttr: function(){
		// summary:
		//		Hook to make get("content") work
		return this.containerNode.innerHTML;
	},

	cancel: function(){
		// summary:
		//		Cancels an in-flight download of content
		if(this._xhrDfd && (this._xhrDfd.fired == -1)){
			this._xhrDfd.cancel();
		}
		delete this._xhrDfd; // garbage collect

		this.onLoadDeferred = null;
	},

	destroy: function(){
		this.cancel();
		this.inherited(arguments);
	},

	destroyRecursive: function(/*Boolean*/ preserveDom){
		// summary:
		//		Destroy the ContentPane and its contents

		// if we have multiple controllers destroying us, bail after the first
		if(this._beingDestroyed){
			return;
		}
		this.inherited(arguments);
	},

	_onShow: function(){
		// summary:
		//		Called when the ContentPane is made visible
		// description:
		//		For a plain ContentPane, this is called on initialization, from startup().
		//		If the ContentPane is a hidden pane of a TabContainer etc., then it's
		//		called whenever the pane is made visible.
		//
		//		Does necessary processing, including href download and layout/resize of
		//		child widget(s)

		this.inherited(arguments);

		if(this.href){
			if(!this._xhrDfd && // if there's an href that isn't already being loaded
				(!this.isLoaded || this._hrefChanged || this.refreshOnShow)
			){
				return this.refresh();	// If child has an href, promise that fires when the load is complete
			}
		}
	},

	refresh: function(){
		// summary:
		//		[Re]download contents of href and display
		// description:
		//		1. cancels any currently in-flight requests
		//		2. posts "loading..." message
		//		3. sends XHR to download new data

		// Cancel possible prior in-flight request
		this.cancel();

		this.onLoadDeferred = new Deferred(lang.hitch(this, "cancel"));
		this.onLoadDeferred.then(lang.hitch(this, "onLoad"));
		this._load();
		return this.onLoadDeferred;		// If child has an href, promise that fires when refresh is complete
	},

	_load: function(){
		// summary:
		//		Load/reload the href specified in this.href

		// display loading message
		this._setContent(this.onDownloadStart(), true);

		var self = this;
		var getArgs = {
			preventCache: (this.preventCache || this.refreshOnShow),
			url: this.href,
			handleAs: "text"
		};
		if(lang.isObject(this.ioArgs)){
			lang.mixin(getArgs, this.ioArgs);
		}

		var hand = (this._xhrDfd = (this.ioMethod || xhr.get)(getArgs)),
			returnedHtml;

		hand.then(
			function(html){
				returnedHtml = html;
				try{
					self._isDownloaded = true;
					return self._setContent(html, false);
				}catch(err){
					self._onError('Content', err); // onContentError
				}
			},
			function(err){
				if(!hand.canceled){
					// show error message in the pane
					self._onError('Download', err); // onDownloadError
				}
				delete self._xhrDfd;
				return err;
			}
		).then(function(){
			self.onDownloadEnd();
			delete self._xhrDfd;
			return returnedHtml;
		});

		// Remove flag saying that a load is needed
		delete this._hrefChanged;
	},

	_onLoadHandler: function(data){
		// summary:
		//		This is called whenever new content is being loaded
		this._set("isLoaded", true);
		try{
			this.onLoadDeferred.resolve(data);
		}catch(e){
			console.error('Error '+this.widgetId+' running custom onLoad code: ' + e.message);
		}
	},

	_onUnloadHandler: function(){
		// summary:
		//		This is called whenever the content is being unloaded
		this._set("isLoaded", false);
		try{
			this.onUnload();
		}catch(e){
			console.error('Error '+this.widgetId+' running custom onUnload code: ' + e.message);
		}
	},

	destroyDescendants: function(/*Boolean*/ preserveDom){
		// summary:
		//		Destroy all the widgets inside the ContentPane and empty containerNode

		// Make sure we call onUnload (but only when the ContentPane has real content)
		if(this.isLoaded){
			this._onUnloadHandler();
		}

		// Even if this.isLoaded == false there might still be a "Loading..." message
		// to erase, so continue...

		// For historical reasons we need to delete all widgets under this.containerNode,
		// even ones that the user has created manually.
		var setter = this._contentSetter;
		array.forEach(this.getChildren(), function(widget){
			if(widget.destroyRecursive){
				// All widgets will hit this branch
				widget.destroyRecursive(preserveDom);
			}else if(widget.destroy){
				// Things like dojo/dnd/Source have destroy(), not destroyRecursive()
				widget.destroy(preserveDom);
			}
			widget._destroyed = true;
		});
		if(setter){
			// Most of the widgets in setter.parseResults have already been destroyed, but
			// things like Menu that have been moved to <body> haven't yet
			array.forEach(setter.parseResults, function(widget){
				if(!widget._destroyed){
					if(widget.destroyRecursive){
						// All widgets will hit this branch
						widget.destroyRecursive(preserveDom);
					}else if(widget.destroy){
						// Things like dojo/dnd/Source have destroy(), not destroyRecursive()
						widget.destroy(preserveDom);
					}
					widget._destroyed = true;
				}
			});
			delete setter.parseResults;
		}

		// And then clear away all the DOM nodes
		if(!preserveDom){
			domConstruct.empty(this.containerNode);
		}

		// Delete any state information we have about current contents
		delete this._singleChild;
	},

	_setContent: function(/*String|DocumentFragment*/ cont, /*Boolean*/ isFakeContent){
		// summary:
		//		Insert the content into the container node
		// returns:
		//		Returns a Deferred promise that is resolved when the content is parsed.

		// first get rid of child widgets
		this.destroyDescendants();

		// html.set will take care of the rest of the details
		// we provide an override for the error handling to ensure the widget gets the errors
		// configure the setter instance with only the relevant widget instance properties
		// NOTE: unless we hook into attr, or provide property setters for each property,
		// we need to re-configure the ContentSetter with each use
		var setter = this._contentSetter;
		if(! (setter && setter instanceof html._ContentSetter)){
			setter = this._contentSetter = new html._ContentSetter({
				node: this.containerNode,
				_onError: lang.hitch(this, this._onError),
				onContentError: lang.hitch(this, function(e){
					// fires if a domfault occurs when we are appending this.errorMessage
					// like for instance if domNode is a UL and we try append a DIV
					var errMess = this.onContentError(e);
					try{
						this.containerNode.innerHTML = errMess;
					}catch(e){
						console.error('Fatal '+this.id+' could not change content due to '+e.message, e);
					}
				})/*,
				_onError */
			});
		}

		var setterParams = lang.mixin({
			cleanContent: this.cleanContent,
			extractContent: this.extractContent,
			parseContent: !cont.domNode && this.parseOnLoad,
			parserScope: this.parserScope,
			startup: false,
			dir: this.dir,
			lang: this.lang,
			textDir: this.textDir
		}, this._contentSetterParams || {});

		var p = setter.set( (lang.isObject(cont) && cont.domNode) ? cont.domNode : cont, setterParams );

		// dojox/layout/html/_base::_ContentSetter.set() returns a Promise that indicates when everything is completed.
		// dojo/html::_ContentSetter.set() currently returns the DOMNode, but that will be changed for 2.0.
		// So, if set() returns a promise then use it, otherwise fallback to waiting on setter.parseDeferred
		var self = this;
		return when(p && p.then ? p : setter.parseDeferred, function(){
			// setter params must be pulled afresh from the ContentPane each time
			delete self._contentSetterParams;
			
			if(!isFakeContent){
				if(self._started){
					// Startup each top level child widget (and they will start their children, recursively)
					self._startChildren();
					
					// Call resize() on each of my child layout widgets,
					// or resize() on my single child layout widget...
					// either now (if I'm currently visible) or when I become visible
					self._scheduleLayout();
				}
				self._onLoadHandler(cont);
			}
		});
	},

	_onError: function(type, err, consoleText){
		this.onLoadDeferred.reject(err);

		// shows user the string that is returned by on[type]Error
		// override on[type]Error and return your own string to customize
		var errText = this['on' + type + 'Error'].call(this, err);
		if(consoleText){
			console.error(consoleText, err);
		}else if(errText){// a empty string won't change current content
			this._setContent(errText, true);
		}
	},

	// EVENT's, should be overide-able
	onLoad: function(/*===== data =====*/){
		// summary:
		//		Event hook, is called after everything is loaded and widgetified
		// tags:
		//		callback
	},

	onUnload: function(){
		// summary:
		//		Event hook, is called before old content is cleared
		// tags:
		//		callback
	},

	onDownloadStart: function(){
		// summary:
		//		Called before download starts.
		// description:
		//		The string returned by this function will be the html
		//		that tells the user we are loading something.
		//		Override with your own function if you want to change text.
		// tags:
		//		extension
		return this.loadingMessage;
	},

	onContentError: function(/*Error*/ /*===== error =====*/){
		// summary:
		//		Called on DOM faults, require faults etc. in content.
		//
		//		In order to display an error message in the pane, return
		//		the error message from this method, as an HTML string.
		//
		//		By default (if this method is not overriden), it returns
		//		nothing, so the error message is just printed to the console.
		// tags:
		//		extension
	},

	onDownloadError: function(/*Error*/ /*===== error =====*/){
		// summary:
		//		Called when download error occurs.
		//
		//		In order to display an error message in the pane, return
		//		the error message from this method, as an HTML string.
		//
		//		Default behavior (if this method is not overriden) is to display
		//		the error message inside the pane.
		// tags:
		//		extension
		return this.errorMessage;
	},

	onDownloadEnd: function(){
		// summary:
		//		Called when download is finished.
		// tags:
		//		callback
	}
});

});

},
'dijit/_Container':function(){
define("dijit/_Container", [
	"dojo/_base/array", // array.forEach array.indexOf
	"dojo/_base/declare", // declare
	"dojo/dom-construct" // domConstruct.place
], function(array, declare, domConstruct){

	// module:
	//		dijit/_Container

	return declare("dijit._Container", null, {
		// summary:
		//		Mixin for widgets that contain HTML and/or a set of widget children.

		buildRendering: function(){
			this.inherited(arguments);
			if(!this.containerNode){
				// all widgets with descendants must set containerNode
				this.containerNode = this.domNode;
			}
		},

		addChild: function(/*dijit/_WidgetBase*/ widget, /*int?*/ insertIndex){
			// summary:
			//		Makes the given widget a child of this widget.
			// description:
			//		Inserts specified child widget's dom node as a child of this widget's
			//		container node, and possibly does other processing (such as layout).
			//
			//		Functionality is undefined if this widget contains anything besides
			//		a list of child widgets (ie, if it contains arbitrary non-widget HTML).

			var refNode = this.containerNode;
			if(insertIndex && typeof insertIndex == "number"){
				var children = this.getChildren();
				if(children && children.length >= insertIndex){
					refNode = children[insertIndex-1].domNode;
					insertIndex = "after";
				}
			}
			domConstruct.place(widget.domNode, refNode, insertIndex);

			// If I've been started but the child widget hasn't been started,
			// start it now.  Make sure to do this after widget has been
			// inserted into the DOM tree, so it can see that it's being controlled by me,
			// so it doesn't try to size itself.
			if(this._started && !widget._started){
				widget.startup();
			}
		},

		removeChild: function(/*Widget|int*/ widget){
			// summary:
			//		Removes the passed widget instance from this widget but does
			//		not destroy it.  You can also pass in an integer indicating
			//		the index within the container to remove (ie, removeChild(5) removes the sixth widget).

			if(typeof widget == "number"){
				widget = this.getChildren()[widget];
			}

			if(widget){
				var node = widget.domNode;
				if(node && node.parentNode){
					node.parentNode.removeChild(node); // detach but don't destroy
				}
			}
		},

		hasChildren: function(){
			// summary:
			//		Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.
			return this.getChildren().length > 0;	// Boolean
		},

		_getSiblingOfChild: function(/*dijit/_WidgetBase*/ child, /*int*/ dir){
			// summary:
			//		Get the next or previous widget sibling of child
			// dir:
			//		if 1, get the next sibling
			//		if -1, get the previous sibling
			// tags:
			//		private
			var children = this.getChildren(),
				idx = array.indexOf(this.getChildren(), child);	// int
			return children[idx + dir];
		},

		getIndexOfChild: function(/*dijit/_WidgetBase*/ child){
			// summary:
			//		Gets the index of the child in this container or -1 if not found
			return array.indexOf(this.getChildren(), child);	// int
		}
	});
});

},
'dijit/layout/_ContentPaneResizeMixin':function(){
define("dijit/layout/_ContentPaneResizeMixin", [
	"dojo/_base/array", // array.filter array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-class",	// domClass.contains domClass.toggle
	"dojo/dom-geometry",// domGeometry.contentBox domGeometry.marginBox
	"dojo/dom-style",
	"dojo/_base/lang", // lang.mixin
	"dojo/query", // query
	"dojo/sniff", // has("ie")
	"../registry",	// registry.byId
	"../Viewport",
	"./utils"	// marginBox2contextBox
], function(array, declare, domClass, domGeometry, domStyle, lang, query, has,
			registry, Viewport, layoutUtils){

// module:
//		dijit/layout/_ContentPaneResizeMixin


return declare("dijit.layout._ContentPaneResizeMixin", null, {
	// summary:
	//		Resize() functionality of ContentPane.   If there's a single layout widget
	//		child then it will call resize() with the same dimensions as the ContentPane.
	//		Otherwise just calls resize on each child.
	//
	//		Also implements basic startup() functionality, where starting the parent
	//		will start the children

	// doLayout: Boolean
	//		- false - don't adjust size of children
	//		- true - if there is a single visible child widget, set it's size to however big the ContentPane is
	doLayout: true,

	// isLayoutContainer: [protected] Boolean
	//		Indicates that this widget will call resize() on it's child widgets
	//		when they become visible.
	isLayoutContainer: true,

	startup: function(){
		// summary:
		//		See `dijit/layout/_LayoutWidget.startup()` for description.
		//		Although ContentPane doesn't extend _LayoutWidget, it does implement
		//		the same API.

		if(this._started){ return; }

		var parent = this.getParent();
		this._childOfLayoutWidget = parent && parent.isLayoutContainer;

		// I need to call resize() on my child/children (when I become visible), unless
		// I'm the child of a layout widget in which case my parent will call resize() on me and I'll do it then.
		this._needLayout = !this._childOfLayoutWidget;

		this.inherited(arguments);

		if(this._isShown()){
			this._onShow();
		}

		if(!this._childOfLayoutWidget){
			// Since my parent isn't a layout container, and my style *may be* width=height=100%
			// or something similar (either set directly or via a CSS class),
			// monitor when viewport size changes so that I can re-layout.
			// This is more for subclasses of ContentPane than ContentPane itself, although it
			// could be useful for a ContentPane if it has a single child widget inheriting ContentPane's size.
			this.own(Viewport.on("resize", lang.hitch(this, "resize")));
		}
	},

	_checkIfSingleChild: function(){
		// summary:
		//		Test if we have exactly one visible widget as a child,
		//		and if so assume that we are a container for that widget,
		//		and should propagate startup() and resize() calls to it.
		//		Skips over things like data stores since they aren't visible.

		var candidateWidgets = [],
			otherVisibleNodes = false;

		query("> *", this.containerNode).some(function(node){
			var widget = registry.byNode(node);
			if(widget && widget.resize){
				candidateWidgets.push(widget);
			}else if(node.offsetHeight){
				otherVisibleNodes = true;
			}
		});

		this._singleChild = candidateWidgets.length == 1 && !otherVisibleNodes ?
			candidateWidgets[0] : null;

		// So we can set overflow: hidden to avoid a safari bug w/scrollbars showing up (#9449)
		domClass.toggle(this.containerNode, this.baseClass + "SingleChild", !!this._singleChild);
	},

	resize: function(changeSize, resultSize){
		// summary:
		//		See `dijit/layout/_LayoutWidget.resize()` for description.
		//		Although ContentPane doesn't extend _LayoutWidget, it does implement
		//		the same API.

		this._resizeCalled = true;

		this._scheduleLayout(changeSize, resultSize);
	},

	_scheduleLayout: function(changeSize, resultSize){
		// summary:
		//		Resize myself, and call resize() on each of my child layout widgets, either now
		//		(if I'm currently visible) or when I become visible
		if(this._isShown()){
			this._layout(changeSize, resultSize);
		}else{
			this._needLayout = true;
			this._changeSize = changeSize;
			this._resultSize = resultSize;
		}
	},

	_layout: function(changeSize, resultSize){
		// summary:
		//		Resize myself according to optional changeSize/resultSize parameters, like a layout widget.
		//		Also, since I am an isLayoutContainer widget, each of my children expects me to
		//		call resize() or layout() on it.
		//
		//		Should be called on initialization and also whenever we get new content
		//		(from an href, or from set('content', ...))... but deferred until
		//		the ContentPane is visible

		delete this._needLayout;

		// For the TabContainer --> BorderContainer --> ContentPane case, _onShow() is
		// never called directly, so resize() is our trigger to do the initial href download (see [20099]).
		// However, don't load href for closed TitlePanes.
		if(!this._wasShown && this.open !== false){
			this._onShow();
		}

		// Set margin box size, unless it wasn't specified, in which case use current size.
		if(changeSize){
			domGeometry.setMarginBox(this.domNode, changeSize);
		}

		// Compute content box size of containerNode in case we [later] need to size our single child.
		var cn = this.containerNode;
		if(cn === this.domNode){
			// If changeSize or resultSize was passed to this method and this.containerNode ==
			// this.domNode then we can compute the content-box size without querying the node,
			// which is more reliable (similar to LayoutWidget.resize) (see for example #9449).
			var mb = resultSize || {};
			lang.mixin(mb, changeSize || {}); // changeSize overrides resultSize
			if(!("h" in mb) || !("w" in mb)){
				mb = lang.mixin(domGeometry.getMarginBox(cn), mb); // just use domGeometry.setMarginBox() to fill in missing values
			}
			this._contentBox = layoutUtils.marginBox2contentBox(cn, mb);
		}else{
			this._contentBox = domGeometry.getContentBox(cn);
		}

		this._layoutChildren();
	},

	_layoutChildren: function(){
		// Call _checkIfSingleChild() again in case app has manually mucked w/the content
		// of the ContentPane (rather than changing it through the set("content", ...) API.
		if(this.doLayout){
			this._checkIfSingleChild();
		}

		if(this._singleChild && this._singleChild.resize){
			var cb = this._contentBox || domGeometry.getContentBox(this.containerNode);

			// note: if widget has padding this._contentBox will have l and t set,
			// but don't pass them to resize() or it will doubly-offset the child
			this._singleChild.resize({w: cb.w, h: cb.h});
		}else{
			// All my child widgets are independently sized (rather than matching my size),
			// but I still need to call resize() on each child to make it layout.
			array.forEach(this.getChildren(), function(widget){
				if(widget.resize){
					widget.resize();
				}
			});
		}
	},

	_isShown: function(){
		// summary:
		//		Returns true if the content is currently shown.
		// description:
		//		If I am a child of a layout widget then it actually returns true if I've ever been visible,
		//		not whether I'm currently visible, since that's much faster than tracing up the DOM/widget
		//		tree every call, and at least solves the performance problem on page load by deferring loading
		//		hidden ContentPanes until they are first shown

		if(this._childOfLayoutWidget){
			// If we are TitlePane, etc - we return that only *IF* we've been resized
			if(this._resizeCalled && "open" in this){
				return this.open;
			}
			return this._resizeCalled;
		}else if("open" in this){
			return this.open;		// for TitlePane, etc.
		}else{
			var node = this.domNode, parent = this.domNode.parentNode;
			return (node.style.display != 'none') && (node.style.visibility != 'hidden') && !domClass.contains(node, "dijitHidden") &&
					parent && parent.style && (parent.style.display != 'none');
		}
	},

	_onShow: function(){
		// summary:
		//		Called when the ContentPane is made visible
		// description:
		//		For a plain ContentPane, this is called on initialization, from startup().
		//		If the ContentPane is a hidden pane of a TabContainer etc., then it's
		//		called whenever the pane is made visible.
		//
		//		Does layout/resize of child widget(s)

		// Need to keep track of whether ContentPane has been shown (which is different than
		// whether or not it's currently visible).
		this._wasShown = true;

		if(this._needLayout){
			// If a layout has been scheduled for when we become visible, do it now
			this._layout(this._changeSize, this._resultSize);
		}

		this.inherited(arguments);
	}
});

});

},
'dijit/Viewport':function(){
define("dijit/Viewport", [
	"dojo/Evented",
	"dojo/on",
	"dojo/ready",
	"dojo/sniff",
	"dojo/_base/window", // global
	"dojo/window" // getBox()
], function(Evented, on, ready, has, win, winUtils){

	// module:
	//		dijit/Viewport

	/*=====
	return {
		// summary:
		//		Utility singleton to watch for viewport resizes, avoiding duplicate notifications
		//		which can lead to infinite loops.
		// description:
		//		Usage: Viewport.on("resize", myCallback).
		//
		//		myCallback() is called without arguments in case it's _WidgetBase.resize(),
		//		which would interpret the argument as the size to make the widget.
	};
	=====*/

	var Viewport = new Evented();

	var focusedNode;

	ready(200, function(){
		var oldBox = winUtils.getBox();
		Viewport._rlh = on(win.global, "resize", function(){
			var newBox = winUtils.getBox();
			if(oldBox.h == newBox.h && oldBox.w == newBox.w){ return; }
			oldBox = newBox;
			Viewport.emit("resize");
		});

		// Also catch zoom changes on IE8, since they don't naturally generate resize events
		if(has("ie") == 8){
			var deviceXDPI = screen.deviceXDPI;
			setInterval(function(){
				if(screen.deviceXDPI != deviceXDPI){
					deviceXDPI = screen.deviceXDPI;
					Viewport.emit("resize");
				}
			}, 500);
		}

		// On iOS, keep track of the focused node so we can guess when the keyboard is/isn't being displayed.
		if(has("ios")){
			on(document, "focusin", function(evt){
				focusedNode = evt.target;
			});
			on(document, "focusout", function(evt){
				focusedNode = null;
			});
		}
	});

	Viewport.getEffectiveBox = function(/*Document*/ doc){
		// summary:
		//		Get the size of the viewport, or on mobile devices, the part of the viewport not obscured by the
		//		virtual keyboard.

		var box = winUtils.getBox(doc);

		// Account for iOS virtual keyboard, if it's being shown.  Unfortunately no direct way to check or measure.
		var tag = focusedNode && focusedNode.tagName && focusedNode.tagName.toLowerCase();
		if(has("ios") && focusedNode && !focusedNode.readOnly && (tag == "textarea" || (tag == "input" &&
			/^(color|email|number|password|search|tel|text|url)$/.test(focusedNode.type)))){

			// Box represents the size of the viewport.  Some of the viewport is likely covered by the keyboard.
			// Estimate height of visible viewport assuming viewport goes to bottom of screen, but is covered by keyboard.
			box.h *= (orientation == 0 || orientation == 180 ? 0.66 : 0.40);

			// Above measurement will be inaccurate if viewport was scrolled up so far that it ends before the bottom
			// of the screen.   In this case, keyboard isn't covering as much of the viewport as we thought.
			// We know the visible size is at least the distance from the top of the viewport to the focused node.
			var rect = focusedNode.getBoundingClientRect();
			box.h = Math.max(box.h, rect.top + rect.height);
		}

		return box;
	};

	return Viewport;
});

},
'dijit/layout/utils':function(){
define("dijit/layout/utils", [
	"dojo/_base/array", // array.filter array.forEach
	"dojo/dom-class", // domClass.add domClass.remove
	"dojo/dom-geometry", // domGeometry.marginBox
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/_base/lang", // lang.mixin
	"../main"	// for exporting symbols to dijit, remove in 2.0
], function(array, domClass, domGeometry, domStyle, lang, dijit){

	// module:
	//		dijit/layout/utils

	var layout = lang.getObject("layout", true, dijit);
	/*=====
	layout = {
		 // summary:
		 //		marginBox2contentBox() and layoutChildren()
	 };
	 =====*/

	layout.marginBox2contentBox = function(/*DomNode*/ node, /*Object*/ mb){
		// summary:
		//		Given the margin-box size of a node, return its content box size.
		//		Functions like domGeometry.contentBox() but is more reliable since it doesn't have
		//		to wait for the browser to compute sizes.
		var cs = domStyle.getComputedStyle(node);
		var me = domGeometry.getMarginExtents(node, cs);
		var pb = domGeometry.getPadBorderExtents(node, cs);
		return {
			l: domStyle.toPixelValue(node, cs.paddingLeft),
			t: domStyle.toPixelValue(node, cs.paddingTop),
			w: mb.w - (me.w + pb.w),
			h: mb.h - (me.h + pb.h)
		};
	};

	function capitalize(word){
		return word.substring(0,1).toUpperCase() + word.substring(1);
	}

	function size(widget, dim){
		// size the child
		var newSize = widget.resize ? widget.resize(dim) : domGeometry.setMarginBox(widget.domNode, dim);

		// record child's size
		if(newSize){
			// if the child returned it's new size then use that
			lang.mixin(widget, newSize);
		}else{
			// otherwise, call getMarginBox(), but favor our own numbers when we have them.
			// the browser lies sometimes
			lang.mixin(widget, domGeometry.getMarginBox(widget.domNode));
			lang.mixin(widget, dim);
		}
	}

	layout.layoutChildren = function(/*DomNode*/ container, /*Object*/ dim, /*Widget[]*/ children,
			/*String?*/ changedRegionId, /*Number?*/ changedRegionSize){
		// summary:
		//		Layout a bunch of child dom nodes within a parent dom node
		// container:
		//		parent node
		// dim:
		//		{l, t, w, h} object specifying dimensions of container into which to place children
		// children:
		//		An array of Widgets or at least objects containing:
		//
		//		- domNode: pointer to DOM node to position
		//		- region or layoutAlign: position to place DOM node
		//		- resize(): (optional) method to set size of node
		//		- id: (optional) Id of widgets, referenced from resize object, below.
		// changedRegionId:
		//		If specified, the slider for the region with the specified id has been dragged, and thus
		//		the region's height or width should be adjusted according to changedRegionSize
		// changedRegionSize:
		//		See changedRegionId.

		// copy dim because we are going to modify it
		dim = lang.mixin({}, dim);

		domClass.add(container, "dijitLayoutContainer");

		// Move "client" elements to the end of the array for layout.  a11y dictates that the author
		// needs to be able to put them in the document in tab-order, but this algorithm requires that
		// client be last.    TODO: move these lines to LayoutContainer?   Unneeded other places I think.
		children = array.filter(children, function(item){ return item.region != "center" && item.layoutAlign != "client"; })
			.concat(array.filter(children, function(item){ return item.region == "center" || item.layoutAlign == "client"; }));

		// set positions/sizes
		array.forEach(children, function(child){
			var elm = child.domNode,
				pos = (child.region || child.layoutAlign);
			if(!pos){
				throw new Error("No region setting for " + child.id)
			}

			// set elem to upper left corner of unused space; may move it later
			var elmStyle = elm.style;
			elmStyle.left = dim.l+"px";
			elmStyle.top = dim.t+"px";
			elmStyle.position = "absolute";

			domClass.add(elm, "dijitAlign" + capitalize(pos));

			// Size adjustments to make to this child widget
			var sizeSetting = {};

			// Check for optional size adjustment due to splitter drag (height adjustment for top/bottom align
			// panes and width adjustment for left/right align panes.
			if(changedRegionId && changedRegionId == child.id){
				sizeSetting[child.region == "top" || child.region == "bottom" ? "h" : "w"] = changedRegionSize;
			}

			// set size && adjust record of remaining space.
			// note that setting the width of a <div> may affect its height.
			if(pos == "top" || pos == "bottom"){
				sizeSetting.w = dim.w;
				size(child, sizeSetting);
				dim.h -= child.h;
				if(pos == "top"){
					dim.t += child.h;
				}else{
					elmStyle.top = dim.t + dim.h + "px";
				}
			}else if(pos == "left" || pos == "right"){
				sizeSetting.h = dim.h;
				size(child, sizeSetting);
				dim.w -= child.w;
				if(pos == "left"){
					dim.l += child.w;
				}else{
					elmStyle.left = dim.l + dim.w + "px";
				}
			}else if(pos == "client" || pos == "center"){
				size(child, dim);
			}
		});
	};


	return {
		marginBox2contentBox: layout.marginBox2contentBox,
		layoutChildren: layout.layoutChildren
	};
});

},
'dojo/html':function(){
define("dojo/html", ["./_base/kernel", "./_base/lang", "./_base/array", "./_base/declare", "./dom", "./dom-construct", "./parser"],
	function(kernel, lang, darray, declare, dom, domConstruct, parser){
	// module:
	//		dojo/html

	var html = {
		// summary:
		//		TODOC
	};
	lang.setObject("dojo.html", html);

	// the parser might be needed..

	// idCounter is incremented with each instantiation to allow assignment of a unique id for tracking, logging purposes
	var idCounter = 0;

	html._secureForInnerHtml = function(/*String*/ cont){
		// summary:
		//		removes !DOCTYPE and title elements from the html string.
		//
		//		khtml is picky about dom faults, you can't attach a style or `<title>` node as child of body
		//		must go into head, so we need to cut out those tags
		// cont:
		//		An html string for insertion into the dom
		//
		return cont.replace(/(?:\s*<!DOCTYPE\s[^>]+>|<title[^>]*>[\s\S]*?<\/title>)/ig, ""); // String
	};

	html._emptyNode = domConstruct.empty;
	/*=====
	 dojo.html._emptyNode = function(node){
		 // summary:
		 //		Removes all child nodes from the given node.   Deprecated, should use dojo/dom-constuct.empty() directly
		 //		instead.
		 // node: DOMNode
		 //		the parent element
	 };
	 =====*/

		html._setNodeContent = function(/*DomNode*/ node, /*String|DomNode|NodeList*/ cont){
		// summary:
		//		inserts the given content into the given node
		// node:
		//		the parent element
		// content:
		//		the content to be set on the parent element.
		//		This can be an html string, a node reference or a NodeList, dojo/NodeList, Array or other enumerable list of nodes

		// always empty
		domConstruct.empty(node);

		if(cont){
			if(typeof cont == "string"){
				cont = domConstruct.toDom(cont, node.ownerDocument);
			}
			if(!cont.nodeType && lang.isArrayLike(cont)){
				// handle as enumerable, but it may shrink as we enumerate it
				for(var startlen=cont.length, i=0; i<cont.length; i=startlen==cont.length ? i+1 : 0){
					domConstruct.place( cont[i], node, "last");
				}
			}else{
				// pass nodes, documentFragments and unknowns through to dojo.place
				domConstruct.place(cont, node, "last");
			}
		}

		// return DomNode
		return node;
	};

	// we wrap up the content-setting operation in a object
	html._ContentSetter = declare("dojo.html._ContentSetter", null,
		{
			// node: DomNode|String
			//		An node which will be the parent element that we set content into
			node: "",

			// content: String|DomNode|DomNode[]
			//		The content to be placed in the node. Can be an HTML string, a node reference, or a enumerable list of nodes
			content: "",

			// id: String?
			//		Usually only used internally, and auto-generated with each instance
			id: "",

			// cleanContent: Boolean
			//		Should the content be treated as a full html document,
			//		and the real content stripped of <html>, <body> wrapper before injection
			cleanContent: false,

			// extractContent: Boolean
			//		Should the content be treated as a full html document,
			//		and the real content stripped of `<html> <body>` wrapper before injection
			extractContent: false,

			// parseContent: Boolean
			//		Should the node by passed to the parser after the new content is set
			parseContent: false,

			// parserScope: String
			//		Flag passed to parser.	Root for attribute names to search for.	  If scopeName is dojo,
			//		will search for data-dojo-type (or dojoType).  For backwards compatibility
			//		reasons defaults to dojo._scopeName (which is "dojo" except when
			//		multi-version support is used, when it will be something like dojo16, dojo20, etc.)
			parserScope: kernel._scopeName,

			// startup: Boolean
			//		Start the child widgets after parsing them.	  Only obeyed if parseContent is true.
			startup: true,

			// lifecycle methods
			constructor: function(/*Object*/ params, /*String|DomNode*/ node){
				// summary:
				//		Provides a configurable, extensible object to wrap the setting on content on a node
				//		call the set() method to actually set the content..

				// the original params are mixed directly into the instance "this"
				lang.mixin(this, params || {});

				// give precedence to params.node vs. the node argument
				// and ensure its a node, not an id string
				node = this.node = dom.byId( this.node || node );

				if(!this.id){
					this.id = [
						"Setter",
						(node) ? node.id || node.tagName : "",
						idCounter++
					].join("_");
				}
			},
			set: function(/* String|DomNode|NodeList? */ cont, /*Object?*/ params){
				// summary:
				//		front-end to the set-content sequence
				// cont:
				//		An html string, node or enumerable list of nodes for insertion into the dom
				//		If not provided, the object's content property will be used
				if(undefined !== cont){
					this.content = cont;
				}
				// in the re-use scenario, set needs to be able to mixin new configuration
				if(params){
					this._mixin(params);
				}

				this.onBegin();
				this.setContent();

				var ret = this.onEnd();

				if(ret && ret.then){
					// Make dojox/html/_ContentSetter.set() return a Promise that resolves when load and parse complete.
					return ret;
				}else{
					// Vanilla dojo/html._ContentSetter.set() returns a DOMNode for back compat.   For 2.0, switch it to
					// return a Deferred like above.
					return this.node;
				}
			},

			setContent: function(){
				// summary:
				//		sets the content on the node

				var node = this.node;
				if(!node){
					// can't proceed
					throw new Error(this.declaredClass + ": setContent given no node");
				}
				try{
					node = html._setNodeContent(node, this.content);
				}catch(e){
					// check if a domfault occurs when we are appending this.errorMessage
					// like for instance if domNode is a UL and we try append a DIV

					// FIXME: need to allow the user to provide a content error message string
					var errMess = this.onContentError(e);
					try{
						node.innerHTML = errMess;
					}catch(e){
						console.error('Fatal ' + this.declaredClass + '.setContent could not change content due to '+e.message, e);
					}
				}
				// always put back the node for the next method
				this.node = node; // DomNode
			},

			empty: function(){
				// summary:
				//		cleanly empty out existing content
				
				// If there is a parse in progress, cancel it.
				if(this.parseDeferred){
					if(!this.parseDeferred.isResolved()){
						this.parseDeferred.cancel();
					}
					delete this.parseDeferred;
				}

				// destroy any widgets from a previous run
				// NOTE: if you don't want this you'll need to empty
				// the parseResults array property yourself to avoid bad things happening
				if(this.parseResults && this.parseResults.length){
					darray.forEach(this.parseResults, function(w){
						if(w.destroy){
							w.destroy();
						}
					});
					delete this.parseResults;
				}
				// this is fast, but if you know its already empty or safe, you could
				// override empty to skip this step
				domConstruct.empty(this.node);
			},

			onBegin: function(){
				// summary:
				//		Called after instantiation, but before set();
				//		It allows modification of any of the object properties -
				//		including the node and content provided - before the set operation actually takes place
				//		This default implementation checks for cleanContent and extractContent flags to
				//		optionally pre-process html string content
				var cont = this.content;

				if(lang.isString(cont)){
					if(this.cleanContent){
						cont = html._secureForInnerHtml(cont);
					}

					if(this.extractContent){
						var match = cont.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
						if(match){ cont = match[1]; }
					}
				}

				// clean out the node and any cruft associated with it - like widgets
				this.empty();

				this.content = cont;
				return this.node; // DomNode
			},

			onEnd: function(){
				// summary:
				//		Called after set(), when the new content has been pushed into the node
				//		It provides an opportunity for post-processing before handing back the node to the caller
				//		This default implementation checks a parseContent flag to optionally run the dojo parser over the new content
				if(this.parseContent){
					// populates this.parseResults and this.parseDeferred if you need those..
					this._parse();
				}
				return this.node; // DomNode
				// TODO: for 2.0 return a Promise indicating that the parse completed.
			},

			tearDown: function(){
				// summary:
				//		manually reset the Setter instance if its being re-used for example for another set()
				// description:
				//		tearDown() is not called automatically.
				//		In normal use, the Setter instance properties are simply allowed to fall out of scope
				//		but the tearDown method can be called to explicitly reset this instance.
				delete this.parseResults;
				delete this.parseDeferred;
				delete this.node;
				delete this.content;
			},

			onContentError: function(err){
				return "Error occurred setting content: " + err;
			},

			onExecError: function(err){
				return "Error occurred executing scripts: " + err;
			},

			_mixin: function(params){
				// mix properties/methods into the instance
				// TODO: the intention with tearDown is to put the Setter's state
				// back to that of the original constructor (vs. deleting/resetting everything regardless of ctor params)
				// so we could do something here to move the original properties aside for later restoration
				var empty = {}, key;
				for(key in params){
					if(key in empty){ continue; }
					// TODO: here's our opportunity to mask the properties we don't consider configurable/overridable
					// .. but history shows we'll almost always guess wrong
					this[key] = params[key];
				}
			},
			_parse: function(){
				// summary:
				//		runs the dojo parser over the node contents, storing any results in this.parseResults
				//		and the parse promise in this.parseDeferred
				//		Any errors resulting from parsing are passed to _onError for handling

				var rootNode = this.node;
				try{
					// store the results (widgets, whatever) for potential retrieval
					var inherited = {};
					darray.forEach(["dir", "lang", "textDir"], function(name){
						if(this[name]){
							inherited[name] = this[name];
						}
					}, this);
					var self = this;
					this.parseDeferred = parser.parse({
						rootNode: rootNode,
						noStart: !this.startup,
						inherited: inherited,
						scope: this.parserScope
					}).then(function(results){
						return self.parseResults = results;
					}, function(e){
						self._onError('Content', e, "Error parsing in _ContentSetter#"+this.id);
					});
				}catch(e){
					this._onError('Content', e, "Error parsing in _ContentSetter#"+this.id);
				}
			},

			_onError: function(type, err, consoleText){
				// summary:
				//		shows user the string that is returned by on[type]Error
				//		override/implement on[type]Error and return your own string to customize
				var errText = this['on' + type + 'Error'].call(this, err);
				if(consoleText){
					console.error(consoleText, err);
				}else if(errText){ // a empty string won't change current content
					html._setNodeContent(this.node, errText, true);
				}
			}
	}); // end declare()

	html.set = function(/*DomNode*/ node, /*String|DomNode|NodeList*/ cont, /*Object?*/ params){
			// summary:
			//		inserts (replaces) the given content into the given node. dojo.place(cont, node, "only")
			//		may be a better choice for simple HTML insertion.
			// description:
			//		Unless you need to use the params capabilities of this method, you should use
			//		dojo.place(cont, node, "only"). dojo.place() has more robust support for injecting
			//		an HTML string into the DOM, but it only handles inserting an HTML string as DOM
			//		elements, or inserting a DOM node. dojo.place does not handle NodeList insertions
			//		or the other capabilities as defined by the params object for this method.
			// node:
			//		the parent element that will receive the content
			// cont:
			//		the content to be set on the parent element.
			//		This can be an html string, a node reference or a NodeList, dojo/NodeList, Array or other enumerable list of nodes
			// params:
			//		Optional flags/properties to configure the content-setting. See dojo/html/_ContentSetter
			// example:
			//		A safe string/node/nodelist content replacement/injection with hooks for extension
			//		Example Usage:
			//	|	html.set(node, "some string");
			//	|	html.set(node, contentNode, {options});
			//	|	html.set(node, myNode.childNodes, {options});
		if(undefined == cont){
			console.warn("dojo.html.set: no cont argument provided, using empty string");
			cont = "";
		}
		if(!params){
			// simple and fast
			return html._setNodeContent(node, cont, true);
		}else{
			// more options but slower
			// note the arguments are reversed in order, to match the convention for instantiation via the parser
			var op = new html._ContentSetter(lang.mixin(
					params,
					{ content: cont, node: node }
			));
			return op.set();
		}
	};

	return html;
});

},
'dojo/parser':function(){
define(
	"dojo/parser", ["require", "./_base/kernel", "./_base/lang", "./_base/array", "./_base/config", "./_base/html", "./_base/window",
		"./_base/url", "./_base/json", "./aspect", "./date/stamp", "./Deferred", "./has", "./query", "./on", "./ready"],
	function(require, dojo, dlang, darray, config, dhtml, dwindow, _Url, djson, aspect, dates, Deferred, has, query, don, ready){

	// module:
	//		dojo/parser

	new Date("X"); // workaround for #11279, new Date("") == NaN


	// Widgets like BorderContainer add properties to _Widget via dojo.extend().
	// If BorderContainer is loaded after _Widget's parameter list has been cached,
	// we need to refresh that parameter list (for _Widget and all widgets that extend _Widget).
	var extendCnt = 0;
	aspect.after(dlang, "extend", function(){
		extendCnt++;
	}, true);

	function getNameMap(ctor){
		// summary:
		//		Returns map from lowercase name to attribute name in class, ex: {onclick: "onClick"}
		var map = ctor._nameCaseMap, proto = ctor.prototype;

		// Create the map if it's undefined.
		// Refresh the map if a superclass was possibly extended with new methods since the map was created.
		if(!map || map._extendCnt < extendCnt){
			map = ctor._nameCaseMap = {};
			for(var name in proto){
				if(name.charAt(0) === "_"){ continue; }	// skip internal properties
				map[name.toLowerCase()] = name;
			}
			map._extendCnt = extendCnt;
		}
		return map;
	}

	// Map from widget name or list of widget names(ex: "dijit/form/Button,acme/MyMixin") to a constructor.
	var _ctorMap = {};

	function getCtor(/*String[]*/ types){
		// summary:
		//		Retrieves a constructor.  If the types array contains more than one class/MID then the
		//		subsequent classes will be mixed into the first class and a unique constructor will be
		//		returned for that array.

		var ts = types.join();
		if(!_ctorMap[ts]){
			var mixins = [];
			for(var i = 0, l = types.length; i < l; i++){
				var t = types[i];
				// TODO: Consider swapping getObject and require in the future
				mixins[mixins.length] = (_ctorMap[t] = _ctorMap[t] || (dlang.getObject(t) || (~t.indexOf('/') && require(t))));
			}
			var ctor = mixins.shift();
			_ctorMap[ts] = mixins.length ? (ctor.createSubclass ? ctor.createSubclass(mixins) : ctor.extend.apply(ctor, mixins)) : ctor;
		}

		return _ctorMap[ts];
	}

	var parser = {
		// summary:
		//		The Dom/Widget parsing package

		_clearCache: function(){
			// summary:
			//		Clear cached data.   Used mainly for benchmarking.
			extendCnt++;
			_ctorMap = {};
		},

		_functionFromScript: function(script, attrData){
			// summary:
			//		Convert a `<script type="dojo/method" args="a, b, c"> ... </script>`
			//		into a function
			// script: DOMNode
			//		The `<script>` DOMNode
			// attrData: String
			//		For HTML5 compliance, searches for attrData + "args" (typically
			//		"data-dojo-args") instead of "args"
			var preamble = "",
				suffix = "",
				argsStr = (script.getAttribute(attrData + "args") || script.getAttribute("args")),
				withStr = script.getAttribute("with");

			// Convert any arguments supplied in script tag into an array to be passed to the
			var fnArgs = (argsStr || "").split(/\s*,\s*/);

			if(withStr && withStr.length){
				darray.forEach(withStr.split(/\s*,\s*/), function(part){
					preamble += "with("+part+"){";
					suffix += "}";
				});
			}

			return new Function(fnArgs, preamble + script.innerHTML + suffix);
		},

		instantiate: function(nodes, mixin, options){
			// summary:
			//		Takes array of nodes, and turns them into class instances and
			//		potentially calls a startup method to allow them to connect with
			//		any children.
			// nodes: Array
			//		Array of DOM nodes
			// mixin: Object?
			//		An object that will be mixed in with each node in the array.
			//		Values in the mixin will override values in the node, if they
			//		exist.
			// options: Object?
			//		An object used to hold kwArgs for instantiation.
			//		See parse.options argument for details.

			mixin = mixin || {};
			options = options || {};

			var dojoType = (options.scope || dojo._scopeName) + "Type",		// typically "dojoType"
				attrData = "data-" + (options.scope || dojo._scopeName) + "-",// typically "data-dojo-"
				dataDojoType = attrData + "type",						// typically "data-dojo-type"
				dataDojoMixins = attrData + "mixins";					// typically "data-dojo-mixins"

			var list = [];
			darray.forEach(nodes, function(node){
				var type = dojoType in mixin ? mixin[dojoType] : node.getAttribute(dataDojoType) || node.getAttribute(dojoType);
				if(type){
					var mixinsValue = node.getAttribute(dataDojoMixins),
						types = mixinsValue ? [type].concat(mixinsValue.split(/\s*,\s*/)) : [type];

					list.push({
						node: node,
						types: types
					});
				}
			});

			// Instantiate the nodes and return the objects
			return this._instantiate(list, mixin, options);
		},

		_instantiate: function(nodes, mixin, options){
			// summary:
			//		Takes array of objects representing nodes, and turns them into class instances and
			//		potentially calls a startup method to allow them to connect with
			//		any children.
			// nodes: Array
			//		Array of objects like
			//	|		{
			//	|			ctor: Function (may be null)
			//	|			types: ["dijit/form/Button", "acme/MyMixin"] (used if ctor not specified)
			//	|			node: DOMNode,
			//	|			scripts: [ ... ],	// array of <script type="dojo/..."> children of node
			//	|			inherited: { ... }	// settings inherited from ancestors like dir, theme, etc.
			//	|		}
			// mixin: Object
			//		An object that will be mixed in with each node in the array.
			//		Values in the mixin will override values in the node, if they
			//		exist.
			// options: Object
			//		An options object used to hold kwArgs for instantiation.
			//		See parse.options argument for details.

			// Call widget constructors
			var thelist = darray.map(nodes, function(obj){
				var ctor = obj.ctor || getCtor(obj.types);
				// If we still haven't resolved a ctor, it is fatal now
				if(!ctor){
					throw new Error("Unable to resolve constructor for: '" + obj.types.join() + "'");
				}
				return this.construct(ctor, obj.node, mixin, options, obj.scripts, obj.inherited);
			}, this);

			// Call startup on each top level instance if it makes sense (as for
			// widgets).  Parent widgets will recursively call startup on their
			// (non-top level) children
			if(!mixin._started && !options.noStart){
				darray.forEach(thelist, function(instance){
					if(typeof instance.startup === "function" && !instance._started){
						instance.startup();
					}
				});
			}

			return thelist;
		},

		construct: function(ctor, node, mixin, options, scripts, inherited){
			// summary:
			//		Calls new ctor(params, node), where params is the hash of parameters specified on the node,
			//		excluding data-dojo-type and data-dojo-mixins.   Does not call startup().   Returns the widget.
			// ctor: Function
			//		Widget constructor.
			// node: DOMNode
			//		This node will be replaced/attached to by the widget.  It also specifies the arguments to pass to ctor.
			// mixin: Object?
			//		Attributes in this object will be passed as parameters to ctor,
			//		overriding attributes specified on the node.
			// options: Object?
			//		An options object used to hold kwArgs for instantiation.   See parse.options argument for details.
			// scripts: DomNode[]?
			//		Array of `<script type="dojo/*">` DOMNodes.  If not specified, will search for `<script>` tags inside node.
			// inherited: Object?
			//		Settings from dir=rtl or lang=... on a node above this node.   Overrides options.inherited.

			var proto = ctor && ctor.prototype;
			options = options || {};

			// Setup hash to hold parameter settings for this widget.	Start with the parameter
			// settings inherited from ancestors ("dir" and "lang").
			// Inherited setting may later be overridden by explicit settings on node itself.
			var params = {};

			if(options.defaults){
				// settings for the document itself (or whatever subtree is being parsed)
				dlang.mixin(params, options.defaults);
			}
			if(inherited){
				// settings from dir=rtl or lang=... on a node above this node
				dlang.mixin(params, inherited);
			}

			// Get list of attributes explicitly listed in the markup
			var attributes;
			if(has("dom-attributes-explicit")){
				// Standard path to get list of user specified attributes
				attributes = node.attributes;
			}else if(has("dom-attributes-specified-flag")){
				// Special processing needed for IE8, to skip a few faux values in attributes[]
				attributes = darray.filter(node.attributes, function(a){ return a.specified;});
			}else{
				// Special path for IE6-7, avoid (sometimes >100) bogus entries in node.attributes
				var clone = /^input$|^img$/i.test(node.nodeName) ? node : node.cloneNode(false),
					attrs = clone.outerHTML.replace(/=[^\s"']+|="[^"]*"|='[^']*'/g, "").replace(/^\s*<[a-zA-Z0-9]*\s*/, "").replace(/\s*>.*$/, "");

				attributes = darray.map(attrs.split(/\s+/), function(name){
					var lcName = name.toLowerCase();
					return {
						name: name,
						// getAttribute() doesn't work for button.value, returns innerHTML of button.
						// but getAttributeNode().value doesn't work for the form.encType or li.value
						value: (node.nodeName == "LI" && name == "value") || lcName == "enctype" ?
								node.getAttribute(lcName) : node.getAttributeNode(lcName).value
					};
				});
			}

			// Hash to convert scoped attribute name (ex: data-dojo17-params) to something friendly (ex: data-dojo-params)
			// TODO: remove scope for 2.0
			var scope = options.scope || dojo._scopeName,
				attrData = "data-" + scope + "-",						// typically "data-dojo-"
				hash = {};
			if(scope !== "dojo"){
				hash[attrData + "props"] = "data-dojo-props";
				hash[attrData + "type"] = "data-dojo-type";
				hash[attrData + "mixins"] = "data-dojo-mixins";
				hash[scope + "type"] = "dojoType";
				hash[attrData + "id"] = "data-dojo-id";
			}

			// Read in attributes and process them, including data-dojo-props, data-dojo-type,
			// dojoAttachPoint, etc., as well as normal foo=bar attributes.
			var i=0, item, funcAttrs=[], jsname, extra;
			while(item = attributes[i++]){
				var name = item.name,
					lcName = name.toLowerCase(),
					value = item.value;

				switch(hash[lcName] || lcName){
				// Already processed, just ignore
				case "data-dojo-type":
				case "dojotype":
				case "data-dojo-mixins":
					break;

				// Data-dojo-props.   Save for later to make sure it overrides direct foo=bar settings
				case "data-dojo-props":
					extra = value;
					break;

				// data-dojo-id or jsId. TODO: drop jsId in 2.0
				case "data-dojo-id":
				case "jsid":
					jsname = value;
					break;

				// For the benefit of _Templated
				case "data-dojo-attach-point":
				case "dojoattachpoint":
					params.dojoAttachPoint = value;
					break;
				case "data-dojo-attach-event":
				case "dojoattachevent":
					params.dojoAttachEvent = value;
					break;

				// Special parameter handling needed for IE
				case "class":
					params["class"] = node.className;
					break;
				case "style":
					params["style"] = node.style && node.style.cssText;
					break;
				default:
					// Normal attribute, ex: value="123"

					// Find attribute in widget corresponding to specified name.
					// May involve case conversion, ex: onclick --> onClick
					if(!(name in proto)){
						var map = getNameMap(ctor);
						name = map[lcName] || name;
					}

					// Set params[name] to value, doing type conversion
					if(name in proto){
						switch(typeof proto[name]){
						case "string":
							params[name] = value;
							break;
						case "number":
							params[name] = value.length ? Number(value) : NaN;
							break;
						case "boolean":
							// for checked/disabled value might be "" or "checked".	 interpret as true.
							params[name] = value.toLowerCase() != "false";
							break;
						case "function":
							if(value === "" || value.search(/[^\w\.]+/i) != -1){
								// The user has specified some text for a function like "return x+5"
								params[name] = new Function(value);
							}else{
								// The user has specified the name of a global function like "myOnClick"
								// or a single word function "return"
								params[name] = dlang.getObject(value, false) || new Function(value);
							}
							funcAttrs.push(name);	// prevent "double connect", see #15026
							break;
						default:
							var pVal = proto[name];
							params[name] =
								(pVal && "length" in pVal) ? (value ? value.split(/\s*,\s*/) : []) :	// array
									(pVal instanceof Date) ?
										(value == "" ? new Date("") :	// the NaN of dates
										value == "now" ? new Date() :	// current date
										dates.fromISOString(value)) :
								(pVal instanceof _Url) ? (dojo.baseUrl + value) :
								djson.fromJson(value);
						}
					}else{
						params[name] = value;
					}
				}
			}

			// Remove function attributes from DOMNode to prevent "double connect" problem, see #15026.
			// Do this as a separate loop since attributes[] is often a live collection (depends on the browser though).
			for(var j=0; j<funcAttrs.length; j++){
				var lcfname = funcAttrs[j].toLowerCase();
				node.removeAttribute(lcfname);
				node[lcfname] = null;
			}

			// Mix things found in data-dojo-props into the params, overriding any direct settings
			if(extra){
				try{
					extra = djson.fromJson.call(options.propsThis, "{" + extra + "}");
					dlang.mixin(params, extra);
				}catch(e){
					// give the user a pointer to their invalid parameters. FIXME: can we kill this in production?
					throw new Error(e.toString() + " in data-dojo-props='" + extra + "'");
				}
			}

			// Any parameters specified in "mixin" override everything else.
			dlang.mixin(params, mixin);

			// Get <script> nodes associated with this widget, if they weren't specified explicitly
			if(!scripts){
				scripts = (ctor && (ctor._noScript || proto._noScript) ? [] : query("> script[type^='dojo/']", node));
			}

			// Process <script type="dojo/*"> script tags
			// <script type="dojo/method" event="foo"> tags are added to params, and passed to
			// the widget on instantiation.
			// <script type="dojo/method"> tags (with no event) are executed after instantiation
			// <script type="dojo/connect" data-dojo-event="foo"> tags are dojo.connected after instantiation
			// <script type="dojo/watch" data-dojo-prop="foo"> tags are dojo.watch after instantiation
			// <script type="dojo/on" data-dojo-event="foo"> tags are dojo.on after instantiation
			// note: dojo/* script tags cannot exist in self closing widgets, like <input />
			var aspects = [],	// aspects to connect after instantiation
				calls = [],		// functions to call after instantiation
				watches = [],  // functions to watch after instantiation
				ons = []; // functions to on after instantiation

			if(scripts){
				for(i=0; i<scripts.length; i++){
					var script = scripts[i];
					node.removeChild(script);
					// FIXME: drop event="" support in 2.0. use data-dojo-event="" instead
					var event = (script.getAttribute(attrData + "event") || script.getAttribute("event")),
						prop = script.getAttribute(attrData + "prop"),
						method = script.getAttribute(attrData + "method"),
						advice = script.getAttribute(attrData + "advice"),
						scriptType = script.getAttribute("type"),
						nf = this._functionFromScript(script, attrData);
					if(event){
						if(scriptType == "dojo/connect"){
							aspects.push({ method: event, func: nf });
						}else if(scriptType == "dojo/on"){
							ons.push({ event: event, func: nf });
						}else{
							params[event] = nf;
						}
					}else if(scriptType == "dojo/aspect"){
						aspects.push({ method: method, advice: advice, func: nf });
					}else if(scriptType == "dojo/watch"){
						watches.push({ prop: prop, func: nf });
					}else{
						calls.push(nf);
					}
				}
			}

			// create the instance
			var markupFactory = ctor.markupFactory || proto.markupFactory;
			var instance = markupFactory ? markupFactory(params, node, ctor) : new ctor(params, node);

			// map it to the JS namespace if that makes sense
			if(jsname){
				dlang.setObject(jsname, instance);
			}

			// process connections and startup functions
			for(i=0; i<aspects.length; i++){
				aspect[aspects[i].advice || "after"](instance, aspects[i].method, dlang.hitch(instance, aspects[i].func), true);
			}
			for(i=0; i<calls.length; i++){
				calls[i].call(instance);
			}
			for(i=0; i<watches.length; i++){
				instance.watch(watches[i].prop, watches[i].func);
			}
			for(i=0; i<ons.length; i++){
				don(instance, ons[i].event, ons[i].func);
			}

			return instance;
		},

		scan: function(root, options){
			// summary:
			//		Scan a DOM tree and return an array of objects representing the DOMNodes
			//		that need to be turned into widgets.
			// description:
			//		Search specified node (or document root node) recursively for class instances
			//		and return an array of objects that represent potential widgets to be
			//		instantiated. Searches for either data-dojo-type="MID" or dojoType="MID" where
			//		"MID" is a module ID like "dijit/form/Button" or a fully qualified Class name
			//		like "dijit/form/Button".  If the MID is not currently available, scan will
			//		attempt to require() in the module.
			//
			//		See parser.parse() for details of markup.
			// root: DomNode?
			//		A default starting root node from which to start the parsing. Can be
			//		omitted, defaulting to the entire document. If omitted, the `options`
			//		object can be passed in this place. If the `options` object has a
			//		`rootNode` member, that is used.
			// options: Object
			//		a kwArgs options object, see parse() for details
			//
			// returns: Promise
			//		A promise that is resolved with the nodes that have been parsed.

			var list = [],  // Output List
				mids = [], // An array of modules that are not yet loaded
				midsHash = {}; // Used to keep the mids array unique

			var dojoType = (options.scope || dojo._scopeName) + "Type",		// typically "dojoType"
				attrData = "data-" + (options.scope || dojo._scopeName) + "-",	// typically "data-dojo-"
				dataDojoType = attrData + "type",						// typically "data-dojo-type"
				dataDojoTextDir = attrData + "textdir",					// typically "data-dojo-textdir"
				dataDojoMixins = attrData + "mixins";					// typically "data-dojo-mixins"

			// Info on DOMNode currently being processed
			var node = root.firstChild;

			// Info on parent of DOMNode currently being processed
			//	- inherited: dir, lang, and textDir setting of parent, or inherited by parent
			//	- parent: pointer to identical structure for my parent (or null if no parent)
			//	- scripts: if specified, collects <script type="dojo/..."> type nodes from children
			var inherited = options.inherited;
			if(!inherited){
				function findAncestorAttr(node, attr){
					return (node.getAttribute && node.getAttribute(attr)) ||
						(node.parentNode && findAncestorAttr(node.parentNode, attr));
				}
				inherited = {
					dir: findAncestorAttr(root, "dir"),
					lang: findAncestorAttr(root, "lang"),
					textDir: findAncestorAttr(root, dataDojoTextDir)
				};
				for(var key in inherited){
					if(!inherited[key]){ delete inherited[key]; }
				}
			}

			// Metadata about parent node
			var parent = {
				inherited: inherited
			};

			// For collecting <script type="dojo/..."> type nodes (when null, we don't need to collect)
			var scripts;

			// when true, only look for <script type="dojo/..."> tags, and don't recurse to children
			var scriptsOnly;

			function getEffective(parent){
				// summary:
				//		Get effective dir, lang, textDir settings for specified obj
				//		(matching "parent" object structure above), and do caching.
				//		Take care not to return null entries.
				if(!parent.inherited){
					parent.inherited = {};
					var node = parent.node,
						grandparent = getEffective(parent.parent);
					var inherited  = {
						dir: node.getAttribute("dir") || grandparent.dir,
						lang: node.getAttribute("lang") || grandparent.lang,
						textDir: node.getAttribute(dataDojoTextDir) || grandparent.textDir
					};
					for(var key in inherited){
						if(inherited[key]){
							parent.inherited[key] = inherited[key];
						}
					}
				}
				return parent.inherited;
			}

			// DFS on DOM tree, collecting nodes with data-dojo-type specified.
			while(true){
				if(!node){
					// Finished this level, continue to parent's next sibling
					if(!parent || !parent.node){
						break;
					}
					node = parent.node.nextSibling;
					scriptsOnly = false;
					parent = parent.parent;
					scripts = parent.scripts;
					continue;
				}

				if(node.nodeType != 1){
					// Text or comment node, skip to next sibling
					node = node.nextSibling;
					continue;
				}

				if(scripts && node.nodeName.toLowerCase() == "script"){
					// Save <script type="dojo/..."> for parent, then continue to next sibling
					type = node.getAttribute("type");
					if(type && /^dojo\/\w/i.test(type)){
						scripts.push(node);
					}
					node = node.nextSibling;
					continue;
				}
				if(scriptsOnly){
					// scriptsOnly flag is set, we have already collected scripts if the parent wants them, so now we shouldn't
					// continue further analysis of the node and will continue to the next sibling
					node = node.nextSibling;
					continue;
				}

				// Check for data-dojo-type attribute, fallback to backward compatible dojoType
				// TODO: Remove dojoType in 2.0
				var type = node.getAttribute(dataDojoType) || node.getAttribute(dojoType);

				// Short circuit for leaf nodes containing nothing [but text]
				var firstChild = node.firstChild;
				if(!type && (!firstChild || (firstChild.nodeType == 3 && !firstChild.nextSibling))){
					node = node.nextSibling;
					continue;
				}

				// Meta data about current node
				var current;

				var ctor = null;
				if(type){
					// If dojoType/data-dojo-type specified, add to output array of nodes to instantiate.
					var mixinsValue = node.getAttribute(dataDojoMixins),
						types = mixinsValue ? [type].concat(mixinsValue.split(/\s*,\s*/)) : [type];

					// Note: won't find classes declared via dojo/Declaration or any modules that haven't been
					// loaded yet so use try/catch to avoid throw from require()
					try{
						ctor = getCtor(types);
					}catch(e){}

					// If the constructor was not found, check to see if it has modules that can be loaded
					if(!ctor){
						darray.forEach(types, function(t){
							if(~t.indexOf('/') && !midsHash[t]){
								// If the type looks like a MID and it currently isn't in the array of MIDs to load, add it.
								midsHash[t] = true;
								mids[mids.length] = t;
							}
						});
					}

					var childScripts = ctor && !ctor.prototype._noScript ? [] : null; // <script> nodes that are parent's children

					// Setup meta data about this widget node, and save it to list of nodes to instantiate
					current = {
						types: types,
						ctor: ctor,
						parent: parent,
						node: node,
						scripts: childScripts
					};
					current.inherited = getEffective(current); // dir & lang settings for current node, explicit or inherited
					list.push(current);
				}else{
					// Meta data about this non-widget node
					current = {
						node: node,
						scripts: scripts,
						parent: parent
					};
				}

				// Recurse, collecting <script type="dojo/..."> children, and also looking for
				// descendant nodes with dojoType specified (unless the widget has the stopParser flag).
				// When finished with children, go to my next sibling.
				node = firstChild;
				scripts = childScripts;
				scriptsOnly = ctor && ctor.prototype.stopParser && !(options.template);
				parent = current;
			}

			var d = new Deferred();

			// If there are modules to load then require them in
			if(mids.length){
				// Warn that there are modules being auto-required
				if(has("dojo-debug-messages")){
					console.warn("WARNING: Modules being Auto-Required: " + mids.join(", "));
				}
				require(mids, function(){
					// Go through list of widget nodes, filling in missing constructors, and filtering out nodes that shouldn't
					// be instantiated due to a stopParser flag on an ancestor that we belatedly learned about due to
					// auto-require of a module like ContentPane.   Assumes list is in DFS order.
					d.resolve(darray.filter(list, function(widget){
						if(!widget.ctor){
							// Attempt to find the constructor again.   Still won't find classes defined via
							// dijit/Declaration so need to try/catch.
							try{
								widget.ctor = getCtor(widget.types);
							}catch(e){}
						}

						// Get the parent widget
						var parent = widget.parent;
						while(parent && !parent.types){
							parent = parent.parent;
						}

						// Return false if this node should be skipped due to stopParser on an ancestor.
						// Since list[] is in DFS order, this loop will always set parent.instantiateChildren before
						// trying to compute widget.instantiate.
						var proto = widget.ctor && widget.ctor.prototype;
						widget.instantiateChildren = !(proto && proto.stopParser && !(options.template));
						widget.instantiate = !parent || (parent.instantiate && parent.instantiateChildren);
						return widget.instantiate;
					}));
				});
			}else{
				// There were no modules to load, so just resolve with the parsed nodes.   This separate code path is for
				// efficiency, to avoid running the require() and the callback code above.
				d.resolve(list);
			}

			// Return the promise
			return d.promise;
		},

		_require: function(/*DOMNode*/ script){
			// summary:
			//		Helper for _scanAMD().  Takes a `<script type=dojo/require>bar: "acme/bar", ...</script>` node,
			//		calls require() to load the specified modules and (asynchronously) assign them to the specified global
			//		variables, and returns a Promise for when that operation completes.
			//
			//		In the example above, it is effectively doing a require(["acme/bar", ...], function(a){ bar = a; }).

			var hash = djson.fromJson("{" + script.innerHTML + "}"),
				vars = [],
				mids = [],
				d = new Deferred();

			for(var name in hash){
				vars.push(name);
				mids.push(hash[name]);
			}

			require(mids, function(){
				for(var i=0; i<vars.length; i++){
					dlang.setObject(vars[i], arguments[i]);
				}
				d.resolve(arguments);
			});

			return d.promise;
		},

		_scanAmd: function(root){
			// summary:
			//		Scans the DOM for any declarative requires and returns their values.
			// description:
			//		Looks for `<script type=dojo/require>bar: "acme/bar", ...</script>` node, calls require() to load the
			//		specified modules and (asynchronously) assign them to the specified global variables,
			//		 and returns a Promise for when those operations complete.
			// root: DomNode
			//		The node to base the scan from.

			// Promise that resolves when all the <script type=dojo/require> nodes have finished loading.
			var deferred = new Deferred(),
				promise = deferred.promise;
			deferred.resolve(true);

			var self = this;
			query("script[type='dojo/require']", root).forEach(function(node){
				// Fire off require() call for specified modules.  Chain this require to fire after
				// any previous requires complete, so that layers can be loaded before individual module require()'s fire.
				promise = promise.then(function(){ return self._require(node); });

				// Remove from DOM so it isn't seen again
				node.parentNode.removeChild(node);
			});

			return promise;
		},

		parse: function(rootNode, options){
			// summary:
			//		Scan the DOM for class instances, and instantiate them.
			// description:
			//		Search specified node (or root node) recursively for class instances,
			//		and instantiate them. Searches for either data-dojo-type="Class" or
			//		dojoType="Class" where "Class" is a a fully qualified class name,
			//		like `dijit/form/Button`
			//
			//		Using `data-dojo-type`:
			//		Attributes using can be mixed into the parameters used to instantiate the
			//		Class by using a `data-dojo-props` attribute on the node being converted.
			//		`data-dojo-props` should be a string attribute to be converted from JSON.
			//
			//		Using `dojoType`:
			//		Attributes are read from the original domNode and converted to appropriate
			//		types by looking up the Class prototype values. This is the default behavior
			//		from Dojo 1.0 to Dojo 1.5. `dojoType` support is deprecated, and will
			//		go away in Dojo 2.0.
			// rootNode: DomNode?
			//		A default starting root node from which to start the parsing. Can be
			//		omitted, defaulting to the entire document. If omitted, the `options`
			//		object can be passed in this place. If the `options` object has a
			//		`rootNode` member, that is used.
			// options: Object?
			//		A hash of options.
			//
			//		- noStart: Boolean?:
			//			when set will prevent the parser from calling .startup()
			//			when locating the nodes.
			//		- rootNode: DomNode?:
			//			identical to the function's `rootNode` argument, though
			//			allowed to be passed in via this `options object.
			//		- template: Boolean:
			//			If true, ignores ContentPane's stopParser flag and parses contents inside of
			//			a ContentPane inside of a template.   This allows dojoAttachPoint on widgets/nodes
			//			nested inside the ContentPane to work.
			//		- inherited: Object:
			//			Hash possibly containing dir and lang settings to be applied to
			//			parsed widgets, unless there's another setting on a sub-node that overrides
			//		- scope: String:
			//			Root for attribute names to search for.   If scopeName is dojo,
			//			will search for data-dojo-type (or dojoType).   For backwards compatibility
			//			reasons defaults to dojo._scopeName (which is "dojo" except when
			//			multi-version support is used, when it will be something like dojo16, dojo20, etc.)
			//		- propsThis: Object:
			//			If specified, "this" referenced from data-dojo-props will refer to propsThis.
			//			Intended for use from the widgets-in-template feature of `dijit._WidgetsInTemplateMixin`
			// returns: Mixed
			//		Returns a blended object that is an array of the instantiated objects, but also can include
			//		a promise that is resolved with the instantiated objects.  This is done for backwards
			//		compatibility.  If the parser auto-requires modules, it will always behave in a promise
			//		fashion and `parser.parse().then(function(instances){...})` should be used.
			// example:
			//		Parse all widgets on a page:
			//	|		parser.parse();
			// example:
			//		Parse all classes within the node with id="foo"
			//	|		parser.parse(dojo.byId('foo'));
			// example:
			//		Parse all classes in a page, but do not call .startup() on any
			//		child
			//	|		parser.parse({ noStart: true })
			// example:
			//		Parse all classes in a node, but do not call .startup()
			//	|		parser.parse(someNode, { noStart:true });
			//	|		// or
			//	|		parser.parse({ noStart:true, rootNode: someNode });

			// determine the root node and options based on the passed arguments.
			var root;
			if(!options && rootNode && rootNode.rootNode){
				options = rootNode;
				root = options.rootNode;
			}else if(rootNode && dlang.isObject(rootNode) && !("nodeType" in rootNode)){
				options = rootNode;
			}else{
				root = rootNode;
			}
			root = root ? dhtml.byId(root) : dwindow.body();

			options = options || {};

			var mixin = options.template ? { template: true } : {},
				instances = [],
				self = this;

			// First scan for any <script type=dojo/require> nodes, and execute.
			// Then scan for all nodes with data-dojo-type, and load any unloaded modules.
			// Then build the object instances.  Add instances to already existing (but empty) instances[] array,
			// which may already have been returned to caller.  Also, use otherwise to collect and throw any errors
			// that occur during the parse().
			var p =
				this._scanAmd(root, options).then(function(){
					return self.scan(root, options);
				}).then(function(parsedNodes){
					return instances = instances.concat(self._instantiate(parsedNodes, mixin, options));
				}).otherwise(function(e){
					// TODO Modify to follow better pattern for promise error managment when available
					console.error("dojo/parser::parse() error", e);
					throw e;
				});

			// Blend the array with the promise
			dlang.mixin(instances, p);
			return instances;
		}
	};

	if( 1 ){
		dojo.parser  = parser;
	}

	// Register the parser callback. It should be the first callback
	// after the a11y test.
	if(config.parseOnLoad){
		ready(100, parser, "parse");
	}

	return parser;
});

},
'dojo/_base/url':function(){
define("dojo/_base/url", ["./kernel"], function(dojo){
	// module:
	//		dojo/url

	var
		ore = new RegExp("^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\\?([^#]*))?(#(.*))?$"),
		ire = new RegExp("^((([^\\[:]+):)?([^@]+)@)?(\\[([^\\]]+)\\]|([^\\[:]*))(:([0-9]+))?$"),
		_Url = function(){
			var n = null,
				_a = arguments,
				uri = [_a[0]];
			// resolve uri components relative to each other
			for(var i = 1; i<_a.length; i++){
				if(!_a[i]){ continue; }

				// Safari doesn't support this.constructor so we have to be explicit
				// FIXME: Tracked (and fixed) in Webkit bug 3537.
				//		http://bugs.webkit.org/show_bug.cgi?id=3537
				var relobj = new _Url(_a[i]+""),
					uriobj = new _Url(uri[0]+"");

				if(
					relobj.path == "" &&
					!relobj.scheme &&
					!relobj.authority &&
					!relobj.query
				){
					if(relobj.fragment != n){
						uriobj.fragment = relobj.fragment;
					}
					relobj = uriobj;
				}else if(!relobj.scheme){
					relobj.scheme = uriobj.scheme;

					if(!relobj.authority){
						relobj.authority = uriobj.authority;

						if(relobj.path.charAt(0) != "/"){
							var path = uriobj.path.substring(0,
								uriobj.path.lastIndexOf("/") + 1) + relobj.path;

							var segs = path.split("/");
							for(var j = 0; j < segs.length; j++){
								if(segs[j] == "."){
									// flatten "./" references
									if(j == segs.length - 1){
										segs[j] = "";
									}else{
										segs.splice(j, 1);
										j--;
									}
								}else if(j > 0 && !(j == 1 && segs[0] == "") &&
									segs[j] == ".." && segs[j-1] != ".."){
									// flatten "../" references
									if(j == (segs.length - 1)){
										segs.splice(j, 1);
										segs[j - 1] = "";
									}else{
										segs.splice(j - 1, 2);
										j -= 2;
									}
								}
							}
							relobj.path = segs.join("/");
						}
					}
				}

				uri = [];
				if(relobj.scheme){
					uri.push(relobj.scheme, ":");
				}
				if(relobj.authority){
					uri.push("//", relobj.authority);
				}
				uri.push(relobj.path);
				if(relobj.query){
					uri.push("?", relobj.query);
				}
				if(relobj.fragment){
					uri.push("#", relobj.fragment);
				}
			}

			this.uri = uri.join("");

			// break the uri into its main components
			var r = this.uri.match(ore);

			this.scheme = r[2] || (r[1] ? "" : n);
			this.authority = r[4] || (r[3] ? "" : n);
			this.path = r[5]; // can never be undefined
			this.query = r[7] || (r[6] ? "" : n);
			this.fragment	 = r[9] || (r[8] ? "" : n);

			if(this.authority != n){
				// server based naming authority
				r = this.authority.match(ire);

				this.user = r[3] || n;
				this.password = r[4] || n;
				this.host = r[6] || r[7]; // ipv6 || ipv4
				this.port = r[9] || n;
			}
		};
	_Url.prototype.toString = function(){ return this.uri; };

	return dojo._Url = _Url;
});

},
'url:dijit/templates/Dialog.html':"<div class=\"dijitDialog\" role=\"dialog\" aria-labelledby=\"${id}_title\">\n\t<div data-dojo-attach-point=\"titleBar\" class=\"dijitDialogTitleBar\">\n\t\t<span data-dojo-attach-point=\"titleNode\" class=\"dijitDialogTitle\" id=\"${id}_title\"\n\t\t\t\trole=\"heading\" level=\"1\"></span>\n\t\t<span data-dojo-attach-point=\"closeButtonNode\" class=\"dijitDialogCloseIcon\" data-dojo-attach-event=\"ondijitclick: onCancel\" title=\"${buttonCancel}\" role=\"button\" tabindex=\"0\">\n\t\t\t<span data-dojo-attach-point=\"closeText\" class=\"closeText\" title=\"${buttonCancel}\">x</span>\n\t\t</span>\n\t</div>\n\t<div data-dojo-attach-point=\"containerNode\" class=\"dijitDialogPaneContent\"></div>\n</div>\n",
'epi/shell/widget/dialog/_DialogMixin':function(){
﻿define("epi/shell/widget/dialog/_DialogMixin", [
    "dojo",
    "dojo/dom-style"
], function (dojo, domStyle) {

    return dojo.declare(null, {
        // summary:
        //    Base class to mixin properties of dialog
        // tags:
        //    public

        // closeIconVisible: [public] Boolean
        //		Flag which indicates whether the close icon should be visible.
        closeIconVisible: true,

        _setCloseIconVisibleAttr: function () {
            // summary:
            //    Show or hide 'Close' icon at top right corner of the dialog
            // tags:
            //    private

            this._set("closeIconVisible", this.closeIconVisible);
            domStyle.set(this.closeButtonNode, "visibility", this.closeIconVisible ? "visible" : "hidden");
            //If the close icon is visible add a right margin to the titleNode
            domStyle.set(this.titleNode, "margin-right", this.closeIconVisible ? "25px" : "0");
        }
    });
});

},
'epi/shell/widget/_FocusableMixin':function(){
﻿define("epi/shell/widget/_FocusableMixin", [
    "dojo/dom-class",
    "dojo/_base/declare",
    "dijit/_FocusMixin"
], function (domClass, declare, _FocusMixin) {
    return declare(_FocusMixin, {
        // tags:
        //    internal

        onFocus: function () {
            domClass.add(this.domNode, "epi-focused");
        },
        onBlur: function () {
            domClass.remove(this.domNode, "epi-focused");
        }
    });
});

},
'epi/shell/widget/_ActionConsumerWidget':function(){
﻿define("epi/shell/widget/_ActionConsumerWidget", ["dojo/_base/declare", "./_ActionConsumer"], function (declare, _ActionConsumer) {

    return declare(_ActionConsumer, {
        // tags:
        //    deprecated

        // definitionConsumer: [protected] Object
        //      The actions consumer that will be the target for action related events.
        definitionConsumer: null,

        startup: function () {
            // summary:
            //      Overridden to populate the connected action consumer
            this.inherited(arguments);
            this._populateActions();
        },

        _setDefinitionConsumerAttr: function (consumer) {
            this._set("definitionConsumer", consumer);

            if (this._started) {
                this._populateActions();
            }
        },

        onActionAdded: function (action) {
            // summary:
            //      Default implementation that checks if a definition consumer exists and registers actions.
            //      Developers should override this if they don't set a definition consumer.
            // tags:
            //      protected
            if (this.definitionConsumer) {
                this.definitionConsumer.add(action);
            }
        },

        onActionRemoved: function (action) {
            // summary:
            //      Default implementation that checks if a definition consumer exists and removes actions.
            //      Developers should override this if they don't set a definition consumer.
            // tags:
            //      protected
            if (this.definitionConsumer) {
                this.definitionConsumer.remove(action.name);
            }
        },

        onActionPropertyChanged: function (action, propertyName, propertyValue) {
            // summary:
            //      Default implementation that checks if a definition consumer exists and updates action state.
            //      Developers should override this if they don't set a definition consumer.
            // tags:
            //      protected
            if (this.definitionConsumer) {
                this.definitionConsumer.setItemProperty(action.name, propertyName, propertyValue);
            }
        },

        _populateActions: function () {
            if (this.definitionConsumer) {
                this.definitionConsumer.add(this.getActions());
            }
        }
    });
});

},
'epi/shell/widget/_ActionConsumer':function(){
﻿define("epi/shell/widget/_ActionConsumer", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/kernel",
    "dojo/_base/array",
    "dojo/aspect",
    "epi/shell/widget/_ActionProviderBase"
],

function (declare, lang, kernel, array, aspect, _ActionProviderBase) {

    return declare(_ActionProviderBase, {
        // summary:
        //    Mixin for components that are responsible for rendering controls and executing custom actions provided by action providers.
        //
        // tags:
        //    deprecated

        // _actionProviders: [private] Array
        //    An array of action providers used by this instance to get and handle the list of custom actions.
        _actionProviders: null,

        constructor: function (args) {
            kernel.deprecated("epi/shell/widget/_ActionConsumer", "use the command pattern with epi/shell/command/_CommandConsumerMixin instead");

            this._actionProviders = [];
            if (args && lang.isArray(args.actionProviders)) {
                args.actionProviders.forEach(function (provider) {
                    this.addProvider(provider);
                }, this);
            }
        },

        isProviderRegistered: function (actionProvider) {
            // summary:
            //    Returns a boolean value indicating whether specified action provider is registered in this action consumer.
            //
            // actionProvider:
            //      An action provider to verify.
            //
            // tags:
            //    public

            if (!actionProvider) {
                return false;
            }
            return this._actionProviders.some(function (providerStruct) {
                return providerStruct.provider === actionProvider;
            });
        },

        addProvider: function (actionProvider) {
            // summary:
            //    Registers specified action provider for this action consumer.
            //
            // actionProvider:
            //      An action provider to add.
            //
            // tags:
            //    public

            if (!actionProvider || this.isProviderRegistered(actionProvider)) {
                return;
            }
            this._actionProviders.push({
                provider: actionProvider,
                connects: [
                    aspect.after(actionProvider, "onActionAdded", lang.hitch(this, "onActionAdded"), true),
                    aspect.after(actionProvider, "onActionRemoved", lang.hitch(this, "onActionRemoved"), true),
                    aspect.after(actionProvider, "onActionPropertyChanged", lang.hitch(this, "onActionPropertyChanged"), this)]
            });
        },

        removeProvider: function (actionProvider) {
            // summary:
            //    Removes specified action provider from the list of this action consumer.
            //
            // actionProvider:
            //      An action provider to remove.
            //
            // tags:
            //    public

            if (!actionProvider || !this.isProviderRegistered(actionProvider)) {
                return;
            }
            for (var i = 0; i < this._actionProviders.length; i++) {
                if (this._actionProviders[i].provider === actionProvider) {
                    var provider = this._actionProviders.splice(i, 1);
                    array.forEach(provider.connects, function (handle) {
                        handle.remove();
                    });
                    break;
                }
            }
        },

        getActions: function () {
            // summary:
            //    Returns an array of custom actions provided by all action providers in actionProviders array.
            //    Returns the empty array if action providers are not defined or don't provide any actions.
            //
            // tags:
            //    protected

            var actions = [];
            this._actionProviders.forEach(function (provider, index) {
                actions = actions.concat(this._getProviderActions(provider.provider));
            }, this);
            return actions;
        },

        _getProviderActions: function (actionProvider) {
            // summary:
            //    Returns an array of custom actions provided by specified action provider.
            //    Returns the empty array if action provider is null or not defined or does not provide any action.
            //
            // actionProvider:
            //      An action provider to get actions from.
            //
            // tags:
            //    private

            if (actionProvider && typeof (actionProvider.getActions) == "function") {
                return actionProvider.getActions();
            } else {
                return [];
            }
        }
    });
});

},
'epi/shell/widget/_ActionProviderBase':function(){
﻿define("epi/shell/widget/_ActionProviderBase", [
    "epi",
    "dojo"],

function (epi, dojo) {

    return dojo.declare(null, {
        // summary:
        //    Base class for components that can provide actions and handlers to execute
        //
        // tags:
        //    deprecated

        getActions: function () {
            // summary:
            //      Provides a list of registered actions.
            //
            // tags:
            //      public abstract

            return [];
        },

        onActionAdded: function (action) {
            // summary:
            //      Event raised when a new action has been added to the provider
            //
            // action: Object
            //      The added action
            //
            // tags:
            //      public event
        },

        onActionRemoved: function (action) {
            // summary:
            //      Event raised when an action has been removed from the provider
            //
            // action: Object
            //      The removed action
            //
            // tags:
            //      public event
        },

        onActionPropertyChanged: function (action, propertyName, propertyValue) {
            // summary:
            //      Event raised when a property of an action has changed
            //
            // action: Object
            //      The action for which the property changed
            //
            // propertyName: String
            //      Name of the changed property
            //
            // propertyValue: Object
            //      The new value of the updated property
            //
            // tags:
            //      public event
        }
    });
});

},
'epi/shell/widget/Toolbar':function(){
define("epi/shell/widget/Toolbar", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dijit/Toolbar",
    "epi/shell/DestroyableByKey"
], function (
    declare,
    lang,
    Toolbar,
    DestroyableByKey
) {

    return declare([Toolbar, DestroyableByKey], {
        // summary:
        //		Overriden toolbar that refines focusing functionality.
        //
        // tags:
        //      internal

        _originalTabIndex: null,
        _focusable: null,
        _firstFocusableItem: null,

        postCreate: function () {
            // summary:
            //      Post widget create initialization
            // description:
            //      Make sure that toolbar can not be tabbed into if it does not contain any visible item.
            // tags:
            //      protected

            this.inherited(arguments);

            this._originalTabIndex = this.get("tabIndex");
            this._correctTabIndex();
        },

        addChild: function (child) {
            // summary:
            //      Add child item to the toolbar
            // tags:
            //      public

            this.inherited(arguments);

            if (child.firstFocusable) {
                this.set("_firstFocusableItem", child);
            }

            // watch item visibility change to correct tab index.
            this.destroyByKey(child);
            this.ownByKey(child, child.watch("itemVisibility", lang.hitch(this, function () {
                this._correctTabIndex();
            })));

            this._correctTabIndex();
        },

        removeChild: function (child) {
            // summary:
            //      Add child item to the toolbar
            // tags:
            //      protected

            this.destroyByKey(child);

            this.inherited(arguments);

            this._correctTabIndex();
        },

        _correctTabIndex: function () {
            // summary:
            //      Overwrite tabIndex to -1 if there is no visible item. Otherwise use the original one.
            // tags:
            //      protected

            this._focusable = (this._getFirstFocusableChild() !== null);
            this.set("tabIndex", this._focusable ? this._originalTabIndex : -1);
        },

        _getFirstFocusableChild: function () {
            // summary:
            //      Overridden to make it easy to specify which toolbar item that should get primary focus.
            // tags:
            //      protected

            return this._firstFocusableItem || this.inherited(arguments);
        },

        _startupChild: function (/*dijit/_WidgetBase*/ widget) {
            // summary:
            //      Override the _startupChild method in _KeyNavContainer to stop blur and focus
            //      event listeners being attached. This is because they break tabbing in the dialog.
            // tags:
            //      private
            widget.set("tabIndex", "-1");
        },

        _onChildBlur: function (/*dijit/_WidgetBase*/ widget) {
            // summary:
            //      Called when focus leaves a child widget to go to a sibling widget.
            //      Ensure that the widget losing focus has it's tab index reset.
            // tags:
            //      protected
            widget.set("tabIndex", "-1");
        }
    });
});

},
'dijit/Toolbar':function(){
define("dijit/Toolbar", [
	"require",
	"dojo/_base/declare", // declare
	"dojo/has",
	"dojo/keys", // keys.LEFT_ARROW keys.RIGHT_ARROW
	"dojo/ready",
	"./_Widget",
	"./_KeyNavContainer",
	"./_TemplatedMixin"
], function(require, declare, has, keys, ready, _Widget, _KeyNavContainer, _TemplatedMixin){

	// module:
	//		dijit/Toolbar


	// Back compat w/1.6, remove for 2.0
	if(has("dijit-legacy-requires")){
		ready(0, function(){
			var requires = ["dijit/ToolbarSeparator"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	return declare("dijit.Toolbar", [_Widget, _TemplatedMixin, _KeyNavContainer], {
		// summary:
		//		A Toolbar widget, used to hold things like `dijit.Editor` buttons

		templateString:
			'<div class="dijit" role="toolbar" tabIndex="${tabIndex}" data-dojo-attach-point="containerNode">' +
			'</div>',

		baseClass: "dijitToolbar",

		postCreate: function(){
			this.inherited(arguments);

			this.connectKeyNavHandlers(
				this.isLeftToRight() ? [keys.LEFT_ARROW] : [keys.RIGHT_ARROW],
				this.isLeftToRight() ? [keys.RIGHT_ARROW] : [keys.LEFT_ARROW]
			);
		}
	});
});

},
'dijit/_KeyNavContainer':function(){
define("dijit/_KeyNavContainer", [
	"dojo/_base/kernel", // kernel.deprecated
	"./_Container",
	"./_FocusMixin",
	"dojo/_base/array", // array.forEach
	"dojo/keys", // keys.END keys.HOME
	"dojo/_base/declare", // declare
	"dojo/_base/event", // event.stop
	"dojo/dom-attr", // domAttr.set
	"dojo/_base/lang" // lang.hitch
], function(kernel, _Container, _FocusMixin, array, keys, declare, event, domAttr, lang){


	// module:
	//		dijit/_KeyNavContainer

	return declare("dijit._KeyNavContainer", [_FocusMixin, _Container], {
		// summary:
		//		A _Container with keyboard navigation of its children.
		// description:
		//		To use this mixin, call connectKeyNavHandlers() in
		//		postCreate().
		//		It provides normalized keyboard and focusing code for Container
		//		widgets.

/*=====
		// focusedChild: [protected] Widget
		//		The currently focused child widget, or null if there isn't one
		focusedChild: null,
=====*/

		// tabIndex: String
		//		Tab index of the container; same as HTML tabIndex attribute.
		//		Note then when user tabs into the container, focus is immediately
		//		moved to the first item in the container.
		tabIndex: "0",

		connectKeyNavHandlers: function(/*keys[]*/ prevKeyCodes, /*keys[]*/ nextKeyCodes){
			// summary:
			//		Call in postCreate() to attach the keyboard handlers
			//		to the container.
			// preKeyCodes: keys[]
			//		Key codes for navigating to the previous child.
			// nextKeyCodes: keys[]
			//		Key codes for navigating to the next child.
			// tags:
			//		protected

			// TODO: call this automatically from my own postCreate()

			var keyCodes = (this._keyNavCodes = {});
			var prev = lang.hitch(this, "focusPrev");
			var next = lang.hitch(this, "focusNext");
			array.forEach(prevKeyCodes, function(code){ keyCodes[code] = prev; });
			array.forEach(nextKeyCodes, function(code){ keyCodes[code] = next; });
			keyCodes[keys.HOME] = lang.hitch(this, "focusFirstChild");
			keyCodes[keys.END] = lang.hitch(this, "focusLastChild");
			this.connect(this.domNode, "onkeypress", "_onContainerKeypress");
			this.connect(this.domNode, "onfocus", "_onContainerFocus");
		},

		startupKeyNavChildren: function(){
			kernel.deprecated("startupKeyNavChildren() call no longer needed", "", "2.0");
		},

		startup: function(){
			this.inherited(arguments);
			array.forEach(this.getChildren(), lang.hitch(this, "_startupChild"));
		},

		addChild: function(/*dijit/_WidgetBase*/ widget, /*int?*/ insertIndex){
			this.inherited(arguments);
			this._startupChild(widget);
		},

		focus: function(){
			// summary:
			//		Default focus() implementation: focus the first child.
			this.focusFirstChild();
		},

		focusFirstChild: function(){
			// summary:
			//		Focus the first focusable child in the container.
			// tags:
			//		protected
			this.focusChild(this._getFirstFocusableChild());
		},

		focusLastChild: function(){
			// summary:
			//		Focus the last focusable child in the container.
			// tags:
			//		protected
			this.focusChild(this._getLastFocusableChild());
		},

		focusNext: function(){
			// summary:
			//		Focus the next widget
			// tags:
			//		protected
			this.focusChild(this._getNextFocusableChild(this.focusedChild, 1));
		},

		focusPrev: function(){
			// summary:
			//		Focus the last focusable node in the previous widget
			//		(ex: go to the ComboButton icon section rather than button section)
			// tags:
			//		protected
			this.focusChild(this._getNextFocusableChild(this.focusedChild, -1), true);
		},

		focusChild: function(/*dijit/_WidgetBase*/ widget, /*Boolean*/ last){
			// summary:
			//		Focus specified child widget.
			// widget:
			//		Reference to container's child widget
			// last:
			//		If true and if widget has multiple focusable nodes, focus the
			//		last one instead of the first one
			// tags:
			//		protected

			if(!widget){ return; }

			if(this.focusedChild && widget !== this.focusedChild){
				this._onChildBlur(this.focusedChild);	// used by _MenuBase
			}
			widget.set("tabIndex", this.tabIndex);	// for IE focus outline to appear, must set tabIndex before focs
			widget.focus(last ? "end" : "start");
			this._set("focusedChild", widget);
		},

		_startupChild: function(/*dijit/_WidgetBase*/ widget){
			// summary:
			//		Setup for each child widget
			// description:
			//		Sets tabIndex=-1 on each child, so that the tab key will
			//		leave the container rather than visiting each child.
			// tags:
			//		private

			widget.set("tabIndex", "-1");

			this.connect(widget, "_onFocus", function(){
				// Set valid tabIndex so tabbing away from widget goes to right place, see #10272
				widget.set("tabIndex", this.tabIndex);
			});
			this.connect(widget, "_onBlur", function(){
				widget.set("tabIndex", "-1");
			});
		},

		_onContainerFocus: function(evt){
			// summary:
			//		Handler for when the container gets focus
			// description:
			//		Initially the container itself has a tabIndex, but when it gets
			//		focus, switch focus to first child...
			// tags:
			//		private

			// Note that we can't use _onFocus() because switching focus from the
			// _onFocus() handler confuses the focus.js code
			// (because it causes _onFocusNode() to be called recursively)
			// Also, _onFocus() would fire when focus went directly to a child widget due to mouse click.

			// Ignore spurious focus events:
			//	1. focus on a child widget bubbles on FF
			//	2. on IE, clicking the scrollbar of a select dropdown moves focus from the focused child item to me
			if(evt.target !== this.domNode || this.focusedChild){ return; }

			this.focusFirstChild();

			// and then set the container's tabIndex to -1,
			// (don't remove as that breaks Safari 4)
			// so that tab or shift-tab will go to the fields after/before
			// the container, rather than the container itself
			domAttr.set(this.domNode, "tabIndex", "-1");
		},

		_onBlur: function(evt){
			// When focus is moved away the container, and its descendant (popup) widgets,
			// then restore the container's tabIndex so that user can tab to it again.
			// Note that using _onBlur() so that this doesn't happen when focus is shifted
			// to one of my child widgets (typically a popup)
			if(this.tabIndex){
				domAttr.set(this.domNode, "tabIndex", this.tabIndex);
			}
			this.focusedChild = null;
			this.inherited(arguments);
		},

		_onContainerKeypress: function(evt){
			// summary:
			//		When a key is pressed, if it's an arrow key etc. then
			//		it's handled here.
			// tags:
			//		private
			if(evt.ctrlKey || evt.altKey){ return; }
			var func = this._keyNavCodes[evt.charOrCode];
			if(func){
				func();
				event.stop(evt);
			}
		},

		_onChildBlur: function(/*dijit/_WidgetBase*/ /*===== widget =====*/){
			// summary:
			//		Called when focus leaves a child widget to go
			//		to a sibling widget.
			//		Used by MenuBase.js (TODO: move code there)
			// tags:
			//		protected
		},

		_getFirstFocusableChild: function(){
			// summary:
			//		Returns first child that can be focused
			return this._getNextFocusableChild(null, 1);	// dijit/_WidgetBase
		},

		_getLastFocusableChild: function(){
			// summary:
			//		Returns last child that can be focused
			return this._getNextFocusableChild(null, -1);	// dijit/_WidgetBase
		},

		_getNextFocusableChild: function(child, dir){
			// summary:
			//		Returns the next or previous focusable child, compared
			//		to "child"
			// child: Widget
			//		The current widget
			// dir: Integer
			//		- 1 = after
			//		- -1 = before
			if(child){
				child = this._getSiblingOfChild(child, dir);
			}
			var children = this.getChildren();
			for(var i=0; i < children.length; i++){
				if(!child){
					child = children[(dir>0) ? 0 : (children.length-1)];
				}
				if(child.isFocusable()){
					return child;	// dijit/_WidgetBase
				}
				child = this._getSiblingOfChild(child, dir);
			}
			// no focusable child found
			return null;	// dijit/_WidgetBase
		}
	});
});

},
'epi/shell/DestroyableByKey':function(){
﻿define("epi/shell/DestroyableByKey", [
    "dojo/_base/array",
    "dojo/_base/declare",
    "dijit/Destroyable"
],
function (
    array,
    declare,
    Destroyable
) {

    return declare([Destroyable], {
        // summary:
        //		A base class which enables owning a set of handles that could
        //      need to be destroyed during this instance's lifetime. For
        //      instance we have an object that's handed a model, it watches
        //      some properties on the model. When our object is handed a new
        //      model we want an easy way to destroy the watches and create
        //      new ones. And we also want these handles to be be destroyed
        //      when our object is destroyed.
        //
        // tags:
        //      public

        // _ownedByKey: Object[]
        //      The owned handles
        _ownedByKey: null,

        constructor: function () {
            this._ownedByKey = [];
        },

        destroy: function () {

            // destroy all handles we have
            var item;
            while (item = this._ownedByKey.pop()) { //eslint-disable-line no-cond-assign
                item.handler.destroy();
            }

            this.inherited(arguments);
        },

        ownByKey: function (/*Object*/ key, /*Object...*/ handle) {
            // summary:
            //      Own a set of handles
            // key:
            //      The key to identify the set of handles. It can be a model
            //      object, but note that the same instance must be used when
            //      destroying the handles
            // handle:
            //      The handle

            var i;
            var ownedByKey = this._findOwnedByKey(key);

            // create a new item if missing
            if (!ownedByKey) {
                ownedByKey = {
                    key: key,
                    handler: new Destroyable()
                };
                this._ownedByKey.push(ownedByKey);
            }

            // own all handles
            for (i = 1; i < arguments.length; i++) {
                ownedByKey.handler.own(arguments[i]);
            }

            // return this instance for easy chaining
            return this;
        },

        destroyByKey: function (key) {
            // summary:
            //      Destroy a set of handles
            // key:
            //      The key to identify the set of handles. It can be a model
            //      object, but note that the same instance must be used as
            //      when owning the handles

            var ownedByKey = this._findOwnedByKey(key);
            var index;

            if (!ownedByKey) {
                return;
            }

            // destroy handlers
            ownedByKey.handler.destroy();

            // remove from array
            index = array.indexOf(this._ownedByKey, ownedByKey);
            this._ownedByKey.splice(index, 1);
        },

        _findOwnedByKey: function (/*Object*/ key) {
            // summary:
            //      find a set of owned handles

            var ownedByKey;
            array.some(this._ownedByKey, function (item) {
                if (item.key === key) {
                    ownedByKey = item;
                    return false;
                }
            });

            return ownedByKey;
        }

    });
});

},
'epi/shell/widget/ToolbarSet':function(){
define("epi/shell/widget/ToolbarSet", [
    "dojo/_base/declare",

    "dijit/_WidgetBase",

    "epi/shell/layout/ToolbarContainer",
    "epi/shell/widget/_AddByDefinition"
], function (
    declare,

    _WidgetBase,

    ToolbarContainer,
    _AddByDefinition) {

    return declare([_WidgetBase, _AddByDefinition], {
        // summary:
        //		A toolbar set widget that supports adding items using a simplified definition
        //
        // tags:
        //      internal xproduct

        // _layoutContainer: dijit/_Container
        //      Reference to the toolbar's current layout container.
        _layoutContainer: null,

        // layoutContainerClass: Constructor
        //      The layout container class used for the toolbar.
        layoutContainerClass: null,

        postMixInProperties: function () {
            // summary:
            //      Post mix in properties initialization
            // description:
            //      Set default layout container if not specified.

            this.inherited(arguments);

            this.layoutContainerClass = this.layoutContainerClass || ToolbarContainer;
        },

        buildRendering: function () {
            // summary:
            //      Build the toolbar set.
            // description:
            //      Instantiate layout container and put it directly under the dom node.
            this.inherited(arguments);

            this._layoutContainer = new this.layoutContainerClass();
            this._layoutContainer.placeAt(this.domNode);

            this.own(this._layoutContainer);
        },

        addChild: function (child) {
            // summary:
            //      Add a child item to the toolbar set.
            // description:
            //      Delegate to the layout container, who is responsible for arranging items.

            return this._layoutContainer.addChild(child);
        },

        _getLayoutContainerAttr: function () {
            return this._layoutContainer;
        }
    });
});

},
'epi/shell/layout/ToolbarContainer':function(){
﻿define("epi/shell/layout/ToolbarContainer", [
    "dojo/_base/declare",
    "dojo/dom-style",
    "dijit/_Container",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetBase"],

function (declare, domStyle, _Container, _TemplatedMixin, _WidgetBase) {

    return declare([_WidgetBase, _TemplatedMixin, _Container], {
        // summary:
        //     Container widget for arranging toolbars in groups.
        //
        // tags:
        //      internal

        cssClasses: {
            container: "epi-toolbarGroupContainer",
            leading: "epi-toolbarGroup epi-toolbarLeading",
            center: "epi-toolbarGroup epi-toolbarCenter",
            trailing: "epi-toolbarGroup epi-toolbarTrailing"
        },

        templateString: "<div class='${cssClasses.container}'>\
                            <div data-dojo-attach-point='leadingNode' class='${cssClasses.leading}'></div>\
                            <div data-dojo-attach-point='centerNode' class='${cssClasses.center}'></div>\
                            <div data-dojo-attach-point='trailingNode' class='${cssClasses.trailing}'></div>\
                        </div>",

        addChild: function (child) {
            // summary: Add a child to the container
            // description: The child should has expected region set, otherwise the container will put it into center region.

            child.region = child.region || "center";

            var nodeToPlace;
            switch (child.region) {
                case "leading":
                    nodeToPlace = this.leadingNode;
                    break;
                case "center":
                    nodeToPlace = this.centerNode;
                    break;
                case "trailing":
                    nodeToPlace = this.trailingNode;
                    break;
            }

            child.placeAt(nodeToPlace);
        }
    });
});

},
'epi/shell/widget/_AddByDefinition':function(){
﻿define("epi/shell/widget/_AddByDefinition", [
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",

    "dojo/Deferred",
    "dojo/dom",
    "dojo/dom-construct",
    "dojo/dom-style",
    "dojo/when",

    "dijit",
    "dijit/_Widget",
    "dijit/ToolbarSeparator",
    "dijit/Menu",
    "dijit/MenuSeparator",

    "epi/shell/widget/WidgetFactory",

    // The following are indirect dependencies, WidgetFactory will require() them when they're instantiated
    "dijit/CheckedMenuItem",
    "dijit/MenuItem",
    "dijit/form/Button",
    "dijit/form/ComboButton",
    "dijit/form/DropDownButton",
    "dijit/form/RadioButton",
    "epi/shell/widget/PopupMenuItem",
    "epi/shell/widget/Toolbar",
    "epi/shell/widget/ToolbarLabel",
    "epi/shell/widget/ToolbarDropDownButton"],

function (array, declare, lang, Deferred, dom, domConstruct, domStyle, when, dijit, _Widget, ToolbarSeparator, Menu, MenuSeparator, WidgetFactory) {

    return declare(null, {
        // summary:
        //    A mixin that supports adding sub-items using a definition object
        //
        // tags:
        //    internal

        /*=====
        // _widgetFactory: [private] epi/shell/widget/WidgetFactory
        //    The instance used for creating widgets from definitions added to the toolbar.
        _widgetFactory: {},
        =====*/

        /*=====
        // _widgetMap: [private] Object
        //    Maps names to widget instances
        _widgetMap: {},
        =====*/

        // template objects for the widget factory
        _widgetTypeTemplates: {
            checkedmenuitem: { widgetType: "dijit/CheckedMenuItem" },
            menu: { widgetType: "dijit/Menu" },
            menuitem: { widgetType: "dijit/MenuItem" },
            popupmenu: { widgetType: "epi/shell/widget/PopupMenuItem" },
            button: { widgetType: "dijit/form/Button" },
            combo: { widgetType: "dijit/form/ComboButton" },
            dropdown: { widgetType: "epi/shell/widget/ToolbarDropDownButton" },
            radio: { widgetType: "dijit/form/RadioButton" },
            toolbargroup: { widgetType: "epi/shell/widget/Toolbar" },
            label: { widgetType: "epi/shell/widget/ToolbarLabel" }
        },

        constructor: function () {
            this._widgetFactory = new WidgetFactory();
            this._widgetMap = {};
        },

        postMixInProperties: function () {
            // summary:
            //    Initializes the widget factory
            // tags:
            //    protected

            this.connect(this._widgetFactory, "onWidgetHierarchyCreated", lang.hitch(this, function (widget, componentDefintion) {
                this._widgetMap[componentDefintion.name] = widget;
                if (componentDefintion.firstChildIsContainer) {
                    widget.containerWidget = widget.getChildren()[0];
                }
            }));

            // handle on-demand creation of menu below combo button
            var addWidgetDropDown = function (rootWidget, widget) {
                var dropDown = rootWidget.get("dropDown");
                if (!dropDown) {
                    dropDown = new Menu();
                    rootWidget.set("dropDown", dropDown);
                }
                dropDown.addChild(widget);
                // Ensure that menu items support an appropriate separator.
                if (!widget.separator) {
                    widget.set("separator", "dijit/MenuSeparator");
                }
            };
            this._widgetFactory.addHandler("dijit/form/ComboButton", addWidgetDropDown);
        },

        buildRendering: function () {
            this.inherited(arguments);
            dom.setSelectable(this.domNode, false);
        },

        _toComponentDefinition: function (definition) {
            // transforms a definition in the added format to a format supported by the widget factory
            var componentDefinition = this._getComponentDefinitionTemplate(definition);
            componentDefinition.name = definition.name;
            // map specific from the definition directly to settings to make the api easier to use
            componentDefinition.settings = lang.mixin({
                label: definition.label || "",
                showLabel: definition.label,
                title: definition.title || "",
                disabled: definition.disabled || false
            }, definition.settings);

            if (definition.iconClass) {
                componentDefinition.settings.iconClass = definition.iconClass;
            }

            // Default connections
            var defaultConnections = {
                onClick: definition.action
            };

            // Mix connections together
            componentDefinition.connections = lang.mixin(defaultConnections, definition.connections);

            return componentDefinition;
        },

        _getComponentDefinitionTemplate: function (definition) {
            // creates an object supported by the widget factory
            var componentDefinition = lang.mixin({}, this._widgetTypeTemplates[definition.type] || this._widgetTypeTemplates["button"]);
            if (definition.widgetType) {
                componentDefinition.widgetType = definition.widgetType;
            }
            return componentDefinition;
        },

        _getWidget: function (name) {
            // gets a previously added item
            return this._widgetMap[name];
        },

        _addSingle: function (definition) {
            // summary:
            //    Adds a single item from a widget definition.
            // tags:
            //    private

            if (this._getWidget(definition.name)) {
                return;
            }

            var parent = this._getWidget(definition.parent) || this;
            var componentDefinition = this._toComponentDefinition(definition);

            var def = new Deferred();
            when(this._widgetFactory.createWidgets(parent, componentDefinition), lang.hitch(this, function (widgets) {
                var widget = widgets[0];

                if (definition.tooltip) {
                    var html = definition.tooltip;
                    var show = function (e) {
                        dijit.showTooltip(html, e.target);
                    };
                    var hide = function (e) {
                        dijit.hideTooltip(e.target);
                    };

                    this.connect(widget.domNode, "onmouseenter", show);
                    this.connect(widget.domNode, "onmouseleave", hide);
                }

                this._addSeparators(widget);

                def.resolve();

            }), function (e) {
                console.error(e.stack || e);

                def.reject(e);
            });

            return def.promise;
        },

        _addSeparators: function (widget) {
            // summary:
            //    Run when a widget is added to the toolbar, adding separators according to the
            //    placing and definition of the widgets involved.
            // tags:
            //    private

            var parent = widget.getParent();
            if (!parent) {
                return;
            }

            var siblings = parent.getChildren();
            var i = array.indexOf(siblings, widget);

            // If this or previous sibling want a separator and none exists then add one.
            var previous = (i > 0) ? siblings[i - 1] : null;
            if (previous && !this._isSeparator(previous)) {
                if (widget.separate) {
                    this.addSeparator(widget, "before");
                } else if (previous.separate) {
                    this.addSeparator(previous, "after");
                }
            }

            // If this or next sibling want a separator and none exists then add one.
            var next = (siblings.length > i) ? siblings[i + 1] : null;
            if (next && !this._isSeparator(next) && (widget.separate || next.separate)) {
                this.addSeparator(widget, "after");
            }
        },

        _getSiblingOfWidget: function (/*dijit/_Widget*/widget, /*String*/dir) {
            // summary:
            //    Get the next or previous widget sibling of child
            // dir:
            //    One of 'nextSibling' or 'previousSibling'
            // tags:
            //    private

            var node = widget.domNode;
            do {
                node = node[dir];
            } while (node && (node.nodeType !== 1 || !dijit.byNode(node)));
            return node && dijit.byNode(node); // dijit/_Widget
        },

        add: function (/* Object|Array */definitions) {
            // summary:
            //    Adds an item or an array of items to the toolbar.
            //
            // definitions:
            //      A toolbar definition, or an array of toolbar definitions to add to the toolbar.
            //
            //      Example of adding a single button:
            //      | // buttons
            //      | toolbar.add({
            //      |     name:"doit",
            //      |     label:"DoIt!",
            //      |     // fallback type:button,
            //      |     action:console.log
            //      | });
            //      |
            //
            //      Example of adding a combo button with a sub menu option:
            //      | toolbar.add([
            //      |     {
            //      |         name: "publish",
            //      |         label: "Publish",
            //      |         type: "combo",
            //      |         action: console.log
            //      |     },
            //      |     {
            //      |         parent: "publish",
            //      |         name: "save",
            //      |         label: "Save",
            //      |         type: "menuitem",
            //      |         action: console.log
            //      |     }]);
            //
            //      Example of adding a drop down with two menu options:
            //      | toolbar.add([
            //      |     {
            //      |         name: "misc",
            //      |         label: "misc stuff",
            //      |         type: "dropdown"
            //      |     },
            //      |     {
            //      |         parent: "misc",
            //      |         name: "hello",
            //      |         label: "hello",
            //      |         type: "menuitem",
            //      |         action: console.log
            //      |     },
            //      |     {
            //      |         parent: "misc",
            //      |         name: "world",
            //      |         label: "world",
            //      |         type: "menuitem",
            //      |         action: console.log
            //      |     }]);
            //
            //
            //      Example adding a custom widget type to the toolbar.
            //      | toolbar.add({
            //      |     name: "custom",
            //      |     label: "Custom widget",
            //      |     widgetType: "dijit/form/ToggleButton",
            //      |     action: console.log,
            //      |     settings: { checked: false },
            //      |     connections: {
            //      |         onChange: function(val) {
            //      |             console.log(this.get("checked"));
            //      |         }
            //      |     }
            //      | });
            //
            //      Example of defining a custom separator type to a toolbar widget.
            //      | toolbar.add({
            //      |     name: "button",
            //      |     label: "Space Cadet",
            //      |     action: console.log,
            //      |     settings: {
            //      |         separate: true,
            //      |         separator: 'epi/shell/widget/SpaceToolbarSeparator'
            //      |     }
            //      | });
            //
            // tags:
            //      public

            var self = this;
            //Chain the deferreds so we call them in a sequence
            var addItem = function (items) {

                var item = items.shift();
                if (!item) {
                    return null;
                }
                return when(self._addSingle(item), function () {
                    return addItem(items);
                });
            };

            if (lang.isArray(definitions)) {
                return addItem(definitions);
            } else {
                this._addSingle(definitions);
            }
        },

        addSeparator: function (/* String|DomNode|_Widget */reference, /* String?|Int? */position) {
            // summary:
            //    Add a separator to the toolbar based on standard domConstruct.place conventions,
            //    or passing a Widget reference. If no reference is given the separator is added to the end of the toolbar.
            // tags:
            //    protected

            var getSeparator = function () {
                if (reference.separator) {
                    var moduleName = reference.separator.replace(/\./g, "/"),
                        deferred = new Deferred();

                    require([moduleName], function (separatorClass) {
                        deferred.resolve(new separatorClass());
                    });

                    return deferred.promise;
                }
                return new ToolbarSeparator();
            };

            when(getSeparator(), lang.hitch(this, function (separator) {
                var refNode = reference ? reference.domNode : this.domNode;
                domConstruct.place(separator.domNode, refNode, position);
            }));
        },

        _removeSingle: function (name) {
            // Removes a single widget using its name.
            var widget = this._widgetMap[name];

            delete this._widgetMap[name];
            if (widget) {
                this._removeSeparators(widget);
                widget.destroy();
            }
        },

        _removeSeparators: function (widget) {
            // summary:
            //    Removes neighboring separators from the widget if required. This is used
            //    when the widget is being removed.
            // tags:
            //    private

            if (!widget.domNode) {
                return;
            }

            var parent = widget.getParent();
            if (!parent) {
                return;
            }

            var separator;
            var siblings = parent.getChildren();
            var i = array.indexOf(siblings, widget);

            var next = (siblings.length > i) ? siblings[i + 1] : null;
            var previous = (i > 0) ? siblings[i - 1] : null;

            // If there is separator between this and the next 'real' sibling, evaluate whether to keep it.
            if (next && this._isSeparator(next) && siblings.length >= i + 2) {
                separator = next;
                next = siblings[i + 2];
                // Check if next wants to separate and this is not the first item.
                if (next.separate && i > 0) {
                    // Remove any previous separator to avoid double separators.
                    if (previous && this._isSeparator(previous)) {
                        previous.destroy();
                        return;
                    }
                } else {
                    separator.destroy();
                }
            }

            // If there is separator between this and the previous 'real' sibling, evaluate whether to keep it.
            if (previous && this._isSeparator(previous) && i - 2 >= 0) {
                separator = previous;
                previous = siblings[i - 2];
                if (!previous.separate) {
                    separator.destroy();
                }
            }
        },

        _isSeparator: function (widget) {
            return widget instanceof ToolbarSeparator || widget instanceof MenuSeparator;
        },

        remove: function (names) {
            // summary:
            //		Removes an item or an array of items from within the toolbar.
            //
            // names:
            //      A string representing the name or an array of names of widgets to be removed.
            //
            // tags:
            //      protected

            if (lang.isArray(names)) {
                array.forEach(names, lang.hitch(this, this._removeSingle));
            } else {
                this._removeSingle(names);
            }
        },

        clear: function () {
            // summary:
            //		Removes all items from the toolbar.
            // tags:
            //      protected

            for (var name in this._widgetMap) {
                this._removeSingle(name);
            }
        },

        destroy: function () {
            this.clear();

            this.inherited(arguments);
        },

        setItemProperty: function (/*String|Array*/names, /*String*/property, /*Object*/value) {
            // summary:
            //    Sets a value to a property of a toolbar item or a collection of toolbar items.
            // tags:
            //    protected

            if (lang.isArray(names)) {
                array.forEach(names, function (name) {
                    this._setInternal(name, property, value);
                });
            } else {
                this._setInternal(names, property, value);
            }
        },

        setItemVisibility: function (/*String*/name, /*Boolean*/visible) {
            // summary:
            //    Sets item visibility.
            // tags:
            //    protected

            var widget = this._widgetMap[name];
            if (widget) {
                domStyle.set(widget.domNode, { display: (visible ? "" : "none") });
                widget.set("itemVisibility", visible);
            }
        },

        _setInternal: function (/*String*/name, /*String*/property, /*Object*/value) {
            var widget = this._widgetMap[name];
            if (widget) {
                widget.set(property, value);
            }
        }
    });
});

},
'dijit/ToolbarSeparator':function(){
define("dijit/ToolbarSeparator", [
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.setSelectable
	"./_Widget",
	"./_TemplatedMixin"
], function(declare, dom, _Widget, _TemplatedMixin){

	// module:
	//		dijit/ToolbarSeparator


	return declare("dijit.ToolbarSeparator", [_Widget, _TemplatedMixin], {
		// summary:
		//		A spacer between two `dijit.Toolbar` items

		templateString: '<div class="dijitToolbarSeparator dijitInline" role="presentation"></div>',

		buildRendering: function(){
			this.inherited(arguments);
			dom.setSelectable(this.domNode, false);
		},

		isFocusable: function(){
			// summary:
			//		This widget isn't focusable, so pass along that fact.
			// tags:
			//		protected
			return false;
		}
	});
});

},
'dijit/Menu':function(){
define("dijit/Menu", [
	"require",
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/_base/event", // event.stop
	"dojo/dom", // dom.byId dom.isDescendant
	"dojo/dom-attr", // domAttr.get domAttr.set domAttr.has domAttr.remove
	"dojo/dom-geometry", // domStyle.getComputedStyle domGeometry.position
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/keys", // keys.F10
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"dojo/sniff", // has("ie"), has("quirks")
	"dojo/_base/window", // win.body win.doc.documentElement win.doc.frames
	"dojo/window", // winUtils.get
	"./popup",
	"./DropDownMenu",
	"dojo/ready"
], function(require, array, declare, event, dom, domAttr, domGeometry, domStyle, keys, lang, on,
			has, win, winUtils, pm, DropDownMenu, ready){

// module:
//		dijit/Menu

// Back compat w/1.6, remove for 2.0
if(has("dijit-legacy-requires")){
	ready(0, function(){
		var requires = ["dijit/MenuItem", "dijit/PopupMenuItem", "dijit/CheckedMenuItem", "dijit/MenuSeparator"];
		require(requires);	// use indirection so modules not rolled into a build
	});
}

return declare("dijit.Menu", DropDownMenu, {
	// summary:
	//		A context menu you can assign to multiple elements

	constructor: function(/*===== params, srcNodeRef =====*/){
		// summary:
		//		Create the widget.
		// params: Object|null
		//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
		//		and functions, typically callbacks like onClick.
		//		The hash can contain any of the widget's properties, excluding read-only properties.
		// srcNodeRef: DOMNode|String?
		//		If a srcNodeRef (DOM node) is specified:
		//
		//		- use srcNodeRef.innerHTML as my contents
		//		- replace srcNodeRef with my generated DOM tree

		this._bindings = [];
	},

	// targetNodeIds: [const] String[]
	//		Array of dom node ids of nodes to attach to.
	//		Fill this with nodeIds upon widget creation and it becomes context menu for those nodes.
	targetNodeIds: [],

	// selector: String?
	//		CSS expression to apply this Menu to descendants of targetNodeIds, rather than to
	//		the nodes specified by targetNodeIds themselves.  Useful for applying a Menu to
	//		a range of rows in a table, tree, etc.
	//
	//		The application must require() an appropriate level of dojo/query to handle the selector.
	selector: "",

	// TODO: in 2.0 remove support for multiple targetNodeIds.   selector gives the same effect.
	// So, change targetNodeIds to a targetNodeId: "", remove bindDomNode()/unBindDomNode(), etc.

	/*=====
	// currentTarget: [readonly] DOMNode
	//		For context menus, set to the current node that the Menu is being displayed for.
	//		Useful so that the menu actions can be tailored according to the node
	currentTarget: null,
	=====*/

	// contextMenuForWindow: [const] Boolean
	//		If true, right clicking anywhere on the window will cause this context menu to open.
	//		If false, must specify targetNodeIds.
	contextMenuForWindow: false,

	// leftClickToOpen: [const] Boolean
	//		If true, menu will open on left click instead of right click, similar to a file menu.
	leftClickToOpen: false,

	// refocus: Boolean
	//		When this menu closes, re-focus the element which had focus before it was opened.
	refocus: true,

	postCreate: function(){
		if(this.contextMenuForWindow){
			this.bindDomNode(this.ownerDocumentBody);
		}else{
			// TODO: should have _setTargetNodeIds() method to handle initialization and a possible
			// later set('targetNodeIds', ...) call.  There's also a problem that targetNodeIds[]
			// gets stale after calls to bindDomNode()/unBindDomNode() as it still is just the original list (see #9610)
			array.forEach(this.targetNodeIds, this.bindDomNode, this);
		}
		this.inherited(arguments);
	},

	// thanks burstlib!
	_iframeContentWindow: function(/* HTMLIFrameElement */iframe_el){
		// summary:
		//		Returns the window reference of the passed iframe
		// tags:
		//		private
		return winUtils.get(this._iframeContentDocument(iframe_el)) ||
			// Moz. TODO: is this available when defaultView isn't?
			this._iframeContentDocument(iframe_el)['__parent__'] ||
			(iframe_el.name && win.doc.frames[iframe_el.name]) || null;	//	Window
	},

	_iframeContentDocument: function(/* HTMLIFrameElement */iframe_el){
		// summary:
		//		Returns a reference to the document object inside iframe_el
		// tags:
		//		protected
		return iframe_el.contentDocument // W3
			|| (iframe_el.contentWindow && iframe_el.contentWindow.document) // IE
			|| (iframe_el.name && win.doc.frames[iframe_el.name] && win.doc.frames[iframe_el.name].document)
			|| null;	//	HTMLDocument
	},

	bindDomNode: function(/*String|DomNode*/ node){
		// summary:
		//		Attach menu to given node
		node = dom.byId(node, this.ownerDocument);

		var cn;	// Connect node

		// Support context menus on iframes.  Rather than binding to the iframe itself we need
		// to bind to the <body> node inside the iframe.
		if(node.tagName.toLowerCase() == "iframe"){
			var iframe = node,
				window = this._iframeContentWindow(iframe);
			cn = win.body(window.document);
		}else{
			// To capture these events at the top level, attach to <html>, not <body>.
			// Otherwise right-click context menu just doesn't work.
			cn = (node == win.body(this.ownerDocument) ? this.ownerDocument.documentElement : node);
		}


		// "binding" is the object to track our connection to the node (ie, the parameter to bindDomNode())
		var binding = {
			node: node,
			iframe: iframe
		};

		// Save info about binding in _bindings[], and make node itself record index(+1) into
		// _bindings[] array.  Prefix w/_dijitMenu to avoid setting an attribute that may
		// start with a number, which fails on FF/safari.
		domAttr.set(node, "_dijitMenu" + this.id, this._bindings.push(binding));

		// Setup the connections to monitor click etc., unless we are connecting to an iframe which hasn't finished
		// loading yet, in which case we need to wait for the onload event first, and then connect
		// On linux Shift-F10 produces the oncontextmenu event, but on Windows it doesn't, so
		// we need to monitor keyboard events in addition to the oncontextmenu event.
		var doConnects = lang.hitch(this, function(cn){
			var selector = this.selector,
				delegatedEvent = selector ?
					function(eventType){
						return on.selector(selector, eventType);
					} :
					function(eventType){
						return eventType;
					},
				self = this;
			return [
				// TODO: when leftClickToOpen is true then shouldn't space/enter key trigger the menu,
				// rather than shift-F10?
				on(cn, delegatedEvent(this.leftClickToOpen ? "click" : "contextmenu"), function(evt){
					event.stop(evt);

					if((new Date()).getTime() < this._lastKeyDown + 500){
						// Ignore contextmenu/click events that were already processed in keydown handler below.
						// But still call evt.stop() (above) so system context menu doesn't appear.
						return;
					}

					// Schedule context menu to be opened.
					// Note that this won't work will if the click was generated by the keyboard, while
					// focused on a <button> etc.   In that case evt.pageX and evt.pageY are either (0,0) or
					// wherever the mouse cursor is.  See keydown handler below.
					self._scheduleOpen(this, iframe, {x: evt.pageX, y: evt.pageY}, evt.target);
				}),
				on(cn, delegatedEvent("keydown"), function(evt){
					if(evt.keyCode == 93 ||									// context menu key
						(evt.shiftKey && evt.keyCode == keys.F10) ||		// shift-F10
						(this.leftClickToOpen && evt.keyCode == keys.SPACE)	// space key
					){
						event.stop(evt);

						// Open the menu around evt.target.  Note that "this" and evt.target
						// are likely different, especially for global context menu, where "this" is <body>.
						self._scheduleOpen(this, iframe, null, evt.target);	// no coords - open near evt.target

						this._lastKeyDown = (new Date()).getTime();
					}
				})
			];
		});
		binding.connects = cn ? doConnects(cn) : [];

		if(iframe){
			// Setup handler to [re]bind to the iframe when the contents are initially loaded,
			// and every time the contents change.
			// Need to do this b/c we are actually binding to the iframe's <body> node.
			// Note: can't use connect.connect(), see #9609.

			binding.onloadHandler = lang.hitch(this, function(){
				// want to remove old connections, but IE throws exceptions when trying to
				// access the <body> node because it's already gone, or at least in a state of limbo

				var window = this._iframeContentWindow(iframe);
					cn = win.body(window.document)
				binding.connects = doConnects(cn);
			});
			if(iframe.addEventListener){
				iframe.addEventListener("load", binding.onloadHandler, false);
			}else{
				iframe.attachEvent("onload", binding.onloadHandler);
			}
		}
	},

	unBindDomNode: function(/*String|DomNode*/ nodeName){
		// summary:
		//		Detach menu from given node

		var node;
		try{
			node = dom.byId(nodeName, this.ownerDocument);
		}catch(e){
			// On IE the dom.byId() call will get an exception if the attach point was
			// the <body> node of an <iframe> that has since been reloaded (and thus the
			// <body> node is in a limbo state of destruction.
			return;
		}

		// node["_dijitMenu" + this.id] contains index(+1) into my _bindings[] array
		var attrName = "_dijitMenu" + this.id;
		if(node && domAttr.has(node, attrName)){
			var bid = domAttr.get(node, attrName) - 1, b = this._bindings[bid], h;
			while((h = b.connects.pop())){
				h.remove();
			}

			// Remove listener for iframe onload events
			var iframe = b.iframe;
			if(iframe){
				if(iframe.removeEventListener){
					iframe.removeEventListener("load", b.onloadHandler, false);
				}else{
					iframe.detachEvent("onload", b.onloadHandler);
				}
			}

			domAttr.remove(node, attrName);
			delete this._bindings[bid];
		}
	},

	_scheduleOpen: function(delegatedTarget, iframe, coords, target){
		// summary:
		//		Set timer to display myself.  Using a timer rather than displaying immediately solves
		//		IE problem: without the delay, focus work in "open" causes the system
		//		context menu to appear in spite of evt.preventDefault().
		// delegatedTarget: Element
		//		The node specified in targetNodeIds or matching selector that the menu is being opened for.
		// iframe: HTMLIframeElement?
		//		Set if target is inside the specified iframe.
		// coords: Object
		//		x/y position to center the menu around.  Undefined if menu was opened via keyboard.
		// target: Element
		//		The actual clicked node, either delegatedTarget or a descendant.

		if(!this._openTimer){
			this._openTimer = this.defer(function(){
				delete this._openTimer;
				this._openMyself({
					target: target,
					delegatedTarget: delegatedTarget,
					iframe: iframe,
					coords: coords
				});
			}, 1);
		}
	},

	_openMyself: function(args){
		// summary:
		//		Internal function for opening myself when the user does a right-click or something similar.
		// args:
		//		This is an Object containing:
		//
		//		- target: The node that is being clicked.
		//		- delegatedTarget: The node from this.targetNodeIds or matching this.selector,
		//		  either the same as target or an ancestor of target.
		//		- iframe: If an `<iframe>` is being clicked, iframe points to that iframe
		//		- coords: Mouse cursor x/y coordinates.  Null when opened via keyboard.
		//		  Put menu at specified position in iframe (if iframe specified) or otherwise in viewport.
		//
		//		_openMyself() formerly took the event object, and since various code references
		//		evt.target (after connecting to _openMyself()), using an Object for parameters
		//		(so that old code still works).

		var target = args.target,
			iframe = args.iframe,
			coords = args.coords;

		// To be used by MenuItem event handlers to tell which node the menu was opened on
		this.currentTarget = args.delegatedTarget;

		// Get coordinates to open menu, either at specified (mouse) position or (if triggered via keyboard)
		// then near the node the menu is assigned to.
		if(coords){
			if(iframe){
				// Specified coordinates are on <body> node of an <iframe>, convert to match main document
				var ifc = domGeometry.position(iframe, true),
					window = this._iframeContentWindow(iframe),
					scroll = domGeometry.docScroll(window.document);

				var cs = domStyle.getComputedStyle(iframe),
					tp = domStyle.toPixelValue,
					left = (has("ie") && has("quirks") ? 0 : tp(iframe, cs.paddingLeft)) + (has("ie") && has("quirks") ? tp(iframe, cs.borderLeftWidth) : 0),
					top = (has("ie") && has("quirks") ? 0 : tp(iframe, cs.paddingTop)) + (has("ie") && has("quirks") ? tp(iframe, cs.borderTopWidth) : 0);

				coords.x += ifc.x + left - scroll.x;
				coords.y += ifc.y + top - scroll.y;
			}
		}else{
			coords = domGeometry.position(target, true);
			coords.x += 10;
			coords.y += 10;
		}

		var self = this;
		var prevFocusNode = this._focusManager.get("prevNode");
		var curFocusNode = this._focusManager.get("curNode");
		var savedFocusNode = !curFocusNode || (dom.isDescendant(curFocusNode, this.domNode)) ? prevFocusNode : curFocusNode;

		function closeAndRestoreFocus(){
			// user has clicked on a menu or popup
			if(self.refocus && savedFocusNode){
				savedFocusNode.focus();
			}
			pm.close(self);
		}
		pm.open({
			popup: this,
			x: coords.x,
			y: coords.y,
			onExecute: closeAndRestoreFocus,
			onCancel: closeAndRestoreFocus,
			orient: this.isLeftToRight() ? 'L' : 'R'
		});
		this.focus();

		this._onBlur = function(){
			this.inherited('_onBlur', arguments);
			// Usually the parent closes the child widget but if this is a context
			// menu then there is no parent
			pm.close(this);
			// don't try to restore focus; user has clicked another part of the screen
			// and set focus there
		};
	},

	destroy: function(){
		array.forEach(this._bindings, function(b){
			if(b){
				this.unBindDomNode(b.node);
			}
		}, this);
		this.inherited(arguments);
	}
});

});

},
'dijit/popup':function(){
define("dijit/popup", [
	"dojo/_base/array", // array.forEach array.some
	"dojo/aspect",
	"dojo/_base/connect",	// connect._keypress
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.isDescendant
	"dojo/dom-attr", // domAttr.set
	"dojo/dom-construct", // domConstruct.create domConstruct.destroy
	"dojo/dom-geometry", // domGeometry.isBodyLtr
	"dojo/dom-style", // domStyle.set
	"dojo/_base/event", // event.stop
	"dojo/has", // has("bgIframe")
	"dojo/keys",
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"./place",
	"./BackgroundIframe",
	"./main"	// dijit (defining dijit.popup to match API doc)
], function(array, aspect, connect, declare, dom, domAttr, domConstruct, domGeometry, domStyle, event, has, keys, lang, on,
			place, BackgroundIframe, dijit){

	// module:
	//		dijit/popup

	/*=====
	var __OpenArgs = {
		// popup: Widget
		//		widget to display
		// parent: Widget
		//		the button etc. that is displaying this popup
		// around: DomNode
		//		DOM node (typically a button); place popup relative to this node.  (Specify this *or* "x" and "y" parameters.)
		// x: Integer
		//		Absolute horizontal position (in pixels) to place node at.  (Specify this *or* "around" parameter.)
		// y: Integer
		//		Absolute vertical position (in pixels) to place node at.  (Specify this *or* "around" parameter.)
		// orient: Object|String
		//		When the around parameter is specified, orient should be a list of positions to try, ex:
		//	|	[ "below", "above" ]
		//		For backwards compatibility it can also be an (ordered) hash of tuples of the form
		//		(around-node-corner, popup-node-corner), ex:
		//	|	{ "BL": "TL", "TL": "BL" }
		//		where BL means "bottom left" and "TL" means "top left", etc.
		//
		//		dijit/popup.open() tries to position the popup according to each specified position, in order,
		//		until the popup appears fully within the viewport.
		//
		//		The default value is ["below", "above"]
		//
		//		When an (x,y) position is specified rather than an around node, orient is either
		//		"R" or "L".  R (for right) means that it tries to put the popup to the right of the mouse,
		//		specifically positioning the popup's top-right corner at the mouse position, and if that doesn't
		//		fit in the viewport, then it tries, in order, the bottom-right corner, the top left corner,
		//		and the top-right corner.
		// onCancel: Function
		//		callback when user has canceled the popup by:
		//
		//		1. hitting ESC or
		//		2. by using the popup widget's proprietary cancel mechanism (like a cancel button in a dialog);
		//		   i.e. whenever popupWidget.onCancel() is called, args.onCancel is called
		// onClose: Function
		//		callback whenever this popup is closed
		// onExecute: Function
		//		callback when user "executed" on the popup/sub-popup by selecting a menu choice, etc. (top menu only)
		// padding: place.__Position
		//		adding a buffer around the opening position. This is only useful when around is not set.
	};
	=====*/

	function destroyWrapper(){
		// summary:
		//		Function to destroy wrapper when popup widget is destroyed.
		//		Left in this scope to avoid memory leak on IE8 on refresh page, see #15206.
		if(this._popupWrapper){
			domConstruct.destroy(this._popupWrapper);
			delete this._popupWrapper;
		}
	}

	var PopupManager = declare(null, {
		// summary:
		//		Used to show drop downs (ex: the select list of a ComboBox)
		//		or popups (ex: right-click context menus).

		// _stack: dijit/_WidgetBase[]
		//		Stack of currently popped up widgets.
		//		(someone opened _stack[0], and then it opened _stack[1], etc.)
		_stack: [],

		// _beginZIndex: Number
		//		Z-index of the first popup.   (If first popup opens other
		//		popups they get a higher z-index.)
		_beginZIndex: 1000,

		_idGen: 1,

		_createWrapper: function(/*Widget*/ widget){
			// summary:
			//		Initialization for widgets that will be used as popups.
			//		Puts widget inside a wrapper DIV (if not already in one),
			//		and returns pointer to that wrapper DIV.

			var wrapper = widget._popupWrapper,
				node = widget.domNode;

			if(!wrapper){
				// Create wrapper <div> for when this widget [in the future] will be used as a popup.
				// This is done early because of IE bugs where creating/moving DOM nodes causes focus
				// to go wonky, see tests/robot/Toolbar.html to reproduce
				wrapper = domConstruct.create("div", {
					"class":"dijitPopup",
					style:{ display: "none"},
					role: "region",
					"aria-label": widget["aria-label"] || widget.label || widget.name || widget.id
				}, widget.ownerDocumentBody);
				wrapper.appendChild(node);

				var s = node.style;
				s.display = "";
				s.visibility = "";
				s.position = "";
				s.top = "0px";

				widget._popupWrapper = wrapper;
				aspect.after(widget, "destroy", destroyWrapper, true);
			}

			return wrapper;
		},

		moveOffScreen: function(/*Widget*/ widget){
			// summary:
			//		Moves the popup widget off-screen.
			//		Do not use this method to hide popups when not in use, because
			//		that will create an accessibility issue: the offscreen popup is
			//		still in the tabbing order.

			// Create wrapper if not already there
			var wrapper = this._createWrapper(widget);

			domStyle.set(wrapper, {
				visibility: "hidden",
				top: "-9999px",		// prevent transient scrollbar causing misalign (#5776), and initial flash in upper left (#10111)
				display: ""
			});
		},

		hide: function(/*Widget*/ widget){
			// summary:
			//		Hide this popup widget (until it is ready to be shown).
			//		Initialization for widgets that will be used as popups
			//
			//		Also puts widget inside a wrapper DIV (if not already in one)
			//
			//		If popup widget needs to layout it should
			//		do so when it is made visible, and popup._onShow() is called.

			// Create wrapper if not already there
			var wrapper = this._createWrapper(widget);

			domStyle.set(wrapper, "display", "none");
		},

		getTopPopup: function(){
			// summary:
			//		Compute the closest ancestor popup that's *not* a child of another popup.
			//		Ex: For a TooltipDialog with a button that spawns a tree of menus, find the popup of the button.
			var stack = this._stack;
			for(var pi=stack.length-1; pi > 0 && stack[pi].parent === stack[pi-1].widget; pi--){
				/* do nothing, just trying to get right value for pi */
			}
			return stack[pi];
		},

		open: function(/*__OpenArgs*/ args){
			// summary:
			//		Popup the widget at the specified position
			//
			// example:
			//		opening at the mouse position
			//		|		popup.open({popup: menuWidget, x: evt.pageX, y: evt.pageY});
			//
			// example:
			//		opening the widget as a dropdown
			//		|		popup.open({parent: this, popup: menuWidget, around: this.domNode, onClose: function(){...}});
			//
			//		Note that whatever widget called dijit/popup.open() should also listen to its own _onBlur callback
			//		(fired from _base/focus.js) to know that focus has moved somewhere else and thus the popup should be closed.

			var stack = this._stack,
				widget = args.popup,
				orient = args.orient || ["below", "below-alt", "above", "above-alt"],
				ltr = args.parent ? args.parent.isLeftToRight() : domGeometry.isBodyLtr(widget.ownerDocument),
				around = args.around,
				id = (args.around && args.around.id) ? (args.around.id+"_dropdown") : ("popup_"+this._idGen++);

			// If we are opening a new popup that isn't a child of a currently opened popup, then
			// close currently opened popup(s).   This should happen automatically when the old popups
			// gets the _onBlur() event, except that the _onBlur() event isn't reliable on IE, see [22198].
			while(stack.length && (!args.parent || !dom.isDescendant(args.parent.domNode, stack[stack.length-1].widget.domNode))){
				this.close(stack[stack.length-1].widget);
			}

			// Get pointer to popup wrapper, and create wrapper if it doesn't exist
			var wrapper = this._createWrapper(widget);


			domAttr.set(wrapper, {
				id: id,
				style: {
					zIndex: this._beginZIndex + stack.length
				},
				"class": "dijitPopup " + (widget.baseClass || widget["class"] || "").split(" ")[0] +"Popup",
				dijitPopupParent: args.parent ? args.parent.id : ""
			});

			if(has("bgIframe") && !widget.bgIframe){
				// setting widget.bgIframe triggers cleanup in _Widget.destroy()
				widget.bgIframe = new BackgroundIframe(wrapper);
			}

			// position the wrapper node and make it visible
			var best = around ?
				place.around(wrapper, around, orient, ltr, widget.orient ? lang.hitch(widget, "orient") : null) :
				place.at(wrapper, args, orient == 'R' ? ['TR','BR','TL','BL'] : ['TL','BL','TR','BR'], args.padding);

			wrapper.style.display = "";
			wrapper.style.visibility = "visible";
			widget.domNode.style.visibility = "visible";	// counteract effects from _HasDropDown

			var handlers = [];

			// provide default escape and tab key handling
			// (this will work for any widget, not just menu)
			handlers.push(on(wrapper, connect._keypress, lang.hitch(this, function(evt){
				if(evt.charOrCode == keys.ESCAPE && args.onCancel){
					event.stop(evt);
					args.onCancel();
				}else if(evt.charOrCode === keys.TAB){
					event.stop(evt);
					var topPopup = this.getTopPopup();
					if(topPopup && topPopup.onCancel){
						topPopup.onCancel();
					}
				}
			})));

			// watch for cancel/execute events on the popup and notify the caller
			// (for a menu, "execute" means clicking an item)
			if(widget.onCancel && args.onCancel){
				handlers.push(widget.on("cancel", args.onCancel));
			}

			handlers.push(widget.on(widget.onExecute ? "execute" : "change", lang.hitch(this, function(){
				var topPopup = this.getTopPopup();
				if(topPopup && topPopup.onExecute){
					topPopup.onExecute();
				}
			})));

			stack.push({
				widget: widget,
				parent: args.parent,
				onExecute: args.onExecute,
				onCancel: args.onCancel,
				onClose: args.onClose,
				handlers: handlers
			});

			if(widget.onOpen){
				// TODO: in 2.0 standardize onShow() (used by StackContainer) and onOpen() (used here)
				widget.onOpen(best);
			}

			return best;
		},

		close: function(/*Widget?*/ popup){
			// summary:
			//		Close specified popup and any popups that it parented.
			//		If no popup is specified, closes all popups.

			var stack = this._stack;

			// Basically work backwards from the top of the stack closing popups
			// until we hit the specified popup, but IIRC there was some issue where closing
			// a popup would cause others to close too.  Thus if we are trying to close B in [A,B,C]
			// closing C might close B indirectly and then the while() condition will run where stack==[A]...
			// so the while condition is constructed defensively.
			while((popup && array.some(stack, function(elem){return elem.widget == popup;})) ||
				(!popup && stack.length)){
				var top = stack.pop(),
					widget = top.widget,
					onClose = top.onClose;

				if(widget.onClose){
					// TODO: in 2.0 standardize onHide() (used by StackContainer) and onClose() (used here)
					widget.onClose();
				}

				var h;
				while(h = top.handlers.pop()){ h.remove(); }

				// Hide the widget and it's wrapper unless it has already been destroyed in above onClose() etc.
				if(widget && widget.domNode){
					this.hide(widget);
				}

				if(onClose){
					onClose();
				}
			}
		}
	});

	return (dijit.popup = new PopupManager());
});

},
'dijit/place':function(){
define("dijit/place", [
	"dojo/_base/array", // array.forEach array.map array.some
	"dojo/dom-geometry", // domGeometry.position
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/window", // win.body
	"./Viewport", // getEffectiveBox
	"./main"	// dijit (defining dijit.place to match API doc)
], function(array, domGeometry, domStyle, kernel, win, Viewport, dijit){

	// module:
	//		dijit/place


	function _place(/*DomNode*/ node, choices, layoutNode, aroundNodeCoords){
		// summary:
		//		Given a list of spots to put node, put it at the first spot where it fits,
		//		of if it doesn't fit anywhere then the place with the least overflow
		// choices: Array
		//		Array of elements like: {corner: 'TL', pos: {x: 10, y: 20} }
		//		Above example says to put the top-left corner of the node at (10,20)
		// layoutNode: Function(node, aroundNodeCorner, nodeCorner, size)
		//		for things like tooltip, they are displayed differently (and have different dimensions)
		//		based on their orientation relative to the parent.	 This adjusts the popup based on orientation.
		//		It also passes in the available size for the popup, which is useful for tooltips to
		//		tell them that their width is limited to a certain amount.	 layoutNode() may return a value expressing
		//		how much the popup had to be modified to fit into the available space.	 This is used to determine
		//		what the best placement is.
		// aroundNodeCoords: Object
		//		Size of aroundNode, ex: {w: 200, h: 50}

		// get {x: 10, y: 10, w: 100, h:100} type obj representing position of
		// viewport over document
		var view = Viewport.getEffectiveBox(node.ownerDocument);

		// This won't work if the node is inside a <div style="position: relative">,
		// so reattach it to win.doc.body.	 (Otherwise, the positioning will be wrong
		// and also it might get cutoff)
		if(!node.parentNode || String(node.parentNode.tagName).toLowerCase() != "body"){
			win.body(node.ownerDocument).appendChild(node);
		}

		var best = null;
		array.some(choices, function(choice){
			var corner = choice.corner;
			var pos = choice.pos;
			var overflow = 0;

			// calculate amount of space available given specified position of node
			var spaceAvailable = {
				w: {
					'L': view.l + view.w - pos.x,
					'R': pos.x - view.l,
					'M': view.w
				   }[corner.charAt(1)],
				h: {
					'T': view.t + view.h - pos.y,
					'B': pos.y - view.t,
					'M': view.h
				   }[corner.charAt(0)]
			};

			// Clear left/right position settings set earlier so they don't interfere with calculations,
			// specifically when layoutNode() (a.k.a. Tooltip.orient()) measures natural width of Tooltip
			var s = node.style;
			s.left = s.right = "auto";

			// configure node to be displayed in given position relative to button
			// (need to do this in order to get an accurate size for the node, because
			// a tooltip's size changes based on position, due to triangle)
			if(layoutNode){
				var res = layoutNode(node, choice.aroundCorner, corner, spaceAvailable, aroundNodeCoords);
				overflow = typeof res == "undefined" ? 0 : res;
			}

			// get node's size
			var style = node.style;
			var oldDisplay = style.display;
			var oldVis = style.visibility;
			if(style.display == "none"){
				style.visibility = "hidden";
				style.display = "";
			}
			var bb = domGeometry.position(node);
			style.display = oldDisplay;
			style.visibility = oldVis;

			// coordinates and size of node with specified corner placed at pos,
			// and clipped by viewport
			var
				startXpos = {
					'L': pos.x,
					'R': pos.x - bb.w,
					'M': Math.max(view.l, Math.min(view.l + view.w, pos.x + (bb.w >> 1)) - bb.w) // M orientation is more flexible
				}[corner.charAt(1)],
				startYpos = {
					'T': pos.y,
					'B': pos.y - bb.h,
					'M': Math.max(view.t, Math.min(view.t + view.h, pos.y + (bb.h >> 1)) - bb.h)
				}[corner.charAt(0)],
				startX = Math.max(view.l, startXpos),
				startY = Math.max(view.t, startYpos),
				endX = Math.min(view.l + view.w, startXpos + bb.w),
				endY = Math.min(view.t + view.h, startYpos + bb.h),
				width = endX - startX,
				height = endY - startY;

			overflow += (bb.w - width) + (bb.h - height);

			if(best == null || overflow < best.overflow){
				best = {
					corner: corner,
					aroundCorner: choice.aroundCorner,
					x: startX,
					y: startY,
					w: width,
					h: height,
					overflow: overflow,
					spaceAvailable: spaceAvailable
				};
			}

			return !overflow;
		});

		// In case the best position is not the last one we checked, need to call
		// layoutNode() again.
		if(best.overflow && layoutNode){
			layoutNode(node, best.aroundCorner, best.corner, best.spaceAvailable, aroundNodeCoords);
		}

		// And then position the node.  Do this last, after the layoutNode() above
		// has sized the node, due to browser quirks when the viewport is scrolled
		// (specifically that a Tooltip will shrink to fit as though the window was
		// scrolled to the left).
		var s = node.style;
		s.top = best.y + "px";
		s.left = best.x + "px";
		s.right = "auto";	// needed for FF or else tooltip goes to far left

		return best;
	}

	var place = {
		// summary:
		//		Code to place a DOMNode relative to another DOMNode.
		//		Load using require(["dijit/place"], function(place){ ... }).

		at: function(node, pos, corners, padding){
			// summary:
			//		Positions one of the node's corners at specified position
			//		such that node is fully visible in viewport.
			// description:
			//		NOTE: node is assumed to be absolutely or relatively positioned.
			// node: DOMNode
			//		The node to position
			// pos: dijit/place.__Position
			//		Object like {x: 10, y: 20}
			// corners: String[]
			//		Array of Strings representing order to try corners in, like ["TR", "BL"].
			//		Possible values are:
			//
			//		- "BL" - bottom left
			//		- "BR" - bottom right
			//		- "TL" - top left
			//		- "TR" - top right
			// padding: dijit/place.__Position?
			//		optional param to set padding, to put some buffer around the element you want to position.
			// example:
			//		Try to place node's top right corner at (10,20).
			//		If that makes node go (partially) off screen, then try placing
			//		bottom left corner at (10,20).
			//	|	place(node, {x: 10, y: 20}, ["TR", "BL"])
			var choices = array.map(corners, function(corner){
				var c = { corner: corner, pos: {x:pos.x,y:pos.y} };
				if(padding){
					c.pos.x += corner.charAt(1) == 'L' ? padding.x : -padding.x;
					c.pos.y += corner.charAt(0) == 'T' ? padding.y : -padding.y;
				}
				return c;
			});

			return _place(node, choices);
		},

		around: function(
			/*DomNode*/		node,
			/*DomNode|dijit/place.__Rectangle*/ anchor,
			/*String[]*/	positions,
			/*Boolean*/		leftToRight,
			/*Function?*/	layoutNode){

			// summary:
			//		Position node adjacent or kitty-corner to anchor
			//		such that it's fully visible in viewport.
			// description:
			//		Place node such that corner of node touches a corner of
			//		aroundNode, and that node is fully visible.
			// anchor:
			//		Either a DOMNode or a rectangle (object with x, y, width, height).
			// positions:
			//		Ordered list of positions to try matching up.
			//
			//		- before: places drop down to the left of the anchor node/widget, or to the right in the case
			//			of RTL scripts like Hebrew and Arabic; aligns either the top of the drop down
			//			with the top of the anchor, or the bottom of the drop down with bottom of the anchor.
			//		- after: places drop down to the right of the anchor node/widget, or to the left in the case
			//			of RTL scripts like Hebrew and Arabic; aligns either the top of the drop down
			//			with the top of the anchor, or the bottom of the drop down with bottom of the anchor.
			//		- before-centered: centers drop down to the left of the anchor node/widget, or to the right
			//			 in the case of RTL scripts like Hebrew and Arabic
			//		- after-centered: centers drop down to the right of the anchor node/widget, or to the left
			//			 in the case of RTL scripts like Hebrew and Arabic
			//		- above-centered: drop down is centered above anchor node
			//		- above: drop down goes above anchor node, left sides aligned
			//		- above-alt: drop down goes above anchor node, right sides aligned
			//		- below-centered: drop down is centered above anchor node
			//		- below: drop down goes below anchor node
			//		- below-alt: drop down goes below anchor node, right sides aligned
			// layoutNode: Function(node, aroundNodeCorner, nodeCorner)
			//		For things like tooltip, they are displayed differently (and have different dimensions)
			//		based on their orientation relative to the parent.	 This adjusts the popup based on orientation.
			// leftToRight:
			//		True if widget is LTR, false if widget is RTL.   Affects the behavior of "above" and "below"
			//		positions slightly.
			// example:
			//	|	placeAroundNode(node, aroundNode, {'BL':'TL', 'TR':'BR'});
			//		This will try to position node such that node's top-left corner is at the same position
			//		as the bottom left corner of the aroundNode (ie, put node below
			//		aroundNode, with left edges aligned).	If that fails it will try to put
			//		the bottom-right corner of node where the top right corner of aroundNode is
			//		(ie, put node above aroundNode, with right edges aligned)
			//

			// if around is a DOMNode (or DOMNode id), convert to coordinates
			var aroundNodePos = (typeof anchor == "string" || "offsetWidth" in anchor || "ownerSVGElement" in anchor)
				? domGeometry.position(anchor, true)
				: anchor;

			// Compute position and size of visible part of anchor (it may be partially hidden by ancestor nodes w/scrollbars)
			if(anchor.parentNode){
				// ignore nodes between position:relative and position:absolute
				var sawPosAbsolute = domStyle.getComputedStyle(anchor).position == "absolute";
				var parent = anchor.parentNode;
				while(parent && parent.nodeType == 1 && parent.nodeName != "BODY"){  //ignoring the body will help performance
					var parentPos = domGeometry.position(parent, true),
						pcs = domStyle.getComputedStyle(parent);
					if(/relative|absolute/.test(pcs.position)){
						sawPosAbsolute = false;
					}
					if(!sawPosAbsolute && /hidden|auto|scroll/.test(pcs.overflow)){
						var bottomYCoord = Math.min(aroundNodePos.y + aroundNodePos.h, parentPos.y + parentPos.h);
						var rightXCoord = Math.min(aroundNodePos.x + aroundNodePos.w, parentPos.x + parentPos.w);
						aroundNodePos.x = Math.max(aroundNodePos.x, parentPos.x);
						aroundNodePos.y = Math.max(aroundNodePos.y, parentPos.y);
						aroundNodePos.h = bottomYCoord - aroundNodePos.y;
						aroundNodePos.w = rightXCoord - aroundNodePos.x;
					}
					if(pcs.position == "absolute"){
						sawPosAbsolute = true;
					}
					parent = parent.parentNode;
				}
			}			

			var x = aroundNodePos.x,
				y = aroundNodePos.y,
				width = "w" in aroundNodePos ? aroundNodePos.w : (aroundNodePos.w = aroundNodePos.width),
				height = "h" in aroundNodePos ? aroundNodePos.h : (kernel.deprecated("place.around: dijit/place.__Rectangle: { x:"+x+", y:"+y+", height:"+aroundNodePos.height+", width:"+width+" } has been deprecated.  Please use { x:"+x+", y:"+y+", h:"+aroundNodePos.height+", w:"+width+" }", "", "2.0"), aroundNodePos.h = aroundNodePos.height);

			// Convert positions arguments into choices argument for _place()
			var choices = [];
			function push(aroundCorner, corner){
				choices.push({
					aroundCorner: aroundCorner,
					corner: corner,
					pos: {
						x: {
							'L': x,
							'R': x + width,
							'M': x + (width >> 1)
						   }[aroundCorner.charAt(1)],
						y: {
							'T': y,
							'B': y + height,
							'M': y + (height >> 1)
						   }[aroundCorner.charAt(0)]
					}
				})
			}
			array.forEach(positions, function(pos){
				var ltr =  leftToRight;
				switch(pos){
					case "above-centered":
						push("TM", "BM");
						break;
					case "below-centered":
						push("BM", "TM");
						break;
					case "after-centered":
						ltr = !ltr;
						// fall through
					case "before-centered":
						push(ltr ? "ML" : "MR", ltr ? "MR" : "ML");
						break;
					case "after":
						ltr = !ltr;
						// fall through
					case "before":
						push(ltr ? "TL" : "TR", ltr ? "TR" : "TL");
						push(ltr ? "BL" : "BR", ltr ? "BR" : "BL");
						break;
					case "below-alt":
						ltr = !ltr;
						// fall through
					case "below":
						// first try to align left borders, next try to align right borders (or reverse for RTL mode)
						push(ltr ? "BL" : "BR", ltr ? "TL" : "TR");
						push(ltr ? "BR" : "BL", ltr ? "TR" : "TL");
						break;
					case "above-alt":
						ltr = !ltr;
						// fall through
					case "above":
						// first try to align left borders, next try to align right borders (or reverse for RTL mode)
						push(ltr ? "TL" : "TR", ltr ? "BL" : "BR");
						push(ltr ? "TR" : "TL", ltr ? "BR" : "BL");
						break;
					default:
						// To assist dijit/_base/place, accept arguments of type {aroundCorner: "BL", corner: "TL"}.
						// Not meant to be used directly.
						push(pos.aroundCorner, pos.corner);
				}
			});

			var position = _place(node, choices, layoutNode, {w: width, h: height});
			position.aroundNodePos = aroundNodePos;

			return position;
		}
	};

	/*=====
	place.__Position = {
		// x: Integer
		//		horizontal coordinate in pixels, relative to document body
		// y: Integer
		//		vertical coordinate in pixels, relative to document body
	};
	place.__Rectangle = {
		// x: Integer
		//		horizontal offset in pixels, relative to document body
		// y: Integer
		//		vertical offset in pixels, relative to document body
		// w: Integer
		//		width in pixels.   Can also be specified as "width" for backwards-compatibility.
		// h: Integer
		//		height in pixels.   Can also be specified as "height" for backwards-compatibility.
	};
	=====*/

	return dijit.place = place;	// setting dijit.place for back-compat, remove for 2.0
});

},
'dijit/DropDownMenu':function(){
require({cache:{
'url:dijit/templates/Menu.html':"<table class=\"dijit dijitMenu dijitMenuPassive dijitReset dijitMenuTable\" role=\"menu\" tabIndex=\"${tabIndex}\"\n\t   data-dojo-attach-event=\"onkeypress:_onKeyPress\" cellspacing=\"0\">\n\t<tbody class=\"dijitReset\" data-dojo-attach-point=\"containerNode\"></tbody>\n</table>\n"}});
define("dijit/DropDownMenu", [
	"dojo/_base/declare", // declare
	"dojo/_base/event", // event.stop
	"dojo/keys", // keys
	"dojo/text!./templates/Menu.html",
	"./_OnDijitClickMixin",
	"./_MenuBase"
], function(declare, event, keys, template, _OnDijitClickMixin, _MenuBase){

	// module:
	//		dijit/DropDownMenu

	return declare("dijit.DropDownMenu", [_MenuBase, _OnDijitClickMixin], {
		// summary:
		//		A menu, without features for context menu (Meaning, drop down menu)

		templateString: template,

		baseClass: "dijitMenu",

		postCreate: function(){
			this.inherited(arguments);
			var l = this.isLeftToRight();
			this._openSubMenuKey = l ? keys.RIGHT_ARROW : keys.LEFT_ARROW;
			this._closeSubMenuKey = l ? keys.LEFT_ARROW : keys.RIGHT_ARROW;
			this.connectKeyNavHandlers([keys.UP_ARROW], [keys.DOWN_ARROW]);
		},

		_onKeyPress: function(/*Event*/ evt){
			// summary:
			//		Handle keyboard based menu navigation.
			// tags:
			//		protected

			if(evt.ctrlKey || evt.altKey){ return; }

			switch(evt.charOrCode){
				case this._openSubMenuKey:
					this._moveToPopup(evt);
					event.stop(evt);
					break;
				case this._closeSubMenuKey:
					if(this.parentMenu){
						if(this.parentMenu._isMenuBar){
							this.parentMenu.focusPrev();
						}else{
							this.onCancel(false);
						}
					}else{
						event.stop(evt);
					}
					break;
			}
		}
	});
});

},
'url:dijit/templates/Menu.html':"<table class=\"dijit dijitMenu dijitMenuPassive dijitReset dijitMenuTable\" role=\"menu\" tabIndex=\"${tabIndex}\"\n\t   data-dojo-attach-event=\"onkeypress:_onKeyPress\" cellspacing=\"0\">\n\t<tbody class=\"dijitReset\" data-dojo-attach-point=\"containerNode\"></tbody>\n</table>\n",
'dijit/_MenuBase':function(){
define("dijit/_MenuBase", [
	"dojo/_base/array",	// array.indexOf
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.isDescendant domClass.replace
	"dojo/dom-attr",
	"dojo/dom-class", // domClass.replace
	"dojo/_base/lang", // lang.hitch
	"dojo/mouse",	// mouse.enter, mouse.leave
	"dojo/on",
	"dojo/window",
	"./a11yclick",
	"./popup",
	"./registry",
	"./_Widget",
	"./_KeyNavContainer",
	"./_TemplatedMixin"
], function(array, declare, dom, domAttr, domClass, lang, mouse, on, winUtils,
			a11yclick, pm, registry, _Widget, _KeyNavContainer, _TemplatedMixin){


// module:
//		dijit/_MenuBase

return declare("dijit._MenuBase",
	[_Widget, _TemplatedMixin, _KeyNavContainer],
{
	// summary:
	//		Base class for Menu and MenuBar

	// parentMenu: [readonly] Widget
	//		pointer to menu that displayed me
	parentMenu: null,

	// popupDelay: Integer
	//		number of milliseconds before hovering (without clicking) causes the popup to automatically open.
	popupDelay: 500,

	// autoFocus: Boolean
	//		A toggle to control whether or not a Menu gets focused when opened as a drop down from a MenuBar
	//		or DropDownButton/ComboButton.   Note though that it always get focused when opened via the keyboard.
	autoFocus: false,

	childSelector: function(/*DOMNode*/ node){
		// summary:
		//		Selector (passed to on.selector()) used to identify MenuItem child widgets, but exclude inert children
		//		like MenuSeparator.  If subclass overrides to a string (ex: "> *"), the subclass must require dojo/query.
		// tags:
		//		protected

		var widget = registry.byNode(node);
		return node.parentNode == this.containerNode && widget && widget.focus;
	},

	postCreate: function(){
		var self = this,
			matches = typeof this.childSelector == "string" ? this.childSelector : lang.hitch(this, "childSelector");
		this.own(
			on(this.containerNode, on.selector(matches, mouse.enter), function(){
				self.onItemHover(registry.byNode(this));
			}),
			on(this.containerNode, on.selector(matches, mouse.leave), function(){
				self.onItemUnhover(registry.byNode(this));
			}),
			on(this.containerNode, on.selector(matches, a11yclick), function(evt){
				self.onItemClick(registry.byNode(this), evt);
				evt.stopPropagation();
				evt.preventDefault();
			})
		);
		this.inherited(arguments);
	},

	onExecute: function(){
		// summary:
		//		Attach point for notification about when a menu item has been executed.
		//		This is an internal mechanism used for Menus to signal to their parent to
		//		close them, because they are about to execute the onClick handler.  In
		//		general developers should not attach to or override this method.
		// tags:
		//		protected
	},

	onCancel: function(/*Boolean*/ /*===== closeAll =====*/){
		// summary:
		//		Attach point for notification about when the user cancels the current menu
		//		This is an internal mechanism used for Menus to signal to their parent to
		//		close them.  In general developers should not attach to or override this method.
		// tags:
		//		protected
	},

	_moveToPopup: function(/*Event*/ evt){
		// summary:
		//		This handles the right arrow key (left arrow key on RTL systems),
		//		which will either open a submenu, or move to the next item in the
		//		ancestor MenuBar
		// tags:
		//		private

		if(this.focusedChild && this.focusedChild.popup && !this.focusedChild.disabled){
			this.onItemClick(this.focusedChild, evt);
		}else{
			var topMenu = this._getTopMenu();
			if(topMenu && topMenu._isMenuBar){
				topMenu.focusNext();
			}
		}
	},

	_onPopupHover: function(/*Event*/ /*===== evt =====*/){
		// summary:
		//		This handler is called when the mouse moves over the popup.
		// tags:
		//		private

		// if the mouse hovers over a menu popup that is in pending-close state,
		// then stop the close operation.
		// This can't be done in onItemHover since some popup targets don't have MenuItems (e.g. ColorPicker)
		if(this.currentPopup && this.currentPopup._pendingClose_timer){
			var parentMenu = this.currentPopup.parentMenu;
			// highlight the parent menu item pointing to this popup
			if(parentMenu.focusedChild){
				parentMenu.focusedChild._setSelected(false);
			}
			parentMenu.focusedChild = this.currentPopup.from_item;
			parentMenu.focusedChild._setSelected(true);
			// cancel the pending close
			this._stopPendingCloseTimer(this.currentPopup);
		}
	},

	onItemHover: function(/*MenuItem*/ item){
		// summary:
		//		Called when cursor is over a MenuItem.
		// tags:
		//		protected

		// Don't do anything unless user has "activated" the menu by:
		//		1) clicking it
		//		2) opening it from a parent menu (which automatically focuses it)
		if(this.isActive){
			this.focusChild(item);
			if(this.focusedChild.popup && !this.focusedChild.disabled && !this.hover_timer){
				this.hover_timer = this.defer("_openPopup", this.popupDelay);
			}
		}
		// if the user is mixing mouse and keyboard navigation,
		// then the menu may not be active but a menu item has focus,
		// but it's not the item that the mouse just hovered over.
		// To avoid both keyboard and mouse selections, use the latest.
		if(this.focusedChild){
			this.focusChild(item);
		}
		this._hoveredChild = item;

		item._set("hovering", true);
	},

	_onChildBlur: function(item){
		// summary:
		//		Called when a child MenuItem becomes inactive because focus
		//		has been removed from the MenuItem *and* it's descendant menus.
		// tags:
		//		private
		this._stopPopupTimer();
		item._setSelected(false);
		// Close all popups that are open and descendants of this menu
		var itemPopup = item.popup;
		if(itemPopup){
			this._stopPendingCloseTimer(itemPopup);
			itemPopup._pendingClose_timer = this.defer(function(){
				itemPopup._pendingClose_timer = null;
				if(itemPopup.parentMenu){
					itemPopup.parentMenu.currentPopup = null;
				}
				pm.close(itemPopup); // this calls onClose
			}, this.popupDelay);
		}
	},

	onItemUnhover: function(/*MenuItem*/ item){
		// summary:
		//		Callback fires when mouse exits a MenuItem
		// tags:
		//		protected

		if(this.isActive){
			this._stopPopupTimer();
		}
		if(this._hoveredChild == item){ this._hoveredChild = null; }

		item._set("hovering", false);
	},

	_stopPopupTimer: function(){
		// summary:
		//		Cancels the popup timer because the user has stop hovering
		//		on the MenuItem, etc.
		// tags:
		//		private
		if(this.hover_timer){
			this.hover_timer = this.hover_timer.remove();
		}
	},

	_stopPendingCloseTimer: function(/*dijit/_WidgetBase*/ popup){
		// summary:
		//		Cancels the pending-close timer because the close has been preempted
		// tags:
		//		private
		if(popup._pendingClose_timer){
			popup._pendingClose_timer = popup._pendingClose_timer.remove();
		}
	},

	_stopFocusTimer: function(){
		// summary:
		//		Cancels the pending-focus timer because the menu was closed before focus occured
		// tags:
		//		private
		if(this._focus_timer){
			this._focus_timer = this._focus_timer.remove();
		}
	},

	_getTopMenu: function(){
		// summary:
		//		Returns the top menu in this chain of Menus
		// tags:
		//		private
		for(var top=this; top.parentMenu; top=top.parentMenu);
		return top;
	},

	onItemClick: function(/*dijit/_WidgetBase*/ item, /*Event*/ evt){
		// summary:
		//		Handle clicks on an item.
		// tags:
		//		private

		// this can't be done in _onFocus since the _onFocus events occurs asynchronously
		if(typeof this.isShowingNow == 'undefined'){ // non-popup menu
			this._markActive();
		}

		this.focusChild(item);

		if(item.disabled){ return false; }

		if(item.popup){
			this._openPopup(evt.type == "keypress");
		}else{
			// before calling user defined handler, close hierarchy of menus
			// and restore focus to place it was when menu was opened
			this.onExecute();

			// user defined handler for click
			item._onClick ? item._onClick(evt) : item.onClick(evt);
		}
	},

	_openPopup: function(/*Boolean*/ focus){
		// summary:
		//		Open the popup to the side of/underneath the current menu item, and optionally focus first item
		// tags:
		//		protected

		this._stopPopupTimer();
		var from_item = this.focusedChild;
		if(!from_item){ return; } // the focused child lost focus since the timer was started
		var popup = from_item.popup;
		if(!popup.isShowingNow){
			if(this.currentPopup){
				this._stopPendingCloseTimer(this.currentPopup);
				pm.close(this.currentPopup);
			}
			popup.parentMenu = this;
			popup.from_item = from_item; // helps finding the parent item that should be focused for this popup
			var self = this;
			pm.open({
				parent: this,
				popup: popup,
				around: from_item.domNode,
				orient: this._orient || ["after", "before"],
				onCancel: function(){ // called when the child menu is canceled
					// set isActive=false (_closeChild vs _cleanUp) so that subsequent hovering will NOT open child menus
					// which seems aligned with the UX of most applications (e.g. notepad, wordpad, paint shop pro)
					self.focusChild(from_item);	// put focus back on my node
					self._cleanUp();			// close the submenu (be sure this is done _after_ focus is moved)
					from_item._setSelected(true); // oops, _cleanUp() deselected the item
					self.focusedChild = from_item;	// and unset focusedChild
				},
				onExecute: lang.hitch(this, "_cleanUp")
			});

			this.currentPopup = popup;

			// detect mouseovers to handle lazy mouse movements that temporarily focus other menu items
			if(this.popupHoverHandle){
				this.popupHoverHandle.remove();
			}
			this.own(this.popupHoverHandle = on.once(popup.domNode, "mouseover", lang.hitch(self, "_onPopupHover")));
		}

		if(focus && popup.focus){
			// If user is opening the popup via keyboard (right arrow, or down arrow for MenuBar), then focus the popup.
			// If the cursor happens to collide with the popup, it will generate an onmouseover event
			// even though the mouse wasn't moved.  Use defer() to call popup.focus so that
			// our focus() call overrides the onmouseover event, rather than vice-versa.  (#8742)
			popup._focus_timer = this.defer(lang.hitch(popup, function(){
				this._focus_timer = null;
				this.focus();
			}));
		}
	},

	_markActive: function(){
		// summary:
		//		Mark this menu's state as active.
		//		Called when this Menu gets focus from:
		//
		//		1. clicking it (mouse or via space/arrow key)
		//		2. being opened by a parent menu.
		//
		//		This is not called just from mouse hover.
		//		Focusing a menu via TAB does NOT automatically set isActive
		//		since TAB is a navigation operation and not a selection one.
		//		For Windows apps, pressing the ALT key focuses the menubar
		//		menus (similar to TAB navigation) but the menu is not active
		//		(ie no dropdown) until an item is clicked.
		this.isActive = true;
		domClass.replace(this.domNode, "dijitMenuActive", "dijitMenuPassive");
	},

	onOpen: function(/*Event*/ /*===== e =====*/){
		// summary:
		//		Callback when this menu is opened.
		//		This is called by the popup manager as notification that the menu
		//		was opened.
		// tags:
		//		private

		this.isShowingNow = true;
		this._markActive();
	},

	_markInactive: function(){
		// summary:
		//		Mark this menu's state as inactive.
		this.isActive = false; // don't do this in _onBlur since the state is pending-close until we get here
		domClass.replace(this.domNode, "dijitMenuPassive", "dijitMenuActive");
	},

	onClose: function(){
		// summary:
		//		Callback when this menu is closed.
		//		This is called by the popup manager as notification that the menu
		//		was closed.
		// tags:
		//		private

		this._stopFocusTimer();
		this._markInactive();
		this.isShowingNow = false;
		this.parentMenu = null;
	},

	_closeChild: function(){
		// summary:
		//		Called when submenu is clicked or focus is lost.  Close hierarchy of menus.
		// tags:
		//		private
		this._stopPopupTimer();

		if(this.currentPopup){
			// If focus is on a descendant MenuItem then move focus to me,
			// because IE doesn't like it when you display:none a node with focus,
			// and also so keyboard users don't lose control.
			// Likely, immediately after a user defined onClick handler will move focus somewhere
			// else, like a Dialog.
			if(array.indexOf(this._focusManager.activeStack, this.id) >= 0){
				domAttr.set(this.focusedChild.focusNode, "tabIndex", this.tabIndex);
				this.focusedChild.focusNode.focus();
			}
			// Close all popups that are open and descendants of this menu
			pm.close(this.currentPopup);
			this.currentPopup = null;
		}

		if(this.focusedChild){ // unhighlight the focused item
			this.focusedChild._setSelected(false);
			this.onItemUnhover(this.focusedChild);
			this.focusedChild = null;
		}
	},

	_onItemFocus: function(/*MenuItem*/ item){
		// summary:
		//		Called when child of this Menu gets focus from:
		//
		//		1. clicking it
		//		2. tabbing into it
		//		3. being opened by a parent menu.
		//
		//		This is not called just from mouse hover.
		if(this._hoveredChild && this._hoveredChild != item){
			this.onItemUnhover(this._hoveredChild);	// any previous mouse movement is trumped by focus selection
		}
	},

	_onBlur: function(){
		// summary:
		//		Called when focus is moved away from this Menu and it's submenus.
		// tags:
		//		protected
		this._cleanUp();
		this.inherited(arguments);
	},

	_cleanUp: function(){
		// summary:
		//		Called when the user is done with this menu.  Closes hierarchy of menus.
		// tags:
		//		private

		this._closeChild(); // don't call this.onClose since that's incorrect for MenuBar's that never close
		if(typeof this.isShowingNow == 'undefined'){ // non-popup menu doesn't call onClose
			this._markInactive();
		}
	}
});

});

},
'dijit/MenuSeparator':function(){
require({cache:{
'url:dijit/templates/MenuSeparator.html':"<tr class=\"dijitMenuSeparator\" role=\"separator\">\n\t<td class=\"dijitMenuSeparatorIconCell\">\n\t\t<div class=\"dijitMenuSeparatorTop\"></div>\n\t\t<div class=\"dijitMenuSeparatorBottom\"></div>\n\t</td>\n\t<td colspan=\"3\" class=\"dijitMenuSeparatorLabelCell\">\n\t\t<div class=\"dijitMenuSeparatorTop dijitMenuSeparatorLabel\"></div>\n\t\t<div class=\"dijitMenuSeparatorBottom\"></div>\n\t</td>\n</tr>"}});
define("dijit/MenuSeparator", [
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.setSelectable
	"./_WidgetBase",
	"./_TemplatedMixin",
	"./_Contained",
	"dojo/text!./templates/MenuSeparator.html"
], function(declare, dom, _WidgetBase, _TemplatedMixin, _Contained, template){

	// module:
	//		dijit/MenuSeparator

	return declare("dijit.MenuSeparator", [_WidgetBase, _TemplatedMixin, _Contained], {
		// summary:
		//		A line between two menu items

		templateString: template,

		buildRendering: function(){
			this.inherited(arguments);
			dom.setSelectable(this.domNode, false);
		},

		isFocusable: function(){
			// summary:
			//		Override to always return false
			// tags:
			//		protected

			return false; // Boolean
		}
	});
});

},
'dijit/_Contained':function(){
define("dijit/_Contained", [
	"dojo/_base/declare", // declare
	"./registry"	// registry.getEnclosingWidget(), registry.byNode()
], function(declare, registry){

	// module:
	//		dijit/_Contained

	return declare("dijit._Contained", null, {
		// summary:
		//		Mixin for widgets that are children of a container widget
		//
		// example:
		//	|	// make a basic custom widget that knows about it's parents
		//	|	declare("my.customClass",[dijit._Widget,dijit._Contained],{});

		_getSibling: function(/*String*/ which){
			// summary:
			//		Returns next or previous sibling
			// which:
			//		Either "next" or "previous"
			// tags:
			//		private
			var node = this.domNode;
			do{
				node = node[which+"Sibling"];
			}while(node && node.nodeType != 1);
			return node && registry.byNode(node);	// dijit/_WidgetBase
		},

		getPreviousSibling: function(){
			// summary:
			//		Returns null if this is the first child of the parent,
			//		otherwise returns the next element sibling to the "left".

			return this._getSibling("previous"); // dijit/_WidgetBase
		},

		getNextSibling: function(){
			// summary:
			//		Returns null if this is the last child of the parent,
			//		otherwise returns the next element sibling to the "right".

			return this._getSibling("next"); // dijit/_WidgetBase
		},

		getIndexInParent: function(){
			// summary:
			//		Returns the index of this widget within its container parent.
			//		It returns -1 if the parent does not exist, or if the parent
			//		is not a dijit._Container

			var p = this.getParent();
			if(!p || !p.getIndexOfChild){
				return -1; // int
			}
			return p.getIndexOfChild(this); // int
		}
	});
});

},
'url:dijit/templates/MenuSeparator.html':"<tr class=\"dijitMenuSeparator\" role=\"separator\">\n\t<td class=\"dijitMenuSeparatorIconCell\">\n\t\t<div class=\"dijitMenuSeparatorTop\"></div>\n\t\t<div class=\"dijitMenuSeparatorBottom\"></div>\n\t</td>\n\t<td colspan=\"3\" class=\"dijitMenuSeparatorLabelCell\">\n\t\t<div class=\"dijitMenuSeparatorTop dijitMenuSeparatorLabel\"></div>\n\t\t<div class=\"dijitMenuSeparatorBottom\"></div>\n\t</td>\n</tr>",
'epi/shell/widget/WidgetFactory':function(){
﻿define("epi/shell/widget/WidgetFactory", [
    "dojo/_base/array",
    "dojo/_base/Deferred",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/window",
    "dojo/dom",
    "dojo/promise/all",
    "dojo/when",
    "epi/dependency",

    // Default handlers added for these widgets in _addDefaultHandlers, added here to prevent incomplete layer builds
    "dijit/_Widget",
    "dijit/_Container",
    "dojox/layout/GridContainerLite",
    "epi/shell/widget/ToolbarDropDownButton",
    "epi/shell/widget/ToolbarSet"
], function (
    array,
    Deferred,
    declare,
    lang,
    win,
    dom,
    all,
    when,
    dependency
) {

    return declare(null, {
        // summary:
        //		Factory for creating widgets
        // tags:
        //      public

        // _domNodeName: [private] String
        // tags:
        //      const
        _domNodeName: "[domnode]",

        // _handlers: [private] String[]
        //      Array with handlers
        // tags:
        //      private
        _handlers: null,

        // moduleManager: [protected] epi.ModuleManager
        //
        moduleManager: null,

        // messageService: [protected] epi/shell/MessageService
        //      A service used for error notifications
        messageService: undefined,

        onWidgetHierarchyCreated: null,
        /*=====
        onWidgetHierarchyCreated: function(widget, componentDefinition){
        // summary:
        //		Called when a widget has been created and added to it's parent.
        //
        // widget: dijit/_Widget
        //		The created widget
        //
        // componentDefinition: Object
        //      Object with parameters for the widget
        //
        // tags:
        //		public callback
        },
        =====*/

        onWidgetCreated: null,
        /*=====
        onWidgetCreated: function(widget, componentDefinition){
        // summary:
        //		Called when a widget has been created.
        //
        // widget: dijit/_Widget
        //		The created widget
        //
        // componentDefinition: Object
        //      Object with parameters for the widget
        //
        // tags:
        //		public callback
        },
        =====*/

        constructor: function (params) {
            declare.safeMixin(this, params);

            // instantiate new array of handlers
            this._handlers = [];

            // add default handlers
            this._addDefaultHandlers();

            this.messageService = this.messageService || dependency.resolve("epi.shell.MessageService");

            this.moduleManager = this.moduleManager || dependency.resolve("epi.ModuleManager");

        },

        createWidgets: function (/* dijit/_Widget|DomNode|String */root, /* Object[] */componentsDefinition,/* bool? */startupWidgets) {
            // summary:
            //		Create widgets from componentDefinition and add them to root
            //
            // root:
            //      Where to place widgets
            //
            // componentsDefinition:
            //      Array of component definitions. Component definition must include properties `widgetType` and `settings` object (instance parameters).
            //      Optional property is `components` with an array of componentDefinitions. For example
            //  |   {
            //  |       widgetType: "dijit/ColorPalette",
            //  |       settings: {
            //  |           palette: "3x4"
            //  |       },
            //  |       anotherProperty: 1
            //  |   }
            //
            //      or:
            //  |   {
            //  |       widgetType: "dijit/layout/BorderContainer",
            //  |       settings: {
            //  |           design: "sidebar"
            //  |       },
            //  |       components: [
            //  |           {
            //  |               widgetType: "dijit/ColorPalette",
            //  |               settings: {
            //  |                   palette: "3x4",
            //  |                   region: center
            //  |               },
            //  |           }
            //  |       ]
            //  |   }
            //
            // example:
            //      Create widgets
            //  |   dojo.require("dijit/ColorPalette");
            //  |   var wf = new epi/shell/widget/WidgetFactory();
            //  |   var node = dojo.create("div");
            //  |   var widgets = wf.createWidgets(node, {
            //  |       widgetType: "dijit/ColorPalette",
            //  |       settings: {
            //  |           palette: "3x4"
            //  |       }
            //  |   });
            //
            // returns:
            //      Array of widgets.
            //
            // tags:
            //      public

            var syntheticRoot,
                getDep,
                widgetsTypes = [],
                moduleStartupDeferreds = [],
                createWidgetsDeferred = new Deferred(),
                errorHandle;

            startupWidgets === false ? startupWidgets = false : startupWidgets = true;

            if (typeof (root) === "string") {
                root = dom.byId(root);
            }

            // wrap definition in array if we only pass one object definition
            if (componentsDefinition && !lang.isArray(componentsDefinition)) {
                componentsDefinition = [componentsDefinition];
            }

            if (this.getHandler(root).type === this._domNodeName) {
                syntheticRoot = win.doc.createDocumentFragment();
                // _WidgetBase.placeAt assumes something without a tagname is a widget
                // so we add one to make it select the correct code path
                syntheticRoot.tagName = "documentFragment";
            }

            //Create a list of all dependencies for the component
            getDep = lang.hitch(this, function (components) {
                if (components instanceof Array) {
                    for (var i in components) {
                        getDep(components[i]);
                    }
                }
                if (components.components instanceof Array) {
                    getDep(components.components);
                }
                if (components.moduleName) {
                    //Startup the module
                    moduleStartupDeferreds.push(this.moduleManager.startModules(components.moduleName));
                }
                if (components.widgetPackage) {
                    widgetsTypes.push(components.widgetPackage);
                } else if (components.widgetType) {
                    widgetsTypes.push(components.widgetType);
                }
            });
            getDep(componentsDefinition);

            //List to errors on require
            errorHandle = require.on("error", function (error) {
                console.error(error.stack || error);

                //Remove error handle
                errorHandle.remove();

                //Reject the deferred and pass the error
                createWidgetsDeferred.reject(error);
            });

            //When all modules has been started, require the components
            all(moduleStartupDeferreds).then(lang.hitch(this, function () {

                //Require all dependencies for the component
                require(widgetsTypes, lang.hitch(this, function () {

                    // Remove error handle
                    // Important, otherwise memory will leak
                    errorHandle.remove();

                    this._createWidgets(syntheticRoot || root, componentsDefinition).then(function (widgets) {

                        // Startup widgets
                        try {

                            syntheticRoot && root.appendChild(syntheticRoot);

                            if (startupWidgets) {
                                array.forEach(widgets, function (w) {
                                    !w._started && w.startup && w.startup();
                                });
                            }
                            createWidgetsDeferred.resolve(widgets);
                        } catch (err) {
                            createWidgetsDeferred.reject(err);
                        }

                    }, createWidgetsDeferred.reject);
                }));
            }));

            return createWidgetsDeferred;
        },

        _createWidgets: function (/* dijit/_Widget|DomNode */root, /* Object[]? */componentsDefinition) {
            // summary:
            //		Create widgets from componentDefinition and add them to root
            //
            // tags:
            //      private
            //
            var dfd = new Deferred(),
                createdWidgets = [],
                handler = this.getHandler(root),
                deferreds;

            if (!handler || !lang.isArray(componentsDefinition)) {
                dfd.resolve(createdWidgets);
                return dfd;
            }

            deferreds = array.map(componentsDefinition, lang.hitch(this, function (componentDefinition) {
                // init modules
                var done = new Deferred(),
                    widgetCreated = this._createInternal(root, componentDefinition, handler);

                widgetCreated.then(function (widget) {
                    if (widget) {
                        createdWidgets.push(widget);
                    }
                    done.resolve(widget);

                }, function (e) {
                    console.error(e.stack || e);

                    done.reject(e);
                });

                return done;

            }));

            all(deferreds).then(
                function () {
                    dfd.resolve(createdWidgets);
                },
                function (result) {
                    console.error("WidgetFactory: Could not resolve one or more widgets");
                    dfd.reject(result);
                }
            );

            return dfd;
        },

        _createInternal: function (root, componentDefinition, handler) {
            var deferred = new Deferred();
            when(handler.instantiateWidgetFunction(componentDefinition), lang.hitch(this, function (widget) {
                if (widget) {

                    try {
                        if (typeof this.onWidgetCreated == "function") {
                            this.onWidgetCreated(widget, componentDefinition);
                        }

                        handler.addWidgetToRootFunction(root, widget);
                    } catch (e) {
                        deferred.reject(e);
                        return;
                    }

                    var childrenCreated = new Deferred();

                    if (componentDefinition.components && lang.isArray(componentDefinition.components) && componentDefinition.components.length > 0) {
                        childrenCreated = this._createWidgets(widget, componentDefinition.components);
                    } else {
                        childrenCreated.resolve();
                    }
                    childrenCreated.then(lang.hitch(this, function () {
                        try {
                            if (typeof this.onWidgetHierarchyCreated == "function") {
                                this.onWidgetHierarchyCreated(widget, componentDefinition);
                            }
                            deferred.resolve(widget);
                        } catch (e) {
                            deferred.reject(e);
                        }
                    }), deferred.reject);
                } else {
                    deferred.reject();
                }
            }), function (error) {
                console.error(error.stack || error);
                deferred.reject(error);
            });
            return deferred;
        },

        defaultWidgetInstantiator: function (/* Object */componentDefinition) {
            // summary:
            //		Instantiates a widget from a componentDefinition. See createWidgets() for info on componentDefinition properties.
            //
            // componentDefinition:
            //      A component definition
            //
            // tags:
            //      public
            //

            var def = new Deferred();

            var settings = lang.mixin({}, componentDefinition.settings, { componentId: componentDefinition.id });

            var widgetType = componentDefinition.widgetType;

            // declareds without define
            var widgetCtor = componentDefinition.widgetPackage && lang.getObject(componentDefinition.widgetType);

            //List to errors on require
            var errorHandle = require.on("error", function (error) {
                console.error(error.stack || error);

                //Remove error handle
                errorHandle.remove();

                //Reject the deferred and pass the error
                def.reject(error);
            });

            function constructorExecutor(ctor) {

                //Create the widget using the resolved widget type
                var widget = new ctor(settings);
                if (componentDefinition.connections) {
                    for (var evt in componentDefinition.connections) {
                        if (componentDefinition.connections[evt]) {
                            widget.connect(widget, evt, componentDefinition.connections[evt]);
                        }
                    }
                }
                errorHandle.remove();
                def.resolve(widget);
            }

            if (widgetCtor) {
                constructorExecutor(widgetCtor);
            } else {
                require([widgetType], lang.hitch(this, constructorExecutor));
            }
            return def;
        },

        _notifyError: function (msg) {
            if (this.messageService) {
                this.messageService.put("error", msg);
            }
        },

        _addDefaultHandlers: function () {
            // summary:
            //		Adds default handlers for most common widgets
            //
            // tags:
            //      private
            //

            // DomNode handler, to be used
            this.addHandler(this._domNodeName, function (rootWidget, widget) {
                widget.placeAt(rootWidget);
            }, null, function () {
                return true;
            });

            // Widgets
            this.addHandler("dijit/_Widget", function (rootWidget, widget) {
                rootWidget.set("content", widget);
            });
            this.addHandler("dijit/_Container", function (rootWidget, widget) {
                rootWidget.addChild(widget);
            });
            this.addHandler("dojox/layout/GridContainerLite", function (rootWidget, widget) {
                rootWidget.addChild(widget, widget.params.column);
            });

            // Implements addChild but does not inherit from _Container
            this.addHandler("epi/shell/widget/ToolbarDropDownButton", function (rootWidget, widget) {
                rootWidget.addChild(widget);
            });
            this.addHandler("epi/shell/widget/ToolbarSet", function (rootWidget, widget) {
                rootWidget.addChild(widget);
            });
        },


        addHandler: function (/* String */typeName, /* Function */addWidgetToRootFunction, /* Function? */instantiateWidgetFunction, /* Function? */canHandle) {
            // summary:
            //      Add a handler for a widget type
            //
            // description:
            //      The handler is an object which will create a widget and then add it to a root (domnode or widget).
            //
            // typeName:
            //      Name of a widget type
            //
            // addWidgetToRootFunction:
            //      A function which adds a widget to a root. For example:
            //  |    function (/* dijit/_Container */ rootWidget, /* dijit/_Widget */widget) {
            //  |       rootWidget.addChild(widget);
            //  |    }
            //
            // instantiateWidgetFunction:
            //      A function to creates and returns a widget. For example:
            //  |   function(componentDefinition) {
            //  |       dojo.require(componentDefinition.widgetType);
            //  |       var widgetConstructor = dojo.getObject(componentDefinition.widgetType);
            //  |       var widget = new widgetConstructor(componentDefinition.settings);
            //  |       return new MyWidget();
            //  |   }
            //
            // canHandle:
            //      Optional function which returns true if handler can handle the root. For example
            //  |   function (/* Object */ object, /* Class */ widgetConstructor) {
            //  |       if (typeof (object["isInstanceOf"]) === "function") {
            //  |           return object.isInstanceOf(aWidgetConstructor);
            //  |       }
            //  |       return false;
            //  |   }
            //
            // tags:
            //      public
            //

            // default handler function, check if handler is an instance of the aWidgetConstructor
            var defaultCanHandle = function (widget, constructor) {
                return constructor && lang.isFunction(widget.isInstanceOf) && widget.isInstanceOf(constructor);
            };

            var callback = lang.hitch(this, function (type) {
                var handler = this.getHandlerByTypeName(typeName);
                var canHandleFunction = canHandle || defaultCanHandle;
                var exists = handler ? true : false;

                if (!exists) {
                    handler = {};
                }

                var widgetInstantiator = instantiateWidgetFunction || lang.hitch(this, this.defaultWidgetInstantiator);

                handler.type = typeName;
                handler.instantiateWidgetFunction = widgetInstantiator;
                handler.addWidgetToRootFunction = addWidgetToRootFunction;
                handler.widgetConstructor = type;
                handler.canHandle = canHandleFunction;

                if (!exists) {
                    this._handlers.push(handler);
                }
            });

            if (typeName !== this._domNodeName) {
                require([typeName], callback);
            } else {
                callback();
            }
        },

        getHandler: function (/* DomNode|dijit/_Widget */root) {
            // summary:
            //		Gets the handler for a root
            //
            // description:
            //      Searches for the most specific handler, i.e. in reverse order of added handlers
            //
            // root:
            //      DomNode or a widget instance
            //
            // returns:
            //      A handler object
            //
            // tags:
            //      private
            //

            var handlers = this._handlers;

            for (var i = handlers.length - 1; i >= 0; i--) {
                if (handlers[i].canHandle(root, handlers[i].widgetConstructor)) {
                    return handlers[i]; // Object
                }
            }
            return null; // Object
        },

        getHandlerByTypeName: function (/* String */typeName) {
            // summary:
            //      Gets a handler for a typeName with an exact match
            //
            // typeName:
            //      Name of the type
            //
            // returns:
            //      A handler object
            //
            // tags:
            //      public
            //
            var handlers = this._handlers;
            for (var i = 0; i < handlers.length; i++) {
                if (handlers[i].type === typeName) {
                    return handlers[i]; // Object
                }
            }
            return null; // Object
        },

        removeHandler: function (/* String */typeName) {
            // summary:
            //      Remove handler by type name
            //
            // typeName: String
            //      Name of the type
            //
            // tags:
            //      public
            //
            var handlers = this._handlers;
            for (var i = 0; i < handlers.length; i++) {
                if (handlers[i].type === typeName) {
                    handlers.splice(i, 1);
                }
            }
        },

        listHandlers: function () {
            // summary:
            //      List handler names
            //
            // returns:
            //      Array of strings
            //
            // tags:
            //      public
            //
            var handlers = this._handlers;

            return array.map(handlers, function (handler) {
                return handler.type;
            }); // String[]
        }
    });
});

},
'dojox/layout/GridContainerLite':function(){
require({cache:{
'url:dojox/layout/resources/GridContainer.html':"<div id=\"${id}\" class=\"gridContainer\" dojoAttachPoint=\"containerNode\" tabIndex=\"0\" dojoAttachEvent=\"onkeypress:_selectFocus\">\n\t<div dojoAttachPoint=\"gridContainerDiv\">\n\t\t<table class=\"gridContainerTable\" dojoAttachPoint=\"gridContainerTable\" cellspacing=\"0\" cellpadding=\"0\">\n\t\t\t<tbody>\n\t\t\t\t<tr dojoAttachPoint=\"gridNode\" >\n\t\t\t\t\t\n\t\t\t\t</tr>\n\t\t\t</tbody>\n\t\t</table>\n\t</div>\n</div>"}});
define("dojox/layout/GridContainerLite", [
	"dojo/_base/kernel",
	"dojo/text!./resources/GridContainer.html",
	"dojo/_base/declare", // declare 
	"dojo/query",
	"dojo/_base/sniff",
	"dojo/dom-class",
	"dojo/dom-style",
	"dojo/dom-geometry",
	"dojo/dom-construct",
	"dojo/dom-attr", // domAttr.get
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/_base/event",
	"dojo/keys", // keys
	"dojo/topic", // topic.publish()
	"dijit/registry",
	"dijit/focus",
	"dijit/_base/focus", // dijit.getFocus()
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dijit/layout/_LayoutWidget",
	"dojo/_base/NodeList",
	"dojox/mdnd/AreaManager", "dojox/mdnd/DropIndicator",
	"dojox/mdnd/dropMode/OverDropMode","dojox/mdnd/AutoScroll"
],function(dojo, template, declare, query, has, domClass, domStyle, geom, domConstruct, domAttr, array, lang, events, keys, topic, registry, focus, baseFocus, _WidgetBase, _TemplatedMixin, _LayoutWidget, NodeList){

	var gcl = declare(
		"dojox.layout.GridContainerLite",
		[_LayoutWidget, _TemplatedMixin],
	{
		// summary:
		//		The GridContainerLite is a container of child elements that are placed in a kind of grid.
		//
		// description:
		//		GridContainerLite displays the child elements by column
		//		(ie: the children widths are fixed by the column width of the grid but
		//		the children heights are free).
		//		Each child is movable by drag and drop inside the GridContainer.
		//		The position of other children is automatically calculated when a child is moved.
		//
		// example:
		// 	|	<div dojoType="dojox.layout.GridContainerLite" nbZones="3" isAutoOrganized="true">
		// 	|		<div dojoType="dijit.layout.ContentPane">Content Pane 1 : Drag Me !</div>
		// 	|		<div dojoType="dijit.layout.ContentPane">Content Pane 2 : Drag Me !</div>
		// 	|		<div dojoType="dijit.layout.ContentPane">Content Pane 3 : Drag Me !</div>
		// 	|	</div>
		//
		// example:
		// 	|	dojo.ready(function(){
		// 	|		var cpane1 = new dijit.layout.ContentPane({
		//	|			title:"cpane1", content: "Content Pane 1 : Drag Me !"
		//	|		}),
		// 	|		cpane2 = new dijit.layout.ContentPane({
		//	|			title:"cpane2",
		//	|			content: "Content Pane 2 : Drag Me !"
		//	|		}),
		// 	|		cpane3 = new dijit.layout.ContentPane({
		//	|			title:"cpane3",
		//	|			content: "Content Pane 3 : Drag Me !"
		//	|		});
		// 	|
		// 	|		var widget = new dojox.layout.GridContainerLite({
		// 	|			nbZones: 3,
		// 	|			isAutoOrganized: true
		// 	|		}, dojo.byId("idNode"));
		// 	|		widget.addChild(cpane1, 0, 0);
		// 	|		widget.addChild(cpane2, 1, 0);
		// 	|		widget.addChild(cpane3, 2, 1);
		// 	|		widget.startup();
		// 	|	});

		// autoRefresh: Boolean
		//		Enable the refresh of registered areas on drag start.
		autoRefresh: true,


		// templateString: String
		//		template of gridContainer.
		templateString: template,

		// dragHandleClass: Array
		//		CSS class enabling a drag handle on a child.
		dragHandleClass: "dojoxDragHandle",

		// nbZones: Integer
		//		The number of dropped zones, by default 1.
		nbZones: 1,

		// doLayout: Boolean
		//		If true, change the size of my currently displayed child to match my size.
		doLayout: true,

		// isAutoOrganized: Boolean
		//		If true, widgets are organized automatically,
		//		else the attribute colum of child will define the right column.
		isAutoOrganized: true,

		// acceptTypes: Array
		//		The GridContainer will only accept the children that fit to the types.
		acceptTypes: [],

		// colWidths: String
		//		A comma separated list of column widths. If the column widths do not add up
		//		to 100, the remaining columns split the rest of the width evenly
		//		between them.
		colWidths: "",

		constructor: function(/*Object*/props, /*DOMNode*/node){
			this.acceptTypes = (props || {}).acceptTypes || ["text"];
			this._disabled = true;
		},

		postCreate: function(){
			//console.log("dojox.layout.GridContainerLite ::: postCreate");
			this.inherited(arguments);
			this._grid = [];

			this._createCells();

			// need to resize dragged child when it's dropped.
			this.subscribe("/dojox/mdnd/drop", "resizeChildAfterDrop");
			this.subscribe("/dojox/mdnd/drag/start", "resizeChildAfterDragStart");

			this._dragManager = dojox.mdnd.areaManager();
			// console.info("autorefresh ::: ", this.autoRefresh);
			this._dragManager.autoRefresh = this.autoRefresh;

			//	Add specific dragHandleClass to the manager.
			this._dragManager.dragHandleClass = this.dragHandleClass;

			if(this.doLayout){
				this._border = {
					h: has("ie") ? geom.getBorderExtents(this.gridContainerTable).h : 0,
					w: (has("ie") == 6) ? 1 : 0
				}
			}else{
				domStyle.set(this.domNode, "overflowY", "hidden");
				domStyle.set(this.gridContainerTable, "height", "auto");
			}
		},

		startup: function(){
			//console.log("dojox.layout.GridContainerLite ::: startup");
			if(this._started){ return; }

			if(this.isAutoOrganized){
				this._organizeChildren();
			}
			else{
				this._organizeChildrenManually();
			}

			// Need to call getChildren because getChildren return null
			// The children are not direct children because of _organizeChildren method
			array.forEach(this.getChildren(), function(child){ 
			  child.startup();
			});

			// Need to enable the Drag And Drop only if the GridContainer is visible.
			if(this._isShown()){
				this.enableDnd();
			}
			this.inherited(arguments);
		},

		resizeChildAfterDrop: function(/*Node*/node, /*Object*/targetArea, /*Integer*/indexChild){
			// summary:
			//		Resize the GridContainerLite inner table and the dropped widget.
			// description:
			//		These components are resized only if the targetArea.node is a
			//		child of this instance of gridContainerLite.
			//		To be resized, the dropped node must have also a method resize.
			// node:
			//		domNode of dropped widget.
			// targetArea:
			//		AreaManager Object containing information of targetArea
			// indexChild:
			//		Index where the dropped widget has been placed
			// returns:
			//		True if resized.

			//console.log("dojox.layout.GridContainerLite ::: resizeChildAfterDrop");
			if(this._disabled){
				return false;
			}
			if(registry.getEnclosingWidget(targetArea.node) == this){
				var widget = registry.byNode(node);
				if(widget.resize && lang.isFunction(widget.resize)){
					widget.resize();
				}

				// Update the column of the widget
				widget.set("column", node.parentNode.cellIndex);

				if(this.doLayout){
					var domNodeHeight = this._contentBox.h,
						divHeight = geom.getContentBox(this.gridContainerDiv).h;
					if(divHeight >= domNodeHeight){
						domStyle.set(this.gridContainerTable, "height",
								(domNodeHeight - this._border.h) + "px");
					}
				}
				return true;
			}
			return false;
		},

		resizeChildAfterDragStart: function(/*Node*/node, /*Object*/sourceArea, /*Integer*/indexChild){
			// summary:
			//		Resize the GridContainerLite inner table only if the drag source
			//		is a child of this gridContainer.
			// node:
			//		domNode of dragged widget.
			// sourceArea:
			//		AreaManager Object containing information of sourceArea
			// indexChild:
			//		Index where the dragged widget has been placed

			//console.log("dojox.layout.GridContainerLite ::: resizeChildAfterDragStart");
			if(this._disabled){
				return false;
			}
			if(registry.getEnclosingWidget(sourceArea.node) == this){
				this._draggedNode = node;
				if(this.doLayout){
					geom.getMarginBox(this.gridContainerTable, {
						h: geom.getContentBox(this.gridContainerDiv).h - this._border.h
					});
				}
				return true;
			}
			return false;
		},

		getChildren: function(){
			// summary:
			//		A specific method which returns children after they were placed in zones.
			// returns:
			//		A NodeList containing all children (widgets).
			// tags:
			//		protected

			var children = new NodeList();
			array.forEach(this._grid, function(dropZone){
				query("> [widgetId]", dropZone.node).map(registry.byNode).forEach(function(item){
				  children.push(item);
				});
			});
			return children;	// Array
		},

		_isShown: function(){
			// summary:
			//		Check if the domNode is visible or not.
			// returns:
			//		true if the content is currently shown
			// tags:
			//		protected

			//console.log("dojox.layout.GridContainerLite ::: _isShown");
			if("open" in this){		// for TitlePane, etc.
				return this.open;		// Boolean
			}
			else{
				var node = this.domNode;
				return (node.style.display != 'none') && (node.style.visibility != 'hidden') && !domClass.contains(node, "dijitHidden"); // Boolean
			}
		},

		layout: function(){
			// summary:
			//		Resize of each child

			//console.log("dojox.layout.GridContainerLite ::: layout");
			if(this.doLayout){
				var contentBox = this._contentBox;
				geom.getMarginBox(this.gridContainerTable, {
					h: contentBox.h - this._border.h
				});
				geom.getContentBox(this.domNode, {
					w: contentBox.w - this._border.w
				});
			}
			array.forEach(this.getChildren(), function(widget){
				if(widget.resize && lang.isFunction(widget.resize)){
					widget.resize();
				}
			});
		},

		onShow: function(){
			// summary:
			//		Enabled the Drag And Drop if it's necessary.

			//console.log("dojox.layout.GridContainerLite ::: onShow");
			if(this._disabled){
				this.enableDnd();
			}
		},

		onHide: function(){
			// summary:
			//		Disabled the Drag And Drop if it's necessary.

			//console.log("dojox.layout.GridContainerLite ::: onHide");
			if(!this._disabled){
				this.disableDnd();
			}
		},

		_createCells: function(){
			// summary:
			//		Create the columns of the GridContainer.
			// tags:
			//		protected

			//console.log("dojox.layout.GridContainerLite ::: _createCells");
			if(this.nbZones === 0){ this.nbZones = 1; }
			var accept = this.acceptTypes.join(","),
				i = 0;

			var origWidths = this.colWidths || [];
			var widths = [];
			var colWidth;
			var widthSum = 0;

			// Calculate the widths of each column.
			for(i = 0; i < this.nbZones; i++){
				if(widths.length < origWidths.length){
					widthSum += origWidths[i];
					widths.push(origWidths[i]);
				}else{
					if(!colWidth){
						colWidth = (100 - widthSum)/(this.nbZones - i);
					}
					widths.push(colWidth);
				}
			}

			i = 0;
			while(i < this.nbZones){
				// Add the parameter accept in each zone used by AreaManager
				// (see method dojox.mdnd.AreaManager:registerByNode)
				this._grid.push({
					node: domConstruct.create("td", {
						'class': "gridContainerZone",
						accept: accept,
						id: this.id + "_dz" + i,
						style: {
							'width': widths[i] + "%"
						}
					}, this.gridNode)
				});
				i++;
			}
		},

		_getZonesAttr: function(){
			// summary:
			//		return array of zone (domNode)
			return query(".gridContainerZone",  this.containerNode);
		},

		enableDnd: function(){
			// summary:
			//		Enable the Drag And Drop for children of GridContainer.

			//console.log("dojox.layout.GridContainerLite ::: enableDnd");
			var m = this._dragManager;
			array.forEach(this._grid, function(dropZone){
				m.registerByNode(dropZone.node);
			});
			m._dropMode.updateAreas(m._areaList);
			this._disabled = false;
		},

		disableDnd: function(){
			// summary:
			//		Disable the Drag And Drop for children of GridContainer.

			//console.log("dojox.layout.GridContainerLite ::: disableDnd");
			var m = this._dragManager;
			array.forEach(this._grid, function(dropZone){
				m.unregister(dropZone.node);
			});
			m._dropMode.updateAreas(m._areaList);
			this._disabled = true;
		},

		_organizeChildren: function(){
			// summary:
			//		List all zones and insert child into columns.

			//console.log("dojox.layout.GridContainerLite ::: _organizeChildren");
			var children = dojox.layout.GridContainerLite.superclass.getChildren.call(this);
			var numZones = this.nbZones,
				numPerZone = Math.floor(children.length / numZones),
				mod = children.length % numZones,
				i = 0;
	//		console.log('numPerZone', numPerZone, ':: mod', mod);
			for(var z = 0; z < numZones; z++){
				for(var r = 0; r < numPerZone; r++){
					this._insertChild(children[i], z);
					i++;
				}
				if(mod > 0){
					try{
						this._insertChild(children[i], z);
						i++;
					}
					catch(e){
						console.error("Unable to insert child in GridContainer", e);
					}
					mod--;
				}
				else if(numPerZone === 0){
					break;	// Optimization
				}
			}
		},

		_organizeChildrenManually: function (){
			// summary:
			//		Organize children by column property of widget.

			//console.log("dojox.layout.GridContainerLite ::: _organizeChildrenManually");
			var children = dojox.layout.GridContainerLite.superclass.getChildren.call(this),
				length = children.length,
				child;
			for(var i = 0; i < length; i++){
				child = children[i];
				try{
					this._insertChild(child, child.column - 1);
				}
				catch(e){
					console.error("Unable to insert child in GridContainer", e);
				}
			}
		},

		_insertChild: function(/*Widget*/child, /*Integer*/column, /*Integer?*/p){
			// summary:
			//		Insert a child in a specific column of the GridContainer widget.
			// column:
			//		Column number
			// p:
			//		Place in the zone (0 - first)
			// returns:
			//		The widget inserted

			//console.log("dojox.layout.GridContainerLite ::: _insertChild", child, column, p);
			var zone = this._grid[column].node,
				length = zone.childNodes.length;
			if(typeof p === "undefined" || p > length){
				p = length;
			}
			if(this._disabled){
				domConstruct.place(child.domNode, zone, p);
				domAttr.set(child.domNode, "tabIndex", "0");
			}
			else{
				if(!child.dragRestriction){
					this._dragManager.addDragItem(zone, child.domNode, p, true);
				}
				else{
					domConstruct.place(child.domNode, zone, p);
					domAttr.set(child.domNode, "tabIndex", "0");
				}
			}
			child.set("column", column);
			return child; // Widget
		},

		removeChild: function(/*Widget*/ widget){
			//console.log("dojox.layout.GridContainerLite ::: removeChild");
			if(this._disabled){
				this.inherited(arguments);
			}
			else{
				this._dragManager.removeDragItem(widget.domNode.parentNode, widget.domNode);
			}
		},

		addService: function(/*Object*/child, /*Integer?*/column, /*Integer?*/p){
			//console.log("dojox.layout.GridContainerLite ::: addService");
			kernel.deprecated("addService is deprecated.", "Please use  instead.", "Future");
			this.addChild(child, column, p);
		},

		addChild: function(/*Object*/child, /*Integer?*/column, /*Integer?*/p){
			// summary:
			//		Add a child in a specific column of the GridContainer widget.
			// child:
			//		widget to insert
			// column:
			//		column number
			// p:
			//		place in the zone (first = 0)
			// returns:
			//		The widget inserted

			//console.log("dojox.layout.GridContainerLite ::: addChild");
			child.domNode.id = child.id;
			dojox.layout.GridContainerLite.superclass.addChild.call(this, child, 0);
			if(column < 0 || column == undefined){ column = 0; }
			if(p <= 0){ p = 0; }
			try{
				return this._insertChild(child, column, p);
			}
			catch(e){
				console.error("Unable to insert child in GridContainer", e);
			}
			return null; 	// Widget
		},

		_setColWidthsAttr: function(value){
			this.colWidths = lang.isString(value) ? value.split(",") : (lang.isArray(value) ? value : [value]);

			if(this._started){
				this._updateColumnsWidth();
			}
		},

		_updateColumnsWidth: function(/*Object*/ manager){
			// summary:
			//		Update the columns width.
			// manager:
			//		dojox.mdnd.AreaManager singleton
			// tags:
			//		private

			//console.log("dojox.layout.GridContainer ::: _updateColumnsWidth");
			var length = this._grid.length;

			var origWidths = this.colWidths || [];
			var widths = [];
			var colWidth;
			var widthSum = 0;
			var i;

			// Calculate the widths of each column.
			for(i = 0; i < length; i++){
				if(widths.length < origWidths.length){
					widthSum += origWidths[i] * 1;
					widths.push(origWidths[i]);
				}else{
					if(!colWidth){
						colWidth = (100 - widthSum)/(this.nbZones - i);

						// If the numbers don't work out, make the remaining columns
						// an even width and let the code below average
						// out the differences.
						if(colWidth < 0){
							colWidth = 100 / this.nbZones;
						}
					}
					widths.push(colWidth);
					widthSum += colWidth * 1;
				}
			}

			// If the numbers are wrong, divide them all so they add up to 100
			if(widthSum > 100){
				var divisor = 100 / widthSum;
				for(i = 0; i < widths.length; i++){
					widths[i] *= divisor;
				}
			}

			// Set the widths of each node
			for(i = 0; i < length; i++){
				this._grid[i].node.style.width = widths[i] + "%";
			}
		},

		_selectFocus: function(/*Event*/event){
			// summary:
			//		Enable keyboard accessibility into the GridContainer.
			// description:
			//		Possibility to move focus into the GridContainer (TAB, LEFT ARROW, RIGHT ARROW, UP ARROW, DOWN ARROW).
			//		Possibility to move GridContainer's children (Drag and Drop) with keyboard. (SHIFT +  ARROW).
			//		If the type of widget is not draggable, a popup is displayed.

			//console.log("dojox.layout.GridContainerLite ::: _selectFocus");
			if(this._disabled){ return; }
			var key = event.keyCode,
				k = keys,
				zone = null,
				cFocus = baseFocus.getFocus(),
				focusNode = cFocus.node,
				m = this._dragManager,
				found,
				i,
				j,
				r,
				children,
				area,
				widget;
			if(focusNode == this.containerNode){
				area = this.gridNode.childNodes;
				switch(key){
					case k.DOWN_ARROW:
					case k.RIGHT_ARROW:
						found = false;
						for(i = 0; i < area.length; i++){
							children = area[i].childNodes;
							for(j = 0; j < children.length; j++){
								zone = children[j];
								if(zone != null && zone.style.display != "none"){
									focus.focus(zone);
									events.stop(event);
									found = true;
									break;
								}
							}
							if(found){ break };
						}
					break;
					case k.UP_ARROW:
					case k.LEFT_ARROW:
						area = this.gridNode.childNodes;
						found = false;
						for(i = area.length-1; i >= 0 ; i--){
							children = area[i].childNodes;
							for(j = children.length; j >= 0; j--){
								zone = children[j];
								if(zone != null && zone.style.display != "none"){
									focus.focus(zone);
									events.stop(event);
									found = true;
									break;
								}
							}
							if(found){ break };
						}
					break;
				}
			}
			else{
				if(focusNode.parentNode.parentNode == this.gridNode){
					var child = (key == k.UP_ARROW || key == k.LEFT_ARROW) ? "lastChild" : "firstChild";
					var pos = (key == k.UP_ARROW || key == k.LEFT_ARROW) ? "previousSibling" : "nextSibling";
					switch(key){
						case k.UP_ARROW:
						case k.DOWN_ARROW:
							events.stop(event);
							found = false;
							var focusTemp = focusNode;
							while(!found){
								children = focusTemp.parentNode.childNodes;
								var num = 0;
								for(i = 0; i < children.length; i++){
									if(children[i].style.display != "none"){ num++ };
									if(num > 1){ break; }
								}
								if(num == 1){ return; }
								if(focusTemp[pos] == null){
									zone = focusTemp.parentNode[child];
								}
								else{
									zone = focusTemp[pos];
								}
								if(zone.style.display === "none"){
									focusTemp = zone;
								}
								else{
									found = true;
								}
							}
							if(event.shiftKey){
								var parent = focusNode.parentNode;
								for(i = 0; i < this.gridNode.childNodes.length; i++){
									if(parent == this.gridNode.childNodes[i]){
										break;
									}
								}
								children = this.gridNode.childNodes[i].childNodes;
								for(j = 0; j < children.length; j++){
									if(zone == children[j]){
										break;
									}
								}
								if(has("mozilla") || has("webkit")){ i-- };

								widget = registry.byNode(focusNode);
								if(!widget.dragRestriction){
									r = m.removeDragItem(parent, focusNode);
									this.addChild(widget, i, j);
									domAttr.set(focusNode, "tabIndex", "0");
									focus.focus(focusNode);
								}
								else{
									topic.publish("/dojox/layout/gridContainer/moveRestriction", this);
								}
							}
							else{
								focus.focus(zone);
							}
						break;
						case k.RIGHT_ARROW:
						case k.LEFT_ARROW:
							events.stop(event);
							if(event.shiftKey){
								var z = 0;
								if(focusNode.parentNode[pos] == null){
									if(has("ie") && key == k.LEFT_ARROW){
										z = this.gridNode.childNodes.length-1;
									}
								}
								else if(focusNode.parentNode[pos].nodeType == 3){
									z = this.gridNode.childNodes.length - 2;
								}
								else{
									for(i = 0; i < this.gridNode.childNodes.length; i++){
										if(focusNode.parentNode[pos] == this.gridNode.childNodes[i]){
											break;
										}
										z++;
									}
									if(has("mozilla") || has("webkit")){ z-- };
								}
								widget = registry.byNode(focusNode);
								var _dndType = focusNode.getAttribute("dndtype");
								if(_dndType == null){
									//check if it's a dijit object
									if(widget && widget.dndType){
										_dndType = widget.dndType.split(/\s*,\s*/);
									}
									else{
										_dndType = ["text"];
									}
								}
								else{
									_dndType = _dndType.split(/\s*,\s*/);
								}
								var accept = false;
								for(i = 0; i < this.acceptTypes.length; i++){
									for(j = 0; j < _dndType.length; j++){
										if(_dndType[j] == this.acceptTypes[i]){
											accept = true;
											break;
										}
									}
								}
								if(accept && !widget.dragRestriction){
									var parentSource = focusNode.parentNode,
										place = 0;
									if(k.LEFT_ARROW == key){
										var t = z;
										if(has("mozilla") || has("webkit")){ t = z + 1 };
										place = this.gridNode.childNodes[t].childNodes.length;
									}
									// delete of manager :
									r = m.removeDragItem(parentSource, focusNode);
									this.addChild(widget, z, place);
									domAttr.set(r, "tabIndex", "0");
									focus.focus(r);
								}
								else{
									topic.publish("/dojox/layout/gridContainer/moveRestriction", this);
								}
							}
							else{
								var node = focusNode.parentNode;
								while(zone === null){
									if(node[pos] !== null && node[pos].nodeType !== 3){
										node = node[pos];
									}
									else{
										if(pos === "previousSibling"){
											node = node.parentNode.childNodes[node.parentNode.childNodes.length-1];
										}
										else{
											node = node.parentNode.childNodes[has("ie") ? 0 : 1];
										}
									}
									zone = node[child];
									if(zone && zone.style.display == "none"){
										// check that all elements are not hidden
										children = zone.parentNode.childNodes;
										var childToSelect = null;
										if(pos == "previousSibling"){
											for(i = children.length-1; i >= 0; i--){
												if(children[i].style.display != "none"){
													childToSelect = children[i];
													break;
												}
											}
										}
										else{
											for(i = 0; i < children.length; i++){
												if(children[i].style.display != "none"){
													childToSelect = children[i];
													break;
												}
											}
										}
										if(!childToSelect){
											focusNode = zone;
											node = focusNode.parentNode;
											zone = null;
										}
										else{
											zone = childToSelect;
										}
									}
								}
								focus.focus(zone);
							}
						break;
					}
				}
			}
		},

		destroy: function(){
			//console.log("dojox.layout.GridContainerLite ::: destroy");
			var m = this._dragManager;
			array.forEach(this._grid, function(dropZone){
				m.unregister(dropZone.node);
			});
			this.inherited(arguments);
		}
	});

	gcl.ChildWidgetProperties = {
		// summary:
		//		Properties set on children of a GridContainerLite

		// column: String
		//		Column of the grid to place the widget.
		//		Defined only if dojo.require("dojox.layout.GridContainerLite") is done.
		column: "1",

		// dragRestriction: Boolean
		//		If true, the widget can not be draggable.
		//		Defined only if dojo.require("dojox.layout.GridContainerLite") is done.
		dragRestriction: false
	};

	// Add to widget base for benefit of parser.   Remove for 2.0.   Also, hide from doc viewer.
	lang.extend(_WidgetBase, /*===== {} || =====*/ gcl.ChildWidgetProperties);

	return gcl;
});

},
'url:dojox/layout/resources/GridContainer.html':"<div id=\"${id}\" class=\"gridContainer\" dojoAttachPoint=\"containerNode\" tabIndex=\"0\" dojoAttachEvent=\"onkeypress:_selectFocus\">\n\t<div dojoAttachPoint=\"gridContainerDiv\">\n\t\t<table class=\"gridContainerTable\" dojoAttachPoint=\"gridContainerTable\" cellspacing=\"0\" cellpadding=\"0\">\n\t\t\t<tbody>\n\t\t\t\t<tr dojoAttachPoint=\"gridNode\" >\n\t\t\t\t\t\n\t\t\t\t</tr>\n\t\t\t</tbody>\n\t\t</table>\n\t</div>\n</div>",
'dijit/_base/focus':function(){
define("dijit/_base/focus", [
	"dojo/_base/array", // array.forEach
	"dojo/dom", // dom.isDescendant
	"dojo/_base/lang", // lang.isArray
	"dojo/topic", // publish
	"dojo/_base/window", // win.doc win.doc.selection win.global win.global.getSelection win.withGlobal
	"../focus",
	"../main"	// for exporting symbols to dijit
], function(array, dom, lang, topic, win, focus, dijit){

	// module:
	//		dijit/_base/focus

	var exports = {
		// summary:
		//		Deprecated module to monitor currently focused node and stack of currently focused widgets.
		//		New code should access dijit/focus directly.

		// _curFocus: DomNode
		//		Currently focused item on screen
		_curFocus: null,

		// _prevFocus: DomNode
		//		Previously focused item on screen
		_prevFocus: null,

		isCollapsed: function(){
			// summary:
			//		Returns true if there is no text selected
			return dijit.getBookmark().isCollapsed;
		},

		getBookmark: function(){
			// summary:
			//		Retrieves a bookmark that can be used with moveToBookmark to return to the same range
			var bm, rg, tg, sel = win.doc.selection, cf = focus.curNode;

			if(win.global.getSelection){
				//W3C Range API for selections.
				sel = win.global.getSelection();
				if(sel){
					if(sel.isCollapsed){
						tg = cf? cf.tagName : "";
						if(tg){
							//Create a fake rangelike item to restore selections.
							tg = tg.toLowerCase();
							if(tg == "textarea" ||
									(tg == "input" && (!cf.type || cf.type.toLowerCase() == "text"))){
								sel = {
									start: cf.selectionStart,
									end: cf.selectionEnd,
									node: cf,
									pRange: true
								};
								return {isCollapsed: (sel.end <= sel.start), mark: sel}; //Object.
							}
						}
						bm = {isCollapsed:true};
						if(sel.rangeCount){
							bm.mark = sel.getRangeAt(0).cloneRange();
						}
					}else{
						rg = sel.getRangeAt(0);
						bm = {isCollapsed: false, mark: rg.cloneRange()};
					}
				}
			}else if(sel){
				// If the current focus was a input of some sort and no selection, don't bother saving
				// a native bookmark.  This is because it causes issues with dialog/page selection restore.
				// So, we need to create psuedo bookmarks to work with.
				tg = cf ? cf.tagName : "";
				tg = tg.toLowerCase();
				if(cf && tg && (tg == "button" || tg == "textarea" || tg == "input")){
					if(sel.type && sel.type.toLowerCase() == "none"){
						return {
							isCollapsed: true,
							mark: null
						}
					}else{
						rg = sel.createRange();
						return {
							isCollapsed: rg.text && rg.text.length?false:true,
							mark: {
								range: rg,
								pRange: true
							}
						};
					}
				}
				bm = {};

				//'IE' way for selections.
				try{
					// createRange() throws exception when dojo in iframe
					//and nothing selected, see #9632
					rg = sel.createRange();
					bm.isCollapsed = !(sel.type == 'Text' ? rg.htmlText.length : rg.length);
				}catch(e){
					bm.isCollapsed = true;
					return bm;
				}
				if(sel.type.toUpperCase() == 'CONTROL'){
					if(rg.length){
						bm.mark=[];
						var i=0,len=rg.length;
						while(i<len){
							bm.mark.push(rg.item(i++));
						}
					}else{
						bm.isCollapsed = true;
						bm.mark = null;
					}
				}else{
					bm.mark = rg.getBookmark();
				}
			}else{
				console.warn("No idea how to store the current selection for this browser!");
			}
			return bm; // Object
		},

		moveToBookmark: function(/*Object*/ bookmark){
			// summary:
			//		Moves current selection to a bookmark
			// bookmark:
			//		This should be a returned object from dijit.getBookmark()

			var _doc = win.doc,
				mark = bookmark.mark;
			if(mark){
				if(win.global.getSelection){
					//W3C Rangi API (FF, WebKit, Opera, etc)
					var sel = win.global.getSelection();
					if(sel && sel.removeAllRanges){
						if(mark.pRange){
							var n = mark.node;
							n.selectionStart = mark.start;
							n.selectionEnd = mark.end;
						}else{
							sel.removeAllRanges();
							sel.addRange(mark);
						}
					}else{
						console.warn("No idea how to restore selection for this browser!");
					}
				}else if(_doc.selection && mark){
					//'IE' way.
					var rg;
					if(mark.pRange){
						rg = mark.range;
					}else if(lang.isArray(mark)){
						rg = _doc.body.createControlRange();
						//rg.addElement does not have call/apply method, so can not call it directly
						//rg is not available in "range.addElement(item)", so can't use that either
						array.forEach(mark, function(n){
							rg.addElement(n);
						});
					}else{
						rg = _doc.body.createTextRange();
						rg.moveToBookmark(mark);
					}
					rg.select();
				}
			}
		},

		getFocus: function(/*Widget?*/ menu, /*Window?*/ openedForWindow){
			// summary:
			//		Called as getFocus(), this returns an Object showing the current focus
			//		and selected text.
			//
			//		Called as getFocus(widget), where widget is a (widget representing) a button
			//		that was just pressed, it returns where focus was before that button
			//		was pressed.   (Pressing the button may have either shifted focus to the button,
			//		or removed focus altogether.)   In this case the selected text is not returned,
			//		since it can't be accurately determined.
			//
			// menu: dijit/_WidgetBase|{domNode: DomNode} structure
			//		The button that was just pressed.  If focus has disappeared or moved
			//		to this button, returns the previous focus.  In this case the bookmark
			//		information is already lost, and null is returned.
			//
			// openedForWindow:
			//		iframe in which menu was opened
			//
			// returns:
			//		A handle to restore focus/selection, to be passed to `dijit.focus`
			var node = !focus.curNode || (menu && dom.isDescendant(focus.curNode, menu.domNode)) ? dijit._prevFocus : focus.curNode;
			return {
				node: node,
				bookmark: node && (node == focus.curNode) && win.withGlobal(openedForWindow || win.global, dijit.getBookmark),
				openedForWindow: openedForWindow
			}; // Object
		},

		// _activeStack: dijit/_WidgetBase[]
		//		List of currently active widgets (focused widget and it's ancestors)
		_activeStack: [],

		registerIframe: function(/*DomNode*/ iframe){
			// summary:
			//		Registers listeners on the specified iframe so that any click
			//		or focus event on that iframe (or anything in it) is reported
			//		as a focus/click event on the `<iframe>` itself.
			// description:
			//		Currently only used by editor.
			// returns:
			//		Handle to pass to unregisterIframe()
			return focus.registerIframe(iframe);
		},

		unregisterIframe: function(/*Object*/ handle){
			// summary:
			//		Unregisters listeners on the specified iframe created by registerIframe.
			//		After calling be sure to delete or null out the handle itself.
			// handle:
			//		Handle returned by registerIframe()

			handle && handle.remove();
		},

		registerWin: function(/*Window?*/targetWindow, /*DomNode?*/ effectiveNode){
			// summary:
			//		Registers listeners on the specified window (either the main
			//		window or an iframe's window) to detect when the user has clicked somewhere
			//		or focused somewhere.
			// description:
			//		Users should call registerIframe() instead of this method.
			// targetWindow:
			//		If specified this is the window associated with the iframe,
			//		i.e. iframe.contentWindow.
			// effectiveNode:
			//		If specified, report any focus events inside targetWindow as
			//		an event on effectiveNode, rather than on evt.target.
			// returns:
			//		Handle to pass to unregisterWin()

			return focus.registerWin(targetWindow, effectiveNode);
		},

		unregisterWin: function(/*Handle*/ handle){
			// summary:
			//		Unregisters listeners on the specified window (either the main
			//		window or an iframe's window) according to handle returned from registerWin().
			//		After calling be sure to delete or null out the handle itself.

			handle && handle.remove();
		}
	};

	// Override focus singleton's focus function so that dijit.focus()
	// has backwards compatible behavior of restoring selection (although
	// probably no one is using that).
	focus.focus = function(/*Object|DomNode */ handle){
		// summary:
		//		Sets the focused node and the selection according to argument.
		//		To set focus to an iframe's content, pass in the iframe itself.
		// handle:
		//		object returned by get(), or a DomNode

		if(!handle){ return; }

		var node = "node" in handle ? handle.node : handle,		// because handle is either DomNode or a composite object
			bookmark = handle.bookmark,
			openedForWindow = handle.openedForWindow,
			collapsed = bookmark ? bookmark.isCollapsed : false;

		// Set the focus
		// Note that for iframe's we need to use the <iframe> to follow the parentNode chain,
		// but we need to set focus to iframe.contentWindow
		if(node){
			var focusNode = (node.tagName.toLowerCase() == "iframe") ? node.contentWindow : node;
			if(focusNode && focusNode.focus){
				try{
					// Gecko throws sometimes if setting focus is impossible,
					// node not displayed or something like that
					focusNode.focus();
				}catch(e){/*quiet*/}
			}
			focus._onFocusNode(node);
		}

		// set the selection
		// do not need to restore if current selection is not empty
		// (use keyboard to select a menu item) or if previous selection was collapsed
		// as it may cause focus shift (Esp in IE).
		if(bookmark && win.withGlobal(openedForWindow || win.global, dijit.isCollapsed) && !collapsed){
			if(openedForWindow){
				openedForWindow.focus();
			}
			try{
				win.withGlobal(openedForWindow || win.global, dijit.moveToBookmark, null, [bookmark]);
			}catch(e2){
				/*squelch IE internal error, see http://trac.dojotoolkit.org/ticket/1984 */
			}
		}
	};

	// For back compatibility, monitor changes to focused node and active widget stack,
	// publishing events and copying changes from focus manager variables into dijit (top level) variables
	focus.watch("curNode", function(name, oldVal, newVal){
		dijit._curFocus = newVal;
		dijit._prevFocus = oldVal;
		if(newVal){
			topic.publish("focusNode", newVal);	// publish
		}
	});
	focus.watch("activeStack", function(name, oldVal, newVal){
		dijit._activeStack = newVal;
	});

	focus.on("widget-blur", function(widget, by){
		topic.publish("widgetBlur", widget, by);	// publish
	});
	focus.on("widget-focus", function(widget, by){
		topic.publish("widgetFocus", widget, by);	// publish
	});

	lang.mixin(dijit, exports);

	/*===== return exports; =====*/
	return dijit;	// for back compat :-(
});

},
'dijit/layout/_LayoutWidget':function(){
define("dijit/layout/_LayoutWidget", [
	"dojo/_base/lang", // lang.mixin
	"../_Widget",
	"../_Container",
	"../_Contained",
	"../Viewport",
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add domClass.remove
	"dojo/dom-geometry", // domGeometry.marginBox
	"dojo/dom-style" // domStyle.getComputedStyle
], function(lang, _Widget, _Container, _Contained, Viewport,
	declare, domClass, domGeometry, domStyle){

	// module:
	//		dijit/layout/_LayoutWidget


	return declare("dijit.layout._LayoutWidget", [_Widget, _Container, _Contained], {
		// summary:
		//		Base class for a _Container widget which is responsible for laying out its children.
		//		Widgets which mixin this code must define layout() to manage placement and sizing of the children.

		// baseClass: [protected extension] String
		//		This class name is applied to the widget's domNode
		//		and also may be used to generate names for sub nodes,
		//		for example dijitTabContainer-content.
		baseClass: "dijitLayoutContainer",

		// isLayoutContainer: [protected] Boolean
		//		Indicates that this widget is going to call resize() on its
		//		children widgets, setting their size, when they become visible.
		isLayoutContainer: true,

		buildRendering: function(){
			this.inherited(arguments);
			domClass.add(this.domNode, "dijitContainer");
		},

		startup: function(){
			// summary:
			//		Called after all the widgets have been instantiated and their
			//		dom nodes have been inserted somewhere under win.doc.body.
			//
			//		Widgets should override this method to do any initialization
			//		dependent on other widgets existing, and then call
			//		this superclass method to finish things off.
			//
			//		startup() in subclasses shouldn't do anything
			//		size related because the size of the widget hasn't been set yet.

			if(this._started){ return; }

			// Need to call inherited first - so that child widgets get started
			// up correctly
			this.inherited(arguments);

			// If I am a not being controlled by a parent layout widget...
			var parent = this.getParent && this.getParent();
			if(!(parent && parent.isLayoutContainer)){
				// Do recursive sizing and layout of all my descendants
				// (passing in no argument to resize means that it has to glean the size itself)
				this.resize();

				// Since my parent isn't a layout container, and my style *may be* width=height=100%
				// or something similar (either set directly or via a CSS class),
				// monitor when viewport size changes so that I can re-layout.
				this.own(Viewport.on("resize", lang.hitch(this, "resize")));
			}
		},

		resize: function(changeSize, resultSize){
			// summary:
			//		Call this to resize a widget, or after its size has changed.
			// description:
			//		####Change size mode:
			//
			//		When changeSize is specified, changes the marginBox of this widget
			//		and forces it to re-layout its contents accordingly.
			//		changeSize may specify height, width, or both.
			//
			//		If resultSize is specified it indicates the size the widget will
			//		become after changeSize has been applied.
			//
			//		####Notification mode:
			//
			//		When changeSize is null, indicates that the caller has already changed
			//		the size of the widget, or perhaps it changed because the browser
			//		window was resized.  Tells widget to re-layout its contents accordingly.
			//
			//		If resultSize is also specified it indicates the size the widget has
			//		become.
			//
			//		In either mode, this method also:
			//
			//		1. Sets this._borderBox and this._contentBox to the new size of
			//			the widget.  Queries the current domNode size if necessary.
			//		2. Calls layout() to resize contents (and maybe adjust child widgets).
			// changeSize: Object?
			//		Sets the widget to this margin-box size and position.
			//		May include any/all of the following properties:
			//	|	{w: int, h: int, l: int, t: int}
			// resultSize: Object?
			//		The margin-box size of this widget after applying changeSize (if
			//		changeSize is specified).  If caller knows this size and
			//		passes it in, we don't need to query the browser to get the size.
			//	|	{w: int, h: int}

			var node = this.domNode;

			// set margin box size, unless it wasn't specified, in which case use current size
			if(changeSize){
				domGeometry.setMarginBox(node, changeSize);
			}

			// If either height or width wasn't specified by the user, then query node for it.
			// But note that setting the margin box and then immediately querying dimensions may return
			// inaccurate results, so try not to depend on it.
			var mb = resultSize || {};
			lang.mixin(mb, changeSize || {});	// changeSize overrides resultSize
			if( !("h" in mb) || !("w" in mb) ){
				mb = lang.mixin(domGeometry.getMarginBox(node), mb);	// just use domGeometry.marginBox() to fill in missing values
			}

			// Compute and save the size of my border box and content box
			// (w/out calling domGeometry.getContentBox() since that may fail if size was recently set)
			var cs = domStyle.getComputedStyle(node);
			var me = domGeometry.getMarginExtents(node, cs);
			var be = domGeometry.getBorderExtents(node, cs);
			var bb = (this._borderBox = {
				w: mb.w - (me.w + be.w),
				h: mb.h - (me.h + be.h)
			});
			var pe = domGeometry.getPadExtents(node, cs);
			this._contentBox = {
				l: domStyle.toPixelValue(node, cs.paddingLeft),
				t: domStyle.toPixelValue(node, cs.paddingTop),
				w: bb.w - pe.w,
				h: bb.h - pe.h
			};

			// Callback for widget to adjust size of its children
			this.layout();
		},

		layout: function(){
			// summary:
			//		Widgets override this method to size and position their contents/children.
			//		When this is called this._contentBox is guaranteed to be set (see resize()).
			//
			//		This is called after startup(), and also when the widget's size has been
			//		changed.
			// tags:
			//		protected extension
		},

		_setupChild: function(/*dijit/_WidgetBase*/child){
			// summary:
			//		Common setup for initial children and children which are added after startup
			// tags:
			//		protected extension

			var cls = this.baseClass + "-child "
				+ (child.baseClass ? this.baseClass + "-" + child.baseClass : "");
			domClass.add(child.domNode, cls);
		},

		addChild: function(/*dijit/_WidgetBase*/ child, /*Integer?*/ insertIndex){
			// Overrides _Container.addChild() to call _setupChild()
			this.inherited(arguments);
			if(this._started){
				this._setupChild(child);
			}
		},

		removeChild: function(/*dijit/_WidgetBase*/ child){
			// Overrides _Container.removeChild() to remove class added by _setupChild()
			var cls = this.baseClass + "-child"
					+ (child.baseClass ?
						" " + this.baseClass + "-" + child.baseClass : "");
			domClass.remove(child.domNode, cls);

			this.inherited(arguments);
		}
	});
});

},
'dojox/mdnd/AreaManager':function(){
define("dojox/mdnd/AreaManager", ["dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/_base/connect",
	"dojo/_base/window",
	"dojo/_base/array",
	"dojo/_base/sniff",
	"dojo/_base/lang",
	"dojo/query",
	"dojo/topic", // topic.publish()
	"dojo/dom-class",
	"dojo/dom-geometry",
	"dojo/dom-construct",
	"dijit/registry",
	"dijit/_Widget",
	"./Moveable"
],function(dojo, declare, connect, win, array, sniff, lang, query, topic, domClass, geom, domConstruct, registry, _Widget, Moveable){
	var am = declare(
		"dojox.mdnd.AreaManager",
		null,
	{
		// summary:
		//		Drag And Drop manager
	
		// autoRefresh: Boolean
		//		Enable the refresh of registered areas on drag start.
		autoRefresh: true,
	
	
		// areaClass: String
		//		CSS class enabled an area if areaClass is defined
		areaClass: "dojoxDndArea",
	
		// dragHandleClass: String
		//		CSS class enabled a drag handle.
		dragHandleClass: "dojoxDragHandle",
	
		constructor: function(){
			// summary:
			//		Constructor of AreaManager class.
			//		Initialize arrays, connects and subscribes.
	
			//console.log("dojox.mdnd.AreaManager ::: constructor");
			this._areaList = [];
			this.resizeHandler = connect.connect(dojo.global,"onresize", this, function(){
				this._dropMode.updateAreas(this._areaList);
			});
			
			this._oldIndexArea = this._currentIndexArea = this._oldDropIndex = this._currentDropIndex = this._sourceIndexArea = this._sourceDropIndex = -1;
		},
	
		init: function(){
			// summary:
			//		Initialize the manager by calling the registerByClass method
	
			//console.log("dojox.mdnd.AreaManager ::: init");
			this.registerByClass();
		},
	
		registerByNode: function(/*DOMNode*/area, /*Boolean*/notInitAreas){
			// summary:
			//		To register Dnd Area : insert the DndArea using the specific sort of dropMode.
			// area:
			//		a DOM node corresponding to the Dnd Area
			// notInitAreas:
			//		if false or undefined, init the areas.
	
			//console.log("dojox.mdnd.AreaManager ::: registerByNode", area);
			var index = this._getIndexArea(area);
			if(area && index == -1){
				var acceptType = area.getAttribute("accept");
				var accept = (acceptType) ? acceptType.split(/\s*,\s*/) : ["text"];
				var obj = {
					'node': area,
					'items': [],
					'coords': {},
					'margin': null,
					'accept': accept,
					'initItems': false
				};
				array.forEach(this._getChildren(area), function(item){
					this._setMarginArea(obj, item);
					obj.items.push(this._addMoveableItem(item));
				}, this);
				this._areaList = this._dropMode.addArea(this._areaList, obj);
				if(!notInitAreas){
					this._dropMode.updateAreas(this._areaList);
				}
				connect.publish("/dojox/mdnd/manager/register",[area]);
			}
		},
	
		registerByClass: function(){
			// summary:
			//		Register all Dnd Areas identified by the attribute areaClass :
			//		insert Dnd Areas using the specific sort of dropMode.
	
			//console.log("dojox.mdnd.AreaManager ::: registerByClass");
			query('.'+this.areaClass).forEach(function(area){
				this.registerByNode(area, true);
			}, this);
			this._dropMode.updateAreas(this._areaList);
		},
	
		unregister: function(/*DOMNode*/area){
			// summary:
			//		Unregister a D&D Area and its children into the AreaManager.
			// area:
			//		A node corresponding to the D&D Area.
			// returns:
			//		True if the area is found and unregistered.
	
			//console.log("dojox.mdnd.AreaManager ::: unregister");
			var index = this._getIndexArea(area);
			if(index != -1){
				array.forEach(this._areaList[index].items, function(item){
					this._deleteMoveableItem(item);
				}, this);
				this._areaList.splice(index,1);
				// refresh target area
				this._dropMode.updateAreas(this._areaList);
				return true; // Boolean
			}
			return false; // Boolean
		},
	
		_addMoveableItem: function(/*DOMNode*/node){
			// summary:
			//		Create a draggable item with a DOM node.
			// node:
			//		A child of the D&D Area.
			// returns:
			//		The draggable item.
			// tags:
			//		protected
	
			//console.log("dojox.mdnd.AreaManager ::: _addMoveableItem");
			node.setAttribute("tabIndex", "0");
			var handle = this._searchDragHandle(node);
			var moveable = new Moveable({ 'handle': handle, 'skip': true }, node);
			// add a css style :
			domClass.add(handle || node, "dragHandle");
			var type = node.getAttribute("dndType");
			var item = {
				'item': moveable,
				'type': type ? type.split(/\s*,\s*/) : ["text"],
				'handlers': [connect.connect(moveable, "onDragStart", this, "onDragStart")]
			}
			// connect to the uninitialize method of dijit._Widget to delete a moveable before a destruct
			if(registry && registry.byNode){
				var widget = registry.byNode(node);
				if(widget){
					item.type = widget.dndType ? widget.dndType.split(/\s*,\s*/) : ["text"];
					item.handlers.push(
						connect.connect(widget, "uninitialize", this, function(){
							this.removeDragItem(node.parentNode, moveable.node);
						})
					);
				}
			}
			return item; // Object
		},
	
		_deleteMoveableItem: function(/*Object*/ objItem){
			// summary:
			//		Delete the Moveable object associated with a node.
			// item:
			//		A moveable Object.
			// tags:
			//		protected
	
			//console.log("dojox.mdnd.AreaManager ::: _deleteMoveableItem", objItem);
			// disconnect the handle
			array.forEach(objItem.handlers, function(handler){
				connect.disconnect(handler);
			});
			// delete css style :
			var node = objItem.item.node,
				handle = this._searchDragHandle(node);
			domClass.remove(handle || node, "dragHandle");
			// call destroy of Moveable class
			objItem.item.destroy();
		},
	
		_getIndexArea: function(/*DOMNode*/area){
			// summary:
			//		Get the index of an area.
			// area:
			//		A moveable Object.
			// returns:
			//		area index or -1
			// tags:
			//		protected
	
			//console.log("dojox.mdnd.AreaManager ::: _getIndexArea");
			if(area){
				for(var i = 0; i < this._areaList.length; i++){
					if(this._areaList[i].node === area){
						return i;	// Integer
					}
				}
			}
			return -1;	// Integer
		},
	
		_searchDragHandle: function(/*DOMNode*/node){
			// summary:
			//		Return the node which contains the first specific CSS class handle.
			// node:
			//		A child of the D&D Area.
			// returns:
			//		The drag handle node.
			// tags:
			//		protected
	
			//console.log("dojox.mdnd.AreaManager ::: _searchDragHandle");
			if(node){
				var cssArray = this.dragHandleClass.split(' '),
					length = cssArray.length,
					queryCss = "";
				array.forEach(cssArray, function(css, i){
					queryCss += "." + css;
					if(i != length - 1){
						queryCss += ", ";
					}
				});
				return query(queryCss, node)[0]; // DomNode
			}
		},
	
		addDragItem: function(/*DOMNode*/area, /*DOMNode*/node, /*Integer*/index, /*Boolean*/notCheckParent){
			// summary:
			//		To add an item programmatically.
			// area:
			//		a node corresponding to the D&D Area
			// node:
			//		the node which has to be treated.
			// index:
			//		the place in the area
			// noCheckParent:
			//		if true, doesn't check if node has a parent.
			// returns:
			//		True if the node has been inserted else false.
	
			//console.log("dojox.mdnd.AreaManager ::: addDragItem");
			var add = true;
			if(!notCheckParent){
				add = area && node && (node.parentNode === null || (node.parentNode && node.parentNode.nodeType !== 1));
			}
			if(add){
				var indexArea = this._getIndexArea(area);
				if(indexArea !== -1){
					var item = this._addMoveableItem(node),
						items = this._areaList[indexArea].items;
					if(0 <= index && index < items.length){
						var firstListChild = items.slice(0, index),
							lastListChild = items.slice(index, items.length);
						firstListChild[firstListChild.length] = item;
						this._areaList[indexArea].items = firstListChild.concat(lastListChild);
						area.insertBefore(node, items[index].item.node);
					}
					else{
						this._areaList[indexArea].items.push(item);
						area.appendChild(node);
					}
					this._setMarginArea(this._areaList[indexArea], node);
					this._areaList[indexArea].initItems = false;
					return true;	// Boolean
				}
			}
			return false;	// Boolean
		},
	
		removeDragItem: function(/*DOMNode*/area, /*DOMNode*/node){
			// summary:
			//		Delete a moveable item programmatically. The node is removed from the area.
			// area:
			//		A node corresponding to the DndArea.
			// node:
			//		The node which has to be treated.
			// returns:
			//		the removed node
	
			//console.log("dojox.mdnd.AreaManager ::: removeDragItem");
			var index = this._getIndexArea(area);
			if(area && index !== -1){
				var items = this._areaList[index].items;
				for(var j = 0; j < items.length; j++){
					if(items[j].item.node === node){
						this._deleteMoveableItem(items[j]);
						// delete item of the array
						items.splice(j, 1);
						return area.removeChild(node); // Object
					}
				}
			}
			return null;
		},
	
		_getChildren: function(/*DOMNode*/area){
			// summary:
			//		Get the children of a D&D area.
			// area:
			//		A DnD area.
			// returns:
			//		The children of a DnD area
			// tags:
			//		protected
	
			//console.log("dojox.mdnd.AreaManager ::: _getChildren");
			var children = [];
			array.forEach(area.childNodes, function(child){
				// delete \n
				if(child.nodeType == 1){
					if(registry && registry.byNode){
						var widget = registry.byNode(child);
						if(widget){
							if(!widget.dragRestriction){
								children.push(child);
							}
						}
						else{
							children.push(child);
						}
					}
					else{
						children.push(child);
					}
				}
			});
			return children;	//Array
		},
	
		_setMarginArea: function(/*Object*/area,/*DOMNode*/node){
			// summary:
			//		Set the value of margin in the data type of areaManager
			//		only when the margin has never been computed.
			// area:
			//		The object of a D&D Area.
			// node:
			//		The node which contains margins
			// tags:
			//		protected
	
			//console.log("dojox.mdnd.AreaManager ::: _setMarginArea");
			if(area && area.margin === null && node){
				area.margin = geom.getMarginExtents(node);
			}
		},
	
		findCurrentIndexArea: function(/*Object*/coords, /*Object*/size){
			// summary:
			//		find the nearest target area according to coordinates.
			//		Coordinates are representing by an object : for example, {'x':10,'y':10}
			// coords:
			//		an object encapsulating X and Y position
			// size:
			//		an object encapsulating the area size
			// returns:
			//		an index of area
	
			//console.log("dojox.mdnd.AreaManager ::: findCurrentIndexArea");
			this._oldIndexArea = this._currentIndexArea;
			this._currentIndexArea = this._dropMode.getTargetArea(this._areaList, coords, this._currentIndexArea);
			if(this._currentIndexArea != this._oldIndexArea){
				if(this._oldIndexArea != -1){
					this.onDragExit(coords, size);
				}
				if(this._currentIndexArea != -1){
					this.onDragEnter(coords, size);
				}
			}
			return this._currentIndexArea;	//Integer
		},
	
		_isAccepted: function(/*Array*/ type, /*Array*/ accept){
			// summary:
			//		True if user can drop widget on this node.
			// type:
			//		Array containing item type
			// accept:
			//		Array containing types
			this._accept = false;
			for(var i = 0; i < accept.length; ++i){
				for(var j = 0; j < type.length;++j){
					if(type[j] == accept[i]){
						this._accept = true;
						break;
					}
				}
			}
		},
	
		onDragStart: function(/*DOMNode*/node, /*Object*/coords, /*Object*/size){
			// summary:
			//		Initialize the drag (see dojox.mdnd.Moveable.initOffsetDrag())
			// node:
			//		The node which is about to be dragged
			// coords:
			//		an object encapsulating X and Y position
			// size:
			//		an object encapsulating width and height values
			// tags:
			//		callback
	
			//console.log("dojox.mdnd.AreaManager ::: onDragStart");
			if(this.autoRefresh){
				this._dropMode.updateAreas(this._areaList);
			}
	
			// Create the cover :
			var _html = (sniff("webkit")) ? dojo.body() : dojo.body().parentNode;
			if(!this._cover){
				this._cover = domConstruct.create('div', {
					'class': "dndCover"
				});
				this._cover2 = lang.clone(this._cover);
				domClass.add(this._cover2, "dndCover2");
			}
			var h = _html.scrollHeight+"px";
			this._cover.style.height = this._cover2.style.height = h;
			dojo.body().appendChild(this._cover);
			dojo.body().appendChild(this._cover2);
	
			this._dragStartHandler = connect.connect(node.ownerDocument, "ondragstart", dojo, "stopEvent");
			// to know the source
			this._sourceIndexArea = this._lastValidIndexArea = this._currentIndexArea = this._getIndexArea(node.parentNode);
			// delete the dragItem into the source area
			var sourceArea = this._areaList[this._sourceIndexArea];
			var children = sourceArea.items;
			for(var i = 0; i < children.length; i++){
				if(children[i].item.node == node){
					this._dragItem = children[i];
					this._dragItem.handlers.push(connect.connect(this._dragItem.item, "onDrag", this, "onDrag"));
					this._dragItem.handlers.push(connect.connect(this._dragItem.item, "onDragEnd", this, "onDrop"));
					children.splice(i,1);
					this._currentDropIndex = this._sourceDropIndex = i;
					break;
				}
			}
			var nodeRef = null;
			if(this._sourceDropIndex !== sourceArea.items.length){
				nodeRef = sourceArea.items[this._sourceDropIndex].item.node;
			}
			// IE7 OPTIMIZATION
			if(sniff("ie")> 7){
				// connect these events on the cover
				this._eventsIE7 = [
					connect.connect(this._cover, "onmouseover", dojo, "stopEvent"),
					connect.connect(this._cover, "onmouseout", dojo, "stopEvent"),
					connect.connect(this._cover, "onmouseenter", dojo, "stopEvent"),
					connect.connect(this._cover, "onmouseleave", dojo, "stopEvent")
				];
			}
	
			var s = node.style;
			s.left = coords.x+"px";
			s.top = coords.y+"px";
			// attach the node to the cover
			if(s.position == "relative" || s.position == ""){
				s.position = "absolute"; // enforcing the absolute mode
			}
			this._cover.appendChild(node);
	
			this._dropIndicator.place(sourceArea.node, nodeRef, size);
			// add a style to place the _dragNode in foreground
			domClass.add(node, "dragNode");
			// A dragged node is always draggable in this source area.
			this._accept = true;
			connect.publish("/dojox/mdnd/drag/start",[node, sourceArea, this._sourceDropIndex]);
		},
	
		onDragEnter: function(/*Object*/coords, /*Object*/size){
			// summary:
			//		Optionally called by the getTargetArea method of TargetFinder class.
			// coords:
			//		coordinates of the dragged Node.
			// size:
			//		size of the dragged Node.
			// tags:
			//		callback
	
			//console.log("dojox.mdnd.AreaManager ::: onDragEnter", coords, size);
			if(this._currentIndexArea === this._sourceIndexArea){
				this._accept = true;
			}
			else{
				this._isAccepted(this._dragItem.type, this._areaList[this._currentIndexArea].accept);
			}
		},
	
		onDragExit: function(/*Object*/coords, /*Object*/size){
			// summary:
			//		Optionally called by the getTargetArea method of TargetFinder class.
			// coords:
			//		coordinates of the dragged Node.
			// size:
			//		size of the dragged Node.
			// tags:
			//		callback
	
			//console.log("dojox.mdnd.AreaManager ::: onDragExit");
			this._accept = false;
		},
	
		onDrag: function(/*DOMNode*/node, /*Object*/coords, /*Object*/size, /*Object*/mousePosition){
			// summary:
			//		Occurs when the dojo.dnd.Moveable.onDrag is fired.
			//		Search the nearest target area and called the placeDropIndicator
			// node:
			//		The node which is dragged
			// coords:
			//		an object encapsulating X and Y position
			// size:
			//		an object encapsulating width and height values
			// mousePosition:
			//		coordinates of mouse
			// tags:
			//		callback
	
			//console.log("dojox.mdnd.AreaManager ::: onDrag", node, ",", coords,size);
			var coordinates = this._dropMode.getDragPoint(coords, size, mousePosition);
			this.findCurrentIndexArea(coordinates, size);
			if(this._currentIndexArea !== -1 && this._accept){
				this.placeDropIndicator(coordinates, size);
			}
		},
	
		placeDropIndicator: function(/*Object*/coords, /*Object*/size){
			// summary:
			//		Search the right place to insert the dropIndicator and display the dropIndicator.
			// coords:
			//		an object encapsulating X and Y position
			// size:
			//		an object encapsulating width and height values
			// returns:
			//		the current drop index
	
			//console.log("dojox.mdnd.AreaManager ::: placeDropIndicator");
			//keep old drop Index
			this._oldDropIndex = this._currentDropIndex;
			// calculate all children marker (see VerticalDropMode.initItems())
			var area = this._areaList[this._currentIndexArea];
			if(!area.initItems){
				this._dropMode.initItems(area);
			}
			//get the index where the drop has to be placed.
			this._currentDropIndex = this._dropMode.getDropIndex(area, coords);
			if(!(this._currentIndexArea === this._oldIndexArea && this._oldDropIndex === this._currentDropIndex)){
				this._placeDropIndicator(size);
			}
			return this._currentDropIndex;	//Integer
		},
	
		_placeDropIndicator: function(/*Object*/size){
			// summary:
			//		place the dropIndicator
			// size:
			//		an object encapsulating width and height values
			// tags:
			//		protected
	
			var oldArea = this._areaList[this._lastValidIndexArea];
			var currentArea = this._areaList[this._currentIndexArea];
			//refresh the previous area after moving out the drop indicator
			this._dropMode.refreshItems(oldArea, this._oldDropIndex, size, false);
			// place dropIndicator
			var node = null;
			if(this._currentDropIndex != -1){
				node = currentArea.items[this._currentDropIndex].item.node;
			}
			this._dropIndicator.place(currentArea.node, node);
			this._lastValidIndexArea = this._currentIndexArea;
			//refresh the current area after placing the drop indicator
			this._dropMode.refreshItems(currentArea, this._currentDropIndex, size, true);
		},
	
		onDropCancel: function(){
			// summary:
			//		Cancel the drop.
			//		The dragNode returns into the source.
			// tags:
			//		callback
	
			//console.log("dojox.mdnd.AreaManager ::: onDropCancel");
			if(!this._accept){
				var index = this._getIndexArea(this._dropIndicator.node.parentNode);
				if(index != -1){
					this._currentIndexArea = index;
				}
				else{
					// case if the dropIndicator is in the area which has been unregistered during the drag.
					// chose by default the first area.
					this._currentIndexArea = 0;
				}
			}
		},
	
		onDrop: function(/*DOMNode*/node){
			// summary:
			//		Drop the dragged item where the dropIndicator is displayed.
			// node:
			//		The node which is about to be dropped
			// tags:
			//		callback
	
			//console.log("dojox.mdnd.AreaManager ::: onDrop");
			//dropCancel
			this.onDropCancel();
			var targetArea = this._areaList[this._currentIndexArea];
			domClass.remove(node, "dragNode");
			var style = node.style;
			style.position = "relative";
			style.left = "0";
			style.top = "0";
			style.width = "auto";
			if(targetArea.node == this._dropIndicator.node.parentNode){
				targetArea.node.insertBefore(node, this._dropIndicator.node);
			}
			else{
				// case if the dropIndicator is in the area which has been unregistered during the drag.
				targetArea.node.appendChild(node);
				this._currentDropIndex = targetArea.items.length;
			}
			// add child into the new target area.
			var indexChild = this._currentDropIndex;
			if(indexChild == -1){
				indexChild = targetArea.items.length;
			}
			var children = targetArea.items;
			var firstListArea = children.slice(0, indexChild);
			var lastListArea = children.slice(indexChild, children.length);
			firstListArea[firstListArea.length] = this._dragItem;
			targetArea.items = firstListArea.concat(lastListArea);
	
			this._setMarginArea(targetArea, node);
			array.forEach(this._areaList, function(obj){
				obj.initItems = false;
			});
			// disconnect onDrop handler
			connect.disconnect(this._dragItem.handlers.pop());
			connect.disconnect(this._dragItem.handlers.pop());
			this._resetAfterDrop();
			// remove the cover
			if(this._cover){
				dojo.body().removeChild(this._cover);
				dojo.body().removeChild(this._cover2);
			}
			connect.publish("/dojox/mdnd/drop",[node, targetArea, indexChild]);
		},
	
		_resetAfterDrop: function(){
			// summary:
			//		reset manager properties after dropping an item
			// tags:
			//		protected
	
			this._accept = false;
			this._dragItem = null;
			this._currentDropIndex = -1;
			this._currentIndexArea = -1;
			this._oldDropIndex = -1;
			this._sourceIndexArea = -1;
			this._sourceDropIndex = -1;
			this._dropIndicator.remove();
			if(this._dragStartHandler){
				connect.disconnect(this._dragStartHandler);
			}
			if(sniff("ie") > 7){
				array.forEach(this._eventsIE7, connect.disconnect);
			}
		},
	
		destroy: function(){
			// summary:
			//		Destroy the component.
	
			//console.log("dojox.mdnd.AreaManager ::: destroy");
			//see implementation of unregister()
			while(this._areaList.length > 0){
				if(!this.unregister(this._areaList[0].node)){
					throw new Error("Error while destroying AreaManager");
				}
			}
			connect.disconnect(this.resizeHandler);
			this._dropIndicator.destroy();
			this._dropMode.destroy();
			if(dojox.mdnd.autoScroll){
				dojox.mdnd.autoScroll.destroy();
			}
			if(this.refreshListener){
				connect.unsubscribe(this.refreshListener);
			}
			// destroy the cover
			if(this._cover){
				domConstruct.destroy(this._cover);
				domConstruct.destroy(this._cover2);
				delete this._cover;
				delete this._cover2;
			}
		}
	});
	
	if(_Widget){
		//	Add a new property to widget
		lang.extend(_Widget, {
			// dndType: String
			//		Defines a type of widget.
			dndType : "text"
		});
	}
	
	dojox.mdnd._areaManager = null;
	dojox.mdnd.areaManager = function(){
		// summary:
		//		Returns the current areaManager, creates one if it is not created yet.
		if(!dojox.mdnd._areaManager){
			dojox.mdnd._areaManager = new dojox.mdnd.AreaManager();
		}
		return dojox.mdnd._areaManager;	// Object
	};
	return am;
});
},
'dojox/mdnd/Moveable':function(){
define("dojox/mdnd/Moveable", [
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/connect",
	"dojo/_base/event",
	"dojo/_base/sniff",
	"dojo/dom",
	"dojo/dom-geometry",
	"dojo/dom-style",
	"dojo/_base/window"
],function(dojo, declare, array, connect, event, sniff, dom, geom, domStyle){
	return declare(
		"dojox.mdnd.Moveable",
		null,
	{
		// summary:
		//		Allow end-users to track a DOM node into the web page
	
		// handle: DOMNode
		//		The node on which the user clicks to drag the main node.
		handle: null,
		
		// skip: Boolean
		//		A flag to control a drag action if a form element has been focused.
		//		If true, the drag action is not executed.
		skip: true,
	
		// dragDistance: Integer
		//		The user clicks on the handle, but the drag action will really begin
		//		if he tracks the main node to more than 3 pixels.
		dragDistance: 3,
		
		constructor: function(/*Object*/params, /*DOMNode*/node){
			// summary:
			//		Configure parameters and listen to mousedown events from handle
			//		node.
			// params:
			//		Hash of parameters
			// node:
			//		The draggable node
	
			//console.log("dojox.mdnd.Moveable ::: constructor");
			this.node = dom.byId(node);
			
			this.d = this.node.ownerDocument;
			
			if(!params){ params = {}; }
			this.handle = params.handle ? dom.byId(params.handle) : null;
			if(!this.handle){ this.handle = this.node; }
			this.skip = params.skip;
			this.events = [
				connect.connect(this.handle, "onmousedown", this, "onMouseDown")
			];
			if(dojox.mdnd.autoScroll){
				this.autoScroll = dojox.mdnd.autoScroll;
			}
			
		},
		
		isFormElement: function(/*DOMEvent*/ e){
			// summary:
			//		identify the type of target node associated with a DOM event.
			// e:
			//		a DOM event
			// returns:
			//		if true, the target is one of those specific nodes.
	
			//console.log("dojox.mdnd.Moveable ::: isFormElement");
			var t = e.target;
			if(t.nodeType == 3 /*TEXT_NODE*/){
				t = t.parentNode;
			}
			return " a button textarea input select option ".indexOf(" " + t.tagName.toLowerCase() + " ") >= 0;	// Boolean
		},
		
		onMouseDown: function(/*DOMEvent*/e){
			// summary:
			//		Occurs when the user clicks on the handle node.
			//		Skip the drag action if a specific node is targeted.
			//		Listens to mouseup and mousemove events on to the HTML document.
			// e:
			//		a DOM event
			// tags:
			//		callback
	
			//console.log("dojox.mdnd.Moveable ::: onMouseDown");
			if(this._isDragging){ return;}
			var isLeftButton = (e.which || e.button) == 1;
			if(!isLeftButton){
				return;
			}
			if(this.skip && this.isFormElement(e)){ return; }
			if(this.autoScroll){
				this.autoScroll.setAutoScrollNode(this.node);
				this.autoScroll.setAutoScrollMaxPage();
			}
			this.events.push(connect.connect(this.d, "onmouseup", this, "onMouseUp"));
			this.events.push(connect.connect(this.d, "onmousemove", this, "onFirstMove"));
			this._selectStart = connect.connect(dojo.body(), "onselectstart", event.stop);
			this._firstX = e.clientX;
			this._firstY = e.clientY;
			event.stop(e);
		},
		
		onFirstMove: function(/*DOMEvent*/e){
			// summary:
			//		Occurs when the user moves the mouse after clicking on the
			//		handle.
			//		Determinate when the drag action will have to begin (see
			//		dragDistance).
			// e:
			//		A DOM event
			// tags:
			//		callback
	
			//console.log("dojox.mdnd.Moveable ::: onFirstMove");
			event.stop(e);
			var d = (this._firstX - e.clientX) * (this._firstX - e.clientX)
					+ (this._firstY - e.clientY) * (this._firstY - e.clientY);
			if(d > this.dragDistance * this.dragDistance){
				this._isDragging = true;
				connect.disconnect(this.events.pop());
				domStyle.set(this.node, "width", geom.getContentBox(this.node).w + "px");
				this.initOffsetDrag(e);
				this.events.push(connect.connect(this.d, "onmousemove", this, "onMove"));
			}
		},
		
		initOffsetDrag: function(/*DOMEvent*/e){
			// summary:
			//		Initialize the gap between main node coordinates and the clicked point.
			//		Call the onDragStart method.
			// e:
			//		A DOM event
	
			//console.log("dojox.mdnd.Moveable ::: initOffsetDrag");
			this.offsetDrag = { 'l': e.pageX, 't': e.pageY };
			var s = this.node.style;
			var position = geom.position(this.node, true);
			/*if(s.position == "relative" || s.position == ""){
				s.position = "absolute"; // enforcing the absolute mode
			}*/
			this.offsetDrag.l = position.x - this.offsetDrag.l;
			this.offsetDrag.t = position.y - this.offsetDrag.t;
			var coords = {
				'x': position.x,
				'y': position.y
			};
			this.size = {
				'w': position.w,
				'h': position.h
			};
			// method to catch
			this.onDragStart(this.node, coords, this.size);
		},
		
		onMove: function(/*DOMEvent*/e){
			// summary:
			//		Occurs when the user moves the mouse.
			//		Calls the onDrag method.
			// e:
			//		a DOM event
			// tags:
			//		callback
	
			//console.log("dojox.mdnd.Moveable ::: onMove");
			event.stop(e);
			// hack to avoid too many calls to onMove in IE8 causing sometimes slowness
			if(sniff("ie") == 8 && new Date() - this.date < 20){
				return;
			}
			if(this.autoScroll){
				this.autoScroll.checkAutoScroll(e);
			}
			var coords = {
				'x': this.offsetDrag.l + e.pageX,
				'y': this.offsetDrag.t + e.pageY
			};
			var s = this.node.style;
			s.left = coords.x + "px";
			s.top = coords.y + "px";
			
			// method to catch
			this.onDrag(this.node, coords, this.size, {'x':e.pageX, 'y':e.pageY});
			if(sniff("ie") == 8){
				this.date = new Date();
			}
		},
		
		onMouseUp: function(/*DOMEvent*/e){
			// summary:
			//		Occurs when the user releases the mouse
			//		Calls the onDragEnd method.
			// e:
			//		a DOM event
	
			if (this._isDragging){
				event.stop(e);
				this._isDragging = false;
				if(this.autoScroll){
					this.autoScroll.stopAutoScroll();
				}
				delete this.onMove;
				this.onDragEnd(this.node);
				this.node.focus();
			}
			connect.disconnect(this.events.pop());
			connect.disconnect(this.events.pop());
		},
		
		onDragStart: function(/*DOMNode*/node, /*Object*/coords, /*Object*/size){
			// summary:
			//		Stub function.
			//		Notes : border box model
			// node:
			//		a DOM node
			// coords:
			//		absolute position of the main node
			// size:
			//		an object encapsulating width an height values
			// tags:
			//		callback
	
		},
		
		onDragEnd: function(/*DOMNode*/node){
			// summary:
			//		Stub function
			//		Notes : Coordinates don't contain margins
			// node:
			//		a DOM node
			// tags:
			//		callback
	
		},
		
		onDrag: function(/*DOMNode*/node, /*Object*/coords, /*Object*/size, /*Object*/mousePosition){
			// summary:
			//		Stub function.
			//		Notes : border box model for size value, margin box model for coordinates
			// node:
			//		a DOM node
			// coords:
			//		position of the main node (equals to css left/top properties)
			// size:
			//		an object encapsulating width and height values
			// mousePosition:
			//		coordiantes of mouse
			// tags:
			//		callback
	
		},
	
		destroy: function(){
			// summary:
			//		Delecte associated events
	
			// console.log("dojox.mdnd.Moveable ::: destroy");
			array.forEach(this.events, connect.disconnect);
			this.events = this.node = null;
		}
	});
});

},
'dojox/mdnd/DropIndicator':function(){
define("dojox/mdnd/DropIndicator", ["dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/dom-class",
	"dojo/dom-construct",
	"./AreaManager"
], function(dojo, declare, domClass, domConstruct){
	var di = declare(
		"dojox.mdnd.DropIndicator",
		null,
	{
		// summary:
		//		DropIndicator managment for DnD.
	
		// node: DOMNode
		//		the drop indicator node
		node : null,
			
		constructor: function(){
			//console.log("dojox.mdnd.DropIndicator ::: constructor");
			var dropIndicator = document.createElement("div");
			var subDropIndicator = document.createElement("div");
			dropIndicator.appendChild(subDropIndicator);
			domClass.add(dropIndicator, "dropIndicator");
			this.node = dropIndicator;
		},
		
		place: function(/*Node*/area, /*Node*/nodeRef, /*Object*/size){
			// summary:
			//		Place the DropIndicator in the right place
			// area:
			//		the dnd targer area node
			// nodeRef:
			//		node where the dropIndicator have to be placed into the area
			// dragNode:
			//		the node which is dragged
			// returns:
			//		the node inserted or null if it crashes
	
			//console.log("dojox.mdnd.DropIndicator ::: place");
			if(size){
				this.node.style.height = size.h + "px";
			}
			try{
				if(nodeRef){
					area.insertBefore(this.node, nodeRef);
				}
				else{
					// empty target area or last node => appendChild
					area.appendChild(this.node);
				}
				return this.node;	// DOMNode
			}catch(e){
				return null;
			}
		},
		
		remove: function(){
			// summary:
			//		remove the DropIndicator (not destroy)
	
			//console.log("dojox.mdnd.DropIndicator ::: remove");
			if(this.node){
				//FIX : IE6 problem
				this.node.style.height = "";
				if(this.node.parentNode){
					this.node.parentNode.removeChild(this.node);
				}
			}
		},
		 
		destroy: function(){
			// summary:
			//		destroy the dropIndicator
	
			//console.log("dojox.mdnd.DropIndicator ::: destroy");
			if(this.node){
				if(this.node.parentNode){
					this.node.parentNode.removeChild(this.node);
				}
				domConstruct.destroy(this.node);
				delete this.node;
			}
		}
	});

	dojox.mdnd.areaManager()._dropIndicator = new dojox.mdnd.DropIndicator();
	
	return di;
});

},
'dojox/mdnd/dropMode/OverDropMode':function(){
define("dojox/mdnd/dropMode/OverDropMode", ["dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/_base/connect",
	"dojo/_base/array",
	"dojo/dom-geometry",
	"dojox/mdnd/AreaManager"
],function(dojo, declare, connect, array, geom){
	var odm = declare(
		"dojox.mdnd.dropMode.OverDropMode",
		null,
	{
		// summary:
		//		Default class to find the nearest target only if the mouse is over an area.
	
		// _oldXPoint: Integer
		//		used to save a X position
		_oldXPoint: null,
	
		// _oldYPoint: Integer
		//		used to save a Y position
		_oldYPoint: null,
	
		// _oldBehaviour: Integer
		//		see getDragpoint()
		_oldBehaviour: "up",
	
		constructor: function(){
			//console.log("dojox.mdnd.dropMode.OverDropMode ::: constructor");
			this._dragHandler = [
				connect.connect(dojox.mdnd.areaManager(), "onDragEnter", function(coords, size){
					var m = dojox.mdnd.areaManager();
					if(m._oldIndexArea == -1){
						m._oldIndexArea = m._lastValidIndexArea;
					}
				})
			];
	
		},
	
		addArea: function(/*Array*/areas, /*Object*/object){
			// summary:
			//		Add a D&D Area into an array sorting by the x position.
			// areas:
			//		array of areas
			// object:
			//		data type of a DndArea
			// returns:
			//		a sorted area
	
			//console.log("dojox.mdnd.dropMode.OverDropMode ::: addArea");
			var length = areas.length,
				position = geom.position(object.node, true);
			object.coords = {'x':position.x, 'y':position.y};
			if(length == 0){
				areas.push(object);
			}
			else{
				var x = object.coords.x;
				for(var i = 0; i < length; i++){
					if(x < areas[i].coords.x){
						for(var j = length-1; j >= i; j--)
							areas[j + 1] = areas[j];
						areas[i] = object;
						break;
					}
				}
				if(i == length){
					areas.push(object);
				}
			}
			return areas;	// Array
		},
	
		updateAreas: function(/*Array*/areaList){
			// summary:
			//		refresh areas position and size to determinate the nearest area to drop an item
			// description:
			//		the area position (and size) is equal to the postion of the domNode associated.
			// areaList:
			//		array of areas
	
			//console.log("dojox.mdnd.dropMode.OverDropMode ::: updateAreas");
			var length = areaList.length;
			for(var i = 0; i < length; i++){
				this._updateArea(areaList[i]);
			}
		},
	
		_updateArea : function(/*Object*/area){
			// summary:
			//		update the D&D area object (i.e. update coordinates of its DOM node)
			// area:
			//		the D&D area.
			// tags:
			//		protected
	
			//console.log("dojox.mdnd.dropMode.OverDropMode ::: addArea");
			var position = geom.position(area.node, true);
			area.coords.x = position.x;
			area.coords.x2 = position.x + position.w;
			area.coords.y = position.y;
		},
	
		initItems: function(/*Object*/area){
			// summary:
			//		initialize the horizontal line in order to determinate the drop zone.
			// area:
			//		the D&D area.
	
			//console.log("dojox.mdnd.dropMode.OverDropMode ::: initItems");
			array.forEach(area.items, function(obj){
				//get the vertical middle of the item
				var node = obj.item.node;
				var position = geom.position(node, true);
				var y = position.y + position.h/2;
				obj.y = y;
			});
			area.initItems = true;
		},
	
		refreshItems: function(/*Object*/area, /*Integer*/indexItem, /*Object*/size, /*Boolean*/added){
			// summary:
			//		take into account the drop indicator DOM element in order to compute horizontal lines
			// area:
			//		a D&D area object
			// indexItem:
			//		index of a draggable item
			// size:
			//		dropIndicator size
			// added:
			//		boolean to know if a dropIndicator has been added or deleted
	
			//console.log("dojox.mdnd.dropMode.OverDropMode ::: refreshItems", area, indexItem, size, added);
			if(indexItem == -1){
				return;
			}
			else if(area && size && size.h){
				var height = size.h;
				if(area.margin){
					height += area.margin.t;
				}
				var length = area.items.length;
				for(var i = indexItem; i < length; i++){
					var item = area.items[i];
					if(added){
						item.y += height;
					}
					else{
						item.y -= height;
					}
				}
			}
		},
	
		getDragPoint: function(/*Object*/coords, /*Object*/size, /*Object*/mousePosition){
			// summary:
			//		return coordinates of the draggable item.
			//
			//		- For X point : the x position of mouse
			//		- For Y point : the y position of mouse
			// returns:
			//		an object of coordinates
			//		examples:{'x':10,'y':10}
			// coords:
			//		an object encapsulating X and Y position
			// size:
			//		an object encapsulating width and height values
			// mousePosition:
			//		coordinates of mouse
	
			//console.log("dojox.mdnd.OverDropMode ::: getDragPoint");
			return {			// Object
				'x': mousePosition.x,
				'y': mousePosition.y
				}
		},
	
	
		getTargetArea: function(/*Array*/areaList, /*Object*/ coords, /*integer*/currentIndexArea ){
			// summary:
			//		get the nearest D&D area.
			// areaList:
			//		a list of D&D areas objects
			// coords:
			//		coordinates [x,y] of the dragItem (see getDragPoint())
			// currentIndexArea:
			//		an index representing the active D&D area
			// returns:
			//		the index of the D&D area
	
			//console.log("dojox.mdnd.dropMode.OverDropMode ::: getTargetArea");
			var index = 0;
			var x = coords.x;
			var y = coords.y;
			var end = areaList.length;
			var start = 0, direction = "right", compute = false;
			if(currentIndexArea == -1 || arguments.length < 3){
				// first time : Need to search the nearest area in all areas.
				compute = true;
			}
			else{
				// check if it's always the same area
				if(this._checkInterval(areaList, currentIndexArea, x, y)){
					index = currentIndexArea;
				}
				else{
					if(this._oldXPoint < x){
						start = currentIndexArea + 1;
					}
					else{
						start = currentIndexArea - 1;
						end = 0;
						direction = "left";
					}
					compute = true;
				}
			}
			if(compute){
				if(direction === "right"){
					for(var i = start; i < end; i++){
						if(this._checkInterval(areaList, i, x, y)){
							index = i;
							break;
						}
					}
					if(i == end){
						index = -1;
					}
				}
				else{
					for(var i = start; i >= end; i--){
						if(this._checkInterval(areaList, i, x, y)){
							index = i;
							break;
						}
					}
					if(i == end-1){
						index = -1;
					}
				}
			}
			this._oldXPoint = x;
			return index; // Integer
		},
	
		_checkInterval: function(/*Array*/areaList, /*Integer*/index, /*Coord*/x, /*Coord*/y){
			// summary:
			//		check if the dragNode is in the interval.
			// returns:
			//		true if the dragNode is in intervall
			// areaList:
			//		a list of D&D areas objects
			// index:
			//		index of a D&D area (to get the interval)
			// x:
			//		coordinate x, of the dragNode (see getDragPoint())
			// tags:
			//		protected
	
			//console.log("dojox.mdnd.dropMode.OverDropMode ::: _checkInterval");
			var area = areaList[index];
			var node = area.node;
			var coords = area.coords;
			var startX = coords.x;
			var endX = coords.x2;
			var startY = coords.y;
			var endY = startY + node.offsetHeight;
			if(startX <= x && x <= endX && startY <= y && y <= endY){
				return true;
			}
			return false; // Boolean
		},
	
		getDropIndex: function(/*Object*/ targetArea, /*Object*/ coords){
			// summary:
			//		Return the index where the drop has to be placed.
			// targetArea:
			//		a D&D area object.
			// coords:
			//		coordinates [x,y] of the draggable item.
			// returns:
			//		a number or -1 if the area has no children or the drop index represents the last position in to the area
	
			//console.log("dojox.mdnd.dropMode.OverDropMode ::: getDropIndex");
			var length = targetArea.items.length;
			var coordinates = targetArea.coords;
			var y = coords.y;
			if(length > 0){
				// course all children in the target area.
				for(var i = 0; i < length; i++){
					// compare y value with y value of children
					if(y < targetArea.items[i].y){
						return i;	// integer
					}
					else{
						if(i == length-1){
							return -1; // integer
						}
					}
				}
			}
			return -1;	//integer
		},
	
		destroy: function(){
			array.forEach(this._dragHandler, connect.disconnect);
		}
	});
	
	dojox.mdnd.areaManager()._dropMode = new dojox.mdnd.dropMode.OverDropMode();
	return odm;
});

},
'dojox/mdnd/AutoScroll':function(){
define("dojox/mdnd/AutoScroll", ["dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/connect",
	"dojo/_base/sniff",
	"dojo/ready",
	"dojo/_base/window"
],function(dojo, declare, lang, connect, sniff, ready){
	var as = declare(
		"dojox.mdnd.AutoScroll",
		null,
	{
		// summary:
		//		Activate scrolling while dragging a widget.
	
		// interval: Integer
		//		default mouse move offset
		interval: 3,
	
		// recursiveTimer: Integer
		recursiveTimer: 10,
	
		// marginMouse: Integer
		//		Default mouse margin
		marginMouse: 50,
	
		constructor: function(){
			//console.log("dojox.mdnd.AutoScroll ::: constructor ");
			this.resizeHandler = connect.connect(dojo.global,"onresize", this, function(){
				this.getViewport();
			});
			ready(lang.hitch(this, "init"));
		},
	
		init: function(){
			//console.log("dojox.mdnd.AutoScroll ::: init ");
			this._html = (sniff("webkit"))? dojo.body() : dojo.body().parentNode;
			this.getViewport();
		},
	
		getViewport:function(){
			// summary:
			//		Set the visible part of the window. Varies accordion to Navigator.
	
			//console.log("dojox.mdnd.AutoScroll ::: getViewport ");
			var d = dojo.doc, dd = d.documentElement, w = window, b = dojo.body();
			if(dojo.isMozilla){
				this._v = { 'w': dd.clientWidth, 'h': w.innerHeight };	// Object
			}
			else if(!dojo.isOpera && w.innerWidth){
				this._v = { 'w': w.innerWidth, 'h': w.innerHeight };		// Object
			}
			else if(!dojo.isOpera && dd && dd.clientWidth){
				this._v = { 'w': dd.clientWidth, 'h': dd.clientHeight };	// Object
			}
			else if(b.clientWidth){
				this._v = { 'w': b.clientWidth, 'h': b.clientHeight };	// Object
			}
		},
	
		setAutoScrollNode: function(/*Node*/node){
			// summary:
			//		set the node which is dragged
			// node:
			//		node to scroll
	
			//console.log("dojox.mdnd.AutoScroll ::: setAutoScrollNode ");
			this._node = node;
		},
	
		setAutoScrollMaxPage: function(){
			// summary:
			//		Set the hightest heigh and width authorized scroll.
	
			//console.log("dojox.mdnd.AutoScroll ::: setAutoScrollMaxPage ");
			this._yMax = this._html.scrollHeight;
			this._xMax = this._html.scrollWidth;
		},
	
		checkAutoScroll: function(/*Event*/e){
			// summary:
			//		Check if an autoScroll have to be launched.
	
			//console.log("dojox.mdnd.AutoScroll ::: checkAutoScroll");
			if(this._autoScrollActive){
				this.stopAutoScroll();
			}
			this._y = e.pageY;
			this._x = e.pageX;
			if(e.clientX < this.marginMouse){
				this._autoScrollActive = true;
				this._autoScrollLeft(e);
			}
			else if(e.clientX > this._v.w - this.marginMouse){
				this._autoScrollActive = true;
				this._autoScrollRight(e);
			}
			if(e.clientY < this.marginMouse){
				this._autoScrollActive = true;
				this._autoScrollUp(e);
				
			}
			else if(e.clientY > this._v.h - this.marginMouse){
				this._autoScrollActive = true;
				this._autoScrollDown();
			}
		},
	
		_autoScrollDown: function(){
			// summary:
			//		Manage the down autoscroll.
			// tags:
			//		protected
	
			//console.log("dojox.mdnd.AutoScroll ::: _autoScrollDown ");
			if(this._timer){
				clearTimeout(this._timer);
			}
			if(this._autoScrollActive && this._y + this.marginMouse < this._yMax){
				this._html.scrollTop += this.interval;
				this._node.style.top = (parseInt(this._node.style.top) + this.interval) + "px";
				this._y += this.interval;
				this._timer = setTimeout(lang.hitch(this, "_autoScrollDown"), this.recursiveTimer);
			}
		},
	
		_autoScrollUp: function(){
			// summary:
			//		Manage the up autoscroll.
			// tags:
			//		protected
	
			//console.log("dojox.mdnd.AutoScroll ::: _autoScrollUp ");
			if(this._timer){
				clearTimeout(this._timer);
			}
			if(this._autoScrollActive && this._y - this.marginMouse > 0){
				this._html.scrollTop -= this.interval;
				this._node.style.top = (parseInt(this._node.style.top) - this.interval) + "px";
				this._y -= this.interval;
				this._timer = setTimeout(lang.hitch(this, "_autoScrollUp"),this.recursiveTimer);
			}
		},
	
		_autoScrollRight: function(){
			// summary:
			//		Manage the right autoscroll.
			// tags:
			//		protected
	
			//console.log("dojox.mdnd.AutoScroll ::: _autoScrollRight ");
			if(this._timer){
				clearTimeout(this._timer);
			}
			if(this._autoScrollActive && this._x + this.marginMouse < this._xMax){
				this._html.scrollLeft += this.interval;
				this._node.style.left = (parseInt(this._node.style.left) + this.interval) + "px";
				this._x += this.interval;
				this._timer = setTimeout(lang.hitch(this, "_autoScrollRight"), this.recursiveTimer);
			}
		},
	
		_autoScrollLeft: function(/*Event*/e){
			// summary:
			//		Manage the left autoscroll.
			// tags:
			//		protected
	
			//console.log("dojox.mdnd.AutoScroll ::: _autoScrollLeft ");
			if(this._timer){
				clearTimeout(this._timer);
			}
			if(this._autoScrollActive && this._x - this.marginMouse > 0){
				this._html.scrollLeft -= this.interval;
				this._node.style.left = (parseInt(this._node.style.left) - this.interval) + "px";
				this._x -= this.interval;
				this._timer = setTimeout(lang.hitch(this, "_autoScrollLeft"),this.recursiveTimer);
			}
		},
	
		stopAutoScroll: function(){
			// summary:
			//		Stop the autoscroll.
			
			//console.log("dojox.mdnd.AutoScroll ::: stopAutoScroll ");
			if(this._timer){
				clearTimeout(this._timer);
			}
			this._autoScrollActive = false;
		},
	
		destroy: function(){
			//console.log("dojox.mdnd.AutoScroll ::: destroy ");
			connect.disconnect(this.resizeHandler);
		}
	});
	
	dojox.mdnd.autoScroll = null;
	
	dojox.mdnd.autoScroll = new dojox.mdnd.AutoScroll();
	return as;
});

},
'epi/shell/widget/ToolbarDropDownButton':function(){
﻿define("epi/shell/widget/ToolbarDropDownButton", [
    "dojo",
    "dojo/dom-style",
    "dijit/form/DropDownButton",
    "dijit/Menu"
],
function (dojo, domStyle, DropDownButton, Menu) {
    return dojo.declare([DropDownButton], {
        // tags:
        //      internal

        constructor: function () {
            // Create the sub menu which encapsulates the actual menu items
            this.dropDown = new Menu();
        },

        buildRendering: function () {
            this.inherited(arguments);
            this._setVisibility();
        },

        addChild: function (/*dijit/_Widget*/widget, /*int?*/insertIndex) {
            // summary:
            //  Adds a widget to the encapsulated menu

            if (!widget.separator) {
                widget.set("separator", "dijit/MenuSeparator");
            }

            var value = this.dropDown.addChild(widget, insertIndex);
            this._setVisibility();
            return value;
        },

        getChildren: function () {
            // summary:
            //  return the children of this button.

            return this.dropDown.getChildren();
        },

        removeChild: function (/*Widget|int*/widget) {
            // summary:
            //		Removes the passed widget instance from this widget but does
            //		not destroy it.  You can also pass in an integer indicating
            //		the index within the container to remove

            var value = this.dropDown.removeChild(widget);
            this._setVisibility();
            return value;
        },

        _setVisibility: function () {
            if (this.hasChildren()) {
                domStyle.set(this.domNode, "display", "");
            } else {
                domStyle.set(this.domNode, "display", "none");
            }
        }
    });
});

},
'dijit/form/DropDownButton':function(){
require({cache:{
'url:dijit/form/templates/DropDownButton.html':"<span class=\"dijit dijitReset dijitInline\"\n\t><span class='dijitReset dijitInline dijitButtonNode'\n\t\tdata-dojo-attach-event=\"ondijitclick:_onClick\" data-dojo-attach-point=\"_buttonNode\"\n\t\t><span class=\"dijitReset dijitStretch dijitButtonContents\"\n\t\t\tdata-dojo-attach-point=\"focusNode,titleNode,_arrowWrapperNode,_popupStateNode\"\n\t\t\trole=\"button\" aria-haspopup=\"true\" aria-labelledby=\"${id}_label\"\n\t\t\t><span class=\"dijitReset dijitInline dijitIcon\"\n\t\t\t\tdata-dojo-attach-point=\"iconNode\"\n\t\t\t></span\n\t\t\t><span class=\"dijitReset dijitInline dijitButtonText\"\n\t\t\t\tdata-dojo-attach-point=\"containerNode\"\n\t\t\t\tid=\"${id}_label\"\n\t\t\t></span\n\t\t\t><span class=\"dijitReset dijitInline dijitArrowButtonInner\"></span\n\t\t\t><span class=\"dijitReset dijitInline dijitArrowButtonChar\">&#9660;</span\n\t\t></span\n\t></span\n\t><input ${!nameAttrSetting} type=\"${type}\" value=\"${value}\" class=\"dijitOffScreen\" tabIndex=\"-1\"\n\t\tdata-dojo-attach-point=\"valueNode\" role=\"presentation\"\n/></span>\n"}});
define("dijit/form/DropDownButton", [
	"dojo/_base/declare", // declare
	"dojo/_base/lang",	// hitch
	"dojo/query", // query
	"../registry",	// registry.byNode
	"../popup",		// dijit.popup2.hide
	"./Button",
	"../_Container",
	"../_HasDropDown",
	"dojo/text!./templates/DropDownButton.html"
], function(declare, lang, query, registry, popup, Button, _Container, _HasDropDown, template){

// module:
//		dijit/form/DropDownButton


return declare("dijit.form.DropDownButton", [Button, _Container, _HasDropDown], {
	// summary:
	//		A button with a drop down
	//
	// example:
	// |	<button data-dojo-type="dijit/form/DropDownButton">
	// |		Hello world
	// |		<div data-dojo-type="dijit/Menu">...</div>
	// |	</button>
	//
	// example:
	// |	var button1 = new DropDownButton({ label: "hi", dropDown: new dijit.Menu(...) });
	// |	win.body().appendChild(button1);
	//

	baseClass : "dijitDropDownButton",

	templateString: template,

	_fillContent: function(){
		// Overrides Button._fillContent().
		//
		// My inner HTML contains both the button contents and a drop down widget, like
		// <DropDownButton>  <span>push me</span>  <Menu> ... </Menu> </DropDownButton>
		// The first node is assumed to be the button content. The widget is the popup.

		if(this.srcNodeRef){ // programatically created buttons might not define srcNodeRef
			//FIXME: figure out how to filter out the widget and use all remaining nodes as button
			//	content, not just nodes[0]
			var nodes = query("*", this.srcNodeRef);
			this.inherited(arguments, [nodes[0]]);

			// save pointer to srcNode so we can grab the drop down widget after it's instantiated
			this.dropDownContainer = this.srcNodeRef;
		}
	},

	startup: function(){
		if(this._started){ return; }

		// the child widget from srcNodeRef is the dropdown widget.  Insert it in the page DOM,
		// make it invisible, and store a reference to pass to the popup code.
		if(!this.dropDown && this.dropDownContainer){
			var dropDownNode = query("[widgetId]", this.dropDownContainer)[0];
			this.dropDown = registry.byNode(dropDownNode);
			delete this.dropDownContainer;
		}
		if(this.dropDown){
			popup.hide(this.dropDown);
		}

		this.inherited(arguments);
	},

	isLoaded: function(){
		// Returns whether or not we are loaded - if our dropdown has an href,
		// then we want to check that.
		var dropDown = this.dropDown;
		return (!!dropDown && (!dropDown.href || dropDown.isLoaded));
	},

	loadDropDown: function(/*Function*/ callback){
		// Default implementation assumes that drop down already exists,
		// but hasn't loaded it's data (ex: ContentPane w/href).
		// App must override if the drop down is lazy-created.
		var dropDown = this.dropDown;
		var handler = dropDown.on("load", lang.hitch(this, function(){
			handler.remove();
			callback();
		}));
		dropDown.refresh();		// tell it to load
	},

	isFocusable: function(){
		// Overridden so that focus is handled by the _HasDropDown mixin, not by
		// the _FormWidget mixin.
		return this.inherited(arguments) && !this._mouseDown;
	}
});

});

},
'dijit/form/Button':function(){
require({cache:{
'url:dijit/form/templates/Button.html':"<span class=\"dijit dijitReset dijitInline\" role=\"presentation\"\n\t><span class=\"dijitReset dijitInline dijitButtonNode\"\n\t\tdata-dojo-attach-event=\"ondijitclick:_onClick\" role=\"presentation\"\n\t\t><span class=\"dijitReset dijitStretch dijitButtonContents\"\n\t\t\tdata-dojo-attach-point=\"titleNode,focusNode\"\n\t\t\trole=\"button\" aria-labelledby=\"${id}_label\"\n\t\t\t><span class=\"dijitReset dijitInline dijitIcon\" data-dojo-attach-point=\"iconNode\"></span\n\t\t\t><span class=\"dijitReset dijitToggleButtonIconChar\">&#x25CF;</span\n\t\t\t><span class=\"dijitReset dijitInline dijitButtonText\"\n\t\t\t\tid=\"${id}_label\"\n\t\t\t\tdata-dojo-attach-point=\"containerNode\"\n\t\t\t></span\n\t\t></span\n\t></span\n\t><input ${!nameAttrSetting} type=\"${type}\" value=\"${value}\" class=\"dijitOffScreen\"\n\t\ttabIndex=\"-1\" role=\"presentation\" data-dojo-attach-point=\"valueNode\"\n/></span>\n"}});
define("dijit/form/Button", [
	"require",
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.toggle
	"dojo/has",			// has("dijit-legacy-requires")
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.trim
	"dojo/ready",
	"./_FormWidget",
	"./_ButtonMixin",
	"dojo/text!./templates/Button.html"
], function(require, declare, domClass, has, kernel, lang, ready, _FormWidget, _ButtonMixin, template){

// module:
//		dijit/form/Button

// Back compat w/1.6, remove for 2.0
if(has("dijit-legacy-requires")){
	ready(0, function(){
		var requires = ["dijit/form/DropDownButton", "dijit/form/ComboButton", "dijit/form/ToggleButton"];
		require(requires);	// use indirection so modules not rolled into a build
	});
}

return declare("dijit.form.Button", [_FormWidget, _ButtonMixin], {
	// summary:
	//		Basically the same thing as a normal HTML button, but with special styling.
	// description:
	//		Buttons can display a label, an icon, or both.
	//		A label should always be specified (through innerHTML) or the label
	//		attribute.  It can be hidden via showLabel=false.
	// example:
	// |	<button data-dojo-type="dijit/form/Button" onClick="...">Hello world</button>
	//
	// example:
	// |	var button1 = new Button({label: "hello world", onClick: foo});
	// |	dojo.body().appendChild(button1.domNode);

	// showLabel: Boolean
	//		Set this to true to hide the label text and display only the icon.
	//		(If showLabel=false then iconClass must be specified.)
	//		Especially useful for toolbars.
	//		If showLabel=true, the label will become the title (a.k.a. tooltip/hint) of the icon.
	//
	//		The exception case is for computers in high-contrast mode, where the label
	//		will still be displayed, since the icon doesn't appear.
	showLabel: true,

	// iconClass: String
	//		Class to apply to DOMNode in button to make it display an icon
	iconClass: "dijitNoIcon",
	_setIconClassAttr: { node: "iconNode", type: "class" },

	baseClass: "dijitButton",

	templateString: template,

	// Map widget attributes to DOMNode attributes.
	_setValueAttr: "valueNode",

	_onClick: function(/*Event*/ e){
		// summary:
		//		Internal function to handle click actions
		var ok = this.inherited(arguments);
		if(ok){
			if(this.valueNode){
				this.valueNode.click();
				e.preventDefault(); // cancel BUTTON click and continue with hidden INPUT click
                e.stopPropagation();    // avoid two events bubbling from Button widget
				// leave ok = true so that subclasses can do what they need to do
			}
		}
		return ok;
	},

	_fillContent: function(/*DomNode*/ source){
		// Overrides _Templated._fillContent().
		// If button label is specified as srcNodeRef.innerHTML rather than
		// this.params.label, handle it here.
		// TODO: remove the method in 2.0, parser will do it all for me
		if(source && (!this.params || !("label" in this.params))){
			var sourceLabel = lang.trim(source.innerHTML);
			if(sourceLabel){
				this.label = sourceLabel; // _applyAttributes will be called after buildRendering completes to update the DOM
			}
		}
	},

	_setShowLabelAttr: function(val){
		if(this.containerNode){
			domClass.toggle(this.containerNode, "dijitDisplayNone", !val);
		}
		this._set("showLabel", val);
	},

	setLabel: function(/*String*/ content){
		// summary:
		//		Deprecated.  Use set('label', ...) instead.
		kernel.deprecated("dijit.form.Button.setLabel() is deprecated.  Use set('label', ...) instead.", "", "2.0");
		this.set("label", content);
	},

	_setLabelAttr: function(/*String*/ content){
		// summary:
		//		Hook for set('label', ...) to work.
		// description:
		//		Set the label (text) of the button; takes an HTML string.
		//		If the label is hidden (showLabel=false) then and no title has
		//		been specified, then label is also set as title attribute of icon.
		this.inherited(arguments);
		if(!this.showLabel && !("title" in this.params)){
			this.titleNode.title = lang.trim(this.containerNode.innerText || this.containerNode.textContent || '');
		}
	}
});


});


},
'dijit/form/_FormWidget':function(){
define("dijit/form/_FormWidget", [
	"dojo/_base/declare",	// declare
	"dojo/has",				// has("dijit-legacy-requires")
	"dojo/_base/kernel",	// kernel.deprecated
	"dojo/ready",
	"../_Widget",
	"../_CssStateMixin",
	"../_TemplatedMixin",
	"./_FormWidgetMixin"
], function(declare, has, kernel, ready, _Widget, _CssStateMixin, _TemplatedMixin, _FormWidgetMixin){


// module:
//		dijit/form/_FormWidget

// Back compat w/1.6, remove for 2.0
if(has("dijit-legacy-requires")){
	ready(0, function(){
		var requires = ["dijit/form/_FormValueWidget"];
		require(requires);	// use indirection so modules not rolled into a build
	});
}

return declare("dijit.form._FormWidget", [_Widget, _TemplatedMixin, _CssStateMixin, _FormWidgetMixin], {
	// summary:
	//		Base class for widgets corresponding to native HTML elements such as `<checkbox>` or `<button>`,
	//		which can be children of a `<form>` node or a `dijit/form/Form` widget.
	//
	// description:
	//		Represents a single HTML element.
	//		All these widgets should have these attributes just like native HTML input elements.
	//		You can set them during widget construction or afterwards, via `dijit/_WidgetBase.set()`.
	//
	//		They also share some common methods.

	setDisabled: function(/*Boolean*/ disabled){
		// summary:
		//		Deprecated.  Use set('disabled', ...) instead.
		kernel.deprecated("setDisabled("+disabled+") is deprecated. Use set('disabled',"+disabled+") instead.", "", "2.0");
		this.set('disabled', disabled);
	},

	setValue: function(/*String*/ value){
		// summary:
		//		Deprecated.  Use set('value', ...) instead.
		kernel.deprecated("dijit.form._FormWidget:setValue("+value+") is deprecated.  Use set('value',"+value+") instead.", "", "2.0");
		this.set('value', value);
	},

	getValue: function(){
		// summary:
		//		Deprecated.  Use get('value') instead.
		kernel.deprecated(this.declaredClass+"::getValue() is deprecated. Use get('value') instead.", "", "2.0");
		return this.get('value');
	},

	postMixInProperties: function(){
		// Setup name=foo string to be referenced from the template (but only if a name has been specified)
		// Unfortunately we can't use _setNameAttr to set the name due to IE limitations, see #8484, #8660.
		// Regarding escaping, see heading "Attribute values" in
		// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
		this.nameAttrSetting = this.name ? ('name="' + this.name.replace(/"/g, "&quot;") + '"') : '';
		this.inherited(arguments);
	},

	// Override automatic assigning type --> focusNode, it causes exception on IE.
	// Instead, type must be specified as ${type} in the template, as part of the original DOM
	_setTypeAttr: null
});

});

},
'dijit/form/_FormWidgetMixin':function(){
define("dijit/form/_FormWidgetMixin", [
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/dom-style", // domStyle.get
	"dojo/_base/lang", // lang.hitch lang.isArray
	"dojo/mouse", // mouse.isLeft
	"dojo/sniff", // has("webkit")
	"dojo/window", // winUtils.scrollIntoView
	"../a11y"	// a11y.hasDefaultTabStop
], function(array, declare, domAttr, domStyle, lang, mouse, has, winUtils, a11y){

// module:
//		dijit/form/_FormWidgetMixin

return declare("dijit.form._FormWidgetMixin", null, {
	// summary:
	//		Mixin for widgets corresponding to native HTML elements such as `<checkbox>` or `<button>`,
	//		which can be children of a `<form>` node or a `dijit/form/Form` widget.
	//
	// description:
	//		Represents a single HTML element.
	//		All these widgets should have these attributes just like native HTML input elements.
	//		You can set them during widget construction or afterwards, via `dijit/_WidgetBase.set()`.
	//
	//		They also share some common methods.

	// name: [const] String
	//		Name used when submitting form; same as "name" attribute or plain HTML elements
	name: "",

	// alt: String
	//		Corresponds to the native HTML `<input>` element's attribute.
	alt: "",

	// value: String
	//		Corresponds to the native HTML `<input>` element's attribute.
	value: "",

	// type: [const] String
	//		Corresponds to the native HTML `<input>` element's attribute.
	type: "text",

	// type: String
	//		Apply aria-label in markup to the widget's focusNode
	"aria-label": "focusNode",

	// tabIndex: String
	//		Order fields are traversed when user hits the tab key
	tabIndex: "0",
	_setTabIndexAttr: "focusNode",	// force copy even when tabIndex default value, needed since Button is <span>

	// disabled: Boolean
	//		Should this widget respond to user input?
	//		In markup, this is specified as "disabled='disabled'", or just "disabled".
	disabled: false,

	// intermediateChanges: Boolean
	//		Fires onChange for each value change or only on demand
	intermediateChanges: false,

	// scrollOnFocus: Boolean
	//		On focus, should this widget scroll into view?
	scrollOnFocus: true,

	// Override _WidgetBase mapping id to this.domNode, needs to be on focusNode so <label> etc.
	// works with screen reader
	_setIdAttr: "focusNode",

	_setDisabledAttr: function(/*Boolean*/ value){
		this._set("disabled", value);
		domAttr.set(this.focusNode, 'disabled', value);
		if(this.valueNode){
			domAttr.set(this.valueNode, 'disabled', value);
		}
		this.focusNode.setAttribute("aria-disabled", value ? "true" : "false");

		if(value){
			// reset these, because after the domNode is disabled, we can no longer receive
			// mouse related events, see #4200
			this._set("hovering", false);
			this._set("active", false);

			// clear tab stop(s) on this widget's focusable node(s)  (ComboBox has two focusable nodes)
			var attachPointNames = "tabIndex" in this.attributeMap ? this.attributeMap.tabIndex :
				("_setTabIndexAttr" in this) ? this._setTabIndexAttr : "focusNode";
			array.forEach(lang.isArray(attachPointNames) ? attachPointNames : [attachPointNames], function(attachPointName){
				var node = this[attachPointName];
				// complex code because tabIndex=-1 on a <div> doesn't work on FF
				if(has("webkit") || a11y.hasDefaultTabStop(node)){	// see #11064 about webkit bug
					node.setAttribute('tabIndex', "-1");
				}else{
					node.removeAttribute('tabIndex');
				}
			}, this);
		}else{
			if(this.tabIndex != ""){
				this.set('tabIndex', this.tabIndex);
			}
		}
	},

	_onFocus: function(/*String*/ by){
		// If user clicks on the widget, even if the mouse is released outside of it,
		// this widget's focusNode should get focus (to mimic native browser hehavior).
		// Browsers often need help to make sure the focus via mouse actually gets to the focusNode.
		if(by == "mouse" && this.isFocusable()){
			// IE exhibits strange scrolling behavior when refocusing a node so only do it when !focused.
			var focusConnector = this.connect(this.focusNode, "onfocus", function(){
				this.disconnect(mouseUpConnector);
				this.disconnect(focusConnector);
			});
			// Set a global event to handle mouseup, so it fires properly
			// even if the cursor leaves this.domNode before the mouse up event.
			var mouseUpConnector = this.connect(this.ownerDocumentBody, "onmouseup", function(){
				this.disconnect(mouseUpConnector);
				this.disconnect(focusConnector);
				// if here, then the mousedown did not focus the focusNode as the default action
				if(this.focused){
					this.focus();
				}
			});
		}
		if(this.scrollOnFocus){
			this.defer(function(){ winUtils.scrollIntoView(this.domNode); }); // without defer, the input caret position can change on mouse click
		}
		this.inherited(arguments);
	},

	isFocusable: function(){
		// summary:
		//		Tells if this widget is focusable or not.  Used internally by dijit.
		// tags:
		//		protected
		return !this.disabled && this.focusNode && (domStyle.get(this.domNode, "display") != "none");
	},

	focus: function(){
		// summary:
		//		Put focus on this widget
		if(!this.disabled && this.focusNode.focus){
			try{ this.focusNode.focus(); }catch(e){}/*squelch errors from hidden nodes*/
		}
	},

	compare: function(/*anything*/ val1, /*anything*/ val2){
		// summary:
		//		Compare 2 values (as returned by get('value') for this widget).
		// tags:
		//		protected
		if(typeof val1 == "number" && typeof val2 == "number"){
			return (isNaN(val1) && isNaN(val2)) ? 0 : val1 - val2;
		}else if(val1 > val2){
			return 1;
		}else if(val1 < val2){
			return -1;
		}else{
			return 0;
		}
	},

	onChange: function(/*===== newValue =====*/){
		// summary:
		//		Callback when this widget's value is changed.
		// tags:
		//		callback
	},

	// _onChangeActive: [private] Boolean
	//		Indicates that changes to the value should call onChange() callback.
	//		This is false during widget initialization, to avoid calling onChange()
	//		when the initial value is set.
	_onChangeActive: false,

	_handleOnChange: function(/*anything*/ newValue, /*Boolean?*/ priorityChange){
		// summary:
		//		Called when the value of the widget is set.  Calls onChange() if appropriate
		// newValue:
		//		the new value
		// priorityChange:
		//		For a slider, for example, dragging the slider is priorityChange==false,
		//		but on mouse up, it's priorityChange==true.  If intermediateChanges==false,
		//		onChange is only called form priorityChange=true events.
		// tags:
		//		private
		if(this._lastValueReported == undefined && (priorityChange === null || !this._onChangeActive)){
			// this block executes not for a change, but during initialization,
			// and is used to store away the original value (or for ToggleButton, the original checked state)
			this._resetValue = this._lastValueReported = newValue;
		}
		this._pendingOnChange = this._pendingOnChange
			|| (typeof newValue != typeof this._lastValueReported)
			|| (this.compare(newValue, this._lastValueReported) != 0);
		if((this.intermediateChanges || priorityChange || priorityChange === undefined) && this._pendingOnChange){
			this._lastValueReported = newValue;
			this._pendingOnChange = false;
			if(this._onChangeActive){
				if(this._onChangeHandle){
					this._onChangeHandle.remove();
				}
				// defer allows hidden value processing to run and
				// also the onChange handler can safely adjust focus, etc
				this._onChangeHandle = this.defer(
					function(){
						this._onChangeHandle = null;
						this.onChange(newValue);
					}); // try to collapse multiple onChange's fired faster than can be processed
			}
		}
	},

	create: function(){
		// Overrides _Widget.create()
		this.inherited(arguments);
		this._onChangeActive = true;
	},

	destroy: function(){
		if(this._onChangeHandle){ // destroy called before last onChange has fired
			this._onChangeHandle.remove();
			this.onChange(this._lastValueReported);
		}
		this.inherited(arguments);
	}
});

});

},
'dijit/form/_ButtonMixin':function(){
define("dijit/form/_ButtonMixin", [
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.setSelectable
	"dojo/_base/event", // event.stop
	"../registry"		// registry.byNode
], function(declare, dom, event, registry){

// module:
//		dijit/form/_ButtonMixin

return declare("dijit.form._ButtonMixin", null, {
	// summary:
	//		A mixin to add a thin standard API wrapper to a normal HTML button
	// description:
	//		A label should always be specified (through innerHTML) or the label attribute.
	//
	//		Attach points:
	//
	//		- focusNode (required): this node receives focus
	//		- valueNode (optional): this node's value gets submitted with FORM elements
	//		- containerNode (optional): this node gets the innerHTML assignment for label
	// example:
	// |	<button data-dojo-type="dijit/form/Button" onClick="...">Hello world</button>
	// example:
	// |	var button1 = new Button({label: "hello world", onClick: foo});
	// |	dojo.body().appendChild(button1.domNode);

	// label: HTML String
	//		Content to display in button.
	label: "",

	// type: [const] String
	//		Type of button (submit, reset, button, checkbox, radio)
	type: "button",

	_onClick: function(/*Event*/ e){
		// summary:
		//		Internal function to handle click actions
		if(this.disabled){
			event.stop(e);
			return false;
		}
		var preventDefault = this.onClick(e) === false; // user click actions
		if(!preventDefault && this.type == "submit" && !(this.valueNode||this.focusNode).form){ // see if a non-form widget needs to be signalled
			for(var node=this.domNode; node.parentNode; node=node.parentNode){
				var widget=registry.byNode(node);
				if(widget && typeof widget._onSubmit == "function"){
					widget._onSubmit(e);
					preventDefault = true;
					break;
				}
			}
		}
		if(preventDefault){
			e.preventDefault();
		}
		return !preventDefault;
	},

	postCreate: function(){
		this.inherited(arguments);
		dom.setSelectable(this.focusNode, false);
	},

	onClick: function(/*Event*/ /*===== e =====*/){
		// summary:
		//		Callback for when button is clicked.
		//		If type="submit", return true to perform submit, or false to cancel it.
		// type:
		//		callback
		return true;		// Boolean
	},

	_setLabelAttr: function(/*String*/ content){
		// summary:
		//		Hook for set('label', ...) to work.
		// description:
		//		Set the label (text) of the button; takes an HTML string.
		this._set("label", content);
		(this.containerNode||this.focusNode).innerHTML = content;
	}
});

});

},
'url:dijit/form/templates/Button.html':"<span class=\"dijit dijitReset dijitInline\" role=\"presentation\"\n\t><span class=\"dijitReset dijitInline dijitButtonNode\"\n\t\tdata-dojo-attach-event=\"ondijitclick:_onClick\" role=\"presentation\"\n\t\t><span class=\"dijitReset dijitStretch dijitButtonContents\"\n\t\t\tdata-dojo-attach-point=\"titleNode,focusNode\"\n\t\t\trole=\"button\" aria-labelledby=\"${id}_label\"\n\t\t\t><span class=\"dijitReset dijitInline dijitIcon\" data-dojo-attach-point=\"iconNode\"></span\n\t\t\t><span class=\"dijitReset dijitToggleButtonIconChar\">&#x25CF;</span\n\t\t\t><span class=\"dijitReset dijitInline dijitButtonText\"\n\t\t\t\tid=\"${id}_label\"\n\t\t\t\tdata-dojo-attach-point=\"containerNode\"\n\t\t\t></span\n\t\t></span\n\t></span\n\t><input ${!nameAttrSetting} type=\"${type}\" value=\"${value}\" class=\"dijitOffScreen\"\n\t\ttabIndex=\"-1\" role=\"presentation\" data-dojo-attach-point=\"valueNode\"\n/></span>\n",
'dijit/_HasDropDown':function(){
define("dijit/_HasDropDown", [
	"dojo/_base/declare", // declare
	"dojo/_base/Deferred",
	"dojo/_base/event", // event.stop
	"dojo/dom", // dom.isDescendant
	"dojo/dom-attr", // domAttr.set
	"dojo/dom-class", // domClass.add domClass.contains domClass.remove
	"dojo/dom-geometry", // domGeometry.marginBox domGeometry.position
	"dojo/dom-style", // domStyle.set
	"dojo/has",	// has("touch")
	"dojo/keys", // keys.DOWN_ARROW keys.ENTER keys.ESCAPE
	"dojo/_base/lang", // lang.hitch lang.isFunction
	"dojo/on",
	"./registry",	// registry.byNode()
	"./focus",
	"./popup",
	"./_FocusMixin",
	"./Viewport"
], function(declare, Deferred, event,dom, domAttr, domClass, domGeometry, domStyle, has, keys, lang, on,
			registry, focus, popup, _FocusMixin, Viewport){


	// module:
	//		dijit/_HasDropDown

	return declare("dijit._HasDropDown", _FocusMixin, {
		// summary:
		//		Mixin for widgets that need drop down ability.

		// _buttonNode: [protected] DomNode
		//		The button/icon/node to click to display the drop down.
		//		Can be set via a data-dojo-attach-point assignment.
		//		If missing, then either focusNode or domNode (if focusNode is also missing) will be used.
		_buttonNode: null,

		// _arrowWrapperNode: [protected] DomNode
		//		Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending
		//		on where the drop down is set to be positioned.
		//		Can be set via a data-dojo-attach-point assignment.
		//		If missing, then _buttonNode will be used.
		_arrowWrapperNode: null,

		// _popupStateNode: [protected] DomNode
		//		The node to set the aria-expanded class on.
		//		Also sets popupActive class but that will be removed in 2.0.
		//		Can be set via a data-dojo-attach-point assignment.
		//		If missing, then focusNode or _buttonNode (if focusNode is missing) will be used.
		_popupStateNode: null,

		// _aroundNode: [protected] DomNode
		//		The node to display the popup around.
		//		Can be set via a data-dojo-attach-point assignment.
		//		If missing, then domNode will be used.
		_aroundNode: null,

		// dropDown: [protected] Widget
		//		The widget to display as a popup.  This widget *must* be
		//		defined before the startup function is called.
		dropDown: null,

		// autoWidth: [protected] Boolean
		//		Set to true to make the drop down at least as wide as this
		//		widget.  Set to false if the drop down should just be its
		//		default width
		autoWidth: true,

		// forceWidth: [protected] Boolean
		//		Set to true to make the drop down exactly as wide as this
		//		widget.  Overrides autoWidth.
		forceWidth: false,

		// maxHeight: [protected] Integer
		//		The max height for our dropdown.
		//		Any dropdown taller than this will have scrollbars.
		//		Set to 0 for no max height, or -1 to limit height to available space in viewport
		maxHeight: 0,

		// dropDownPosition: [const] String[]
		//		This variable controls the position of the drop down.
		//		It's an array of strings with the following values:
		//
		//		- before: places drop down to the left of the target node/widget, or to the right in
		//		  the case of RTL scripts like Hebrew and Arabic
		//		- after: places drop down to the right of the target node/widget, or to the left in
		//		  the case of RTL scripts like Hebrew and Arabic
		//		- above: drop down goes above target node
		//		- below: drop down goes below target node
		//
		//		The list is positions is tried, in order, until a position is found where the drop down fits
		//		within the viewport.
		//
		dropDownPosition: ["below","above"],

		// _stopClickEvents: Boolean
		//		When set to false, the click events will not be stopped, in
		//		case you want to use them in your subclass
		_stopClickEvents: true,

		_onDropDownMouseDown: function(/*Event*/ e){
			// summary:
			//		Callback when the user mousedown's on the arrow icon
			if(this.disabled || this.readOnly){ return; }

			// Prevent default to stop things like text selection, but don't stop propagation, so that:
			//		1. TimeTextBox etc. can focus the <input> on mousedown
			//		2. dropDownButtonActive class applied by _CssStateMixin (on button depress)
			//		3. user defined onMouseDown handler fires
			//
			// Also, don't call preventDefault() on MSPointerDown event (on IE10) because that prevents the button
			// from getting focus, and then the focus manager doesn't know what's going on (#17262)
			if(e.type != "MSPointerDown" && e.type != "pointerdown"){
				e.preventDefault();
			}

			this._docHandler = this.connect(this.ownerDocument, "mouseup", "_onDropDownMouseUp");

			this.toggleDropDown();
		},

		_onDropDownMouseUp: function(/*Event?*/ e){
			// summary:
			//		Callback when the user lifts their mouse after mouse down on the arrow icon.
			//		If the drop down is a simple menu and the mouse is over the menu, we execute it, otherwise, we focus our
			//		drop down widget.  If the event is missing, then we are not
			//		a mouseup event.
			//
			//		This is useful for the common mouse movement pattern
			//		with native browser `<select>` nodes:
			//
			//		1. mouse down on the select node (probably on the arrow)
			//		2. move mouse to a menu item while holding down the mouse button
			//		3. mouse up.  this selects the menu item as though the user had clicked it.

			if(e && this._docHandler){
				this.disconnect(this._docHandler);
			}
			var dropDown = this.dropDown, overMenu = false;

			if(e && this._opened){
				// This code deals with the corner-case when the drop down covers the original widget,
				// because it's so large.  In that case mouse-up shouldn't select a value from the menu.
				// Find out if our target is somewhere in our dropdown widget,
				// but not over our _buttonNode (the clickable node)
				var c = domGeometry.position(this._buttonNode, true);
				if(!(e.pageX >= c.x && e.pageX <= c.x + c.w) ||
					!(e.pageY >= c.y && e.pageY <= c.y + c.h)){
					var t = e.target;
					while(t && !overMenu){
						if(domClass.contains(t, "dijitPopup")){
							overMenu = true;
						}else{
							t = t.parentNode;
						}
					}
					if(overMenu){
						t = e.target;
						if(dropDown.onItemClick){
							var menuItem;
							while(t && !(menuItem = registry.byNode(t))){
								t = t.parentNode;
							}
							if(menuItem && menuItem.onClick && menuItem.getParent){
								menuItem.getParent().onItemClick(menuItem, e);
							}
						}
						return;
					}
				}
			}
			if(this._opened){
				// Focus the dropdown widget unless it's a menu (in which case autoFocus is set to false).
				// Even if it's a menu, we need to focus it if this is a fake mouse event caused by the user typing
				// SPACE/ENTER while using JAWS.  Jaws converts the SPACE/ENTER key into mousedown/mouseup events.
				// If this.hovering is false then it's presumably actually a keyboard event.
				if(dropDown.focus && (dropDown.autoFocus !== false || (e.type == "mouseup" && !this.hovering))){
					// Do it on a delay so that we don't steal back focus from the dropdown.
					this._focusDropDownTimer = this.defer(function(){
						dropDown.focus();
						delete this._focusDropDownTimer;
					});
				}
			}else{
				// The drop down arrow icon probably can't receive focus, but widget itself should get focus.
				// defer() needed to make it work on IE (test DateTextBox)
				if(this.focus){
					this.defer("focus");
				}
			}

			if(has("touch")){
				this._justGotMouseUp = true;
				this.defer(function(){
					this._justGotMouseUp = false;
				});
			}
		},

		_onDropDownClick: function(/*Event*/ e){
			if(has("touch") && !this._justGotMouseUp){
				// If there was no preceding mousedown/mouseup (like on android), then simulate them to
				// toggle the drop down.
				//
				// The if(has("touch") is necessary since IE and desktop safari get spurious onclick events
				// when there are nested tables (specifically, clicking on a table that holds a dijit/form/Select,
				// but not on the Select itself, causes an onclick event on the Select)
				this._onDropDownMouseDown(e);
				this._onDropDownMouseUp(e);
			}

			// The drop down was already opened on mousedown/keydown; just need to call stopEvent().
			if(this._stopClickEvents){
				event.stop(e);
			}
		},

		buildRendering: function(){
			this.inherited(arguments);

			this._buttonNode = this._buttonNode || this.focusNode || this.domNode;
			this._popupStateNode = this._popupStateNode || this.focusNode || this._buttonNode;

			// Add a class to the "dijitDownArrowButton" type class to _buttonNode so theme can set direction of arrow
			// based on where drop down will normally appear
			var defaultPos = {
					"after" : this.isLeftToRight() ? "Right" : "Left",
					"before" : this.isLeftToRight() ? "Left" : "Right",
					"above" : "Up",
					"below" : "Down",
					"left" : "Left",
					"right" : "Right"
			}[this.dropDownPosition[0]] || this.dropDownPosition[0] || "Down";
			domClass.add(this._arrowWrapperNode || this._buttonNode, "dijit" + defaultPos + "ArrowButton");
		},

		postCreate: function(){
			// summary:
			//		set up nodes and connect our mouse and keyboard events

			this.inherited(arguments);

			var keyboardEventNode = this.focusNode || this.domNode;
			this.own(
				on(this._buttonNode, "mousedown", lang.hitch(this, "_onDropDownMouseDown")),
				on(this._buttonNode, "click", lang.hitch(this, "_onDropDownClick")),
				on(keyboardEventNode, "keydown", lang.hitch(this, "_onKey")),
				on(keyboardEventNode, "keyup", lang.hitch(this, "_onKeyUp"))
			);
		},

		destroy: function(){
			if(this.dropDown){
				// Destroy the drop down, unless it's already been destroyed.  This can happen because
				// the drop down is a direct child of <body> even though it's logically my child.
				if(!this.dropDown._destroyed){
					this.dropDown.destroyRecursive();
				}
				delete this.dropDown;
			}
			this.inherited(arguments);
		},

		_onKey: function(/*Event*/ e){
			// summary:
			//		Callback when the user presses a key while focused on the button node

			if(this.disabled || this.readOnly){ return; }
			var d = this.dropDown, target = e.target;
			if(d && this._opened && d.handleKey){
				if(d.handleKey(e) === false){
					/* false return code means that the drop down handled the key */
					event.stop(e);
					return;
				}
			}
			if(d && this._opened && e.keyCode == keys.ESCAPE){
				this.closeDropDown();
				event.stop(e);
			}else if(!this._opened &&
					(e.keyCode == keys.DOWN_ARROW ||
						( (e.keyCode == keys.ENTER || e.keyCode == keys.SPACE) &&
						  //ignore enter and space if the event is for a text input
						  ((target.tagName || "").toLowerCase() !== 'input' ||
						     (target.type && target.type.toLowerCase() !== 'text'))))){
				// Toggle the drop down, but wait until keyup so that the drop down doesn't
				// get a stray keyup event, or in the case of key-repeat (because user held
				// down key for too long), stray keydown events
				this._toggleOnKeyUp = true;
				event.stop(e);
			}
		},

		_onKeyUp: function(){
			if(this._toggleOnKeyUp){
				delete this._toggleOnKeyUp;
				this.toggleDropDown();
				var d = this.dropDown;	// drop down may not exist until toggleDropDown() call
				if(d && d.focus){
					this.defer(lang.hitch(d, "focus"), 1);
				}
			}
		},

		_onBlur: function(){
			// summary:
			//		Called magically when focus has shifted away from this widget and it's dropdown

			// Don't focus on button if the user has explicitly focused on something else (happens
			// when user clicks another control causing the current popup to close)..
			// But if focus is inside of the drop down then reset focus to me, because IE doesn't like
			// it when you display:none a node with focus.
			var focusMe = focus.curNode && this.dropDown && dom.isDescendant(focus.curNode, this.dropDown.domNode);

			this.closeDropDown(focusMe);

			this.inherited(arguments);
		},

		isLoaded: function(){
			// summary:
			//		Returns true if the dropdown exists and it's data is loaded.  This can
			//		be overridden in order to force a call to loadDropDown().
			// tags:
			//		protected

			return true;
		},

		loadDropDown: function(/*Function*/ loadCallback){
			// summary:
			//		Creates the drop down if it doesn't exist, loads the data
			//		if there's an href and it hasn't been loaded yet, and then calls
			//		the given callback.
			// tags:
			//		protected

			// TODO: for 2.0, change API to return a Deferred, instead of calling loadCallback?
			loadCallback();
		},

		loadAndOpenDropDown: function(){
			// summary:
			//		Creates the drop down if it doesn't exist, loads the data
			//		if there's an href and it hasn't been loaded yet, and
			//		then opens the drop down.  This is basically a callback when the
			//		user presses the down arrow button to open the drop down.
			// returns: Deferred
			//		Deferred for the drop down widget that
			//		fires when drop down is created and loaded
			// tags:
			//		protected
			var d = new Deferred(),
				afterLoad = lang.hitch(this, function(){
					this.openDropDown();
					d.resolve(this.dropDown);
				});
			if(!this.isLoaded()){
				this.loadDropDown(afterLoad);
			}else{
				afterLoad();
			}
			return d;
		},

		toggleDropDown: function(){
			// summary:
			//		Callback when the user presses the down arrow button or presses
			//		the down arrow key to open/close the drop down.
			//		Toggle the drop-down widget; if it is up, close it, if not, open it
			// tags:
			//		protected

			if(this.disabled || this.readOnly){ return; }
			if(!this._opened){
				this.loadAndOpenDropDown();
			}else{
				this.closeDropDown();
			}
		},

		openDropDown: function(){
			// summary:
			//		Opens the dropdown for this widget.   To be called only when this.dropDown
			//		has been created and is ready to display (ie, it's data is loaded).
			// returns:
			//		return value of dijit/popup.open()
			// tags:
			//		protected

			var dropDown = this.dropDown,
				ddNode = dropDown.domNode,
				aroundNode = this._aroundNode || this.domNode,
				self = this;

			// Prepare our popup's height and honor maxHeight if it exists.

			// TODO: isn't maxHeight dependent on the return value from dijit/popup.open(),
			// ie, dependent on how much space is available (BK)

			if(!this._preparedNode){
				this._preparedNode = true;
				// Check if we have explicitly set width and height on the dropdown widget dom node
				if(ddNode.style.width){
					this._explicitDDWidth = true;
				}
				if(ddNode.style.height){
					this._explicitDDHeight = true;
				}
			}

			// Code for resizing dropdown (height limitation, or increasing width to match my width)
			if(this.maxHeight || this.forceWidth || this.autoWidth){
				var myStyle = {
					display: "",
					visibility: "hidden"
				};
				if(!this._explicitDDWidth){
					myStyle.width = "";
				}
				if(!this._explicitDDHeight){
					myStyle.height = "";
				}
				domStyle.set(ddNode, myStyle);

				// Figure out maximum height allowed (if there is a height restriction)
				var maxHeight = this.maxHeight;
				if(maxHeight == -1){
					// limit height to space available in viewport either above or below my domNode
					// (whichever side has more room)
					var viewport = Viewport.getEffectiveBox(this.ownerDocument),
						position = domGeometry.position(aroundNode, false);
					maxHeight = Math.floor(Math.max(position.y, viewport.h - (position.y + position.h)));
				}

				// Attach dropDown to DOM and make make visibility:hidden rather than display:none
				// so we call startup() and also get the size
				popup.moveOffScreen(dropDown);

				if(dropDown.startup && !dropDown._started){
					dropDown.startup(); // this has to be done after being added to the DOM
				}
				// Get size of drop down, and determine if vertical scroll bar needed.  If no scroll bar needed,
				// use overflow:visible rather than overflow:hidden so off-by-one errors don't hide drop down border.
				var mb = domGeometry.getMarginSize(ddNode);
				var overHeight = (maxHeight && mb.h > maxHeight);
				domStyle.set(ddNode, {
					overflow: overHeight ? "auto" : "visible"
				});
				if(overHeight){
					mb.h = maxHeight;
					if("w" in mb){
						mb.w += 16;	// room for vertical scrollbar
					}
				}else{
					delete mb.h;
				}

				// Adjust dropdown width to match or be larger than my width
				if(this.forceWidth){
					mb.w = aroundNode.offsetWidth;
				}else if(this.autoWidth){
					mb.w = Math.max(mb.w, aroundNode.offsetWidth);
				}else{
					delete mb.w;
				}

				// And finally, resize the dropdown to calculated height and width
				if(lang.isFunction(dropDown.resize)){
					dropDown.resize(mb);
				}else{
					domGeometry.setMarginBox(ddNode, mb);
				}
			}

			var retVal = popup.open({
				parent: this,
				popup: dropDown,
				around: aroundNode,
				orient: this.dropDownPosition,
				onExecute: function(){
					self.closeDropDown(true);
				},
				onCancel: function(){
					self.closeDropDown(true);
				},
				onClose: function(){
					domAttr.set(self._popupStateNode, "popupActive", false);
					domClass.remove(self._popupStateNode, "dijitHasDropDownOpen");
					self._set("_opened", false);	// use set() because _CssStateMixin is watching
				}
			});
			domAttr.set(this._popupStateNode, "popupActive", "true");
			domClass.add(this._popupStateNode, "dijitHasDropDownOpen");
			this._set("_opened", true);	// use set() because _CssStateMixin is watching
			
			this._popupStateNode.setAttribute("aria-expanded", "true");
			this._popupStateNode.setAttribute("aria-owns", dropDown.id);

			// Set aria-labelledby on dropdown if it's not already set to something more meaningful
			if(ddNode.getAttribute("role") !== "presentation" && !ddNode.getAttribute("aria-labelledby")){
				ddNode.setAttribute("aria-labelledby", this.id);
			}
			
			return retVal;
		},

		closeDropDown: function(/*Boolean*/ focus){
			// summary:
			//		Closes the drop down on this widget
			// focus:
			//		If true, refocuses the button widget
			// tags:
			//		protected

			if(this._focusDropDownTimer){
				this._focusDropDownTimer.remove();
				delete this._focusDropDownTimer;
			}
			
			if(this._opened){
				this._popupStateNode.setAttribute("aria-expanded", "false");
				if(focus){ this.focus(); }
				popup.close(this.dropDown);
				this._opened = false;
			}
		}

	});
});

},
'url:dijit/form/templates/DropDownButton.html':"<span class=\"dijit dijitReset dijitInline\"\n\t><span class='dijitReset dijitInline dijitButtonNode'\n\t\tdata-dojo-attach-event=\"ondijitclick:_onClick\" data-dojo-attach-point=\"_buttonNode\"\n\t\t><span class=\"dijitReset dijitStretch dijitButtonContents\"\n\t\t\tdata-dojo-attach-point=\"focusNode,titleNode,_arrowWrapperNode,_popupStateNode\"\n\t\t\trole=\"button\" aria-haspopup=\"true\" aria-labelledby=\"${id}_label\"\n\t\t\t><span class=\"dijitReset dijitInline dijitIcon\"\n\t\t\t\tdata-dojo-attach-point=\"iconNode\"\n\t\t\t></span\n\t\t\t><span class=\"dijitReset dijitInline dijitButtonText\"\n\t\t\t\tdata-dojo-attach-point=\"containerNode\"\n\t\t\t\tid=\"${id}_label\"\n\t\t\t></span\n\t\t\t><span class=\"dijitReset dijitInline dijitArrowButtonInner\"></span\n\t\t\t><span class=\"dijitReset dijitInline dijitArrowButtonChar\">&#9660;</span\n\t\t></span\n\t></span\n\t><input ${!nameAttrSetting} type=\"${type}\" value=\"${value}\" class=\"dijitOffScreen\" tabIndex=\"-1\"\n\t\tdata-dojo-attach-point=\"valueNode\" role=\"presentation\"\n/></span>\n",
'dijit/CheckedMenuItem':function(){
require({cache:{
'url:dijit/templates/CheckedMenuItem.html':"<tr class=\"dijitReset dijitMenuItem\" data-dojo-attach-point=\"focusNode\" role=\"menuitemcheckbox\" tabIndex=\"-1\" aria-checked=\"${checked}\">\n\t<td class=\"dijitReset dijitMenuItemIconCell\" role=\"presentation\">\n\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitMenuItemIcon dijitCheckedMenuItemIcon\" data-dojo-attach-point=\"iconNode\"/>\n\t\t<span class=\"dijitCheckedMenuItemIconChar\">&#10003;</span>\n\t</td>\n\t<td class=\"dijitReset dijitMenuItemLabel\" colspan=\"2\" data-dojo-attach-point=\"containerNode,labelNode\"></td>\n\t<td class=\"dijitReset dijitMenuItemAccelKey\" style=\"display: none\" data-dojo-attach-point=\"accelKeyNode\"></td>\n\t<td class=\"dijitReset dijitMenuArrowCell\" role=\"presentation\">&#160;</td>\n</tr>\n"}});
define("dijit/CheckedMenuItem", [
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.toggle
	"./MenuItem",
	"dojo/text!./templates/CheckedMenuItem.html",
	"./hccss"
], function(declare, domClass, MenuItem, template){

	// module:
	//		dijit/CheckedMenuItem

	return declare("dijit.CheckedMenuItem", MenuItem, {
		// summary:
		//		A checkbox-like menu item for toggling on and off

		templateString: template,

		// checked: Boolean
		//		Our checked state
		checked: false,
		_setCheckedAttr: function(/*Boolean*/ checked){
			// summary:
			//		Hook so attr('checked', bool) works.
			//		Sets the class and state for the check box.
			domClass.toggle(this.domNode, "dijitCheckedMenuItemChecked", checked);
			this.domNode.setAttribute("aria-checked", checked ? "true" : "false");
			this._set("checked", checked);
		},

		iconClass: "",	// override dijitNoIcon

		onChange: function(/*Boolean*/ /*===== checked =====*/){
			// summary:
			//		User defined function to handle check/uncheck events
			// tags:
			//		callback
		},

		_onClick: function(evt){
			// summary:
			//		Clicking this item just toggles its state
			// tags:
			//		private
			if(!this.disabled){
				this.set("checked", !this.checked);
				this.onChange(this.checked);
			}
			this.onClick(evt);
		}
	});
});

},
'dijit/MenuItem':function(){
require({cache:{
'url:dijit/templates/MenuItem.html':"<tr class=\"dijitReset dijitMenuItem\" data-dojo-attach-point=\"focusNode\" role=\"menuitem\" tabIndex=\"-1\">\n\t<td class=\"dijitReset dijitMenuItemIconCell\" role=\"presentation\">\n\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitIcon dijitMenuItemIcon\" data-dojo-attach-point=\"iconNode\"/>\n\t</td>\n\t<td class=\"dijitReset dijitMenuItemLabel\" colspan=\"2\" data-dojo-attach-point=\"containerNode\"></td>\n\t<td class=\"dijitReset dijitMenuItemAccelKey\" style=\"display: none\" data-dojo-attach-point=\"accelKeyNode\"></td>\n\t<td class=\"dijitReset dijitMenuArrowCell\" role=\"presentation\">\n\t\t<div data-dojo-attach-point=\"arrowWrapper\" style=\"visibility: hidden\">\n\t\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitMenuExpand\"/>\n\t\t\t<span class=\"dijitMenuExpandA11y\">+</span>\n\t\t</div>\n\t</td>\n</tr>\n"}});
define("dijit/MenuItem", [
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.setSelectable
	"dojo/dom-attr", // domAttr.set
	"dojo/dom-class", // domClass.toggle
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/sniff", // has("ie")
	"./_Widget",
	"./_TemplatedMixin",
	"./_Contained",
	"./_CssStateMixin",
	"dojo/text!./templates/MenuItem.html"
], function(declare, dom, domAttr, domClass, kernel, has,
			_Widget, _TemplatedMixin, _Contained, _CssStateMixin, template){

	// module:
	//		dijit/MenuItem

	return declare("dijit.MenuItem",
		[_Widget, _TemplatedMixin, _Contained, _CssStateMixin],
		{
		// summary:
		//		A line item in a Menu Widget

		// Make 3 columns
		// icon, label, and expand arrow (BiDi-dependent) indicating sub-menu
		templateString: template,

		baseClass: "dijitMenuItem",

		// label: String
		//		Menu text
		label: "",
		_setLabelAttr: function(val){
			this.containerNode.innerHTML = 	val;
			this._set("label", val);
			if(this.textDir === "auto"){
				this.applyTextDir(this.focusNode, this.label);
			}
		},

		// iconClass: String
		//		Class to apply to DOMNode to make it display an icon.
		iconClass: "dijitNoIcon",
		_setIconClassAttr: { node: "iconNode", type: "class" },

		// accelKey: String
		//		Text for the accelerator (shortcut) key combination.
		//		Note that although Menu can display accelerator keys there
		//		is no infrastructure to actually catch and execute these
		//		accelerators.
		accelKey: "",

		// disabled: Boolean
		//		If true, the menu item is disabled.
		//		If false, the menu item is enabled.
		disabled: false,

		_fillContent: function(/*DomNode*/ source){
			// If button label is specified as srcNodeRef.innerHTML rather than
			// this.params.label, handle it here.
			if(source && !("label" in this.params)){
				this.set('label', source.innerHTML);
			}
		},

		buildRendering: function(){
			this.inherited(arguments);
			var label = this.id+"_text";
			domAttr.set(this.containerNode, "id", label);
			if(this.accelKeyNode){
				domAttr.set(this.accelKeyNode, "id", this.id + "_accel");
				label += " " + this.id + "_accel";
			}
			this.domNode.setAttribute("aria-labelledby", label);
			dom.setSelectable(this.domNode, false);
		},

		onClick: function(/*Event*/){
			// summary:
			//		User defined function to handle clicks
			// tags:
			//		callback
		},

		focus: function(){
			// summary:
			//		Focus on this MenuItem
			try{
				if(has("ie") == 8){
					// needed for IE8 which won't scroll TR tags into view on focus yet calling scrollIntoView creates flicker (#10275)
					this.containerNode.focus();
				}
				this.focusNode.focus();
			}catch(e){
				// this throws on IE (at least) in some scenarios
			}
		},

		_onFocus: function(){
			// summary:
			//		This is called by the focus manager when focus
			//		goes to this MenuItem or a child menu.
			// tags:
			//		protected
			this._setSelected(true);
			this.getParent()._onItemFocus(this);

			this.inherited(arguments);
		},

		_setSelected: function(selected){
			// summary:
			//		Indicate that this node is the currently selected one
			// tags:
			//		private

			/***
			 * TODO: remove this method and calls to it, when _onBlur() is working for MenuItem.
			 * Currently _onBlur() gets called when focus is moved from the MenuItem to a child menu.
			 * That's not supposed to happen, but the problem is:
			 * In order to allow dijit.popup's getTopPopup() to work,a sub menu's popupParent
			 * points to the parent Menu, bypassing the parent MenuItem... thus the
			 * MenuItem is not in the chain of active widgets and gets a premature call to
			 * _onBlur()
			 */

			domClass.toggle(this.domNode, "dijitMenuItemSelected", selected);
		},

		setLabel: function(/*String*/ content){
			// summary:
			//		Deprecated.   Use set('label', ...) instead.
			// tags:
			//		deprecated
			kernel.deprecated("dijit.MenuItem.setLabel() is deprecated.  Use set('label', ...) instead.", "", "2.0");
			this.set("label", content);
		},

		setDisabled: function(/*Boolean*/ disabled){
			// summary:
			//		Deprecated.   Use set('disabled', bool) instead.
			// tags:
			//		deprecated
			kernel.deprecated("dijit.Menu.setDisabled() is deprecated.  Use set('disabled', bool) instead.", "", "2.0");
			this.set('disabled', disabled);
		},
		_setDisabledAttr: function(/*Boolean*/ value){
			// summary:
			//		Hook for attr('disabled', ...) to work.
			//		Enable or disable this menu item.

			this.focusNode.setAttribute('aria-disabled', value ? 'true' : 'false');
			this._set("disabled", value);
		},
		_setAccelKeyAttr: function(/*String*/ value){
			// summary:
			//		Hook for attr('accelKey', ...) to work.
			//		Set accelKey on this menu item.

			this.accelKeyNode.style.display=value?"":"none";
			this.accelKeyNode.innerHTML=value;
			//have to use colSpan to make it work in IE
			domAttr.set(this.containerNode,'colSpan',value?"1":"2");

			this._set("accelKey", value);
		},
		_setTextDirAttr: function(/*String*/ textDir){
			// summary:
			//		Setter for textDir.
			// description:
			//		Users shouldn't call this function; they should be calling
			//		set('textDir', value)
			// tags:
			//		private

			// only if new textDir is different from the old one
			// and on widgets creation.
			if(!this._created || this.textDir != textDir){
				this._set("textDir", textDir);
				this.applyTextDir(this.focusNode, this.label);
			}
		}		
	});
});

},
'url:dijit/templates/MenuItem.html':"<tr class=\"dijitReset dijitMenuItem\" data-dojo-attach-point=\"focusNode\" role=\"menuitem\" tabIndex=\"-1\">\n\t<td class=\"dijitReset dijitMenuItemIconCell\" role=\"presentation\">\n\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitIcon dijitMenuItemIcon\" data-dojo-attach-point=\"iconNode\"/>\n\t</td>\n\t<td class=\"dijitReset dijitMenuItemLabel\" colspan=\"2\" data-dojo-attach-point=\"containerNode\"></td>\n\t<td class=\"dijitReset dijitMenuItemAccelKey\" style=\"display: none\" data-dojo-attach-point=\"accelKeyNode\"></td>\n\t<td class=\"dijitReset dijitMenuArrowCell\" role=\"presentation\">\n\t\t<div data-dojo-attach-point=\"arrowWrapper\" style=\"visibility: hidden\">\n\t\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitMenuExpand\"/>\n\t\t\t<span class=\"dijitMenuExpandA11y\">+</span>\n\t\t</div>\n\t</td>\n</tr>\n",
'url:dijit/templates/CheckedMenuItem.html':"<tr class=\"dijitReset dijitMenuItem\" data-dojo-attach-point=\"focusNode\" role=\"menuitemcheckbox\" tabIndex=\"-1\" aria-checked=\"${checked}\">\n\t<td class=\"dijitReset dijitMenuItemIconCell\" role=\"presentation\">\n\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitMenuItemIcon dijitCheckedMenuItemIcon\" data-dojo-attach-point=\"iconNode\"/>\n\t\t<span class=\"dijitCheckedMenuItemIconChar\">&#10003;</span>\n\t</td>\n\t<td class=\"dijitReset dijitMenuItemLabel\" colspan=\"2\" data-dojo-attach-point=\"containerNode,labelNode\"></td>\n\t<td class=\"dijitReset dijitMenuItemAccelKey\" style=\"display: none\" data-dojo-attach-point=\"accelKeyNode\"></td>\n\t<td class=\"dijitReset dijitMenuArrowCell\" role=\"presentation\">&#160;</td>\n</tr>\n",
'dijit/form/ComboButton':function(){
require({cache:{
'url:dijit/form/templates/ComboButton.html':"<table class=\"dijit dijitReset dijitInline dijitLeft\"\n\tcellspacing='0' cellpadding='0' role=\"presentation\"\n\t><tbody role=\"presentation\"><tr role=\"presentation\"\n\t\t><td class=\"dijitReset dijitStretch dijitButtonNode\" data-dojo-attach-point=\"buttonNode\" data-dojo-attach-event=\"ondijitclick:_onClick,onkeypress:_onButtonKeyPress\"\n\t\t><div id=\"${id}_button\" class=\"dijitReset dijitButtonContents\"\n\t\t\tdata-dojo-attach-point=\"titleNode\"\n\t\t\trole=\"button\" aria-labelledby=\"${id}_label\"\n\t\t\t><div class=\"dijitReset dijitInline dijitIcon\" data-dojo-attach-point=\"iconNode\" role=\"presentation\"></div\n\t\t\t><div class=\"dijitReset dijitInline dijitButtonText\" id=\"${id}_label\" data-dojo-attach-point=\"containerNode\" role=\"presentation\"></div\n\t\t></div\n\t\t></td\n\t\t><td id=\"${id}_arrow\" class='dijitReset dijitRight dijitButtonNode dijitArrowButton'\n\t\t\tdata-dojo-attach-point=\"_popupStateNode,focusNode,_buttonNode\"\n\t\t\tdata-dojo-attach-event=\"onkeypress:_onArrowKeyPress\"\n\t\t\ttitle=\"${optionsTitle}\"\n\t\t\trole=\"button\" aria-haspopup=\"true\"\n\t\t\t><div class=\"dijitReset dijitArrowButtonInner\" role=\"presentation\"></div\n\t\t\t><div class=\"dijitReset dijitArrowButtonChar\" role=\"presentation\">&#9660;</div\n\t\t></td\n\t\t><td style=\"display:none !important;\"\n\t\t\t><input ${!nameAttrSetting} type=\"${type}\" value=\"${value}\" data-dojo-attach-point=\"valueNode\" role=\"presentation\"\n\t\t/></td></tr></tbody\n></table>\n"}});
define("dijit/form/ComboButton", [
	"dojo/_base/declare", // declare
	"dojo/_base/event", // event.stop
	"dojo/keys", // keys
	"../focus",		// focus.focus()
	"./DropDownButton",
	"dojo/text!./templates/ComboButton.html"
], function(declare, event, keys, focus, DropDownButton, template){

// module:
//		dijit/form/ComboButton

return declare("dijit.form.ComboButton", DropDownButton, {
	// summary:
	//		A combination button and drop-down button.
	//		Users can click one side to "press" the button, or click an arrow
	//		icon to display the drop down.
	//
	// example:
	// |	<button data-dojo-type="dijit/form/ComboButton" onClick="...">
	// |		<span>Hello world</span>
	// |		<div data-dojo-type="dijit/Menu">...</div>
	// |	</button>
	//
	// example:
	// |	var button1 = new ComboButton({label: "hello world", onClick: foo, dropDown: "myMenu"});
	// |	dojo.body().appendChild(button1.domNode);
	//

	templateString: template,

	// Map widget attributes to DOMNode attributes.
	_setIdAttr: "",	// override _FormWidgetMixin which puts id on the focusNode
	_setTabIndexAttr: ["focusNode", "titleNode"],
	_setTitleAttr: "titleNode",

	// optionsTitle: String
	//		Text that describes the options menu (accessibility)
	optionsTitle: "",

	baseClass: "dijitComboButton",

	// Set classes like dijitButtonContentsHover or dijitArrowButtonActive depending on
	// mouse action over specified node
	cssStateNodes: {
		"buttonNode": "dijitButtonNode",
		"titleNode": "dijitButtonContents",
		"_popupStateNode": "dijitDownArrowButton"
	},

	_focusedNode: null,

	_onButtonKeyPress: function(/*Event*/ evt){
		// summary:
		//		Handler for right arrow key when focus is on left part of button
		if(evt.charOrCode == keys[this.isLeftToRight() ? "RIGHT_ARROW" : "LEFT_ARROW"]){
			focus.focus(this._popupStateNode);
			event.stop(evt);
		}
	},

	_onArrowKeyPress: function(/*Event*/ evt){
		// summary:
		//		Handler for left arrow key when focus is on right part of button
		if(evt.charOrCode == keys[this.isLeftToRight() ? "LEFT_ARROW" : "RIGHT_ARROW"]){
			focus.focus(this.titleNode);
			event.stop(evt);
		}
	},

	focus: function(/*String*/ position){
		// summary:
		//		Focuses this widget to according to position, if specified,
		//		otherwise on arrow node
		// position:
		//		"start" or "end"
		if(!this.disabled){
			focus.focus(position == "start" ? this.titleNode : this._popupStateNode);
		}
	}
});

});

},
'url:dijit/form/templates/ComboButton.html':"<table class=\"dijit dijitReset dijitInline dijitLeft\"\n\tcellspacing='0' cellpadding='0' role=\"presentation\"\n\t><tbody role=\"presentation\"><tr role=\"presentation\"\n\t\t><td class=\"dijitReset dijitStretch dijitButtonNode\" data-dojo-attach-point=\"buttonNode\" data-dojo-attach-event=\"ondijitclick:_onClick,onkeypress:_onButtonKeyPress\"\n\t\t><div id=\"${id}_button\" class=\"dijitReset dijitButtonContents\"\n\t\t\tdata-dojo-attach-point=\"titleNode\"\n\t\t\trole=\"button\" aria-labelledby=\"${id}_label\"\n\t\t\t><div class=\"dijitReset dijitInline dijitIcon\" data-dojo-attach-point=\"iconNode\" role=\"presentation\"></div\n\t\t\t><div class=\"dijitReset dijitInline dijitButtonText\" id=\"${id}_label\" data-dojo-attach-point=\"containerNode\" role=\"presentation\"></div\n\t\t></div\n\t\t></td\n\t\t><td id=\"${id}_arrow\" class='dijitReset dijitRight dijitButtonNode dijitArrowButton'\n\t\t\tdata-dojo-attach-point=\"_popupStateNode,focusNode,_buttonNode\"\n\t\t\tdata-dojo-attach-event=\"onkeypress:_onArrowKeyPress\"\n\t\t\ttitle=\"${optionsTitle}\"\n\t\t\trole=\"button\" aria-haspopup=\"true\"\n\t\t\t><div class=\"dijitReset dijitArrowButtonInner\" role=\"presentation\"></div\n\t\t\t><div class=\"dijitReset dijitArrowButtonChar\" role=\"presentation\">&#9660;</div\n\t\t></td\n\t\t><td style=\"display:none !important;\"\n\t\t\t><input ${!nameAttrSetting} type=\"${type}\" value=\"${value}\" data-dojo-attach-point=\"valueNode\" role=\"presentation\"\n\t\t/></td></tr></tbody\n></table>\n",
'dijit/form/RadioButton':function(){
define("dijit/form/RadioButton", [
	"dojo/_base/declare", // declare
	"./CheckBox",
	"./_RadioButtonMixin"
], function(declare, CheckBox, _RadioButtonMixin){

	// module:
	//		dijit/form/RadioButton

	return declare("dijit.form.RadioButton", [CheckBox, _RadioButtonMixin], {
		// summary:
		//		Same as an HTML radio, but with fancy styling.

		baseClass: "dijitRadio"
	});
});

},
'dijit/form/CheckBox':function(){
require({cache:{
'url:dijit/form/templates/CheckBox.html':"<div class=\"dijit dijitReset dijitInline\" role=\"presentation\"\n\t><input\n\t \t${!nameAttrSetting} type=\"${type}\" role=\"${type}\" aria-checked=\"false\" ${checkedAttrSetting}\n\t\tclass=\"dijitReset dijitCheckBoxInput\"\n\t\tdata-dojo-attach-point=\"focusNode\"\n\t \tdata-dojo-attach-event=\"onclick:_onClick\"\n/></div>\n"}});
define("dijit/form/CheckBox", [
	"require",
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/has",		// has("dijit-legacy-requires")
	"dojo/query", // query
	"dojo/ready",
	"./ToggleButton",
	"./_CheckBoxMixin",
	"dojo/text!./templates/CheckBox.html",
	"dojo/NodeList-dom" // NodeList.addClass/removeClass
], function(require, declare, domAttr, has, query, ready, ToggleButton, _CheckBoxMixin, template){

	// module:
	//		dijit/form/CheckBox

	// Back compat w/1.6, remove for 2.0
	if(has("dijit-legacy-requires")){
		ready(0, function(){
			var requires = ["dijit/form/RadioButton"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	return declare("dijit.form.CheckBox", [ToggleButton, _CheckBoxMixin], {
		// summary:
		//		Same as an HTML checkbox, but with fancy styling.
		//
		// description:
		//		User interacts with real html inputs.
		//		On onclick (which occurs by mouse click, space-bar, or
		//		using the arrow keys to switch the selected radio button),
		//		we update the state of the checkbox/radio.
		//
		//		There are two modes:
		//
		//		1. High contrast mode
		//		2. Normal mode
		//
		//		In case 1, the regular html inputs are shown and used by the user.
		//		In case 2, the regular html inputs are invisible but still used by
		//		the user. They are turned quasi-invisible and overlay the background-image.

		templateString: template,

		baseClass: "dijitCheckBox",

		_setValueAttr: function(/*String|Boolean*/ newValue, /*Boolean*/ priorityChange){
			// summary:
			//		Handler for value= attribute to constructor, and also calls to
			//		set('value', val).
			// description:
			//		During initialization, just saves as attribute to the `<input type=checkbox>`.
			//
			//		After initialization,
			//		when passed a boolean, controls whether or not the CheckBox is checked.
			//		If passed a string, changes the value attribute of the CheckBox (the one
			//		specified as "value" when the CheckBox was constructed
			//		(ex: `<input data-dojo-type="dijit/CheckBox" value="chicken">`).
			//
			//		`widget.set('value', string)` will check the checkbox and change the value to the
			//		specified string.
			//
			//		`widget.set('value', boolean)` will change the checked state.

			if(typeof newValue == "string"){
				this.inherited(arguments);
				newValue = true;
			}
			if(this._created){
				this.set('checked', newValue, priorityChange);
			}
		},
		_getValueAttr: function(){
			// summary:
			//		Hook so get('value') works.
			// description:
			//		If the CheckBox is checked, returns the value attribute.
			//		Otherwise returns false.
			return (this.checked ? this.value : false);
		},

		// Override behavior from Button, since we don't have an iconNode
		_setIconClassAttr: null,

		postMixInProperties: function(){
			this.inherited(arguments);

			// Need to set initial checked state via node.setAttribute so that form submit works
			// and IE8 radio button tab order is preserved.
			// domAttr.set(node, "checked", bool) doesn't work on IE until node has been attached
			// to <body>, see #8666
			this.checkedAttrSetting = "";
		},

		 _fillContent: function(){
			// Override Button::_fillContent() since it doesn't make sense for CheckBox,
			// since CheckBox doesn't even have a container
		},

		_onFocus: function(){
			if(this.id){
				query("label[for='"+this.id+"']").addClass("dijitFocusedLabel");
			}
			this.inherited(arguments);
		},

		_onBlur: function(){
			if(this.id){
				query("label[for='"+this.id+"']").removeClass("dijitFocusedLabel");
			}
			this.inherited(arguments);
		}
	});
});

},
'dijit/form/ToggleButton':function(){
define("dijit/form/ToggleButton", [
	"dojo/_base/declare", // declare
	"dojo/_base/kernel", // kernel.deprecated
	"./Button",
	"./_ToggleButtonMixin"
], function(declare, kernel, Button, _ToggleButtonMixin){

	// module:
	//		dijit/form/ToggleButton


	return declare("dijit.form.ToggleButton", [Button, _ToggleButtonMixin], {
		// summary:
		//		A templated button widget that can be in two states (checked or not).
		//		Can be base class for things like tabs or checkbox or radio buttons.

		baseClass: "dijitToggleButton",

		setChecked: function(/*Boolean*/ checked){
			// summary:
			//		Deprecated.  Use set('checked', true/false) instead.
			kernel.deprecated("setChecked("+checked+") is deprecated. Use set('checked',"+checked+") instead.", "", "2.0");
			this.set('checked', checked);
		}
	});
});

},
'dijit/form/_ToggleButtonMixin':function(){
define("dijit/form/_ToggleButtonMixin", [
	"dojo/_base/declare", // declare
	"dojo/dom-attr" // domAttr.set
], function(declare, domAttr){

// module:
//		dijit/form/_ToggleButtonMixin

return declare("dijit.form._ToggleButtonMixin", null, {
	// summary:
	//		A mixin to provide functionality to allow a button that can be in two states (checked or not).

	// checked: Boolean
	//		Corresponds to the native HTML `<input>` element's attribute.
	//		In markup, specified as "checked='checked'" or just "checked".
	//		True if the button is depressed, or the checkbox is checked,
	//		or the radio button is selected, etc.
	checked: false,

	// aria-pressed for toggle buttons, and aria-checked for checkboxes
	_aria_attr: "aria-pressed",

	_onClick: function(/*Event*/ evt){
		var original = this.checked;
		this._set('checked', !original); // partially set the toggled value, assuming the toggle will work, so it can be overridden in the onclick handler
		var ret = this.inherited(arguments); // the user could reset the value here
		this.set('checked', ret ? this.checked : original); // officially set the toggled or user value, or reset it back
		return ret;
	},

	_setCheckedAttr: function(/*Boolean*/ value, /*Boolean?*/ priorityChange){
		this._set("checked", value);
		var node = this.focusNode || this.domNode;
		if(this._created){ // IE is not ready to handle checked attribute (affects tab order)
			// needlessly setting "checked" upsets IE's tab order
			if(domAttr.get(node, "checked") != !!value){
				domAttr.set(node, "checked", !!value); // "mixed" -> true
			}
		}
		node.setAttribute(this._aria_attr, String(value)); // aria values should be strings
		this._handleOnChange(value, priorityChange);
	},

	postCreate: function(){ // use postCreate instead of startup so users forgetting to call startup are OK
		this.inherited(arguments);
		var node = this.focusNode || this.domNode;
		if(this.checked){
			// need this here instead of on the template so IE8 tab order works
			node.setAttribute('checked', 'checked');
		}

		// Update our reset value if it hasn't yet been set (because this.set()
		// is only called when there *is* a value)
		if(this._resetValue === undefined){
			this._lastValueReported = this._resetValue = this.checked;
		}
	},

	reset: function(){
		// summary:
		//		Reset the widget's value to what it was at initialization time

		this._hasBeenBlurred = false;

		// set checked state to original setting
		this.set('checked', this.params.checked || false);
	}
});

});

},
'dijit/form/_CheckBoxMixin':function(){
define("dijit/form/_CheckBoxMixin", [
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/_base/event" // event.stop
], function(declare, domAttr, event){

	// module:
	//		dijit/form/_CheckBoxMixin

	return declare("dijit.form._CheckBoxMixin", null, {
		// summary:
		//		Mixin to provide widget functionality corresponding to an HTML checkbox
		//
		// description:
		//		User interacts with real html inputs.
		//		On onclick (which occurs by mouse click, space-bar, or
		//		using the arrow keys to switch the selected radio button),
		//		we update the state of the checkbox/radio.
		//

		// type: [private] String
		//		type attribute on `<input>` node.
		//		Overrides `dijit/form/Button.type`.  Users should not change this value.
		type: "checkbox",

		// value: String
		//		As an initialization parameter, equivalent to value field on normal checkbox
		//		(if checked, the value is passed as the value when form is submitted).
		value: "on",

		// readOnly: Boolean
		//		Should this widget respond to user input?
		//		In markup, this is specified as "readOnly".
		//		Similar to disabled except readOnly form values are submitted.
		readOnly: false,
		
		// aria-pressed for toggle buttons, and aria-checked for checkboxes
		_aria_attr: "aria-checked",

		_setReadOnlyAttr: function(/*Boolean*/ value){
			this._set("readOnly", value);
			domAttr.set(this.focusNode, 'readOnly', value);
		},

		// Override dijit/form/Button._setLabelAttr() since we don't even have a containerNode.
		// Normally users won't try to set label, except when CheckBox or RadioButton is the child of a dojox/layout/TabContainer
		_setLabelAttr: undefined,

		_getSubmitValue: function(/*String*/ value){
			return (value == null || value === "") ? "on" : value;
		},

		_setValueAttr: function(newValue){
			newValue = this._getSubmitValue(newValue);	// "on" to match browser native behavior when value unspecified
			this._set("value", newValue);
			domAttr.set(this.focusNode, "value", newValue);
		},

		reset: function(){
			this.inherited(arguments);
			// Handle unlikely event that the <input type=checkbox> value attribute has changed
			this._set("value", this._getSubmitValue(this.params.value));
			domAttr.set(this.focusNode, 'value', this.value);
		},

		_onClick: function(/*Event*/ e){
			// summary:
			//		Internal function to handle click actions - need to check
			//		readOnly, since button no longer does that check.
			if(this.readOnly){
				event.stop(e);
				return false;
			}
			return this.inherited(arguments);
		}
	});
});

},
'url:dijit/form/templates/CheckBox.html':"<div class=\"dijit dijitReset dijitInline\" role=\"presentation\"\n\t><input\n\t \t${!nameAttrSetting} type=\"${type}\" role=\"${type}\" aria-checked=\"false\" ${checkedAttrSetting}\n\t\tclass=\"dijitReset dijitCheckBoxInput\"\n\t\tdata-dojo-attach-point=\"focusNode\"\n\t \tdata-dojo-attach-event=\"onclick:_onClick\"\n/></div>\n",
'dijit/form/_RadioButtonMixin':function(){
define("dijit/form/_RadioButtonMixin", [
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/_base/event", // event.stop
	"dojo/_base/lang", // lang.hitch
	"dojo/query", // query
	"../registry"	// registry.getEnclosingWidget
], function(array, declare, domAttr, event, lang, query, registry){

	// module:
	//		dijit/form/_RadioButtonMixin

	return declare("dijit.form._RadioButtonMixin", null, {
		// summary:
		//		Mixin to provide widget functionality for an HTML radio button

		// type: [private] String
		//		type attribute on `<input>` node.
		//		Users should not change this value.
		type: "radio",

		_getRelatedWidgets: function(){
			// Private function needed to help iterate over all radio buttons in a group.
			var ary = [];
			query("input[type=radio]", this.focusNode.form || this.ownerDocument).forEach( // can't use name= since query doesn't support [] in the name
				lang.hitch(this, function(inputNode){
					if(inputNode.name == this.name && inputNode.form == this.focusNode.form){
						var widget = registry.getEnclosingWidget(inputNode);
						if(widget){
							ary.push(widget);
						}
					}
				})
			);
			return ary;
		},

		_setCheckedAttr: function(/*Boolean*/ value){
			// If I am being checked then have to deselect currently checked radio button
			this.inherited(arguments);
			if(!this._created){ return; }
			if(value){
				array.forEach(this._getRelatedWidgets(), lang.hitch(this, function(widget){
					if(widget != this && widget.checked){
						widget.set('checked', false);
					}
				}));
			}
		},

		_getSubmitValue: function(/*String*/ value){
			return value == null ? "on" : value;
		},

		_onClick: function(/*Event*/ e){
			if(this.checked || this.disabled){ // nothing to do
				event.stop(e);
				return false;
			}
			if(this.readOnly){ // ignored by some browsers so we have to resync the DOM elements with widget values
				event.stop(e);
				array.forEach(this._getRelatedWidgets(), lang.hitch(this, function(widget){
					domAttr.set(this.focusNode || this.domNode, 'checked', widget.checked);
				}));
				return false;
			}
			return this.inherited(arguments);
		}
	});
});

},
'epi/shell/widget/PopupMenuItem':function(){
﻿define("epi/shell/widget/PopupMenuItem", ["dojo/_base/declare", "dijit/_Container", "dijit/Menu", "dijit/PopupMenuItem"], function (declare, _Container, Menu, PopupMenuItem) {

    return declare([PopupMenuItem, _Container], {
        // summary:
        //  A custom popup menu implementation made to behave like a proper container
        //
        // tags:
        //      internal

        constructor: function () {
            // Create the popup sub menu which encapsulates the actual menu items
            this.popup = new Menu();
        },

        addChild: function (/*dijit/_Widget*/widget, /*int?*/insertIndex) {
            // summary:
            //  Adds a widget to the encapsulated menu

            return this.popup.addChild(widget, insertIndex);
        },

        getChildren: function () {
            // summary:
            //  return the children of this menu.

            return this.popup.getChildren();
        },

        removeChild: function (/*Widget|int*/ widget) {
            // summary:
            //		Removes the passed widget instance from this widget but does
            //		not destroy it.  You can also pass in an integer indicating
            //		the index within the container to remove

            return this.popup.removeChild(widget);
        }
    });
});

},
'dijit/PopupMenuItem':function(){
define("dijit/PopupMenuItem", [
	"dojo/_base/declare", // declare
	"dojo/dom-style", // domStyle.set
	"dojo/query", // query
	"./registry",	// registry.byNode
	"./MenuItem",
	"./hccss"
], function(declare, domStyle, query, registry, MenuItem){

	// module:
	//		dijit/PopupMenuItem

	return declare("dijit.PopupMenuItem", MenuItem, {
		// summary:
		//		An item in a Menu that spawn a drop down (usually a drop down menu)

		_fillContent: function(){
			// summary:
			//		When Menu is declared in markup, this code gets the menu label and
			//		the popup widget from the srcNodeRef.
			// description:
			//		srcNodeRefinnerHTML contains both the menu item text and a popup widget
			//		The first part holds the menu item text and the second part is the popup
			// example:
			// |	<div data-dojo-type="dijit/PopupMenuItem">
			// |		<span>pick me</span>
			// |		<popup> ... </popup>
			// |	</div>
			// tags:
			//		protected

			if(this.srcNodeRef){
				var nodes = query("*", this.srcNodeRef);
				this.inherited(arguments, [nodes[0]]);

				// save pointer to srcNode so we can grab the drop down widget after it's instantiated
				this.dropDownContainer = this.srcNodeRef;
			}
		},

		startup: function(){
			if(this._started){ return; }
			this.inherited(arguments);

			// we didn't copy the dropdown widget from the this.srcNodeRef, so it's in no-man's
			// land now.  move it to win.doc.body.
			if(!this.popup){
				var node = query("[widgetId]", this.dropDownContainer)[0];
				this.popup = registry.byNode(node);
			}
			this.ownerDocumentBody.appendChild(this.popup.domNode);
			this.popup.startup();

			this.popup.domNode.style.display="none";
			if(this.arrowWrapper){
				domStyle.set(this.arrowWrapper, "visibility", "");
			}
			this.focusNode.setAttribute("aria-haspopup", "true");
		},

		destroyDescendants: function(/*Boolean*/ preserveDom){
			if(this.popup){
				// Destroy the popup, unless it's already been destroyed.  This can happen because
				// the popup is a direct child of <body> even though it's logically my child.
				if(!this.popup._destroyed){
					this.popup.destroyRecursive(preserveDom);
				}
				delete this.popup;
			}
			this.inherited(arguments);
		}
	});
});

},
'epi/shell/widget/ToolbarLabel':function(){
﻿define("epi/shell/widget/ToolbarLabel", [
    "dojo",
    "dojo/dom-construct",
    "dijit/_WidgetBase"
], function (dojo, construct, _WidgetBase) {

    return dojo.declare([_WidgetBase], {
        // summary:
        //      A label widget inteaded for display in the toolbar.
        //
        // tags:
        //      internal xproduct

        // baseClass: [protected] String
        //		Root CSS class of the widget.
        baseClass: "epi-breadCrumbsCurrentItem dijitInline",

        // label: [public] String
        //		Value to be displayed in the label.
        label: null,

        buildRendering: function () {
            // summary:
            //		Construct the UI for this widget, setting this.domNode.
            // tags:
            //		protected

            if (!this.domNode) {
                this.domNode = this.srcNodeRef || construct.create("h1");
            }

            this.inherited(arguments);
        },

        _setLabelAttr: { node: "domNode", type: "innerText" }
    });
});

},
'epi/shell/widget/LoginForm':function(){
require({cache:{
'url:epi/shell/widget/templates/LoginForm.htm':"﻿<div class=\"epi-form\">\n    <ul class=\"dijitReset\" data-dojo-attach-point=\"containerNode\">\n        <li>${resources.loggedout}</li>\n        <li>\n            <label>${resources.username}</label>\n            <input data-dojo-type=\"dijit/form/ValidationTextBox\" required=\"true\" data-dojo-attach-point=\"usernameNode\" name=\"username\" />\n        </li>\n        <li>\n            <label>${resources.password}</label>\n            <input data-dojo-type=\"dijit/form/ValidationTextBox\" required=\"true\" data-dojo-attach-point=\"passwordNode\" name=\"password\" type=\"password\" />\n        </li>\n    </ul>\n</div>\n"}});
define("epi/shell/widget/LoginForm", [
    "epi",
    "dojo",
    "dojo/keys",
    "dojo/text!./templates/LoginForm.htm",
    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/form/ValidationTextBox", // Needed for the template
    "dojo/i18n!epi/nls/EPiServer.Shell.UI.Resources.LoginForm"
], function (epi, dojo, keys, template, _WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, ValidationTextBox, localization) {

    return dojo.declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin], {
        // summary:
        //      A login form widget
        //
        // tags:
        //      internal

        // templateString: [protected] String
        //		A string that represents the login form template.
        templateString: template,

        postMixInProperties: function () {
            // summary:
            //		Mixin language resources to the class.
            // tags:
            //		protected

            this.inherited(arguments);
            this.resources = dojo.mixin(epi.resources.action, localization);

            this.connect(this.passwordNode, "onkeydown", this._onKeyDown);
        },

        serializeForm: function () {
            // summary:
            //		Gets the username and password as entered by the user.

            return {
                username: this.usernameNode.get("value"),
                password: this.passwordNode.get("value")
            };
        },

        authenticationFailed: function () {
            // summary:
            //		Clears the password box and sets it's validation message to login failed.

            this.passwordNode.set("value", "");
            this.passwordNode.focus();
            this.passwordNode.displayMessage(this.resources.loginfailed);
        },

        _onKeyDown: function (e) {
            if (e.keyCode === keys.ENTER && !(e.ctrlKey || e.altKey || e.metaKey)) {
                this.onSubmit(e);
            }
        },

        onSubmit: function () {
            // summary:
            //		Callback when user submits the form.
            // tags:
            //		callback
        }
    });
});

},
'url:epi/shell/widget/templates/LoginForm.htm':"﻿<div class=\"epi-form\">\n    <ul class=\"dijitReset\" data-dojo-attach-point=\"containerNode\">\n        <li>${resources.loggedout}</li>\n        <li>\n            <label>${resources.username}</label>\n            <input data-dojo-type=\"dijit/form/ValidationTextBox\" required=\"true\" data-dojo-attach-point=\"usernameNode\" name=\"username\" />\n        </li>\n        <li>\n            <label>${resources.password}</label>\n            <input data-dojo-type=\"dijit/form/ValidationTextBox\" required=\"true\" data-dojo-attach-point=\"passwordNode\" name=\"password\" type=\"password\" />\n        </li>\n    </ul>\n</div>\n",
'dijit/_WidgetsInTemplateMixin':function(){
define("dijit/_WidgetsInTemplateMixin", [
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/parser" // parser.parse
], function(array, declare, parser){

	// module:
	//		dijit/_WidgetsInTemplateMixin

	return declare("dijit._WidgetsInTemplateMixin", null, {
		// summary:
		//		Mixin to supplement _TemplatedMixin when template contains widgets

		// _earlyTemplatedStartup: Boolean
		//		A fallback to preserve the 1.0 - 1.3 behavior of children in
		//		templates having their startup called before the parent widget
		//		fires postCreate. Defaults to 'false', causing child widgets to
		//		have their .startup() called immediately before a parent widget
		//		.startup(), but always after the parent .postCreate(). Set to
		//		'true' to re-enable to previous, arguably broken, behavior.
		_earlyTemplatedStartup: false,

		// widgetsInTemplate: [protected] Boolean
		//		Should we parse the template to find widgets that might be
		//		declared in markup inside it?  (Remove for 2.0 and assume true)
		widgetsInTemplate: true,

		_beforeFillContent: function(){
			if(this.widgetsInTemplate){
				// Before copying over content, instantiate widgets in template
				var node = this.domNode;

				var cw = (this._startupWidgets = parser.parse(node, {
					noStart: !this._earlyTemplatedStartup,
					template: true,
					inherited: {dir: this.dir, lang: this.lang, textDir: this.textDir},
					propsThis: this,	// so data-dojo-props of widgets in the template can reference "this" to refer to me
					scope: "dojo"	// even in multi-version mode templates use dojoType/data-dojo-type
				}));

				if(!cw.isFulfilled()){
					throw new Error(this.declaredClass + ": parser returned unfilled promise (probably waiting for module auto-load), " +
						"unsupported by _WidgetsInTemplateMixin.   Must pre-load all supporting widgets before instantiation.");
				}

				// _WidgetBase::destroy() will destroy any supporting widgets under this.domNode.
				// If we wanted to, we could call this.own() on anything in this._startupWidgets that was moved outside
				// of this.domNode (like Dialog, which is moved to <body>).

				this._attachTemplateNodes(cw, function(n,p){
					return n[p];
				});
			}
		},

		startup: function(){
			array.forEach(this._startupWidgets, function(w){
				if(w && !w._started && w.startup){
					w.startup();
				}
			});
			this.inherited(arguments);
		}
	});
});

},
'dijit/form/ValidationTextBox':function(){
require({cache:{
'url:dijit/form/templates/ValidationTextBox.html':"<div class=\"dijit dijitReset dijitInline dijitLeft\"\n\tid=\"widget_${id}\" role=\"presentation\"\n\t><div class='dijitReset dijitValidationContainer'\n\t\t><input class=\"dijitReset dijitInputField dijitValidationIcon dijitValidationInner\" value=\"&#935; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t/></div\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\n\t\t><input class=\"dijitReset dijitInputInner\" data-dojo-attach-point='textbox,focusNode' autocomplete=\"off\"\n\t\t\t${!nameAttrSetting} type='${type}'\n\t/></div\n></div>\n"}});
define("dijit/form/ValidationTextBox", [
	"dojo/_base/declare", // declare
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/i18n", // i18n.getLocalization
	"./TextBox",
	"../Tooltip",
	"dojo/text!./templates/ValidationTextBox.html",
	"dojo/i18n!./nls/validate"
], function(declare, kernel, i18n, TextBox, Tooltip, template){

	// module:
	//		dijit/form/ValidationTextBox


	/*=====
	var __Constraints = {
		// locale: String
		//		locale used for validation, picks up value from this widget's lang attribute
		// _flags_: anything
		//		various flags passed to pattern function
	};
	=====*/

	var ValidationTextBox;
	return ValidationTextBox = declare("dijit.form.ValidationTextBox", TextBox, {
		// summary:
		//		Base class for textbox widgets with the ability to validate content of various types and provide user feedback.

		templateString: template,

		// required: Boolean
		//		User is required to enter data into this field.
		required: false,

		// promptMessage: String
		//		If defined, display this hint string immediately on focus to the textbox, if empty.
		//		Also displays if the textbox value is Incomplete (not yet valid but will be with additional input).
		//		Think of this like a tooltip that tells the user what to do, not an error message
		//		that tells the user what they've done wrong.
		//
		//		Message disappears when user starts typing.
		promptMessage: "",

		// invalidMessage: String
		//		The message to display if value is invalid.
		//		The translated string value is read from the message file by default.
		//		Set to "" to use the promptMessage instead.
		invalidMessage: "$_unset_$",

		// missingMessage: String
		//		The message to display if value is empty and the field is required.
		//		The translated string value is read from the message file by default.
		//		Set to "" to use the invalidMessage instead.
		missingMessage: "$_unset_$",

		// message: String
		//		Currently error/prompt message.
		//		When using the default tooltip implementation, this will only be
		//		displayed when the field is focused.
		message: "",

		// constraints: __Constraints
		//		user-defined object needed to pass parameters to the validator functions
		constraints: {},

		// pattern: [extension protected] String|Function(constraints) returning a string.
		//		This defines the regular expression used to validate the input.
		//		Do not add leading ^ or $ characters since the widget adds these.
		//		A function may be used to generate a valid pattern when dependent on constraints or other runtime factors.
		//		set('pattern', String|Function).
		pattern: ".*",

		// regExp: Deprecated [extension protected] String.  Use "pattern" instead.
		regExp: "",

		regExpGen: function(/*__Constraints*/ /*===== constraints =====*/){
			// summary:
			//		Deprecated.  Use set('pattern', Function) instead.
		},

		// state: [readonly] String
		//		Shows current state (ie, validation result) of input (""=Normal, Incomplete, or Error)
		state: "",

		// tooltipPosition: String[]
		//		See description of `dijit/Tooltip.defaultPosition` for details on this parameter.
		tooltipPosition: [],

		_deprecateRegExp: function(attr, value){
			if(value != ValidationTextBox.prototype[attr]){
				kernel.deprecated("ValidationTextBox id="+this.id+", set('" + attr + "', ...) is deprecated.  Use set('pattern', ...) instead.", "", "2.0");
				this.set('pattern', value);
			}
		},
		_setRegExpGenAttr: function(/*Function*/ newFcn){
			this._deprecateRegExp("regExpGen", newFcn);
			this.regExpGen = this._getPatternAttr; // backward compat with this.regExpGen(this.constraints)
		},
		_setRegExpAttr: function(/*String*/ value){
			this._deprecateRegExp("regExp", value);
		},

		_setValueAttr: function(){
			// summary:
			//		Hook so set('value', ...) works.
			this.inherited(arguments);
			this._refreshState();
		},

		validator: function(/*anything*/ value, /*__Constraints*/ constraints){
			// summary:
			//		Overridable function used to validate the text input against the regular expression.
			// tags:
			//		protected
			return (new RegExp("^(?:" + this._getPatternAttr(constraints) + ")"+(this.required?"":"?")+"$")).test(value) &&
				(!this.required || !this._isEmpty(value)) &&
				(this._isEmpty(value) || this.parse(value, constraints) !== undefined); // Boolean
		},

		_isValidSubset: function(){
			// summary:
			//		Returns true if the value is either already valid or could be made valid by appending characters.
			//		This is used for validation while the user [may be] still typing.
			return this.textbox.value.search(this._partialre) == 0;
		},

		isValid: function(/*Boolean*/ /*===== isFocused =====*/){
			// summary:
			//		Tests if value is valid.
			//		Can override with your own routine in a subclass.
			// tags:
			//		protected
			return this.validator(this.textbox.value, this.constraints);
		},

		_isEmpty: function(value){
			// summary:
			//		Checks for whitespace
			return (this.trim ? /^\s*$/ : /^$/).test(value); // Boolean
		},

		getErrorMessage: function(/*Boolean*/ /*===== isFocused =====*/){
			// summary:
			//		Return an error message to show if appropriate
			// tags:
			//		protected
			var invalid = this.invalidMessage == "$_unset_$" ? this.messages.invalidMessage :
				!this.invalidMessage ? this.promptMessage : this.invalidMessage;
			var missing = this.missingMessage == "$_unset_$" ? this.messages.missingMessage :
				!this.missingMessage ? invalid : this.missingMessage;
			return (this.required && this._isEmpty(this.textbox.value)) ? missing : invalid; // String
		},

		getPromptMessage: function(/*Boolean*/ /*===== isFocused =====*/){
			// summary:
			//		Return a hint message to show when widget is first focused
			// tags:
			//		protected
			return this.promptMessage; // String
		},

		_maskValidSubsetError: true,
		validate: function(/*Boolean*/ isFocused){
			// summary:
			//		Called by oninit, onblur, and onkeypress.
			// description:
			//		Show missing or invalid messages if appropriate, and highlight textbox field.
			// tags:
			//		protected
			var message = "";
			var isValid = this.disabled || this.isValid(isFocused);
			if(isValid){ this._maskValidSubsetError = true; }
			var isEmpty = this._isEmpty(this.textbox.value);
			var isValidSubset = !isValid && isFocused && this._isValidSubset();
			this._set("state", isValid ? "" : (((((!this._hasBeenBlurred || isFocused) && isEmpty) || isValidSubset) && (this._maskValidSubsetError || (isValidSubset && !this._hasBeenBlurred && isFocused))) ? "Incomplete" : "Error"));
			this.focusNode.setAttribute("aria-invalid", isValid ? "false" : "true");

			if(this.state == "Error"){
				this._maskValidSubsetError = isFocused && isValidSubset; // we want the error to show up after a blur and refocus
				message = this.getErrorMessage(isFocused);
			}else if(this.state == "Incomplete"){
				message = this.getPromptMessage(isFocused); // show the prompt whenever the value is not yet complete
				this._maskValidSubsetError = !this._hasBeenBlurred || isFocused; // no Incomplete warnings while focused
			}else if(isEmpty){
				message = this.getPromptMessage(isFocused); // show the prompt whenever there's no error and no text
			}
			this.set("message", message);

			return isValid;
		},

		displayMessage: function(/*String*/ message){
			// summary:
			//		Overridable method to display validation errors/hints.
			//		By default uses a tooltip.
			// tags:
			//		extension
			if(message && this.focused){
				Tooltip.show(message, this.domNode, this.tooltipPosition, !this.isLeftToRight());
			}else{
				Tooltip.hide(this.domNode);
			}
		},

		_refreshState: function(){
			// Overrides TextBox._refreshState()
			if(this._created){ // should instead be this._started but that would require all programmatic ValidationTextBox instantiations to call startup()
				this.validate(this.focused);
			}
			this.inherited(arguments);
		},

		//////////// INITIALIZATION METHODS ///////////////////////////////////////

		constructor: function(params /*===== , srcNodeRef =====*/){
			// summary:
			//		Create the widget.
			// params: Object|null
			//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
			//		and functions, typically callbacks like onClick.
			//		The hash can contain any of the widget's properties, excluding read-only properties.
			// srcNodeRef: DOMNode|String?
			//		If a srcNodeRef (DOM node) is specified, replace srcNodeRef with my generated DOM tree.

			this.constraints = {};
			this.baseClass += ' dijitValidationTextBox';
		},

		startup: function(){
			this.inherited(arguments);
			this._refreshState(); // after all _set* methods have run
		},

		_setConstraintsAttr: function(/*__Constraints*/ constraints){
			if(!constraints.locale && this.lang){
				constraints.locale = this.lang;
			}
			this._set("constraints", constraints);
			this._refreshState();
		},

		_setPatternAttr: function(/*String|Function*/ pattern){
			this._set("pattern", pattern); // don't set on INPUT to avoid native HTML5 validation
			this._refreshState();
		},

		_getPatternAttr: function(/*__Constraints*/ constraints){
			// summary:
			//		Hook to get the current regExp and to compute the partial validation RE.
			var p = this.pattern;
			var type = (typeof p).toLowerCase();
			if(type == "function"){
				p = this.pattern(constraints || this.constraints);
			}
			if(p != this._lastRegExp){
				var partialre = "";
				this._lastRegExp = p;
				// parse the regexp and produce a new regexp that matches valid subsets
				// if the regexp is .* then there's no use in matching subsets since everything is valid
				if(p != ".*"){
					p.replace(/\\.|\[\]|\[.*?[^\\]{1}\]|\{.*?\}|\(\?[=:!]|./g,
					function(re){
						switch(re.charAt(0)){
							case '{':
							case '+':
							case '?':
							case '*':
							case '^':
							case '$':
							case '|':
							case '(':
								partialre += re;
								break;
							case ")":
								partialre += "|$)";
								break;
							 default:
								partialre += "(?:"+re+"|$)";
								break;
						}
					});
				}
				try{ // this is needed for now since the above regexp parsing needs more test verification
					"".search(partialre);
				}catch(e){ // should never be here unless the original RE is bad or the parsing is bad
					partialre = this.pattern;
					console.warn('RegExp error in ' + this.declaredClass + ': ' + this.pattern);
				} // should never be here unless the original RE is bad or the parsing is bad
				this._partialre = "^(?:" + partialre + ")$";
			}
			return p;
		},

		postMixInProperties: function(){
			this.inherited(arguments);
			this.messages = i18n.getLocalization("dijit.form", "validate", this.lang);
			this._setConstraintsAttr(this.constraints); // this needs to happen now (and later) due to codependency on _set*Attr calls attachPoints
		},

		_setDisabledAttr: function(/*Boolean*/ value){
			this.inherited(arguments);	// call FormValueWidget._setDisabledAttr()
			this._refreshState();
		},

		_setRequiredAttr: function(/*Boolean*/ value){
			this._set("required", value);
			this.focusNode.setAttribute("aria-required", value);
			this._refreshState();
		},

		_setMessageAttr: function(/*String*/ message){
			this._set("message", message);
			this.displayMessage(message);
		},

		reset:function(){
			// Overrides dijit/form/TextBox.reset() by also
			// hiding errors about partial matches
			this._maskValidSubsetError = true;
			this.inherited(arguments);
		},

		_onBlur: function(){
			// the message still exists but for back-compat, and to erase the tooltip
			// (if the message is being displayed as a tooltip), call displayMessage('')
			this.displayMessage('');

			this.inherited(arguments);
		}
	});
});

},
'dijit/form/TextBox':function(){
require({cache:{
'url:dijit/form/templates/TextBox.html':"<div class=\"dijit dijitReset dijitInline dijitLeft\" id=\"widget_${id}\" role=\"presentation\"\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\n\t\t><input class=\"dijitReset dijitInputInner\" data-dojo-attach-point='textbox,focusNode' autocomplete=\"off\"\n\t\t\t${!nameAttrSetting} type='${type}'\n\t/></div\n></div>\n"}});
define("dijit/form/TextBox", [
	"dojo/_base/declare", // declare
	"dojo/dom-construct", // domConstruct.create
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.hitch
	"dojo/sniff", // has("ie") has("mozilla")
	"./_FormValueWidget",
	"./_TextBoxMixin",
	"dojo/text!./templates/TextBox.html",
	"../main"	// to export dijit._setSelectionRange, remove in 2.0
], function(declare, domConstruct, domStyle, kernel, lang, has,
			_FormValueWidget, _TextBoxMixin, template, dijit){

	// module:
	//		dijit/form/TextBox

	var TextBox = declare("dijit.form.TextBox", [_FormValueWidget, _TextBoxMixin], {
		// summary:
		//		A base class for textbox form inputs

		templateString: template,
		_singleNodeTemplate: '<input class="dijit dijitReset dijitLeft dijitInputField" data-dojo-attach-point="textbox,focusNode" autocomplete="off" type="${type}" ${!nameAttrSetting} />',

		_buttonInputDisabled: has("ie") ? "disabled" : "", // allows IE to disallow focus, but Firefox cannot be disabled for mousedown events

		baseClass: "dijitTextBox",

		postMixInProperties: function(){
			var type = this.type.toLowerCase();
			if(this.templateString && this.templateString.toLowerCase() == "input" || ((type == "hidden" || type == "file") && this.templateString == this.constructor.prototype.templateString)){
				this.templateString = this._singleNodeTemplate;
			}
			this.inherited(arguments);
		},

		postCreate: function(){
			this.inherited(arguments);

			if(has("ie") < 9){
				// IE INPUT tag fontFamily has to be set directly using STYLE
				// the defer gives IE a chance to render the TextBox and to deal with font inheritance
				this.defer(function(){
					try{
						var s = domStyle.getComputedStyle(this.domNode); // can throw an exception if widget is immediately destroyed
						if(s){
							var ff = s.fontFamily;
							if(ff){
								var inputs = this.domNode.getElementsByTagName("INPUT");
								if(inputs){
									for(var i=0; i < inputs.length; i++){
										inputs[i].style.fontFamily = ff;
									}
								}
							}
						}
					}catch(e){/*when used in a Dialog, and this is called before the dialog is
					 shown, s.fontFamily would trigger "Invalid Argument" error.*/}
				});
			}
		},

		_setPlaceHolderAttr: function(v){
			this._set("placeHolder", v);
			if(!this._phspan){
				this._attachPoints.push('_phspan');
				// dijitInputField class gives placeHolder same padding as the input field
				// parent node already has dijitInputField class but it doesn't affect this <span>
				// since it's position: absolute.
				this._phspan = domConstruct.create('span',{ onmousedown:function(e){ e.preventDefault(); }, className:'dijitPlaceHolder dijitInputField'},this.textbox,'after');
			}
			this._phspan.innerHTML="";
			this._phspan.appendChild(this._phspan.ownerDocument.createTextNode(v));
			this._updatePlaceHolder();
		},

		_onInput: function(/*Event*/ evt){
			// summary:
			//		Called AFTER the input event has happened
			//		See if the placeHolder text should be removed or added while editing.
			this.inherited(arguments);
			this._updatePlaceHolder();
		},

		_updatePlaceHolder: function(){
			if(this._phspan){
				this._phspan.style.display = (this.placeHolder && !this.textbox.value) ? "" : "none";
			}
		},

		_setValueAttr: function(value, /*Boolean?*/ priorityChange, /*String?*/ formattedValue){
			this.inherited(arguments);
			this._updatePlaceHolder();
		},

		getDisplayedValue: function(){
			// summary:
			//		Deprecated.  Use get('displayedValue') instead.
			// tags:
			//		deprecated
			kernel.deprecated(this.declaredClass+"::getDisplayedValue() is deprecated. Use get('displayedValue') instead.", "", "2.0");
			return this.get('displayedValue');
		},

		setDisplayedValue: function(/*String*/ value){
			// summary:
			//		Deprecated.  Use set('displayedValue', ...) instead.
			// tags:
			//		deprecated
			kernel.deprecated(this.declaredClass+"::setDisplayedValue() is deprecated. Use set('displayedValue', ...) instead.", "", "2.0");
			this.set('displayedValue', value);
		},

		_onBlur: function(e){
			if(this.disabled){ return; }
			this.inherited(arguments);
			this._updatePlaceHolder();

			if(has("mozilla")){
				if(this.selectOnClick){
					// clear selection so that the next mouse click doesn't reselect
					this.textbox.selectionStart = this.textbox.selectionEnd = undefined;
				}
			}
		},

		_onFocus: function(/*String*/ by){
			if(this.disabled || this.readOnly){ return; }
			this.inherited(arguments);
			this._updatePlaceHolder();
		}
	});

	if(has("ie") < 9){
		TextBox.prototype._isTextSelected = function(){
			var range = this.ownerDocument.selection.createRange();
			var parent = range.parentElement();
			return parent == this.textbox && range.text.length > 0;
		};

		// Overrides definition of _setSelectionRange from _TextBoxMixin (TODO: move to _TextBoxMixin.js?)
		dijit._setSelectionRange = _TextBoxMixin._setSelectionRange = function(/*DomNode*/ element, /*Number?*/ start, /*Number?*/ stop){
			if(element.createTextRange){
				var r = element.createTextRange();
				r.collapse(true);
				r.moveStart("character", -99999); // move to 0
				r.moveStart("character", start); // delta from 0 is the correct position
				r.moveEnd("character", stop-start);
				r.select();
			}
		}
	}

	return TextBox;
});

},
'dijit/form/_FormValueWidget':function(){
define("dijit/form/_FormValueWidget", [
	"dojo/_base/declare", // declare
	"dojo/sniff", // has("ie")
	"./_FormWidget",
	"./_FormValueMixin"
], function(declare, has, _FormWidget, _FormValueMixin){

// module:
//		dijit/form/_FormValueWidget

return declare("dijit.form._FormValueWidget", [_FormWidget, _FormValueMixin],
{
	// summary:
	//		Base class for widgets corresponding to native HTML elements such as `<input>` or `<select>`
	//		that have user changeable values.
	// description:
	//		Each _FormValueWidget represents a single input value, and has a (possibly hidden) `<input>` element,
	//		to which it serializes it's input value, so that form submission (either normal submission or via FormBind?)
	//		works as expected.

	// Don't attempt to mixin the 'type', 'name' attributes here programatically -- they must be declared
	// directly in the template as read by the parser in order to function. IE is known to specifically
	// require the 'name' attribute at element creation time.  See #8484, #8660.

	_layoutHackIE7: function(){
		// summary:
		//		Work around table sizing bugs on IE7 by forcing redraw

		if(has("ie") == 7){ // fix IE7 layout bug when the widget is scrolled out of sight
			var domNode = this.domNode;
			var parent = domNode.parentNode;
			var pingNode = domNode.firstChild || domNode; // target node most unlikely to have a custom filter
			var origFilter = pingNode.style.filter; // save custom filter, most likely nothing
			var _this = this;
			while(parent && parent.clientHeight == 0){ // search for parents that haven't rendered yet
				(function ping(){
					var disconnectHandle = _this.connect(parent, "onscroll",
						function(){
							_this.disconnect(disconnectHandle); // only call once
							pingNode.style.filter = (new Date()).getMilliseconds(); // set to anything that's unique
							_this.defer(function(){ pingNode.style.filter = origFilter; }); // restore custom filter, if any
						}
					);
				})();
				parent = parent.parentNode;
			}
		}
	}
});

});

},
'dijit/form/_FormValueMixin':function(){
define("dijit/form/_FormValueMixin", [
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/keys", // keys.ESCAPE
	"dojo/sniff", // has("ie"), has("quirks")
	"./_FormWidgetMixin"
], function(declare, domAttr, keys, has, _FormWidgetMixin){

	// module:
	//		dijit/form/_FormValueMixin

	return declare("dijit.form._FormValueMixin", _FormWidgetMixin, {
		// summary:
		//		Mixin for widgets corresponding to native HTML elements such as `<input>` or `<select>`
		//		that have user changeable values.
		// description:
		//		Each _FormValueMixin represents a single input value, and has a (possibly hidden) `<input>` element,
		//		to which it serializes it's input value, so that form submission (either normal submission or via FormBind?)
		//		works as expected.

		// readOnly: Boolean
		//		Should this widget respond to user input?
		//		In markup, this is specified as "readOnly".
		//		Similar to disabled except readOnly form values are submitted.
		readOnly: false,

		_setReadOnlyAttr: function(/*Boolean*/ value){
			domAttr.set(this.focusNode, 'readOnly', value);
			this._set("readOnly", value);
		},

		postCreate: function(){
			this.inherited(arguments);

			if(has("ie")){ // IE won't stop the event with keypress
				this.connect(this.focusNode || this.domNode, "onkeydown", this._onKeyDown);
			}
			// Update our reset value if it hasn't yet been set (because this.set()
			// is only called when there *is* a value)
			if(this._resetValue === undefined){
				this._lastValueReported = this._resetValue = this.value;
			}
		},

		_setValueAttr: function(/*anything*/ newValue, /*Boolean?*/ priorityChange){
			// summary:
			//		Hook so set('value', value) works.
			// description:
			//		Sets the value of the widget.
			//		If the value has changed, then fire onChange event, unless priorityChange
			//		is specified as null (or false?)
			this._handleOnChange(newValue, priorityChange);
		},

		_handleOnChange: function(/*anything*/ newValue, /*Boolean?*/ priorityChange){
			// summary:
			//		Called when the value of the widget has changed.  Saves the new value in this.value,
			//		and calls onChange() if appropriate.   See _FormWidget._handleOnChange() for details.
			this._set("value", newValue);
			this.inherited(arguments);
		},

		undo: function(){
			// summary:
			//		Restore the value to the last value passed to onChange
			this._setValueAttr(this._lastValueReported, false);
		},

		reset: function(){
			// summary:
			//		Reset the widget's value to what it was at initialization time
			this._hasBeenBlurred = false;
			this._setValueAttr(this._resetValue, true);
		},

		_onKeyDown: function(e){
			if(e.keyCode == keys.ESCAPE && !(e.ctrlKey || e.altKey || e.metaKey)){
				if(has("ie") < 9 || (has("ie") && has("quirks"))){
					e.preventDefault(); // default behavior needs to be stopped here since keypress is too late
					var node = e.srcElement,
						te = node.ownerDocument.createEventObject();
					te.keyCode = keys.ESCAPE;
					te.shiftKey = e.shiftKey;
					node.fireEvent('onkeypress', te);
				}
			}
		}
	});
});

},
'dijit/form/_TextBoxMixin':function(){
define("dijit/form/_TextBoxMixin", [
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.byId
	"dojo/_base/event", // event.stop
	"dojo/keys", // keys.ALT keys.CAPS_LOCK keys.CTRL keys.META keys.SHIFT
	"dojo/_base/lang", // lang.mixin
	"dojo/on", // on
	"../main"	// for exporting dijit._setSelectionRange, dijit.selectInputText
], function(array, declare, dom, event, keys, lang, on, dijit){

// module:
//		dijit/form/_TextBoxMixin

var _TextBoxMixin = declare("dijit.form._TextBoxMixin", null, {
	// summary:
	//		A mixin for textbox form input widgets

	// trim: Boolean
	//		Removes leading and trailing whitespace if true.  Default is false.
	trim: false,

	// uppercase: Boolean
	//		Converts all characters to uppercase if true.  Default is false.
	uppercase: false,

	// lowercase: Boolean
	//		Converts all characters to lowercase if true.  Default is false.
	lowercase: false,

	// propercase: Boolean
	//		Converts the first character of each word to uppercase if true.
	propercase: false,

	// maxLength: String
	//		HTML INPUT tag maxLength declaration.
	maxLength: "",

	// selectOnClick: [const] Boolean
	//		If true, all text will be selected when focused with mouse
	selectOnClick: false,

	// placeHolder: String
	//		Defines a hint to help users fill out the input field (as defined in HTML 5).
	//		This should only contain plain text (no html markup).
	placeHolder: "",

	_getValueAttr: function(){
		// summary:
		//		Hook so get('value') works as we like.
		// description:
		//		For `dijit/form/TextBox` this basically returns the value of the `<input>`.
		//
		//		For `dijit/form/MappedTextBox` subclasses, which have both
		//		a "displayed value" and a separate "submit value",
		//		This treats the "displayed value" as the master value, computing the
		//		submit value from it via this.parse().
		return this.parse(this.get('displayedValue'), this.constraints);
	},

	_setValueAttr: function(value, /*Boolean?*/ priorityChange, /*String?*/ formattedValue){
		// summary:
		//		Hook so set('value', ...) works.
		//
		// description:
		//		Sets the value of the widget to "value" which can be of
		//		any type as determined by the widget.
		//
		// value:
		//		The visual element value is also set to a corresponding,
		//		but not necessarily the same, value.
		//
		// formattedValue:
		//		If specified, used to set the visual element value,
		//		otherwise a computed visual value is used.
		//
		// priorityChange:
		//		If true, an onChange event is fired immediately instead of
		//		waiting for the next blur event.

		var filteredValue;
		if(value !== undefined){
			// TODO: this is calling filter() on both the display value and the actual value.
			// I added a comment to the filter() definition about this, but it should be changed.
			filteredValue = this.filter(value);
			if(typeof formattedValue != "string"){
				if(filteredValue !== null && ((typeof filteredValue != "number") || !isNaN(filteredValue))){
					formattedValue = this.filter(this.format(filteredValue, this.constraints));
				}else{ formattedValue = ''; }
			}
		}
		if(formattedValue != null /* and !undefined */ && ((typeof formattedValue) != "number" || !isNaN(formattedValue)) && this.textbox.value != formattedValue){
			this.textbox.value = formattedValue;
			this._set("displayedValue", this.get("displayedValue"));
		}

		if(this.textDir == "auto"){
			this.applyTextDir(this.focusNode, formattedValue);
		}

		this.inherited(arguments, [filteredValue, priorityChange]);
	},

	// displayedValue: String
	//		For subclasses like ComboBox where the displayed value
	//		(ex: Kentucky) and the serialized value (ex: KY) are different,
	//		this represents the displayed value.
	//
	//		Setting 'displayedValue' through set('displayedValue', ...)
	//		updates 'value', and vice-versa.  Otherwise 'value' is updated
	//		from 'displayedValue' periodically, like onBlur etc.
	//
	//		TODO: move declaration to MappedTextBox?
	//		Problem is that ComboBox references displayedValue,
	//		for benefit of FilteringSelect.
	displayedValue: "",

	_getDisplayedValueAttr: function(){
		// summary:
		//		Hook so get('displayedValue') works.
		// description:
		//		Returns the displayed value (what the user sees on the screen),
		//		after filtering (ie, trimming spaces etc.).
		//
		//		For some subclasses of TextBox (like ComboBox), the displayed value
		//		is different from the serialized value that's actually
		//		sent to the server (see `dijit/form/ValidationTextBox.serialize()`)

		// TODO: maybe we should update this.displayedValue on every keystroke so that we don't need
		// this method
		// TODO: this isn't really the displayed value when the user is typing
		return this.filter(this.textbox.value);
	},

	_setDisplayedValueAttr: function(/*String*/ value){
		// summary:
		//		Hook so set('displayedValue', ...) works.
		// description:
		//		Sets the value of the visual element to the string "value".
		//		The widget value is also set to a corresponding,
		//		but not necessarily the same, value.

		if(value == null /* or undefined */){ value = '' }
		else if(typeof value != "string"){ value = String(value) }

		this.textbox.value = value;

		// sets the serialized value to something corresponding to specified displayedValue
		// (if possible), and also updates the textbox.value, for example converting "123"
		// to "123.00"
		this._setValueAttr(this.get('value'), undefined);

		this._set("displayedValue", this.get('displayedValue'));

		// textDir support
		if(this.textDir == "auto"){
			this.applyTextDir(this.focusNode, value);
		}
	},

	format: function(value /*=====, constraints =====*/){
		// summary:
		//		Replaceable function to convert a value to a properly formatted string.
		// value: String
		// constraints: Object
		// tags:
		//		protected extension
		return value == null /* or undefined */ ? "" : (value.toString ? value.toString() : value);
	},

	parse: function(value /*=====, constraints =====*/){
		// summary:
		//		Replaceable function to convert a formatted string to a value
		// value: String
		// constraints: Object
		// tags:
		//		protected extension

		return value;	// String
	},

	_refreshState: function(){
		// summary:
		//		After the user types some characters, etc., this method is
		//		called to check the field for validity etc.  The base method
		//		in `dijit/form/TextBox` does nothing, but subclasses override.
		// tags:
		//		protected
	},

	/*=====
	onInput: function(event){
		// summary:
		//		Connect to this function to receive notifications of various user data-input events.
		//		Return false to cancel the event and prevent it from being processed.
		// event:
		//		keydown | keypress | cut | paste | input
		// tags:
		//		callback
	},
	=====*/
	onInput: function(){},

	__skipInputEvent: false,
	_onInput: function(/*Event*/ evt){
		// summary:
		//		Called AFTER the input event has happened

		// set text direction according to textDir that was defined in creation
		if(this.textDir == "auto"){
			this.applyTextDir(this.focusNode, this.focusNode.value);
		}

		this._processInput(evt);

		if(this.intermediateChanges){
			// allow the key to post to the widget input box
			this.defer(function(){ this._handleOnChange(this.get('value'), false); });
		}
	},

	_processInput: function(/*Event*/ evt){
		// summary:
		//		Default action handler for user input events

		this._refreshState();

		// In case someone is watch()'ing for changes to displayedValue
		this._set("displayedValue", this.get("displayedValue"));
	},

	postCreate: function(){
		// setting the value here is needed since value="" in the template causes "undefined"
		// and setting in the DOM (instead of the JS object) helps with form reset actions
		this.textbox.setAttribute("value", this.textbox.value); // DOM and JS values should be the same

		this.inherited(arguments);

		// normalize input events to reduce spurious event processing
		//	onkeydown: do not forward modifier keys
		//		       set charOrCode to numeric keycode
		//	onkeypress: do not forward numeric charOrCode keys (already sent through onkeydown)
		//	onpaste & oncut: set charOrCode to 229 (IME)
		//	oninput: if primary event not already processed, set charOrCode to 229 (IME), else do not forward
		var handleEvent = function(e){
			var charOrCode;
			if(e.type == "keydown"){
				charOrCode = e.keyCode;
				switch(charOrCode){ // ignore state keys
					case keys.SHIFT:
					case keys.ALT:
					case keys.CTRL:
					case keys.META:
					case keys.CAPS_LOCK:
					case keys.NUM_LOCK:
					case keys.SCROLL_LOCK:
						return;
				}
				if(!e.ctrlKey && !e.metaKey && !e.altKey){ // no modifiers
					switch(charOrCode){ // ignore location keys
						case keys.NUMPAD_0:
						case keys.NUMPAD_1:
						case keys.NUMPAD_2:
						case keys.NUMPAD_3:
						case keys.NUMPAD_4:
						case keys.NUMPAD_5:
						case keys.NUMPAD_6:
						case keys.NUMPAD_7:
						case keys.NUMPAD_8:
						case keys.NUMPAD_9:
						case keys.NUMPAD_MULTIPLY:
						case keys.NUMPAD_PLUS:
						case keys.NUMPAD_ENTER:
						case keys.NUMPAD_MINUS:
						case keys.NUMPAD_PERIOD:
						case keys.NUMPAD_DIVIDE:
							return;
					}
					if((charOrCode >= 65 && charOrCode <= 90) || (charOrCode >= 48 && charOrCode <= 57) || charOrCode == keys.SPACE){
						return; // keypress will handle simple non-modified printable keys
					}
					var named = false;
					for(var i in keys){
						if(keys[i] === e.keyCode){
							named = true;
							break;
						}
					}
					if(!named){ return; } // only allow named ones through
				}
			}
			charOrCode = e.charCode >= 32 ? String.fromCharCode(e.charCode) : e.charCode;
			if(!charOrCode){
				charOrCode = (e.keyCode >= 65 && e.keyCode <= 90) || (e.keyCode >= 48 && e.keyCode <= 57) || e.keyCode == keys.SPACE ? String.fromCharCode(e.keyCode) : e.keyCode;
			}
			if(!charOrCode){
				charOrCode = 229; // IME
			}
			if(e.type == "keypress"){
				if(typeof charOrCode != "string"){ return; }
				if((charOrCode >= 'a' && charOrCode <= 'z') || (charOrCode >= 'A' && charOrCode <= 'Z') || (charOrCode >= '0' && charOrCode <= '9') || (charOrCode === ' ')){
					if(e.ctrlKey || e.metaKey || e.altKey){ return; } // can only be stopped reliably in keydown
				}
			}
			if(e.type == "input"){
				if(this.__skipInputEvent){ // duplicate event
					this.__skipInputEvent = false;
					return;
				}
			}else{
				this.__skipInputEvent = true;
			}
			// create fake event to set charOrCode and to know if preventDefault() was called
			var faux = { faux: true }, attr;
			for(attr in e){
				if(attr != "layerX" && attr != "layerY"){ // prevent WebKit warnings
					var v = e[attr];
					if(typeof v != "function" && typeof v != "undefined"){ faux[attr] = v; }
				}
			}
			lang.mixin(faux, {
				charOrCode: charOrCode,
				_wasConsumed: false,
				preventDefault: function(){
					faux._wasConsumed = true;
					e.preventDefault();
				},
				stopPropagation: function(){ e.stopPropagation(); }
			});
			// give web page author a chance to consume the event
			//console.log(faux.type + ', charOrCode = (' + (typeof charOrCode) + ') ' + charOrCode + ', ctrl ' + !!faux.ctrlKey + ', alt ' + !!faux.altKey + ', meta ' + !!faux.metaKey + ', shift ' + !!faux.shiftKey);
			if(this.onInput(faux) === false){ // return false means stop
				faux.preventDefault();
				faux.stopPropagation();
			}
			if(faux._wasConsumed){ return; } // if preventDefault was called
			this.defer(function(){ this._onInput(faux); }); // widget notification after key has posted
		};
		this.own(on(this.textbox, "keydown, keypress, paste, cut, input, compositionend", lang.hitch(this, handleEvent)));
	},

	_blankValue: '', // if the textbox is blank, what value should be reported
	filter: function(val){
		// summary:
		//		Auto-corrections (such as trimming) that are applied to textbox
		//		value on blur or form submit.
		// description:
		//		For MappedTextBox subclasses, this is called twice
		//
		//		- once with the display value
		//		- once the value as set/returned by set('value', ...)
		//
		//		and get('value'), ex: a Number for NumberTextBox.
		//
		//		In the latter case it does corrections like converting null to NaN.  In
		//		the former case the NumberTextBox.filter() method calls this.inherited()
		//		to execute standard trimming code in TextBox.filter().
		//
		//		TODO: break this into two methods in 2.0
		//
		// tags:
		//		protected extension
		if(val === null){ return this._blankValue; }
		if(typeof val != "string"){ return val; }
		if(this.trim){
			val = lang.trim(val);
		}
		if(this.uppercase){
			val = val.toUpperCase();
		}
		if(this.lowercase){
			val = val.toLowerCase();
		}
		if(this.propercase){
			val = val.replace(/[^\s]+/g, function(word){
				return word.substring(0,1).toUpperCase() + word.substring(1);
			});
		}
		return val;
	},

	_setBlurValue: function(){
		this._setValueAttr(this.get('value'), true);
	},

	_onBlur: function(e){
		if(this.disabled){ return; }
		this._setBlurValue();
		this.inherited(arguments);
	},

	_isTextSelected: function(){
		return this.textbox.selectionStart != this.textbox.selectionEnd;
	},

	_onFocus: function(/*String*/ by){
		if(this.disabled || this.readOnly){ return; }

		// Select all text on focus via click if nothing already selected.
		// Since mouse-up will clear the selection, need to defer selection until after mouse-up.
		// Don't do anything on focus by tabbing into the widget since there's no associated mouse-up event.
		if(this.selectOnClick && by == "mouse"){
			this._selectOnClickHandle = this.connect(this.domNode, "onmouseup", function(){
				// Only select all text on first click; otherwise users would have no way to clear
				// the selection.
				this.disconnect(this._selectOnClickHandle);
				this._selectOnClickHandle = null;

				// Check if the user selected some text manually (mouse-down, mouse-move, mouse-up)
				// and if not, then select all the text
				if(!this._isTextSelected()){
					_TextBoxMixin.selectInputText(this.textbox);
				}
			});
			// in case the mouseup never comes
			this.defer(function(){ 
				if(this._selectOnClickHandle){
					this.disconnect(this._selectOnClickHandle);
					this._selectOnClickHandle = null;
				}
			}, 500); // if mouseup not received soon, then treat it as some gesture
		}
		// call this.inherited() before refreshState(), since this.inherited() will possibly scroll the viewport
		// (to scroll the TextBox into view), which will affect how _refreshState() positions the tooltip
		this.inherited(arguments);

		this._refreshState();
	},

	reset: function(){
		// Overrides `dijit/_FormWidget/reset()`.
		// Additionally resets the displayed textbox value to ''
		this.textbox.value = '';
		this.inherited(arguments);
	},

	_setTextDirAttr: function(/*String*/ textDir){
		// summary:
		//		Setter for textDir.
		// description:
		//		Users shouldn't call this function; they should be calling
		//		set('textDir', value)
		// tags:
		//		private

		// only if new textDir is different from the old one
		// and on widgets creation.
		if(!this._created
			|| this.textDir != textDir){
				this._set("textDir", textDir);
				// so the change of the textDir will take place immediately.
				this.applyTextDir(this.focusNode, this.focusNode.value);
		}
	}
});


_TextBoxMixin._setSelectionRange = dijit._setSelectionRange = function(/*DomNode*/ element, /*Number?*/ start, /*Number?*/ stop){
	if(element.setSelectionRange){
		element.setSelectionRange(start, stop);
	}
};

_TextBoxMixin.selectInputText = dijit.selectInputText = function(/*DomNode*/ element, /*Number?*/ start, /*Number?*/ stop){
	// summary:
	//		Select text in the input element argument, from start (default 0), to stop (default end).

	// TODO: use functions in _editor/selection.js?
	element = dom.byId(element);
	if(isNaN(start)){ start = 0; }
	if(isNaN(stop)){ stop = element.value ? element.value.length : 0; }
	try{
		element.focus();
		_TextBoxMixin._setSelectionRange(element, start, stop);
	}catch(e){ /* squelch random errors (esp. on IE) from unexpected focus changes or DOM nodes being hidden */ }
};

return _TextBoxMixin;
});

},
'url:dijit/form/templates/TextBox.html':"<div class=\"dijit dijitReset dijitInline dijitLeft\" id=\"widget_${id}\" role=\"presentation\"\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\n\t\t><input class=\"dijitReset dijitInputInner\" data-dojo-attach-point='textbox,focusNode' autocomplete=\"off\"\n\t\t\t${!nameAttrSetting} type='${type}'\n\t/></div\n></div>\n",
'dijit/Tooltip':function(){
require({cache:{
'url:dijit/templates/Tooltip.html':"<div class=\"dijitTooltip dijitTooltipLeft\" id=\"dojoTooltip\"\n\t><div class=\"dijitTooltipContainer dijitTooltipContents\" data-dojo-attach-point=\"containerNode\" role='alert'></div\n\t><div class=\"dijitTooltipConnector\" data-dojo-attach-point=\"connectorNode\"></div\n></div>\n"}});
define("dijit/Tooltip", [
	"dojo/_base/array", // array.forEach array.indexOf array.map
	"dojo/_base/declare", // declare
	"dojo/_base/fx", // fx.fadeIn fx.fadeOut
	"dojo/dom", // dom.byId
	"dojo/dom-class", // domClass.add
	"dojo/dom-geometry", // domGeometry.position
	"dojo/dom-style", // domStyle.set, domStyle.get
	"dojo/_base/lang", // lang.hitch lang.isArrayLike
	"dojo/mouse",
	"dojo/on",
	"dojo/sniff", // has("ie"), has("trident")
	"./_base/manager",	// manager.defaultDuration
	"./place",
	"./_Widget",
	"./_TemplatedMixin",
	"./BackgroundIframe",
	"dojo/text!./templates/Tooltip.html",
	"./main"		// sets dijit.showTooltip etc. for back-compat
], function(array, declare, fx, dom, domClass, domGeometry, domStyle, lang, mouse, on, has,
			manager, place, _Widget, _TemplatedMixin, BackgroundIframe, template, dijit){

	// module:
	//		dijit/Tooltip


	// TODO: Tooltip should really share more positioning code with TooltipDialog, like:
	//		- the orient() method
	//		- the connector positioning code in show()
	//		- the dijitTooltip[Dialog] class
	//
	// The problem is that Tooltip's implementation supplies it's own <iframe> and interacts directly
	// with dijit/place, rather than going through dijit/popup like TooltipDialog and other popups (ex: Menu).

	var MasterTooltip = declare("dijit._MasterTooltip", [_Widget, _TemplatedMixin], {
		// summary:
		//		Internal widget that holds the actual tooltip markup,
		//		which occurs once per page.
		//		Called by Tooltip widgets which are just containers to hold
		//		the markup
		// tags:
		//		protected

		// duration: Integer
		//		Milliseconds to fade in/fade out
		duration: manager.defaultDuration,

		templateString: template,

		postCreate: function(){
			this.ownerDocumentBody.appendChild(this.domNode);

			this.bgIframe = new BackgroundIframe(this.domNode);

			// Setup fade-in and fade-out functions.
			this.fadeIn = fx.fadeIn({ node: this.domNode, duration: this.duration, onEnd: lang.hitch(this, "_onShow") });
			this.fadeOut = fx.fadeOut({ node: this.domNode, duration: this.duration, onEnd: lang.hitch(this, "_onHide") });
		},

		show: function(innerHTML, aroundNode, position, rtl, textDir){
			// summary:
			//		Display tooltip w/specified contents to right of specified node
			//		(To left if there's no space on the right, or if rtl == true)
			// innerHTML: String
			//		Contents of the tooltip
			// aroundNode: DomNode|dijit/place.__Rectangle
			//		Specifies that tooltip should be next to this node / area
			// position: String[]?
			//		List of positions to try to position tooltip (ex: ["right", "above"])
			// rtl: Boolean?
			//		Corresponds to `WidgetBase.dir` attribute, where false means "ltr" and true
			//		means "rtl"; specifies GUI direction, not text direction.
			// textDir: String?
			//		Corresponds to `WidgetBase.textdir` attribute; specifies direction of text.


			if(this.aroundNode && this.aroundNode === aroundNode && this.containerNode.innerHTML == innerHTML){
				return;
			}

			if(this.fadeOut.status() == "playing"){
				// previous tooltip is being hidden; wait until the hide completes then show new one
				this._onDeck=arguments;
				return;
			}
			this.containerNode.innerHTML=innerHTML;

			if(textDir){
				this.set("textDir", textDir);
			}

			this.containerNode.align = rtl? "right" : "left"; //fix the text alignment

			var pos = place.around(this.domNode, aroundNode,
				position && position.length ? position : Tooltip.defaultPosition, !rtl, lang.hitch(this, "orient"));

			// Position the tooltip connector for middle alignment.
			// This could not have been done in orient() since the tooltip wasn't positioned at that time.
			var aroundNodeCoords = pos.aroundNodePos;
			if(pos.corner.charAt(0) == 'M' && pos.aroundCorner.charAt(0) == 'M'){
				this.connectorNode.style.top = aroundNodeCoords.y + ((aroundNodeCoords.h - this.connectorNode.offsetHeight) >> 1) - pos.y + "px";
				this.connectorNode.style.left = "";
			}else if(pos.corner.charAt(1) == 'M' && pos.aroundCorner.charAt(1) == 'M'){
				this.connectorNode.style.left = aroundNodeCoords.x + ((aroundNodeCoords.w - this.connectorNode.offsetWidth) >> 1) - pos.x + "px";
			}else{
				// Not *-centered, but just above/below/after/before
				this.connectorNode.style.left = "";
				this.connectorNode.style.top = "";
			}

			// show it
			domStyle.set(this.domNode, "opacity", 0);
			this.fadeIn.play();
			this.isShowingNow = true;
			this.aroundNode = aroundNode;
		},

		orient: function(/*DomNode*/ node, /*String*/ aroundCorner, /*String*/ tooltipCorner, /*Object*/ spaceAvailable, /*Object*/ aroundNodeCoords){
			// summary:
			//		Private function to set CSS for tooltip node based on which position it's in.
			//		This is called by the dijit popup code.   It will also reduce the tooltip's
			//		width to whatever width is available
			// tags:
			//		protected

			this.connectorNode.style.top = ""; //reset to default

			var heightAvailable = spaceAvailable.h,
				widthAvailable = spaceAvailable.w;

			node.className = "dijitTooltip " +
				{
					"MR-ML": "dijitTooltipRight",
					"ML-MR": "dijitTooltipLeft",
					"TM-BM": "dijitTooltipAbove",
					"BM-TM": "dijitTooltipBelow",
					"BL-TL": "dijitTooltipBelow dijitTooltipABLeft",
					"TL-BL": "dijitTooltipAbove dijitTooltipABLeft",
					"BR-TR": "dijitTooltipBelow dijitTooltipABRight",
					"TR-BR": "dijitTooltipAbove dijitTooltipABRight",
					"BR-BL": "dijitTooltipRight",
					"BL-BR": "dijitTooltipLeft"
				}[aroundCorner + "-" + tooltipCorner];

			// reset width; it may have been set by orient() on a previous tooltip show()
			this.domNode.style.width = "auto";

			// Reduce tooltip's width to the amount of width available, so that it doesn't overflow screen.
			// Note that sometimes widthAvailable is negative, but we guard against setting style.width to a
			// negative number since that causes an exception on IE.
			var size = domGeometry.position(this.domNode);
			if(has("ie") || has("trident")){
				// workaround strange IE bug where setting width to offsetWidth causes words to wrap
				size.w += 2;
			}

			var width = Math.min((Math.max(widthAvailable,1)), size.w);

			domGeometry.setMarginBox(this.domNode, {w: width});

			// Reposition the tooltip connector.
			if(tooltipCorner.charAt(0) == 'B' && aroundCorner.charAt(0) == 'B'){
				var bb = domGeometry.position(node);
				var tooltipConnectorHeight = this.connectorNode.offsetHeight;
				if(bb.h > heightAvailable){
					// The tooltip starts at the top of the page and will extend past the aroundNode
					var aroundNodePlacement = heightAvailable - ((aroundNodeCoords.h + tooltipConnectorHeight) >> 1);
					this.connectorNode.style.top = aroundNodePlacement + "px";
					this.connectorNode.style.bottom = "";
				}else{
					// Align center of connector with center of aroundNode, except don't let bottom
					// of connector extend below bottom of tooltip content, or top of connector
					// extend past top of tooltip content
					this.connectorNode.style.bottom = Math.min(
						Math.max(aroundNodeCoords.h/2 - tooltipConnectorHeight/2, 0),
						bb.h - tooltipConnectorHeight) + "px";
					this.connectorNode.style.top = "";
				}
			}else{
				// reset the tooltip back to the defaults
				this.connectorNode.style.top = "";
				this.connectorNode.style.bottom = "";
			}

			return Math.max(0, size.w - widthAvailable);
		},

		_onShow: function(){
			// summary:
			//		Called at end of fade-in operation
			// tags:
			//		protected
			if(has("ie")){
				// the arrow won't show up on a node w/an opacity filter
				this.domNode.style.filter="";
			}
		},

		hide: function(aroundNode){
			// summary:
			//		Hide the tooltip

			if(this._onDeck && this._onDeck[1] == aroundNode){
				// this hide request is for a show() that hasn't even started yet;
				// just cancel the pending show()
				this._onDeck=null;
			}else if(this.aroundNode === aroundNode){
				// this hide request is for the currently displayed tooltip
				this.fadeIn.stop();
				this.isShowingNow = false;
				this.aroundNode = null;
				this.fadeOut.play();
			}else{
				// just ignore the call, it's for a tooltip that has already been erased
			}
		},

		_onHide: function(){
			// summary:
			//		Called at end of fade-out operation
			// tags:
			//		protected

			this.domNode.style.cssText="";	// to position offscreen again
			this.containerNode.innerHTML="";
			if(this._onDeck){
				// a show request has been queued up; do it now
				this.show.apply(this, this._onDeck);
				this._onDeck=null;
			}
		},

		_setAutoTextDir: function(/*Object*/node){
			// summary:
			//		Resolve "auto" text direction for children nodes
			// tags:
			//		private

			this.applyTextDir(node, has("ie") ? node.outerText : node.textContent);
			array.forEach(node.children, function(child){this._setAutoTextDir(child); }, this);
		},

		_setTextDirAttr: function(/*String*/ textDir){
			// summary:
			//		Setter for textDir.
			// description:
			//		Users shouldn't call this function; they should be calling
			//		set('textDir', value)
			// tags:
			//		private

			this._set("textDir", textDir);

			if (textDir == "auto"){
				this._setAutoTextDir(this.containerNode);
			}else{
				this.containerNode.dir = this.textDir;
			}
		}
	});

	dijit.showTooltip = function(innerHTML, aroundNode, position, rtl, textDir){
		// summary:
		//		Static method to display tooltip w/specified contents in specified position.
		//		See description of dijit/Tooltip.defaultPosition for details on position parameter.
		//		If position is not specified then dijit/Tooltip.defaultPosition is used.
		// innerHTML: String
		//		Contents of the tooltip
		// aroundNode: place.__Rectangle
		//		Specifies that tooltip should be next to this node / area
		// position: String[]?
		//		List of positions to try to position tooltip (ex: ["right", "above"])
		// rtl: Boolean?
		//		Corresponds to `WidgetBase.dir` attribute, where false means "ltr" and true
		//		means "rtl"; specifies GUI direction, not text direction.
		// textDir: String?
		//		Corresponds to `WidgetBase.textdir` attribute; specifies direction of text.

		// After/before don't work, but for back-compat convert them to the working after-centered, before-centered.
		// Possibly remove this in 2.0.   Alternately, get before/after to work.
		if(position){
			position = array.map(position, function(val){
				return {after: "after-centered", before: "before-centered"}[val] || val;
			});
		}

		if(!Tooltip._masterTT){ dijit._masterTT = Tooltip._masterTT = new MasterTooltip(); }
		return Tooltip._masterTT.show(innerHTML, aroundNode, position, rtl, textDir);
	};

	dijit.hideTooltip = function(aroundNode){
		// summary:
		//		Static method to hide the tooltip displayed via showTooltip()
		return Tooltip._masterTT && Tooltip._masterTT.hide(aroundNode);
	};

	var Tooltip = declare("dijit.Tooltip", _Widget, {
		// summary:
		//		Pops up a tooltip (a help message) when you hover over a node.
		//		Also provides static show() and hide() methods that can be used without instantiating a dijit/Tooltip.

		// label: String
		//		Text to display in the tooltip.
		//		Specified as innerHTML when creating the widget from markup.
		label: "",

		// showDelay: Integer
		//		Number of milliseconds to wait after hovering over/focusing on the object, before
		//		the tooltip is displayed.
		showDelay: 400,

		// connectId: String|String[]|DomNode|DomNode[]
		//		Id of domNode(s) to attach the tooltip to.
		//		When user hovers over specified dom node(s), the tooltip will appear.
		connectId: [],

		// position: String[]
		//		See description of `dijit/Tooltip.defaultPosition` for details on position parameter.
		position: [],

		// selector: String?
		//		CSS expression to apply this Tooltip to descendants of connectIds, rather than to
		//		the nodes specified by connectIds themselves.    Useful for applying a Tooltip to
		//		a range of rows in a table, tree, etc.   Use in conjunction with getContent() parameter.
		//		Ex: connectId: myTable, selector: "tr", getContent: function(node){ return ...; }
		//
		//		The application must require() an appropriate level of dojo/query to handle the selector.
		selector: "",

		// TODO: in 2.0 remove support for multiple connectIds.   selector gives the same effect.
		// So, change connectId to a "", remove addTarget()/removeTarget(), etc.

		_setConnectIdAttr: function(/*String|String[]}DomNode|DomNode[]*/ newId){
			// summary:
			//		Connect to specified node(s)

			// Remove connections to old nodes (if there are any)
			array.forEach(this._connections || [], function(nested){
				array.forEach(nested, function(handle){ handle.remove(); });
			}, this);

			// Make array of id's to connect to, excluding entries for nodes that don't exist yet, see startup()
			this._connectIds = array.filter(lang.isArrayLike(newId) ? newId : (newId ? [newId] : []),
					function(id){ return dom.byId(id, this.ownerDocument); }, this);

			// Make connections
			this._connections = array.map(this._connectIds, function(id){
				var node = dom.byId(id, this.ownerDocument),
					selector = this.selector,
					delegatedEvent = selector ?
						function(eventType){ return on.selector(selector, eventType); } :
						function(eventType){ return eventType; },
					self = this;
				return [
					on(node, delegatedEvent(mouse.enter), function(){
						self._onHover(this);
					}),
					on(node, delegatedEvent("focusin"), function(){
						self._onHover(this);
					}),
					on(node, delegatedEvent(mouse.leave), lang.hitch(self, "_onUnHover")),
					on(node, delegatedEvent("focusout"), lang.hitch(self, "_onUnHover"))
				];
			}, this);

			this._set("connectId", newId);
		},

		addTarget: function(/*OomNode|String*/ node){
			// summary:
			//		Attach tooltip to specified node if it's not already connected

			// TODO: remove in 2.0 and just use set("connectId", ...) interface

			var id = node.id || node;
			if(array.indexOf(this._connectIds, id) == -1){
				this.set("connectId", this._connectIds.concat(id));
			}
		},

		removeTarget: function(/*DomNode|String*/ node){
			// summary:
			//		Detach tooltip from specified node

			// TODO: remove in 2.0 and just use set("connectId", ...) interface

			var id = node.id || node,	// map from DOMNode back to plain id string
				idx = array.indexOf(this._connectIds, id);
			if(idx >= 0){
				// remove id (modifies original this._connectIds but that's OK in this case)
				this._connectIds.splice(idx, 1);
				this.set("connectId", this._connectIds);
			}
		},

		buildRendering: function(){
			this.inherited(arguments);
			domClass.add(this.domNode,"dijitTooltipData");
		},

		startup: function(){
			this.inherited(arguments);

			// If this tooltip was created in a template, or for some other reason the specified connectId[s]
			// didn't exist during the widget's initialization, then connect now.
			var ids = this.connectId;
			array.forEach(lang.isArrayLike(ids) ? ids : [ids], this.addTarget, this);
		},

		getContent: function(/*DomNode*/ node){
			// summary:
			//		User overridable function that return the text to display in the tooltip.
			// tags:
			//		extension
			return this.label || this.domNode.innerHTML;
		},

		_onHover: function(/*DomNode*/ target){
			// summary:
			//		Despite the name of this method, it actually handles both hover and focus
			//		events on the target node, setting a timer to show the tooltip.
			// tags:
			//		private
			if(!this._showTimer){
				this._showTimer = this.defer(function(){ this.open(target); }, this.showDelay);
			}
		},

		_onUnHover: function(){
			// summary:
			//		Despite the name of this method, it actually handles both mouseleave and blur
			//		events on the target node, hiding the tooltip.
			// tags:
			//		private

			if(this._showTimer){
				this._showTimer.remove();
				delete this._showTimer;
			}
			this.close();
		},

		open: function(/*DomNode*/ target){
			// summary:
			//		Display the tooltip; usually not called directly.
			// tags:
			//		private

			if(this._showTimer){
				this._showTimer.remove();
				delete this._showTimer;
			}

			var content = this.getContent(target);
			if(!content){
				return;
			}
			Tooltip.show(content, target, this.position, !this.isLeftToRight(), this.textDir);

			this._connectNode = target;		// _connectNode means "tooltip currently displayed for this node"
			this.onShow(target, this.position);
		},

		close: function(){
			// summary:
			//		Hide the tooltip or cancel timer for show of tooltip
			// tags:
			//		private

			if(this._connectNode){
				// if tooltip is currently shown
				Tooltip.hide(this._connectNode);
				delete this._connectNode;
				this.onHide();
			}
			if(this._showTimer){
				// if tooltip is scheduled to be shown (after a brief delay)
				this._showTimer.remove();
				delete this._showTimer;
			}
		},

		onShow: function(/*===== target, position =====*/){
			// summary:
			//		Called when the tooltip is shown
			// tags:
			//		callback
		},

		onHide: function(){
			// summary:
			//		Called when the tooltip is hidden
			// tags:
			//		callback
		},

		destroy: function(){
			this.close();

			// Remove connections manually since they aren't registered to be removed by _WidgetBase
			array.forEach(this._connections || [], function(nested){
				array.forEach(nested, function(handle){ handle.remove(); });
			}, this);

			this.inherited(arguments);
		}
	});

	Tooltip._MasterTooltip = MasterTooltip;		// for monkey patching
	Tooltip.show = dijit.showTooltip;		// export function through module return value
	Tooltip.hide = dijit.hideTooltip;		// export function through module return value

	Tooltip.defaultPosition = ["after-centered", "before-centered"];

	/*=====
	lang.mixin(Tooltip, {
		 // defaultPosition: String[]
		 //		This variable controls the position of tooltips, if the position is not specified to
		 //		the Tooltip widget or *TextBox widget itself.  It's an array of strings with the values
		 //		possible for `dijit/place.around()`.   The recommended values are:
		 //
		 //		- before-centered: centers tooltip to the left of the anchor node/widget, or to the right
		 //		  in the case of RTL scripts like Hebrew and Arabic
		 //		- after-centered: centers tooltip to the right of the anchor node/widget, or to the left
		 //		  in the case of RTL scripts like Hebrew and Arabic
		 //		- above-centered: tooltip is centered above anchor node
		 //		- below-centered: tooltip is centered above anchor node
		 //
		 //		The list is positions is tried, in order, until a position is found where the tooltip fits
		 //		within the viewport.
		 //
		 //		Be careful setting this parameter.  A value of "above-centered" may work fine until the user scrolls
		 //		the screen so that there's no room above the target node.   Nodes with drop downs, like
		 //		DropDownButton or FilteringSelect, are especially problematic, in that you need to be sure
		 //		that the drop down and tooltip don't overlap, even when the viewport is scrolled so that there
		 //		is only room below (or above) the target node, but not both.
	 });
	=====*/
	return Tooltip;
});

},
'url:dijit/templates/Tooltip.html':"<div class=\"dijitTooltip dijitTooltipLeft\" id=\"dojoTooltip\"\n\t><div class=\"dijitTooltipContainer dijitTooltipContents\" data-dojo-attach-point=\"containerNode\" role='alert'></div\n\t><div class=\"dijitTooltipConnector\" data-dojo-attach-point=\"connectorNode\"></div\n></div>\n",
'url:dijit/form/templates/ValidationTextBox.html':"<div class=\"dijit dijitReset dijitInline dijitLeft\"\n\tid=\"widget_${id}\" role=\"presentation\"\n\t><div class='dijitReset dijitValidationContainer'\n\t\t><input class=\"dijitReset dijitInputField dijitValidationIcon dijitValidationInner\" value=\"&#935; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t/></div\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\n\t\t><input class=\"dijitReset dijitInputInner\" data-dojo-attach-point='textbox,focusNode' autocomplete=\"off\"\n\t\t\t${!nameAttrSetting} type='${type}'\n\t/></div\n></div>\n",
'epi-saas-base/RetryableXhrWrapper':function(){
define("epi-saas-base/RetryableXhrWrapper", [
    "dojo/_base/declare",
    "epi/shell/XhrWrapper",
    "./_RetryableXhrWrapperMixin"
],
function(declare, XhrWrapper, _RetryableXhrWrapperMixin) {
    return declare([XhrWrapper, _RetryableXhrWrapperMixin]);
});
},
'epi-saas-base/_RetryableXhrWrapperMixin':function(){
﻿define("epi-saas-base/_RetryableXhrWrapperMixin", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/Deferred",
    "dojo/request/handlers",
    "dojo/when",
    "dojo/json",

    "./widgets/RetryConfirmation"
],
function (declare, lang, array, Deferred, handlers, when, json,
    RetryConfirmation) {

    return declare(null, {
        // summary: 
        //      Hooks to XHR calls and allows retrying when original request is failed.
        
        // enabled: Boolean
        //      Enables or disables retrying functionality
        enabled: true,

        // retryConfirmation: Object?
        //      An object that has method confirm() and this method returns a promise that indicates whether original call should be retried.
        retryConfirmation: null,

        // statusCodesToRetry: Array
        //      The list of status codes when the last failed call can be retried.
        statusCodesToRetry: null,

        // maxNumberOfRetries: Number
        //      The maximum number of retries when request is failed. Default value is 50. 
        //      Value less than 1 means that retrying is disabled.
        maxNumberOfRetries: 50,

        // _numberOfRetries: Number
        //      Current number of retries of failed requests. It is used to avoid loops when retrying.
        //      This value is shared for all requests and retries and 
        //      it is reset to 0 as soon as any request was resolved or failed after maximum number of retries.
        _numberOfRetries: 0,

        authToken: null,

        _readValidationToken: function () {
            //	summary:
            //		Reads server generated antiforgery token

            if (this.sharedXsrfHeader) {
                return this.sharedXsrfHeader;
            }

            this.sharedXsrfHeader = document.getElementsByName("__RequestVerificationToken")[0].value;
            return this.sharedXsrfHeader;
        },
        
        constructor: function (options) {
            // summary: 
            //      Called when new instance is created. Creates default retry confirmation.

            this.preventLocalizationHeader = true;

            declare.safeMixin(this, options);

            if (!this.retryConfirmation) {
                this.retryConfirmation = new RetryConfirmation();
            }

            if (!this.statusCodesToRetry) {
                // Default status codes when retries should be enabled
                this.statusCodesToRetry = [
                    "404", // Not Found, usually when there is no connection to the REST API
                    "408", // Request Timeout
                    "429", // Too Many Requests (draft)
                    "444", // No Response (Nginx)
                    "449", // Retry With (Microsoft)
                    "499", // Client Closed Request (Nginx)
                    "502", // Bad Gateway
                    "503", // Service Unavailable                    
                    "504", // Gateway Timeout
                    "509", // Bandwidth Limit Exceeded (Apache)
                    "511", // Network Authentication Required (draft)
                    "598", // Network read timeout error
                    "599"  // Network connect timeout error
                ];
            } else {
                if (!(this.statusCodesToRetry instanceof Array)) {
                    this.statusCodesToRetry = [this.statusCodesToRetry];
                }
            }
        },

        xhr: function (/*String*/method, /*dojo.__XhrArgs*/args, /*Boolean?*/hasBody) {
            // summary: 
            //      Overriden to enable retries for failed calls.
            var result;
            args.xsrfProtection = false;
            args.headers = args.headers || {};
            //try to get antiforgery header name
            var _xsrHeaderElement = document.querySelectorAll("[data-epi-antiforgery-header-name]");
            var _xsrHeaderName = _xsrHeaderElement && _xsrHeaderElement.length > 0 ? _xsrHeaderElement[0].getAttribute("data-epi-antiforgery-header-name") : "RequestVerificationToken";
            _xsrHeaderName = _xsrHeaderName || "RequestVerificationToken";
            args.headers[_xsrHeaderName] = this._readValidationToken();
            
            if (this.authToken) {
                if (this.epiAuthHeader) {
                    args.headers['EpiFindAuthorization'] = "Bearer " + this.authToken;
                }
                else {
                    args.headers.Authorization = "Bearer " + this.authToken;
                }
                result = this.inherited(arguments, [method, args, hasBody]);
            }
            else {
                result = this.inherited(arguments);
            }
            return this._handle(result, arguments);
        },

        _handle: function (result, xhrArguments) {
            // summary: 
            //     Wraps and handles the result of the original call.
            
            if (!this.enabled) {
                return result;
            }
            var deferred = new Deferred();
            deferred.ioArgs = result.ioArgs;

            when(result,
                lang.hitch(this, function (data) {
                    // Return results of successful call
                    this._resetNumberOfRetries();
                    deferred.ioArgs = result.ioArgs;
                    deferred.resolve(data);
            }),
                lang.hitch(this, function (error) {
                    // Handling failure, possibly retrying
                    var statusCode = error.status || result.ioArgs.xhr.status;
                    this._handleErrorData(error);
                    // Retry only when it makes sense, for example if server was not available
                    if (this._isRetryableError(error, statusCode)) {
                        this._handleFailure(error, xhrArguments, deferred);
                    } else {
                        this._resetNumberOfRetries();
                        deferred.reject(error);
                    }
            }),
                deferred.progress);

            return deferred;
        },

        _handleFailure: function (error, xhrArguments, deferred) {
            // summary: 
            //      Handles failed call result and enables retries.

            // Correct and increase current number of retries
            if (typeof this._numberOfRetries != "number" || !this._numberOfRetries) {
                this._resetNumberOfRetries();
            }
            this._numberOfRetries++;

            // Don't retry if reached the maximum number of retries or 
            // when retrying is disabled 
            // or retrying cannot be confirmed
            if (this._numberOfRetries > this.maxNumberOfRetries ||
                !this.retryConfirmation ||
                !this.retryConfirmation.confirm ||
                typeof this.retryConfirmation.confirm != "function") {
                this._resetNumberOfRetries();
                deferred.reject(error);
                return;
            }

            // Ask confirmation to retry the failed call
            var retryConfirmResult = this.retryConfirmation.confirm();

            // Check if confirmation result is already resolved to warn about possible loop.
            if (retryConfirmResult && retryConfirmResult.isResolved &&
                typeof retryConfirmResult.isResolved == "function") {
                try {
                    if (retryConfirmResult.isResolved()) {
                        console.warn("Deferred result of retry confirmation is already resolved. Having resolved confirmation and failed retry on each attempt can cause a loop until number of retries is bigger than maxNumberOfRetries.");
                    }
                } catch(e) {
                }
            }

            when(retryConfirmResult, lang.hitch(this, function () {
                // Retrying the original call
                var retryResult = this.xhr.apply(this, xhrArguments);
                when(retryResult, deferred.resolve, deferred.reject, deferred.progress);
            }), lang.hitch(this, function () {
                // Don't retry
                this._resetNumberOfRetries();
                deferred.reject(error);
            }));
        },
        
        _resetNumberOfRetries: function () {
            this._numberOfRetries = 0;
        },
        
        _handleErrorData: function (error) {
            var data = error.response ? error.response.data : undefined;
            var responseText = error.response ? error.response.text : undefined;

            // Data is already parsed or there is no text to parse
            if ((data && typeof data == "object") || !responseText) {
                return;
            }

            // Try to parse response data if it is not an object
            // Detect data handler options for response and request
            var originalResponseOptions = error.response.options;
            var responseOptions = originalResponseOptions || {};
            var responseHandleAs = responseOptions.handleAs;
            var requestHandleAs = error.request && error.request.options ? error.request.options.handleAs : null;
            var defaultHandleAs = "json";
            // parse data using handler that was specified for request or using json handler as default
            if (!responseHandleAs || responseHandleAs != requestHandleAs) {
                responseOptions.handleAs = requestHandleAs || defaultHandleAs;
                error.response.options = responseOptions;
            }
            try {
                // parse response data
                handlers(error.response);
            } catch (e) { }

            // restore original values if data is not parsed
            if (error.response.data === data || 
                error.response.data === responseText) {
                if (originalResponseOptions) {
                    originalResponseOptions.handleAs = responseHandleAs;
                }
                error.response.options = originalResponseOptions;
                error.response.data = data;
            }
        },
      
        _isRetryableError: function (error, statusCode) {
            // Check that response status codes is defined as retryable
            if (!array.some(this.statusCodesToRetry, function(item) {
                return statusCode == item;
            })) {
                return false;
            }

            // Check that 404 error is not business logic error
            if (statusCode == "404") {
                // Response contains JSON data if it is business logic error.
                // In this case there is no point in executing identical retry, probably data should be fixed.
                if (error.response && error.response.data && typeof error.response.data == "object") {
                    return false;
                }
            }

            return true;
        }
        

    });
});

},
'epi-saas-base/widgets/RetryConfirmation':function(){
﻿define("epi-saas-base/widgets/RetryConfirmation", [
    "dojo/_base/declare",
    "dojo/_base/Deferred",
    "epi/shell/widget/dialog/Confirmation",
    "dojo/i18n!./nls/RetryConfirmation"
],
function (declare, Deferred,
    Confirmation, i18n) {

    return declare(null, {
        // summary: 
        //      Confirmation to retry failed operation.

        i18n: i18n,
        
        confirm: function () {
            // summary: 
            //      Shows confirmation dialog to notify user about failure and suggestion to retry.
            
            var deferred = new Deferred();
            
            var dialog = new Confirmation({
                title: this.i18n.retryConfirmation_title,
                description: this.i18n.retryConfirmation_description,
                confirmActionText: this.i18n.retryConfirmation_okTitle,
                cancelActionText: this.i18n.retryConfirmation_cancelTitle,
                onAction: function (retry) {
                    if (retry) {
                        deferred.resolve();
                    } else {
                        deferred.reject();
                    }
                },
                destroyOnHide: true
            });
            dialog.show();

            return deferred;
        }
    });
});

},
'epi/shell/widget/dialog/Confirmation':function(){
﻿define("epi/shell/widget/dialog/Confirmation", [
// Dojo
    "dojo/_base/declare",
    "dojo/_base/lang",

    // EPi
    "epi",
    "epi/shell/widget/dialog/Dialog"
], function (
// Dojo
    declare,
    lang,

    // EPi
    epi,
    Dialog
) {

    return declare([Dialog], {
        // summary:
        //		A modal confirmation dialog widget.
        //
        // tags:
        //      public

        // confirmActionText: [public] String
        //		Label to be displayed for the confirm (positive) action.
        confirmActionText: epi.resources.action.yes,

        // cancelActionText: [public] String
        //		Label to be displayed for the cancel (negative) action.
        cancelActionText: epi.resources.action.no,

        // closeIconVisible: [public] Boolean
        //		Flag which indicates whether the close icon should be visible.
        //		Users are requried to answer the confirmation so don't allow
        //		closing via the close icon.
        closeIconVisible: false,

        // dialogClass: [protected] String
        //		Class to apply to the root DOMNode of the dialog.
        dialogClass: "epi-dialog-confirm",

        getActions: function () {
            // summary:
            //      Overridden from Dialog base to assemble the action collection
            // returns:
            //		A collection of action definitions that can be added to the action pane.
            // tags:
            //      protected

            var confirmButton = {
                    name: this._okButtonName,
                    label: this.confirmActionText,
                    title: null,
                    action: lang.hitch(this, this._onConfirm)
                },
                cancelButton = {
                    name: this._cancelButtonName,
                    label: this.cancelActionText,
                    title: null,
                    action: lang.hitch(this, this._onCancel)
                };

            if (this.setFocusOnConfirmButton) {
                confirmButton.settings = { "class": "Salt" };
            }

            return [confirmButton, cancelButton];
        },

        _onConfirm: function () {
            // summary:
            //		Called when user has triggered the dialog's confirm action.
            // type:
            //		callback

            this.hide();
            this.onAction(true);
        },

        _onCancel: function () {
            // summary:
            //		Called when user has triggered the dialog's cancel action.
            // type:
            //		callback

            this.hide();
            this.onAction(false);
        },

        onAction: function (/*Boolean*/ /*===== confirm =====*/) {
            // summary:
            //		Called when the user has triggered an action on the dialog.
            // type:
            //		public callback
        }
    });
});

},
'epi-find/store/ResourceStore':function(){
﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/Deferred",
    "dojo/store/util/QueryResults",
    "epi/shell/store/JsonRest",
    "epi-saas-base/RetryableXhrWrapper"
],
function(declare,
         lang,
         Deferred,
         QueryResults,
         JsonRest,
         RetryableXhrWrapper) {
    return declare([JsonRest], {
        // listProperty: string
        //      The property to extract the list of items from.
        //      Treats the entire result as the results list if not set.
        listProperty: null,

        // itemProperty: string
        //      The property to extract the item.
        //      Treats the entire result as the item if not set.
        itemProperty: null,

        // returnUpdatedItemFromPut: Boolean
        //    Indicates that put method should return an updated object from the store
        //    by issuing a separate GET request
        returnUpdatedItemFromPut: false,

        // preventCache: Boolean
        //    Prevents browser caching by adding a parameter to requests URL that changes with every new request.
        //    The parameter addition is handled by the base store implementation, here we just overriding the default value.
        preventCache: true,

        constructor: function(options) {
            this.xhrHandler = new RetryableXhrWrapper(options ? options.handlerOptions : null);
            declare.safeMixin(this, options);
        },

        _getList: function(data, options) {
            //  summary:
            //      Extracts a items from a property specified in the listProperty
            //  tags: protected
            if (this.listProperty) {
                return data[this.listProperty];
            } else {
                return data;
            }
        },

        _getTotal: function(result) {
            //  summary:
            //      Returns a total amount of items
            //  tags: protected
            var self = this;

            return result.then(function(data) {
                if (data.total) {
                    return data.total;
                }
                var result = self._getList(data, null);
                if (result && result instanceof Array) {
                    return result.length;
                }
            });
        },

        query: function(query, options){
            var self = this, queryResult = new Deferred(), newQuery = lang.clone(query || {}),
                newOptions = lang.clone(options || {}), result;

            if (newOptions.count) {
                newQuery.size = newOptions.count;
            }
            if (newOptions.start) {
                newQuery.from = newOptions.start;
            }

            if(query && query.tags && query.tags.join) {
                newQuery.tags = query.tags.join();
            }

            if (newOptions.sort && !query.sort) {
                var sortValue = "";
                for(var i = 0; i<options.sort.length; i++){
                    var sort = options.sort[i];
                    sortValue += (i > 0 ? "," : "") + (sort.descending ? "-" : "") + encodeURIComponent(sort.attribute);
                }
                newQuery.sort = sortValue;
                newOptions.sort = null;
            }

            result = this.inherited(arguments, [newQuery, newOptions]);

            result.then(function(data) {
                    queryResult.resolve(self._getList(data, options));
                }, function(error) {
                    queryResult.reject(error);
                });

            queryResult.total = this._getTotal(result);
            return QueryResults(queryResult);
        },

        get: function() {
            var result = this.inherited(arguments),
                self = this;

            return result.then(function(data) {
                return self._getItem(data);
            });
        },

        _getItem: function(data) {
            //  summary:
            //      Extracts a items from a property specified in the itemProperty
            //  tags: protected

            if (this.itemProperty && typeof data[this.itemProperty] != "undefined") {
                return data[this.itemProperty];
            } else {
                return data;
            }

        },

        put: function(object, options) {
            // Summary:
            //      Assumes that the REST-endpoint does not modify the object, but simply informs of the new id.
            //      This function will add that id to the object and return the entire object.

            var result = this.inherited(arguments),
                self = this;

            return result.then(function(data) {
                var dataItem = self._getItem(data), id = self.getIdentity(dataItem || {});
                if (self.returnUpdatedItemFromPut) { // in case if an endpoint does modify the object
                    return self.get(id);
                }
                if (dataItem && id) {
                    object[self.idProperty] = id;
                }
                return object;
            });
        },

        remove: function(id, options) {
            var self = this, deferred = new Deferred(), result = this.inherited(arguments);

            result.then(function(data) {
                if (data && typeof data === "object") {
                    // a workaround for observable, remove should return plain ID instead of object
                    deferred.resolve(self.getIdentity(data));
                }
                else {
                    deferred.resolve(data);
                }
            }, function(error) {
                deferred.reject(error);
            });

            return deferred;
        }
    });
});

},
'epi-find/store/BucketMixin':function(){
define([
    "dojo/_base/declare",
    "dojo/date",
    "dojo/date/stamp",
    "dojo/when"
],
function (declare, date, stamp, when) {

    return declare([], {
        // summary:
        //      Bucket mixin to pad out empty intervals of requested result set with "0"

        // keyName: String
        //      Most commonly it will be a timestamp range that needs padding
        keyName: "timestamp",

        // keyName: String
        //      This mixin currently only supports one "value" per bucket and this is the key
        valueName: "count",

        query: function (query) {
            // summary:
            //      Queries the store for objects. After a query has returned the results from
            //      server, values need to be put into buckets to fill out any non-returned
            //      empty results.
            // query: String|Object|Function
            //      The query to use for retrieving objects from the store.
            if(!query || !query.to || !query.from || !query.interval) {
                return this.inherited(arguments);
            }

            var keyName = this.keyName,
                valueName = this.valueName;

            return when(this.inherited(arguments)).then(function (results) {
                var buckets = {},
                    to = stamp.fromISOString(query.to),
                    from = stamp.fromISOString(query.from),
                    interval = query.interval,
                    fromOffset = from.getTimezoneOffset();

                if(interval === "month") {
                    from.setUTCDate(1);
                    to.setUTCDate(1);
                }

                // create a bucket for every interval
                for (var i=0, j=date.difference(from, to, query.interval); i<j; i++) {
                    var curDate = date.add(from, interval, i);
                    curDate = date.add(curDate, "minute", fromOffset-curDate.getTimezoneOffset()); // Compensate offset for "i", needed if span has transitioned to DST
                    buckets[stamp.toISOString(curDate, {
                        zulu: true
                    })] = 0;
                }

                // populate buckets value from result
                for(var i2=0, j2=results.length; i2<j2; i2+=1) {
                    buckets[results[i2].timestamp] = results[i2][valueName];
                }

                var data = [];
                for (var key in buckets) {
                    var o = {};
                    o[keyName] = key;
                    o[valueName] = buckets[key];
                    data.push(o);
                }
                return data;
            });
        }
    });
});
},
'epi-find/store/Statistics':function(){
﻿define([
        "dojo/_base/declare",
        "dojo/_base/lang",
        "./ResourceStore"
],
function (declare,
          lang,
          ResourceStore) {
    // No base class, but for purposes of documentation, the base class is dojo/store/api/Store

    return declare(ResourceStore, {
        // summary:
        //		This is a basic store for RESTful communicating with Statistics.
        //		It implements dojo/store/api/Store.

        constructor: function (options) {
            this.idProperty = "query";
            this.listProperty = "hits";
            declare.safeMixin(this, options);
        },

        _getList: function(data, options) {
            var result = this.inherited(arguments);
            if (options && result instanceof Array) {
                return result.slice(options.start, options.count);
            }
            return result;
        },

        _getTotal: function(total) {
            return this.inherited(arguments).then(function(total) {
                if(total > 100) { // since we cannot page stats in the backed we stop att a 100
                    return 100;
                }
                return total;
            });
        },

        query: function (query, options) {
            var newQuery = lang.clone(query || {}),
                newOptions = lang.clone(options || {});
            if (newQuery.previousQuery) { // TODO: Consider removing this transform and use q_previous
                newQuery["query.previous"] = newQuery.previousQuery;
                delete newQuery.previousQuery;
            }
            var start = 0;
            if (newOptions && newOptions.start) {
                start = newOptions.start;
                newQuery.start = start;
            }
            delete newOptions.start;
            if (newOptions && newOptions.count) {
                newQuery.size = newOptions.count;
            delete newOptions.count;
            }
            var results = this.inherited(arguments, [newQuery, newOptions]);
            return results;
        }
    });

});
},
'epi-find/store/Optimization':function(){
﻿define([
        "dojo/_base/declare",
        "dojo/_base/lang",
        "dojo/Deferred",
        "dojo/Evented",
        "./ResourceStore"
],
function (declare,
          lang,
          Deferred,
          Evented,
          ResourceStore) {

    return declare([ResourceStore, Evented], {
        // summary:
        //		This is a store for RelateQueries and Autocomplete.
        //		It implements dojo/store/api/Store.

        priorityProperty: "priority",
        maxPriority: 2147483647,

        constructor: function (options) {
            this.idProperty = "id";
            this.listProperty = "hits";
            this.itemProperty = "item";
            declare.safeMixin(this, options);
        },

        query: function (query, options) {
            // summary:
            //      Query method adapted to the nature of related queries endpoint.
            // description:
            //      Utilises the epi/shell/store/JsonRest "id-property to endpoint path" rewrite functionality
            //      to add "/list" to the endpoint target url on query requests.
            // tags:
            //      public
            query = lang.clone(query || {});
            query[this.idProperty] = "list"; // HACK: RelatedQueries and Autocomplete endpoints requires "/list" in order to get collection
            return this.inherited(arguments, [query, options]);
        },

        put: function(object, options) {

            // "before" property appears when Drag & Drop in dGrid is used
            if (options && options.hasOwnProperty("before")) {
                var superClassPut = this.getInherited(arguments),
                    results = new Deferred(),
                    newObject = lang.clone(object),
                    newOptions = lang.clone(options),
                    self = this,
                    query = {},
                    afterItem,
                    afterPriority = self.maxPriority,
                    beforePriority = 0;
                if (options.before) {
                    beforePriority = options.before[this.priorityProperty];
                    query = {priorityFrom: beforePriority};
                }
                var queryResults = self.query(query,{
                        start: 0,
                        count: 1,
                        sort: [{attribute: this.priorityProperty, descending: false}]
                });
                queryResults.then(function(afterItems) {
                    // find an item that should be after the current item
                    if (afterItems && afterItems.length) {
                        afterItem = afterItems[0];
                        afterPriority = afterItem[self.priorityProperty];
                    }
                    // calculate a new priority value for the current item as an average between
                    // priorities of "before" and "after" items
                    var newPriority = beforePriority + Math.round((afterPriority - beforePriority) / 2);
                    if (newPriority === beforePriority || newPriority === afterPriority) {
                        // notify about refresh with delay, since update on the elastic is delayed
                        setTimeout(function() {self.emit("refresh");}, 2000);
                    }
                    newObject[self.priorityProperty] = newPriority;
                    superClassPut.apply(self, [newObject, newOptions]).then(function (putResults) {
                        results.resolve(putResults);
                        self.emit("dndComplete", putResults);
                    }, function (error) {
                        results.reject(error);
                        self.emit("dndError", error);
                    });
                });

                return results;

            }
            return this.inherited(arguments);
        }

    });

});
},
'epi-find/store/BestBets':function(){
﻿define([
    "dojo/_base/config",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/Deferred",
    "./ResourceStore",
    "dojo/_base/json",
    "epi-saas-base/RetryableXhrWrapper"
],
    function (config,
        declare,
        lang,
        Deferred,
        ResourceStore,
        json,
        RetryableXhrWrapper
    ) {

        return declare([ResourceStore], {
            // summary:
            //		This is a store for RelateQueries and Autocomplete.
            //		It implements dojo/store/api/Store.

            priorityProperty: "priority",
            maxPriority: 2147483647,

            constructor: function (options) {
                this.idProperty = "id";
                this.listProperty = "hits";
                this.itemProperty = "item";
                this.xhrHandler = new RetryableXhrWrapper(options ? options.handlerOptions : null);
                declare.safeMixin(this, options);
            },

            query: function (query, options) {
                // summary:
                //      Query method adapted to the nature of related queries endpoint.
                // description:
                //      Utilises the epi/shell/store/JsonRest "id-property to endpoint path" rewrite functionality
                //      to add "/list" to the endpoint target url on query requests.
                // tags:
                //      public
                query = lang.clone(query || {});
                query[this.idProperty] = options && options.apiName ? options.apiName : "list";
                return this.inherited(arguments, [query, options]);
            },

            get: function (id, options) {
                // summary:
                //      Retrieves an object by its identity. This will trigger a GET request to the server using
                //      the url `this.target + id`.
                // id: Number
                //      The identity to use to lookup the object.
                // options: Object?
                //      The optional options for the request header.
                // returns:
                //      The object in the store that matches the given id.
                // tags:
                //      public

                var headers = options || {};
                headers.Accept = "application/javascript, application/json";
                var url = config.find.siteServiceApiBaseUrl + "bestbets/get?id=";
                var hasId = typeof id != "undefined";

                var params = {
                    url: hasId ? url + id : url,
                    handleAs: "json",
                    headers: headers,
                    preventCache: this.preventCache
                };

                var result = this.xhrHandler.xhrGet(this._mixinDefaultParams(params));
                self = this;

                return result.then(function (data) {
                    return self._getItem(data);
                });
            },

            put: function (object, options) {

                options = options || {};

                var paramHeaders = {
                    "Content-Type": "application/json",
                    "If-Match": options.overwrite === true ? "*" : null,
                    "If-None-Match": options.overwrite === false ? "*" : null,
                    Accept: "application/javascript, application/json"
                };

                var url = config.find.siteServiceApiBaseUrl + "bestbets/post";
                var params = {
                    url: url,
                    postData: json.toJson(object),
                    handleAs: "json",
                    headers: paramHeaders
                };

                var result = this.xhrHandler.xhr("POST", this._mixinDefaultParams(params));
                var self = this;
                var deferred = new Deferred()
                return result.then(function (data) {
                    if (data && typeof data === "object") {
                        var dataItem = self._getItem(data), id = self.getIdentity(dataItem || {});
                        if (self.returnUpdatedItemFromPut) { // in case if an endpoint does modify the object
                            return self.get(id);
                        }
                        if (dataItem && id) {
                            object[self.idProperty] = id;
                        }
                        return object;
                    }
                    else {
                        deferred.resolve(data);
                    }
                });
            },

            remove: function (id, options) {
                var headers = {};
                headers.Accept = "application/javascript, application/json";
                headers["Content-Type"] = "application/json";

                options = options || {};

                var result = this.xhrHandler.xhr("DELETE", this._mixinDefaultParams({
                    url: config.find.siteServiceApiBaseUrl + "bestbets/delete?id=" + id,
                    headers: headers,
                    postData: json.toJson(options),
                    handleAs: "json"
                }));

                var self = this, deferred = new Deferred();
                result.then(function (data) {
                    if (data && typeof data === "object") {
                        // a workaround for observable, remove should return plain ID instead of object
                        deferred.resolve(self.getIdentity(data));
                    }
                    else {
                        deferred.resolve(data);
                    }
                }, function (error) {
                    deferred.reject(error);
                });

                return deferred;
            },

            _mixinDefaultParams: function (params) {
                // summary:
                //      Mix the params and the defaultRequestParams into a new object.
                // params: Object
                //      The params to mix with defaultRequestParams.
                // returns: Object
                //      The new combined params.
                // tags:
                //      private

                return lang.mixin({}, this.defaultRequestParams, params);
            }

        });

    });
},
'epi-find/store/SearchWithWeights':function(){
﻿define([
        "dojo/_base/lang",
        "dojo/Deferred",
        "dojo/json",
        "dojo/_base/declare",
        "dojo/store/util/QueryResults",
        "epi/shell/store/JsonRest",
        "epi-saas-base/RetryableXhrWrapper"
],
function (lang, Deferred, JSON, declare, QueryResults, JsonRest, RetryableXhrWrapper) {
    return declare(JsonRest, {
        // summary:
        //		This is a basic store for RESTful communicating with searchwithweights

        constructor: function (options) {
            this.xhrHandler = new RetryableXhrWrapper(options ? options.handlerOptions : null);
            this.headers = {"Content-Type": "application/json"};
            declare.safeMixin(this, options);
        },

        // idProperty: String
        //		Indicates the property to use as the identity property. The values of this
        //		property should be unique.
        idProperty: "Url",

        query: function (query, options) {
            options = options || {};
            query = query || {};

            if (options.count) {
                query.size = options.count;
            }
            if (options.start) {
                query.from = options.start;
            }
            var results;
            if (query.q) {
                results = this.inherited(arguments);
            } else {
                var headers = lang.mixin({ Accept: this.accepts }, this.headers, options.headers);
                results = this.xhrHandler.xhr("POST", {
                    url: this.target,
                    postData: JSON.stringify(query),
                    handleAs: "json",
                    headers: headers
                });
            }

            var queryResults = new Deferred();
            results.then(function(data) {
                queryResults.resolve(data.hits);
            }, function(error) {
                queryResults.reject(error);
            });

            queryResults.total = results.then(function (data) {
                return data.total;
            });

            queryResults.query = query;

            return QueryResults(queryResults);
        }
    });

});
},
'epi-find/store/ServiceHealthStore':function(){
﻿define([
    "dojo/_base/lang",
    "dojo/Deferred",
    "dojo/_base/declare",
    "dojo/store/util/QueryResults",
    "epi/shell/store/JsonRest",
    "epi-saas-base/RetryableXhrWrapper"
],
    function (lang, Deferred, declare, QueryResults, JsonRest, RetryableXhrWrapper) {
        return declare(JsonRest, {
            // summary:
            //      This is a basic store for RESTful communicating with Service Health

            constructor: function (options) {
                this.xhrHandler = new RetryableXhrWrapper(options ? options.handlerOptions : null);
                this.headers = { "Content-Type": "application/json" };
                declare.safeMixin(this, options);
            },

            query: function (options) {
                options = options || {};
                var headers = lang.mixin({ Accept: this.accepts }, this.headers, options.headers);
                results = this.xhrHandler.xhr("GET", {
                    url: this.target,
                    handleAs: "json",
                    headers: headers
                });

                var queryResults = new Deferred();
                results.then(function (data) {
                    queryResults.resolve(data);
                }, function (error) {
                    queryResults.reject(error);
                });

                queryResults.data = results.then(function (data) {
                    return data;
                });

                return QueryResults(queryResults);
            }
        });
    });

},
'epi-find/store/LangMixin':function(){
﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang"
],
function (declare, lang) {
    return declare(null, {
        // languageParameter: string
        //      The query parameter to send when querying for a specific language.
        languageParameter: null,

        // languagePrefix: string
        //      Prefix of the language value.
        languagePrefix: null,

        // emptyLanguage: string
        //      Value to pass when language is null.
        emptyLanguage: null,

        // addEmptyToQuery: bool
        //      Indicates if emptyLanguage should always be added to the query using the pipe syntax,
        //      even if language is specified
        addEmptyToQuery: false,

        // language: String
        //      The identifying string for the language
        language: null,

        // addToObject: Boolean
        //      Indicates that additional parameters should be added to the object when it is about to be saved
        addToObject: false,

        query: function(query, options) {

            var newQuery = this._addLanguage(lang.clone(query), this.addEmptyToQuery);
            return this.inherited(arguments, [newQuery, options]);
        },

        put: function(object, options){
            if (this.addToObject) {
                object = this._addLanguage(object);
            }
            return this.inherited(arguments, [object, options]);
        },

        _addLanguage: function(/*Object*/ query, /*Boolean*/ addEmpty) {
            // summary:
            //      This method adds information about the current language
            //      to the specified `query` object
            // query: Object
            //      Entity or query where the language parameter should be set
            // addEmpty: Boolean
            //      Indicates that value of `emptyLanguage` property should be
            //      added to the `query` parameter
            // tags:
            //      private
            if (!this.languageParameter || !this.languagePrefix) {
                return query;
            }
            var langQueryValue, existingParam, langValues = [];
            query = query || {};
            if (this.emptyLanguage && (addEmpty || !this.language)) {
                langValues.push(this.languagePrefix + this.emptyLanguage);
            }
            if (this.language) {
                langValues.push(this.languagePrefix + this.language);
            }

            if (langValues.length === 0)
            {
                return query;
            }

            langQueryValue = langValues.join("|");
            existingParam = query[this.languageParameter];
            if (existingParam) {
                if (existingParam instanceof Array) {
                    var exists = false;
                    for (var i=0; i < existingParam.length; i++) {
                        if (existingParam[i].indexOf(this.languagePrefix) === 0) {
                            existingParam[i] = langQueryValue;
                            exists = true;
                            break;
                        }
                    }
                    if (!exists) {
                        query[this.languageParameter].push(langQueryValue);
                    }
                } else {
                    query[this.languageParameter] = [existingParam, langQueryValue];
                }
            } else {
                query[this.languageParameter] = [langQueryValue];
            }

            return query;
        },

        setLanguage: function(/* string */ language) {
            // summary:
            //      Sets the language to a new value. 
            //      The language is the identifying language code of the language.
            //      null means all languages
            var oldLanguage = this.language;

            this.language = language;

            if (language !== oldLanguage) {
                this.onLanguageChange(oldLanguage, language);
            }
        },

        // Overridden in children.
        onLanguageChange: function(oldLanguage, newLanguage) {
        }
    });
});

},
'epi-find/store/SiteMixin':function(){
define([
    "dojo/_base/declare",
    "dojo/_base/lang"
],
function (declare, lang) {
    return declare(null, {
        // siteParameter: string
        //      The query parameter to send when querying for a specific site.
        siteParameter: null,

        // sitePrefix: string
        //      An optional prefix of the site value.
        sitePrefix: "",

        // emptySite: string
        //      Value to pass when site is null.
        emptySite: null,

        // addEmptyToQuery: bool
        //      Indicates if emptySite should always be added to the query using the pipe syntax,
        //      even if site is specified
        addEmptyToQuery: false,

        // site: String
        //      The identifying string for the site
        site: null,

        // addToObject: Boolean
        //      Indicates that additional parameters should be added to the object when it is about to be saved
        addToObject: false,

        query: function(query, options) {
            // Override any query function to add the site parameter
            var newQuery = this._addSite(lang.clone(query), this.addEmptyToQuery);
            return this.inherited(arguments, [newQuery, options]);
        },

        put: function(object, options){
            if (this.addToObject) {
                object = this._addSite(object);
            }
            return this.inherited(arguments, [object, options]);
        },

        _addSite: function(query, /*bool*/ addEmpty) {
            // summary:
            //      This method adds information about the current site
            //      to the specified `query` object
            // query: Object
            //      Entity or query where the site parameter should be set
            // addEmpty: Boolean
            //      Indicates that value of `emptySite` property should be
            //      added to the `query` parameter
            // tags:
            //      private
            if (!this.siteParameter || !this.sitePrefix) {
                return query;
            }
            var siteQueryValue, existingParam, siteValues = [];
            query = query || {};
            if (this.emptySite && (addEmpty || !this.site)) {
                siteValues.push(this.sitePrefix + this.emptySite);
            }
            if (this.site) {
                siteValues.push(this.sitePrefix + this.site);
            }

            if (siteValues.length === 0)
            {
                return query;
            }

            siteQueryValue = siteValues.join("|");
            existingParam = query[this.siteParameter];
            if (existingParam) {
                if (existingParam instanceof Array) {
                    var exists = false;
                    for (var i=0; i < existingParam.length; i++) {
                        if (existingParam[i].indexOf(this.sitePrefix) === 0) {
                            existingParam[i] = siteQueryValue;
                            exists = true;
                            break;
                        }
                    }
                    if (!exists) {
                        query[this.siteParameter].push(siteQueryValue);
                    }
                } else {
                    query[this.siteParameter] = [existingParam, siteQueryValue];
                }
            } else {
                query[this.siteParameter] = [siteQueryValue];
            }

            return query;
        },

        setSite: function(/* object */ site) {
            // Summary:
            //      Sets the site to a new value, null means all sites.
            var oldSite = this.site;

            this.site = site;

            if (site !== oldSite) {
                this.onSiteChange(oldSite, site);
            }
        },

        // Overridden in children.
        onSiteChange: function(oldSite, newSite) {
        }
    });
});
},
'epi-find/AccessModel':function(){
define("epi-find/AccessModel", [
    "dojo/_base/array",
    "dojo/_base/config",
    "dojo/_base/declare",
    "dojo/_base/lang",

    "./ConfigModel",
    "./Intents!"
],
    function(array,
             config,
             declare,
             lang,
             ConfigModel
        ) {

        // module:
        //      epi-find/AccessModel

        var AccessItem = declare(null, {
            // summary:
            //      Represents a single access item

            intent: null,
            allowedOptions: null,
            enabledOptions: null,
            allowedRoles: null,
            enabledRoles: null,

            constructor: function(options) {
                declare.safeMixin(this, options);

                this.allowedOptions = this.allowedOptions || {};
                this.enabledOptions = this.enabledOptions || {};

                this.allowedRoles = this.allowedRoles || [];
                this.enabledRoles = this.enabledRoles || [];
            },

            enableFor: function() {
                // summary:
                //      This method change enabledOptions or enabledRoles depending on parameter type.
                //      It can accept multiple arguments.
                // example:
                //      access.view.synonyms.enableFor({admin:true}, ["Admin", "SearchAdmin"]);
                for (var i=0; i<arguments.length; i++) {
                    var param = arguments[i];
                    if (typeof param === "string" || param instanceof String) {
                        this.enabledRoles.push(param);
                    }
                    else if (param instanceof Array) {
                        this.enabledRoles = this.enabledRoles.concat(param);
                    }
                    else if (param instanceof Object) {
                        lang.mixin(this.enabledOptions, param);
                    }

                }
            },

            allowFor: function() {
                // summary:
                //      This method change allowedOptions or allowedRoles depending on parameter type.
                //      It can accept multiple arguments.
                // example:
                //      access.view.synonyms.allowFor({admin:true}, ["Admin", "SearchAdmin"]);
                for (var i=0; i<arguments.length; i++) {
                    var param = arguments[i];
                    if (param instanceof Object) {
                        lang.mixin(this.allowedOptions, param);
                    }
                    else if (param instanceof String) {
                        this.allowedRoles.push(param);
                    }
                    else if (param instanceof Array) {
                        this.allowedRoles = this.allowedRoles.concat(param);
                    }
                }
            },
            isEnabled: function() {
                return this._isMatch(this.enabledOptions, this.enabledRoles);
            },
            isAllowed: function() {
                return this._isMatch(this.allowedOptions, this.allowedRoles);
            },

            _isMatch: function(options, roles) {
                if(ConfigModel.config === null) {
                    throw new Error("Config is not loaded yet. Access model can not be used.");
                }

                // Check options first
                if (options) {
                    for (var option in options) {
                        if (options[option] !== ConfigModel.config[option]) {
                            return false;
                        }
                    }
                }

                // Check roles if access options fulfilled
                if (roles && roles.length && ConfigModel.config.roles) {
                    return array.some(roles, function(role) {
                        return array.indexOf(ConfigModel.config.roles, role) > -1;
                    });
                }
                return true;
            }
        });

        var AccessModel = declare(null, {
            // summary:
            //     Model for access rights.

            // access: Object
            //      Holds access settings
            access: null,

            constructor: function(options) {
                declare.safeMixin(this, options);
                this._configureAccess();
            },

            isEnabled: function(intent) {
                // Summary:
                //      Check if specified intent is enabled for the current user
                var access = this._getAccessForIntent(this.access, intent);
                if (!access) {
                    return false;
                }

                return access.isEnabled();
            },

            isAllowed: function(intent) {
                // summary:
                //      Check if specified intent is allowed for the current user
                var access = this._getAccessForIntent(this.access, intent);
                if (!access) {
                    return false;
                }
                return access.isAllowed();
            },

            _configureAccess: function() {
                // Summary:
                //      Configures default access for each intent

                this.access = this._createDefaultAccess(config.intents);
            },

            _createDefaultAccess: function (intents) {
                var access = {};

                for (var i in intents) {
                    var intent = intents[i];
                    if (intent instanceof Object) {
                        access[i] = this._createDefaultAccess(intent);
                    }
                    else {
                        access[i] = new AccessItem({intent: intent});
                    }
                }
                return access;
            },

            _getAccessForIntent: function(accessSet, intent) {
                if (!accessSet) {
                    return null;
                }
                for (var a in accessSet) {
                    var access = accessSet[a];
                    if (!access.intent) {
                        var child = this._getAccessForIntent(access, intent);
                        if (child) {
                            return child;
                        }
                    }
                    else {
                        if (access.intent === intent) {
                            return access;
                        }
                    }
                }
                return null;
            }
        });
        var accessModel = new AccessModel();
        return accessModel;
    });
},
'epi-find/ConfigModel':function(){
define("epi-find/ConfigModel", [
    "dojo/_base/declare",
    "dojo/_base/config",

    "dojo/Deferred",
    "dojo/Stateful",
    "dojo/topic",

    "epi-saas-base/RetryableXhrWrapper",
    "dojo/i18n!./nls/ConfigModel"
],
    function(declare, config,
             Deferred, Stateful,
             topic, RetryableXhrWrapper,
             i18n) {

        // module:
        //      epi-find/ConfigModel

        var Model = declare([Stateful], {
            // summary:
            //     Model for the application configuration.

            // xhrHandler: object
            //      An XHR implementation to use when reading configuration
            xhrHandler: null,

            //  config: object
            //      An object representing application configuration
            config: null,

            //  target: string
            //      Url to the configuration endpoint
            target: null,

            constructor: function(options) {
                declare.safeMixin(this, options);
                this.xhrHandler = this.xhrHandler || new RetryableXhrWrapper();
            },

            refresh: function() {
                // summary:
                //      Updates the model with configuration values from server side.
                //  tags:
                //      public
                if (!this.target) {
                    throw new Error("The 'target' property is not set.");
                }

                var self = this;

                return this.xhrHandler.xhrGet({
                    url: this.target,
                    handleAs: "json"
                }).then(function(result) {
                    self.set("config", result.item);
                    return result.item;
                }, function(error) {
                    var notification = {topic: "global", key: "ReadConfig"};
                    notification.type = "error";
                    notification.message = i18n.configErrorMessage;
                    notification.details = error;
                    topic.publish("epi/notification", notification);
                });
            },

            getValue: function(/*String*/ path) {
                var pathParts = path.split("."),
                    currentConfigPath = this.config;
                for (var i=0; i< pathParts.length; i++) {
                    if (currentConfigPath !== null && currentConfigPath !== undefined) {
                        currentConfigPath = currentConfigPath[pathParts[i]];
                    }
                    else {
                        break;
                    }
                }
                return currentConfigPath;
            }
        });
        var configModel = new Model();
        return configModel;
    });

},
'epi-find/widget/SiteMenu':function(){
define([
    "dojo/_base/declare",
    "dojo/_base/array",

    "dojox/html/entities",

    "dijit/_WidgetBase",
    "dijit/_Container",
    "dijit/_TemplatedMixin",

    "dijit/form/DropDownButton",
    "dijit/MenuItem",
    "dijit/DropDownMenu",

    "./_HelpMixin",

    "dojo/i18n!./nls/SiteMenu",
    "dojo/text!./templates/SiteMenu.html"
],
function(
    declare,
    array,
    entities,
    _WidgetBase,
    _Container,
    _TemplatedMixin,
    DropDownButton,
    MenuItem,
    DropDownMenu,
    _HelpMixin,
    i18n,
    template) {

    // module:
    //      epi-find/widget/SiteMenu

    return declare([_WidgetBase, _Container, _TemplatedMixin, _HelpMixin], {
        // summary: 
        //     Menu for site, language and help.

        i18n: i18n,

        templateString: template,

        siteMenu: null,
        languageMenu: null,

        sites: null,
        _setSitesAttr: function(value) {
            var self = this,
                clickHandler = function() {
                    var selected = array.filter(self.get("sites"), function (item) {
                        return item.id === this.value;
                    }, this);
                    if (selected.length>0) {
                        self.set("selectedSite", selected[0]);
                    }
                };
            if (value && this.siteMenu) {
                while(this.siteMenu.hasChildren()) {
                    this.siteMenu.removeChild(0);
                }
                for (var i=0; i<value.length; i++) {
                    this.siteMenu.addChild(new MenuItem({
                        label: entities.encode(value[i].name),
                        value: value[i].id,
                        onClick: clickHandler
                    }));
                }
            }
            this._set("sites", value);
        },

        selectedSite: null,
        _setSelectedSiteAttr: function(value) {
            if (value && this.siteDropDown) {
                this.siteDropDown.set("label", entities.encode(value.name));
            }
            this._set("selectedSite", value);
        },

        sitesDisabled: false,
        _setSitesDisabledAttr: function(value) {
            if (this.siteDropDown) {
                this.siteDropDown.setDisabled(value);
            }
        },

        languages: null,
        _setLanguagesAttr: function(value) {
            var self = this,
                clickHandler = function() {
                    var selected = array.filter(self.get("languages"), function (item) {
                        return item.id === this.value;
                    }, this);
                    if (selected.length>0) {
                        self.set("selectedLanguage", selected[0]);
                    }
                };
            if (value && this.languageMenu) {
                while(this.languageMenu.hasChildren()) {
                    this.languageMenu.removeChild(0);
                }

                for (var i=0; i<value.length; i++) {
                    this.languageMenu.addChild(new MenuItem({
                        label: value[i].name,
                        value: value[i].id,
                        onClick: clickHandler
                    }));
                }
            }
            this._set("languages", value);
        },

        selectedLanguage: null,
        _setSelectedLanguageAttr: function(value) {
            if (value && this.languageDropDown) {
                this.languageDropDown.set("label", value.name);
            }
            this._set("selectedLanguage", value);
        },
        languagesDisabled: false,
        _setLanguagesDisabledAttr: function(value) {
            if (this.languageDropDown) {
                this.languageDropDown.setDisabled(value);
            }
        },

        postMixInProperties: function() {
            this.inherited(arguments);
        },

        postCreate: function() {
            this.inherited(arguments);

            this._createSiteMenu(this.sitesNode);
            this._createLanguageMenu(this.languagesNode);

        },

        _createSiteMenu: function(node) {
            var siteMenu = this.siteMenu = new DropDownMenu({
                style: "display: none;"
            });

            this.siteDropDown = new DropDownButton({
                "class": "epi-chromeless epi-chromeless--with-arrow epi-flat epi-background--dark-grey",
                name: "sites",
                dropDown: siteMenu,
                id: "siteButton"
            }, node);
        },

        _createLanguageMenu: function(node) {
            var languageMenu = this.languageMenu = new DropDownMenu({
                style: "display: none;"
            });
            this.languageDropDown = new DropDownButton({
                "class": "epi-chromeless epi-chromeless--with-arrow epi-flat epi-background--dark-grey",
                name: "languages",
                dropDown: languageMenu,
                id: "languagesButton"
            }, node);
        }
    });
});
},
'epi-find/widget/_HelpMixin':function(){
define([
    "dojo/_base/declare",
    "dojo/query",
    "dojo/dom-class",

    "dijit/_WidgetBase",
    "dijit/_Container",
    "dijit/_TemplatedMixin",
    "dijit/form/Button",

    "dojo/i18n!./nls/_HelpMixin",
    "dojo/ready"
], function(declare, query, domClass,
    _WidgetBase, _Container, _TemplatedMixin, Button,
    i18n) {

    var HelpButton = declare([Button], {

        // container: _HelpMixin
        //      The _HelpMixin container widget to toggle "help mode" for
        "class": "epi-chromeless epi-mediumButton epi-help-button",
        iconClass:"epi-iconHelp"
    });

    // module:
    //      epi-find/widget/_HelpMixin

    return declare([_WidgetBase, _TemplatedMixin, _Container], {
        // summary:
        //     Help toggling mixin
        // description:
        //     This mixin toggles the visibility of all elements with the "help" class
        //     when the element with the attachment point "helpButtonNode" is clicked.

        showHelp: false,
        _setShowHelpAttr: function(value) {
            this._set("showHelp", value);
            this._updateLabel();
            this._syncHelp();
        },

        helpClassName: "epi-help",
        activeClassName: "is-active",

        helpContainer: null,
        helpButton: null,
        helpButtonNode: null,

        // globalHelpState: Boolean
        //      Set this to true if one global help state  is desired, false if every instance
        //      should hold its own state (using the classes on the domNodes inside the widget)
        globalHelpState: true,

        postMixInProperties: function() {
            this.inherited(arguments);
            this.helpContainer = this.helpContainer || this.domNode;
        },

        postCreate: function() {
            var self = this;
            this.inherited(arguments);
            if (!this.helpButtonNode) {
                throw new Error("The _HelpMixin should be mixed to a widget that has a domNode reference stored in 'helpButtonNode' property");
            }

            this.helpButton = new HelpButton({
                onClick: function() {
                    self.toggleHelp();
                }
            }, this.helpButtonNode);
            this.own(this.helpButton);
            this._updateLabel();

        },

        startup: function() {
            this.inherited(arguments);
            this._syncHelp();
        },

        toggleHelp: function() {
            this.set("showHelp", !this.showHelp);
        },

        refresh: function() {
            this.set("showHelp", this._isHelpActive());
        },

        _updateLabel: function() {
            if (this.helpButton) {
                this.helpButton.set("label", this.showHelp ? i18n.hideHelp : i18n.showHelp);
            }
        },

        _isHelpActive: function() {
            return this.globalHelpState ? this.showHelp : query("." + this.helpClassName+"."+this.activeClassName, this.helpContainer).length > 0;
        },

        _syncHelp: function() {
            var showHelp = this.showHelp, activeClassName = this.activeClassName;

            query("." + this.helpClassName, this.helpContainer).forEach(function(node) {
                if (showHelp) {
                    domClass.add(node, activeClassName);
                } else {
                    domClass.remove(node, activeClassName);
                }
            });
        }
    });
});
},
'url:epi-find/widget/templates/SiteMenu.html':"<div class=\"epi-site-menu\">\r\n    ${i18n.for}\r\n    <span data-dojo-attach-point=\"sitesNode\"></span><!--\r\n --><span class=\"epi-lozenge\">${i18n.in}</span><!--\r\n --><span data-dojo-attach-point=\"languagesNode\"></span>\r\n\r\n    <span data-dojo-attach-point=\"helpButtonNode\"></span>\r\n</div>\r\n",
'epi-find/widget/SiteMenuModel':function(){
﻿define([
    "dojo/_base/declare",
    "dojo/_base/array",
    "dojo/_base/config",

    "dojo/Stateful",
    "dijit/Destroyable",

    "dojox/lang/functional",
    "dojox/mvc/sync",

    "epi-cms/ApplicationSettings",
    "epi/dependency",

    "../store/LangMixin",
    "../store/SiteMixin",
    "../ConfigModel",
    "../ApplicationState",

    "dojo/i18n!epi-saas-base/nls/Languages",
    "dojo/i18n!./nls/SiteMenu"
],
    function(declare,
             array,
             config,
             Stateful,
             Destroyable,
             functional,
             sync,
             ApplicationSettings,
             dependency,
             LangMixin,
             SiteMixin,
             ConfigModel,
             ApplicationState,
             i18nLanguages,
             i18n) {

        // module:
        //      epi-find/widget/SiteMenuModel

        return declare([Stateful, Destroyable], {
            // summary:
            //     Model for the site menu for site and language.

            showHelp: false,

            sites: null,

            selectedSite: null,
            _selectedSiteSetter: function (site) {
                if (site) {
                    ApplicationSettings.startPage = site.startPage;
                    ApplicationSettings.rootPage = site.rootPage;
                    ApplicationSettings.wastebasketPage = site.wasteBasket || "2"; // fallback for backward compatibility

                    functional.forEach(config.dependencies, function (dependency) {
                        // TODO: Consider avoid changes in stores state like this since stores are often used
                        // with 'delegate', which might hinder changing the store state.
                        // Here we have to use a workaround with the 'unwrapped' property in order to reach
                        // the original store object.
                        if (dependency.isInstanceOf(SiteMixin)) {
                            if (dependency.unwrapped) {
                                dependency.unwrapped.setSite(site.id);
                            }
                            else {
                                dependency.setSite(site.id);
                            }
                        }
                    });
                }
                this.selectedSite = site;
                ApplicationState.set("site", site);
            },
            sitesDisabled: false,

            languages: null,
            selectedLanguage: null,
            _selectedLanguageSetter: function (language) {
                if (language) {
                    functional.forEach(config.dependencies, function (dependency) {
                        // TODO: Consider avoid changes in stores state like this since stores are often used
                        // with 'delegate', which might hinder changing the store state.
                        // Here we have to use a workaround with the 'unwrapped' property in order to reach
                        // the original store object.
                        if (dependency.isInstanceOf(LangMixin)) {
                            if (dependency.unwrapped) {
                                dependency.unwrapped.setLanguage(language.id);
                            }
                            else {
                                dependency.setLanguage(language.id);
                            }
                        }
                    });
                }
                this.selectedLanguage = language;
                ApplicationState.set("language", language);


                if (!dependency.exists("epi.shell.Profile")) {
                    dependency.register("epi.shell.Profile", {});
                }
                var profile = dependency.resolve("epi.shell.Profile");
                profile.contentLanguage =  this.selectedLanguage.id;
            },
            languagesDisabled: false,

            // config: object
            //      Configuration parameters
            config: null,
            _configSetter: function(value) {
                if (!value) {
                    return;
                }
                var languages = value.languages, languageItems, sites = value.sites, rootPage;

                languageItems = array.map(languages, function(item) {
                    return {id: item, name: i18nLanguages[item] || item };
                });
                languageItems.unshift({ id: null, name: i18n.allLanguages });
                this.set("languages", languageItems);
                this.set("selectedLanguage", languageItems[0]);


                if (sites.length > 0) {
                    rootPage = sites[0].rootPage;
                }
                sites.unshift({ id: null, name: i18n.allSites, rootPage: rootPage, startPage: rootPage });

                this.set("sites", sites);
                this.set("selectedSite", sites[0]);

                this.config = value;
            },

            constructor: function(options) {
                declare.safeMixin(this, options);
                var self = this;

                // read current configuration
                this.set("config", ConfigModel.config);
                // and watch for configuration changes
                this.own(ConfigModel.watch("config", function(name, oldValue, value) {
                    self.set("config", value);
                }));
            }
        });
    });
},
'dojox/lang/functional':function(){
define("dojox/lang/functional", ["./functional/lambda", "./functional/array", "./functional/object"], function(df){
	return df;
});

},
'dojox/lang/functional/lambda':function(){
define("dojox/lang/functional/lambda", ["../..", "dojo/_base/kernel", "dojo/_base/lang", "dojo/_base/array"], function(dojox, dojo, lang, arr){
	var df = lang.getObject("lang.functional", true, dojox);

// This module adds high-level functions and related constructs:
//	- anonymous functions built from the string

// Acknowledgements:
//	- lambda() is based on work by Oliver Steele
//		(http://osteele.com/sources/javascript/functional/functional.js)
//		which was published under MIT License

// Notes:
//	- lambda() produces functions, which after the compilation step are
//		as fast as regular JS functions (at least theoretically).

// Lambda input values:
//	- returns functions unchanged
//	- converts strings to functions
//	- converts arrays to a functional composition

	var lcache = {};

	// split() is augmented on IE6 to ensure the uniform behavior
	var split = "ab".split(/a*/).length > 1 ? String.prototype.split :
			function(sep){
				 var r = this.split.call(this, sep),
					 m = sep.exec(this);
				 if(m && m.index == 0){ r.unshift(""); }
				 return r;
			};
			
	var lambda = function(/*String*/ s){
		var args = [], sects = split.call(s, /\s*->\s*/m);
		if(sects.length > 1){
			while(sects.length){
				s = sects.pop();
				args = sects.pop().split(/\s*,\s*|\s+/m);
				if(sects.length){ sects.push("(function(" + args + "){return (" + s + ")})"); }
			}
		}else if(s.match(/\b_\b/)){
			args = ["_"];
		}else{
			var l = s.match(/^\s*(?:[+*\/%&|\^\.=<>]|!=)/m),
				r = s.match(/[+\-*\/%&|\^\.=<>!]\s*$/m);
			if(l || r){
				if(l){
					args.push("$1");
					s = "$1" + s;
				}
				if(r){
					args.push("$2");
					s = s + "$2";
				}
			}else{
				// the point of the long regex below is to exclude all well-known
				// lower-case words from the list of potential arguments
				var vars = s.
					replace(/(?:\b[A-Z]|\.[a-zA-Z_$])[a-zA-Z_$\d]*|[a-zA-Z_$][a-zA-Z_$\d]*:|this|true|false|null|undefined|typeof|instanceof|in|delete|new|void|arguments|decodeURI|decodeURIComponent|encodeURI|encodeURIComponent|escape|eval|isFinite|isNaN|parseFloat|parseInt|unescape|dojo|dijit|dojox|window|document|'(?:[^'\\]|\\.)*'|"(?:[^"\\]|\\.)*"/g, "").
					match(/([a-z_$][a-z_$\d]*)/gi) || [], t = {};
				arr.forEach(vars, function(v){
					if(!(v in t)){
						args.push(v);
						t[v] = 1;
					}
				});
			}
		}
		return {args: args, body: s};	// Object
	};

	var compose = function(/*Array*/ a){
		return a.length ?
					function(){
						var i = a.length - 1, x = df.lambda(a[i]).apply(this, arguments);
						for(--i; i >= 0; --i){ x = df.lambda(a[i]).call(this, x); }
						return x;
					}
				:
					// identity
					function(x){ return x; };
	};

	lang.mixin(df, {
		// lambda
		rawLambda: function(/*String*/ s){
			// summary:
			//		builds a function from a snippet, or array (composing),
			//		returns an object describing the function; functions are
			//		passed through unmodified.
			// description:
			//		This method is to normalize a functional representation (a
			//		text snippet) to an object that contains an array of
			//		arguments, and a body , which is used to calculate the
			//		returning value.
			return lambda(s);	// Object
		},
		buildLambda: function(/*String*/ s){
			// summary:
			//		builds a function from a snippet, returns a string, which
			//		represents the function.
			// description:
			//		This method returns a textual representation of a function
			//		built from the snippet. It is meant to be evaled in the
			//		proper context, so local variables can be pulled from the
			//		environment.
			s = lambda(s);
			return "function(" + s.args.join(",") + "){return (" + s.body + ");}";	// String
		},
		lambda: function(/*Function|String|Array*/ s){
			// summary:
			//		builds a function from a snippet, or array (composing),
			//		returns a function object; functions are passed through
			//		unmodified.
			// description:
			//		This method is used to normalize a functional
			//		representation (a text snippet, an array, or a function) to
			//		a function object.
			if(typeof s == "function"){ return s; }
			if(s instanceof Array){ return compose(s); }
			if(s in lcache){ return lcache[s]; }
			s = lambda(s);
			return lcache[s] = new Function(s.args, "return (" + s.body + ");");	// Function
		},
		clearLambdaCache: function(){
			// summary:
			//		clears internal cache of lambdas
			lcache = {};
		}
	});
	
	return df;
});

},
'dojox/main':function(){
define("dojox/main", ["dojo/_base/kernel"], function(dojo) {
	// module:
	//		dojox/main

	/*=====
	return {
		// summary:
		//		The dojox package main module; dojox package is somewhat unusual in that the main module currently just provides an empty object.
		//		Apps should require modules from the dojox packages directly, rather than loading this module.
	};
	=====*/

	return dojo.dojox;
});
},
'dojox/lang/functional/array':function(){
define("dojox/lang/functional/array", ["dojo/_base/kernel", "dojo/_base/lang", "dojo/_base/array", "./lambda"],
	function(kernel, lang, arr, df){

// This module adds high-level functions and related constructs:
//	- array-processing functions similar to standard JS functions

// Notes:
//	- this module provides JS standard methods similar to high-level functions in dojo/_base/array.js:
//		forEach, map, filter, every, some

// Defined methods:
//	- take any valid lambda argument as the functional argument
//	- operate on dense arrays
//	- take a string as the array argument
//	- take an iterator objects as the array argument

	var empty = {};

	lang.mixin(df, {
		// JS 1.6 standard array functions, which can take a lambda as a parameter.
		// Consider using dojo._base.array functions, if you don't need the lambda support.
		filter: function(/*Array|String|Object*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary:
			//		creates a new array with all elements that pass the test
			//		implemented by the provided function.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || kernel.global; f = df.lambda(f);
			var t = [], v, i, n;
			if(lang.isArray(a)){
				// array
				for(i = 0, n = a.length; i < n; ++i){
					v = a[i];
					if(f.call(o, v, i, a)){ t.push(v); }
				}
			}else if(typeof a.hasNext == "function" && typeof a.next == "function"){
				// iterator
				for(i = 0; a.hasNext();){
					v = a.next();
					if(f.call(o, v, i++, a)){ t.push(v); }
				}
			}else{
				// object/dictionary
				for(i in a){
					if(!(i in empty)){
						v = a[i];
						if(f.call(o, v, i, a)){ t.push(v); }
					}
				}
			}
			return t;	// Array
		},
		forEach: function(/*Array|String|Object*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary:
			//		executes a provided function once per array element.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || kernel.global; f = df.lambda(f);
			var i, n;
			if(lang.isArray(a)){
				// array
				for(i = 0, n = a.length; i < n; f.call(o, a[i], i, a), ++i);
			}else if(typeof a.hasNext == "function" && typeof a.next == "function"){
				// iterator
				for(i = 0; a.hasNext(); f.call(o, a.next(), i++, a));
			}else{
				// object/dictionary
				for(i in a){
					if(!(i in empty)){
						f.call(o, a[i], i, a);
					}
				}
			}
			return o;	// Object
		},
		map: function(/*Array|String|Object*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary:
			//		creates a new array with the results of calling
			//		a provided function on every element in this array.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || kernel.global; f = df.lambda(f);
			var t, n, i;
			if(lang.isArray(a)){
				// array
				t = new Array(n = a.length);
				for(i = 0; i < n; t[i] = f.call(o, a[i], i, a), ++i);
			}else if(typeof a.hasNext == "function" && typeof a.next == "function"){
				// iterator
				t = [];
				for(i = 0; a.hasNext(); t.push(f.call(o, a.next(), i++, a)));
			}else{
				// object/dictionary
				t = [];
				for(i in a){
					if(!(i in empty)){
						t.push(f.call(o, a[i], i, a));
					}
				}
			}
			return t;	// Array
		},
		every: function(/*Array|String|Object*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary:
			//		tests whether all elements in the array pass the test
			//		implemented by the provided function.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || kernel.global; f = df.lambda(f);
			var i, n;
			if(lang.isArray(a)){
				// array
				for(i = 0, n = a.length; i < n; ++i){
					if(!f.call(o, a[i], i, a)){
						return false;	// Boolean
					}
				}
			}else if(typeof a.hasNext == "function" && typeof a.next == "function"){
				// iterator
				for(i = 0; a.hasNext();){
					if(!f.call(o, a.next(), i++, a)){
						return false;	// Boolean
					}
				}
			}else{
				// object/dictionary
				for(i in a){
					if(!(i in empty)){
						if(!f.call(o, a[i], i, a)){
							return false;	// Boolean
						}
					}
				}
			}
			return true;	// Boolean
		},
		some: function(/*Array|String|Object*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary:
			//		tests whether some element in the array passes the test
			//		implemented by the provided function.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || kernel.global; f = df.lambda(f);
			var i, n;
			if(lang.isArray(a)){
				// array
				for(i = 0, n = a.length; i < n; ++i){
					if(f.call(o, a[i], i, a)){
						return true;	// Boolean
					}
				}
			}else if(typeof a.hasNext == "function" && typeof a.next == "function"){
				// iterator
				for(i = 0; a.hasNext();){
					if(f.call(o, a.next(), i++, a)){
						return true;	// Boolean
					}
				}
			}else{
				// object/dictionary
				for(i in a){
					if(!(i in empty)){
						if(f.call(o, a[i], i, a)){
							return true;	// Boolean
						}
					}
				}
			}
			return false;	// Boolean
		}
	});
	
	return df;
});

},
'dojox/lang/functional/object':function(){
define("dojox/lang/functional/object", ["dojo/_base/kernel", "dojo/_base/lang", "./lambda"], function(kernel, lang, df){

// This module adds high-level functions and related constructs:
//	- object/dictionary helpers

// Defined methods:
//	- take any valid lambda argument as the functional argument
//	- skip all attributes that are present in the empty object
//		(IE and/or 3rd-party libraries).

	var empty = {};

	lang.mixin(df, {
		// object helpers
		keys: function(/*Object*/ obj){
			// summary:
			//		returns an array of all keys in the object
			var t = [];
			for(var i in obj){
				if(!(i in empty)){
					t.push(i);
				}
			}
			return	t; // Array
		},
		values: function(/*Object*/ obj){
			// summary:
			//		returns an array of all values in the object
			var t = [];
			for(var i in obj){
				if(!(i in empty)){
					t.push(obj[i]);
				}
			}
			return	t; // Array
		},
		filterIn: function(/*Object*/ obj, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary:
			//		creates new object with all attributes that pass the test
			//		implemented by the provided function.
			o = o || kernel.global; f = df.lambda(f);
			var t = {}, v, i;
			for(i in obj){
				if(!(i in empty)){
					v = obj[i];
					if(f.call(o, v, i, obj)){ t[i] = v; }
				}
			}
			return t;	// Object
		},
		forIn: function(/*Object*/ obj, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary:
			//		iterates over all object attributes.
			o = o || kernel.global; f = df.lambda(f);
			for(var i in obj){
				if(!(i in empty)){
					f.call(o, obj[i], i, obj);
				}
			}
			return o;	// Object
		},
		mapIn: function(/*Object*/ obj, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary:
			//		creates new object with the results of calling
			//		a provided function on every attribute in this object.
			o = o || kernel.global; f = df.lambda(f);
			var t = {}, i;
			for(i in obj){
				if(!(i in empty)){
					t[i] = f.call(o, obj[i], i, obj);
				}
			}
			return t;	// Object
		}
	});
	
	return df;
});

},
'epi-cms/ApplicationSettings':function(){
﻿define("epi-cms/ApplicationSettings", [],
    function () {
        return {
        // summary:
        //      epi-cms/ApplicationSettings
        // description:
        //      Application settings for CMS.
        // tags:
        //      public

        };
    });

},
'epi-find/ApplicationState':function(){
﻿define("epi-find/ApplicationState", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/config",
    "dojo/Stateful",
    "dojo/when"
], function (declare,
             lang,
             config,
             Stateful,
             when) {
    var ApplicationState = declare([Stateful], {
            // current: dojo/Stateful
            //      Current state bag
            current: null,

            // _stateHistory: Array
            //      State history
            _stateHistory: null,

            // widgetsInAnimation: Array
            //      Array of widgets currently animating
            widgetsInAnimation: null,
            
            username: null,

            context: null,

            site: null,

            language: null,

            reset: function() {
                // summary:
                //      Reset current state
               if (!this.current) {
                   return;
               }
               for (var prop in this.current) {
                   if (this.current.hasOwnProperty(prop)) {
                       delete this.current[prop];
                   }
               }
            },

            push: function() {
                // summary:
                //      Push current state to the state history
                if (!this.current) {
                    return;
                }
                var currentState = this.current;
                this._stateHistory.push(currentState);
                this.current = new Stateful();
            },

            pop: function() {
                // summary:
                //      Pop the last state from the history
                var currentState = this._stateHistory.pop();
                if (!currentState) {
                    this.reset();
                }
                else {
                    this.current = currentState;
                }

                return this.current;
            },
            
            _contextGetter: function() {
                if (!this.context) {
                    var contextStore = config.dependencies["epi-find.ContextStore"];
                    this.context = contextStore.query({}).then(lang.hitch(this, function (value) {
                        this.set("context", value);
                        return value;
                    }));
                }
                return this.context;
            },
            
            _contextSetter: function(value) {
                this.set("username", value.Username);
            },

            isThisMe: function (username) {
                return username && username.toLowerCase() === this.get("username").toLowerCase();
            },

            constructor: function () {
                this.widgetsInAnimation = [];
                this.current = new Stateful();
                this.set("username", config.find && config.find.init ? config.find.init.username : "");
                // TODO: Consider read user name from contextStore
                this._stateHistory = [];
            }
        }
    );
    var appState = new ApplicationState();
    return appState;
});

},
'epi-find/IndexStateModel':function(){
﻿define("epi-find/IndexStateModel", [
    "../dojo/_base/config",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/Stateful",
    "dojo/topic",
    "dojo/when",
    "./ConfigModel",
    "dojo/i18n!./widget/nls/IndexStateModel"
], function (
    config,
    declare,
    lang,
    Stateful,
    topic,
    when,
    ConfigModel,
    i18n
    ) {
    var IndexState = declare([Stateful], {

            // maxDocumentsInIndex: Number
            //      Limit of maximal documents count that can be saved in the index
            maxDocumentsInIndex: null,
            _maxDocumentsInIndexSetter: function(value) {
                this.maxDocumentsInIndex = value;
                this._verifyDocumentsCount();
            },

            // documentsInIndex: Number
            //      Current documents count in the index
            documentsInIndex: null,
            _documentsInIndexSetter: function(value) {
                this.documentsInIndex = value;
                this._verifyDocumentsCount();
            },

            // showWarningOnAndBelow: Number
            //      Percentage of index filling when a warning message is shown. The message can be ignored and closed by the user.
            showWarningOnAndBelow: 10,
            // showPermanentWarningOnAndBelow: Number
            //      Percentage of index filling when a warning message is shown and can not be closed by the user.
            showPermanentWarningOnAndBelow: 5,
            // showErrorOnAndBelow: Number
            //      Percentage of index filling when an error message is shown and can not be closed by the user.
            showErrorOnAndBelow: 0,

            init: function() {
                // summary:
                //      Initialize the model
                ConfigModel.watch("config", lang.hitch(this, function(name, oldValue, value) {
                    this.set("maxDocumentsInIndex", value.max_docs);
                }));
            },

            refresh: function() {
                // summary:
                //      Refresh all model parameters
                this.refreshMaxDocumentsInIndex();
                this.refreshDocumentsInIndex();
            },

            refreshDocumentsInIndex: function() {
                // summary:
                //      Refreshes documents count in the index by issuing a server request
                var countStore = config.dependencies["epi-find.ElasticsearchStore"];
                when(countStore.query({size:0}).total, lang.hitch(this, function(total) {
                    this.set("documentsInIndex", total);
                }));
            },

            refreshMaxDocumentsInIndex: function() {
                // summary:
                //      Refreshes maximal documents count in the index by refreshing the configuration
                ConfigModel.refresh();
            },

            _verifyDocumentsCount: function() {
                // summary:
                //      This method verifies if index is filled above certain limits and issues a notification message if needed.
                if (!this.maxDocumentsInIndex || !this.documentsInIndex) {
                    return;
                }
                var percentageLeft = this.percentageLeft = Math.max(Math.round((this.maxDocumentsInIndex-this.documentsInIndex)*100 / this.maxDocumentsInIndex), 0),
                    notification = {topic: "global", key: "IndexLimit"};

                if (percentageLeft > this.showWarningOnAndBelow) {
                    notification.type = "clear";
                    notification.message = null;
                }
                else if (percentageLeft > this.showPermanentWarningOnAndBelow) {
                    notification.type = "warning";
                    notification.message = this._getWarning();
                }
                else if (percentageLeft > this.showErrorOnAndBelow) {
                    notification.type = "warning";
                    notification.message = this._getWarning();
                    notification.persistent = true;
                }
                else {
                    notification.type = "error";
                    notification.message = this._getError();
                    notification.persistent = true;
                }
                topic.publish("epi/notification", notification);
            },

            _getWarning: function() {
                // summary:
                //      Creates a notification message parametrised with index filling percentage and "upgrade your account" link.
                return lang.replace(i18n.indexLimitReaching, {percent: this.percentageLeft, link: config.find.orderFindUrl});
            },

            _getError: function() {
                // summary:
                //      Creates a notification message parametrised with "upgrade your account" link.
                return lang.replace(i18n.indexLimitReached, {link: config.find.orderFindUrl});
            }
        }
    );
    var indexState = new IndexState(); // this model is a singleton.
    return indexState;
});

},
'epi-saas-base/NotificationHandler':function(){
define("epi-saas-base/NotificationHandler", [
        "dojo/_base/declare",
        "dojo/topic",

        "epi/shell/widget/dialog/Confirmation",

        "./_PropagatingDestroyableMixin",
        "./NotificationHub",

        "./widgets/Toaster",
        "./widgets/NotificationBar"
    ],
    function(declare,
             topic,
             Confirmation,
             _PropagatingDestroyableMixin,
             NotificationHub,
             Toaster,
             NotificationBar
    ) {

        return declare([_PropagatingDestroyableMixin], {
            // summary:
            //     Initializes basic notification handling and displaying.

            rootNode: null,

            constructor: function(options) {
                declare.safeMixin(this, options);
            },

            startup: function() {
                // summary:
                //      Defines basic notification handlers and creates widgets to display.
                //  tags:
                //      public
                if (!this.rootNode) {
                    throw new Error("The 'rootNode' property is not set.");
                }

                var globalNotification = new NotificationBar({
                    topics: "epi/notification/global"
                });
                globalNotification.placeAt(this.rootNode, "first");
                globalNotification.startup();
                this.own(globalNotification);

                var toaster = new Toaster({
                        topics: "epi/notification/default",
                        orientation: "bottom-left"
                    });
                toaster.placeAt(this.rootNode);
                toaster.startup();
                this.own(toaster);


                this.own(NotificationHub.addHandler(this._handleNotificationWithAction));
            },

            _handleNotificationWithAction: function (notification) {
                if (notification && notification.message && notification.action &&
                    typeof notification.action === "function") {

                    var confirmationOptions = {
                        destroyOnHide: true,
                        setFocusOnConfirmButton: true,
                        title: notification.title || notification.type || "",
                        description: notification.message,
                        onAction: function(confirmed) {
                            if (confirmed) {
                                notification.action();
                            }
                            else if (notification.cancelAction &&
                                typeof notification.cancelAction === "function") {
                                notification.cancelAction();
                            }
                        }
                    };
                    //do not override default values if undefined
                    if(notification.confirmActionText) {
                        confirmationOptions.confirmActionText = notification.confirmActionText;
                    }
                    if(notification.cancelActionText) {
                        confirmationOptions.cancelActionText = notification.cancelActionText;
                    }

                    new Confirmation(confirmationOptions).show();

                    notification.stopPropagation = true;
                }
            }
        });
    });
},
'epi-saas-base/_PropagatingDestroyableMixin':function(){
define("epi-saas-base/_PropagatingDestroyableMixin", [
    "dojo/_base/declare",
    "dijit/Destroyable"
],
function (declare, Destroyable) {
    return declare(null, {
        //  summary:
        //      A mixin that that can be used to add own/destroy methods to existing non-dijit-widget-based classes (like dgrid List)

        // _destroyableObject: dijit/Destroyable
        //      A dijit/Destroyable instance to delegate calls to
        _destroyableObject: null,

        destroy: function () {
            this.inherited(arguments);
            if (this._destroyableObject) {
                if (this._destroyableObject._destroyed !== true) {
                    this._destroyableObject.destroy();
                }
                this._destroyableObject = null;
            }
        },

        own: function () {
            if (!this._destroyableObject || this._destroyableObject._destroyed === true)
            {
                this._destroyableObject = new Destroyable();
            }
            this._destroyableObject.own.apply(this._destroyableObject, arguments);
        }
    });
});
},
'epi-saas-base/NotificationHub':function(){
define("epi-saas-base/NotificationHub", [
        "dojo/_base/declare",
        "dojo/_base/lang",
        "dojo/_base/array",
        "dojo/topic",

        "./_PropagatingDestroyableMixin"
    ],
function(declare,
         lang,
         array,
         topic,
         _PropagatingDestroyableMixin
    ) {
    var NotificationHub = declare([_PropagatingDestroyableMixin], {
        // summary:
        //      The hub object that receives notifications and allows to define handlers to process notifications before publishing them

        // notificationTopic: String
        //      Topic name that is used to receive notifications
        notificationTopic: "epi/notification",

        // defaultTopic: String
        //      Topic name that is used to publish notifications if specific topic is not defined
        defaultTopic: "epi/notification/default",

        _topicSeparator: "/",

        _handlers: null,

        _defaultHandlerOrder: 0,

        _subscriptionHandle: null,

        constructor: function () {
            this._handlers = [];

            this.own(topic.subscribe(this.notificationTopic, lang.hitch(this, this.submit)));
        },

        addHandler: function (/*Object*/ handler) {
            // summary:
            //      Add specified method or object as a handler for incoming notifications.
            // handler:
            //      Function to process notification or object that must have handle function and may have an order property
            //      (integer value that defines the order of execution).
            if (handler) {
                if (typeof handler === "function" ) {
                    handler = { handle: handler };
                }

                if (handler.handle &&
                    typeof handler.handle === "function" &&
                    !array.some(this._handlers, function(item){
                        return item === handler ||
                               item.handle === handler.handle;
                    })) {
                    this._handlers.push(handler);
                    this._sortHandlers();
                }
                var hub = this;
                return {
                    remove: function () {
                        if (hub) {
                            hub.removeHandler(handler);
                            hub = null;
                            handler = null;
                        }
                    }
                };
            }
            return null;
        },

        removeHandler: function (/*Object*/ handler) {
            // summary:
            //      Removes specified object from the list of handlers, so this handler will not process incoming notifications.
            // handler:
            //      Function or object that was added as a handler before.
            if (!handler) {
                return;
            }
            var handlersToRemove = array.filter(this._handlers, function(item){
                return item === handler || item.handle === handler;
            });
            array.forEach(handlersToRemove, function (handlerToRemove) {
                var index = this._handlers.indexOf(handlerToRemove);
                if (index > -1) {
                    this._handlers.splice(index, 1);
                }
            }, this);
        },

        submit: function (/*Object|String*/ notification) {
            // summary:
            //      Executes defined handlers for notification and publishes the notification.
            //      Execution of handlers stops and notification is not published if one of the handlers sets stopPropagation property to true.
            //      Processed notification is published using topic notificationTopic/notification.topic if topic property is defined,
            //      for example: { message: "hello", topic: "find/optimization" } is going to be published using topic "epi/notification/find/optimization".
            //      Processed notification is published in defaultTopic "epi/notification/default" if topic property is not defined.
            // notification:
            //      Notification object to process and publish. Notification can have topic property.
            if (!notification) {
                return;
            }
            if (typeof notification === "string" || notification instanceof String) {
                notification = {
                    message: notification
                };
            }
            array.forEach(this._handlers, function (handler) {
                if (!notification.stopPropagation) {
                    handler.handle(notification);
                }
            }, this);
            if (!notification.stopPropagation) {
                this._publishNotification(notification);
            }
        },

        reset: function () {
            // summary:
            //      Clears the list of defined handlers.
            this._handlers = [];
        },

        _publishNotification: function (notification) {
            var notificationTopic = (notification.topic || "").trim();
            if (notificationTopic) {
                notificationTopic = this.notificationTopic + this._topicSeparator + notificationTopic;
            }
            else {
                if (notification.topic !== undefined) {
                    notification.topic = null;
                }
                notificationTopic = this.defaultTopic;
            }
            topic.publish(notificationTopic, notification);
        },

        _sortHandlers: function () {
            this._handlers.sort(lang.hitch(this, function (handler1, handler2) {
                return (handler1.order || this._defaultHandlerOrder) -
                    (handler2.order || this._defaultHandlerOrder);
            }));
        }
    });

    var hub = new NotificationHub();
    return hub;
});

},
'epi-saas-base/widgets/Toaster':function(){
define("epi-saas-base/widgets/Toaster", [
    "dojo/_base/declare",
    "dojo/dom-class",
    "dojo/query",

    "./NotificationBar",
    "./ToasterNotificationItem"
], function (declare,
         domClass,
         query,
         NotificationBar,
         ToasterNotificationItem) {

    return declare([NotificationBar], {
        rootNode: null,

        orientation: null,

        _defaultItemType: ToasterNotificationItem,

        postCreate: function () {
            this.inherited(arguments);

            if (!this.rootNode) {
                this.rootNode = query("body")[0];
            }

            if (!this.orientation) {
                this.orientation = "top-right";
            }

            domClass.add(this.domNode, "epi-notification-container");
            domClass.add(this.domNode, "epi-notification-container--toaster");
            domClass.add(this.domNode, "epi-notification-toaster--" + this.orientation);
        },

        startup: function () {
            if (this._started) {
                return;
            }
            this.placeAt(this.rootNode);
            this.inherited(arguments);
        },

        _addItemWidget: function (notificationWidget) {
            this.addChild(notificationWidget, this.orientation.indexOf("bottom") >= 0 ? "first" : "last");
        }
    });
});

},
'epi-saas-base/widgets/NotificationBar':function(){
require({cache:{
'url:epi-saas-base/widgets/templates/NotificationBar.html':"<div data-dojo-attach-point=\"containerNode\">\r\n</div>"}});
define("epi-saas-base/widgets/NotificationBar", [
    "dojo/_base/declare",
    "dojo/_base/array",
    "dojo/_base/lang",
    "dojo/topic",

    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/_Container",

    "./NotificationItem",

    "dojo/text!./templates/NotificationBar.html"
], function (declare,
             array,
             lang,
             topic,
             WidgetBase,
             _TemplatedMixin,
             WidgetsInTemplateMixin,
             _Container,
             NotificationItem,
             template
             ) {

    return declare([WidgetBase, _TemplatedMixin, WidgetsInTemplateMixin, _Container], {
        // summary:
        //      A notification bar widget - a placeholder for notifications.
        //  description:
        //      This widget accepts a topic (or array of topics) to listen to in order to receive notifications that
        //      should be displayed.
        //      "Notification" is a string or an object. Some properties specified in the notification are used by the
        //      NotificationBar itself and some are processed by the NotificationItem.
        //      Here is notification properties:
        //          type: String - a notification type. Can be "clear", "ok", "warning", "error" or any other supported
        //                          by the notification items.
        //          itemType: Widget type - a type of the notification item widget that should be instantiated when a
        //                          new notification is received if type is not defined in the notification object.
        //                          _defaultItemType is used if itemType is not specified.
        //                          _defaultItemType is set to NotificationItem by default.
        //          key: String - a grouping key for notifications related to the same kind. This can be used to avoid
        //                          duplication of messages with the same kind.
        //          message: String - a text that should be shown in the notification
        //          content: Widget - a widget that should be shown in the notification (can be combined with message)
        //          persistent: Boolean - indicates if notification can be removed by the user by clicking
        //                          on the close button
        templateString: template,

        // topics: Array|String
        //      Topics to subscribe in order to recieve messages
        topics: null,

        // itemType: Widget type
        //      A widget type used for notifications if it is not defined in notification object
        itemType: null,

        _defaultItemType: NotificationItem,

        CLEAR_TYPE: "clear",

        postCreate: function () {
            this.inherited(arguments);

            this.itemType = this.itemType || this._defaultItemType;

            if(!(this.topics instanceof Array))
            {
                this.topics = [this.topics];
            }
            this.topics = array.filter(this.topics, function (item) {
                return typeof item === "string" || item instanceof String;
            });

            array.forEach(this.topics, function (item) {
                this.own(topic.subscribe(item, lang.hitch(this, this.addNotification)));
            }, this);
        },

        addNotification: function (notification) {
            // summary:
            //      Adds a new notification to the notification bar.

            if (typeof notification === "string" || notification instanceof String) {
                notification = {
                    message: notification
                };
            }

            // remove notifications with the same key or all if no key specified and type is "clear"
            array.forEach(this.getChildren(), function (item) {
                if ((notification.key && item.key === notification.key)
                    || (!notification.key && notification.type === this.CLEAR_TYPE)) {
                    item.destroy();
                }
            }, this);

            // items with the same key should be removed by now,
            // if notification type is "clear" - we have nothing to do any more.
            if (notification.type === this.CLEAR_TYPE) {
                return;
            }

            var notificationWidget = this._createItemWidget(notification);
            this.own(notificationWidget);

            this._addItemWidget(notificationWidget);

            if (notificationWidget.show &&
                typeof notificationWidget.show === "function") {
                notificationWidget.show();
            }
        },

        _createItemWidget: function (notification) {
            // resolve type of notification widget and instantiate it
            var ItemType = notification.itemType || this.itemType;
            return new ItemType(notification);
        },

        _addItemWidget: function (notificationWidget) {
            this.addChild(notificationWidget);
        }

    });
});

},
'epi-saas-base/widgets/NotificationItem':function(){
require({cache:{
'url:epi-saas-base/widgets/templates/NotificationItem.html':"<div class=\"epi-notification\">\r\n    <span data-dojo-attach-point=\"closeIconNode\" class=\"dijitInline  dijitReset  dijitIcon  epi-iconClose  epi-icon--inverted\"></span>\r\n    <span data-dojo-attach-point=\"iconNode\" class=\"dijitInline  dijitReset  dijitIcon  epi-icon--inverted\"></span>\r\n\r\n    <span data-dojo-attach-point=\"containerNode\" class=\"epi-notification__header\">${title}</span>\r\n    <span class=\"epi-notification__text\">${message}</span>\r\n\r\n    <div data-dojo-attach-point=\"detailsContainer\" class=\"epi-notification__details  epi-hidden\">\r\n        <hr class=\"epi-separator\" />\r\n        <span data-dojo-attach-point=\"toggleDetailsButton\" class=\"epi-notification__details__toggle\">\r\n            <span data-dojo-attach-point=\"toggleDetailsNode\" class=\"dijitReset dijitInline dijitIcon epi-disclosure-arrow  epi-icon--inverted\"></span> ${detailsTitle}\r\n        </span>\r\n        <span class=\"epi-notification__text  is-hidden\" data-dojo-attach-point=\"detailsNode\">${details}</span>\r\n    </div>\r\n</div>"}});
define("epi-saas-base/widgets/NotificationItem", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/on",
    "dojo/dom-class",
    "dojo/query",

    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/_Container",

    "dojo/i18n!./nls/Notifications",
    "dojo/text!./templates/NotificationItem.html"
], function (declare,
             lang,
             on,
             domClass,
             query,
             WidgetBase,
             _TemplatedMixin,
             WidgetsInTemplateMixin,
             _Container,
             i18n,
             template
             ) {

    return declare([WidgetBase, _TemplatedMixin, WidgetsInTemplateMixin, _Container], {
        templateString: template,

        // Nodes from widgets template
        closeIconNode: null,
        iconNode: null,

        // title: String
        //      A text for notification title
        title: null,

        // message: String
        //      A text for notification message
        message: null,

        // detailsTitle: String
        //      A text for notification details title
        detailsTitle: null,

        // details: String
        //      A text for notification details message
        details: null,

        // content: Widget
        //      A widget inserted in the notification
        content: null,
        // type: String
        //      Notification type: "warning", "error"
        type: null,

        // persistent: Boolean
        //      If true - notification should not be hidden automatically
        persistent: false,

        // closable: Boolean
        //      If true - notification can not be closed by the user
        closable: true,

        // showOnStartup: Boolean
        //      If true - notification should be displayed when startup is being executed (show() method is called)
        showOnStartup: true,

        _defaultStyles: {
            "success": {
                "icon": "epi-iconSuccess",
                "background": "epi-background--success"
            },
            "info": {
                "icon": "epi-iconInfo",
                "background": "epi-background--info"

            },
            "warning": {
                "icon": "epi-iconWarning",
                "background": "epi-background--warning"

            },
            "error": {
                "icon": "epi-iconError",
                "background": "epi-background--danger"

            },
            "fatal": {
                "icon": "epi-iconError",
                "background": "epi-background--danger"
            }
        },

        postMixInProperties: function () {
            this.inherited(arguments);
            this.closable = !this.persistent;

            this.title = this.title || "";
            this.message = this.message || "";
            if (this.message && !this.title) {
                this.title = this.message;
                this.message = "";
            }

            this.details = this.details || "";
            this.detailsTitle = this.detailsTitle || i18n.detailsDefaultTitle;

            this.type = this.type || "info";

            this.iconClass = this._getIconClass(this) || this.iconClass;
            this.backgroundClass = this.backgroundClass || this._getBackgroundClass(this);
        },

        postCreate: function () {
            this.inherited(arguments);

            if (this.type === "info") {
                query(".epi-icon--inverted", this.domNode).forEach(function(node){
                    domClass.remove(node, "epi-icon--inverted");
                });
            }

            if (this.closable) {
                domClass.add(this.domNode, "epi-notification--closable");
                this.own(
                    on(this.closeIconNode, "click", lang.hitch(this, function () {
                        this.hide();
                    }))
                );
            }
            else {
                domClass.add(this.closeIconNode, "epi-hidden");
            }

            if (this.iconClass) {
                domClass.add(this.iconNode, this.iconClass);
            }

            if (this.backgroundClass) {
                domClass.add(this.domNode, this.backgroundClass);
            }

            if (this.details) {
                domClass.remove(this.detailsContainer, "epi-hidden");
                this.own(
                    on(this.toggleDetailsButton, "click", lang.hitch(this, function (e) {
                        e.preventDefault();
                        e.stopPropagation();
                        domClass.toggle(this.detailsNode, "is-hidden");
                        domClass.toggle(this.toggleDetailsNode, "epi-disclosure-arrow--down");
                    }))
                );
            }

            if (this.content) {
                this.addChild(this.content);
            }
        },

        startup: function () {
            this.inherited(arguments);
            if (this.showOnStartup) {
                this.show();
            }
        },

        show: function()
        {
        },

        hide: function()
        {
            if (this._destroyed) {
                return;
            }
            this.destroy();
        },

        _getIconClass: function (notification) {
            if (!notification) {
                return null;
            }
            var type = (notification.type || "info").toLowerCase();
            var styles = this._defaultStyles[type];
            return  styles ? styles.icon : null;
        },

        _getBackgroundClass: function (notification) {
            if (!notification) {
                return null;
            }
            var type = (notification.type || "info").toLowerCase();
            var styles = this._defaultStyles[type];
            return  styles ? styles.background : null;
        }

    });
});
},
'url:epi-saas-base/widgets/templates/NotificationItem.html':"<div class=\"epi-notification\">\r\n    <span data-dojo-attach-point=\"closeIconNode\" class=\"dijitInline  dijitReset  dijitIcon  epi-iconClose  epi-icon--inverted\"></span>\r\n    <span data-dojo-attach-point=\"iconNode\" class=\"dijitInline  dijitReset  dijitIcon  epi-icon--inverted\"></span>\r\n\r\n    <span data-dojo-attach-point=\"containerNode\" class=\"epi-notification__header\">${title}</span>\r\n    <span class=\"epi-notification__text\">${message}</span>\r\n\r\n    <div data-dojo-attach-point=\"detailsContainer\" class=\"epi-notification__details  epi-hidden\">\r\n        <hr class=\"epi-separator\" />\r\n        <span data-dojo-attach-point=\"toggleDetailsButton\" class=\"epi-notification__details__toggle\">\r\n            <span data-dojo-attach-point=\"toggleDetailsNode\" class=\"dijitReset dijitInline dijitIcon epi-disclosure-arrow  epi-icon--inverted\"></span> ${detailsTitle}\r\n        </span>\r\n        <span class=\"epi-notification__text  is-hidden\" data-dojo-attach-point=\"detailsNode\">${details}</span>\r\n    </div>\r\n</div>",
'url:epi-saas-base/widgets/templates/NotificationBar.html':"<div data-dojo-attach-point=\"containerNode\">\r\n</div>",
'epi-saas-base/widgets/ToasterNotificationItem':function(){
define("epi-saas-base/widgets/ToasterNotificationItem", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-class",
    "dojo/mouse",
    "dojo/on",

    "./NotificationItem"
], function (declare,
             lang,
             domClass,
             mouse,
             on,
             NotificationItem) {
    return declare([NotificationItem], {

        duration: 5000,

        _hideTimeout: null,

        postCreate: function()
        {
            // Toaster notifications are always closable
            this.closable = true;

            this.inherited(arguments);

            if (this.duration <= 0) {
                this.duration = 0;
                this.persistent = true;
            }

            this.showOnStartup = false;
        },

        startup: function () {
            this.inherited(arguments);
            if(!this.persistent) {
                this._delayedHide();
            }

            this.own(
                on(this.domNode, mouse.enter, lang.hitch(this, function() {
                    clearTimeout(this._hideTimeout); // do not hide automatically if user hovers mouse pointer over the message
                })),
                on(this.domNode, mouse.leave, lang.hitch(this, function() {
                    if (!this.persistent) {
                        this._delayedHide(); // hide automatically when user moves mouse pointer away
                    }
                }))
            );
        },

        hide: function()
        {
            if (this._destroyed) {
                return;
            }
            var self = this;
            setTimeout(function() {
                on.once(self.domNode, "transitionend", function() {
                   self.destroy();
                });
                domClass.add(self.domNode, "epi-notification--animate-out");
            }, 10);
        },

        _delayedHide: function() {
            this._hideTimeout = setTimeout(lang.hitch(this, this.hide), this.duration);
        }

    });
});

},
'epi-find/CommonModule':function(){
﻿define("epi-find/CommonModule", [
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/config",
    "dojo/store/Memory",
    "epi/routes",
    "epi/dependency",
    "epi/shell/store/JsonRest",
    "epi/shell/store/Registry",
    "epi/shell/store/Throttle",
    "epi/shell/TypeDescriptorManager",

    "epi-cms/ApplicationSettings",
    "epi-cms/store/CustomQueryEngine",

    "./ConfigModel",
    "epi-cms/contentediting/ContentHierarchyService",
    "epi-cms/ContentRepositoryDescriptorService"
],
    function(
            array,
            declare,
            config,
            Memory,
            routes,
            dependency,
            JsonRest,
            Registry,
            Throttle,
            TypeDescriptorManager,
            ApplicationSettings,
            CustomQueryEngine,
            ConfigModel,
            ContentHierarchyService,
            ContentRepositoryDescriptorService
        ) {


        var CommonModule = declare([], {
            // _initialized: bool
            //      Indicates that the initialization is complete
            _initialized: false,

            initialize: function() {
                if (this._initialized) {
                    return;
                }

                this.registerRoutes();

                this.createStores();

                if (!dependency.exists("epi.shell.Profile")) {
                    dependency.register("epi.shell.Profile", {contentLanguage: "en"});
                }
                // TODO: Why do we need to not need these? :s
                dependency.register("epi.shell.MessageService", null);
                dependency.register("epi.ModuleManager", null);
                //dependency.register("epi.commerce.global", {get: function(){return {watch: function() {}};}});
                dependency.register("epi.cms.contentRepositoryDescriptors", new ContentRepositoryDescriptorService(ConfigModel.getValue("repository_descriptors") || config.find.defaultDescriptors));
                dependency.register("epi.cms.ContentHierarchyService",  new ContentHierarchyService());

                TypeDescriptorManager.initialize();

                this._initialized = true;
            },

            registerRoutes: function() {
                var shellRoutes = {
                    "routeBasePath": config.find.siteModulesBaseUrl + "{moduleArea}/{controller}/{action}/{id}",
                    "routeDefaults": {
                        "action": "Index",
                        "id": "",
                        "moduleArea": "Shell"
                    }
                };
                routes.registerRoute(shellRoutes.routeBasePath, shellRoutes.routeDefaults);

                var findRoutes = {
                    "routeBasePath": config.find.siteModulesBaseUrl + "{moduleArea}/{controller}/{action}/{id}",
                    "routeDefaults": {
                        "action": "Index",
                        "id": "",
                        "moduleArea": "Find"
                    }
                };
                routes.registerRoute(findRoutes.routeBasePath, findRoutes.routeDefaults);
            },

            createStores: function() {
                var registry = new Registry();
                // Prepare a providers list based on default configuration or "searchProviders" configuration value
                var providers = config.find.defaultSearchProviders,
                    configuredProviders = ConfigModel.getValue("searchProviders"), providersMap = {};
                if (configuredProviders && configuredProviders.length) {
                    array.forEach(providers, function(provider) {
                        providersMap[provider.searchArea] = provider;
                    });
                    array.forEach(configuredProviders, function(provider) {
                        if (!provider || !provider.settings) {
                            return;
                        }
                        provider.settings.fullName = provider.settings.fullName || provider.settings.FullName;
                        if (!providersMap[provider.searchArea] ||
                            (provider.settings.fullName && provider.settings.fullName.indexOf(".Find") > 0)) { // prefer Find based providers
                            providersMap[provider.searchArea] = provider;
                        }
                    });
                    providers = [];
                    for (var provider in providersMap) {
                        if (providersMap.hasOwnProperty(provider)) {
                            providers.push({searchArea: provider, id: providersMap[provider].settings.fullName});
                        }
                }
                }
                // Create a combined search provider
                var pageAndFilesProviders = array.filter(providers,
                        function(item) { return (item.searchArea === "CMS/pages" || item.searchArea === "CMS/files"); }
                    ).sort(function(item1, item2) { // sort it so that pages goes before files
                        return item1.searchArea > item2.searchArea ? -1 : 1;
                    });
                var combinedProvider = {
                    searchArea: "CMS/pages&files",
                    id: array.map(pageAndFilesProviders, function(item) {return item.id;}).join()
                };
                providers.push(combinedProvider);

                // Create mock search providers store with Find providers
                var searchProvidersStore = new Memory({
                    data: providers
                });
                registry.add("epi.shell.searchproviders", searchProvidersStore);

                // Create search results store
                registry.create("epi.shell.searchresults", routes.getRestPath({ moduleArea: "find", storeName: "searchresults" }));

                registry.create("epi.shell.uidescriptor", routes.getRestPath({ moduleArea: "find", storeName: "finduidescriptor" }));
                var contentStoreName = ConfigModel.getValue("api.best_bets.content_store_name") || "content";
                registry.create("epi.cms.content.light", routes.getRestPath({ moduleArea: "find", storeName: contentStoreName}), { idProperty: "contentLink", queryEngine: CustomQueryEngine });
                registry.add("epi.cms.contentdata", {}); // this store is required by some components but not needed by the ContentSelector.
                registry.add("epi.cms.contenttype", {get: function() {}}); // this store is required by some components related to ContentSelector.

                registry.add("epi.cms.content.search",
                    new Throttle(
                        new JsonRest({
                            target: routes.getRestPath({ moduleArea: "find", storeName: contentStoreName }),
                            idProperty: "contentLink"
                        })
                    ));
                registry.add("epi.commerce.relation", {}); // this store is required by Commerce selector.

                dependency.register("epi.storeregistry", registry);
                }
        });

        var module = new CommonModule();
        return module;
    }
);

},
'epi/shell/store/Registry':function(){
﻿define("epi/shell/store/Registry", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/store/Memory",
    "epi/shell/store/EventedStore",
    "epi/shell/store/JsonRest",
    "epi/shell/store/Patchable",
    "epi/shell/store/Throttle",
    "epi/shell/store/Realtime"
], function (declare, lang, Memory, EventedStore, JsonRest, Patchable, Throttle, Realtime) {

    return declare(null, {
        // summary:
        //		Implementation of a store registry used for handling store sharing between different components.
        // tags:
        //		public

        constructor: function (options) {
            lang.mixin(this, options);
            this._stores = {};
        },

        add: function (name, store) {
            // summary:
            //		Adds a store to the collection with the given name. This will throw an
            //		error if the name already exists in the collection.
            // name: String
            //		The name to be associated with the store.
            // store: Object
            //		The store that will be added.

            if (this._stores[name]) {
                throw new Error("A store with the name '" + name + "' already exists.");
            }
            return (this._stores[name] = store);
        },

        create: function (name, url, options) {
            // summary:
            //		Creates an observable JsonRest store with a backing cache store and
            //		adds it to the collection with the given name. This will throw an error
            //		if the name already exists in the collection.
            // name: String
            //		The name to be associated with the store.
            // url: String
            //		The url to be used as the JsonRest store target.
            // options: Object
            //		This provides any configuration information that will be mixed into the store.

            if (this._stores[name]) {
                throw new Error("A store with the name '" + name + "' already exists.");
            }

            var memory = new Memory(options),
                rest = new JsonRest(lang.mixin({
                    target: url,
                    preventCache: true
                }, options)),

                // Allow only one concurrent get to the server for the same data.
                patchable = Patchable(Throttle(rest, "get"), memory),

                evented = EventedStore(patchable),

                // The Observable isn't happy about several concurrent query calls.
                store = Throttle(evented, "query");

            if (options && options.realtimeInfo) {
                store = Realtime(store, options.realtimeInfo);
            }

            return (this._stores[name] = store);
        },

        get: function (name) {
            // summary:
            //		Get a store by its registered name.
            // name: String
            //		The name of the store to return.

            var store = this._stores[name];
            if (!store) {
                throw new Error("No store by the name '" + name + "' exists.");
            }
            return store;
        }
    });
});

},
'epi/shell/store/EventedStore':function(){
define("epi/shell/store/EventedStore", [
    "dojo/Evented",
    "dojo/when"
], function (Evented, when) {

    return function (store) {
        // summary:
        //      The evented store wrapper takes a store and wraps its methods in order to emit
        //      dstore-like events when the methods are executed. Also adds the evented methods on
        //      and emit to the store object.
        // store:
        //      The store to make evented.
        // tags:
        //      internal

        // Create an evented object and add evented methods to the store that delegate to the
        // evented object.
        var evented = new Evented();
        store.emit = evented.emit.bind(evented);
        store.on = evented.on.bind(evented);

        var inMethod;
        function whenFinished(method, action) {
            var original = store[method];
            if (original) {
                store[method] = function () {
                    var args = arguments;
                    // If one method calls another (like add calling put or refresh calling evict)
                    // we don't want two events.
                    if (inMethod) {
                        return original.apply(this, args);
                    }
                    inMethod = true;
                    try {
                        var results = original.apply(this, args);
                        when(results).then(function (result) {
                            action(result, args);
                        });
                        return results;
                    } finally {
                        inMethod = false;
                    }
                };
            }
        }

        // Emits a dstore-like update event.
        function emitUpdateEvent(type) {
            return function (result) {
                when(result).then(function (result) {
                    store.emit(type, { target: result });
                });
                return result;
            };
        }

        // Emits a dstore-like remove event.
        function emitRemoveEvent(type) {
            return function (result, args) {
                when(result).then(function () {
                    store.emit(type, { id: args[0] });
                });
                return result;
            };
        }

        // Wrap the methods that should generate events.
        whenFinished("add", emitUpdateEvent("update")); // to match the local and remote client events, we need to emit update on both add/put
        whenFinished("put", emitUpdateEvent("update"));
        whenFinished("refresh", emitUpdateEvent("update"));
        whenFinished("remove", emitRemoveEvent("delete"));
        whenFinished("evict", emitRemoveEvent("delete"));

        return store;
    };
});

},
'epi/shell/store/Patchable':function(){
﻿define("epi/shell/store/Patchable", [
    "dojo/_base/lang",
    "dojo/aspect",
    "dojo/Deferred",
    "dojo/promise/all",
    "dojo/store/Cache",
    "dojo/store/Observable",
    "dojo/when",
    "epi/shell/store/storeDelegate"
], function (lang, aspect, Deferred, all, Cache, Observable, when, storeDelegate) {

    return function (masterStore, cachingStore, /*dojo/store/__CacheArgs*/options) {
        // masterStore:
        //		This is the authoritative store, all uncached requests or non-safe requests will
        //		be made against this store.
        // cachingStore:
        //		This is the caching store that will be used to store responses for quick access.
        //		Typically this should be a local store.
        // options:
        //		These are additional options for how caching is handled.
        //
        // tags:
        //      internal

        var cache = Cache(masterStore, cachingStore, options || {}),
            dependencies = [];

        var checkTypeEquality = function (queriedValue, actualValue) {
            if (typeof queriedValue !== typeof actualValue) {
                console.error("The store was queried with a [" + typeof queriedValue + "] but the entity id is a [" + typeof actualValue + "]");
            }
        };

        // Override put and add methods in Cache since the default implementation doesn't update returned id.
        var updateId = function (object, result) {
            // now put result in cache
            if (typeof result == "object") {
                object[store.idProperty] = result[store.idProperty]; //eslint-disable-line no-use-before-define
            } else {
                object[store.idProperty] = result; //eslint-disable-line no-use-before-define
            }
        };

        var stamp = function (object) {
            // summary: Time stamp the object
            if (object._cached) {
                object._cached = new Date();
            } else {
                Object.defineProperty(object, "_cached", {
                    value: new Date(),
                    enumerable: false
                });
            }

            return object;
        };

        var isOld = function (object) {
            //If the object does not have a cache key, then it is not considered as old
            if (!object._cached) {
                return false;
            }

            //Check if the object is older than 5 minutes, if so consider it as old
            var now = new Date();
            now.setMinutes(now.getMinutes() - 5);

            return now > object._cached;
        };

        cache.get = function (id, directives) {
            return when(cachingStore.get(id), function (result) {

                if (result) {
                    if (isOld(result)) {
                        //If the result is old, remove it and get new data
                        cache.evict(id, directives);
                        result = null;
                    }
                }

                return result || when(masterStore.get(id, directives), function (result) {
                    if (result) {
                        checkTypeEquality(id, result[masterStore.idProperty]);
                        cachingStore.put(stamp(result), { id: id });
                    }
                    return result;
                });
            });
        };

        cache.put = function (object, directives) {
            // first remove from the cache, so it is empty until we get a response from the master store
            cachingStore.remove((directives && directives.id) || this.getIdentity(object));
            return when(masterStore.put(object, directives), function (result) {
                updateId(object, result);
                cachingStore.put(typeof result == "object" ? stamp(result) : stamp(object), directives);
                return result;
            });
        };

        cache.add = function (object, directives) {
            return when(masterStore.add(object, directives), function (result) {
                // now put result in cache
                updateId(object, result);
                cachingStore.add(typeof result == "object" ? stamp(result) : stamp(object), directives);
                return result;
            });
        };

        var _isObject = function (obj) {
            // summary:
            //      Returns true if obj is a non-null object, but not an array or function.

            return obj && lang.isObject(obj) && !lang.isArray(obj) && !lang.isFunction(obj);
        };

        var _mixin = function (dest, source) {
            // summary:
            //      Updates properties in dest using properties from source.
            //      Differences from dojo/mixin are:
            //          * When mixing in recursively this mixin method merges objects
            //            whereas dojo/mixin overwrites sub-properties in dest.
            //          * Properties that doesn't exist in dest won't be created

            var name, s, empty = {};

            for (name in source) {
                // the (!(name in empty) || empty[name] !== s) condition avoids copying properties in "source"
                // inherited from Object.prototype.	 For example, if dest has a custom toString() method,
                // don't overwrite it with the toString() method that source inherited from Object.prototype

                s = source[name];
                if (name in dest && (dest[name] !== s && (!(name in empty) || empty[name] !== s))) {

                    // When both source and target are objects merge recursively.
                    // objects, arrays and functions are mixed
                    if (_isObject(s) && _isObject(dest[name])) {
                        _mixin(dest[name], s);
                    } else {
                        dest[name] = s;
                    }
                }
            }

            return dest;
        };

        var observable = Observable(cache);

        // We need to create a store wrapper which can trigger notify from obserable to patchable and vice versa.
        // Because dojo's aspect doesn't work properly with the lang.delegate
        var store = storeDelegate(observable, {

            getCached: function (id) {
                // summary:
                //		Returns the item from cache if it exists. Doesn't make any call to backend.
                // id: Number
                //		The identifier for the object in question.
                // tags:
                //      internal

                return cachingStore.get(id);
            },

            refresh: function (id, silent) {
                // summary:
                //		Removes the object with the given id from the underlying caching store if it exists
                //		and then gets the object with the given id from the master store and caches it again.
                // id: Number
                //		The identifier for the object in question.
                // silent: Boolean
                //		Prevent notifications on the item refreshed is set to true.
                // returns: Object
                //		The updated object in the store that matches the given id.
                // tags:
                //		public

                return when(this.evict(id), lang.hitch(this, function () {
                    return when(this.get(id), lang.hitch(this, function (result) {
                        if (result) {
                            !silent && this.notify(result, id);
                            this.updateDependentStores(result);
                            !silent && this.onItemChanged(id, result);
                        }
                        return result;
                    }));
                }));
            },

            removeRange: function (ids) {
                // summary:
                //      Removes a range of entities based on their IDs.
                // ids: Array
                //      An array of the IDs to be removed.
                // returns: Promise
                //      A promise that resolves to the results of the evictions and removal.
                // tags:
                //      internal

                var self = this;

                // Evict each ID from the cache and create an array of the resulting promises.
                var evictionPromises = ids.map(function (id) {
                    self.evict(id);
                });

                // Execute the delete range method on the store and add the
                // resulting promise to the list of promises.
                var result = all(evictionPromises)
                    .then(function () {
                        return when(self.executeMethod("DeleteRange", "", ids));
                    });

                // Notify for each removal after the result has resolved.
                result.then(function () {
                    ids.forEach(function (id) {
                        self.notify(undefined, id);
                    });
                });

                return result;
            },

            patchCache: function (object) {
                // summary:
                //		Patches an object in the caching store but not in the master store.
                // object: Object
                //		The object to update in the caching store.
                // returns: Object
                //		The updated object in the caching store.
                // tags:
                //		public

                if (!object) {
                    return;
                }

                var id = this.getIdentity(object);
                return when(this.get(id), lang.hitch(this, function (result) {
                    if (result) {
                        result = _mixin(result, object);
                        when(cachingStore.put(result, { overwrite: true }), lang.hitch(this, function () {
                            this.notify(result, id);
                            this.updateDependentStores(object);
                            this.onItemChanged(id, result);
                        }));
                    }
                    return result;
                }));
            },

            addDependentStore: function (store, options) {
                // summary:
                //		Adds a dependent store which will get updated when this store is updated.
                // store: epi/shell/store/Patchable
                //		The store to update when this store gets updated.
                // options: Object (Optional)
                //      options.refreshOnPatch: bool
                //          If set to true the a refresh will be done against the store instead of a transformDelegate
                //      options.refresh: function(Optional)
                //          If specifed the default refresh behaviour is overriden
                //      transformDelegate: function (Optional)
                //          A function taking the patch object as argument and returning a transformed
                //          object for patching the store. Take care not to modify the source object,
                //          since that would cause side effects in other dependency patched stores.
                // tags:
                //		public
                //
                // example:
                //      In the following example store will be updated whenever authoritativeStore is updated
                //      and name will be inserted as itemName.
                //  |   authoritativeStore.addDependentStore(store, function(sourceObject){
                //  |       return {
                //  |           itemName: sourceObject.name,
                //  |       };
                //  |   }));

                if (!store.patchCache) {
                    throw new Error("addDependentStore: The store must implement the patch method.");
                }
                dependencies.push({ store: store, options: options || {} });
            },

            updateDependentStores: function (object) {
                // summary:
                //		Adds a dependent store which will get updated when this store is updated.
                // object: Object
                //		The object to update in the dependent stores.
                // tags:
                //		public

                for (var i = dependencies.length - 1; i >= 0; i--) {
                    var store = dependencies[i].store;

                    if (dependencies[i].options.refreshOnPatch) {
                        var refresh = dependencies[i].options.refresh;
                        if (typeof refresh === "function") {
                            refresh(object);
                        } else {
                            store.refresh(this.getIdentity(object));
                        }
                    } else {

                        var transform = dependencies[i].options.transformDelegate;
                        store.patchCache(typeof transform === "function" ? transform(object) : object);
                    }
                }
            },

            onItemChanged: function (id, item) {
                // summary:
                //		Notification event when an item has been patched or reloaded.
                //      Deprecated, use notify instead.
                // id: object
                //		The id of the updated item.
                // item: object
                //		The updated item.
                // tags:
                //		public deprecated
            }
        });

        return store;
    };
});

},
'dojo/store/Cache':function(){
define("dojo/store/Cache", ["../_base/lang","../_base/Deferred" /*=====, "../_base/declare", "./api/Store" =====*/],
function(lang, Deferred /*=====, declare, Store =====*/){

// module:
//		dojo/store/Cache

var Cache = function(masterStore, cachingStore, options){
	options = options || {};
	return lang.delegate(masterStore, {
		query: function(query, directives){
			var results = masterStore.query(query, directives);
			results.forEach(function(object){
				if(!options.isLoaded || options.isLoaded(object)){
					cachingStore.put(object);
				}
			});
			return results;
		},
		// look for a queryEngine in either store
		queryEngine: masterStore.queryEngine || cachingStore.queryEngine,
		get: function(id, directives){
			return Deferred.when(cachingStore.get(id), function(result){
				return result || Deferred.when(masterStore.get(id, directives), function(result){
					if(result){
						cachingStore.put(result, {id: id});
					}
					return result;
				});
			});
		},
		add: function(object, directives){
			return Deferred.when(masterStore.add(object, directives), function(result){
				// now put result in cache
				cachingStore.add(typeof result == "object" ? result : object, directives);
				return result; // the result from the add should be dictated by the masterStore and be unaffected by the cachingStore
			});
		},
		put: function(object, directives){
			// first remove from the cache, so it is empty until we get a response from the master store
			cachingStore.remove((directives && directives.id) || this.getIdentity(object));
			return Deferred.when(masterStore.put(object, directives), function(result){
				// now put result in cache
				cachingStore.put(typeof result == "object" ? result : object, directives);
				return result; // the result from the put should be dictated by the masterStore and be unaffected by the cachingStore
			});
		},
		remove: function(id, directives){
			return Deferred.when(masterStore.remove(id, directives), function(result){
				return cachingStore.remove(id, directives);
			});
		},
		evict: function(id){
			return cachingStore.remove(id);
		}
	});
};
lang.setObject("dojo.store.Cache", Cache);

/*=====
var __CacheArgs = {
	// summary:
	//		These are additional options for how caching is handled.
	// isLoaded: Function?
	//		This is a function that will be called for each item in a query response to determine
	//		if it is cacheable. If isLoaded returns true, the item will be cached, otherwise it
	//		will not be cached. If isLoaded is not provided, all items will be cached.
};

Cache = declare(Store, {
	// summary:
	//		The Cache store wrapper takes a master store and a caching store,
	//		caches data from the master into the caching store for faster
	//		lookup. Normally one would use a memory store for the caching
	//		store and a server store like JsonRest for the master store.
	// example:
	//	|	var master = new Memory(data);
	//	|	var cacher = new Memory();
	//	|	var store = new Cache(master, cacher);
	//
	constructor: function(masterStore, cachingStore, options){
		// masterStore:
		//		This is the authoritative store, all uncached requests or non-safe requests will
		//		be made against this store.
		// cachingStore:
		//		This is the caching store that will be used to store responses for quick access.
		//		Typically this should be a local store.
		// options: __CacheArgs?
		//		These are additional options for how caching is handled.
	},
	query: function(query, directives){
		// summary:
		//		Query the underlying master store and cache any results.
		// query: Object|String
		//		The object or string containing query information. Dependent on the query engine used.
		// directives: dojo/store/api/Store.QueryOptions?
		//		An optional keyword arguments object with additional parameters describing the query.
		// returns: dojo/store/api/Store.QueryResults
		//		A QueryResults object that can be used to iterate over.
	},
	get: function(id, directives){
		// summary:
		//		Get the object with the specific id.
		// id: Number
		//		The identifier for the object in question.
		// directives: Object?
		//		Any additional parameters needed to describe how the get should be performed.
		// returns: dojo/store/api/Store.QueryResults
		//		A QueryResults object.
	},
	add: function(object, directives){
		// summary:
		//		Add the given object to the store.
		// object: Object
		//		The object to add to the store.
		// directives: dojo/store/api/Store.AddOptions?
		//		Any additional parameters needed to describe how the add should be performed.
		// returns: Number
		//		The new id for the object.
	},
	put: function(object, directives){
		// summary:
		//		Put the object into the store (similar to an HTTP PUT).
		// object: Object
		//		The object to put to the store.
		// directives: dojo/store/api/Store.PutDirectives?
		//		Any additional parameters needed to describe how the put should be performed.
		// returns: Number
		//		The new id for the object.
	},
	remove: function(id){
		// summary:
		//		Remove the object with the specific id.
		// id: Number
		//		The identifier for the object in question.
	},
	evict: function(id){
		// summary:
		//		Remove the object with the given id from the underlying caching store.
		// id: Number
		//		The identifier for the object in question.
	}
});
=====*/

return Cache;
});

},
'dojo/store/Observable':function(){
define("dojo/store/Observable", ["../_base/kernel", "../_base/lang", "../_base/Deferred", "../_base/array" /*=====, "./api/Store" =====*/
], function(kernel, lang, Deferred, array /*=====, Store =====*/){

// module:
//		dojo/store/Observable

var Observable = function(/*Store*/ store){
	// summary:
	//		The Observable store wrapper takes a store and sets an observe method on query()
	//		results that can be used to monitor results for changes.
	//
	// description:
	//		Observable wraps an existing store so that notifications can be made when a query
	//		is performed.
	//
	// example:
	//		Create a Memory store that returns an observable query, and then log some
	//		information about that query.
	//
	//	|	var store = Observable(new Memory({
	//	|		data: [
	//	|			{id: 1, name: "one", prime: false},
	//	|			{id: 2, name: "two", even: true, prime: true},
	//	|			{id: 3, name: "three", prime: true},
	//	|			{id: 4, name: "four", even: true, prime: false},
	//	|			{id: 5, name: "five", prime: true}
	//	|		]
	//	|	}));
	//	|	var changes = [], results = store.query({ prime: true });
	//	|	var observer = results.observe(function(object, previousIndex, newIndex){
	//	|		changes.push({previousIndex:previousIndex, newIndex:newIndex, object:object});
	//	|	});
	//
	//		See the Observable tests for more information.

	var undef, queryUpdaters = [], revision = 0;
	// a Comet driven store could directly call notify to notify observers when data has
	// changed on the backend
	// create a new instance
	store = lang.delegate(store);
	
	store.notify = function(object, existingId){
		revision++;
		var updaters = queryUpdaters.slice();
		for(var i = 0, l = updaters.length; i < l; i++){
			updaters[i](object, existingId);
		}
	};
	var originalQuery = store.query;
	store.query = function(query, options){
		options = options || {};
		var results = originalQuery.apply(this, arguments);
		if(results && results.forEach){
			var nonPagedOptions = lang.mixin({}, options);
			delete nonPagedOptions.start;
			delete nonPagedOptions.count;

			var queryExecutor = store.queryEngine && store.queryEngine(query, nonPagedOptions);
			var queryRevision = revision;
			var listeners = [], queryUpdater;
			results.observe = function(listener, includeObjectUpdates){
				if(listeners.push(listener) == 1){
					// first listener was added, create the query checker and updater
					queryUpdaters.push(queryUpdater = function(changed, existingId){
						Deferred.when(results, function(resultsArray){
							var atEnd = resultsArray.length != options.count;
							var i, l, listener;
							if(++queryRevision != revision){
								throw new Error("Query is out of date, you must observe() the query prior to any data modifications");
							}
							var removedObject, removedFrom = -1, insertedInto = -1;
							if(existingId !== undef){
								// remove the old one
								for(i = 0, l = resultsArray.length; i < l; i++){
									var object = resultsArray[i];
									if(store.getIdentity(object) == existingId){
										removedObject = object;
										removedFrom = i;
										if(queryExecutor || !changed){// if it was changed and we don't have a queryExecutor, we shouldn't remove it because updated objects would be eliminated
											resultsArray.splice(i, 1);
										}
										break;
									}
								}
							}
							if(queryExecutor){
								// add the new one
								if(changed &&
										// if a matches function exists, use that (probably more efficient)
										(queryExecutor.matches ? queryExecutor.matches(changed) : queryExecutor([changed]).length)){

									var firstInsertedInto = removedFrom > -1 ? 
										removedFrom : // put back in the original slot so it doesn't move unless it needs to (relying on a stable sort below)
										resultsArray.length;
									resultsArray.splice(firstInsertedInto, 0, changed); // add the new item
									insertedInto = array.indexOf(queryExecutor(resultsArray), changed); // sort it
									// we now need to push the chagne back into the original results array
									resultsArray.splice(firstInsertedInto, 1); // remove the inserted item from the previous index
									
									if((options.start && insertedInto == 0) ||
										(!atEnd && insertedInto == resultsArray.length)){
										// if it is at the end of the page, assume it goes into the prev or next page
										insertedInto = -1;
									}else{
										resultsArray.splice(insertedInto, 0, changed); // and insert into the results array with the correct index
									}
								}
							}else if(changed){
								// we don't have a queryEngine, so we can't provide any information
								// about where it was inserted or moved to. If it is an update, we leave it's position alone, other we at least indicate a new object
								if(existingId !== undef){
									// an update, keep the index the same
									insertedInto = removedFrom;
								}else if(!options.start){
									// a new object
									insertedInto = store.defaultIndex || 0;
									resultsArray.splice(insertedInto, 0, changed);
								}
							}
							if((removedFrom > -1 || insertedInto > -1) &&
									(includeObjectUpdates || !queryExecutor || (removedFrom != insertedInto))){
								var copyListeners = listeners.slice();
								for(i = 0;listener = copyListeners[i]; i++){
									listener(changed || removedObject, removedFrom, insertedInto);
								}
							}
						});
					});
				}
				var handle = {};
				// TODO: Remove cancel in 2.0.
				handle.remove = handle.cancel = function(){
					// remove this listener
					var index = array.indexOf(listeners, listener);
					if(index > -1){ // check to make sure we haven't already called cancel
						listeners.splice(index, 1);
						if(!listeners.length){
							// no more listeners, remove the query updater too
							queryUpdaters.splice(array.indexOf(queryUpdaters, queryUpdater), 1);
						}
					}
				};
				return handle;
			};
		}
		return results;
	};
	var inMethod;
	function whenFinished(method, action){
		var original = store[method];
		if(original){
			store[method] = function(value){
				if(inMethod){
					// if one method calls another (like add() calling put()) we don't want two events
					return original.apply(this, arguments);
				}
				inMethod = true;
				try{
					var results = original.apply(this, arguments);
					Deferred.when(results, function(results){
						action((typeof results == "object" && results) || value);
					});
					return results;
				}finally{
					inMethod = false;
				}
			};
		}
	}
	// monitor for updates by listening to these methods
	whenFinished("put", function(object){
		store.notify(object, store.getIdentity(object));
	});
	whenFinished("add", function(object){
		store.notify(object);
	});
	whenFinished("remove", function(id){
		store.notify(undefined, id);
	});

	return store;
};

lang.setObject("dojo.store.Observable", Observable);

return Observable;
});

},
'epi/shell/store/storeDelegate':function(){
﻿define("epi/shell/store/storeDelegate", [
    "dojo/_base/lang",
    "dojo/aspect"
], function (lang, aspect) {

    return function (masterStore, childObject) {
        // summary:
        //      Provides same functionality as lang.delegate but allows the store's notify method
        //      to be propagated from child to master and vice versa
        // masterStore:
        //      The store instance (usually an instance of observable store)
        // childObject:
        //      The object which needs to be extended.
        // tags:
        //      internal

        var store = lang.delegate(masterStore, childObject);

        if ((typeof masterStore.notify !== "function") || (typeof childObject !== "object")) {
            return store;
        }

        var inNotify = false;
        aspect.before(masterStore, "notify", function () {
            if (inNotify) {
                return;
            }

            inNotify = true;
            store.notify.apply(store, arguments);
            inNotify = false;
        }, true);

        store.notify = function () {
            if (!inNotify) {
                inNotify = true;
                masterStore.notify.apply(store, arguments);
            }

            inNotify = false;
        };

        return store;
    };
});

},
'epi/shell/store/Realtime':function(){
﻿define("epi/shell/store/Realtime", [
    "dojo/_base/lang",
    "dojo/aspect",
    "dojo/when",
    "dojo/Deferred",
    "dojo/promise/all",
    "epi/shell/socket/hub",
    "epi/shell/TaskQueue",
    "epi/shell/store/storeDelegate"
],
function (
    lang,
    aspect,
    when,
    Deferred,
    all,
    hub,
    TaskQueue,
    storeDelegate) {

    return function (patchableStore, options) {
        // patchableStore:
        //       An instance to patchable store which needs realtime updates
        // options:
        //		RealtimeInfo object which must contains subscriptionKey.
        //
        // tags:
        //      internal

        if (!options || !options.subscriptionKey) {
            throw new Error("Realtime functionality can't be added to a store without providing realtimeInfo (i.e subscriptionKey & idProperty of the store)");
        }

        var idProperty = patchableStore.idProperty;
        var store = patchableStore;

        // provides functionality to listen to socket and process the requests as first come first serve with start/stop ability
        function socketRequestHandler() {

            // Try to refresh the store if possible based on given socket message
            function handleStoreRefresh(recordId, socketItem) {

                function refreshStoreByModified(itemInCache) {
                    // if socket message doesn't have modified (date) on item then refresh the store. (This might be the only extra refresh for Project store.)
                    if (!socketItem.modified) {
                        return store.refresh(recordId);
                    }

                    // in case local cached copy has same timestamp as the socket message, then avoid refresh.
                    var socketItemTimestamp = new Date(socketItem.modified).getTime();
                    var cachedItemTimeStamp = new Date(itemInCache.modified).getTime();
                    if (socketItemTimestamp !== cachedItemTimeStamp) {
                        return store.refresh(recordId);
                    } else {
                        // Item has changed but been updated in the store by other means. Just notify.
                        store.notify(itemInCache, recordId);
                    }

                    return true;
                }

                // we need to get the item from local cache without touching the backend.
                return when(store.getCached(recordId), function (itemInCache) {

                    // if there is no item in local cache, then it is a remote client and it must refresh
                    if (!itemInCache) {
                        return store.refresh(recordId);
                    } else {
                        return refreshStoreByModified(itemInCache);
                    }
                });
            }

            // Executes once per valid socket message and try to notify the changes
            function processMessage(message) {
                if (message) {
                    if (!(message instanceof Array)) {
                        message = [message];
                    }

                    var results = message.map(function (item) {
                        var id = item[idProperty];
                        var action = item.action;

                        switch (action) {
                            case 1: // Save/Update
                                return handleStoreRefresh(id, item);
                            case 2: // Delete
                                // notify the deletion of cached item only if it exists.
                                return when(store.getCached(id), function (itemInCache) {
                                    if (itemInCache) {
                                        store.evict(id);
                                        store.notify(undefined, id);
                                    }
                                });
                            default: // in case some unknown action from server
                                return true;
                        }
                    });

                    return all(results);
                }

                return true;
            }

            var queue = new TaskQueue(processMessage, { throttle: 5 });
            // subscribe to the given subscription key
            hub.subscribe(options.subscriptionKey, queue.enqueue);

            // starts processing the requests if there is any
            this.start = function () {
                queue.start();
            };

            // stops processing the requests
            this.stop = function () {
                queue.stop();
            };
        }

        var handler = new socketRequestHandler();

        var stopQueueAndExecute = function (backingStore, method) {
            return function () {
                handler.stop();
                var result = when(backingStore[method].apply(backingStore, arguments));
                result.always(function () {
                    handler.start();
                });

                return result;
            };
        };

        var realTimeStore = storeDelegate(store, {
            // wraper around store's add/put/remove requests so we can start/stop the processing of socket messages until the request finishes.
            add: stopQueueAndExecute(patchableStore, "add"),
            put: stopQueueAndExecute(patchableStore, "put"),
            remove: stopQueueAndExecute(patchableStore, "remove"),
            removeRange: stopQueueAndExecute(patchableStore, "removeRange")
        });





        return realTimeStore;
    };
});

},
'epi/shell/socket/hub':function(){
define("epi/shell/socket/hub", [
    "epi/routes",
    "./MessageHub",
    "./notificationService",
    "dojo/i18n!epi/nls/episerver.shell.ui.resources.socket.connectionmessages"
], function (
    routes,
    MessageHub,
    notificationService,
    res
) {
    // summary:
    //      Returns an active epi/shell/socket/MessageHub instance
    var messageHub;

    function handleConnectionFailure(message) {
        var description = null;

        if (!message.hasBeenConnected && message.lastAttempt) {
            description = res.noconnection;
        } else if (message.hasBeenConnected) {
            // Wait a few re-connect attempts before showing a notification.
            // The 3rd attempt will be after 25 seconds with the default re-connect delay.
            if (message.connectionAttempts === 3) {
                description = res.reconnect;
            }
            if (message.lastAttempt) {
                description = res.reconnectfailed;
            }
        }

        description && notificationService.show({
            title: res.title,
            description: description
        });
    }

    messageHub = new MessageHub(function () {
        return routes.getActionPath({ moduleArea: "Shell", controller: "socket", action: "endpoint" });
    });
    messageHub.subscribe("/epi/socket/connectionfailed", handleConnectionFailure);

    return messageHub;
});

},
'epi/shell/socket/MessageHub':function(){
define("epi/shell/socket/MessageHub", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/url",
    "dojo/Evented",
    "dojo/on",
    "./WebSocket"
], function (
    declare,
    lang,
    Url,
    Evented,
    on,
    WebSocket
) {

    return declare(null, {
        // summary:
        //      A message hub receiving and emitting push messages from the server.
        //      The underlying connection uses a web socket for the client-server communication.
        // tags: internal

        // maxConnectionAttempts: Number
        //      The maximal number of attempts to reconnect if the socket is closed
        maxConnectionAttempts: 8,

        // reconnectPause: Number
        //      The base time to wait between reconnect attempts.
        reconnectPause: 5000,

        // statusTopic: [readonly] Object
        //      A dictionary of status topics emitted by the message hub
        statusTopic: {
            // open: [readonly] String
            //      Emitted when the socket connection has been opened
            open: "/epi/socket/open",

            // error: [readonly] String
            //      Emitted when an error occurs on the socket
            error: "/epi/socket/error",

            // reconnect: [readonly] String
            //      Emitted before trying to re-establish a socket connection
            reconnect: "/epi/socket/reconnect",

            // connectionfailed: [readonly] String
            //      Emitted when a socket connection can not be established
            connectionfailed: "/epi/socket/connectionfailed",

            // message: [readonly] String
            //      Emitted when a socket message is received from the server
            message: "/epi/socket/message"
        },

        // heartBeatInterval: Number
        //      The time in ms between heartbeat messages sent to the server when idle.
        //      If set to 0 heartbeat will be disabled.
        heartbeatInterval: 3000,

        // heartbeatMessage: String
        //      The string sent as a heartbeat message to the server.
        heartbeatMessage: "/epi/heartbeat",

        _path: null,
        _eventListeners: null,
        _topicHub: null,
        _connectionAttempts: 0,
        _hasBeenConnected: false,
        _reconnectHandle: null,

        _heartbeatHandle: null,
        _outstandingHeartbeats: 0,

        constructor: function (socketPathResolver) {
            // summary:
            //      Creates a new instance of the MessageHub
            // socketPathResolver: function
            //      Callback to resolve the path to the server-side socket endpoint

            this._topicHub = new Evented();
            this._socketPathResolver = socketPathResolver;
            this._eventListeners = [];
        },

        start: function () {
            // summary:
            //      Opens the connection to the server and attaches message event listeners
            // tags: public

            var socket;

            if (this._socket && this._socket.readyState !== WebSocket.CLOSED) {
                return;
            }

            socket = new WebSocket(this._resolvePath());

            this._eventListeners.push(
                on(socket, "error", lang.hitch(this, "_onError")),
                on(socket, "open", lang.hitch(this, "_onOpen")),
                on(socket, "close", lang.hitch(this, "_onClose")),
                on(socket, "message", lang.hitch(this, "_onMessage"))
            );

            this._socket = socket;
        },

        stop: function () {
            // summary:
            //      Closes the connection to the server and removes message event listeners
            // tags: public

            if (!this._socket) {
                return;
            }

            this._reconnectHandle && clearTimeout(this._reconnectHandle);
            this._reconnectHandle = null;

            this._socket.close();
            this._socket = null;
            this._eventListeners.forEach(function (listener) {
                listener.remove();
            });
            this._eventListeners = [];
        },

        subscribe: function (topic, listener) {
            // summary:
            //      subscribe to a topic published from server
            // tags: public

            return this._topicHub.on(topic, listener);
        },

        _resolvePath: function () {
            // summary:
            //      Resolves the websocket path based on the current location
            // tags: protected

            var base = (document.baseURI || window.location.href).replace(/^http/i, "ws");

            return new Url(base, this._socketPathResolver());
        },

        _publish: function (topic, data) {
            // summary:
            //      Executes a publish on the message hub
            // tags: protected

            this._topicHub.emit(topic, data);
        },

        _onMessage: function (messageEvent) {
            // summary:
            //      Handles incoming messages from the websocket client
            // tags: private

            var data;

            if (messageEvent.data && !this._handleHeartbeat(messageEvent.data)) {
                data = JSON.parse(messageEvent.data);
                this._publish(this.statusTopic.message, data);
                if (data && data.topic) {
                    this._publish(data.topic, data.data);
                }
            }
        },

        _onError: function (e) {
            // summary:
            //      Handles error events on the websocket client
            // tags: private

            // Connection failed and closed when opening, it can't be configured right
            // make sure we get a final fail when the close event comes
            if (!this._hasBeenConnected && this._socket.readyState === WebSocket.CLOSED) {
                this._connectionAttempts = this.maxConnectionAttempts;
            }

            this._publish(this.statusTopic.error, e);
        },

        _onOpen: function (e) {
            // summary:
            //      Handles open events on the websocket client
            // tags: private

            this._hasBeenConnected = true;
            this._connectionAttempts = 0;
            this._publish(this.statusTopic.open, e);

            this._resetHeartbeat();
        },

        _onClose: function (e) {
            // summary:
            //      Handles close events on the websocket client
            // tags: private

            var nextAttempt,
                publish = function (topic) {
                    this._publish(topic, {
                        connectionAttempts: this._connectionAttempts,
                        lastAttempt: this._connectionAttempts >= this.maxConnectionAttempts,
                        hasBeenConnected: this._hasBeenConnected
                    });
                }.bind(this),
                reConnect = function () {
                    publish(this.statusTopic.reconnect);
                    this._reconnectHandle = null;
                    this.stop();
                    this.start();
                }.bind(this);

            this._heartbeatHandle && clearTimeout(this._heartbeatHandle);
            this._heartbeatHandle = null;

            if (!this._socket) {
                return;
            }

            if (!e.wasClean) {
                this._connectionAttempts++;
                publish(this.statusTopic.connectionfailed);
            }

            if (this._connectionAttempts <= this.maxConnectionAttempts) {
                nextAttempt = this.reconnectPause * Math.pow(this._connectionAttempts, 2);
                this._reconnectHandle = setTimeout(reConnect, nextAttempt);
            }
        },

        _resetHeartbeat: function () {
            // summary:
            //
            // tags: private

            var checkHeartbeat = function () {
                if (this._outstandingHeartbeats > 1) {
                    this._socket.close();
                } else {
                    this._outstandingHeartbeats++;
                    this._socket.send(this.heartbeatMessage);
                    this._heartbeatHandle = setTimeout(checkHeartbeat, this.heartbeatInterval);
                }
            }.bind(this);

            if (this.heartbeatInterval === 0) {
                return;
            }

            this._outstandingHeartbeats = 0;

            if (this._heartbeatHandle) {
                clearTimeout(this._heartbeatHandle);
                this._heartbeatHandle = setTimeout(checkHeartbeat, this.heartbeatInterval);
            } else {
                checkHeartbeat();
            }
        },

        _handleHeartbeat: function (data) {
            // summary:
            //
            // tags: private

            this._resetHeartbeat();

            if (data === this.heartbeatMessage) {
                return true;
            }
        }
    });
});

},
'epi/shell/socket/WebSocket':function(){
define("epi/shell/socket/WebSocket", function () {
    // summary:
    //      A native browser WebSocket client.

    return WebSocket;
});

},
'epi/shell/socket/notificationService':function(){
define("epi/shell/socket/notificationService", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "epi/shell/widget/dialog/Confirmation"
], function (
    declare,
    lang,
    Confirmation
) {
    var currentDialog,
        Alert = declare([Confirmation], {
            getActions: function () {
                return [{
                    name: this._okButtonName,
                    label: this.buttonOk,
                    title: null,
                    action: lang.hitch(this, this._onCancel)
                }];
            }
        });

    return {
        // summary:
        //      Service handling showing notifications in the UI.
        // tags: internal

        show: function (notification) {
            currentDialog && currentDialog.hide();
            currentDialog = new Alert(notification);

            currentDialog.show();
        }
    };
});

},
'epi/shell/TaskQueue':function(){
﻿define("epi/shell/TaskQueue", [
    "dojo/_base/lang",
    "dojo/when"

], function (lang, when) {

    return function (task, options) {
        // summary:
        //    A simple message queue which accepts a function and then apply that on every incoming message sequentially
        //    Once the function applied to a message it gets removed from queue.
        //
        // task: Function
        //       A function applied on every message when processed
        // options: Object
        //		Option contains throttle
        //
        // tags:
        //      internal

        if ((typeof task !== "function")) {
            throw new Error("TaskQueue can't be created without providing a task processing callback.");
        }

        // throttle is used to only process message after a quiet period
        var throttle = (options && options.throttle) || 0;
        var processingInProgress = false;

        // contains all incoming messages
        var messageQueue = [];

        // processing of messages can be switched on/off via this flag.
        var enableProcessing = true;

        function processQueue() {
            // summary:
            //      Starts the message processing only if it is enabled.

            if (enableProcessing && !processingInProgress && messageQueue.length > 0) {
                processingInProgress = true;
                var currentMessage = messageQueue.shift();

                when(task(currentMessage)).always(function () {
                    processingInProgress = false;
                    setTimeout(processQueue, throttle);
                }).otherwise(function (err) {
                    console.error("Something went wrong while running provided function to TaskQueue. " + err);
                });
            }
        }

        this.start = function () {
            // summary:
            //      Enables the processing of queued messages and start the process.

            enableProcessing = true;
            processQueue();
        };

        this.stop = function () {
            // summary:
            //      Stops the processing of messages.
            enableProcessing = false;
        };

        this.enqueue = function (message) {
            // summary:
            //      Enques the given message in queue and start the processing
            // message:
            //       Any message. (Could be an object, an array etc. but null or undefined will not be processed)
            if (message) {
                messageQueue.push(message);
                processQueue();
            }
        };

        this.getLength = function () {
            // summary:
            //      Returns the length of message queue.
            return messageQueue.length;
        };

    };
});

},
'epi/shell/TypeDescriptorManager':function(){
define("epi/shell/TypeDescriptorManager", [
// dojo
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/promise/all",
    "dojo/when",
    // epi
    "epi/dependency",
    "dojo/i18n!epi/nls/contenttypes"
], function (
// dojo
    array,
    declare,
    lang,
    all,
    when,

    // epi
    dependency,
    typeResources
) {
    return {
        // summary:
        //    This class manages ui behavior for classes including resolving inherited values.
        // tags:
        //    public

        // _uiDescriptors: Array
        //    An array of descriptors
        _uiDescriptors: {},

        resources: typeResources,

        store: null,

        initialize: function () {
            this.store = this.store || dependency.resolve("epi.storeregistry").get("epi.shell.uidescriptor");

            return when(this.store.query()).then(this.registerTypeSettings.bind(this));
        },

        clear: function () {
            // summary:
            //    Clears all registered settings.
            this._uiDescriptors = {};
        },

        registerTypeSettings: function (uiDescriptors) {
            var types = this._uiDescriptors = {};

            function getOrCreateDeclaration(typeKey) {
                var type = types[typeKey],
                    uiDescriptor = type.descriptor,
                    baseTypes = uiDescriptor.baseTypes || [];

                if (type.ctor) {
                    return type.ctor;
                }

                if (type.visited) {
                    throw new Error("Cyclic dependency detected in descriptor: " + typeKey);
                }

                type.visited = true;
                type.ctor = declare(baseTypes.map(getOrCreateDeclaration), uiDescriptor);
                type.instance = new type.ctor();

                return type.ctor;
            }

            uiDescriptors.forEach(function (uiDescriptor) {
                types[uiDescriptor.typeIdentifier] = this._prepareDescriptor(uiDescriptor);
            }, this);

            Object.keys(types).forEach(getOrCreateDeclaration);

            return types;
        },

        registerTypeSetting: function (name, setting) {
            // summary:
            //    Gets a given setting for a type

            setting.typeIdentifier = name;
            this.registerTypeSettings([setting]);
        },

        getValue: function (type, settingName) {
            // summary:
            //    Gets a given setting for a type
            //
            // type: string
            //    The type that you want to fetch the settings for.
            //
            // settingName: string
            //    The settings that you want to fetch.

            var typeDescriptor = this._uiDescriptors[type];

            return typeDescriptor ? typeDescriptor.instance[settingName] : undefined;
        },

        getResourceValue: function (type, resourceName, primaryType) {
            // summary:
            //      Gets a resource for a given type.
            // type: [String]
            //      The type that you want to fetch the settings for.
            // resourceName: [String]
            //      The settings that you want to fetch.
            // primaryType: [Boolean]
            //      Flag indicates that needed to get resources from primary type or not
            // tags:
            //      public

            var typeDescriptor = this._uiDescriptors[type];

            return typeDescriptor ? typeDescriptor.instance.getResource(resourceName, primaryType) : undefined;
        },

        getAndConcatenateValues: function (type, settingName) {
            // summary:
            //    Gets a given setting for a type and it's ancestors returning an array with all values.
            //
            // type: string
            //    The type that you want to fetch the settings for.
            //
            // settingName: string
            //    The settings that you want to fetch.

            var values = [],
                typeDescriptor = this._uiDescriptors[type];

            typeDescriptor && typeDescriptor.instance.getRecursive(settingName, values);

            return values;
        },

        getInheritanceChain: function (type) {
            // summary:
            //    Returns an array containing the type identifiers for the given type and it's inherited types.
            //
            // type: string
            //    The type that you want to fetch the inheritance chain for.

            var value = [],
                typeDescriptor = this._uiDescriptors[type];

            if (!typeDescriptor) {
                return value;
            }

            typeDescriptor.instance.constructor._meta.bases.forEach(function (base) {
                value.push(base._meta.hidden.typeIdentifier);
            });

            return value;
        },

        isBaseTypeIdentifier: function (childType, parentType) {
            // summary:
            //      Test if parent type is base of child type/
            // childType: String
            //      the child type
            // parentType: String
            //      the parent type

            childType = this._uiDescriptors[childType];
            parentType = this._uiDescriptors[parentType];

            if (!childType || !parentType) {
                return false;
            }

            return childType.instance.isInstanceOf(parentType.ctor);
        },

        removeIntersectingTypes: function (allowedTypes, restrictedTypes) {
            // summary:
            //      Returns the list of types which are present in allowedTypes but not in restrictedTypes or their children
            // allowedTypes: Array
            //      the allowed types
            // restrictedTypes: restrictedTypes
            //      the restricted types
            // tags:
            //      public

            var result = [];

            if (allowedTypes) {
                allowedTypes.forEach(function (allowed) {

                    // allowedTypes should not be part of restricted or its child
                    if (array.indexOf(restrictedTypes, allowed) === -1 && array.some(restrictedTypes, function (restricted) {
                        return this.isBaseTypeIdentifier(allowed, restricted);
                    }, this) === false) {
                        result.push(allowed);
                    }
                }, this);
            }

            return result;
        },

        getValidAcceptedTypes: function (allAvailableTypes, allowedTypes, restrictedTypes) {
            // summary:
            //      Returns a filtered list of given by availableTypes and the filter criteria is based on given allowed and restricted types.
            // allAvailableTypes: Array
            //      The available types
            // allowedTypes: Array
            //      the allowed types
            // restrictedTypes: Array
            //      the restricted types
            // tags:
            //      public

            var allowed = allowedTypes || [];
            var restricted = restrictedTypes || [];
            var available = allAvailableTypes || [];

            // if there are no allowed/restricted types then no need to perform any filtering
            if (allowed.length === 0 && restricted.length === 0) {
                return available;
            }

            var isAnyElementSameOrParent = function (child /* typeIdentifier as string */, parents /* array of typeIdentifiers */) {
                return array.some(parents, function (parent) {
                    return this.isBaseTypeIdentifier(child, parent);
                }, this);
            };

            var filteredTypes = array.filter(available, function (availableType) {
                return (!(isAnyElementSameOrParent.call(this, availableType, restricted)) && isAnyElementSameOrParent.call(this, availableType, allowed));
            }, this);

            return filteredTypes;
        },

        _prepareDescriptor: function (uiDescriptor) {

            var resource,
                languageKey = uiDescriptor.languageKey;

            Object.keys(uiDescriptor).forEach(function (key) {
                var prop = uiDescriptor[key];
                if (prop === null || prop === undefined) {
                    delete uiDescriptor[key];
                }
            });

            if (languageKey) {
                resource = this.resources[languageKey];
            }

            lang.mixin(uiDescriptor, {
                getRecursive: function (key, values) {
                    var value = uiDescriptor[key];
                    if (value) {
                        if (value instanceof Array) {
                            value.forEach(function (v) {
                                values.push(v);
                            });
                        } else {
                            values.push(value);
                        }
                    }

                    this.inherited(arguments);
                },
                getResource: function (key, getPrimary) {

                    var resources = uiDescriptor.resource,
                        resource;

                    if (resources) {
                        if (!getPrimary || (getPrimary && uiDescriptor.isPrimaryType)) {
                            resource = lang.getObject(key, false, resources);
                            if (resource !== undefined) {
                                return resource;
                            }
                        }
                    }

                    return this.inherited(arguments);
                },
                resource: resource
            });

            return {
                instance: null,
                ctor: null,
                descriptor: uiDescriptor,
                visited: false
            };
        }
    };
});

},
'epi-cms/store/CustomQueryEngine':function(){
﻿define("epi-cms/store/CustomQueryEngine", ["dojo/_base/array"], function (arrayUtil) {

    return function (query, options) {
        // summary:
        //      The module defines a filtering query engine that can be passed an array of ignore
        //      parameters enabling specific query parameters to be ignored when analyzing the query.
        // description:
        //		Simple query engine that matches using filter functions, named filter
        //		functions or objects by name-value on a query object hash.
        //
        // tags:
        //      internal xproduct deprecated

        // create our matching query function
        switch (typeof query) {
            case "object": case "undefined":
                var queryObject = query;
                query = function (object) {
                    for (var key in queryObject) {
                        if (options && options.ignore) {
                            var skipKey;
                            for (var ignoreKey in options.ignore) {
                                if (key === options.ignore[ignoreKey]) {
                                    skipKey = true;
                                    break;
                                }
                            }
                            if (skipKey) {
                                skipKey = false;
                                continue;
                            }
                        }
                        var required = queryObject[key];
                        // check if we have external property comparer to use it.
                        // For example: comparer for typeIdentifiers, which is array.
                        // Only return false if the comparer returns false, otherwise, let the loop continue. Do NOT return true
                        if (options && options.comparers && options.comparers[key]) {
                            if (!options.comparers[key](required, object)) {
                                return false;
                            }
                        } else if (required && required.test) {
                            if (!required.test(object[key])) {
                                return false;
                            }
                        } else if (required && required !== object[key]) {
                            return false;
                        }
                    }
                    return true;
                };
                break;
            case "string":
                // named query
                if (!this[query]) {
                    throw new Error("No filter function " + query + " was found in store");
                }
                query = this[query];
                break;
            case "function":
                break;
            default:
                throw new Error("Can not query with a " + typeof query);
        }
        function execute(array) {
            // execute the whole query, first we filter
            var results = arrayUtil.filter(array, query);
            // next we sort
            if (options && options.sort) {
                results.sort(function (a, b) {
                    for (var sort, i = 0; (sort = options.sort[i]); i++) {
                        var aValue = a[sort.attribute];
                        var bValue = b[sort.attribute];

                        if (typeof aValue === "string" && typeof bValue === "string") {
                            aValue = aValue.toLowerCase();
                            bValue = bValue.toLowerCase();
                        }

                        if (aValue !== bValue) {
                            return sort.descending === (aValue > bValue) ? -1 : 1;
                        }
                    }
                    return 0;
                });
            }
            // now we paginate
            if (options && (options.start || options.count)) {
                var total = results.length;
                results = results.slice(options.start || 0, (options.start || 0) + (options.count || Infinity));
                results.total = total;
            }
            return results;
        }
        execute.matches = query;
        return execute;
    };
});

},
'epi-cms/contentediting/ContentHierarchyService':function(){
define("epi-cms/contentediting/ContentHierarchyService", [
// dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/when",
    "dojo/Deferred",
    "dojo/promise/all",

    // epi
    "epi/dependency",
    "epi/shell/DialogService",
    "epi-cms/core/ContentReference"
], function (
// dojo
    declare,
    lang,
    when,
    Deferred,
    all,

    // epi
    dependency,
    DialogService,
    ContentReference
) {

    return declare([], {
        // summary:
        //      A service for interacting with the content hierarchy
        // tags:
        //      internal

        // store: [readonly] Store
        //      A REST store for interacting with content items.
        store: null,

        constructor: function (params) {
            declare.safeMixin(this, params);

            this.store = this.store || dependency.resolve("epi.storeregistry").get("epi.cms.content.light");
        },

        move: function (sourceIds, targetId, createAsLocalAsset, sortIndex) {
            // summary:
            //		Move one or several content items from one parent to another.
            // sourceIds: ContentReference || ContentReference[]
            //      The items that should be copied or moved
            // targetId: ContentReference
            //      The target item for the operation
            // createAsLocalAsset: Boolean
            //      If set to true the items will be added to the targets asset folder
            // sortIndex: Number
            //      Optional sortIndex in the child collection of the new parent
            // returns: Response object
            //      An object indicating the result of the operation
            //
            // tags: public

            return this._execute("MoveMany", sourceIds, targetId, createAsLocalAsset, sortIndex);
        },

        copy: function (sourceIds, targetId, createAsLocalAsset, sortIndex) {
            // summary:
            //		Copies one or several  content items from one parent to another.
            // sourceIds: ContentReference || ContentReference[]
            //      The items that should be copied or moved
            // targetId: ContentReference
            //      The target item for the operation
            // createAsLocalAsset: Boolean
            //      If set to true the items will be added to the targets asset folder
            // sortIndex: Number
            //      Optional sortIndex in the child collection of the new parent
            // returns: Response object
            //      An object indicating the result of the operation
            //
            // tags: public

            return this._execute("CopyMany", sourceIds, targetId, createAsLocalAsset, sortIndex);
        },

        _execute: function (method, sourceIds, targetId, createAsLocalAsset, sortIndex) {
            // summary:
            //		Calls the store to execute the defined method.
            //
            // tags: private

            var params = {
                ids: sourceIds instanceof Array ? sourceIds : [sourceIds],
                targetId: targetId + "",
                createAsLocalAsset: createAsLocalAsset,
                sortIndex: sortIndex
            };

            return this.store.executeMethod(method, null, params)
                .then(this._checkForErrors.bind(this))
                .then(this._handleUpdates.bind(this));
        },

        _checkForErrors: function (response) {
            // summary:
            //		Extracts failed operations from the response and id any shows
            //      an error dialog.
            //
            // tags: private

            var deferred = new Deferred(),
                errors = this.store.getAllResponsesWithError(response).map(function (error) {
                    return error.message;
                });

            if (errors.length > 0) {
                DialogService.alertWithErrors({}, errors).then(function () {
                    deferred.resolve(response);
                });
            } else {
                deferred.resolve(response);
            }

            return deferred.promise;
        },

        _handleUpdates: function (response) {
            // summary:
            //		Uses the response items to refresh items in the store
            //      and notify listeners of the changes.
            //
            // tags: private

            var store = this.store,
                itemsToNotify = this._getUpdatesFromResponse(response);

            return all(itemsToNotify.map(function (item) {
                var id = ContentReference.toContentReference(item.extraInformation).createVersionUnspecificReference().toString();
                return store.refresh(id, true);
            })).then(function (items) {
                items.forEach(function (item) {
                    store.notify(item, store.getIdentity(item));
                });

                return response;
            });
        },

        _getUpdatesFromResponse: function (response) {
            // summary:
            //		Extracts all successful changes from the response.
            //
            // tags: private

            var items = response.extraInformation,
                itemsToNotify = [];

            Object.keys(items).forEach(function (id) {
                var item = items[id];
                if (item.statusCode === 200) {
                    itemsToNotify.push(item);
                }
            });

            return itemsToNotify;
        }
    });
});

},
'epi/shell/DialogService':function(){
define("epi/shell/DialogService", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/Deferred",
    "dojo/on",
    "dgrid/List",
    "dgrid/extensions/DijitRegistry",
    "put-selector/put",
    "epi/shell/widget/dialog/Alert",
    "epi/shell/widget/dialog/Confirmation",
    "epi/shell/widget/dialog/Dialog"
], function (
    declare,
    lang,
    Deferred,
    on,
    List,
    DijitRegistry,
    put,
    Alert,
    Confirmation,
    Dialog
) {
    var ErrorList = declare([List, DijitRegistry], {
        renderRow: function (value, options) {
            return put("div", "", {
                innerHTML: value
            });
        }
    });

    return {
        // summary:
        //		Show different dialogs.
        //
        // description:
        //		Use the "show" method to pick the type of dialog by its name (useful for data objects where
        //      the message type is a part of it) or use the stand-alone methods to easily show a dialog with sensible
        //      defaults.
        //
        // tags:
        //      public

        show: function (type, settings) {
            // summary:
            //      Convenience method to show a dialog based on the "type" instead of calling the specific method.
            // description:
            //      type: The type of dialog. I.e. alert, confirm.
            //      settings: Include "title" and "description" to set the text. Optionally override the default settings.
            //      Returns a promise.
            // tags:
            //      public

            if (!this[type]) {
                throw new Error("Dialog type " + type + " is not a valid DialogService option.");
            }

            return this[type].call(this, settings);
        },

        alert: function (settings) {
            // summary:
            //      Displays an Alert dialog.
            // description:
            //      settings: Include "title" and "description" to set the text. Optionally override the default settings.
            //      Returns a promise. Always resolves.
            // tags:
            //      public

            return this.alertWithErrors(settings);
        },

        alertWithErrors: function (settings, errors) {
            // summary:
            //      Displays an Alert dialog with a list errors
            // settings: Object
            //      Include "heading" and "description" to set the text. Optionally override the default settings.
            // errors:  [string]
            //      A list of errors to display, it will replace the description passed in the settings object
            // returns:
            //      Returns a promise. Always resolves.
            // tags:
            //      public

            var deferred = new Deferred();

            if (errors instanceof Array) {
                var list = new ErrorList({ className: "epi-grid-max-height--300" });
                list.renderArray(errors);
                list.startup();

                settings.content = list;
                settings.description = null;
            }

            this._callDialog(Alert, settings, deferred.resolve);

            return deferred.promise;
        },

        confirmation: function (settings) {
            // summary:
            //      Displays an Confirmation dialog.
            // description:
            //      settings: Include "title" and "description" to set the text. Optionally override the default settings.
            //      Returns a promise. Resolves on confirm, rejects on cancel.
            // tags:
            //      public

            var deferred = new Deferred();

            this._callDialog(Confirmation, settings, function (confirm) {
                if (confirm) {
                    deferred.resolve();
                } else {
                    deferred.reject();
                }
            });

            return deferred.promise;
        },

        dialog: function (settings) {
            // summary:
            //      Displays a dialog.
            // description:
            //      settings: Include "title" and "description" to set the text. Optionally override the default settings.
            //      Returns a promise. Resolves on dialog submit, rejects on cancel.
            // tags:
            //      internal

            var deferred = new Deferred();

            var dialog = new Dialog(settings);
            on.once(dialog, "execute", function () {
                deferred.resolve();
            });
            on.once(dialog, "cancel", function () {
                deferred.reject();
            });

            dialog.show();

            return deferred.promise;
        },

        _callDialog: function (dialogClass, settings, callback) {
            var dialog = new dialogClass(settings);

            dialog.on("action", callback);
            dialog.show();
        }
    };
});

},
'dgrid/List':function(){
define("dgrid/List", ["dojo/_base/kernel", "dojo/_base/declare", "dojo/dom", "dojo/on", "dojo/has", "./util/misc", "dojo/has!touch?./TouchScroll", "xstyle/has-class", "put-selector/put", "dojo/_base/sniff", "xstyle/css!./css/dgrid.css"],
function(kernel, declare, dom, listen, has, miscUtil, TouchScroll, hasClass, put){
	// Add user agent/feature CSS classes 
	hasClass("mozilla", "opera", "webkit", "ie", "ie-6", "ie-6-7", "quirks", "no-quirks", "touch");
	
	var oddClass = "dgrid-row-odd",
		evenClass = "dgrid-row-even",
		scrollbarWidth, scrollbarHeight;
	
	function byId(id){
		return document.getElementById(id);
	}
	
	function cleanupTestElement(element){
		element.className = "";
		document.body.removeChild(element);
	}
	
	function getScrollbarSize(element, dimension){
		// Used by has tests for scrollbar width/height
		put(document.body, element, ".dgrid-scrollbar-measure");
		var size = element["offset" + dimension] - element["client" + dimension];
		cleanupTestElement(element);
		return size;
	}
	has.add("dom-scrollbar-width", function(global, doc, element){
		return getScrollbarSize(element, "Width");
	});
	has.add("dom-scrollbar-height", function(global, doc, element){
		return getScrollbarSize(element, "Height");
	});
	
	has.add("dom-rtl-scrollbar-left", function(global, doc, element){
		var div = put("div"),
			isLeft;
		
		put(document.body, element, ".dgrid-scrollbar-measure[dir=rtl]");
		put(element, div);
		
		// position: absolute makes IE always report child's offsetLeft as 0,
		// but it conveniently makes other browsers reset to 0 as base, and all
		// versions of IE are known to move the scrollbar to the left side for rtl
		isLeft = !!has("ie") || !!has("trident") || div.offsetLeft >= has("dom-scrollbar-width");
		cleanupTestElement(element);
		put(div, "!");
		element.removeAttribute("dir");
		return isLeft;
	});
	
	// var and function for autogenerating ID when one isn't provided
	var autogen = 0;
	function generateId(){
		return "dgrid_" + autogen++;
	}
	
	// common functions for class and className setters/getters
	// (these are run in instance context)
	var spaceRx = / +/g;
	function setClass(cls){
		// Format input appropriately for use with put...
		var putClass = cls ? "." + cls.replace(spaceRx, ".") : "";
		
		// Remove any old classes, and add new ones.
		if(this._class){
			putClass = "!" + this._class.replace(spaceRx, "!") + putClass;
		}
		put(this.domNode, putClass);
		
		// Store for later retrieval/removal.
		this._class = cls;
	}
	function getClass(){
		return this._class;
	}
	
	// window resize event handler, run in context of List instance
	var winResizeHandler = has("ie") < 7 && !has("quirks") ? function(){
		// IE6 triggers window.resize on any element resize;
		// avoid useless calls (and infinite loop if height: auto).
		// The measurement logic here is based on dojo/window logic.
		var root, w, h, dims;
		
		if(!this._started){ return; } // no sense calling resize yet
		
		root = document.documentElement;
		w = root.clientWidth;
		h = root.clientHeight;
		dims = this._prevWinDims || [];
		if(dims[0] !== w || dims[1] !== h){
			this.resize();
			this._prevWinDims = [w, h];
		}
	} :
	function(){
		if(this._started){ this.resize(); }
	};
	
	// Desktop versions of functions, deferred to when there is no touch support,
	// or when the useTouchScroll instance property is set to false
	
	function desktopGetScrollPosition(){
		return {
			x: this.bodyNode.scrollLeft,
			y: this.bodyNode.scrollTop
		};
	}
	
	function desktopScrollTo(options){
		if(typeof options.x !== "undefined"){
			this.bodyNode.scrollLeft = options.x;
		}
		if(typeof options.y !== "undefined"){
			this.bodyNode.scrollTop = options.y;
		}
	}
	
	return declare(has("touch") ? TouchScroll : null, {
		tabableHeader: false,
		// showHeader: Boolean
		//		Whether to render header (sub)rows.
		showHeader: false,
		
		// showFooter: Boolean
		//		Whether to render footer area.  Extensions which display content
		//		in the footer area should set this to true.
		showFooter: false,
		
		// maintainOddEven: Boolean
		//		Whether to maintain the odd/even classes when new rows are inserted.
		//		This can be disabled to improve insertion performance if odd/even styling is not employed.
		maintainOddEven: true,
		
		// cleanAddedRules: Boolean
		//		Whether to track rules added via the addCssRule method to be removed
		//		when the list is destroyed.  Note this is effective at the time of
		//		the call to addCssRule, not at the time of destruction.
		cleanAddedRules: true,
		
		// useTouchScroll: Boolean
		//		If touch support is available, this determines whether to
		//		incorporate logic from the TouchScroll module (at the expense of
		//		normal desktop/mouse or native mobile scrolling functionality).
		useTouchScroll: null,
		
		// addUiClasses: Boolean
		//		Whether to add jQuery UI classes to various elements in dgrid's DOM.
		addUiClasses: true,

		// shouldObserveStore: Boolean
		//		Whether this instance should observe any observable store it is passed.
		shouldObserveStore: true,
		
		// cleanEmptyObservers: Boolean
		//		Whether to clean up observers for empty result sets.
		cleanEmptyObservers: true,

		// highlightDuration: Integer
		//		The amount of time (in milliseconds) that a row should remain
		//		highlighted after it has been updated.
		highlightDuration: 250,
		
		postscript: function(params, srcNodeRef){
			// perform setup and invoke create in postScript to allow descendants to
			// perform logic before create/postCreate happen (a la dijit/_WidgetBase)
			var grid = this;
			
			(this._Row = function(id, object, element){
				this.id = id;
				this.data = object;
				this.element = element;
			}).prototype.remove = function(){
				grid.removeRow(this.element);
			};
			
			if(srcNodeRef){
				// normalize srcNodeRef and store on instance during create process.
				// Doing this in postscript is a bit earlier than dijit would do it,
				// but allows subclasses to access it pre-normalized during create.
				this.srcNodeRef = srcNodeRef =
					srcNodeRef.nodeType ? srcNodeRef : byId(srcNodeRef);
			}
			this.create(params, srcNodeRef);
		},
		listType: "list",
		
		create: function(params, srcNodeRef){
			var domNode = this.domNode = srcNodeRef || put("div"),
				cls;
			
			if(params){
				this.params = params;
				declare.safeMixin(this, params);
				
				// Check for initial class or className in params or on domNode
				cls = params["class"] || params.className || domNode.className;
				
				// handle sort param - TODO: revise @ 0.4 when _sort -> sort
				this._sort = params.sort || [];
				delete this.sort; // ensure back-compat method isn't shadowed
			}else{
				this._sort = [];
			}
			
			// ensure arrays and hashes are initialized
			this.observers = [];
			this._numObservers = 0;
			this._listeners = [];
			this._rowIdToObject = {};
			
			this.postMixInProperties && this.postMixInProperties();
			
			// Apply id to widget and domNode,
			// from incoming node, widget params, or autogenerated.
			this.id = domNode.id = domNode.id || this.id || generateId();
			
			// If useTouchScroll wasn't explicitly set on the instance, set it
			// now during creation (not up-front, in case document isn't ready)
			if(this.useTouchScroll === null){
				this.useTouchScroll = !has("dom-scrollbar-width");
			}
			
			// Perform initial rendering, and apply classes if any were specified.
			this.buildRendering();
			if(cls){ setClass.call(this, cls); }
			
			this.postCreate();
			
			// remove srcNodeRef instance property post-create
			delete this.srcNodeRef;
			// to preserve "it just works" behavior, call startup if we're visible
			if(this.domNode.offsetHeight){
				this.startup();
			}
		},
		buildRendering: function(){
			var domNode = this.domNode,
				addUiClasses = this.addUiClasses,
				self = this,
				headerNode, spacerNode, bodyNode, footerNode, isRTL;
			
			// Detect RTL on html/body nodes; taken from dojo/dom-geometry
			isRTL = this.isRTL = (document.body.dir || document.documentElement.dir ||
				document.body.style.direction).toLowerCase() == "rtl";
			
			// Clear out className (any pre-applied classes will be re-applied via the
			// class / className setter), then apply standard classes/attributes
			domNode.className = "";
			
			put(domNode, "[role=grid].dgrid.dgrid-" + this.listType +
				(addUiClasses ? ".ui-widget" : ""));
			
			// Place header node (initially hidden if showHeader is false).
			headerNode = this.headerNode = put(domNode, 
				"div.dgrid-header.dgrid-header-row" +
				(addUiClasses ? ".ui-widget-header" : "") +
				(this.showHeader ? "" : ".dgrid-header-hidden"));
			if(has("quirks") || has("ie") < 8){
				spacerNode = put(domNode, "div.dgrid-spacer");
			}
			bodyNode = this.bodyNode = put(domNode, "div.dgrid-scroller");
			
			// Firefox 4+ adds overflow: auto elements to the tab index by default;
			// force them to not be tabbable, but restrict this to Firefox,
			// since it breaks accessibility support in other browsers
			if(has("ff")){
				bodyNode.tabIndex = -1;
			}
			
			this.headerScrollNode = put(domNode, "div.dgrid-header.dgrid-header-scroll.dgrid-scrollbar-width" +
				(addUiClasses ? ".ui-widget-header" : ""));
			
			// Place footer node (initially hidden if showFooter is false).
			footerNode = this.footerNode = put("div.dgrid-footer" +
				(this.showFooter ? "" : ".dgrid-footer-hidden"));
			put(domNode, footerNode);
			
			if(isRTL){
				domNode.className += " dgrid-rtl" +
					(has("dom-rtl-scrollbar-left") ? " dgrid-rtl-swap" : "");
			}
			
			listen(bodyNode, "scroll", function(event){
				if(self.showHeader){
					// keep the header aligned with the body
					headerNode.scrollLeft = event.scrollLeft || bodyNode.scrollLeft;
				}
				// re-fire, since browsers are not consistent about propagation here
				event.stopPropagation();
				listen.emit(domNode, "scroll", {scrollTarget: bodyNode});
			});
			this.configStructure();
			this.renderHeader();
			
			this.contentNode = this.touchNode = put(this.bodyNode,
				"div.dgrid-content" + (addUiClasses ? ".ui-widget-content" : ""));
			// add window resize handler, with reference for later removal if needed
			this._listeners.push(this._resizeHandle = listen(window, "resize",
				miscUtil.throttleDelayed(winResizeHandler, this)));
		},
		
		postCreate: has("touch") ? function(){
			if(this.useTouchScroll){
				this.inherited(arguments);
			}
		} : function(){},
		
		startup: function(){
			// summary:
			//		Called automatically after postCreate if the component is already
			//		visible; otherwise, should be called manually once placed.
			
			if(this._started){ return; } // prevent double-triggering
			this.inherited(arguments);
			this._started = true;
			this.resize();
			// apply sort (and refresh) now that we're ready to render
			this.set("sort", this._sort);
		},
		
		configStructure: function(){
			// does nothing in List, this is more of a hook for the Grid
		},
		resize: function(){
			var
				bodyNode = this.bodyNode,
				headerNode = this.headerNode,
				footerNode = this.footerNode,
				headerHeight = headerNode.offsetHeight,
				footerHeight = this.showFooter ? footerNode.offsetHeight : 0,
				quirks = has("quirks") || has("ie") < 7;
			
			this.headerScrollNode.style.height = bodyNode.style.marginTop = headerHeight + "px";
			bodyNode.style.marginBottom = footerHeight + "px";
			
			if(quirks){
				// in IE6 and quirks mode, the "bottom" CSS property is ignored.
				// We guard against negative values in case of issues with external CSS.
				bodyNode.style.height = ""; // reset first
				bodyNode.style.height =
					Math.max((this.domNode.offsetHeight - headerHeight - footerHeight), 0) + "px";
				if (footerHeight) {
					// Work around additional glitch where IE 6 / quirks fails to update
					// the position of the bottom-aligned footer; this jogs its memory.
					footerNode.style.bottom = '1px';
					setTimeout(function(){ footerNode.style.bottom = ''; }, 0);
				}
			}
			
			if(!scrollbarWidth){
				// Measure the browser's scrollbar width using a DIV we'll delete right away
				scrollbarWidth = has("dom-scrollbar-width");
				scrollbarHeight = has("dom-scrollbar-height");
				
				// Avoid issues with certain widgets inside in IE7, and
				// ColumnSet scroll issues with all supported IE versions
				if(has("ie")){
					scrollbarWidth++;
					scrollbarHeight++;
				}
				
				// add rules that can be used where scrollbar width/height is needed
				miscUtil.addCssRule(".dgrid-scrollbar-width", "width: " + scrollbarWidth + "px");
				miscUtil.addCssRule(".dgrid-scrollbar-height", "height: " + scrollbarHeight + "px");
				
				if(scrollbarWidth != 17 && !quirks){
					// for modern browsers, we can perform a one-time operation which adds
					// a rule to account for scrollbar width in all grid headers.
					miscUtil.addCssRule(".dgrid-header-row", "right: " + scrollbarWidth + "px");
					// add another for RTL grids
					miscUtil.addCssRule(".dgrid-rtl-swap .dgrid-header-row", "left: " + scrollbarWidth + "px");
				}
			}
			
			if(quirks){
				// old IE doesn't support left + right + width:auto; set width directly
				headerNode.style.width = bodyNode.clientWidth + "px";
				setTimeout(function(){
					// sync up (after the browser catches up with the new width)
					headerNode.scrollLeft = bodyNode.scrollLeft;
				}, 0);
			}
		},
		
		addCssRule: function(selector, css){
			// summary:
			//		Version of util/misc.addCssRule which tracks added rules and removes
			//		them when the List is destroyed.
			
			var rule = miscUtil.addCssRule(selector, css);
			if(this.cleanAddedRules){
				// Although this isn't a listener, it shares the same remove contract
				this._listeners.push(rule);
			}
			return rule;
		},
		
		on: function(eventType, listener){
			// delegate events to the domNode
			var signal = listen(this.domNode, eventType, listener);
			if(!has("dom-addeventlistener")){
				this._listeners.push(signal);
			}
			return signal;
		},
		
		cleanup: function(){
			// summary:
			//		Clears out all rows currently in the list.
			
			var observers = this.observers,
				i;
			for(i in this._rowIdToObject){
				if(this._rowIdToObject[i] != this.columns){
					var rowElement = byId(i);
					if(rowElement){
						this.removeRow(rowElement, true);
					}
				}
			}
			// remove any store observers
			for(i = 0;i < observers.length; i++){
				var observer = observers[i];
				observer && observer.cancel();
			}
			this.observers = [];
			this._numObservers = 0;
			this.preload = null;
		},
		destroy: function(){
			// summary:
			//		Destroys this grid
			
			// Remove any event listeners and other such removables
			if(this._listeners){ // Guard against accidental subsequent calls to destroy
				for(var i = this._listeners.length; i--;){
					this._listeners[i].remove();
				}
				delete this._listeners;
			}
			
			this._started = false;
			this.cleanup();
			// destroy DOM
			put(this.domNode, "!");
			
			if(this.useTouchScroll){
				// Only call TouchScroll#destroy if we also initialized it
				this.inherited(arguments);
			}
		},
		refresh: function(){
			// summary:
			//		refreshes the contents of the grid
			this.cleanup();
			this._rowIdToObject = {};
			this._autoId = 0;
			
			// make sure all the content has been removed so it can be recreated
			this.contentNode.innerHTML = "";
			// Ensure scroll position always resets (especially for TouchScroll).
			this.scrollTo({ x: 0, y: 0 });
		},
		
		newRow: function(object, parentNode, beforeNode, i, options){
			if(parentNode){
				var row = this.insertRow(object, parentNode, beforeNode, i, options);
				put(row, ".dgrid-highlight" +
					(this.addUiClasses ? ".ui-state-highlight" : ""));
				setTimeout(function(){
					put(row, "!dgrid-highlight!ui-state-highlight");
				}, this.highlightDuration);
				return row;
			}
		},
		adjustRowIndices: function(firstRow){
			// this traverses through rows to maintain odd/even classes on the rows when indexes shift;
			var next = firstRow;
			var rowIndex = next.rowIndex;
			if(rowIndex > -1){ // make sure we have a real number in case this is called on a non-row
				do{
					// Skip non-numeric, non-rows
					if(next.rowIndex > -1){
						if(this.maintainOddEven){
							if((next.className + ' ').indexOf("dgrid-row ") > -1){
								put(next, '.' + (rowIndex % 2 == 1 ? oddClass : evenClass) + '!' + (rowIndex % 2 == 0 ? oddClass : evenClass));
							}
						}
						next.rowIndex = rowIndex++;
					}
				}while((next = next.nextSibling) && next.rowIndex != rowIndex);
			}
		},
		renderArray: function(results, beforeNode, options){
			// summary:
			//		This renders an array or collection of objects as rows in the grid, before the
			//		given node. This will listen for changes in the collection if an observe method
			//		is available (as it should be if it comes from an Observable data store).
			options = options || {};
			var self = this,
				start = options.start || 0,
				observers = this.observers,
				rows, container, observerIndex;
			
			if(!beforeNode){
				this._lastCollection = results;
			}
			if(results.observe && this.shouldObserveStore){
				// observe the results for changes
				self._numObservers++;
				var observer = results.observe(function(object, from, to){
					var row, firstRow, nextNode, parentNode;
					
					function advanceNext() {
						nextNode = (nextNode.connected || nextNode).nextSibling;
					}
					
					// a change in the data took place
					if(from > -1 && rows[from]){
						// remove from old slot
						row = rows.splice(from, 1)[0];
						// check to make sure the node is still there before we try to remove it
						// (in case it was moved to a different place in the DOM)
						if(row.parentNode == container){
							firstRow = row.nextSibling;
							if(firstRow){ // it's possible for this to have been already removed if it is in overlapping query results
								if(from != to){ // if from and to are identical, it is an in-place update and we don't want to alter the rowIndex at all
									firstRow.rowIndex--; // adjust the rowIndex so adjustRowIndices has the right starting point
								}
							}
							self.removeRow(row);
						}
						// Update count to reflect that we lost one row
						options.count--;
						// The removal of rows could cause us to need to page in more items
						if(self._processScroll){
							self._processScroll();
						}
					}
					if(to > -1){
						// Add to new slot (either before an existing row, or at the end)
						// First determine the DOM node that this should be placed before.
						if(rows.length){
							if(to === 0){ // if it is the first row, we can safely get the next item
								nextNode = rows[to];
								// Re-retrieve the element in case we are referring to an orphan
								nextNode = nextNode && correctElement(nextNode);
							}else{
								// If we are near the end of the page, we may not be able to retrieve the 
								// result from our own array, so go from the previous row and advance one
								nextNode = rows[to - 1];
								if(nextNode){
									nextNode = correctElement(nextNode);
									// Make sure to skip connected nodes, so we don't accidentally
									// insert a row in between a parent and its children.
									advanceNext();
								}
							}
						}else{
							// There are no rows.  Allow for subclasses to insert new rows somewhere other than
							// at the end of the parent node.
							nextNode = self._getFirstRowSibling && self._getFirstRowSibling(container);
						}
						// Make sure we don't trip over a stale reference to a
						// node that was removed, or try to place a node before
						// itself (due to overlapped queries)
						if(row && nextNode && row.id === nextNode.id){
							advanceNext();
						}
						if(nextNode && !nextNode.parentNode){
							nextNode = byId(nextNode.id);
						}
						parentNode = (beforeNode && beforeNode.parentNode) ||
							(nextNode && nextNode.parentNode) || self.contentNode;
						row = self.newRow(object, parentNode, nextNode, options.start + to, options);
						
						if(row){
							row.observerIndex = observerIndex;
							rows.splice(to, 0, row);
							if(!firstRow || to < from){
								// the inserted row is first, so we update firstRow to point to it
								var previous = row.previousSibling;
								// if we are not in sync with the previous row, roll the firstRow back one so adjustRowIndices can sync everything back up.
								firstRow = !previous || previous.rowIndex + 1 == row.rowIndex || row.rowIndex == 0 ?
									row : previous;
							}
						}
						options.count++;
					}
					
					if(from === 0){
						overlapRows(1, 1);
					}else if(from === results.length - (to === -1 ? 0 : 1)){
						// It was (re)moved from the end
						// (which was the previous length if it was a removal)
						overlapRows(0, 0);
					}
					
					from != to && firstRow && self.adjustRowIndices(firstRow);
					self._onNotification(rows, object, from, to);
				}, true);
				observerIndex = observers.push(observer) - 1;
			}
			var rowsFragment = document.createDocumentFragment(),
				lastRow;

			function overlapRows(){
				// This is responsible for setting row overlaps in result sets to
				// ensure that observable can always properly determine which page
				// an object belongs to.
				// This function uses kind of an esoteric argument, optimized for
				// performance and size, since it is called quite frequently.
				// `sides` is an array of overlapping operations, with a falsy item indicating
				// to add an overlap to the top, and a truthy item means to add an overlap
				// to the bottom (so [0, 1] adds one overlap to the top and the bottom)
				
				var sides = arguments;
				// Only perform row overlap in the case of observable results
				if(observerIndex > -1){
					// Iterate through the sides operations
					for(var i = 0; i < sides.length; i++){
						var top = sides[i];
						var lastRow = rows[top ? 0 : rows.length-1];
						lastRow = lastRow && correctElement(lastRow);
						// check to make sure we have a row, we won't if we don't have any rows
						if(lastRow){
							// Make sure we have the correct row element
							// (not one that was previously removed)
							var row = lastRow[top ? "previousSibling" : "nextSibling"];
							if(row){
								row = self.row(row);
							}
							if(row && row.element != lastRow){
								var method = top ? "unshift" : "push";
								// Take the row and data from the adjacent page and unshift to the
								// top or push to the bottom of our array of rows and results,
								// and adjust the count
								results[method](row.data);
								rows[method](row.element);
								options.count++;
							}
						}
					}
				}
			}
			function correctElement(row){
				// If a node has been orphaned, try to retrieve the correct in-document element
				// (use isDescendant since offsetParent is faulty in IE<9)
				if(!dom.isDescendant(row, self.domNode) && byId(row.id)){
					return self.row(row.id.slice(self.id.length + 5)).element;
				}
				// Fall back to the originally-specified element
				return row;
			}
			
			function mapEach(object){
				lastRow = self.insertRow(object, rowsFragment, null, start++, options);
				lastRow.observerIndex = observerIndex;
				return lastRow;
			}
			function whenError(error){
				if(typeof observerIndex !== "undefined"){
					observers[observerIndex].cancel();
					observers[observerIndex] = 0;
					self._numObservers--;
				}
				if(error){
					throw error;
				}
			}
			var originalRows;
			function whenDone(resolvedRows){
				// Save the original rows, before the overlapping is performed
				originalRows = resolvedRows.slice(0);
				container = beforeNode ? beforeNode.parentNode : self.contentNode;
				if(container && container.parentNode &&
						(container !== self.contentNode || resolvedRows.length)){
					container.insertBefore(rowsFragment, beforeNode || null);
					lastRow = resolvedRows[resolvedRows.length - 1];
					lastRow && self.adjustRowIndices(lastRow);
				}else if(observers[observerIndex] && self.cleanEmptyObservers){
					// Remove the observer and don't bother inserting;
					// rows are already out of view or there were none to track
					whenError();
				}
				rows = resolvedRows;
				if(observer){
					observer.rows = rows;
				}
			}
			
			// Now render the results
			if(results.map){
				rows = results.map(mapEach, console.error);
				if(rows.then){
					return results.then(function(resultsArray){
						results = resultsArray;
						return rows.then(function(resolvedRows){
							whenDone(resolvedRows);
							// Overlap rows in the results array when using observable
							// so that we can determine page boundary changes
							// (but return the original set)
							overlapRows(1, 1, 0, 0);
							return originalRows;
						});
					});
				}
			}else{
				rows = [];
				for(var i = 0, l = results.length; i < l; i++){
					rows[i] = mapEach(results[i]);
				}
			}
			
			whenDone(rows);
			overlapRows(1, 1, 0, 0);
			// Return the original rows, not the overlapped set
			return originalRows;
		},

		_onNotification: function(rows, object, from, to){
			// summary:
			//		Protected method called whenever a store notification is observed.
			//		Intended to be extended as necessary by mixins/extensions.
		},

		renderHeader: function(){
			// no-op in a plain list
		},
		
		_autoId: 0,
		insertRow: function(object, parent, beforeNode, i, options){
			// summary:
			//		Creates a single row in the grid.
			
			// Include parentId within row identifier if one was specified in options.
			// (This is used by tree to allow the same object to appear under
			// multiple parents.)
			var parentId = options.parentId,
				id = this.id + "-row-" + (parentId ? parentId + "-" : "") + 
					((this.store && this.store.getIdentity) ? 
						this.store.getIdentity(object) : this._autoId++),
				row = byId(id),
				previousRow = row && row.previousSibling;
			
			if(row){// if it existed elsewhere in the DOM, we will remove it, so we can recreate it
				if(row === beforeNode){
					beforeNode = (beforeNode.connected || beforeNode).nextSibling;
				}
				this.removeRow(row);
			}
			row = this.renderRow(object, options);
			row.className = (row.className || "") + " dgrid-row " +
				(i % 2 == 1 ? oddClass : evenClass) +
				(this.addUiClasses ? " ui-state-default" : "");
			// get the row id for easy retrieval
			this._rowIdToObject[row.id = id] = object;
			parent.insertBefore(row, beforeNode || null);
			if(previousRow){
				// in this case, we are pulling the row from another location in the grid, and we need to readjust the rowIndices from the point it was removed
				this.adjustRowIndices(previousRow);
			}
			row.rowIndex = i;
			return row;
		},
		renderRow: function(value, options){
			// summary:
			//		Responsible for returning the DOM for a single row in the grid.
			
			return put("div", "" + value);
		},
		removeRow: function(rowElement, justCleanup){
			// summary:
			//		Simply deletes the node in a plain List.
			//		Column plugins may aspect this to implement their own cleanup routines.
			// rowElement: Object|DOMNode
			//		Object or element representing the row to be removed.
			// justCleanup: Boolean
			//		If true, the row element will not be removed from the DOM; this can
			//		be used by extensions/plugins in cases where the DOM will be
			//		massively cleaned up at a later point in time.
			
			rowElement = rowElement.element || rowElement;
			delete this._rowIdToObject[rowElement.id];
			if(!justCleanup){
				put(rowElement, "!");
			}
		},
		
		row: function(target){
			// summary:
			//		Get the row object by id, object, node, or event
			var id;
			
			if(target instanceof this._Row){ return target; } // no-op; already a row
			
			if(target.target && target.target.nodeType){
				// event
				target = target.target;
			}
			if(target.nodeType){
				var object;
				do{
					var rowId = target.id;
					if((object = this._rowIdToObject[rowId])){
						return new this._Row(rowId.substring(this.id.length + 5), object, target); 
					}
					target = target.parentNode;
				}while(target && target != this.domNode);
				return;
			}
			if(typeof target == "object"){
				// assume target represents a store item
				id = this.store.getIdentity(target);
			}else{
				// assume target is a row ID
				id = target;
				target = this._rowIdToObject[this.id + "-row-" + id];
			}
			return new this._Row(id, target, byId(this.id + "-row-" + id));
		},
		cell: function(target){
			// this doesn't do much in a plain list
			return {
				row: this.row(target)
			};
		},
		
		_move: function(item, steps, targetClass, visible){
			var nextSibling, current, element;
			// Start at the element indicated by the provided row or cell object.
			element = current = item.element;
			steps = steps || 1;
			
			do{
				// Outer loop: move in the appropriate direction.
				if((nextSibling = current[steps < 0 ? "previousSibling" : "nextSibling"])){
					do{
						// Inner loop: advance, and dig into children if applicable.
						current = nextSibling;
						if(current && (current.className + " ").indexOf(targetClass + " ") > -1){
							// Element with the appropriate class name; count step, stop digging.
							element = current;
							steps += steps < 0 ? 1 : -1;
							break;
						}
						// If the next sibling isn't a match, drill down to search, unless
						// visible is true and children are hidden.
					}while((nextSibling = (!visible || !current.hidden) && current[steps < 0 ? "lastChild" : "firstChild"]));
				}else{
					current = current.parentNode;
					if(!current || current === this.bodyNode || current === this.headerNode){
						// Break out if we step out of the navigation area entirely.
						break;
					}
				}
			}while(steps);
			// Return the final element we arrived at, which might still be the
			// starting element if we couldn't navigate further in that direction.
			return element;
		},
		
		up: function(row, steps, visible){
			// summary:
			//		Returns the row that is the given number of steps (1 by default)
			//		above the row represented by the given object.
			// row:
			//		The row to navigate upward from.
			// steps:
			//		Number of steps to navigate up from the given row; default is 1.
			// visible:
			//		If true, rows that are currently hidden (i.e. children of
			//		collapsed tree rows) will not be counted in the traversal.
			// returns:
			//		A row object representing the appropriate row.  If the top of the
			//		list is reached before the given number of steps, the first row will
			//		be returned.
			if(!row.element){ row = this.row(row); }
			return this.row(this._move(row, -(steps || 1), "dgrid-row", visible));
		},
		down: function(row, steps, visible){
			// summary:
			//		Returns the row that is the given number of steps (1 by default)
			//		below the row represented by the given object.
			// row:
			//		The row to navigate downward from.
			// steps:
			//		Number of steps to navigate down from the given row; default is 1.
			// visible:
			//		If true, rows that are currently hidden (i.e. children of
			//		collapsed tree rows) will not be counted in the traversal.
			// returns:
			//		A row object representing the appropriate row.  If the bottom of the
			//		list is reached before the given number of steps, the last row will
			//		be returned.
			if(!row.element){ row = this.row(row); }
			return this.row(this._move(row, steps || 1, "dgrid-row", visible));
		},
		
		scrollTo: has("touch") ? function(options){
			// If TouchScroll is the superclass, defer to its implementation.
			return this.useTouchScroll ? this.inherited(arguments) :
				desktopScrollTo.call(this, options);
		} : desktopScrollTo,
		
		getScrollPosition: has("touch") ? function(){
			// If TouchScroll is the superclass, defer to its implementation.
			return this.useTouchScroll ? this.inherited(arguments) :
				desktopGetScrollPosition.call(this);
		} : desktopGetScrollPosition,
		
		get: function(/*String*/ name /*, ... */){
			// summary:
			//		Get a property on a List instance.
			//	name:
			//		The property to get.
			//	returns:
			//		The property value on this List instance.
			// description:
			//		Get a named property on a List object. The property may
			//		potentially be retrieved via a getter method in subclasses. In the base class
			//		this just retrieves the object's property.
			
			var fn = "_get" + name.charAt(0).toUpperCase() + name.slice(1);
			
			if(typeof this[fn] === "function"){
				return this[fn].apply(this, [].slice.call(arguments, 1));
			}
			
			// Alert users that try to use Dijit-style getter/setters so they don’t get confused
			// if they try to use them and it does not work
			if(! 1  && typeof this[fn + "Attr"] === "function"){
				console.warn("dgrid: Use " + fn + " instead of " + fn + "Attr for getting " + name);
			}
			
			return this[name];
		},
		
		set: function(/*String*/ name, /*Object*/ value /*, ... */){
			//	summary:
			//		Set a property on a List instance
			//	name:
			//		The property to set.
			//	value:
			//		The value to set in the property.
			//	returns:
			//		The function returns this List instance.
			//	description:
			//		Sets named properties on a List object.
			//		A programmatic setter may be defined in subclasses.
			//
			//		set() may also be called with a hash of name/value pairs, ex:
			//	|	myObj.set({
			//	|		foo: "Howdy",
			//	|		bar: 3
			//	|	})
			//		This is equivalent to calling set(foo, "Howdy") and set(bar, 3)
			
			if(typeof name === "object"){
				for(var k in name){
					this.set(k, name[k]);
				}
			}else{
				var fn = "_set" + name.charAt(0).toUpperCase() + name.slice(1);
				
				if(typeof this[fn] === "function"){
					this[fn].apply(this, [].slice.call(arguments, 1));
				}else{
					// Alert users that try to use Dijit-style getter/setters so they don’t get confused
					// if they try to use them and it does not work
					if(! 1  && typeof this[fn + "Attr"] === "function"){
						console.warn("dgrid: Use " + fn + " instead of " + fn + "Attr for setting " + name);
					}
					
					this[name] = value;
				}
			}
			
			return this;
		},
		
		// Accept both class and className programmatically to set domNode class.
		_getClass: getClass,
		_setClass: setClass,
		_getClassName: getClass,
		_setClassName: setClass,
		
		_setSort: function(property, descending){
			// summary:
			//		Sort the content
			// property: String|Array
			//		String specifying field to sort by, or actual array of objects
			//		with attribute and descending properties
			// descending: boolean
			//		In the case where property is a string, this argument
			//		specifies whether to sort ascending (false) or descending (true)
			
			this._sort = typeof property != "string" ? property :
				[{attribute: property, descending: descending}];
			
			this.refresh();
			
			if(this._lastCollection){
				if(property.length){
					// if an array was passed in, flatten to just first sort attribute
					// for default array sort logic
					if(typeof property != "string"){
						descending = property[0].descending;
						property = property[0].attribute;
					}
					
					this._lastCollection.sort(function(a,b){
						var aVal = a[property], bVal = b[property];
						// fall back undefined values to "" for more consistent behavior
						if(aVal === undefined){ aVal = ""; }
						if(bVal === undefined){ bVal = ""; }
						return aVal == bVal ? 0 : (aVal > bVal == !descending ? 1 : -1);
					});
				}
				this.renderArray(this._lastCollection);
			}
		},
		// TODO: remove the following two (and rename _sort to sort) in 0.4
		sort: function(property, descending){
			kernel.deprecated("sort(...)", 'use set("sort", ...) instead', "dgrid 0.4");
			this.set("sort", property, descending);
		},
		_getSort: function(){
			return this._sort;
		},
		
		_setShowHeader: function(show){
			// this is in List rather than just in Grid, primarily for two reasons:
			// (1) just in case someone *does* want to show a header in a List
			// (2) helps address IE < 8 header display issue in List
			
			var headerNode = this.headerNode;
			
			this.showHeader = show;
			
			// add/remove class which has styles for "hiding" header
			put(headerNode, (show ? "!" : ".") + "dgrid-header-hidden");
			
			this.renderHeader();
			this.resize(); // resize to account for (dis)appearance of header
			
			if(show){
				// Update scroll position of header to make sure it's in sync.
				headerNode.scrollLeft = this.getScrollPosition().x;
			}
		},
		setShowHeader: function(show){
			kernel.deprecated("setShowHeader(...)", 'use set("showHeader", ...) instead', "dgrid 0.4");
			this.set("showHeader", show);
		},
		
		_setShowFooter: function(show){
			this.showFooter = show;
			
			// add/remove class which has styles for hiding footer
			put(this.footerNode, (show ? "!" : ".") + "dgrid-footer-hidden");
			
			this.resize(); // to account for (dis)appearance of footer
		}
	});
});

},
'dgrid/util/misc':function(){
define("dgrid/util/misc", [
	"dojo/has",
	"put-selector/put"
], function(has, put){
	// summary:
	//		This module defines miscellaneous utility methods for purposes of
	//		adding styles, and throttling/debouncing function calls.
	
	has.add("dom-contains", function(global, doc, element){
		return !!element.contains; // not supported by FF < 9
	});
	
	// establish an extra stylesheet which addCssRule calls will use,
	// plus an array to track actual indices in stylesheet for removal
	var extraRules = [],
		extraSheet,
		removeMethod,
		rulesProperty,
		invalidCssChars = /([^A-Za-z0-9_\u00A0-\uFFFF-])/g;
	
	function removeRule(index){
		// Function called by the remove method on objects returned by addCssRule.
		var realIndex = extraRules[index],
			i, l;
		if (realIndex === undefined) { return; } // already removed
		
		// remove rule indicated in internal array at index
		extraSheet[removeMethod](realIndex);
		
		// Clear internal array item representing rule that was just deleted.
		// NOTE: we do NOT splice, since the point of this array is specifically
		// to negotiate the splicing that occurs in the stylesheet itself!
		extraRules[index] = undefined;
		
		// Then update array items as necessary to downshift remaining rule indices.
		// Can start at index + 1, since array is sparse but strictly increasing.
		for(i = index + 1, l = extraRules.length; i < l; i++){
			if(extraRules[i] > realIndex){ extraRules[i]--; }
		}
	}
	
	var util = {
		// Throttle/debounce functions
		
		defaultDelay: 15,
		throttle: function(cb, context, delay){
			// summary:
			//		Returns a function which calls the given callback at most once per
			//		delay milliseconds.  (Inspired by plugd)
			var ran = false;
			delay = delay || util.defaultDelay;
			return function(){
				if(ran){ return; }
				ran = true;
				cb.apply(context, arguments);
				setTimeout(function(){ ran = false; }, delay);
			};
		},
		throttleDelayed: function(cb, context, delay){
			// summary:
			//		Like throttle, except that the callback runs after the delay,
			//		rather than before it.
			var ran = false;
			delay = delay || util.defaultDelay;
			return function(){
				if(ran){ return; }
				ran = true;
				var a = arguments;
				setTimeout(function(){
					ran = false;
					cb.apply(context, a);
				}, delay);
			};
		},
		debounce: function(cb, context, delay){
			// summary:
			//		Returns a function which calls the given callback only after a
			//		certain time has passed without successive calls.  (Inspired by plugd)
			var timer;
			delay = delay || util.defaultDelay;
			return function(){
				if(timer){
					clearTimeout(timer);
					timer = null;
				}
				var a = arguments;
				timer = setTimeout(function(){
					cb.apply(context, a);
				}, delay);
			};
		},
		
		// Iterative functions
		
		each: function(arrayOrObject, callback, context){
			// summary:
			//		Given an array or object, iterates through its keys.
			//		Does not use hasOwnProperty (since even Dojo does not
			//		consistently use it), but will iterate using a for or for-in
			//		loop as appropriate.
			
			var i, len;
			
			if(!arrayOrObject){
				return;
			}
			
			if(typeof arrayOrObject.length === "number"){
				for(i = 0, len = arrayOrObject.length; i < len; i++){
					callback.call(context, arrayOrObject[i], i, arrayOrObject);
				}
			}else{
				for(i in arrayOrObject){
					callback.call(context, arrayOrObject[i], i, arrayOrObject);
				}
			}
		},
		
		// DOM-related functions
		
		contains: function(parent, node){
			// summary:
			//		Checks to see if an element is contained in another element.
			
			if(has("dom-contains")){
				return parent.contains(node);
			}else{
				return parent.compareDocumentPosition(node) & 8 /* DOCUMENT_POSITION_CONTAINS */;
			}
		},
		
		// CSS-related functions
		
		addCssRule: function(selector, css){
			// summary:
			//		Dynamically adds a style rule to the document.  Returns an object
			//		with a remove method which can be called to later remove the rule.
			
			if(!extraSheet){
				// First time, create an extra stylesheet for adding rules
				extraSheet = put(document.getElementsByTagName("head")[0], "style");
				// Keep reference to actual StyleSheet object (`styleSheet` for IE < 9)
				extraSheet = extraSheet.sheet || extraSheet.styleSheet;
				// Store name of method used to remove rules (`removeRule` for IE < 9)
				removeMethod = extraSheet.deleteRule ? "deleteRule" : "removeRule";
				// Store name of property used to access rules (`rules` for IE < 9)
				rulesProperty = extraSheet.cssRules ? "cssRules" : "rules";
			}
			
			var index = extraRules.length;
			extraRules[index] = (extraSheet.cssRules || extraSheet.rules).length;
			extraSheet.addRule ?
				extraSheet.addRule(selector, css) :
				extraSheet.insertRule(selector + '{' + css + '}', extraRules[index]);
			
			return {
				get: function(prop) {
					return extraSheet[rulesProperty][extraRules[index]].style[prop];
				},
				set: function(prop, value) {
					if (typeof extraRules[index] !== "undefined") {
						extraSheet[rulesProperty][extraRules[index]].style[prop] = value;
					}
				},
				remove: function(){
					removeRule(index);
				}
			};
		},
		
		escapeCssIdentifier: function(id, replace){
			// summary:
			//		Escapes normally-invalid characters in a CSS identifier (such as . or :);
			//		see http://www.w3.org/TR/CSS2/syndata.html#value-def-identifier
			// id: String
			//		CSS identifier (e.g. tag name, class, or id) to be escaped
			// replace: String?
			//		If specified, indicates that invalid characters should be
			//		replaced by the given string rather than being escaped
			
			return typeof id === 'string' ? id.replace(invalidCssChars, replace || "\\$1") : id;
		}
	};
	return util;
});
},
'put-selector/put':function(){
(function(define){
var forDocument, fragmentFasterHeuristic = /[-+,> ]/; // if it has any of these combinators, it is probably going to be faster with a document fragment 
define("put-selector/put", [], forDocument = function(doc, newFragmentFasterHeuristic){
"use strict";
	// module:
	//		put-selector/put
	// summary:
	//		This module defines a fast lightweight function for updating and creating new elements
	//		terse, CSS selector-based syntax. The single function from this module creates
	// 		new DOM elements and updates existing elements. See README.md for more information.
	//	examples:
	//		To create a simple div with a class name of "foo":
	//		|	put("div.foo");
	fragmentFasterHeuristic = newFragmentFasterHeuristic || fragmentFasterHeuristic;
	var selectorParse = /(?:\s*([-+ ,<>]))?\s*(\.|!\.?|#)?([-\w%$|]+)?(?:\[([^\]=]+)=?['"]?([^\]'"]*)['"]?\])?/g,
		undefined, namespaceIndex, namespaces = false,
		doc = doc || document,
		ieCreateElement = typeof doc.createElement == "object"; // telltale sign of the old IE behavior with createElement that does not support later addition of name 
	function insertTextNode(element, text){
		element.appendChild(doc.createTextNode(text));
	}
	function put(topReferenceElement){
		var fragment, lastSelectorArg, nextSibling, referenceElement, current,
			args = arguments,
			returnValue = args[0]; // use the first argument as the default return value in case only an element is passed in
		function insertLastElement(){
			// we perform insertBefore actions after the element is fully created to work properly with 
			// <input> tags in older versions of IE that require type attributes
			//	to be set before it is attached to a parent.
			// We also handle top level as a document fragment actions in a complex creation 
			// are done on a detached DOM which is much faster
			// Also if there is a parse error, we generally error out before doing any DOM operations (more atomic) 
			if(current && referenceElement && current != referenceElement){
				(referenceElement == topReferenceElement &&
					// top level, may use fragment for faster access 
					(fragment || 
						// fragment doesn't exist yet, check to see if we really want to create it 
						(fragment = fragmentFasterHeuristic.test(argument) && doc.createDocumentFragment()))
							// any of the above fails just use the referenceElement  
							 ? fragment : referenceElement).
								insertBefore(current, nextSibling || null); // do the actual insertion
			}
		}
		for(var i = 0; i < args.length; i++){
			var argument = args[i];
			if(typeof argument == "object"){
				lastSelectorArg = false;
				if(argument instanceof Array){
					// an array
					current = doc.createDocumentFragment();
					for(var key = 0; key < argument.length; key++){
						current.appendChild(put(argument[key]));
					}
					argument = current;
				}
				if(argument.nodeType){
					current = argument;
					insertLastElement();
					referenceElement = argument;
					nextSibling = 0;
				}else{
					// an object hash
					for(var key in argument){
						current[key] = argument[key];
					}				
				}
			}else if(lastSelectorArg){
				// a text node should be created
				// take a scalar value, use createTextNode so it is properly escaped
				// createTextNode is generally several times faster than doing an escaped innerHTML insertion: http://jsperf.com/createtextnode-vs-innerhtml/2
				lastSelectorArg = false;
				insertTextNode(current, argument);
			}else{
				if(i < 1){
					// if we are starting with a selector, there is no top element
					topReferenceElement = null;
				}
				lastSelectorArg = true;
				var leftoverCharacters = argument.replace(selectorParse, function(t, combinator, prefix, value, attrName, attrValue){
					if(combinator){
						// insert the last current object
						insertLastElement();
						if(combinator == '-' || combinator == '+'){
							// + or - combinator, 
							// TODO: add support for >- as a means of indicating before the first child?
							referenceElement = (nextSibling = (current || referenceElement)).parentNode;
							current = null;
							if(combinator == "+"){
								nextSibling = nextSibling.nextSibling;
							}// else a - operator, again not in CSS, but obvious in it's meaning (create next element before the current/referenceElement)
						}else{
							if(combinator == "<"){
								// parent combinator (not really in CSS, but theorized, and obvious in it's meaning)
								referenceElement = current = (current || referenceElement).parentNode;
							}else{
								if(combinator == ","){
									// comma combinator, start a new selector
									referenceElement = topReferenceElement;
								}else if(current){
									// else descendent or child selector (doesn't matter, treated the same),
									referenceElement = current;
								}
								current = null;
							}
							nextSibling = 0;
						}
						if(current){
							referenceElement = current;
						}
					}
					var tag = !prefix && value;
					if(tag || (!current && (prefix || attrName))){
						if(tag == "$"){
							// this is a variable to be replaced with a text node
							insertTextNode(referenceElement, args[++i]);
						}else{
							// Need to create an element
							tag = tag || put.defaultTag;
							var ieInputName = ieCreateElement && args[i +1] && args[i +1].name;
							if(ieInputName){
								// in IE, we have to use the crazy non-standard createElement to create input's that have a name 
								tag = '<' + tag + ' name="' + ieInputName + '">';
							}
							// we swtich between creation methods based on namespace usage
							current = namespaces && ~(namespaceIndex = tag.indexOf('|')) ?
								doc.createElementNS(namespaces[tag.slice(0, namespaceIndex)], tag.slice(namespaceIndex + 1)) : 
								doc.createElement(tag);
						}
					}
					if(prefix){
						if(value == "$"){
							value = args[++i];
						}
						if(prefix == "#"){
							// #id was specified
							current.id = value;
						}else{
							// we are in the className addition and removal branch
							var currentClassName = current.className;
							// remove the className (needed for addition or removal)
							// see http://jsperf.com/remove-class-name-algorithm/2 for some tests on this
							var removed = currentClassName && (" " + currentClassName + " ").replace(" " + value + " ", " ");
							if(prefix == "."){
								// addition, add the className
								current.className = currentClassName ? (removed + value).substring(1) : value;
							}else{
								// else a '!' class removal
								if(argument == "!"){
									var parentNode;
									// special signal to delete this element
									if(ieCreateElement){
										// use the ol' innerHTML trick to get IE to do some cleanup
										put("div", current, '<').innerHTML = "";
									}else if(parentNode = current.parentNode){ // intentional assigment
										// use a faster, and more correct (for namespaced elements) removal (http://jsperf.com/removechild-innerhtml)
										parentNode.removeChild(current);
									}
								}else{
									// we already have removed the class, just need to trim
									removed = removed.substring(1, removed.length - 1);
									// only assign if it changed, this can save a lot of time
									if(removed != currentClassName){
										current.className = removed;
									}
								}
							}
							// CSS class removal
						}
					}
					if(attrName){
						if(attrValue == "$"){
							attrValue = args[++i];
						}
						// [name=value]
						if(attrName == "style"){
							// handle the special case of setAttribute not working in old IE
							current.style.cssText = attrValue;
						}else{
							var method = attrName.charAt(0) == "!" ? (attrName = attrName.substring(1)) && 'removeAttribute' : 'setAttribute';
							attrValue = attrValue === '' ? attrName : attrValue;
							// determine if we need to use a namespace
							namespaces && ~(namespaceIndex = attrName.indexOf('|')) ?
								current[method + "NS"](namespaces[attrName.slice(0, namespaceIndex)], attrName.slice(namespaceIndex + 1), attrValue) :
								current[method](attrName, attrValue);
						}
					}
					return '';
				});
				if(leftoverCharacters){
					throw new SyntaxError("Unexpected char " + leftoverCharacters + " in " + argument);
				}
				insertLastElement();
				referenceElement = returnValue = current || referenceElement;
			}
		}
		if(topReferenceElement && fragment){
			// we now insert the top level elements for the fragment if it exists
			topReferenceElement.appendChild(fragment);
		}
		return returnValue;
	}
	put.addNamespace = function(name, uri){
		if(doc.createElementNS){
			(namespaces || (namespaces = {}))[name] = uri;
		}else{
			// for old IE
			doc.namespaces.add(name, uri);
		}
	};
	put.defaultTag = "div";
	put.forDocument = forDocument;
	return put;
});
})(function(id, deps, factory){
	factory = factory || deps;
	if(typeof define === "function"){
		// AMD loader
		define([], function(){
			return factory();
		});
	}else if(typeof window == "undefined"){
		// server side JavaScript, probably (hopefully) NodeJS
		require("./node-html")(module, factory);
	}else{
		// plain script in a browser
		put = factory();
	}
});

},
'xstyle/has-class':function(){
define("xstyle/has-class", ['dojo/has'], function(has){
	var tested = {};
	return function(){
		var test, args = arguments;
		for(var i = 0; i < args.length; i++){
			var test = args[i];
			if(!tested[test]){
				tested[test] = true;
				var parts = test.match(/^(no-)?(.+?)((-[\d\.]+)(-[\d\.]+)?)?$/), // parse the class name
					hasResult = has(parts[2]), // the actual has test
					lower = -parts[4]; // lower bound if it is in the form of test-4 or test-4-6 (would be 4)
				// if it has a range boundary, compare to see if we are in it
				if((lower > 0 ? lower <= hasResult && (-parts[5] || lower) >= hasResult :
						!!hasResult) == !parts[1]){ // parts[1] is the no- prefix that can negate the result
					document.documentElement.className += ' has-' + test;
				}
			}
		}
	};
});
},
'xstyle/css':function(){
define("xstyle/css", ["require"], function(moduleRequire){
"use strict";
/*
 * AMD css! plugin
 * This plugin will load and wait for css files. This allows JavaScript resources to 
 * fully there dependencies on stylesheets. This can also be used when
 * loading css files as part of a layer or as a way to apply a run-time theme. This
 * module checks to see if the CSS is already loaded before incurring the cost
 * of loading the full CSS loader codebase
 */
 	function testElementStyle(tag, id, property){
 		// test an element's style
		var docElement = document.documentElement;
		var testDiv = docElement.insertBefore(document.createElement(tag), docElement.firstChild);
		testDiv.id = id;
		var styleValue = (testDiv.currentStyle || getComputedStyle(testDiv, null) || {})[property];
		docElement.removeChild(testDiv);
 		return styleValue;
 	} 
 	return {
		load: function(resourceDef, require, callback, config) {
			var url = require.toUrl(resourceDef);
			var options;
			if(url.match(/!$/)){
				// a final ! can be used to indicate not to wait for the stylesheet to load
				options = {
					wait: false
				};
				url = url.slice(0, -1);
			}
			var cachedCss = require.cache && require.cache['url:' + url];
			if(cachedCss){
				// we have CSS cached inline in the build
				if(cachedCss.xCss){
					var parser = cachedCss.parser;
					var xCss =cachedCss.xCss;
					cachedCss = cachedCss.cssText;
				}
				moduleRequire(['./core/load-css'],function(load){
					checkForParser(load.insertCss(cachedCss));
				});
				if(xCss){
					//require([parsed], callback);
				}
				return;
			}
			function checkForParser(styleSheetElement){
				var parser = testElementStyle('x-parse', null, 'content');
				var sheet = styleSheetElement && 
					(styleSheetElement.sheet || styleSheetElement.styleSheet);
				if(parser && parser != 'none' && parser != 'normal'){
					// TODO: wait for parser to load
					require([eval(parser)], function(parser){
						if(styleSheetElement){
							parser.process(styleSheetElement, callback);
						}else{
							parser.processAll();
							callback(sheet);
						}
					});
				}else{
					callback(sheet);
				}
			}
			
			// if there is an id test available, see if the referenced rule is already loaded,
			// and if so we can completely avoid any dynamic CSS loading. If it is
			// not present, we need to use the dynamic CSS loader.
			var displayStyle = testElementStyle('div', resourceDef.replace(/\//g,'-').replace(/\..*/,'') + "-loaded", 'display');
			if(displayStyle == "none"){
				return checkForParser();
			}
			// use dynamic loader
			moduleRequire(["./core/load-css"], function(load){
				load(url, checkForParser, options);
			});
		}
	};
});

},
'dgrid/extensions/DijitRegistry':function(){
define("dgrid/extensions/DijitRegistry", ["dojo/_base/declare", "dojo/dom-geometry", "dijit/registry"],
function(declare, domGeometry, registry){
	return declare(null, {
		// summary:
		//		A dgrid extension which will add the grid to the dijit registry,
		//		so that startup() will be successfully called by dijit layout widgets
		//		with dgrid children.
		
		// Defaults normally imposed on _WidgetBase by container widget modules:
		minSize: 0, // BorderContainer
		maxSize: Infinity, // BorderContainer
		layoutPriority: 0, // BorderContainer
		showTitle: true, // StackContainer
		
		buildRendering: function(){
			registry.add(this);
			this.inherited(arguments);
			// Note: for dojo 2.0 may rename widgetId to dojo._scopeName + "_widgetId"
			this.domNode.setAttribute("widgetId", this.id);
		},
		
		startup: function(){
			if(this._started){ return; }
			this.inherited(arguments);
			
			var widget = registry.getEnclosingWidget(this.domNode.parentNode);
			// If we have a parent layout container widget, it will handle resize,
			// so remove the window resize listener added by List.
			if(widget && widget.isLayoutContainer){
				this._resizeHandle.remove();
			}
		},

		destroyRecursive: function() {
			this.destroy();
		},
		
		destroy: function(){
			this.inherited(arguments);
			registry.remove(this.id);
		},
		
		getChildren: function(){
			// provide hollow implementation for logic which assumes its existence
			// (e.g. dijit/form/_FormMixin)
			return [];
		},
		
		isLeftToRight: function(){
			// Implement method expected by Dijit layout widgets
			return !this.isRTL;
		},
		
		resize: function(changeSize){
			// Honor changeSize parameter used by layout widgets, and resize grid
			if(changeSize){
				domGeometry.setMarginBox(this.domNode, changeSize);
			}
			
			this.inherited(arguments);
		},
		
		_set: function(prop, value){
			// summary:
			//		Simple analogue of _WidgetBase#_set for compatibility with some
			//		Dijit layout widgets which assume its existence.
			this[prop] = value;
		},
		
		watch: function(){
			// summary:
			//		dgrid doesn't support watch; this is a no-op for compatibility with
			//		some Dijit layout widgets which assume its existence.
		},
		
		getParent: function(){
			// summary:
			//		Analogue of _WidgetBase#getParent for compatibility with for example
			//		dijit._KeyNavContainer.
			return registry.getEnclosingWidget(this.domNode.parentNode);
		}
	});
});

},
'epi/shell/widget/dialog/Alert':function(){
﻿define("epi/shell/widget/dialog/Alert", [
    "epi",
    "dojo",
    "dijit/form/Button",
    "epi/shell/widget/dialog/_DialogBase"
], function (epi, dojo, Button, _DialogBase) {

    return dojo.declare([_DialogBase], {
        // summary:
        //		A modal alert dialog widget. Does not display a dialog title.
        //
        // tags:
        //      public

        // acknowledgeActionText: [public] String
        //		Label to be displayed for the acknowledge action.
        acknowledgeActionText: epi.resources.action.ok,

        // closeIconVisible: [public] Boolean
        //		Flag which indicates whether the close icon should be visible.
        //		To encourage users to read alert messages only allow closing
        //		via the acknowledge action.
        closeIconVisible: false,

        // dialogClass: [protected] String
        //		Class to apply to the root DOMNode of the dialog.
        dialogClass: "epi-dialog-alert",

        buildRendering: function () {
            // summary:
            //		Construct the UI for this widget from a template, adding
            //		an acknowledge button.
            // tags:
            //		protected

            this.inherited(arguments);

            this.domNode.setAttribute("role", "alertdialog");
            this.descriptionNode.setAttribute("role", "alert");

            var acknowledge = new Button({
                "class": "Salt",
                label: this.acknowledgeActionText,
                onClick: dojo.hitch(this, this._onAcknowledge)
            });
            dojo.place(acknowledge.domNode, this.actionContainerNode);
        },

        _onAcknowledge: function () {
            // summary:
            //		Called when user has triggered the dialog's acknowledge action.
            // type:
            //		callback

            this.hide();
            typeof this.onAction === "function" && this.onAction();
        },

        onAction: function () {
            // summary:
            //		Called when the user has triggered an action on the dialog.
            // type:
            //		public callback
        }
    });
});

},
'epi-cms/core/ContentReference':function(){
define("epi-cms/core/ContentReference", ["dojo/_base/declare", "epi/epi"], function (declare, epi) {

    var ContentReference = declare(null, {
        // summary:
        //      An object uniquely identifying a CMS content item.
        //
        // example:
        //      Create a content reference from an object with properties
        //      |   var contentReference = new epi.cms.core.ContentReference({ id:3, workId:4, providerName: ""});
        //
        // example:
        //      Create a content reference from its serialized string representation
        //      |   var contentReference = new epi.cms.core.ContentReference("3__providerName");
        //
        // tags:
        //      public

        // id: [public] Integer
        //      The id number of an item
        id: 0,

        // workId: [public] Integer
        //      The version id of an item.
        workId: 0,

        // providerName: [public] String
        //      A string that identifies a if an external provider serves the item.
        providerName: null,

        // isSelfReference: [public] Boolean
        //      Denotes if the reference is a self reference
        isSelfReference: false,

        constructor: function (value) {
            // summary:
            //      Creates a new content reference instance from the provided value
            //
            // value: Object|String|Number|epi.cms.core.ContentReference
            //      Can be either an object in the format

            if (value) {
                this._parse(value);
            }
        },

        _parse: function (value) {
            // summary:
            //      Parses an object or a string describing a content reference int this object
            //
            // value: Object|String|Number
            //      The value to parse as a content reference
            //
            // tags:
            //      public

            if (typeof value === "object") {
                // Assume it's an object serialized by the server or an instance of this class
                this.id = parseInt(value.ID || value.id || value.iD || value.Id || 0, 10);
                this.workId = parseInt(value.WorkID || value.workId || 0, 10);
                this.providerName = value.ProviderName || value.providerName || null;
            } else if (typeof value === "string") {
                if (value === "-") {
                    this.isSelfReference = true;
                    this.id = 0;
                    this.workId = -1;
                } else {
                    this.isSelfReference = false;
                    var v = String();
                    v.split("_");
                    var segments = value.split("_");
                    this.id = parseInt(segments[0], 10);
                    this.workId = parseInt(segments[1], 10) || 0;
                    this.providerName = segments[2] || "";
                }
            } else if (typeof value === "number") {
                this.id = value;
                this.workId = 0;
                this.providerName = "";
            }

            // validate data
            if (isNaN(this.id)) {
                throw new Error("The id parameter of a content reference must be an integer");
            }

            if (isNaN(this.workId) && !this.isSelfReference) {
                throw new Error("The workId of a content reference must be an integer");
            }
        },

        toString: function () {
            // summary:
            //      Returns a serialized version of this content reference in the format id[_workId][_providerName]
            //
            // tags:
            //      public

            if (this.isSelfReference) {
                return "-";
            }

            var contentReference = String(this.id);
            if (this.workId !== 0) {
                contentReference += "_" + this.workId;
            }
            if (this.providerName) {
                contentReference += (this.workId === 0 ? "__" : "_") + this.providerName;
            }
            return contentReference;
        },

        createVersionUnspecificReference: function () {
            // summary:
            //      Creates a content link that ignores the work id
            //
            // tags:
            //      public

            return new ContentReference({ id: this.id, providerName: this.providerName });
        }
    });

    ContentReference.isContentReference = function (value) {
        // summary:
        //      Tests whether the given value can be parsed as a content reference.
        //
        // value: Object|String|Number|epi.cms.core.ContentReference
        //      The value to convert to a content reference
        //
        // tags:
        //      public

        try {
            return !!(new ContentReference(value));
        } catch (error) {
            return false;
        }
    };

    ContentReference.toContentReference = function (obj) {
        // summary:
        //      Returns the parameter obj if it's already a ContentReference, or
        //      creates a new ContentReference if it's a string or number.
        //      Otherwise returns an empty ContentReference.
        //
        // value: Object|String|Number|epi.cms.core.ContentReference
        //      The value to create a new content reference
        //
        // tags:
        //      public

        if (obj instanceof ContentReference) {
            return obj;
        } else if (typeof obj === "string" || typeof obj === "number") {
            return new ContentReference(obj);
        }

        return ContentReference.empty;
    };

    ContentReference.tryConvert = function (value, fallback) {
        // summary:
        //      Tries to create a new ContentReference, if it fails
        //      returns the fallback if given otherwise an empty ContentReference.
        //
        // value: Object|String|Number|epi.cms.core.ContentReference
        //      The value to convert to a content reference
        //
        // fallback: Object [optional]
        //      The value to return if the convertion fails, otherwise an empty ContentReference is returned.
        //
        // tags:
        //      public

        if (value instanceof ContentReference) {
            return value;
        }

        try {
            return new ContentReference(value);
        } catch (error) {
            // Allow falsy values as fallback, unless it's undefined.
            return fallback !== undefined ? fallback : ContentReference.empty;
        }
    };

    ContentReference.compareIgnoreVersion = function (object1, object2) {
        // summary:
        //      Compares two ContentReference's while ignoring their versions.
        //
        // object1: epi.cms.core.ContentReference
        //      Value to compare against object2
        //
        // object2: epi.cms.core.ContentReference
        //      Value to compare against object1
        //
        // tags:
        //      public

        var c1 = ContentReference.toContentReference(object1);
        var c2 = ContentReference.toContentReference(object2);

        var pub1 = c1.createVersionUnspecificReference();
        var pub2 = c2.createVersionUnspecificReference();

        return epi.areEqual(pub1, pub2);
    };

    // Deprecate emptyContentReference, and use empty instead.
    ContentReference.empty = new ContentReference({ id: -1, workId: -1 });
    ContentReference.emptyContentReference = ContentReference.empty;

    return ContentReference;
});

},
'epi-cms/ContentRepositoryDescriptorService':function(){
define("epi-cms/ContentRepositoryDescriptorService", [
    "epi/assign"
], function (
    assign
) {
    function ContentRepositoryDescriptorService(contentRepositoryDescriptors) {
        // summary:
        //      Service that contains all registered IContentRepositoryDescriptors from the server
        // tags:
        //      internal

        assign(this, contentRepositoryDescriptors);
    }

    ContentRepositoryDescriptorService.prototype.get = function (key) {
        // summary:
        //      Gets the repository descriptor with the matching key
        // tags:
        //      public

        return !key ? null : this[key.toLowerCase()];
    };

    ContentRepositoryDescriptorService.prototype.list = function () {
        // summary:
        //      Returns all the descriptor keys present in the repository
        // tags:
        //      public

        return Object.keys(this);
    };

    return ContentRepositoryDescriptorService;
});

},
'epi/assign':function(){
﻿define("epi/assign", [], function () {
    // summary:
    //      Polyfill for Object.assign for Internet Explorer
    //      Source: https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/assign#Polyfill
    // tags:
    //      internal ie-polyfill

    if (typeof Object.assign === "function") {
        return Object.assign;
    }

    return function (target, varArgs) { // .length of function is 2
        "use strict";
        if (target == null) { // TypeError if undefined or null
            throw new TypeError("Cannot convert undefined or null to object");
        }

        var to = Object(target);

        for (var index = 1; index < arguments.length; index++) {
            var nextSource = arguments[index];

            if (nextSource != null) { // Skip over if undefined or null
                for (var nextKey in nextSource) {
                    // Avoid bugs when hasOwnProperty is shadowed
                    if (Object.prototype.hasOwnProperty.call(nextSource, nextKey)) {
                        to[nextKey] = nextSource[nextKey];
                    }
                }
            }
        }
        return to;
    };
});

},
'epi-find/manage/DashboardController':function(){
﻿define([
    "dojo/_base/declare",
    "dojo/_base/config",

    "../_ControllerBase",
    "../widget/SwoshContainer",
    "../widget/_SwoshPaneBase",
    "../widget/_ActionableMixin",

    "../widget/Dashboard",
    "./DashboardModel"
],
function(declare, config,
    _ControllerBase, SwoshContainer, _SwoshPaneBase, _ActionableMixin,
    DashboardWidget, DashboardModel
) {
    return declare([_ControllerBase, _ActionableMixin], {
        // summary:
        //      Controller for Dashboard views.

        model: null,
        swoshContainer: null,
        dashboard: null,

        startup: function() {
            if (this.isStarted()) {
                return;
            }
            this.inherited(arguments);
            this.model = new DashboardModel();
            this.model.init();

            var DashboardPane = declare([_SwoshPaneBase, DashboardWidget]);
            this.dashboard = new DashboardPane({
                    model: this.model,
                    name: "dashboard",
                    location: "left"
                });

            this.registerAction(this.dashboard.name, this.dashboard);

            this.swoshContainer = new SwoshContainer({
                model: this.model,
                swoshPanes: [{
                    intent: config.intents.view.dashboard,
                    widget: this.dashboard
                }]
            });
            this.own(this.swoshContainer);

            this.currentAction = this.dashboard.name;
            this.previousActionFallback = true;
        },

        takeAction: function(actions, params) {
            if (!actions || !actions.length) {
                return this.inherited(arguments);
            }
            var activePane = this.findActionable(actions[0]);

            for(var i in this.swoshContainer.swoshPanes) {
                var widget = this.swoshContainer.swoshPanes[i].widget;
                if (activePane == widget) {
                    this.swoshContainer.setActiveWidget(i);
                }
            }
            this.inherited(arguments);
            return activePane;
        },

        refresh: function() {
            this.dashboard.refresh();
        },

        show: function() {
            this._setupView([this.swoshContainer]);
        }
    });
});
},
'epi-find/_ControllerBase':function(){
﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/query",
    "dojo/on",

    "dijit/layout/ContentPane",
    "dojo/dom-class",

    "epi-saas-base/_PropagatingDestroyableMixin",
    "./ApplicationState",
    "./ResponsiveMode"
],

function (declare,
    lang,
    array,
    query,
    on,
    ContentPane,
    domClass,
    _PropagatingDestroyableMixin,
    ApplicationState,
    ResponsiveMode
    ) {

    return declare([_PropagatingDestroyableMixin], {
        // summary:
        //          Controller for views.

        applicationContainer: null,

        rootNode: null,
        rootWidget: null,
        currentResponsiveMode: null,

        widgetsInAnimation: null,
        _started: false,

        constructor: function(application, rootNode) {
            // summary:
            //          Initializes the controller instance
            this.widgetsInAnimation = ApplicationState.widgetsInAnimation;
            this.application = application;
            this.applicationContainer = rootNode;
            this.globalNavigation = query(".epi-navigationContainer")[0];
        },

        isStarted: function() {
            return this._started;
        },

        startup: function() {
            if (this.isStarted()) {
                return;
            }
            this._createRootWidget();
            this.own(
                on(ResponsiveMode, "modeChanged", lang.hitch(this, function(mode) {
                    this.onModeChanged(mode);
                    this.currentResponsiveMode = mode;
                }))
            );
            this.currentResponsiveMode = ResponsiveMode.currentMode.name;
            this._started = true;
        },

        refresh: function() {
            //  summary:
            //      Called when model data has been invalidated and needs to be refreshed
            //      Default behaviour is to try to refresh the rootWidget

            var children = this.rootWidget.getChildren();
            for(var i in children) {
                var widget = children[i];
                if (widget.refresh) {
                    widget.refresh();
                }
            }
        },

        destroy: function () {
            if (!this.isStarted()) {
                return;
            }
            this.inherited(arguments);
            this._started = false;
        },

        onModeChanged: function(mode) {
        },

        _createRootWidget: function () {

            if(!this.rootWidget) {
                this.rootWidget = new ContentPane();
                domClass.add(this.rootWidget.domNode, "epi-pane-wrapper");
                this.own(this.rootWidget);
            }

        },

        _setupView: function (childWidgets, /* bool */destroy) {

            // This whole method needs refactoring (and rethinking). Right now, the _setupView needs to move already existing
            // views out of the way.

            var appNode = this.applicationContainer;
            if (array.indexOf(appNode.children, this.rootWidget.domNode) === -1) {
                while(appNode.firstChild) {
                    appNode.removeChild(appNode.firstChild);    // Removing lazy space hoggers
                }
                this.rootWidget.placeAt(appNode);
                this.rootWidget.startup();
            }

            if (!lang.isArray(childWidgets)) {
                childWidgets = [childWidgets];
            }
            // remove views that are not in the required childWidgets
            array.forEach(this.rootWidget.getChildren(), lang.hitch(this, function (child) {
                if (array.indexOf(childWidgets, child) === -1 &&// if not in the specified array
                    !this._isWidgetAnimating(child)) { // and not in the currently animated widgets
                    this.rootWidget.removeChild(child);
                    if (destroy) {
                        child.destroyRecursive();
                    }
                }
            }));
            array.forEach(childWidgets, lang.hitch(this, function (child) {
                if (!this._isWidgetAdded(child)) {
                    this.rootWidget.addChild(child);
                    child.startup();
                    if (child.resize) {
                       child.resize();
                    }
                }
            }));
            this.onModeChanged(this.currentResponsiveMode);
        },

        _isWidgetAdded: function (widget) {
            return (array.indexOf(this.rootWidget.getChildren(), widget) !== -1);
        },

        _isWidgetAnimating: function (widget) {
            return array.indexOf(this.widgetsInAnimation, widget) >= 0;
        },

        _removeWidgetsInAnimation: function (destroy) {
            array.forEach(this.widgetsInAnimation, lang.hitch(this, function (child) {
                if (this.rootWidget.getIndexOfChild(child)!==-1)
                {
                    this.rootWidget.removeChild(child);
                    if (destroy) {
                        child.destroyRecursive();
                    }
                }
            }));
            this.widgetsInAnimation.splice(0);
        }
    });
});

},
'epi-find/ResponsiveMode':function(){
﻿define("epi-find/ResponsiveMode", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/window",

    "dijit/Destroyable",
    "dijit/Viewport",

    "./_ResponsiveMixin"

], function (declare, lang, window,
             Destroyable, Viewport,
             _ResponsiveMixin) {
    var ResponsiveMode = declare([Destroyable, _ResponsiveMixin], {

        constructor: function() {
            this.own(
                Viewport.on("resize", lang.hitch(this, this.resize))
            );
            this.startup();
        },

        getNewSize : function() {
            return  window.getBox();
        }

    });
    var responsiveMode = new ResponsiveMode();
    return responsiveMode;
});

},
'epi-find/_ResponsiveMixin':function(){
﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/_base/Deferred",
    "dojo/on",
    "dojo/Evented",
    "dojo/window",
    "dojo/dom-geometry"

],
function (declare, lang, array, Deferred, on, Evented, window, domGeometry) {
    // This module responsive mode switching

    return declare([Evented], {
        // modes: Object
        //      The available modes list
        modes: null,
        
        // currentWidth: number
        //      The current width in pixels
        currentWidth: null,
        
        // currentMode: Mode
        //      The current mode
        currentMode: null,

        constructor: function (options) {
            this.currentMode = null;
            this.modes = {
                small: { width: 768, name: "small" },
                medium: { width: 1024, name: "medium" },
                full: { width: Infinity, name: "full" }
            };
            if (options && options.modes) {
                declare.safeMixin(this.modes, options.modes);
            }
        },

        startup: function() {
            this.inherited(arguments);
            this._viewResize(); // initial responsive mode detection
        },

        resize: function() {
            this.inherited(arguments);
            this._viewResize();
        },

        getNewSize: function () {
            return this._contentBox || domGeometry.getMarginBox(this.domNode);
        },

        _viewResize: function() {
            var newSize = this.getNewSize();
            if (!newSize.w) {
                return;
            }
            var previousModeWidth = 0;
            this.currentWidth = newSize.w;
            this.objEach(this.modes, function (mode, key) {
                var currentWidth = newSize.w;
                if (mode.width && currentWidth <= mode.width && currentWidth > previousModeWidth) {
                    if (this.currentMode !== mode) {
                        this.currentMode = mode;
                        this.modeChanged(key);
                    }
                }
                previousModeWidth = mode.width; 
            }, this);
        },

        modeChanged: function(modeName) {
            // summary:
            //      A callback function for mode changed notification
            this.emit("modeChanged", modeName);
        },
        
        objEach: function (obj, f, scope) {
            for(var key in obj){
                if(obj.hasOwnProperty(key)){
                    f.call(scope, obj[key], key);
                }
            }
        }
    });
});

},
'epi-find/widget/SwoshContainer':function(){
define([
    "dojo/_base/declare",
    "dojo/dom-class",
    "dojo/dom-style",
    "dojo/Stateful",
    "dojo/topic",

    "dijit/layout/ContentPane",
    "dijit/focus"
],
function(declare, domClass, domStyle, Stateful, topic,
    ContentPane, focusUtil
) {

    // module:
    //      epi-find/widget/Swosh

    return declare([ContentPane], {
        // summary:
        //     A base widget for swosh-containers.

        // swoshPanes: { intent: string,  widget: epi-find/widget/_SwoshPaneBase[] }
        //      A list of content panes to switch amongst.
        swoshPanes: null,

        // model: dojo/Stateful
        //      Used to store the property "activePaneIndex"
        model: null,

        activeClassName: "is-active",

        hiddenClassName: "is-hidden",

        postMixInProperties: function() {
            this.inherited(arguments);

            if (! this.model) {
                this.model = new Stateful();
            }

            if (!this.model.get("activePaneIndex")) {
                this.model.set("activePaneIndex", 0);
            }

            if( !this.swoshPanes) {
                this.swoshPanes = [];
            }
        },

        postCreate: function() {
            this.inherited(arguments);

            for (var i in this.swoshPanes) {
                var widget = this.swoshPanes[i].widget;
                this.addChild(widget);
                this.own(widget);

                domClass.add(widget.domNode, this.hiddenClassName);
            }
            //TODO: MAKE A CLASS!
            domStyle.set(this.domNode, "height", "100%");
            domStyle.set(this.domNode, "position", "relative");
            domStyle.set(this.domNode, "overflow", "hidden");
            domClass.replace(this.getActiveWidget().domNode, this.activeClassName, this.hiddenClassName);

            var self = this;
            this.own(
                // The widget will express the intent of switching views on focus change
                focusUtil.watch("activeStack", function(name, oldFocusStack, newFocusStack) {
                    for(var i in self.swoshPanes) {
                        var pane = self.swoshPanes[i],
                            hadFocus = oldFocusStack.indexOf(pane.widget.id) > -1,
                            hasFocus = newFocusStack.indexOf(pane.widget.id) > -1,
                            isActivePane = self.model.activePaneIndex === i;
                        if (! hadFocus && hasFocus && ! isActivePane) {
                            topic.publish(pane.intent, {});
                            return;
                        }
                    }
                }),

                // The intent will hopefully bubble up and end up in a call to setActiveWidget will will trigger
                // the fancy switching of panes.
                this.model.watch("activePaneIndex", function() {
                    var activePane = self._getActiveWidgetWrapper();

                    if (self.swoshPanes.indexOf(activePane) === -1) {
                        return;
                    }

                    for (var i in self.swoshPanes) {
                        var widget = self.swoshPanes[i].widget;

                        if (widget === activePane.widget) {
                            domClass.replace(widget.domNode, self.activeClassName, self.hiddenClassName);

                            var hasFocus = focusUtil.activeStack.indexOf(widget.id) > -1;
                            if (! hasFocus) {
                                focusUtil.focus(widget.domNode);
                            }

                            widget.showChildren();
                            widget.resize();    // So that widget resizes child elements if needed
                        } else {
                            domClass.replace(widget.domNode, self.hiddenClassName, self.activeClassName);
                            widget.hideChildren();
                        }
                    }
                })
            );
        },

        getActiveWidget: function() {
            return this._getActiveWidgetWrapper().widget;
        },

        _getActiveWidgetWrapper: function() {
            return this.swoshPanes[this.model.get("activePaneIndex")];
        },

        setActiveWidget: function(index) {
            this.model.set("activePaneIndex", index);
        },

        startup: function() {
            this.inherited(arguments);

            // Trigger initial swosh
            this.model.set("activePaneIndex", this.model.get("activePaneIndex"));
        },

        destroy: function() {
            this.inherited(arguments);
        }
    });
});
},
'epi-find/widget/_SwoshPaneBase':function(){
define([
    "dojo/_base/declare",
    "dojo/dom-class",
    "dojo/dom-construct",
    "dojo/dom-style",

    "dijit/layout/ContentPane",

    "./_StyleFocusMixin"

],
function(declare, domClass, domConstruct, domStyle,
    ContentPane,
    _StyleFocusMixin
) {
    // module:
    //      epi-find/widget/_SwoshPaneBase

    // TODO: Perhaps we would like to destroy the entire subwidget? Enable this and make it configurable.

    return declare([ContentPane, _StyleFocusMixin], {
        // name: String
        //      The name of the pane, will be used to define the css class
        name: null,

        // i18n: object
        //      The i18n object which contains strings for the template
        i18n: null,

        // removedChildren: _WidgetBase[]
        //      Array to save references to children removed onBlur
        removedChildren: null,

        // location: String (left|right)
        //      If the swosh pane is located to the right or the left
        location: null,

        // scrollNode: DomNode
        //      Defines what DomNode of the widget that is interesting for connecting scroll listeners to
        scrollNode: null,

        constructor: function() {
            this.removedChildren = [];
        },

        buildRendering: function() {

            var original = this.domNode; // Keep reference to original domNode to place it inside containernode.
            this.domNode = domConstruct.toDom("<div tabindex\"0\" class=\"epi-pane  epi-pane--"+this.location+"\"><a class=\"epi-pane-selector\"><span class=\"epi-pane-selector__arrow\">›</span></a>");
            this.wrapperNode = domConstruct.toDom("<div class=\"epi-pane__content-wrapper\"><div class=\"epi-pane__header\"><h1>"+ (this.i18n && this.i18n.header ? this.i18n.header : "") +"</h1><p class=\"epi-pane__description\">"+ (this.i18n && this.i18n.description ? this.i18n.description : "") +"</p></div></div></div>");
            this.scrollNode = this.containerNode = domConstruct.toDom("<div class=\"epi-pane__content\"><p class=\"epi-pane__description\">"+ (this.i18n && this.i18n.description ? this.i18n.description : "") +"</p></div>");

            domConstruct.place(this.containerNode, this.wrapperNode);
            domConstruct.place(this.wrapperNode, this.domNode);
            if(original) {
                domConstruct.place(original, this.containerNode);
            }

            this.inherited(arguments);
        },

        postCreate: function() {
            this.inherited(arguments);

            if(this.name) {
                domClass.add(this.domNode, "epi-pane--"+this.name);
            }
        },

        showChildren: function () {
            while(this.removedChildren.length > 0) {
                this.addChild(this.removedChildren.shift());
            }
        },

        hideChildren: function() {
            while(this.getChildren().length > 0) {
                var child = this.getChildren()[0];
                this.removeChild(child);
                this.removedChildren.push(child);
            }
        }
    });
});
},
'epi-find/widget/_StyleFocusMixin':function(){
define([
    "dojo/dom-class",
    "dojo/_base/declare",
    "dijit/_FocusMixin"
], function (domClass, declare, _FocusMixin) {

    // module:
    //      epi-find/widget/_StyleFocusMixin

    // Copied from "epi/shell/widget/_FocusableMixin"

    return declare( _FocusMixin, {
        onFocus: function () {
            // summary:
            //      Called when the widget becomes "active" because
            //      it or a widget inside of it either has focus, or has recently
            //      been clicked.
            this.inherited(arguments);
            domClass.add(this.domNode, "epi-focused");
        },
        onBlur: function () {
            // summary:
            //      Called when the widget stops being "active" because
            //      focus moved to something outside of it, or the user
            //      clicked somewhere outside of it, or the widget was
            //      hidden.
            this.inherited(arguments);
            domClass.remove(this.domNode, "epi-focused");
        }
    });
});
},
'epi-find/widget/_ActionableMixin':function(){
define([
    "dojo/_base/declare"
],
function(declare) {
    //  summary:
    //      A mixin that should be used for anything taking part of navigation.
    //      An actionable is:
    //          * Something that maintains a list of actions that can be taken.
    //          * When urged to, choses one of the child-Actionables based on what is on top on the provided action stack
    //              and delegates further action to that child-Actionable.
    //      If an actual action should be taken, any inheriting object can override takeAction.

    // module:
    //      epi-find/widget/_ActionableMixin

    return declare([], {

        _actions: null,
        _previousActionable: null,

        // previousActionFallback: Boolean
        //       Indicates whether previous action should be taken if unable to resolve required action by arguments
        previousActionFallback: false,

        currentAction: null,

        registerAction: function( /* string */ name, /* _ActionableMixin | function */ actionable) {
            this._actions = this._actions || {};
            this._actions[name] = actionable;
        },

        findActionable: function(context) {
            for (var name in this._actions) {
                var actionable = this._actions[name];
                if (name === context) {
                    return actionable;
                }
            }
            return null;
        },

        takeAction: function(actions, params) {
            if (!actions) {
                return null;
            }
            if (!(actions instanceof Array)) {
                actions = [actions];
            }
            var context = actions.shift();
            var actionable = this.findActionable(context);
            if (actionable) {
                this._previousActionable = actionable;
                this._executeActionable(actionable, actions, params);
                if (actionable.currentAction) {
                    this.currentAction = context + "/" + actionable.currentAction;
                }
                else {
                    this.currentAction = context;
                }
                return actionable;
            }
            else if (this.previousActionFallback && this._previousActionable) {
                this._executeActionable(this._previousActionable, actions, params);
                return this._previousActionable;
            }

            // TODO: How do we want to handle route failures?
            return null;
        },

        _executeActionable: function (actionable, actions, params) {
            if (actionable.takeAction) {
                return actionable.takeAction(actions, params);
            } else {
                actionable(actions, params);
                return actionable;
            }
        }
    });
});
},
'epi-find/widget/Dashboard':function(){
define([
        "dojo/_base/declare",
        "dojo/_base/lang",
        "dojo/_base/config",
        "dojo/_base/array",
        "dojo/dom-class",
        "dojo/dom-style",
        "dojo/on",
        "dojo/date/stamp",
        "dojo/number",
        "dojo/topic",
        "dojo/when",
        "dijit/layout/ContentPane",
        "dijit/_Contained",
        "dijit/form/Button",
        "dijit/Viewport",
        "dijit/_WidgetBase",
        "dijit/_Container",
        "dojox/charting/Chart",
        "dojox/charting/action2d/Tooltip",
        "dojox/charting/action2d/Magnify",
        "dojox/charting/StoreSeries",
        "dojox/mvc/at",
        "put-selector/put",
        "epi-saas-base/Sticker",
        "epi-saas-base/form/RadioButtonGroupList",
        "../AccessModel",
        "../themes/charting/Dashboard",
        "./_ActionableMixin",
        "./_DisabledViewMixin",
        "./ContentWrapperWithSidebar",
        "./PhraseDashboardDialog",
        "./HelpText",
        "./CustomRanges",
        "./DashboardRanges",
        "./DashboardGrid",
        "dojo/i18n!./nls/Dashboard",
        // Chart plugins (not used as arguments)
        "dojox/charting/plot2d/StackedAreas",
        "dojox/charting/plot2d/Markers",
        "dojox/charting/axis2d/Default"
    ],
    function(
            // dojo
            declare,
            lang,
            config,
            array,
            domClass,
            domStyle,
            on,
            stamp,
            number,
            topic,
            when,
            // dijit
            ContentPane,
            _Contained,
            Button,
            Viewport,
            _WidgetBase,
            _Container,
            // dojox
            Chart,
            Tooltip,
            Magnify,
            StoreSeries,
            at,
            put,
            // epi-saas-base
            Sticker,
            RadioButtonGroupList,
            // .
            AccessModel,
            theme,
            _ActionableMixin,
            _DisabledViewMixin,
            ContentWrapperWithSidebar,
            PhraseDashboardDialog,
            HelpText,
            CustomRanges,
            DashboardRanges,
            DashboardGrid,

            i18n) {

        // module:
        //      epi-find/widget/Dashboard

        var Views = {
            // summary:
            //     These are the views avaiable in the Dashboard widget
            mostFrequent: {
                // name: string
                //      String identifier for most frequent searches
                name: "mostFrequent",
                // queryType: string
                //      Query parameter "type" added to endpoint queries
                queryType: "top"
            },
            withoutHits: {
                // name: string
                //      String identifier for searches without hits
                name: "withoutHits",
                // queryType: string
                //      Query parameter "type" added to endpoint queries
                queryType: "null"
            },
            withoutRelevantHits: {
                // name: string
                //      String identifier for searches without relevant hits
                name: "withoutRelevantHits",
                // queryType: string
                //      Query parameter "type" added to endpoint queries
                queryType: "nullclick"
            }
        };

        var labels = {
            view: {
                mostFrequent: i18n.radioButtons_view_mostFrequent,
                withoutHits: i18n.radioButtons_view_withoutHits,
                withoutRelevantHits: i18n.radioButtons_view_withoutRelevantHits
            },
            span: {
                last24hours: i18n.radioButtons_span_last24hours,
                lastWeek: i18n.radioButtons_span_lastWeek,
                lastMonth: i18n.radioButtons_span_lastMonth,
                lastYear: i18n.radioButtons_span_lastYear,
                dateRange: i18n.radioButtons_span_dateRange
            }
        };

        return declare([ContentPane, _ActionableMixin, _DisabledViewMixin], {
            // summary:
            //      Dashboard widget

            // model: epi-find/manage/ManageModel
            //      A stateful object containing current ranges, queries and stores
            model: null,
            // chart: dojox/charting/Chart
            //      The Chart instance
            chart: null,
            // chartPane: dijit/layout/ContentPane
            //      The ContentPane containing the Chart instance
            chartPane: null,
            // grid: dgrid/OnDemandGrid
            //      The dgrid/Grid instance
            grid: null,
            // chartPane: dijit/layout/ContentPane
            //      The ContentPane containing the Grid instance
            gridPane: null,

            _createBestBetsButton: null,
            _createRelatedQueriesButton: null,
            _createSynonymsButton: null,
            _downloadRawButton: null,

            i18n: i18n,

            constructor: function(model) {
                this.model = model;
            },

            postMixInProperties: function() {
                this.inherited(arguments);
                var model = this.model,
                    _getTS = function( /*Date*/ date) {
                        return stamp.toISOString(date, {
                            zulu: true
                        });
                    };

                if (!model.get("view")) {
                    model.set("view", Views.mostFrequent.name);
                }

                if (!model.get("span")) {
                    model.set("span", DashboardRanges.last24hours.name);
                }

                if (!model.get("range")) {
                    model.set("range", DashboardRanges[model.get("span")].getRange());
                }

                // Init query
                if (!model.get("query")) {
                    model.set("query", {
                        from: _getTS(model.get("range").from),
                        to: _getTS(model.get("range").to),
                        interval: model.get("range").interval
                    });
                }

                this.own(model.watch(function(name, oldValue, newValue) {

                    if (oldValue === newValue) {
                        return;
                    }
                    var currentQuery = lang.mixin({}, model.get("query"));

                    switch (name) {
                        case "span":
                            if (oldValue !== newValue) {
                                model.set("range", DashboardRanges[newValue].getRange());

                                if (newValue !== DashboardRanges.dateRange.name) {
                                    CustomRanges.setDisabled(true);
                                } else {
                                    // Enable datepicker to allow customer to pick a range they want
                                    CustomRanges.setDisabled(false);
                                }
                            }
                            break;
                        case "range":
                            model.set("query", lang.mixin({}, currentQuery, {
                                "to": _getTS(newValue.to),
                                "from": _getTS(newValue.from),
                                "interval": newValue.interval
                            }));
                            break;
                        case "rowQuery":
                            model.set("query", lang.mixin({}, currentQuery, {
                                "query": newValue
                            }));
                            break;
                        case "view":
                            var typeQuery = { "type": Views[newValue].queryType },
                                extendedQuery = Views[newValue].queryType === Views.withoutRelevantHits.queryType ? { extended: true } : {};
                            delete currentQuery.type;
                            delete currentQuery.extended;
                            model.set("query", lang.mixin({}, currentQuery, typeQuery, extendedQuery ));
                            break;
                        default:
                            break;
                    }
                }));

                this.own(CustomRanges.startDate.watch(function(name, oldValue, newValue) {
                    if (oldValue === newValue || name !== "value") {
                        return;
                    }
                    if (!CustomRanges.disabledStatus) {
                        model.set("range", DashboardRanges.dateRange.getRange({from: newValue}));
                    }
                }));
                this.own(CustomRanges.endDate.watch(function(name, oldValue, newValue) {
                    if (oldValue === newValue || name !== "value") {
                        return;
                    }
                    if (!CustomRanges.disabledStatus) {
                        model.set("range", DashboardRanges.dateRange.getRange({to: newValue}));
                    }
                }))
            },
            makeButton: function(grid, label, iconClass, view) {
                return new Button({
                    label: label,
                    disabled: true,
                    iconClass: "owl-function-icon--24x24 " + iconClass,
                    onClick: function() {
                        var phrases = [];
                        for (var key in grid.selection) {
                            phrases.push(key);
                        }
                        topic.publish(view, {
                            phrases: phrases.join()
                        });
                    }
                });
            },
            buildRendering: function() {
                this.inherited(arguments);

                var self = this;
                // Navbar
                var model = this.model,
                    ContainedContentPane = declare([ContentPane, _Contained]),
                    viewFormList = new RadioButtonGroupList({
                        "class": "epi-pane__content__nav  clearfix",
                        name: "view",
                        options: [
                            Views.mostFrequent.name,
                            Views.withoutHits.name,
                            Views.withoutRelevantHits.name
                        ],
                        labels: labels,
                        value: at(model, "view")
                    }),
                    // Chart
                    chartPane = this.chartPane = new ContainedContentPane({
                        "class": "epi-vertical-spacer"
                    }),
                    ChartWrapper = declare([_WidgetBase, _Container], {
                        hide: function() {
                            domClass.add(chartWrapper.domNode, "epi-hidden");
                        },
                        show: function() {
                            domClass.remove(chartWrapper.domNode, "epi-hidden");
                        },
                        isVisible: function() {
                            return !domClass.contains(chartWrapper.domNode, "epi-hidden");
                        }
                    }),
                    chartWrapper = this.chartWrapper = new ChartWrapper({}),
                    // Grid
                    gridPane = this.gridPane = new ContentPane({
                        "class": "epi-vertical-spacer--double"
                    }),
                    grid = this.grid = new DashboardGrid({
                        "class": "epi-plain-grid  epi-plain-grid--no-border dgrid-autoheight",
                        query: this.model.get("query"),
                        store: this.model.topQueryStore,
                        loadingMessage: "<div class='dgrid-loading-icon'></div><div class='dgrid-loading-message'>" + i18n.dataLoading + "</div>",
                        showLoadingMessage: true,
                        shouldObserveStore: false
                    }),
                    // "Create"-buttons
                    createSynonymsButton = this._createSynonymsButton = this.makeButton(grid, i18n.createSynonyms, "epi-iconSynonyms", config.intents.create.synonym),
                    createRelatedQueriesButton = this._createRelatedQueriesButton = this.makeButton(grid, i18n.createRelatedQueries, "epi-iconRelatedQueries", config.intents.create.relatedqueries),
                    createBestBetsButton = this._createBestBetsButton = this.makeButton(grid, i18n.createBestBets, "epi-iconBestBets", config.intents.create.bestbet),
                    downloadRawButton = this._downloadRawButton = new Button({
                        label: i18n.downloadRawData,
                        disabled: true,
                        iconClass: "owl-function-icon--24x24 " + "epi-iconDownloadData",
                        onClick: function() {
                            if (grid._total > 0) {
                                // Download top_query
                                when(self.model.topQueryStore.query(Object.assign({}, grid.query, {size: grid._total})),
                                    function(results) {
                                        if (results) {
                                            var csvTitle = "data:text/csv;charset=utf-8," + i18n.fields_query + "," + i18n.fields_numberOfSearches,
                                                csvFileName = "statistic_most_frequent_searches.csv";

                                            if (grid.query.type == "nullclick") {
                                                csvTitle += "," + i18n.fields_clickRate;
                                                csvFileName = "statistic_searches_without_relevant_hits.csv";
                                            } else if (grid.query.type == "null") {
                                                csvFileName = "statistic_searches_without_hits.csv";
                                            }

                                            var data = array.map(results, function(item) {
                                                if (grid.query.type == "nullclick") {
                                                    // Searches without relevant hits
                                                    var total = 0,
                                                        percentage = 0;

                                                    if (item.extended && item.extended.total > 0) {
                                                        total = item.extended.total;
                                                        percentage = 100 - Math.round(item.count / total * 100);
                                                    }

                                                    return item.query + "," + total + "," + percentage + "%";
                                                }
                                                return item.query + "," + item.count;
                                            }).join("\n");
                                            var csvContent = csvTitle + "\n" + data;
                                            var encodedUri = encodeURI(csvContent);

                                            dojo.create("a", { href: encodedUri, download: csvFileName, id: "linkDownloadRawData" }, dojo.body());
                                            dojo.byId('linkDownloadRawData').click();
                                            dojo.destroy('linkDownloadRawData');
                                        }
                                    },
                                    function(error) {
                                        alert("Can't download data! Error: " + error);
                                    }
                                );

                                if (grid.query.type != "nullclick") {
                                    // Download count_query
                                    when(self.model.queryStore.query(grid.query),
                                        function(results) {
                                            if (results) {
                                                var csvTitle = "data:text/csv;charset=utf-8," + i18n.fields_timestamp + "," + i18n.fields_numberOfSearches;
                                                var csvFileName = "statistic_most_frequent_searches_count.csv";
                                                if (grid.query.type == "null") {
                                                    csvFileName = "statistic_searches_without_hits_count.csv";
                                                }

                                                var data = array.map(results, function(item) {
                                                    var timestamp = "";
                                                    var timestampArr = item.timestamp.split('T');
                                                    if (timestampArr.length < 2) {
                                                        return "";
                                                    }
                                                    timestamp += timestampArr[0];

                                                    if (grid.query.interval === "hour") {
                                                        var timestampHour = timestampArr[1].split(":");
                                                        if (timestampHour.length < 2) {
                                                            return "";
                                                        }
                                                        timestamp += " " + timestampHour[0] + ":" + timestampHour[1];
                                                    }

                                                    return timestamp + "," + item.count;
                                                }).join("\n");
                                                var csvContent = csvTitle + "\n" + data;
                                                var encodedUri = encodeURI(csvContent);

                                                dojo.create("a", { href: encodedUri, download: csvFileName, id: "linkDownloadFacetData" }, dojo.body());
                                                dojo.byId('linkDownloadFacetData').click();
                                                dojo.destroy('linkDownloadFacetData');
                                            }
                                        },
                                        function(error) {
                                            alert("Can't download data! Error: " + error);
                                        }
                                    );
                                }
                            }
                        }
                    }),
                    phraseDashboardDialog = new PhraseDashboardDialog({
                        model: this.model,
                        destroyOnHide: false
                    });

                this.own(phraseDashboardDialog);

                // Span radio buttons
                this.spanFormList = new RadioButtonGroupList({
                    "class": "epi-filter-list",
                    name: "span",
                    options: [
                        DashboardRanges.last24hours.name,
                        DashboardRanges.lastWeek.name,
                        DashboardRanges.lastMonth.name,
                        DashboardRanges.lastYear.name,
                        DashboardRanges.dateRange.name
                    ],
                    labels: labels,
                    value: at(model, "span")
                });

                // View form list (navigation)
                this.own(viewFormList);
                this.addChild(viewFormList);

                this.set("notificationPosition", 1);

                // Content wrapper for chart and span selector
                var contentWrapper = this.contentWrapper = new ContentWrapperWithSidebar();
                this.own(contentWrapper);
                this.addChild(contentWrapper);

                this.set("disabledNode", contentWrapper.domNode);

                // Chart - has to be wrapped with help text for proper hide/show
                chartWrapper.addChild(new HelpText({
                    help: i18n.help_chart
                }));
                this.own(chartPane);
                this.own(chartWrapper);
                chartWrapper.addChild(chartPane);
                contentWrapper.addChild(chartWrapper);
                domStyle.set(chartPane.containerNode, "height", "300px"); // Needed for chart to claim its space properly
                domStyle.set(chartPane.containerNode, "overflow", "visible");


                // Span form list (sidebar)
                this.own(this.spanFormList);
                this.rightSideBar = new ContentPane();
                this.rightSideBar.addChild(this.spanFormList);
                this.rightSideBar.addChild(CustomRanges.dateRange);
                contentWrapper.addSidebar(this.rightSideBar);

                // Grid
                contentWrapper.addChild(new HelpText({
                    help: i18n.help_grid
                }));
                gridPane.own(grid);
                gridPane.addChild(grid);

                this.own(gridPane);
                contentWrapper.addChild(gridPane);

                var self = this;

                // Grid-based button behavior
                this.own(
                    // Toggle disabled-state on buttons depending on selection in grid
                    on(grid, "dgrid-select", function(event) {
                        self._enableButtons(event.grid);
                    }),
                    // Toggle disabled-state on buttons depending on selection in grid
                    on(grid, "dgrid-deselect", function(event) {
                        self._enableButtons(event.grid);
                    }),
                    on(grid, "dgrid-refresh-complete", function(event) {
                        self._downloadRawButton.set("disabled", grid._total === 0);
                    }),

                    on(grid, "phrase-click", function(e) {
                        phraseDashboardDialog.set("showMostClicked", (model.get("view") === "mostFrequent"));
                        phraseDashboardDialog.set("query", e.phrase.query);
                        phraseDashboardDialog.show();
                    })
                );

                // Buttons
                contentWrapper.addChild(new HelpText({
                    help: i18n.help_buttons
                }));
                this.own(createSynonymsButton, createBestBetsButton, createRelatedQueriesButton, downloadRawButton);
                this.buttonWrapper = new ContentPane({
                    "class": "epi-pane__content__button-wrapper"
                });

                this.buttonWrapper.addChild(createBestBetsButton);
                this.buttonWrapper.addChild(createRelatedQueriesButton);
                if (AccessModel.isEnabled(config.intents.create.synonym)) {
                    this.buttonWrapper.addChild(createSynonymsButton);
                }
                this.buttonWrapper.addChild(downloadRawButton);
                contentWrapper.addChild(this.buttonWrapper);

                grid.own(
                    this.model.watch("query", function(name, oldValue, newValue) {
                        if (oldValue.to !== newValue.to ||
                            oldValue.from !== newValue.from ||
                            oldValue.query !== newValue.query ||
                            oldValue.type !== newValue.type) {
                            grid.set("query", newValue);
                        }
                    })
                );
            },

            takeAction: function() {
                this.refresh();
                return this;
            },

            _enableButtons: function(grid) {
                var count = 0;
                for (var key in grid.selection) {
                    count += grid.selection.hasOwnProperty(key) ? 1 : 0;
                }
                this._createSynonymsButton.set("disabled", count === 0);
                this._createBestBetsButton.set("disabled", count === 0);
                this._createRelatedQueriesButton.set("disabled", count !== 1);
            },

            refresh: function() {
                if (!this._started) {
                    return;
                }
                when(this.grid.refresh(), lang.hitch(this, function () {
                    this._updateStickers();
                }));
                this._enableButtons(this.grid);

                var ss = new StoreSeries(this.model.queryStore, {
                    query: this.model.query
                }, "count");
                this.chart.updateSeries("y", ss, true);
            },

            destroyRecursive: function() {
                // summary:
                //      This widget will destroy its children and other related non-widgets
                this.inherited(arguments);

                if (this.chart) {
                    this.chart.destroy();
                    delete this.chart;
                }
                if (this.grid) {
                    this.grid.destroy();
                    delete this.grid;
                }
            },

            resize: function() {
                this.inherited(arguments);
                if (this.chart && this.chartWrapper.isVisible()) {
                    this.chart.resize();
                }
                if (this.grid) {
                    this.grid.resize();
                }
                this._updateStickers();
            },

            startup: function() {
                this.inherited(arguments);
                this.grid.startup();
                var model = this.model,
                    chart = this.chart = this._createChart(this.chartPane.containerNode),
                    chartWrapper = this.chartWrapper,
                    grid = this.grid,
                    tooltip = new Tooltip(chart, "default"),
                    magnify = new Magnify(chart, "default"),
                    ss = new StoreSeries(model.queryStore, {
                        query: model.get("query")
                    }, "count");

                chart.addSeries("y", ss).render();
                this.own(model.watch("query", function(name, oldValue, newValue) {

                    if(newValue.type && newValue.type === Views.withoutRelevantHits.queryType) {
                        grid.toggleColumnHiddenState("number", true);
                        grid.toggleColumnHiddenState("total", false);
                        grid.toggleColumnHiddenState("percentage", false);
                        chartWrapper.hide();
                    } else {
                        grid.toggleColumnHiddenState("number", false);
                        grid.toggleColumnHiddenState("total", true);
                        grid.toggleColumnHiddenState("percentage", true);
                        chartWrapper.show();
                        chart.resize();
                    }

                    var ss = new StoreSeries(model.queryStore, {
                        query: newValue
                    }, "count");
                    if (oldValue.type !== newValue.type) {
                        chart.addSeries("y", ss,
                            newValue.type && newValue.type === Views.withoutHits.queryType ? {
                                stroke: {
                                    color: "#ef9e03"
                                },
                                fill: "#fdc860"
                            } :
                            null).render();
                    } else {
                        chart.updateSeries("y", ss, true).render();
                    }
                }));

                // Chart has static size and needs to get a #resize call when viewport resizes
                var resizeTimeOut = null;
                this.own(Viewport.on("resize", lang.hitch(this, function() {
                    if (this.hasChildren()) { // Only resize when there are
                        window.clearTimeout(resizeTimeOut);
                        resizeTimeOut = window.setTimeout(function() {
                            if (chartWrapper.isVisible()) {
                                chart.resize();
                            }
                        }, 10); // Use timeout to prevent mass-resizing
                    }
                })));

                this._stickers = [
                    new Sticker({ domNode: this.buttonWrapper.domNode, offset: {bottom: 0}, scrollNode: this.containerNode}),
                    new Sticker({ domNode: grid.headerNode, offset: {top: 0}, scrollNode: this.containerNode, limitNode: grid.domNode}),
                    new Sticker({ domNode: this.spanFormList.domNode, offset: {top: 0}, scrollNode: this.containerNode})
                ];

                array.forEach(this._stickers, function (sticker) {
                    this.own(sticker);
                }, this);
            },

            _updateStickers: function () {
                if (this._stickers) {
                    array.forEach(this._stickers, function (sticker) {
                        sticker.update(true);
                    });
                }
            },

            _createChart: function( /*DomNode*/ chartNode) {
                // summary:
                //      Creates a chart instance
                // description:
                //      Creates and returns the chart that will be used in dashboard view.
                // chartNode: DomNode|null
                //      The DomNode where to place the grid. Required!
                // tags:
                //      private
                var model = this.model,
                    chart = new Chart(chartNode);

                chart.setTheme(theme);
                chart.addPlot("default", {
                    type: "StackedAreas",
                    markers: true,
                    precision: 0
                }).addAxis("x", {
                    labelFunc: function(i) {
                        return DashboardRanges[model.get("span")].labelFormat(
                            stamp.fromISOString(chart.series[0].source.objects[number.parse(i) - 1].timestamp)
                        );
                    },
                    majorTicks: true,
                    minorTicks: false,
                    microTicks: false,
                    majorTickStep: 1
                }).addAxis("y", {
                    vertical: true,
                    fixLower: "major",
                    fixUpper: "major",
                    min: 0
                });

                return chart;
            }
        });
    });

},
'dojo/number':function(){
define("dojo/number", [/*===== "./_base/declare", =====*/ "./_base/lang", "./i18n", "./i18n!./cldr/nls/number", "./string", "./regexp"],
	function(/*===== declare, =====*/ lang, i18n, nlsNumber, dstring, dregexp){

// module:
//		dojo/number

var number = {
	// summary:
	//		localized formatting and parsing routines for Number
};
lang.setObject("dojo.number", number);

/*=====
number.__FormatOptions = declare(null, {
	// pattern: String?
	//		override [formatting pattern](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	//		with this string.  Default value is based on locale.  Overriding this property will defeat
	//		localization.  Literal characters in patterns are not supported.
	// type: String?
	//		choose a format type based on the locale from the following:
	//		decimal, scientific (not yet supported), percent, currency. decimal by default.
	// places: Number?
	//		fixed number of decimal places to show.  This overrides any
	//		information in the provided pattern.
	// round: Number?
	//		5 rounds to nearest .5; 0 rounds to nearest whole (default). -1
	//		means do not round.
	// locale: String?
	//		override the locale used to determine formatting rules
	// fractional: Boolean?
	//		If false, show no decimal places, overriding places and pattern settings.
});
=====*/

number.format = function(/*Number*/ value, /*number.__FormatOptions?*/ options){
	// summary:
	//		Format a Number as a String, using locale-specific settings
	// description:
	//		Create a string from a Number using a known localized pattern.
	//		Formatting patterns appropriate to the locale are chosen from the
	//		[Common Locale Data Repository](http://unicode.org/cldr) as well as the appropriate symbols and
	//		delimiters.
	//		If value is Infinity, -Infinity, or is not a valid JavaScript number, return null.
	// value:
	//		the number to be formatted

	options = lang.mixin({}, options || {});
	var locale = i18n.normalizeLocale(options.locale),
		bundle = i18n.getLocalization("dojo.cldr", "number", locale);
	options.customs = bundle;
	var pattern = options.pattern || bundle[(options.type || "decimal") + "Format"];
	if(isNaN(value) || Math.abs(value) == Infinity){ return null; } // null
	return number._applyPattern(value, pattern, options); // String
};

//number._numberPatternRE = /(?:[#0]*,?)*[#0](?:\.0*#*)?/; // not precise, but good enough
number._numberPatternRE = /[#0,]*[#0](?:\.0*#*)?/; // not precise, but good enough

number._applyPattern = function(/*Number*/ value, /*String*/ pattern, /*number.__FormatOptions?*/ options){
	// summary:
	//		Apply pattern to format value as a string using options. Gives no
	//		consideration to local customs.
	// value:
	//		the number to be formatted.
	// pattern:
	//		a pattern string as described by
	//		[unicode.org TR35](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	// options: number.__FormatOptions?
	//		_applyPattern is usually called via `dojo/number.format()` which
	//		populates an extra property in the options parameter, "customs".
	//		The customs object specifies group and decimal parameters if set.

	//TODO: support escapes
	options = options || {};
	var group = options.customs.group,
		decimal = options.customs.decimal,
		patternList = pattern.split(';'),
		positivePattern = patternList[0];
	pattern = patternList[(value < 0) ? 1 : 0] || ("-" + positivePattern);

	//TODO: only test against unescaped
	if(pattern.indexOf('%') != -1){
		value *= 100;
	}else if(pattern.indexOf('\u2030') != -1){
		value *= 1000; // per mille
	}else if(pattern.indexOf('\u00a4') != -1){
		group = options.customs.currencyGroup || group;//mixins instead?
		decimal = options.customs.currencyDecimal || decimal;// Should these be mixins instead?
		pattern = pattern.replace(/\u00a4{1,3}/, function(match){
			var prop = ["symbol", "currency", "displayName"][match.length-1];
			return options[prop] || options.currency || "";
		});
	}else if(pattern.indexOf('E') != -1){
		throw new Error("exponential notation not supported");
	}

	//TODO: support @ sig figs?
	var numberPatternRE = number._numberPatternRE;
	var numberPattern = positivePattern.match(numberPatternRE);
	if(!numberPattern){
		throw new Error("unable to find a number expression in pattern: "+pattern);
	}
	if(options.fractional === false){ options.places = 0; }
	return pattern.replace(numberPatternRE,
		number._formatAbsolute(value, numberPattern[0], {decimal: decimal, group: group, places: options.places, round: options.round}));
};

number.round = function(/*Number*/ value, /*Number?*/ places, /*Number?*/ increment){
	// summary:
	//		Rounds to the nearest value with the given number of decimal places, away from zero
	// description:
	//		Rounds to the nearest value with the given number of decimal places, away from zero if equal.
	//		Similar to Number.toFixed(), but compensates for browser quirks. Rounding can be done by
	//		fractional increments also, such as the nearest quarter.
	//		NOTE: Subject to floating point errors.  See dojox/math/round for experimental workaround.
	// value:
	//		The number to round
	// places:
	//		The number of decimal places where rounding takes place.  Defaults to 0 for whole rounding.
	//		Must be non-negative.
	// increment:
	//		Rounds next place to nearest value of increment/10.  10 by default.
	// example:
	// |	>>> number.round(-0.5)
	// |	-1
	// |	>>> number.round(162.295, 2)
	// |	162.29  // note floating point error.  Should be 162.3
	// |	>>> number.round(10.71, 0, 2.5)
	// |	10.75
	var factor = 10 / (increment || 10);
	return (factor * +value).toFixed(places) / factor; // Number
};

if((0.9).toFixed() == 0){
	// (isIE) toFixed() bug workaround: Rounding fails on IE when most significant digit
	// is just after the rounding place and is >=5
	var round = number.round;
	number.round = function(v, p, m){
		var d = Math.pow(10, -p || 0), a = Math.abs(v);
		if(!v || a >= d){
			d = 0;
		}else{
			a /= d;
			if(a < 0.5 || a >= 0.95){
				d = 0;
			}
		}
		return round(v, p, m) + (v > 0 ? d : -d);
	};

	// Use "doc hint" so the doc parser ignores this new definition of round(), and uses the one above.
	/*===== number.round = round; =====*/
}

/*=====
number.__FormatAbsoluteOptions = declare(null, {
	// decimal: String?
	//		the decimal separator
	// group: String?
	//		the group separator
	// places: Number|String?
	//		number of decimal places.  the range "n,m" will format to m places.
	// round: Number?
	//		5 rounds to nearest .5; 0 rounds to nearest whole (default). -1
	//		means don't round.
});
=====*/

number._formatAbsolute = function(/*Number*/ value, /*String*/ pattern, /*number.__FormatAbsoluteOptions?*/ options){
	// summary:
	//		Apply numeric pattern to absolute value using options. Gives no
	//		consideration to local customs.
	// value:
	//		the number to be formatted, ignores sign
	// pattern:
	//		the number portion of a pattern (e.g. `#,##0.00`)
	options = options || {};
	if(options.places === true){options.places=0;}
	if(options.places === Infinity){options.places=6;} // avoid a loop; pick a limit

	var patternParts = pattern.split("."),
		comma = typeof options.places == "string" && options.places.indexOf(","),
		maxPlaces = options.places;
	if(comma){
		maxPlaces = options.places.substring(comma + 1);
	}else if(!(maxPlaces >= 0)){
		maxPlaces = (patternParts[1] || []).length;
	}
	if(!(options.round < 0)){
		value = number.round(value, maxPlaces, options.round);
	}

	var valueParts = String(Math.abs(value)).split("."),
		fractional = valueParts[1] || "";
	if(patternParts[1] || options.places){
		if(comma){
			options.places = options.places.substring(0, comma);
		}
		// Pad fractional with trailing zeros
		var pad = options.places !== undefined ? options.places : (patternParts[1] && patternParts[1].lastIndexOf("0") + 1);
		if(pad > fractional.length){
			valueParts[1] = dstring.pad(fractional, pad, '0', true);
		}

		// Truncate fractional
		if(maxPlaces < fractional.length){
			valueParts[1] = fractional.substr(0, maxPlaces);
		}
	}else{
		if(valueParts[1]){ valueParts.pop(); }
	}

	// Pad whole with leading zeros
	var patternDigits = patternParts[0].replace(',', '');
	pad = patternDigits.indexOf("0");
	if(pad != -1){
		pad = patternDigits.length - pad;
		if(pad > valueParts[0].length){
			valueParts[0] = dstring.pad(valueParts[0], pad);
		}

		// Truncate whole
		if(patternDigits.indexOf("#") == -1){
			valueParts[0] = valueParts[0].substr(valueParts[0].length - pad);
		}
	}

	// Add group separators
	var index = patternParts[0].lastIndexOf(','),
		groupSize, groupSize2;
	if(index != -1){
		groupSize = patternParts[0].length - index - 1;
		var remainder = patternParts[0].substr(0, index);
		index = remainder.lastIndexOf(',');
		if(index != -1){
			groupSize2 = remainder.length - index - 1;
		}
	}
	var pieces = [];
	for(var whole = valueParts[0]; whole;){
		var off = whole.length - groupSize;
		pieces.push((off > 0) ? whole.substr(off) : whole);
		whole = (off > 0) ? whole.slice(0, off) : "";
		if(groupSize2){
			groupSize = groupSize2;
			delete groupSize2;
		}
	}
	valueParts[0] = pieces.reverse().join(options.group || ",");

	return valueParts.join(options.decimal || ".");
};

/*=====
number.__RegexpOptions = declare(null, {
	// pattern: String?
	//		override [formatting pattern](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	//		with this string.  Default value is based on locale.  Overriding this property will defeat
	//		localization.
	// type: String?
	//		choose a format type based on the locale from the following:
	//		decimal, scientific (not yet supported), percent, currency. decimal by default.
	// locale: String?
	//		override the locale used to determine formatting rules
	// strict: Boolean?
	//		strict parsing, false by default.  Strict parsing requires input as produced by the format() method.
	//		Non-strict is more permissive, e.g. flexible on white space, omitting thousands separators
	// places: Number|String?
	//		number of decimal places to accept: Infinity, a positive number, or
	//		a range "n,m".  Defined by pattern or Infinity if pattern not provided.
});
=====*/
number.regexp = function(/*number.__RegexpOptions?*/ options){
	// summary:
	//		Builds the regular needed to parse a number
	// description:
	//		Returns regular expression with positive and negative match, group
	//		and decimal separators
	return number._parseInfo(options).regexp; // String
};

number._parseInfo = function(/*Object?*/ options){
	options = options || {};
	var locale = i18n.normalizeLocale(options.locale),
		bundle = i18n.getLocalization("dojo.cldr", "number", locale),
		pattern = options.pattern || bundle[(options.type || "decimal") + "Format"],
//TODO: memoize?
		group = bundle.group,
		decimal = bundle.decimal,
		factor = 1;

	if(pattern.indexOf('%') != -1){
		factor /= 100;
	}else if(pattern.indexOf('\u2030') != -1){
		factor /= 1000; // per mille
	}else{
		var isCurrency = pattern.indexOf('\u00a4') != -1;
		if(isCurrency){
			group = bundle.currencyGroup || group;
			decimal = bundle.currencyDecimal || decimal;
		}
	}

	//TODO: handle quoted escapes
	var patternList = pattern.split(';');
	if(patternList.length == 1){
		patternList.push("-" + patternList[0]);
	}

	var re = dregexp.buildGroupRE(patternList, function(pattern){
		pattern = "(?:"+dregexp.escapeString(pattern, '.')+")";
		return pattern.replace(number._numberPatternRE, function(format){
			var flags = {
				signed: false,
				separator: options.strict ? group : [group,""],
				fractional: options.fractional,
				decimal: decimal,
				exponent: false
				},

				parts = format.split('.'),
				places = options.places;

			// special condition for percent (factor != 1)
			// allow decimal places even if not specified in pattern
			if(parts.length == 1 && factor != 1){
			    parts[1] = "###";
			}
			if(parts.length == 1 || places === 0){
				flags.fractional = false;
			}else{
				if(places === undefined){ places = options.pattern ? parts[1].lastIndexOf('0') + 1 : Infinity; }
				if(places && options.fractional == undefined){flags.fractional = true;} // required fractional, unless otherwise specified
				if(!options.places && (places < parts[1].length)){ places += "," + parts[1].length; }
				flags.places = places;
			}
			var groups = parts[0].split(',');
			if(groups.length > 1){
				flags.groupSize = groups.pop().length;
				if(groups.length > 1){
					flags.groupSize2 = groups.pop().length;
				}
			}
			return "("+number._realNumberRegexp(flags)+")";
		});
	}, true);

	if(isCurrency){
		// substitute the currency symbol for the placeholder in the pattern
		re = re.replace(/([\s\xa0]*)(\u00a4{1,3})([\s\xa0]*)/g, function(match, before, target, after){
			var prop = ["symbol", "currency", "displayName"][target.length-1],
				symbol = dregexp.escapeString(options[prop] || options.currency || "");
			before = before ? "[\\s\\xa0]" : "";
			after = after ? "[\\s\\xa0]" : "";
			if(!options.strict){
				if(before){before += "*";}
				if(after){after += "*";}
				return "(?:"+before+symbol+after+")?";
			}
			return before+symbol+after;
		});
	}

//TODO: substitute localized sign/percent/permille/etc.?

	// normalize whitespace and return
	return {regexp: re.replace(/[\xa0 ]/g, "[\\s\\xa0]"), group: group, decimal: decimal, factor: factor}; // Object
};

/*=====
number.__ParseOptions = declare(null, {
	// pattern: String?
	//		override [formatting pattern](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	//		with this string.  Default value is based on locale.  Overriding this property will defeat
	//		localization.  Literal characters in patterns are not supported.
	// type: String?
	//		choose a format type based on the locale from the following:
	//		decimal, scientific (not yet supported), percent, currency. decimal by default.
	// locale: String?
	//		override the locale used to determine formatting rules
	// strict: Boolean?
	//		strict parsing, false by default.  Strict parsing requires input as produced by the format() method.
	//		Non-strict is more permissive, e.g. flexible on white space, omitting thousands separators
	// fractional: Boolean|Array?
	//		Whether to include the fractional portion, where the number of decimal places are implied by pattern
	//		or explicit 'places' parameter.  The value [true,false] makes the fractional portion optional.
});
=====*/
number.parse = function(/*String*/ expression, /*number.__ParseOptions?*/ options){
	// summary:
	//		Convert a properly formatted string to a primitive Number, using
	//		locale-specific settings.
	// description:
	//		Create a Number from a string using a known localized pattern.
	//		Formatting patterns are chosen appropriate to the locale
	//		and follow the syntax described by
	//		[unicode.org TR35](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
    	//		Note that literal characters in patterns are not supported.
	// expression:
	//		A string representation of a Number
	var info = number._parseInfo(options),
		results = (new RegExp("^"+info.regexp+"$")).exec(expression);
	if(!results){
		return NaN; //NaN
	}
	var absoluteMatch = results[1]; // match for the positive expression
	if(!results[1]){
		if(!results[2]){
			return NaN; //NaN
		}
		// matched the negative pattern
		absoluteMatch =results[2];
		info.factor *= -1;
	}

	// Transform it to something Javascript can parse as a number.  Normalize
	// decimal point and strip out group separators or alternate forms of whitespace
	absoluteMatch = absoluteMatch.
		replace(new RegExp("["+info.group + "\\s\\xa0"+"]", "g"), "").
		replace(info.decimal, ".");
	// Adjust for negative sign, percent, etc. as necessary
	return absoluteMatch * info.factor; //Number
};

/*=====
number.__RealNumberRegexpFlags = declare(null, {
	// places: Number?
	//		The integer number of decimal places or a range given as "n,m".  If
	//		not given, the decimal part is optional and the number of places is
	//		unlimited.
	// decimal: String?
	//		A string for the character used as the decimal point.  Default
	//		is ".".
	// fractional: Boolean|Array?
	//		Whether decimal places are used.  Can be true, false, or [true,
	//		false].  Default is [true, false] which means optional.
	// exponent: Boolean|Array?
	//		Express in exponential notation.  Can be true, false, or [true,
	//		false]. Default is [true, false], (i.e. will match if the
	//		exponential part is present are not).
	// eSigned: Boolean|Array?
	//		The leading plus-or-minus sign on the exponent.  Can be true,
	//		false, or [true, false].  Default is [true, false], (i.e. will
	//		match if it is signed or unsigned).  flags in regexp.integer can be
	//		applied.
});
=====*/

number._realNumberRegexp = function(/*__RealNumberRegexpFlags?*/ flags){
	// summary:
	//		Builds a regular expression to match a real number in exponential
	//		notation

	// assign default values to missing parameters
	flags = flags || {};
	//TODO: use mixin instead?
	if(!("places" in flags)){ flags.places = Infinity; }
	if(typeof flags.decimal != "string"){ flags.decimal = "."; }
	if(!("fractional" in flags) || /^0/.test(flags.places)){ flags.fractional = [true, false]; }
	if(!("exponent" in flags)){ flags.exponent = [true, false]; }
	if(!("eSigned" in flags)){ flags.eSigned = [true, false]; }

	var integerRE = number._integerRegexp(flags),
		decimalRE = dregexp.buildGroupRE(flags.fractional,
		function(q){
			var re = "";
			if(q && (flags.places!==0)){
				re = "\\" + flags.decimal;
				if(flags.places == Infinity){
					re = "(?:" + re + "\\d+)?";
				}else{
					re += "\\d{" + flags.places + "}";
				}
			}
			return re;
		},
		true
	);

	var exponentRE = dregexp.buildGroupRE(flags.exponent,
		function(q){
			if(q){ return "([eE]" + number._integerRegexp({ signed: flags.eSigned}) + ")"; }
			return "";
		}
	);

	var realRE = integerRE + decimalRE;
	// allow for decimals without integers, e.g. .25
	if(decimalRE){realRE = "(?:(?:"+ realRE + ")|(?:" + decimalRE + "))";}
	return realRE + exponentRE; // String
};

/*=====
number.__IntegerRegexpFlags = declare(null, {
	// signed: Boolean?
	//		The leading plus-or-minus sign. Can be true, false, or `[true,false]`.
	//		Default is `[true, false]`, (i.e. will match if it is signed
	//		or unsigned).
	// separator: String?
	//		The character used as the thousands separator. Default is no
	//		separator. For more than one symbol use an array, e.g. `[",", ""]`,
	//		makes ',' optional.
	// groupSize: Number?
	//		group size between separators
	// groupSize2: Number?
	//		second grouping, where separators 2..n have a different interval than the first separator (for India)
});
=====*/

number._integerRegexp = function(/*number.__IntegerRegexpFlags?*/ flags){
	// summary:
	//		Builds a regular expression that matches an integer

	// assign default values to missing parameters
	flags = flags || {};
	if(!("signed" in flags)){ flags.signed = [true, false]; }
	if(!("separator" in flags)){
		flags.separator = "";
	}else if(!("groupSize" in flags)){
		flags.groupSize = 3;
	}

	var signRE = dregexp.buildGroupRE(flags.signed,
		function(q){ return q ? "[-+]" : ""; },
		true
	);

	var numberRE = dregexp.buildGroupRE(flags.separator,
		function(sep){
			if(!sep){
				return "(?:\\d+)";
			}

			sep = dregexp.escapeString(sep);
			if(sep == " "){ sep = "\\s"; }
			else if(sep == "\xa0"){ sep = "\\s\\xa0"; }

			var grp = flags.groupSize, grp2 = flags.groupSize2;
			//TODO: should we continue to enforce that numbers with separators begin with 1-9?  See #6933
			if(grp2){
				var grp2RE = "(?:0|[1-9]\\d{0," + (grp2-1) + "}(?:[" + sep + "]\\d{" + grp2 + "})*[" + sep + "]\\d{" + grp + "})";
				return ((grp-grp2) > 0) ? "(?:" + grp2RE + "|(?:0|[1-9]\\d{0," + (grp-1) + "}))" : grp2RE;
			}
			return "(?:0|[1-9]\\d{0," + (grp-1) + "}(?:[" + sep + "]\\d{" + grp + "})*)";
		},
		true
	);

	return signRE + numberRE; // String
};

return number;
});

},
'dojox/charting/Chart':function(){
define("dojox/charting/Chart", ["../main", "dojo/_base/lang", "dojo/_base/array","dojo/_base/declare", "dojo/dom-style",
	"dojo/dom", "dojo/dom-geometry", "dojo/dom-construct","dojo/_base/Color", "dojo/_base/sniff",
	"./Element", "./SimpleTheme", "./Series", "./axis2d/common", "dojox/gfx/shape",
	"dojox/gfx", "dojox/lang/functional", "dojox/lang/functional/fold", "dojox/lang/functional/reversed"], 
	function(dojox, lang, arr, declare, domStyle,
	 		 dom, domGeom, domConstruct, Color, has,
	 		 Element, SimpleTheme, Series, common, shape,
	 		 g, func, funcFold, funcReversed){
	/*=====
	var __ChartCtorArgs = {
		// summary:
		//		The keyword arguments that can be passed in a Chart constructor.
		// margins: Object?
		//		Optional margins for the chart, in the form of { l, t, r, b}.
		// stroke: dojox.gfx.Stroke?
		//		An optional outline/stroke for the chart.
		// fill: dojox.gfx.Fill?
		//		An optional fill for the chart.
		// delayInMs: Number
		//		Delay in ms for delayedRender(). Default: 200.
	};
	=====*/

	/*=====
	var __SeriesCtorArgs = {
		// summary:
		//		An optional arguments object that can be used in the Series constructor.
		// plot: String?
		//		The plot (by name) that this series belongs to.
	};
	=====*/

	/*=====
	var __BaseAxisCtorArgs = {
		// summary:
		//		Optional arguments used in the definition of an invisible axis.
		// vertical: Boolean?
		//		A flag that says whether an axis is vertical (i.e. y axis) or horizontal. Default is false (horizontal).
		// min: Number?
		//		The smallest value on an axis. Default is 0.
		// max: Number?
		//		The largest value on an axis. Default is 1.
	};
	=====*/

	var dc = lang.getObject("charting", true, dojox),
		clear = func.lambda("item.clear()"),
		purge = func.lambda("item.purgeGroup()"),
		destroy = func.lambda("item.destroy()"),
		makeClean = func.lambda("item.dirty = false"),
		makeDirty = func.lambda("item.dirty = true"),
		getName = func.lambda("item.name");

	var Chart = declare("dojox.charting.Chart", null, {
		// summary:
		//		The main chart object in dojox.charting.  This will create a two dimensional
		//		chart based on dojox.gfx.
		//
		// description:
		//		dojox.charting.Chart is the primary object used for any kind of charts.  It
		//		is simple to create--just pass it a node reference, which is used as the
		//		container for the chart--and a set of optional keyword arguments and go.
		//
		//		Note that like most of dojox.gfx, most of dojox.charting.Chart's methods are
		//		designed to return a reference to the chart itself, to allow for functional
		//		chaining.  This makes defining everything on a Chart very easy to do.
		//
		// example:
		//		Create an area chart, with smoothing.
		//	|	require(["dojox/charting/Chart", "dojox/charting/themes/Shrooms", "dojox/charting/plot2d/Areas", ...],
		// 	|		function(Chart, Shrooms, Areas, ...){
		//	|		new Chart(node)
		//	|			.addPlot("default", { type: Areas, tension: "X" })
		//	|			.setTheme(Shrooms)
		//	|			.addSeries("Series A", [1, 2, 0.5, 1.5, 1, 2.8, 0.4])
		//	|			.addSeries("Series B", [2.6, 1.8, 2, 1, 1.4, 0.7, 2])
		//	|			.addSeries("Series C", [6.3, 1.8, 3, 0.5, 4.4, 2.7, 2])
		//	|			.render();
		//	|	});
		//
		// example:
		//		The form of data in a data series can take a number of forms: a simple array,
		//		an array of objects {x,y}, or something custom (as determined by the plot).
		//		Here's an example of a Candlestick chart, which expects an object of
		//		{ open, high, low, close }.
		//	|	require(["dojox/charting/Chart", "dojox/charting/plot2d/Candlesticks", ...],
		// 	|		function(Chart, Candlesticks, ...){
		//	|		new Chart(node)
		//	|			.addPlot("default", {type: Candlesticks, gap: 1})
		//	|			.addAxis("x", {fixLower: "major", fixUpper: "major", includeZero: true})
		//	|			.addAxis("y", {vertical: true, fixLower: "major", fixUpper: "major", natural: true})
		//	|			.addSeries("Series A", [
		//	|					{ open: 20, close: 16, high: 22, low: 8 },
		//	|					{ open: 16, close: 22, high: 26, low: 6, mid: 18 },
		//	|					{ open: 22, close: 18, high: 22, low: 11, mid: 21 },
		//	|					{ open: 18, close: 29, high: 32, low: 14, mid: 27 },
		//	|					{ open: 29, close: 24, high: 29, low: 13, mid: 27 },
		//	|					{ open: 24, close: 8, high: 24, low: 5 },
		//	|					{ open: 8, close: 16, high: 22, low: 2 },
		//	|					{ open: 16, close: 12, high: 19, low: 7 },
		//	|					{ open: 12, close: 20, high: 22, low: 8 },
		//	|					{ open: 20, close: 16, high: 22, low: 8 },
		//	|					{ open: 16, close: 22, high: 26, low: 6, mid: 18 },
		//	|					{ open: 22, close: 18, high: 22, low: 11, mid: 21 },
		//	|					{ open: 18, close: 29, high: 32, low: 14, mid: 27 },
		//	|					{ open: 29, close: 24, high: 29, low: 13, mid: 27 },
		//	|					{ open: 24, close: 8, high: 24, low: 5 },
		//	|					{ open: 8, close: 16, high: 22, low: 2 },
		//	|					{ open: 16, close: 12, high: 19, low: 7 },
		//	|					{ open: 12, close: 20, high: 22, low: 8 },
		//	|					{ open: 20, close: 16, high: 22, low: 8 },
		//	|					{ open: 16, close: 22, high: 26, low: 6 },
		//	|					{ open: 22, close: 18, high: 22, low: 11 },
		//	|					{ open: 18, close: 29, high: 32, low: 14 },
		//	|					{ open: 29, close: 24, high: 29, low: 13 },
		//	|					{ open: 24, close: 8, high: 24, low: 5 },
		//	|					{ open: 8, close: 16, high: 22, low: 2 },
		//	|					{ open: 16, close: 12, high: 19, low: 7 },
		//	|					{ open: 12, close: 20, high: 22, low: 8 },
		//	|					{ open: 20, close: 16, high: 22, low: 8 }
		//	|				],
		//	|				{ stroke: { color: "green" }, fill: "lightgreen" }
		//	|			)
		//	|			.render();
		//	|	});
		
		// theme: dojox/charting/SimpleTheme?
		//		An optional theme to use for styling the chart.
		// axes: dojox/charting/axis2d/Base{}?
		//		A map of axes for use in plotting a chart.
		// stack: dojox/charting/plot2d/Base[]
		//		A stack of plotters.
		// plots: dojox/charting/plot2d/Base{}
		//		A map of plotter indices
		// series: dojox/charting/Series[]
		//		The stack of data runs used to create plots.
		// runs: dojox/charting/Series{}
		//		A map of series indices
		// margins: Object?
		//		The margins around the chart. Default is { l:10, t:10, r:10, b:10 }.
		// stroke: dojox.gfx.Stroke?
		//		The outline of the chart (stroke in vector graphics terms).
		// fill: dojox.gfx.Fill?
		//		The color for the chart.
		// node: DOMNode
		//		The container node passed to the constructor.
		// surface: dojox/gfx/shape.Surface
		//		The main graphics surface upon which a chart is drawn.
		// dirty: Boolean
		//		A boolean flag indicating whether or not the chart needs to be updated/re-rendered.

		constructor: function(/* DOMNode */node, /* __ChartCtorArgs? */kwArgs){
			// summary:
			//		The constructor for a new Chart.  Initializes all parameters used for a chart.
			// returns: dojox/charting/Chart
			//		The newly created chart.

			// initialize parameters
			if(!kwArgs){ kwArgs = {}; }
			this.margins   = kwArgs.margins ? kwArgs.margins : {l: 10, t: 10, r: 10, b: 10};
			this.stroke    = kwArgs.stroke;
			this.fill      = kwArgs.fill;
			this.delayInMs = kwArgs.delayInMs || 200;
			this.title     = kwArgs.title;
			this.titleGap  = kwArgs.titleGap;
			this.titlePos  = kwArgs.titlePos;
			this.titleFont = kwArgs.titleFont;
			this.titleFontColor = kwArgs.titleFontColor;
			this.chartTitle = null;

			// default initialization
			this.theme = null;
			this.axes = {};		// map of axes
			this.stack = [];	// stack of plotters
			this.plots = {};	// map of plotter indices
			this.series = [];	// stack of data runs
			this.runs = {};		// map of data run indices
			this.dirty = true;

			// create a surface
			this.node = dom.byId(node);
			var box = domGeom.getMarginBox(node);
			this.surface = g.createSurface(this.node, box.w || 400, box.h || 300);
			if(this.surface.declaredClass.indexOf("vml") == -1){
				// except if vml use native clipping
				this.plotGroup = this.surface.createGroup();
			}
		},
		destroy: function(){
			// summary:
			//		Cleanup when a chart is to be destroyed.
			// returns: void
			arr.forEach(this.series, destroy);
			arr.forEach(this.stack,  destroy);
			func.forIn(this.axes, destroy);
			this.surface.destroy();
			if(this.chartTitle && this.chartTitle.tagName){
				// destroy title if it is a DOM node
				domConstruct.destroy(this.chartTitle);
			}
		},
		getCoords: function(){
			// summary:
			//		Get the coordinates and dimensions of the containing DOMNode, as
			//		returned by dojo.coords.
			// returns: Object
			//		The resulting coordinates of the chart.  See dojo.coords for details.
			var node = this.node;
			var s = domStyle.getComputedStyle(node), coords = domGeom.getMarginBox(node, s);
			var abs = domGeom.position(node, true);
			coords.x = abs.x;
			coords.y = abs.y;
			return coords;	//	Object
		},
		setTheme: function(theme){
			// summary:
			//		Set a theme of the chart.
			// theme: dojox/charting/SimpleTheme
			//		The theme to be used for visual rendering.
			// returns: dojox/charting/Chart
			//		A reference to the current chart for functional chaining.
			this.theme = theme.clone();
			this.dirty = true;
			return this;	//	dojox/charting/Chart
		},
		addAxis: function(name, kwArgs){
			// summary:
			//		Add an axis to the chart, for rendering.
			// name: String
			//		The name of the axis.
			// kwArgs: __BaseAxisCtorArgs?
			//		An optional keyword arguments object for use in defining details of an axis.
			// returns: dojox/charting/Chart
			//		A reference to the current chart for functional chaining.
			var axis, axisType = kwArgs && kwArgs.type || "Default";
			if(typeof axisType == "string"){
				if(!dc.axis2d || !dc.axis2d[axisType]){
					throw Error("Can't find axis: " + axisType + " - Check " + "require() dependencies.");
				}
				axis = new dc.axis2d[axisType](this, kwArgs);
			}else{
				axis = new axisType(this, kwArgs);
			}
			axis.name = name;
			axis.dirty = true;
			if(name in this.axes){
				this.axes[name].destroy();
			}
			this.axes[name] = axis;
			this.dirty = true;
			return this;	//	dojox/charting/Chart
		},
		getAxis: function(name){
			// summary:
			//		Get the given axis, by name.
			// name: String
			//		The name the axis was defined by.
			// returns: dojox/charting/axis2d/Default
			//		The axis as stored in the chart's axis map.
			return this.axes[name];	//	dojox/charting/axis2d/Default
		},
		removeAxis: function(name){
			// summary:
			//		Remove the axis that was defined using name.
			// name: String
			//		The axis name, as defined in addAxis.
			// returns: dojox/charting/Chart
			//		A reference to the current chart for functional chaining.
			if(name in this.axes){
				// destroy the axis
				this.axes[name].destroy();
				delete this.axes[name];
				// mark the chart as dirty
				this.dirty = true;
			}
			return this;	//	dojox/charting/Chart
		},
		addPlot: function(name, kwArgs){
			// summary:
			//		Add a new plot to the chart, defined by name and using the optional keyword arguments object.
			//		Note that dojox.charting assumes the main plot to be called "default"; if you do not have
			//		a plot called "default" and attempt to add data series to the chart without specifying the
			//		plot to be rendered on, you WILL get errors.
			// name: String
			//		The name of the plot to be added to the chart.  If you only plan on using one plot, call it "default".
			// kwArgs: dojox.charting.plot2d.__PlotCtorArgs
			//		An object with optional parameters for the plot in question.
			// returns: dojox/charting/Chart
			//		A reference to the current chart for functional chaining.
			var plot, plotType = kwArgs && kwArgs.type || "Default";
			if(typeof plotType == "string"){
				if(!dc.plot2d || !dc.plot2d[plotType]){
					throw Error("Can't find plot: " + plotType + " - didn't you forget to dojo" + ".require() it?");
				}
				plot = new dc.plot2d[plotType](this, kwArgs);
			}else{
				plot = new plotType(this, kwArgs);
			}
			plot.name = name;
			plot.dirty = true;
			if(name in this.plots){
				this.stack[this.plots[name]].destroy();
				this.stack[this.plots[name]] = plot;
			}else{
				this.plots[name] = this.stack.length;
				this.stack.push(plot);
			}
			this.dirty = true;
			return this;	//	dojox/charting/Chart
		},
		getPlot: function(name){
			// summary:
			//		Get the given plot, by name.
			// name: String
			//		The name the plot was defined by.
			// returns: dojox/charting/plot2d/Base
			//		The plot.
			return this.stack[this.plots[name]];
		},
		removePlot: function(name){
			// summary:
			//		Remove the plot defined using name from the chart's plot stack.
			// name: String
			//		The name of the plot as defined using addPlot.
			// returns: dojox/charting/Chart
			//		A reference to the current chart for functional chaining.
			if(name in this.plots){
				// get the index and remove the name
				var index = this.plots[name];
				delete this.plots[name];
				// destroy the plot
				this.stack[index].destroy();
				// remove the plot from the stack
				this.stack.splice(index, 1);
				// update indices to reflect the shift
				func.forIn(this.plots, function(idx, name, plots){
					if(idx > index){
						plots[name] = idx - 1;
					}
				});
				// remove all related series
				var ns = arr.filter(this.series, function(run){ return run.plot != name; });
				if(ns.length < this.series.length){
					// kill all removed series
					arr.forEach(this.series, function(run){
						if(run.plot == name){
							run.destroy();
						}
					});
					// rebuild all necessary data structures
					this.runs = {};
					arr.forEach(ns, function(run, index){
						this.runs[run.plot] = index;
					}, this);
					this.series = ns;
				}
				// mark the chart as dirty
				this.dirty = true;
			}
			return this;	//	dojox/charting/Chart
		},
		getPlotOrder: function(){
			// summary:
			//		Returns an array of plot names in the current order
			//		(the top-most plot is the first).
			// returns: Array
			return func.map(this.stack, getName); // Array
		},
		setPlotOrder: function(newOrder){
			// summary:
			//		Sets new order of plots. newOrder cannot add or remove
			//		plots. Wrong names, or dups are ignored.
			// newOrder: Array
			//		Array of plot names compatible with getPlotOrder().
			// returns: dojox/charting/Chart
			//		A reference to the current chart for functional chaining.
			var names = {},
				order = func.filter(newOrder, function(name){
					if(!(name in this.plots) || (name in names)){
						return false;
					}
					names[name] = 1;
					return true;
				}, this);
			if(order.length < this.stack.length){
				func.forEach(this.stack, function(plot){
					var name = plot.name;
					if(!(name in names)){
						order.push(name);
					}
				});
			}
			var newStack = func.map(order, function(name){
					return this.stack[this.plots[name]];
				}, this);
			func.forEach(newStack, function(plot, i){
				this.plots[plot.name] = i;
			}, this);
			this.stack = newStack;
			this.dirty = true;
			return this;	//	dojox/charting/Chart
		},
		movePlotToFront: function(name){
			// summary:
			//		Moves a given plot to front.
			// name: String
			//		Plot's name to move.
			// returns: dojox/charting/Chart
			//		A reference to the current chart for functional chaining.
			if(name in this.plots){
				var index = this.plots[name];
				if(index){
					var newOrder = this.getPlotOrder();
					newOrder.splice(index, 1);
					newOrder.unshift(name);
					return this.setPlotOrder(newOrder);	//	dojox/charting/Chart
				}
			}
			return this;	//	dojox/charting/Chart
		},
		movePlotToBack: function(name){
			// summary:
			//		Moves a given plot to back.
			// name: String
			//		Plot's name to move.
			// returns: dojox/charting/Chart
			//		A reference to the current chart for functional chaining.
			if(name in this.plots){
				var index = this.plots[name];
				if(index < this.stack.length - 1){
					var newOrder = this.getPlotOrder();
					newOrder.splice(index, 1);
					newOrder.push(name);
					return this.setPlotOrder(newOrder);	//	dojox/charting/Chart
				}
			}
			return this;	//	dojox/charting/Chart
		},
		addSeries: function(name, data, kwArgs){
			// summary:
			//		Add a data series to the chart for rendering.
			// name: String
			//		The name of the data series to be plotted.
			// data: Array|Object
			//		The array of data points (either numbers or objects) that
			//		represents the data to be drawn. Or it can be an object. In
			//		the latter case, it should have a property "data" (an array),
			//		destroy(), and setSeriesObject().
			// kwArgs: __SeriesCtorArgs?
			//		An optional keyword arguments object that will be mixed into
			//		the resultant series object.
			// returns: dojox/charting/Chart
			//		A reference to the current chart for functional chaining.
			var run = new Series(this, data, kwArgs);
			run.name = name;
			if(name in this.runs){
				this.series[this.runs[name]].destroy();
				this.series[this.runs[name]] = run;
			}else{
				this.runs[name] = this.series.length;
				this.series.push(run);
			}
			this.dirty = true;
			// fix min/max
			if(!("ymin" in run) && "min" in run){ run.ymin = run.min; }
			if(!("ymax" in run) && "max" in run){ run.ymax = run.max; }
			return this;	//	dojox/charting/Chart
		},
		getSeries: function(name){
			// summary:
			//		Get the given series, by name.
			// name: String
			//		The name the series was defined by.
			// returns: dojox/charting/Series
			//		The series.
			return this.series[this.runs[name]];
		},
		removeSeries: function(name){
			// summary:
			//		Remove the series defined by name from the chart.
			// name: String
			//		The name of the series as defined by addSeries.
			// returns: dojox/charting/Chart
			//		A reference to the current chart for functional chaining.
			if(name in this.runs){
				// get the index and remove the name
				var index = this.runs[name];
				delete this.runs[name];
				// destroy the run
				this.series[index].destroy();
				// remove the run from the stack of series
				this.series.splice(index, 1);
				// update indices to reflect the shift
				func.forIn(this.runs, function(idx, name, runs){
					if(idx > index){
						runs[name] = idx - 1;
					}
				});
				this.dirty = true;
			}
			return this;	//	dojox/charting/Chart
		},
		updateSeries: function(name, data, offsets){
			// summary:
			//		Update the given series with a new set of data points.
			// name: String
			//		The name of the series as defined in addSeries.
			// data: Array|Object
			//		The array of data points (either numbers or objects) that
			//		represents the data to be drawn. Or it can be an object. In
			//		the latter case, it should have a property "data" (an array),
			//		destroy(), and setSeriesObject().
			// offsets: Boolean?
			//		If true recomputes the offsets of the chart based on the new
			//		data. This is useful if the range of data is drastically changing
			//		and offsets need to be recomputed.
			// returns: dojox/charting/Chart
			//		A reference to the current chart for functional chaining.
			if(name in this.runs){
				var run = this.series[this.runs[name]];
				run.update(data);
				if(offsets){
					this.dirty = true;
				}else{
					this._invalidateDependentPlots(run.plot, false);
					this._invalidateDependentPlots(run.plot, true);
				}
			}
			return this;	//	dojox/charting/Chart
		},
		getSeriesOrder: function(plotName){
			// summary:
			//		Returns an array of series names in the current order
			//		(the top-most series is the first) within a plot.
			// plotName: String
			//		Plot's name.
			// returns: Array
			return func.map(func.filter(this.series, function(run){
					return run.plot == plotName;
				}), getName);
		},
		setSeriesOrder: function(newOrder){
			// summary:
			//		Sets new order of series within a plot. newOrder cannot add
			//		or remove series. Wrong names, or dups are ignored.
			// newOrder: Array
			//		Array of series names compatible with getPlotOrder(). All
			//		series should belong to the same plot.
			// returns: dojox/charting/Chart
			//		A reference to the current chart for functional chaining.
			var plotName, names = {},
				order = func.filter(newOrder, function(name){
					if(!(name in this.runs) || (name in names)){
						return false;
					}
					var run = this.series[this.runs[name]];
					if(plotName){
						if(run.plot != plotName){
							return false;
						}
					}else{
						plotName = run.plot;
					}
					names[name] = 1;
					return true;
				}, this);
			func.forEach(this.series, function(run){
				var name = run.name;
				if(!(name in names) && run.plot == plotName){
					order.push(name);
				}
			});
			var newSeries = func.map(order, function(name){
					return this.series[this.runs[name]];
				}, this);
			this.series = newSeries.concat(func.filter(this.series, function(run){
				return run.plot != plotName;
			}));
			func.forEach(this.series, function(run, i){
				this.runs[run.name] = i;
			}, this);
			this.dirty = true;
			return this;	//	dojox/charting/Chart
		},
		moveSeriesToFront: function(name){
			// summary:
			//		Moves a given series to front of a plot.
			// name: String
			//		Series' name to move.
			// returns: dojox/charting/Chart
			//		A reference to the current chart for functional chaining.
			if(name in this.runs){
				var index = this.runs[name],
					newOrder = this.getSeriesOrder(this.series[index].plot);
				if(name != newOrder[0]){
					newOrder.splice(index, 1);
					newOrder.unshift(name);
					return this.setSeriesOrder(newOrder);	//	dojox/charting/Chart
				}
			}
			return this;	//	dojox/charting/Chart
		},
		moveSeriesToBack: function(name){
			// summary:
			//		Moves a given series to back of a plot.
			// name: String
			//		Series' name to move.
			// returns: dojox/charting/Chart
			//		A reference to the current chart for functional chaining.
			if(name in this.runs){
				var index = this.runs[name],
					newOrder = this.getSeriesOrder(this.series[index].plot);
				if(name != newOrder[newOrder.length - 1]){
					newOrder.splice(index, 1);
					newOrder.push(name);
					return this.setSeriesOrder(newOrder);	//	dojox/charting/Chart
				}
			}
			return this;	//	dojox/charting/Chart
		},
		resize: function(width, height){
			// summary:
			//		Resize the chart to the dimensions of width and height.
			// description:
			//		Resize the chart and its surface to the width and height dimensions.
			//		If no width/height or box is provided, resize the surface to the marginBox of the chart.
			// width: Number
			//		The new width of the chart.
			// height: Number
			//		The new height of the chart.
			// returns: dojox/charting/Chart
			//		A reference to the current chart for functional chaining.
			var box;
			switch(arguments.length){
				// case 0, do not resize the div, just the surface
				case 1:
					// argument, override node box
					box = lang.mixin({}, width);
					domGeom.setMarginBox(this.node, box);
					break;
				case 2:
					box = {w: width, h: height};
					// argument, override node box
					domGeom.setMarginBox(this.node, box);
					break;
			}
			// in all cases take back the computed box
			box = domGeom.getMarginBox(this.node);
			var d = this.surface.getDimensions();
			if(d.width != box.w || d.height != box.h){
				// and set it on the surface
				this.surface.setDimensions(box.w, box.h);
				this.dirty = true;
				return this.render();	//	dojox/charting/Chart
			}else{
				return this;
			}
		},
		getGeometry: function(){
			// summary:
			//		Returns a map of information about all axes in a chart and what they represent
			//		in terms of scaling (see dojox.charting.axis2d.Default.getScaler).
			// returns: Object
			//		An map of geometry objects, a one-to-one mapping of axes.
			var ret = {};
			func.forIn(this.axes, function(axis){
				if(axis.initialized()){
					ret[axis.name] = {
						name:		axis.name,
						vertical:	axis.vertical,
						scaler:		axis.scaler,
						ticks:		axis.ticks
					};
				}
			});
			return ret;	//	Object
		},
		setAxisWindow: function(name, scale, offset, zoom){
			// summary:
			//		Zooms an axis and all dependent plots. Can be used to zoom in 1D.
			// name: String
			//		The name of the axis as defined by addAxis.
			// scale: Number
			//		The scale on the target axis.
			// offset: Number
			//		Any offest, as measured by axis tick
			// zoom: Boolean|Object?
			//		The chart zooming animation trigger.  This is null by default,
			//		e.g. {duration: 1200}, or just set true.
			// returns: dojox/charting/Chart
			//		A reference to the current chart for functional chaining.
			var axis = this.axes[name];
			if(axis){
				axis.setWindow(scale, offset);
				arr.forEach(this.stack,function(plot){
					if(plot.hAxis == name || plot.vAxis == name){
						plot.zoom = zoom;
					}
				});
			}
			return this;	//	dojox/charting/Chart
		},
		setWindow: function(sx, sy, dx, dy, zoom){
			// summary:
			//		Zooms in or out any plots in two dimensions.
			// sx: Number
			//		The scale for the x axis.
			// sy: Number
			//		The scale for the y axis.
			// dx: Number
			//		The pixel offset on the x axis.
			// dy: Number
			//		The pixel offset on the y axis.
			// zoom: Boolean|Object?
			//		The chart zooming animation trigger.  This is null by default,
			//		e.g. {duration: 1200}, or just set true.
			// returns: dojox/charting/Chart
			//		A reference to the current chart for functional chaining.
			if(!("plotArea" in this)){
				this.calculateGeometry();
			}
			func.forIn(this.axes, function(axis){
				var scale, offset, bounds = axis.getScaler().bounds,
					s = bounds.span / (bounds.upper - bounds.lower);
				if(axis.vertical){
					scale  = sy;
					offset = dy / s / scale;
				}else{
					scale  = sx;
					offset = dx / s / scale;
				}
				axis.setWindow(scale, offset);
			});
			arr.forEach(this.stack, function(plot){ plot.zoom = zoom; });
			return this;	//	dojox/charting/Chart
		},
		zoomIn:	function(name, range, delayed){
			// summary:
			//		Zoom the chart to a specific range on one axis.  This calls render()
			//		directly as a convenience method.
			// name: String
			//		The name of the axis as defined by addAxis.
			// range: Array
			//		The end points of the zoom range, measured in axis ticks.
			var axis = this.axes[name];
			if(axis){
				var scale, offset, bounds = axis.getScaler().bounds;
				var lower = Math.min(range[0],range[1]);
				var upper = Math.max(range[0],range[1]);
				lower = range[0] < bounds.lower ? bounds.lower : lower;
				upper = range[1] > bounds.upper ? bounds.upper : upper;
				scale = (bounds.upper - bounds.lower) / (upper - lower);
				offset = lower - bounds.lower;
				this.setAxisWindow(name, scale, offset);
				if(delayed){
					this.delayedRender();
				}else{
					this.render();
				}
			}
		},
		calculateGeometry: function(){
			// summary:
			//		Calculate the geometry of the chart based on the defined axes of
			//		a chart.
			// returns: dojox/charting/Chart
			//		A reference to the current chart for functional chaining.
			if(this.dirty){
				return this.fullGeometry();
			}

			// calculate geometry
			var dirty = arr.filter(this.stack, function(plot){
					return plot.dirty ||
						(plot.hAxis && this.axes[plot.hAxis].dirty) ||
						(plot.vAxis && this.axes[plot.vAxis].dirty);
				}, this);
			calculateAxes(dirty, this.plotArea);

			return this;	//	dojox/charting/Chart
		},
		fullGeometry: function(){
			// summary:
			//		Calculate the full geometry of the chart.  This includes passing
			//		over all major elements of a chart (plots, axes, series, container)
			//		in order to ensure proper rendering.
			// returns: dojox/charting/Chart
			//		A reference to the current chart for functional chaining.
			this._makeDirty();

			// clear old values
			arr.forEach(this.stack, clear);

			// rebuild new connections, and add defaults

			// set up a theme
			if(!this.theme){
				this.setTheme(new SimpleTheme());
			}

			// assign series
			arr.forEach(this.series, function(run){
				if(!(run.plot in this.plots)){
					if(!dc.plot2d || !dc.plot2d.Default){
						throw Error("Can't find plot: Default - didn't you forget to dojo" + ".require() it?");
					}
					var plot = new dc.plot2d.Default(this, {});
					plot.name = run.plot;
					this.plots[run.plot] = this.stack.length;
					this.stack.push(plot);
				}
				this.stack[this.plots[run.plot]].addSeries(run);
			}, this);
			// assign axes
			arr.forEach(this.stack, function(plot){
				if(plot.assignAxes){
					plot.assignAxes(this.axes);
				}
			}, this);

			// calculate geometry

			// 1st pass
			var dim = this.dim = this.surface.getDimensions();
			dim.width  = g.normalizedLength(dim.width);
			dim.height = g.normalizedLength(dim.height);
			func.forIn(this.axes, clear);
			calculateAxes(this.stack, dim);

			// assumption: we don't have stacked axes yet
			var offsets = this.offsets = {l: 0, r: 0, t: 0, b: 0};
			func.forIn(this.axes, function(axis){
				func.forIn(axis.getOffsets(), function(o, i){ offsets[i] = Math.max(o, offsets[i]); });
			});
			// add title area
			if(this.title){
				this.titleGap = (this.titleGap==0) ? 0 : this.titleGap || this.theme.chart.titleGap || 20;
				this.titlePos = this.titlePos || this.theme.chart.titlePos || "top";
				this.titleFont = this.titleFont || this.theme.chart.titleFont;
				this.titleFontColor = this.titleFontColor || this.theme.chart.titleFontColor || "black";
				var tsize = g.normalizedLength(g.splitFontString(this.titleFont).size);
				offsets[this.titlePos == "top" ? "t" : "b"] += (tsize + this.titleGap);
			}
			// add margins
			func.forIn(this.margins, function(o, i){ offsets[i] += o; });

			// 2nd pass with realistic dimensions
			this.plotArea = {
				width: dim.width - offsets.l - offsets.r,
				height: dim.height - offsets.t - offsets.b
			};
			func.forIn(this.axes, clear);
			calculateAxes(this.stack, this.plotArea);

			return this;	//	dojox/charting/Chart
		},
		render: function(){
			// summary:
			//		Render the chart according to the current information defined.  This should
			//		be the last call made when defining/creating a chart, or if data within the
			//		chart has been changed.
			// returns: dojox/charting/Chart
			//		A reference to the current chart for functional chaining.

			// do we have a delayed renderer pending? If yes we need to clear it
			if(this._delayedRenderHandle){
				clearTimeout(this._delayedRenderHandle);
				this._delayedRenderHandle = null;
			}
			
			if(this.theme){
				this.theme.clear();
			}

			if(this.dirty){
				return this.fullRender();
			}

			this.calculateGeometry();

			// go over the stack backwards
			func.forEachRev(this.stack, function(plot){ plot.render(this.dim, this.offsets); }, this);

			// go over axes
			func.forIn(this.axes, function(axis){ axis.render(this.dim, this.offsets); }, this);

			this._makeClean();

			return this;	//	dojox/charting/Chart
		},
		fullRender: function(){
			// summary:
			//		Force a full rendering of the chart, including full resets on the chart itself.
			//		You should not call this method directly unless absolutely necessary.
			// returns: dojox/charting/Chart
			//		A reference to the current chart for functional chaining.

			// calculate geometry
			this.fullGeometry();
			var offsets = this.offsets, dim = this.dim;
			var w = Math.max(0, dim.width  - offsets.l - offsets.r),
				h = Math.max(0, dim.height - offsets.t - offsets.b);

			// get required colors
			//var requiredColors = func.foldl(this.stack, "z + plot.getRequiredColors()", 0);
			//this.theme.defineColors({num: requiredColors, cache: false});

			// clear old shapes
			arr.forEach(this.series, purge);
			func.forIn(this.axes, purge);
			arr.forEach(this.stack,  purge);
			var children = this.surface.children;
			for(var i = 0; i < children.length;++i){
				shape.dispose(children[i]);
			}
			if(this.chartTitle && this.chartTitle.tagName){
				// destroy title if it is a DOM node
			    domConstruct.destroy(this.chartTitle);
			}
			if(this.plotGroup){
				this.plotGroup.clear();
			}
			this.surface.clear();
			this.chartTitle = null;

			if(this.plotGroup){
				this._renderChartBackground(dim, offsets);
				this._renderPlotBackground(dim, offsets, w, h);
				this.surface.add(this.plotGroup);
				this.plotGroup.setClip({ x: offsets.l, y: offsets.t, width: w, height: h });
			}else{
				// VML
				this._renderPlotBackground(dim, offsets, w, h);
			}

			// go over the stack backwards
			func.foldr(this.stack, function(z, plot){ return plot.render(dim, offsets), 0; }, 0);

			if(!this.plotGroup){
				// VML, matting-clipping
				this._renderChartBackground(dim, offsets);
			}

			//create title: Whether to make chart title as a widget which extends dojox.charting.Element?
			if(this.title){
				var forceHtmlLabels = (g.renderer == "canvas"),
					labelType = forceHtmlLabels || !has("ie") && !has("opera") ? "html" : "gfx",
					tsize = g.normalizedLength(g.splitFontString(this.titleFont).size);
				this.chartTitle = common.createText[labelType](
					this,
					this.surface,
					dim.width/2,
					this.titlePos=="top" ? tsize + this.margins.t : dim.height - this.margins.b,
					"middle",
					this.title,
					this.titleFont,
					this.titleFontColor
				);
			}

			// go over axes
			func.forIn(this.axes, function(axis){ axis.render(dim, offsets); });

			this._makeClean();

			return this;	//	dojox/charting/Chart
		},
		_renderChartBackground: function(dim, offsets){
			var t = this.theme, rect;
			// chart background
			var fill   = this.fill   !== undefined ? this.fill   : (t.chart && t.chart.fill);
			var stroke = this.stroke !== undefined ? this.stroke : (t.chart && t.chart.stroke);

			// TRT: support for "inherit" as a named value in a theme.
			if(fill == "inherit"){
				//	find the background color of the nearest ancestor node, and use that explicitly.
				var node = this.node;
				fill = new Color(domStyle.get(node, "backgroundColor"));
				while(fill.a==0 && node!=document.documentElement){
					fill = new Color(domStyle.get(node, "backgroundColor"));
					node = node.parentNode;
				}
			}

			if(fill){
				if(this.plotGroup){
					fill = Element.prototype._shapeFill(Element.prototype._plotFill(fill, dim),
						{ x:0, y: 0, width: dim.width + 1, height: dim.height + 1 });
					this.surface.createRect({ width: dim.width + 1, height: dim.height + 1 }).setFill(fill);
				}else{
					// VML
					fill = Element.prototype._plotFill(fill, dim, offsets);
					if(offsets.l){	// left
						rect = {
							x: 0,
							y: 0,
							width:  offsets.l,
							height: dim.height + 1
						};
						this.surface.createRect(rect).setFill(Element.prototype._shapeFill(fill, rect));
					}
					if(offsets.r){	// right
						rect = {
							x: dim.width - offsets.r,
							y: 0,
							width:  offsets.r + 1,
							height: dim.height + 2
						};
						this.surface.createRect(rect).setFill(Element.prototype._shapeFill(fill, rect));
					}
					if(offsets.t){	// top
						rect = {
							x: 0,
							y: 0,
							width:  dim.width + 1,
							height: offsets.t
						};
						this.surface.createRect(rect).setFill(Element.prototype._shapeFill(fill, rect));
					}
					if(offsets.b){	// bottom
						rect = {
							x: 0,
							y: dim.height - offsets.b,
							width:  dim.width + 1,
							height: offsets.b + 2
						};
						this.surface.createRect(rect).setFill(Element.prototype._shapeFill(fill, rect));
					}
				}
			}
			if(stroke){
				this.surface.createRect({
					width:  dim.width - 1,
					height: dim.height - 1
				}).setStroke(stroke);
			}
		},
		_renderPlotBackground: function(dim, offsets, w, h){
			var t = this.theme;

			// draw a plot background
			var fill   = t.plotarea && t.plotarea.fill;
			var stroke = t.plotarea && t.plotarea.stroke;
			// size might be neg if offsets are bigger that chart size this happens quite often at
			// initialization time if the chart widget is used in a BorderContainer
			// this will fail on IE/VML
			var rect = {
				x: offsets.l - 1, y: offsets.t - 1,
				width:  w + 2,
				height: h + 2
			};
			if(fill){
				fill = Element.prototype._shapeFill(Element.prototype._plotFill(fill, dim, offsets), rect);
				this.surface.createRect(rect).setFill(fill);
			}
			if(stroke){
				this.surface.createRect({
					x: offsets.l, y: offsets.t,
					width:  w + 1,
					height: h + 1
				}).setStroke(stroke);
			}
		},
		delayedRender: function(){
			// summary:
			//		Delayed render, which is used to collect multiple updates
			//		within a delayInMs time window.
			// returns: dojox/charting/Chart
			//		A reference to the current chart for functional chaining.

			if(!this._delayedRenderHandle){
				this._delayedRenderHandle = setTimeout(
					lang.hitch(this, function(){
						this.render();
					}),
					this.delayInMs
				);
			}

			return this;	//	dojox/charting/Chart
		},
		connectToPlot: function(name, object, method){
			// summary:
			//		A convenience method to connect a function to a plot.
			// name: String
			//		The name of the plot as defined by addPlot.
			// object: Object
			//		The object to be connected.
			// method: Function
			//		The function to be executed.
			// returns: Array
			//		A handle to the connection, as defined by dojo.connect (see dojo.connect).
			return name in this.plots ? this.stack[this.plots[name]].connect(object, method) : null;	//	Array
		},
		fireEvent: function(seriesName, eventName, index){
			// summary:
			//		Fires a synthetic event for a series item.
			// seriesName: String
			//		Series name.
			// eventName: String
			//		Event name to simulate: onmouseover, onmouseout, onclick.
			// index: Number
			//		Valid data value index for the event.
			// returns: dojox/charting/Chart
			//		A reference to the current chart for functional chaining.
			if(seriesName in this.runs){
				var plotName = this.series[this.runs[seriesName]].plot;
				if(plotName in this.plots){
					var plot = this.stack[this.plots[plotName]];
					if(plot){
						plot.fireEvent(seriesName, eventName, index);
					}
				}
			}
			return this;	//	dojox/charting/Chart
		},
		_makeClean: function(){
			// reset dirty flags
			arr.forEach(this.axes,   makeClean);
			arr.forEach(this.stack,  makeClean);
			arr.forEach(this.series, makeClean);
			this.dirty = false;
		},
		_makeDirty: function(){
			// reset dirty flags
			arr.forEach(this.axes,   makeDirty);
			arr.forEach(this.stack,  makeDirty);
			arr.forEach(this.series, makeDirty);
			this.dirty = true;
		},
		_invalidateDependentPlots: function(plotName, /* Boolean */ verticalAxis){
			if(plotName in this.plots){
				var plot = this.stack[this.plots[plotName]], axis,
					axisName = verticalAxis ? "vAxis" : "hAxis";
				if(plot[axisName]){
					axis = this.axes[plot[axisName]];
					if(axis && axis.dependOnData()){
						axis.dirty = true;
						// find all plots and mark them dirty
						arr.forEach(this.stack, function(p){
							if(p[axisName] && p[axisName] == plot[axisName]){
								p.dirty = true;
							}
						});
					}
				}else{
					plot.dirty = true;
				}
			}
		}
	});

	function hSection(stats){
		return {min: stats.hmin, max: stats.hmax};
	}

	function vSection(stats){
		return {min: stats.vmin, max: stats.vmax};
	}

	function hReplace(stats, h){
		stats.hmin = h.min;
		stats.hmax = h.max;
	}

	function vReplace(stats, v){
		stats.vmin = v.min;
		stats.vmax = v.max;
	}

	function combineStats(target, source){
		if(target && source){
			target.min = Math.min(target.min, source.min);
			target.max = Math.max(target.max, source.max);
		}
		return target || source;
	}

	function calculateAxes(stack, plotArea){
		var plots = {}, axes = {};
		arr.forEach(stack, function(plot){
			var stats = plots[plot.name] = plot.getSeriesStats();
			if(plot.hAxis){
				axes[plot.hAxis] = combineStats(axes[plot.hAxis], hSection(stats));
			}
			if(plot.vAxis){
				axes[plot.vAxis] = combineStats(axes[plot.vAxis], vSection(stats));
			}
		});
		arr.forEach(stack, function(plot){
			var stats = plots[plot.name];
			if(plot.hAxis){
				hReplace(stats, axes[plot.hAxis]);
			}
			if(plot.vAxis){
				vReplace(stats, axes[plot.vAxis]);
			}
			plot.initializeScalers(plotArea, stats);
		});
	}
	
	return Chart;
});

},
'dojox/charting/Element':function(){
define("dojox/charting/Element", ["dojo/_base/lang", "dojo/_base/array", "dojo/dom-construct","dojo/_base/declare", "dojox/gfx", "dojox/gfx/shape"],
	function(lang, arr, domConstruct, declare, gfx, shape){

	return declare("dojox.charting.Element", null, {
		// summary:
		//		A base class that is used to build other elements of a chart, such as
		//		a series.
		// chart: dojox/charting/Chart
		//		The parent chart for this element.
		// group: dojox/gfx/shape.Group
		//		The visual GFX group representing this element.
		// htmlElement: Array
		//		Any DOMNodes used as a part of this element (such as HTML-based labels).
		// dirty: Boolean
		//		A flag indicating whether or not this element needs to be rendered.

		chart: null,
		group: null,
		htmlElements: null,
		dirty: true,

		constructor: function(chart){
			// summary:
			//		Creates a new charting element.
			// chart: dojox/charting/Chart
			//		The chart that this element belongs to.
			this.chart = chart;
			this.group = null;
			this.htmlElements = [];
			this.dirty = true;
			this.trailingSymbol = "...";
			this._events = [];
		},
		purgeGroup: function(){
			// summary:
			//		Clear any elements out of our group, and destroy the group.
			// returns: dojox/charting/Element
			//		A reference to this object for functional chaining.
			this.destroyHtmlElements();
			if(this.group){
				// since 1.7.x we need dispose shape otherwise there is a memoryleak
				this.group.removeShape();
				var children = this.group.children;
				for(var i = 0; i < children.length;++i){
					shape.dispose(children[i], true);
				}
				if(this.group.rawNode){
					domConstruct.empty(this.group.rawNode);
				}
				this.group.clear();
				shape.dispose(this.group, true);
				this.group = null;
			}
			this.dirty = true;
			if(this._events.length){
				arr.forEach(this._events, function(item){
					item.shape.disconnect(item.handle);
				});
				this._events = [];
			}
			return this;	//	dojox.charting.Element
		},
		cleanGroup: function(creator){
			// summary:
			//		Clean any elements (HTML or GFX-based) out of our group, and create a new one.
			// creator: dojox/gfx/shape.Surface?
			//		An optional surface to work with.
			// returns: dojox/charting/Element
			//		A reference to this object for functional chaining.
			this.destroyHtmlElements();
			if(!creator){ creator = this.chart.surface; }
			if(this.group){
				var bgnode;
				var children = this.group.children;
				for(var i = 0; i < children.length;++i){
					shape.dispose(children[i], true);
				}
				if(this.group.rawNode){
					bgnode = this.group.bgNode;
					domConstruct.empty(this.group.rawNode);
				}
				this.group.clear();
				if(bgnode){
					this.group.rawNode.appendChild(bgnode);
				}
			}else{
				this.group = creator.createGroup();
			}
			this.dirty = true;
			return this;	//	dojox.charting.Element
		},
		destroyHtmlElements: function(){
			// summary:
			//		Destroy any DOMNodes that may have been created as a part of this element.
			if(this.htmlElements.length){
				arr.forEach(this.htmlElements, domConstruct.destroy);
				this.htmlElements = [];
			}
		},
		destroy: function(){
			// summary:
			//		API addition to conform to the rest of the Dojo Toolkit's standard.
			this.purgeGroup();
		},
		//text utilities
		getTextWidth: function(s, font){
			return gfx._base._getTextBox(s, {font: font}).w || 0;
		},
		getTextWithLimitLength: function(s, font, limitWidth, truncated){
			// summary:
			//		Get the truncated string based on the limited width in px(dichotomy algorithm)
			// s: String?
			//		candidate text.
			// font: String?
			//		text's font style.
			// limitWidth: Number?
			//		text limited width in px.
			// truncated: Boolean?
			//		whether the input text(s) has already been truncated.
			// returns: Object
			// |	{
			// |		text: processed text, maybe truncated or not,
			// |		truncated: whether text has been truncated
			// |	}
			if(!s || s.length <= 0){
				return {
					text: "",
					truncated: truncated || false
				};
			}
			if(!limitWidth || limitWidth <= 0){
				return {
					text: s,
					truncated: truncated || false
				};
			}
			var delta = 2,
				//golden section for dichotomy algorithm
				trucPercentage = 0.618,
				minStr = s.substring(0,1) + this.trailingSymbol,
				minWidth = this.getTextWidth(minStr, font);
			if(limitWidth <= minWidth){
				return {
					text: minStr,
					truncated: true
				};
			}
			var width = this.getTextWidth(s, font);
			if(width <= limitWidth){
				return {
					text: s,
					truncated: truncated || false
				};
			}else{
				var begin = 0,
					end = s.length;
				while(begin < end){
					if(end - begin <= delta ){
						while (this.getTextWidth(s.substring(0, begin) + this.trailingSymbol, font) > limitWidth) {
							begin -= 1;
						}
						return {
							text: (s.substring(0,begin) + this.trailingSymbol),
							truncated: true
							};
					}
					var index = begin + Math.round((end - begin) * trucPercentage),
						widthIntercepted = this.getTextWidth(s.substring(0, index), font);
					if(widthIntercepted < limitWidth){
						begin = index;
						end = end;
					}else{
						begin = begin;
						end = index;
					}
				}
			}
		},
		getTextWithLimitCharCount: function(s, font, wcLimit, truncated){
			// summary:
			//		Get the truncated string based on the limited character count(dichotomy algorithm)
			// s: String?
			//		candidate text.
			// font: String?
			//		text's font style.
			// wcLimit: Number?
			//		text limited character count.
			// truncated: Boolean?
			//		whether the input text(s) has already been truncated.
			// returns: Object
			// |	{
			// |		text: processed text, maybe truncated or not,
			// |		truncated: whether text has been truncated
			// |	}
			if (!s || s.length <= 0) {
				return {
					text: "",
					truncated: truncated || false
				};
			}
			if(!wcLimit || wcLimit <= 0 || s.length <= wcLimit){
				return {
					text: s,
					truncated: truncated || false
				};
			}
			return {
				text: s.substring(0, wcLimit) + this.trailingSymbol,
				truncated: true
			};
		},
		// fill utilities
		_plotFill: function(fill, dim, offsets){
			// process a plot-wide fill
			if(!fill || !fill.type || !fill.space){
				return fill;
			}
			var space = fill.space, span;
			switch(fill.type){
				case "linear":
					if(space === "plot" || space === "shapeX" || space === "shapeY"){
						// clone a fill so we can modify properly directly
						fill = gfx.makeParameters(gfx.defaultLinearGradient, fill);
						fill.space = space;
						// process dimensions
						if(space === "plot" || space === "shapeX"){
							// process Y
							span = dim.height - offsets.t - offsets.b;
							fill.y1 = offsets.t + span * fill.y1 / 100;
							fill.y2 = offsets.t + span * fill.y2 / 100;
						}
						if(space === "plot" || space === "shapeY"){
							// process X
							span = dim.width - offsets.l - offsets.r;
							fill.x1 = offsets.l + span * fill.x1 / 100;
							fill.x2 = offsets.l + span * fill.x2 / 100;
						}
					}
					break;
				case "radial":
					if(space === "plot"){
						// this one is used exclusively for scatter charts
						// clone a fill so we can modify properly directly
						fill = gfx.makeParameters(gfx.defaultRadialGradient, fill);
						fill.space = space;
						// process both dimensions
						var spanX = dim.width  - offsets.l - offsets.r,
							spanY = dim.height - offsets.t - offsets.b;
						fill.cx = offsets.l + spanX * fill.cx / 100;
						fill.cy = offsets.t + spanY * fill.cy / 100;
						fill.r  = fill.r * Math.sqrt(spanX * spanX + spanY * spanY) / 200;
					}
					break;
				case "pattern":
					if(space === "plot" || space === "shapeX" || space === "shapeY"){
						// clone a fill so we can modify properly directly
						fill = gfx.makeParameters(gfx.defaultPattern, fill);
						fill.space = space;
						// process dimensions
						if(space === "plot" || space === "shapeX"){
							// process Y
							span = dim.height - offsets.t - offsets.b;
							fill.y = offsets.t + span * fill.y / 100;
							fill.height = span * fill.height / 100;
						}
						if(space === "plot" || space === "shapeY"){
							// process X
							span = dim.width - offsets.l - offsets.r;
							fill.x = offsets.l + span * fill.x / 100;
							fill.width = span * fill.width / 100;
						}
					}
					break;
			}
			return fill;
		},
		_shapeFill: function(fill, bbox){
			// process shape-specific fill
			if(!fill || !fill.space){
				return fill;
			}
			var space = fill.space, span;
			switch(fill.type){
				case "linear":
					if(space === "shape" || space === "shapeX" || space === "shapeY"){
						// clone a fill so we can modify properly directly
						fill = gfx.makeParameters(gfx.defaultLinearGradient, fill);
						fill.space = space;
						// process dimensions
						if(space === "shape" || space === "shapeX"){
							// process X
							span = bbox.width;
							fill.x1 = bbox.x + span * fill.x1 / 100;
							fill.x2 = bbox.x + span * fill.x2 / 100;
						}
						if(space === "shape" || space === "shapeY"){
							// process Y
							span = bbox.height;
							fill.y1 = bbox.y + span * fill.y1 / 100;
							fill.y2 = bbox.y + span * fill.y2 / 100;
						}
					}
					break;
				case "radial":
					if(space === "shape"){
						// this one is used exclusively for bubble charts and pie charts
						// clone a fill so we can modify properly directly
						fill = gfx.makeParameters(gfx.defaultRadialGradient, fill);
						fill.space = space;
						// process both dimensions
						fill.cx = bbox.x + bbox.width  / 2;
						fill.cy = bbox.y + bbox.height / 2;
						fill.r  = fill.r * bbox.width  / 200;
					}
					break;
				case "pattern":
					if(space === "shape" || space === "shapeX" || space === "shapeY"){
						// clone a fill so we can modify properly directly
						fill = gfx.makeParameters(gfx.defaultPattern, fill);
						fill.space = space;
						// process dimensions
						if(space === "shape" || space === "shapeX"){
							// process X
							span = bbox.width;
							fill.x = bbox.x + span * fill.x / 100;
							fill.width = span * fill.width / 100;
						}
						if(space === "shape" || space === "shapeY"){
							// process Y
							span = bbox.height;
							fill.y = bbox.y + span * fill.y / 100;
							fill.height = span * fill.height / 100;
						}
					}
					break;
			}
			return fill;
		},
		_pseudoRadialFill: function(fill, center, radius, start, end){
			// process pseudo-radial fills
			if(!fill || fill.type !== "radial" || fill.space !== "shape"){
				return fill;
			}
			// clone and normalize fill
			var space = fill.space;
			fill = gfx.makeParameters(gfx.defaultRadialGradient, fill);
			fill.space = space;
			if(arguments.length < 4){
				// process both dimensions
				fill.cx = center.x;
				fill.cy = center.y;
				fill.r  = fill.r * radius / 100;
				return fill;
			}
			// convert to a linear gradient
			var angle = arguments.length < 5 ? start : (end + start) / 2;
			return {
				type: "linear",
				x1: center.x,
				y1: center.y,
				x2: center.x + fill.r * radius * Math.cos(angle) / 100,
				y2: center.y + fill.r * radius * Math.sin(angle) / 100,
				colors: fill.colors
			};
			return fill;
		}
	});
});

},
'dojox/gfx':function(){
define("dojox/gfx", ["dojo/_base/lang", "./gfx/_base", "./gfx/renderer!"], 
  function(lang, gfxBase, renderer){
	// module:
	//		dojox/gfx
	// summary:
	//		This the root of the Dojo Graphics package
	gfxBase.switchTo(renderer);
	return gfxBase;
});

},
'dojox/gfx/_base':function(){
define("dojox/gfx/_base", ["dojo/_base/kernel", "dojo/_base/lang", "dojo/_base/Color", "dojo/_base/sniff", "dojo/_base/window",
	    "dojo/_base/array","dojo/dom", "dojo/dom-construct","dojo/dom-geometry"],
function(kernel, lang, Color, has, win, arr, dom, domConstruct, domGeom){
	// module:
	//		dojox/gfx
	// summary:
	//		This module contains common core Graphics API used by different graphics renderers.

	var g = lang.getObject("dojox.gfx", true),
		b = g._base = {};
	
	// candidates for dojox.style (work on VML and SVG nodes)
	g._hasClass = function(/*DomNode*/node, /*String*/classStr){
		// summary:
		//		Returns whether or not the specified classes are a portion of the
		//		class list currently applied to the node.
		
		// return (new RegExp('(^|\\s+)'+classStr+'(\\s+|$)')).test(node.className)	// Boolean
		var cls = node.getAttribute("className");
		return cls && (" " + cls + " ").indexOf(" " + classStr + " ") >= 0;  // Boolean
	};
	g._addClass = function(/*DomNode*/node, /*String*/classStr){
		// summary:
		//		Adds the specified classes to the end of the class list on the
		//		passed node.
		var cls = node.getAttribute("className") || "";
		if(!cls || (" " + cls + " ").indexOf(" " + classStr + " ") < 0){
			node.setAttribute("className", cls + (cls ? " " : "") + classStr);
		}
	};
	g._removeClass = function(/*DomNode*/node, /*String*/classStr){
		// summary:
		//		Removes classes from node.
		var cls = node.getAttribute("className");
		if(cls){
			node.setAttribute(
				"className",
				cls.replace(new RegExp('(^|\\s+)' + classStr + '(\\s+|$)'), "$1$2")
			);
		}
	};

	// candidate for dojox.html.metrics (dynamic font resize handler is not implemented here)

	//		derived from Morris John's emResized measurer
	b._getFontMeasurements = function(){
		// summary:
		//		Returns an object that has pixel equivilents of standard font
		//		size values.
		var heights = {
			'1em': 0, '1ex': 0, '100%': 0, '12pt': 0, '16px': 0, 'xx-small': 0,
			'x-small': 0, 'small': 0, 'medium': 0, 'large': 0, 'x-large': 0,
			'xx-large': 0
		};
		var p, oldStyle;	
		if(has("ie")){
			//	We do a font-size fix if and only if one isn't applied already.
			// NOTE: If someone set the fontSize on the HTML Element, this will kill it.
			oldStyle = win.doc.documentElement.style.fontSize || "";
			if(!oldStyle){
				win.doc.documentElement.style.fontSize="100%";
			}
		}

		//		set up the measuring node.
		var div = domConstruct.create("div", {style: {
				position: "absolute",
				left: "0",
				top: "-100px",
				width: "30px",
				height: "1000em",
				borderWidth: "0",
				margin: "0",
				padding: "0",
				outline: "none",
				lineHeight: "1",
				overflow: "hidden"
			}}, win.body());

		//		do the measurements.
		for(p in heights){
			div.style.fontSize = p;
			heights[p] = Math.round(div.offsetHeight * 12/16) * 16/12 / 1000;
		}

		if(has("ie")){
			// Restore the font to its old style.
			win.doc.documentElement.style.fontSize = oldStyle;
		}
		win.body().removeChild(div);
		return heights; //object
	};

	var fontMeasurements = null;

	b._getCachedFontMeasurements = function(recalculate){
		if(recalculate || !fontMeasurements){
			fontMeasurements = b._getFontMeasurements();
		}
		return fontMeasurements;
	};

	// candidate for dojox.html.metrics

	var measuringNode = null, empty = {};
	b._getTextBox = function(	/*String*/ text,
								/*Object*/ style,
								/*String?*/ className){
		var m, s, al = arguments.length;
		var i;
		if(!measuringNode){
			measuringNode = domConstruct.create("div", {style: {
				position: "absolute",
				top: "-10000px",
				left: "0"
			}}, win.body());
		}
		m = measuringNode;
		// reset styles
		m.className = "";
		s = m.style;
		s.borderWidth = "0";
		s.margin = "0";
		s.padding = "0";
		s.outline = "0";
		// set new style
		if(al > 1 && style){
			for(i in style){
				if(i in empty){ continue; }
				s[i] = style[i];
			}
		}
		// set classes
		if(al > 2 && className){
			m.className = className;
		}
		// take a measure
		m.innerHTML = text;

		if(m["getBoundingClientRect"]){
			var bcr = m.getBoundingClientRect();
			return {l: bcr.left, t: bcr.top, w: bcr.width || (bcr.right - bcr.left), h: bcr.height || (bcr.bottom - bcr.top)};
		}else{
			return domGeom.getMarginBox(m);
		}
	};

	// candidate for dojo.dom

	var uniqueId = 0;
	b._getUniqueId = function(){
		// summary:
		//		returns a unique string for use with any DOM element
		var id;
		do{
			id = kernel._scopeName + "xUnique" + (++uniqueId);
		}while(dom.byId(id));
		return id;
	};

	/*=====
	g.Stroke = {
		// summary:
		//		A stroke defines stylistic properties that are used when drawing a path.

		// color: String
		//		The color of the stroke, default value 'black'.
		color: "black",

		// style: String
		//		The style of the stroke, one of 'solid', ... . Default value 'solid'.
		style: "solid",

		// width: Number
		//		The width of a stroke, default value 1.
		width: 1,

		// cap: String
		//		The endcap style of the path. One of 'butt', 'round', ... . Default value 'butt'.
		cap: "butt",

		// join: Number
		//		The join style to use when combining path segments. Default value 4.
		join: 4
	};
	
	g.Fill = {
		// summary:
		//		Defines how to fill a shape. Four types of fills can be used: solid, linear gradient, radial gradient and pattern.
		//		See dojox/gfx.LinearGradient, dojox/gfx.RadialGradient and dojox/gfx.Pattern respectively for more information about the properties supported by each type.
		
		// type: String?
		//		The type of fill. One of 'linear', 'radial', 'pattern' or undefined. If not specified, a solid fill is assumed.
		type:"",
		
		// color: String|dojo/Color?
		//		The color of a solid fill type.
		color:null,
		
	};
	
	g.LinearGradient = {
		// summary:
		//		An object defining the default stylistic properties used for Linear Gradient fills.
		//		Linear gradients are drawn along a virtual line, which results in appearance of a rotated pattern in a given direction/orientation.

		// type: String
		//		Specifies this object is a Linear Gradient, value 'linear'
		type: "linear",

		// x1: Number
		//		The X coordinate of the start of the virtual line along which the gradient is drawn, default value 0.
		x1: 0,

		// y1: Number
		//		The Y coordinate of the start of the virtual line along which the gradient is drawn, default value 0.
		y1: 0,

		// x2: Number
		//		The X coordinate of the end of the virtual line along which the gradient is drawn, default value 100.
		x2: 100,

		// y2: Number
		//		The Y coordinate of the end of the virtual line along which the gradient is drawn, default value 100.
		y2: 100,

		// colors: Array
		//		An array of colors at given offsets (from the start of the line).  The start of the line is
		//		defined at offest 0 with the end of the line at offset 1.
		//		Default value, [{ offset: 0, color: 'black'},{offset: 1, color: 'white'}], is a gradient from black to white.
		colors: []
	};
	
	g.RadialGradient = {
		// summary:
		//		Specifies the properties for RadialGradients using in fills patterns.

		// type: String
		//		Specifies this is a RadialGradient, value 'radial'
		type: "radial",

		// cx: Number
		//		The X coordinate of the center of the radial gradient, default value 0.
		cx: 0,

		// cy: Number
		//		The Y coordinate of the center of the radial gradient, default value 0.
		cy: 0,

		// r: Number
		//		The radius to the end of the radial gradient, default value 100.
		r: 100,

		// colors: Array
		//		An array of colors at given offsets (from the center of the radial gradient).
		//		The center is defined at offest 0 with the outer edge of the gradient at offset 1.
		//		Default value, [{ offset: 0, color: 'black'},{offset: 1, color: 'white'}], is a gradient from black to white.
		colors: []
	};
	
	g.Pattern = {
		// summary:
		//		An object specifying the default properties for a Pattern using in fill operations.

		// type: String
		//		Specifies this object is a Pattern, value 'pattern'.
		type: "pattern",

		// x: Number
		//		The X coordinate of the position of the pattern, default value is 0.
		x: 0,

		// y: Number
		//		The Y coordinate of the position of the pattern, default value is 0.
		y: 0,

		// width: Number
		//		The width of the pattern image, default value is 0.
		width: 0,

		// height: Number
		//		The height of the pattern image, default value is 0.
		height: 0,

		// src: String
		//		A url specifying the image to use for the pattern.
		src: ""
	};

	g.Text = {
		//	summary:
		//		A keyword argument object defining both the text to be rendered in a VectorText shape,
		//		and specifying position, alignment, and fitting.
		//	text: String
		//		The text to be rendered.
		//	x: Number?
		//		The left coordinate for the text's bounding box.
		//	y: Number?
		//		The top coordinate for the text's bounding box.
		//	width: Number?
		//		The width of the text's bounding box.
		//	height: Number?
		//		The height of the text's bounding box.
		//	align: String?
		//		The alignment of the text, as defined in SVG. Can be "start", "end" or "middle".
		//	fitting: Number?
		//		How the text is to be fitted to the bounding box. Can be 0 (no fitting), 1 (fitting based on
		//		passed width of the bounding box and the size of the font), or 2 (fit text to the bounding box,
		//		and ignore any size parameters).
		//	leading: Number?
		//		The leading to be used between lines in the text.
		//	decoration: String?
		//		Any text decoration to be used.
	};

	g.Font = {
		// summary:
		//		An object specifying the properties for a Font used in text operations.
	
		// type: String
		//		Specifies this object is a Font, value 'font'.
		type: "font",
	
		// style: String
		//		The font style, one of 'normal', 'bold', default value 'normal'.
		style: "normal",
	
		// variant: String
		//		The font variant, one of 'normal', ... , default value 'normal'.
		variant: "normal",
	
		// weight: String
		//		The font weight, one of 'normal', ..., default value 'normal'.
		weight: "normal",
	
		// size: String
		//		The font size (including units), default value '10pt'.
		size: "10pt",
	
		// family: String
		//		The font family, one of 'serif', 'sanserif', ..., default value 'serif'.
		family: "serif"
	};

	=====*/

	lang.mixin(g, {
		// summary:
		//		defines constants, prototypes, and utility functions for the core Graphics API

		// default shapes, which are used to fill in missing parameters
		defaultPath: {
			// summary:
			//		Defines the default Path prototype object.

			// type: String
			//		Specifies this object is a Path, default value 'path'.
			type: "path", 

			// path: String
			//		The path commands. See W32C SVG 1.0 specification.
			//		Defaults to empty string value.
			path: ""
		},
		defaultPolyline: {
			// summary:
			//		Defines the default PolyLine prototype.

			// type: String
			//		Specifies this object is a PolyLine, default value 'polyline'.
			type: "polyline",

			// points: Array
			//		An array of point objects [{x:0,y:0},...] defining the default polyline's line segments. Value is an empty array [].
			points: []
		},
		defaultRect: {
			// summary:
			//		Defines the default Rect prototype.

			// type: String
			//		Specifies this default object is a type of Rect. Value is 'rect'
			type: "rect",

			// x: Number
			//		The X coordinate of the default rectangles position, value 0.
			x: 0,

			// y: Number
			//		The Y coordinate of the default rectangle's position, value 0.
			y: 0,

			// width: Number
			//		The width of the default rectangle, value 100.
			width: 100,

			// height: Number
			//		The height of the default rectangle, value 100.
			height: 100,

			// r: Number
			//		The corner radius for the default rectangle, value 0.
			r: 0
		},
		defaultEllipse: {
			// summary:
			//		Defines the default Ellipse prototype.

			// type: String
			//		Specifies that this object is a type of Ellipse, value is 'ellipse'
			type: "ellipse",

			// cx: Number
			//		The X coordinate of the center of the ellipse, default value 0.
			cx: 0,

			// cy: Number
			//		The Y coordinate of the center of the ellipse, default value 0.
			cy: 0,

			// rx: Number
			//		The radius of the ellipse in the X direction, default value 200.
			rx: 200,

			// ry: Number
			//		The radius of the ellipse in the Y direction, default value 200.
			ry: 100
		},
		defaultCircle: {
			// summary:
			//		An object defining the default Circle prototype.

			// type: String
			//		Specifies this object is a circle, value 'circle'
			type: "circle",

			// cx: Number
			//		The X coordinate of the center of the circle, default value 0.
			cx: 0,
			// cy: Number
			//		The Y coordinate of the center of the circle, default value 0.
			cy: 0,

			// r: Number
			//		The radius, default value 100.
			r: 100
		},
		defaultLine: {
			// summary:
			//		An object defining the default Line prototype.

			// type: String
			//		Specifies this is a Line, value 'line'
			type: "line",

			// x1: Number
			//		The X coordinate of the start of the line, default value 0.
			x1: 0,

			// y1: Number
			//		The Y coordinate of the start of the line, default value 0.
			y1: 0,

			// x2: Number
			//		The X coordinate of the end of the line, default value 100.
			x2: 100,

			// y2: Number
			//		The Y coordinate of the end of the line, default value 100.
			y2: 100
		},
		defaultImage: {
			// summary:
			//		Defines the default Image prototype.

			// type: String
			//		Specifies this object is an image, value 'image'.
			type: "image",

			// x: Number
			//		The X coordinate of the image's position, default value 0.
			x: 0,

			// y: Number
			//		The Y coordinate of the image's position, default value 0.
			y: 0,

			// width: Number
			//		The width of the image, default value 0.
			width: 0,

			// height: Number
			//		The height of the image, default value 0.
			height: 0,

			// src: String
			//		The src url of the image, defaults to empty string.
			src: ""
		},
		defaultText: {
			// summary:
			//		Defines the default Text prototype.

			// type: String
			//		Specifies this is a Text shape, value 'text'.
			type: "text",

			// x: Number
			//		The X coordinate of the text position, default value 0.
			x: 0,

			// y: Number
			//		The Y coordinate of the text position, default value 0.
			y: 0,

			// text: String
			//		The text to be displayed, default value empty string.
			text: "",

			// align:	String
			//		The horizontal text alignment, one of 'start', 'end', 'center'. Default value 'start'.
			align: "start",

			// decoration: String
			//		The text decoration , one of 'none', ... . Default value 'none'.
			decoration: "none",

			// rotated: Boolean
			//		Whether the text is rotated, boolean default value false.
			rotated: false,

			// kerning: Boolean
			//		Whether kerning is used on the text, boolean default value true.
			kerning: true
		},
		defaultTextPath: {
			// summary:
			//		Defines the default TextPath prototype.

			// type: String
			//		Specifies this is a TextPath, value 'textpath'.
			type: "textpath",

			// text: String
			//		The text to be displayed, default value empty string.
			text: "",

			// align: String
			//		The horizontal text alignment, one of 'start', 'end', 'center'. Default value 'start'.
			align: "start",

			// decoration: String
			//		The text decoration , one of 'none', ... . Default value 'none'.
			decoration: "none",

			// rotated: Boolean
			//		Whether the text is rotated, boolean default value false.
			rotated: false,

			// kerning: Boolean
			//		Whether kerning is used on the text, boolean default value true.
			kerning: true
		},

		// default stylistic attributes
		defaultStroke: {
			// summary:
			//		A stroke defines stylistic properties that are used when drawing a path.
			//		This object defines the default Stroke prototype.
			// type: String
			//		Specifies this object is a type of Stroke, value 'stroke'.
			type: "stroke",

			// color: String
			//		The color of the stroke, default value 'black'.
			color: "black",

			// style: String
			//		The style of the stroke, one of 'solid', ... . Default value 'solid'.
			style: "solid",

			// width: Number
			//		The width of a stroke, default value 1.
			width: 1,

			// cap: String
			//		The endcap style of the path. One of 'butt', 'round', ... . Default value 'butt'.
			cap: "butt",

			// join: Number
			//		The join style to use when combining path segments. Default value 4.
			join: 4
		},
		defaultLinearGradient: {
			// summary:
			//		An object defining the default stylistic properties used for Linear Gradient fills.
			//		Linear gradients are drawn along a virtual line, which results in appearance of a rotated pattern in a given direction/orientation.

			// type: String
			//		Specifies this object is a Linear Gradient, value 'linear'
			type: "linear",

			// x1: Number
			//		The X coordinate of the start of the virtual line along which the gradient is drawn, default value 0.
			x1: 0,

			// y1: Number
			//		The Y coordinate of the start of the virtual line along which the gradient is drawn, default value 0.
			y1: 0,

			// x2: Number
			//		The X coordinate of the end of the virtual line along which the gradient is drawn, default value 100.
			x2: 100,

			// y2: Number
			//		The Y coordinate of the end of the virtual line along which the gradient is drawn, default value 100.
			y2: 100,

			// colors: Array
			//		An array of colors at given offsets (from the start of the line).  The start of the line is
			//		defined at offest 0 with the end of the line at offset 1.
			//		Default value, [{ offset: 0, color: 'black'},{offset: 1, color: 'white'}], is a gradient from black to white.
			colors: [
				{ offset: 0, color: "black" }, { offset: 1, color: "white" }
			]
		},
		defaultRadialGradient: {
			// summary:
			//		An object specifying the default properties for RadialGradients using in fills patterns.

			// type: String
			//		Specifies this is a RadialGradient, value 'radial'
			type: "radial",

			// cx: Number
			//		The X coordinate of the center of the radial gradient, default value 0.
			cx: 0,

			// cy: Number
			//		The Y coordinate of the center of the radial gradient, default value 0.
			cy: 0,

			// r: Number
			//		The radius to the end of the radial gradient, default value 100.
			r: 100,

			// colors: Array
			//		An array of colors at given offsets (from the center of the radial gradient).
			//		The center is defined at offest 0 with the outer edge of the gradient at offset 1.
			//		Default value, [{ offset: 0, color: 'black'},{offset: 1, color: 'white'}], is a gradient from black to white.
			colors: [
				{ offset: 0, color: "black" }, { offset: 1, color: "white" }
			]
		},
		defaultPattern: {
			// summary:
			//		An object specifying the default properties for a Pattern using in fill operations.

			// type: String
			//		Specifies this object is a Pattern, value 'pattern'.
			type: "pattern",

			// x: Number
			//		The X coordinate of the position of the pattern, default value is 0.
			x: 0,

			// y: Number
			//		The Y coordinate of the position of the pattern, default value is 0.
			y: 0,

			// width: Number
			//		The width of the pattern image, default value is 0.
			width: 0,

			// height: Number
			//		The height of the pattern image, default value is 0.
			height: 0,

			// src: String
			//		A url specifying the image to use for the pattern.
			src: ""
		},
		defaultFont: {
			// summary:
			//		An object specifying the default properties for a Font used in text operations.

			// type: String
			//		Specifies this object is a Font, value 'font'.
			type: "font",

			// style: String
			//		The font style, one of 'normal', 'bold', default value 'normal'.
			style: "normal",

			// variant: String
			//		The font variant, one of 'normal', ... , default value 'normal'.
			variant: "normal",

			// weight: String
			//		The font weight, one of 'normal', ..., default value 'normal'.
			weight: "normal",

			// size: String
			//		The font size (including units), default value '10pt'.
			size: "10pt",

			// family: String
			//		The font family, one of 'serif', 'sanserif', ..., default value 'serif'.
			family: "serif"
		},

		getDefault: (function(){
			// summary:
			//		Returns a function used to access default memoized prototype objects (see them defined above).
			var typeCtorCache = {};
			// a memoized delegate()
			return function(/*String*/ type){
				var t = typeCtorCache[type];
				if(t){
					return new t();
				}
				t = typeCtorCache[type] = new Function();
				t.prototype = g[ "default" + type ];
				return new t();
			}
		})(),

		normalizeColor: function(/*dojo/Color|Array|string|Object*/ color){
			// summary:
			//		converts any legal color representation to normalized
			//		dojo/Color object
			// color:
			//		A color representation.
			return (color instanceof Color) ? color : new Color(color); // dojo/Color
		},
		normalizeParameters: function(existed, update){
			// summary:
			//		updates an existing object with properties from an 'update'
			//		object
			// existed: Object
			//		the target object to be updated
			// update: Object
			//		the 'update' object, whose properties will be used to update
			//		the existed object
			var x;
			if(update){
				var empty = {};
				for(x in existed){
					if(x in update && !(x in empty)){
						existed[x] = update[x];
					}
				}
			}
			return existed;	// Object
		},
		makeParameters: function(defaults, update){
			// summary:
			//		copies the original object, and all copied properties from the
			//		'update' object
			// defaults: Object
			//		the object to be cloned before updating
			// update: Object
			//		the object, which properties are to be cloned during updating
			// returns: Object
			//      new object with new and default properties
			var i = null;
			if(!update){
				// return dojo.clone(defaults);
				return lang.delegate(defaults);
			}
			var result = {};
			for(i in defaults){
				if(!(i in result)){
					result[i] = lang.clone((i in update) ? update[i] : defaults[i]);
				}
			}
			return result; // Object
		},
		formatNumber: function(x, addSpace){
			// summary:
			//		converts a number to a string using a fixed notation
			// x: Number
			//		number to be converted
			// addSpace: Boolean
			//		whether to add a space before a positive number
			// returns: String
			//      the formatted value
			var val = x.toString();
			if(val.indexOf("e") >= 0){
				val = x.toFixed(4);
			}else{
				var point = val.indexOf(".");
				if(point >= 0 && val.length - point > 5){
					val = x.toFixed(4);
				}
			}
			if(x < 0){
				return val; // String
			}
			return addSpace ? " " + val : val; // String
		},
		// font operations
		makeFontString: function(font){
			// summary:
			//		converts a font object to a CSS font string
			// font: Object
			//		font object (see dojox/gfx.defaultFont)
			return font.style + " " + font.variant + " " + font.weight + " " + font.size + " " + font.family; // Object
		},
		splitFontString: function(str){
			// summary:
			//		converts a CSS font string to a font object
			// description:
			//		Converts a CSS font string to a gfx font object. The CSS font
			//		string components should follow the W3C specified order
			//		(see http://www.w3.org/TR/CSS2/fonts.html#font-shorthand):
			//		style, variant, weight, size, optional line height (will be
			//		ignored), and family. Note that the Font.size attribute is limited to numeric CSS length.
			// str: String
			//		a CSS font string.
			// returns: Object
			//      object in dojox/gfx.defaultFont format
			var font = g.getDefault("Font");
			var t = str.split(/\s+/);
			do{
				if(t.length < 5){ break; }
				font.style   = t[0];
				font.variant = t[1];
				font.weight  = t[2];
				var i = t[3].indexOf("/");
				font.size = i < 0 ? t[3] : t[3].substring(0, i);
				var j = 4;
				if(i < 0){
					if(t[4] == "/"){
						j = 6;
					}else if(t[4].charAt(0) == "/"){
						j = 5;
					}
				}
				if(j < t.length){
					font.family = t.slice(j).join(" ");
				}
			}while(false);
			return font;	// Object
		},
		// length operations

		// cm_in_pt: Number
		//		points per centimeter (constant)
		cm_in_pt: 72 / 2.54,

		// mm_in_pt: Number
		//		points per millimeter (constant)
		mm_in_pt: 7.2 / 2.54,

		px_in_pt: function(){
			// summary:
			//		returns the current number of pixels per point.
			return g._base._getCachedFontMeasurements()["12pt"] / 12;	// Number
		},

		pt2px: function(len){
			// summary:
			//		converts points to pixels
			// len: Number
			//		a value in points
			return len * g.px_in_pt();	// Number
		},

		px2pt: function(len){
			// summary:
			//		converts pixels to points
			// len: Number
			//		a value in pixels
			return len / g.px_in_pt();	// Number
		},

		normalizedLength: function(len) {
			// summary:
			//		converts any length value to pixels
			// len: String
			//		a length, e.g., '12pc'
			// returns: Number
			//      pixels
			if(len.length === 0){ return 0; }
			if(len.length > 2){
				var px_in_pt = g.px_in_pt();
				var val = parseFloat(len);
				switch(len.slice(-2)){
					case "px": return val;
					case "pt": return val * px_in_pt;
					case "in": return val * 72 * px_in_pt;
					case "pc": return val * 12 * px_in_pt;
					case "mm": return val * g.mm_in_pt * px_in_pt;
					case "cm": return val * g.cm_in_pt * px_in_pt;
				}
			}
			return parseFloat(len);	// Number
		},

		// pathVmlRegExp: RegExp
		//		a constant regular expression used to split a SVG/VML path into primitive components
		// tags:
		//		private
		pathVmlRegExp: /([A-Za-z]+)|(\d+(\.\d+)?)|(\.\d+)|(-\d+(\.\d+)?)|(-\.\d+)/g,

		// pathVmlRegExp: RegExp
		//		a constant regular expression used to split a SVG/VML path into primitive components
		// tags:
		//		private
		pathSvgRegExp: /([A-Za-z])|(\d+(\.\d+)?)|(\.\d+)|(-\d+(\.\d+)?)|(-\.\d+)/g,

		equalSources: function(a, b){
			// summary:
			//		compares event sources, returns true if they are equal
			// a: Object
			//		first event source
			// b: Object
			//		event source to compare against a
			// returns: Boolean
			//      true, if objects are truthy and the same
			return a && b && a === b;
		},

		switchTo: function(/*String|Object*/ renderer){
			// summary:
			//		switch the graphics implementation to the specified renderer.
			// renderer:
			//		Either the string name of a renderer (eg. 'canvas', 'svg, ...) or the renderer
			//		object to switch to.
			var ns = typeof renderer == "string" ? g[renderer] : renderer;
			if(ns){
				// If more options are added, update the docblock at the end of shape.js!
				arr.forEach(["Group", "Rect", "Ellipse", "Circle", "Line",
						"Polyline", "Image", "Text", "Path", "TextPath",
						"Surface", "createSurface", "fixTarget"], function(name){
					g[name] = ns[name];
				});
			}
		}
	});
	
	/*=====
		g.createSurface = function(parentNode, width, height){
			// summary:
			//		creates a surface
			// parentNode: Node
			//		a parent node
			// width: String|Number
			//		width of surface, e.g., "100px" or 100
			// height: String|Number
			//		height of surface, e.g., "100px" or 100
			// returns: dojox/gfx.Surface
			//     newly created surface
		};
		g.fixTarget = function(){
			// tags:
			//		private
		};
	=====*/
	
	return g; // defaults object api
});

},
'dojox/gfx/renderer':function(){
define("dojox/gfx/renderer", ["./_base","dojo/_base/lang", "dojo/_base/sniff", "dojo/_base/window", "dojo/_base/config"],
  function(g, lang, has, win, config){
  //>> noBuildResolver
	var currentRenderer = null;

	has.add("vml", function(global, document, element){
		element.innerHTML = "<v:shape adj=\"1\"/>";
		var supported = ("adj" in element.firstChild);
		element.innerHTML = "";
		return supported;
	});

	return {
		// summary:
		//		This module is an AMD loader plugin that loads the appropriate graphics renderer
		//		implementation based on detected environment and current configuration settings.
		
		load: function(id, require, load){
			// tags:
			//      private
			if(currentRenderer && id != "force"){
				load(currentRenderer);
				return;
			}
			var renderer = config.forceGfxRenderer,
				renderers = !renderer && (lang.isString(config.gfxRenderer) ?
					config.gfxRenderer : "svg,vml,canvas,silverlight").split(","),
				silverlightObject, silverlightFlag;

			while(!renderer && renderers.length){
				switch(renderers.shift()){
					case "svg":
						// the next test is from https://github.com/phiggins42/has.js
						if("SVGAngle" in win.global){
							renderer = "svg";
						}
						break;
					case "vml":
						if(has("vml")){
							renderer = "vml";
						}
						break;
					case "silverlight":
						try{
							if(has("ie")){
								silverlightObject = new ActiveXObject("AgControl.AgControl");
								if(silverlightObject && silverlightObject.IsVersionSupported("1.0")){
									silverlightFlag = true;
								}
							}else{
								if(navigator.plugins["Silverlight Plug-In"]){
									silverlightFlag = true;
								}
							}
						}catch(e){
							silverlightFlag = false;
						}finally{
							silverlightObject = null;
						}
						if(silverlightFlag){
							renderer = "silverlight";
						}
						break;
					case "canvas":
						if(win.global.CanvasRenderingContext2D){
							renderer = "canvas";
						}
						break;
				}
			}

			if (renderer === 'canvas' && config.canvasEvents !== false) {
				renderer = "canvasWithEvents";
			}

			if(config.isDebug){
				console.log("gfx renderer = " + renderer);
			}

			function loadRenderer(){
				require(["dojox/gfx/" + renderer], function(module){
					g.renderer = renderer;
					// memorize the renderer module
					currentRenderer = module;
					// now load it
					load(module);
				});
			}
			if(renderer == "svg" && typeof window.svgweb != "undefined"){
				window.svgweb.addOnLoad(loadRenderer);
			}else{
				loadRenderer();
			}
		}
	};
});

},
'dojox/gfx/shape':function(){
define("dojox/gfx/shape", ["./_base", "dojo/_base/lang", "dojo/_base/declare", "dojo/_base/kernel", "dojo/_base/sniff",
	"dojo/_base/connect", "dojo/_base/array", "dojo/dom-construct", "dojo/_base/Color", "./matrix" /*===== , "./path" =====*/ ], 
	function(g, lang, declare, kernel, has, events, arr, domConstruct, Color, matrixLib){

	var shape = g.shape = {
		// summary:
		//		This module contains the core graphics Shape API.
		//		Different graphics renderer implementation modules (svg, canvas, vml, silverlight, etc.) extend this
		//		basic api to provide renderer-specific implementations for each shape.
	};
	
	// a set of ids (keys=type)
	var _ids = {};
	// a simple set impl to map shape<->id
	var registry = {};
	var disposeCount = 0, fixIELeak = has("ie") < 9;

	function repack(oldRegistry){
		var newRegistry = {};
		for(var key in oldRegistry){
			if(oldRegistry.hasOwnProperty(key)){
				newRegistry[key] = oldRegistry[key]
			}
		}
		return newRegistry;
	}

	shape.register = function(/*dojox/gfx/shape.Shape*/s){
		// summary:
		//		Register the specified shape into the graphics registry.
		// s: dojox/gfx/shape.Shape
		//		The shape to register.
		// returns: Number
		//		The unique id associated with this shape.
		
		// the id pattern : type+number (ex: Rect0,Rect1,etc)
		var t = s.declaredClass.split('.').pop();
		var i = t in _ids ? ++_ids[t] : ((_ids[t] = 0));
		var uid = t+i;
		registry[uid] = s;
		return uid;
	};
	
	shape.byId = function(/*String*/id){
		// summary:
		//		Returns the shape that matches the specified id.
		// id: String
		//		The unique identifier for this Shape.
		// returns: dojox/gfx/shape.Shape
		return registry[id]; //dojox/gfx/shape.Shape
	};

	shape.dispose = function(/*dojox/gfx/shape.Shape*/s, /*Boolean?*/recurse){
		// summary:
		//		Removes the specified shape from the registry.
		// s: dojox/gfx/shape.Shape
		//		The shape to unregister.
		if(recurse && s.children){
			for(var i=0; i< s.children.length; ++i){
				shape.dispose(s.children[i], true);
			}
		}
		delete registry[s.getUID()];
		++disposeCount;
		if(fixIELeak && disposeCount>10000){
			registry = repack(registry);
			disposeCount = 0;
		}
	};
	
	shape.Shape = declare("dojox.gfx.shape.Shape", null, {
		// summary:
		//		a Shape object, which knows how to apply
		//		graphical attributes and transformations
	
		constructor: function(){
			// rawNode: Node
			//		underlying graphics-renderer-specific implementation object (if applicable)
			this.rawNode = null;

			// shape: Object
			//		an abstract shape object
			//		(see dojox/gfx.defaultPath,
			//		dojox/gfx.defaultPolyline,
			//		dojox/gfx.defaultRect,
			//		dojox/gfx.defaultEllipse,
			//		dojox/gfx.defaultCircle,
			//		dojox/gfx.defaultLine,
			//		or dojox/gfx.defaultImage)
			this.shape = null;
	
			// matrix: dojox/gfx/matrix.Matrix2D
			//		a transformation matrix
			this.matrix = null;
	
			// fillStyle: dojox/gfx.Fill
			//		a fill object
			//		(see dojox/gfx.defaultLinearGradient,
			//		dojox/gfx.defaultRadialGradient,
			//		dojox/gfx.defaultPattern,
			//		or dojo/Color)
			this.fillStyle = null;
	
			// strokeStyle: dojox/gfx.Stroke
			//		a stroke object
			//		(see dojox/gfx.defaultStroke)
			this.strokeStyle = null;
	
			// bbox: dojox/gfx.Rectangle
			//		a bounding box of this shape
			//		(see dojox/gfx.defaultRect)
			this.bbox = null;
	
			// virtual group structure
	
			// parent: Object
			//		a parent or null
			//		(see dojox/gfx/shape.Surface,
			//		or dojox/gfx.Group)
			this.parent = null;
	
			// parentMatrix: dojox/gfx/matrix.Matrix2D
			//		a transformation matrix inherited from the parent
			this.parentMatrix = null;
			
			var uid = shape.register(this);
			this.getUID = function(){
				return uid;
			}
		},
		
		destroy: function(){
			// summary:
			//		Releases all internal resources owned by this shape. Once this method has been called,
			//		the instance is considered destroyed and should not be used anymore.
			shape.dispose(this);
		},
	
		// trivial getters
	
		getNode: function(){
			// summary:
			//		Different graphics rendering subsystems implement shapes in different ways.  This
			//		method provides access to the underlying graphics subsystem object.  Clients calling this
			//		method and using the return value must be careful not to try sharing or using the underlying node
			//		in a general way across renderer implementation.
			//		Returns the underlying graphics Node, or null if no underlying graphics node is used by this shape.
			return this.rawNode; // Node
		},
		getShape: function(){
			// summary:
			//		returns the current Shape object or null
			//		(see dojox/gfx.defaultPath,
			//		dojox/gfx.defaultPolyline,
			//		dojox/gfx.defaultRect,
			//		dojox/gfx.defaultEllipse,
			//		dojox/gfx.defaultCircle,
			//		dojox/gfx.defaultLine,
			//		or dojox/gfx.defaultImage)
			return this.shape; // Object
		},
		getTransform: function(){
			// summary:
			//		Returns the current transformation matrix applied to this Shape or null
			return this.matrix;	// dojox/gfx/matrix.Matrix2D
		},
		getFill: function(){
			// summary:
			//		Returns the current fill object or null
			//		(see dojox/gfx.defaultLinearGradient,
			//		dojox/gfx.defaultRadialGradient,
			//		dojox/gfx.defaultPattern,
			//		or dojo/Color)
			return this.fillStyle;	// Object
		},
		getStroke: function(){
			// summary:
			//		Returns the current stroke object or null
			//		(see dojox/gfx.defaultStroke)
			return this.strokeStyle;	// Object
		},
		getParent: function(){
			// summary:
			//		Returns the parent Shape, Group or null if this Shape is unparented.
			//		(see dojox/gfx/shape.Surface,
			//		or dojox/gfx.Group)
			return this.parent;	// Object
		},
		getBoundingBox: function(){
			// summary:
			//		Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be
			//		calculated for the shape on the current renderer or for shapes with no geometric area (points).
			//		A bounding box is a rectangular geometric region
			//		defining the X and Y extent of the shape.
			//		(see dojox/gfx.defaultRect)
			//		Note that this method returns a direct reference to the attribute of this instance. Therefore you should
			//		not modify its value directly but clone it instead.
			return this.bbox;	// dojox/gfx.Rectangle
		},
		getTransformedBoundingBox: function(){
			// summary:
			//		returns an array of four points or null
			//		four points represent four corners of the untransformed bounding box
			var b = this.getBoundingBox();
			if(!b){
				return null;	// null
			}
			var m = this._getRealMatrix(),
				gm = matrixLib;
			return [	// Array
					gm.multiplyPoint(m, b.x, b.y),
					gm.multiplyPoint(m, b.x + b.width, b.y),
					gm.multiplyPoint(m, b.x + b.width, b.y + b.height),
					gm.multiplyPoint(m, b.x, b.y + b.height)
				];
		},
		getEventSource: function(){
			// summary:
			//		returns a Node, which is used as
			//		a source of events for this shape
			
			// COULD BE RE-IMPLEMENTED BY THE RENDERER!
			return this.rawNode;	// Node
		},
	
		// empty settings
		
		setClip: function(clip){
			// summary:
			//		sets the clipping area of this shape.
			// description:
			//		The clipping area defines the shape area that will be effectively visible. Everything that
			//		would be drawn outside of the clipping area will not be rendered.
			//		The possible clipping area types are rectangle, ellipse, polyline and path, but all are not
			//		supported by all the renderers. vml only supports rectangle clipping, while the gfx silverlight renderer does not
			//		support path clipping.
			//		The clip parameter defines the clipping area geometry, and should be an object with the following properties:
			//
			//		- {x:Number, y:Number, width:Number, height:Number} for rectangular clip
			//		- {cx:Number, cy:Number, rx:Number, ry:Number} for ellipse clip
			//		- {points:Array} for polyline clip
			//		- {d:String} for a path clip.
			//
			//		The clip geometry coordinates are expressed in the coordinate system used to draw the shape. In other
			//		words, the clipping area is defined in the shape parent coordinate system and the shape transform is automatically applied.
			// example:
			//		The following example shows how to clip a gfx image with all the possible clip geometry: a rectangle,
			//		an ellipse, a circle (using the ellipse geometry), a polyline and a path:
			//
			//	|	surface.createImage({src:img, width:200,height:200}).setClip({x:10,y:10,width:50,height:50});
			//	|	surface.createImage({src:img, x:100,y:50,width:200,height:200}).setClip({cx:200,cy:100,rx:20,ry:30});
			//	|	surface.createImage({src:img, x:0,y:350,width:200,height:200}).setClip({cx:100,cy:425,rx:60,ry:60});
			//	|	surface.createImage({src:img, x:300,y:0,width:200,height:200}).setClip({points:[350,0,450,50,380,130,300,110]});
			//	|	surface.createImage({src:img, x:300,y:350,width:200,height:200}).setClip({d:"M 350,350 C314,414 317,557 373,450.0000 z"});

			// clip: Object
			//		an object that defines the clipping geometry, or null to remove clip.
			
			// COULD BE RE-IMPLEMENTED BY THE RENDERER!
			this.clip = clip;
		},
		
		getClip: function(){
			return this.clip;
		},
	
		setShape: function(shape){
			// summary:
			//		sets a shape object
			//		(the default implementation simply ignores it)
			// shape: Object
			//		a shape object
			//		(see dojox/gfx.defaultPath,
			//		dojox/gfx.defaultPolyline,
			//		dojox/gfx.defaultRect,
			//		dojox/gfx.defaultEllipse,
			//		dojox/gfx.defaultCircle,
			//		dojox/gfx.defaultLine,
			//		or dojox/gfx.defaultImage)
			
			// COULD BE RE-IMPLEMENTED BY THE RENDERER!
			this.shape = g.makeParameters(this.shape, shape);
			this.bbox = null;
			return this;	// self
		},
		setFill: function(fill){
			// summary:
			//		sets a fill object
			//		(the default implementation simply ignores it)
			// fill: Object
			//		a fill object
			//		(see dojox/gfx.defaultLinearGradient,
			//		dojox/gfx.defaultRadialGradient,
			//		dojox/gfx.defaultPattern,
			//		or dojo/_base/Color)
			
			// COULD BE RE-IMPLEMENTED BY THE RENDERER!
			if(!fill){
				// don't fill
				this.fillStyle = null;
				return this;	// self
			}
			var f = null;
			if(typeof(fill) == "object" && "type" in fill){
				// gradient or pattern
				switch(fill.type){
					case "linear":
						f = g.makeParameters(g.defaultLinearGradient, fill);
						break;
					case "radial":
						f = g.makeParameters(g.defaultRadialGradient, fill);
						break;
					case "pattern":
						f = g.makeParameters(g.defaultPattern, fill);
						break;
				}
			}else{
				// color object
				f = g.normalizeColor(fill);
			}
			this.fillStyle = f;
			return this;	// self
		},
		setStroke: function(stroke){
			// summary:
			//		sets a stroke object
			//		(the default implementation simply ignores it)
			// stroke: Object
			//		a stroke object
			//		(see dojox/gfx.defaultStroke)
			
			// COULD BE RE-IMPLEMENTED BY THE RENDERER!
			if(!stroke){
				// don't stroke
				this.strokeStyle = null;
				return this;	// self
			}
			// normalize the stroke
			if(typeof stroke == "string" || lang.isArray(stroke) || stroke instanceof Color){
				stroke = {color: stroke};
			}
			var s = this.strokeStyle = g.makeParameters(g.defaultStroke, stroke);
			s.color = g.normalizeColor(s.color);
			return this;	// self
		},
		setTransform: function(matrix){
			// summary:
			//		sets a transformation matrix
			// matrix: dojox/gfx/matrix.Matrix2D
			//		a matrix or a matrix-like object
			//		(see an argument of dojox/gfx/matrix.Matrix2D
			//		constructor for a list of acceptable arguments)
			
			// COULD BE RE-IMPLEMENTED BY THE RENDERER!
			this.matrix = matrixLib.clone(matrix ? matrixLib.normalize(matrix) : matrixLib.identity);
			return this._applyTransform();	// self
		},
	
		_applyTransform: function(){
			// summary:
			//		physically sets a matrix
			
			// COULD BE RE-IMPLEMENTED BY THE RENDERER!
			return this;	// self
		},
	
		// z-index
	
		moveToFront: function(){
			// summary:
			//		moves a shape to front of its parent's list of shapes
			var p = this.getParent();
			if(p){
				p._moveChildToFront(this);
				this._moveToFront();	// execute renderer-specific action
			}
			return this;	// self
		},
		moveToBack: function(){
			// summary:
			//		moves a shape to back of its parent's list of shapes
			var p = this.getParent();
			if(p){
				p._moveChildToBack(this);
				this._moveToBack();	// execute renderer-specific action
			}
			return this;
		},
		_moveToFront: function(){
			// summary:
			//		renderer-specific hook, see dojox/gfx/shape.Shape.moveToFront()
			
			// COULD BE RE-IMPLEMENTED BY THE RENDERER!
		},
		_moveToBack: function(){
			// summary:
			//		renderer-specific hook, see dojox/gfx/shape.Shape.moveToFront()
			
			// COULD BE RE-IMPLEMENTED BY THE RENDERER!
		},
	
		// apply left & right transformation
	
		applyRightTransform: function(matrix){
			// summary:
			//		multiplies the existing matrix with an argument on right side
			//		(this.matrix * matrix)
			// matrix: dojox/gfx/matrix.Matrix2D
			//		a matrix or a matrix-like object
			//		(see an argument of dojox/gfx/matrix.Matrix2D
			//		constructor for a list of acceptable arguments)
			return matrix ? this.setTransform([this.matrix, matrix]) : this;	// self
		},
		applyLeftTransform: function(matrix){
			// summary:
			//		multiplies the existing matrix with an argument on left side
			//		(matrix * this.matrix)
			// matrix: dojox/gfx/matrix.Matrix2D
			//		a matrix or a matrix-like object
			//		(see an argument of dojox/gfx/matrix.Matrix2D
			//		constructor for a list of acceptable arguments)
			return matrix ? this.setTransform([matrix, this.matrix]) : this;	// self
		},
		applyTransform: function(matrix){
			// summary:
			//		a shortcut for dojox/gfx/shape.Shape.applyRightTransform
			// matrix: dojox/gfx/matrix.Matrix2D
			//		a matrix or a matrix-like object
			//		(see an argument of dojox/gfx/matrix.Matrix2D
			//		constructor for a list of acceptable arguments)
			return matrix ? this.setTransform([this.matrix, matrix]) : this;	// self
		},
	
		// virtual group methods
	
		removeShape: function(silently){
			// summary:
			//		removes the shape from its parent's list of shapes
			// silently: Boolean
			//		if true, do not redraw a picture yet
			if(this.parent){
				this.parent.remove(this, silently);
			}
			return this;	// self
		},
		_setParent: function(parent, matrix){
			// summary:
			//		sets a parent
			// parent: Object
			//		a parent or null
			//		(see dojox/gfx/shape.Surface,
			//		or dojox/gfx.Group)
			// matrix: dojox/gfx/matrix.Matrix2D
			//		a 2D matrix or a matrix-like object
			this.parent = parent;
			return this._updateParentMatrix(matrix);	// self
		},
		_updateParentMatrix: function(matrix){
			// summary:
			//		updates the parent matrix with new matrix
			// matrix: dojox/gfx/Matrix2D
			//		a 2D matrix or a matrix-like object
			this.parentMatrix = matrix ? matrixLib.clone(matrix) : null;
			return this._applyTransform();	// self
		},
		_getRealMatrix: function(){
			// summary:
			//		returns the cumulative ('real') transformation matrix
			//		by combining the shape's matrix with its parent's matrix
			var m = this.matrix;
			var p = this.parent;
			while(p){
				if(p.matrix){
					m = matrixLib.multiply(p.matrix, m);
				}
				p = p.parent;
			}
			return m;	// dojox/gfx/matrix.Matrix2D
		}
	});
	
	shape._eventsProcessing = {
		connect: function(name, object, method){
			// summary:
			//		connects a handler to an event on this shape
			
			// COULD BE RE-IMPLEMENTED BY THE RENDERER!
			// redirect to fixCallback to normalize events and add the gfxTarget to the event. The latter
			// is done by dojox/gfx.fixTarget which is defined by each renderer
			return events.connect(this.getEventSource(), name, shape.fixCallback(this, g.fixTarget, object, method));
			
		},
		disconnect: function(token){
			// summary:
			//		connects a handler by token from an event on this shape
			
			// COULD BE RE-IMPLEMENTED BY THE RENDERER!
	
			events.disconnect(token);
		}
	};
	
	shape.fixCallback = function(gfxElement, fixFunction, scope, method){
		// summary:
		//		Wraps the callback to allow for tests and event normalization
		//		before it gets invoked. This is where 'fixTarget' is invoked.
		// tags:
		//      private
		// gfxElement: Object
		//		The GFX object that triggers the action (ex.:
		//		dojox/gfx.Surface and dojox/gfx/shape.Shape). A new event property
		//		'gfxTarget' is added to the event to reference this object.
		//		for easy manipulation of GFX objects by the event handlers.
		// fixFunction: Function
		//		The function that implements the logic to set the 'gfxTarget'
		//		property to the event. It should be 'dojox/gfx.fixTarget' for
		//		most of the cases
		// scope: Object
		//		Optional. The scope to be used when invoking 'method'. If
		//		omitted, a global scope is used.
		// method: Function|String
		//		The original callback to be invoked.
		if(!method){
			method = scope;
			scope = null;
		}
		if(lang.isString(method)){
			scope = scope || kernel.global;
			if(!scope[method]){ throw(['dojox.gfx.shape.fixCallback: scope["', method, '"] is null (scope="', scope, '")'].join('')); }
			return function(e){  
				return fixFunction(e,gfxElement) ? scope[method].apply(scope, arguments || []) : undefined; }; // Function
		}
		return !scope 
			? function(e){ 
				return fixFunction(e,gfxElement) ? method.apply(scope, arguments) : undefined; } 
			: function(e){ 
				return fixFunction(e,gfxElement) ? method.apply(scope, arguments || []) : undefined; }; // Function
	};
	lang.extend(shape.Shape, shape._eventsProcessing);
	
	shape.Container = {
		// summary:
		//		a container of shapes, which can be used
		//		as a foundation for renderer-specific groups, or as a way
		//		to logically group shapes (e.g, to propagate matricies)
	
		_init: function() {
			// children: Array
			//		a list of children
			this.children = [];
		},
	
		// group management
	
		openBatch: function() {
			// summary:
			//		starts a new batch, subsequent new child shapes will be held in
			//		the batch instead of appending to the container directly
		},
		closeBatch: function() {
			// summary:
			//		submits the current batch, append all pending child shapes to DOM
		},
		add: function(shape){
			// summary:
			//		adds a shape to the list
			// shape: dojox/gfx/shape.Shape
			//		the shape to add to the list
			var oldParent = shape.getParent();
			if(oldParent){
				oldParent.remove(shape, true);
			}
			this.children.push(shape);
			return shape._setParent(this, this._getRealMatrix());	// self
		},
		remove: function(shape, silently){
			// summary:
			//		removes a shape from the list
			// shape: dojox/gfx/shape.Shape
			//		the shape to remove
			// silently: Boolean
			//		if true, do not redraw a picture yet
			for(var i = 0; i < this.children.length; ++i){
				if(this.children[i] == shape){
					if(silently){
						// skip for now
					}else{
						shape.parent = null;
						shape.parentMatrix = null;
					}
					this.children.splice(i, 1);
					break;
				}
			}
			return this;	// self
		},
		clear: function(/*Boolean?*/ destroy){
			// summary:
			//		removes all shapes from a group/surface.
			// destroy: Boolean
			//		Indicates whether the children should be destroyed. Optional.
			var shape;
			for(var i = 0; i < this.children.length;++i){
				shape = this.children[i];
				shape.parent = null;
				shape.parentMatrix = null;
				if(destroy){
					shape.destroy();
				}
			}
			this.children = [];
			return this;	// self
		},
		getBoundingBox: function(){
			// summary:
			//		Returns the bounding box Rectangle for this shape.
			if(this.children){
				// if this is a composite shape, then sum up all the children
				var result = null;
				arr.forEach(this.children, function(shape){
					var bb = shape.getBoundingBox();
					if(bb){
						var ct = shape.getTransform();
						if(ct){
							bb = matrixLib.multiplyRectangle(ct, bb);
						}
						if(result){
							// merge two bbox 
							result.x = Math.min(result.x, bb.x);
							result.y = Math.min(result.y, bb.y);
							result.endX = Math.max(result.endX, bb.x + bb.width);
							result.endY = Math.max(result.endY, bb.y + bb.height);
						}else{
							// first bbox 
							result = {
								x: bb.x,
								y: bb.y,
								endX: bb.x + bb.width,
								endY: bb.y + bb.height
							};
						}
					}
				});
				if(result){
					result.width = result.endX - result.x;
					result.height = result.endY - result.y;
				}
				return result; // dojox/gfx.Rectangle
			}
			// unknown/empty bounding box, subclass shall override this impl 
			return null;
		},
		// moving child nodes
		_moveChildToFront: function(shape){
			// summary:
			//		moves a shape to front of the list of shapes
			// shape: dojox/gfx/shape.Shape
			//		one of the child shapes to move to the front
			for(var i = 0; i < this.children.length; ++i){
				if(this.children[i] == shape){
					this.children.splice(i, 1);
					this.children.push(shape);
					break;
				}
			}
			return this;	// self
		},
		_moveChildToBack: function(shape){
			// summary:
			//		moves a shape to back of the list of shapes
			// shape: dojox/gfx/shape.Shape
			//		one of the child shapes to move to the front
			for(var i = 0; i < this.children.length; ++i){
				if(this.children[i] == shape){
					this.children.splice(i, 1);
					this.children.unshift(shape);
					break;
				}
			}
			return this;	// self
		}
	};

	shape.Surface = declare("dojox.gfx.shape.Surface", null, {
		// summary:
		//		a surface object to be used for drawings
		constructor: function(){
			// underlying node
			this.rawNode = null;
			// the parent node
			this._parent = null;
			// the list of DOM nodes to be deleted in the case of destruction
			this._nodes = [];
			// the list of events to be detached in the case of destruction
			this._events = [];
		},
		destroy: function(){
			// summary:
			//		destroy all relevant external resources and release all
			//		external references to make this object garbage-collectible
			arr.forEach(this._nodes, domConstruct.destroy);
			this._nodes = [];
			arr.forEach(this._events, events.disconnect);
			this._events = [];
			this.rawNode = null;	// recycle it in _nodes, if it needs to be recycled
			if(has("ie")){
				while(this._parent.lastChild){
					domConstruct.destroy(this._parent.lastChild);
				}
			}else{
				this._parent.innerHTML = "";
			}
			this._parent = null;
		},
		getEventSource: function(){
			// summary:
			//		returns a node, which can be used to attach event listeners
			return this.rawNode; // Node
		},
		_getRealMatrix: function(){
			// summary:
			//		always returns the identity matrix
			return null;	// dojox/gfx/Matrix2D
		},
		/*=====
		 setDimensions: function(width, height){
			 // summary:
			 //		sets the width and height of the rawNode
			 // width: String
			 //		width of surface, e.g., "100px"
			 // height: String
			 //		height of surface, e.g., "100px"
			 return this;	// self
		 },
		 getDimensions: function(){
			 // summary:
			 //     gets current width and height in pixels
			 // returns: Object
			 //     object with properties "width" and "height"
		 },
		 =====*/
		isLoaded: true,
		onLoad: function(/*dojox/gfx/shape.Surface*/ surface){
			// summary:
			//		local event, fired once when the surface is created
			//		asynchronously, used only when isLoaded is false, required
			//		only for Silverlight.
		},
		whenLoaded: function(/*Object|Null*/ context, /*Function|String*/ method){
			var f = lang.hitch(context, method);
			if(this.isLoaded){
				f(this);
			}else{
				var h = events.connect(this, "onLoad", function(surface){
					events.disconnect(h);
					f(surface);
				});
			}
		}
	});
	lang.extend(shape.Surface, shape._eventsProcessing);

	/*=====
	g.Point = declare("dojox/gfx.Point", null, {
		// summary:
		//		2D point for drawings - {x, y}
		// description:
		//		Do not use this object directly!
		//		Use the naked object instead: {x: 1, y: 2}.
	});

	g.Rectangle = declare("dojox.gfx.Rectangle", null, {
		// summary:
		//		rectangle - {x, y, width, height}
		// description:
		//		Do not use this object directly!
		//		Use the naked object instead: {x: 1, y: 2, width: 100, height: 200}.
	});
	 =====*/


	shape.Rect = declare("dojox.gfx.shape.Rect", shape.Shape, {
		// summary:
		//		a generic rectangle
		constructor: function(rawNode){
			// rawNode: Node
			//		The underlying graphics system object (typically a DOM Node)
			this.shape = g.getDefault("Rect");
			this.rawNode = rawNode;
		},
		getBoundingBox: function(){
			// summary:
			//		returns the bounding box (its shape in this case)
			return this.shape;	// dojox/gfx.Rectangle
		}
	});
	
	shape.Ellipse = declare("dojox.gfx.shape.Ellipse", shape.Shape, {
		// summary:
		//		a generic ellipse
		constructor: function(rawNode){
			// rawNode: Node
			//		a DOM Node
			this.shape = g.getDefault("Ellipse");
			this.rawNode = rawNode;
		},
		getBoundingBox: function(){
			// summary:
			//		returns the bounding box
			if(!this.bbox){
				var shape = this.shape;
				this.bbox = {x: shape.cx - shape.rx, y: shape.cy - shape.ry,
					width: 2 * shape.rx, height: 2 * shape.ry};
			}
			return this.bbox;	// dojox/gfx.Rectangle
		}
	});
	
	shape.Circle = declare("dojox.gfx.shape.Circle", shape.Shape, {
		// summary:
		//		a generic circle
		constructor: function(rawNode){
			// rawNode: Node
			//		a DOM Node
			this.shape = g.getDefault("Circle");
			this.rawNode = rawNode;
		},
		getBoundingBox: function(){
			// summary:
			//		returns the bounding box
			if(!this.bbox){
				var shape = this.shape;
				this.bbox = {x: shape.cx - shape.r, y: shape.cy - shape.r,
					width: 2 * shape.r, height: 2 * shape.r};
			}
			return this.bbox;	// dojox/gfx.Rectangle
		}
	});
	
	shape.Line = declare("dojox.gfx.shape.Line", shape.Shape, {
		// summary:
		//		a generic line (do not instantiate it directly)
		constructor: function(rawNode){
			// rawNode: Node
			//		a DOM Node
			this.shape = g.getDefault("Line");
			this.rawNode = rawNode;
		},
		getBoundingBox: function(){
			// summary:
			//		returns the bounding box
			if(!this.bbox){
				var shape = this.shape;
				this.bbox = {
					x:		Math.min(shape.x1, shape.x2),
					y:		Math.min(shape.y1, shape.y2),
					width:	Math.abs(shape.x2 - shape.x1),
					height:	Math.abs(shape.y2 - shape.y1)
				};
			}
			return this.bbox;	// dojox/gfx.Rectangle
		}
	});
	
	shape.Polyline = declare("dojox.gfx.shape.Polyline", shape.Shape, {
		// summary:
		//		a generic polyline/polygon (do not instantiate it directly)
		constructor: function(rawNode){
			// rawNode: Node
			//		a DOM Node
			this.shape = g.getDefault("Polyline");
			this.rawNode = rawNode;
		},
		setShape: function(points, closed){
			// summary:
			//		sets a polyline/polygon shape object
			// points: Object|Array
			//		a polyline/polygon shape object, or an array of points
			// closed: Boolean
			//		close the polyline to make a polygon
			if(points && points instanceof Array){
				this.inherited(arguments, [{points: points}]);
				if(closed && this.shape.points.length){
					this.shape.points.push(this.shape.points[0]);
				}
			}else{
				this.inherited(arguments, [points]);
			}
			return this;	// self
		},
		_normalizePoints: function(){
			// summary:
			//		normalize points to array of {x:number, y:number}
			var p = this.shape.points, l = p && p.length;
			if(l && typeof p[0] == "number"){
				var points = [];
				for(var i = 0; i < l; i += 2){
					points.push({x: p[i], y: p[i + 1]});
				}
				this.shape.points = points;
			}
		},
		getBoundingBox: function(){
			// summary:
			//		returns the bounding box
			if(!this.bbox && this.shape.points.length){
				var p = this.shape.points;
				var l = p.length;
				var t = p[0];
				var bbox = {l: t.x, t: t.y, r: t.x, b: t.y};
				for(var i = 1; i < l; ++i){
					t = p[i];
					if(bbox.l > t.x) bbox.l = t.x;
					if(bbox.r < t.x) bbox.r = t.x;
					if(bbox.t > t.y) bbox.t = t.y;
					if(bbox.b < t.y) bbox.b = t.y;
				}
				this.bbox = {
					x:		bbox.l,
					y:		bbox.t,
					width:	bbox.r - bbox.l,
					height:	bbox.b - bbox.t
				};
			}
			return this.bbox;	// dojox/gfx.Rectangle
		}
	});
	
	shape.Image = declare("dojox.gfx.shape.Image", shape.Shape, {
		// summary:
		//		a generic image (do not instantiate it directly)
		constructor: function(rawNode){
			// rawNode: Node
			//		a DOM Node
			this.shape = g.getDefault("Image");
			this.rawNode = rawNode;
		},
		getBoundingBox: function(){
			// summary:
			//		returns the bounding box (its shape in this case)
			return this.shape;	// dojox/gfx.Rectangle
		},
		setStroke: function(){
			// summary:
			//		ignore setting a stroke style
			return this;	// self
		},
		setFill: function(){
			// summary:
			//		ignore setting a fill style
			return this;	// self
		}
	});
	
	shape.Text = declare(shape.Shape, {
		// summary:
		//		a generic text (do not instantiate it directly)
		constructor: function(rawNode){
			// rawNode: Node
			//		a DOM Node
			this.fontStyle = null;
			this.shape = g.getDefault("Text");
			this.rawNode = rawNode;
		},
		getFont: function(){
			// summary:
			//		returns the current font object or null
			return this.fontStyle;	// Object
		},
		setFont: function(newFont){
			// summary:
			//		sets a font for text
			// newFont: Object
			//		a font object (see dojox/gfx.defaultFont) or a font string
			this.fontStyle = typeof newFont == "string" ? g.splitFontString(newFont) :
				g.makeParameters(g.defaultFont, newFont);
			this._setFont();
			return this;	// self
		}
	});
	
	shape.Creator = {
		// summary:
		//		shape creators
		createShape: function(shape){
			// summary:
			//		creates a shape object based on its type; it is meant to be used
			//		by group-like objects
			// shape: Object
			//		a shape descriptor object
			// returns: dojox/gfx/shape.Shape | Null
			//      a fully instantiated surface-specific Shape object
			switch(shape.type){
				case g.defaultPath.type:		return this.createPath(shape);
				case g.defaultRect.type:		return this.createRect(shape);
				case g.defaultCircle.type:	    return this.createCircle(shape);
				case g.defaultEllipse.type:	    return this.createEllipse(shape);
				case g.defaultLine.type:		return this.createLine(shape);
				case g.defaultPolyline.type:	return this.createPolyline(shape);
				case g.defaultImage.type:		return this.createImage(shape);
				case g.defaultText.type:		return this.createText(shape);
				case g.defaultTextPath.type:	return this.createTextPath(shape);
			}
			return null;
		},
		createGroup: function(){
			// summary:
			//		creates a group shape
			return this.createObject(g.Group);	// dojox/gfx/Group
		},
		createRect: function(rect){
			// summary:
			//		creates a rectangle shape
			// rect: Object
			//		a path object (see dojox/gfx.defaultRect)
			return this.createObject(g.Rect, rect);	// dojox/gfx/shape.Rect
		},
		createEllipse: function(ellipse){
			// summary:
			//		creates an ellipse shape
			// ellipse: Object
			//		an ellipse object (see dojox/gfx.defaultEllipse)
			return this.createObject(g.Ellipse, ellipse);	// dojox/gfx/shape.Ellipse
		},
		createCircle: function(circle){
			// summary:
			//		creates a circle shape
			// circle: Object
			//		a circle object (see dojox/gfx.defaultCircle)
			return this.createObject(g.Circle, circle);	// dojox/gfx/shape.Circle
		},
		createLine: function(line){
			// summary:
			//		creates a line shape
			// line: Object
			//		a line object (see dojox/gfx.defaultLine)
			return this.createObject(g.Line, line);	// dojox/gfx/shape.Line
		},
		createPolyline: function(points){
			// summary:
			//		creates a polyline/polygon shape
			// points: Object
			//		a points object (see dojox/gfx.defaultPolyline)
			//		or an Array of points
			return this.createObject(g.Polyline, points);	// dojox/gfx/shape.Polyline
		},
		createImage: function(image){
			// summary:
			//		creates a image shape
			// image: Object
			//		an image object (see dojox/gfx.defaultImage)
			return this.createObject(g.Image, image);	// dojox/gfx/shape.Image
		},
		createText: function(text){
			// summary:
			//		creates a text shape
			// text: Object
			//		a text object (see dojox/gfx.defaultText)
			return this.createObject(g.Text, text);	// dojox/gfx/shape.Text
		},
		createPath: function(path){
			// summary:
			//		creates a path shape
			// path: Object
			//		a path object (see dojox/gfx.defaultPath)
			return this.createObject(g.Path, path);	// dojox/gfx/shape.Path
		},
		createTextPath: function(text){
			// summary:
			//		creates a text shape
			// text: Object
			//		a textpath object (see dojox/gfx.defaultTextPath)
			return this.createObject(g.TextPath, {}).setText(text);	// dojox/gfx/shape.TextPath
		},
		createObject: function(shapeType, rawShape){
			// summary:
			//		creates an instance of the passed shapeType class
			// shapeType: Function
			//		a class constructor to create an instance of
			// rawShape: Object 
			//		properties to be passed in to the classes 'setShape' method
	
			// SHOULD BE RE-IMPLEMENTED BY THE RENDERER!
			return null;	// dojox/gfx/shape.Shape
		}
	};
	
	/*=====
	 lang.extend(shape.Surface, shape.Container);
	 lang.extend(shape.Surface, shape.Creator);

	 g.Group = declare(shape.Shape, {
		// summary:
		//		a group shape, which can be used
		//		to logically group shapes (e.g, to propagate matricies)
	});
	lang.extend(g.Group, shape.Container);
	lang.extend(g.Group, shape.Creator);

	g.Rect     = shape.Rect;
	g.Circle   = shape.Circle;
	g.Ellipse  = shape.Ellipse;
	g.Line     = shape.Line;
	g.Polyline = shape.Polyline;
	g.Path     = shape.Path;
	g.Text     = shape.Text;
	g.Surface  = shape.Surface;

	g.Path = g.path.Path;
	g.TextPath = g.path.TextPath;
	=====*/

	return shape;
});

},
'dojox/gfx/matrix':function(){
define("dojox/gfx/matrix", ["./_base","dojo/_base/lang"], 
  function(g, lang){
	var m = g.matrix = {};

	// candidates for dojox.math:
	var _degToRadCache = {};
	m._degToRad = function(degree){
		return _degToRadCache[degree] || (_degToRadCache[degree] = (Math.PI * degree / 180));
	};
	m._radToDeg = function(radian){ return radian / Math.PI * 180; };

	m.Matrix2D = function(arg){
		// summary:
		//		a 2D matrix object
		// description:
		//		Normalizes a 2D matrix-like object. If arrays is passed,
		//		all objects of the array are normalized and multiplied sequentially.
		// arg: Object
		//		a 2D matrix-like object, a number, or an array of such objects
		if(arg){
			if(typeof arg == "number"){
				this.xx = this.yy = arg;
			}else if(arg instanceof Array){
				if(arg.length > 0){
					var matrix = m.normalize(arg[0]);
					// combine matrices
					for(var i = 1; i < arg.length; ++i){
						var l = matrix, r = m.normalize(arg[i]);
						matrix = new m.Matrix2D();
						matrix.xx = l.xx * r.xx + l.xy * r.yx;
						matrix.xy = l.xx * r.xy + l.xy * r.yy;
						matrix.yx = l.yx * r.xx + l.yy * r.yx;
						matrix.yy = l.yx * r.xy + l.yy * r.yy;
						matrix.dx = l.xx * r.dx + l.xy * r.dy + l.dx;
						matrix.dy = l.yx * r.dx + l.yy * r.dy + l.dy;
					}
					lang.mixin(this, matrix);
				}
			}else{
				lang.mixin(this, arg);
			}
		}
	};

	// the default (identity) matrix, which is used to fill in missing values
	lang.extend(m.Matrix2D, {xx: 1, xy: 0, yx: 0, yy: 1, dx: 0, dy: 0});

	lang.mixin(m, {
		// summary:
		//		class constants, and methods of dojox/gfx/matrix

		// matrix constants

		// identity: dojox/gfx/matrix.Matrix2D
		//		an identity matrix constant: identity * (x, y) == (x, y)
		identity: new m.Matrix2D(),

		// flipX: dojox/gfx/matrix.Matrix2D
		//		a matrix, which reflects points at x = 0 line: flipX * (x, y) == (-x, y)
		flipX:    new m.Matrix2D({xx: -1}),

		// flipY: dojox/gfx/matrix.Matrix2D
		//		a matrix, which reflects points at y = 0 line: flipY * (x, y) == (x, -y)
		flipY:    new m.Matrix2D({yy: -1}),

		// flipXY: dojox/gfx/matrix.Matrix2D
		//		a matrix, which reflects points at the origin of coordinates: flipXY * (x, y) == (-x, -y)
		flipXY:   new m.Matrix2D({xx: -1, yy: -1}),

		// matrix creators

		translate: function(a, b){
			// summary:
			//		forms a translation matrix
			// description:
			//		The resulting matrix is used to translate (move) points by specified offsets.
			// a: Number|dojox/gfx.Point
			//		an x coordinate value, or a point-like object, which specifies offsets for both dimensions
			// b: Number?
			//		a y coordinate value
			// returns: dojox/gfx/matrix.Matrix2D
			if(arguments.length > 1){
				return new m.Matrix2D({dx: a, dy: b}); // dojox/gfx/matrix.Matrix2D
			}
			// branch
			return new m.Matrix2D({dx: a.x, dy: a.y}); // dojox/gfx/matrix.Matrix2D
		},
		scale: function(a, b){
			// summary:
			//		forms a scaling matrix
			// description:
			//		The resulting matrix is used to scale (magnify) points by specified offsets.
			// a: Number|dojox/gfx.Point
			//		a scaling factor used for the x coordinate, or
			//		a uniform scaling factor used for the both coordinates, or
			//		a point-like object, which specifies scale factors for both dimensions
			// b: Number?
			//		a scaling factor used for the y coordinate
			// returns: dojox/gfx/matrix.Matrix2D
			if(arguments.length > 1){
				return new m.Matrix2D({xx: a, yy: b}); // dojox/gfx/matrix.Matrix2D
			}
			if(typeof a == "number"){
				return new m.Matrix2D({xx: a, yy: a}); // dojox/gfx/matrix.Matrix2D
			}
			return new m.Matrix2D({xx: a.x, yy: a.y}); // dojox/gfx/matrix.Matrix2D
		},
		rotate: function(angle){
			// summary:
			//		forms a rotating matrix
			// description:
			//		The resulting matrix is used to rotate points
			//		around the origin of coordinates (0, 0) by specified angle.
			// angle: Number
			//		an angle of rotation in radians (>0 for CW)
			// returns: dojox/gfx/matrix.Matrix2D
			var c = Math.cos(angle);
			var s = Math.sin(angle);
			return new m.Matrix2D({xx: c, xy: -s, yx: s, yy: c}); // dojox/gfx/matrix.Matrix2D
		},
		rotateg: function(degree){
			// summary:
			//		forms a rotating matrix
			// description:
			//		The resulting matrix is used to rotate points
			//		around the origin of coordinates (0, 0) by specified degree.
			//		See dojox/gfx/matrix.rotate() for comparison.
			// degree: Number
			//		an angle of rotation in degrees (>0 for CW)
			// returns: dojox/gfx/matrix.Matrix2D
			return m.rotate(m._degToRad(degree)); // dojox/gfx/matrix.Matrix2D
		},
		skewX: function(angle) {
			// summary:
			//		forms an x skewing matrix
			// description:
			//		The resulting matrix is used to skew points in the x dimension
			//		around the origin of coordinates (0, 0) by specified angle.
			// angle: Number
			//		a skewing angle in radians
			// returns: dojox/gfx/matrix.Matrix2D
			return new m.Matrix2D({xy: Math.tan(angle)}); // dojox/gfx/matrix.Matrix2D
		},
		skewXg: function(degree){
			// summary:
			//		forms an x skewing matrix
			// description:
			//		The resulting matrix is used to skew points in the x dimension
			//		around the origin of coordinates (0, 0) by specified degree.
			//		See dojox/gfx/matrix.skewX() for comparison.
			// degree: Number
			//		a skewing angle in degrees
			// returns: dojox/gfx/matrix.Matrix2D
			return m.skewX(m._degToRad(degree)); // dojox/gfx/matrix.Matrix2D
		},
		skewY: function(angle){
			// summary:
			//		forms a y skewing matrix
			// description:
			//		The resulting matrix is used to skew points in the y dimension
			//		around the origin of coordinates (0, 0) by specified angle.
			// angle: Number
			//		a skewing angle in radians
			// returns: dojox/gfx/matrix.Matrix2D
			return new m.Matrix2D({yx: Math.tan(angle)}); // dojox/gfx/matrix.Matrix2D
		},
		skewYg: function(degree){
			// summary:
			//		forms a y skewing matrix
			// description:
			//		The resulting matrix is used to skew points in the y dimension
			//		around the origin of coordinates (0, 0) by specified degree.
			//		See dojox/gfx/matrix.skewY() for comparison.
			// degree: Number
			//		a skewing angle in degrees
			// returns: dojox/gfx/matrix.Matrix2D
			return m.skewY(m._degToRad(degree)); // dojox/gfx/matrix.Matrix2D
		},
		reflect: function(a, b){
			// summary:
			//		forms a reflection matrix
			// description:
			//		The resulting matrix is used to reflect points around a vector,
			//		which goes through the origin.
			// a: dojox/gfx.Point|Number
			//		a point-like object, which specifies a vector of reflection, or an X value
			// b: Number?
			//		a Y value
			// returns: dojox/gfx/matrix.Matrix2D
			if(arguments.length == 1){
				b = a.y;
				a = a.x;
			}
			// make a unit vector
			var a2 = a * a, b2 = b * b, n2 = a2 + b2, xy = 2 * a * b / n2;
			return new m.Matrix2D({xx: 2 * a2 / n2 - 1, xy: xy, yx: xy, yy: 2 * b2 / n2 - 1}); // dojox/gfx/matrix.Matrix2D
		},
		project: function(a, b){
			// summary:
			//		forms an orthogonal projection matrix
			// description:
			//		The resulting matrix is used to project points orthogonally on a vector,
			//		which goes through the origin.
			// a: dojox/gfx.Point|Number
			//		a point-like object, which specifies a vector of projection, or
			//		an x coordinate value
			// b: Number?
			//		a y coordinate value
			// returns: dojox/gfx/matrix.Matrix2D
			if(arguments.length == 1){
				b = a.y;
				a = a.x;
			}
			// make a unit vector
			var a2 = a * a, b2 = b * b, n2 = a2 + b2, xy = a * b / n2;
			return new m.Matrix2D({xx: a2 / n2, xy: xy, yx: xy, yy: b2 / n2}); // dojox/gfx/matrix.Matrix2D
		},

		// ensure matrix 2D conformance
		normalize: function(matrix){
			// summary:
			//		converts an object to a matrix, if necessary
			// description:
			//		Converts any 2D matrix-like object or an array of
			//		such objects to a valid dojox/gfx/matrix.Matrix2D object.
			// matrix: Object
			//		an object, which is converted to a matrix, if necessary
			// returns: dojox/gfx/matrix.Matrix2D
			return (matrix instanceof m.Matrix2D) ? matrix : new m.Matrix2D(matrix); // dojox/gfx/matrix.Matrix2D
		},

		// common operations

		isIdentity: function(matrix){
			// summary:
			//		returns whether the specified matrix is the identity.
			// matrix: dojox/gfx/matrix.Matrix2D
			//		a 2D matrix object to be tested
			// returns: Boolean
			return matrix.xx == 1 && matrix.xy == 0 && matrix.yx == 0 && matrix.yy == 1 && matrix.dx == 0 && matrix.dy == 0; // Boolean
		},
		clone: function(matrix){
			// summary:
			//		creates a copy of a 2D matrix
			// matrix: dojox/gfx/matrix.Matrix2D
			//		a 2D matrix-like object to be cloned
			// returns: dojox/gfx/matrix.Matrix2D
			var obj = new m.Matrix2D();
			for(var i in matrix){
				if(typeof(matrix[i]) == "number" && typeof(obj[i]) == "number" && obj[i] != matrix[i]) obj[i] = matrix[i];
			}
			return obj; // dojox/gfx/matrix.Matrix2D
		},
		invert: function(matrix){
			// summary:
			//		inverts a 2D matrix
			// matrix: dojox/gfx/matrix.Matrix2D
			//		a 2D matrix-like object to be inverted
			// returns: dojox/gfx/matrix.Matrix2D
			var M = m.normalize(matrix),
				D = M.xx * M.yy - M.xy * M.yx;
				M = new m.Matrix2D({
					xx: M.yy/D, xy: -M.xy/D,
					yx: -M.yx/D, yy: M.xx/D,
					dx: (M.xy * M.dy - M.yy * M.dx) / D,
					dy: (M.yx * M.dx - M.xx * M.dy) / D
				});
			return M; // dojox/gfx/matrix.Matrix2D
		},
		_multiplyPoint: function(matrix, x, y){
			// summary:
			//		applies a matrix to a point
			// matrix: dojox/gfx/matrix.Matrix2D
			//		a 2D matrix object to be applied
			// x: Number
			//		an x coordinate of a point
			// y: Number
			//		a y coordinate of a point
			// returns: dojox/gfx.Point
			return {x: matrix.xx * x + matrix.xy * y + matrix.dx, y: matrix.yx * x + matrix.yy * y + matrix.dy}; // dojox/gfx.Point
		},
		multiplyPoint: function(matrix, /* Number||Point */ a, /* Number? */ b){
			// summary:
			//		applies a matrix to a point
			// matrix: dojox/gfx/matrix.Matrix2D
			//		a 2D matrix object to be applied
			// a: Number|dojox/gfx.Point
			//		an x coordinate of a point, or a point
			// b: Number?
			//		a y coordinate of a point
			// returns: dojox/gfx.Point
			var M = m.normalize(matrix);
			if(typeof a == "number" && typeof b == "number"){
				return m._multiplyPoint(M, a, b); // dojox/gfx.Point
			}
			return m._multiplyPoint(M, a.x, a.y); // dojox/gfx.Point
		},
		multiplyRectangle: function(matrix, /*Rectangle*/ rect){
			// summary:
			//		Applies a matrix to a rectangle.
			// description:
			//		The method applies the transformation on all corners of the
			//		rectangle and returns the smallest rectangle enclosing the 4 transformed
			//		points.
			// matrix: dojox/gfx/matrix.Matrix2D
			//		a 2D matrix object to be applied.
			// rect: Rectangle
			//		the rectangle to transform.
			// returns: dojox/gfx.Rectangle
			var M = m.normalize(matrix);
			rect = rect || {x:0, y:0, width:0, height:0}; 
			if(m.isIdentity(M))
				return {x: rect.x, y: rect.y, width: rect.width, height: rect.height}; // dojo/gfx.Rectangle
			var p0 = m.multiplyPoint(M, rect.x, rect.y),
				p1 = m.multiplyPoint(M, rect.x, rect.y + rect.height),
				p2 = m.multiplyPoint(M, rect.x + rect.width, rect.y),
				p3 = m.multiplyPoint(M, rect.x + rect.width, rect.y + rect.height),
				minx = Math.min(p0.x, p1.x, p2.x, p3.x),
				miny = Math.min(p0.y, p1.y, p2.y, p3.y),
				maxx = Math.max(p0.x, p1.x, p2.x, p3.x),
				maxy = Math.max(p0.y, p1.y, p2.y, p3.y);
			return{ // dojo/gfx.Rectangle
				x: minx,
				y: miny,
				width: maxx - minx,
				height: maxy - miny
			};
		},
		multiply: function(matrix){
			// summary:
			//		combines matrices by multiplying them sequentially in the given order
			// matrix: dojox/gfx/matrix.Matrix2D
			//		a 2D matrix-like object,
			//		all subsequent arguments are matrix-like objects too
			var M = m.normalize(matrix);
			// combine matrices
			for(var i = 1; i < arguments.length; ++i){
				var l = M, r = m.normalize(arguments[i]);
				M = new m.Matrix2D();
				M.xx = l.xx * r.xx + l.xy * r.yx;
				M.xy = l.xx * r.xy + l.xy * r.yy;
				M.yx = l.yx * r.xx + l.yy * r.yx;
				M.yy = l.yx * r.xy + l.yy * r.yy;
				M.dx = l.xx * r.dx + l.xy * r.dy + l.dx;
				M.dy = l.yx * r.dx + l.yy * r.dy + l.dy;
			}
			return M; // dojox/gfx/matrix.Matrix2D
		},

		// high level operations

		_sandwich: function(matrix, x, y){
			// summary:
			//		applies a matrix at a central point
			// matrix: dojox/gfx/matrix.Matrix2D
			//		a 2D matrix-like object, which is applied at a central point
			// x: Number
			//		an x component of the central point
			// y: Number
			//		a y component of the central point
			return m.multiply(m.translate(x, y), matrix, m.translate(-x, -y)); // dojox/gfx/matrix.Matrix2D
		},
		scaleAt: function(a, b, c, d){
			// summary:
			//		scales a picture using a specified point as a center of scaling
			// description:
			//		Compare with dojox/gfx/matrix.scale().
			// a: Number
			//		a scaling factor used for the x coordinate, or a uniform scaling factor used for both coordinates
			// b: Number?
			//		a scaling factor used for the y coordinate
			// c: Number|Point
			//		an x component of a central point, or a central point
			// d: Number
			//		a y component of a central point
			// returns: dojox/gfx/matrix.Matrix2D
			switch(arguments.length){
				case 4:
					// a and b are scale factor components, c and d are components of a point
					return m._sandwich(m.scale(a, b), c, d); // dojox/gfx/matrix.Matrix2D
				case 3:
					if(typeof c == "number"){
						return m._sandwich(m.scale(a), b, c); // dojox/gfx/matrix.Matrix2D
					}
					return m._sandwich(m.scale(a, b), c.x, c.y); // dojox/gfx/matrix.Matrix2D
			}
			return m._sandwich(m.scale(a), b.x, b.y); // dojox/gfx/matrix.Matrix2D
		},
		rotateAt: function(angle, a, b){
			// summary:
			//		rotates a picture using a specified point as a center of rotation
			// description:
			//		Compare with dojox/gfx/matrix.rotate().
			// angle: Number
			//		an angle of rotation in radians (>0 for CW)
			// a: Number|dojox/gfx.Point
			//		an x component of a central point, or a central point
			// b: Number?
			//		a y component of a central point
			// returns: dojox/gfx/matrix.Matrix2D
			if(arguments.length > 2){
				return m._sandwich(m.rotate(angle), a, b); // dojox/gfx/matrix.Matrix2D
			}
			return m._sandwich(m.rotate(angle), a.x, a.y); // dojox/gfx/matrix.Matrix2D
		},
		rotategAt: function(degree, a, b){
			// summary:
			//		rotates a picture using a specified point as a center of rotation
			// description:
			//		Compare with dojox/gfx/matrix.rotateg().
			// degree: Number
			//		an angle of rotation in degrees (>0 for CW)
			// a: Number|dojox/gfx.Point
			//		an x component of a central point, or a central point
			// b: Number?
			//		a y component of a central point
			// returns: dojox/gfx/matrix.Matrix2D
			if(arguments.length > 2){
				return m._sandwich(m.rotateg(degree), a, b); // dojox/gfx/matrix.Matrix2D
			}
			return m._sandwich(m.rotateg(degree), a.x, a.y); // dojox/gfx/matrix.Matrix2D
		},
		skewXAt: function(angle, a, b){
			// summary:
			//		skews a picture along the x axis using a specified point as a center of skewing
			// description:
			//		Compare with dojox/gfx/matrix.skewX().
			// angle: Number
			//		a skewing angle in radians
			// a: Number|dojox/gfx.Point
			//		an x component of a central point, or a central point
			// b: Number?
			//		a y component of a central point
			// returns: dojox/gfx/matrix.Matrix2D
			if(arguments.length > 2){
				return m._sandwich(m.skewX(angle), a, b); // dojox/gfx/matrix.Matrix2D
			}
			return m._sandwich(m.skewX(angle), a.x, a.y); // dojox/gfx/matrix.Matrix2D
		},
		skewXgAt: function(degree, a, b){
			// summary:
			//		skews a picture along the x axis using a specified point as a center of skewing
			// description:
			//		Compare with dojox/gfx/matrix.skewXg().
			// degree: Number
			//		a skewing angle in degrees
			// a: Number|dojox/gfx.Point
			//		an x component of a central point, or a central point
			// b: Number?
			//		a y component of a central point
			// returns: dojox/gfx/matrix.Matrix2D
			if(arguments.length > 2){
				return m._sandwich(m.skewXg(degree), a, b); // dojox/gfx/matrix.Matrix2D
			}
			return m._sandwich(m.skewXg(degree), a.x, a.y); // dojox/gfx/matrix.Matrix2D
		},
		skewYAt: function(angle, a, b){
			// summary:
			//		skews a picture along the y axis using a specified point as a center of skewing
			// description:
			//		Compare with dojox/gfx/matrix.skewY().
			// angle: Number
			//		a skewing angle in radians
			// a: Number|dojox/gfx.Point
			//		an x component of a central point, or a central point
			// b: Number?
			//		a y component of a central point
			// returns: dojox/gfx/matrix.Matrix2D
			if(arguments.length > 2){
				return m._sandwich(m.skewY(angle), a, b); // dojox/gfx/matrix.Matrix2D
			}
			return m._sandwich(m.skewY(angle), a.x, a.y); // dojox/gfx/matrix.Matrix2D
		},
		skewYgAt: function(/* Number */ degree, /* Number||Point */ a, /* Number? */ b){
			// summary:
			//		skews a picture along the y axis using a specified point as a center of skewing
			// description:
			//		Compare with dojox/gfx/matrix.skewYg().
			// degree: Number
			//		a skewing angle in degrees
			// a: Number|dojox/gfx.Point
			//		an x component of a central point, or a central point
			// b: Number?
			//		a y component of a central point
			// returns: dojox/gfx/matrix.Matrix2D
			if(arguments.length > 2){
				return m._sandwich(m.skewYg(degree), a, b); // dojox/gfx/matrix.Matrix2D
			}
			return m._sandwich(m.skewYg(degree), a.x, a.y); // dojox/gfx/matrix.Matrix2D
		}

		//TODO: rect-to-rect mapping, scale-to-fit (isotropic and anisotropic versions)

	});
	// propagate Matrix2D up
	g.Matrix2D = m.Matrix2D;

	return m;
});



},
'dojox/charting/SimpleTheme':function(){
define("dojox/charting/SimpleTheme", ["dojo/_base/lang", "dojo/_base/array","dojo/_base/declare","dojo/_base/Color", "dojox/lang/utils", "dojox/gfx/gradutils"],
	function(lang, arr, declare, Color, dlu, dgg){
	
	var SimpleTheme = declare("dojox.charting.SimpleTheme", null, {
	// summary:
	//		A SimpleTheme or Theme is a pre-defined object, primarily JSON-based, that makes up the definitions to
	//		style a chart.
	//
	// description:
	//		While you can set up style definitions on a chart directly (usually through the various add methods
	//		on a dojox.charting.Chart object), a Theme simplifies this manual setup by allowing you to
	//		pre-define all of the various visual parameters of each element in a chart.
	//
	//		Most of the properties of a Theme are straight-forward; if something is line-based (such as
	//		an axis or the ticks on an axis), they will be defined using basic stroke parameters.  Likewise,
	//		if an element is primarily block-based (such as the background of a chart), it will be primarily
	//		fill-based.
	//
	//		In addition (for convenience), a Theme definition does not have to contain the entire JSON-based
	//		structure.  Each theme is built on top of a default theme (which serves as the basis for the theme
	//		"GreySkies"), and is mixed into the default theme object.  This allows you to create a theme based,
	//		say, solely on colors for data series.
	//
	//		Defining a new theme is relatively easy; see any of the themes in dojox.charting.themes for examples
	//		on how to define your own.
	//
	//		When you set a theme on a chart, the theme itself is deep-cloned.  This means that you cannot alter
	//		the theme itself after setting the theme value on a chart, and expect it to change your chart.  If you
	//		are looking to make alterations to a theme for a chart, the suggestion would be to create your own
	//		theme, based on the one you want to use, that makes those alterations before it is applied to a chart.
	//
	//		Finally, a Theme contains a number of functions to facilitate rendering operations on a chart--the main
	//		helper of which is the ~next~ method, in which a chart asks for the information for the next data series
	//		to be rendered.
	//
	//		A note on colors:
	//		A theme palette is usually comprised of 5 different color definitions, and
	//		no more.  If you have a need to render a chart with more than 5 data elements, you can simply "push"
	//		new color definitions into the theme's .color array.  Make sure that you do that with the actual
	//		theme object from a Chart, and not in the theme itself (i.e. either do that before using .setTheme
	//		on a chart).
	//
	// example:
	//		The default theme (and structure) looks like so:
	//	|	// all objects are structs used directly in dojox.gfx
	//	|	chart:{
	//	|		stroke: null,
	//	|		fill: "white",
	//	|		pageStyle: null // suggested page style as an object suitable for dojo.style()
	//	|	},
	//	|	plotarea:{
	//	|		stroke: null,
	//	|		fill: "white"
	//	|	},
	//	|	axis:{
	//	|		stroke:	{ // the axis itself
	//	|			color: "#333",
	//	|			width: 1
	//	|		},
	//	|		tick: {	// used as a foundation for all ticks
	//	|			color:     "#666",
	//	|			position:  "center",
	//	|			font:      "normal normal normal 7pt Tahoma",	// labels on axis
	//	|			fontColor: "#333"								// color of labels
	//	|		},
	//	|		majorTick:	{ // major ticks on axis, and used for major gridlines
	//	|			width:  1,
	//	|			length: 6
	//	|		},
	//	|		minorTick:	{ // minor ticks on axis, and used for minor gridlines
	//	|			width:  0.8,
	//	|			length: 3
	//	|		},
	//	|		microTick:	{ // minor ticks on axis, and used for minor gridlines
	//	|			width:  0.5,
	//	|			length: 1
	//	|		},
	//	|		title: {
	//	|			gap:  15,
	//	|			font: "normal normal normal 11pt Tahoma",	// title font
	//	|			fontColor: "#333",							// title font color
	//	|			orientation: "axis"						// "axis": facing the axis, "away": facing away
	//	|		}
	//	|	},
	//	|	series: {
	//	|		stroke:  {width: 1.5, color: "#333"},		// line
	//	|		outline: {width: 0.1, color: "#ccc"},		// outline
	//	|		//shadow:  {dx: 1, dy: 1, width: 2, color: [0, 0, 0, 0.3]},
	//	|		shadow: null,								// no shadow
	//	|		fill:    "#ccc",							// fill, if appropriate
	//	|		font:    "normal normal normal 8pt Tahoma",	// if there's a label
	//	|		fontColor: "#000"							// color of labels
	//	|		labelWiring: {width: 1, color: "#ccc"},		// connect marker and target data item(slice, column, bar...)
	//	|	},
	//	|	marker: {	// any markers on a series
	//	|		symbol:  "m-3,3 l3,-6 3,6 z",				// symbol
	//	|		stroke:  {width: 1.5, color: "#333"},		// stroke
	//	|		outline: {width: 0.1, color: "#ccc"},		// outline
	//	|		shadow: null,								// no shadow
	//	|		fill:    "#ccc",							// fill if needed
	//	|		font:    "normal normal normal 8pt Tahoma",	// label
	//	|		fontColor: "#000"
	//	|	},
	//	|	grid: {	// grid, when not present axis tick strokes are used instead
	//	|		majorLine: {	// major grid line
	//	|			color:     "#666",
	//	|			width:  1,
	//	|			length: 6
	//	|		},
	//	|		minorLine: {	// minor grid line
	//	|			color:     "#666",
	//	|			width:  0.8,
	//	|			length: 3
	//	|		}
	//	|	},
	//	|	indicator: {
	//	|		lineStroke:  {width: 1.5, color: "#333"},		// line
	//	|		lineOutline: {width: 0.1, color: "#ccc"},		// line outline
	//	|		lineShadow: null,								// no line shadow
	//	|		stroke:  {width: 1.5, color: "#333"},			// label background stroke
	//	|		outline: {width: 0.1, color: "#ccc"},			// label background outline
	//	|		shadow: null,									// no label background shadow
	//	|		fill:  "#ccc",									// label background fill
	//	|		radius: 3,										// radius of the label background
	//	|		font:    "normal normal normal 10pt Tahoma",	// label font
	//	|		fontColor: "#000"								// label color
	//	|		markerFill:    "#ccc",							// marker fill
	//	|		markerSymbol:  "m-3,0 c0,-4 6,-4 6,0 m-6,0 c0,4 6,4 6,0",	// marker symbol
	//	|		markerStroke:  {width: 1.5, color: "#333"},		// marker stroke
	//	|		markerOutline: {width: 0.1, color: "#ccc"},		// marker outline
	//	|		markerShadow: null,								// no marker shadow
	//	|	}
	//
	// example:
	//		Defining a new theme is pretty simple:
	//	|	var Grasslands = new SimpleTheme({
	//	|		colors: [ "#70803a", "#dde574", "#788062", "#b1cc5d", "#eff2c2" ]
	//	|	});
	//	|
	//	|	myChart.setTheme(Grasslands);

	shapeSpaces: {shape: 1, shapeX: 1, shapeY: 1},

	constructor: function(kwArgs){
		// summary:
		//		Initialize a theme using the keyword arguments.  Note that the arguments
		//		look like the example (above), and may include a few more parameters.
		kwArgs = kwArgs || {};

		// populate theme with defaults updating them if needed
		var def = SimpleTheme.defaultTheme;
		arr.forEach(["chart", "plotarea", "axis", "grid", "series", "marker", "indicator"], function(name){
			this[name] = lang.delegate(def[name], kwArgs[name]);
		}, this);

		// personalize theme
		if(kwArgs.seriesThemes && kwArgs.seriesThemes.length){
			this.colors  = null;
			this.seriesThemes = kwArgs.seriesThemes.slice(0);
		}else{
			this.seriesThemes = null;
			this.colors = (kwArgs.colors || SimpleTheme.defaultColors).slice(0);
		}
		this.markerThemes = null;
		if(kwArgs.markerThemes && kwArgs.markerThemes.length){
			this.markerThemes = kwArgs.markerThemes.slice(0);
		}
		this.markers = kwArgs.markers ? lang.clone(kwArgs.markers) : lang.delegate(SimpleTheme.defaultMarkers);

		// set flags
		this.noGradConv = kwArgs.noGradConv;
		this.noRadialConv = kwArgs.noRadialConv;
		if(kwArgs.reverseFills){
			this.reverseFills();
		}

		//	private housekeeping
		this._current = 0;
		this._buildMarkerArray();
	},

	clone: function(){
		// summary:
		//		Clone the current theme.
		// returns: dojox.charting.SimpleTheme
		//		The cloned theme; any alterations made will not affect the original.
		var theme = new this.constructor({
			// theme components
			chart: this.chart,
			plotarea: this.plotarea,
			axis: this.axis,
			grid: this.grid,
			series: this.series,
			marker: this.marker,
			// individual arrays
			colors: this.colors,
			markers: this.markers,
			indicator: this.indicator,
			seriesThemes: this.seriesThemes,
			markerThemes: this.markerThemes,
			// flags
			noGradConv: this.noGradConv,
			noRadialConv: this.noRadialConv
		});
		// copy custom methods
		arr.forEach(
			["clone", "clear", "next", "skip", "addMixin", "post", "getTick"],
			function(name){
				if(this.hasOwnProperty(name)){
					theme[name] = this[name];
				}
			},
			this
		);
		return theme;	//	dojox.charting.SimpleTheme
	},

	clear: function(){
		// summary:
		//		Clear and reset the internal pointer to start fresh.
		this._current = 0;
	},

	next: function(elementType, mixin, doPost){
		// summary:
		//		Get the next color or series theme.
		// elementType: String?
		//		An optional element type (for use with series themes)
		// mixin: Object?
		//		An optional object to mix into the theme.
		// doPost: Boolean?
		//		A flag to post-process the results.
		// returns: Object
		//		An object of the structure { series, marker, symbol }
		var merge = dlu.merge, series, marker;
		if(this.colors){
			series = lang.delegate(this.series);
			marker = lang.delegate(this.marker);
			var color = new Color(this.colors[this._current % this.colors.length]), old;
			// modify the stroke
			if(series.stroke && series.stroke.color){
				series.stroke = lang.delegate(series.stroke);
				old = new Color(series.stroke.color);
				series.stroke.color = new Color(color);
				series.stroke.color.a = old.a;
			}else{
				series.stroke = {color: color};
			}
			if(marker.stroke && marker.stroke.color){
				marker.stroke = lang.delegate(marker.stroke);
				old = new Color(marker.stroke.color);
				marker.stroke.color = new Color(color);
				marker.stroke.color.a = old.a;
			}else{
				marker.stroke = {color: color};
			}
			// modify the fill
			if(!series.fill || series.fill.type){
				series.fill = color;
			}else{
				old = new Color(series.fill);
				series.fill = new Color(color);
				series.fill.a = old.a;
			}
			if(!marker.fill || marker.fill.type){
				marker.fill = color;
			}else{
				old = new Color(marker.fill);
				marker.fill = new Color(color);
				marker.fill.a = old.a;
			}
		}else{
			series = this.seriesThemes ?
				merge(this.series, this.seriesThemes[this._current % this.seriesThemes.length]) :
				this.series;
			marker = this.markerThemes ?
				merge(this.marker, this.markerThemes[this._current % this.markerThemes.length]) :
				series;
		}

		var symbol = marker && marker.symbol || this._markers[this._current % this._markers.length];

		var theme = {series: series, marker: marker, symbol: symbol};
		
		// advance the counter
		++this._current;

		if(mixin){
			theme = this.addMixin(theme, elementType, mixin);
		}
		if(doPost){
			theme = this.post(theme, elementType);
		}

		return theme;	//	Object
	},

	skip: function(){
		// summary:
		//		Skip the next internal color.
		++this._current;
	},

	addMixin: function(theme, elementType, mixin, doPost){
		// summary:
		//		Add a mixin object to the passed theme and process.
		// theme: dojox/charting/SimpleTheme
		//		The theme to mixin to.
		// elementType: String
		//		The type of element in question. Can be "line", "bar" or "circle"
		// mixin: Object|Array
		//		The object or objects to mix into the theme.
		// doPost: Boolean
		//		If true, run the new theme through the post-processor.
		// returns: dojox/charting/SimpleTheme
		//		The new theme.
		if(lang.isArray(mixin)){
			arr.forEach(mixin, function(m){
				theme = this.addMixin(theme, elementType, m);
			}, this);
		}else{
			var t = {};
			if("color" in mixin){
				if(elementType == "line" || elementType == "area"){
					lang.setObject("series.stroke.color", mixin.color, t);
					lang.setObject("marker.stroke.color", mixin.color, t);
				}else{
					lang.setObject("series.fill", mixin.color, t);
				}
			}
			arr.forEach(["stroke", "outline", "shadow", "fill", "font", "fontColor", "labelWiring"], function(name){
				var markerName = "marker" + name.charAt(0).toUpperCase() + name.substr(1),
					b = markerName in mixin;
				if(name in mixin){
					lang.setObject("series." + name, mixin[name], t);
					if(!b){
						lang.setObject("marker." + name, mixin[name], t);
					}
				}
				if(b){
					lang.setObject("marker." + name, mixin[markerName], t);
				}
			});
			if("marker" in mixin){
				t.symbol = mixin.marker;
				t.symbol = mixin.marker;
			}
			theme = dlu.merge(theme, t);
		}
		if(doPost){
			theme = this.post(theme, elementType);
		}
		return theme;	//	dojox/charting/SimpleTheme
	},

	post: function(theme, elementType){
		// summary:
		//		Process any post-shape fills.
		// theme: dojox/charting/SimpleTheme
		//		The theme to post process with.
		// elementType: String
		//		The type of element being filled.  Can be "bar" or "circle".
		// returns: dojox/charting/SimpleTheme
		//		The post-processed theme.
		var fill = theme.series.fill, t;
		if(!this.noGradConv && this.shapeSpaces[fill.space] && fill.type == "linear"){
			if(elementType == "bar"){
				// transpose start and end points
				t = {
					x1: fill.y1,
					y1: fill.x1,
					x2: fill.y2,
					y2: fill.x2
				};
			}else if(!this.noRadialConv && fill.space == "shape" && (elementType == "slice" || elementType == "circle")){
				// switch to radial
				t = {
					type: "radial",
					cx: 0,
					cy: 0,
					r:  100
				};
			}
			if(t){
				return dlu.merge(theme, {series: {fill: t}});
			}
		}
		return theme;	//	dojox/charting/SimpleTheme
	},

	getTick: function(name, mixin){
		// summary:
		//		Calculates and merges tick parameters.
		// name: String
		//		Tick name, can be "major", "minor", or "micro".
		// mixin: Object?
		//		Optional object to mix in to the tick.
		var tick = this.axis.tick, tickName = name + "Tick",
			merge = dlu.merge;
		if(tick){
			if(this.axis[tickName]){
				tick = merge(tick, this.axis[tickName]);
			}
		}else{
			tick = this.axis[tickName];
		}
		if(mixin){
			if(tick){
				if(mixin[tickName]){
					tick = merge(tick, mixin[tickName]);
				}
			}else{
				tick = mixin[tickName];
			}
		}
		return tick;	//	Object
	},

	inspectObjects: function(f){
		arr.forEach(["chart", "plotarea", "axis", "grid", "series", "marker", "indicator"], function(name){
			f(this[name]);
		}, this);
		if(this.seriesThemes){
			arr.forEach(this.seriesThemes, f);
		}
		if(this.markerThemes){
			arr.forEach(this.markerThemes, f);
		}
	},

	reverseFills: function(){
		this.inspectObjects(function(o){
			if(o && o.fill){
				o.fill = dgg.reverse(o.fill);
			}
		});
	},

	addMarker:function(/*String*/ name, /*String*/ segment){
		// summary:
		//		Add a custom marker to this theme.
		// example:
		//	|	myTheme.addMarker("Ellipse", foo);
		this.markers[name] = segment;
		this._buildMarkerArray();
	},

	setMarkers:function(/*Object*/ obj){
		// summary:
		//		Set all the markers of this theme at once.  obj should be a
		//		dictionary of keys and path segments.
		//
		// example:
		//	|	myTheme.setMarkers({ "CIRCLE": foo });
		this.markers = obj;
		this._buildMarkerArray();
	},

	_buildMarkerArray: function(){
		this._markers = [];
		for(var p in this.markers){
			this._markers.push(this.markers[p]);
		}
	}
});

lang.mixin(SimpleTheme, {
	defaultMarkers: {
		CIRCLE:   "m-3,0 c0,-4 6,-4 6,0 m-6,0 c0,4 6,4 6,0",
		SQUARE:   "m-3,-3 l0,6 6,0 0,-6 z",
		DIAMOND:  "m0,-3 l3,3 -3,3 -3,-3 z",
		CROSS:    "m0,-3 l0,6 m-3,-3 l6,0",
		X:        "m-3,-3 l6,6 m0,-6 l-6,6",
		TRIANGLE: "m-3,3 l3,-6 3,6 z",
		TRIANGLE_INVERTED: "m-3,-3 l3,6 3,-6 z"
	},

	defaultColors:[
		// gray skies
		"#54544c", "#858e94", "#6e767a", "#948585", "#474747"
	],

	defaultTheme: {
		// all objects are structs used directly in dojox.gfx
		chart:{
			stroke: null,
			fill: "white",
			pageStyle: null,
			titleGap:		20,
			titlePos:		"top",
			titleFont:      "normal normal bold 14pt Tahoma",	// chart title
			titleFontColor: "#333"
		},
		plotarea:{
			stroke: null,
			fill: "white"
		},
		// TODO: label rotation on axis
		axis:{
			stroke:	{ // the axis itself
				color: "#333",
				width: 1
			},
			tick: {	// used as a foundation for all ticks
				color:     "#666",
				position:  "center",
				font:      "normal normal normal 7pt Tahoma",	// labels on axis
				fontColor: "#333",								// color of labels
				labelGap:  4                                    // gap between a tick and its label in pixels
			},
			majorTick:	{ // major ticks on axis, and used for major gridlines
				width:  1,
				length: 6
			},
			minorTick:	{ // minor ticks on axis, and used for minor gridlines
				width:  0.8,
				length: 3
			},
			microTick:	{ // minor ticks on axis, and used for minor gridlines
				width:  0.5,
				length: 1
			},
			title: {
				gap:  15,
				font: "normal normal normal 11pt Tahoma",	// title font
				fontColor: "#333",							// title font color
				orientation: "axis"						// "axis": facing the axis, "away": facing away
			}
		},
		series: {
			// used as a "main" theme for series, sThemes augment it
			stroke:  {width: 1.5, color: "#333"},		// line
			outline: {width: 0.1, color: "#ccc"},		// outline
			//shadow:  {dx: 1, dy: 1, width: 2, color: [0, 0, 0, 0.3]},
			shadow: null,								// no shadow
			fill:    "#ccc",							// fill, if appropriate
			font:    "normal normal normal 8pt Tahoma",	// if there's a label
			fontColor: "#000",							// color of labels
			labelWiring: {width: 1, color: "#ccc"}		// connect marker and target data item(slice, column, bar...)
		},
		marker: {	// any markers on a series
			stroke:  {width: 1.5, color: "#333"},		// stroke
			outline: {width: 0.1, color: "#ccc"},		// outline
			//shadow:  {dx: 1, dy: 1, width: 2, color: [0, 0, 0, 0.3]},
			shadow: null,								// no shadow
			fill:    "#ccc",							// fill if needed
			font:    "normal normal normal 8pt Tahoma",	// label
			fontColor: "#000"
		},
		indicator: {
			lineStroke:  {width: 1.5, color: "#333"},		
			lineOutline: {width: 0.1, color: "#ccc"},		
			lineShadow: null,
			stroke:  {width: 1.5, color: "#333"},		
			outline: {width: 0.1, color: "#ccc"},		
			shadow: null,								
			fill : "#ccc",
			radius: 3,
			font:    "normal normal normal 10pt Tahoma",	
			fontColor: "#000",							
			markerFill:    "#ccc",							
			markerSymbol:  "m-3,0 c0,-4 6,-4 6,0 m-6,0 c0,4 6,4 6,0",			
			markerStroke:  {width: 1.5, color: "#333"},		
			markerOutline: {width: 0.1, color: "#ccc"},		
			markerShadow: null								
		}
	}
});

return SimpleTheme;
});

},
'dojox/lang/utils':function(){
define("dojox/lang/utils", ["..", "dojo/_base/lang"], 
  function(dojox, lang){
	var du = lang.getObject("lang.utils", true, dojox);
	
	var empty = {}, opts = Object.prototype.toString;

	var clone = function(o){
		if(o){
			switch(opts.call(o)){
				case "[object Array]":
					return o.slice(0);
				case "[object Object]":
					return lang.delegate(o);
			}
		}
		return o;
	}
	
	lang.mixin(du, {
		coerceType: function(target, source){
			// summary:
			//		Coerces one object to the type of another.
			// target: Object
			//		object, which typeof result is used to coerce "source" object.
			// source: Object
			//		object, which will be forced to change type.
			switch(typeof target){
				case "number":	return Number(eval("(" + source + ")"));
				case "string":	return String(source);
				case "boolean":	return Boolean(eval("(" + source + ")"));
			}
			return eval("(" + source + ")");
		},
		
		updateWithObject: function(target, source, conv){
			// summary:
			//		Updates an existing object in place with properties from an "source" object.
			// target: Object
			//		the "target" object to be updated
			// source: Object
			//		the "source" object, whose properties will be used to source the existed object.
			// conv: Boolean?
			//		force conversion to the original type
			if(!source){ return target; }
			for(var x in target){
				if(x in source && !(x in empty)){
					var t = target[x];
					if(t && typeof t == "object"){
						du.updateWithObject(t, source[x], conv);
					}else{
						target[x] = conv ? du.coerceType(t, source[x]) : clone(source[x]);
					}
				}
			}
			return target;	// Object
		},
	
		updateWithPattern: function(target, source, pattern, conv){
			// summary:
			//		Updates an existing object in place with properties from an "source" object.
			// target: Object
			//		the "target" object to be updated
			// source: Object
			//		the "source" object, whose properties will be used to source the existed object.
			// pattern: Object
			//		object, whose properties will be used to pull values from the "source"
			// conv: Boolean?
			//		force conversion to the original type
			if(!source || !pattern){ return target; }
			for(var x in pattern){
				if(x in source && !(x in empty)){
					target[x] = conv ? du.coerceType(pattern[x], source[x]) : clone(source[x]);
				}
			}
			return target;	// Object
		},
		
		merge: function(object, mixin){
			// summary:
			//		Merge two objects structurally, mixin properties will override object's properties.
			// object: Object
			//		original object.
			// mixin: Object
			//		additional object, which properties will override object's properties.
			if(mixin){
				var otype = opts.call(object), mtype = opts.call(mixin), t, i, l, m;
				switch(mtype){
					case "[object Array]":
						if(mtype == otype){
							t = new Array(Math.max(object.length, mixin.length));
							for(i = 0, l = t.length; i < l; ++i){
								t[i] = du.merge(object[i], mixin[i]);
							}
							return t;
						}
						return mixin.slice(0);
					case "[object Object]":
						if(mtype == otype && object){
							t = lang.delegate(object);
							for(i in mixin){
								if(i in object){
									l = object[i];
									m = mixin[i];
									if(m !== l){
										t[i] = du.merge(l, m);
									}
								}else{
									t[i] = lang.clone(mixin[i]);
								}
							}
							return t;
						}
						return lang.clone(mixin);
				}
			}
			return mixin;
		}
	});
	
	return du;
});

},
'dojox/gfx/gradutils':function(){
// Various generic utilities to deal with a linear gradient

define("dojox/gfx/gradutils", ["./_base", "dojo/_base/lang", "./matrix", "dojo/_base/Color"], 
  function(g, lang, m, Color){
  
	var gradutils = g.gradutils = {};

	function findColor(o, c){
		if(o <= 0){
			return c[0].color;
		}
		var len = c.length;
		if(o >= 1){
			return c[len - 1].color;
		}
		//TODO: use binary search
		for(var i = 0; i < len; ++i){
			var stop = c[i];
			if(stop.offset >= o){
				if(i){
					var prev = c[i - 1];
					return Color.blendColors(new Color(prev.color), new Color(stop.color),
						(o - prev.offset) / (stop.offset - prev.offset));
				}
				return stop.color;
			}
		}
		return c[len - 1].color;
	}

	gradutils.getColor = function(fill, pt){
		// summary:
		//		sample a color from a gradient using a point
		// fill: Object
		//		fill object
		// pt: dojox/gfx.Point
		//		point where to sample a color
		var o;
		if(fill){
			switch(fill.type){
				case "linear":
					var angle = Math.atan2(fill.y2 - fill.y1, fill.x2 - fill.x1),
						rotation = m.rotate(-angle),
						projection = m.project(fill.x2 - fill.x1, fill.y2 - fill.y1),
						p = m.multiplyPoint(projection, pt),
						pf1 = m.multiplyPoint(projection, fill.x1, fill.y1),
						pf2 = m.multiplyPoint(projection, fill.x2, fill.y2),
						scale = m.multiplyPoint(rotation, pf2.x - pf1.x, pf2.y - pf1.y).x;
					o = m.multiplyPoint(rotation, p.x - pf1.x, p.y - pf1.y).x / scale;
					break;
				case "radial":
					var dx = pt.x - fill.cx, dy = pt.y - fill.cy;
					o = Math.sqrt(dx * dx + dy * dy) / fill.r;
					break;
			}
			return findColor(o, fill.colors);	// dojo/_base/Color
		}
		// simple color
		return new Color(fill || [0, 0, 0, 0]);	// dojo/_base/Color
	};

	gradutils.reverse = function(fill){
		// summary:
		//		reverses a gradient
		// fill: Object
		//		fill object
		if(fill){
			switch(fill.type){
				case "linear":
				case "radial":
					fill = lang.delegate(fill);
					if(fill.colors){
						var c = fill.colors, l = c.length, i = 0, stop,
							n = fill.colors = new Array(c.length);
						for(; i < l; ++i){
							stop = c[i];
							n[i] = {
								offset: 1 - stop.offset,
								color:  stop.color
							};
						}
						n.sort(function(a, b){ return a.offset - b.offset; });
					}
					break;
			}
		}
		return fill;	// Object
	};

	return gradutils;
});

},
'dojox/charting/Series':function(){
define("dojox/charting/Series", ["dojo/_base/lang", "dojo/_base/declare", "./Element"], 
	function(lang, declare, Element){ 
	/*=====
	var __SeriesCtorArgs = {
		// summary:
		//		An optional arguments object that can be used in the Series constructor.
		// plot: String?
		//		The plot (by name) that this series belongs to.
	};
	=====*/
	return declare("dojox.charting.Series", Element, {
		// summary:
		//		An object representing a series of data for plotting on a chart.
		constructor: function(chart, data, kwArgs){
			// summary:
			//		Create a new data series object for use within charting.
			// chart: dojox/charting/Chart
			//		The chart that this series belongs to.
			// data: Array|Object
			//		The array of data points (either numbers or objects) that
			//		represents the data to be drawn. Or it can be an object. In
			//		the latter case, it should have a property "data" (an array),
			//		destroy(), and setSeriesObject().
			// kwArgs: __SeriesCtorArgs?
			//		An optional keyword arguments object to set details for this series.
			lang.mixin(this, kwArgs);
			if(typeof this.plot != "string"){ this.plot = "default"; }
			this.update(data);
		},
	
		clear: function(){
			// summary:
			//		Clear the calculated additional parameters set on this series.
			this.dyn = {};
		},
		
		update: function(data){
			// summary:
			//		Set data and make this object dirty, so it can be redrawn.
			// data: Array|Object
			//		The array of data points (either numbers or objects) that
			//		represents the data to be drawn. Or it can be an object. In
			//		the latter case, it should have a property "data" (an array),
			//		destroy(), and setSeriesObject().
			if(lang.isArray(data)){
				this.data = data;
			}else{
				this.source = data;
				this.data = this.source.data;
				if(this.source.setSeriesObject){
					this.source.setSeriesObject(this);
				}
			}
			this.dirty = true;
			this.clear();
		}
	});
});

},
'dojox/charting/axis2d/common':function(){
define("dojox/charting/axis2d/common", ["dojo/_base/lang", "dojo/_base/window", "dojo/dom-geometry", "dojox/gfx"],
	function(lang, win, domGeom, g){

	var common = lang.getObject("dojox.charting.axis2d.common", true);
	
	var clearNode = function(s){
		s.marginLeft   = "0px";
		s.marginTop    = "0px";
		s.marginRight  = "0px";
		s.marginBottom = "0px";
		s.paddingLeft   = "0px";
		s.paddingTop    = "0px";
		s.paddingRight  = "0px";
		s.paddingBottom = "0px";
		s.borderLeftWidth   = "0px";
		s.borderTopWidth    = "0px";
		s.borderRightWidth  = "0px";
		s.borderBottomWidth = "0px";
	};

	var getBoxWidth = function(n){
		// marginBox is incredibly slow, so avoid it if we can
		if(n["getBoundingClientRect"]){
			var bcr = n.getBoundingClientRect();
			return bcr.width || (bcr.right - bcr.left);
		}else{
			return domGeom.getMarginBox(n).w;
		}
	};

	return lang.mixin(common, {
		// summary:
		//		Common methods to be used by any axis.  This is considered "static".
		createText: {
			gfx: function(chart, creator, x, y, align, text, font, fontColor){
				// summary:
				//		Use dojox.gfx to create any text.
				// chart: dojox.charting.Chart
				//		The chart to create the text into.
				// creator: dojox.gfx.Surface
				//		The graphics surface to use for creating the text.
				// x: Number
				//		Where to create the text along the x axis (CSS left).
				// y: Number
				//		Where to create the text along the y axis (CSS top).
				// align: String
				//		How to align the text.  Can be "left", "right", "center".
				// text: String
				//		The text to render.
				// font: String
				//		The font definition, a la CSS "font".
				// fontColor: String|dojo.Color
				//		The color of the resultant text.
				// returns: dojox.gfx.Text
				//		The resultant GFX object.
				return creator.createText({
					x: x, y: y, text: text, align: align
				}).setFont(font).setFill(fontColor);	//	dojox.gfx.Text
			},
			html: function(chart, creator, x, y, align, text, font, fontColor, labelWidth){
				// summary:
				//		Use the HTML DOM to create any text.
				// chart: dojox.charting.Chart
				//		The chart to create the text into.
				// creator: dojox.gfx.Surface
				//		The graphics surface to use for creating the text.
				// x: Number
				//		Where to create the text along the x axis (CSS left).
				// y: Number
				//		Where to create the text along the y axis (CSS top).
				// align: String
				//		How to align the text.  Can be "left", "right", "center".
				// text: String
				//		The text to render.
				// font: String
				//		The font definition, a la CSS "font".
				// fontColor: String|dojo.Color
				//		The color of the resultant text.
				// labelWidth: Number?
				//		The maximum width of the resultant DOM node.
				// returns: DOMNode
				//		The resultant DOMNode (a "div" element).

				// setup the text node
				var p = win.doc.createElement("div"), s = p.style, boxWidth;
				// bidi support, if this function exists the module was loaded 
				if(chart.getTextDir){
					p.dir = chart.getTextDir(text);
				}
				clearNode(s);
				s.font = font;
				p.innerHTML = String(text).replace(/\s/g, "&nbsp;");
				s.color = fontColor;
				// measure the size
				s.position = "absolute";
				s.left = "-10000px";
				win.body().appendChild(p);
				var size = g.normalizedLength(g.splitFontString(font).size);

				// do we need to calculate the label width?
				if(!labelWidth){
					boxWidth = getBoxWidth(p);
				}
				// when the textDir is rtl, but the UI ltr needs
				// to recalculate the starting point
				if(p.dir == "rtl"){
					x += labelWidth ? labelWidth : boxWidth;
				}

				// new settings for the text node
				win.body().removeChild(p);

				s.position = "relative";
				if(labelWidth){
					s.width = labelWidth + "px";
					// s.border = "1px dotted grey";
					switch(align){
						case "middle":
							s.textAlign = "center";
							s.left = (x - labelWidth / 2) + "px";
							break;
						case "end":
							s.textAlign = "right";
							s.left = (x - labelWidth) + "px";
							break;
						default:
							s.left = x + "px";
							s.textAlign = "left";
							break;
					}
				}else{
					switch(align){
						case "middle":
							s.left = Math.floor(x - boxWidth / 2) + "px";
							// s.left = Math.floor(x - p.offsetWidth / 2) + "px";
							break;
						case "end":
							s.left = Math.floor(x - boxWidth) + "px";
							// s.left = Math.floor(x - p.offsetWidth) + "px";
							break;
						//case "start":
						default:
							s.left = Math.floor(x) + "px";
							break;
					}
				}
				s.top = Math.floor(y - size) + "px";
				s.whiteSpace = "nowrap";	// hack for WebKit
				// setup the wrapper node
				var wrap = win.doc.createElement("div"), w = wrap.style;
				clearNode(w);
				w.width = "0px";
				w.height = "0px";
				// insert nodes
				wrap.appendChild(p);
				chart.node.insertBefore(wrap, chart.node.firstChild);
				return wrap;	//	DOMNode
			}
		}
	});
});

},
'dojox/lang/functional/fold':function(){
define("dojox/lang/functional/fold", ["dojo/_base/lang", "dojo/_base/array", "dojo/_base/kernel", "./lambda"],
	function(lang, arr, kernel, df){

// This module adds high-level functions and related constructs:
//	- "fold" family of functions

// Notes:
//	- missing high-level functions are provided with the compatible API:
//		foldl, foldl1, foldr, foldr1
//	- missing JS standard functions are provided with the compatible API:
//		reduce, reduceRight
//	- the fold's counterpart: unfold

// Defined methods:
//	- take any valid lambda argument as the functional argument
//	- operate on dense arrays
//	- take a string as the array argument
//	- take an iterator objects as the array argument (only foldl, foldl1, and reduce)

	var empty = {};

	lang.mixin(df, {
		// classic reduce-class functions
		foldl: function(/*Array|String|Object*/ a, /*Function*/ f, /*Object*/ z, /*Object?*/ o){
			// summary:
			//		repeatedly applies a binary function to an array from left
			//		to right using a seed value as a starting point; returns the final
			//		value.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || kernel.global; f = df.lambda(f);
			var i, n;
			if(lang.isArray(a)){
				// array
				for(i = 0, n = a.length; i < n; z = f.call(o, z, a[i], i, a), ++i);
			}else if(typeof a.hasNext == "function" && typeof a.next == "function"){
				// iterator
				for(i = 0; a.hasNext(); z = f.call(o, z, a.next(), i++, a));
			}else{
				// object/dictionary
				for(i in a){
					if(!(i in empty)){
						z = f.call(o, z, a[i], i, a);
					}
				}
			}
			return z;	// Object
		},
		foldl1: function(/*Array|String|Object*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary:
			//		repeatedly applies a binary function to an array from left
			//		to right; returns the final value.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || kernel.global; f = df.lambda(f);
			var z, i, n;
			if(lang.isArray(a)){
				// array
				z = a[0];
				for(i = 1, n = a.length; i < n; z = f.call(o, z, a[i], i, a), ++i);
			}else if(typeof a.hasNext == "function" && typeof a.next == "function"){
				// iterator
				if(a.hasNext()){
					z = a.next();
					for(i = 1; a.hasNext(); z = f.call(o, z, a.next(), i++, a));
				}
			}else{
				// object/dictionary
				var first = true;
				for(i in a){
					if(!(i in empty)){
						if(first){
							z = a[i];
							first = false;
						}else{
							z = f.call(o, z, a[i], i, a);
						}
					}
				}
			}
			return z;	// Object
		},
		foldr: function(/*Array|String*/ a, /*Function|String|Array*/ f, /*Object*/ z, /*Object?*/ o){
			// summary:
			//		repeatedly applies a binary function to an array from right
			//		to left using a seed value as a starting point; returns the final
			//		value.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || kernel.global; f = df.lambda(f);
			for(var i = a.length; i > 0; --i, z = f.call(o, z, a[i], i, a));
			return z;	// Object
		},
		foldr1: function(/*Array|String*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary:
			//		repeatedly applies a binary function to an array from right
			//		to left; returns the final value.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || kernel.global; f = df.lambda(f);
			var n = a.length, z = a[n - 1], i = n - 1;
			for(; i > 0; --i, z = f.call(o, z, a[i], i, a));
			return z;	// Object
		},
		// JS 1.8 standard array functions, which can take a lambda as a parameter.
		reduce: function(/*Array|String|Object*/ a, /*Function|String|Array*/ f, /*Object?*/ z){
			// summary:
			//		apply a function simultaneously against two values of the array
			//		(from left-to-right) as to reduce it to a single value.
			return arguments.length < 3 ? df.foldl1(a, f) : df.foldl(a, f, z);	// Object
		},
		reduceRight: function(/*Array|String*/ a, /*Function|String|Array*/ f, /*Object?*/ z){
			// summary:
			//		apply a function simultaneously against two values of the array
			//		(from right-to-left) as to reduce it to a single value.
			return arguments.length < 3 ? df.foldr1(a, f) : df.foldr(a, f, z);	// Object
		},
		// the fold's counterpart: unfold
		unfold: function(/*Function|String|Array*/ pr, /*Function|String|Array*/ f,
						/*Function|String|Array*/ g, /*Object*/ z, /*Object?*/ o){
			// summary:
			//		builds an array by unfolding a value
			o = o || kernel.global; f = df.lambda(f); g = df.lambda(g); pr = df.lambda(pr);
			var t = [];
			for(; !pr.call(o, z); t.push(f.call(o, z)), z = g.call(o, z));
			return t;	// Array
		}
	});
});

},
'dojox/lang/functional/reversed':function(){
define("dojox/lang/functional/reversed", ["dojo/_base/lang", "dojo/_base/kernel" ,"./lambda"],
	function(lang, kernel, df){
// This module adds high-level functions and related constructs:
//	- reversed versions of array-processing functions similar to standard JS functions

// Notes:
//	- this module provides reversed versions of standard array-processing functions:
//		forEachRev, mapRev, filterRev

// Defined methods:
//	- take any valid lambda argument as the functional argument
//	- operate on dense arrays
//	- take a string as the array argument

	lang.mixin(df, {
		// JS 1.6 standard array functions, which can take a lambda as a parameter.
		// Consider using dojo._base.array functions, if you don't need the lambda support.
		filterRev: function(/*Array|String*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary:
			//		creates a new array with all elements that pass the test
			//		implemented by the provided function.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || kernel.global; f = df.lambda(f);
			var t = [], v, i = a.length - 1;
			for(; i >= 0; --i){
				v = a[i];
				if(f.call(o, v, i, a)){ t.push(v); }
			}
			return t;	// Array
		},
		forEachRev: function(/*Array|String*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary:
			//		executes a provided function once per array element.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || kernel.global; f = df.lambda(f);
			for(var i = a.length - 1; i >= 0; f.call(o, a[i], i, a), --i);
		},
		mapRev: function(/*Array|String*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary:
			//		creates a new array with the results of calling
			//		a provided function on every element in this array.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || kernel.global; f = df.lambda(f);
			var n = a.length, t = new Array(n), i = n - 1, j = 0;
			for(; i >= 0; t[j++] = f.call(o, a[i], i, a), --i);
			return t;	// Array
		},
		everyRev: function(/*Array|String*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary:
			//		tests whether all elements in the array pass the test
			//		implemented by the provided function.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || kernel.global; f = df.lambda(f);
			for(var i = a.length - 1; i >= 0; --i){
				if(!f.call(o, a[i], i, a)){
					return false;	// Boolean
				}
			}
			return true;	// Boolean
		},
		someRev: function(/*Array|String*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary:
			//		tests whether some element in the array passes the test
			//		implemented by the provided function.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || kernel.global; f = df.lambda(f);
			for(var i = a.length - 1; i >= 0; --i){
				if(f.call(o, a[i], i, a)){
					return true;	// Boolean
				}
			}
			return false;	// Boolean
		}
	});
	
	return df;
});

},
'dojox/charting/action2d/Tooltip':function(){
define("dojox/charting/action2d/Tooltip", ["dojo/_base/kernel", "dijit/Tooltip","dojo/_base/lang", "dojo/_base/declare", "dojo/dom-style", "./PlotAction",
	"dojox/gfx/matrix", "dojox/lang/functional", "dojox/lang/functional/scan", "dojox/lang/functional/fold"], 
	function(dojo, Tooltip, lang, declare, domStyle, PlotAction, m, df, dfs, dff){
	
	/*=====
	var __TooltipCtorArgs = {
			// summary:
			//		Additional arguments for tooltip actions.
			// duration: Number?
			//		The amount of time in milliseconds for an animation to last.  Default is 400.
			// easing: dojo/fx/easing/*?
			//		An easing object (see dojo.fx.easing) for use in an animation.  The
			//		default is dojo.fx.easing.backOut.
			// text: Function?
			//		The function that produces the text to be shown within a tooltip.  By default this will be
			//		set by the plot in question, by returning the value of the element.
	};
	=====*/

	var DEFAULT_TEXT = function(o){
		var t = o.run && o.run.data && o.run.data[o.index];
		if(t && typeof t != "number" && (t.tooltip || t.text)){
			return t.tooltip || t.text;
		}
		if(o.element == "candlestick"){
			return '<table cellpadding="1" cellspacing="0" border="0" style="font-size:0.9em;">'
				+ '<tr><td>Open:</td><td align="right"><strong>' + o.data.open + '</strong></td></tr>'
				+ '<tr><td>High:</td><td align="right"><strong>' + o.data.high + '</strong></td></tr>'
				+ '<tr><td>Low:</td><td align="right"><strong>' + o.data.low + '</strong></td></tr>'
				+ '<tr><td>Close:</td><td align="right"><strong>' + o.data.close + '</strong></td></tr>'
				+ (o.data.mid !== undefined ? '<tr><td>Mid:</td><td align="right"><strong>' + o.data.mid + '</strong></td></tr>' : '')
				+ '</table>';
		}
		return o.element == "bar" ? o.x : o.y;
	};

	var pi4 = Math.PI / 4, pi2 = Math.PI / 2;
	
	return declare("dojox.charting.action2d.Tooltip", PlotAction, {
		// summary:
		//		Create an action on a plot where a tooltip is shown when hovering over an element.

		// the data description block for the widget parser
		defaultParams: {
			text: DEFAULT_TEXT	// the function to produce a tooltip from the object
		},
		optionalParams: {},	// no optional parameters

		constructor: function(chart, plot, kwArgs){
			// summary:
			//		Create the tooltip action and connect it to the plot.
			// chart: dojox/charting/Chart
			//		The chart this action belongs to.
			// plot: String?
			//		The plot this action is attached to.  If not passed, "default" is assumed.
			// kwArgs: __TooltipCtorArgs?
			//		Optional keyword arguments object for setting parameters.
			this.text = kwArgs && kwArgs.text ? kwArgs.text : DEFAULT_TEXT;
			
			this.connect();
		},
		
		process: function(o){
			// summary:
			//		Process the action on the given object.
			// o: dojox/gfx/shape.Shape
			//		The object on which to process the highlighting action.
			if(o.type === "onplotreset" || o.type === "onmouseout"){
                Tooltip.hide(this.aroundRect);
				this.aroundRect = null;
				if(o.type === "onplotreset"){
					delete this.angles;
				}
				return;
			}
			
			if(!o.shape || o.type !== "onmouseover"){ return; }
			
			// calculate relative coordinates and the position
			var aroundRect = {type: "rect"}, position = ["after-centered", "before-centered"];
			switch(o.element){
				case "marker":
					aroundRect.x = o.cx;
					aroundRect.y = o.cy;
					aroundRect.w = aroundRect.h = 1;
					break;
				case "circle":
					aroundRect.x = o.cx - o.cr;
					aroundRect.y = o.cy - o.cr;
					aroundRect.w = aroundRect.h = 2 * o.cr;
					break;
				case "spider_circle":
					aroundRect.x = o.cx;
					aroundRect.y = o.cy ;
					aroundRect.w = aroundRect.h = 1;
					break;
				case "spider_plot":
					return;
				case "column":
					position = ["above-centered", "below-centered"];
					// intentional fall down
				case "bar":
					aroundRect = lang.clone(o.shape.getShape());
					aroundRect.w = aroundRect.width;
					aroundRect.h = aroundRect.height;
					break;
				case "candlestick":
					aroundRect.x = o.x;
					aroundRect.y = o.y;
					aroundRect.w = o.width;
					aroundRect.h = o.height;
					break;
				default:
				//case "slice":
					if(!this.angles){
						// calculate the running total of slice angles
						if(typeof o.run.data[0] == "number"){
							this.angles = df.map(df.scanl(o.run.data, "+", 0),
								"* 2 * Math.PI / this", df.foldl(o.run.data, "+", 0));
						}else{
							this.angles = df.map(df.scanl(o.run.data, "a + b.y", 0),
								"* 2 * Math.PI / this", df.foldl(o.run.data, "a + b.y", 0));
						}
					}
					var startAngle = m._degToRad(o.plot.opt.startAngle),
						angle = (this.angles[o.index] + this.angles[o.index + 1]) / 2 + startAngle;
					aroundRect.x = o.cx + o.cr * Math.cos(angle);
					aroundRect.y = o.cy + o.cr * Math.sin(angle);
					aroundRect.w = aroundRect.h = 1;
                    // depending on startAngle we might go out of the 0-2*PI range, normalize that
					if(startAngle && (angle < 0 || angle > 2 * Math.PI)){
						angle = Math.abs(2 * Math.PI  - Math.abs(angle));
					}
					// calculate the position
					if(angle < pi4){
						// do nothing: the position is right
					}else if(angle < pi2 + pi4){
						position = ["below-centered", "above-centered"];
					}else if(angle < Math.PI + pi4){
						position = ["before-centered", "after-centered"];
					}else if(angle < 2 * Math.PI - pi4){
						position = ["above-centered", "below-centered"];
					}
					/*
					else{
						// do nothing: the position is right
					}
					*/
					break;
			}
			
			// adjust relative coordinates to absolute, and remove fractions
			var lt = this.chart.getCoords();
			aroundRect.x += lt.x;
			aroundRect.y += lt.y;
			aroundRect.x = Math.round(aroundRect.x);
			aroundRect.y = Math.round(aroundRect.y);
			aroundRect.w = Math.ceil(aroundRect.w);
			aroundRect.h = Math.ceil(aroundRect.h);
			this.aroundRect = aroundRect;

			var tooltip = this.text(o);
			if(this.chart.getTextDir){
				var isChartDirectionRtl = (domStyle.get(this.chart.node, "direction") == "rtl");
				var isBaseTextDirRtl = (this.chart.getTextDir(tooltip) == "rtl");
			}
			if(tooltip){
				if(isBaseTextDirRtl && !isChartDirectionRtl){
					Tooltip.show("<span dir = 'rtl'>" + tooltip +"</span>", this.aroundRect, position);
				}
				else if(!isBaseTextDirRtl && isChartDirectionRtl){
					Tooltip.show("<span dir = 'ltr'>" + tooltip +"</span>", this.aroundRect, position);
				}else{
					Tooltip.show(tooltip, this.aroundRect, position);
				}
			}
		}
	});
});

},
'dojox/charting/action2d/PlotAction':function(){
define("dojox/charting/action2d/PlotAction", ["dojo/_base/connect", "dojo/_base/declare", "./Base", "dojo/fx/easing", "dojox/lang/functional", 
		"dojox/lang/functional/object"], 
	function(hub, declare, Base, dfe, df, dlfo){
	
	/*=====
	var __PlotActionCtorArgs = {
	 	// summary:
		//		The base keyword arguments object for creating an action2d.
		// duration: Number?
		//		The amount of time in milliseconds for an animation to last.  Default is 400.
		// easing: dojo/fx/easing/*?
		//		An easing object (see dojo.fx.easing) for use in an animation.  The
		//		default is dojo.fx.easing.backOut.
	};
	=====*/

	var DEFAULT_DURATION = 400,	// ms
		DEFAULT_EASING   = dfe.backOut;

	return declare("dojox.charting.action2d.PlotAction", Base, {
		// summary:
		//		Base action class for plot actions.

		overOutEvents: {onmouseover: 1, onmouseout: 1},

		constructor: function(chart, plot, kwargs){
			// summary:
			//		Create a new base PlotAction.
			// chart: dojox/charting/Chart
			//		The chart this action applies to.
			// plot: String?
			//		The name of the plot this action belongs to.  If none is passed "default" is assumed.
			// kwargs: __PlotActionCtorArgs?
			//		Optional arguments for the action.
			this.anim = {};

			// process common optional named parameters
			if(!kwargs){ kwargs = {}; }
			this.duration = kwargs.duration ? kwargs.duration : DEFAULT_DURATION;
			this.easing   = kwargs.easing   ? kwargs.easing   : DEFAULT_EASING;
		},

		connect: function(){
			// summary:
			//		Connect this action to the given plot.
			this.handle = this.chart.connectToPlot(this.plot.name, this, "process");
		},

		disconnect: function(){
			// summary:
			//		Disconnect this action from the given plot, if connected.
			if(this.handle){
				hub.disconnect(this.handle);
				this.handle = null;
			}
		},

		reset: function(){
			// summary:
			//		Reset the action.
		},

		destroy: function(){
			// summary:
			//		Do any cleanup needed when destroying parent elements.
			this.inherited(arguments);
			df.forIn(this.anim, function(o){
				df.forIn(o, function(anim){
					anim.action.stop(true);
				});
			});
			this.anim = {};
		}
	});
});

},
'dojox/charting/action2d/Base':function(){
define("dojox/charting/action2d/Base", ["dojo/_base/lang", "dojo/_base/declare"], 
	function(lang, declare){

	return declare("dojox.charting.action2d.Base", null, {
		// summary:
		//		Base action class for plot and chart actions.
	
		constructor: function(chart, plot){
			// summary:
			//		Create a new base action.  This can either be a plot or a chart action.
			// chart: dojox/charting/Chart
			//		The chart this action applies to.
			// plot: String|dojox/charting/plot2d/Base?
			//		Optional target plot for this action.  Default is "default".
			this.chart = chart;
			this.plot = plot ? (lang.isString(plot) ? this.chart.getPlot(plot) : plot) : this.chart.getPlot("default");
		},
	
		connect: function(){
			// summary:
			//		Connect this action to the plot or the chart.
		},
	
		disconnect: function(){
			// summary:
			//		Disconnect this action from the plot or the chart.
		},
		
		destroy: function(){
			// summary:
			//		Do any cleanup needed when destroying parent elements.
			this.disconnect();
		}
	});

});

},
'dojo/fx/easing':function(){
define("dojo/fx/easing", ["../_base/lang"], function(lang){

// module:
//		dojo/fx/easing

var easingFuncs = {
	// summary:
	//		Collection of easing functions to use beyond the default
	//		`dojo._defaultEasing` function.
	// description:
	//		Easing functions are used to manipulate the iteration through
	//		an `dojo.Animation`s _Line. _Line being the properties of an Animation,
	//		and the easing function progresses through that Line determining
	//		how quickly (or slowly) it should go. Or more accurately: modify
	//		the value of the _Line based on the percentage of animation completed.
	//
	//		All functions follow a simple naming convention of "ease type" + "when".
	//		If the name of the function ends in Out, the easing described appears
	//		towards the end of the animation. "In" means during the beginning,
	//		and InOut means both ranges of the Animation will applied, both
	//		beginning and end.
	//
	//		One does not call the easing function directly, it must be passed to
	//		the `easing` property of an animation.
	// example:
	//	|	dojo.require("dojo.fx.easing");
	//	|	var anim = dojo.fadeOut({
	//	|		node: 'node',
	//	|		duration: 2000,
	//	|		//	note there is no ()
	//	|		easing: dojo.fx.easing.quadIn
	//	|	}).play();
	//

	linear: function(/* Decimal? */n){
		// summary:
		//		A linear easing function
		return n;
	},

	quadIn: function(/* Decimal? */n){
		return Math.pow(n, 2);
	},

	quadOut: function(/* Decimal? */n){
		return n * (n - 2) * -1;
	},

	quadInOut: function(/* Decimal? */n){
		n = n * 2;
		if(n < 1){ return Math.pow(n, 2) / 2; }
		return -1 * ((--n) * (n - 2) - 1) / 2;
	},

	cubicIn: function(/* Decimal? */n){
		return Math.pow(n, 3);
	},

	cubicOut: function(/* Decimal? */n){
		return Math.pow(n - 1, 3) + 1;
	},

	cubicInOut: function(/* Decimal? */n){
		n = n * 2;
		if(n < 1){ return Math.pow(n, 3) / 2; }
		n -= 2;
		return (Math.pow(n, 3) + 2) / 2;
	},

	quartIn: function(/* Decimal? */n){
		return Math.pow(n, 4);
	},

	quartOut: function(/* Decimal? */n){
		return -1 * (Math.pow(n - 1, 4) - 1);
	},

	quartInOut: function(/* Decimal? */n){
		n = n * 2;
		if(n < 1){ return Math.pow(n, 4) / 2; }
		n -= 2;
		return -1 / 2 * (Math.pow(n, 4) - 2);
	},

	quintIn: function(/* Decimal? */n){
		return Math.pow(n, 5);
	},

	quintOut: function(/* Decimal? */n){
		return Math.pow(n - 1, 5) + 1;
	},

	quintInOut: function(/* Decimal? */n){
		n = n * 2;
		if(n < 1){ return Math.pow(n, 5) / 2; }
		n -= 2;
		return (Math.pow(n, 5) + 2) / 2;
	},

	sineIn: function(/* Decimal? */n){
		return -1 * Math.cos(n * (Math.PI / 2)) + 1;
	},

	sineOut: function(/* Decimal? */n){
		return Math.sin(n * (Math.PI / 2));
	},

	sineInOut: function(/* Decimal? */n){
		return -1 * (Math.cos(Math.PI * n) - 1) / 2;
	},

	expoIn: function(/* Decimal? */n){
		return (n == 0) ? 0 : Math.pow(2, 10 * (n - 1));
	},

	expoOut: function(/* Decimal? */n){
		return (n == 1) ? 1 : (-1 * Math.pow(2, -10 * n) + 1);
	},

	expoInOut: function(/* Decimal? */n){
		if(n == 0){ return 0; }
		if(n == 1){ return 1; }
		n = n * 2;
		if(n < 1){ return Math.pow(2, 10 * (n - 1)) / 2; }
		--n;
		return (-1 * Math.pow(2, -10 * n) + 2) / 2;
	},

	circIn: function(/* Decimal? */n){
		return -1 * (Math.sqrt(1 - Math.pow(n, 2)) - 1);
	},

	circOut: function(/* Decimal? */n){
		n = n - 1;
		return Math.sqrt(1 - Math.pow(n, 2));
	},

	circInOut: function(/* Decimal? */n){
		n = n * 2;
		if(n < 1){ return -1 / 2 * (Math.sqrt(1 - Math.pow(n, 2)) - 1); }
		n -= 2;
		return 1 / 2 * (Math.sqrt(1 - Math.pow(n, 2)) + 1);
	},

	backIn: function(/* Decimal? */n){
		// summary:
		//		An easing function that starts away from the target,
		//		and quickly accelerates towards the end value.
		//
		//		Use caution when the easing will cause values to become
		//		negative as some properties cannot be set to negative values.
		var s = 1.70158;
		return Math.pow(n, 2) * ((s + 1) * n - s);
	},

	backOut: function(/* Decimal? */n){
		// summary:
		//		An easing function that pops past the range briefly, and slowly comes back.
		// description:
		//		An easing function that pops past the range briefly, and slowly comes back.
		//
		//		Use caution when the easing will cause values to become negative as some
		//		properties cannot be set to negative values.

		n = n - 1;
		var s = 1.70158;
		return Math.pow(n, 2) * ((s + 1) * n + s) + 1;
	},

	backInOut: function(/* Decimal? */n){
		// summary:
		//		An easing function combining the effects of `backIn` and `backOut`
		// description:
		//		An easing function combining the effects of `backIn` and `backOut`.
		//		Use caution when the easing will cause values to become negative
		//		as some properties cannot be set to negative values.
		var s = 1.70158 * 1.525;
		n = n * 2;
		if(n < 1){ return (Math.pow(n, 2) * ((s + 1) * n - s)) / 2; }
		n-=2;
		return (Math.pow(n, 2) * ((s + 1) * n + s) + 2) / 2;
	},

	elasticIn: function(/* Decimal? */n){
		// summary:
		//		An easing function the elastically snaps from the start value
		// description:
		//		An easing function the elastically snaps from the start value
		//
		//		Use caution when the elasticity will cause values to become negative
		//		as some properties cannot be set to negative values.
		if(n == 0 || n == 1){ return n; }
		var p = .3;
		var s = p / 4;
		n = n - 1;
		return -1 * Math.pow(2, 10 * n) * Math.sin((n - s) * (2 * Math.PI) / p);
	},

	elasticOut: function(/* Decimal? */n){
		// summary:
		//		An easing function that elasticly snaps around the target value,
		//		near the end of the Animation
		// description:
		//		An easing function that elasticly snaps around the target value,
		//		near the end of the Animation
		//
		//		Use caution when the elasticity will cause values to become
		//		negative as some properties cannot be set to negative values.
		if(n==0 || n == 1){ return n; }
		var p = .3;
		var s = p / 4;
		return Math.pow(2, -10 * n) * Math.sin((n - s) * (2 * Math.PI) / p) + 1;
	},

	elasticInOut: function(/* Decimal? */n){
		// summary:
		//		An easing function that elasticly snaps around the value, near
		//		the beginning and end of the Animation.
		// description:
		//		An easing function that elasticly snaps around the value, near
		//		the beginning and end of the Animation.
		//
		//		Use caution when the elasticity will cause values to become
		//		negative as some properties cannot be set to negative values.
		if(n == 0) return 0;
		n = n * 2;
		if(n == 2) return 1;
		var p = .3 * 1.5;
		var s = p / 4;
		if(n < 1){
			n -= 1;
			return -.5 * (Math.pow(2, 10 * n) * Math.sin((n - s) * (2 * Math.PI) / p));
		}
		n -= 1;
		return .5 * (Math.pow(2, -10 * n) * Math.sin((n - s) * (2 * Math.PI) / p)) + 1;
	},

	bounceIn: function(/* Decimal? */n){
		// summary:
		//		An easing function that 'bounces' near the beginning of an Animation
		return (1 - easingFuncs.bounceOut(1 - n)); // Decimal
	},

	bounceOut: function(/* Decimal? */n){
		// summary:
		//		An easing function that 'bounces' near the end of an Animation
		var s = 7.5625;
		var p = 2.75;
		var l;
		if(n < (1 / p)){
			l = s * Math.pow(n, 2);
		}else if(n < (2 / p)){
			n -= (1.5 / p);
			l = s * Math.pow(n, 2) + .75;
		}else if(n < (2.5 / p)){
			n -= (2.25 / p);
			l = s * Math.pow(n, 2) + .9375;
		}else{
			n -= (2.625 / p);
			l = s * Math.pow(n, 2) + .984375;
		}
		return l;
	},

	bounceInOut: function(/* Decimal? */n){
		// summary:
		//		An easing function that 'bounces' at the beginning and end of the Animation
		if(n < 0.5){ return easingFuncs.bounceIn(n * 2) / 2; }
		return (easingFuncs.bounceOut(n * 2 - 1) / 2) + 0.5; // Decimal
	}
};

lang.setObject("dojo.fx.easing", easingFuncs);

return easingFuncs;
});

},
'dojox/lang/functional/scan':function(){
define("dojox/lang/functional/scan", ["dojo/_base/kernel", "dojo/_base/lang", "./lambda"], function(kernel, lang, df){

// This module adds high-level functions and related constructs:
//	- "scan" family of functions

// Notes:
//	- missing high-level functions are provided with the compatible API:
//		scanl, scanl1, scanr, scanr1

// Defined methods:
//	- take any valid lambda argument as the functional argument
//	- operate on dense arrays
//	- take a string as the array argument
//	- take an iterator objects as the array argument (only scanl, and scanl1)

	var empty = {};

	lang.mixin(df, {
		// classic reduce-class functions
		scanl: function(/*Array|String|Object*/ a, /*Function|String|Array*/ f, /*Object*/ z, /*Object?*/ o){
			// summary:
			//		repeatedly applies a binary function to an array from left
			//		to right using a seed value as a starting point; returns an array
			//		of values produced by foldl() at that point.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || kernel.global; f = df.lambda(f);
			var t, n, i;
			if(lang.isArray(a)){
				// array
				t = new Array((n = a.length) + 1);
				t[0] = z;
				for(i = 0; i < n; z = f.call(o, z, a[i], i, a), t[++i] = z);
			}else if(typeof a.hasNext == "function" && typeof a.next == "function"){
				// iterator
				t = [z];
				for(i = 0; a.hasNext(); t.push(z = f.call(o, z, a.next(), i++, a)));
			}else{
				// object/dictionary
				t = [z];
				for(i in a){
					if(!(i in empty)){
						t.push(z = f.call(o, z, a[i], i, a));
					}
				}
			}
			return t;	// Array
		},
		scanl1: function(/*Array|String|Object*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary:
			//		repeatedly applies a binary function to an array from left
			//		to right; returns an array of values produced by foldl1() at that
			//		point.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || kernel.global; f = df.lambda(f);
			var t, n, z, first = true;
			if(lang.isArray(a)){
				// array
				t = new Array(n = a.length);
				t[0] = z = a[0];
				for(var i = 1; i < n; t[i] = z = f.call(o, z, a[i], i, a), ++i);
			}else if(typeof a.hasNext == "function" && typeof a.next == "function"){
				// iterator
				if(a.hasNext()){
					t = [z = a.next()];
					for(i = 1; a.hasNext(); t.push(z = f.call(o, z, a.next(), i++, a)));
				}
			}else{
				// object/dictionary
				for(i in a){
					if(!(i in empty)){
						if(first){
							t = [z = a[i]];
							first = false;
						}else{
							t.push(z = f.call(o, z, a[i], i, a));
						}
					}
				}
			}
			return t;	// Array
		},
		scanr: function(/*Array|String*/ a, /*Function|String|Array*/ f, /*Object*/ z, /*Object?*/ o){
			// summary:
			//		repeatedly applies a binary function to an array from right
			//		to left using a seed value as a starting point; returns an array
			//		of values produced by foldr() at that point.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || kernel.global; f = df.lambda(f);
			var n = a.length, t = new Array(n + 1), i = n;
			t[n] = z;
			for(; i > 0; --i, z = f.call(o, z, a[i], i, a), t[i] = z);
			return t;	// Array
		},
		scanr1: function(/*Array|String*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary:
			//		repeatedly applies a binary function to an array from right
			//		to left; returns an array of values produced by foldr1() at that
			//		point.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || kernel.global; f = df.lambda(f);
			var n = a.length, t = new Array(n), z = a[n - 1], i = n - 1;
			t[i] = z;
			for(; i > 0; --i, z = f.call(o, z, a[i], i, a), t[i] = z);
			return t;	// Array
		}
	});
});

},
'dojox/charting/action2d/Magnify':function(){
define("dojox/charting/action2d/Magnify", ["dojo/_base/connect", "dojo/_base/declare", 
	"./PlotAction", "dojox/gfx/matrix", 
	"dojox/gfx/fx", "dojo/fx", "dojo/fx/easing"], 
	function(Hub, declare, PlotAction, m, gf, df, dfe){

	/*=====
	var __MagnifyCtorArgs = {
		// summary:
		//		Additional arguments for magnifying actions.
		// duration: Number?
		//		The amount of time in milliseconds for an animation to last.  Default is 400.
		// easing: dojo/fx/easing/*?
		//		An easing object (see dojo.fx.easing) for use in an animation.  The
		//		default is dojo.fx.easing.backOut.
		// scale: Number?
		//		The amount to magnify the given object to.  Default is 2.
	};
	=====*/
	
	var DEFAULT_SCALE = 2;

	return declare("dojox.charting.action2d.Magnify", PlotAction, {
		// summary:
		//		Create an action that magnifies the object the action is applied to.

		// the data description block for the widget parser
		defaultParams: {
			duration: 400,	// duration of the action in ms
			easing:   dfe.backOut,	// easing for the action
			scale:    DEFAULT_SCALE	// scale of magnification
		},
		optionalParams: {},	// no optional parameters

		constructor: function(chart, plot, kwArgs){
			// summary:
			//		Create the magnifying action.
			// chart: dojox/charting/Chart
			//		The chart this action belongs to.
			// plot: String?
			//		The plot to apply the action to. If not passed, "default" is assumed.
			// kwArgs: __MagnifyCtorArgs?
			//		Optional keyword arguments for this action.

			// process optional named parameters
			this.scale = kwArgs && typeof kwArgs.scale == "number" ? kwArgs.scale : DEFAULT_SCALE;

			this.connect();
		},

		process: function(o){
			// summary:
			//		Process the action on the given object.
			// o: dojox/gfx/shape.Shape
			//		The object on which to process the magnifying action.
			if(!o.shape || !(o.type in this.overOutEvents) ||
				!("cx" in o) || !("cy" in o)){ return; }

			var runName = o.run.name, index = o.index, vector = [], anim, init, scale;

			if(runName in this.anim){
				anim = this.anim[runName][index];
			}else{
				this.anim[runName] = {};
			}

			if(anim){
				anim.action.stop(true);
			}else{
				this.anim[runName][index] = anim = {};
			}

			if(o.type == "onmouseover"){
				init  = m.identity;
				scale = this.scale;
			}else{
				init  = m.scaleAt(this.scale, o.cx, o.cy);
				scale = 1 / this.scale;
			}

			var kwArgs = {
				shape:    o.shape,
				duration: this.duration,
				easing:   this.easing,
				transform: [
					{name: "scaleAt", start: [1, o.cx, o.cy], end: [scale, o.cx, o.cy]},
					init
				]
			};
			if(o.shape){
				vector.push(gf.animateTransform(kwArgs));
			}
			if(o.outline){
				kwArgs.shape = o.outline;
				vector.push(gf.animateTransform(kwArgs));
			}
			if(o.shadow){
				kwArgs.shape = o.shadow;
				vector.push(gf.animateTransform(kwArgs));
			}

			if(!vector.length){
				delete this.anim[runName][index];
				return;
			}

			anim.action = df.combine(vector);
			if(o.type == "onmouseout"){
				Hub.connect(anim.action, "onEnd", this, function(){
					if(this.anim[runName]){
						delete this.anim[runName][index];
					}
				});
			}
			anim.action.play();
		}
	});
	
});

},
'dojox/gfx/fx':function(){
define("dojox/gfx/fx", ["dojo/_base/lang", "./_base", "./matrix", "dojo/_base/Color", "dojo/_base/array", "dojo/_base/fx", "dojo/_base/connect"], 
  function(lang, g, m, Color, arr, fx, Hub){
	var fxg = g.fx = {};

	// Generic interpolators. Should they be moved to dojox.fx?

	function InterpolNumber(start, end){
		this.start = start, this.end = end;
	}
	InterpolNumber.prototype.getValue = function(r){
		return (this.end - this.start) * r + this.start;
	};

	function InterpolUnit(start, end, units){
		this.start = start, this.end = end;
		this.units = units;
	}
	InterpolUnit.prototype.getValue = function(r){
		return (this.end - this.start) * r + this.start + this.units;
	};

	function InterpolColor(start, end){
		this.start = start, this.end = end;
		this.temp = new Color();
	}
	InterpolColor.prototype.getValue = function(r){
		return Color.blendColors(this.start, this.end, r, this.temp);
	};

	function InterpolValues(values){
		this.values = values;
		this.length = values.length;
	}
	InterpolValues.prototype.getValue = function(r){
		return this.values[Math.min(Math.floor(r * this.length), this.length - 1)];
	};

	function InterpolObject(values, def){
		this.values = values;
		this.def = def ? def : {};
	}
	InterpolObject.prototype.getValue = function(r){
		var ret = lang.clone(this.def);
		for(var i in this.values){
			ret[i] = this.values[i].getValue(r);
		}
		return ret;
	};

	function InterpolTransform(stack, original){
		this.stack = stack;
		this.original = original;
	}
	InterpolTransform.prototype.getValue = function(r){
		var ret = [];
		arr.forEach(this.stack, function(t){
			if(t instanceof m.Matrix2D){
				ret.push(t);
				return;
			}
			if(t.name == "original" && this.original){
				ret.push(this.original);
				return;
			}
 			// Adding support for custom matrices
 			if(t.name == "matrix"){
 				if((t.start instanceof m.Matrix2D) && (t.end instanceof m.Matrix2D)){
 					var transfMatrix = new m.Matrix2D();
 					for(var p in t.start) {
 						transfMatrix[p] = (t.end[p] - t.start[p])*r + t.start[p];
 					}
 					ret.push(transfMatrix);
 				}
 				return;
 			}
			if(!(t.name in m)){ return; }
			var f = m[t.name];
			if(typeof f != "function"){
				// constant
				ret.push(f);
				return;
			}
			var val = arr.map(t.start, function(v, i){
							return (t.end[i] - v) * r + v;
						}),
				matrix = f.apply(m, val);
			if(matrix instanceof m.Matrix2D){
				ret.push(matrix);
			}
		}, this);
		return ret;
	};

	var transparent = new Color(0, 0, 0, 0);

	function getColorInterpol(prop, obj, name, def){
		if(prop.values){
			return new InterpolValues(prop.values);
		}
		var value, start, end;
		if(prop.start){
			start = g.normalizeColor(prop.start);
		}else{
			start = value = obj ? (name ? obj[name] : obj) : def;
		}
		if(prop.end){
			end = g.normalizeColor(prop.end);
		}else{
			if(!value){
				value = obj ? (name ? obj[name] : obj) : def;
			}
			end = value;
		}
		return new InterpolColor(start, end);
	}

	function getNumberInterpol(prop, obj, name, def){
		if(prop.values){
			return new InterpolValues(prop.values);
		}
		var value, start, end;
		if(prop.start){
			start = prop.start;
		}else{
			start = value = obj ? obj[name] : def;
		}
		if(prop.end){
			end = prop.end;
		}else{
			if(typeof value != "number"){
				value = obj ? obj[name] : def;
			}
			end = value;
		}
		return new InterpolNumber(start, end);
	}

	fxg.animateStroke = function(/*Object*/ args){
		// summary:
		//		Returns an animation which will change stroke properties over time.
		// args:
		//		an object defining the animation setting.
		// example:
		//	|	fxg.animateStroke{{
		//	|		shape: shape,
		//	|		duration: 500,
		//	|		color: {start: "red", end: "green"},
		//	|		width: {end: 15},
		//	|		join:  {values: ["miter", "bevel", "round"]}
		//	|	}).play();
		if(!args.easing){ args.easing = fx._defaultEasing; }
		var anim = new fx.Animation(args), shape = args.shape, stroke;
		Hub.connect(anim, "beforeBegin", anim, function(){
			stroke = shape.getStroke();
			var prop = args.color, values = {}, value, start, end;
			if(prop){
				values.color = getColorInterpol(prop, stroke, "color", transparent);
			}
			prop = args.style;
			if(prop && prop.values){
				values.style = new InterpolValues(prop.values);
			}
			prop = args.width;
			if(prop){
				values.width = getNumberInterpol(prop, stroke, "width", 1);
			}
			prop = args.cap;
			if(prop && prop.values){
				values.cap = new InterpolValues(prop.values);
			}
			prop = args.join;
			if(prop){
				if(prop.values){
					values.join = new InterpolValues(prop.values);
				}else{
					start = prop.start ? prop.start : (stroke && stroke.join || 0);
					end = prop.end ? prop.end : (stroke && stroke.join || 0);
					if(typeof start == "number" && typeof end == "number"){
						values.join = new InterpolNumber(start, end);
					}
				}
			}
			this.curve = new InterpolObject(values, stroke);
		});
		Hub.connect(anim, "onAnimate", shape, "setStroke");
		return anim; // dojo.Animation
	};

	fxg.animateFill = function(/*Object*/ args){
		// summary:
		//		Returns an animation which will change fill color over time.
		//		Only solid fill color is supported at the moment
		// args:
		//		an object defining the animation setting.
		// example:
		//	|	gfx.animateFill{{
		//	|		shape: shape,
		//	|		duration: 500,
		//	|		color: {start: "red", end: "green"}
		//	|	}).play();
		if(!args.easing){ args.easing = fx._defaultEasing; }
		var anim = new fx.Animation(args), shape = args.shape, fill;
		Hub.connect(anim, "beforeBegin", anim, function(){
			fill = shape.getFill();
			var prop = args.color, values = {};
			if(prop){
				this.curve = getColorInterpol(prop, fill, "", transparent);
			}
		});
		Hub.connect(anim, "onAnimate", shape, "setFill");
		return anim; // dojo.Animation
	};

	fxg.animateFont = function(/*Object*/ args){
		// summary:
		//		Returns an animation which will change font properties over time.
		// args:
		//		an object defining the animation setting.
		// example:
		//	|	gfx.animateFont{{
		//	|		shape: shape,
		//	|		duration: 500,
		//	|		variant: {values: ["normal", "small-caps"]},
		//	|		size:  {end: 10, units: "pt"}
		//	|	}).play();
		if(!args.easing){ args.easing = fx._defaultEasing; }
		var anim = new fx.Animation(args), shape = args.shape, font;
		Hub.connect(anim, "beforeBegin", anim, function(){
			font = shape.getFont();
			var prop = args.style, values = {}, value, start, end;
			if(prop && prop.values){
				values.style = new InterpolValues(prop.values);
			}
			prop = args.variant;
			if(prop && prop.values){
				values.variant = new InterpolValues(prop.values);
			}
			prop = args.weight;
			if(prop && prop.values){
				values.weight = new InterpolValues(prop.values);
			}
			prop = args.family;
			if(prop && prop.values){
				values.family = new InterpolValues(prop.values);
			}
			prop = args.size;
			if(prop && prop.units){
				start = parseFloat(prop.start ? prop.start : (shape.font && shape.font.size || "0"));
				end = parseFloat(prop.end ? prop.end : (shape.font && shape.font.size || "0"));
				values.size = new InterpolUnit(start, end, prop.units);
			}
			this.curve = new InterpolObject(values, font);
		});
		Hub.connect(anim, "onAnimate", shape, "setFont");
		return anim; // dojo.Animation
	};

	fxg.animateTransform = function(/*Object*/ args){
		// summary:
		//		Returns an animation which will change transformation over time.
		// args:
		//		an object defining the animation setting.
		// example:
		//	|	gfx.animateTransform{{
		//	|		shape: shape,
		//	|		duration: 500,
		//	|		transform: [
		//	|			{name: "translate", start: [0, 0], end: [200, 200]},
		//	|			{name: "original"}
		//	|		]
		//	|	}).play();
		if(!args.easing){ args.easing = fx._defaultEasing; }
		var anim = new fx.Animation(args), shape = args.shape, original;
		Hub.connect(anim, "beforeBegin", anim, function(){
			original = shape.getTransform();
			this.curve = new InterpolTransform(args.transform, original);
		});
		Hub.connect(anim, "onAnimate", shape, "setTransform");
		return anim; // dojo.Animation
	};
	
	return fxg;
});

},
'dojo/fx':function(){
define("dojo/fx", [
	"./_base/lang",
	"./Evented",
	"./_base/kernel",
	"./_base/array",
	"./_base/connect",
	"./_base/fx",
	"./dom",
	"./dom-style",
	"./dom-geometry",
	"./ready",
	"require" // for context sensitive loading of Toggler
], function(lang, Evented, dojo, arrayUtil, connect, baseFx, dom, domStyle, geom, ready, require){

	// module:
	//		dojo/fx
	
	// For back-compat, remove in 2.0.
	if(!dojo.isAsync){
		ready(0, function(){
			var requires = ["./fx/Toggler"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	var coreFx = dojo.fx = {
		// summary:
		//		Effects library on top of Base animations
	};

	var _baseObj = {
			_fire: function(evt, args){
				if(this[evt]){
					this[evt].apply(this, args||[]);
				}
				return this;
			}
		};

	var _chain = function(animations){
		this._index = -1;
		this._animations = animations||[];
		this._current = this._onAnimateCtx = this._onEndCtx = null;

		this.duration = 0;
		arrayUtil.forEach(this._animations, function(a){
			this.duration += a.duration;
			if(a.delay){ this.duration += a.delay; }
		}, this);
	};
	_chain.prototype = new Evented();
	lang.extend(_chain, {
		_onAnimate: function(){
			this._fire("onAnimate", arguments);
		},
		_onEnd: function(){
			connect.disconnect(this._onAnimateCtx);
			connect.disconnect(this._onEndCtx);
			this._onAnimateCtx = this._onEndCtx = null;
			if(this._index + 1 == this._animations.length){
				this._fire("onEnd");
			}else{
				// switch animations
				this._current = this._animations[++this._index];
				this._onAnimateCtx = connect.connect(this._current, "onAnimate", this, "_onAnimate");
				this._onEndCtx = connect.connect(this._current, "onEnd", this, "_onEnd");
				this._current.play(0, true);
			}
		},
		play: function(/*int?*/ delay, /*Boolean?*/ gotoStart){
			if(!this._current){ this._current = this._animations[this._index = 0]; }
			if(!gotoStart && this._current.status() == "playing"){ return this; }
			var beforeBegin = connect.connect(this._current, "beforeBegin", this, function(){
					this._fire("beforeBegin");
				}),
				onBegin = connect.connect(this._current, "onBegin", this, function(arg){
					this._fire("onBegin", arguments);
				}),
				onPlay = connect.connect(this._current, "onPlay", this, function(arg){
					this._fire("onPlay", arguments);
					connect.disconnect(beforeBegin);
					connect.disconnect(onBegin);
					connect.disconnect(onPlay);
				});
			if(this._onAnimateCtx){
				connect.disconnect(this._onAnimateCtx);
			}
			this._onAnimateCtx = connect.connect(this._current, "onAnimate", this, "_onAnimate");
			if(this._onEndCtx){
				connect.disconnect(this._onEndCtx);
			}
			this._onEndCtx = connect.connect(this._current, "onEnd", this, "_onEnd");
			this._current.play.apply(this._current, arguments);
			return this;
		},
		pause: function(){
			if(this._current){
				var e = connect.connect(this._current, "onPause", this, function(arg){
						this._fire("onPause", arguments);
						connect.disconnect(e);
					});
				this._current.pause();
			}
			return this;
		},
		gotoPercent: function(/*Decimal*/percent, /*Boolean?*/ andPlay){
			this.pause();
			var offset = this.duration * percent;
			this._current = null;
			arrayUtil.some(this._animations, function(a){
				if(a.duration <= offset){
					this._current = a;
					return true;
				}
				offset -= a.duration;
				return false;
			});
			if(this._current){
				this._current.gotoPercent(offset / this._current.duration, andPlay);
			}
			return this;
		},
		stop: function(/*boolean?*/ gotoEnd){
			if(this._current){
				if(gotoEnd){
					for(; this._index + 1 < this._animations.length; ++this._index){
						this._animations[this._index].stop(true);
					}
					this._current = this._animations[this._index];
				}
				var e = connect.connect(this._current, "onStop", this, function(arg){
						this._fire("onStop", arguments);
						connect.disconnect(e);
					});
				this._current.stop();
			}
			return this;
		},
		status: function(){
			return this._current ? this._current.status() : "stopped";
		},
		destroy: function(){
			if(this._onAnimateCtx){ connect.disconnect(this._onAnimateCtx); }
			if(this._onEndCtx){ connect.disconnect(this._onEndCtx); }
		}
	});
	lang.extend(_chain, _baseObj);

	coreFx.chain = function(/*dojo/_base/fx.Animation[]*/ animations){
		// summary:
		//		Chain a list of `dojo.Animation`s to run in sequence
		//
		// description:
		//		Return a `dojo.Animation` which will play all passed
		//		`dojo.Animation` instances in sequence, firing its own
		//		synthesized events simulating a single animation. (eg:
		//		onEnd of this animation means the end of the chain,
		//		not the individual animations within)
		//
		// example:
		//	Once `node` is faded out, fade in `otherNode`
		//	|	fx.chain([
		//	|		dojo.fadeIn({ node:node }),
		//	|		dojo.fadeOut({ node:otherNode })
		//	|	]).play();
		//
		return new _chain(animations); // dojo/_base/fx.Animation
	};

	var _combine = function(animations){
		this._animations = animations||[];
		this._connects = [];
		this._finished = 0;

		this.duration = 0;
		arrayUtil.forEach(animations, function(a){
			var duration = a.duration;
			if(a.delay){ duration += a.delay; }
			if(this.duration < duration){ this.duration = duration; }
			this._connects.push(connect.connect(a, "onEnd", this, "_onEnd"));
		}, this);

		this._pseudoAnimation = new baseFx.Animation({curve: [0, 1], duration: this.duration});
		var self = this;
		arrayUtil.forEach(["beforeBegin", "onBegin", "onPlay", "onAnimate", "onPause", "onStop", "onEnd"],
			function(evt){
				self._connects.push(connect.connect(self._pseudoAnimation, evt,
					function(){ self._fire(evt, arguments); }
				));
			}
		);
	};
	lang.extend(_combine, {
		_doAction: function(action, args){
			arrayUtil.forEach(this._animations, function(a){
				a[action].apply(a, args);
			});
			return this;
		},
		_onEnd: function(){
			if(++this._finished > this._animations.length){
				this._fire("onEnd");
			}
		},
		_call: function(action, args){
			var t = this._pseudoAnimation;
			t[action].apply(t, args);
		},
		play: function(/*int?*/ delay, /*Boolean?*/ gotoStart){
			this._finished = 0;
			this._doAction("play", arguments);
			this._call("play", arguments);
			return this;
		},
		pause: function(){
			this._doAction("pause", arguments);
			this._call("pause", arguments);
			return this;
		},
		gotoPercent: function(/*Decimal*/percent, /*Boolean?*/ andPlay){
			var ms = this.duration * percent;
			arrayUtil.forEach(this._animations, function(a){
				a.gotoPercent(a.duration < ms ? 1 : (ms / a.duration), andPlay);
			});
			this._call("gotoPercent", arguments);
			return this;
		},
		stop: function(/*boolean?*/ gotoEnd){
			this._doAction("stop", arguments);
			this._call("stop", arguments);
			return this;
		},
		status: function(){
			return this._pseudoAnimation.status();
		},
		destroy: function(){
			arrayUtil.forEach(this._connects, connect.disconnect);
		}
	});
	lang.extend(_combine, _baseObj);

	coreFx.combine = function(/*dojo/_base/fx.Animation[]*/ animations){
		// summary:
		//		Combine a list of `dojo.Animation`s to run in parallel
		//
		// description:
		//		Combine an array of `dojo.Animation`s to run in parallel,
		//		providing a new `dojo.Animation` instance encompasing each
		//		animation, firing standard animation events.
		//
		// example:
		//	Fade out `node` while fading in `otherNode` simultaneously
		//	|	fx.combine([
		//	|		dojo.fadeIn({ node:node }),
		//	|		dojo.fadeOut({ node:otherNode })
		//	|	]).play();
		//
		// example:
		//	When the longest animation ends, execute a function:
		//	|	var anim = fx.combine([
		//	|		dojo.fadeIn({ node: n, duration:700 }),
		//	|		dojo.fadeOut({ node: otherNode, duration: 300 })
		//	|	]);
		//	|	dojo.connect(anim, "onEnd", function(){
		//	|		// overall animation is done.
		//	|	});
		//	|	anim.play(); // play the animation
		//
		return new _combine(animations); // dojo/_base/fx.Animation
	};

	coreFx.wipeIn = function(/*Object*/ args){
		// summary:
		//		Expand a node to it's natural height.
		//
		// description:
		//		Returns an animation that will expand the
		//		node defined in 'args' object from it's current height to
		//		it's natural height (with no scrollbar).
		//		Node must have no margin/border/padding.
		//
		// args: Object
		//		A hash-map of standard `dojo.Animation` constructor properties
		//		(such as easing: node: duration: and so on)
		//
		// example:
		//	|	fx.wipeIn({
		//	|		node:"someId"
		//	|	}).play()
		var node = args.node = dom.byId(args.node), s = node.style, o;

		var anim = baseFx.animateProperty(lang.mixin({
			properties: {
				height: {
					// wrapped in functions so we wait till the last second to query (in case value has changed)
					start: function(){
						// start at current [computed] height, but use 1px rather than 0
						// because 0 causes IE to display the whole panel
						o = s.overflow;
						s.overflow = "hidden";
						if(s.visibility == "hidden" || s.display == "none"){
							s.height = "1px";
							s.display = "";
							s.visibility = "";
							return 1;
						}else{
							var height = domStyle.get(node, "height");
							return Math.max(height, 1);
						}
					},
					end: function(){
						return node.scrollHeight;
					}
				}
			}
		}, args));

		var fini = function(){
			s.height = "auto";
			s.overflow = o;
		};
		connect.connect(anim, "onStop", fini);
		connect.connect(anim, "onEnd", fini);

		return anim; // dojo/_base/fx.Animation
	};

	coreFx.wipeOut = function(/*Object*/ args){
		// summary:
		//		Shrink a node to nothing and hide it.
		//
		// description:
		//		Returns an animation that will shrink node defined in "args"
		//		from it's current height to 1px, and then hide it.
		//
		// args: Object
		//		A hash-map of standard `dojo.Animation` constructor properties
		//		(such as easing: node: duration: and so on)
		//
		// example:
		//	|	fx.wipeOut({ node:"someId" }).play()

		var node = args.node = dom.byId(args.node), s = node.style, o;

		var anim = baseFx.animateProperty(lang.mixin({
			properties: {
				height: {
					end: 1 // 0 causes IE to display the whole panel
				}
			}
		}, args));

		connect.connect(anim, "beforeBegin", function(){
			o = s.overflow;
			s.overflow = "hidden";
			s.display = "";
		});
		var fini = function(){
			s.overflow = o;
			s.height = "auto";
			s.display = "none";
		};
		connect.connect(anim, "onStop", fini);
		connect.connect(anim, "onEnd", fini);

		return anim; // dojo/_base/fx.Animation
	};

	coreFx.slideTo = function(/*Object*/ args){
		// summary:
		//		Slide a node to a new top/left position
		//
		// description:
		//		Returns an animation that will slide "node"
		//		defined in args Object from its current position to
		//		the position defined by (args.left, args.top).
		//
		// args: Object
		//		A hash-map of standard `dojo.Animation` constructor properties
		//		(such as easing: node: duration: and so on). Special args members
		//		are `top` and `left`, which indicate the new position to slide to.
		//
		// example:
		//	|	.slideTo({ node: node, left:"40", top:"50", units:"px" }).play()

		var node = args.node = dom.byId(args.node),
			top = null, left = null;

		var init = (function(n){
			return function(){
				var cs = domStyle.getComputedStyle(n);
				var pos = cs.position;
				top = (pos == 'absolute' ? n.offsetTop : parseInt(cs.top) || 0);
				left = (pos == 'absolute' ? n.offsetLeft : parseInt(cs.left) || 0);
				if(pos != 'absolute' && pos != 'relative'){
					var ret = geom.position(n, true);
					top = ret.y;
					left = ret.x;
					n.style.position="absolute";
					n.style.top=top+"px";
					n.style.left=left+"px";
				}
			};
		})(node);
		init();

		var anim = baseFx.animateProperty(lang.mixin({
			properties: {
				top: args.top || 0,
				left: args.left || 0
			}
		}, args));
		connect.connect(anim, "beforeBegin", anim, init);

		return anim; // dojo/_base/fx.Animation
	};

	return coreFx;
});

},
'dojox/charting/StoreSeries':function(){
define("dojox/charting/StoreSeries", ["dojo/_base/array", "dojo/_base/declare", "dojo/_base/Deferred"], 
  function(arr, declare, Deferred){
	
	return declare("dojox.charting.StoreSeries", null, {
		constructor: function(store, kwArgs, value){
			// summary:
			//		Series adapter for dojo object stores (dojo.store).
			// store: Object
			//		A dojo object store.
			// kwArgs: Object
			//		A store-specific keyword parameters used for querying objects.
			//		See dojo.store docs
			// value: Function|Object|String
			//		Function, which takes an object handle, and
			//		produces an output possibly inspecting the store's item. Or
			//		a dictionary object, which tells what names to extract from
			//		an object and how to map them to an output. Or a string, which
			//		is a numeric field name to use for plotting. If undefined, null
			//		or empty string (the default), "value" field is extracted.
			this.store = store;
			this.kwArgs = kwArgs;
	
			if(value){
				if(typeof value == "function"){
					this.value = value;
				}else if(typeof value == "object"){
					this.value = function(object){
						var o = {};
						for(var key in value){
							o[key] = object[value[key]];
						}
						return o;
					};
				}else{
					this.value = function(object){
						return object[value];
					};
				}
			}else{
				this.value = function(object){
					return object.value;
				};
			}
	
			this.data = [];

			this._initialRendering = true;
			this.fetch();
		},
	
		destroy: function(){
			// summary:
			//		Clean up before GC.
			if(this.observeHandle){
				this.observeHandle.remove();
			}
		},
	
		setSeriesObject: function(series){
			// summary:
			//		Sets a dojox.charting.Series object we will be working with.
			// series: dojox/charting/Series
			//		Our interface to the chart.
			this.series = series;
		},
	
		// store fetch loop
	
		fetch: function(){
			// summary:
			//		Fetches data from the store and updates a chart.
			var objects = this.objects = [];
			var self = this;
			if(this.observeHandle){
				this.observeHandle.remove();
			}
			var results = this.store.query(this.kwArgs.query, this.kwArgs);
			Deferred.when(results, function(objects){
				self.objects = objects;
				update();
			});
			if(results.observe){
				this.observeHandle = results.observe(update, true);
			}
			function update(){
				self.data = arr.map(self.objects, function(object){
					return self.value(object, self.store);
				});
				self._pushDataChanges();
			}
		},
	
		_pushDataChanges: function(){
			if(this.series){
				this.series.chart.updateSeries(this.series.name, this, this._initialRendering);
				this._initialRendering = false;
				this.series.chart.delayedRender();
			}
		}
	
	});
});

},
'epi-saas-base/Sticker':function(){
define("epi-saas-base/Sticker", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/window",
    "dojo/dom-class",
    "dojo/dom-construct",
    "dojo/dom-attr",
    "dojo/dom-style",
    "dojo/dom-geometry",
    "dojo/query",
    "dijit/registry",
    "./_StickyEventMixin"
], function (declare,
             lang,
             window,
             domClass,
             domConstruct,
             domAttr,
             domStyle,
             domGeometry,
             query,
             registry,
             _StickyEventMixin) {
    return declare([_StickyEventMixin], {
        // module:
        //      epi-saas-base/Sticker

        offset: 0,

        subscribe: true,

        _resetCss: "stick stick-top stick-bottom",

        constructor:function (options) {
            // summary:
            //      Creates the sticker instance
            // options: Object?
            //      The optional set of parameters like node to stick to, scroll element, limit element:
            //          {
            //              domNode: DOMNode|Widget
            //              stickToNode: HTML element stick to. Document will be used if this option is not specified.
            //              scrollNode: HTML element that has a scrolling that sticker should react to. Windows will be used if this option is not specified.
            //              limitNode: HTML element that limits the area where sticker should stick. Sticker should unstick when this element is not visible.
            //              stickZIndex: Z-index that should be assigned to sticker.
            //              subscribe: Indicates whether sticker should subscribe to scroll, resize and click events. True by default.
            //          }
            // tags:
            //      public

            declare.safeMixin(this, options || {});

            if (!(this.domNode instanceof HTMLElement)) {
                if (this.domNode.domNode instanceof HTMLElement) {
                    this.domNode = this.domNode.domNode;
                }
                else {
                    throw "Provide HtmlElement or Widget when creating Sticker.";
                }
            }

            this.positionNode = this.domNode;

            this.stickToNode = this.stickToNode || this.scrollNode || window.doc;

            this._createReplacer();

            if (this.subscribe) {
                this.attachEvents(lang.hitch(this, function (resized) {
                    this.update(resized);
                }), this.scrollNode, this.stickToNode);

                this.triggerDelayedAction(600);
            }
        },

        update: function(resized, options) {
            // summary:
            //      Updates the state and position of the node depending on scrolling, container and other options
            // resized: Boolean
            //      Indicates whether elements were resized
            // options: Object?
            //      The set of optional parameters like container position and height, scroll top and additional margins
            // tags:
            //      public
            if (domStyle.get(this.domNode, "display") === "none") {
                return;
            }

            options = options || {};

            var position = domGeometry.position(this.positionNode),
                stick;

            // get the height of the scrolling area
            var scrollHeight = options.scrollHeight || (this.scrollNode ? this.scrollNode.clientHeight : window.doc.height);

            // get the position of the node to stick to
            var stickToNodePosition = options.stickToNodePosition || domGeometry.position(this.stickToNode);

            // get the position of the node that should be visible when element sticks
            var limitNodePosition = this.limitNode ? domGeometry.position(this.limitNode) : stickToNodePosition;

            // get the defined top and bottom offsets in pixels
            var offset = this._getOffset();

            // get additional offsets, usually it is used in stack of stickers.
            var additionalTop = options.offset && options.offset.top ? options.offset.top : 0;
            var additionalBottom = options.offset && options.offset.bottom ? options.offset.bottom : 0;

            // check if the node should stick on the top
            if (offset.top !== null && offset.top !== undefined) {
                if (position.y < stickToNodePosition.y + additionalTop &&
                    limitNodePosition.y + limitNodePosition.h - position.h > stickToNodePosition.y + additionalTop) {
                    stick = "top";
                }
            }

            // check if the node should stick on the bottom
            if (offset.bottom !== null && offset.bottom !== undefined) {
                if (position.y + position.h >= stickToNodePosition.y + scrollHeight - additionalBottom &&
                    limitNodePosition.y + position.h < stickToNodePosition.y + scrollHeight - additionalBottom) {
                    stick = "bottom";
                }
            }

            if (!stick && !this.stuckTo) {
                this.stickerBox = domGeometry.getMarginBox(this.domNode);
                return;
            }

            this.stuckTo = stick;

            this._getStyles(resized);

            if (this.stuckTo) {
                // set size and offset for stick node
                domStyle.set(this.domNode, "width", this.nodeStyle.width);
                domStyle.set(this.domNode, "height", this.nodeStyle.height);
                domStyle.set(this.domNode, "left", position.x + "px");
                domClass.replace(this.domNode, "stick" + (stick ? "-" + stick : ""), this._resetCss);

                var temporaryStickerBox = domGeometry.getMarginBox(this.domNode);

                // set new position for stick node
                if (this.stuckTo  === "top") {
                    domStyle.set(this.domNode, "top", (stickToNodePosition.y + offset.top + additionalTop) + "px");
                }
                if (this.stuckTo  === "bottom") {
                    domStyle.set(this.domNode, "top", (stickToNodePosition.y + stickToNodePosition.h - offset.bottom - additionalBottom - temporaryStickerBox.h)  + "px");
                }
                if (this.stickZIndex) {
                    domStyle.set(this.domNode, "z-index", "" + this.stickZIndex);
                }
                // make the stick node position fixed and display replacing node in place of original
                domStyle.set(this.domNode, "position", "fixed");
                domStyle.set(this.replacer, "display", this.nodeStyle.display || "block");
                this.positionNode = this.replacer;
                this.stickerBox = domGeometry.getMarginBox(this.domNode);
                this.position = domGeometry.position(this.domNode);
            }
            else {
                this.unstick();
            }

        },

        unstick: function () {
            // summary:
            //      Unsticks the element and resets it's original state
            // tags:
            //      public

            this._getStyles(true);
            // restore original inline styles for original node and put the original node on the original place
            domStyle.set(this.domNode, this.nodeStyleInline);
            // hide replacer
            domStyle.set(this.replacer, "display", "none");
            this.positionNode = this.domNode;
            this.nodeStyle = null;
            this.nodeStyleInline = null;
            this.stuckTo = null;
            domClass.replace(this.domNode, "stick", this._resetCss);
            this.stickerBox = domGeometry.getMarginBox(this.domNode);
            this.position = domGeometry.position(this.domNode);
        },

        _getStyles: function (resized) {
            if (this.nodeStyle && this.nodeStyleInline && !resized) {
                return;
            }
            // store original dimensions and style of the node.
            var styleNode = resized? this.replacer : this.positionNode;

            var inlineStyle = styleNode.style;
            this.nodeStyleInline = {
                width: inlineStyle ? inlineStyle.width : "",
                height: inlineStyle ? inlineStyle.height : "",
                left: inlineStyle ? inlineStyle.left : "",
                top: inlineStyle ? inlineStyle.top : "",
                zIndex: inlineStyle ? inlineStyle.zIndex : "",
                position: inlineStyle ? inlineStyle.position : ""
            };

            var style = domStyle.get(styleNode);
            this.nodeStyle = {
                width: style.width,
                height: style.height,
                display: style.display && style.display !== "none" ? style.display : null
            };
        },

        _createReplacer: function () {
            // summary:
            //      Creates a replacer or avatar for the node that is used to detect original position when original node sticks
            // tags:
            //      private
            this.replacer = lang.clone(this.domNode);
            query("*", this.replacer).removeAttr("id"); // remove IDs from all nodes inside replacer to avoid ID duplicates

            domAttr.set(this.replacer, "id", registry.getUniqueId("epi-saas-base/StickerReplacer"));
            domStyle.set(this.replacer, "visibility", "hidden");
            domStyle.set(this.replacer, "display", "none");
            domConstruct.place(this.replacer, this.domNode, "after");
        },

        _getOffset: function () {
            // summary:
            //      Calculates the top and bottom offset values based on defined option values or functions
            // tags:
            //      private
            var offset = {};
            if (typeof this.offset !== "object") {
                offset.bottom = offset.top = this.offset;
            } else {
                if (typeof this.offset.top === "function") {
                    offset.top = this.offset.top();
                } else {
                    offset.top = this.offset.top;
                }
                if (typeof this.offset.bottom === "function") {
                    offset.bottom = this.offset.bottom();
                } else {
                    offset.bottom = this.offset.bottom;
                }
            }
            return offset;
        }
    });
});
},
'epi-saas-base/_StickyEventMixin':function(){
define("epi-saas-base/_StickyEventMixin", [
    "dojo/_base/declare",
    "dojo/_base/array",
    "dojo/_base/lang",
    "dojo/_base/window",
    "dojo/on",
    "dojo/aspect",
    "dijit/registry",
    "./_PropagatingDestroyableMixin"
], function (declare,
             array,
             lang,
             window,
             on,
             aspect,
             registry,
             _PropagatingDestroyableMixin
             ) {
    return declare([_PropagatingDestroyableMixin], {
        // module:
        //      epi-saas-base/_StickyEventMixin

        _actionTimer: null,
        _handlerAction: null,

        attachEvents: function (handlerAction, scrollNodes, resizeNodes) {
            // summary:
            //      Subscribes on events of specified nodes that may require executing specified action.
            // handlerAction: Function
            //      Function to execute when event is triggered.
            // scrollNodes: Array|DOMNode
            //      Array of DOM nodes that may trigger scroll event that should be watched.
            // handlerAction: Array|DOMNode
            //      Array of DOM nodes that may trigger resize and click events that should be watched.
            // tags:
            //      protected

            if (handlerAction === null ||
                handlerAction === undefined ||
                typeof(handlerAction) !== "function" ) {
                throw "Handler for scroll, resize and click events should be defined.";
            }

            var defaultScrollDelay = 0,
                defaultResizeDelay = 20;

            this._handlerAction = handlerAction;

            scrollNodes = scrollNodes && (scrollNodes instanceof Array) ? scrollNodes : [scrollNodes];
            // always handle windows events
            if (array.indexOf(scrollNodes, window.global) === -1) {
                scrollNodes.push(window.global);
            }

            resizeNodes = resizeNodes && (resizeNodes instanceof Array) ? resizeNodes : [resizeNodes];

            var self = this;

            // subscribing to scroll event of the window and specified scroll nodes
            array.forEach(scrollNodes, function(node) {
                this.own(
                    on(node, "scroll", function () {
                        self.triggerDelayedAction(self.scrollDelay || defaultScrollDelay);
                    })
                );
            }, this);

            // subscribing to resize and click events of the window, all specified nodes and enclosing widgets
            array.forEach(resizeNodes.concat(scrollNodes), function(node) {
                var widget = registry.getEnclosingWidget(node);
                if(widget && typeof(widget.resize) === "function") {
                    this.own(
                        aspect.after(widget, "resize", function() {
                            self.triggerDelayedAction(self.resizeDelay || defaultResizeDelay, true);
                        })
                    );
                }
                this.own(
                    on(node, "resize", function () {
                        self.triggerDelayedAction(self.resizeDelay || defaultResizeDelay, true);
                    }),
                    on(node, "click", function () {
                        self.triggerDelayedAction(self.resizeDelay || defaultResizeDelay);
                    })
                );
            }, this);
        },

        triggerDelayedAction: function(timeOut, resized) {
            // summary:
            //      Executes the handler action after delay in order to avoid extra calls.
            // resized: Boolean?
            //      Indicates whether elements were resized.
            // tags:
            //      protected

            if (this._actionTimer) {

                clearTimeout(this._actionTimer);
            }
            this._actionTimer = setTimeout(lang.hitch(this, function () {
                this._actionTimer = null;
                this._handlerAction(resized);
            }), timeOut);
        }
    });
});
},
'epi-saas-base/form/RadioButtonGroupList':function(){
define("epi-saas-base/form/RadioButtonGroupList", [
        "dojo/_base/declare",
        "dojo/dom-construct",
        "dojo/dom-class",
        "dijit/form/RadioButton",
        "dojox/mvc/at",
        "./FormList",
        "./FormListItem"
    ],
    function(
            // dojo
            declare,
            domConstruct,
            domClass,
            // dijit
            RadioButton,
            // dojox
            at,
            // .
            FormList,
            FormListItem) {

        // module:
        //      epi-saas-base/form/RadioButtonGroupList

        return declare([FormList], {

            // label: string
            //      The value of the radiobutton group
            value: "",

            postCreate: function() {
                if(this.name && this.options) {
                    var rbs = this._createRadioButtonGroup(this.name, this.options);
                    for (var i = 0, j = rbs.length; i < j; i += 1) {
                        this.createListItem(rbs[i]);
                    }
                }
            },

            createListItem: function(radioButton) {
                var listItem = new FormListItem({
                    blockClass: ""
                });
                listItem.addChild(radioButton);
                this.addChild(listItem);
                this.own(
                    radioButton.watch("checked", function() {
                        domClass.toggle(listItem.domNode, "is-active");
                    })
                );
                radioButton.startup();
            },

            _createRadioButtonGroup: function(name, options) {
                // summary:
                //      Creates a radio button group
                // description:
                //      Creates a group of radio buttons based on the array of options provided
                // name: string
                //      The name of the radio button group
                // options: string[]
                //      The string array of options to represent in radio button group
                // tags:
                //      private
                options = options instanceof Array ? options : Array.prototype.slice.call(arguments, 1);

                var self = this,
                    rbs = [],
                    createRb = function(option) {
                        var rb = new RadioButton({
                            baseClass: "",
                            "class": "" + option,
                            name: name,
                            value: option,
                            checked: at(self, "value")
                                .transform({
                                    format: function(value) {
                                        return value === rb.value;
                                    },
                                    parse: function(checked) {
                                        return checked ? rb.value : self.value;
                                    }
                                })
                            }),
                            label = domConstruct.toDom("<label for=\"" + rb.id + "\" class=\"epi-pane__content__header-link--" + option + "\">" + self.labels[name][option] + "</label>");
                        domConstruct.place(label, rb.focusNode, "before");
                        domConstruct.place(rb.focusNode, label, "first");
                        return rb;
                    };

                for (var i = 0, j = options.length; i < j; i += 1) {
                    rbs.push(createRb(options[i]));
                }
                return rbs;
            }
        });
    });

},
'epi-saas-base/form/FormList':function(){
define("epi-saas-base/form/FormList", [
    "dojo/_base/declare",
    "dojo/dom",
    "dojo/dom-class",
    "dojo/dom-construct",
    "dojo/dom-geometry",
    "dojo/dom-style",

    "dijit/_WidgetBase",
    "dijit/_Container",

    "./FormListItem"
], function(declare, dom, domClass, domConstruct, domGeometry, domStyle,
    _WidgetBase, _Container,
    FormListItem
    ){

return declare([_WidgetBase, _Container], {
    // summary:
    //      FormList
    // description:
    //      Form items should be placed inside list elements

    // containerType: String
    //      The node type used for the container node.
    containerType: "ol",

    // blockClass: string
    //      Block class to use, according to BEM naming convention
    blockClass: "",

    // originalContainerNode: DomNode
    //      Holds the original container node
    originalContainerNode: null,

    buildRendering: function(){
        // summary:
        //      Creates a new containerNode of type this.containerType
        this.domNode = this.srcNodeRef || this.ownerDocument.createElement(this.containerType);
        this.domNode.setAttribute("class", this.blockClass + (this["class"] !== "" ? " " + this["class"] : ""));
        this.inherited(arguments);
    },

    addChild: function(/*dijit/_WidgetBase*/ child){
        // summary:
        //      Add a child widget to the list, wrapped in a FormListItem
        // child:
        //      a widget to add
        if(child instanceof FormListItem){
            this.own(child); // FormListItems belong to the FormList
            this.inherited(arguments);
        } else {
            var newArgs = Array.prototype.slice.call(arguments, 0);
            newArgs[0] = new FormListItem({ item: child });  // Wrap any child widgets that are not FormListItems
            this.own(newArgs[0]);
            this.inherited(arguments, newArgs);
        }
    }

});
});
},
'epi-saas-base/form/FormListItem':function(){
define("epi-saas-base/form/FormListItem", [
    "dojo/_base/declare",
    "dojo/dom-class",
    "dojo/dom-construct",
    "dojo/dom-attr",

    "dijit/_WidgetBase",
    "dijit/_Container",
    "dijit/registry"

], function(declare, domClass, domConstruct, domAttr,
    _WidgetBase, _Container, registry
    ){

// module:
//      epi-saas-base/form/FormListItem

return declare([_WidgetBase, _Container], {
    // summary:
    //     Form input wrapping widget for use with FormList
    // desciption:
    //     Creates list items, using the extra properties applied to its item
    //     property.
    // example:
    //  |   var textBox = new TextBox(),
    //  |   listItem = new FormsListItem({
    //  |               item: textBox,
    //  |               label: "Fantastic",
    //  |               description: "This is a fantastic text box",
    //  |               help: "Just write something fantastic in the box and press submit",
    //  |               inputStyleType: "custom-style-type-modifier"
    //  |           });

    // blockClass: string
    //      Block class to use, according to BEM naming convention
    blockClass: "epi-formsRow",

    // inputStyleType: string
    //      Used as modifier with blockClass as style on li-element
    inputStyleType: "fullWidthInput",

    // item: dijit/_WidgetBase
    //      This is the input widget that is represented in the FormListItem
    item: null,

    // wrapperNode: [readonly] DomNode
    //      This is the span, on sibling level to its label, that wraps the
    //      input field and any description and help paragraphs.
    wrapperNode: null,

    // labelNode: [readonly] DomNode
    //      This is the label node that refers to the input field. If there
    //      is no label, this property is null and this.domNode has no-label-input
    //      modifier class added to it.
    labelNode: null,

    // descriptionNode: [readonly] DomNode
    //      This is the node holding the description of the input element, if any.
    descriptionNode: null,

    // helpNode: [readonly] DomNode
    //      This is the node that holds the help text for the input element
    helpNode: null,

    // label: string
    //      This is the content of the label node for this item
    label: null,
    _setLabelAttr: function(value) {
        if (value) {
            this.labelNode = domConstruct.toDom("<label for=\"" + (this._getItemId() || "") + "\">"+ value + "</label>");
            domConstruct.place(this.labelNode, this.domNode, "first");
            domClass.remove(this.domNode, this.blockClass + "--no-label-input");
        }
        this._set("label",value);
    },

    // description: string
    //      This is the content of the description node for this item
    description: null,
    _setDescriptionAttr: function(value) {
        this.descriptionNode = domConstruct.toDom("<p class=\"" + this.blockClass + "__" + "description" + "\">" + value + "</p>");
        domConstruct.place(this.descriptionNode, this.containerNode);
        this._set("description", value);
    },

    // help: string
    //      This is the content of the help node for this item
    help: null,
    _setHelpAttr: function(value) {
        this.helpNode = domConstruct.toDom("<p class=\"" + "epi-help" + "\">"+ value + "</p>");
        domConstruct.place(this.helpNode, this.containerNode);
        this._set("help", value);
    },

    buildRendering: function(){
        // summary:
        //      Creates a new containerNode of type this.containerType
        this.inherited(arguments);

        var blockClass = this.blockClass,
            liClass = blockClass ? blockClass + "--" + this.inputStyleType : "",
            liNode = this.domNode = domConstruct.toDom("<li class=\"" + blockClass + (blockClass && liClass ? " " : "") + liClass + "\"></li>"),
            wrapperClass = liClass ? liClass + "__" + "input-wrapper" : "",
            wrapperNode = this.wrapperNode = domConstruct.toDom("<span class=\"epi-formsRow__input-wrapper\"></span>"),
            itemNodeId = this._getItemId();

        // if containerNode has any children - then probably the widget was used in a template and we need to
        // copy nodes that was placed inside the widget to the wrapperNode
        if (this.containerNode) {
            var children = this.containerNode.childNodes;
            while(children.length) {
                var currentNode = children[0];
                domConstruct.place(currentNode, wrapperNode);
                if (!itemNodeId && (currentNode.nodeName === "INPUT" ||
                                   currentNode.nodeName === "SELECT" ||
                                   currentNode.nodeName === "TEXTAREA")) {
                    itemNodeId = domAttr.get(currentNode, "id");
                }
            }
        }
        // For #_getDescendantFormWidgets in dijit/form/_FormMixin to work properly, item must be a child
        this.containerNode = wrapperNode;

        if(this.item && this.getChildren().length === 0) { // If this.item is set via property on instantiation, add it as child
            this.addChild(this.item);
        }

        if(blockClass) {
            domClass.add(liNode, blockClass + "--no-label-input");
        }

        domConstruct.place(wrapperNode, liNode);
    },

    addChild: function(/*dijit/_WidgetBase*/ child){
        // summary:
        //      Add a child widget to the list, wrapped in a FormListItem
        // child:
        //      a widget to add
        if(this.getChildren().length === 0) {
            this.item = child;
            this.inherited(arguments);
        } else {
            throw "Each FormListItem should only have one widget";
        }
    },

    _getItemId: function(){
        return this.item && this.item.id ? this.item.id : null;
    }

});
});

},
'epi-find/themes/charting/Dashboard':function(){
define(["dojox/charting/Theme", "dojox/gfx/gradutils", "dojox/charting/themes/common"], function(Theme, gradutils, themes){

    // Based on dojox/charting/themes/Charged

    var g = Theme.generateGradient,
        defaultFill = {type: "linear", space: "shape", x1: 0, y1: 0, x2: 0, y2: 75};

    themes.Dashboard = new Theme({
        chart: {
            fill: "#faf9f4"
        },
        plotarea: {
            fill: "transparent"
        },
        axis:{
            stroke: { // the axis itself
                color: "#333",
                width: 1
            },
            tick: { // used as a foundation for all ticks
                color:     "#333",
                position:  "center",
                font:      "normal normal normal 7pt Helvetica, Arial, sans-serif", // labels on axis
                fontColor: "#333"                                // color of labels
            }
        },
        series: {
            stroke:  {width: 2, color: "#95c532"},
            fill: "#c6da5c",
            outline: null,
            font: "normal normal normal 8pt Helvetica, Arial, sans-serif",
            fontColor: "#808078"
        },
        marker: {
            stroke:  {width: 3, color: "#333"},
            outline: null,
            font: "normal normal normal 8pt Helvetica, Arial, sans-serif",
            fontColor: "#333"
        },
        seriesThemes: [
            {fill: g(defaultFill, "#cbe25e", "#cbe25e")}    // Gradient!
        ],
        markerThemes: [
            {fill: "#c6d964", stroke: {color: "#95c532"}}
        ]
    });

    themes.Dashboard.next = function(elementType, mixin, doPost){
        var isLine = elementType == "line";
        if(isLine || elementType == "area"){
            // custom processing for lines: substitute colors
            var s = this.seriesThemes[this._current % this.seriesThemes.length];
            s.fill.space = "plot";
            if(isLine){
                s.stroke  = { width: 2.5, color: s.fill.colors[1].color};
            }
            if(elementType == "area"){
                s.fill.y2 = 90;
            }
            var theme = Theme.prototype.next.apply(this, arguments);
            // cleanup
            delete s.stroke;
            s.fill.y2 = 75;
            s.fill.space = "shape";
            return theme;
        }
        return Theme.prototype.next.apply(this, arguments);
    };

    themes.Dashboard.post = function(theme, elementType){
        theme = Theme.prototype.post.apply(this, arguments);
        if((elementType == "slice" || elementType == "circle") && theme.series.fill && theme.series.fill.type == "radial"){
            theme.series.fill = gradutils.reverse(theme.series.fill);
        }
        return theme;
    };

    return themes.Dashboard;
});

},
'dojox/charting/Theme':function(){
define("dojox/charting/Theme", ["dojo/_base/lang", "dojo/_base/declare", "dojo/_base/Color", "./SimpleTheme",
	    "dojox/color/_base", "dojox/color/Palette", "dojox/gfx/gradutils"],
	function(lang, declare, Color, SimpleTheme, colorX, Palette){
	
	var Theme = declare("dojox.charting.Theme", SimpleTheme, {
	// summary:
	//		A Theme is a pre-defined object, primarily JSON-based, that makes up the definitions to
	//		style a chart. It extends SimpleTheme with additional features like color definition by
	//		palettes and gradients definition.
	});

	/*=====
	var __DefineColorArgs = {
		// summary:
		//		The arguments object that can be passed to define colors for a theme.
		// num: Number?
		//		The number of colors to generate.  Defaults to 5.
		// colors: String[]|dojo/_base/Color[]?
		//		A pre-defined set of colors; this is passed through to the Theme directly.
		// hue: Number?
		//		A hue to base the generated colors from (a number from 0 - 359).
		// saturation: Number?
		//		If a hue is passed, this is used for the saturation value (0 - 100).
		// low: Number?
		//		An optional value to determine the lowest value used to generate a color (HSV model)
		// high: Number?
		//		An optional value to determine the highest value used to generate a color (HSV model)
		// base: String|dojo/_base/Color?
		//		A base color to use if we are defining colors using dojox.color.Palette
		// generator: String?
		//		The generator function name from dojox/color/Palette.
	};
	=====*/
	lang.mixin(Theme, {

		defineColors: function(kwArgs){
			// summary:
			//		Generate a set of colors for the theme based on keyword
			//		arguments.
			// kwArgs: __DefineColorArgs
			//		The arguments object used to define colors.
			// returns: dojo/_base/Color[]
			//		An array of colors for use in a theme.
			//
			// example:
			//	|	var colors = Theme.defineColors({
			//	|		base: "#369",
			//	|		generator: "compound"
			//	|	});
			//
			// example:
			//	|	var colors = Theme.defineColors({
			//	|		hue: 60,
			//	|		saturation: 90,
			//	|		low: 30,
			//	|		high: 80
			//	|	});
			kwArgs = kwArgs || {};
			var l, c = [], n = kwArgs.num || 5;	// the number of colors to generate
			if(kwArgs.colors){
				// we have an array of colors predefined, so fix for the number of series.
				l = kwArgs.colors.length;
				for(var i = 0; i < n; i++){
					c.push(kwArgs.colors[i % l]);
				}
				return c;	//	dojo.Color[]
			}
			if(kwArgs.hue){
				// single hue, generate a set based on brightness
				var s = kwArgs.saturation || 100,	// saturation
					st = kwArgs.low || 30,
					end = kwArgs.high || 90;
				// we'd like it to be a little on the darker side.
				l = (end + st) / 2;
				// alternately, use "shades"
				return Palette.generate(
					colorX.fromHsv(kwArgs.hue, s, l), "monochromatic"
				).colors;
			}
			if(kwArgs.generator){
				//	pass a base color and the name of a generator
				return colorX.Palette.generate(kwArgs.base, kwArgs.generator).colors;
			}
			return c;	//	dojo.Color[]
		},

		generateGradient: function(fillPattern, colorFrom, colorTo){
			var fill = lang.delegate(fillPattern);
			fill.colors = [
				{offset: 0, color: colorFrom},
				{offset: 1, color: colorTo}
			];
			return fill;
		},

		generateHslColor: function(color, luminance){
			color = new Color(color);
			var hsl    = color.toHsl(),
				result = colorX.fromHsl(hsl.h, hsl.s, luminance);
			result.a = color.a;	// add missing opacity
			return result;
		},

		generateHslGradient: function(color, fillPattern, lumFrom, lumTo){
			color = new Color(color);
			var hsl       = color.toHsl(),
				colorFrom = colorX.fromHsl(hsl.h, hsl.s, lumFrom),
				colorTo   = colorX.fromHsl(hsl.h, hsl.s, lumTo);
			colorFrom.a = colorTo.a = color.a;	// add missing opacity
			return Theme.generateGradient(fillPattern, colorFrom, colorTo);	// Object
		}
	});

	// for compatibility
	Theme.defaultMarkers = SimpleTheme.defaultMarkers;
	Theme.defaultColors = SimpleTheme.defaultColors;
	Theme.defaultTheme = SimpleTheme.defaultTheme;

	return Theme;
});

},
'dojox/color/_base':function(){
define("dojox/color/_base", ["../main", "dojo/_base/lang", "dojo/_base/Color", "dojo/colors"],
	function(dojox, lang, Color, colors){

var cx = lang.getObject("color", true, dojox);
/*===== cx = dojox.color =====*/
		
//	alias all the dojo.Color mechanisms
cx.Color=Color;
cx.blend=Color.blendColors;
cx.fromRgb=Color.fromRgb;
cx.fromHex=Color.fromHex;
cx.fromArray=Color.fromArray;
cx.fromString=Color.fromString;

//	alias the dojo.colors mechanisms
cx.greyscale=colors.makeGrey;

lang.mixin(cx,{
	fromCmy: function(/* Object|Array|int */cyan, /*int*/magenta, /*int*/yellow){
		// summary:
		//		Create a dojox.color.Color from a CMY defined color.
		//		All colors should be expressed as 0-100 (percentage)
	
		if(lang.isArray(cyan)){
			magenta=cyan[1], yellow=cyan[2], cyan=cyan[0];
		} else if(lang.isObject(cyan)){
			magenta=cyan.m, yellow=cyan.y, cyan=cyan.c;
		}
		cyan/=100, magenta/=100, yellow/=100;
	
		var r=1-cyan, g=1-magenta, b=1-yellow;
		return new Color({ r:Math.round(r*255), g:Math.round(g*255), b:Math.round(b*255) });	//	dojox.color.Color
	},
	
	fromCmyk: function(/* Object|Array|int */cyan, /*int*/magenta, /*int*/yellow, /*int*/black){
		// summary:
		//		Create a dojox.color.Color from a CMYK defined color.
		//		All colors should be expressed as 0-100 (percentage)
	
		if(lang.isArray(cyan)){
			magenta=cyan[1], yellow=cyan[2], black=cyan[3], cyan=cyan[0];
		} else if(lang.isObject(cyan)){
			magenta=cyan.m, yellow=cyan.y, black=cyan.b, cyan=cyan.c;
		}
		cyan/=100, magenta/=100, yellow/=100, black/=100;
		var r,g,b;
		r = 1-Math.min(1, cyan*(1-black)+black);
		g = 1-Math.min(1, magenta*(1-black)+black);
		b = 1-Math.min(1, yellow*(1-black)+black);
		return new Color({ r:Math.round(r*255), g:Math.round(g*255), b:Math.round(b*255) });	//	dojox.color.Color
	},
		
	fromHsl: function(/* Object|Array|int */hue, /* int */saturation, /* int */luminosity){
		// summary:
		//		Create a dojox.color.Color from an HSL defined color.
		//		hue from 0-359 (degrees), saturation and luminosity 0-100.
	
		if(lang.isArray(hue)){
			saturation=hue[1], luminosity=hue[2], hue=hue[0];
		} else if(lang.isObject(hue)){
			saturation=hue.s, luminosity=hue.l, hue=hue.h;
		}
		saturation/=100;
		luminosity/=100;
	
		while(hue<0){ hue+=360; }
		while(hue>=360){ hue-=360; }
		
		var r, g, b;
		if(hue<120){
			r=(120-hue)/60, g=hue/60, b=0;
		} else if (hue<240){
			r=0, g=(240-hue)/60, b=(hue-120)/60;
		} else {
			r=(hue-240)/60, g=0, b=(360-hue)/60;
		}
		
		r=2*saturation*Math.min(r, 1)+(1-saturation);
		g=2*saturation*Math.min(g, 1)+(1-saturation);
		b=2*saturation*Math.min(b, 1)+(1-saturation);
		if(luminosity<0.5){
			r*=luminosity, g*=luminosity, b*=luminosity;
		}else{
			r=(1-luminosity)*r+2*luminosity-1;
			g=(1-luminosity)*g+2*luminosity-1;
			b=(1-luminosity)*b+2*luminosity-1;
		}
		return new Color({ r:Math.round(r*255), g:Math.round(g*255), b:Math.round(b*255) });	//	dojox.color.Color
	}
});
	
cx.fromHsv = function(/* Object|Array|int */hue, /* int */saturation, /* int */value){
	// summary:
	//		Create a dojox.color.Color from an HSV defined color.
	//		hue from 0-359 (degrees), saturation and value 0-100.

	if(lang.isArray(hue)){
		saturation=hue[1], value=hue[2], hue=hue[0];
	} else if (lang.isObject(hue)){
		saturation=hue.s, value=hue.v, hue=hue.h;
	}
	
	if(hue==360){ hue=0; }
	saturation/=100;
	value/=100;
	
	var r, g, b;
	if(saturation==0){
		r=value, b=value, g=value;
	}else{
		var hTemp=hue/60, i=Math.floor(hTemp), f=hTemp-i;
		var p=value*(1-saturation);
		var q=value*(1-(saturation*f));
		var t=value*(1-(saturation*(1-f)));
		switch(i){
			case 0:{ r=value, g=t, b=p; break; }
			case 1:{ r=q, g=value, b=p; break; }
			case 2:{ r=p, g=value, b=t; break; }
			case 3:{ r=p, g=q, b=value; break; }
			case 4:{ r=t, g=p, b=value; break; }
			case 5:{ r=value, g=p, b=q; break; }
		}
	}
	return new Color({ r:Math.round(r*255), g:Math.round(g*255), b:Math.round(b*255) });	//	dojox.color.Color
};
lang.extend(Color,{
	toCmy: function(){
		// summary:
		//		Convert this Color to a CMY definition.
		var cyan=1-(this.r/255), magenta=1-(this.g/255), yellow=1-(this.b/255);
		return { c:Math.round(cyan*100), m:Math.round(magenta*100), y:Math.round(yellow*100) };		//	Object
	},
		
	toCmyk: function(){
		// summary:
		//		Convert this Color to a CMYK definition.
		var cyan, magenta, yellow, black;
		var r=this.r/255, g=this.g/255, b=this.b/255;
		black = Math.min(1-r, 1-g, 1-b);
		cyan = (1-r-black)/(1-black);
		magenta = (1-g-black)/(1-black);
		yellow = (1-b-black)/(1-black);
		return { c:Math.round(cyan*100), m:Math.round(magenta*100), y:Math.round(yellow*100), b:Math.round(black*100) };	//	Object
	},
		
	toHsl: function(){
		// summary:
		//		Convert this Color to an HSL definition.
		var r=this.r/255, g=this.g/255, b=this.b/255;
		var min = Math.min(r, b, g), max = Math.max(r, g, b);
		var delta = max-min;
		var h=0, s=0, l=(min+max)/2;
		if(l>0 && l<1){
			s = delta/((l<0.5)?(2*l):(2-2*l));
		}
		if(delta>0){
			if(max==r && max!=g){
				h+=(g-b)/delta;
			}
			if(max==g && max!=b){
				h+=(2+(b-r)/delta);
			}
			if(max==b && max!=r){
				h+=(4+(r-g)/delta);
			}
			h*=60;
		}
		return { h:h, s:Math.round(s*100), l:Math.round(l*100) };	//	Object
	},
	
	toHsv: function(){
		// summary:
		//		Convert this Color to an HSV definition.
		var r=this.r/255, g=this.g/255, b=this.b/255;
		var min = Math.min(r, b, g), max = Math.max(r, g, b);
		var delta = max-min;
		var h = null, s = (max==0)?0:(delta/max);
		if(s==0){
			h = 0;
		}else{
			if(r==max){
				h = 60*(g-b)/delta;
			}else if(g==max){
				h = 120 + 60*(b-r)/delta;
			}else{
				h = 240 + 60*(r-g)/delta;
			}
	
			if(h<0){ h+=360; }
		}
		return { h:h, s:Math.round(s*100), v:Math.round(max*100) };	//	Object
	}
});

return cx;
});

},
'dojo/colors':function(){
define("dojo/colors", ["./_base/kernel", "./_base/lang", "./_base/Color", "./_base/array"], function(dojo, lang, Color, ArrayUtil){
	// module:
	//		dojo/colors

	/*=====
	return {
		// summary:
		//		Color utilities, extending Base dojo.Color
	};
	=====*/

	var ColorExt = {};
	lang.setObject("dojo.colors", ColorExt);

//TODO: this module appears to break naming conventions

	// this is a standard conversion prescribed by the CSS3 Color Module
	var hue2rgb = function(m1, m2, h){
		if(h < 0){ ++h; }
		if(h > 1){ --h; }
		var h6 = 6 * h;
		if(h6 < 1){ return m1 + (m2 - m1) * h6; }
		if(2 * h < 1){ return m2; }
		if(3 * h < 2){ return m1 + (m2 - m1) * (2 / 3 - h) * 6; }
		return m1;
	};
	// Override base Color.fromRgb with the impl in this module
	dojo.colorFromRgb = Color.fromRgb = function(/*String*/ color, /*dojo/_base/Color?*/ obj){
		// summary:
		//		get rgb(a) array from css-style color declarations
		// description:
		//		this function can handle all 4 CSS3 Color Module formats: rgb,
		//		rgba, hsl, hsla, including rgb(a) with percentage values.
		var m = color.toLowerCase().match(/^(rgba?|hsla?)\(([\s\.\-,%0-9]+)\)/);
		if(m){
			var c = m[2].split(/\s*,\s*/), l = c.length, t = m[1], a;
			if((t == "rgb" && l == 3) || (t == "rgba" && l == 4)){
				var r = c[0];
				if(r.charAt(r.length - 1) == "%"){
					// 3 rgb percentage values
					a = ArrayUtil.map(c, function(x){
						return parseFloat(x) * 2.56;
					});
					if(l == 4){ a[3] = c[3]; }
					return Color.fromArray(a, obj); // dojo/_base/Color
				}
				return Color.fromArray(c, obj); // dojo/_base/Color
			}
			if((t == "hsl" && l == 3) || (t == "hsla" && l == 4)){
				// normalize hsl values
				var H = ((parseFloat(c[0]) % 360) + 360) % 360 / 360,
					S = parseFloat(c[1]) / 100,
					L = parseFloat(c[2]) / 100,
					// calculate rgb according to the algorithm
					// recommended by the CSS3 Color Module
					m2 = L <= 0.5 ? L * (S + 1) : L + S - L * S,
					m1 = 2 * L - m2;
				a = [
					hue2rgb(m1, m2, H + 1 / 3) * 256,
					hue2rgb(m1, m2, H) * 256,
					hue2rgb(m1, m2, H - 1 / 3) * 256,
					1
				];
				if(l == 4){ a[3] = c[3]; }
				return Color.fromArray(a, obj); // dojo/_base/Color
			}
		}
		return null;	// dojo/_base/Color
	};

	var confine = function(c, low, high){
		// summary:
		//		sanitize a color component by making sure it is a number,
		//		and clamping it to valid values
		c = Number(c);
		return isNaN(c) ? high : c < low ? low : c > high ? high : c;	// Number
	};

	Color.prototype.sanitize = function(){
		// summary:
		//		makes sure that the object has correct attributes
		var t = this;
		t.r = Math.round(confine(t.r, 0, 255));
		t.g = Math.round(confine(t.g, 0, 255));
		t.b = Math.round(confine(t.b, 0, 255));
		t.a = confine(t.a, 0, 1);
		return this;	// dojo/_base/Color
	};

	ColorExt.makeGrey = Color.makeGrey = function(/*Number*/ g, /*Number?*/ a){
		// summary:
		//		creates a greyscale color with an optional alpha
		return Color.fromArray([g, g, g, a]);	// dojo/_base/Color
	};

	// mixin all CSS3 named colors not already in _base, along with SVG 1.0 variant spellings
	lang.mixin(Color.named, {
		"aliceblue":	[240,248,255],
		"antiquewhite": [250,235,215],
		"aquamarine":	[127,255,212],
		"azure":	[240,255,255],
		"beige":	[245,245,220],
		"bisque":	[255,228,196],
		"blanchedalmond":	[255,235,205],
		"blueviolet":	[138,43,226],
		"brown":	[165,42,42],
		"burlywood":	[222,184,135],
		"cadetblue":	[95,158,160],
		"chartreuse":	[127,255,0],
		"chocolate":	[210,105,30],
		"coral":	[255,127,80],
		"cornflowerblue":	[100,149,237],
		"cornsilk": [255,248,220],
		"crimson":	[220,20,60],
		"cyan": [0,255,255],
		"darkblue": [0,0,139],
		"darkcyan": [0,139,139],
		"darkgoldenrod":	[184,134,11],
		"darkgray": [169,169,169],
		"darkgreen":	[0,100,0],
		"darkgrey": [169,169,169],
		"darkkhaki":	[189,183,107],
		"darkmagenta":	[139,0,139],
		"darkolivegreen":	[85,107,47],
		"darkorange":	[255,140,0],
		"darkorchid":	[153,50,204],
		"darkred":	[139,0,0],
		"darksalmon":	[233,150,122],
		"darkseagreen": [143,188,143],
		"darkslateblue":	[72,61,139],
		"darkslategray":	[47,79,79],
		"darkslategrey":	[47,79,79],
		"darkturquoise":	[0,206,209],
		"darkviolet":	[148,0,211],
		"deeppink": [255,20,147],
		"deepskyblue":	[0,191,255],
		"dimgray":	[105,105,105],
		"dimgrey":	[105,105,105],
		"dodgerblue":	[30,144,255],
		"firebrick":	[178,34,34],
		"floralwhite":	[255,250,240],
		"forestgreen":	[34,139,34],
		"gainsboro":	[220,220,220],
		"ghostwhite":	[248,248,255],
		"gold": [255,215,0],
		"goldenrod":	[218,165,32],
		"greenyellow":	[173,255,47],
		"grey": [128,128,128],
		"honeydew": [240,255,240],
		"hotpink":	[255,105,180],
		"indianred":	[205,92,92],
		"indigo":	[75,0,130],
		"ivory":	[255,255,240],
		"khaki":	[240,230,140],
		"lavender": [230,230,250],
		"lavenderblush":	[255,240,245],
		"lawngreen":	[124,252,0],
		"lemonchiffon": [255,250,205],
		"lightblue":	[173,216,230],
		"lightcoral":	[240,128,128],
		"lightcyan":	[224,255,255],
		"lightgoldenrodyellow": [250,250,210],
		"lightgray":	[211,211,211],
		"lightgreen":	[144,238,144],
		"lightgrey":	[211,211,211],
		"lightpink":	[255,182,193],
		"lightsalmon":	[255,160,122],
		"lightseagreen":	[32,178,170],
		"lightskyblue": [135,206,250],
		"lightslategray":	[119,136,153],
		"lightslategrey":	[119,136,153],
		"lightsteelblue":	[176,196,222],
		"lightyellow":	[255,255,224],
		"limegreen":	[50,205,50],
		"linen":	[250,240,230],
		"magenta":	[255,0,255],
		"mediumaquamarine": [102,205,170],
		"mediumblue":	[0,0,205],
		"mediumorchid": [186,85,211],
		"mediumpurple": [147,112,219],
		"mediumseagreen":	[60,179,113],
		"mediumslateblue":	[123,104,238],
		"mediumspringgreen":	[0,250,154],
		"mediumturquoise":	[72,209,204],
		"mediumvioletred":	[199,21,133],
		"midnightblue": [25,25,112],
		"mintcream":	[245,255,250],
		"mistyrose":	[255,228,225],
		"moccasin": [255,228,181],
		"navajowhite":	[255,222,173],
		"oldlace":	[253,245,230],
		"olivedrab":	[107,142,35],
		"orange":	[255,165,0],
		"orangered":	[255,69,0],
		"orchid":	[218,112,214],
		"palegoldenrod":	[238,232,170],
		"palegreen":	[152,251,152],
		"paleturquoise":	[175,238,238],
		"palevioletred":	[219,112,147],
		"papayawhip":	[255,239,213],
		"peachpuff":	[255,218,185],
		"peru": [205,133,63],
		"pink": [255,192,203],
		"plum": [221,160,221],
		"powderblue":	[176,224,230],
		"rosybrown":	[188,143,143],
		"royalblue":	[65,105,225],
		"saddlebrown":	[139,69,19],
		"salmon":	[250,128,114],
		"sandybrown":	[244,164,96],
		"seagreen": [46,139,87],
		"seashell": [255,245,238],
		"sienna":	[160,82,45],
		"skyblue":	[135,206,235],
		"slateblue":	[106,90,205],
		"slategray":	[112,128,144],
		"slategrey":	[112,128,144],
		"snow": [255,250,250],
		"springgreen":	[0,255,127],
		"steelblue":	[70,130,180],
		"tan":	[210,180,140],
		"thistle":	[216,191,216],
		"tomato":	[255,99,71],
		"turquoise":	[64,224,208],
		"violet":	[238,130,238],
		"wheat":	[245,222,179],
		"whitesmoke":	[245,245,245],
		"yellowgreen":	[154,205,50]
	});

	return Color;	// TODO: return ColorExt, not Color
});

},
'dojox/color/Palette':function(){
define("dojox/color/Palette", ["dojo/_base/lang", "dojo/_base/array", "./_base"],
	function(lang, arr, dxc){

	/***************************************************************
	*	dojox.color.Palette
	*
	*	The Palette object is loosely based on the color palettes
	*	at Kuler (http://kuler.adobe.com).  They are 5 color palettes
	*	with the base color considered to be the third color in the
	*	palette (for generation purposes).
	*
	*	Palettes can be generated from well-known algorithms or they
	* 	can be manually created by passing an array to the constructor.
	*
	*	Palettes can be transformed, using a set of specific params
	*	similar to the way shapes can be transformed with dojox.gfx.
	*	However, unlike with transformations in dojox.gfx, transforming
	* 	a palette will return you a new Palette object, in effect
	* 	a clone of the original.
	***************************************************************/

	//	ctor ----------------------------------------------------------------------------
	dxc.Palette = function(/* String|Array|dojox.color.Color|dojox.color.Palette */base){
		// summary:
		//		An object that represents a palette of colors.
		// description:
		//		A Palette is a representation of a set of colors.  While the standard
		//		number of colors contained in a palette is 5, it can really handle any
		//		number of colors.
		//
		//		A palette is useful for the ability to transform all the colors in it
		//		using a simple object-based approach.  In addition, you can generate
		//		palettes using dojox.color.Palette.generate; these generated palettes
		//		are based on the palette generators at http://kuler.adobe.com.

		// colors: dojox.color.Color[]
		//		The actual color references in this palette.
		this.colors = [];
		if(base instanceof dxc.Palette){
			this.colors = base.colors.slice(0);
		}
		else if(base instanceof dxc.Color){
			this.colors = [ null, null, base, null, null ];
		}
		else if(lang.isArray(base)){
			this.colors = arr.map(base.slice(0), function(item){
				if(lang.isString(item)){ return new dxc.Color(item); }
				return item;
			});
		}
		else if (lang.isString(base)){
			this.colors = [ null, null, new dxc.Color(base), null, null ];
		}
	}

	//	private functions ---------------------------------------------------------------

	//	transformations
	function tRGBA(p, param, val){
		var ret = new dxc.Palette();
		ret.colors = [];
		arr.forEach(p.colors, function(item){
			var r=(param=="dr")?item.r+val:item.r,
				g=(param=="dg")?item.g+val:item.g,
				b=(param=="db")?item.b+val:item.b,
				a=(param=="da")?item.a+val:item.a
			ret.colors.push(new dxc.Color({
				r: Math.min(255, Math.max(0, r)),
				g: Math.min(255, Math.max(0, g)),
				b: Math.min(255, Math.max(0, b)),
				a: Math.min(1, Math.max(0, a))
			}));
		});
		return ret;
	}

	function tCMY(p, param, val){
		var ret = new dxc.Palette();
		ret.colors = [];
		arr.forEach(p.colors, function(item){
			var o=item.toCmy(),
				c=(param=="dc")?o.c+val:o.c,
				m=(param=="dm")?o.m+val:o.m,
				y=(param=="dy")?o.y+val:o.y;
			ret.colors.push(dxc.fromCmy(
				Math.min(100, Math.max(0, c)),
				Math.min(100, Math.max(0, m)),
				Math.min(100, Math.max(0, y))
			));
		});
		return ret;
	}

	function tCMYK(p, param, val){
		var ret = new dxc.Palette();
		ret.colors = [];
		arr.forEach(p.colors, function(item){
			var o=item.toCmyk(),
				c=(param=="dc")?o.c+val:o.c,
				m=(param=="dm")?o.m+val:o.m,
				y=(param=="dy")?o.y+val:o.y,
				k=(param=="dk")?o.b+val:o.b;
			ret.colors.push(dxc.fromCmyk(
				Math.min(100, Math.max(0, c)),
				Math.min(100, Math.max(0, m)),
				Math.min(100, Math.max(0, y)),
				Math.min(100, Math.max(0, k))
			));
		});
		return ret;
	}

	function tHSL(p, param, val){
		var ret = new dxc.Palette();
		ret.colors = [];
		arr.forEach(p.colors, function(item){
			var o=item.toHsl(),
				h=(param=="dh")?o.h+val:o.h,
				s=(param=="ds")?o.s+val:o.s,
				l=(param=="dl")?o.l+val:o.l;
			ret.colors.push(dxc.fromHsl(h%360, Math.min(100, Math.max(0, s)), Math.min(100, Math.max(0, l))));
		});
		return ret;
	}

	function tHSV(p, param, val){
		var ret = new dxc.Palette();
		ret.colors = [];
		arr.forEach(p.colors, function(item){
			var o=item.toHsv(),
				h=(param=="dh")?o.h+val:o.h,
				s=(param=="ds")?o.s+val:o.s,
				v=(param=="dv")?o.v+val:o.v;
			ret.colors.push(dxc.fromHsv(h%360, Math.min(100, Math.max(0, s)), Math.min(100, Math.max(0, v))));
		});
		return ret;
	}

	//	helper functions
	function rangeDiff(val, low, high){
		//	given the value in a range from 0 to high, find the equiv
		//		using the range low to high.
		return high-((high-val)*((high-low)/high));
	}

/*=====
var __transformArgs = {
	// summary:
	//		The keywords argument to be passed to the dojox.color.Palette.transform function.  Note that
	//		while all arguments are optional, *some* arguments must be passed.  The basic concept is that
	//		you pass a delta value for a specific aspect of a color model (or multiple aspects of the same
	//		color model); for instance, if you wish to transform a palette based on the HSV color model,
	//		you would pass one of "dh", "ds", or "dv" as a value.
	// use: String?
	//		Specify the color model to use for the transformation.  Can be "rgb", "rgba", "hsv", "hsl", "cmy", "cmyk".
	// dr: Number?
	//		The delta to be applied to the red aspect of the RGB/RGBA color model.
	// dg: Number?
	//		The delta to be applied to the green aspect of the RGB/RGBA color model.
	// db: Number?
	//		The delta to be applied to the blue aspect of the RGB/RGBA color model.
	// da: Number?
	//		The delta to be applied to the alpha aspect of the RGBA color model.
	// dc: Number?
	//		The delta to be applied to the cyan aspect of the CMY/CMYK color model.
	// dm: Number?
	//		The delta to be applied to the magenta aspect of the CMY/CMYK color model.
	// dy: Number?
	//		The delta to be applied to the yellow aspect of the CMY/CMYK color model.
	// dk: Number?
	//		The delta to be applied to the black aspect of the CMYK color model.
	// dh: Number?
	//		The delta to be applied to the hue aspect of the HSL/HSV color model.
	// ds: Number?
	//		The delta to be applied to the saturation aspect of the HSL/HSV color model.
	// dl: Number?
	//		The delta to be applied to the luminosity aspect of the HSL color model.
	// dv: Number?
	//		The delta to be applied to the value aspect of the HSV color model.
};
var __generatorArgs = {
	// summary:
	//		The keyword arguments object used to create a palette based on a base color.
	// base: dojo/_base/Color
	//		The base color to be used to generate the palette.
};
var __analogousArgs = {
	// summary:
	//		The keyword arguments object that is used to create a 5 color palette based on the
	//		analogous rules as implemented at http://kuler.adobe.com, using the HSV color model.
	// base: dojo/_base/Color
	//		The base color to be used to generate the palette.
	// high: Number?
	//		The difference between the hue of the base color and the highest hue.  In degrees, default is 60.
	// low: Number?
	//		The difference between the hue of the base color and the lowest hue.  In degrees, default is 18.
};
var __splitComplementaryArgs = {
	// summary:
	//		The keyword arguments object used to create a palette based on the split complementary rules
	//		as implemented at http://kuler.adobe.com.
	// base: dojo/_base/Color
	//		The base color to be used to generate the palette.
	// da: Number?
	//		The delta angle to be used to determine where the split for the complementary rules happen.
	//		In degrees, the default is 30.
};
=====*/

	//	object methods ---------------------------------------------------------------
	lang.extend(dxc.Palette, {
		transform: function(/*__transformArgs*/kwArgs){
			// summary:
			//		Transform the palette using a specific transformation function
			//		and a set of transformation parameters.
			// description:
			//		{palette}.transform is a simple way to uniformly transform
			//		all of the colors in a palette using any of 5 formulae:
			//		RGBA, HSL, HSV, CMYK or CMY.
			//
			//		Once the forumula to be used is determined, you can pass any
			//		number of parameters based on the formula "d"[param]; for instance,
			//		{ use: "rgba", dr: 20, dg: -50 } will take all of the colors in
			//		palette, add 20 to the R value and subtract 50 from the G value.
			//
			//		Unlike other types of transformations, transform does *not* alter
			//		the original palette but will instead return a new one.
			var fn=tRGBA;	//	the default transform function.
			if(kwArgs.use){
				//	we are being specific about the algo we want to use.
				var use=kwArgs.use.toLowerCase();
				if(use.indexOf("hs")==0){
					if(use.charAt(2)=="l"){ fn=tHSL; }
					else { fn=tHSV; }
				}
				else if(use.indexOf("cmy")==0){
					if(use.charAt(3)=="k"){ fn=tCMYK; }
					else { fn=tCMY; }
				}
			}
			//	try to guess the best choice.
			else if("dc" in kwArgs || "dm" in kwArgs || "dy" in kwArgs){
				if("dk" in kwArgs){ fn = tCMYK; }
				else { fn = tCMY; }
			}
			else if("dh" in kwArgs || "ds" in kwArgs){
				if("dv" in kwArgs){ fn = tHSV; }
				else { fn = tHSL; }
			}

			var palette = this;
			for(var p in kwArgs){
				//	ignore use
				if(p=="use"){ continue; }
				palette = fn(palette, p, kwArgs[p]);
			}
			return palette;		//	dojox.color.Palette
		},
		clone: function(){
			// summary:
			//		Clones the current palette.
			return new dxc.Palette(this);	//	dojox.color.Palette
		}
	});

	lang.mixin(dxc.Palette, {
		generators: {
			analogous:function(/* __analogousArgs */args){
				// summary:
				//		Create a 5 color palette based on the analogous rules as implemented at
				//		http://kuler.adobe.com.
				var high=args.high||60, 	//	delta between base hue and highest hue (subtracted from base)
					low=args.low||18,		//	delta between base hue and lowest hue (added to base)
					base = lang.isString(args.base)?new dxc.Color(args.base):args.base,
					hsv=base.toHsv();

				//	generate our hue angle differences
				var h=[
					(hsv.h+low+360)%360,
					(hsv.h+Math.round(low/2)+360)%360,
					hsv.h,
					(hsv.h-Math.round(high/2)+360)%360,
					(hsv.h-high+360)%360
				];

				var s1=Math.max(10, (hsv.s<=95)?hsv.s+5:(100-(hsv.s-95))),
					s2=(hsv.s>1)?hsv.s-1:21-hsv.s,
					v1=(hsv.v>=92)?hsv.v-9:Math.max(hsv.v+9, 20),
					v2=(hsv.v<=90)?Math.max(hsv.v+5, 20):(95+Math.ceil((hsv.v-90)/2)),
					s=[ s1, s2, hsv.s, s1, s1 ],
					v=[ v1, v2, hsv.v, v1, v2 ]

				return new dxc.Palette(arr.map(h, function(hue, i){
					return dxc.fromHsv(hue, s[i], v[i]);
				}));		//	dojox.color.Palette
			},

			monochromatic: function(/* __generatorArgs */args){
				// summary:
				//		Create a 5 color palette based on the monochromatic rules as implemented at
				//		http://kuler.adobe.com.
				var base = lang.isString(args.base)?new dxc.Color(args.base):args.base,
					hsv = base.toHsv();
				
				//	figure out the saturation and value
				var s1 = (hsv.s-30>9)?hsv.s-30:hsv.s+30,
					s2 = hsv.s,
					v1 = rangeDiff(hsv.v, 20, 100),
					v2 = (hsv.v-20>20)?hsv.v-20:hsv.v+60,
					v3 = (hsv.v-50>20)?hsv.v-50:hsv.v+30;

				return new dxc.Palette([
					dxc.fromHsv(hsv.h, s1, v1),
					dxc.fromHsv(hsv.h, s2, v3),
					base,
					dxc.fromHsv(hsv.h, s1, v3),
					dxc.fromHsv(hsv.h, s2, v2)
				]);		//	dojox.color.Palette
			},

			triadic: function(/* __generatorArgs */args){
				// summary:
				//		Create a 5 color palette based on the triadic rules as implemented at
				//		http://kuler.adobe.com.
				var base = lang.isString(args.base)?new dxc.Color(args.base):args.base,
					hsv = base.toHsv();

				var h1 = (hsv.h+57+360)%360,
					h2 = (hsv.h-157+360)%360,
					s1 = (hsv.s>20)?hsv.s-10:hsv.s+10,
					s2 = (hsv.s>90)?hsv.s-10:hsv.s+10,
					s3 = (hsv.s>95)?hsv.s-5:hsv.s+5,
					v1 = (hsv.v-20>20)?hsv.v-20:hsv.v+20,
					v2 = (hsv.v-30>20)?hsv.v-30:hsv.v+30,
					v3 = (hsv.v-30>70)?hsv.v-30:hsv.v+30;

				return new dxc.Palette([
					dxc.fromHsv(h1, s1, hsv.v),
					dxc.fromHsv(hsv.h, s2, v2),
					base,
					dxc.fromHsv(h2, s2, v1),
					dxc.fromHsv(h2, s3, v3)
				]);		//	dojox.color.Palette
			},

			complementary: function(/* __generatorArgs */args){
				// summary:
				//		Create a 5 color palette based on the complementary rules as implemented at
				//		http://kuler.adobe.com.
				var base = lang.isString(args.base)?new dxc.Color(args.base):args.base,
					hsv = base.toHsv();

				var h1 = ((hsv.h*2)+137<360)?(hsv.h*2)+137:Math.floor(hsv.h/2)-137,
					s1 = Math.max(hsv.s-10, 0),
					s2 = rangeDiff(hsv.s, 10, 100),
					s3 = Math.min(100, hsv.s+20),
					v1 = Math.min(100, hsv.v+30),
					v2 = (hsv.v>20)?hsv.v-30:hsv.v+30;

				return new dxc.Palette([
					dxc.fromHsv(hsv.h, s1, v1),
					dxc.fromHsv(hsv.h, s2, v2),
					base,
					dxc.fromHsv(h1, s3, v2),
					dxc.fromHsv(h1, hsv.s, hsv.v)
				]);		//	dojox.color.Palette
			},

			splitComplementary: function(/* __splitComplementaryArgs */args){
				// summary:
				//		Create a 5 color palette based on the split complementary rules as implemented at
				//		http://kuler.adobe.com.
				var base = lang.isString(args.base)?new dxc.Color(args.base):args.base,
					dangle = args.da || 30,
					hsv = base.toHsv();

				var baseh = ((hsv.h*2)+137<360)?(hsv.h*2)+137:Math.floor(hsv.h/2)-137,
					h1 = (baseh-dangle+360)%360,
					h2 = (baseh+dangle)%360,
					s1 = Math.max(hsv.s-10, 0),
					s2 = rangeDiff(hsv.s, 10, 100),
					s3 = Math.min(100, hsv.s+20),
					v1 = Math.min(100, hsv.v+30),
					v2 = (hsv.v>20)?hsv.v-30:hsv.v+30;

				return new dxc.Palette([
					dxc.fromHsv(h1, s1, v1),
					dxc.fromHsv(h1, s2, v2),
					base,
					dxc.fromHsv(h2, s3, v2),
					dxc.fromHsv(h2, hsv.s, hsv.v)
				]);		//	dojox.color.Palette
			},

			compound: function(/* __generatorArgs */args){
				// summary:
				//		Create a 5 color palette based on the compound rules as implemented at
				//		http://kuler.adobe.com.
				var base = lang.isString(args.base)?new dxc.Color(args.base):args.base,
					hsv = base.toHsv();

				var h1 = ((hsv.h*2)+18<360)?(hsv.h*2)+18:Math.floor(hsv.h/2)-18,
					h2 = ((hsv.h*2)+120<360)?(hsv.h*2)+120:Math.floor(hsv.h/2)-120,
					h3 = ((hsv.h*2)+99<360)?(hsv.h*2)+99:Math.floor(hsv.h/2)-99,
					s1 = (hsv.s-40>10)?hsv.s-40:hsv.s+40,
					s2 = (hsv.s-10>80)?hsv.s-10:hsv.s+10,
					s3 = (hsv.s-25>10)?hsv.s-25:hsv.s+25,
					v1 = (hsv.v-40>10)?hsv.v-40:hsv.v+40,
					v2 = (hsv.v-20>80)?hsv.v-20:hsv.v+20,
					v3 = Math.max(hsv.v, 20);

				return new dxc.Palette([
					dxc.fromHsv(h1, s1, v1),
					dxc.fromHsv(h1, s2, v2),
					base,
					dxc.fromHsv(h2, s3, v3),
					dxc.fromHsv(h3, s2, v2)
				]);		//	dojox.color.Palette
			},

			shades: function(/* __generatorArgs */args){
				// summary:
				//		Create a 5 color palette based on the shades rules as implemented at
				//		http://kuler.adobe.com.
				var base = lang.isString(args.base)?new dxc.Color(args.base):args.base,
					hsv = base.toHsv();

				var s  = (hsv.s==100 && hsv.v==0)?0:hsv.s,
					v1 = (hsv.v-50>20)?hsv.v-50:hsv.v+30,
					v2 = (hsv.v-25>=20)?hsv.v-25:hsv.v+55,
					v3 = (hsv.v-75>=20)?hsv.v-75:hsv.v+5,
					v4 = Math.max(hsv.v-10, 20);

				return new dxc.Palette([
					new dxc.fromHsv(hsv.h, s, v1),
					new dxc.fromHsv(hsv.h, s, v2),
					base,
					new dxc.fromHsv(hsv.h, s, v3),
					new dxc.fromHsv(hsv.h, s, v4)
				]);		//	dojox.color.Palette
			}
		},
		generate: function(/* String|dojox.color.Color */base, /* Function|String */type){
			// summary:
			//		Generate a new Palette using any of the named functions in
			//		dojox.color.Palette.generators or an optional function definition.  Current
			//		generators include "analogous", "monochromatic", "triadic", "complementary",
			//		"splitComplementary", and "shades".
			if(lang.isFunction(type)){
				return type({ base: base });	//	dojox.color.Palette
			}
			else if(dxc.Palette.generators[type]){
				return dxc.Palette.generators[type]({ base: base });	//	dojox.color.Palette
			}
			throw new Error("dojox.color.Palette.generate: the specified generator ('" + type + "') does not exist.");
		}
	});
	
	return dxc.Palette;
});

},
'dojox/charting/themes/common':function(){
define("dojox/charting/themes/common", ["dojo/_base/lang"], function(lang){
	return lang.getObject("dojox.charting.themes", true);
});

},
'epi-find/widget/_DisabledViewMixin':function(){
define([
    "dojo/_base/config",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-class",

    "epi-saas-base/widgets/NotificationBar",

    "dojo/i18n!./nls/_DisabledViewMixin"
], function (
    config,
    declare,
    lang,
    domClass,

    NotificationBar,

    i18n
    ) {
    return declare([], {
        // summary:
        //     Allows to enable and disable views.

        // disabledView: Boolean
        //		Should this widget respond to user actions
        disabledView: false,

        _setDisabledViewAttr: function(/*Boolean*/ disabled) {
            // summary:
            //		Sets the disable state of the view widget.
            // tags:
            //		protected

            if (disabled === this.disabledView) {
                return;
            }
            this._set("disabledView", disabled);

            if(this._created) {
                if (disabled) {
                    this.disable();
                }
                else {
                    this.enable();
                }
            }
        },

        //  disabledNode: Object
        //    DOM node of the area that should be disabled.
        disabledNode: null,

        _getDisabledNodeAttr: function () {
            // summary:
            //		Gets the DOM node of the area that should be disabled. Can be overriden in order to change default behaviour.
            // returns:
            //      By default widget DOM node is returned.
            // tags:
            //		protected

            return this.disabledNode || this.domNode;
        },

        //  notificationParent: Object
        //    Widget where disabled notification should be added.
        notificationParent: null,

        _getNotificationParentAttr: function () {
            // summary:
            //		Gets the widget that should be used as parent where disabled notification should be added.
            //      Can be overriden in order to change default behaviour.
            // returns:
            //      By default this widget is returned.
            // tags:
            //		protected

            return this.notificationParent || this;
        },

        //  notificationPosition: Integer
        //    Integer position where disabled notification should be added in the parent widget.
        notificationPosition: null,

        _getNotificationPositionAttr: function () {
            // summary:
            //		Gets the position where disabled notification should be added in the parent widget.
            //      Can be overriden in order to change default behaviour.
            // returns:
            //      By default 0 is returned, so notification bar should be added on the first place inside parent widget.
            // tags:
            //		protected

            return this.notificationPosition || 0;
        },

        // notificationClass: String
        //      CSS class that should be used for notification bar.
        notificationClass: "epi-notificationBar-disabledView",

        // disabledClass: String
        //      CSS class that should be used for the DOM node that visually represents disabled area.
        disabledClass: "epi-pane__content--disabled",

        // _disabledNotification: Object
        //      Disabled notification bar widget.
        _disabledNotification: null,

        disable: function () {
            //  summary:
            //    Disables the view by adding overlay and notification bar.
            //    This method can be overriden if additional custom behaviour is required when disabling the view.
            //
            // tags:
            //		public

            var disabledNode = this.get("disabledNode");
            if (disabledNode) {
                domClass.add(disabledNode, this.disabledClass);
            }

            var position = this.get("notificationPosition");
            var parent = this.get("notificationParent");

            if (this._disabledNotification || !parent) {
                return;
            }
            this._disabledNotification = this._createNotification();
            this.own(this._disabledNotification);

            this._disabledNotification.placeAt(parent, position);
        },

        enable: function () {
            //  summary:
            //    Enables the view by removing overlay and notification bar.
            //    This method can be overriden if additional custom behaviour is required when enabling the view.
            //
            // tags:
            //		public

            if (this._disabledNotification) {
                this._disabledNotification.destroy();
            }
            this._disabledNotification = null;

            var disabledNode = this.get("disabledNode");
            if (disabledNode) {
                domClass.remove(disabledNode, this.disabledClass);
            }
        },

        postCreate: function () {
            //  summary:
            //    Overriden in order to disable the view if it is required
            //
            // tags:
            //		public

            this.inherited(arguments);
            if (this.disabledView) {
                this.disable();
            }
        },

        _createNotification: function () {
            //  summary:
            //    Creates notification bar that should be displayed on disabled views.
            //    This method can be overriden if custom notification widget should be created for disabled views.
            //
            // tags:
            //		protected

            var notification = new NotificationBar({
                "class": this.notificationClass
            });

            var orderFindUrl = i18n["orderFindUrl"] ||
                ( config.find && config.find.orderFindUrl ? config.find.orderFindUrl : "http://episerver.com" );

            notification.addNotification({
                message: lang.replace(i18n.disabledMessage, [ orderFindUrl ]),
                type: "info",
                persistent: true
            });

            return notification;
        }

    });
});

},
'epi-find/widget/ContentWrapperWithSidebar':function(){
define([
    "dojo/_base/declare",
    "dojo/dom-construct",
    "dijit/_WidgetBase",
    "dijit/_Container",
    "dijit/_TemplatedMixin",

    "dojo/text!./templates/ContentWrapperWithSidebar.html"
],
function (declare, domConstruct,
    _WidgetBase, _Container, _TemplatedMixin,
    template
) {

    // module:
    //      epi-find/widget/ContentWrapperWithSidebar

    return declare([_WidgetBase, _Container, _TemplatedMixin], {
        // summary:
        //     Content wrapper with sidebar

        templateString: template,

        sidebarWidget: null,

        addSidebar: function(/*dijit/_WidgetBase*/ widget){
            // summary:
            //      Makes the given widget a sidebar of this widget.

            var refNode = this.sidebarNode,
                sidebarWidget = this.sidebarWidget = widget;

            domConstruct.place(sidebarWidget.domNode, refNode);

            if(this._started && !sidebarWidget._started){
                sidebarWidget.startup();
            }
        },

        startup: function() {
            this.inherited(arguments);
            this.sidebarWidget.startup();
        }

    });

});
},
'url:epi-find/widget/templates/ContentWrapperWithSidebar.html':"<div class=\"epi-pane__content--with-sidebar  epi-pane__content__content-container\">\r\n    <div class=\"epi-pane__content__sidebar epi-pane__content__sidebar--right\" data-dojo-attach-point=\"sidebarNode\">\r\n    </div>\r\n    <div class=\"epi-pane__content__main\" data-dojo-attach-point=\"containerNode\">\r\n    </div>\r\n</div>",
'epi-find/widget/PhraseDashboardDialog':function(){
﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/when",
    "dojo/dom-class",
    "dojo/dom-attr",

    "dojo/store/Memory",

    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",

    "dojox/html/entities",

    "dgrid/OnDemandGrid",

    "epi/shell/widget/dialog/Dialog",

    "epi-saas-base/widgets/_NotificationMixin",
    "epi-saas-base/widgets/_GridNumberedColumnMixin",

    "dojo/i18n!./nls/PhraseDashboardDialog",
    "dojo/text!./templates/PhraseDashboardDialog.html"
], function(
        declare,
        lang,
        when,
        domClass,
        domAttr,

        MemoryStore,

        _WidgetBase,
        _TemplatedMixin,
        _WidgetsInTemplateMixin,

        entities,

        OnDemandGrid,

        Dialog,
        _NotificationMixin,
        _GridNumberedColumnMixin,

        i18n,
        template) {

    var bold = function(text) { return "<strong>"+text+"</strong>"; };
    var DialogContent = declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin], {

        model: null,

        i18n: i18n,

        templateString: template,

        dialogTitle: null,

        mostClickedTitle: null,
        _setMostClickedTitleAttr: {node: "mostClickedTitleNode", type: "innerHTML"},

        alsoSearchedTitle: null,
        _setAlsoSearchedTitleAttr: {node: "alsoSearchedTitleNode", type: "innerHTML"},

        showMostClicked: true,
        _setShowMostClickedAttr: function(value) {
            if (value)
            {
                domClass.add(this.alsoSearchedContainerNode, "epi-column");
                domClass.remove(this.mostClickedContainerNode, "epi-hidden");
            }
            else {
                domClass.remove(this.alsoSearchedContainerNode, "epi-column");
                domClass.add(this.mostClickedContainerNode, "epi-hidden");
            }
            this._set("showMostClicked", value);
        },

        showAlsoSearched: true,
        _setShowAlsoSearchedAttr: function(value) {
            if (value)
            {
                domClass.add(this.mostClickedContainerNode, "epi-column");
                domClass.remove(this.alsoSearchedContainerNode, "epi-hidden");
            }
            else {
                domClass.remove(this.mostClickedContainerNode, "epi-column");
                domClass.add(this.alsoSearchedContainerNode, "epi-hidden");
            }
            this._set("showAlsoSearched", value);
        },

        // query: string
        //      The search query that this dialog is currently showing, should be set externally.
        query: null,
        _setQueryAttr: function(value) {
            this._set("query", value);
            if (this.showMostClicked) {
                this._updateMostClicked();
            }
            if (this.showAlsoSearched) {
                this._updateAlsoSearched();
            }
        },

        _hitsStore: null,
        _hitGrid: null,
        _relatedQueriesStore: null,
        _relatedQueriesGrid: null,

        postCreate: function() {
            this.inherited(arguments);
            this._createGrids();
        },

        onError: function() {
            // summary:
            //      A callback for errors handling
        },

        _createGrids: function() {
            this._hitsStore = new MemoryStore();
            var NumberedGrid = declare([OnDemandGrid, _GridNumberedColumnMixin]);
            this._hitGrid = new NumberedGrid({
                    numberedColumnId: "phrase",
                    "class": "epi-plain-grid  epi-plain-grid--no-border  epi-vertical-spacer  epi-grid-max-height--700",
                    columns: {
                        phrase: {
                            field: "name",
                            formatter: function (value) {
                                if (!value) {
                                    return "<span class='epi-page-not-found'>" + i18n.notFound + "</span>";
                                }
                                return value;
                            },
                            label: i18n.searchResult,
                            sortable: false
                        },
                        count: {
                            field: "count",
                            label: i18n.clicks,
                            className: "epi-grid-column--centered  epi-positive",
                            sortable: false
                        }
                    },
                    noDataMessage: i18n.noResults,
                    store: this._hitsStore
            }, this.mostClickedGridNode);
            this.own(this._hitGrid);

            this._relatedQueriesStore = new MemoryStore();
            this._relatedQueriesGrid = new NumberedGrid({
                    numberedColumnId: "phrase",
                    "class": "epi-plain-grid  epi-plain-grid--no-border  epi-vertical-spacer  epi-grid-max-height--700",
                    columns: {
                        phrase: {
                            field: "query",
                                label: i18n.searchPhrase,
                            sortable: false
                        },
                        count: {
                            field: "count",
                                label: i18n.numberOfSearches,
                                className: "epi-grid-column--centered  epi-positive",
                                sortable: false
                        }
                    },
                    noDataMessage: i18n.noResults,
                    selectionMode: "toggle",
                    store: this._relatedQueriesStore
            }, this.alsoSearchedGridNode);
            this.own(this._relatedQueriesGrid);
        },

        _updateMostClicked: function() {
            var self = this, query = this.query;
            when(this.model.topHitStore.query({
                    size: 5,
                    query: query
                }),
                function(results) {
                    self.set("mostClickedTitle",
                        lang.replace(i18n.templates_mostClicked,
                            {clicks: bold(results.length), phrase: bold(entities.encode(query))}));

                    self._hitsStore.setData(results);
                    self._hitGrid.refresh();
                },
                function(error){
                    self._hitsStore.setData([]);
                    self._hitGrid.refresh();
                    self.onError(error);
                }
            );
        },

        _updateAlsoSearched: function() {
            var self = this;
            when(this.model.topQueryStore.query({
                    type: "related",
                    previousQuery: self.query,
                    size: 5
                }),
                function(results) {
                    self._relatedQueriesStore.setData(results);
                    self._relatedQueriesGrid.refresh();
                },
                function(error) {
                    self._relatedQueriesStore.setData([]);
                    self._relatedQueriesGrid.refresh();
                    self.onError(error);
                }
            );
            this.set("alsoSearchedTitle",
                lang.replace(i18n.templates_alsoSearched, {phrase: bold(entities.encode(self.query))}));

        }
    });

    return declare([Dialog, _NotificationMixin], {
        model: null,

        // errors: Array
        //      Holds array of errors that arise but wasn't shown yet to the user.
        errors: null,

        query: null,
        _setQueryAttr: function(value) {
            this.dialogContent.set("query", value);
            this.set("title", lang.replace(i18n.templates_title, {phrase: bold(entities.encode(value))}));
            domAttr.set(this.titleBar, "title", lang.replace(i18n.templates_title, {phrase: entities.encode(value)}));
            this._set("query", value);
        },

        showMostClicked: true,
        _setShowMostClickedAttr: function(value) {
            this.dialogContent.set("showMostClicked", value);
            this._set("showMostClicked", value);
            this._changeWidth();
        },

        showAlsoSearched: true,
        _setShowAlsoSearchedAttr: function(value) {
            this.dialogContent.set("showAlsoSearched", value);
            this._set("showAlsoSearched", value);
            this._changeWidth();
        },

        dialogClass:"epi-phrase-dashboard-dialog",
        dialogContent: null,
        defaultActionsVisible: false,

        getActions: function() {
            return [{
                label: i18n.close,
                title: null,
                settings: { type: "submit", "class": "Salt" }
            }];
        },

        constructor: function(options) {
            var self = this;
            this.errors = [];
            declare.safeMixin(this, options);
            this.dialogContent = new DialogContent({
                model: this.model,
                dialog: this,
                onError: function(error) {
                    if (self.open) {
                        self.showError(error);
                    }
                    else {
                        // Since this is a modal dialog we can't show error messages
                        // as another model confirm dialog until this dialog is opened.
                        // So we save error for later usage.
                        self.errors.push(error);
                    }
                }
            });
        },

        postCreate: function() {
            this.inherited(arguments);
            this.set("content", this.dialogContent);
        },

        show: function() {
            //  summary:
            //      Shows the dialog. After the dialog is shown, displays error messages that occurred meanwhile.
            var result = this.inherited(arguments), self = this;
            when(result, function() {
                var error;
                while ((error = self.errors.shift())) {
                    self.showError(error);
                }
            });
            return result;
        },

        _changeWidth: function () {
            if (this.showMostClicked && this.showAlsoSearched) {
                domClass.remove(this.domNode, "epi-phrase-dashboard-dialog__narrow");
            }
            else {
                domClass.add(this.domNode, "epi-phrase-dashboard-dialog__narrow");
            }
        }

    });
});

},
'dgrid/OnDemandGrid':function(){
define("dgrid/OnDemandGrid", ["dojo/_base/declare", "./Grid", "./OnDemandList"], function(declare, Grid, OnDemandList){
	return declare([Grid, OnDemandList], {});
});
},
'dgrid/Grid':function(){
define("dgrid/Grid", ["dojo/_base/kernel", "dojo/_base/declare", "dojo/on", "dojo/has", "put-selector/put", "./List", "./util/misc", "dojo/_base/sniff"],
function(kernel, declare, listen, has, put, List, miscUtil){
	var contentBoxSizing = has("ie") < 8 && !has("quirks");
	
	function appendIfNode(parent, subNode){
		if(subNode && subNode.nodeType){
			parent.appendChild(subNode);
		}
	}
	
	function replaceInvalidChars(str) {
		// Replaces invalid characters for a CSS identifier with hyphen,
		// as dgrid does for field names / column IDs when adding classes.
		return miscUtil.escapeCssIdentifier(str, "-");
	}
	
	var Grid = declare(List, {
		columns: null,
		// cellNavigation: Boolean
		//		This indicates that focus is at the cell level. This may be set to false to cause
		//		focus to be at the row level, which is useful if you want only want row-level
		//		navigation.
		cellNavigation: true,
		tabableHeader: true,
		showHeader: true,
		column: function(target){
			// summary:
			//		Get the column object by node, or event, or a columnId
			if(typeof target != "object"){
				return this.columns[target];
			}else{
				return this.cell(target).column;
			}
		},
		listType: "grid",
		cell: function(target, columnId){
			// summary:
			//		Get the cell object by node, or event, id, plus a columnId
			
			if(target.column && target.element){ return target; }
			
			if(target.target && target.target.nodeType){
				// event
				target = target.target;
			}
			var element;
			if(target.nodeType){
				var object;
				do{
					if(this._rowIdToObject[target.id]){
						break;
					}
					var colId = target.columnId;
					if(colId){
						columnId = colId;
						element = target;
						break;
					}
					target = target.parentNode;
				}while(target && target != this.domNode);
			}
			if(!element && typeof columnId != "undefined"){
				var row = this.row(target),
					rowElement = row && row.element;
				if(rowElement){
					var elements = rowElement.getElementsByTagName("td");
					for(var i = 0; i < elements.length; i++){
						if(elements[i].columnId == columnId){
							element = elements[i];
							break;
						}
					}
				}
			}
			if(target != null){
				return {
					row: row || this.row(target),
					column: columnId && this.column(columnId),
					element: element
				};
			}
		},
		
		createRowCells: function(tag, each, subRows, object){
			// summary:
			//		Generates the grid for each row (used by renderHeader and and renderRow)
			var row = put("table.dgrid-row-table[role=presentation]"),
				cellNavigation = this.cellNavigation,
				// IE < 9 needs an explicit tbody; other browsers do not
				tbody = (has("ie") < 9 || has("quirks")) ? put(row, "tbody") : row,
				tr,
				si, sl, i, l, // iterators
				subRow, column, id, extraClasses, className,
				cell, innerCell, colSpan, rowSpan; // used inside loops
			
			// Allow specification of custom/specific subRows, falling back to
			// those defined on the instance.
			subRows = subRows || this.subRows;
			
			for(si = 0, sl = subRows.length; si < sl; si++){
				subRow = subRows[si];
				// for single-subrow cases in modern browsers, TR can be skipped
				// http://jsperf.com/table-without-trs
				tr = put(tbody, "tr");
				if(subRow.className){
					put(tr, "." + subRow.className);
				}

				for(i = 0, l = subRow.length; i < l; i++){
					// iterate through the columns
					column = subRow[i];
					id = column.id;

					extraClasses = column.field ?
						".field-" + replaceInvalidChars(column.field) :
						"";
					className = typeof column.className === "function" ?
						column.className(object) : column.className;
					if(className){
						extraClasses += "." + className;
					}

					cell = put(tag + (
							".dgrid-cell.dgrid-cell-padding" +
							(id ? ".dgrid-column-" + replaceInvalidChars(id) : "") +
							extraClasses.replace(/ +/g, ".")
						) + "[role=" + (tag === "th" ? "columnheader" : "gridcell") + "]");
					cell.columnId = id;
					if(contentBoxSizing){
						// The browser (IE7-) does not support box-sizing: border-box, so we emulate it with a padding div
						innerCell = put(cell, "!dgrid-cell-padding div.dgrid-cell-padding");// remove the dgrid-cell-padding, and create a child with that class
						cell.contents = innerCell;
					}else{
						innerCell = cell;
					}
					colSpan = column.colSpan;
					if(colSpan){
						cell.colSpan = colSpan;
					}
					rowSpan = column.rowSpan;
					if(rowSpan){
						cell.rowSpan = rowSpan;
					}
					each(innerCell, column);
					// add the td to the tr at the end for better performance
					tr.appendChild(cell);
				}
			}
			return row;
		},
		
		left: function(cell, steps){
			if(!cell.element){ cell = this.cell(cell); }
			return this.cell(this._move(cell, -(steps || 1), "dgrid-cell"));
		},
		right: function(cell, steps){
			if(!cell.element){ cell = this.cell(cell); }
			return this.cell(this._move(cell, steps || 1, "dgrid-cell"));
		},
		
		renderRow: function(object, options){
			var self = this;
			var row = this.createRowCells("td", function(td, column){
				var data = object;
				// Support get function or field property (similar to DataGrid)
				if(column.get){
					data = column.get(object);
				}else if("field" in column && column.field != "_item"){
					data = data[column.field];
				}
				
				if(column.renderCell){
					// A column can provide a renderCell method to do its own DOM manipulation,
					// event handling, etc.
					appendIfNode(td, column.renderCell(object, data, td, options));
				}else{
					defaultRenderCell.call(column, object, data, td, options);
				}
			}, options && options.subRows, object);
			// row gets a wrapper div for a couple reasons:
			//	1. So that one can set a fixed height on rows (heights can't be set on <table>'s AFAICT)
			// 2. So that outline style can be set on a row when it is focused, and Safari's outline style is broken on <table>
			return put("div[role=row]>", row);
		},
		renderHeader: function(){
			// summary:
			//		Setup the headers for the grid
			var
				grid = this,
				columns = this.columns,
				headerNode = this.headerNode,
				i = headerNode.childNodes.length;
			
			headerNode.setAttribute("role", "row");
			
			// clear out existing header in case we're resetting
			while(i--){
				put(headerNode.childNodes[i], "!");
			}
			
			var row = this.createRowCells("th", function(th, column){
				var contentNode = column.headerNode = th;
				if(contentBoxSizing){
					// we're interested in the th, but we're passed the inner div
					th = th.parentNode;
				}
				var field = column.field;
				if(field){
					th.field = field;
				}
				// allow for custom header content manipulation
				if(column.renderHeaderCell){
					appendIfNode(contentNode, column.renderHeaderCell(contentNode));
				}else if("label" in column || column.field){
					contentNode.appendChild(document.createTextNode(
						"label" in column ? column.label : column.field));
				}
				if(column.sortable !== false && field && field != "_item"){
					th.sortable = true;
					th.className += " dgrid-sortable";
				}
			}, this.subRows && this.subRows.headerRows);
			this._rowIdToObject[row.id = this.id + "-header"] = this.columns;
			headerNode.appendChild(row);
			
			// If the columns are sortable, re-sort on clicks.
			// Use a separate listener property to be managed by renderHeader in case
			// of subsequent calls.
			if(this._sortListener){
				this._sortListener.remove();
			}
			this._sortListener = listen(row, "click,keydown", function(event){
				// respond to click, space keypress, or enter keypress
				if(event.type == "click" || event.keyCode == 32 /* space bar */ || (!has("opera") && event.keyCode == 13) /* enter */){
					var target = event.target,
						field, sort, newSort, eventObj;
					do{
						if(target.sortable){
							// If the click is on the same column as the active sort,
							// reverse sort direction
							newSort = [{
								attribute: (field = target.field || target.columnId),
								descending: (sort = grid._sort[0]) && sort.attribute == field &&
									!sort.descending
							}];
							
							// Emit an event with the new sort
							eventObj = {
								bubbles: true,
								cancelable: true,
								grid: grid,
								parentType: event.type,
								sort: newSort
							};
							
							if (listen.emit(event.target, "dgrid-sort", eventObj)){
								// Stash node subject to DOM manipulations,
								// to be referenced then removed by sort()
								grid._sortNode = target;
								grid.set("sort", newSort);
							}
							
							break;
						}
					}while((target = target.parentNode) && target != headerNode);
				}
			});
		},
		
		resize: function(){
			// extension of List.resize to allow accounting for
			// column sizes larger than actual grid area
			var
				headerTableNode = this.headerNode.firstChild,
				contentNode = this.contentNode,
				width;
			
			this.inherited(arguments);
			
			if(!has("ie") || (has("ie") > 7 && !has("quirks"))){
				// Force contentNode width to match up with header width.
				// (Old IEs don't have a problem due to how they layout.)
				
				contentNode.style.width = ""; // reset first
				
				if(contentNode && headerTableNode){
					if((width = headerTableNode.offsetWidth) != contentNode.offsetWidth){
						// update size of content node if necessary (to match size of rows)
						// (if headerTableNode can't be found, there isn't much we can do)
						contentNode.style.width = width + "px";
					}
				}
			}
		},
		
		destroy: function(){
			// Run _destroyColumns first to perform any column plugin tear-down logic.
			this._destroyColumns();
			if(this._sortListener){
				this._sortListener.remove();
			}
			
			this.inherited(arguments);
		},
		
		_setSort: function(property, descending){
			// summary:
			//		Extension of List.js sort to update sort arrow in UI
			
			// Normalize _sort first via inherited logic, then update the sort arrow
			this.inherited(arguments);
			this.updateSortArrow(this._sort);
		},
		
		_findSortArrowParent: function(field){
			// summary:
			//		Method responsible for finding cell that sort arrow should be
			//		added under.  Called by updateSortArrow; separated for extensibility.
			
			var columns = this.columns;
			for(var i in columns){
				var column = columns[i];
				if(column.field == field){
					return column.headerNode;
				}
			}
		},
		
		updateSortArrow: function(sort, updateSort){
			// summary:
			//		Method responsible for updating the placement of the arrow in the
			//		appropriate header cell.  Typically this should not be called (call
			//		set("sort", ...) when actually updating sort programmatically), but
			//		this method may be used by code which is customizing sort (e.g.
			//		by reacting to the dgrid-sort event, canceling it, then
			//		performing logic and calling this manually).
			// sort: Array
			//		Standard sort parameter - array of object(s) containing attribute
			//		and optionally descending property
			// updateSort: Boolean?
			//		If true, will update this._sort based on the passed sort array
			//		(i.e. to keep it in sync when custom logic is otherwise preventing
			//		it from being updated); defaults to false
			
			// Clean up UI from any previous sort
			if(this._lastSortedArrow){
				// Remove the sort classes from the parent node
				put(this._lastSortedArrow, "<!dgrid-sort-up!dgrid-sort-down");
				// Destroy the lastSortedArrow node
				put(this._lastSortedArrow, "!");
				delete this._lastSortedArrow;
			}
			
			if(updateSort){ this._sort = sort; }
			if(!sort[0]){ return; } // nothing to do if no sort is specified
			
			var prop = sort[0].attribute,
				desc = sort[0].descending,
				// if invoked from header click, target is stashed in _sortNode
				target = this._sortNode || this._findSortArrowParent(prop),
				arrowNode;
			
			delete this._sortNode;
			
			// Skip this logic if field being sorted isn't actually displayed
			if(target){
				target = target.contents || target;
				// Place sort arrow under clicked node, and add up/down sort class
				arrowNode = this._lastSortedArrow = put("div.dgrid-sort-arrow.ui-icon[role=presentation]");
				arrowNode.innerHTML = "&nbsp;";
				target.insertBefore(arrowNode, target.firstChild);
				put(target, desc ? ".dgrid-sort-down" : ".dgrid-sort-up");
				// Call resize in case relocation of sort arrow caused any height changes
				this.resize();
			}
		},
		
		styleColumn: function(colId, css){
			// summary:
			//		Dynamically creates a stylesheet rule to alter a column's style.
			
			return this.addCssRule("#" + miscUtil.escapeCssIdentifier(this.domNode.id) +
				" .dgrid-column-" + replaceInvalidChars(colId), css);
		},
		
		/*=====
		_configColumn: function(column, columnId, rowColumns, prefix){
			// summary:
			//		Method called when normalizing base configuration of a single
			//		column.  Can be used as an extension point for behavior requiring
			//		access to columns when a new configuration is applied.
		},=====*/
		
		_configColumns: function(prefix, rowColumns){
			// configure the current column
			var subRow = [],
				isArray = rowColumns instanceof Array;
			
			function configColumn(column, columnId){
				if(typeof column == "string"){
					rowColumns[columnId] = column = {label:column};
				}
				if(!isArray && !column.field){
					column.field = columnId;
				}
				columnId = column.id = column.id || (isNaN(columnId) ? columnId : (prefix + columnId));
				// allow further base configuration in subclasses
				if(this._configColumn){
					this._configColumn(column, columnId, rowColumns, prefix);
					// Allow the subclasses to modify the column id.
					columnId = column.id;
				}
				if(isArray){ this.columns[columnId] = column; }

				// add grid reference to each column object for potential use by plugins
				column.grid = this;
				if(typeof column.init === "function"){ column.init(); }
				
				subRow.push(column); // make sure it can be iterated on
			}
			
			miscUtil.each(rowColumns, configColumn, this);
			return isArray ? rowColumns : subRow;
		},
		
		_destroyColumns: function(){
			// summary:
			//		Iterates existing subRows looking for any column definitions with
			//		destroy methods (defined by plugins) and calls them.  This is called
			//		immediately before configuring a new column structure.
			
			var subRows = this.subRows,
				// if we have column sets, then we don't need to do anything with the missing subRows, ColumnSet will handle it
				subRowsLength = subRows && subRows.length,
				i, j, column, len;
			
			// First remove rows (since they'll be refreshed after we're done),
			// so that anything aspected onto removeRow by plugins can run.
			// (cleanup will end up running again, but with nothing to iterate.)
			this.cleanup();
			
			for(i = 0; i < subRowsLength; i++){
				for(j = 0, len = subRows[i].length; j < len; j++){
					column = subRows[i][j];
					if(typeof column.destroy === "function"){ column.destroy(); }
				}
			}
		},
		
		configStructure: function(){
			// configure the columns and subRows
			var subRows = this.subRows,
				columns = this._columns = this.columns;
			
			// Reset this.columns unless it was already passed in as an object
			this.columns = !columns || columns instanceof Array ? {} : columns;
			
			if(subRows){
				// Process subrows, which will in turn populate the this.columns object
				for(var i = 0; i < subRows.length; i++){
					subRows[i] = this._configColumns(i + "-", subRows[i]);
				}
			}else{
				this.subRows = [this._configColumns("", columns)];
			}
		},
		
		_getColumns: function(){
			// _columns preserves what was passed to set("columns"), but if subRows
			// was set instead, columns contains the "object-ified" version, which
			// was always accessible in the past, so maintain that accessibility going
			// forward.
			return this._columns || this.columns;
		},
		_setColumns: function(columns){
			this._destroyColumns();
			// reset instance variables
			this.subRows = null;
			this.columns = columns;
			// re-run logic
			this._updateColumns();
		},
		
		_setSubRows: function(subrows){
			this._destroyColumns();
			this.subRows = subrows;
			this._updateColumns();
		},
		
		setColumns: function(columns){
			kernel.deprecated("setColumns(...)", 'use set("columns", ...) instead', "dgrid 0.4");
			this.set("columns", columns);
		},
		setSubRows: function(subrows){
			kernel.deprecated("setSubRows(...)", 'use set("subRows", ...) instead', "dgrid 0.4");
			this.set("subRows", subrows);
		},
		
		_updateColumns: function(){
			// summary:
			//		Called when columns, subRows, or columnSets are reset
			
			this.configStructure();
			this.renderHeader();
			
			this.refresh();
			// re-render last collection if present
			this._lastCollection && this.renderArray(this._lastCollection);
			
			// After re-rendering the header, re-apply the sort arrow if needed.
			if(this._started){
				if(this._sort && this._sort.length){
					this.updateSortArrow(this._sort);
				} else {
					// Only call resize directly if we didn't call updateSortArrow,
					// since that calls resize itself when it updates.
					this.resize();
				}
			}
		}
	});
	
	function defaultRenderCell(object, data, td, options){
		if(this.formatter){
			// Support formatter, with or without formatterScope
			var formatter = this.formatter,
				formatterScope = this.grid.formatterScope;
			td.innerHTML = typeof formatter === "string" && formatterScope ?
				formatterScope[formatter](data, object) : this.formatter(data, object);
		}else if(data != null){
			td.appendChild(document.createTextNode(data)); 
		}
	}
	
	// expose appendIfNode and default implementation of renderCell,
	// e.g. for use by column plugins
	Grid.appendIfNode = appendIfNode;
	Grid.defaultRenderCell = defaultRenderCell;
	
	return Grid;
});

},
'dgrid/OnDemandList':function(){
define("dgrid/OnDemandList", ["./List", "./_StoreMixin", "dojo/_base/declare", "dojo/_base/lang", "dojo/_base/Deferred", "dojo/dom", "dojo/on", "./util/misc", "put-selector/put"],
function(List, _StoreMixin, declare, lang, Deferred, dom, listen, miscUtil, put){

return declare([List, _StoreMixin], {
	// summary:
	//		Extends List to include virtual scrolling functionality, querying a
	//		dojo/store instance for the appropriate range when the user scrolls.
	
	// minRowsPerPage: Integer
	//		The minimum number of rows to request at one time.
	minRowsPerPage: 25,
	
	// maxRowsPerPage: Integer
	//		The maximum number of rows to request at one time.
	maxRowsPerPage: 250,
	
	// maxEmptySpace: Integer
	//		Defines the maximum size (in pixels) of unrendered space below the
	//		currently-rendered rows. Setting this to less than Infinity can be useful if you
	//		wish to limit the initial vertical scrolling of the grid so that the scrolling is
	// 		not excessively sensitive. With very large grids of data this may make scrolling
	//		easier to use, albiet it can limit the ability to instantly scroll to the end.
	maxEmptySpace: Infinity,	
	
	// bufferRows: Integer
	//	  The number of rows to keep ready on each side of the viewport area so that the user can
	//	  perform local scrolling without seeing the grid being built. Increasing this number can
	//	  improve perceived performance when the data is being retrieved over a slow network.
	bufferRows: 10,
	
	// farOffRemoval: Integer
	//		Defines the minimum distance (in pixels) from the visible viewport area
	//		rows must be in order to be removed.  Setting to Infinity causes rows
	//		to never be removed.
	farOffRemoval: 2000,
	
	// queryRowsOverlap: Integer
	//		Indicates the number of rows to overlap queries. This helps keep
	//		continuous data when underlying data changes (and thus pages don't
	//		exactly align)
	queryRowsOverlap: 0,
	
	// pagingMethod: String
	//		Method (from dgrid/util/misc) to use to either throttle or debounce
	//		requests.  Default is "debounce" which will cause the grid to wait until
	//		the user pauses scrolling before firing any requests; can be set to
	//		"throttleDelayed" instead to progressively request as the user scrolls,
	//		which generally incurs more overhead but might appear more responsive.
	pagingMethod: "debounce",
	
	// pagingDelay: Integer
	//		Indicates the delay (in milliseconds) imposed upon pagingMethod, to wait
	//		before paging in more data on scroll events. This can be increased to
	//		reduce client-side overhead or the number of requests sent to a server.
	pagingDelay: miscUtil.defaultDelay,
	
	// keepScrollPosition: Boolean
	//		When refreshing the list, controls whether the scroll position is
	//		preserved, or reset to the top.  This can also be overridden for
	//		specific calls to refresh.
	keepScrollPosition: false,
	
	rowHeight: 22,
	
	postCreate: function(){
		this.inherited(arguments);
		var self = this;
		// check visibility on scroll events
		listen(this.bodyNode, "scroll",
			miscUtil[this.pagingMethod](function(event){ self._processScroll(event); },
				null, this.pagingDelay));
	},
	
	destroy: function(){
		this.inherited(arguments);
		if (this._refreshTimeout) {
			clearTimeout(this._refreshTimeout);
		}
	},
	
	renderQuery: function(query, options){
		// summary:
		//		Creates a preload node for rendering a query into, and executes the query
		//		for the first page of data. Subsequent data will be downloaded as it comes
		//		into view.
		var self = this,
			container = (options && options.container) || this.contentNode,
			preload = {
				query: query,
				count: 0,
				options: options
			},
			preloadNode,
			priorPreload = this.preload,
			results;
		
		// Initial query; set up top and bottom preload nodes
		var topPreload = {
			node: put(container, "div.dgrid-preload", {
				rowIndex: 0
			}),
			count: 0,
			query: query,
			next: preload,
			options: options
		};
		topPreload.node.style.height = "0";
		preload.node = preloadNode = put(container, "div.dgrid-preload");
		preload.previous = topPreload;
		
		// this preload node is used to represent the area of the grid that hasn't been
		// downloaded yet
		preloadNode.rowIndex = this.minRowsPerPage;

		if(priorPreload){
			// the preload nodes (if there are multiple) are represented as a linked list, need to insert it
			if((preload.next = priorPreload.next) && 
					// is this preload node below the prior preload node?
					preloadNode.offsetTop >= priorPreload.node.offsetTop){
				// the prior preload is above/before in the linked list
				preload.previous = priorPreload;
			}else{
				// the prior preload is below/after in the linked list
				preload.next = priorPreload;
				preload.previous = priorPreload.previous;
			}
			// adjust the previous and next links so the linked list is proper
			preload.previous.next = preload;
			preload.next.previous = preload; 
		}else{
			this.preload = preload;
		}
		
		var loadingNode = put(preloadNode, "-div.dgrid-loading"),
			innerNode = put(loadingNode, "div.dgrid-below");
		innerNode.innerHTML = this.loadingMessage;

		function errback(err) {
			// Used as errback for when calls;
			// remove the loadingNode and re-throw if an error was passed
			put(loadingNode, "!");
			
			if(err){
				if(self._refreshDeferred){
					self._refreshDeferred.reject(err);
					delete self._refreshDeferred;
				}
				throw err;
			}
		}

		// Establish query options, mixing in our own.
		// (The getter returns a delegated object, so simply using mixin is safe.)
		options = lang.mixin(this.get("queryOptions"), options, 
			{ start: 0, count: this.minRowsPerPage },
			"level" in query ? { queryLevel: query.level } : null);
		
		// Protect the query within a _trackError call, but return the QueryResults
		this._trackError(function(){ return results = query(options); });
		
		if(typeof results === "undefined"){
			// Synchronous error occurred (but was caught by _trackError)
			errback();
			return;
		}
		
		// Render the result set
		Deferred.when(self.renderArray(results, preloadNode, options), function(trs){
			var total = typeof results.total === "undefined" ?
				results.length : results.total;
			return Deferred.when(total, function(total){
				var trCount = trs.length,
					parentNode = preloadNode.parentNode,
					noDataNode = self.noDataNode;
				
				put(loadingNode, "!");
				if(!("queryLevel" in options)){
					self._total = total;
				}
				// now we need to adjust the height and total count based on the first result set
				if(total === 0 && parentNode){
					if(noDataNode){
						put(noDataNode, "!");
						delete self.noDataNode;
					}
					self.noDataNode = noDataNode = put("div.dgrid-no-data");
					parentNode.insertBefore(noDataNode, self._getFirstRowSibling(parentNode));
					noDataNode.innerHTML = self.noDataMessage;
				}
				var height = 0;
				for(var i = 0; i < trCount; i++){
					height += self._calcRowHeight(trs[i]);
				}
				// only update rowHeight if we actually got results and are visible
				if(trCount && height){ self.rowHeight = height / trCount; }
				
				total -= trCount;
				preload.count = total;
				preloadNode.rowIndex = trCount;
				if(total){
					preloadNode.style.height = Math.min(total * self.rowHeight, self.maxEmptySpace) + "px";
				}else{
					// if total is 0, IE quirks mode can't handle 0px height for some reason, I don't know why, but we are setting display: none for now
					preloadNode.style.display = "none";
					// This is a hack to get Observable to recognize that this is the
					// last page, like is done in the processScroll function
					options.count++;
				}
				
				if (self._previousScrollPosition) {
					// Restore position after a refresh operation w/ keepScrollPosition
					self.scrollTo(self._previousScrollPosition);
					delete self._previousScrollPosition;
				}
				
				// Redo scroll processing in case the query didn't fill the screen,
				// or in case scroll position was restored
				self._processScroll();
				
				// If _refreshDeferred is still defined after calling _processScroll,
				// resolve it now (_processScroll will remove it and resolve it itself
				// otherwise)
				if(self._refreshDeferred){
					self._refreshDeferred.resolve(results);
					delete self._refreshDeferred;
				}
				
				return trs;
			}, errback);
		}, errback);
		
		return results;
	},
	
	refresh: function(options){
		// summary:
		//		Refreshes the contents of the grid.
		// options: Object?
		//		Optional object, supporting the following parameters:
		//		* keepScrollPosition: like the keepScrollPosition instance property;
		//			specifying it in the options here will override the instance
		//			property's value for this specific refresh call only.
		
		var self = this,
			keep = (options && options.keepScrollPosition),
			dfd, results;
		
		// Fall back to instance property if option is not defined
		if(typeof keep === "undefined"){ keep = this.keepScrollPosition; }
		
		// Store scroll position to be restored after new total is received
		if(keep){ this._previousScrollPosition = this.getScrollPosition(); }
		
		this.inherited(arguments);
		if(this.store && this.shouldObserveStore){
			// render the query
			dfd = this._refreshDeferred = new Deferred();
			
			// renderQuery calls _trackError internally
			results = self.renderQuery(function(queryOptions){
				return self.store.query(self.query, queryOptions);
			});
			if(typeof results === "undefined"){
				// Synchronous error occurred; reject the refresh promise.
				dfd.reject();
			}
			
			// Internally, _refreshDeferred will always be resolved with an object
			// containing `results` (QueryResults) and `rows` (the rendered rows);
			// externally the promise will resolve simply with the QueryResults, but
			// the event will be emitted with both under respective properties.
			return dfd.then(function(results){
				// Emit on a separate turn to enable event to be used consistently for
				// initial render, regardless of whether the backing store is async
				self._refreshTimeout = setTimeout(function() {
					listen.emit(self.domNode, "dgrid-refresh-complete", {
						bubbles: true,
						cancelable: false,
						grid: self,
						results: results // QueryResults object (may be a wrapped promise)
					});
					self._refreshTimeout = null;
				}, 0);
				
				// Delete the Deferred immediately so nothing tries to re-resolve
				delete self._refreshDeferred;
				
				// Resolve externally with just the QueryResults
				return results;
			}, function(err){
				delete self._refreshDeferred;
				throw err;
			});
		}
	},
	
	resize: function(){
		this.inherited(arguments);
		this._processScroll();
	},

	_getFirstRowSibling: function(container){
		// summary:
		//		Returns the DOM node that a new row should be inserted before
		//		when there are no other rows in the current result set.
		//		In the case of OnDemandList, this will always be the last child
		//		of the container (which will be a trailing preload node).
		return container.lastChild;
	},
	
	_calcRowHeight: function(rowElement){
		// summary:
		//		Calculate the height of a row. This is a method so it can be overriden for
		//		plugins that add connected elements to a row, like the tree
		
		var sibling = rowElement.nextSibling;
		
		// If a next row exists, compare the top of this row with the
		// next one (in case "rows" are actually rendering side-by-side).
		// If no next row exists, this is either the last or only row,
		// in which case we count its own height.
		if(sibling && !/\bdgrid-preload\b/.test(sibling.className)){
			return sibling.offsetTop - rowElement.offsetTop;
		}
		
		return rowElement.offsetHeight;
	},
	
	lastScrollTop: 0,
	_processScroll: function(evt){
		// summary:
		//		Checks to make sure that everything in the viewable area has been
		//		downloaded, and triggering a request for the necessary data when needed.
		var grid = this,
			scrollNode = grid.bodyNode,
			// grab current visible top from event if provided, otherwise from node
			visibleTop = (evt && evt.scrollTop) || this.getScrollPosition().y,
			visibleBottom = scrollNode.offsetHeight + visibleTop,
			priorPreload, preloadNode, preload = grid.preload,
			lastScrollTop = grid.lastScrollTop,
			requestBuffer = grid.bufferRows * grid.rowHeight,
			searchBuffer = requestBuffer - grid.rowHeight, // Avoid rounding causing multiple queries
			// References related to emitting dgrid-refresh-complete if applicable
			refreshDfd,
			lastResults,
			lastRows,
			preloadSearchNext = true;
		
		// XXX: I do not know why this happens.
		// munging the actual location of the viewport relative to the preload node by a few pixels in either
		// direction is necessary because at least WebKit on Windows seems to have an error that causes it to
		// not quite get the entire element being focused in the viewport during keyboard navigation,
		// which means it becomes impossible to load more data using keyboard navigation because there is
		// no more data to scroll to to trigger the fetch.
		// 1 is arbitrary and just gets it to work correctly with our current test cases; don’t wanna go
		// crazy and set it to a big number without understanding more about what is going on.
		// wondering if it has to do with border-box or something, but changing the border widths does not
		// seem to make it break more or less, so I do not know…
		var mungeAmount = 1;
		
		grid.lastScrollTop = visibleTop;

		function removeDistantNodes(preload, distanceOff, traversal, below){
			// we check to see the the nodes are "far off"
			var farOffRemoval = grid.farOffRemoval,
				preloadNode = preload.node;
			// by checking to see if it is the farOffRemoval distance away
			if(distanceOff > 2 * farOffRemoval){
				// ok, there is preloadNode that is far off, let's remove rows until we get to in the current viewpoint
				var row, nextRow = preloadNode[traversal];
				var reclaimedHeight = 0;
				var count = 0;
				var toDelete = [];
				while((row = nextRow)){
					var rowHeight = grid._calcRowHeight(row);
					if(reclaimedHeight + rowHeight + farOffRemoval > distanceOff || (nextRow.className.indexOf("dgrid-row") < 0 && nextRow.className.indexOf("dgrid-loading") < 0)){
						// we have reclaimed enough rows or we have gone beyond grid rows, let's call it good
						break;
					}
					var nextRow = row[traversal]; // have to do this before removing it
					reclaimedHeight += rowHeight;
					count += row.count || 1;
					// we just do cleanup here, as we will do a more efficient node destruction in the setTimeout below
					grid.removeRow(row, true);
					toDelete.push(row);
				}
				// now adjust the preloadNode based on the reclaimed space
				preload.count += count;
				if(below){
					preloadNode.rowIndex -= count;
					adjustHeight(preload);
				}else{
					// if it is above, we can calculate the change in exact row changes, which we must do to not mess with the scrolling
					preloadNode.style.height = (preloadNode.offsetHeight + reclaimedHeight) + "px";
				}
				// we remove the elements after expanding the preload node so that the contraction doesn't alter the scroll position
				var trashBin = put("div", toDelete);
				setTimeout(function(){
					// we can defer the destruction until later
					put(trashBin, "!");
				},1);
			}
		}
		
		function adjustHeight(preload, noMax){
			preload.node.style.height = Math.min(preload.count * grid.rowHeight, noMax ? Infinity : grid.maxEmptySpace) + "px";
		}
		function traversePreload(preload, moveNext){
			do{
				preload = moveNext ? preload.next : preload.previous;
			}while(preload && !preload.node.offsetWidth);// skip past preloads that are not currently connected
			return preload;
		}
		while(preload && !preload.node.offsetWidth){
			// skip past preloads that are not currently connected
			preload = preload.previous;
		}
		// there can be multiple preloadNodes (if they split, or multiple queries are created),
		//	so we can traverse them until we find whatever is in the current viewport, making
		//	sure we don't backtrack
		while(preload && preload != priorPreload){
			priorPreload = grid.preload;
			grid.preload = preload;
			preloadNode = preload.node;
			var preloadTop = preloadNode.offsetTop;
			var preloadHeight;
			
			if(visibleBottom + mungeAmount + searchBuffer < preloadTop){
				// the preload is below the line of sight
				preload = traversePreload(preload, (preloadSearchNext = false));
			}else if(visibleTop - mungeAmount - searchBuffer > (preloadTop + (preloadHeight = preloadNode.offsetHeight))){
				// the preload is above the line of sight
				preload = traversePreload(preload, (preloadSearchNext = true));
			}else{
				// the preload node is visible, or close to visible, better show it
				var offset = ((preloadNode.rowIndex ? visibleTop - requestBuffer : visibleBottom) - preloadTop) / grid.rowHeight;
				var count = (visibleBottom - visibleTop + 2 * requestBuffer) / grid.rowHeight;
				// utilize momentum for predictions
				var momentum = Math.max(Math.min((visibleTop - lastScrollTop) * grid.rowHeight, grid.maxRowsPerPage/2), grid.maxRowsPerPage/-2);
				count += Math.min(Math.abs(momentum), 10);
				if(preloadNode.rowIndex == 0){
					// at the top, adjust from bottom to top
					offset -= count;
				}
				offset = Math.max(offset, 0);
				if(offset < 10 && offset > 0 && count + offset < grid.maxRowsPerPage){
					// connect to the top of the preloadNode if possible to avoid excessive adjustments
					count += Math.max(0, offset);
					offset = 0;
				}
				count = Math.min(Math.max(count, grid.minRowsPerPage),
									grid.maxRowsPerPage, preload.count);
				
				if(count == 0){
					preload = traversePreload(preload, preloadSearchNext);
					continue;
				}
				
				count = Math.ceil(count);
				offset = Math.min(Math.floor(offset), preload.count - count);
				var options = lang.mixin(grid.get("queryOptions"), preload.options);
				preload.count -= count;
				var beforeNode = preloadNode,
					keepScrollTo, queryRowsOverlap = grid.queryRowsOverlap,
					below = (preloadNode.rowIndex > 0 || preloadNode.offsetTop > visibleTop) && preload;
				if(below){
					// add new rows below
					var previous = preload.previous;
					if(previous){
						removeDistantNodes(previous, visibleTop - (previous.node.offsetTop + previous.node.offsetHeight), 'nextSibling');
						if(offset > 0 && previous.node == preloadNode.previousSibling){
							// all of the nodes above were removed
							offset = Math.min(preload.count, offset);
							preload.previous.count += offset;
							adjustHeight(preload.previous, true);
							preloadNode.rowIndex += offset;
							queryRowsOverlap = 0;
						}else{
							count += offset;
						}
						preload.count -= offset;
					}
					options.start = preloadNode.rowIndex - queryRowsOverlap;
					options.count = Math.min(count + queryRowsOverlap, grid.maxRowsPerPage);
					preloadNode.rowIndex = options.start + options.count;
				}else{
					// add new rows above
					if(preload.next){
						// remove out of sight nodes first
						removeDistantNodes(preload.next, preload.next.node.offsetTop - visibleBottom, 'previousSibling', true);
						var beforeNode = preloadNode.nextSibling;
						if(beforeNode == preload.next.node){
							// all of the nodes were removed, can position wherever we want
							preload.next.count += preload.count - offset;
							preload.next.node.rowIndex = offset + count;
							adjustHeight(preload.next);
							preload.count = offset;
							queryRowsOverlap = 0;
						}else{
							keepScrollTo = true;
						}
						
					}
					options.start = preload.count;
					options.count = Math.min(count + queryRowsOverlap, grid.maxRowsPerPage);
				}
				if(keepScrollTo && beforeNode && beforeNode.offsetWidth){
					keepScrollTo = beforeNode.offsetTop;
				}

				adjustHeight(preload);
				
				// use the query associated with the preload node to get the next "page"
				if("level" in preload.query){
					options.queryLevel = preload.query.level;
				}
				
				// Avoid spurious queries (ideally this should be unnecessary...)
				if(!("queryLevel" in options) && (options.start > grid._total || options.count < 0)){
					continue;
				}
				
				// create a loading node as a placeholder while the data is loaded
				var loadingNode = put(beforeNode, "-div.dgrid-loading[style=height:" + count * grid.rowHeight + "px]"),
					innerNode = put(loadingNode, "div.dgrid-" + (below ? "below" : "above"));
				innerNode.innerHTML = grid.loadingMessage;
				loadingNode.count = count;
				
				// Query now to fill in these rows.
				// Keep _trackError-wrapped results separate, since if results is a
				// promise, it will lose QueryResults functions when chained by `when`
				var results = preload.query(options),
					trackedResults = grid._trackError(function(){ return results; });
				
				if(trackedResults === undefined){
					// Sync query failed
					put(loadingNode, "!");
					return;
				}

				// Isolate the variables in case we make multiple requests
				// (which can happen if we need to render on both sides of an island of already-rendered rows)
				(function(loadingNode, below, keepScrollTo, results){
					lastRows = Deferred.when(grid.renderArray(results, loadingNode, options), function(rows){
						lastResults = results;
						
						// can remove the loading node now
						beforeNode = loadingNode.nextSibling;
						put(loadingNode, "!");
						if(keepScrollTo && beforeNode && beforeNode.offsetWidth){ // beforeNode may have been removed if the query results loading node was a removed as a distant node before rendering 
							// if the preload area above the nodes is approximated based on average
							// row height, we may need to adjust the scroll once they are filled in
							// so we don't "jump" in the scrolling position
							var pos = grid.getScrollPosition();
							grid.scrollTo({
								// Since we already had to query the scroll position,
								// include x to avoid TouchScroll querying it again on its end.
								x: pos.x,
								y: pos.y + beforeNode.offsetTop - keepScrollTo,
								// Don't kill momentum mid-scroll (for TouchScroll only).
								preserveMomentum: true
							});
						}
						
						Deferred.when(results.total || results.length, function(total){
							if(!("queryLevel" in options)){
								grid._total = total;
							}
							if(below){
								// if it is below, we will use the total from the results to update
								// the count of the last preload in case the total changes as later pages are retrieved
								// (not uncommon when total counts are estimated for db perf reasons)
								
								// recalculate the count
								below.count = total - below.node.rowIndex;
								// check to see if we are on the last page
								if(below.count === 0){
									// This is a hack to get Observable to recognize that this is the
									// last page; if the count doesn't match results.length, Observable
									// will think this is the last page and properly handle additions to the bottom
									options.count++;
								}
								// readjust the height
								adjustHeight(below);
							}
						});
						
						// make sure we have covered the visible area
						grid._processScroll();
						return rows;
					}, function (e) {
						put(loadingNode, "!");
						throw e;
					});
				}).call(this, loadingNode, below, keepScrollTo, results);
				preload = preload.previous;
			}
		}
		
		// After iterating, if additional requests have been made mid-refresh,
		// resolve the refresh promise based on the latest results obtained
		if (lastRows && (refreshDfd = this._refreshDeferred)) {
			delete this._refreshDeferred;
			Deferred.when(lastRows, function() {
				refreshDfd.resolve(lastResults);
			});
		}
	},

	removeRow: function(rowElement, justCleanup){
		function chooseIndex(index1, index2){
			return index1 != null ? index1 : index2;
		}

		if(rowElement){
			// Clean up observers that need to be cleaned up.
			var previousNode = rowElement.previousSibling,
				nextNode = rowElement.nextSibling,
				prevIndex = previousNode && chooseIndex(previousNode.observerIndex, previousNode.previousObserverIndex),
				nextIndex = nextNode && chooseIndex(nextNode.observerIndex, nextNode.nextObserverIndex),
				thisIndex = rowElement.observerIndex;

			// Clear the observerIndex on the node being removed so it will not be considered any longer.
			rowElement.observerIndex = undefined;
			if(justCleanup){
				// Save the indexes from the siblings for future calls to removeRow.
				rowElement.nextObserverIndex = nextIndex;
				rowElement.previousObserverIndex = prevIndex;
			}

			// Is this row's observer index different than those on either side?
			if(this.cleanEmptyObservers && thisIndex > -1 && thisIndex !== prevIndex && thisIndex !== nextIndex){
				// This is the last row that references the observer index.  Cancel the observer.
				var observers = this.observers;
				var observer = observers[thisIndex];
				if(observer){
					// justCleanup is set to true when the list is being cleaned out.  The rows are left in the DOM
					// and later they are removed altogether.  Skip the check for overlapping rows because
					// in the end, all of the rows will be removed and all of the observers need to be canceled.
					if(!justCleanup){
					// We need to verify that all the rows really have been removed. If there
					// are overlapping rows, it is possible another element exists
						var rows = observer.rows;
						for(var i = 0; i < rows.length; i++){
							if(rows[i] != rowElement && dom.isDescendant(rows[i], this.domNode)){
								// still rows in this list, abandon
								return this.inherited(arguments);
							}
						}
					}
					observer.cancel();
					this._numObservers--;
					observers[thisIndex] = 0; // remove it so we don't call cancel twice
				}
			}
		}
		// Finish the row removal.
		this.inherited(arguments);
	}
});

});

},
'dgrid/_StoreMixin':function(){
define("dgrid/_StoreMixin", ["dojo/_base/kernel", "dojo/_base/declare", "dojo/_base/lang", "dojo/_base/Deferred", "dojo/on", "dojo/aspect", "put-selector/put"],
function(kernel, declare, lang, Deferred, listen, aspect, put){
	// This module isolates the base logic required by store-aware list/grid
	// components, e.g. OnDemandList/Grid and the Pagination extension.
	
	// Noop function, needed for _trackError when callback due to a bug in 1.8
	// (see http://bugs.dojotoolkit.org/ticket/16667)
	function noop(value){ return value; }
	
	function emitError(err){
		// called by _trackError in context of list/grid, if an error is encountered
		if(typeof err !== "object"){
			// Ensure we actually have an error object, so we can attach a reference.
			err = new Error(err);
		}else if(err.dojoType === "cancel"){
			// Don't fire dgrid-error events for errors due to canceled requests
			// (unfortunately, the Deferred instrumentation will still log them)
			return;
		}
		// TODO: remove this @ 0.4 (prefer grid property directly on event object)
		err.grid = this;
		
		if(listen.emit(this.domNode, "dgrid-error", {
				grid: this,
				error: err,
				cancelable: true,
				bubbles: true })){
			console.error(err);
		}
	}
	
	return declare(null, {
		// store: Object
		//		The object store (implementing the dojo/store API) from which data is
		//		to be fetched.
		store: null,
		
		// query: Object
		//		Specifies query parameter(s) to pass to store.query calls.
		query: null,
		
		// queryOptions: Object
		//		Specifies additional query options to mix in when calling store.query;
		//		sort, start, and count are already handled.
		queryOptions: null,
		
		// getBeforePut: boolean
		//		If true, a get request will be performed to the store before each put
		//		as a baseline when saving; otherwise, existing row data will be used.
		getBeforePut: true,
		
		// noDataMessage: String
		//		Message to be displayed when no results exist for a query, whether at
		//		the time of the initial query or upon subsequent observed changes.
		//		Defined by _StoreMixin, but to be implemented by subclasses.
		noDataMessage: "",
		
		// loadingMessage: String
		//		Message displayed when data is loading.
		//		Defined by _StoreMixin, but to be implemented by subclasses.
		loadingMessage: "",
		
		constructor: function(){
			// Create empty objects on each instance, not the prototype
			this.query = {};
			this.queryOptions = {};
			this.dirty = {};
			this._updating = {}; // Tracks rows that are mid-update
			this._columnsWithSet = {};

			// Reset _columnsWithSet whenever column configuration is reset
			aspect.before(this, "configStructure", lang.hitch(this, function(){
				this._columnsWithSet = {};
			}));
		},
		
		postCreate: function(){
			this.inherited(arguments);
			if(this.store){
				this._updateNotifyHandle(this.store);
			}
		},
		
		destroy: function(){
			this.inherited(arguments);
			if(this._notifyHandle){
				this._notifyHandle.remove();
			}
		},
		
		_configColumn: function(column){
			// summary:
			//		Implements extension point provided by Grid to store references to
			//		any columns with `set` methods, for use during `save`.
			if (column.set){
				this._columnsWithSet[column.field] = column;
			}
			this.inherited(arguments);
		},
		
		_updateNotifyHandle: function(store){
			// summary:
			//		Unhooks any previously-existing store.notify handle, and
			//		hooks up a new one for the given store.
			
			if(this._notifyHandle){
				// Unhook notify handler from previous store
				this._notifyHandle.remove();
				delete this._notifyHandle;
			}
			if(store && typeof store.notify === "function" && this.shouldObserveStore){
				this._notifyHandle = aspect.after(store, "notify",
					lang.hitch(this, "_onNotify"), true);
				
				var sort = this.get("sort");
				if (!sort || !sort.length) {
					console.warn("Observable store detected, but no sort order specified. " +
						"You may experience quirks when adding/updating items.  " +
						"These can be resolved by setting a sort order on the list or grid.");
				}
			}
		},
		
		_setStore: function(store, query, queryOptions){
			// summary:
			//		Assigns a new store (and optionally query/queryOptions) to the list,
			//		and tells it to refresh.
			
			this._updateNotifyHandle(store);
			
			this.store = store;
			this.dirty = {}; // discard dirty map, as it applied to a previous store
			this.set("query", query, queryOptions);
		},
		_setQuery: function(query, queryOptions){
			// summary:
			//		Assigns a new query (and optionally queryOptions) to the list,
			//		and tells it to refresh.
			
			var sort = queryOptions && queryOptions.sort;
			
			this.query = query !== undefined ? query : this.query;
			this.queryOptions = queryOptions || this.queryOptions;
			
			// Avoid unnecessary refresh if instance hasn't started yet (startup will refresh)
			if (this._started) {
				// If we have new sort criteria, pass them through sort
				// (which will update _sort and call refresh in itself).
				// Otherwise, just refresh.
				sort ? this.set("sort", sort) : this.refresh();
			}
		},
		setStore: function(store, query, queryOptions){
			kernel.deprecated("setStore(...)", 'use set("store", ...) instead', "dgrid 0.4");
			this.set("store", store, query, queryOptions);
		},
		setQuery: function(query, queryOptions){
			kernel.deprecated("setQuery(...)", 'use set("query", ...) instead', "dgrid 0.4");
			this.set("query", query, queryOptions);
		},
		
		_getQueryOptions: function(){
			// summary:
			//		Get a fresh queryOptions object, also including the current sort
			var options = lang.delegate(this.queryOptions, {});
			if(typeof(this._sort) === "function" || this._sort.length){
				// Prevents SimpleQueryEngine from doing unnecessary "null" sorts (which can
				// change the ordering in browsers that don't use a stable sort algorithm, eg Chrome)
				options.sort = this._sort;
			}
			return options;
		},
		_getQuery: function(){
			// summary:
			//		Implemented consistent with _getQueryOptions so that if query is
			//		an object, this returns a protected (delegated) object instead of
			//		the original.
			var q = this.query;
			return typeof q == "object" && q != null ? lang.delegate(q, {}) : q;
		},
		
		_setSort: function(property, descending){
			// summary:
			//		Sort the content
			
			// prevent default storeless sort logic as long as we have a store
			if(this.store){ this._lastCollection = null; }
			this.inherited(arguments);
		},
		
		_onNotify: function(object, existingId){
			// summary:
			//		Method called when the store's notify method is called.
			
			// Call inherited in case anything was mixed in earlier
			this.inherited(arguments);
			
			// For adds/puts, check whether any observers are hooked up;
			// if not, force a refresh to properly hook one up now that there is data
			if(object && this._numObservers < 1){
				this.refresh({ keepScrollPosition: true });
			}
		},
		
		refresh: function(){
			var result = this.inherited(arguments);
			
			if(!this.store){
				this.noDataNode = put(this.contentNode, "div.dgrid-no-data");
				this.noDataNode.innerHTML = this.noDataMessage;
			}
			
			return result;
		},
		
		renderArray: function(){
			var self = this;
			var rows = this.inherited(arguments);
			
			if(!this.store){
				Deferred.when(rows, function(resolvedRows){
					if(resolvedRows.length && self.noDataNode){
						put(self.noDataNode, "!");
					}
				});
			}
			return rows;
		},
		
		insertRow: function(object, parent, beforeNode, i, options){
			var store = this.store,
				dirty = this.dirty,
				id = store && store.getIdentity(object),
				dirtyObj;
			
			if(id in dirty && !(id in this._updating)){ dirtyObj = dirty[id]; }
			if(dirtyObj){
				// restore dirty object as delegate on top of original object,
				// to provide protection for subsequent changes as well
				object = lang.delegate(object, dirtyObj);
			}
			return this.inherited(arguments);
		},
		
		updateDirty: function(id, field, value){
			// summary:
			//		Updates dirty data of a field for the item with the specified ID.
			var dirty = this.dirty,
				dirtyObj = dirty[id];
			
			if(!dirtyObj){
				dirtyObj = dirty[id] = {};
			}
			dirtyObj[field] = value;
		},
		setDirty: function(id, field, value){
			kernel.deprecated("setDirty(...)", "use updateDirty() instead", "dgrid 0.4");
			this.updateDirty(id, field, value);
		},
		
		save: function() {
			// Keep track of the store and puts
			var self = this,
				store = this.store,
				dirty = this.dirty,
				dfd = new Deferred(), promise = dfd.promise,
				getFunc = function(id){
					// returns a function to pass as a step in the promise chain,
					// with the id variable closured
					var data;
					return (self.getBeforePut || !(data = self.row(id).data)) ?
						function(){ return store.get(id); } :
						function(){ return data; };
				};
			
			// function called within loop to generate a function for putting an item
			function putter(id, dirtyObj) {
				// Return a function handler
				return function(object) {
					var colsWithSet = self._columnsWithSet,
						updating = self._updating,
						key, data;

					if (typeof object.set === "function") {
						object.set(dirtyObj);
					} else {
						// Copy dirty props to the original, applying setters if applicable
						for(key in dirtyObj){
							object[key] = dirtyObj[key];
						}
					}

					// Apply any set methods in column definitions.
					// Note that while in the most common cases column.set is intended
					// to return transformed data for the key in question, it is also
					// possible to directly modify the object to be saved.
					for(key in colsWithSet){
						data = colsWithSet[key].set(object);
						if(data !== undefined){ object[key] = data; }
					}
					
					updating[id] = true;
					// Put it in the store, returning the result/promise
					return Deferred.when(store.put(object), function() {
						// Clear the item now that it's been confirmed updated
						delete dirty[id];
						delete updating[id];
					});
				};
			}
			
			// For every dirty item, grab the ID
			for(var id in dirty) {
				// Create put function to handle the saving of the the item
				var put = putter(id, dirty[id]);
				
				// Add this item onto the promise chain,
				// getting the item from the store first if desired.
				promise = promise.then(getFunc(id)).then(put);
			}
			
			// Kick off and return the promise representing all applicable get/put ops.
			// If the success callback is fired, all operations succeeded; otherwise,
			// save will stop at the first error it encounters.
			dfd.resolve();
			return promise;
		},
		
		revert: function(){
			// summary:
			//		Reverts any changes since the previous save.
			this.dirty = {};
			this.refresh();
		},
		
		_trackError: function(func){
			// summary:
			//		Utility function to handle emitting of error events.
			// func: Function|String
			//		A function which performs some store operation, or a String identifying
			//		a function to be invoked (sans arguments) hitched against the instance.
			//		If sync, it can return a value, but may throw an error on failure.
			//		If async, it should return a promise, which would fire the error
			//		callback on failure.
			// tags:
			//		protected
			
			var result;
			
			if(typeof func == "string"){ func = lang.hitch(this, func); }
			
			try{
				result = func();
			}catch(err){
				// report sync error
				emitError.call(this, err);
			}
			
			// wrap in when call to handle reporting of potential async error
			return Deferred.when(result, noop, lang.hitch(this, emitError));
		},
		
		newRow: function(){
			// Override to remove no data message when a new row appears.
			// Run inherited logic first to prevent confusion due to noDataNode
			// no longer being present as a sibling.
			var row = this.inherited(arguments);
			if(this.noDataNode){
				put(this.noDataNode, "!");
				delete this.noDataNode;
			}
			return row;
		},
		removeRow: function(rowElement, justCleanup){
			var row = {element: rowElement};
			// Check to see if we are now empty...
			if(!justCleanup && this.noDataMessage &&
					(this.up(row).element === rowElement) &&
					(this.down(row).element === rowElement)){
				// ...we are empty, so show the no data message.
				this.noDataNode = put(this.contentNode, "div.dgrid-no-data");
				this.noDataNode.innerHTML = this.noDataMessage;
			}
			return this.inherited(arguments);
		}
	});
});

},
'epi-saas-base/widgets/_NotificationMixin':function(){
define("epi-saas-base/widgets/_NotificationMixin", [
        "dojo/_base/declare",
        "dojo/_base/lang",
        "dojo/topic",

        "put-selector/put",

        "dojo/i18n!./nls/_NotificationMixin"
    ],
    function(declare,
             lang,
             topic,
             put,
             i18n
        ) {

        return declare([], {
            // summary:
            //      A mixin for errors notification in widgets

            i18nFallback: i18n,
            dialog: null,

            postCreate: function() {
                this.inherited(arguments);
                this.i18n = this.i18n || {};
            },

            showNotification: function(notification) {
                topic.publish("epi/notification", notification);
            },

            showError: function(/*Object*/ error,  /*String*/ message, /*String*/ title, /*Callback*/ action) {
                var details = this._getErrorDetails(error);

                message = message || this._getErrorDescription(error);
                title = title || this._getErrorTitle(error);

                var notification = {
                    title: title,
                    message: message,
                    details: details,
                    type: "error",
                    persistent: true
                };

                if (action) {
                    notification.action = action;
                    this.showConfirmation(notification);
                }
                else {
                    this.showNotification(notification);
                }
            },

            showErrorInGrid: function(/*Object*/ error){
                this.showInGrid(this.i18n.dataLoadError || this._getErrorDescription(error));
            },

            _getErrorDescription: function(/* Object */ error) {
                if (!(error instanceof Object)) {
                    return error;
                }
                var errorMsgKey = "error" + error.status;
                var errorDefaultKey = "errorMessage";
                var description = lang.replace((this.i18n[errorMsgKey] || this.i18nFallback[errorMsgKey] ||
                    this.i18n[errorDefaultKey] || this.i18nFallback[errorDefaultKey]), error);
                return description;
            },

            _getErrorTitle: function(/* Object */ error) {
                if (!(error instanceof Object)) {
                    return null;
                }
                var errorMsgKey = "error" + error.status + "Title";
                var errorDefaultKey = "errorTitle";
                var title = lang.replace((this.i18n[errorMsgKey] || this.i18nFallback[errorMsgKey] ||
                    this.i18n[errorDefaultKey] || this.i18nFallback[errorDefaultKey]), error);
                return title;
            },

            _getErrorDetails: function(/* Object */ error) {
                if (!(error instanceof Object)) {
                    return null;
                }
                return error.message;
            },

            showInGrid: function(/*String*/ message) {
                this.inherited(arguments);
                if (!this.contentNode) {
                    throw Error("showInGrid method can be used only with grid");
                }
                if(this.noDataNode){
                    put(this.noDataNode, "!");
                    delete this.noDataNode;
                }
                this.noDataNode = put(this.contentNode, "div.dgrid-no-data");
                this.noDataNode.innerHTML = message;
            },

            showInfo: function(/*String*/ message, /*String*/ title) {
                this.showNotification({
                    title: title,
                    message: message,
                    type: "info"
                });
            },

            showSuccess: function(/*String*/ message, /*String*/ title) {
                this.showNotification({
                    title: title,
                    message: message,
                    type: "success"
                });
            },

            showConfirmation: function(options) {
                this.showNotification({
                    title: options.title,
                    message: options.message,
                    confirmActionText: options.confirmActionText,
                    cancelActionText: options.cancelActionText,
                    type: options.type || "warning",
                    action: options.action,
                    cancelAction: options.cancelAction
                });
            }
        });
    });

},
'epi-saas-base/widgets/_GridNumberedColumnMixin':function(){
define("epi-saas-base/widgets/_GridNumberedColumnMixin", [
    // Dojo
    "dojo/_base/declare",

    "dojo/dom-construct",
    "dojo/query"


], function(
    declare,
    domConstruct,
    query
) {
    // module: 
    //      epi-saas-base/widgets/_GridNumberedColumnMixin
    // summary:
    //      This mixin allows add numbering for a specified column

    return declare([], {
        // numberedColumnId: String
        //      Column ID where the numbering is going to be inserted.
        numberedColumnId: null,

        insertRow: function() {
            var row = this.inherited(arguments),
                numberCell = query(".dgrid-column-"+this.numberedColumnId, row),
                numberNode = domConstruct.create("span", {"class":"epi-row-number", innerHTML: row.rowIndex + 1});
            domConstruct.place(numberNode, numberCell[0], "first");
            return row;
        }
    });

});
},
'url:epi-find/widget/templates/PhraseDashboardDialog.html':"<div>\r\n    <div class=\"epi-column clearfix\" data-dojo-attach-point=\"mostClickedContainerNode\">\r\n        <p data-dojo-attach-point=\"mostClickedTitleNode\"></p>\r\n        <div data-dojo-attach-point=\"mostClickedGridNode\"></div>\r\n    </div>\r\n    <div class=\"epi-column clearfix\" data-dojo-attach-point=\"alsoSearchedContainerNode\">\r\n        <p data-dojo-attach-point=\"alsoSearchedTitleNode\"></p>\r\n        <div data-dojo-attach-point=\"alsoSearchedGridNode\"></div>\r\n    </div>\r\n</div>\r\n",
'epi-find/widget/HelpText':function(){
define([
    "dojo/_base/declare",
    "dojo/dom-construct",
    "dijit/_WidgetBase"

], function(declare, domConstruct, _WidgetBase) {

    // module:
    //      epi-find/widget/HelpText

    return declare(_WidgetBase, {
        // summary:
        //     Simple help text container.

        help: null,
        _setHelpAttr: { node: "domNode", type: "innerHTML" },
        buildRendering: function(){
           this.domNode = domConstruct.create("p", {"class":"epi-help", innerHTML: this.help});
        }
    });
});
},
'epi-find/widget/CustomRanges':function(){
define([
        "dojo/date",
        "dijit/layout/ContentPane",
        "dijit/form/DateTextBox",
        "dojo/i18n!./nls/Dashboard",
    ],
    function(
        date,
        ContentPane,
        DateTextBox,
        i18n
    ) {
    var today = new Date();

    var Ranges = {
        endDate: new DateTextBox({
            value: today,
            name: "endDate",
            disabled: true
        }),
        startDate: new DateTextBox({
            value: new Date(today.setDate(today.getDate()-1)),
            name: "startDate",
            disabled: true
        }),
        interval: "day",
        dateRange: new ContentPane({
            "class": "epi-pane__content__date-range-wrapper"
        }),

        disabledStatus: true,
        setDisabled: function(value) {
            Ranges.disabledStatus = value;
            Ranges.startDate.set("disabled", Ranges.disabledStatus);
            Ranges.endDate.set("disabled", Ranges.disabledStatus);
        }
    };

    var startDateDiv = dojo.create("div", { "class": "height-35" });
    dojo.place("<div class='float-left padding-top-5 padding-left-15'><label for='startDate'>" + i18n.startDate + "</label></div>", startDateDiv);
    var startDatePicker = dojo.create("div", { "class": "float-right" });
    dojo.place(Ranges.startDate.domNode, startDatePicker);
    dojo.place(startDatePicker, startDateDiv);

    var endDateDiv = dojo.create("div", { "class": "height-35" });
    dojo.place("<div class='float-left padding-top-5 padding-left-15'><label for='endDate'>" + i18n.endDate + "</label></div>", endDateDiv);
    var endDatePicker = dojo.create("div", { "class": "float-right" });
    dojo.place(Ranges.endDate.domNode, endDatePicker);
    dojo.place(endDatePicker, endDateDiv);

    Ranges.dateRange.domNode.appendChild(startDateDiv);
    Ranges.dateRange.domNode.appendChild(endDateDiv);

    return Ranges;
});

},
'dijit/form/DateTextBox':function(){
define("dijit/form/DateTextBox", [
	"dojo/_base/declare", // declare
	"../Calendar",
	"./_DateTimeTextBox"
], function(declare, Calendar, _DateTimeTextBox){

	// module:
	//		dijit/form/DateTextBox

	return declare("dijit.form.DateTextBox", _DateTimeTextBox, {
		// summary:
		//		A validating, serializable, range-bound date text box with a drop down calendar
		// example:
		// |	new DateTextBox({value: new Date(2009, 0, 20)})
		// example:
		// |	<input data-dojo-type='dijit/form/DateTextBox' value='2009-01-20'>

		baseClass: "dijitTextBox dijitComboBox dijitDateTextBox",
		popupClass: Calendar,
		_selector: "date",

		// value: Date
		//		The value of this widget as a JavaScript Date object, with only year/month/day specified.
		//		If specified in markup, use the format specified in `stamp.fromISOString`.
		//		set("value", ...) accepts either a Date object or a string.
		value: new Date("")	// value.toString()="NaN"
	});
});

},
'dijit/Calendar':function(){
define("dijit/Calendar", [
	"dojo/_base/array", // array.map
	"dojo/date",
	"dojo/date/locale",
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.get
	"dojo/dom-class", // domClass.add domClass.contains domClass.remove domClass.toggle
	"dojo/_base/event", // event.stop
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/keys", // keys
	"dojo/_base/lang", // lang.hitch
	"dojo/sniff", // has("ie")
	"./CalendarLite",
	"./_Widget",
	"./_CssStateMixin",
	"./_TemplatedMixin",
	"./form/DropDownButton"
], function(array, date, local, declare, domAttr, domClass, event, kernel, keys, lang, has,
			CalendarLite, _Widget, _CssStateMixin, _TemplatedMixin, DropDownButton){

	// module:
	//		dijit/Calendar

	var Calendar = declare("dijit.Calendar",
		[CalendarLite, _Widget, _CssStateMixin], // _Widget for deprecated methods like setAttribute()
		{
		// summary:
		//		A simple GUI for choosing a date in the context of a monthly calendar.
		//
		// description:
		//		See CalendarLite for general description.   Calendar extends CalendarLite, adding:
		//
		//		- month drop down list
		//		- keyboard navigation
		//		- CSS classes for hover/mousepress on date, month, and year nodes
		//		- support of deprecated methods (will be removed in 2.0)

		// Set node classes for various mouse events, see dijit._CssStateMixin for more details
		cssStateNodes: {
			"decrementMonth": "dijitCalendarArrow",
			"incrementMonth": "dijitCalendarArrow",
			"previousYearLabelNode": "dijitCalendarPreviousYear",
			"nextYearLabelNode": "dijitCalendarNextYear"
		},

		setValue: function(/*Date*/ value){
			// summary:
			//		Deprecated.   Use set('value', ...) instead.
			// tags:
			//		deprecated
			kernel.deprecated("dijit.Calendar:setValue() is deprecated.  Use set('value', ...) instead.", "", "2.0");
			this.set('value', value);
		},

		_createMonthWidget: function(){
			// summary:
			//		Creates the drop down button that displays the current month and lets user pick a new one

			return new Calendar._MonthDropDownButton({
				id: this.id + "_mddb",
				tabIndex: -1,
				onMonthSelect: lang.hitch(this, "_onMonthSelect"),
				lang: this.lang,
				dateLocaleModule: this.dateLocaleModule
			}, this.monthNode);
		},

		postCreate: function(){
			this.inherited(arguments);

			// Events specific to Calendar, not used in CalendarLite
			this.connect(this.domNode, "onkeydown", "_onKeyDown");
			this.connect(this.dateRowsNode, "onmouseover", "_onDayMouseOver");
			this.connect(this.dateRowsNode, "onmouseout", "_onDayMouseOut");
			this.connect(this.dateRowsNode, "onmousedown", "_onDayMouseDown");
			this.connect(this.dateRowsNode, "onmouseup", "_onDayMouseUp");
		},

		_onMonthSelect: function(/*Number*/ newMonth){
			// summary:
			//		Handler for when user selects a month from the drop down list
			// tags:
			//		protected

			// move to selected month, bounding by the number of days in the month
			// (ex: jan 31 --> feb 28, not feb 31)
			var date  = new this.dateClassObj(this.currentFocus);
			date.setDate(1);
			date.setMonth(newMonth);
			var daysInMonth = this.dateModule.getDaysInMonth(date);
			var currentDate = this.currentFocus.getDate();
			date.setDate(Math.min(currentDate, daysInMonth));
			this._setCurrentFocusAttr(date);
		},

		_onDayMouseOver: function(/*Event*/ evt){
			// summary:
			//		Handler for mouse over events on days, sets hovered style
			// tags:
			//		protected

			// event can occur on <td> or the <span> inside the td,
			// set node to the <td>.
			var node =
				domClass.contains(evt.target, "dijitCalendarDateLabel") ?
				evt.target.parentNode :
				evt.target;

			if(node && (
				(node.dijitDateValue && !domClass.contains(node, "dijitCalendarDisabledDate"))
					|| node == this.previousYearLabelNode || node == this.nextYearLabelNode
				)){
				domClass.add(node, "dijitCalendarHoveredDate");
				this._currentNode = node;
			}
		},

		_onDayMouseOut: function(/*Event*/ evt){
			// summary:
			//		Handler for mouse out events on days, clears hovered style
			// tags:
			//		protected

			if(!this._currentNode){ return; }

			// if mouse out occurs moving from <td> to <span> inside <td>, ignore it
			if(evt.relatedTarget && evt.relatedTarget.parentNode == this._currentNode){ return; }
			var cls = "dijitCalendarHoveredDate";
			if(domClass.contains(this._currentNode, "dijitCalendarActiveDate")){
				cls += " dijitCalendarActiveDate";
			}
			domClass.remove(this._currentNode, cls);
			this._currentNode = null;
		},

		_onDayMouseDown: function(/*Event*/ evt){
			var node = evt.target.parentNode;
			if(node && node.dijitDateValue && !domClass.contains(node, "dijitCalendarDisabledDate")){
				domClass.add(node, "dijitCalendarActiveDate");
				this._currentNode = node;
			}
		},

		_onDayMouseUp: function(/*Event*/ evt){
			var node = evt.target.parentNode;
			if(node && node.dijitDateValue){
				domClass.remove(node, "dijitCalendarActiveDate");
			}
		},

		handleKey: function(/*Event*/ evt){
			// summary:
			//		Provides keyboard navigation of calendar.
			// description:
			//		Called from _onKeyDown() to handle keypress on a stand alone Calendar,
			//		and also from `dijit/form/_DateTimeTextBox` to pass a keydown event
			//		from the `dijit/form/DateTextBox` to be handled in this widget
			// returns:
			//		False if the key was recognized as a navigation key,
			//		to indicate that the event was handled by Calendar and shouldn't be propagated
			// tags:
			//		protected
			var increment = -1,
				interval,
				newValue = this.currentFocus;
			switch(evt.keyCode){
				case keys.RIGHT_ARROW:
					increment = 1;
					//fallthrough...
				case keys.LEFT_ARROW:
					interval = "day";
					if(!this.isLeftToRight()){ increment *= -1; }
					break;
				case keys.DOWN_ARROW:
					increment = 1;
					//fallthrough...
				case keys.UP_ARROW:
					interval = "week";
					break;
				case keys.PAGE_DOWN:
					increment = 1;
					//fallthrough...
				case keys.PAGE_UP:
					interval = evt.ctrlKey || evt.altKey ? "year" : "month";
					break;
				case keys.END:
					// go to the next month
					newValue = this.dateModule.add(newValue, "month", 1);
					// subtract a day from the result when we're done
					interval = "day";
					//fallthrough...
				case keys.HOME:
					newValue = new this.dateClassObj(newValue);
					newValue.setDate(1);
					break;
				case keys.ENTER:
				case keys.SPACE:
					this.set("value", this.currentFocus);
					break;
				default:
					return true;
			}

			if(interval){
				newValue = this.dateModule.add(newValue, interval, increment);
			}

			this._setCurrentFocusAttr(newValue);

			return false;
		},

		_onKeyDown: function(/*Event*/ evt){
			// summary:
			//		For handling keypress events on a stand alone calendar
			if(!this.handleKey(evt)){
				event.stop(evt);
			}
		},

		onValueSelected: function(/*Date*/ /*===== date =====*/){
			// summary:
			//		Deprecated.   Notification that a date cell was selected.  It may be the same as the previous value.
			// description:
			//		Formerly used by `dijit/form/_DateTimeTextBox` (and thus `dijit/form/DateTextBox`)
			//		to get notification when the user has clicked a date.  Now onExecute() (above) is used.
			// tags:
			//		protected
		},

		onChange: function(value){
			this.onValueSelected(value);	// remove in 2.0
		},

		getClassForDate: function(/*===== dateObject, locale =====*/){
			// summary:
			//		May be overridden to return CSS classes to associate with the date entry for the given dateObject,
			//		for example to indicate a holiday in specified locale.
			// dateObject: Date
			// locale: String?
			// tags:
			//		extension

/*=====
			return ""; // String
=====*/
		}
	});

	Calendar._MonthDropDownButton = declare("dijit.Calendar._MonthDropDownButton", DropDownButton, {
		// summary:
		//		DropDownButton for the current month.    Displays name of current month
		//		and a list of month names in the drop down

		onMonthSelect: function(){ },

		postCreate: function(){
			this.inherited(arguments);
			this.dropDown = new Calendar._MonthDropDown({
				id: this.id + "_mdd", //do not change this id because it is referenced in the template
				onChange: this.onMonthSelect
			});
		},
		_setMonthAttr: function(month){
			// summary:
			//		Set the current month to display as a label
			var monthNames = this.dateLocaleModule.getNames('months', 'wide', 'standAlone', this.lang, month);
			this.dropDown.set("months", monthNames);

			// Set name of current month and also fill in spacer element with all the month names
			// (invisible) so that the maximum width will affect layout.   But not on IE6 because then
			// the center <TH> overlaps the right <TH> (due to a browser bug).
			this.containerNode.innerHTML =
				(has("ie") == 6 ? "" : "<div class='dijitSpacer'>" + this.dropDown.domNode.innerHTML + "</div>") +
				"<div class='dijitCalendarMonthLabel dijitCalendarCurrentMonthLabel'>" +  monthNames[month.getMonth()] + "</div>";
		}
	});

	Calendar._MonthDropDown = declare("dijit.Calendar._MonthDropDown", [_Widget, _TemplatedMixin], {
		// summary:
		//		The list-of-months drop down from the MonthDropDownButton

		// months: String[]
		//		List of names of months, possibly w/some undefined entries for Hebrew leap months
		//		(ex: ["January", "February", undefined, "April", ...])
		months: [],

		templateString: "<div class='dijitCalendarMonthMenu dijitMenu' " +
			"data-dojo-attach-event='onclick:_onClick,onmouseover:_onMenuHover,onmouseout:_onMenuHover'></div>",

		_setMonthsAttr: function(/*String[]*/ months){
			this.domNode.innerHTML = array.map(months, function(month, idx){
					return month ? "<div class='dijitCalendarMonthLabel' month='" + idx +"'>" + month + "</div>" : "";
				}).join("");
		},

		_onClick: function(/*Event*/ evt){
			this.onChange(domAttr.get(evt.target, "month"));
		},

		onChange: function(/*Number*/ /*===== month =====*/){
			// summary:
			//		Callback when month is selected from drop down
		},

		_onMenuHover: function(evt){
			domClass.toggle(evt.target, "dijitCalendarMonthLabelHover", evt.type == "mouseover");
		}
	});

	return Calendar;
});

},
'dijit/CalendarLite':function(){
require({cache:{
'url:dijit/templates/Calendar.html':"<table cellspacing=\"0\" cellpadding=\"0\" class=\"dijitCalendarContainer\" role=\"grid\" aria-labelledby=\"${id}_mddb ${id}_year\" data-dojo-attach-point=\"gridNode\">\n\t<thead>\n\t\t<tr class=\"dijitReset dijitCalendarMonthContainer\" valign=\"top\">\n\t\t\t<th class='dijitReset dijitCalendarArrow' data-dojo-attach-point=\"decrementMonth\" scope=\"col\">\n\t\t\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitCalendarIncrementControl dijitCalendarDecrease\" role=\"presentation\"/>\n\t\t\t\t<span data-dojo-attach-point=\"decreaseArrowNode\" class=\"dijitA11ySideArrow\">-</span>\n\t\t\t</th>\n\t\t\t<th class='dijitReset' colspan=\"5\" scope=\"col\">\n\t\t\t\t<div data-dojo-attach-point=\"monthNode\">\n\t\t\t\t</div>\n\t\t\t</th>\n\t\t\t<th class='dijitReset dijitCalendarArrow' scope=\"col\" data-dojo-attach-point=\"incrementMonth\">\n\t\t\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitCalendarIncrementControl dijitCalendarIncrease\" role=\"presentation\"/>\n\t\t\t\t<span data-dojo-attach-point=\"increaseArrowNode\" class=\"dijitA11ySideArrow\">+</span>\n\t\t\t</th>\n\t\t</tr>\n\t\t<tr role=\"row\">\n\t\t\t${!dayCellsHtml}\n\t\t</tr>\n\t</thead>\n\t<tbody data-dojo-attach-point=\"dateRowsNode\" data-dojo-attach-event=\"onclick: _onDayClick\" class=\"dijitReset dijitCalendarBodyContainer\">\n\t\t\t${!dateRowsHtml}\n\t</tbody>\n\t<tfoot class=\"dijitReset dijitCalendarYearContainer\">\n\t\t<tr>\n\t\t\t<td class='dijitReset' valign=\"top\" colspan=\"7\" role=\"presentation\">\n\t\t\t\t<div class=\"dijitCalendarYearLabel\">\n\t\t\t\t\t<span data-dojo-attach-point=\"previousYearLabelNode\" class=\"dijitInline dijitCalendarPreviousYear\" role=\"button\"></span>\n\t\t\t\t\t<span data-dojo-attach-point=\"currentYearLabelNode\" class=\"dijitInline dijitCalendarSelectedYear\" role=\"button\" id=\"${id}_year\"></span>\n\t\t\t\t\t<span data-dojo-attach-point=\"nextYearLabelNode\" class=\"dijitInline dijitCalendarNextYear\" role=\"button\"></span>\n\t\t\t\t</div>\n\t\t\t</td>\n\t\t</tr>\n\t</tfoot>\n</table>\n"}});
define("dijit/CalendarLite", [
	"dojo/_base/array", // array.forEach array.map
	"dojo/_base/declare", // declare
	"dojo/cldr/supplemental", // cldrSupplemental.getFirstDayOfWeek
	"dojo/date", // date
	"dojo/date/locale",
	"dojo/date/stamp", // stamp.fromISOString
	"dojo/dom", // dom.setSelectable
	"dojo/dom-class", // domClass.contains
	"dojo/_base/event", // event.stop
	"dojo/_base/lang", // lang.getObject, lang.hitch
	"dojo/sniff", // has("ie") has("webkit")
	"dojo/string", // string.substitute
	"./_WidgetBase",
	"./_TemplatedMixin",
	"dojo/text!./templates/Calendar.html",
	"./hccss"	// not used directly, but sets CSS class on <body>
], function(array, declare, cldrSupplemental, date, locale, stamp, dom, domClass, event, lang, has, string,
			_WidgetBase, _TemplatedMixin, template){


	// module:
	//		dijit/CalendarLite

	var CalendarLite = declare("dijit.CalendarLite", [_WidgetBase, _TemplatedMixin], {
		// summary:
		//		Lightweight version of Calendar widget aimed towards mobile use
		//
		// description:
		//		A simple GUI for choosing a date in the context of a monthly calendar.
		//		This widget can't be used in a form because it doesn't serialize the date to an
		//		`<input>` field.  For a form element, use dijit/form/DateTextBox instead.
		//
		//		Note that the parser takes all dates attributes passed in the
		//		[RFC 3339 format](http://www.faqs.org/rfcs/rfc3339.html), e.g. `2005-06-30T08:05:00-07:00`
		//		so that they are serializable and locale-independent.
		//
		//		Also note that this widget isn't keyboard accessible; use dijit.Calendar for that
		// example:
		//	|	var calendar = new dijit.CalendarLite({}, dojo.byId("calendarNode"));
		//
		// example:
		//	|	<div data-dojo-type="dijit/CalendarLite"></div>

		// Template for main calendar
		templateString: template,

		// Template for cell for a day of the week (ex: M)
		dowTemplateString: '<th class="dijitReset dijitCalendarDayLabelTemplate" role="columnheader" scope="col"><span class="dijitCalendarDayLabel">${d}</span></th>',

		// Templates for a single date (ex: 13), and for a row for a week (ex: 20 21 22 23 24 25 26)
		dateTemplateString: '<td class="dijitReset" role="gridcell" data-dojo-attach-point="dateCells"><span class="dijitCalendarDateLabel" data-dojo-attach-point="dateLabels"></span></td>',
		weekTemplateString: '<tr class="dijitReset dijitCalendarWeekTemplate" role="row">${d}${d}${d}${d}${d}${d}${d}</tr>',

		// value: Date
		//		The currently selected Date, initially set to invalid date to indicate no selection.
		value: new Date(""),
		// TODO: for 2.0 make this a string (ISO format) rather than a Date

		// datePackage: String
		//		JavaScript namespace to find calendar routines.	 If unspecified, uses Gregorian calendar routines
		//		at dojo/date and dojo/date/locale.
		datePackage: "",
		//		TODO: for 2.0, replace datePackage with dateModule and dateLocalModule attributes specifying MIDs,
		//		or alternately just get rid of this completely and tell user to use module ID remapping
		//		via require

		// dayWidth: String
		//		How to represent the days of the week in the calendar header. See locale
		dayWidth: "narrow",

		// tabIndex: String
		//		Order fields are traversed when user hits the tab key
		tabIndex: "0",

		// currentFocus: Date
		//		Date object containing the currently focused date, or the date which would be focused
		//		if the calendar itself was focused.   Also indicates which year and month to display,
		//		i.e. the current "page" the calendar is on.
		currentFocus: new Date(),

		// Put the summary to the node with role=grid
		_setSummaryAttr: "gridNode",

		baseClass:"dijitCalendar",

		_isValidDate: function(/*Date*/ value){
			// summary:
			//		Runs various tests on the value, checking that it's a valid date, rather
			//		than blank or NaN.
			// tags:
			//		private
			return value && !isNaN(value) && typeof value == "object" &&
				value.toString() != this.constructor.prototype.value.toString();
		},

		_getValueAttr: function(){
			// summary:
			//		Support get('value')

			// this.value is set to 1AM, but return midnight, local time for back-compat
			if(this.value && !isNaN(this.value)){
				var value = new this.dateClassObj(this.value);
				value.setHours(0, 0, 0, 0);

				// If daylight savings pushes midnight to the previous date, fix the Date
				// object to point at 1am so it will represent the correct day. See #9366
				if(value.getDate() < this.value.getDate()){
					value = this.dateModule.add(value, "hour", 1);
				}
				return value;
			}else{
				return null;
			}
		},

		_setValueAttr: function(/*Date|Number*/ value, /*Boolean*/ priorityChange){
			// summary:
			//		Support set("value", ...)
			// description:
			//		Set the current date and update the UI.  If the date is disabled, the value will
			//		not change, but the display will change to the corresponding month.
			// value:
			//		Either a Date or the number of seconds since 1970.
			// tags:
			//		protected
			if(typeof value == "string"){
				value = stamp.fromISOString(value);
			}
			value = this._patchDate(value);
			
			if(this._isValidDate(value) && !this.isDisabledDate(value, this.lang)){
				this._set("value", value);

				// Set focus cell to the new value.   Arguably this should only happen when there isn't a current
				// focus point.   This will also repopulate the grid to new month/year if necessary.
				this.set("currentFocus", value);

				// Mark the selected date
				this._markSelectedDates([value]);

				if(this._created && (priorityChange || typeof priorityChange == "undefined")){
					this.onChange(this.get('value'));
				}
			}else{
				// clear value, and mark all dates as unselected
				this._set("value", null);
				this._markSelectedDates([]);
			}
		},

		_patchDate: function(/*Date|Number*/ value){
			// summary:
			//		Convert Number into Date, or copy Date object.   Then, round to nearest day,
			//		setting to 1am to avoid issues when DST shift occurs at midnight, see #8521, #9366)
			if(value){
				value = new this.dateClassObj(value);
				value.setHours(1, 0, 0, 0);
			}
			return value;
		},

		_setText: function(node, text){
			// summary:
			//		This just sets the content of node to the specified text.
			//		Can't do "node.innerHTML=text" because of an IE bug w/tables, see #3434.
			// tags:
			//		private
			while(node.firstChild){
				node.removeChild(node.firstChild);
			}
			node.appendChild(node.ownerDocument.createTextNode(text));
		},

		_populateGrid: function(){
			// summary:
			//		Fills in the calendar grid with each day (1-31).
			//		Call this on creation, when moving to a new month.
			// tags:
			//		private

			var month = new this.dateClassObj(this.currentFocus);
			month.setDate(1);

			var firstDay = month.getDay(),
				daysInMonth = this.dateModule.getDaysInMonth(month),
				daysInPreviousMonth = this.dateModule.getDaysInMonth(this.dateModule.add(month, "month", -1)),
				today = new this.dateClassObj(),
				dayOffset = cldrSupplemental.getFirstDayOfWeek(this.lang);
			if(dayOffset > firstDay){ dayOffset -= 7; }

			// If they didn't provide a summary, change the default summary to match with the new month
			if(!this.summary){
					var monthNames = this.dateLocaleModule.getNames('months', 'wide', 'standAlone', this.lang, month)
					this.gridNode.setAttribute("summary", monthNames[month.getMonth()]);
			}

			// Mapping from date (as specified by number returned from Date.valueOf()) to corresponding <td>
			this._date2cell = {};

			// Iterate through dates in the calendar and fill in date numbers and style info
			array.forEach(this.dateCells, function(template, idx){
				var i = idx + dayOffset;
				var date = new this.dateClassObj(month),
					number, clazz = "dijitCalendar", adj = 0;

				if(i < firstDay){
					number = daysInPreviousMonth - firstDay + i + 1;
					adj = -1;
					clazz += "Previous";
				}else if(i >= (firstDay + daysInMonth)){
					number = i - firstDay - daysInMonth + 1;
					adj = 1;
					clazz += "Next";
				}else{
					number = i - firstDay + 1;
					clazz += "Current";
				}

				if(adj){
					date = this.dateModule.add(date, "month", adj);
				}
				date.setDate(number);

				if(!this.dateModule.compare(date, today, "date")){
					clazz = "dijitCalendarCurrentDate " + clazz;
				}

				if(this.isDisabledDate(date, this.lang)){
					clazz = "dijitCalendarDisabledDate " + clazz;
					template.setAttribute("aria-disabled", "true");
				}else{
					clazz = "dijitCalendarEnabledDate " + clazz;
					template.removeAttribute("aria-disabled");
					template.setAttribute("aria-selected", "false");
				}

				var clazz2 = this.getClassForDate(date, this.lang);
				if(clazz2){
					clazz = clazz2 + " " + clazz;
				}

				template.className = clazz + "Month dijitCalendarDateTemplate";

				// Each cell has an associated integer value representing it's date
				var dateVal = date.valueOf();
				this._date2cell[dateVal] = template;
				template.dijitDateValue = dateVal;

				// Set Date string (ex: "13").
				this._setText(this.dateLabels[idx], date.getDateLocalized ? date.getDateLocalized(this.lang) : date.getDate());
			}, this);
		},
		
		_populateControls: function(){
			// summary:
			//		Fill in localized month, and prev/current/next years
			// tags:
			//		protected

			var month = new this.dateClassObj(this.currentFocus);
			month.setDate(1);
			
			// set name of this month
			this.monthWidget.set("month", month);
			
			var y = month.getFullYear() - 1;
			var d = new this.dateClassObj();
			array.forEach(["previous", "current", "next"], function(name){
				d.setFullYear(y++);
				this._setText(this[name+"YearLabelNode"],
					this.dateLocaleModule.format(d, {selector:'year', locale:this.lang}));
			}, this);
		},

		goToToday: function(){
			// summary:
			//		Sets calendar's value to today's date
			this.set('value', new this.dateClassObj());
		},

		constructor: function(params /*===== , srcNodeRef =====*/){
			// summary:
			//		Create the widget.
			// params: Object|null
			//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
			//		and functions, typically callbacks like onClick.
			//		The hash can contain any of the widget's properties, excluding read-only properties.
			// srcNodeRef: DOMNode|String?
			//		If a srcNodeRef (DOM node) is specified, replace srcNodeRef with my generated DOM tree

			this.dateModule = params.datePackage ? lang.getObject(params.datePackage, false) : date;
			this.dateClassObj = this.dateModule.Date || Date;
			this.dateLocaleModule = params.datePackage ? lang.getObject(params.datePackage+".locale", false) : locale;
		},

		_createMonthWidget: function(){
			// summary:
			//		Creates the drop down button that displays the current month and lets user pick a new one

			return CalendarLite._MonthWidget({
				id: this.id + "_mddb",
				lang: this.lang,
				dateLocaleModule: this.dateLocaleModule
			}, this.monthNode);
		},

		buildRendering: function(){
			// Markup for days of the week (referenced from template)
			var d = this.dowTemplateString,
				dayNames = this.dateLocaleModule.getNames('days', this.dayWidth, 'standAlone', this.lang),
				dayOffset = cldrSupplemental.getFirstDayOfWeek(this.lang);
			this.dayCellsHtml = string.substitute([d,d,d,d,d,d,d].join(""), {d: ""}, function(){
				return dayNames[dayOffset++ % 7];
			});

			// Markup for dates of the month (referenced from template), but without numbers filled in
			var r = string.substitute(this.weekTemplateString, {d: this.dateTemplateString});
			this.dateRowsHtml = [r,r,r,r,r,r].join("");

			// Instantiate from template.
			// dateCells and dateLabels arrays filled when _Templated parses my template.
			this.dateCells = [];
			this.dateLabels = [];
			this.inherited(arguments);

			dom.setSelectable(this.domNode, false);

			var dateObj = new this.dateClassObj(this.currentFocus);

			this.monthWidget = this._createMonthWidget();

			this.set('currentFocus', dateObj, false);	// draw the grid to the month specified by currentFocus
		},

		postCreate: function(){
			this.inherited(arguments);
			this._connectControls();
		},

		_connectControls: function(){
			// summary:
			//		Set up connects for increment/decrement of months/years
			// tags:
			//		protected

			var connect = lang.hitch(this, function(nodeProp, part, amount){
				this.connect(this[nodeProp], "onclick", function(){
					this._setCurrentFocusAttr(this.dateModule.add(this.currentFocus, part, amount));
				});
			});
			
			connect("incrementMonth", "month", 1);
			connect("decrementMonth", "month", -1);
			connect("nextYearLabelNode", "year", 1);
			connect("previousYearLabelNode", "year", -1);
		},
		
		_setCurrentFocusAttr: function(/*Date*/ date, /*Boolean*/ forceFocus){
			// summary:
			//		If the calendar currently has focus, then focuses specified date,
			//		changing the currently displayed month/year if necessary.
			//		If the calendar doesn't have focus, updates currently
			//		displayed month/year, and sets the cell that will get focus
			//		when Calendar is focused.
			// forceFocus:
			//		If true, will focus() the cell even if calendar itself doesn't have focus

			var oldFocus = this.currentFocus,
				oldCell = this._getNodeByDate(oldFocus);
			date = this._patchDate(date);

			this._set("currentFocus", date);

			// If the focus is on a different month than the current calendar month, switch the displayed month.
			// Also will populate the grid initially, on Calendar creation.
			if(!this._date2cell || this.dateModule.difference(oldFocus, date, "month") != 0){
				this._populateGrid();
				this._populateControls();
				this._markSelectedDates([this.value]);
			}
			
			// set tabIndex=0 on new cell, and focus it (but only if Calendar itself is focused)
			var newCell = this._getNodeByDate(date);
			newCell.setAttribute("tabIndex", this.tabIndex);
			if(this.focused || forceFocus){
				newCell.focus();
			}

			// set tabIndex=-1 on old focusable cell
			if(oldCell && oldCell != newCell){
				if(has("webkit")){	// see #11064 about webkit bug
					oldCell.setAttribute("tabIndex", "-1");
				}else{
					oldCell.removeAttribute("tabIndex");
				}
			}
		},

		focus: function(){
			// summary:
			//		Focus the calendar by focusing one of the calendar cells
			this._setCurrentFocusAttr(this.currentFocus, true);
		},

		_onDayClick: function(/*Event*/ evt){
			// summary:
			//		Handler for day clicks, selects the date if appropriate
			// tags:
			//		protected
			event.stop(evt);
			for(var node = evt.target; node && !node.dijitDateValue; node = node.parentNode);
			if(node && !domClass.contains(node, "dijitCalendarDisabledDate")){
				this.set('value', node.dijitDateValue);
			}
		},

		_getNodeByDate : function(/*Date*/ value){
			// summary:
			//		Returns the cell corresponding to the date, or null if the date is not within the currently
			//		displayed month.
			value = this._patchDate(value);
			return value && this._date2cell ? this._date2cell[value.valueOf()] : null;
		},

		_markSelectedDates: function(/*Date[]*/ dates){
			// summary:
			//		Marks the specified cells as selected, and clears cells previously marked as selected.
			//		For CalendarLite at most one cell is selected at any point, but this allows an array
			//		for easy subclassing.

			// Function to mark a cell as selected or unselected
			function mark(/*Boolean*/ selected, /*DomNode*/ cell){
				domClass.toggle(cell, "dijitCalendarSelectedDate", selected);
				cell.setAttribute("aria-selected", selected ? "true" : "false");
			}

			// Clear previously selected cells.
			array.forEach(this._selectedCells || [], lang.partial(mark, false));

			// Mark newly selected cells.  Ignore dates outside the currently displayed month.
			this._selectedCells = array.filter(array.map(dates, this._getNodeByDate, this), function(n){ return n;});
			array.forEach(this._selectedCells, lang.partial(mark, true));
		},

		onChange: function(/*Date*/ /*===== date =====*/){
			// summary:
			//		Called only when the selected date has changed
		},

		isDisabledDate: function(/*===== dateObject, locale =====*/){
			// summary:
			//		May be overridden to disable certain dates in the calendar e.g. `isDisabledDate=dojo.date.locale.isWeekend`
			// dateObject: Date
			// locale: String?
			// tags:
			//		extension
/*=====
			return false; // Boolean
=====*/
		},

		getClassForDate: function(/*===== dateObject, locale =====*/){
			// summary:
			//		May be overridden to return CSS classes to associate with the date entry for the given dateObject,
			//		for example to indicate a holiday in specified locale.
			// dateObject: Date
			// locale: String?
			// tags:
			//		extension

/*=====
			return ""; // String
=====*/
		}
	});

	CalendarLite._MonthWidget = declare("dijit.CalendarLite._MonthWidget", _WidgetBase, {
		// summary:
		//		Displays name of current month padded to the width of the month
		//		w/the longest name, so that changing months doesn't change width.
		//
		//		Create as:
		// |	new Calendar._MonthWidget({
		// |			lang: ...,
		// |			dateLocaleModule: ...
		// |		})

		_setMonthAttr: function(month){
			// summary:
			//		Set the current month to display as a label
			var monthNames = this.dateLocaleModule.getNames('months', 'wide', 'standAlone', this.lang, month),
				spacer =
					(has("ie") == 6 ? "" :	"<div class='dijitSpacer'>" +
						array.map(monthNames, function(s){ return "<div>" + s + "</div>"; }).join("") + "</div>");

			// Set name of current month and also fill in spacer element with all the month names
			// (invisible) so that the maximum width will affect layout.   But not on IE6 because then
			// the center <TH> overlaps the right <TH> (due to a browser bug).
			this.domNode.innerHTML =
				spacer +
				"<div class='dijitCalendarMonthLabel dijitCalendarCurrentMonthLabel'>" +
				monthNames[month.getMonth()] + "</div>";
		}
	});

	return CalendarLite;
});

},
'url:dijit/templates/Calendar.html':"<table cellspacing=\"0\" cellpadding=\"0\" class=\"dijitCalendarContainer\" role=\"grid\" aria-labelledby=\"${id}_mddb ${id}_year\" data-dojo-attach-point=\"gridNode\">\n\t<thead>\n\t\t<tr class=\"dijitReset dijitCalendarMonthContainer\" valign=\"top\">\n\t\t\t<th class='dijitReset dijitCalendarArrow' data-dojo-attach-point=\"decrementMonth\" scope=\"col\">\n\t\t\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitCalendarIncrementControl dijitCalendarDecrease\" role=\"presentation\"/>\n\t\t\t\t<span data-dojo-attach-point=\"decreaseArrowNode\" class=\"dijitA11ySideArrow\">-</span>\n\t\t\t</th>\n\t\t\t<th class='dijitReset' colspan=\"5\" scope=\"col\">\n\t\t\t\t<div data-dojo-attach-point=\"monthNode\">\n\t\t\t\t</div>\n\t\t\t</th>\n\t\t\t<th class='dijitReset dijitCalendarArrow' scope=\"col\" data-dojo-attach-point=\"incrementMonth\">\n\t\t\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitCalendarIncrementControl dijitCalendarIncrease\" role=\"presentation\"/>\n\t\t\t\t<span data-dojo-attach-point=\"increaseArrowNode\" class=\"dijitA11ySideArrow\">+</span>\n\t\t\t</th>\n\t\t</tr>\n\t\t<tr role=\"row\">\n\t\t\t${!dayCellsHtml}\n\t\t</tr>\n\t</thead>\n\t<tbody data-dojo-attach-point=\"dateRowsNode\" data-dojo-attach-event=\"onclick: _onDayClick\" class=\"dijitReset dijitCalendarBodyContainer\">\n\t\t\t${!dateRowsHtml}\n\t</tbody>\n\t<tfoot class=\"dijitReset dijitCalendarYearContainer\">\n\t\t<tr>\n\t\t\t<td class='dijitReset' valign=\"top\" colspan=\"7\" role=\"presentation\">\n\t\t\t\t<div class=\"dijitCalendarYearLabel\">\n\t\t\t\t\t<span data-dojo-attach-point=\"previousYearLabelNode\" class=\"dijitInline dijitCalendarPreviousYear\" role=\"button\"></span>\n\t\t\t\t\t<span data-dojo-attach-point=\"currentYearLabelNode\" class=\"dijitInline dijitCalendarSelectedYear\" role=\"button\" id=\"${id}_year\"></span>\n\t\t\t\t\t<span data-dojo-attach-point=\"nextYearLabelNode\" class=\"dijitInline dijitCalendarNextYear\" role=\"button\"></span>\n\t\t\t\t</div>\n\t\t\t</td>\n\t\t</tr>\n\t</tfoot>\n</table>\n",
'dijit/form/_DateTimeTextBox':function(){
require({cache:{
'url:dijit/form/templates/DropDownBox.html':"<div class=\"dijit dijitReset dijitInline dijitLeft\"\n\tid=\"widget_${id}\"\n\trole=\"combobox\"\n\taria-haspopup=\"true\"\n\tdata-dojo-attach-point=\"_popupStateNode\"\n\t><div class='dijitReset dijitRight dijitButtonNode dijitArrowButton dijitDownArrowButton dijitArrowButtonContainer'\n\t\tdata-dojo-attach-point=\"_buttonNode\" role=\"presentation\"\n\t\t><input class=\"dijitReset dijitInputField dijitArrowButtonInner\" value=\"&#9660; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"button presentation\" aria-hidden=\"true\"\n\t\t\t${_buttonInputDisabled}\n\t/></div\n\t><div class='dijitReset dijitValidationContainer'\n\t\t><input class=\"dijitReset dijitInputField dijitValidationIcon dijitValidationInner\" value=\"&#935; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t/></div\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\n\t\t><input class='dijitReset dijitInputInner' ${!nameAttrSetting} type=\"text\" autocomplete=\"off\"\n\t\t\tdata-dojo-attach-point=\"textbox,focusNode\" role=\"textbox\"\n\t/></div\n></div>\n"}});
define("dijit/form/_DateTimeTextBox", [
	"dojo/date", // date date.compare
	"dojo/date/locale", // locale.regexp
	"dojo/date/stamp", // stamp.fromISOString stamp.toISOString
	"dojo/_base/declare", // declare
	"dojo/_base/lang", // lang.getObject
	"./RangeBoundTextBox",
	"../_HasDropDown",
	"dojo/text!./templates/DropDownBox.html"
], function(date, locale, stamp, declare, lang, RangeBoundTextBox, _HasDropDown, template){

	// module:
	//		dijit/form/_DateTimeTextBox

	new Date("X"); // workaround for #11279, new Date("") == NaN

	var _DateTimeTextBox = declare("dijit.form._DateTimeTextBox", [RangeBoundTextBox, _HasDropDown], {
		// summary:
		//		Base class for validating, serializable, range-bound date or time text box.

		templateString: template,

		// hasDownArrow: [const] Boolean
		//		Set this textbox to display a down arrow button, to open the drop down list.
		hasDownArrow: true,

		// Set classes like dijitDownArrowButtonHover depending on mouse action over button node
		cssStateNodes: {
			"_buttonNode": "dijitDownArrowButton"
		},

		/*=====
		// constraints: _DateTimeTextBox.__Constraints
		//		Despite the name, this parameter specifies both constraints on the input
		//		(including starting/ending dates/times allowed) as well as
		//		formatting options like whether the date is displayed in long (ex: December 25, 2005)
		//		or short (ex: 12/25/2005) format.  See `dijit/form/_DateTimeTextBox.__Constraints` for details.
		constraints: {},
		======*/

		// Override ValidationTextBox.pattern.... we use a reg-ex generating function rather
		// than a straight regexp to deal with locale  (plus formatting options too?)
		pattern: locale.regexp,

		// datePackage: String
		//		JavaScript namespace to find calendar routines.	 If unspecified, uses Gregorian calendar routines
		//		at dojo/date and dojo/date/locale.
		datePackage: "",
		//		TODO: for 2.0, replace datePackage with dateModule and dateLocalModule attributes specifying MIDs,
		//		or alternately just get rid of this completely and tell user to use module ID remapping
		//		via require

		postMixInProperties: function(){
			this.inherited(arguments);
			this._set("type", "text"); // in case type="date"|"time" was specified which messes up parse/format
		},

		// Override _FormWidget.compare() to work for dates/times
		compare: function(/*Date*/ val1, /*Date*/ val2){
			var isInvalid1 = this._isInvalidDate(val1);
			var isInvalid2 = this._isInvalidDate(val2);
			return isInvalid1 ? (isInvalid2 ? 0 : -1) : (isInvalid2 ? 1 : date.compare(val1, val2, this._selector));
		},

		// flag to _HasDropDown to make drop down Calendar width == <input> width
		forceWidth: true,

		format: function(/*Date*/ value, /*locale.__FormatOptions*/ constraints){
			// summary:
			//		Formats the value as a Date, according to specified locale (second argument)
			// tags:
			//		protected
			if(!value){ return ''; }
			return this.dateLocaleModule.format(value, constraints);
		},

		"parse": function(/*String*/ value, /*locale.__FormatOptions*/ constraints){
			// summary:
			//		Parses as string as a Date, according to constraints
			// tags:
			//		protected

			return this.dateLocaleModule.parse(value, constraints) || (this._isEmpty(value) ? null : undefined);	 // Date
		},

		// Overrides ValidationTextBox.serialize() to serialize a date in canonical ISO format.
		serialize: function(/*anything*/ val, /*Object?*/ options){
			if(val.toGregorian){
				val = val.toGregorian();
			}
			return stamp.toISOString(val, options);
		},

		// dropDownDefaultValue: Date
		//		The default value to focus in the popupClass widget when the textbox value is empty.
		dropDownDefaultValue : new Date(),

		// value: Date
		//		The value of this widget as a JavaScript Date object.  Use get("value") / set("value", val) to manipulate.
		//		When passed to the parser in markup, must be specified according to `dojo/date/stamp.fromISOString()`
		value: new Date(""),	// value.toString()="NaN"

		_blankValue: null,	// used by filter() when the textbox is blank

		// popupClass: [protected extension] String
		//		Name of the popup widget class used to select a date/time.
		//		Subclasses should specify this.
		popupClass: "", // default is no popup = text only


		// _selector: [protected extension] String
		//		Specifies constraints.selector passed to dojo.date functions, should be either
		//		"date" or "time".
		//		Subclass must specify this.
		_selector: "",

		constructor: function(params /*===== , srcNodeRef =====*/){
			// summary:
			//		Create the widget.
			// params: Object|null
			//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
			//		and functions, typically callbacks like onClick.
			//		The hash can contain any of the widget's properties, excluding read-only properties.
			// srcNodeRef: DOMNode|String?
			//		If a srcNodeRef (DOM node) is specified, replace srcNodeRef with my generated DOM tree

			this.dateModule = params.datePackage ? lang.getObject(params.datePackage, false) : date;
			this.dateClassObj = this.dateModule.Date || Date;
			this.dateLocaleModule = params.datePackage ? lang.getObject(params.datePackage+".locale", false) : locale;
			this._set('pattern', this.dateLocaleModule.regexp);
			this._invalidDate = this.constructor.prototype.value.toString();
		},

		buildRendering: function(){
			this.inherited(arguments);

			if(!this.hasDownArrow){
				this._buttonNode.style.display = "none";
			}

			// If hasDownArrow is false, we basically just want to treat the whole widget as the
			// button.
			if(!this.hasDownArrow){
				this._buttonNode = this.domNode;
				this.baseClass += " dijitComboBoxOpenOnClick";
			}
		},

		_setConstraintsAttr: function(/*Object*/ constraints){
			constraints.selector = this._selector;
			constraints.fullYear = true; // see #5465 - always format with 4-digit years
			var fromISO = stamp.fromISOString;
			if(typeof constraints.min == "string"){ constraints.min = fromISO(constraints.min); }
			if(typeof constraints.max == "string"){ constraints.max = fromISO(constraints.max); }
			this.inherited(arguments);
		},

		_isInvalidDate: function(/*Date*/ value){
			// summary:
			//		Runs various tests on the value, checking for invalid conditions
			// tags:
			//		private
			return !value || isNaN(value) || typeof value != "object" || value.toString() == this._invalidDate;
		},

		_setValueAttr: function(/*Date|String*/ value, /*Boolean?*/ priorityChange, /*String?*/ formattedValue){
			// summary:
			//		Sets the date on this textbox. Note: value can be a JavaScript Date literal or a string to be parsed.
			if(value !== undefined){
				if(typeof value == "string"){
					value = stamp.fromISOString(value);
				}
				if(this._isInvalidDate(value)){
					value = null;
				}
				if(value instanceof Date && !(this.dateClassObj instanceof Date)){
					value = new this.dateClassObj(value);
				}
			}
			this.inherited(arguments);
			if(this.value instanceof Date){
				this.filterString = "";
			}
			if(this.dropDown){
				this.dropDown.set('value', value, false);
			}
		},

		_set: function(attr, value){
			// Avoid spurious watch() notifications when value is changed to new Date object w/the same value
			if(attr == "value" && this.value instanceof Date && this.compare(value, this.value) == 0){
				return;
			}
			this.inherited(arguments);
		},

		_setDropDownDefaultValueAttr: function(/*Date*/ val){
			if(this._isInvalidDate(val)){
				// convert null setting into today's date, since there needs to be *some* default at all times.
				 val = new this.dateClassObj();
			}
			this.dropDownDefaultValue = val;
		},

		openDropDown: function(/*Function*/ callback){
			// rebuild drop down every time, so that constraints get copied (#6002)
			if(this.dropDown){
				this.dropDown.destroy();
			}
			var PopupProto = lang.isString(this.popupClass) ? lang.getObject(this.popupClass, false) : this.popupClass,
				textBox = this,
				value = this.get("value");
			this.dropDown = new PopupProto({
				onChange: function(value){
					// this will cause InlineEditBox and other handlers to do stuff so make sure it's last
					textBox.set('value', value, true);
				},
				id: this.id + "_popup",
				dir: textBox.dir,
				lang: textBox.lang,
				value: value,
				currentFocus: !this._isInvalidDate(value) ? value : this.dropDownDefaultValue,
				constraints: textBox.constraints,
				filterString: textBox.filterString, // for TimeTextBox, to filter times shown
				datePackage: textBox.params.datePackage,
				isDisabledDate: function(/*Date*/ date){
					// summary:
					//		disables dates outside of the min/max of the _DateTimeTextBox
					return !textBox.rangeCheck(date, textBox.constraints);
				}
			});

			this.inherited(arguments);
		},

		_getDisplayedValueAttr: function(){
			return this.textbox.value;
		},

		_setDisplayedValueAttr: function(/*String*/ value, /*Boolean?*/ priorityChange){
			this._setValueAttr(this.parse(value, this.constraints), priorityChange, value);
		}
	});


	/*=====
	 _DateTimeTextBox.__Constraints = declare([RangeBoundTextBox.__Constraints, locale.__FormatOptions], {
		 // summary:
		 //		Specifies both the rules on valid/invalid values (first/last date/time allowed),
		 //		and also formatting options for how the date/time is displayed.
		 // example:
		 //		To restrict to dates within 2004, displayed in a long format like "December 25, 2005":
		 //	|		{min:'2004-01-01',max:'2004-12-31', formatLength:'long'}
	 });
	 =====*/

	return _DateTimeTextBox;
});

},
'dijit/form/RangeBoundTextBox':function(){
define("dijit/form/RangeBoundTextBox", [
	"dojo/_base/declare", // declare
	"dojo/i18n", // i18n.getLocalization
	"./MappedTextBox"
], function(declare, i18n, MappedTextBox){

	// module:
	//		dijit/form/RangeBoundTextBox


	var RangeBoundTextBox = declare("dijit.form.RangeBoundTextBox", MappedTextBox, {
		// summary:
		//		Base class for textbox form widgets which defines a range of valid values.

		// rangeMessage: String
		//		The message to display if value is out-of-range
		rangeMessage: "",

		/*=====
		// constraints: RangeBoundTextBox.__Constraints
		constraints: {},
		======*/

		rangeCheck: function(/*Number*/ primitive, /*dijit/form/RangeBoundTextBox.__Constraints*/ constraints){
			// summary:
			//		Overridable function used to validate the range of the numeric input value.
			// tags:
			//		protected
			return	("min" in constraints? (this.compare(primitive,constraints.min) >= 0) : true) &&
				("max" in constraints? (this.compare(primitive,constraints.max) <= 0) : true); // Boolean
		},

		isInRange: function(/*Boolean*/ /*===== isFocused =====*/){
			// summary:
			//		Tests if the value is in the min/max range specified in constraints
			// tags:
			//		protected
			return this.rangeCheck(this.get('value'), this.constraints);
		},

		_isDefinitelyOutOfRange: function(){
			// summary:
			//		Returns true if the value is out of range and will remain
			//		out of range even if the user types more characters
			var val = this.get('value');
			if(val == null){ return false; } // not yet valid enough to compare to
			var outOfRange = false;
			if("min" in this.constraints){
				var min = this.constraints.min;
				outOfRange = this.compare(val, ((typeof min == "number") && min >= 0 && val != 0) ? 0 : min) < 0;
			}
			if(!outOfRange && ("max" in this.constraints)){
				var max = this.constraints.max;
				outOfRange = this.compare(val, ((typeof max != "number") || max > 0) ? max : 0) > 0;
			}
			return outOfRange;
		},

		_isValidSubset: function(){
			// summary:
			//		Overrides `dijit/form/ValidationTextBox._isValidSubset()`.
			//		Returns true if the input is syntactically valid, and either within
			//		range or could be made in range by more typing.
			return this.inherited(arguments) && !this._isDefinitelyOutOfRange();
		},

		isValid: function(/*Boolean*/ isFocused){
			// Overrides dijit/form/ValidationTextBox.isValid() to check that the value is also in range.
			return this.inherited(arguments) &&
				((this._isEmpty(this.textbox.value) && !this.required) || this.isInRange(isFocused)); // Boolean
		},

		getErrorMessage: function(/*Boolean*/ isFocused){
			// Overrides dijit/form/ValidationTextBox.getErrorMessage() to print "out of range" message if appropriate
			var v = this.get('value');
			if(v != null /* and !undefined */ && v !== '' && (typeof v != "number" || !isNaN(v)) && !this.isInRange(isFocused)){ // don't check isInRange w/o a real value
				return this.rangeMessage; // String
			}
			return this.inherited(arguments);
		},

		postMixInProperties: function(){
			this.inherited(arguments);
			if(!this.rangeMessage){
				this.messages = i18n.getLocalization("dijit.form", "validate", this.lang);
				this.rangeMessage = this.messages.rangeMessage;
			}
		},

		applyTextDir: function(/*===== element, text =====*/){
			// summary:
			//		The function overridden in the _BidiSupport module,
			//		originally used for setting element.dir according to this.textDir.
			//		In this case does nothing.
			// element: Object
			// text: String
			// tags:
			//		protected.
		}
	});
	/*=====
	RangeBoundTextBox.__Constraints = declare(null, {
		// min: Number
		//		Minimum signed value.  Default is -Infinity
		// max: Number
		//		Maximum signed value.  Default is +Infinity
	});
	=====*/
	return RangeBoundTextBox;
});

},
'dijit/form/MappedTextBox':function(){
define("dijit/form/MappedTextBox", [
	"dojo/_base/declare", // declare
	"dojo/dom-construct", // domConstruct.place
	"./ValidationTextBox"
], function(declare, domConstruct, ValidationTextBox){

	// module:
	//		dijit/form/MappedTextBox

	return declare("dijit.form.MappedTextBox", ValidationTextBox, {
		// summary:
		//		A dijit/form/ValidationTextBox subclass which provides a base class for widgets that have
		//		a visible formatted display value, and a serializable
		//		value in a hidden input field which is actually sent to the server.
		// description:
		//		The visible display may
		//		be locale-dependent and interactive.  The value sent to the server is stored in a hidden
		//		input field which uses the `name` attribute declared by the original widget.  That value sent
		//		to the server is defined by the dijit/form/MappedTextBox.serialize() method and is typically
		//		locale-neutral.
		// tags:
		//		protected

		postMixInProperties: function(){
			this.inherited(arguments);

			// we want the name attribute to go to the hidden <input>, not the displayed <input>,
			// so override _FormWidget.postMixInProperties() setting of nameAttrSetting
			this.nameAttrSetting = "";
		},

		// Override default behavior to assign name to focusNode
		_setNameAttr: null,

		serialize: function(val /*=====, options =====*/){
			// summary:
			//		Overridable function used to convert the get('value') result to a canonical
			//		(non-localized) string.  For example, will print dates in ISO format, and
			//		numbers the same way as they are represented in javascript.
			// val: anything
			// options: Object?
			// tags:
			//		protected extension
			return val.toString ? val.toString() : ""; // String
		},

		toString: function(){
			// summary:
			//		Returns widget as a printable string using the widget's value
			// tags:
			//		protected
			var val = this.filter(this.get('value')); // call filter in case value is nonstring and filter has been customized
			return val != null ? (typeof val == "string" ? val : this.serialize(val, this.constraints)) : ""; // String
		},

		validate: function(){
			// Overrides `dijit/form/TextBox.validate`
			this.valueNode.value = this.toString();
			return this.inherited(arguments);
		},

		buildRendering: function(){
			// Overrides `dijit/_TemplatedMixin/buildRendering`

			this.inherited(arguments);

			// Create a hidden <input> node with the serialized value used for submit
			// (as opposed to the displayed value).
			// Passing in name as markup rather than calling domConstruct.create() with an attrs argument
			// to make query(input[name=...]) work on IE. (see #8660)
			this.valueNode = domConstruct.place("<input type='hidden'" + (this.name ? ' name="' + this.name.replace(/"/g, "&quot;") + '"' : "") + "/>", this.textbox, "after");
		},

		reset: function(){
			// Overrides `dijit/form/ValidationTextBox.reset` to
			// reset the hidden textbox value to ''
			this.valueNode.value = '';
			this.inherited(arguments);
		}
	});
});

},
'url:dijit/form/templates/DropDownBox.html':"<div class=\"dijit dijitReset dijitInline dijitLeft\"\n\tid=\"widget_${id}\"\n\trole=\"combobox\"\n\taria-haspopup=\"true\"\n\tdata-dojo-attach-point=\"_popupStateNode\"\n\t><div class='dijitReset dijitRight dijitButtonNode dijitArrowButton dijitDownArrowButton dijitArrowButtonContainer'\n\t\tdata-dojo-attach-point=\"_buttonNode\" role=\"presentation\"\n\t\t><input class=\"dijitReset dijitInputField dijitArrowButtonInner\" value=\"&#9660; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"button presentation\" aria-hidden=\"true\"\n\t\t\t${_buttonInputDisabled}\n\t/></div\n\t><div class='dijitReset dijitValidationContainer'\n\t\t><input class=\"dijitReset dijitInputField dijitValidationIcon dijitValidationInner\" value=\"&#935; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t/></div\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\n\t\t><input class='dijitReset dijitInputInner' ${!nameAttrSetting} type=\"text\" autocomplete=\"off\"\n\t\t\tdata-dojo-attach-point=\"textbox,focusNode\" role=\"textbox\"\n\t/></div\n></div>\n",
'epi-find/widget/DashboardRanges':function(){
define([
        "dojo/date",
        "dojo/date/locale"
    ],
    function(
        // dojo
        date,
        locale
    ) {

    var Ranges = {};

    Ranges.last24hours = {
        name: "last24hours",
        interval: "hour",
        getStartDate: function(/*Date*/endDate) {
            // summary:
            //      Gets the start date based on end date
            // description:
            //      Minutes and seconds will be set to 0 to neutralise any DST effects
            // returns:
            //      Date
            var startDate = new Date(endDate.getTime());
            startDate.setDate(endDate.getDate() - 1);
            startDate.setUTCMinutes(0, 0);
            return startDate;
        },
        getIntervalEnd: function(/*Date*/intervalDate) {
            // summary:
            //      Gets the end of the current interval
            // description:
            //      This is most commonly used to retrieve a full interval span
            //      _including_ the specified date
            // returns:
            //      Date
            var endDate = date.add(intervalDate, Ranges.last24hours.interval, 1);
            endDate.setUTCMinutes(0, 0, 0);
            return endDate;
        },
        getRange: function(/*Object*/options) {
            // summary:
            //      Gets a range object based on passed options
            // description:
            //      The range object will include to, from and interval properties. If
            //      no object is passed, range will be calculated based on `to: new Date()`
            // returns:
            //      Object
            options = options || { to: new Date() };
            var to = Ranges.last24hours.getIntervalEnd(options.to);
            return {
                to: to,
                from: Ranges.last24hours.getStartDate(to),
                interval: Ranges.last24hours.interval
            };
        },
        labelFormat: function(/*Date*/date) {
            // summary:
            //      Returns a label formatted suitable for this range type
            // returns:
            //      string
            return locale.format(date, {
                selector: "date",
                datePattern: "HH" + (date.getHours() === 0 ? "\nd MMM" : "")
            });
        }
    };
    Object.freeze(Ranges.last24hours);

    Ranges.lastWeek = {
        name: "lastWeek",
        interval: "day",
        getStartDate: function(/*Date*/endDate) {
            // summary:
            //      Gets the start date based on end date
            // description:
            //      Minutes and seconds will be set to 0 to neutralise any DST effects
            // returns:
            //      Date
            var startDate = new Date(endDate.getTime());
            startDate.setDate(endDate.getDate() - 7);
            startDate.setUTCHours(0, 0, 0);
            return startDate;
        },
        getIntervalEnd: function(/*Date*/intervalDate) {
            // summary:
            //      Gets the end of the current interval
            // description:
            //      This is most commonly used to retrieve a full interval span
            //      _including_ the specified date
            // returns:
            //      Date
            var endDate = date.add(intervalDate, "day", 1);
            endDate.setUTCHours(0, 0, 0, 0);
            return endDate;
        },
        getRange: function(/*Object*/options) {
            // summary:
            //      Gets a range object based on passed options
            // description:
            //      The range object will include to, from and interval properties. If
            //      no object is passed, range will be calculated based on `to: new Date()`
            // returns:
            //      Object
            options = options || { to: new Date() };
            var to = Ranges.lastWeek.getIntervalEnd(options.to);
            return {
                to: to,
                from: Ranges.lastWeek.getStartDate(to),
                interval: Ranges.lastWeek.interval
            };
        },
        labelFormat: function(/*Date*/date) {
            // summary:
            //      Returns a label formatted suitable for this range type
            // returns:
            //      string
            return locale.format(date, {
                selector: "date",
                datePattern: "d" + (date.getDate() === 1 ? "\nMMMM" : "")
            });
        }
    };
    Object.freeze(Ranges.lastWeek);

    Ranges.lastMonth = {
        name: "lastMonth",
        interval: "day",
        getStartDate: function(/*Date*/endDate) {
            // summary:
            //      Gets the start date based on end date
            // description:
            //      Minutes and seconds will be set to 0 to neutralise any DST effects
            // returns:
            //      Date
            var startDate = new Date(endDate.getTime());
            startDate.setMonth(endDate.getMonth() - 1);
            startDate.setUTCHours(0, 0, 0);
            return startDate;
        },
        getIntervalEnd: function(/*Date*/intervalDate) {
            // summary:
            //      Gets the end of the current interval
            // description:
            //      This is most commonly used to retrieve a full interval span
            //      _including_ the specified date
            // returns:
            //      Date
            var endDate = date.add(intervalDate, "day", 1);
            endDate.setUTCHours(0, 0, 0, 0);
            return endDate;
        },
        getRange: function(/*Object*/options) {
            // summary:
            //      Gets a range object based on passed options
            // description:
            //      The range object will include to, from and interval properties. If
            //      no object is passed, range will be calculated based on `to: new Date()`
            // returns:
            //      Object
            options = options || { to: new Date() };
            var to = Ranges.lastMonth.getIntervalEnd(options.to);
            return {
                to: to,
                from: Ranges.lastMonth.getStartDate(to),
                interval: Ranges.lastMonth.interval
            };
        },
        labelFormat: function(/*Date*/date) {
            // summary:
            //      Returns a label formatted suitable for this range type
            // returns:
            //      string
            return locale.format(date, {
                selector: "date",
                datePattern: "d" + (date.getDate() === 1 ? "\nMMMM" : "")
            });
        }
    };
    Object.freeze(Ranges.lastMonth);

    Ranges.lastYear = {
        name: "lastYear",
        interval: "month",
        getStartDate: function(/*Date*/endDate) {
            // summary:
            //      Gets the start date based on end date
            // description:
            //      Minutes and seconds will be set to 0 to neutralise any DST effects
            // returns:
            //      Date
            var startDate = new Date(endDate.getTime());
            startDate.setYear(endDate.getFullYear() - 1);
            startDate.setUTCHours(0, 0, 0);
            return startDate;
        },
        getIntervalEnd: function(/*Date*/intervalDate) {
            // summary:
            //      Gets the end of the current interval
            // description:
            //      This is most commonly used to retrieve a full interval span
            //      _including_ the specified date
            // returns:
            //      Date
            var endDate = date.add(intervalDate, "day", 1);
            endDate.setUTCHours(0, 0, 0, 0);
            return endDate;
        },
        getRange: function(options) {
            // summary:
            //      Gets a range object based on passed options
            // description:
            //      The range object will include to, from and interval properties. If
            //      no object is passed, range will be calculated based on `to: new Date()`
            // returns:
            //      Object
            options = options || { to: new Date() };
            var to = Ranges.lastYear.getIntervalEnd(options.to);
            return {
                to: to,
                from: Ranges.lastYear.getStartDate(to),
                interval: Ranges.lastYear.interval
            };
        },
        labelFormat: function(/*Date*/date) {
            // summary:
            //      Returns a label formatted suitable for this range type
            // returns:
            //      string
            return locale.format(date, {
                selector: "date",
                datePattern: "MMM" + (date.getDate() === 1 && date.getMonth() === 0 ? "\nyyy" : "")
            });
        }
    };
    Object.freeze(Ranges.lastYear);

    /* TODO: Implement daterange span selection in Dashboard, post-beta */
    Ranges.dateRange = {
        name: "dateRange",
        interval: "day",
        startDate: null,
        endDate: null,
        getStartDate: function(/*Date*/from) {
            // summary:
            //      Gets the start date
            // description:
            //      Minutes and seconds will be set to 0 to neutralise any DST effects
            // returns:
            //      Date
            if (from) {
                var startDate = date.add(from, "day", 1);
                startDate.setUTCHours(0, 0, 0, 0);
                Ranges.dateRange.startDate = startDate;
            }
            if (!Ranges.dateRange.startDate) {
                var startDate = date.add(new Date(), "day", -1)
                startDate.setUTCHours(0, 0, 0, 0);
                Ranges.dateRange.startDate = startDate;
            }

            return Ranges.dateRange.startDate;
        },
        getIntervalEnd: function(/*Date*/intervalDate) {
            // summary:
            //      Gets the end of the current interval
            // description:
            //      This is most commonly used to retrieve a full interval span
            //      _including_ the specified date
            // returns:
            //      Date
            if (intervalDate) {
                var endDate = date.add(intervalDate, "day", 2);
                endDate.setUTCHours(0, 0, 0, 0);
                Ranges.dateRange.endDate = endDate;
            }
            if (!Ranges.dateRange.endDate) {
                var endDate = date.add(new Date(), "day", 1)
                endDate.setUTCHours(0, 0, 0, 0);
                Ranges.dateRange.endDate = endDate;
            }

            return Ranges.dateRange.endDate;
        },
        getRange: function(/*Object*/options) {
            // summary:
            //      Gets a range object based on passed options
            // description:
            //      The range object will include to, from and interval properties. If
            //      no object is passed, range will be calculated based on `to: new Date()`
            // returns:
            //      Object
            options = options || {};
            return {
                to: Ranges.dateRange.getIntervalEnd(options.to),
                from: Ranges.dateRange.getStartDate(options.from),
                interval: Ranges.dateRange.interval
            };
        },
        labelFormat: function(/*Date*/date) {
            // summary:
            //      Returns a label formatted suitable for this range type
            // returns:
            //      string
            return locale.format(date, {
                selector: "date",
                datePattern: "d" + (date.getDate() === 1 ? "\nMMMM" : "")
            });
        }
    };

    return Ranges;
});

},
'epi-find/widget/DashboardGrid':function(){
define([
        "dojo/_base/declare",
        "dojo/dom-construct",
        "dojo/dom-class",
        "dojo/dom-style",
        "dojo/on",
        "dojo/query",
        "dijit/Destroyable",
        "dgrid/Selection",
        "dgrid/selector",
        "dgrid/OnDemandGrid",
        "dgrid/extensions/ColumnHider",
        "dgrid/extensions/Pagination",
        "dojox/html/entities",
        "epi-saas-base/widgets/_GridNumberedColumnMixin",
        "dojo/i18n!./nls/Dashboard"
    ],
    function(
            // dojo
            declare,
            domConstruct,
            domClass,
            domStyle,
            on,
            query,
            // dijit
            Destroyable,
            // dgrid
            Selection,
            selector,
            OnDemandGrid,
            ColumnHider,
            Pagination,
            // dojox
            entities,
            _GridNumberedColumnMixin,
            // .
            i18n) {

        return    declare([Destroyable, OnDemandGrid, Selection, ColumnHider, Pagination, _GridNumberedColumnMixin], {

            numberedColumnId: "phrase",
            columns: {
                selector: selector({
                    className: "epi-grid-column--snug",
                    label: "",
                    selectorType: "checkbox",
                    sortable: false
                }),

                phrase: {
                    field: "query",
                    label: i18n.fields_query,
                    renderCell: function (object, value, node) {
                        var grid = this.grid,
                            anchor = domConstruct.create("a", {
                                innerHTML: entities.encode(value),
                                "class": "epi-visibleLink"
                            }, node);
                        grid.own(
                            on(anchor, "mousedown, click", function (e) {
                                e.preventDefault();
                                e.stopPropagation();
                                e.phrase = object;
                                on.emit(grid.contentNode, "phrase-click", e);
                            })
                        );
                    },
                    sortable: false
                },
                number: {
                    field: "count",
                    className: "epi-grid-column--centered",
                    label: i18n.fields_numberOfSearches,
                    renderCell: function (object, value, node) {
                        domClass.add(node, "epi-grid-column--positive");
                        node.innerHTML = value;
                    },
                    sortable: false
                },
                total: {
                    className: "epi-grid-column--centered",
                    label: i18n.fields_numberOfSearches,
                    get: function (object) {
                        if (object.extended) {
                            return object.extended.total;
                        }
                    },
                    renderCell: function (object, value, node) {
                        domClass.add(node, "epi-grid-column--positive");
                        node.innerHTML = value;
                    },
                    sortable: false,
                    hidden: true
                },
                percentage: {
                    className: "epi-grid-column--centered",
                    label: i18n.fields_clickRate,
                    get: function (object) {
                        if (object.extended) {
                            return 100 - Math.round(object.count / object.extended.total * 100) + "%"; // Returning how many percent of all searches that resulted in any clicks
                        }
                    },
                    renderCell: function (object, value, node) {
                        domClass.add(node, "epi-grid-column--negative");
                        node.innerHTML = value;
                    },
                    sortable: false,
                    hidden: true
                }
            },
            firstLastArrows: true,
            pageSizeOptions: [10, 15, 25, 50],

            selectionMode: "toggle",
            allowTextSelection: false,
            noDataMessage: i18n.noResults,

            startup: function () {
                this.inherited(arguments);
                domStyle.set(this.hiderToggleNode, "display", "none"); // We only want to show/hide columns programmatically
            }

        });
});
},
'dgrid/Selection':function(){
define("dgrid/Selection", ["dojo/_base/kernel", "dojo/_base/declare", "dojo/_base/Deferred", "dojo/on", "dojo/has", "dojo/aspect", "./List", "dojo/has!touch?./util/touch", "put-selector/put", "dojo/query", "dojo/_base/sniff"],
function(kernel, declare, Deferred, on, has, aspect, List, touchUtil, put){

has.add("dom-comparedocumentposition", function(global, doc, element){
	return !!element.compareDocumentPosition;
});

has.add("pointer", function(global){
	return "PointerEvent" in global ? "pointer" :
		"MSPointerEvent" in global ? "MSPointer" : false;
});

// Add feature test for user-select CSS property for optionally disabling
// text selection.
// (Can't use dom.setSelectable prior to 1.8.2 because of bad sniffs, see #15990)
has.add("css-user-select", function(global, doc, element){
	var style = element.style,
		prefixes = ["Khtml", "O", "ms", "Moz", "Webkit"],
		i = prefixes.length,
		name = "userSelect";

	// Iterate prefixes from most to least likely
	do{
		if(typeof style[name] !== "undefined"){
			// Supported; return property name
			return name;
		}
	}while(i-- && (name = prefixes[i] + "UserSelect"));

	// Not supported if we didn't return before now
	return false;
});

// Also add a feature test for the onselectstart event, which offers a more
// graceful fallback solution than node.unselectable.
has.add("dom-selectstart", typeof document.onselectstart !== "undefined");

var ctrlEquiv = has("mac") ? "metaKey" : "ctrlKey",
	hasUserSelect = has("css-user-select"),
	hasPointer = has("pointer"),
	hasMSPointer = hasPointer && hasPointer.slice(0, 2) === "MS",
	downType = hasPointer ? hasPointer + (hasMSPointer ? "Down" : "down") : "mousedown",
	upType = hasPointer ? hasPointer + (hasMSPointer ? "Up" : "up") : "mouseup";

function makeUnselectable(node, unselectable){
	// Utility function used in fallback path for recursively setting unselectable
	var value = node.unselectable = unselectable ? "on" : "",
		elements = node.getElementsByTagName("*"),
		i = elements.length;
	
	while(--i){
		if(elements[i].tagName === "INPUT" || elements[i].tagName === "TEXTAREA"){
			continue; // Don't prevent text selection in text input fields.
		}
		elements[i].unselectable = value;
	}
}

function setSelectable(grid, selectable){
	// Alternative version of dojo/dom.setSelectable based on feature detection.
	
	// For FF < 21, use -moz-none, which will respect -moz-user-select: text on
	// child elements (e.g. form inputs).  In FF 21, none behaves the same.
	// See https://developer.mozilla.org/en-US/docs/CSS/user-select
	var node = grid.bodyNode,
		value = selectable ? "text" : has("ff") < 21 ? "-moz-none" : "none";
	
	// In IE10+, -ms-user-select: none will block selection from starting within the
	// element, but will not block an existing selection from entering the element.
	// When using a modifier key, IE will select text inside of the element as well
	// as outside of the element, because it thinks the selection started outside.
	// Therefore, fall back to other means of blocking selection for IE10+.
	if(hasUserSelect && hasUserSelect !== "msUserSelect"){
		node.style[hasUserSelect] = value;
	}else if(has("dom-selectstart")){
		// For browsers that don't support user-select but support selectstart (IE<10),
		// we can hook up an event handler as necessary.  Since selectstart bubbles,
		// it will handle any child elements as well.
		// Note, however, that both this and the unselectable fallback below are
		// incapable of preventing text selection from outside the targeted node.
		if(!selectable && !grid._selectstartHandle){
			grid._selectstartHandle = on(node, "selectstart", function(evt){
				var tag = evt.target && evt.target.tagName;
				
				// Prevent selection except where a text input field is involved.
				if(tag !== "INPUT" && tag !== "TEXTAREA"){
					evt.preventDefault();
				}
			});
		}else if(selectable && grid._selectstartHandle){
			grid._selectstartHandle.remove();
			delete grid._selectstartHandle;
		}
	}else{
		// For browsers that don't support either user-select or selectstart (Opera),
		// we need to resort to setting the unselectable attribute on all nodes
		// involved.  Since this doesn't automatically apply to child nodes, we also
		// need to re-apply it whenever rows are rendered.
		makeUnselectable(node, !selectable);
		if(!selectable && !grid._unselectableHandle){
			grid._unselectableHandle = aspect.after(grid, "renderRow", function(row){
				makeUnselectable(row, true);
				return row;
			});
		}else if(selectable && grid._unselectableHandle){
			grid._unselectableHandle.remove();
			delete grid._unselectableHandle;
		}
	}
}

return declare(null, {
	// summary:
	//		Add selection capabilities to a grid. The grid will have a selection property and
	//		fire "dgrid-select" and "dgrid-deselect" events.
	
	// selectionDelegate: String
	//		Selector to delegate to as target of selection events.
	selectionDelegate: ".dgrid-row",
	
	// selectionEvents: String|Function
	//		Event (or comma-delimited events, or extension event) to listen on
	//		to trigger select logic.
	selectionEvents: downType + "," + upType + ",dgrid-cellfocusin",
	
	// selectionTouchEvents: String|Function
	//		Event (or comma-delimited events, or extension event) to listen on
	//		in addition to selectionEvents for touch devices.
	selectionTouchEvents: has("touch") ? touchUtil.tap : null,
	
	// deselectOnRefresh: Boolean
	//		If true, the selection object will be cleared when refresh is called.
	deselectOnRefresh: true,
	
	// allowSelectAll: Boolean
	//		If true, allow ctrl/cmd+A to select all rows.
	//		Also consulted by the selector plugin for showing select-all checkbox.
	allowSelectAll: false,
	
	// selection:
	//		An object where the property names correspond to 
	//		object ids and values are true or false depending on whether an item is selected
	selection: {},
	
	// selectionMode: String
	//		The selection mode to use, can be "none", "multiple", "single", or "extended".
	selectionMode: "extended",
	
	// allowTextSelection: Boolean
	//		Whether to still allow text within cells to be selected.  The default
	//		behavior is to allow text selection only when selectionMode is none;
	//		setting this property to either true or false will explicitly set the
	//		behavior regardless of selectionMode.
	allowTextSelection: undefined,
	
	// _selectionTargetType: String
	//		Indicates the property added to emitted events for selected targets;
	//		overridden in CellSelection
	_selectionTargetType: "rows",
	
	create: function(){
		this.selection = {};
		return this.inherited(arguments);
	},
	postCreate: function(){
		this.inherited(arguments);
		
		this._initSelectionEvents();
		
		// Force selectionMode setter to run
		var selectionMode = this.selectionMode;
		this.selectionMode = "";
		this._setSelectionMode(selectionMode);
	},
	
	destroy: function(){
		this.inherited(arguments);
		
		// Remove any extra handles added by Selection.
		if(this._selectstartHandle){ this._selectstartHandle.remove(); }
		if(this._unselectableHandle){ this._unselectableHandle.remove(); }
		if(this._removeDeselectSignals){ this._removeDeselectSignals(); }
	},
	
	_setSelectionMode: function(mode){
		// summary:
		//		Updates selectionMode, resetting necessary variables.
		if(mode == this.selectionMode){ return; } // prevent unnecessary spinning
		
		// Start selection fresh when switching mode.
		this.clearSelection();
		
		this.selectionMode = mode;
		
		// Compute name of selection handler for this mode once
		// (in the form of _fooSelectionHandler)
		this._selectionHandlerName = "_" + mode + "SelectionHandler";
		
		// Also re-run allowTextSelection setter in case it is in automatic mode.
		this._setAllowTextSelection(this.allowTextSelection);
	},
	setSelectionMode: function(mode){
		kernel.deprecated("setSelectionMode(...)", 'use set("selectionMode", ...) instead', "dgrid 0.4");
		this.set("selectionMode", mode);
	},
	
	_setAllowTextSelection: function(allow){
		if(typeof allow !== "undefined"){
			setSelectable(this, allow);
		}else{
			setSelectable(this, this.selectionMode === "none");
		}
		this.allowTextSelection = allow;
	},
	
	_handleSelect: function(event, target){
		// Don't run if selection mode doesn't have a handler (incl. "none"), target can't be selected,
		// or if coming from a dgrid-cellfocusin from a mousedown
		if(!this[this._selectionHandlerName] || !this.allowSelect(this.row(target)) ||
				(event.type === "dgrid-cellfocusin" && event.parentType === "mousedown") ||
				(event.type === upType && target != this._waitForMouseUp)){
			return;
		}
		this._waitForMouseUp = null;
		this._selectionTriggerEvent = event;
		
		// Don't call select handler for ctrl+navigation
		if(!event.keyCode || !event.ctrlKey || event.keyCode == 32){
			// If clicking a selected item, wait for mouseup so that drag n' drop
			// is possible without losing our selection
			if(!event.shiftKey && event.type === downType && this.isSelected(target)){
				this._waitForMouseUp = target;
			}else{
				this[this._selectionHandlerName](event, target);
			}
		}
		this._selectionTriggerEvent = null;
	},
	
	_singleSelectionHandler: function(event, target){
		// summary:
		//		Selection handler for "single" mode, where only one target may be
		//		selected at a time.
		
		var ctrlKey = event.keyCode ? event.ctrlKey : event[ctrlEquiv];
		if(this._lastSelected === target){
			// Allow ctrl to toggle selection, even within single select mode.
			this.select(target, null, !ctrlKey || !this.isSelected(target));
		}else{
			this.clearSelection();
			this.select(target);
			this._lastSelected = target;
		}
	},
	
	_multipleSelectionHandler: function(event, target){
		// summary:
		//		Selection handler for "multiple" mode, where shift can be held to
		//		select ranges, ctrl/cmd can be held to toggle, and clicks/keystrokes
		//		without modifier keys will add to the current selection.
		
		var lastRow = this._lastSelected,
			ctrlKey = event.keyCode ? event.ctrlKey : event[ctrlEquiv],
			value;
		
		if(!event.shiftKey){
			// Toggle if ctrl is held; otherwise select
			value = ctrlKey ? null : true;
			lastRow = null;
		}
		this.select(target, lastRow, value);

		if(!lastRow){
			// Update reference for potential subsequent shift+select
			// (current row was already selected above)
			this._lastSelected = target;
		}
	},
	
	_extendedSelectionHandler: function(event, target){
		// summary:
		//		Selection handler for "extended" mode, which is like multiple mode
		//		except that clicks/keystrokes without modifier keys will clear
		//		the previous selection.
		
		// Clear selection first for right-clicks outside selection and non-ctrl-clicks;
		// otherwise, extended mode logic is identical to multiple mode
		if(event.button === 2 ? !this.isSelected(target) :
				!(event.keyCode ? event.ctrlKey : event[ctrlEquiv])){
			this.clearSelection(null, true);
		}
		this._multipleSelectionHandler(event, target);
	},
	
	_toggleSelectionHandler: function(event, target){
		// summary:
		//		Selection handler for "toggle" mode which simply toggles the selection
		//		of the given target.  Primarily useful for touch input.
		
		this.select(target, null, null);
	},

	_initSelectionEvents: function(){
		// summary:
		//		Performs first-time hookup of event handlers containing logic
		//		required for selection to operate.
		
		var grid = this,
			contentNode = this.contentNode,
			selector = this.selectionDelegate;
		
		this._selectionEventQueues = {
			deselect: [],
			select: []
		};
		
		if(has("touch") && !has("pointer") && this.selectionTouchEvents){
			// Listen for taps, and also for mouse/keyboard, making sure not
			// to trigger both for the same interaction
			on(contentNode, touchUtil.selector(selector, this.selectionTouchEvents), function(evt){
				grid._handleSelect(evt, this);
				grid._ignoreMouseSelect = this;
			});
			on(contentNode, on.selector(selector, this.selectionEvents), function(event){
				if(grid._ignoreMouseSelect !== this){
					grid._handleSelect(event, this);
				}else if(event.type === upType){
					grid._ignoreMouseSelect = null;
				}
			});
		}else{
			// Listen for mouse/keyboard actions that should cause selections
			on(contentNode, on.selector(selector, this.selectionEvents), function(event){
				grid._handleSelect(event, this);
			});
		}
		
		// Also hook up spacebar (for ctrl+space)
		if(this.addKeyHandler){
			this.addKeyHandler(32, function(event){
				grid._handleSelect(event, event.target);
			});
		}
		
		// If allowSelectAll is true, bind ctrl/cmd+A to (de)select all rows,
		// unless the event was received from an editor component.
		// (Handler further checks against _allowSelectAll, which may be updated
		// if selectionMode is changed post-init.)
		if(this.allowSelectAll){
			this.on("keydown", function(event) {
				if(event[ctrlEquiv] && event.keyCode == 65 &&
						!/\bdgrid-input\b/.test(event.target.className)){
					event.preventDefault();
					grid[grid.allSelected ? "clearSelection" : "selectAll"]();
				}
			});
		}
		
		// Update aspects if there is a store change
		if(this._setStore){
			aspect.after(this, "_setStore", function(){
				grid._updateDeselectionAspect();
			});
		}
		this._updateDeselectionAspect();
	},
	
	_updateDeselectionAspect: function(){
		// summary:
		//		Hooks up logic to handle deselection of removed items.
		//		Aspects to an observable store's notify method if applicable,
		//		or to the list/grid's removeRow method otherwise.
		
		var self = this,
			store = this.store,
			beforeSignal,
			afterSignal;

		function ifSelected(object, idToUpdate, methodName){
			// Calls a method if the row corresponding to the object is selected.
			var id = idToUpdate || (object && object[self.idProperty || "id"]);
			if(id != null){
				var row = self.row(id),
					selection = row && self.selection[row.id];
				// Is the row currently in the selection list.
				if(selection){
					self[methodName](row, null, selection);
				}
			}
		}
		
		// Remove anything previously configured
		if(this._removeDeselectSignals){
			this._removeDeselectSignals();
		}

		// Is there currently an observable store?
		if(store && store.notify){
			beforeSignal = aspect.before(store, "notify", function(object, idToUpdate){
				if(!object){
					// Call deselect on the row if the object is being removed.  This allows the
					// deselect event to reference the row element while it still exists in the DOM.
					ifSelected(object, idToUpdate, "deselect");
				}
			});
			afterSignal = aspect.after(store, "notify", function(object, idToUpdate){
				// When List updates an item, the row element is removed and a new one inserted.
				// If at this point the object is still in grid.selection, then call select on the row so the
				// element's CSS is updated.  If the object was removed then the aspect-before has already deselected it.
				ifSelected(object, idToUpdate, "select");
			}, true);
			
			this._removeDeselectSignals = function(){
				beforeSignal.remove();
				afterSignal.remove();
			};
		}else{
			beforeSignal = aspect.before(this, "removeRow", function(rowElement, justCleanup){
				var row;
				if(!justCleanup){
					row = this.row(rowElement);
					// if it is a real row removal for a selected item, deselect it
					if(row && (row.id in this.selection)){
						this.deselect(row);
					}
				}
			});
			this._removeDeselectSignals = function(){
				beforeSignal.remove();
			};
		}
	},
	
	allowSelect: function(row){
		// summary:
		//		A method that can be overriden to determine whether or not a row (or 
		//		cell) can be selected. By default, all rows (or cells) are selectable.
		return true;
	},
	
	_fireSelectionEvent: function(type){
		// summary:
		//		Fires an event for the accumulated rows once a selection
		//		operation is finished (whether singular or for a range)
		
		var queue = this._selectionEventQueues[type],
			triggerEvent = this._selectionTriggerEvent,
			eventObject;
		
		eventObject = {
			bubbles: true,
			grid: this
		};
		if(triggerEvent){
			eventObject.parentType = triggerEvent.type;
		}
		eventObject[this._selectionTargetType] = queue;
		
		// Clear the queue so that the next round of (de)selections starts anew
		this._selectionEventQueues[type] = [];
		
		on.emit(this.contentNode, "dgrid-" + type, eventObject);
	},
	
	_fireSelectionEvents: function(){
		var queues = this._selectionEventQueues,
			type;
		
		for(type in queues){
			if(queues[type].length){
				this._fireSelectionEvent(type);
			}
		}
	},
	
	_select: function(row, toRow, value){
		// summary:
		//		Contains logic for determining whether to select targets, but
		//		does not emit events.  Called from select, deselect, selectAll,
		//		and clearSelection.
		
		var selection,
			previousValue,
			element,
			toElement,
			direction;
		
		if(typeof value === "undefined"){
			// default to true
			value = true;
		} 
		if(!row.element){
			row = this.row(row);
		}
		
		// Check whether we're allowed to select the given row before proceeding.
		// If a deselect operation is being performed, this check is skipped,
		// to avoid errors when changing column definitions, and since disabled
		// rows shouldn't ever be selected anyway.
		if(value === false || this.allowSelect(row)){
			selection = this.selection;
			previousValue = !!selection[row.id];
			if(value === null){
				// indicates a toggle
				value = !previousValue;
			}
			element = row.element;
			if(!value && !this.allSelected){
				delete this.selection[row.id];
			}else{
				selection[row.id] = value;
			}
			if(element){
				// add or remove classes as appropriate
				if(value){
					put(element, ".dgrid-selected" +
						(this.addUiClasses ? ".ui-state-active" : ""));
				}else{
					put(element, "!dgrid-selected!ui-state-active");
				}
			}
			if(value !== previousValue && element){
				// add to the queue of row events
				this._selectionEventQueues[(value ? "" : "de") + "select"].push(row);
			}
			
			if(toRow){
				if(!toRow.element){
					toRow = this.row(toRow);
				}
				
				if(!toRow){
					this._lastSelected = element;
					console.warn("The selection range has been reset because the " +
						"beginning of the selection is no longer in the DOM. " +
						"If you are using OnDemandList, you may wish to increase " +
						"farOffRemoval to avoid this, but note that keeping more nodes " +
						"in the DOM may impact performance.");
					return;
				}
				
				toElement = toRow.element;
				if(toElement){
					direction = this._determineSelectionDirection(element, toElement);
					if(!direction){
						// The original element was actually replaced
						toElement = document.getElementById(toElement.id);
						direction = this._determineSelectionDirection(element, toElement);
					}
					while(row.element != toElement && (row = this[direction](row))){
						this._select(row, null, value);
					}
				}
			}
		}
	},
	
	// Implement _determineSelectionDirection differently based on whether the
	// browser supports element.compareDocumentPosition; use sourceIndex for IE<9
	_determineSelectionDirection: has("dom-comparedocumentposition") ? function (from, to) {
		var result = to.compareDocumentPosition(from);
		if(result & 1){
			return false; // Out of document
		}
		return result === 2 ? "down" : "up";
	} : function(from, to) {
		if(to.sourceIndex < 1){
			return false; // Out of document
		}
		return to.sourceIndex > from.sourceIndex ? "down" : "up";
	},
	
	select: function(row, toRow, value){
		// summary:
		//		Selects or deselects the given row or range of rows.
		// row: Mixed
		//		Row object (or something that can resolve to one) to (de)select
		// toRow: Mixed
		//		If specified, the inclusive range between row and toRow will
		//		be (de)selected
		// value: Boolean|Null
		//		Whether to select (true/default), deselect (false), or toggle
		//		(null) the row
		
		this._select(row, toRow, value);
		this._fireSelectionEvents();
	},
	deselect: function(row, toRow){
		// summary:
		//		Deselects the given row or range of rows.
		// row: Mixed
		//		Row object (or something that can resolve to one) to deselect
		// toRow: Mixed
		//		If specified, the inclusive range between row and toRow will
		//		be deselected
		
		this.select(row, toRow, false);
	},
	
	clearSelection: function(exceptId, dontResetLastSelected){
		// summary:
		//		Deselects any currently-selected items.
		// exceptId: Mixed?
		//		If specified, the given id will not be deselected.
		
		this.allSelected = false;
		for(var id in this.selection){
			if(exceptId !== id){
				this._select(id, null, false);
			}
		}
		if(!dontResetLastSelected){
			this._lastSelected = null;
		}
		this._fireSelectionEvents();
	},
	selectAll: function(){
		this.allSelected = true;
		this.selection = {}; // we do this to clear out pages from previous sorts
		for(var i in this._rowIdToObject){
			var row = this.row(this._rowIdToObject[i]);
			this._select(row.id, null, true);
		}
		this._fireSelectionEvents();
	},
	
	isSelected: function(object){
		// summary:
		//		Returns true if the indicated row is selected.
		
		if(typeof object === "undefined" || object === null){
			return false;
		}
		if(!object.element){
			object = this.row(object);
		}
		
		// First check whether the given row is indicated in the selection hash;
		// failing that, check if allSelected is true (testing against the
		// allowSelect method if possible)
		return (object.id in this.selection) ? !!this.selection[object.id] :
			this.allSelected && (!object.data || this.allowSelect(object));
	},
	
	refresh: function(){
		if(this.deselectOnRefresh){
			this.clearSelection();
		}
		this._lastSelected = null;
		return this.inherited(arguments);
	},
	
	renderArray: function(){
		var grid = this,
			rows = this.inherited(arguments);
		
		Deferred.when(rows, function(rows){
			var selection = grid.selection,
				i, row, selected;
			for(i = 0; i < rows.length; i++){
				row = grid.row(rows[i]);
				selected = row.id in selection ? selection[row.id] : grid.allSelected;
				if(selected){
					grid._select(row, null, selected);
				}
			}
			grid._fireSelectionEvents();
		});
		return rows;
	}
});

});

},
'dgrid/selector':function(){
define("dgrid/selector", ["dojo/_base/kernel", "dojo/_base/array", "dojo/on", "dojo/aspect", "dojo/_base/sniff", "put-selector/put"],
function(kernel, arrayUtil, on, aspect, has, put){
	return function(column, type){
		
		var listeners = [],
			grid, headerCheckbox;
		
		if(!column){ column = {}; }
		
		if(column.type){
			column.selectorType = column.type;
			kernel.deprecated("columndef.type", "use columndef.selectorType instead", "dgrid 0.4");
		}
		// accept type as argument to Selector function, or from column def
		column.selectorType = type = type || column.selectorType || "checkbox";
		column.sortable = false;

		function disabled(item) {
			return !grid.allowSelect(grid.row(item));
		}
		
		function changeInput(value){
			// creates a function that modifies the input on an event
			return function(event){
				var rows = event.rows,
					len = rows.length,
					state = "false",
					selection, mixed, i;
				
				for(i = 0; i < len; i++){
					var element = grid.cell(rows[i], column.id).element;
					if(!element){ continue; } // skip if row has been entirely removed
					element = (element.contents || element).input;
					if(element && !element.disabled){
						// only change the value if it is not disabled
						element.checked = value;
						element.setAttribute("aria-checked", value);
					}
				}
				if(headerCheckbox.type == "checkbox"){
					selection = grid.selection;
					mixed = false;
					// see if the header checkbox needs to be indeterminate
					for(i in selection){
						// if there is anything in the selection, than it is indeterminate
						if(selection[i] != grid.allSelected){
							mixed = true;
							break;
						}
					}
					headerCheckbox.indeterminate = mixed;
					headerCheckbox.checked = grid.allSelected;
					if (mixed) {
						state = "mixed";
					} else if (grid.allSelected) {
						state = "true";
					}
					headerCheckbox.setAttribute("aria-checked", state);
				}
			};
		}
		
		function onSelect(event){
			// we would really only care about click, since other input sources, like spacebar
			// trigger a click, but the click event doesn't provide access to the shift key in firefox, so
			// listen for keydown's as well to get an event in firefox that we can properly retrieve
			// the shiftKey property from
			if(event.type == "click" || event.keyCode == 32 || (!has("opera") && event.keyCode == 13) || event.keyCode === 0){
				var row = grid.row(event);
				grid._selectionTriggerEvent = event;
				
				if(row){
					if(grid.allowSelect(row)){
						var lastRow = grid._lastSelected && grid.row(grid._lastSelected);
						
						if(type == "radio"){
							if(!lastRow || lastRow.id != row.id){
								grid.clearSelection();
								grid.select(row, null, true);
								grid._lastSelected = row.element;
							}
						}else{
							if(row){
								if(event.shiftKey){
									// make sure the last input always ends up checked for shift key
									changeInput(true)({rows: [row]});
								}else{
									// no shift key, so no range selection
									lastRow = null;
								}
								lastRow = event.shiftKey ? lastRow : null;
								grid.select(lastRow || row, row, lastRow ? undefined : null);
								grid._lastSelected = row.element;
							}
						}
					}
				}else{
					// No row resolved; must be the select-all checkbox.
					put(this, (grid.allSelected ? "!" : ".") + "dgrid-select-all");
					grid[grid.allSelected ? "clearSelection" : "selectAll"]();
				}
				grid._selectionTriggerEvent = null;
			}
		}
		
		function setupSelectionEvents(){
			// register one listener at the top level that receives events delegated
			grid._hasSelectorInputListener = true;
			listeners.push(grid.on(".dgrid-selector:click,.dgrid-selector:keydown", onSelect));
			var handleSelect = grid._handleSelect;
			grid._handleSelect = function(event){
				// ignore the default select handler for events that originate from the selector column
				if(this.cell(event).column != column){
					handleSelect.apply(this, arguments);
				}
			};
			
			// Set up disabled and grid.allowSelect to match each other's behaviors
			if(typeof column.disabled == "function"){
				var originalAllowSelect = grid.allowSelect,
					originalDisabled = column.disabled;

				// Wrap allowSelect to consult both the original allowSelect and disabled
				grid.allowSelect = function(row){
					var allow = originalAllowSelect.call(this, row);

					if (originalDisabled === disabled) {
						return allow;
					} else {
						return allow && !originalDisabled.call(column, row.data);
					}
				};

				// Then wrap disabled to simply call the new allowSelect
				column.disabled = disabled;
			}else{
				// If no disabled function was specified, institute a default one
				// which honors allowSelect
				column.disabled = disabled;
			}
			// register listeners to the select and deselect events to change the input checked value
			listeners.push(grid.on("dgrid-select", changeInput(true)));
			listeners.push(grid.on("dgrid-deselect", changeInput(false)));
		}
		
		var renderInput = typeof type == "function" ? type : function(value, cell, object){
			var parent = cell.parentNode,
				disabled;
			
			if(!grid._hasSelectorInputListener){
				setupSelectionEvents();
			}
			
			// column.disabled gets initialized or wrapped in setupSelectionEvents
			disabled = column.disabled;

			// must set the class name on the outer cell in IE for keystrokes to be intercepted
			put(parent && parent.contents ? parent : cell, ".dgrid-selector");
			var input = cell.input || (cell.input = put(cell, "input[type="+type + "]", {
				tabIndex: isNaN(column.tabIndex) ? -1 : column.tabIndex,
				disabled: disabled && (typeof disabled == "function" ?
					disabled.call(column, object) : disabled),
				checked: value
			}));
			input.setAttribute("aria-checked", !!value);
			
			return input;
		};
		
		aspect.after(column, "init", function(){
			grid = column.grid;
		});
		
		aspect.after(column, "destroy", function(){
			arrayUtil.forEach(listeners, function(l){ l.remove(); });
			grid._hasSelectorInputListener = false;
		});
		
		column.renderCell = function(object, value, cell, options, header){
			var row = object && grid.row(object);
			value = row && grid.selection[row.id];
			renderInput(value, cell, object);
		};
		column.renderHeaderCell = function(th){
			var label = "label" in column ? column.label :
				column.field || "";
			
			if(type == "radio" || !grid.allowSelectAll){
				th.appendChild(document.createTextNode(label));
				if(!grid._hasSelectorInputListener){
					setupSelectionEvents();
				}
			}else{
				renderInput(false, th, {});
			}
			headerCheckbox = th.lastChild;
		};
		
		return column;
	};
});

},
'dgrid/extensions/ColumnHider':function(){
define("dgrid/extensions/ColumnHider", ["dojo/_base/declare", "dojo/has", "dojo/on", "../util/misc", "put-selector/put", "dojo/i18n!./nls/columnHider", "xstyle/css!../css/extensions/ColumnHider.css"],
function(declare, has, listen, miscUtil, put, i18n){
/*
 *	Column Hider plugin for dgrid
 *	Originally contributed by TRT 2011-09-28
 *
 *	A dGrid plugin that attaches a menu to a dgrid, along with a way of opening it,
 *	that will allow you to show and hide columns.  A few caveats:
 *
 *	1. Menu placement is entirely based on CSS definitions.
 *	2. If you want columns initially hidden, you must add "hidden: true" to your
 *		column definition.
 *	3. This implementation does NOT support ColumnSet, and has not been tested
 *		with multi-subrow records.
 *	4. Column show/hide is controlled via straight up HTML checkboxes.  If you
 *		are looking for something more fancy, you'll probably need to use this
 *		definition as a template to write your own plugin.
 *
 */
	
	var activeGrid, // references grid for which the menu is currently open
		bodyListener, // references pausable event handler for body mousedown
		// Need to handle old IE specially for checkbox listener and for attribute.
		hasIE = has("ie"),
		hasIEQuirks = hasIE && has("quirks"),
		forAttr = hasIE < 8 || hasIEQuirks ? "htmlFor" : "for";
	
	function getColumnIdFromCheckbox(cb, grid){
		// Given one of the checkboxes from the hider menu,
		// return the id of the corresponding column.
		// (e.g. gridIDhere-hider-menu-check-colIDhere -> colIDhere)
		return cb.id.substr(grid.id.length + 18);
	}
	
	return declare(null, {
		// hiderMenuNode: DOMNode
		//		The node for the menu to show/hide columns.
		hiderMenuNode: null,
		
		// hiderToggleNode: DOMNode
		//		The node for the toggler to open the menu.
		hiderToggleNode: null,
		
		// i18nColumnHider: Object
		//		This object contains all of the internationalized strings for
		//		the ColumnHider extension as key/value pairs.
		i18nColumnHider: i18n,
		
		// _hiderMenuOpened: Boolean
		//		Records the current open/closed state of the menu.
		_hiderMenuOpened: false,
		
		// _columnHiderRules: Object
		//		Hash containing handles returned from addCssRule.
		_columnHiderRules: null,
		
		// _columnHiderCheckboxes: Object
		//		Hash containing checkboxes generated for menu items.
		_columnHiderCheckboxes: null,
		
		_renderHiderMenuEntries: function(){
			// summary:
			//		Iterates over subRows for the sake of adding items to the
			//		column hider menu.
			
			var subRows = this.subRows,
				first = true,
				srLength, cLength, sr, c;
			
			delete this._columnHiderFirstCheckbox;
			
			for(sr = 0, srLength = subRows.length; sr < srLength; sr++){
				for(c = 0, cLength = subRows[sr].length; c < cLength; c++){
					this._renderHiderMenuEntry(subRows[sr][c]);
					if(first){
						first = false;
						this._columnHiderFirstCheckbox =
							this._columnHiderCheckboxes[subRows[sr][c].id];
					}
				}
			}
		},
		
		_renderHiderMenuEntry: function(col){
			var id = col.id,
				replacedId = miscUtil.escapeCssIdentifier(id, "-"),
				div,
				checkId,
				checkbox,
				label;
			
			if(col.hidden){
				// Hide the column (reset first to avoid short-circuiting logic)
				col.hidden = false;
				this._hideColumn(id);
				col.hidden = true;
			}
			
			// Allow cols to opt out of the hider (e.g. for selector column).
			if(col.unhidable){ return; }
			
			// Create the checkbox and label for each column selector.
			div = put("div.dgrid-hider-menu-row");
			checkId = this.domNode.id + "-hider-menu-check-" + replacedId;
			
			// put-selector can't handle invalid selector characters, and the
			// ID could have some, so add it directly
			checkbox = this._columnHiderCheckboxes[id] =
				put(div, "input.dgrid-hider-menu-check.hider-menu-check-" + replacedId + "[type=checkbox]");
			checkbox.id = checkId;
			
			label = put(div, "label.dgrid-hider-menu-label.hider-menu-label-" + replacedId +
				"[" + forAttr + "=" + checkId + "]",
				col.label || col.field || "");
			
			put(this.hiderMenuNode, div);
			
			if(!col.hidden){
				// Hidden state is false; checkbox should be initially checked.
				// (Need to do this after adding to DOM to avoid IE6 clobbering it.)
				checkbox.checked = true;
			}
		},
		
		renderHeader: function(){
			var grid = this,
				hiderMenuNode = this.hiderMenuNode,
				hiderToggleNode = this.hiderToggleNode,
				id;
			
			function stopPropagation(event){
				event.stopPropagation();
			}
			
			this.inherited(arguments);
			
			if(!hiderMenuNode){ // first run
				// Assume that if this plugin is used, then columns are hidable.
				// Create the toggle node.
				hiderToggleNode = this.hiderToggleNode =
					put(this.domNode, "button.ui-icon.dgrid-hider-toggle[type=button][aria-label=" +
						this.i18nColumnHider.popupTriggerLabel + "]");
				
				this._listeners.push(listen(hiderToggleNode, "click", function(e){
					grid._toggleColumnHiderMenu(e);
				}));
	
				// Create the column list, with checkboxes.
				hiderMenuNode = this.hiderMenuNode =
					put("div.dgrid-hider-menu[role=dialog][aria-label=" +
						this.i18nColumnHider.popupLabel + "]");
				hiderMenuNode.id = this.id + "-hider-menu";

				this._listeners.push(listen(hiderMenuNode, "keyup", function (e) {
					var charOrCode = e.charCode || e.keyCode;
					if(charOrCode === /*ESCAPE*/ 27){
						grid._toggleColumnHiderMenu(e);
						hiderToggleNode.focus();
					}
				}));
				
				// Make sure our menu is initially hidden, then attach to the document.
				hiderMenuNode.style.display = "none";
				put(this.domNode, hiderMenuNode);
				
				// Hook up delegated listener for modifications to checkboxes.
				this._listeners.push(listen(hiderMenuNode,
						".dgrid-hider-menu-check:" + (hasIE < 9 || hasIEQuirks ? "click" : "change"),
					function(e){
						grid._updateColumnHiddenState(
							getColumnIdFromCheckbox(e.target, grid), !e.target.checked);
					}
				));
				
				// Stop click events from propagating from menu or trigger nodes,
				// so that we can simply track body clicks for hide without
				// having to drill-up to check.
				this._listeners.push(
					listen(hiderMenuNode, "mousedown", stopPropagation),
					listen(hiderToggleNode, "mousedown", stopPropagation)
				);
				
				// Hook up top-level mousedown listener if it hasn't been yet.
				if(!bodyListener){
					bodyListener = listen.pausable(document, "mousedown", function(e){
						// If an event reaches this listener, the menu is open,
						// but a click occurred outside, so close the dropdown.
						activeGrid && activeGrid._toggleColumnHiderMenu(e);
					});
					bodyListener.pause(); // pause initially; will resume when menu opens
				}
			}else{ // subsequent run
				// Remove active rules, and clear out the menu (to be repopulated).
				for(id in this._columnHiderRules){
					this._columnHiderRules[id].remove();
				}
				hiderMenuNode.innerHTML = "";
			}
			
			this._columnHiderCheckboxes = {};
			this._columnHiderRules = {};

			// Populate menu with checkboxes/labels based on current columns.
			this._renderHiderMenuEntries();
		},
		
		destroy: function(){
			this.inherited(arguments);
			// Remove any remaining rules applied to hidden columns.
			for(var id in this._columnHiderRules){
				this._columnHiderRules[id].remove();
			}
		},
		
		left: function(cell, steps){
			return this.right(cell, -steps);
		},
		
		right: function(cell, steps){
			if(!cell.element){
				cell = this.cell(cell);
			}
			var nextCell = this.inherited(arguments),
				prevCell = cell;
			
			// Skip over hidden cells
			while(nextCell.column.hidden){
				nextCell = this.inherited(arguments, [nextCell, steps > 0 ? 1 : -1]);
				if(prevCell.element === nextCell.element){
					// No further visible cell found - return original
					return cell;
				}
				prevCell = nextCell;
			}
			return nextCell;
		},
		
		isColumnHidden: function(id){
			// summary:
			//		Convenience method to determine current hidden state of a column
			return !!this._columnHiderRules[id];
		},
		
		_toggleColumnHiderMenu: function(){
			var hidden = this._hiderMenuOpened, // reflects hidden state after toggle
				hiderMenuNode = this.hiderMenuNode,
				domNode = this.domNode,
				firstCheckbox;

			// Show or hide the hider menu
			hiderMenuNode.style.display = (hidden ? "none" : "");

			// Adjust height of menu
			if (hidden) {
				// Clear the set size
				hiderMenuNode.style.height = "";
			} else {
				// Adjust height of the menu if necessary
				// Why 12? Based on menu default paddings and border, we need
				// to adjust to be 12 pixels shorter. Given the infrequency of
				// this style changing, we're assuming it will remain this
				// static value of 12 for now, to avoid pulling in any sort of
				// computed styles.
				if (hiderMenuNode.offsetHeight > domNode.offsetHeight - 12) {
					hiderMenuNode.style.height = (domNode.offsetHeight - 12) + "px";
				}
				// focus on the first checkbox
				(firstCheckbox = this._columnHiderFirstCheckbox) && firstCheckbox.focus();
			}

			// Pause or resume the listener for clicks outside the menu
			bodyListener[hidden ? "pause" : "resume"]();

			// Update activeGrid appropriately
			activeGrid = hidden ? null : this;

			// Toggle the instance property
			this._hiderMenuOpened = !hidden;
		},
		
		_hideColumn: function(id){
			// summary:
			//		Hides the column indicated by the given id.
			
			// Use miscUtil function directly, since we clean these up ourselves anyway
			var grid = this,
				selectorPrefix = "#" + miscUtil.escapeCssIdentifier(this.domNode.id) + " .dgrid-column-",
				tableRule; // used in IE8 code path

			if (this._columnHiderRules[id]) {
				return;
			}

			this._columnHiderRules[id] =
				miscUtil.addCssRule(selectorPrefix + miscUtil.escapeCssIdentifier(id, "-"),
					"display: none;");

			if((has("ie") === 8 || has("ie") === 10) && !has("quirks")){
				tableRule = miscUtil.addCssRule(".dgrid-row-table", "display: inline-table;");

				window.setTimeout(function(){
					tableRule.remove();
					grid.resize();
				}, 0);
			}
		},
		
		_showColumn: function(id){
			// summary:
			//		Shows the column indicated by the given id
			//		(by removing the rule responsible for hiding it).
			
			if(this._columnHiderRules[id]){
				this._columnHiderRules[id].remove();
				delete this._columnHiderRules[id];
			}
		},
		
		_updateColumnHiddenState: function(id, hidden){
			// summary:
			//		Performs internal work for toggleColumnHiddenState; see the public
			//		method for more information.
			
			this[hidden ? '_hideColumn' : '_showColumn'](id);
			
			// Update hidden state in actual column definition,
			// in case columns are re-rendered.
			this.columns[id].hidden = hidden;
			
			// Emit event to notify of column state change.
			listen.emit(this.domNode, "dgrid-columnstatechange", {
				grid: this,
				column: this.columns[id],
				hidden: hidden,
				bubbles: true
			});

			// Adjust the size of the header.
			this.resize();
		},
		
		toggleColumnHiddenState: function(id, hidden){
			// summary:
			//		Shows or hides the column with the given id.
			// id: String
			//		ID of column to show/hide.
			// hide: Boolean?
			//		If specified, explicitly sets the hidden state of the specified
			//		column.  If unspecified, toggles the column from the current state.
			
			if(typeof hidden === "undefined"){ hidden = !this._columnHiderRules[id]; }
			this._updateColumnHiddenState(id, hidden);
			
			// Since this can be called directly, re-sync the appropriate checkbox.
			this._columnHiderCheckboxes[id].checked = !hidden;
		}
	});
});

},
'dgrid/extensions/Pagination':function(){
define("dgrid/extensions/Pagination", ["../_StoreMixin", "dojo/_base/declare", "dojo/_base/array", "dojo/_base/lang", "dojo/_base/Deferred",
	"dojo/on", "dojo/query", "dojo/string", "dojo/has", "put-selector/put", "../util/misc", "dojo/i18n!./nls/pagination",
	"dojo/_base/sniff", "xstyle/css!../css/extensions/Pagination.css"],
function(_StoreMixin, declare, arrayUtil, lang, Deferred, on, query, string, has, put, miscUtil, i18n){
	function cleanupContent(grid){
		// Remove any currently-rendered rows, or noDataMessage
		if(grid.noDataNode){
			put(grid.noDataNode, "!");
			delete grid.noDataNode;
		}else{
			grid.cleanup();
		}
		grid.contentNode.innerHTML = "";
	}
	function cleanupLoading(grid){
		if(grid.loadingNode){
			put(grid.loadingNode, "!");
			delete grid.loadingNode;
		}else if(grid._oldPageNodes){
			// If cleaning up after a load w/ showLoadingMessage: false,
			// be careful to only clean up rows from the old page, not the new one
			for(var id in grid._oldPageNodes){
				grid.removeRow(grid._oldPageNodes[id]);
			}
			delete grid._oldPageNodes;
			// Also remove the observer from the previous page, if there is one
			if(grid._oldPageObserver){
				grid._oldPageObserver.cancel();
				grid._numObservers--;
				delete grid._oldPageObserver;
			}
		}
		delete grid._isLoading;
	}
	
	return declare(_StoreMixin, {
		// summary:
		//		An extension for adding discrete pagination to a List or Grid.
		
		// rowsPerPage: Number
		//		Number of rows (items) to show on a given page.
		rowsPerPage: 10,
		
		// pagingTextBox: Boolean
		//		Indicates whether or not to show a textbox for paging.
		pagingTextBox: false,
		// previousNextArrows: Boolean
		//		Indicates whether or not to show the previous and next arrow links.
		previousNextArrows: true,
		// firstLastArrows: Boolean
		//		Indicates whether or not to show the first and last arrow links.
		firstLastArrows: false,
		
		// pagingLinks: Number
		//		The number of page links to show on each side of the current page
		//		Set to 0 (or false) to disable page links.
		pagingLinks: 2,
		// pageSizeOptions: Array[Number]
		//		This provides options for different page sizes in a drop-down.
		//		If it is empty (default), no page size drop-down will be displayed.
		pageSizeOptions: null,
		
		// showLoadingMessage: Boolean
		//		If true, clears previous data and displays loading node when requesting
		//		another page; if false, leaves previous data in place until new data
		//		arrives, then replaces it immediately.
		showLoadingMessage: true,
		
		// i18nPagination: Object
		//		This object contains all of the internationalized strings as
		//		key/value pairs.
		i18nPagination: i18n,
		
		showFooter: true,
		_currentPage: 1,
		_total: 0,
		
		buildRendering: function(){
			this.inherited(arguments);
			
			// add pagination to footer
			var grid = this,
				paginationNode = this.paginationNode =
					put(this.footerNode, "div.dgrid-pagination"),
				statusNode = this.paginationStatusNode =
					put(paginationNode, "div.dgrid-status"),
				i18n = this.i18nPagination,
				navigationNode,
				node,
				i;
			
			statusNode.tabIndex = 0;
			
			// Initialize UI based on pageSizeOptions and rowsPerPage
			this._updatePaginationSizeSelect();
			this._updateRowsPerPageOption();
			
			// initialize some content into paginationStatusNode, to ensure
			// accurate results on initial resize call
			statusNode.innerHTML = string.substitute(i18n.status,
				{ start: 1, end: 1, total: 0 });
			
			navigationNode = this.paginationNavigationNode =
				put(paginationNode, "div.dgrid-navigation");
			
			if(this.firstLastArrows){
				// create a first-page link
				node = this.paginationFirstNode =
					put(navigationNode,  "span.dgrid-first.dgrid-page-link", "«");
				node.setAttribute("aria-label", i18n.gotoFirst);
				node.tabIndex = 0;
			}
			if(this.previousNextArrows){
				// create a previous link
				node = this.paginationPreviousNode =
					put(navigationNode,  "span.dgrid-previous.dgrid-page-link", "‹");
				node.setAttribute("aria-label", i18n.gotoPrev);
				node.tabIndex = 0;
			}
			
			this.paginationLinksNode = put(navigationNode, "span.dgrid-pagination-links");
			if(this.previousNextArrows){
				// create a next link
				node = this.paginationNextNode =
					put(navigationNode, "span.dgrid-next.dgrid-page-link", "›");
				node.setAttribute("aria-label", i18n.gotoNext);
				node.tabIndex = 0;
			}
			if(this.firstLastArrows){
				// create a last-page link
				node = this.paginationLastNode =
					put(navigationNode,  "span.dgrid-last.dgrid-page-link", "»");
				node.setAttribute("aria-label", i18n.gotoLast);
				node.tabIndex = 0;
			}
			
			this._listeners.push(on(navigationNode, ".dgrid-page-link:click,.dgrid-page-link:keydown", function(event){
				// For keyboard events, only respond to enter
				if(event.type === "keydown" && event.keyCode !== 13){
					return;
				}
				
				var cls = this.className,
					curr, max;
				
				if(grid._isLoading || cls.indexOf("dgrid-page-disabled") > -1){
					return;
				}
				
				curr = grid._currentPage;
				max = Math.ceil(grid._total / grid.rowsPerPage);
				
				// determine navigation target based on clicked link's class
				if(this === grid.paginationPreviousNode){
					grid.gotoPage(curr - 1);
				}else if(this === grid.paginationNextNode){
					grid.gotoPage(curr + 1);
				}else if(this === grid.paginationFirstNode){
					grid.gotoPage(1);
				}else if(this === grid.paginationLastNode){
					grid.gotoPage(max);
				}else if(cls === "dgrid-page-link"){
					grid.gotoPage(+this.innerHTML); // the innerHTML has the page number
				}
			}));
		},
		
		destroy: function(){
			this.inherited(arguments);
			if(this._pagingTextBoxHandle){
				this._pagingTextBoxHandle.remove();
			}
		},

		_updatePaginationSizeSelect: function(){
			// summary:
			//		Creates or repopulates the pagination size selector based on
			//		the values in pageSizeOptions. Called from buildRendering
			//		and _setPageSizeOptions.
			
			var pageSizeOptions = this.pageSizeOptions,
				paginationSizeSelect = this.paginationSizeSelect,
				handle;
			
			if(pageSizeOptions && pageSizeOptions.length){
				if(!paginationSizeSelect){
					// First time setting page options; create the select
					paginationSizeSelect = this.paginationSizeSelect =
						put(this.paginationNode, "select.dgrid-page-size[aria-label=" +
							this.i18nPagination.rowsPerPage + "]");
					
					handle = this._paginationSizeChangeHandle =
						on(paginationSizeSelect, "change", lang.hitch(this, function(){
							this.set("rowsPerPage", +this.paginationSizeSelect.value);
						}));
					this._listeners.push(handle);
				}
				
				// Repopulate options
				paginationSizeSelect.options.length = 0;
				for(var i = 0; i < pageSizeOptions.length; i++){
					put(paginationSizeSelect, "option", pageSizeOptions[i], {
						value: pageSizeOptions[i],
						selected: this.rowsPerPage === pageSizeOptions[i]
					});
				}
				// Ensure current rowsPerPage value is in options
				this._updateRowsPerPageOption();
			}else if(!(pageSizeOptions && pageSizeOptions.length) && paginationSizeSelect){
				// pageSizeOptions was removed; remove/unhook the drop-down
				put(paginationSizeSelect, "!");
				this.paginationSizeSelect = null;
				this._paginationSizeChangeHandle.remove();
			}
		},

		_setPageSizeOptions: function(pageSizeOptions){
			this.pageSizeOptions = pageSizeOptions && pageSizeOptions.sort(function(a, b){
				return a - b;
			});
			this._updatePaginationSizeSelect();
		},

		_updateRowsPerPageOption: function(){
			// summary:
			//		Ensures that an option for rowsPerPage's value exists in the
			//		paginationSizeSelect drop-down (if one is rendered).
			//		Called from buildRendering and _setRowsPerPage.
			
			var rowsPerPage = this.rowsPerPage,
				pageSizeOptions = this.pageSizeOptions,
				paginationSizeSelect = this.paginationSizeSelect;
			
			if(paginationSizeSelect){
				if(arrayUtil.indexOf(pageSizeOptions, rowsPerPage) < 0){
					this._setPageSizeOptions(pageSizeOptions.concat([rowsPerPage])); 
				}else{
					paginationSizeSelect.value = "" + rowsPerPage;
				}
			}
		},
		
		_setRowsPerPage: function(rowsPerPage){
			this.rowsPerPage = rowsPerPage;
			this._updateRowsPerPageOption();
			this.gotoPage(1);
		},

		_updateNavigation: function(){
			// summary:
			//		Update status and navigation controls based on total count from query
			
			var grid = this,
				i18n = this.i18nPagination,
				linksNode = this.paginationLinksNode,
				currentPage = this._currentPage,
				pagingLinks = this.pagingLinks,
				paginationNavigationNode = this.paginationNavigationNode,
				end = Math.ceil(this._total / this.rowsPerPage),
				pagingTextBoxHandle = this._pagingTextBoxHandle,
				focused = document.activeElement,
				focusedPage,
				lastFocusablePageLink,
				focusableNodes;
			
			function pageLink(page, addSpace){
				var link;
				var disabled;
				if(grid.pagingTextBox && page == currentPage && end > 1){
					// use a paging text box if enabled instead of just a number
					link = put(linksNode, 'input.dgrid-page-input[type=text][value=$]', currentPage);
					link.setAttribute("aria-label", i18n.jumpPage);
					grid._pagingTextBoxHandle = on(link, "change", function(){
						var value = +this.value;
						if(!isNaN(value) && value > 0 && value <= end){
							grid.gotoPage(+this.value);
						}
					});
					if(focused && focused.tagName === "INPUT"){
						link.focus();
					}
				}else{
					// normal link
					disabled = page === currentPage;
					link = put(linksNode,
						'span' + (disabled ? '.dgrid-page-disabled' : '') + '.dgrid-page-link',
						page + (addSpace ? " " : ""));
					link.setAttribute("aria-label", i18n.gotoPage);
					link.tabIndex = disabled ? -1 : 0;
					
					// Try to restore focus if applicable;
					// if we need to but can't, try on the previous or next page,
					// depending on whether we're at the end
					if(focusedPage === page){
						if(!disabled){
							link.focus();
						}else if(page < end){
							focusedPage++;
						}else{
							lastFocusablePageLink.focus();
						}
					}
					
					if(!disabled){
						lastFocusablePageLink = link;
					}
				}
			}
			
			function setDisabled(link, disabled){
				put(link, (disabled ? "." : "!") + "dgrid-page-disabled");
				link.tabIndex = disabled ? -1 : 0;
			}
			
			if(!focused || !miscUtil.contains(this.paginationNavigationNode, focused)){
				focused = null;
			}else if(focused.className === "dgrid-page-link"){
				focusedPage = +focused.innerHTML;
			}
			
			if(pagingTextBoxHandle){ pagingTextBoxHandle.remove(); }
			linksNode.innerHTML = "";
			query(".dgrid-first, .dgrid-previous", paginationNavigationNode).forEach(function(link){
				setDisabled(link, currentPage === 1);
			});
			query(".dgrid-last, .dgrid-next", paginationNavigationNode).forEach(function(link){
				setDisabled(link, currentPage >= end);
			});
			
			if(pagingLinks && end > 0){
				// always include the first page (back to the beginning)
				pageLink(1, true);
				var start = currentPage - pagingLinks;
				if(start > 2) {
					// visual indication of skipped page links
					put(linksNode, "span.dgrid-page-skip", "...");
				}else{
					start = 2;
				}
				// now iterate through all the page links we should show
				for(var i = start; i < Math.min(currentPage + pagingLinks + 1, end); i++){
					pageLink(i, true);
				}
				if(currentPage + pagingLinks + 1 < end){
					put(linksNode, "span.dgrid-page-skip", "...");
				}
				// last link
				if(end > 1){
					pageLink(end);
				}
			}else if(grid.pagingTextBox){
				// The pageLink function is also used to create the paging textbox.
				pageLink(currentPage);
			}
			
			if (focused && focused.tabIndex === -1) {
				// One of the first/last or prev/next links was focused but
				// is now disabled, so find something focusable
				focusableNodes = query("[tabindex='0']", this.paginationNavigationNode);
				if(focused === this.paginationPreviousNode || focused === this.paginationFirstNode){
					focused = focusableNodes[0];
				}else if(focusableNodes.length){
					focused = focusableNodes[focusableNodes.length - 1];
				}
				if(focused){
					focused.focus();
				}
			}
		},
		
		refresh: function(){
			var self = this;
			
			this.inherited(arguments);
			
			if(!this.store){
				console.warn("Pagination requires a store to operate.");
				return;
			}
			
			// Reset to first page and return promise from gotoPage
			return this.gotoPage(1).then(function(results){
				// Emit on a separate turn to enable event to be used consistently for
				// initial render, regardless of whether the backing store is async
				setTimeout(function() {
					on.emit(self.domNode, "dgrid-refresh-complete", {
						bubbles: true,
						cancelable: false,
						grid: self,
						results: results // QueryResults object (may be a wrapped promise)
					});
				}, 0);
				
				return results;
			});
		},
		
		_onNotification: function(rows){
			if(rows.length !== this._rowsOnPage){
				// Refresh the current page to maintain correct number of rows on page
				this.gotoPage(this._currentPage);
			}
		},
		
		renderArray: function(results, beforeNode){
			var grid = this,
				rows = this.inherited(arguments);
			
			// Make sure _lastCollection is cleared (due to logic in List)
			this._lastCollection = null;
			
			if(!beforeNode){
				if(this._topLevelRequest && this._topLevelRequest !== results){
					// Cancel previous async request that didn't finish
					this._topLevelRequest.cancel();
					delete this._topLevelRequest;
				}
				
				if (typeof results.cancel === "function") {
					// Store reference to new async request in progress
					this._topLevelRequest = results;
				}
				
				Deferred.when(results, function(){
					if(grid._topLevelRequest){
						// Remove reference to request now that it's finished
						delete grid._topLevelRequest;
					}
				});
			}
			
			return rows;
		},
		
		insertRow: function(){
			var oldNodes = this._oldPageNodes,
				row = this.inherited(arguments);
			
			if(oldNodes && row === oldNodes[row.id]){
				// If the previous row was reused, avoid removing it in cleanup
				delete oldNodes[row.id];
			}
			
			return row;
		},
		
		gotoPage: function(page){
			// summary:
			//		Loads the given page.  Note that page numbers start at 1.
			var grid = this,
				dfd = new Deferred();
			
			var result = this._trackError(function(){
				var count = grid.rowsPerPage,
					start = (page - 1) * count,
					options = lang.mixin(grid.get("queryOptions"), {
						start: start,
						count: count
						// current sort is also included by get("queryOptions")
					}),
					results,
					contentNode = grid.contentNode,
					loadingNode,
					oldNodes,
					children,
					i,
					len;
				
				if(grid.showLoadingMessage){
					cleanupContent(grid);
					loadingNode = grid.loadingNode = put(contentNode, "div.dgrid-loading");
					loadingNode.innerHTML = grid.loadingMessage;
				}else{
					// Reference nodes to be cleared later, rather than now;
					// iterate manually since IE < 9 doesn't like slicing HTMLCollections
					grid._oldPageNodes = oldNodes = {};
					children = contentNode.children;
					for(i = 0, len = children.length; i < len; i++){
						oldNodes[children[i].id] = children[i];
					}
					// Also reference the current page's observer (if any)
					grid._oldPageObserver = grid.observers.pop();
				}
				
				// set flag to deactivate pagination event handlers until loaded
				grid._isLoading = true;
				
				// Run new query and pass it into renderArray
				results = grid.store.query(grid.query, options);
				
				Deferred.when(grid.renderArray(results, null, options), function(rows){
					cleanupLoading(grid);
					// Reset scroll Y-position now that new page is loaded.
					grid.scrollTo({ y: 0 });
					
					Deferred.when(results.total, function(total){
						if(!total){
							if(grid.noDataNode){
								put(grid.noDataNode, "!");
								delete grid.noDataNode;
							}
							// If there are no results, display the no data message.
							grid.noDataNode = put(grid.contentNode, "div.dgrid-no-data");
							grid.noDataNode.innerHTML = grid.noDataMessage;
						}
						
						// Update status text based on now-current page and total.
						grid.paginationStatusNode.innerHTML = string.substitute(grid.i18nPagination.status, {
							start: Math.min(start + 1, total),
							end: Math.min(total, start + count),
							total: total
						});
						grid._total = total;
						grid._currentPage = page;
						grid._rowsOnPage = rows.length;
						
						// It's especially important that _updateNavigation is called only
						// after renderArray is resolved as well (to prevent jumping).
						grid._updateNavigation();
					});
					
					if (has("ie") < 7 || (has("ie") && has("quirks"))) {
						// call resize in old IE in case grid is set to height: auto
						grid.resize();
					}
					
					dfd.resolve(results);
				}, function(error){
					cleanupLoading(grid);
					dfd.reject(error);
				});
				
				return dfd.promise;
			});
			
			if (!result) {
				// A synchronous error occurred; reject the promise.
				dfd.reject();
			}
			return dfd.promise;
		}
	});
});

},
'dojox/charting/plot2d/StackedAreas':function(){
define("dojox/charting/plot2d/StackedAreas", ["dojo/_base/declare", "./Stacked"], function(declare, Stacked){

	return declare("dojox.charting.plot2d.StackedAreas", Stacked, {
		// summary:
		//		A convenience object to set up a stacked area plot.
		constructor: function(){
			// summary:
			//		Force our Stacked plotter to include both lines and areas.
			this.opt.lines = true;
			this.opt.areas = true;
		}
	});
});


},
'dojox/charting/plot2d/Stacked':function(){
define("dojox/charting/plot2d/Stacked", ["dojo/_base/declare", "./Default", "./commonStacked"], 
	function(declare, Default, commonStacked){

	return declare("dojox.charting.plot2d.Stacked", Default, {
		// summary:
		//		Like the default plot, Stacked sets up lines, areas and markers
		//		in a stacked fashion (values on the y axis added to each other)
		//		as opposed to a direct one.
		getSeriesStats: function(){
			// summary:
			//		Calculate the min/max on all attached series in both directions.
			// returns: Object
			//		{hmin, hmax, vmin, vmax} min/max in both directions.
			var stats = commonStacked.collectStats(this.series);
			return stats; // Object
		},
		
		buildSegments: function(i, indexed){
			var run = this.series[i],
				min = indexed?Math.max(0, Math.floor(this._hScaler.bounds.from - 1)):0,
				max = indexed?Math.min(run.data.length-1, Math.ceil(this._hScaler.bounds.to)):run.data.length-1,
				rseg = null, segments = [];
			// split the run data into dense segments (each containing no nulls)
			// except if interpolates is false in which case ignore null between valid data
			for(var j = min; j <= max; j++){
				var value = indexed ? commonStacked.getIndexValue(this.series, i, j) : commonStacked.getValue(this.series, i, run.data[j] ?run.data[j].x: null);
				if(value != null && (indexed || value.y != null)){
					if(!rseg){
						rseg = [];
						segments.push({index: j, rseg: rseg});
					}
					rseg.push(value);
				}else{
					if(!this.opt.interpolate || indexed){
						// we break the line only if not interpolating or if we have indexed data
						rseg = null;
					}
				}
			}
			return segments;
		}
		
	});
});

},
'dojox/charting/plot2d/Default':function(){
define("dojox/charting/plot2d/Default", ["dojo/_base/lang", "dojo/_base/declare", "dojo/_base/array", 
		"./CartesianBase", "./_PlotEvents", "./common", "dojox/lang/functional", "dojox/lang/functional/reversed", "dojox/lang/utils", "dojox/gfx/fx"],
	function(lang, declare, arr, CartesianBase, _PlotEvents, dc, df, dfr, du, fx){

	/*=====
	declare("dojox.charting.plot2d.__DefaultCtorArgs", dojox.charting.plot2d.__PlotCtorArgs, {
		// summary:
		//		The arguments used for any/most plots.
	
		// hAxis: String?
		//		The horizontal axis name.
		hAxis: "x",
	
		// vAxis: String?
		//		The vertical axis name
		vAxis: "y",
	
		// lines: Boolean?
		//		Whether or not to draw lines on this plot.  Defaults to true.
		lines:   true,
	
		// areas: Boolean?
		//		Whether or not to draw areas on this plot. Defaults to false.
		areas:   false,
	
		// markers: Boolean?
		//		Whether or not to draw markers at data points on this plot. Default is false.
		markers: false,
	
		// tension: Number|String?
		//		Whether or not to apply 'tensioning' to the lines on this chart.
		//		Options include a number, "X", "x", or "S"; if a number is used, the
		//		simpler bezier curve calculations are used to draw the lines.  If X, x or S
		//		is used, the more accurate smoothing algorithm is used.
		tension: "",
	
		// animate: Boolean?
		//		Whether or not to animate the chart to place.
		animate: false,
	
		// stroke: dojox.gfx.Stroke?
		//		An optional stroke to use for any series on the plot.
		stroke:		{},
	
		// outline: dojox.gfx.Stroke?
		//		An optional stroke used to outline any series on the plot.
		outline:	{},
	
		// shadow: dojox.gfx.Stroke?
		//		An optional stroke to use to draw any shadows for a series on a plot.
		shadow:		{},
	
		// fill: dojox.gfx.Fill?
		//		Any fill to be used for elements on the plot (such as areas).
		fill:		{},

		// styleFunc: Function?
		//		A function that returns a styling object for the a given data item.
		styleFunc:	null,
	
		// font: String?
		//		A font definition to be used for labels and other text-based elements on the plot.
		font:		"",
	
		// fontColor: String|dojo.Color?
		//		The color to be used for any text-based elements on the plot.
		fontColor:	"",
	
		// markerStroke: dojo.gfx.Stroke?
		//		An optional stroke to use for any markers on the plot.
		markerStroke:		{},
	
		// markerOutline: dojo.gfx.Stroke?
		//		An optional outline to use for any markers on the plot.
		markerOutline:		{},
	
		// markerShadow: dojo.gfx.Stroke?
		//		An optional shadow to use for any markers on the plot.
		markerShadow:		{},
	
		// markerFill: dojo.gfx.Fill?
		//		An optional fill to use for any markers on the plot.
		markerFill:			{},
	
		// markerFont: String?
		//		An optional font definition to use for any markers on the plot.
		markerFont:			"",
	
		// markerFontColor: String|dojo.Color?
		//		An optional color to use for any marker text on the plot.
		markerFontColor:	"",
		
		// enableCache: Boolean?
		//		Whether the markers are cached from one rendering to another. This improves the rendering performance of
		//		successive rendering but penalize the first rendering.  Default false.
		enableCache: false,

		// interpolate: Boolean?
		//		Whether when there is a null data point in the data the plot interpolates it or if the lines is split at that
		//		point.	Default false.
		interpolate: false
	});
=====*/

	var purgeGroup = dfr.lambda("item.purgeGroup()");

	var DEFAULT_ANIMATION_LENGTH = 1200;	// in ms

	return declare("dojox.charting.plot2d.Default", [CartesianBase, _PlotEvents], {
		
		// defaultParams:
		//		The default parameters of this plot.
		defaultParams: {
			hAxis: "x",		// use a horizontal axis named "x"
			vAxis: "y",		// use a vertical axis named "y"
			lines:   true,	// draw lines
			areas:   false,	// draw areas
			markers: false,	// draw markers
			tension: "",	// draw curved lines (tension is "X", "x", or "S")
			animate: false, // animate chart to place
			enableCache: false,
			interpolate: false
		},
		
		// optionalParams:
		//		The optional parameters of this plot.
		optionalParams: {
			// theme component
			stroke:		{},
			outline:	{},
			shadow:		{},
			fill:		{},
			styleFunc: null,
			font:		"",
			fontColor:	"",
			marker:             "",
			markerStroke:		{},
			markerOutline:		{},
			markerShadow:		{},
			markerFill:			{},
			markerFont:			"",
			markerFontColor:	""
		},

		constructor: function(chart, kwArgs){
			// summary:
			//		Return a new plot.
			// chart: dojox/charting/Chart
			//		The chart this plot belongs to.
			// kwArgs: dojox.charting.plot2d.__DefaultCtorArgs?
			//		An optional arguments object to help define this plot.
			this.opt = lang.clone(this.defaultParams);
			du.updateWithObject(this.opt, kwArgs);
			du.updateWithPattern(this.opt, kwArgs, this.optionalParams);
			this.series = [];
			this.hAxis = this.opt.hAxis;
			this.vAxis = this.opt.vAxis;

			// animation properties
			this.animate = this.opt.animate;
		},

		createPath: function(run, creator, params){
			var path;
			if(this.opt.enableCache && run._pathFreePool.length > 0){
				path = run._pathFreePool.pop();
				path.setShape(params);
				// was cleared, add it back
				creator.add(path);
			}else{
				path = creator.createPath(params);
			}
			if(this.opt.enableCache){
				run._pathUsePool.push(path);
			}
			return path;
		},

		buildSegments: function(i, indexed){
			var run = this.series[i],
				min = indexed?Math.max(0, Math.floor(this._hScaler.bounds.from - 1)):0,
				max = indexed?Math.min(run.data.length, Math.ceil(this._hScaler.bounds.to)):run.data.length,
				rseg = null, segments = [];

			// split the run data into dense segments (each containing no nulls)
			// except if interpolates is false in which case ignore null between valid data
			for(var j = min; j < max; j++){
				if(run.data[j] != null && (indexed || run.data[j].y != null)){
					if(!rseg){
						rseg = [];
						segments.push({index: j, rseg: rseg});
					}
					rseg.push((indexed && run.data[j].hasOwnProperty("y"))?run.data[j].y:run.data[j]);
				}else{
					if(!this.opt.interpolate || indexed){
						// we break the line only if not interpolating or if we have indexed data
						rseg = null;
					}
				}
			}
			return segments;
		},

		render: function(dim, offsets){
			// summary:
			//		Render/draw everything on this plot.
			// dim: Object
			//		An object of the form { width, height }
			// offsets: Object
			//		An object of the form { l, r, t, b }
			// returns: dojox/charting/plot2d/Default
			//		A reference to this plot for functional chaining.

			// make sure all the series is not modified
			if(this.zoom && !this.isDataDirty()){
				return this.performZoom(dim, offsets);
			}

			this.resetEvents();
			this.dirty = this.isDirty();
			var s;
			if(this.dirty){
				arr.forEach(this.series, purgeGroup);
				this._eventSeries = {};
				this.cleanGroup();
				this.group.setTransform(null);
				s = this.group;
				df.forEachRev(this.series, function(item){ item.cleanGroup(s); });
			}
			var t = this.chart.theme, stroke, outline, marker, events = this.events();

			for(var i = this.series.length - 1; i >= 0; --i){
				var run = this.series[i];
				if(!this.dirty && !run.dirty){
					t.skip();
					this._reconnectEvents(run.name);
					continue;
				}
				run.cleanGroup();
				if(this.opt.enableCache){
					run._pathFreePool = (run._pathFreePool?run._pathFreePool:[]).concat(run._pathUsePool?run._pathUsePool:[]);
					run._pathUsePool = [];
				}
				if(!run.data.length){
					run.dirty = false;
					t.skip();
					continue;
				}

				var theme = t.next(this.opt.areas ? "area" : "line", [this.opt, run], true),
					lpoly,
					ht = this._hScaler.scaler.getTransformerFromModel(this._hScaler),
					vt = this._vScaler.scaler.getTransformerFromModel(this._vScaler),
					eventSeries = this._eventSeries[run.name] = new Array(run.data.length);

				s = run.group;
				
				// optim works only for index based case
				var indexed = arr.some(run.data, function(item){
					return typeof item == "number" || (item && !item.hasOwnProperty("x"));
				});

				var rsegments = this.buildSegments(i, indexed);
				for(var seg = 0; seg < rsegments.length; seg++){
					var rsegment = rsegments[seg];
					if(indexed){
						lpoly = arr.map(rsegment.rseg, function(v, i){
							return {
								x: ht(i + rsegment.index + 1) + offsets.l,
								y: dim.height - offsets.b - vt(v),
								data: v
							};
						}, this);
					}else{
						lpoly = arr.map(rsegment.rseg, function(v){
							return {
								x: ht(v.x) + offsets.l,
								y: dim.height - offsets.b - vt(v.y),
								data: v
							};
						}, this);
					}

					// if we are indexed & we interpolate we need to put all the segments as a single one now
					if(indexed && this.opt.interpolate){
						while(seg < rsegments.length) {
							seg++;
							rsegment = rsegments[seg];
							if(rsegment){
								lpoly = lpoly.concat(arr.map(rsegment.rseg, function(v, i){
									return {
										x: ht(i + rsegment.index + 1) + offsets.l,
										y: dim.height - offsets.b - vt(v),
										data: v
									};
								}, this));
							}
						}
					} 

					var lpath = this.opt.tension ? dc.curve(lpoly, this.opt.tension) : "";

					if(this.opt.areas && lpoly.length > 1){
						var fill = this._plotFill(theme.series.fill, dim, offsets), apoly = lang.clone(lpoly);
						if(this.opt.tension){
							var apath = "L" + apoly[apoly.length-1].x + "," + (dim.height - offsets.b) +
								" L" + apoly[0].x + "," + (dim.height - offsets.b) +
								" L" + apoly[0].x + "," + apoly[0].y;
							run.dyn.fill = s.createPath(lpath + " " + apath).setFill(fill).getFill();
						} else {
							apoly.push({x: lpoly[lpoly.length - 1].x, y: dim.height - offsets.b});
							apoly.push({x: lpoly[0].x, y: dim.height - offsets.b});
							apoly.push(lpoly[0]);
							run.dyn.fill = s.createPolyline(apoly).setFill(fill).getFill();
						}
					}
					if(this.opt.lines || this.opt.markers){
						// need a stroke
						stroke = theme.series.stroke;
						if(theme.series.outline){
							outline = run.dyn.outline = dc.makeStroke(theme.series.outline);
							outline.width = 2 * outline.width + stroke.width;
						}
					}
					if(this.opt.markers){
						run.dyn.marker = theme.symbol;
					}
					var frontMarkers = null, outlineMarkers = null, shadowMarkers = null;
					if(stroke && theme.series.shadow && lpoly.length > 1){
						var shadow = theme.series.shadow,
							spoly = arr.map(lpoly, function(c){
								return {x: c.x + shadow.dx, y: c.y + shadow.dy};
							});
						if(this.opt.lines){
							if(this.opt.tension){
								run.dyn.shadow = s.createPath(dc.curve(spoly, this.opt.tension)).setStroke(shadow).getStroke();
							} else {
								run.dyn.shadow = s.createPolyline(spoly).setStroke(shadow).getStroke();
							}
						}
						if(this.opt.markers && theme.marker.shadow){
							shadow = theme.marker.shadow;
							shadowMarkers = arr.map(spoly, function(c){
								return this.createPath(run, s, "M" + c.x + " " + c.y + " " + theme.symbol).
									setStroke(shadow).setFill(shadow.color);
							}, this);
						}
					}
					if(this.opt.lines && lpoly.length > 1){
						if(outline){
							if(this.opt.tension){
								run.dyn.outline = s.createPath(lpath).setStroke(outline).getStroke();
							} else {
								run.dyn.outline = s.createPolyline(lpoly).setStroke(outline).getStroke();
							}
						}
						if(this.opt.tension){
							run.dyn.stroke = s.createPath(lpath).setStroke(stroke).getStroke();
						} else {
							run.dyn.stroke = s.createPolyline(lpoly).setStroke(stroke).getStroke();
						}
					}
					if(this.opt.markers){
						var markerTheme = theme; 
						frontMarkers = new Array(lpoly.length);
						outlineMarkers = new Array(lpoly.length);
						outline = null;
						if(markerTheme.marker.outline){
							outline = dc.makeStroke(markerTheme.marker.outline);
							outline.width = 2 * outline.width + (markerTheme.marker.stroke ? markerTheme.marker.stroke.width : 0);
						}
						arr.forEach(lpoly, function(c, i){
							if(this.opt.styleFunc || typeof c.data != "number"){
								var tMixin = typeof c.data != "number" ? [c.data] : [];
								if(this.opt.styleFunc){
									tMixin.push(this.opt.styleFunc(c.data));
								}
								markerTheme = t.addMixin(theme, "marker", tMixin, true);
							}else{
								markerTheme = t.post(theme, "marker");
							}
							var path = "M" + c.x + " " + c.y + " " + markerTheme.symbol;
							if(outline){
								outlineMarkers[i] = this.createPath(run, s, path).setStroke(outline);
							}
							frontMarkers[i] = this.createPath(run, s, path).setStroke(markerTheme.marker.stroke).setFill(markerTheme.marker.fill);
						}, this);
						run.dyn.markerFill = markerTheme.marker.fill;
						run.dyn.markerStroke = markerTheme.marker.stroke;
						if(events){
							arr.forEach(frontMarkers, function(s, i){
								var o = {
									element: "marker",
									index:   i + rsegment.index,
									run:     run,
									shape:   s,
									outline: outlineMarkers[i] || null,
									shadow:  shadowMarkers && shadowMarkers[i] || null,
									cx:      lpoly[i].x,
									cy:      lpoly[i].y
								};
								if(indexed){
									o.x = i + rsegment.index + 1;
									o.y = rsegment.rseg[i];
								}else{
									o.x = rsegment.rseg[i].x;
									o.y = rsegment.rseg[i].y;
								}
								this._connectEvents(o);
								eventSeries[i + rsegment.index] = o;
							}, this);
						}else{
							delete this._eventSeries[run.name];
						}
					}
				}
				run.dirty = false;
			}
			if(this.animate){
				// grow from the bottom
				var plotGroup = this.group;
				fx.animateTransform(lang.delegate({
					shape: plotGroup,
					duration: DEFAULT_ANIMATION_LENGTH,
					transform:[
						{name:"translate", start: [0, dim.height - offsets.b], end: [0, 0]},
						{name:"scale", start: [1, 0], end:[1, 1]},
						{name:"original"}
					]
				}, this.animate)).play();
			}
			this.dirty = false;
			return this;	//	dojox/charting/plot2d/Default
		}
	});
});

},
'dojox/charting/plot2d/CartesianBase':function(){
define("dojox/charting/plot2d/CartesianBase", ["dojo/_base/lang", "dojo/_base/declare", "dojo/_base/connect", 
		"./Base",
		"../scaler/primitive", "dojox/gfx/fx"],
	function(lang, declare, hub, Base, primitive, fx){

	return declare("dojox.charting.plot2d.CartesianBase", Base, {
		// summary:
		//		Base class for cartesian plot types.
		constructor: function(chart, kwArgs){
			// summary:
			//		Create a cartesian base plot for cartesian charts.
			// chart: dojox/chart/Chart
			//		The chart this plot belongs to.
			// kwArgs: dojox.charting.plot2d.__PlotCtorArgs?
			//		An optional arguments object to help define the plot.
			this.axes = ["hAxis", "vAxis"];
			this.zoom = null,
			this.zoomQueue = [];	// zooming action task queue
			this.lastWindow = {vscale: 1, hscale: 1, xoffset: 0, yoffset: 0};
		},
		clear: function(){
			// summary:
			//		Clear out all of the information tied to this plot.
			// returns: dojox/charting/plot2d/CartesianBase
			//		A reference to this plot for functional chaining.
			this.inherited(arguments);
			this._hAxis = null;
			this._vAxis = null;
			return this;	//	dojox/charting/plot2d/CartesianBase
		},
		cleanGroup: function(creator){
			this.inherited(arguments, [creator || this.chart.plotGroup]);
		},
		setAxis: function(axis){
			// summary:
			//		Set an axis for this plot.
			// axis: dojox/charting/axis2d/Base
			//		The axis to set.
			// returns: dojox/charting/plot2d/CartesianBase
			//		A reference to this plot for functional chaining.
			if(axis){
				this[axis.vertical ? "_vAxis" : "_hAxis"] = axis;
			}
			return this;	//	dojox/charting/plot2d/CartesianBase
		},
		toPage: function(coord){
			// summary:
			//		Compute page coordinates from plot axis data coordinates.
			// coord: Object?
			//		The coordinates in plot axis data coordinate space. For cartesian charts that is of the following form:
			//		`{ hAxisName: 50, vAxisName: 200 }`
			//		If not provided return the transform method instead of the result of the transformation.
			// returns: Object
			//		The resulting page pixel coordinates. That is of the following form:
			//		`{ x: 50, y: 200 }`
			var ah = this._hAxis, av = this._vAxis,
				sh = ah.getScaler(), sv = av.getScaler(),
				th = sh.scaler.getTransformerFromModel(sh),
				tv = sv.scaler.getTransformerFromModel(sv),
				c = this.chart.getCoords(),
				o = this.chart.offsets, dim = this.chart.dim;
			var t = function(coord){
				var r = {};
				r.x = th(coord[ah.name]) + c.x + o.l;
				r.y = c.y + dim.height - o.b - tv(coord[av.name]);
				return r;
			};
			// if no coord return the function so that we can capture the current transforms
			// and reuse them later on
			return coord?t(coord):t; // Object
		},
		toData: function(coord){
			// summary:
			//		Compute plot axis data coordinates from page coordinates.
			// coord: Object
			//		The pixel coordinate in page coordinate space. That is of the following form:
			//		`{ x: 50, y: 200 }`
			//		If not provided return the tranform method instead of the result of the transformation.
			// returns: Object
			//		The resulting plot axis data coordinates. For cartesian charts that is of the following form:
			//		`{ hAxisName: 50, vAxisName: 200 }`
			var ah = this._hAxis, av = this._vAxis,
				sh = ah.getScaler(), sv = av.getScaler(),
				th = sh.scaler.getTransformerFromPlot(sh),
				tv = sv.scaler.getTransformerFromPlot(sv),
				c = this.chart.getCoords(),
				o = this.chart.offsets, dim = this.chart.dim;
			var t = function(coord){
				var r = {};
				r[ah.name] = th(coord.x - c.x - o.l);
				r[av.name] = tv(c.y + dim.height - coord.y  - o.b);
				return r;
			};
			// if no coord return the function so that we can capture the current transforms
			// and reuse them later on
			return coord?t(coord):t; // Object
		},
		isDirty: function(){
			// summary:
			//		Returns whether or not this plot needs to be rendered.
			// returns: Boolean
			//		The state of the plot.
			return this.dirty || this._hAxis && this._hAxis.dirty || this._vAxis && this._vAxis.dirty;	//	Boolean
		},
		performZoom: function(dim, offsets){
			// summary:
			//		Create/alter any zooming windows on this plot.
			// dim: Object
			//		An object of the form { width, height }.
			// offsets: Object
			//		An object of the form { l, r, t, b }.
			// returns: dojox/charting/plot2d/CartesianBase
			//		A reference to this plot for functional chaining.

			// get current zooming various
			var vs = this._vAxis.scale || 1,
				hs = this._hAxis.scale || 1,
				vOffset = dim.height - offsets.b,
				hBounds = this._hScaler.bounds,
				xOffset = (hBounds.from - hBounds.lower) * hBounds.scale,
				vBounds = this._vScaler.bounds,
				yOffset = (vBounds.from - vBounds.lower) * vBounds.scale,
				// get incremental zooming various
				rVScale = vs / this.lastWindow.vscale,
				rHScale = hs / this.lastWindow.hscale,
				rXOffset = (this.lastWindow.xoffset - xOffset)/
					((this.lastWindow.hscale == 1)? hs : this.lastWindow.hscale),
				rYOffset = (yOffset - this.lastWindow.yoffset)/
					((this.lastWindow.vscale == 1)? vs : this.lastWindow.vscale),

				shape = this.group,
				anim = fx.animateTransform(lang.delegate({
					shape: shape,
					duration: 1200,
					transform:[
						{name:"translate", start:[0, 0], end: [offsets.l * (1 - rHScale), vOffset * (1 - rVScale)]},
						{name:"scale", start:[1, 1], end: [rHScale, rVScale]},
						{name:"original"},
						{name:"translate", start: [0, 0], end: [rXOffset, rYOffset]}
					]}, this.zoom));

			lang.mixin(this.lastWindow, {vscale: vs, hscale: hs, xoffset: xOffset, yoffset: yOffset});
			//add anim to zooming action queue,
			//in order to avoid several zooming action happened at the same time
			this.zoomQueue.push(anim);
			//perform each anim one by one in zoomQueue
			hub.connect(anim, "onEnd", this, function(){
				this.zoom = null;
				this.zoomQueue.shift();
				if(this.zoomQueue.length > 0){
					this.zoomQueue[0].play();
				}
			});
			if(this.zoomQueue.length == 1){
				this.zoomQueue[0].play();
			}
			return this;	//	dojox/charting/plot2d/CartesianBase
		},
		initializeScalers: function(dim, stats){
			// summary:
			//		Initializes scalers using attached axes.
			// dim: Object
			//		Size of a plot area in pixels as {width, height}.
			// stats: Object
			//		Min/max of data in both directions as {hmin, hmax, vmin, vmax}.
			// returns: dojox/charting/plot2d/CartesianBase
			//		A reference to this plot for functional chaining.
			if(this._hAxis){
				if(!this._hAxis.initialized()){
					this._hAxis.calculate(stats.hmin, stats.hmax, dim.width);
				}
				this._hScaler = this._hAxis.getScaler();
			}else{
				this._hScaler = primitive.buildScaler(stats.hmin, stats.hmax, dim.width);
			}
			if(this._vAxis){
				if(!this._vAxis.initialized()){
					this._vAxis.calculate(stats.vmin, stats.vmax, dim.height);
				}
				this._vScaler = this._vAxis.getScaler();
			}else{
				this._vScaler = primitive.buildScaler(stats.vmin, stats.vmax, dim.height);
			}
			return this;	//	dojox/charting/plot2d/CartesianBase
		}
	});
});

},
'dojox/charting/plot2d/Base':function(){
define("dojox/charting/plot2d/Base", ["dojo/_base/declare",
		"../Element", "dojo/_base/array",
	    "./common"],
	function(declare, Element, arr,  common){
/*=====
dojox.charting.plot2d.__PlotCtorArgs = {
	// summary:
	//		The base keyword arguments object for plot constructors.
	//		Note that the parameters for this may change based on the
	//		specific plot type (see the corresponding plot type for
	//		details).
};
=====*/
return declare("dojox.charting.plot2d.Base", Element, {
	// summary:
	//		Base class for all plot types.
	constructor: function(chart, kwArgs){
		// summary:
		//		Create a base plot for charting.
		// chart: dojox/chart/Chart
		//		The chart this plot belongs to.
		// kwArgs: dojox.charting.plot2d.__PlotCtorArgs?
		//		An optional arguments object to help define the plot.
	},
	clear: function(){
		// summary:
		//		Clear out all of the information tied to this plot.
		// returns: dojox.charting.plot2d.Base
		//		A reference to this plot for functional chaining.
		this.series = [];
		this.dirty = true;
		return this;	//	dojox/charting/plot2d/Base
	},
	setAxis: function(axis){
		// summary:
		//		Set an axis for this plot.
		// axis: dojox.charting.axis2d.Base
		//		The axis to set.
		// returns: dojox/charting/plot2d/Base
		//		A reference to this plot for functional chaining.
		return this;	//	dojox/charting/plot2d/Base
	},
	assignAxes: function(axes){
		// summary:
		//		From an array of axes pick the ones that correspond to this plot and
		//		assign them to the plot using setAxis method.
		// axes: Array
		//		An array of dojox/charting/axis2d/Base
		// tags:
		//		protected
		arr.forEach(this.axes, function(axis){
			if(this[axis]){
				this.setAxis(axes[this[axis]]);
			}
		}, this);
	},
	addSeries: function(run){
		// summary:
		//		Add a data series to this plot.
		// run: dojox.charting.Series
		//		The series to be added.
		// returns: dojox/charting/plot2d/Base
		//		A reference to this plot for functional chaining.
		this.series.push(run);
		return this;	//	dojox/charting/plot2d/Base
	},
	getSeriesStats: function(){
		// summary:
		//		Calculate the min/max on all attached series in both directions.
		// returns: Object
		//		{hmin, hmax, vmin, vmax} min/max in both directions.
		return common.collectSimpleStats(this.series);
	},
	calculateAxes: function(dim){
		// summary:
		//		Stub function for running the axis calculations (deprecated).
		// dim: Object
		//		An object of the form { width, height }
		// returns: dojox/charting/plot2d/Base
		//		A reference to this plot for functional chaining.
		this.initializeScalers(dim, this.getSeriesStats());
		return this;	//	dojox/charting/plot2d/Base
	},
	initializeScalers: function(){
		// summary:
		//		Does nothing.
		return this;
	},
	isDataDirty: function(){
		// summary:
		//		Returns whether or not any of this plot's data series need to be rendered.
		// returns: Boolean
		//		Flag indicating if any of this plot's series are invalid and need rendering.
		return arr.some(this.series, function(item){ return item.dirty; });	//	Boolean
	},
	render: function(dim, offsets){
		// summary:
		//		Render the plot on the chart.
		// dim: Object
		//		An object of the form { width, height }.
		// offsets: Object
		//		An object of the form { l, r, t, b }.
		// returns: dojox/charting/plot2d/Base
		//		A reference to this plot for functional chaining.
		return this;	//	dojox/charting/plot2d/Base
	},
	getRequiredColors: function(){
		// summary:
		//		Get how many data series we have, so we know how many colors to use.
		// returns: Number
		//		The number of colors needed.
		return this.series.length;	//	Number
	}
});
});

},
'dojox/charting/plot2d/common':function(){
define("dojox/charting/plot2d/common", ["dojo/_base/lang", "dojo/_base/array", "dojo/_base/Color", 
		"dojox/gfx", "dojox/lang/functional", "../scaler/common"], 
	function(lang, arr, Color, g, df, sc){
	
	var common = lang.getObject("dojox.charting.plot2d.common", true);
	
	return lang.mixin(common, {	
		doIfLoaded: sc.doIfLoaded,
		makeStroke: function(stroke){
			if(!stroke){ return stroke; }
			if(typeof stroke == "string" || stroke instanceof Color){
				stroke = {color: stroke};
			}
			return g.makeParameters(g.defaultStroke, stroke);
		},
		augmentColor: function(target, color){
			var t = new Color(target),
				c = new Color(color);
			c.a = t.a;
			return c;
		},
		augmentStroke: function(stroke, color){
			var s = common.makeStroke(stroke);
			if(s){
				s.color = common.augmentColor(s.color, color);
			}
			return s;
		},
		augmentFill: function(fill, color){
			var fc, c = new Color(color);
			if(typeof fill == "string" || fill instanceof Color){
				return common.augmentColor(fill, color);
			}
			return fill;
		},

		defaultStats: {
			vmin: Number.POSITIVE_INFINITY, vmax: Number.NEGATIVE_INFINITY,
			hmin: Number.POSITIVE_INFINITY, hmax: Number.NEGATIVE_INFINITY
		},

		collectSimpleStats: function(series){
			var stats = lang.delegate(common.defaultStats);
			for(var i = 0; i < series.length; ++i){
				var run = series[i];
				for(var j = 0; j < run.data.length; j++){
					if(run.data[j] !== null){
						if(typeof run.data[j] == "number"){
							// 1D case
							var old_vmin = stats.vmin, old_vmax = stats.vmax;
							if(!("ymin" in run) || !("ymax" in run)){
								arr.forEach(run.data, function(val, i){
									if(val !== null){
										var x = i + 1, y = val;
										if(isNaN(y)){ y = 0; }
										stats.hmin = Math.min(stats.hmin, x);
										stats.hmax = Math.max(stats.hmax, x);
										stats.vmin = Math.min(stats.vmin, y);
										stats.vmax = Math.max(stats.vmax, y);
									}
								});
							}
							if("ymin" in run){ stats.vmin = Math.min(old_vmin, run.ymin); }
							if("ymax" in run){ stats.vmax = Math.max(old_vmax, run.ymax); }
						}else{
							// 2D case
							var old_hmin = stats.hmin, old_hmax = stats.hmax,
								old_vmin = stats.vmin, old_vmax = stats.vmax;
							if(!("xmin" in run) || !("xmax" in run) || !("ymin" in run) || !("ymax" in run)){
								arr.forEach(run.data, function(val, i){
									if(val !== null){
										var x = "x" in val ? val.x : i + 1, y = val.y;
										if(isNaN(x)){ x = 0; }
										if(isNaN(y)){ y = 0; }
										stats.hmin = Math.min(stats.hmin, x);
										stats.hmax = Math.max(stats.hmax, x);
										stats.vmin = Math.min(stats.vmin, y);
										stats.vmax = Math.max(stats.vmax, y);
									}
								});
							}
							if("xmin" in run){ stats.hmin = Math.min(old_hmin, run.xmin); }
							if("xmax" in run){ stats.hmax = Math.max(old_hmax, run.xmax); }
							if("ymin" in run){ stats.vmin = Math.min(old_vmin, run.ymin); }
							if("ymax" in run){ stats.vmax = Math.max(old_vmax, run.ymax); }
						}

						break;
					}
				}
			}
			return stats;
		},

		calculateBarSize: function(/* Number */ availableSize, /* Object */ opt, /* Number? */ clusterSize){
			if(!clusterSize){
				clusterSize = 1;
			}
			var gap = opt.gap, size = (availableSize - 2 * gap) / clusterSize;
			if("minBarSize" in opt){
				size = Math.max(size, opt.minBarSize);
			}
			if("maxBarSize" in opt){
				size = Math.min(size, opt.maxBarSize);
			}
			size = Math.max(size, 1);
			gap = (availableSize - size * clusterSize) / 2;
			return {size: size, gap: gap};	// Object
		},

		collectStackedStats: function(series){
			// collect statistics
			var stats = lang.clone(common.defaultStats);
			if(series.length){
				// 1st pass: find the maximal length of runs
				stats.hmin = Math.min(stats.hmin, 1);
				stats.hmax = df.foldl(series, "seed, run -> Math.max(seed, run.data.length)", stats.hmax);
				// 2nd pass: stack values
				for(var i = 0; i < stats.hmax; ++i){
					var v = series[0].data[i];
					v = v && (typeof v == "number" ? v : v.y);
					if(isNaN(v)){ v = 0; }
					stats.vmin = Math.min(stats.vmin, v);
					for(var j = 1; j < series.length; ++j){
						var t = series[j].data[i];
						t = t && (typeof t == "number" ? t : t.y);
						if(isNaN(t)){ t = 0; }
						v += t;
					}
					stats.vmax = Math.max(stats.vmax, v);
				}
			}
			return stats;
		},

		curve: function(/* Number[] */a, /* Number|String */tension){
			//	FIX for #7235, submitted by Enzo Michelangeli.
			//	Emulates the smoothing algorithms used in a famous, unnamed spreadsheet
			//		program ;)
			var array = a.slice(0);
			if(tension == "x") {
				array[array.length] = array[0];   // add a last element equal to the first, closing the loop
			}
			var p=arr.map(array, function(item, i){
				if(i==0){ return "M" + item.x + "," + item.y; }
				if(!isNaN(tension)) { // use standard Dojo smoothing in tension is numeric
					var dx=item.x-array[i-1].x, dy=array[i-1].y;
					return "C"+(item.x-(tension-1)*(dx/tension))+","+dy+" "+(item.x-(dx/tension))+","+item.y+" "+item.x+","+item.y;
				} else if(tension == "X" || tension == "x" || tension == "S") {
					// use Excel "line smoothing" algorithm (http://xlrotor.com/resources/files.shtml)
					var p0, p1 = array[i-1], p2 = array[i], p3;
					var bz1x, bz1y, bz2x, bz2y;
					var f = 1/6;
					if(i==1) {
						if(tension == "x") {
							p0 = array[array.length-2];
						} else { // "tension == X || tension == "S"
							p0 = p1;
						}
						f = 1/3;
					} else {
						p0 = array[i-2];
					}
					if(i==(array.length-1)) {
						if(tension == "x") {
							p3 = array[1];
						} else { // "tension == X || tension == "S"
							p3 = p2;
						}
						f = 1/3;
					} else {
						p3 = array[i+1];
					}
					var p1p2 = Math.sqrt((p2.x-p1.x)*(p2.x-p1.x)+(p2.y-p1.y)*(p2.y-p1.y));
					var p0p2 = Math.sqrt((p2.x-p0.x)*(p2.x-p0.x)+(p2.y-p0.y)*(p2.y-p0.y));
					var p1p3 = Math.sqrt((p3.x-p1.x)*(p3.x-p1.x)+(p3.y-p1.y)*(p3.y-p1.y));

					var p0p2f = p0p2 * f;
					var p1p3f = p1p3 * f;

					if(p0p2f > p1p2/2 && p1p3f > p1p2/2) {
						p0p2f = p1p2/2;
						p1p3f = p1p2/2;
					} else if(p0p2f > p1p2/2) {
						p0p2f = p1p2/2;
						p1p3f = p1p2/2 * p1p3/p0p2;
					} else if(p1p3f > p1p2/2) {
						p1p3f = p1p2/2;
						p0p2f = p1p2/2 * p0p2/p1p3;
					}

					if(tension == "S") {
						if(p0 == p1) { p0p2f = 0; }
						if(p2 == p3) { p1p3f = 0; }
					}

					bz1x = p1.x + p0p2f*(p2.x - p0.x)/p0p2;
					bz1y = p1.y + p0p2f*(p2.y - p0.y)/p0p2;
					bz2x = p2.x - p1p3f*(p3.x - p1.x)/p1p3;
					bz2y = p2.y - p1p3f*(p3.y - p1.y)/p1p3;
				}
				return "C"+(bz1x+","+bz1y+" "+bz2x+","+bz2y+" "+p2.x+","+p2.y);
			});
			return p.join(" ");
		},
		
		getLabel: function(/*Number*/number, /*Boolean*/fixed, /*Number*/precision){
			return sc.doIfLoaded("dojo/number", function(numberLib){
				return (fixed ? numberLib.format(number, {places : precision}) :
					numberLib.format(number)) || "";
			}, function(){
				return fixed ? number.toFixed(precision) : number.toString();
			});
		}
	});
});

},
'dojox/charting/scaler/common':function(){
define("dojox/charting/scaler/common", ["dojo/_base/lang"], function(lang){

	var eq = function(/*Number*/ a, /*Number*/ b){
		// summary:
		//		compare two FP numbers for equality
		return Math.abs(a - b) <= 1e-6 * (Math.abs(a) + Math.abs(b));	// Boolean
	};
	
	var common = lang.getObject("dojox.charting.scaler.common", true);
	
	var testedModules = {};

	return lang.mixin(common, {
		doIfLoaded: function(moduleName, ifloaded, ifnotloaded){
			if(testedModules[moduleName] == undefined){
				try{
					testedModules[moduleName] = require(moduleName);
				}catch(e){
					testedModules[moduleName] = null;
				}
			}
			if(testedModules[moduleName]){
				return ifloaded(testedModules[moduleName]);
			}else{
				return ifnotloaded();
			}
		},
		getNumericLabel: function(/*Number*/ number, /*Number*/ precision, /*Object*/ kwArgs){
			var def = "";
			common.doIfLoaded("dojo/number", function(numberLib){
				def = (kwArgs.fixed ? numberLib.format(number, {places : precision < 0 ? -precision : 0}) :
					numberLib.format(number)) || "";
			}, function(){
				def = kwArgs.fixed ? number.toFixed(precision < 0 ? -precision : 0) : number.toString();
			});
			if(kwArgs.labelFunc){
				var r = kwArgs.labelFunc(def, number, precision);
				if(r){ return r; }
				// else fall through to the regular labels search
			}
			if(kwArgs.labels){
				// classic binary search
				// TODO: working only if the array is sorted per value should be better documented or sorted automatically
				var l = kwArgs.labels, lo = 0, hi = l.length;
				while(lo < hi){
					var mid = Math.floor((lo + hi) / 2), val = l[mid].value;
					if(val < number){
						lo = mid + 1;
					}else{
						hi = mid;
					}
				}
				// lets take into account FP errors
				if(lo < l.length && eq(l[lo].value, number)){
					return l[lo].text;
				}
				--lo;
				if(lo >= 0 && lo < l.length && eq(l[lo].value, number)){
					return l[lo].text;
				}
				lo += 2;
				if(lo < l.length && eq(l[lo].value, number)){
					return l[lo].text;
				}
				// otherwise we will produce a number
			}
			return def;
		}
	});
});

},
'dojox/charting/scaler/primitive':function(){
define("dojox/charting/scaler/primitive", ["dojo/_base/lang"], 
  function(lang){
	var primitive = lang.getObject("dojox.charting.scaler.primitive", true);
	return lang.mixin(primitive, {
		buildScaler: function(/*Number*/ min, /*Number*/ max, /*Number*/ span, /*Object*/ kwArgs){
			if(min == max){
				// artificially extend bounds
				min -= 0.5;
				max += 0.5;
				// now the line will be centered
			}
			return {
				bounds: {
					lower: min,
					upper: max,
					from:  min,
					to:    max,
					scale: span / (max - min),
					span:  span
				},
				scaler: primitive
			};
		},
		buildTicks: function(/*Object*/ scaler, /*Object*/ kwArgs){
			return {major: [], minor: [], micro: []};	// Object
		},
		getTransformerFromModel: function(/*Object*/ scaler){
			var offset = scaler.bounds.from, scale = scaler.bounds.scale;
			return function(x){ return (x - offset) * scale; };	// Function
		},
		getTransformerFromPlot: function(/*Object*/ scaler){
			var offset = scaler.bounds.from, scale = scaler.bounds.scale;
			return function(x){ return x / scale + offset; };	// Function
		}
	});
});

},
'dojox/charting/plot2d/_PlotEvents':function(){
define("dojox/charting/plot2d/_PlotEvents", ["dojo/_base/lang", "dojo/_base/array", "dojo/_base/declare", "dojo/_base/connect"], 
	function(lang, arr, declare, hub){

	return declare("dojox.charting.plot2d._PlotEvents", null, {
		constructor: function(){
			this._shapeEvents = [];
			this._eventSeries = {};
		},
		destroy: function(){
			// summary:
			//		Destroy any internal elements and event handlers.
			this.resetEvents();
			this.inherited(arguments);
		},
		plotEvent: function(o){
			// summary:
			//		Stub function for use by specific plots.
			// o: Object
			//		An object intended to represent event parameters.
		},
		raiseEvent: function(o){
			// summary:
			//		Raises events in predefined order
			// o: Object
			//		An object intended to represent event parameters.
			this.plotEvent(o);
			var t = lang.delegate(o);
			t.originalEvent = o.type;
			t.originalPlot  = o.plot;
			t.type = "onindirect";
			arr.forEach(this.chart.stack, function(plot){
				if(plot !== this && plot.plotEvent){
					t.plot = plot;
					plot.plotEvent(t);
				}
			}, this);
		},
		connect: function(object, method){
			// summary:
			//		Helper function to connect any object's method to our plotEvent.
			// object: Object
			//		The object to connect to.
			// method: String|Function
			//		The method to fire when our plotEvent is fired.
			// returns: Array
			//		The handle as returned from dojo.connect (see dojo.connect).
			this.dirty = true;
			return hub.connect(this, "plotEvent", object, method);	//	Array
		},
		events: function(){
			// summary:
			//		Find out if any event handlers have been connected to our plotEvent.
			// returns: Boolean
			//		A flag indicating that there are handlers attached.
			return !!this.plotEvent.after;
		},
		resetEvents: function(){
			// summary:
			//		Reset all events attached to our plotEvent (i.e. disconnect).
			if(this._shapeEvents.length){
				arr.forEach(this._shapeEvents, function(item){
					item.shape.disconnect(item.handle);
				});
				this._shapeEvents = [];
			}
			this.raiseEvent({type: "onplotreset", plot: this});
		},
		_connectSingleEvent: function(o, eventName){
			this._shapeEvents.push({
				shape:  o.eventMask,
				handle: o.eventMask.connect(eventName, this, function(e){
					o.type  = eventName;
					o.event = e;
					this.raiseEvent(o);
					o.event = null;
				})
			});
		},
		_connectEvents: function(o){
			if(o){
				o.chart = this.chart;
				o.plot  = this;
				o.hAxis = this.hAxis || null;
				o.vAxis = this.vAxis || null;
				o.eventMask = o.eventMask || o.shape;
				this._connectSingleEvent(o, "onmouseover");
				this._connectSingleEvent(o, "onmouseout");
				this._connectSingleEvent(o, "onclick");
			}
		},
		_reconnectEvents: function(seriesName){
			var a = this._eventSeries[seriesName];
			if(a){
				arr.forEach(a, this._connectEvents, this);
			}
		},
		fireEvent: function(seriesName, eventName, index, eventObject){
			// summary:
			//		Emulates firing an event for a given data value (specified by
			//		an index) of a given series.
			// seriesName: String
			//		Series name.
			// eventName: String
			//		Event name to emulate.
			// index: Number
			//		Valid data value index used to raise an event.
			// eventObject: Object?
			//		Optional event object. Especially useful for synthetic events.
			//		Default: null.
			var s = this._eventSeries[seriesName];
			if(s && s.length && index < s.length){
				var o = s[index];
				o.type  = eventName;
				o.event = eventObject || null;
				this.raiseEvent(o);
				o.event = null;
			}
		}
	});
});

},
'dojox/charting/plot2d/commonStacked':function(){
define("dojox/charting/plot2d/commonStacked", [
	"dojo/_base/lang",
	"./common"
], function(lang, common){
	
	var commonStacked = lang.getObject("dojox.charting.plot2d.commonStacked", true);
	return lang.mixin(commonStacked, {
		collectStats: function(series){
			var stats = lang.delegate(common.defaultStats);
			for(var i = 0; i < series.length; ++i){
				var run = series[i];
				for(var j = 0; j < run.data.length; j++){
					var x, y;
					if(run.data[j] !== null){
						if(typeof run.data[j] == "number" || !run.data[j].hasOwnProperty("x")){
							y = commonStacked.getIndexValue(series, i, j);
							x = j+1;
						}else{
							x = run.data[j].x;
							if(x !== null){
								y = commonStacked.getValue(series, i, x);
								y = y != null && y.y ? y.y:null; 
							}
						}
						stats.hmin = Math.min(stats.hmin, x);
						stats.hmax = Math.max(stats.hmax, x);
						stats.vmin = Math.min(stats.vmin, y);
						stats.vmax = Math.max(stats.vmax, y);
					
					}
				}
			}
			return stats;
		},
		getIndexValue: function(series, i, index){
			var value = 0, v, j;
			for(j = 0; j <= i; ++j){
				v = series[j].data[index];
				if(v != null){
					if(isNaN(v)){ v = v.y || 0; }
					value += v;
				}
			}
			return value;
		},
		
		getValue: function(series, i, x){
			var value = null, j, z;
			for(j = 0; j <= i; ++j){
				for(z = 0; z < series[j].data.length; z++){
					v = series[j].data[z];
					if(v !== null){
						if(v.x == x){
							if(!value){
								value = {x: x};
							}
							if(v.y != null){
								if(value.y == null){
									value.y = 0;
								}
								value.y += v.y;
							}
							break;
						}else if(v.x > x){break;}
					}
				}
			}
			return value;
		}

		
	});
});

},
'dojox/charting/plot2d/Markers':function(){
define("dojox/charting/plot2d/Markers", ["dojo/_base/declare", "./Default"], function(declare, Default){

	return declare("dojox.charting.plot2d.Markers", Default, {
		// summary:
		//		A convenience plot to draw a line chart with markers.
		constructor: function(){
			// summary:
			//		Set up the plot for lines and markers.
			this.opt.markers = true;
		}
	});
});

},
'dojox/charting/axis2d/Default':function(){
define("dojox/charting/axis2d/Default", ["dojo/_base/lang", "dojo/_base/array", "dojo/_base/sniff", "dojo/_base/declare",
	"dojo/_base/connect", "dojo/dom-geometry", "./Invisible",
	"../scaler/common", "../scaler/linear", "./common", "dojox/gfx", "dojox/lang/utils", "dojox/lang/functional"],
	function(lang, arr, has, declare, connect, domGeom, Invisible, scommon,
			lin, acommon, g, du, df){

	/*=====
	var __AxisCtorArgs = {
		// summary:
		//		Optional arguments used in the definition of an axis.
		// vertical: Boolean?
		//		A flag that says whether an axis is vertical (i.e. y axis) or horizontal. Default is false (horizontal).
		// fixUpper: String?
		//		Align the greatest value on the axis with the specified tick level. Options are "major", "minor", "micro", or "none".  Defaults to "none".
		// fixLower: String?
		//		Align the smallest value on the axis with the specified tick level. Options are "major", "minor", "micro", or "none".  Defaults to "none".
		// natural: Boolean?
		//		Ensure tick marks are made on "natural" numbers. Defaults to false.
		// leftBottom: Boolean?
		//		The position of a vertical axis; if true, will be placed against the left-bottom corner of the chart.  Defaults to true.
		// includeZero: Boolean?
		//		Include 0 on the axis rendering.  Default is false.
		// fixed: Boolean?
		//		Force all axis labels to be fixed numbers.  Default is true.
		// majorLabels: Boolean?
		//		Flag to draw labels at major ticks. Default is true.
		// minorTicks: Boolean?
		//		Flag to draw minor ticks on an axis.  Default is true.
		// minorLabels: Boolean?
		//		Flag to labels on minor ticks when there is enough space. Default is true.
		// microTicks: Boolean?
		//		Flag to draw micro ticks on an axis. Default is false.
		// htmlLabels: Boolean?
		//		Flag to use HTML (as opposed to the native vector graphics engine) to draw labels. Default is true.
		// min: Number?
		//		The smallest value on an axis. Default is 0.
		// max: Number?
		//		The largest value on an axis. Default is 1.
		// from: Number?
		//		Force the chart to render data visible from this value. Default is 0.
		// to: Number?
		//		Force the chart to render data visible to this value. Default is 1.
		// majorTickStep: Number?
		//		The amount to skip before a major tick is drawn. When not set the major ticks step is computed from
		//		the data range.
		// minorTickStep: Number?
		//		The amount to skip before a minor tick is drawn. When not set the minor ticks step is computed from
		//		the data range.
		// microTickStep: Number?
		//		The amount to skip before a micro tick is drawn. When not set the micro ticks step is computed from
		// labels: Object[]?
		//		An array of labels for major ticks, with corresponding numeric values, ordered by value.
		// labelFunc: Function?
		//		An optional function to use to compute label text. It takes precedence over
		//		the default text when available. The function must be of the following form:
		//	|		function labelFunc(text, value, precision) {}
		//		`text` is the already pre-formatted text. Pre-formatting is done using `dojo/number` is available, `Date.toFixed` otherwise.
		//		`value`  is the raw axis value.
		//		`precision` is the requested precision to be applied.
		// maxLabelSize: Number?
		//		The maximum size, in pixels, for a label.  To be used with the optional label function.
		// stroke: dojox.gfx.Stroke?
		//		An optional stroke to be used for drawing an axis.
		// majorTick: Object?
		//		An object containing a dojox.gfx.Stroke, and a length (number) for a major tick.
		// minorTick: Object?
		//		An object containing a dojox.gfx.Stroke, and a length (number) for a minor tick.
		// microTick: Object?
		//		An object containing a dojox.gfx.Stroke, and a length (number) for a micro tick.
		// tick: Object?
		//		An object containing a dojox.gfx.Stroke, and a length (number) for a tick.
		// font: String?
		//		An optional font definition (as used in the CSS font property) for labels.
		// fontColor: String|dojo.Color?
		//		An optional color to be used in drawing labels.
		// titleGap: Number?
		//		An optional grap between axis title and axis label
		// titleFont: String?
		//		An optional font definition for axis title
		// titleFontColor: String?
		//		An optional axis title color
		// titleOrientation: String?
		//		An optional orientation for axis title. "axis" means the title facing the axis, "away" means facing away.
		//		If no value is set "axis" is used.
		// enableCache: Boolean?
		//		Whether the ticks and labels are cached from one rendering to another. This improves the rendering performance of
		//		successive rendering but penalize the first rendering. For labels it is only working with gfx labels
		//		not html ones.  Default false.
		// dropLabels: Boolean?
		//		Whether the axis automatically drops labels at regular interval or not to avoid labels overlapping.
		//		This gives better results but require more computations.  You can disable it to save computation
		//		time when you know your labels won't overlap. Default is true.
		// labelSizeChange: Boolean?
		//		Indicates to the axis whether the axis labels are changing their size on zoom. If false this allows to
		//		optimize the axis by avoiding recomputing labels maximum size on zoom actions. Default is false.
	};
	=====*/

	var centerAnchorLimit = 45;	// in degrees

	return declare("dojox.charting.axis2d.Default", Invisible, {
		// summary:
		//		The default axis object used in dojox.charting.  See dojox.charting.Chart.addAxis for details.

		// defaultParams: Object
		//		The default parameters used to define any axis.
		// optionalParams: Object
		//		Any optional parameters needed to define an axis.

		/*=====
		// TODO: the documentation tools need these to be pre-defined in order to pick them up
		//	correctly, but the code here is partially predicated on whether or not the properties
		//	actually exist.  For now, we will leave these undocumented but in the code for later. -- TRT

		// opt: Object
		//		The actual options used to define this axis, created at initialization.
		// scaler: Object
		//		The calculated helper object to tell charts how to draw an axis and any data.
		// ticks: Object
		//		The calculated tick object that helps a chart draw the scaling on an axis.
		// dirty: Boolean
		//		The state of the axis (whether it needs to be redrawn or not)
		// scale: Number
		//		The current scale of the axis.
		// offset: Number
		//		The current offset of the axis.

		opt: null,
		scaler: null,
		ticks: null,
		dirty: true,
		scale: 1,
		offset: 0,
		=====*/
		defaultParams: {
			vertical:	false,		// true for vertical axis
			fixUpper:	"none",	// align the upper on ticks: "major", "minor", "micro", "none"
			fixLower:	"none",	// align the lower on ticks: "major", "minor", "micro", "none"
			natural:	 false,		// all tick marks should be made on natural numbers
			leftBottom:  true,		// position of the axis, used with "vertical"
			includeZero: false,		// 0 should be included
			fixed:	   true,		// all labels are fixed numbers
			majorLabels: true,		// draw major labels
			minorTicks:  true,		// draw minor ticks
			minorLabels: true,		// draw minor labels
			microTicks:  false,		// draw micro ticks
			rotation:	0,			// label rotation angle in degrees
			htmlLabels:  true,		// use HTML to draw labels
			enableCache: false,		// whether we cache or not
			dropLabels: true,		// whether we automatically drop overlapping labels or not
			labelSizeChange: false // whether the labels size change on zoom
		},
		optionalParams: {
			min:			0,	// minimal value on this axis
			max:			1,	// maximal value on this axis
			from:			0,	// visible from this value
			to:				1,	// visible to this value
			majorTickStep:	4,	// major tick step
			minorTickStep:	2,	// minor tick step
			microTickStep:	1,	// micro tick step
			labels:			[],	// array of labels for major ticks
			// with corresponding numeric values
			// ordered by values
			labelFunc:		null, // function to compute label values
			maxLabelSize:	0,	// size in px. For use with labelFunc
			maxLabelCharCount:	0,	// size in word count.
			trailingSymbol:	null,

			// TODO: add support for minRange!
			// minRange:		1,	// smallest distance from min allowed on the axis

			// theme components
			stroke:			{},	// stroke for an axis
			majorTick:		{},	// stroke + length for a tick
			minorTick:		{},	// stroke + length for a tick
			microTick:		{},	// stroke + length for a tick
			tick:		   {},	// stroke + length for a tick
			font:			"",	// font for labels
			fontColor:		"",	// color for labels as a string
			title:				 "",	// axis title
			titleGap:			 0,		// gap between axis title and axis label
			titleFont:			 "",		// axis title font
			titleFontColor:		 "",		// axis title font color
			titleOrientation:	 ""		// "axis" means the title facing the axis, "away" means facing away
		},

		constructor: function(chart, kwArgs){
			// summary:
			//		The constructor for an axis.
			// chart: dojox/charting/Chart
			//		The chart the axis belongs to.
			// kwArgs: __AxisCtorArgs?
			//		Any optional keyword arguments to be used to define this axis.
			this.opt = lang.clone(this.defaultParams);
			du.updateWithObject(this.opt, kwArgs);
			du.updateWithPattern(this.opt, kwArgs, this.optionalParams);
			if(this.opt.enableCache){
				this._textFreePool = [];
				this._lineFreePool = [];
				this._textUsePool = [];
				this._lineUsePool = [];
			}
			this._invalidMaxLabelSize = true;
		},
		setWindow: function(scale, offset){
			// summary:
			//		Set the drawing "window" for the axis.
			// scale: Number
			//		The new scale for the axis.
			// offset: Number
			//		The new offset for the axis.
			// returns: dojox/charting/axis2d/Default
			//		The reference to the axis for functional chaining.
			if(scale != this.scale){
				// if scale changed we need to recompute new max label size
				this._invalidMaxLabelSize = true;
			}
			return this.inherited(arguments);
		},

		_groupLabelWidth: function(labels, font, wcLimit){
			if(!labels.length){
				return 0;
			}
			if(labels.length > 50){
				// let's avoid degenerated cases
				labels.length = 50;
			}
			if(lang.isObject(labels[0])){
				labels = df.map(labels, function(label){ return label.text; });
			}
			if(wcLimit){
				labels = df.map(labels, function(label){
					return lang.trim(label).length == 0 ? "" : label.substring(0, wcLimit) + this.trailingSymbol;
				}, this);
			}
			var s = labels.join("<br>");
			return g._base._getTextBox(s, {font: font}).w || 0;
		},

		_getMaxLabelSize: function(min, max, span, rotation, font, size){
			if(this._maxLabelSize == null && arguments.length == 6){
				var o = this.opt;
				// everything might have changed, reset the minMinorStep value
				this.scaler.minMinorStep = this._prevMinMinorStep = 0;
				var ob = lang.clone(o);
				delete ob.to;
				delete ob.from;
				// build all the ticks from min, to max not from to to _but_ using the step
				// that would be used if we where just displaying from to to from.
				var sb = lin.buildScaler(min, max, span, ob, o.to - o.from);
				sb.minMinorStep = 0;
				this._majorStart = sb.major.start;
				// we build all the ticks not only the ones we need to draw in order to get
				// a correct drop rate computation that works for any offset of this scale
				var tb = lin.buildTicks(sb, o);
				// if there is not tick at all tb is null
				if(size && tb){
					var majLabelW = 0, minLabelW = 0; // non rotated versions
					// we first collect all labels when needed
					var tickLabelFunc = function(tick){
						if(tick.label){
							this.push(tick.label);
						}
					};
					var labels = [];
					if(this.opt.majorLabels){
						arr.forEach(tb.major, tickLabelFunc, labels);
						majLabelW = this._groupLabelWidth(labels, font, ob.maxLabelCharCount);
						if(ob.maxLabelSize){
							majLabelW = Math.min(ob.maxLabelSize, majLabelW);
						}
					}
					// do the minor labels computation only if dropLabels is set
					labels = [];
					if(this.opt.dropLabels && this.opt.minorLabels){
						arr.forEach(tb.minor, tickLabelFunc, labels);
						minLabelW = this._groupLabelWidth(labels, font, ob.maxLabelCharCount);
						if(ob.maxLabelSize){
							minLabelW = Math.min(ob.maxLabelSize, minLabelW);
						}
					}
					this._maxLabelSize = {
						majLabelW: majLabelW, minLabelW: minLabelW,
						majLabelH: size, minLabelH: size
					};
				}else{
					this._maxLabelSize = null;
				}
			}
			return this._maxLabelSize;
		},

		calculate: function(min, max, span){
			this.inherited(arguments);
			// when the scale has not changed there is no reason for minMinorStep to change
			this.scaler.minMinorStep = this._prevMinMinorStep;
			// we want to recompute the dropping mechanism only when the scale or the size of the axis is changing
			// not when for example when we scroll (otherwise effect would be weird)
			if((this._invalidMaxLabelSize || span != this._oldSpan) && (min != Infinity && max != -Infinity)){
				this._invalidMaxLabelSize = false;
				if(this.opt.labelSizeChange){
					this._maxLabelSize = null;
				}
				this._oldSpan = span;
				var o = this.opt;
				var ta = this.chart.theme.axis, rotation = o.rotation % 360,
					labelGap = this.chart.theme.axis.tick.labelGap,
					// TODO: we use one font --- of major tick, we need to use major and minor fonts
					font = o.font || (ta.majorTick && ta.majorTick.font) || (ta.tick && ta.tick.font),
					size = font ? g.normalizedLength(g.splitFontString(font).size) : 0,
					// even if we don't drop label we need to compute max size for offsets
					labelW = this._getMaxLabelSize(min, max, span, rotation, font, size);
				if(typeof labelGap != "number"){
					labelGap = 4; // in pixels
				}
				if(labelW && o.dropLabels){
					var cosr = Math.abs(Math.cos(rotation * Math.PI / 180)),
						sinr = Math.abs(Math.sin(rotation * Math.PI / 180));
					var majLabelW, minLabelW;
					if(rotation < 0){
						rotation += 360;
					}
					switch(rotation){
						case 0:
						case 180:
							// trivial cases: horizontal labels
							if(this.vertical){
								majLabelW = minLabelW = size;
							}else{
								majLabelW = labelW.majLabelW;
								minLabelW = labelW.minLabelW;
							}
							break;
						case 90:
						case 270:
							// trivial cases: vertical
							if(this.vertical){
								majLabelW = labelW.majLabelW;
								minLabelW = labelW.minLabelW;
							}else{
								majLabelW = minLabelW = size;
							}
							break;
						default:
							// all major labels are parallel they can't collapse except if the two ticks are
							// closer than the height of the text * cos(90-rotation)
							majLabelW  = this.vertical ? Math.min(labelW.majLabelW, size / cosr) : Math.min(labelW.majLabelW, size / sinr);
							// for minor labels we need to rotated them
							var gap1 = Math.sqrt(labelW.minLabelW * labelW.minLabelW + size * size),
								gap2 = this.vertical ? size * cosr + labelW.minLabelW * sinr : labelW.minLabelW * cosr + size * sinr;
							minLabelW = Math.min(gap1, gap2);
							break;
					}
					// we need to check both minor and major labels fit a minor step
					this.scaler.minMinorStep = this._prevMinMinorStep =  Math.max(majLabelW, minLabelW) + labelGap;
					var canMinorLabel = this.scaler.minMinorStep <= this.scaler.minor.tick * this.scaler.bounds.scale;
					if(!canMinorLabel){
						// we can't place minor labels, let's see if we can place major ones
						// in a major step and if not which skip interval we must follow
						this._skipInterval = Math.floor((majLabelW + labelGap) / (this.scaler.major.tick * this.scaler.bounds.scale));
					}else{
						// everything fit well
						this._skipInterval = 0;
					}
				}else{
					// drop label disabled
					this._skipInterval = 0;
				}
			}
			// computes the tick subset we need for that scale/offset
			this.ticks = lin.buildTicks(this.scaler, this.opt);
			return this;
		},

		getOffsets: function(){
			// summary:
			//		Get the physical offset values for this axis (used in drawing data series). This method is not
			//		supposed to be called by the users but internally.
			// returns: Object
			//		The calculated offsets in the form of { l, r, t, b } (left, right, top, bottom).
			var s = this.scaler, offsets = { l: 0, r: 0, t: 0, b: 0 };
			if(!s){
				return offsets;
			}
			var o = this.opt, a, b, c, d,
				gl = scommon.getNumericLabel,
				offset = 0, ma = s.major, mi = s.minor,
				ta = this.chart.theme.axis,
				labelGap = this.chart.theme.axis.tick.labelGap,
				// TODO: we use one font --- of major tick, we need to use major and minor fonts
				taTitleFont = o.titleFont || (ta.title && ta.title.font),
				taTitleGap = (o.titleGap==0) ? 0 : o.titleGap || (ta.title && ta.title.gap),
				taMajorTick = this.chart.theme.getTick("major", o),
				taMinorTick = this.chart.theme.getTick("minor", o),
				tsize = taTitleFont ? g.normalizedLength(g.splitFontString(taTitleFont).size) : 0,
				rotation = o.rotation % 360, leftBottom = o.leftBottom,
				cosr = Math.abs(Math.cos(rotation * Math.PI / 180)),
				sinr = Math.abs(Math.sin(rotation * Math.PI / 180));
			this.trailingSymbol = (o.trailingSymbol === undefined || o.trailingSymbol === null) ?
				this.trailingSymbol : o.trailingSymbol;
			if(typeof labelGap != "number"){
				labelGap = 4; // in pixels
			}
			if(rotation < 0){
				rotation += 360;
			}
			var maxLabelSize = this._getMaxLabelSize(); // don't need parameters, calculate has been called before => we use cached value
			if(maxLabelSize){
				var side;
				var labelWidth = Math.ceil(Math.max(maxLabelSize.majLabelW, maxLabelSize.minLabelW)) + 1,
					size = Math.ceil(Math.max(maxLabelSize.majLabelH, maxLabelSize.minLabelH)) + 1;
				if(this.vertical){
					side = leftBottom ? "l" : "r";
					switch(rotation){
						case 0:
						case 180:
							offsets[side] = labelWidth;
							offsets.t = offsets.b = size / 2;
							break;
						case 90:
						case 270:
							offsets[side] = size;
							offsets.t = offsets.b = labelWidth / 2;
							break;
						default:
							if(rotation <= centerAnchorLimit || (180 < rotation && rotation <= (180 + centerAnchorLimit))){
								offsets[side] = size * sinr / 2 + labelWidth * cosr;
								offsets[leftBottom ? "t" : "b"] = size * cosr / 2 + labelWidth * sinr;
								offsets[leftBottom ? "b" : "t"] = size * cosr / 2;
							}else if(rotation > (360 - centerAnchorLimit) || (180 > rotation && rotation > (180 - centerAnchorLimit))){
								offsets[side] = size * sinr / 2 + labelWidth * cosr;
								offsets[leftBottom ? "b" : "t"] = size * cosr / 2 + labelWidth * sinr;
								offsets[leftBottom ? "t" : "b"] = size * cosr / 2;
							}else if(rotation < 90 || (180 < rotation && rotation < 270)){
								offsets[side] = size * sinr + labelWidth * cosr;
								offsets[leftBottom ? "t" : "b"] = size * cosr + labelWidth * sinr;
							}else{
								offsets[side] = size * sinr + labelWidth * cosr;
								offsets[leftBottom ? "b" : "t"] = size * cosr + labelWidth * sinr;
							}
							break;
					}
					offsets[side] += labelGap + Math.max(taMajorTick.length, taMinorTick.length) + (o.title ? (tsize + taTitleGap) : 0);
				}else{
					side = leftBottom ? "b" : "t";
					switch(rotation){
						case 0:
						case 180:
							offsets[side] = size;
							offsets.l = offsets.r = labelWidth / 2;
							break;
						case 90:
						case 270:
							offsets[side] = labelWidth;
							offsets.l = offsets.r = size / 2;
							break;
						default:
							if((90 - centerAnchorLimit) <= rotation && rotation <= 90 || (270 - centerAnchorLimit) <= rotation && rotation <= 270){
								offsets[side] = size * cosr / 2 + labelWidth * sinr;
								offsets[leftBottom ? "r" : "l"] = size * sinr / 2 + labelWidth * cosr;
								offsets[leftBottom ? "l" : "r"] = size * sinr / 2;
							}else if(90 <= rotation && rotation <= (90 + centerAnchorLimit) || 270 <= rotation && rotation <= (270 + centerAnchorLimit)){
								offsets[side] = size * cosr / 2 + labelWidth * sinr;
								offsets[leftBottom ? "l" : "r"] = size * sinr / 2 + labelWidth * cosr;
								offsets[leftBottom ? "r" : "l"] = size * sinr / 2;
							}else if(rotation < centerAnchorLimit || (180 < rotation && rotation < (180 + centerAnchorLimit))){
								offsets[side] = size * cosr + labelWidth * sinr;
								offsets[leftBottom ? "r" : "l"] = size * sinr + labelWidth * cosr;
							}else{
								offsets[side] = size * cosr + labelWidth * sinr;
								offsets[leftBottom ? "l" : "r"] = size * sinr + labelWidth * cosr;
							}
							break;
					}
					offsets[side] += labelGap + Math.max(taMajorTick.length, taMinorTick.length) + (o.title ? (tsize + taTitleGap) : 0);
				}
			}
			return offsets;	//	Object
		},
		cleanGroup: function(creator){
			if(this.opt.enableCache && this.group){
				this._lineFreePool = this._lineFreePool.concat(this._lineUsePool);
				this._lineUsePool = [];
				this._textFreePool = this._textFreePool.concat(this._textUsePool);
				this._textUsePool = [];
			}
			this.inherited(arguments);
		},
		createText: function(labelType, creator, x, y, align, textContent, font, fontColor, labelWidth){
			if(!this.opt.enableCache || labelType=="html"){
				return acommon.createText[labelType](
						this.chart,
						creator,
						x,
						y,
						align,
						textContent,
						font,
						fontColor,
						labelWidth
					);
			}
			var text;
			if (this._textFreePool.length > 0){
				text = this._textFreePool.pop();
				text.setShape({x: x, y: y, text: textContent, align: align});
				// For now all items share the same font, no need to re-set it
				//.setFont(font).setFill(fontColor);
				// was cleared, add it back
				creator.add(text);
			}else{
				text = acommon.createText[labelType](
						this.chart,
						creator,
						x,
						y,
						align,
						textContent,
						font,
						fontColor						
					);			
			}
			this._textUsePool.push(text);
			return text;
		},
		createLine: function(creator, params){
			var line;
			if(this.opt.enableCache && this._lineFreePool.length > 0){
				line = this._lineFreePool.pop();
				line.setShape(params);
				// was cleared, add it back
				creator.add(line);
			}else{
				line = creator.createLine(params);
			}
			if(this.opt.enableCache){
				this._lineUsePool.push(line);
			}
			return line;
		},
		render: function(dim, offsets){
			// summary:
			//		Render/draw the axis.
			// dim: Object
			//		An object of the form { width, height}.
			// offsets: Object
			//		An object of the form { l, r, t, b }.
			// returns: dojox/charting/axis2d/Default
			//		The reference to the axis for functional chaining.
			if(!this.dirty || !this.scaler){
				return this;	//	dojox/charting/axis2d/Default
			}
			// prepare variable
			var o = this.opt, ta = this.chart.theme.axis, leftBottom = o.leftBottom, rotation = o.rotation % 360,
				start, stop, titlePos, titleRotation=0, titleOffset, axisVector, tickVector, anchorOffset, labelOffset, labelAlign,
				labelGap = this.chart.theme.axis.tick.labelGap,
				// TODO: we use one font --- of major tick, we need to use major and minor fonts
				taFont = o.font || (ta.majorTick && ta.majorTick.font) || (ta.tick && ta.tick.font),
				taTitleFont = o.titleFont || (ta.title && ta.title.font),
				// TODO: we use one font color --- we need to use different colors
				taFontColor = o.fontColor || (ta.majorTick && ta.majorTick.fontColor) || (ta.tick && ta.tick.fontColor) || "black",
				taTitleFontColor = o.titleFontColor || (ta.title && ta.title.fontColor) || "black",
				taTitleGap = (o.titleGap==0) ? 0 : o.titleGap || (ta.title && ta.title.gap) || 15,
				taTitleOrientation = o.titleOrientation || (ta.title && ta.title.orientation) || "axis",
				taMajorTick = this.chart.theme.getTick("major", o),
				taMinorTick = this.chart.theme.getTick("minor", o),
				taMicroTick = this.chart.theme.getTick("micro", o),

				tickSize = Math.max(taMajorTick.length, taMinorTick.length, taMicroTick.length),
				taStroke = "stroke" in o ? o.stroke : ta.stroke,
				size = taFont ? g.normalizedLength(g.splitFontString(taFont).size) : 0,
				cosr = Math.abs(Math.cos(rotation * Math.PI / 180)),
				sinr = Math.abs(Math.sin(rotation * Math.PI / 180)),
				tsize = taTitleFont ? g.normalizedLength(g.splitFontString(taTitleFont).size) : 0;
			if(typeof labelGap != "number"){
				labelGap = 4; // in pixels
			}
			if(rotation < 0){
				rotation += 360;
			}
			var cachedLabelW = this._getMaxLabelSize();
			cachedLabelW = cachedLabelW && cachedLabelW.majLabelW;
			if(this.vertical){
				start = {y: dim.height - offsets.b};
				stop  = {y: offsets.t};
				titlePos = {y: (dim.height - offsets.b + offsets.t)/2};
				titleOffset = size * sinr + (cachedLabelW || 0) * cosr + labelGap + Math.max(taMajorTick.length, taMinorTick.length) + tsize + taTitleGap;
				axisVector = {x: 0, y: -1};
				labelOffset = {x: 0, y: 0};
				tickVector = {x: 1, y: 0};
				anchorOffset = {x: labelGap, y: 0};
				switch(rotation){
					case 0:
						labelAlign = "end";
						labelOffset.y = size * 0.4;
						break;
					case 90:
						labelAlign = "middle";
						labelOffset.x = -size;
						break;
					case 180:
						labelAlign = "start";
						labelOffset.y = -size * 0.4;
						break;
					case 270:
						labelAlign = "middle";
						break;
					default:
						if(rotation < centerAnchorLimit){
							labelAlign = "end";
							labelOffset.y = size * 0.4;
						}else if(rotation < 90){
							labelAlign = "end";
							labelOffset.y = size * 0.4;
						}else if(rotation < (180 - centerAnchorLimit)){
							labelAlign = "start";
						}else if(rotation < (180 + centerAnchorLimit)){
							labelAlign = "start";
							labelOffset.y = -size * 0.4;
						}else if(rotation < 270){
							labelAlign = "start";
							labelOffset.x = leftBottom ? 0 : size * 0.4;
						}else if(rotation < (360 - centerAnchorLimit)){
							labelAlign = "end";
							labelOffset.x = leftBottom ? 0 : size * 0.4;
						}else{
							labelAlign = "end";
							labelOffset.y = size * 0.4;
						}
				}
				if(leftBottom){
					start.x = stop.x = offsets.l;
					titleRotation = (taTitleOrientation && taTitleOrientation == "away") ? 90 : 270;
					titlePos.x = offsets.l - titleOffset + (titleRotation == 270 ? tsize : 0);
					tickVector.x = -1;
					anchorOffset.x = -anchorOffset.x;
				}else{
					start.x = stop.x = dim.width - offsets.r;
					titleRotation = (taTitleOrientation && taTitleOrientation == "axis") ? 90 : 270;
					titlePos.x = dim.width - offsets.r + titleOffset - (titleRotation == 270 ? 0 : tsize);
					switch(labelAlign){
						case "start":
							labelAlign = "end";
							break;
						case "end":
							labelAlign = "start";
							break;
						case "middle":
							labelOffset.x += size;
							break;
					}
				}
			}else{
				start = {x: offsets.l};
				stop  = {x: dim.width - offsets.r};
				titlePos = {x: (dim.width - offsets.r + offsets.l)/2};
				titleOffset = size * cosr + (cachedLabelW || 0) * sinr + labelGap + Math.max(taMajorTick.length, taMinorTick.length) + tsize + taTitleGap;
				axisVector = {x: 1, y: 0};
				labelOffset = {x: 0, y: 0};
				tickVector = {x: 0, y: 1};
				anchorOffset = {x: 0, y: labelGap};
				switch(rotation){
					case 0:
						labelAlign = "middle";
						labelOffset.y = size;
						break;
					case 90:
						labelAlign = "start";
						labelOffset.x = -size * 0.4;
						break;
					case 180:
						labelAlign = "middle";
						break;
					case 270:
						labelAlign = "end";
						labelOffset.x = size * 0.4;
						break;
					default:
						if(rotation < (90 - centerAnchorLimit)){
							labelAlign = "start";
							labelOffset.y = leftBottom ? size : 0;
						}else if(rotation < (90 + centerAnchorLimit)){
							labelAlign = "start";
							labelOffset.x = -size * 0.4;
						}else if(rotation < 180){
							labelAlign = "start";
							labelOffset.y = leftBottom ? 0 : -size;
						}else if(rotation < (270 - centerAnchorLimit)){
							labelAlign = "end";
							labelOffset.y = leftBottom ? 0 : -size;
						}else if(rotation < (270 + centerAnchorLimit)){
							labelAlign = "end";
							labelOffset.y = leftBottom ? size * 0.4 : 0;
						}else{
							labelAlign = "end";
							labelOffset.y = leftBottom ? size : 0;
						}
				}
				if(leftBottom){
					start.y = stop.y = dim.height - offsets.b;
					titleRotation = (taTitleOrientation && taTitleOrientation == "axis") ? 180 : 0;
					titlePos.y = dim.height - offsets.b + titleOffset - (titleRotation ? tsize : 0);
				}else{
					start.y = stop.y = offsets.t;
					titleRotation = (taTitleOrientation && taTitleOrientation == "away") ? 180 : 0;
					titlePos.y = offsets.t - titleOffset + (titleRotation ? 0 : tsize);
					tickVector.y = -1;
					anchorOffset.y = -anchorOffset.y;
					switch(labelAlign){
						case "start":
							labelAlign = "end";
							break;
						case "end":
							labelAlign = "start";
							break;
						case "middle":
							labelOffset.y -= size;
							break;
					}
				}
			}

			// render shapes

			this.cleanGroup();

			var s = this.group,
				c = this.scaler,
				t = this.ticks,
				f = lin.getTransformerFromModel(this.scaler),
				// GFX Canvas now supports labels, so let's _not_ fallback to HTML anymore on canvas, just use
				// HTML labels if explicitly asked + no rotation + no IE + no Opera
				labelType = (!o.title || !titleRotation) && !rotation && this.opt.htmlLabels && !has("ie") && !has("opera") ? "html" : "gfx",
				dx = tickVector.x * taMajorTick.length,
				dy = tickVector.y * taMajorTick.length,
				skip = this._skipInterval;

			s.createLine({
				x1: start.x,
				y1: start.y,
				x2: stop.x,
				y2: stop.y
			}).setStroke(taStroke);

			//create axis title
			if(o.title){
				var axisTitle = acommon.createText[labelType](
					this.chart,
					s,
					titlePos.x,
					titlePos.y,
					"middle",
					o.title,
					taTitleFont,
					taTitleFontColor
				);
				if(labelType == "html"){
					this.htmlElements.push(axisTitle);
				}else{
					//as soon as rotation is provided, labelType won't be "html"
					//rotate gfx labels
					axisTitle.setTransform(g.matrix.rotategAt(titleRotation, titlePos.x, titlePos.y));
				}
			}

			// go out nicely instead of try/catch
			if(t == null){
				this.dirty = false;
				return this;
			}

			var rel = (t.major.length > 0)?(t.major[0].value - this._majorStart) / c.major.tick:0;
			var canLabel = this.opt.majorLabels;
			arr.forEach(t.major, function(tick, i){
				var offset = f(tick.value), elem,
					x = start.x + axisVector.x * offset,
					y = start.y + axisVector.y * offset;
				i += rel;
				this.createLine(s, {
					x1: x, y1: y,
					x2: x + dx,
					y2: y + dy
				}).setStroke(taMajorTick);
				if(tick.label && (!skip || (i - (1 + skip)) % (1 + skip) == 0)){
					var label = o.maxLabelCharCount ? this.getTextWithLimitCharCount(tick.label, taFont, o.maxLabelCharCount) : {
						text: tick.label,
						truncated: false
					};
					label = o.maxLabelSize ? this.getTextWithLimitLength(label.text, taFont, o.maxLabelSize, label.truncated) : label;
					elem = this.createText(labelType,
						s,
						x + dx + anchorOffset.x + (rotation ? 0 : labelOffset.x),
						y + dy + anchorOffset.y + (rotation ? 0 : labelOffset.y),
						labelAlign,
						label.text,
						taFont,
						taFontColor
						//cachedLabelW
					);
					// if bidi support was required, the textDir is "auto" and truncation
					// took place, we need to update the dir of the element for cases as:
					// Fool label: 111111W (W for bidi character)
					// truncated label: 11...
					// in this case for auto textDir the dir will be "ltr" which is wrong.
					if(this.chart.truncateBidi  && label.truncated){
						this.chart.truncateBidi(elem, tick.label, labelType);
					}
					label.truncated && this.labelTooltip(elem, this.chart, tick.label, label.text, taFont, labelType);
					if(labelType == "html"){
						this.htmlElements.push(elem);
					}else if(rotation){
						elem.setTransform([
							{dx: labelOffset.x, dy: labelOffset.y},
							g.matrix.rotategAt(
								rotation,
								x + dx + anchorOffset.x,
								y + dy + anchorOffset.y
							)
						]);
					}
				}
			}, this);

			dx = tickVector.x * taMinorTick.length;
			dy = tickVector.y * taMinorTick.length;
			canLabel = this.opt.minorLabels && c.minMinorStep <= c.minor.tick * c.bounds.scale;
			arr.forEach(t.minor, function(tick){
				var offset = f(tick.value), elem,
					x = start.x + axisVector.x * offset,
					y = start.y + axisVector.y * offset;
				this.createLine(s, {
					x1: x, y1: y,
					x2: x + dx,
					y2: y + dy
				}).setStroke(taMinorTick);
				if(canLabel && tick.label){
					var label = o.maxLabelCharCount ? this.getTextWithLimitCharCount(tick.label, taFont, o.maxLabelCharCount) : {
						text: tick.label,
						truncated: false
					};
					label = o.maxLabelSize ? this.getTextWithLimitLength(label.text, taFont, o.maxLabelSize, label.truncated) : label;
					elem = this.createText(labelType,
						s,
						x + dx + anchorOffset.x + (rotation ? 0 : labelOffset.x),
						y + dy + anchorOffset.y + (rotation ? 0 : labelOffset.y),
						labelAlign,
						label.text,
						taFont,
						taFontColor
						//cachedLabelW
					);
					// if bidi support was required, the textDir is "auto" and truncation
					// took place, we need to update the dir of the element for cases as:
					// Fool label: 111111W (W for bidi character)
					// truncated label: 11...
					// in this case for auto textDir the dir will be "ltr" which is wrong.
					if(this.chart.getTextDir && label.truncated){
						this.chart.truncateBidi(elem, tick.label, labelType);
					}
					label.truncated && this.labelTooltip(elem, this.chart, tick.label, label.text, taFont, labelType);
					if(labelType == "html"){
						this.htmlElements.push(elem);
					}else if(rotation){
						elem.setTransform([
							{dx: labelOffset.x, dy: labelOffset.y},
							g.matrix.rotategAt(
								rotation,
								x + dx + anchorOffset.x,
								y + dy + anchorOffset.y
							)
						]);
					}
				}
			}, this);

			dx = tickVector.x * taMicroTick.length;
			dy = tickVector.y * taMicroTick.length;
			arr.forEach(t.micro, function(tick){
				var offset = f(tick.value), elem,
					x = start.x + axisVector.x * offset,
					y = start.y + axisVector.y * offset;
					this.createLine(s, {
						x1: x, y1: y,
						x2: x + dx,
						y2: y + dy
					}).setStroke(taMicroTick);
			}, this);

			this.dirty = false;
			return this;	//	dojox/charting/axis2d/Default
		},
		labelTooltip: function(elem, chart, label, truncatedLabel, font, elemType){
			var modules = ["dijit/Tooltip"];
			var aroundRect = {type: "rect"}, position = ["above", "below"],
				fontWidth = g._base._getTextBox(truncatedLabel, {font: font}).w || 0,
				fontHeight = font ? g.normalizedLength(g.splitFontString(font).size) : 0;
			if(elemType == "html"){
				lang.mixin(aroundRect, domGeom.position(elem.firstChild, true));
				aroundRect.width = Math.ceil(fontWidth);
				aroundRect.height = Math.ceil(fontHeight);
				this._events.push({
					shape:  dojo,
					handle: connect.connect(elem.firstChild, "onmouseover", this, function(e){
						require(modules, function(Tooltip){
							Tooltip.show(label, aroundRect, position);
						});
					})
				});
				this._events.push({
					shape:  dojo,
					handle: connect.connect(elem.firstChild, "onmouseout", this, function(e){
						require(modules, function(Tooltip){
							Tooltip.hide(aroundRect);
						});
					})
				});
			}else{
				var shp = elem.getShape(),
					lt = chart.getCoords();
				aroundRect = lang.mixin(aroundRect, {
					x: shp.x - fontWidth / 2,
					y: shp.y
				});
				aroundRect.x += lt.x;
				aroundRect.y += lt.y;
				aroundRect.x = Math.round(aroundRect.x);
				aroundRect.y = Math.round(aroundRect.y);
				aroundRect.width = Math.ceil(fontWidth);
				aroundRect.height = Math.ceil(fontHeight);
				this._events.push({
					shape:  elem,
					handle: elem.connect("onmouseenter", this, function(e){
						require(modules, function(Tooltip){
							Tooltip.show(label, aroundRect, position);
						});
					})
				});
				this._events.push({
					shape:  elem,
					handle: elem.connect("onmouseleave", this, function(e){
						require(modules, function(Tooltip){
							Tooltip.hide(aroundRect);
						});
					})
				});
			}
		}
	});
});

},
'dojox/charting/axis2d/Invisible':function(){
define("dojox/charting/axis2d/Invisible", ["dojo/_base/lang", "dojo/_base/declare", "./Base", "../scaler/linear",
	"dojox/gfx", "dojox/lang/utils"],
	function(lang, declare, Base, lin, g, du){

/*=====
	var __InvisibleAxisCtorArgs = {
		// summary:
		//		Optional arguments used in the definition of an invisible axis.
		// vertical: Boolean?
		//		A flag that says whether an axis is vertical (i.e. y axis) or horizontal. Default is false (horizontal).
		// fixUpper: String?
		//		Align the greatest value on the axis with the specified tick level. Options are "major", "minor", "micro", or "none".  Defaults to "none".
		// fixLower: String?
		//		Align the smallest value on the axis with the specified tick level. Options are "major", "minor", "micro", or "none".  Defaults to "none".
		// natural: Boolean?
		//		Ensure tick marks are made on "natural" numbers. Defaults to false.
		// leftBottom: Boolean?
		//		The position of a vertical axis; if true, will be placed against the left-bottom corner of the chart.  Defaults to true.
		// includeZero: Boolean?
		//		Include 0 on the axis rendering.  Default is false.
		// fixed: Boolean?
		//		Force all axis labels to be fixed numbers.  Default is true.
		// min: Number?
		//		The smallest value on an axis. Default is 0.
		// max: Number?
		//		The largest value on an axis. Default is 1.
		// from: Number?
		//		Force the chart to render data visible from this value. Default is 0.
		// to: Number?
		//		Force the chart to render data visible to this value. Default is 1.
		// majorTickStep: Number?
		//		The amount to skip before a major tick is drawn. When not set the major ticks step is computed from
		//		the data range.
		// minorTickStep: Number?
		//		The amount to skip before a minor tick is drawn. When not set the minor ticks step is computed from
		//		the data range.
		// microTickStep: Number?
		//		The amount to skip before a micro tick is drawn. When not set the micro ticks step is computed from
	};
=====*/

	return declare("dojox.charting.axis2d.Invisible", Base, {
		// summary:
		//		A axis object used in dojox.charting.  You can use that axis if you want the axis to be invisible.
		//		See dojox.charting.Chart.addAxis for details.
		//
		// defaultParams: Object
		//		The default parameters used to define any axis.
		// optionalParams: Object
		//		Any optional parameters needed to define an axis.

		/*
		// TODO: the documentation tools need these to be pre-defined in order to pick them up
		//	correctly, but the code here is partially predicated on whether or not the properties
		//	actually exist.  For now, we will leave these undocumented but in the code for later. -- TRT

		// opt: Object
		//		The actual options used to define this axis, created at initialization.
		// scaler: Object
		//		The calculated helper object to tell charts how to draw an axis and any data.
		// ticks: Object
		//		The calculated tick object that helps a chart draw the scaling on an axis.
		// dirty: Boolean
		//		The state of the axis (whether it needs to be redrawn or not)
		// scale: Number
		//		The current scale of the axis.
		// offset: Number
		//		The current offset of the axis.

		opt: null,
		scaler: null,
		ticks: null,
		dirty: true,
		scale: 1,
		offset: 0,
		*/
		defaultParams: {
			vertical:    false,		// true for vertical axis
			fixUpper:    "none",	// align the upper on ticks: "major", "minor", "micro", "none"
			fixLower:    "none",	// align the lower on ticks: "major", "minor", "micro", "none"
			natural:     false,		// all tick marks should be made on natural numbers
			leftBottom:  true,		// position of the axis, used with "vertical"
			includeZero: false,		// 0 should be included
			fixed:       true		// all labels are fixed numbers
		},
		optionalParams: {
			min:			0,	// minimal value on this axis
			max:			1,	// maximal value on this axis
			from:			0,	// visible from this value
			to:				1,	// visible to this value
			majorTickStep:	4,	// major tick step
			minorTickStep:	2,	// minor tick step
			microTickStep:	1	// micro tick step
		},

		constructor: function(chart, kwArgs){
			// summary:
			//		The constructor for an invisible axis.
			// chart: dojox/charting/Chart
			//		The chart the axis belongs to.
			// kwArgs: __InvisibleAxisCtorArgs?
			//		Any optional keyword arguments to be used to define this axis.
			this.opt = lang.clone(this.defaultParams);
            du.updateWithObject(this.opt, kwArgs);
			du.updateWithPattern(this.opt, kwArgs, this.optionalParams);
		},
		dependOnData: function(){
			// summary:
			//		Find out whether or not the axis options depend on the data in the axis.
			return !("min" in this.opt) || !("max" in this.opt);	//	Boolean
		},
		clear: function(){
			// summary:
			//		Clear out all calculated properties on this axis;
			// returns: dojox/charting/axis2d/Invisible
			//		The reference to the axis for functional chaining.
			delete this.scaler;
			delete this.ticks;
			this.dirty = true;
			return this;	//	dojox/charting/axis2d/Invisible
		},
		initialized: function(){
			// summary:
			//		Finds out if this axis has been initialized or not.
			// returns: Boolean
			//		Whether a scaler has been calculated and if the axis is not dirty.
			return "scaler" in this && !(this.dirty && this.dependOnData());
		},
		setWindow: function(scale, offset){
			// summary:
			//		Set the drawing "window" for the axis.
			// scale: Number
			//		The new scale for the axis.
			// offset: Number
			//		The new offset for the axis.
			// returns: dojox/charting/axis2d/Invisible
			//		The reference to the axis for functional chaining.
			this.scale  = scale;
			this.offset = offset;
			return this.clear();	//	dojox/charting/axis2d/Invisible
		},
		getWindowScale: function(){
			// summary:
			//		Get the current windowing scale of the axis.
			return "scale" in this ? this.scale : 1;	//	Number
		},
		getWindowOffset: function(){
			// summary:
			//		Get the current windowing offset for the axis.
			return "offset" in this ? this.offset : 0;	//	Number
		},
		calculate: function(min, max, span){
			// summary:
			//		Perform all calculations needed to render this axis.
			// min: Number
			//		The smallest value represented on this axis.
			// max: Number
			//		The largest value represented on this axis.
			// span: Number
			//		The span in pixels over which axis calculations are made.
			// returns: dojox/charting/axis2d/Invisible
			//		The reference to the axis for functional chaining.
			if(this.initialized()){
				return this;
			}
			var o = this.opt;
			// we used to have a 4th function parameter to reach labels but
			// nobody was calling it with 4 parameters.
			this.labels = o.labels;
			this.scaler = lin.buildScaler(min, max, span, o);
			// store the absolute major tick start, this will be useful when dropping a label every n labels
			// TODO: if o.lower then it does not work
			var tsb = this.scaler.bounds;
			if("scale" in this){
				// calculate new range
				o.from = tsb.lower + this.offset;
				o.to   = (tsb.upper - tsb.lower) / this.scale + o.from;
				// make sure that bounds are correct
				if( !isFinite(o.from) ||
					isNaN(o.from) ||
					!isFinite(o.to) ||
					isNaN(o.to) ||
					o.to - o.from >= tsb.upper - tsb.lower
				){
					// any error --- remove from/to bounds
					delete o.from;
					delete o.to;
					delete this.scale;
					delete this.offset;
				}else{
					// shift the window, if we are out of bounds
					if(o.from < tsb.lower){
						o.to += tsb.lower - o.from;
						o.from = tsb.lower;
					}else if(o.to > tsb.upper){
						o.from += tsb.upper - o.to;
						o.to = tsb.upper;
					}
					// update the offset
					this.offset = o.from - tsb.lower;
				}
				// re-calculate the scaler
				this.scaler = lin.buildScaler(min, max, span, o);
				tsb = this.scaler.bounds;
				// cleanup
				if(this.scale == 1 && this.offset == 0){
					delete this.scale;
					delete this.offset;
				}
			}
			return this;	//	dojox/charting/axis2d/Invisible
		},
		getScaler: function(){
			// summary:
			//		Get the pre-calculated scaler object.
			return this.scaler;	//	Object
		},
		getTicks: function(){
			// summary:
			//		Get the pre-calculated ticks object.
			return this.ticks;	//	Object
		}
	});
});

},
'dojox/charting/axis2d/Base':function(){
define("dojox/charting/axis2d/Base", ["dojo/_base/declare", "../Element"],
	function(declare, Element){
	/*=====
	var __BaseAxisCtorArgs = {
		// summary:
		//		Optional arguments used in the definition of an invisible axis.
		// vertical: Boolean?
		//		A flag that says whether an axis is vertical (i.e. y axis) or horizontal. Default is false (horizontal).
		// min: Number?
		//		The smallest value on an axis. Default is 0.
		// max: Number?
		//		The largest value on an axis. Default is 1.
	};
	=====*/
	return declare("dojox.charting.axis2d.Base", Element, {
		// summary:
		//		The base class for any axis.  This is more of an interface/API
		//		definition than anything else; see dojox.charting.axis2d.Default
		//		for more details.
		constructor: function(chart, kwArgs){
			// summary:
			//		Return a new base axis.
			// chart: dojox/charting/Chart
			//		The chart this axis belongs to.
			// kwArgs: __BaseAxisCtorArgs?
			//		An optional arguments object to define the axis parameters.
			this.vertical = kwArgs && kwArgs.vertical;
			this.opt = {};
			this.opt.min = kwArgs && kwArgs.min;
			this.opt.max = kwArgs && kwArgs.max;
		},
		clear: function(){
			// summary:
			//		Stub function for clearing the axis.
			// returns: dojox/charting/axis2d/Base
			//		A reference to the axis for functional chaining.
			return this;	//	dojox/charting/axis2d/Base
		},
		initialized: function(){
			// summary:
			//		Return a flag as to whether or not this axis has been initialized.
			// returns: Boolean
			//		If the axis is initialized or not.
			return false;	//	Boolean
		},
		calculate: function(min, max, span){
			// summary:
			//		Stub function to run the calcuations needed for drawing this axis.
			// returns: dojox/charting/axis2d/Base
			//		A reference to the axis for functional chaining.
			return this;	//	dojox/charting/axis2d/Base
		},
		getScaler: function(){
			// summary:
			//		A stub function to return the scaler object created during calculate.
			// returns: Object
			//		The scaler object (see dojox.charting.scaler.linear for more information)
			return null;	//	Object
		},
		getTicks: function(){
			// summary:
			//		A stub function to return the object that helps define how ticks are rendered.
			// returns: Object
			//		The ticks object.
			return null;	//	Object
		},
		getOffsets: function(){
			// summary:
			//		A stub function to return any offsets needed for axis and series rendering.
			// returns: Object
			//		An object of the form { l, r, t, b }.
			return {l: 0, r: 0, t: 0, b: 0};	//	Object
		},
		render: function(dim, offsets){
			// summary:
			//		Stub function to render this axis.
			// returns: dojox/charting/axis2d/Base
			//		A reference to the axis for functional chaining.
			this.dirty = false;
			return this;	//	dojox/charting/axis2d/Base
		}
	});
});

},
'dojox/charting/scaler/linear':function(){
define("dojox/charting/scaler/linear", ["dojo/_base/lang", "./common"], 
	function(lang, common){
	var linear = lang.getObject("dojox.charting.scaler.linear", true);
	
	var deltaLimit = 3,	// pixels
		getLabel = common.getNumericLabel;

		function findString(/*String*/ val, /*Array*/ text){
			val = val.toLowerCase();
			for(var i = text.length - 1; i >= 0; --i){
				if(val === text[i]){
					return true;
				}
			}
			return false;
		}
	
	var calcTicks = function(min, max, kwArgs, majorTick, minorTick, microTick, span){
		kwArgs = lang.delegate(kwArgs);
		if(!majorTick){
			if(kwArgs.fixUpper == "major"){ kwArgs.fixUpper = "minor"; }
			if(kwArgs.fixLower == "major"){ kwArgs.fixLower = "minor"; }
		}
		if(!minorTick){
			if(kwArgs.fixUpper == "minor"){ kwArgs.fixUpper = "micro"; }
			if(kwArgs.fixLower == "minor"){ kwArgs.fixLower = "micro"; }
		}
		if(!microTick){
			if(kwArgs.fixUpper == "micro"){ kwArgs.fixUpper = "none"; }
			if(kwArgs.fixLower == "micro"){ kwArgs.fixLower = "none"; }
		}
		var lowerBound = findString(kwArgs.fixLower, ["major"]) ?
				Math.floor(kwArgs.min / majorTick) * majorTick :
					findString(kwArgs.fixLower, ["minor"]) ?
						Math.floor(kwArgs.min / minorTick) * minorTick :
							findString(kwArgs.fixLower, ["micro"]) ?
								Math.floor(kwArgs.min / microTick) * microTick : kwArgs.min,
			upperBound = findString(kwArgs.fixUpper, ["major"]) ?
				Math.ceil(kwArgs.max / majorTick) * majorTick :
					findString(kwArgs.fixUpper, ["minor"]) ?
						Math.ceil(kwArgs.max / minorTick) * minorTick :
							findString(kwArgs.fixUpper, ["micro"]) ?
								Math.ceil(kwArgs.max / microTick) * microTick : kwArgs.max;
								
		if(kwArgs.useMin){ min = lowerBound; }
		if(kwArgs.useMax){ max = upperBound; }
		
		var majorStart = (!majorTick || kwArgs.useMin && findString(kwArgs.fixLower, ["major"])) ?
				min : Math.ceil(min / majorTick) * majorTick,
			minorStart = (!minorTick || kwArgs.useMin && findString(kwArgs.fixLower, ["major", "minor"])) ?
				min : Math.ceil(min / minorTick) * minorTick,
			microStart = (! microTick || kwArgs.useMin && findString(kwArgs.fixLower, ["major", "minor", "micro"])) ?
				min : Math.ceil(min / microTick) * microTick,
			majorCount = !majorTick ? 0 : (kwArgs.useMax && findString(kwArgs.fixUpper, ["major"]) ?
				Math.round((max - majorStart) / majorTick) :
				Math.floor((max - majorStart) / majorTick)) + 1,
			minorCount = !minorTick ? 0 : (kwArgs.useMax && findString(kwArgs.fixUpper, ["major", "minor"]) ?
				Math.round((max - minorStart) / minorTick) :
				Math.floor((max - minorStart) / minorTick)) + 1,
			microCount = !microTick ? 0 : (kwArgs.useMax && findString(kwArgs.fixUpper, ["major", "minor", "micro"]) ?
				Math.round((max - microStart) / microTick) :
				Math.floor((max - microStart) / microTick)) + 1,
			minorPerMajor  = minorTick ? Math.round(majorTick / minorTick) : 0,
			microPerMinor  = microTick ? Math.round(minorTick / microTick) : 0,
			majorPrecision = majorTick ? Math.floor(Math.log(majorTick) / Math.LN10) : 0,
			minorPrecision = minorTick ? Math.floor(Math.log(minorTick) / Math.LN10) : 0,
			scale = span / (max - min);
		if(!isFinite(scale)){ scale = 1; }
		
		return {
			bounds: {
				lower:	lowerBound,
				upper:	upperBound,
				from:	min,
				to:		max,
				scale:	scale,
				span:	span
			},
			major: {
				tick:	majorTick,
				start:	majorStart,
				count:	majorCount,
				prec:	majorPrecision
			},
			minor: {
				tick:	minorTick,
				start:	minorStart,
				count:	minorCount,
				prec:	minorPrecision
			},
			micro: {
				tick:	microTick,
				start:	microStart,
				count:	microCount,
				prec:	0
			},
			minorPerMajor:	minorPerMajor,
			microPerMinor:	microPerMinor,
			scaler:			linear
		};
	};
	
	return lang.mixin(linear, {
		buildScaler: function(/*Number*/ min, /*Number*/ max, /*Number*/ span, /*Object*/ kwArgs, /*Number?*/ delta, /*Number?*/ minorDelta){
			var h = {fixUpper: "none", fixLower: "none", natural: false};
			if(kwArgs){
				if("fixUpper" in kwArgs){ h.fixUpper = String(kwArgs.fixUpper); }
				if("fixLower" in kwArgs){ h.fixLower = String(kwArgs.fixLower); }
				if("natural"  in kwArgs){ h.natural  = Boolean(kwArgs.natural); }
			}
			minorDelta = !minorDelta || minorDelta < deltaLimit ? deltaLimit : minorDelta;
			
			// update bounds
			if("min" in kwArgs){ min = kwArgs.min; }
			if("max" in kwArgs){ max = kwArgs.max; }
			if(kwArgs.includeZero){
				if(min > 0){ min = 0; }
				if(max < 0){ max = 0; }
			}
			h.min = min;
			h.useMin = true;
			h.max = max;
			h.useMax = true;
			
			if("from" in kwArgs){
				min = kwArgs.from;
				h.useMin = false;
			}
			if("to" in kwArgs){
				max = kwArgs.to;
				h.useMax = false;
			}
			
			// check for erroneous condition
			if(max <= min){
				return calcTicks(min, max, h, 0, 0, 0, span);	// Object
			}
			if(!delta){
				delta = max - min;
			}
			var mag = Math.floor(Math.log(delta) / Math.LN10),
				major = kwArgs && ("majorTickStep" in kwArgs) ? kwArgs.majorTickStep : Math.pow(10, mag),
				minor = 0, micro = 0, ticks;
				
			// calculate minor ticks
			if(kwArgs && ("minorTickStep" in kwArgs)){
				minor = kwArgs.minorTickStep;
			}else{
				do{
					minor = major / 10;
					if(!h.natural || minor > 0.9){
						ticks = calcTicks(min, max, h, major, minor, 0, span);
						if(ticks.bounds.scale * ticks.minor.tick > minorDelta){ break; }
					}
					minor = major / 5;
					if(!h.natural || minor > 0.9){
						ticks = calcTicks(min, max, h, major, minor, 0, span);
						if(ticks.bounds.scale * ticks.minor.tick > minorDelta){ break; }
					}
					minor = major / 2;
					if(!h.natural || minor > 0.9){
						ticks = calcTicks(min, max, h, major, minor, 0, span);
						if(ticks.bounds.scale * ticks.minor.tick > minorDelta){ break; }
					}
					return calcTicks(min, max, h, major, 0, 0, span);	// Object
				}while(false);
			}
	
			// calculate micro ticks
			if(kwArgs && ("microTickStep" in kwArgs)){
				micro = kwArgs.microTickStep;
				ticks = calcTicks(min, max, h, major, minor, micro, span);
			}else{
				do{
					micro = minor / 10;
					if(!h.natural || micro > 0.9){
						ticks = calcTicks(min, max, h, major, minor, micro, span);
						if(ticks.bounds.scale * ticks.micro.tick > deltaLimit){ break; }
					}
					micro = minor / 5;
					if(!h.natural || micro > 0.9){
						ticks = calcTicks(min, max, h, major, minor, micro, span);
						if(ticks.bounds.scale * ticks.micro.tick > deltaLimit){ break; }
					}
					micro = minor / 2;
					if(!h.natural || micro > 0.9){
						ticks = calcTicks(min, max, h, major, minor, micro, span);
						if(ticks.bounds.scale * ticks.micro.tick > deltaLimit){ break; }
					}
					micro = 0;
				}while(false);
			}
	
			return micro ? ticks : calcTicks(min, max, h, major, minor, 0, span);	// Object
		},
		buildTicks: function(/*Object*/ scaler, /*Object*/ kwArgs){
			var step, next, tick,
				nextMajor = scaler.major.start,
				nextMinor = scaler.minor.start,
				nextMicro = scaler.micro.start;
			if(kwArgs.microTicks && scaler.micro.tick){
				step = scaler.micro.tick, next = nextMicro;
			}else if(kwArgs.minorTicks && scaler.minor.tick){
				step = scaler.minor.tick, next = nextMinor;
			}else if(scaler.major.tick){
				step = scaler.major.tick, next = nextMajor;
			}else{
				// no ticks
				return null;
			}
			// make sure that we have finite bounds
			var revScale = 1 / scaler.bounds.scale;
			if(scaler.bounds.to <= scaler.bounds.from || isNaN(revScale) || !isFinite(revScale) ||
					step <= 0 || isNaN(step) || !isFinite(step)){
				// no ticks
				return null;
			}
			// loop over all ticks
			var majorTicks = [], minorTicks = [], microTicks = [];
			while(next <= scaler.bounds.to + revScale){
				if(Math.abs(nextMajor - next) < step / 2){
					// major tick
					tick = {value: nextMajor};
					if(kwArgs.majorLabels){
						tick.label = getLabel(nextMajor, scaler.major.prec, kwArgs);
					}
					majorTicks.push(tick);
					nextMajor += scaler.major.tick;
					nextMinor += scaler.minor.tick;
					nextMicro += scaler.micro.tick;
				}else if(Math.abs(nextMinor - next) < step / 2){
					// minor tick
					if(kwArgs.minorTicks){
						tick = {value: nextMinor};
						if(kwArgs.minorLabels && (scaler.minMinorStep <= scaler.minor.tick * scaler.bounds.scale)){
							tick.label = getLabel(nextMinor, scaler.minor.prec, kwArgs);
						}
						minorTicks.push(tick);
					}
					nextMinor += scaler.minor.tick;
					nextMicro += scaler.micro.tick;
				}else{
					// micro tick
					if(kwArgs.microTicks){
						microTicks.push({value: nextMicro});
					}
					nextMicro += scaler.micro.tick;
				}
				next += step;
			}
			return {major: majorTicks, minor: minorTicks, micro: microTicks};	// Object
		},
		getTransformerFromModel: function(/*Object*/ scaler){
			var offset = scaler.bounds.from, scale = scaler.bounds.scale;
			return function(x){ return (x - offset) * scale; };	// Function
		},
		getTransformerFromPlot: function(/*Object*/ scaler){
			var offset = scaler.bounds.from, scale = scaler.bounds.scale;
			return function(x){ return x / scale + offset; };	// Function
		}
	});
});

},
'epi-find/manage/DashboardModel':function(){
﻿define([
    "dojo/_base/declare",
    "dojo/_base/config",
    "dojo/Stateful",
    "dijit/Destroyable"
],
function (
    declare,
    config,
    Stateful,
    Destroyable
    ) {

    return declare([Stateful, Destroyable], {
        // summary:
        //      Statistics model.

        topQueryStore: null,
        queryStore: null,
        topHitStore: null,
        nullQueryStore: null,

        constructor: function () {
            this.queryStore = config.dependencies["epi-find.QueryStore"];
            this.topQueryStore = config.dependencies["epi-find.TopQueryStore"];
            this.topHitStore = config.dependencies["epi-find.TopHitStore"];
            this.nullQueryStore = config.dependencies["epi-find.NullQueryStore"];
        },

        init: function () {
        }
    });
});

},
'epi-find/optimize/OptimizationController':function(){
﻿define([
    "dojo/_base/declare",

    "../_ControllerBase",
    "./OptimizationModel",
    "../widget/Optimization",
    "../widget/_ActionableMixin"
],
function(declare,
    _ControllerBase,
    OptimizationModel, Optimization, _ActionableMixin
) {
    return declare([_ControllerBase, _ActionableMixin], {
        // summary:
        //      Controller for Optimization views.

        model: null,
        widget: null,

        startup: function() {
            if (this.isStarted()) {
                return;
            }
            this.inherited(arguments);
            this.model = new OptimizationModel();
            this.model.init();

            this.widget = new Optimization({
                model: this.model
            });

            this.own(this.widget);
        },

        show: function() {
            this._setupView([this.widget]);
        },

        takeAction: function(actions, params) {
            this.widget.takeAction(actions, params);
            this.currentAction = this.widget.currentAction;
            return this.widget;
        }
    });
});
},
'epi-find/optimize/OptimizationModel':function(){
﻿define([
    "dojo/_base/declare",
    "dojo/_base/config",
    "dojo/Stateful",

    "dojo/store/Observable",
    "./RelatedQueriesModel",
    "./AutocompleteModel",

    "dijit/Destroyable"
],
function (
    declare,
    config,
    Stateful,
    Observable,
    RelatedQueriesModel,
    AutocompleteModel,
    Destroyable
    ) {

    return declare([Stateful, Destroyable], {
        // summary:
        //      Optimization model.

        relatedQueriesStore: null,
        autocompleteStore: null,

        relatedQueriesModel: null,
        autocompleteModel: null,

        constructor: function () {
            this.relatedQueriesStore = config.dependencies["epi-find.RelatedQueries"];
            this.autocompleteStore = config.dependencies["epi-find.Autocomplete"];
        },

        init: function () {
            this.relatedQueriesModel = new RelatedQueriesModel({
                store: this.relatedQueriesStore
            });

            this.autocompleteModel = new AutocompleteModel({
                store: new Observable(this.autocompleteStore)
            });
        }
    });
});

},
'epi-find/optimize/RelatedQueriesModel':function(){
define([
    "dojo/_base/declare",
    "dojo/_base/array",
    "dojo/_base/lang",

    "dojo/promise/all",
    "dojo/when",

    "dojo/Stateful",
    "dijit/Destroyable",

    "dojox/mvc/EditStoreRefController",
    "epi-saas-base/mvc/_CommitReturnMixin",
    "epi-saas-base/mvc/_EmptiableRefControllerMixin"
],
    function(declare, array, lang,
             all, when,
             Stateful, Destroyable,
             EditStoreRefController, _CommitReturnMixin, _EmptiableRefControllerMixin) {

        // module:
        //      epi-find/manage/RelatedQueriesModel

        return declare([Stateful, Destroyable], {
            // summary:
            //     Model for the Related Queries view.

            // store: dojo/store/JsonRest
            //      A store that is used for saving data
            store: null,

            // ctrl: dojox/mvc/EditStoreRefController
            //      A controller for binding
            ctrl: null,

            // searchText: String
            //      A search query
            searchText: null,
            _searchTextSetter: function(value) {
                if (value) {
                    this.set("gridQuery", {q: value + "*"});
                }
                else {
                    this.set("gridQuery", {});
                }
            },

            // gridQuery: Object
            //      Query for the grid
            gridQuery: null,

            postscript: function() {
                this.inherited(arguments);
                var BindingController = declare([EditStoreRefController, _CommitReturnMixin, _EmptiableRefControllerMixin]);
                this.ctrl = new BindingController({
                    store: this.store,
                    emptyModel: new Stateful({priority:1})
                });
                this.gridQuery = {};
            },

            moveUp: function(data) {
                // summary:
                //      Places specified suggestion above the previous suggestion in
                //      a sorted by priority list of suggestions that belongs to the same query.
                // data: Object
                //      A suggestion entity that is going to be moved up
                // returns: dojo/promise/Promise
                var suggestionId = this._findPreviousSuggestion(data);
                if (suggestionId === null) {
                    return null;
                }
                return this._swapSuggestions(data, suggestionId);
            },


            moveDown: function(data) {
                // summary:
                //      Places specified suggestion below the next suggestion in
                //      a sorted by priority list of suggestions that belongs to the same query.
                // data: Object
                //      A suggestion entity that is going to be moved down
                // returns: dojo/promise/Promise
                var suggestionId = this._findNextSuggestion(data);
                if (suggestionId === null) {
                    return null;
                }
                return this._swapSuggestions(data, suggestionId);
            },

            remove: function(id) {
                // summary:
                //      Removes a suggestion with specified ID
                return this.store.remove(id);
            },

            canMoveUp: function(data) {
                // summary:
                //      Verifies if a specified suggestion can be moved up in a sorted by priority
                //      list of suggestions that belongs to the same query
                // returns: Boolean
                return this._findPreviousSuggestion(data) !== null;
            },

            canMoveDown: function(data) {
                // summary:
                //      Verifies if a specified suggestion can be moved down in a sorted by priority
                //      list of suggestions that belongs to the same query
                // returns: Boolean

                return this._findNextSuggestion(data) !== null;
            },

            saveChanges: function() {
                // summary:
                //      Saves changes made in binding controller (ctrl).
                //  description:
                //      Before a new item is saved a check for existing suggestions is performed.
                //      If there are suggestions with the same query, the priority of the new item
                //      is set to the highest priority of existing suggestions plus one.
                //      Method also transforms multiple suggestions entered as a comma-separated list
                //      to a series of commits that create a separate entity for each suggestion.
                // returns: dojo/promise/Promise
                var self = this, query = this.ctrl.get("query");

                return when(this.store.query({query: query}), function(otherSuggestions) {
                    var commits, suggestions = self.ctrl.get("suggestion").split(","),
                        id = self.ctrl.get(self.store.idProperty),
                        // If a language is set on the store, we want to set a high base priority so
                        // suggestion comes before other suggestions from "all languages"
                        basePriority = self.store.language ? 10000 : 0;

                    if (otherSuggestions && otherSuggestions.length) {
                        basePriority = otherSuggestions[0].priority; // get the topmost suggestion priority
                    }
                    commits = array.map(suggestions, function (item, index) {
                        // keep ID only for the first suggestion and reset ID for others
                        // in order to perform add instead of put
                        if (index > 0) {
                            var model = lang.clone(self.ctrl.get("model"));
                            delete model[self.store.idProperty];
                            self.ctrl.set("model", model);
                        }
                        if (!id || index > 0) { // set priority for new items
                            self.ctrl.set("priority", basePriority + suggestions.length - index);
                        }

                        self.ctrl.set("suggestion", item);
                        return self.ctrl.commit();
                    });
                    return all(commits);
                });
            },

            reorderAll: function (data) {
                //  summary:
                //      Range all suggestions related to a specified item by priority
                //  data: Object
                //      A related query entity with other_suggestions property holding
                //      an array of other suggestions IDs related to the same query.
                var self = this;
                return all(
                    array.map(data.other_suggestions, function (id, index) {
                        return when(self.store.get(id), function (otherSuggestion) {
                            var newPriority = data.other_suggestions.length - index;
                            if (otherSuggestion.priority !== newPriority) {
                                otherSuggestion.priority = newPriority;
                                return self.store.put(otherSuggestion);
                            }
                        });
                    })
                );
            },

            _findPreviousSuggestion: function (data) {
                var previousSuggestion = null;
                if (data && data.other_suggestions) {
                    var currentItemIndex = array.indexOf(data.other_suggestions, data.id);
                    if (currentItemIndex-1 >= 0) {
                        previousSuggestion = data.other_suggestions[currentItemIndex-1];
                    }
                }
                return previousSuggestion;
            },

            _findNextSuggestion: function(data) {
                var nextSuggestion = null;
                if (data && data.other_suggestions) {
                    var currentItemIndex = array.indexOf(data.other_suggestions, data.id);
                    if (currentItemIndex > -1 && currentItemIndex+1 < data.other_suggestions.length) {
                        nextSuggestion = data.other_suggestions[currentItemIndex+1];
                    }
                }
                return nextSuggestion;
            },

            _swapSuggestions: function (data, suggestionId) {
                var self = this;
                return when(this.store.get(suggestionId), function (suggestion) {
                    if (data.priority === suggestion.priority) {
                        var currentIndex = array.indexOf(data.other_suggestions, data.id),
                            suggestionIndex = array.indexOf(data.other_suggestions, suggestion.id);
                        data.other_suggestions[currentIndex] = suggestion.id;
                        data.other_suggestions[suggestionIndex] = data.id;
                        return self.reorderAll(data);
                    }

                    self._swapPriority(data, suggestion);
                    return all([
                        self.store.put(data),
                        self.store.put(suggestion)
                    ]);
                });
            },

            _swapPriority: function(data, otherSuggestion) {
                var tmp = data.priority;
                data.priority = otherSuggestion.priority;
                otherSuggestion.priority = tmp;
                otherSuggestion.query = data.query;
                otherSuggestion.tags = lang.clone(data.tags);
            }

        });
    });

},
'dojox/mvc/EditStoreRefController':function(){
define("dojox/mvc/EditStoreRefController", [
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/when",
	"./getPlainValue",
	"./EditModelRefController",
	"./StoreRefController"
], function(declare, lang, when, getPlainValue, EditModelRefController, StoreRefController){
	return declare("dojox.mvc.EditStoreRefController", [StoreRefController, EditModelRefController], {
		// summary:
		//		A child class of dojox/mvc/StoreRefController, managing edits.
		// description:
		//		In addition to what dojox/mvc/StoreRefController does, the commit() method sends the data model as well as the removed entries in array to the data store.
		//		NOTE - If this class is used with a widget by data-dojo-mixins, make sure putting the widget in data-dojo-type and putting this class to data-dojo-mixins.
		// example:
		//		The check box refers to "value" property in the controller (with "ctrl" ID).
		//		The controller provides the "value" property, from the data coming from data store ("store" property in the controller), using the first one in array.
		//		Two seconds later, the check box changes from unchecked to checked.
		//		The change is committed to the data store, which is reflected to dojo/store/Observable callback. 
		// |		<html>
		// |			<head>
		// |				<script src="/path/to/dojo-toolkit/dojo/dojo.js" type="text/javascript" data-dojo-config="parseOnLoad: 0"></script>
		// |				<script type="text/javascript">
		// |					require([
		// |						"dojo/dom", "dojo/parser", "dojo/when", "dojo/store/Observable", "dojo/store/Memory", "dijit/registry", "dojo/domReady!"
		// |					], function(ddom, parser, when, Observable, Memory, registry){
		// |						store = Observable(new Memory({data: [{id: "Foo", value: false}]}));
		// |						when(parser.parse(), function(){
		// |							registry.byId("ctrl").queryStore().observe(function(object, previousIndex, newIndex){
		// |								alert("ID: " + object.id + ", value: " + object.value);
		// |							}, true);
		// |							var count = 0;
		// |							var h = setInterval(function(){
		// |								ddom.byId("check").click();
		// |								registry.byId("ctrl").commit();
		// |								if(++count >= 2){ clearInterval(h); }
		// |							}, 2000);
		// |						});
		// |					});
		// |				</script>
		// |			</head>
		// |			<body>
		// |				<script type="dojo/require">at: "dojox/mvc/at"</script>
		// |				<span id="ctrl" data-dojo-type="dojox/mvc/EditStoreRefController" data-dojo-mixins="dojox/mvc/ListController"
		// |				 data-dojo-props="store: store, cursorIndex: 0"></span>
		// |				<input id="check" type="checkbox" data-dojo-type="dijit/form/CheckBox" data-dojo-props="checked: at('widget:ctrl', 'value')">
		// |			</body>
		// |		</html>

		// getPlainValueOptions: dojox/mvc/getPlainValueOptions
		//		The options to get plain value from stateful object.
		getPlainValueOptions: null,

		// _removals: Object[]
		//		The list of removed elements.
		_removals: [],

		// _resultsWatchHandle: dojox/mvc/StatefulArray.watchElements.handle
		//		The watch handle for model array elements.
		_resultsWatchHandle: null,

		// _refSourceModelProp: String
		//		The property name for the data model, that serves as the data source.
		_refSourceModelProp: "sourceModel",

		queryStore: function(/*Object*/ query, /*dojo/store/api/Store.QueryOptions?*/ options){
			// summary:
			//		Queries the store for objects.
			// query: Object
			//		The query to use for retrieving objects from the store.
			// options: dojo/store/api/Store.QueryOptions?
			//		The optional arguments to apply to the resultset.
			// returns: dojo/store/api/Store.QueryResults
			//		The results of the query, extended with iterative methods.

			if(!(this.store || {}).query){ return; }
			if(this._resultsWatchHandle){ this._resultsWatchHandle.unwatch(); }
			this._removals = [];
			var _self = this,
			 queryResult = this.inherited(arguments),
			 result = when(queryResult, function(results){
				if(_self._beingDestroyed){ return; }
				if(lang.isArray(results)){
					_self._resultsWatchHandle = results.watchElements(function(idx, removals, adds){
						[].push.apply(_self._removals, removals);
					});
				}
				return results;
			});
			if(result.then){
				result = lang.delegate(result);
			}
			// For dojo/store/Observable, which adds a function to query result
			for(var s in queryResult){
				if(isNaN(s) && queryResult.hasOwnProperty(s) && lang.isFunction(queryResult[s])){
					result[s] = queryResult[s];
				}
			}
			return result;
		},

		getStore: function(/*Number*/ id, /*Object*/ options){
			// summary:
			//		Retrieves an object by its identity.
			// id: Number
			//		The identity to use to lookup the object.
			// options: Object
			//		The options for dojo/store/*/get().
			// returns: Object
			//		The object in the store that matches the given id.

			if(this._resultsWatchHandle){ this._resultsWatchHandle.unwatch(); }
			return this.inherited(arguments);
		},

		commit: function(){
			// summary:
			//		Send the change back to the data source.

			if(this._removals){
				for(var i = 0; i < this._removals.length; i++){
					this.store.remove(this.store.getIdentity(this._removals[i]));
				}
				this._removals = [];
			}
			var data = getPlainValue(this.get(this._refEditModelProp), this.getPlainValueOptions);
			if(lang.isArray(data)){
				for(var i = 0; i < data.length; i++){
					this.store.put(data[i]);
				}
			}else{
				this.store.put(data);
			}
			this.inherited(arguments);
		},

		reset: function(){
			// summary:
			//		Change the model back to its original state.

			this.inherited(arguments);
			this._removals = [];
		},

		destroy: function(){
			// summary:
			//		Clean up model watch handle as this object is destroyed.

			if(this._resultsWatchHandle){ this._resultsWatchHandle.unwatch(); }
			this.inherited(arguments);
		}
	});
});

},
'dojox/mvc/getPlainValue':function(){
define("dojox/mvc/getPlainValue", [
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/Stateful"
], function(array, lang, Stateful){
	var getPlainValueOptions = {
		// summary:
		//		Options used for dojox/mvc/getPlainValue().

		getType: function(/*Anything*/ v){
			// summary:
			//		Returns the type of the given value.
			// v: Anything
			//		The value.
			// returns:
			//		 The type of the given value.

			return lang.isArray(v) ? "array" : v != null && {}.toString.call(v) == "[object Object]" ? "object" : "value";
		},

		getPlainArray: function(/*Anything[]*/ a){
			// summary:
			//		Returns the stateful version of the given array.
			// a: Anything[]
			//		The array.
			// returns:
			//		 The converted array.

			return array.map(a, function(item){ return getPlainValue(item, this); }, this); // Anything[]
		},

		getPlainObject: function(/*Object*/ o){
			// summary:
			//		Returns the stateful version of the given object.
			// o: Object
			//		The object.

			var plain = {};
			for(var s in o){
				if(!(s in Stateful.prototype) && s != "_watchCallbacks"){
					plain[s] = getPlainValue(o[s], this);
				}
			}
			return plain; // Object
		},

		getPlainValue: function(/*Anything*/ v){
			// summary:
			//		Just returns the given value.

			return v; // Anything
		}
	};

	var getPlainValue = function(/*Anything*/ value, /*dojox/mvc/getPlainValueOptions*/ options){
		// summary:
		//		Create a raw value from a dojo/Stateful object.
		// description:
		//		Recursively iterates the stateful value given, and convert them to raw ones.
		// value: Anything
		//		The stateful value.
		// options: dojox/mvc/getPlainValueOptions
		//		The object that defines how plain value should be created from stateful value.
		// returns:
		//		 The converted value.

		return (options || getPlainValue)["getPlain" + (options || getPlainValue).getType(value).replace(/^[a-z]/, function(c){ return c.toUpperCase(); })](value); // Anything
	};

	// lang.setObject() thing is for back-compat, remove it in 2.0
	return lang.setObject("dojox.mvc.getPlainValue", lang.mixin(getPlainValue, getPlainValueOptions));
});

},
'dojox/mvc/EditModelRefController':function(){
define("dojox/mvc/EditModelRefController", [
	"dojo/_base/declare",
	"dojo/_base/lang",
	"./getPlainValue",
	"./getStateful",
	"./ModelRefController"
], function(declare, lang, getPlainValue, getStateful, ModelRefController){
	// module:
	//		dojox/mvc/EditModelRefController

	function setRefSourceModel(/*dojox/mvc/EditModelRefController*/ ctrl, /*Anything*/ old, /*Anything*/ current){
		// summary:
		//		A function called when this controller gets newer value as the data source.
		// ctrl: dojox/mvc/EditModelRefController
		//		The controller.
		// old: Anything
		//		The older value.
		// current: Anything
		//		The newer value.

		if(old !== current){
			ctrl.set(ctrl._refOriginalModelProp, ctrl.holdModelUntilCommit ? current : ctrl.cloneModel(current));
			ctrl.set(ctrl._refEditModelProp, ctrl.holdModelUntilCommit ? ctrl.cloneModel(current) : current);
		}
	}

	return declare("dojox.mvc.EditModelRefController", ModelRefController, {
		// summary:
		//		A child class of dojox/mvc/ModelRefController.
		//		Keeps a copy (originalModel) of given data model (sourceModel) so that it can manage the data model of before/after the edit.
		// description:
		//		Has two modes:
		//
		//		- Directly reflect the edits to sourceModel (holdModelUntilCommit=false)
		//		- Don't reflect the edits to sourceModel, until commit() is called (holdModelUntilCommit=true)
		//
		//		For the 1st case, dojo/Stateful get()/set()/watch() interfaces will work with sourceModel.
		//		For the 2nd case, dojo/Stateful get()/set()/watch() interfaces will work with a copy of sourceModel, and sourceModel will be replaced with such copy when commit() is called.
		//
		//		NOTE - If this class is used with a widget by data-dojo-mixins, make sure putting the widget in data-dojo-type and putting this class to data-dojo-mixins.
		// example:
		//		The check box refers to "value" property in the controller (with "ctrl" ID).
		//		The controller provides the "value" property on behalf of the model ("model" property in the controller, which comes from "sourceModel" property).
		//		Two seconds later, the check box changes from unchecked to checked, and the controller saves the state.
		//		Two seconds later then, the check box changes from checked to unchecked.
		//		Two seconds later then, the controller goes back to the last saved state, and the check box changes from unchecked to checked as the result.
		// |		<html>
		// |			<head>
		// |				<script src="/path/to/dojo-toolkit/dojo/dojo.js" type="text/javascript" data-dojo-config="parseOnLoad: 0"></script>
		// |				<script type="text/javascript">
		// |					require([
		// |						"dojo/dom", "dojo/parser", "dojo/Stateful", "dijit/registry", "dijit/form/CheckBox", "dojox/mvc/EditModelRefController", "dojo/domReady!"
		// |					], function(ddom, parser, Stateful, registry){
		// |						model = new Stateful({value: false});
		// |						setTimeout(function(){
		// |							ddom.byId("check").click();
		// |							registry.byId("ctrl").commit();
		// |							setTimeout(function(){
		// |								ddom.byId("check").click();
		// |								setTimeout(function(){
		// |									registry.byId("ctrl").reset();
		// |								}, 2000);
		// |							}, 2000);
		// |						}, 2000);
		// |						parser.parse();
		// |					});
		// |				</script>
		// |			</head>
		// |			<body>
		// |				<script type="dojo/require">at: "dojox/mvc/at"</script>
		// |				<span id="ctrl" data-dojo-type="dojox/mvc/EditModelRefController" data-dojo-props="sourceModel: model"></span>
		// |				<input id="check" type="checkbox" data-dojo-type="dijit/form/CheckBox" data-dojo-props="checked: at('widget:ctrl', 'value')">
		// |			</body>
		// |		</html>
		// example:
		//		The controller with "ctrlSource" ID specifies holding changes until commit() is called (by setting true to holdModelUntilCommit).
		//		As the change in the second check box is committed two seconds later from the change, the first check box is checked at then (when the change is committed).
		// |		<html>
		// |			<head>
		// |				<script src="/path/to/dojo-toolkit/dojo/dojo.js" type="text/javascript" data-dojo-config="parseOnLoad: 0"></script>
		// |				<script type="text/javascript">
		// |					require([
		// |						"dojo/dom", "dojo/parser", "dojo/Stateful", "dijit/registry",
		// |						"dijit/form/CheckBox", "dojox/mvc/ModelRefController", "dojox/mvc/EditModelRefController", "dojo/domReady!"
		// |					], function(ddom, parser, Stateful, registry){
		// |						model = new Stateful({value: false});
		// |						setTimeout(function(){
		// |							ddom.byId("checkEdit").click();
		// |							setTimeout(function(){
		// |								registry.byId("ctrlEdit").commit();
		// |							}, 2000);
		// |						}, 2000);
		// |						parser.parse();
		// |					});
		// |				</script>
		// |			</head>
		// |			<body>
		// |				<script type="dojo/require">at: "dojox/mvc/at"</script>
		// |				<span id="ctrlSource" data-dojo-type="dojox/mvc/ModelRefController" data-dojo-props="model: model"></span>
		// |				<span id="ctrlEdit" data-dojo-type="dojox/mvc/EditModelRefController"
		// |				 data-dojo-props="sourceModel: at('widget:ctrlSource', 'model'), holdModelUntilCommit: true"></span>
		// |				Source:
		// |				<input id="checkSource" type="checkbox" data-dojo-type="dijit/form/CheckBox"
		// |				 data-dojo-props="checked: at('widget:ctrlSource', 'value')">
		// |				Edit:
		// |				<input id="checkEdit" type="checkbox" data-dojo-type="dijit/form/CheckBox"
		// |				 data-dojo-props="checked: at('widget:ctrlEdit', 'value')">
		// |			</body>
		// |		</html>

		// getStatefulOptions: dojox/mvc/getStatefulOptions
		//		The options to get stateful object from plain value.
		getStatefulOptions: null,

		// getPlainValueOptions: dojox/mvc/getPlainValueOptions
		//		The options to get plain value from stateful object.
		getPlainValueOptions: null,

		// holdModelUntilCommit: Boolean
		//		True not to send the change in model back to sourceModel until commit() is called.
		holdModelUntilCommit: false,

		// originalModel: dojo/Stateful
		//		The data model, that serves as the original data.
		originalModel: null,

		// originalModel: dojo/Stateful
		//		The data model, that serves as the data source.
		sourceModel: null,

		// _refOriginalModelProp: String
		//		The property name for the data model, that serves as the original data.
		_refOriginalModelProp: "originalModel",

		// _refSourceModelProp: String
		//		The property name for the data model, that serves as the data source.
		_refSourceModelProp: "sourceModel",

		// _refEditModelProp: String
		//		The property name for the data model, that is being edited.
		_refEditModelProp: "model",

		postscript: function(/*Object?*/ params, /*DomNode|String?*/ srcNodeRef){
			// summary:
			//		Sets certain properties before setting models.

			for(var s in {getStatefulOptions: 1, getPlainValueOptions: 1, holdModelUntilCommit: 1}){
				var value = (params || {})[s];
				if(typeof value != "undefined"){
					this[s] = value;
				}
			}
			this.inherited(arguments);
		},

		set: function(/*String*/ name, /*Anything*/ value){
			// summary:
			//		Set a property to this.
			// name: String
			//		The property to set.
			// value: Anything
			//		The value to set in the property.

			if(name == this._refSourceModelProp){
				setRefSourceModel(this, this[this._refSourceModelProp], value);
			}
			this.inherited(arguments);
		},

		cloneModel: function(/*Anything*/ value){
			// summary:
			//		Create a clone object of the data source.
			//		Child classes of this controller can override it to achieve its specific needs.
			// value: Anything
			//		The data serving as the data source.

			var plain = lang.isFunction((value || {}).set) && lang.isFunction((value || {}).watch) ? getPlainValue(value, this.getPlainValueOptions) : value;
			return getStateful(plain, this.getStatefulOptions);
		},

		commit: function(){
			// summary:
			//		Send the change back to the data source.

			this.set(this.holdModelUntilCommit ? this._refSourceModelProp : this._refOriginalModelProp, this.cloneModel(this.get(this._refEditModelProp)));
		},

		reset: function(){
			// summary:
			//		Change the model back to its original state.

			this.set(this.holdModelUntilCommit ? this._refEditModelProp : this._refSourceModelProp, this.cloneModel(this.get(this._refOriginalModelProp)));
		},

		hasControllerProperty: function(/*String*/ name){
			// summary:
			//		Returns true if this controller itself owns the given property.
			// name: String
			//		The property name.

			return this.inherited(arguments) || name == this._refOriginalModelProp || name == this._refSourceModelProp;
		}
	});
});

},
'dojox/mvc/getStateful':function(){
define("dojox/mvc/getStateful", [
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/Stateful",
	"./StatefulArray"
], function(array, lang, Stateful, StatefulArray){
	var getStatefulOptions = {
		// summary:
		//		Options used for dojox/mvc/getStateful().

		getType: function(/*Anything*/ v){
			// summary:
			//		Returns the type of the given value.
			// v: Anything
			//		The value.

			return lang.isArray(v) ? "array" : v != null && {}.toString.call(v) == "[object Object]" ? "object" : "value";
		},

		getStatefulArray: function(/*Anything[]*/ a){
			// summary:
			//		Returns the stateful version of the given array.
			// a: Anything[]
			//		The array.

			return new StatefulArray(array.map(a, function(item){ return getStateful(item, this); }, this)); // dojox/mvc/StatefulArray
		},

		getStatefulObject: function(/*Object*/ o){
			// summary:
			//		Returns the stateful version of the given object.
			// o: Object
			//		The object.

			var stateful = new Stateful();
			for(var s in o){
				stateful[s] = getStateful(o[s], this);
			}
			return stateful; // dojo/Stateful
		},

		getStatefulValue: function(/*Anything*/ v){
			// summary:
			//		Just returns the given value.

			return v; // Anything
		}
	};

	var getStateful = function(/*Anything*/ value, /*dojox/mvc/getStatefulOptions*/ options){
		// summary:
		//		Create a dojo/Stateful object from a raw value.
		// description:
		//		Recursively iterates the raw value given, and convert them to stateful ones.
		// value: Anything
		//		The raw value.
		// options: dojox/mvc/getStatefulOptions
		//		The object that defines how model object should be created from plain object hierarchy.
		// returns: Anything
		//		 The converted value.

		return (options || getStateful)["getStateful" + (options || getStateful).getType(value).replace(/^[a-z]/, function(c){ return c.toUpperCase(); })](value); // Anything
	};

	// lang.setObject() thing is for back-compat, remove it in 2.0
	return lang.setObject("dojox.mvc.getStateful", lang.mixin(getStateful, getStatefulOptions));
});

},
'dojox/mvc/StatefulArray':function(){
define("dojox/mvc/StatefulArray", [
	"dojo/_base/lang",
	"dojo/Stateful"
], function(lang, Stateful){
	function update(/*dojox/mvc/StatefulArray*/ a){
		// summary:
		//		Set all array elements as stateful so that watch function runs.
		// a: dojox/mvc/StatefulArray
		//		The array.

		// Notify change of elements.
		if(a._watchElementCallbacks){
			a._watchElementCallbacks();
		}

		return a; // dojox/mvc/StatefulArray
	}

	var StatefulArray = function(/*Anything[]*/ a){
		// summary:
		//		An inheritance of native JavaScript array, that adds dojo/Stateful capability.
		// description:
		//		Supported methods are:
		//
		//		- pop() - Stateful update is done for the removed element, as well as the length.
		//		- push() - Stateful update is done for the added element, as well as the length.
		//		- reverse() - Stateful update is done for the elements.
		//		- shift() - Stateful update is done for the removed element, as well as the length.
		//		- sort() - Stateful update is done for the elements.
		//		- splice() - Stateful update is done for the removed/added elements, as well as the length. Returns an instance of StatefulArray instead of the native array.
		//		- unshift() - Stateful update is done for the added element, as well as the length.
		//		- concat() - Returns an instance of StatefulArray instead of the native Array.
		//		- join() - The length as well as the elements are obtained via stateful getters, instead of direct access.
		//		- slice() - The length as well as the elements are obtained via stateful getters, instead of direct access.
		//		- Setting an element to this array via set() - Stateful update is done for the new element as well as the new length.
		//		- Setting a length to this array via set() - Stateful update is done for the removed/added elements as well as the new length.

		var array = lang._toArray(a || []);
		var ctor = StatefulArray;
		ctor._meta = {bases: [Stateful]}; // For isInstanceOf()
		array.constructor = ctor;
		return lang.mixin(array, {
			pop: function(){
				return this.splice(this.get("length") - 1, 1)[0];
			},
			push: function(){
				this.splice.apply(this, [this.get("length"), 0].concat(lang._toArray(arguments)));
				return this.get("length");
			},
			reverse: function(){
				return update([].reverse.apply(this, lang._toArray(arguments)));
			},
			shift: function(){
				return this.splice(0, 1)[0];
			},
			sort: function(){
				return update([].sort.apply(this, lang._toArray(arguments)));
			},
			splice: function(/*Number*/ idx, /*Number*/ n){
				// summary:
				//		Removes and then adds some elements to an array.
				//		Updates the removed/added elements, as well as the length, as stateful.
				// idx: Number
				//		The index where removal/addition should be done.
				// n: Number
				//		How many elements to be removed at idx.
				// varargs: Anything[]
				//		The elements to be added to idx.
				// returns: dojox/mvc/StatefulArray
				//		The removed elements.

				var l = this.get("length");

				idx += idx < 0 ? l : 0;

				var p = Math.min(idx, l),
				 removals = this.slice(idx, idx + n),
				 adds = lang._toArray(arguments).slice(2);

				// Do the modification in a native manner except for setting additions
				[].splice.apply(this, [idx, n].concat(new Array(adds.length)));

				// Set additions in a stateful manner
				for(var i = 0; i < adds.length; i++){
					this.set(p + i, adds[i]);
				}

				// Notify change of elements.
				if(this._watchElementCallbacks){
					this._watchElementCallbacks(idx, removals, adds);
				}

				// Notify change of length.
				// Not calling the setter for "length" though, given removal/addition of array automatically changes the length.
				if(this._watchCallbacks){
					this._watchCallbacks("length", l, l - removals.length + adds.length);
				}

				return removals; // dojox/mvc/StatefulArray
			},
			unshift: function(){
				this.splice.apply(this, [0, 0].concat(lang._toArray(arguments)));
				return this.get("length");
			},
			concat: function(/*Array*/ a){
				return new StatefulArray([].concat.apply(this, arguments));
			},
			join: function(/*String*/ sep){
				// summary:
				//		Returns a string joining string elements in a, with a separator.
				// sep: String
				//		The separator.

				var list = [];
				for(var l = this.get("length"), i = 0; i < l; i++){
					list.push(this.get(i));
				}
				return list.join(sep); // String
			},
			slice: function(/*Number*/ start, /*Number*/ end){
				// summary:
				//		Returns partial elements of an array.
				// start: Number
				//		The index to begin with.
				// end: Number
				//		The index to end at. (a[end] won't be picked up)

				var l = this.get("length");

				start += start < 0 ? l : 0;
				end = (end === void 0 ? l : end) + (end < 0 ? l : 0);

				var slice = [];
				for(var i = start || 0; i < Math.min(end, this.get("length")); i++){
					slice.push(this.get(i));
				}
				return new StatefulArray(slice); // dojox/mvc/StatefuArray
			},
			watchElements: function(/*Function*/ callback){
				// summary:
				//		Watch for change in array elements.
				// callback: Function
				//		The callback function, which should take: The array index, the removed elements, and the added elements.

				var callbacks = this._watchElementCallbacks, _self = this;
				if(!callbacks){
					callbacks = this._watchElementCallbacks = function(idx, removals, adds){
						for(var list = [].concat(callbacks.list), i = 0; i < list.length; i++){
							list[i].call(_self, idx, removals, adds);
						}
					};
					callbacks.list = [];
				}

				callbacks.list.push(callback);

				var h = {};
				h.unwatch = h.remove = function(){
					for(var list = callbacks.list, i = 0; i < list.length; i++){
						if(list[i] == callback){
							list.splice(i, 1);
							break;
						}
					}
				}; 
				return h; // dojo/handle
			}
		}, Stateful.prototype, {
			set: function(/*Number|String*/ name, /*Anything*/ value){
				// summary:
				//		Sets a new value to an array.
				// name: Number|String
				//		The property name.
				// value: Anything
				//		The new value.

				if(name == "length"){
					var old = this.get("length");
					if(old < value){
						this.splice.apply(this, [old, 0].concat(new Array(value - old)));
					}else if(value > old){
						this.splice.apply(this, [value, old - value]);
					}
					return this;
				}else{
					var oldLength = this.length;
					Stateful.prototype.set.call(this, name, value);
					if(oldLength != this.length){
						Stateful.prototype.set.call(this, "length", this.length);
					}
					return this;
				}
			}
		});
	};

	return lang.setObject("dojox.mvc.StatefulArray", StatefulArray);
});

},
'dojox/mvc/ModelRefController':function(){
define("dojox/mvc/ModelRefController", [
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/Stateful",
	"./_Controller"
], function(array, declare, lang, Stateful, _Controller){
	return declare("dojox.mvc.ModelRefController", _Controller, {
		// summary:
		//		A controller that keeps a reference to dojo/Stateful-based data model.
		// description:
		//		Does the following on behalf of such model:
		//
		//		- Provides data from model via dojo/Stateful get() interface
		//		- Stores data to model via dojo/Stateful set() interface
		//		- Watches for change in model via dojo/Stateful watch() interface (The callback is called when there is a change in data model, as well as when the data model itself is replaced with different one)
		//
		//		Can also be used to do some application-specific stuffs upon change in properties in model, by defining setter functions. 
		//		Doing so will help keep models and widgets free from application-specific logic, and will help keep application logic free from specifics of models and widgets.
		//		Such kind of setter functions can be defined in the same manner as widgets (_setXXXAttr()).
		//
		//		NOTE - If this class is used with a widget by data-dojo-mixins, make sure putting the widget in data-dojo-type and putting this class to data-dojo-mixins.
		// example:
		//		The text box refers to "value" property in the controller (with "ctrl" ID).
		//		The controller provides the "value" property on behalf of the model ("model" property in the controller).
		//		Two seconds later, the text box changes from "Foo" to "Bar" as the controller changes the data model it refers to.
		// |		<html>
		// |			<head>
		// |				<script src="/path/to/dojo-toolkit/dojo/dojo.js" type="text/javascript" data-dojo-config="parseOnLoad: 0"></script>
		// |				<script type="text/javascript">
		// |					require([
		// |						"dojo/parser", "dojo/Stateful", "dijit/registry",
		// |						"dijit/form/TextBox", "dojox/mvc/ModelRefController", "dojo/domReady!"
		// |					], function(parser, Stateful, registry){
		// |						modelFoo = new Stateful({value: "Foo"});
		// |						modelBar = new Stateful({value: "Bar"});
		// |						setTimeout(function(){ registry.byId("ctrl").set("model", modelBar); }, 2000);
		// |						parser.parse();
		// |					});
		// |				</script>
		// |			</head>
		// |			<body>
		// |				<script type="dojo/require">at: "dojox/mvc/at"</script>
		// |				<span id="ctrl" data-dojo-type="dojox/mvc/ModelRefController" data-dojo-props="model: modelFoo"></span>
		// |				<input type="text" data-dojo-type="dijit/form/TextBox" data-dojo-props="value: at('widget:ctrl', 'value')">
		// |			</body>
		// |		</html>

		// ownProps: Object
		//		List of property names owned by this controller, instead of the data model.
		ownProps: null,

		// _refModelProp: String
		//		The property name for the data model.
		_refModelProp: "model",

		// _refInModelProp: String
		//		The property name for the data model, used as the input.
		//		Used when this controller needs data model (as input) that is different from the data model this controller provides.
		_refInModelProp: "model",

		// model: dojo/Stateful
		//		The data model.
		model: null,

		postscript: function(/*Object?*/ params, /*DomNode|String?*/ srcNodeRef){
			// summary:
			//		Sets _relTargetProp so that the property specified by _refModelProp is used for relative data binding.

			this._relTargetProp = (params || {})._refModelProp || this._refModelProp;
			this.inherited(arguments);
		},

		get: function(/*String*/ name){
			// summary:
			//		If getter function is there, use it. Otherwise, get the data from data model of this object.
			// name: String
			//		The property name.

			if(!this.hasControllerProperty(name)){
				var model = this[this._refModelProp];
				return !model ? void 0 : model.get ? model.get(name) : model[name];
			}
			return this.inherited(arguments);
		},

		_set: function(/*String*/ name, /*Anything*/ value){
			// summary:
			//		Set the value to the data model or to this object.
			// name: String
			//		The property name.
			// value: Anything
			//		The property value.

			if(!this.hasControllerProperty(name)){
				var model = this[this._refModelProp];
				model && (model.set ? model.set(name, value) : (model[name] = value));
				return this;
			}
			return this.inherited(arguments);
		},

		watch: function(/*String?*/ name, /*Function*/ callback){
			// summary:
			//		Watch a property in the data model or in this object.
			// name: String?
			//		The property name.
			// callback: Function
			//		The callback function.

			if(this.hasControllerProperty(name)){
				return this.inherited(arguments);
			}

			if(!callback){
				callback = name;
				name = null;
			}

			var hm = null, hp = null, _self = this;

			function watchPropertiesInModel(/*dojo/Stateful*/ model){
				// summary:
				//		Watch properties in referred model.
				// model: dojo/Stateful
				//		The model to watch for.

				// Unwatch properties of older model.
				if(hp){ hp.unwatch(); }
				// Watch properties of newer model.
				if(model && lang.isFunction(model.set) && lang.isFunction(model.watch)){
					hp = model.watch.apply(model, (name ? [name] : []).concat([function(name, old, current){ callback.call(_self, name, old, current); }]));
				}
			}

			function reflectChangeInModel(/*dojo/Stateful*/ old, /*dojo/Stateful*/ current){
				// summary:
				//		Upon change in model, detect change in properties, and call watch callbacks.
				// old: dojo/Stateful
				//		The older model.
				// current: dojo/Stateful
				//		The newer model.

				// Gather list of properties to notify change in value as model changes.
				var props = {};
				if(!name){
					// If all properties are being watched, find out all properties from older model as well as from newer model.
					array.forEach([old, current], function(model){
						var list = model && model.get("properties");
						if(list){
							// If the model explicitly specifies the list of properties, use it.
							array.forEach(list, function(item){
								if(!_self.hasControllerProperty(item)){ props[item] = 1; }
							});
						}else{
							// Otherwise, iterate through own properties.
							for(var s in model){
								if(model.hasOwnProperty(s) && !_self.hasControllerProperty(s)){ props[s] = 1; }
							}
						}
					});
				}else{
					props[name] = 1;
				}

				// Call watch callbacks for properties.
				for(var s in props){
					callback.call(_self, s, !old ? void 0 : old.get ? old.get(s) : old[s], !current ? void 0 : current.get ? current.get(s) : current[s]);
				}
			}

			// Watch for change in model.
			hm = Stateful.prototype.watch.call(this, this._refModelProp, function(name, old, current){
				if(old === current){ return; }
				reflectChangeInModel(old, current);
				watchPropertiesInModel(current);
			});

			// Watch for properties in model.
			watchPropertiesInModel(this.get(this._refModelProp));

			var h = {};
			h.unwatch = h.remove = function(){
				if(hp){ hp.unwatch(); hp = null; } if(hm){ hm.unwatch(); hm = null; }
			};
			return h; // dojo/handle
		},

		hasControllerProperty: function(/*String*/ name){
			// summary:
			//		Returns true if this controller itself owns the given property.
			// name: String
			//		The property name.

			return name == "_watchCallbacks" || name == this._refModelProp || name == this._refInModelProp || (name in (this.ownProps || {})) || (name in this.constructor.prototype) || /^dojoAttach(Point|Event)$/i.test(name); // Let dojoAttachPoint/dojoAttachEvent be this controller's property to support <span data-dojo-type="dojox/mvc/ModelRefController" data-dojo-attach-point="controllerNode"> in widgets-in-template
		}
	});
});

},
'dojox/mvc/_Controller':function(){
define("dojox/mvc/_Controller", [
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/Stateful",
	"./_atBindingMixin"
], function(declare, lang, Stateful, _atBindingMixin){
	return declare("dojox.mvc._Controller", [Stateful, _atBindingMixin], {
		postscript: function(/*Object?*/ params, /*DomNode|String?*/ srcNodeRef){
			// summary:
			//		If this object is not called from Dojo parser, starts this up right away.
			//		Also, if widget registry is available, register this object.

			// If there is dijit/_WidgetBase in upper class hierarchy (happens when this descendant is mixed into a widget), let _WidgetBase do all work
			if(this._applyAttributes){
				this.inherited(arguments);
			}
			// Look for dojox/mvc/at handles in the parameters
			this._dbpostscript(params, srcNodeRef);
			// Merge the parameters to this
			if(params){
				this.params = params;
				for(var s in params){
					this.set(s, params[s]);
				}
			}
			// Add this instance to dijit/registry, if it's available
			var registry;
			try{
				// Usage of dijit/registry module is optional. Do not use it if it's not already loaded.
				registry = require("dijit/registry");
				this.id = this.id || (srcNodeRef || {}).id || registry.getUniqueId(this.declaredClass.replace(/\./g, "_"));
				registry.add(this);
			}catch(e){}
			if(!srcNodeRef){
				// If this instance is not created via Dojo parser, start this up right away
				this.startup();
			}else{
				// If this is created via Dojo parser, set widgetId attribute so that destroyDescendants() of parent widget works
				srcNodeRef.setAttribute("widgetId", this.id); 
			}
		},

		startup: function(){
			// summary:
			//		Starts up data binding as this object starts up.

			if(!this._applyAttributes){
				this._startAtWatchHandles();
			}
			// If there is dijit/_WidgetBase in upper class hierarchy (happens when this descendant is mixed into a widget), let _WidgetBase do all work
			this.inherited(arguments);
		},

		destroy: function(){
			// summary:
			//		Stops data binding as this object is destroyed.

			this._beingDestroyed = true;
			if(!this._applyAttributes){
				this._stopAtWatchHandles();
			}
			// If there is dijit/_WidgetBase in upper class hierarchy (happens when this descendant is mixed into a widget), let _WidgetBase do all work
			this.inherited(arguments);
			if(!this._applyAttributes){
				try{
					// Remove this instance from dijit/registry
					// Usage of dijit/registry module is optional. Do not use it if it's not already loaded.
					require("dijit/registry").remove(this.id);
				}catch(e){}
			}
			this._destroyed = true;
		},

		set: function(/*String*/ name, /*Anything*/ value){
			// summary:
			//		If the value given is dojox/mvc/at handle, use it for data binding.
			//		Otherwise, if setter function is there, use it.
			//		Otherwise, set the value to the data model or to this object.
			// name: String
			//		The property name.
			// value: Anything
			//		The property value.

			// If an object is used, iterate through object
			if(typeof name === "object"){
				for(var x in name){
					if(name.hasOwnProperty(x)){
						this.set(x, name[x]);
					}
				}
				return this;
			}

			if(!this._applyAttributes){
				if((value || {}).atsignature == "dojox.mvc.at"){
					// If dojox/mvc/at handle is given, use it for data binding
					return this._setAtWatchHandle(name, value);
				}else{
					// Otherwise align the setter interface to _WidgetBase
					var setterName = "_set" + name.replace(/^[a-z]/, function(c){ return c.toUpperCase(); }) + "Attr";
					if(this[setterName]){
						this[setterName](value);
					}else{
						this._set(name, value);
					}
					return this;
				}
			}

			// If there is dijit/_WidgetBase in upper class hierarchy (happens when this descendant is mixed into a widget), let _WidgetBase do all work
			return this.inherited(arguments);
		},

		_set: function(/*String*/ name, /*Anything*/ value){
			// summary:
			//		Implement _set() interface so that _set() behavior is consistent whether the instance inherits _WidgetBase or not.
			//		If the instance does not inherit _WidgetBase, use dojo/Stateful/_changeAttrValue() that's equivalent to dijit/_WidgetBase._set().
			// name: String
			//		The property name.
			// value: Anything
			//		The property value.

			if(!this._applyAttributes){
				// Call dojo/Stateful/_changeAttrValue() that's equivalent to dijit/_WidgetBase/_set()
				return this._changeAttrValue(name, value);
			}
			// If there is dijit/_WidgetBase in upper class hierarchy (happens when this descendant is mixed into a widget), let _WidgetBase do all work
			return this.inherited(arguments);
		}
	});
});

},
'dojox/mvc/StoreRefController':function(){
define("dojox/mvc/StoreRefController", [
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/when",
	"./getStateful",
	"./ModelRefController"
], function(declare, lang, when, getStateful, ModelRefController){
	return declare("dojox.mvc.StoreRefController", ModelRefController, {
		// summary:
		//		A child class of dojox.mvc.ModelRefController, which keeps a reference to Dojo Object Store (in store property).
		// description:
		//		Has several methods to work with the store:
		//
		//		- queryStore(): Runs query() against the store, and creates a data model from retrieved data
		//		- getStore(): Runs get() against the store, and creates a data model from retrieved data
		//		- putStore(): Runs put() against the store
		//		- addStore(): Runs add() against the store
		//		- removeStore(): Runs remove() against the store
		//
		//		dojo.Stateful get()/set()/watch() interfaces in dojox.mvc.StoreRefController will work with the data model from queryStore() or getStore().
		//
		//		NOTE - If this class is used with a widget by data-dojo-mixins, make sure putting the widget in data-dojo-type and putting this class to data-dojo-mixins.
		// example:
		//		The text box refers to "value" property in the controller (with "ctrl" ID).
		//		The controller provides the "value" property, from the data coming from data store ("store" property in the controller).
		//		Two seconds later, the text box changes from "Foo" to "Bar" as the controller gets new data from data store.
		// |		<html>
		// |			<head>
		// |				<script src="/path/to/dojo-toolkit/dojo/dojo.js" type="text/javascript" data-dojo-config="parseOnLoad: 0"></script>
		// |				<script type="text/javascript">
		// |					require([
		// |						"dojo/parser", "dojo/when", "dojo/store/Memory", "dijit/registry", "dojo/domReady!"
		// |					], function(parser, when, Memory, registry){
		// |						store = new Memory({data: [{id: "Foo", value: "Foo"}, {id: "Bar", value: "Bar"}]});
		// |						when(parser.parse(), function(){
		// |							registry.byId("ctrl").getStore("Foo");
		// |							setTimeout(function(){ registry.byId("ctrl").getStore("Bar"); }, 2000);
		// |						});
		// |					});
		// |				</script>
		// |			</head>
		// |			<body>
		// |				<script type="dojo/require">at: "dojox/mvc/at"</script>
		// |				<span id="ctrl" data-dojo-type="dojox.mvc.StoreRefController" data-dojo-props="store: store"></span>
		// |				<input type="text" data-dojo-type="dijit/form/TextBox" data-dojo-props="value: at('widget:ctrl', 'value')">
		// |			</body>
		// |		</html>

		// store: dojo/store/*
		//		The Dojo Object Store in use.
		store: null,

		// getStatefulOptions: dojox.mvc.getStatefulOptions
		//		The options to get stateful object from plain value.
		getStatefulOptions: null,

		// _refSourceModelProp: String
		//		The property name for the data model, that serves as the data source.
		_refSourceModelProp: "model",

		queryStore: function(/*Object*/ query, /*dojo/store/api/Store.QueryOptions?*/ options){
			// summary:
			//		Queries the store for objects.
			// query: Object
			//		The query to use for retrieving objects from the store.
			// options: dojo/store/api/Store.QueryOptions?
			//		The optional arguments to apply to the resultset.
			// returns: dojo/store/api/Store.QueryResults
			//		The results of the query, extended with iterative methods.

			if(!(this.store || {}).query){ return; }
			if(this._queryObserveHandle){ this._queryObserveHandle.cancel(); }

			var _self = this,
			 queryResult = this.store.query(query, options),
			 result = when(queryResult, function(results){
				if(_self._beingDestroyed){ return; }
				results = getStateful(results, _self.getStatefulOptions);
				_self.set(_self._refSourceModelProp, results);
				return results;
			});
            if(result.then){
				result = lang.delegate(result);
			}
			// For dojo/store/Observable, which adds a function to query result
			for(var s in queryResult){
				if(isNaN(s) && queryResult.hasOwnProperty(s) && lang.isFunction(queryResult[s])){
					result[s] = queryResult[s];
				}
			}
			return result;
		},

		getStore: function(/*Number*/ id, /*Object*/ options){
			// summary:
			//		Retrieves an object by its identity.
			// id: Number
			//		The identity to use to lookup the object.
			// options: Object
			//		The options for dojo/store.*.get().
			// returns: Object
			//		The object in the store that matches the given id.

			if(!(this.store || {}).get){ return; }
			if(this._queryObserveHandle){ this._queryObserveHandle.cancel(); }
			var _self = this,
				result = when(this.store.get(id, options), function(result){
					if(_self._beingDestroyed){ return; }
					result = getStateful(result, _self.getStatefulOptions);
					_self.set(_self._refSourceModelProp, result);
					return result;
				});
			return result;
		},

		putStore: function(/*Object*/ object, /*dojo/store/api/Store.PutDirectives?*/ options){
			// summary:
			//		Stores an object.
			// object: Object
			//		The object to store.
			// options: dojo/store/api/Store.PutDirectives?
			//		Additional metadata for storing the data.  Includes an "id" property if a specific id is to be used.
			// returns: Number

			if(!(this.store || {}).put){ return; }
			return this.store.put(object, options);
		},

		addStore: function(object, options){
			// summary:
			//		Creates an object, throws an error if the object already exists.
			// object: Object
			//		The object to store.
			// options: dojo/store/api/Store.PutDirectives?
			//		Additional metadata for storing the data.  Includes an "id" property if a specific id is to be used.
			// returns: Number

			if(!(this.store || {}).add){ return; }
			return this.store.add(object, options);
		},

		removeStore: function(/*Number*/ id, /*Object*/ options){
			// summary:
			//		Deletes an object by its identity
			// id: Number
			//		The identity to use to delete the object
			// options: Object
			//		The options for dojo/store/*.remove().
			// returns: Boolean
			//		Returns true if an object was removed, falsy (undefined) if no object matched the id.

			if(!(this.store || {}).remove){ return; }
			return this.store.remove(id, options);
		}
	});
});

},
'epi-saas-base/mvc/_CommitReturnMixin':function(){
﻿define("epi-saas-base/mvc/_CommitReturnMixin", [
    "dojo/_base/declare",
    "dojox/mvc/getPlainValue"
], function (declare, getPlainValue) {

    // module:
    //      epi-saas-base/mvc/_CommitReturnMixin

    return declare([], {
        // summary:
        //      Mixin to override commit()
        // description:
        //      commit() is overridden to return the promises for put and add.
        //      This allows the use of then() in the UX in order to provide callbacks for success and failure

        commit: function () {
            // summary:
            //      Send the change back to the data source.

            //TODO: looking for a better alternative
            this.set(this.holdModelUntilCommit ? this._refSourceModelProp : this._refOriginalModelProp, this.cloneModel(this.get(this._refEditModelProp)));

            var data = getPlainValue(this.get(this._refEditModelProp), this.getPlainValueOptions);

            var id = this.store.getIdentity(data);
            if (id) {
                return this.store.put(data);
            }
            else {
                return this.store.add(data);
            }
        }
    });
});

},
'epi-saas-base/mvc/_EmptiableRefControllerMixin':function(){
define("epi-saas-base/mvc/_EmptiableRefControllerMixin", [
    "dojo/_base/declare"
], function (declare) {

    // module:
    //      epi-saas-base/mvc/_EmptiableRefControllerMixin

    return declare([], {
        // summary:
        //      Mixin to enable resetting model to a predefined empty state
        // description:
        //      When using *RefControllers, using the commit method overwrites the original model. This mixin enables
        //      resetting controller to a clean slate.

        _refEmptyModelProp: "emptyModel",

        empty: function() {
            // summary:
            //      Sets model to a clone of the empty model defined on instantiation.
            this.set(this.holdModelUntilCommit ? this._refEditModelProp : this._refSourceModelProp, this.cloneModel(this.get(this._refEmptyModelProp)));
        },

        hasControllerProperty: function(/*String*/ name){
            // summary:
            //      Returns true if this controller itself owns the given property.
            // name: String
            //      The property name.
            return this.inherited(arguments) || name == this._refEmptyModelProp;
        }
    });
});
},
'epi-find/optimize/AutocompleteModel':function(){
define([
    "dojo/_base/declare",
    "dojo/Deferred",
    "dojo/regexp",

    "dojo/when",

    "dojo/Stateful",
    "dijit/Destroyable",

    "dojox/mvc/EditStoreRefController",
    "epi-saas-base/mvc/_CommitReturnMixin",
    "epi-saas-base/mvc/_EmptiableRefControllerMixin"
],
    function(declare,
             Deferred,
             regexp,
             when,
             Stateful,
             Destroyable,
             EditStoreRefController,
             _CommitReturnMixin,
             _EmptiableRefControllerMixin) {

        // module:
        //      epi-find/manage/RelatedQueriesModel

        var SearchQuery = function(value, properties) {
            // summary:
            //      This is a native JavaScript object used as a query when requesting backend store.

            this.properties = (properties instanceof Array) ?  properties : [properties];
            this.value = value;

            this.toString = function() {
                return this.value;
            };

            this.test = function(property, object) {
                // summary:
                //      This method used when dojo/store/Observable tries to verify if an object that was just
                //      updated matches to the current query. By default dojo/store/util/SimpleQueryEngine is used
                //      to verify, if an object matches with the current query. It has an ability to provide
                //      a custom verification logic by implementing a 'test' method.

                // replace asterisks and question mark with a correspondent regular expressions
                var escapedValue = regexp.escapeString(this.value, "*?").replace("*", ".*").replace("?", "."),
                    regularExpression = new RegExp(escapedValue, "i"),
                    result = false;
                for (var i=0; i<this.properties.length; i++) {
                    var testValue = object[this.properties[i]];
                    result = result || regularExpression.test(testValue);
                }
                return result;
            };
        };

        return declare([Stateful, Destroyable], {
            // summary:
            //     Model for the Autocomplete view.

            // store: dojo/store/JsonRest
            //      A store that is used for saving data
            store: null,

            // ctrl: dojox/mvc/EditStoreRefController
            //      A controller for binding
            ctrl: null,

            // searchText: String
            //      A search query
            searchText: null,
            _searchTextSetter: function(value) {
                if (value) {
                    var queryValue = {q: new SearchQuery(value + "*", "query")};
                    this.set("gridQuery", queryValue);
                }
                else {
                    this.set("gridQuery", {});
                }
            },

            // gridQuery: Object
            //      Query for the grid
            gridQuery: null,

            postscript: function() {
                this.inherited(arguments);
                var BindingController = declare([EditStoreRefController, _CommitReturnMixin, _EmptiableRefControllerMixin]);
                this.ctrl = new BindingController({
                    store: this.store,
                    emptyModel: new Stateful({priority:1})
                });
                this.gridQuery = {};
            },

            save: function() {
                if (!this.store.getIdentity(this.ctrl.sourceModel)) { // if this is a new item
                    var self = this, results = new Deferred();
                    // calculate priority value for a new item
                    when(this.store.query({}, {
                        start:0,
                        count: 1,
                        sort: [
                            {attribute: "priority", descending: true}
                        ]
                    }), function(topItem) {
                        var topPriority = 0;
                        if (topItem && topItem.length) {
                            topPriority = topItem[0].priority;
                        }
                        // priority for new items is calculated as a quarter of the difference between
                        // maximal priority of existing item and maximal possible priority
                        var newPriority = topPriority + Math.round((self.store.maxPriority - topPriority) / 4);
                        self.ctrl.set("priority", newPriority);
                        when(self.ctrl.commit(), function(commitResults) {
                            if (newPriority === topPriority) {
                                setTimeout(function() {self.store.emit("refresh");}, 2000); // init displayed data refresh
                            }
                            results.resolve(commitResults);
                        }, function(error) {
                            results.reject(error);
                        });
                    }, function(error) {
                        results.reject(error);
                    });
                    return results;
                }
                return this.ctrl.commit();
            },

            remove: function(id) {
                // summary:
                //      Removes an autocomplete entry with specified ID
                return this.store.remove(id);
            }

        });
    });
},
'epi-find/widget/Optimization':function(){
﻿define([
    "dojo/_base/declare",
    "dojo/_base/config",
    "dojo/on",
    "dojo/topic",
    "dojo/query",

    "dojo/dom-class",

    "dijit/_TemplatedMixin",
    "dijit/layout/ContentPane",

    "./_ActionableMixin",
    "./Synonyms",
    "./Bestbets",
    "./RelatedQueries",
    "./Autocomplete",

    "../AccessModel",

    "dojo/i18n!./nls/Optimization",
    "dojo/text!./templates/Optimization.html"
],
function (declare,
        config,
        on,
        topic,
        query,
        domClass,
        _TemplatedMixin,
        ContentPane,
        _ActionableMixin,
        Synonyms,
        BestBets,
        RelatedQueries,
        Autocomplete,
        AccessModel,
        i18n,
        template) {

    // module:
    //      epi-find/widget/Optimization

    return declare([ContentPane, _TemplatedMixin, _ActionableMixin], {
        // summary:
        //     Optimization view.

        i18n: i18n,
        templateString: template,

        _synonyms: null,
        _bestBets: null,
        _relatedQueries: null,

        postCreate: function() {
            this.inherited(arguments);

            this._bestBets = new BestBets({
                scrollNode: this.scrollNode
            });
            this.own(this._bestBets);

            this._relatedQueries = new RelatedQueries({
                model: this.model.relatedQueriesModel,
                scrollNode: this.scrollNode
            });
            this.own(this._relatedQueries);

            this._synonyms = new Synonyms({
                disabledView: !AccessModel.isEnabled(config.intents.view.synonyms),
                scrollNode: this.scrollNode
            });
            this.own(this._synonyms);

            this._autocomplete = new Autocomplete({
                model: this.model.autocompleteModel,
                scrollNode: this.scrollNode
            });
            this.own(this._autocomplete);

            var self = this,
                bestBetsAction = {
                    currentAction: "bestbets",
                    takeAction: function(actions, params) {
                        self.removeChildren();
                        self.addChild(self._bestBets);
                        self._bestBets.takeAction(actions, params);
                        self._switchSelection(self.bestBetsLink.parentNode);
                        return self._bestBets;
                    }
                },
                relatedQueriesAction = {
                    currentAction: "relatedqueries",
                    takeAction: function(actions, params) {
                        self.removeChildren();
                        self.addChild(self._relatedQueries);
                        self._relatedQueries.takeAction(actions, params);
                        self._switchSelection(self.relatedQueriesLink.parentNode);
                        return self._relatedQueries;
                    }
                },
                synonymsAction = {
                    currentAction: "synonyms",
                    takeAction: function(actions, params) {
                        self.removeChildren();
                        self.addChild(self._synonyms);
                        self._synonyms.takeAction(actions, params);
                        self._switchSelection(self.synonymsLink.parentNode);
                        return self._synonyms;
                    }
                },
                autocompleteAction = {
                    currentAction: "autocomplete",
                    takeAction: function(actions, params) {
                        self.removeChildren();
                        self.addChild(self._autocomplete);
                        self._autocomplete.takeAction(actions, params);
                        self._switchSelection(self.autocompleteLink.parentNode);
                        return self._autocomplete;
                    }
                };
            this.currentAction = bestBetsAction.currentAction;
            this.registerAction(bestBetsAction.currentAction, bestBetsAction);
            this.registerAction(relatedQueriesAction.currentAction, relatedQueriesAction);
            this.registerAction(synonymsAction.currentAction, synonymsAction);
            this.registerAction(autocompleteAction.currentAction, autocompleteAction);

            this.own(
                on(this.bestBetsLink, "click", function(event) {
                    topic.publish(config.intents.view.bestBets);
                    event.preventDefault();
                }),
                on(this.relatedQueriesLink, "click", function(event) {
                    topic.publish(config.intents.view.relatedQueries);
                    event.preventDefault();
                }),
                on(this.synonymsLink, "click", function(event) {
                    topic.publish(config.intents.view.synonyms);
                    event.preventDefault();
                }),
                on(this.autocompleteLink, "click", function(event) {
                    topic.publish(config.intents.view.autocomplete);
                    event.preventDefault();
                })
            );

            this.previousActionFallback = true;
        },

        startup: function() {
            this.inherited(arguments);
            this.takeAction([this.currentAction]);
        },

        removeChildren: function () {
             while(this.getChildren().length > 0) {
                this.removeChild(0);
             }
        },

        refresh: function() {
            this._synonyms.refresh();
            this._bestBets.refresh();
            this._relatedQueries.refresh();
            this._autocomplete.refresh();
        },

        hideChildren: function() {
            this.inherited(arguments);
            domClass.add(this.links, "epi-hidden");
        },

        showChildren: function() {
            this.inherited(arguments);
            domClass.remove(this.links, "epi-hidden");
        },

        _switchSelection: function(/*domNode*/ node) {
            // summary:
            //      Removes current selection indication and puts selection indicator to a new node
            query(".epi-pane__content__nav li.is-active", this.domNode).removeClass("is-active");
            domClass.add(node, "is-active");
        }

    });
});
},
'epi-find/widget/Synonyms':function(){
define([
    "dojo/_base/config",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-construct",
    "dojo/dom-class",
    "dojo/store/Memory",
    "dojo/store/Observable",
    "dojo/Stateful",
    "dojo/when",

    "dojox/mvc/EditStoreRefController",

    "dijit/_WidgetBase",
    "dijit/_Container",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/_FocusMixin",
    "dijit/MenuItem",
    "dijit/form/TextBox",

    "dgrid/Grid",
    "dgrid/extensions/DijitRegistry",

    "epi-saas-base/Sticker",
    "epi-saas-base/StickyContainer",
    "epi-saas-base/mvc/_CommitReturnMixin",
    "epi-saas-base/mvc/_EmptiableRefControllerMixin",
    "epi-saas-base/widgets/_NotificationMixin",
    "epi-saas-base/widgets/_GridContextMenuMixin",
    "epi-saas-base/widgets/_GridMoreMixin",
    "epi-saas-base/widgets/_AutoLoadScrollMixin",

    "./_SiblingFocusMixin",
    "./_ActionableMixin",
    "./_DisabledViewMixin",

    "./SynonymForm",

    "epi/shell/widget/ContextMenu",
    "epi/shell/dgrid/Formatter",
    "epi-cms/dgrid/formatters",

    "dojo/i18n!./nls/Synonyms",
    "dojo/text!./templates/Synonyms.html"

],
function(config,
        declare,
        lang,
        domConstruct,
        domClass,
        MemoryStore,
        Observable,
        Stateful,
        when,
        EditStoreRefController,
        _WidgetBase,
        _Container,
        _TemplatedMixin,
        _WidgetsInTemplateMixin,
        _FocusMixin,
        MenuItem,
        TextBox,

        Grid,
        DijitRegistry,

         Sticker,
         StickyContainer,
        _CommitReturnMixin,
        _EmptiableRefControllerMixin,
        _NotificationMixin,
        _GridContextMenuMixin,
        _GridMoreMixin,
        _AutoLoadScrollMixin,

        _SiblingFocusMixin,
        _ActionableMixin,
        _DisabledViewMixin,

        SynonymForm,

        ContextMenu,
        Formatter,
        formatters,
        i18n,
        template) {

    // module:
    //      epi-find/widget/Synonyms

    return declare([_WidgetBase, _Container, _TemplatedMixin, _WidgetsInTemplateMixin, _FocusMixin, _SiblingFocusMixin, _ActionableMixin, _DisabledViewMixin, _NotificationMixin], {
        // summary:
        //     Synonyms view.

        templateString: template,

        i18n: i18n,

        store: null,

        ctrl: null,

        _synonymForm: null,

        _searchText: null,

        _searchUpdateTimeout: 400,
        _searchUpdateTimer: null,

        constructor: function() {
            this.store = new Observable(config.dependencies["epi-find.SynonymStore"]);
            this.own(this.store);

        },

        takeAction: function(actions, params) {
            if (params && params.phrases) {
                this._synonymForm.reset();
                this.ctrl.set("phrase", decodeURIComponent(params.phrases));
            }
            this.grid.refresh();
        },

        refresh: function() {
            this.grid.refresh();
            this._synonymForm.reset();
        },

        postCreate: function() {
            this.inherited(arguments);

            var self = this,
                CommitController = declare([EditStoreRefController, _CommitReturnMixin, _EmptiableRefControllerMixin]);

            this.ctrl = new CommitController({
                store: this.store,
                originalModel: new Stateful({ phrase: null, synonym: null, bidirectional: false }),
                emptyModel:  new Stateful({ phrase: null, synonym: null, bidirectional: false })
            });

            this._synonymForm = new SynonymForm({
                ctrl: this.ctrl
            }, this.synonymForm);
            this._synonymForm.reset();
            this.own(this._synonymForm);
            this._synonymForm.startup();

            var SynonymGrid = declare([Grid, _GridMoreMixin, _AutoLoadScrollMixin, Formatter, _GridContextMenuMixin, _NotificationMixin], {
                    constructor: function() {
                        // Sets the loader (#loadNextPage, available in _GridMoreMixn) to be used by the _AutoLoadScrollMixin
                        this.loader = lang.hitch(this, this.loadNextPage);
                    },
                    i18n: i18n,
                    contextMenu: new ContextMenu(),
                    showHeader: true,
                    margin: 140,
                    columns: {
                        phrase: {
                            label: i18n.fields_phrase,
                            sortable: true
                        },
                        bidirectional: {
                            label: " ",
                            renderCell: function (object, value, node) {
                                var directionNode = domConstruct.create("span", {
                                    "class": "dijitReset dijitInline dijitIcon",
                                    title: value ? i18n.fields_bidirectional : i18n.fields_oneWay
                                });
                                domClass.add(directionNode, value ? "epi-gridIconBiDirectional" : "epi-gridIconDirectional");
                                domConstruct.place(directionNode, node);
                            },
                            sortable: false,
                            className: "epi-grid--5  epi-grid-column--centered"
                        },
                        synonym: {
                            label: i18n.fields_synonym,
                            sortable: true
                        },
                        context: {
                            label: " ",
                            sortable: false,
                            formatter: function() {
                                return formatters.menu({title: i18n.contextMenuTitle});
                            }
                        }
                    }
                }),
                grid = this.grid = new SynonymGrid({
                    "class": "epi-plain-grid epi-plain-grid--no-border",
                    store: self.store,
                    noDataMessage: i18n.noSynonyms,
                    loadingMessage: i18n.dataLoading,
                    showLoadingMessage: true,
                    sort: [{attribute: "phrase", descending: false}]
                }, this.synonymGrid);

            this.own(grid,
                grid.on("dgrid-error", function(event) {
                    grid.showErrorInGrid(event.error);
                }),
                this.searchNode.watch("value", function(name, oldValue, newValue) {
                    self._searchText = newValue;
                    self._delayedReload();
                })
            );

            grid.contextMenu.addChild(new MenuItem({
                label: i18n.edit,
                onClick: function() {
                    var row = grid.getRow(this);
                    self._synonymForm.edit(row.id);
                }
            }));
            grid.contextMenu.addChild(new MenuItem({
                label: i18n["delete"],
                onClick: function() {
                    var row = grid.getRow(this);
                    self.showConfirmation({
                        title: i18n.deleteSynonym,
                        message: i18n.areYouSure,
                        confirmActionText: i18n.buttons_confirmDelete,
                        cancelActionText: i18n.buttons_cancelDelete,
                        action: function() {
                            when(self.store.remove(row.id),
                                function() {
                                    self.showSuccess(i18n.deletedMessage);
                                },
                                function(error) {
                                    self.showError(error);
                                    self.grid.refresh();
                                }
                            );
                        }
                    });
                }
            }));
        },

        _delayedReload: function() {
            //  summary:
            //      Updates grids query with text from the search textbox with delay.
            if (this._searchUpdateTimer) {
                clearTimeout(this._searchUpdateTimer);
            }
            this._searchUpdateTimer = setTimeout(lang.hitch(this, function () {
                this._searchUpdateTimer = null;
                this.grid.set("query", this._searchText ? {q: "(@ |\"\")" + this._searchText.toLowerCase() + ".*"} : {});
            }), this._searchUpdateTimeout);
        },

        startup: function() {
            this.inherited(arguments);
            var scrollNode = this.scrollNode || this.domNode.parentNode;
            this.grid.set("scrollNode", scrollNode);
            this.grid.startup();

            this.own(
                new StickyContainer({
                    scrollNode: scrollNode,
                    stickers: [
                        new Sticker({ domNode: this.synonymsHeader, offset: {top: 0}, limitNode: this.grid.domNode, subscribe: false}),
                        new Sticker({ domNode: this.grid.headerNode, offset: {top: 0}, limitNode: this.grid.domNode, subscribe: false})
                    ]
                })
            );
        }
    });
});

},
'epi-saas-base/StickyContainer':function(){
define("epi-saas-base/StickyContainer", [
    "dojo/_base/declare",
    "dojo/_base/array",
    "dojo/_base/lang",
    "dojo/_base/window",
    "dojo/dom-style",
    "dojo/dom-geometry",

    "./_StickyEventMixin",
    "./Sticker"
], function (declare,
             array,
             lang,
             window,
             domStyle,
             domGeometry,
             _StickyEventMixin,
             Sticker) {
    return declare([_StickyEventMixin], {
        // module:
        //      epi-saas-base/StickyContainer

        stickers: null,

        constructor:function (options) {
            // summary:
            //      Creates the container that owns several stickers that should stick as a stack.
            // options: Object?
            //      The optional set of parameters like the list of stickers, the node to stick to, scroll element:
            //          {
            //              stickers: Array of Sticker objects or HTML elements or widgets that should stick.
            //              stickToNode: HTML element to stick. Document will be used if this option is not specified.
            //              scrollNode: HTML element that has a scrolling that stickers should react to. Windows will be used if this option is not specified.
            //          }
            // tags:
            //      public

            declare.safeMixin(this, options || {});

            this.stickToNode = this.stickToNode || this.scrollNode || window.doc;

            this.stickers = this._prepareStickers(this.stickers && (this.stickers instanceof Array) ? this.stickers : []);

            this.attachEvents(lang.hitch(this, function (resized) {
                this.update(resized);
            }), this.scrollNode, this.stickToNode);

            this.triggerDelayedAction(600);
        },

        update: function(resized) {
            // summary:
            //      Updates the state and position of the contained stickers depending on scrolling, container and other options
            // resized: Boolean?
            //      Indicates whether elements were resized
            // tags:
            //      public

            if (domStyle.get(this.stickToNode, "display") === "none" || this.stickers.length == 0) {
                return;
            }

            var scrollHeight = this.scrollNode ? this.scrollNode.clientHeight : window.doc.height,

                stickToNodePosition = domGeometry.position(this.stickToNode),

                options = {
                    offset: {
                        top: 0,
                        bottom: 0
                    },
                    stickToNodePosition: stickToNodePosition,
                    scrollHeight: scrollHeight
                };

            var unstuckTopLimits = [],
                unstuckBottomLimits = [];

            var unstickIfRequired = lang.hitch(this, function (sticker) {
                if (sticker.stuckTo === "top" &&
                    array.indexOf(unstuckTopLimits, sticker.limitNode || this.stickToNode) >= 0 ) {
                    sticker.unstick();
                    return true;
                }
                if (sticker.stuckTo === "bottom" &&
                    array.indexOf(unstuckBottomLimits, sticker.limitNode || this.stickToNode) >= 0 ) {
                    sticker.unstick();
                    return true;
                }
                return false;
            });

            array.forEach(this.stickers, function (sticker) {
                // calculate sticker position and state
                sticker.update(resized, options);

                // unstick it, if we already know that stickers in this limiter node should be unstuck.
                if (unstickIfRequired(sticker)) {
                    return;
                }

                // store offsets if the element is stuck
                if (sticker.stuckTo) {
                    options.offset[sticker.stuckTo] += sticker.stickerBox.h;
                }
                // check if the element is not stuck and was scrolled out of the limiter node
                else if (sticker.position){
                    // is the sticker above the scrolling area?
                    if (sticker.position.y < stickToNodePosition.y + options.offset.top) {
                        var limitNode = sticker.limitNode || this.stickToNode;
                        if (array.indexOf(unstuckTopLimits, limitNode) == -1) {
                            // store the limiter to unstick all other top stickers that are linked to this limiter
                            unstuckTopLimits.push(limitNode);
                        }
                    }
                    // is the sticker below the scrolling area?
                    if (sticker.position.y > stickToNodePosition.y + scrollHeight - options.offset.bottom) {
                        var limitNode = sticker.limitNode || this.stickToNode;
                        if (array.indexOf(unstuckBottomLimits, limitNode) == -1) {
                            // store the limiter to unstick all other bottom stickers that are linked to this limiter
                            unstuckBottomLimits.push(limitNode);
                        }
                    }
                }
            }, this);

            // if we have any limiters to unstick
            if (unstuckTopLimits.length > 0 || unstuckBottomLimits.length > 0) {
                var hadSomethingToUntick = false;

                // try to unstick any element that is linked to the known limiters
                // and was processed before we knew about that limiters.
                array.forEach(this.stickers, lang.hitch(this, function (sticker) {
                    hadSomethingToUntick |= unstickIfRequired(sticker);
                }), this);

                // if we had unstuck any elements at this point, some stickers may need to recalculate their position
                if (hadSomethingToUntick) {
                    options = {
                        offset: {
                            top: 0,
                            bottom: 0
                        },
                        stickToNodePosition: stickToNodePosition,
                        scrollHeight: scrollHeight
                    };

                    // update all stuck stickers to recalculate their positions
                    array.forEach(this.stickers, lang.hitch(this, function (sticker) {
                        if (sticker.stuckTo) {
                            sticker.update(true, options);
                            if (sticker.stuckTo) {
                                options.offset[sticker.stuckTo] += sticker.stickerBox.h;
                            }
                        }
                    }), this);
                }
            }
        },

        _prepareStickers: function (stickers) {
            var createSticker = function (element) {
                return new Sticker({ domNode: element, offset: {top: 0}, limitNode: this.stickToNode, subscribe: false});
            };
            this.stickers = [];
            stickers = stickers && (stickers instanceof Array) ? stickers : [stickers];
            stickers = array.map(stickers, function (sticker) {
                if (sticker === null || sticker === undefined) {
                    return null;
                }
                if (sticker instanceof Sticker) {
                    return sticker;
                }
                if (sticker instanceof HTMLElement) {
                    return createSticker(sticker);
                }
                if (sticker.domNode instanceof HTMLElement) {
                    return createSticker(sticker.domNode);
                }
            }, this);

            // take and own sticker that are not null
            stickers = array.filter(stickers, function (sticker) {
                return sticker;
            });
            array.forEach(stickers, function (sticker) {
                this.own(sticker);
            }, this);

            // sort by position to make sure that we process stickers in right order
            stickers.sort(function (s1, s2) {
                var p1 = domGeometry.position(s1.domNode),
                    p2 = domGeometry.position(s2.domNode);
                return p1.y - p2.y;
            });
            return stickers;
        }
    });
});
},
'epi-saas-base/widgets/_GridContextMenuMixin':function(){
define("epi-saas-base/widgets/_GridContextMenuMixin", [
    // Dojo
    "dojo/_base/declare",
    "dojo/_base/lang",

    "epi-cms/dgrid/WithContextMenu"

], function(
    declare,
    lang,
    WithContextMenu
) {
    // module: 
    //      epi-saas-base/widgets/_GridContextMenuMixin
    // summary:
    //      This is slightly modified epi-cms/dgrid/WithContextMenu extension for dgrid that
    //      simplifies finding current row in the MenuItem OnClick handler.

    return declare([WithContextMenu], {

        onContextMenuClick: function(e) {
            // summary:
            //      Triggered when a click on the grids context menu is registered.
            // remarks: 
            //      It will open the context menu.
            // tags:
            //      protected

            this.contextMenu.scheduleOpen(e.target, null, {
                x: e.pageX,
                y: e.pageY
            });
        },

        getRow: function(/*Object*/ context) {
            // summary:
            //      Returns row where context menu was opened
            // context:
            //      Here should be specified current context of the OnClick event

            return this.row(context.getParent().currentTarget);
        }
    });

});
},
'epi-cms/dgrid/WithContextMenu':function(){
define("epi-cms/dgrid/WithContextMenu", [
    "dojo/_base/declare",
    "dojo/_base/event",
    "dojo/dom-geometry",
    "dojo/dom-style",
    "dojo/on",
    "dojo/query",
    "dgrid/extensions/DijitRegistry",
    "epi/shell/DestroyableByKey",
    "epi/shell/widget/ContextMenu",
    "epi-cms/extension/events"
], function (
    declare,
    event,
    domGeometry,
    domStyle,
    on,
    query,
    DijitRegistry,
    DestroyableByKey,
    ContextMenu,
    events
) {

    return declare([DestroyableByKey, DijitRegistry], {
        // summary:
        //      Extension for dgrid that adds a context menu.
        // tags:
        //      public

        // contextMenu: [public] Object
        //      The context menu widget that is bound to the context menu node of each row
        //      in a grid or list.
        contextMenu: null,

        // commandCategory: [public] String
        //      The command category to be used when determining which command should
        //      appear in the context menu.
        commandCategory: "context",

        // stopPropagation: [public] Boolean
        //      Bool to stop propagation when clicking the context menu
        stopPropagation: false,

        // _contextMenuClass: [private] String
        //      The CSS class used to identify the context menu node within a dgrid-row.
        _contextMenuClass: "epi-iconContextMenu",

        postCreate: function () {
            // summary:
            //      Called after the list or grid has finished being created but before
            //      startup is called.
            // tags:
            //      protected

            this.inherited(arguments);

            this.contextMenu = this.contextMenu || new ContextMenu({ category: this.commandCategory, popupParent: this });

            this.own(
                this.contextMenu,
                this.on("." + this._contextMenuClass + ":click", this._contextMouseHandler.bind(this)),
                this.on(on.selector(".dgrid-row", events.keys.shiftf10), this._contextKeyHandler.bind(this))
            );
        },

        startup: function () {
            // summary:
            //      Called automatically after postCreate if the component is already
            //      visible; otherwise, it should be called manually once placed.
            // tags:
            //      protected

            this.inherited(arguments);

            this.contextMenu.startup();
        },

        onContextMenuClick: function (e) {
            // summary:
            //      Opens the context menu at the position the event occurred.
            // tags:
            //      protected callback

            // If the grid supports selection then ensure the row is selected before opening the
            // context menu.
            if (this.selection) {
                this.select(e.target);
            }

            on.emit(e.target, "dgrid-contextmenu", { bubbles: true });

            this.contextMenu.scheduleOpen(this.domNode, null, {
                x: e.pageX,
                y: e.pageY
            });
        },

        _extendedSelectionHandler: function (e, target) {
            // summary:
            //      Extend the selection handler for "extended" mode, so that clicks on
            //      the context menu of a selected row will not clear the selection.
            // tags:
            //      protected

            var hasContextMenuClass = new RegExp("\\b" + this._contextMenuClass + "\\b");

            // If we are clicking on the context menu and the target row is already
            // selected, then do an early exit in order to avoid deselecting the currently
            // selected rows.
            if (hasContextMenuClass.test(e.target.className) && this.isSelected(target)) {
                return;
            }

            this.inherited(arguments);
        },

        _isNodeVisible: function (domNode) {
            // summary:
            //      Checks the computed style on a domNode to see if display is other than "none"
            //      and visibility is other than "hidden".
            // tags:
            //      private

            if (!domNode) {
                return false;
            }
            var style  = domStyle.get(domNode);
            return style.display !== "none" && style.visibility !== "hidden";
        },

        _contextMouseHandler: function (e) {
            // tags:
            //      private

            if (this.stopPropagation) {
                e.stopImmediatePropagation();
                e.stopPropagation();
            }

            this.onContextMenuClick(e);
        },

        _contextKeyHandler: function (e) {
            // tags:
            //      private

            var contextMenuNode = query("." + this._contextMenuClass, e.target)[0],
                position;

            if (!this._isNodeVisible(contextMenuNode)) {
                return;
            }

            e.stopPropagation();
            e.preventDefault();
            position = domGeometry.position(contextMenuNode);

            this.onContextMenuClick({
                target: contextMenuNode,
                pageX: position.x,
                pageY: position.y
            });
        }
    });
});

},
'epi/shell/widget/ContextMenu':function(){
﻿define("epi/shell/widget/ContextMenu", [
    "dojo/_base/declare",
    "dojo/aspect",
    "dijit/Menu",
    "dijit/popup",

    "epi/shell/command/_CommandConsumerMixin",
    "epi/shell/command/builder/MenuAssembler",
    "epi/shell/command/builder/MenuBuilder",

    "epi/shell/command/builder/MenuWithSeparatorBuilder"
], function (
    declare,
    aspect,
    Menu,
    popup,

    _CommandConsumerMixin,
    MenuAssembler,
    MenuBuilder,
    MenuWithSeparatorBuilder
) {

    var ContextMenu;

    aspect.before(popup, "open", function (args) {
        // Propagate the popupParent property from our ContextMenu to the dijit popup machinery.
        // This keeps the popupParent, the owner of our popup, in the dijit.focus.activeStack.
        // Otherwise we'll lose the dijitFocused class from the parent and the onBlur will
        // be raised on it, etc.

        var popup = args.popup;
        if (popup && popup instanceof ContextMenu && popup.popupParent) {
            args.parent = popup.popupParent;
            return [args];
        }
    });

    return ContextMenu = declare([Menu, _CommandConsumerMixin], {
        // summary:
        //      A context menu that can display context commands for an item.
        //
        // tags:
        //      public

        // category: string
        //		If set the menu will be filtered on the category.
        category: null,

        // leftClickToOpen: [readonly] Boolean
        //		Menu will open on left click instead of right click.
        leftClickToOpen: true,

        postCreate: function () {

            this.inherited(arguments);

            var defaultConfiguration = { builder: new MenuBuilder(), target: this, category: this.category },
                assembler;

            var configuration = [
                defaultConfiguration,
                { builder: new MenuWithSeparatorBuilder(), target: this, category: "menuWithSeparator" },
                { builder: new MenuBuilder(), target: this, category: "popup" }
            ];
            assembler = new MenuAssembler({ configuration: configuration, commandSource: this });

            assembler.build();
        },

        scheduleOpen: function (/*DomNode?*/delegatedTarget, /*DomNode?*/iframe, /*Object?*/coords, /*DomNode?*/target) {
            // summary:
            //      Display the menu
            // remarks:
            //      See more details in the base class(Dijit/Menu)
            // tags:
            //      public

            this._scheduleOpen(delegatedTarget, iframe, coords, target);
        }
    });
});

},
'epi/shell/command/_CommandConsumerMixin':function(){
﻿define("epi/shell/command/_CommandConsumerMixin", [
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang"
], function (array, declare, lang) {

    return declare(null, {
        // summary:
        //      A mixin for objects that will consume commands from providers.
        // tags:
        //      private

        // providers: [readonly] Array
        //		Array of providers that commands are being consumed from.
        providers: null,

        constructor: function () {
            // summary:
            //		Ensure that an array of providers has been initialized.
            // tags:
            //		public

            this.providers = this.providers || [];
        },

        destroy: function () {
            // summary:
            //		Detach all the watch handles before uninitializing.
            // tags:
            //		public

            this.inherited(arguments);

            // make sure no providers are left behind with watches
            array.forEach(this.providers, function (item) {
                item.watch.unwatch();
            });
        },

        getConsumer: function () {
            // summary:
            //		Returns a command consumer, in this case this.
            return this;
        },

        addProvider: function (provider) {
            // summary:
            //		Add a provider to consume commands from.
            // tags:
            //		public

            var watch = provider.watch("commands", lang.hitch(this, this._commandsChanged));

            this.providers.push({
                provider: provider,
                watch: watch
            });

            // Notify a change to commands with the added provider's commands.
            this.onCommandsChanged("commands", null, provider.commands);

            return {
                removeProvider: lang.hitch(this, this.removeProvider, provider)
            };
        },

        removeProvider: function (provider) {
            // summary:
            //		Remove a provider that commands are being consumed from.
            // tags:
            //		public

            var providers = this.providers;

            // Find the index of the given provider.
            for (var i = providers.length - 1; i >= 0; i--) {
                if (providers[i].provider === provider) {
                    break;
                }
            }

            if (i >= 0) {
                var item = providers.splice(i, 1)[0];

                // release watch
                item.watch.unwatch();
                item = null;

                // Notify a change to commands with the removed provider's commands.
                this.onCommandsChanged("commands", provider.commands, null);
            }
        },

        getCommands: function () {
            // summary:
            //		Gets all the commands being consumed from providers.
            // tags:
            //		public

            var commands = [];
            array.forEach(this.providers, function (item) {
                commands = commands.concat(item.provider.commands);
            });
            return commands;
        },

        onCommandsChanged: function (name, removed, added) {
            // summary:
            //		Callback when available commands have been changed.
            // name: String
            //      Name of the affected command collection
            // removed: Array
            //      An array of removed commands
            // added: Array
            //      An array of added commands
            // tags:
            //		public callback
        },

        _commandsChanged: function () {
            this.onCommandsChanged.apply(this, arguments);
        }
    });
});

},
'epi/shell/command/builder/MenuAssembler':function(){
﻿define("epi/shell/command/builder/MenuAssembler", [
    "dojo/_base/array",
    "dojo/_base/connect",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/Stateful",
    "epi/shell/_StatefulGetterSetterMixin"
],
function (array, connect, declare, lang, Stateful, _StatefulGetterSetterMixin) {

    return declare([Stateful, _StatefulGetterSetterMixin], {
        // summary:
        //      The MenuAssembler serves as a bridge between command sources, i.e. an instance of
        //      epi/shell/command/_CommandConsumer and the builder instances inheriting from
        //      epi/shell/command/builder/_Builder
        //
        // tags: private

        // commandSource: [public] epi/shell/command/_CommandConsumerMixin
        //      The source providing commands for this assembler. Use set("commandSource", value) when updating this property
        commandSource: null,

        // configuration: [public]
        //
        configuration: null,

        // _commandSourceConnect: [private]
        //      A reference to the event listener for observing command changes on the commandSource
        _commandSourceConnect: null,

        destroy: function () {
            connect.disconnect(this._commandSourceConnect);
        },

        build: function () {
            // summary:
            //      Trigger an explicit ui assembly of the currently available commands
            // tags: public

            this.set("commandSource", this.commandSource);
        },

        _setCommandSourceAttr: function (value) {
            // summary:
            //      Sets the commandSource property and re-assembles the ui using the new commands
            // tags: private

            if (this._commandSourceConnect && this.commandSource) {
                // When used from the build method and set for the first time we don't wanna tear down anything that hasn't yet been created.
                // When this._commandSourceConnect is set it means that we've populated previously.
                connect.disconnect(this._commandSourceConnect);
                this._removeCommands(this.commandSource.getCommands());
            }
            this.commandSource = value;
            if (this.commandSource) {
                this._addCommands(this.commandSource.getCommands());
                this._commandSourceConnect = connect.connect(this.commandSource, "onCommandsChanged", this, this._commandsChanged);
            }
        },

        _commandsChanged: function (name, removed, added) {
            // summary:
            //      Event handler invoked when the available commands changes
            // name: String
            //      Name of the affected array in the observed object
            // removed: Array
            //      An array of removed commands
            // added: Array
            //      An array with added commands
            // tags: private

            if (removed) {
                this._removeCommands(removed);
            }
            if (added) {
                this._addCommands(added);
            }
        },

        _getBuildInfo: function (command) {
            // summary:
            //      Return the builder information for a specific command based on its category property
            // tags: private

            var buildInfo,
                category = command.category;

            array.some(this.configuration, function (item) {
                if (item.category == category) { // eslint-disable-line eqeqeq
                    buildInfo = item;
                    return true;
                }
                return false;
            });

            return buildInfo;
        },


        _addCommands: function (commands) {
            // summary:
            //      Adds a collection of commands to the
            // commands: Array[epi/shell/command/_Command]
            //      An array of commands to build a ui representation for
            // tags: private

            array.forEach(commands, this._addCommand, this);
        },

        _addCommand: function (command) {
            // summary:
            //
            // command: epi/shell/command/_Command
            //      An array of commands to build a ui representation for
            // tags: private

            var buildInfo = this._getBuildInfo(command);
            if (buildInfo) {
                buildInfo.builder.create(command, buildInfo.target);
            }
        },

        _removeCommands: function (commands) {
            // summary:
            //		Removes the commands from the toolbar.
            // commands: Array[epi/shell/command/_Commands]
            // tags:
            //		private

            array.forEach(commands, function (command) {
                var buildInfo = this._getBuildInfo(command);
                if (buildInfo) {
                    buildInfo.builder.remove(command, buildInfo.target);
                }
            }, this);
        }

    });

});

},
'epi/shell/_StatefulGetterSetterMixin':function(){
﻿define("epi/shell/_StatefulGetterSetterMixin", [
    "dojo/_base/declare",
    "dojo/_base/lang"
],

function (
    declare,
    lang
) {

    return declare(null, {
        // summary:
        //		A mixin for Stateful object with getters and setters like the dijit Widget implementation.
        //		Each property can be accessed or set with a function like:
        //
        //		`myObject.get("propertyName")` will be handled by `myObject._getPropertyNameAttr()` if it exists
        //		and `myObject.set("propertyName", value)` will be handled by
        //		`myObject._setPropertyNameAttr(value)` if it exists
        //
        // tags:
        //    internal

        get: function (name) {
            // summary:
            //		Get a property.
            //	name:
            //		The property to get.
            // description:
            //		Get a named property. The property may
            //		potentially be retrieved via a getter method. If no getter is defined, this
            //		just retrieves the object's property.

            var names = this._getAttrNames(name);
            return this[names.g] ? this[names.g]() : this[name];
        },

        set: function (name, value) {
            // summary:
            //		Set a property
            //	name:
            //		The property to set.
            //	value:
            //		The value to set in the property.
            // description:
            //		Sets named properties on this object which may potentially be handled by a
            //		setter.
            //
            //		set() may also be called with a hash of name/value pairs, ex:
            //
            //	|	myWidget.set({
            //	|		foo: "Howdy",
            //	|		bar: 3
            //	|	});

            if (typeof name === "object") {
                for (var x in name) {
                    this.set(x, name[x]);
                }
                return this;
            }

            var names = this._getAttrNames(name);
            var setter = this[names.s];

            if (lang.isFunction(setter)) {
                // use the explicit setter
                setter.apply(this, Array.prototype.slice.call(arguments, 1));
            } else {
                this._set(name, value);
            }

            return this;
        },

        _attrPairNames: {},  // shared between all widgets

        _getAttrNames: function (name) {
            // summary:
            //		Helper function for get() and set().
            //		Caches attribute name values so we don't do the string ops every time.
            // tags:
            //		private

            var apn = this._attrPairNames;
            if (apn[name]) {
                return apn[name];
            }
            var uc = name.replace(/^[a-z]|-[a-zA-Z]/g, function (c) {
                return c.charAt(c.length - 1).toUpperCase();
            });
            return (apn[name] = {
                s: "_set" + uc + "Attr", // converts dashes to camel case, ex: accept-charset --> _setAcceptCharsetAttr
                g: "_get" + uc + "Attr",
                l: uc.toLowerCase()		// lowercase name w/out dashes, ex: acceptcharset
            });
        },

        _set: function (/*String*/name, /*anything*/value) {
            // summary:
            //		Helper function to set new value for specified attribute, and call handlers
            //		registered with watch() if the value has changed.

            var oldValue = this[name];
            this[name] = value;
            if (this._watchCallbacks) {
                this._watchCallbacks(name, oldValue, value);
            }
            return this;
        }
    });
});

},
'epi/shell/command/builder/MenuBuilder':function(){
﻿define("epi/shell/command/builder/MenuBuilder", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dijit/Menu",
    "dijit/MenuItem",
    "dijit/CheckedMenuItem",
    "dijit/PopupMenuItem",
    "epi/shell/command/builder/_Builder",
    "epi/shell/command/builder/OptionsBuilderMixin",
    "epi/shell/command/_CommandModelBindingMixin"
], function (declare, lang, Menu, MenuItem, CheckedMenuItem, PopupMenuItem, _Builder, OptionsBuilderMixin, _CommandModelBindingMixin) {

    return declare([_Builder], {
        // summary:
        //      Builds a drop down button widget with commands as child menu items.
        // tags:
        //      internal

        // _menuItemClass: [private] class
        //      The class used for representing a menu item
        _menuItemClass: declare([MenuItem, _CommandModelBindingMixin]),

        // _checkedMenuItemClass: [private] class
        //      The class used for representing a checked menu item for a checked command
        _checkedMenuItemClass: declare([CheckedMenuItem, _CommandModelBindingMixin]),

        // _popupMenuItemClass: [private] class
        //      The class used for representing a popup menu item for a option command
        _popupMenuItemClass: declare([PopupMenuItem, _CommandModelBindingMixin]),

        _optionsMenuClass: declare([Menu, OptionsBuilderMixin]),

        _create: function (command) {
            // summary:
            //		Builds a menu item widget from the given command.
            // tags:
            //		public

            var isToggle = typeof command.active == "boolean",
                isOption = lang.isArray(command.options),
                isPopup = "popup" in command,
                cls = isToggle ? this._checkedMenuItemClass : ((isOption || isPopup) ? this._popupMenuItemClass : this._menuItemClass),
                options = lang.mixin({ model: command }, this.itemSettings);

            var widget = new cls(options);

            // If creating a command that already has a popup menu then associate it with the widget.
            if (isPopup) {
                if (command.popupClass) {
                    // Construct the popup class defined in the command.
                    command.set("popup", new command.popupClass({ category: command.popupCategory }));
                }
                widget.popup = command.popup;
            }

            // If creating a command that has options then create the options on the widget.
            if (isOption) {
                widget.popup = new this._optionsMenuClass({model: command});
            }

            return widget;
        }
    });
});

},
'epi/shell/command/builder/_Builder':function(){
﻿define("epi/shell/command/builder/_Builder", [
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dijit/registry"
], function (array, declare, lang, registry) {

    return declare(null, {
        // summary:
        //      Builds a widget and connects events based on a command object.
        //
        // tags:
        //      internal xproduct

        // settings: [public] Object
        //		An optional settings object that will be applied to all widgets created.
        settings: null,

        postscript: function (mixin) {
            // summary:
            //		Mix constructor arguments into the object after construction.
            // tags:
            //		protected

            if (mixin) {
                lang.mixin(this, mixin);
            }
        },

        create: function (command, container) {
            // summary:
            //		Creates a widget from the given command and adds it to the given container.
            // tags:
            //		public

            var widget = this._create(command);

            // Create back-reference to command so that we can identify this widget.
            widget._command = command.settings && command.settings.model ? command.settings.model : command;
            widget._commandCategory = command.category;
            this._addToContainer(widget, container);
        },


        remove: function (command, container) {
            // summary:
            //      Remove the ui representation of a specific command from a container
            // command: epi/shell/command/_Command
            //      The command to remove
            // container: dijit/_Widget
            //      The container displaying the command
            // tags:
            //      public

            var children = container.getChildren(),
                len = children.length,
                i = 0;

            for (; i < len; i++) {
                var child = children[i];
                if (child._command === command) {
                    container.removeChild(child);
                    child.destroy();
                    return true;
                }
            }
            return false;
        },

        _create: function (/*===== command =====*/) {
            // summary:
            //		Builds a widget from the given command. Subclasses should override this method.
            // tags:
            //		protected
        },

        _addToContainer: function (widget, container) {
            // summary:
            //		Adds the widget to the container.
            // tags:
            //		protected
            var insertIndex = null;

            // If the command has specified a order try to find the insert index
            if (widget._command) {
                var order = this._getSortOrder(widget);
                this._getChildren(container).some(function (child, index) {
                    var childOrder = this._getSortOrder(child);
                    if (childOrder > order) {
                        insertIndex = index;
                        // Break out of some
                        return true;
                    }
                }, this);
            }

            widget.placeAt(container, insertIndex);
        },

        _getSortOrder: function (widget) {
            // summary:
            //      Get the sort order for the widget
            //      If the command and/or the order of the command is undefined
            //      the order will be 0
            // widget: Widget
            //      The widget to get the sort order for
            //
            // tags:
            //      protected

            return (widget._command && widget._command.order) ? widget._command.order : 0;
        },

        _getChildren: function (container) {
            // summary:
            //      Gets the child widgets
            //
            // container: DomNode|Widget
            //      The container to get the children for
            // tags:
            //      protected

            // If the container is a container widget, get the children using getChildren
            // otherwise try to get the child widgets using the dijit registry
            if (typeof (container.getChildren) === "function") {
                return container.getChildren();
            } else {
                return registry.findWidgets(container);
            }
        }
    });
});

},
'epi/shell/command/builder/OptionsBuilderMixin':function(){
define("epi/shell/command/builder/OptionsBuilderMixin", [
    "epi",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/on",
    "dojox/html/entities",
    "dijit/CheckedMenuItem",
    "../OptionGroup",
    "epi/shell/widget/MenuHeaderSeparator",
    "epi/shell/DestroyableByKey"
], function (
    epi,
    declare,
    lang,
    on,
    entities,
    CheckedMenuItem,
    OptionGroup,
    MenuHeaderSeparator,
    DestroyableByKey) {

    return declare([DestroyableByKey], {
        // summary:
        //      Mixin class for creating option menu items from the options collection provided by an OptionCommand and add them to the dijit/Menu
        // tags:
        //      internal

        // optionItemClass: [public] String
        //      An optional CSS class added to all created menu items
        optionItemClass: "",

        constructor: function (options) {
            declare.safeMixin(this, options);
        },

        startup: function () {
            this.inherited(arguments);

            //Add the options
            this._addOptionMenuItems();
        },

        _addOptionMenuItems: function () {
            // summary:
            //      Adds the menu items from the options collection in the model to the menu
            // tags:
            //      private

            // We only want to create the menu items when the widget has started
            if (!this._started) {
                return;
            }

            // Destroy the old options
            this.getChildren().forEach(function (child) {
                this.removeChild(child);
                if (typeof child.destroy === "function") {
                    child.destroy();
                }
            }, this);

            this._createOptionMenuItems(this.model).forEach(this.addChild.bind(this));
        },

        _createOptionMenuItems: function (model) {
            // summary:
            //      Creates menu items from the options collection in the supplied model.
            //      Where model is an OptionCommand or an OptionGroup
            //
            // returns:
            //      An array with the created menu items
            //
            // tags:
            //      protected

            var items = [];

            var options = model.get("options");
            if (!lang.isArray(options)) {
                return items;
            }

            if (model.optionsLabel) {
                items.push(new MenuHeaderSeparator({label: model.optionsLabel}));
            }

            options.forEach(function (option) {
                if (option instanceof OptionGroup) {
                    items.push(new MenuHeaderSeparator({label: option.label}));
                    items = items.concat(this._createOptionMenuItems(option));
                } else {
                    items.push(this._createMenuItem(option, model));
                }
            }, this);

            return items;
        },

        _setModelAttr: function (model) {
            this._set("model", model);

            // Listen to changes on the options property and re-create the options if it has changed
            this.destroyByKey("optionsWatch");
            this.ownByKey("optionsWatch", model.watch("options", this._addOptionMenuItems.bind(this)));
        },

        _createMenuItem: function (option, model) {
            // summary:
            //      Creates a menu item from an option and binds it to the model
            //
            // tags: private

            var menuItem = new CheckedMenuItem({
                label: entities.encode(option.label),
                checked: epi.areEqual(model.get("selected"), option.value),
                "class": this.optionItemClass
            });
            menuItem.own(on(menuItem, "click", function () {
                model.set("selected", option.value);
            }));
            menuItem.own(model.watch("selected", function (name, oldValue, newValue) {
                menuItem.set("checked", epi.areEqual(newValue, option.value));
            }));

            return menuItem;
        }

    });
});

},
'epi/shell/command/OptionGroup':function(){
define("epi/shell/command/OptionGroup", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/Stateful",
    "epi/shell/DestroyableByKey"
], function (declare, lang, Stateful, DestroyableByKey) {

    return declare([Stateful, DestroyableByKey], {
        // summary:
        //      Defines a group of options when creating option menus
        // tags: internal

        // label: [public] String
        //      The heading for the option group. To be used in visual elements.
        label: "",

        // options: [public] Array
        //      An array of options objects within this group.
        options: null,

        // model: [public] Object
        //      The model on which the selected option is updated.
        model: null,

        // selected: [public] String|Number
        //      The value of the selected item from the options array.
        selected: null,

        // selected: [public] String
        //      Name of the property on the model to update when the selected option changes.
        property: "",

        constructor: function () {
            this.options = [];
        },

        _selectedSetter: function (value) {
            // summary:
            //      Updates the selected property if changed and propagates the new value to the backing model.
            if (this.selected !== value) {
                this.selected = value;
                this.model && this.model.set(this.property, value);
            }
        },

        _modelSetter: function (model) {
            this.destroyByKey("_model");
            this.model = model;
            if (model) {
                this.set("selected", model.get(this.property));
                this.ownByKey("_model", model.watch(this.property, lang.hitch(this, function (name, oldValue, newValue) {
                    this.set("selected", newValue);
                })));
            }
        }

    });
});


},
'epi/shell/widget/MenuHeaderSeparator':function(){
require({cache:{
'url:epi/shell/widget/templates/MenuHeaderSeparator.html':"<tr class=\"dijitMenuSeparator epi-invertedTooltip\" role=\"separator\">\n    <td colspan=\"4\">\n        <div class=\"epi-tooltipDialogTop\">\n            <span data-dojo-attach-point=\"labelNode\"></span>\n            <div class=\"dijitTooltipConnector\"></div>\n        </div>\n    </td>\n</tr>\n"}});
define("epi/shell/widget/MenuHeaderSeparator", [
    "dojo/_base/declare",
    "dijit/MenuSeparator",
    "dojo/text!./templates/MenuHeaderSeparator.html"
],
function (declare, MenuSeparator, template) {

    return declare([MenuSeparator], {
        // summary:
        //      Drop down menu header. A fancy menu separator with a title.
        // tags:
        //      internal

        templateString: template,

        // label: [public] String
        //      Sets the label text in the header
        _setLabelAttr: {node: "labelNode", type: "innerHTML" }
    });
});

},
'url:epi/shell/widget/templates/MenuHeaderSeparator.html':"<tr class=\"dijitMenuSeparator epi-invertedTooltip\" role=\"separator\">\n    <td colspan=\"4\">\n        <div class=\"epi-tooltipDialogTop\">\n            <span data-dojo-attach-point=\"labelNode\"></span>\n            <div class=\"dijitTooltipConnector\"></div>\n        </div>\n    </td>\n</tr>\n",
'epi/shell/command/_CommandModelBindingMixin':function(){
﻿define("epi/shell/command/_CommandModelBindingMixin", [
    "dojo/_base/declare",
    "dojo/dom-style",
    "dojo/dom-class",
    "../widget/_ModelBindingMixin"
],
function (
    declare,
    domStyle,
    domClass,
    _ModelBindingMixin
) {

    return declare(_ModelBindingMixin, {
        // summary:
        //      A widget mixin for binding command properties to widget properties.
        //
        // tags:
        //      internal xproduct

        // viewModelBindingMap: [protected] Object
        //      Configuration of command property to widget property mappings.
        modelBindingMap: {
            label: ["label"],
            tooltip: ["tooltip"],
            iconClass: ["iconClass"],
            canExecute: ["canExecute"],
            isExecuting: ["isExecuting"],
            isAvailable: ["isAvailable"],
            active: ["checked", "isExpand"]
        },

        _setCanExecuteAttr: function (/*Boolean*/canExecute) {
            // summary:
            //      Sets the widget as enabled or disabled based on the value of canExecute
            // tags:
            //      private

            this.set("disabled", !canExecute);
        },

        _setIsAvailableAttr: function (/*Boolean*/isAvailable) {
            // summary:
            //      Sets the widget as visible or invisible based on the value of isAvailable
            // tags:
            //      private

            if (this.domNode) {
                domStyle.set(this.domNode, "display", isAvailable ? "" : "none");
            }
        },

        _setIsExecutingAttr: function (/*Boolean*/isExecuting) {
            // summary:
            //      Adds or removes a css class while the command is executing
            // tags:
            //      private

            this.isExecutingClass && domClass.toggle(this.domNode, this.isExecutingClass, isExecuting);
        },

        onClick: function () {
            // summary:
            //      Event raised when the button is clicked
            //
            // tags:
            //      public

            this.inherited(arguments);
            this.model && this.model.execute.apply(this.model, arguments);
        }
    });

});

},
'epi/shell/widget/_ModelBindingMixin':function(){
﻿define("epi/shell/widget/_ModelBindingMixin", [
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "epi/shell/DestroyableByKey"],

function (array, declare, lang, DestroyableByKey) {

    return declare([DestroyableByKey], {
        // summary:
        //      Add the capability to bind a widget to a view model object.
        //      By providing a binding map, the widget can observe the view model and update its properties accordingly.
        //      The purpose is to separate presentation logic from the widgets and make them testable. The widget properties
        //      can then be reflected in the dom by using dijit attribute mapping.
        //      The mixin will take care of
        //
        // description:
        //      Declare the binding map in widget declaration to enable property propagtation from model to widget. One property
        //      in the model can be propagated to many widget properties.
        //
        //      |   define([
        //      |       "dojo/dom-style",
        //      |       "dijit/_Widget",
        //      |       "dijit/_TemplatedMixin",
        //      |       "epi/shell/widget/_ModelBindingMixin"
        //      |   ],
        //      |
        //      |   function (domStyle, _Widget, _TemplatedMixin, _ModelBindingMixin) {
        //      |       return dojo.declare([_Widget, _TemplatedMixin, _ModelBindingMixin], {
        //      |
        //      |           templateString: '<div><div data-dojo-attach-point="statusTextNode"></div><div data-dojo-attach-point="statusTextNode2"></div></div>',
        //      |
        //      |           // Declare view model binding
        //      |           modelBindingMap: {
        //      |               "statusText": ["statusText", "statusText2"],
        //      |               "visible": ["visible"],
        //      |           },
        //      |
        //      |
        //      |           // other formats supported by modelBindingMap:
        //      |           // * array of properties mapped 1-1
        //      |           //      this.modelBindingMap = ["prop1", "prop2", "prop3"]
        //      |           //      is equivalent to:
        //      |           //      this.modelBindingMap = {
        //      |           //         "prop1": ["prop1"],
        //      |           //         "prop2": ["prop2"],
        //      |           //         "prop3": ["prop3"]
        //      |           //      }
        //      |
        //      |           // * values of modelBindingMap are strings instead of array of strings
        //      |           //      this.modelBindingMap = {
        //      |           //         "prop1": "widgetProperty1",
        //      |           //         "prop2": "widgetProperty2",
        //      |           //         "prop3": "widgetProperty3"
        //      |           //      }
        //      |           //     equivalent is same object with values surrounded as array
        //      |
        //      |           _setModelAttr: function(newModel) {
        //      |               this.inherited(arguments);
        //      |
        //      |               // Own any additional model bindinds (i.e. on or connects) here if needed,
        //      |               // inherited call will release model handles, so we don't need to do at destroyByKey
        //      |               // this.ownByKey(this.model, handle1, handle2)
        //      |           },
        //      |
        //      |           _setStatusTextAttr: { node: "statusTextNode", type: "innerHTML" },
        //      |
        //      |           _setStatusText2Attr: { node: "statusTextNode2", type: "innerHTML" },
        //      |
        //      |           _setVisibleAttr: function (val) {
        //      |               domStyle.set(this.domNode, "visibility", val ? "visible" : "hidden");
        //      |           },
        //      |
        //      |           // The other stuffs come here
        //      |       });
        //      |   });
        //
        // tags:
        //    internal

        // model: dojo/Stateful
        //      The view model which is a stateful object.
        model: null,

        // modelBindingMap: Object
        //      The binding map. Each key in the map must be an observable property in the model and each item in the map
        //      is a list of widget properties
        modelBindingMap: null,

        constructor: function (options) {
        },

        postMixInProperties: function () {
            this.inherited(arguments);

            if (!this.modelBindingMap) {
                this.modelBindingMap = {};
            }
        },

        _setModelAttr: function (value) {
            // remove existing handles
            this.destroyByKey(this.model);

            this._set("model", value);

            this._setupModelBindings();
        },

        _setupModelBindings: function () {

            if (!this.model) {
                return;
            }

            var callback = lang.hitch(this, "_modelWatch");

            var mappings = this._getNormalizedBindingMap();

            for (var modelProperty in mappings) {
                var widgetProperties = mappings[modelProperty];

                this.ownByKey(this.model, this.model.watch(modelProperty, callback));

                // copy intial values to widget
                array.forEach(widgetProperties, function (widgetProperty) {
                    this.set(widgetProperty, this.model.get(modelProperty));
                }, this);

            }
        },

        _modelWatch: function (modelProperty, oldValue, value) {
            var mappings = this._getNormalizedBindingMap();

            var widgetProperties = mappings[modelProperty];

            array.forEach(widgetProperties, function (widgetProperty) {
                this.set(widgetProperty, value);
            }, this);
        },

        _getNormalizedBindingMap: function () {
            // convert modelBindingMap to format:
            /*
             this.modelBindingMap = {
                 "prop1" = ["prop1"],
                 "prop2"= ["prop2"],
                 "prop3"= ["prop3"],
             }
             */

            var mappings = {};

            // modelBindingMap is an array and model properties match 1-1
            /*
                this.modelBindingMap = ["prop1", "prop2", "prop3"];
             */
            if (this.modelBindingMap instanceof Array) {
                this.modelBindingMap.forEach(function (property) {
                    mappings[property] = [property];
                });

                return mappings;
            }

            // modelBindingMap is an object
            /*
                this.modelBindingMap = { "prop1": ["widgetProp1"], "prop2": ["widgetProp2"] };
             */
            for (var modelProperty in this.modelBindingMap) {
                if (!this.modelBindingMap.hasOwnProperty(modelProperty)) {
                    continue;
                }
                var widgetProperties = this.modelBindingMap[modelProperty];

                // check if value of bindingMap is an array and convert if necessary
                /*
                    prop1: "widgetProp1"  =>  prop1: ["widgetProp1"]
                 */
                if (!(widgetProperties instanceof Array)) {
                    widgetProperties = [widgetProperties];
                }

                mappings[modelProperty] = widgetProperties;
            }

            return mappings;
        }
    });
});

},
'epi/shell/command/builder/MenuWithSeparatorBuilder':function(){
﻿define("epi/shell/command/builder/MenuWithSeparatorBuilder", [
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/on",

    "dijit/MenuSeparator",
    "./MenuBuilder"
], function (array, declare, lang, on, MenuSeparator, MenuBuilder) {

    return declare([MenuBuilder], {
        // summary:
        //      Builds a context menu with separator.
        //
        // tags:
        //      internal

        _addToContainer: function (widget, container) {
            // summary:
            //		Adds the widget to the container.
            // tags:
            //		protected

            this.inherited(arguments, [widget, container]);

            // Add the separator
            var separator = new MenuSeparator({ _command: { order: widget._command.order } });
            this.inherited(arguments, [separator, container]);
        }
    });
});

},
'epi-cms/extension/events':function(){
﻿define("epi-cms/extension/events", [
    "dojo/keys",                                            // used to detect keyboard event
    "dojo/on"                                               // used to query dom by query string inside this.domNode
], function (
    keys,
    on
) {
    // =======================================================================
    // Helper method
    //
    function isDeleteKey(e) {
        return e.keyCode === keys.DELETE;
    }
    function isShiftKey(e) {
        return e.shiftKey;
    }
    function isF10(e) {
        return e.keyCode === keys.F10;
    }
    function isShiftF10(e) {
        return isShiftKey(e) && isF10(e);
    }

    // =======================================================================
    // Common function to create a customize event
    //
    function eventHandler(type, selectHandler) {
        var handler = function (node, listener) {
            return on(node, type, function (evt) {
                if (selectHandler) {
                    return selectHandler(evt, listener);
                }
                return listener.call(this, evt);
            });
        };

        return handler;
    }

    return {
        // summary:
        //      An static class that add default events (keyboard/mouse) for common uses.
        // tags:
        //      internal

        // Support this for more convenient
        selector: on.selector,

        contextmenu: eventHandler("contextmenu", function (e, listener) {
            // summary:
            //      Shortcut way to listen to the contextmenu event
            // tags:
            //      public event

            listener.call(this, e);
        }),

        // =======================================================================
        // Keyboard events
        //

        keys: {
            shiftf10: eventHandler("keyup", function (e, listener) {
                // summary:
                //      Shortcut way to listen shift-f10 key event
                // tags:
                //      public event

                if (isShiftF10(e)) {
                    listener.call(this, e);
                }
            }),

            del: eventHandler("keydown", function (e, listener) {
                // summary:
                //      Shortcut way to listen delete keydown event
                // tags:
                //      public event

                if (isDeleteKey(e)) {
                    listener.call(this, e);
                }
            })
        }
    };
});

},
'epi-saas-base/widgets/_GridMoreMixin':function(){
﻿define("epi-saas-base/widgets/_GridMoreMixin", [
    "dgrid/_StoreMixin",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/Deferred",
    "dojo/on",
    "dojo/query",
    "dojo/dom-style",
    "dojo/string",
    "dojo/has",
    "dojo/dom-geometry",
    "put-selector/put",
    "dojo/i18n!./nls/_GridMoreMixin"
],
    function (_StoreMixin, declare, lang, Deferred, on, query, domStyle, string, has, domGeom, put, i18n) {
        return declare(_StoreMixin, {
            // summary:
            //		An extension for adding More link for loading next data portion to a List or Grid.

            // rowsPerPage: Number
            //		Number of rows (items) to show on a given page.
            rowsPerPage: 25,

            // i18nMore: Object
            //		This object contains all of the internationalized strings as
            //		key/value pairs.
            i18nMore: i18n,
            // showFooter: Boolean
            //      Indicates if footer should be visible. Initially footer should be hidden.
            showFooter: false,
            _currentPage: 1,
            _total: null,
            _loadingNode: null,
            // loadingMessage: String
            //      The message shown when fetching data for the grid.
            loadingMessage: null,

            buildRendering: function () {
                this.inherited(arguments);

                // add more to footer
                this.renderFooter();
            },

            renderFooter: function () {
                var grid = this,
                    moreNode = this.moreNode =
                        put(this.footerNode, "div.dgrid-more"),
                    statusNode = this.moreStatusNode =
                        put(moreNode, "div.dgrid-status"),
                    i18n = this.i18nMore,
                    navigationNode;

                statusNode.tabIndex = 0;

                // initialize some content into moreStatusNode, to ensure
                // accurate results on initial resize call
                statusNode.innerHTML = string.substitute(i18n.status,
                    { start: 1, end: 1, total: 0 });

                navigationNode = this.moreNavigationNode =
                    put(moreNode, "div.dgrid-navigation");


                this.moreLinksNode = put(navigationNode, "span.dgrid-more-links");
                this._listeners.push(on(navigationNode, "a:click", function () {
                    var cls = this.className;

                    if (cls.indexOf("dgrid-more-disabled") > -1) {
                        return;
                    }
                    var emitResult = grid.emitMoreClick();
                    if (emitResult) {
                        grid.loadNextPage();
                    }
                }));
            },

            emitMoreClick: function () {
                return on.emit(this.domNode, "dgrid-more-click", { cancelable: true, grid: this});
            },

            _updateNavigation: function (focusLink) {
                // summary:
                //		Update status and navigation controls based on total count from query

                var i18n = this.i18nMore,
                    currentPage = this._currentPage,
                    end = Math.ceil(this._total / this.rowsPerPage),
                    moreDisabled = (end === 0 || end === currentPage);

                this.updateFooter(moreDisabled, focusLink);
            },

            updateFooter: function (moreDisabled, focusLink) {
                var linksNode = this.moreLinksNode;

                 linksNode.innerHTML = "";
                // normal link
                var link = put(linksNode,
                    "a[href=javascript:]" + (moreDisabled ? ".dgrid-more-disabled" : "") + ".dgrid-more-link",
                    i18n.more);
                link.setAttribute("aria-label", i18n.more);

                if (focusLink) {
                    // focus on it if we are supposed to retain the focus
                    link.focus();
                }
            },

            refresh: function () {
                var self = this, contentHeight = domGeom.position(this.contentNode).h;
                if (contentHeight > 0) {
                    domStyle.set(this.contentNode, "height", contentHeight + "px"); // preserve contentNode height to avoid grids footer jumping
                }

                // hide footer
                this.updateFooter(true, false);
                this.inherited(arguments);

                if (!this.store) {
                    console.warn("_GridMoreMixin requires a store to operate.");
                    return;
                }

                // Reset to first page and return promise from loadNextPage
                this._total = null;
                this._currentPage = 0;

                return this.loadNextPage().then(function (results) {
                    // Emit on a separate turn to enable event to be used consistently for
                    // initial render, regardless of whether the backing store is async
                    setTimeout(function () {
                        on.emit(self.domNode, "dgrid-refresh-complete", {
                            bubbles: true,
                            cancelable: false,
                            grid: self,
                            results: results // QueryResults object (may be a wrapped promise)
                        });
                    }, 0);
                    // Delete the Deferred immediately so nothing tries to re-resolve
                    delete self._refreshDeferred;

                    return results;
                }, function (err) {
                    delete self._refreshDeferred;
                    throw err;
                });
            },

            renderArray: function () {
                var rows = this.inherited(arguments);

                // Make sure _lastCollection is cleared (due to logic in List)
                this._lastCollection = null;

                return rows;
            },
            cleanupContent: function (grid){
                // Remove any currently-rendered rows, or noDataMessage
                if(grid.noDataNode){
                    put(grid.noDataNode, "!");
                    delete grid.noDataNode;
                }else{
                    grid.cleanup();
                }
                grid.contentNode.innerHTML = "";
            },
            _clearLoadingNode: function () {
            // summary:
            //      Internal function to clear the node displaying the loadingMessage
                if(this._loadingNode){
                    put(this._loadingNode, "!");
                    delete this._loadingNode;
                }
                delete this._isLoading;
            },
            _clearNoDataNode: function() {
            // summary:
            //      Internal function to clear the node displaying the noDataMessage
                if(this.noDataNode){
                    put(this.noDataNode, "!");
                    delete this.noDataNode;
                }
            },

            showInGrid: function() {
            // summary:
            //      Clears the loading node before displaying messages in grid (used by epi-saas-base/widgets/_NotificationMixin)
                this._clearLoadingNode();
                this.inherited(arguments);
            },

            loadNextPage: function (focusLink) {
                // summary:
                //		Loads the next page.
                var grid = this,
                    loadingNode,
                    contentNode = grid.contentNode,
                    dfd = grid._refreshDeferred;

                if (dfd && this._currentPage === 0 && grid._oldQuery === grid.query) {
                    return dfd.promise;
                }

                dfd = this._refreshDeferred = new Deferred()

                // Stop loading pages once we have reached the total
                if(this._total !== null && this._total <= this._currentPage * this.rowsPerPage) {
                    dfd.resolve();
                    return dfd.promise;
                }

                var page = this._currentPage + 1;

                if(grid.showLoadingMessage && (!grid.showLoadingOnlyOnFirstPage || (grid.showLoadingOnlyOnFirstPage && page === 1))){
                    if(page === 1) {
                      grid.cleanupContent(grid);
                    }
                    this._clearLoadingNode();
                    loadingNode = grid._loadingNode = put(contentNode, "div.dgrid-loading");
                    put(loadingNode, "div.dgrid-loading-icon");
                    put(loadingNode, "div.dgrid-loading-message", grid.loadingMessage);
                }

                var result = this._trackError(function () {
                    var count = grid.rowsPerPage,
                        start = (page - 1) * count,
                        options = lang.mixin(grid.get("queryOptions"), {
                            start: start,
                            count: count
                            // current sort is also included by get("queryOptions")
                        }),
                        results;

                    grid._clearNoDataNode();

                    // Run new query and pass it into renderArray
                    var oldQuery = grid._oldQuery = grid.query;
                    results = grid.store.query(grid.query, options);
                    Deferred.when(results, function() {
                        if (oldQuery != grid.query) {
                            dfd.resolve();
                            return;
                        }
                        Deferred.when(grid.renderArray(results, this.contentNode, options), function (rows) {
                            Deferred.when(results.total, function (total) {
                                grid._clearLoadingNode();
                                grid._clearNoDataNode();
                                if (!total) {
                                    // If there are no results, display the no data message.
                                    if (grid.noDataNode) {
                                        put(grid.noDataNode, "!");
                                    }
                                    grid.noDataNode = put(grid.contentNode, "div.dgrid-no-data");
                                    grid.noDataNode.innerHTML = grid.noDataMessage;
                                }

                                // Update status text based on now-current page and total.
                                if(grid.moreStatusNode) {
                                    grid.moreStatusNode.innerHTML = string.substitute(grid.i18nMore.status, {
                                        start: Math.min(1, total),
                                        end: Math.min(total, start + count),
                                        total: total
                                    });
                                }
                                grid._total = total;
                                grid._currentPage = page;
                                grid._oldQuery = null;
                                grid._rowsOnPage = rows.length;

                                domStyle.set(grid.contentNode, "height", "auto"); // reset height back to "auto" after refresh is complete

                                if (has("ie") < 7 || (has("ie") && has("quirks"))) {
                                    // call resize in old IE in case grid is set to height: auto
                                    grid.resize();
                                }

                                // It's especially important that _updateNavigation is called only
                                // after renderArray is resolved as well (to prevent jumping).
                                if(focusLink) {
                                    grid._updateNavigation(focusLink);
                                }

                                dfd.resolve(results);
                            });


                        }, function (error) {
                            dfd.reject(error);
                        });

                    });
                    return dfd.promise;
                });

                if (!result) {
                    // A synchronous error occurred; reject the promise.
                    dfd.reject();
                }
                return dfd.promise;
            }
        });
    });

},
'epi-saas-base/widgets/_AutoLoadScrollMixin':function(){
define("epi-saas-base/widgets/_AutoLoadScrollMixin", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-geometry",
    "dojo/on",
    "dojo/aspect",
    "dojo/when",
    "dojo/Deferred",
    "dijit/Destroyable",
    "dijit/registry"
],
    function (
        declare,
        lang,
        domGeom,
        on,
        aspect,
        when,
        Deferred,
        Destroyable,
        registry
    ) {

    // module:
    //      epi-saas-base/widgets/_AutoLoadScrollMixin

    return declare([Destroyable], {
        // summary:
        //      An extension for loading conditionally

        // _listeners: Object
        //      An object keeping track of listeners placed on scrollNode
        // tags:
        //      private
        _listeners: null,
        // _delayedLoad: dojo/Deferred
        //      If a scrollNode is not set when #load is called, we wait until one is available using a Deferred
        // tags:
        //      private
        _delayedLoad: null,
        // loading: Boolean
        //      A flag preventing load to be triggered while a previous one is running
        loading: false,
        // loader: Function
        //      A function reference to the loader to trig when needed
        loader: null,
        // margin: Integer
        //      The margin in pixels to consider when measuring if bottom of grid is to be considered visible
        margin: 0,
        // scrollNode: DomNode
        //      The DomNode to listen to scroll events from, as well as looking for widget resize function calls from
        scrollNode: null,

        constructor: function() {
            this._listeners = {};
        },

        refresh: function() {
            var result = this.inherited(arguments);
            return when(result).then(lang.hitch(this, function (result) {
                this.load();
                return result;
            }));
        },

        startup: function() {
            this.inherited(arguments);
            this.load();
        },

        _conditionalLoad: function (/*Function*/loader, /*Function|bool*/condition) {
            // summary:
            //      Help method to call a loader method based on a condition.
            if(typeof(condition) == "function" && condition.call() ||
                typeof(condition) == "boolean" && condition) {
                return loader.call();
            }
        },

        _checkIfBottomIsVisible: function (/*domNode*/node, /*domNode*/containerNode, /*integer*/margin) {
            // summary:
            //      Help method to determine if the bottom of a node is visible
            //      inside a container, with margin taken into account.
            // node: DomNode
            //      This is the node whose bottom to examine
            // containerNode: DomNode
            //      The node containing the node being examined
            // margin: integer
            //      Allows adding a margin to the calculation
            // tags:
            //      private

            // Find container position and size
            var containerPosition = domGeom.position(containerNode),
                containerBottom = containerPosition.y + containerPosition.h,
                // Find node bottom
                nodePosition = domGeom.position(node),
                nodeBottom = nodePosition.y + nodePosition.h;
            if (nodeBottom === 0) { // invalid node or it is not in the DOM yet
                return false;
            }
            // Calculate whether bottom of node is visible
            return containerBottom - nodeBottom + margin > 0;
        },

        load: function () {
            // summary:
            //      Executes the conditional load based on current loader and condition evaluation
            if(!this.scrollNode) {
                this._delayedLoad = this._delayedLoad || new Deferred();
                return this._delayedLoad.promise;
            }

            if(!this.loading) {
                this.loading = true;
                return when(this._conditionalLoad(this.loader, this._checkIfBottomIsVisible(this.domNode, this.scrollNode, this.margin)))
                .then(lang.hitch(this, function (result) {
                    this.loading = false;
                    if(result) {
                        // Making sure bottom of grid still is not visible
                        this.load();
                    }
                    return result;
                }));
            }
        },

        _setScrollNode: function(/*DomNode*/ node){
            // summary:
            //      Allows us to set (or exchange) what node is monitored for scrolling action.
            // node: DomNode
            //      This is the node to be monitored
            // tags:
            //      private
            this.scrollNode = node;

            // Clear old listeners
            for (var key in this._listeners) {
                if(this._listeners[key] && typeof(this._listeners[key].remove) == "function") {
                    this._listeners[key].remove();
                    delete this._listeners[key];
                }
            }

            // Listen to scroll event
            this._listeners.scroll = on(node, "scroll", lang.hitch(this, function() {
                    this.load();
                }));
            this.own(this._listeners.scroll);

            // Listen to resize function calls from enclosing widget (if any)
            var widget = registry.getEnclosingWidget(node);
            if(widget && typeof(widget.resize) === "function") {
                this._listeners.resize = aspect.after(widget, "resize", lang.hitch(this, function() {
                        this.load();
                    }));
                this.own(this._listeners.resize);
            }

            // If there's a delayed load waiting, do it now
            if(this._delayedLoad) {
                this._delayedLoad.resolve(this.load());
                this._delayedLoad = null;
            }
        }
    });
});

},
'epi-find/widget/_SiblingFocusMixin':function(){
define([
    "dojo/_base/declare",
    "dojo/_base/array",
    "dijit/dijit",
    "dijit/_WidgetBase",
    "dijit/_Container",
    "dijit/focus"
],
function (declare, array,
    dijit, _WidgetBase, _Container, focusUtil
) {

    // module:
    //      epi-find/widget/_SiblingHasFocus

    return declare([], {
        // summary: 
        //     Sibling has focus mixin
        // desciption:
        //     This mixin is used to determine whether any of the siblings of a widget has focus 

        siblingWithFocus: function() {
            var parent = this.getParent();
            if (!parent) {
                return;
            }
            var siblings = parent.getChildren();
            for(var i=0,j=focusUtil.activeStack.length;i<j;i+=1) {
                var focusedWidget = dijit.byId(focusUtil.activeStack[i]);
                if(array.indexOf(siblings, focusedWidget) > -1) {
                    return focusedWidget;
                }
            }
        },

        onBlurToSibling: function(/* _Widget */ focusedSibling) {},

        onFocus: function () {
            var parent = this.getParent();
            if (!parent) {
                return;
            }
            var siblings = parent.getChildren();
            for(var i=0,j=siblings.length;i<j;i+=1) {
                if(siblings[i] !== this && siblings[i].onBlurToSibling) {
                    siblings[i].onBlurToSibling();
                }
            }
        },

        onBlur: function () {
            var focusedSibling = this.siblingWithFocus();
            if(focusedSibling) {
                this.onBlurToSibling(focusedSibling);
            }
        }

    });
});
},
'dijit/dijit':function(){
define("dijit/dijit", [
	"./main",
	"./_base",
	"dojo/parser",
	"./_Widget",
	"./_TemplatedMixin",
	"./_Container",
	"./layout/_LayoutWidget",
	"./form/_FormWidget",
	"./form/_FormValueWidget"
], function(dijit){

	// module:
	//		dijit/dijit

	/*=====
	return {
		// summary:
		//		A roll-up for common dijit methods
		//		All the stuff in _base (these are the function that are guaranteed available without an explicit dojo.require)
		//		And some other stuff that we tend to pull in all the time anyway
	};
	=====*/

	return dijit;
});

},
'dijit/_base':function(){
define("dijit/_base", [
	"./main",
	"./a11y",	// used to be in dijit/_base/manager
	"./WidgetSet",	// used to be in dijit/_base/manager
	"./_base/focus",
	"./_base/manager",
	"./_base/place",
	"./_base/popup",
	"./_base/scroll",
	"./_base/sniff",
	"./_base/typematic",
	"./_base/wai",
	"./_base/window"
], function(dijit){

	// module:
	//		dijit/_base

	/*=====
	return {
		// summary:
		//		Includes all the modules in dijit/_base
	};
	=====*/

	return dijit._base;
});

},
'dijit/WidgetSet':function(){
define("dijit/WidgetSet", [
	"dojo/_base/array", // array.forEach array.map
	"dojo/_base/declare", // declare
	"dojo/_base/kernel", // kernel.global
	"./registry"	// to add functions to dijit.registry
], function(array, declare, kernel, registry){

	// module:
	//		dijit/WidgetSet

	var WidgetSet = declare("dijit.WidgetSet", null, {
		// summary:
		//		A set of widgets indexed by id.
		//		Deprecated, will be removed in 2.0.
		//
		// example:
		//		Create a small list of widgets:
		//		|	require(["dijit/WidgetSet", "dijit/registry"],
		//		|		function(WidgetSet, registry){
		//		|		var ws = new WidgetSet();
		//		|		ws.add(registry.byId("one"));
		//		|		ws.add(registry.byId("two"));
		//		|		// destroy both:
		//		|		ws.forEach(function(w){ w.destroy(); });
		//		|	});

		constructor: function(){
			this._hash = {};
			this.length = 0;
		},

		add: function(/*dijit/_WidgetBase*/ widget){
			// summary:
			//		Add a widget to this list. If a duplicate ID is detected, a error is thrown.
			//
			// widget: dijit/_WidgetBase
			//		Any dijit/_WidgetBase subclass.
			if(this._hash[widget.id]){
				throw new Error("Tried to register widget with id==" + widget.id + " but that id is already registered");
			}
			this._hash[widget.id] = widget;
			this.length++;
		},

		remove: function(/*String*/ id){
			// summary:
			//		Remove a widget from this WidgetSet. Does not destroy the widget; simply
			//		removes the reference.
			if(this._hash[id]){
				delete this._hash[id];
				this.length--;
			}
		},

		forEach: function(/*Function*/ func, /* Object? */thisObj){
			// summary:
			//		Call specified function for each widget in this set.
			//
			// func:
			//		A callback function to run for each item. Is passed the widget, the index
			//		in the iteration, and the full hash, similar to `array.forEach`.
			//
			// thisObj:
			//		An optional scope parameter
			//
			// example:
			//		Using the default `dijit.registry` instance:
			//		|	require(["dijit/WidgetSet", "dijit/registry"],
			//		|		function(WidgetSet, registry){
			//		|		registry.forEach(function(widget){
			//		|			console.log(widget.declaredClass);
			//		|		});
			//		|	});
			//
			// returns:
			//		Returns self, in order to allow for further chaining.

			thisObj = thisObj || kernel.global;
			var i = 0, id;
			for(id in this._hash){
				func.call(thisObj, this._hash[id], i++, this._hash);
			}
			return this;	// dijit/WidgetSet
		},

		filter: function(/*Function*/ filter, /* Object? */thisObj){
			// summary:
			//		Filter down this WidgetSet to a smaller new WidgetSet
			//		Works the same as `array.filter` and `NodeList.filter`
			//
			// filter:
			//		Callback function to test truthiness. Is passed the widget
			//		reference and the pseudo-index in the object.
			//
			// thisObj: Object?
			//		Option scope to use for the filter function.
			//
			// example:
			//		Arbitrary: select the odd widgets in this list
			//		|	
			//		|		
			//		|	
			//		|	require(["dijit/WidgetSet", "dijit/registry"],
			//		|		function(WidgetSet, registry){
			//		|		registry.filter(function(w, i){
			//		|			return i % 2 == 0;
			//		|		}).forEach(function(w){ /* odd ones */ });
			//		|	});

			thisObj = thisObj || kernel.global;
			var res = new WidgetSet(), i = 0, id;
			for(id in this._hash){
				var w = this._hash[id];
				if(filter.call(thisObj, w, i++, this._hash)){
					res.add(w);
				}
			}
			return res; // dijit/WidgetSet
		},

		byId: function(/*String*/ id){
			// summary:
			//		Find a widget in this list by it's id.
			// example:
			//		Test if an id is in a particular WidgetSet
			//		|	require(["dijit/WidgetSet", "dijit/registry"],
			//		|		function(WidgetSet, registry){
			//		|		var ws = new WidgetSet();
			//		|		ws.add(registry.byId("bar"));
			//		|		var t = ws.byId("bar") // returns a widget
			//		|		var x = ws.byId("foo"); // returns undefined
			//		|	});

			return this._hash[id];	// dijit/_WidgetBase
		},

		byClass: function(/*String*/ cls){
			// summary:
			//		Reduce this widgetset to a new WidgetSet of a particular `declaredClass`
			//
			// cls: String
			//		The Class to scan for. Full dot-notated string.
			//
			// example:
			//		Find all `dijit.TitlePane`s in a page:
			//		|	require(["dijit/WidgetSet", "dijit/registry"],
			//		|		function(WidgetSet, registry){
			//		|		registry.byClass("dijit.TitlePane").forEach(function(tp){ tp.close(); });
			//		|	});

			var res = new WidgetSet(), id, widget;
			for(id in this._hash){
				widget = this._hash[id];
				if(widget.declaredClass == cls){
					res.add(widget);
				}
			 }
			 return res; // dijit/WidgetSet
		},

		toArray: function(){
			// summary:
			//		Convert this WidgetSet into a true Array
			//
			// example:
			//		Work with the widget .domNodes in a real Array
			//		|	require(["dijit/WidgetSet", "dijit/registry"],
			//		|		function(WidgetSet, registry){
			//		|		array.map(registry.toArray(), function(w){ return w.domNode; });
			//		|	});


			var ar = [];
			for(var id in this._hash){
				ar.push(this._hash[id]);
			}
			return ar;	// dijit/_WidgetBase[]
		},

		map: function(/* Function */func, /* Object? */thisObj){
			// summary:
			//		Create a new Array from this WidgetSet, following the same rules as `array.map`
			// example:
			//		|	require(["dijit/WidgetSet", "dijit/registry"],
			//		|		function(WidgetSet, registry){
			//		|		var nodes = registry.map(function(w){ return w.domNode; });
			//		|	});
			//
			// returns:
			//		A new array of the returned values.
			return array.map(this.toArray(), func, thisObj); // Array
		},

		every: function(func, thisObj){
			// summary:
			//		A synthetic clone of `array.every` acting explicitly on this WidgetSet
			//
			// func: Function
			//		A callback function run for every widget in this list. Exits loop
			//		when the first false return is encountered.
			//
			// thisObj: Object?
			//		Optional scope parameter to use for the callback

			thisObj = thisObj || kernel.global;
			var x = 0, i;
			for(i in this._hash){
				if(!func.call(thisObj, this._hash[i], x++, this._hash)){
					return false; // Boolean
				}
			}
			return true; // Boolean
		},

		some: function(func, thisObj){
			// summary:
			//		A synthetic clone of `array.some` acting explicitly on this WidgetSet
			//
			// func: Function
			//		A callback function run for every widget in this list. Exits loop
			//		when the first true return is encountered.
			//
			// thisObj: Object?
			//		Optional scope parameter to use for the callback

			thisObj = thisObj || kernel.global;
			var x = 0, i;
			for(i in this._hash){
				if(func.call(thisObj, this._hash[i], x++, this._hash)){
					return true; // Boolean
				}
			}
			return false; // Boolean
		}

	});

	// Add in 1.x compatibility methods to dijit/registry.
	// These functions won't show up in the API doc but since they are deprecated anyway,
	// that's probably for the best.
	array.forEach(["forEach", "filter", "byClass", "map", "every", "some"], function(func){
		registry[func] = WidgetSet.prototype[func];
	});


	return WidgetSet;
});

},
'dijit/_base/place':function(){
define("dijit/_base/place", [
	"dojo/_base/array", // array.forEach
	"dojo/_base/lang", // lang.isArray, lang.mixin
	"dojo/window", // windowUtils.getBox
	"../place",
	"../main"	// export to dijit namespace
], function(array, lang, windowUtils, place, dijit){

	// module:
	//		dijit/_base/place


	var exports = {
		// summary:
		//		Deprecated back compatibility module, new code should use dijit/place directly instead of using this module.
	};

	exports.getViewport = function(){
		// summary:
		//		Deprecated method to return the dimensions and scroll position of the viewable area of a browser window.
		//		New code should use windowUtils.getBox()

		return windowUtils.getBox();
	};

	exports.placeOnScreen = place.at;

	exports.placeOnScreenAroundElement = function(node, aroundNode, aroundCorners, layoutNode){
		// summary:
		//		Like dijit.placeOnScreenAroundNode(), except it accepts an arbitrary object
		//		for the "around" argument and finds a proper processor to place a node.
		//		Deprecated, new code should use dijit/place.around() instead.

		// Convert old style {"BL": "TL", "BR": "TR"} type argument
		// to style needed by dijit.place code:
		//		[
		//			{aroundCorner: "BL", corner: "TL" },
		//			{aroundCorner: "BR", corner: "TR" }
		//		]
		var positions;
		if(lang.isArray(aroundCorners)){
			positions = aroundCorners;
		}else{
			positions = [];
			for(var key in aroundCorners){
				positions.push({aroundCorner: key, corner: aroundCorners[key]});
			}
		}

		return place.around(node, aroundNode, positions, true, layoutNode);
	};

	exports.placeOnScreenAroundNode = exports.placeOnScreenAroundElement;
	/*=====
	exports.placeOnScreenAroundNode = function(node, aroundNode, aroundCorners, layoutNode){
		// summary:
		//		Position node adjacent or kitty-corner to aroundNode
		//		such that it's fully visible in viewport.
		//		Deprecated, new code should use dijit/place.around() instead.
	};
	=====*/

	exports.placeOnScreenAroundRectangle = exports.placeOnScreenAroundElement;
	/*=====
	exports.placeOnScreenAroundRectangle = function(node, aroundRect, aroundCorners, layoutNode){
		// summary:
		//		Like dijit.placeOnScreenAroundNode(), except that the "around"
		//		parameter is an arbitrary rectangle on the screen (x, y, width, height)
		//		instead of a dom node.
		//		Deprecated, new code should use dijit/place.around() instead.
	};
	=====*/

	exports.getPopupAroundAlignment = function(/*Array*/ position, /*Boolean*/ leftToRight){
		// summary:
		//		Deprecated method, unneeded when using dijit/place directly.
		//		Transforms the passed array of preferred positions into a format suitable for
		//		passing as the aroundCorners argument to dijit/place.placeOnScreenAroundElement.
		// position: String[]
		//		This variable controls the position of the drop down.
		//		It's an array of strings with the following values:
		//
		//		- before: places drop down to the left of the target node/widget, or to the right in
		//		  the case of RTL scripts like Hebrew and Arabic
		//		- after: places drop down to the right of the target node/widget, or to the left in
		//		  the case of RTL scripts like Hebrew and Arabic
		//		- above: drop down goes above target node
		//		- below: drop down goes below target node
		//
		//		The list is positions is tried, in order, until a position is found where the drop down fits
		//		within the viewport.
		// leftToRight: Boolean
		//		Whether the popup will be displaying in leftToRight mode.

		var align = {};
		array.forEach(position, function(pos){
			var ltr = leftToRight;
			switch(pos){
				case "after":
					align[leftToRight ? "BR" : "BL"] = leftToRight ? "BL" : "BR";
					break;
				case "before":
					align[leftToRight ? "BL" : "BR"] = leftToRight ? "BR" : "BL";
					break;
				case "below-alt":
					ltr = !ltr;
					// fall through
				case "below":
					// first try to align left borders, next try to align right borders (or reverse for RTL mode)
					align[ltr ? "BL" : "BR"] = ltr ? "TL" : "TR";
					align[ltr ? "BR" : "BL"] = ltr ? "TR" : "TL";
					break;
				case "above-alt":
					ltr = !ltr;
					// fall through
				case "above":
				default:
					// first try to align left borders, next try to align right borders (or reverse for RTL mode)
					align[ltr ? "TL" : "TR"] = ltr ? "BL" : "BR";
					align[ltr ? "TR" : "TL"] = ltr ? "BR" : "BL";
					break;
			}
		});
		return align;
	};

	lang.mixin(dijit, exports);

	/*===== return exports; =====*/
	return dijit;	// for back compat :-(
});

},
'dijit/_base/popup':function(){
define("dijit/_base/popup", [
	"dojo/dom-class", // domClass.contains
	"dojo/_base/window",
	"../popup",
	"../BackgroundIframe"	// just loading for back-compat, in case client code is referencing it
], function(domClass, win, popup){

// module:
//		dijit/_base/popup

/*=====
return {
	// summary:
	//		Deprecated.   Old module for popups, new code should use dijit/popup directly.
};
=====*/


// Hack support for old API passing in node instead of a widget (to various methods)
var origCreateWrapper = popup._createWrapper;
popup._createWrapper = function(widget){
	if(!widget.declaredClass){
		// make fake widget to pass to new API
		widget = {
			_popupWrapper: (widget.parentNode && domClass.contains(widget.parentNode, "dijitPopup")) ?
				widget.parentNode : null,
			domNode: widget,
			destroy: function(){},
			ownerDocument: widget.ownerDocument,
			ownerDocumentBody: win.body(widget.ownerDocument)
		};
	}
	return origCreateWrapper.call(this, widget);
};

// Support old format of orient parameter
var origOpen = popup.open;
popup.open = function(/*__OpenArgs*/ args){
	// Convert old hash structure (ex: {"BL": "TL", ...}) of orient to format compatible w/new popup.open() API.
	// Don't do conversion for:
	//		- null parameter (that means to use the default positioning)
	//		- "R" or "L" strings used to indicate positioning for context menus (when there is no around node)
	//		- new format, ex: ["below", "above"]
	//		- return value from deprecated dijit.getPopupAroundAlignment() method,
	//			ex: ["below", "above"]
	if(args.orient && typeof args.orient != "string" && !("length" in args.orient)){
		var ary = [];
		for(var key in args.orient){
			ary.push({aroundCorner: key, corner: args.orient[key]});
		}
		args.orient = ary;
	}

	return origOpen.call(this, args);
};

return popup;
});

},
'dijit/_base/scroll':function(){
define("dijit/_base/scroll", [
	"dojo/window", // windowUtils.scrollIntoView
	"../main"	// export symbol to dijit
], function(windowUtils, dijit){
	// module:
	//		dijit/_base/scroll

	/*=====
	return {
		// summary:
		//		Back compatibility module, new code should use windowUtils directly instead of using this module.
	};
	=====*/

	dijit.scrollIntoView = function(/*DomNode*/ node, /*Object?*/ pos){
		// summary:
		//		Scroll the passed node into view, if it is not already.
		//		Deprecated, use `windowUtils.scrollIntoView` instead.

		windowUtils.scrollIntoView(node, pos);
	};
});

},
'dijit/_base/sniff':function(){
define("dijit/_base/sniff", [ "dojo/uacss" ], function(){

	// module:
	//		dijit/_base/sniff

	/*=====
	return {
		// summary:
		//		Deprecated, back compatibility module, new code should require dojo/uacss directly instead of this module.
	};
	=====*/
});

},
'dijit/_base/typematic':function(){
define("dijit/_base/typematic", ["../typematic"], function(){

	/*=====
	return {
		// summary:
		//		Deprecated, for back-compat, just loads top level module
	};
	=====*/

});

},
'dijit/typematic':function(){
define("dijit/typematic", [
	"dojo/_base/array", // array.forEach
	"dojo/_base/connect", // connect.connect
	"dojo/_base/event", // event.stop
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.mixin, lang.hitch
	"dojo/on",
	"dojo/sniff", // has("ie")
	"./main"		// setting dijit.typematic global
], function(array, connect, event, kernel, lang, on, has, dijit){

// module:
//		dijit/typematic

var typematic = (dijit.typematic = {
	// summary:
	//		These functions are used to repetitively call a user specified callback
	//		method when a specific key or mouse click over a specific DOM node is
	//		held down for a specific amount of time.
	//		Only 1 such event is allowed to occur on the browser page at 1 time.

	_fireEventAndReload: function(){
		this._timer = null;
		this._callback(++this._count, this._node, this._evt);

		// Schedule next event, timer is at most minDelay (default 10ms) to avoid
		// browser overload (particularly avoiding starving DOH robot so it never gets to send a mouseup)
		this._currentTimeout = Math.max(
			this._currentTimeout < 0 ? this._initialDelay :
				(this._subsequentDelay > 1 ? this._subsequentDelay : Math.round(this._currentTimeout * this._subsequentDelay)),
			this._minDelay);
		this._timer = setTimeout(lang.hitch(this, "_fireEventAndReload"), this._currentTimeout);
	},

	trigger: function(/*Event*/ evt, /*Object*/ _this, /*DOMNode*/ node, /*Function*/ callback, /*Object*/ obj, /*Number?*/ subsequentDelay, /*Number?*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start a timed, repeating callback sequence.
		//		If already started, the function call is ignored.
		//		This method is not normally called by the user but can be
		//		when the normal listener code is insufficient.
		// evt:
		//		key or mouse event object to pass to the user callback
		// _this:
		//		pointer to the user's widget space.
		// node:
		//		the DOM node object to pass the the callback function
		// callback:
		//		function to call until the sequence is stopped called with 3 parameters:
		// count:
		//		integer representing number of repeated calls (0..n) with -1 indicating the iteration has stopped
		// node:
		//		the DOM node object passed in
		// evt:
		//		key or mouse event object
		// obj:
		//		user space object used to uniquely identify each typematic sequence
		// subsequentDelay:
		//		if > 1, the number of milliseconds until the 3->n events occur
		//		or else the fractional time multiplier for the next event's delay, default=0.9
		// initialDelay:
		//		the number of milliseconds until the 2nd event occurs, default=500ms
		// minDelay:
		//		the maximum delay in milliseconds for event to fire, default=10ms
		if(obj != this._obj){
			this.stop();
			this._initialDelay = initialDelay || 500;
			this._subsequentDelay = subsequentDelay || 0.90;
			this._minDelay = minDelay || 10;
			this._obj = obj;
			this._node = node;
			this._currentTimeout = -1;
			this._count = -1;
			this._callback = lang.hitch(_this, callback);
			this._evt = { faux: true };
			for(var attr in evt){
				if(attr != "layerX" && attr != "layerY"){ // prevent WebKit warnings
					var v = evt[attr];
					if(typeof v != "function" && typeof v != "undefined"){ this._evt[attr] = v }
				}
			}
			this._fireEventAndReload();
		}
	},

	stop: function(){
		// summary:
		//		Stop an ongoing timed, repeating callback sequence.
		if(this._timer){
			clearTimeout(this._timer);
			this._timer = null;
		}
		if(this._obj){
			this._callback(-1, this._node, this._evt);
			this._obj = null;
		}
	},

	addKeyListener: function(/*DOMNode*/ node, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start listening for a specific typematic key.
		//		See also the trigger method for other parameters.
		// keyObject:
		//		an object defining the key to listen for:
		//
		//		- charOrCode: the printable character (string) or keyCode (number) to listen for.
		//		- keyCode: (deprecated - use charOrCode) the keyCode (number) to listen for (implies charCode = 0).
		//		- charCode: (deprecated - use charOrCode) the charCode (number) to listen for.
		//		- ctrlKey: desired ctrl key state to initiate the callback sequence:
		//			- pressed (true)
		//			- released (false)
		//			- either (unspecified)
		//		- altKey: same as ctrlKey but for the alt key
		//		- shiftKey: same as ctrlKey but for the shift key
		// returns:
		//		a connection handle

		if(keyObject.keyCode){
			keyObject.charOrCode = keyObject.keyCode;
			kernel.deprecated("keyCode attribute parameter for dijit.typematic.addKeyListener is deprecated. Use charOrCode instead.", "", "2.0");
		}else if(keyObject.charCode){
			keyObject.charOrCode = String.fromCharCode(keyObject.charCode);
			kernel.deprecated("charCode attribute parameter for dijit.typematic.addKeyListener is deprecated. Use charOrCode instead.", "", "2.0");
		}
		var handles = [
			on(node, connect._keypress, lang.hitch(this, function(evt){
				if(evt.charOrCode == keyObject.charOrCode &&
				(keyObject.ctrlKey === undefined || keyObject.ctrlKey == evt.ctrlKey) &&
				(keyObject.altKey === undefined || keyObject.altKey == evt.altKey) &&
				(keyObject.metaKey === undefined || keyObject.metaKey == (evt.metaKey || false)) && // IE doesn't even set metaKey
				(keyObject.shiftKey === undefined || keyObject.shiftKey == evt.shiftKey)){
					event.stop(evt);
					typematic.trigger(evt, _this, node, callback, keyObject, subsequentDelay, initialDelay, minDelay);
				}else if(typematic._obj == keyObject){
					typematic.stop();
				}
			})),
			on(node, "keyup", lang.hitch(this, function(){
				if(typematic._obj == keyObject){
					typematic.stop();
				}
			}))
		];
		return { remove: function(){ array.forEach(handles, function(h){ h.remove(); }); } };
	},

	addMouseListener: function(/*DOMNode*/ node, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start listening for a typematic mouse click.
		//		See the trigger method for other parameters.
		// returns:
		//		a connection handle
		var handles = [
			on(node, "mousedown", lang.hitch(this, function(evt){
				evt.preventDefault();
				typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay, minDelay);
			})),
			on(node, "mouseup", lang.hitch(this, function(evt){
				if(this._obj){
					evt.preventDefault();
				}
				typematic.stop();
			})),
			on(node, "mouseout", lang.hitch(this, function(evt){
				if(this._obj){
					evt.preventDefault();
				}
				typematic.stop();
			})),
			on(node, "dblclick", lang.hitch(this, function(evt){
				evt.preventDefault();
				if(has("ie") < 9){
					typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay, minDelay);
					setTimeout(lang.hitch(this, typematic.stop), 50);
				}
			}))
		];
		return { remove: function(){ array.forEach(handles, function(h){ h.remove(); }); } };
	},

	addListener: function(/*Node*/ mouseNode, /*Node*/ keyNode, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start listening for a specific typematic key and mouseclick.
		//		This is a thin wrapper to addKeyListener and addMouseListener.
		//		See the addMouseListener and addKeyListener methods for other parameters.
		// mouseNode:
		//		the DOM node object to listen on for mouse events.
		// keyNode:
		//		the DOM node object to listen on for key events.
		// returns:
		//		a connection handle
		var handles = [
			this.addKeyListener(keyNode, keyObject, _this, callback, subsequentDelay, initialDelay, minDelay),
			this.addMouseListener(mouseNode, _this, callback, subsequentDelay, initialDelay, minDelay)
		];
		return { remove: function(){ array.forEach(handles, function(h){ h.remove(); }); } };
	}
});

return typematic;

});

},
'dijit/_base/wai':function(){
define("dijit/_base/wai", [
	"dojo/dom-attr", // domAttr.attr
	"dojo/_base/lang", // lang.mixin
	"../main",	// export symbols to dijit
	"../hccss"			// not using this module directly, but loading it sets CSS flag on <html>
], function(domAttr, lang, dijit){

	// module:
	//		dijit/_base/wai

	var exports = {
		// summary:
		//		Deprecated methods for setting/getting wai roles and states.
		//		New code should call setAttribute()/getAttribute() directly.
		//
		//		Also loads hccss to apply dj_a11y class to root node if machine is in high-contrast mode.

		hasWaiRole: function(/*Element*/ elem, /*String?*/ role){
			// summary:
			//		Determines if an element has a particular role.
			// returns:
			//		True if elem has the specific role attribute and false if not.
			//		For backwards compatibility if role parameter not provided,
			//		returns true if has a role
			var waiRole = this.getWaiRole(elem);
			return role ? (waiRole.indexOf(role) > -1) : (waiRole.length > 0);
		},

		getWaiRole: function(/*Element*/ elem){
			// summary:
			//		Gets the role for an element (which should be a wai role).
			// returns:
			//		The role of elem or an empty string if elem
			//		does not have a role.
			 return lang.trim((domAttr.get(elem, "role") || "").replace("wairole:",""));
		},

		setWaiRole: function(/*Element*/ elem, /*String*/ role){
			// summary:
			//		Sets the role on an element.
			// description:
			//		Replace existing role attribute with new role.

			domAttr.set(elem, "role", role);
		},

		removeWaiRole: function(/*Element*/ elem, /*String*/ role){
			// summary:
			//		Removes the specified role from an element.
			//		Removes role attribute if no specific role provided (for backwards compat.)

			var roleValue = domAttr.get(elem, "role");
			if(!roleValue){ return; }
			if(role){
				var t = lang.trim((" " + roleValue + " ").replace(" " + role + " ", " "));
				domAttr.set(elem, "role", t);
			}else{
				elem.removeAttribute("role");
			}
		},

		hasWaiState: function(/*Element*/ elem, /*String*/ state){
			// summary:
			//		Determines if an element has a given state.
			// description:
			//		Checks for an attribute called "aria-"+state.
			// returns:
			//		true if elem has a value for the given state and
			//		false if it does not.

			return elem.hasAttribute ? elem.hasAttribute("aria-"+state) : !!elem.getAttribute("aria-"+state);
		},

		getWaiState: function(/*Element*/ elem, /*String*/ state){
			// summary:
			//		Gets the value of a state on an element.
			// description:
			//		Checks for an attribute called "aria-"+state.
			// returns:
			//		The value of the requested state on elem
			//		or an empty string if elem has no value for state.

			return elem.getAttribute("aria-"+state) || "";
		},

		setWaiState: function(/*Element*/ elem, /*String*/ state, /*String*/ value){
			// summary:
			//		Sets a state on an element.
			// description:
			//		Sets an attribute called "aria-"+state.

			elem.setAttribute("aria-"+state, value);
		},

		removeWaiState: function(/*Element*/ elem, /*String*/ state){
			// summary:
			//		Removes a state from an element.
			// description:
			//		Sets an attribute called "aria-"+state.

			elem.removeAttribute("aria-"+state);
		}
	};

	lang.mixin(dijit, exports);

	/*===== return exports; =====*/
	return dijit;	// for back compat :-(
});

},
'dijit/_base/window':function(){
define("dijit/_base/window", [
	"dojo/window", // windowUtils.get
	"../main"	// export symbol to dijit
], function(windowUtils, dijit){
	// module:
	//		dijit/_base/window

	/*=====
	return {
		// summary:
		//		Back compatibility module, new code should use windowUtils directly instead of using this module.
	};
	=====*/

	dijit.getDocumentWindow = function(doc){
		return windowUtils.get(doc);
	};
});

},
'epi-find/widget/SynonymForm':function(){
﻿define([
    "dojo/_base/declare",
    "dojo/_base/event",
    "dojo/_base/config",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/topic",
    "dojo/when",

    "dijit/form/Form",
    "dijit/form/CheckBox",
    "dijit/form/Button",
    "dijit/form/ValidationTextBox",

    "dijit/_WidgetsInTemplateMixin",

    "dojox/mvc/at",

    "epi-saas-base/form/FormListItem",
    "epi-saas-base/UnicodeLetterRegexp",
    "epi-saas-base/widgets/_NotificationMixin",
    "epi-saas-base/widgets/_ActionButtonsMixin",
    "../ConfigModel",
    "../util/arrayx",

    "dojo/text!./templates/SynonymForm.html",
    "dojo/i18n!./nls/Synonyms"

], function(
    declare,
    event,
    config,
    lang,
    array,
    topic,
    when,

    Form,
    CheckBox,
    Button,
    ValidationTextBox,

    _WidgetsInTemplateMixin,

    at,

    FormListItem,
    UnicodeLetterRegExp,
    _NotificationMixin,
    _ActionButtonsMixin,
    ConfigModel,
    arrayx,

    template,
    i18n
    ) {
    return declare([Form, _WidgetsInTemplateMixin, _NotificationMixin, _ActionButtonsMixin], {

        "class": "epi-formsWidgetWrapper",
        ctrl: null,
        templateString: template,
        i18n: i18n,

        i18nPhraseHelp: i18n.help_phrase,
        i18nSynonymHelp: i18n.help_synonym,

        // phraseValidationRegExp: string
        //      A string with regular expression defining a comma-separated list of words in unicode
        phraseValidationRegExp: "[^~`!@%^*()\\[\\]\\{\\}:;'|/<>?_-]((("+UnicodeLetterRegExp+"|[0-9&+#$'_.\\-])+)(,\\s*)*)*",
        multiPhraseValidationRegExp: "[^~`!@%^*()\\[\\]\\{\\}:;'|/<>?_-]((("+UnicodeLetterRegExp+"|[0-9&+#$'_.\\-])*[\\s]?){1,3}\\s*,\\s*)*(("+UnicodeLetterRegExp+"|[0-9&+#$'_.\\-])*[\\s]?){1,3}",

        // synonymValidationRegExp: string
        //      A string with regular expression defining a single word in unicode
        synonymValidationRegExp: "[^~`!@%^*()\\[\\]\\{\\}:;'|/<>?_-]("+UnicodeLetterRegExp+"|[0-9&+#$'_.\\-])*",
        multiSynonymValidationRegExp: "[^~`!@%^*()\\[\\]\\{\\}:;'|/<>?_-](("+UnicodeLetterRegExp+"|[0-9&+#$'_.\\-])*[\\s]?){1,3}",


        phraseField: null,
        synonymField: null,
        bidirectionalCheckbox: null,
        submitButton: null,

        _hasMultiToken: false,

        postMixInProperties: function () {
            this.inherited(arguments);
            if (ConfigModel.getValue("api.synonyms.multi_token") === true) {
                this.i18nPhraseHelp = this.i18n.help_multiPhrase;
                this.i18nSynonymHelp = this.i18n.help_multiSynonym;
            }
        },

        postCreate: function() {
            this.inherited(arguments);

            this.phraseFieldListItem.set("label", i18n.fields_phrase);
            this.phraseFieldListItem.set("description", i18n.separateMultiplePhrases);
            this.phraseFieldListItem.set("help", this.i18nPhraseHelp);
            this.synonymFieldListItem.set("label", i18n.fields_synonym);
            this.bidirectionalCheckboxListItem.set("label",i18n.fields_bidirectional);
            this.submitButton.set("label", i18n.addSynonym);
            this.resetButton.set("label", i18n.cancel);

            this.actionButtons = [this.submitButton, this.resetButton];
            this.enabledWhenNoChangesButtons = [this.resetButton];

            var compareValidator = lang.hitch(this, this.validatePhraseSynonymNotEqual);
            var phraseFieldValidator = lang.hitch(this.phraseField, this.phraseField.validator);
            this.phraseField.validator = function (value, constraints) {
                return  phraseFieldValidator(value, constraints) &&
                    compareValidator(value, constraints);
            };

            var synonymFieldValidator = lang.hitch(this.synonymField, this.synonymField.validator);
            this.synonymField.validator = function (value, constraints) {
                return synonymFieldValidator(value, constraints) &&
                    compareValidator(value, constraints);
            };

            this._hasMultiToken = ConfigModel.getValue("api.synonyms.multi_token") === true;

            this.phraseField.set("value", at(this.ctrl, "phrase"));
            this.phraseField.set("pattern", this._hasMultiToken ? this.multiPhraseValidationRegExp : this.phraseValidationRegExp);

            this.synonymField.set("value", at(this.ctrl, "synonym"));
            this.synonymField.set("pattern", this._hasMultiToken ? this.multiSynonymValidationRegExp : this.synonymValidationRegExp);

            this.bidirectionalCheckbox.set("checked", at(this.ctrl, "bidirectional"));

            this.own(this.watchDataChanges(this.ctrl));
        },

        onSubmit: function(e) {
            var self = this;
            event.stop(e);
            if (!this.validate()) {
                return false;
            }
            this.submitButton.focus();
            this.indicateInProgress(this.submitButton);
            when(this.ctrl.commit(), function() {
                    self.reset();
                    self.showSuccess(i18n.savedMessage);
                    self.indicateSuccess();
                    topic.publish(config.intents.view.synonyms);
                    self.phraseField.focus();
                },
                function (error) {
                    self.showError(error, null, i18n.saveErrorTitle);
                    self.indicateError();
                });
        },

        edit: function(synonymId) {
            var self = this;

            if (this.editMode) {
                this.reset();
            }
            when(this.ctrl.getStore(synonymId), function() {
                    self.phraseField.focus();
                    self.submitButton.set("label", i18n.saveSynonym);
                    self.set("hasChanges", false);
                },
                function (error) {
                    self.showError(error);
                }
            );
        },

        onReset: function() {
            this.ctrl.reset();
            this.ctrl.empty();
            this.submitButton.set("label", i18n.addSynonym);
            return this.inherited(arguments);
        },

        submit: function(){
            // summary:
            //      programmatically submit form if and only if the `onSubmit` returns true
            if(this.onSubmit() === false){
                return;
            }
            this.originalContainerNode.submit();
        },

        validatePhraseSynonymNotEqual: function(value, constraints) {
            var phrase = this.phraseField.getValue();
            var synonym = this.synonymField.getValue();
            if (!phrase || !synonym) {
                return true;
            }
            return !arrayx.intersected(this._hasMultiToken ? arrayx.parse(phrase, ",") : phrase, synonym);
        }
    });
});

},
'dijit/form/Form':function(){
define("dijit/form/Form", [
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/_base/event", // event.stop
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/sniff", // has("ie")
	"../_Widget",
	"../_TemplatedMixin",
	"./_FormMixin",
	"../layout/_ContentPaneResizeMixin"
], function(declare, domAttr, event, kernel, has, _Widget, _TemplatedMixin, _FormMixin, _ContentPaneResizeMixin){

	// module:
	//		dijit/form/Form


	return declare("dijit.form.Form", [_Widget, _TemplatedMixin, _FormMixin, _ContentPaneResizeMixin], {
		// summary:
		//		Widget corresponding to HTML form tag, for validation and serialization
		//
		// example:
		//	|	<form data-dojo-type="dijit/form/Form" id="myForm">
		//	|		Name: <input type="text" name="name" />
		//	|	</form>
		//	|	myObj = {name: "John Doe"};
		//	|	dijit.byId('myForm').set('value', myObj);
		//	|
		//	|	myObj=dijit.byId('myForm').get('value');

		// HTML <FORM> attributes

		// name: String?
		//		Name of form for scripting.
		name: "",

		// action: String?
		//		Server-side form handler.
		action: "",

		// method: String?
		//		HTTP method used to submit the form, either "GET" or "POST".
		method: "",

		// encType: String?
		//		Encoding type for the form, ex: application/x-www-form-urlencoded.
		encType: "",

		// accept-charset: String?
		//		List of supported charsets.
		"accept-charset": "",

		// accept: String?
		//		List of MIME types for file upload.
		accept: "",

		// target: String?
		//		Target frame for the document to be opened in.
		target: "",

		templateString: "<form data-dojo-attach-point='containerNode' data-dojo-attach-event='onreset:_onReset,onsubmit:_onSubmit' ${!nameAttrSetting}></form>",

		postMixInProperties: function(){
			// Setup name=foo string to be referenced from the template (but only if a name has been specified)
			// Unfortunately we can't use _setNameAttr to set the name due to IE limitations, see #8660
			this.nameAttrSetting = this.name ? ("name='" + this.name + "'") : "";
			this.inherited(arguments);
		},

		execute: function(/*Object*/ /*===== formContents =====*/){
			// summary:
			//		Deprecated: use submit()
			// tags:
			//		deprecated
		},

		onExecute: function(){
			// summary:
			//		Deprecated: use onSubmit()
			// tags:
			//		deprecated
		},

		_setEncTypeAttr: function(/*String*/ value){
			this.encType = value;
			domAttr.set(this.domNode, "encType", value);
			if(has("ie")){ this.domNode.encoding = value; }
		},

		reset: function(/*Event?*/ e){
			// summary:
			//		restores all widget values back to their init values,
			//		calls onReset() which can cancel the reset by returning false

			// create fake event so we can know if preventDefault() is called
			var faux = {
				returnValue: true, // the IE way
				preventDefault: function(){ // not IE
							this.returnValue = false;
						},
				stopPropagation: function(){},
				currentTarget: e ? e.target : this.domNode,
				target: e ? e.target : this.domNode
			};
			// if return value is not exactly false, and haven't called preventDefault(), then reset
			if(!(this.onReset(faux) === false) && faux.returnValue){
				this.inherited(arguments, []);
			}
		},

		onReset: function(/*Event?*/ /*===== e =====*/){
			// summary:
			//		Callback when user resets the form. This method is intended
			//		to be over-ridden. When the `reset` method is called
			//		programmatically, the return value from `onReset` is used
			//		to compute whether or not resetting should proceed
			// tags:
			//		callback
			return true; // Boolean
		},

		_onReset: function(e){
			this.reset(e);
			event.stop(e);
			return false;
		},

		_onSubmit: function(e){
			var fp = this.constructor.prototype;
			// TODO: remove this if statement beginning with 2.0
			if(this.execute != fp.execute || this.onExecute != fp.onExecute){
				kernel.deprecated("dijit.form.Form:execute()/onExecute() are deprecated. Use onSubmit() instead.", "", "2.0");
				this.onExecute();
				this.execute(this.getValues());
			}
			if(this.onSubmit(e) === false){ // only exactly false stops submit
				event.stop(e);
			}
		},

		onSubmit: function(/*Event?*/ /*===== e =====*/){
			// summary:
			//		Callback when user submits the form.
			// description:
			//		This method is intended to be over-ridden, but by default it checks and
			//		returns the validity of form elements. When the `submit`
			//		method is called programmatically, the return value from
			//		`onSubmit` is used to compute whether or not submission
			//		should proceed
			// tags:
			//		extension

			return this.isValid(); // Boolean
		},

		submit: function(){
			// summary:
			//		programmatically submit form if and only if the `onSubmit` returns true
			if(!(this.onSubmit() === false)){
				this.containerNode.submit();
			}
		}
	});
});

},
'epi-saas-base/UnicodeLetterRegexp':function(){
define("epi-saas-base/UnicodeLetterRegexp", [], function() {
    // summary:
    //      A string containing a regexp matching any letter in unicode
    return "[\\u0041-\\u005A\\u0061-\\u007A\\u00AA\\u00B5\\u00BA\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02C1\\u02C6-\\u02D1\\u02E0-\\u02E4\\u02EC\\u02EE\\u0370-\\u0374\\u0376\\u0377\\u037A-\\u037D\\u0386\\u0388-\\u038A\\u038C\\u038E-\\u03A1\\u03A3-\\u03F5\\u03F7-\\u0481\\u048A-\\u0527\\u0531-\\u0556\\u0559\\u0561-\\u0587\\u05D0-\\u05EA\\u05F0-\\u05F2\\u0620-\\u064A\\u066E\\u066F\\u0671-\\u06D3\\u06D5\\u06E5\\u06E6\\u06EE\\u06EF\\u06FA-\\u06FC\\u06FF\\u0710\\u0712-\\u072F\\u074D-\\u07A5\\u07B1\\u07CA-\\u07EA\\u07F4\\u07F5\\u07FA\\u0800-\\u0815\\u081A\\u0824\\u0828\\u0840-\\u0858\\u08A0\\u08A2-\\u08AC\\u0904-\\u0939\\u093D\\u0950\\u0958-\\u0961\\u0971-\\u0977\\u0979-\\u097F\\u0985-\\u098C\\u098F\\u0990\\u0993-\\u09A8\\u09AA-\\u09B0\\u09B2\\u09B6-\\u09B9\\u09BD\\u09CE\\u09DC\\u09DD\\u09DF-\\u09E1\\u09F0\\u09F1\\u0A05-\\u0A0A\\u0A0F\\u0A10\\u0A13-\\u0A28\\u0A2A-\\u0A30\\u0A32\\u0A33\\u0A35\\u0A36\\u0A38\\u0A39\\u0A59-\\u0A5C\\u0A5E\\u0A72-\\u0A74\\u0A85-\\u0A8D\\u0A8F-\\u0A91\\u0A93-\\u0AA8\\u0AAA-\\u0AB0\\u0AB2\\u0AB3\\u0AB5-\\u0AB9\\u0ABD\\u0AD0\\u0AE0\\u0AE1\\u0B05-\\u0B0C\\u0B0F\\u0B10\\u0B13-\\u0B28\\u0B2A-\\u0B30\\u0B32\\u0B33\\u0B35-\\u0B39\\u0B3D\\u0B5C\\u0B5D\\u0B5F-\\u0B61\\u0B71\\u0B83\\u0B85-\\u0B8A\\u0B8E-\\u0B90\\u0B92-\\u0B95\\u0B99\\u0B9A\\u0B9C\\u0B9E\\u0B9F\\u0BA3\\u0BA4\\u0BA8-\\u0BAA\\u0BAE-\\u0BB9\\u0BD0\\u0C05-\\u0C0C\\u0C0E-\\u0C10\\u0C12-\\u0C28\\u0C2A-\\u0C33\\u0C35-\\u0C39\\u0C3D\\u0C58\\u0C59\\u0C60\\u0C61\\u0C85-\\u0C8C\\u0C8E-\\u0C90\\u0C92-\\u0CA8\\u0CAA-\\u0CB3\\u0CB5-\\u0CB9\\u0CBD\\u0CDE\\u0CE0\\u0CE1\\u0CF1\\u0CF2\\u0D05-\\u0D0C\\u0D0E-\\u0D10\\u0D12-\\u0D3A\\u0D3D\\u0D4E\\u0D60\\u0D61\\u0D7A-\\u0D7F\\u0D85-\\u0D96\\u0D9A-\\u0DB1\\u0DB3-\\u0DBB\\u0DBD\\u0DC0-\\u0DC6\\u0E01-\\u0E30\\u0E32\\u0E33\\u0E40-\\u0E46\\u0E81\\u0E82\\u0E84\\u0E87\\u0E88\\u0E8A\\u0E8D\\u0E94-\\u0E97\\u0E99-\\u0E9F\\u0EA1-\\u0EA3\\u0EA5\\u0EA7\\u0EAA\\u0EAB\\u0EAD-\\u0EB0\\u0EB2\\u0EB3\\u0EBD\\u0EC0-\\u0EC4\\u0EC6\\u0EDC-\\u0EDF\\u0F00\\u0F40-\\u0F47\\u0F49-\\u0F6C\\u0F88-\\u0F8C\\u1000-\\u102A\\u103F\\u1050-\\u1055\\u105A-\\u105D\\u1061\\u1065\\u1066\\u106E-\\u1070\\u1075-\\u1081\\u108E\\u10A0-\\u10C5\\u10C7\\u10CD\\u10D0-\\u10FA\\u10FC-\\u1248\\u124A-\\u124D\\u1250-\\u1256\\u1258\\u125A-\\u125D\\u1260-\\u1288\\u128A-\\u128D\\u1290-\\u12B0\\u12B2-\\u12B5\\u12B8-\\u12BE\\u12C0\\u12C2-\\u12C5\\u12C8-\\u12D6\\u12D8-\\u1310\\u1312-\\u1315\\u1318-\\u135A\\u1380-\\u138F\\u13A0-\\u13F4\\u1401-\\u166C\\u166F-\\u167F\\u1681-\\u169A\\u16A0-\\u16EA\\u1700-\\u170C\\u170E-\\u1711\\u1720-\\u1731\\u1740-\\u1751\\u1760-\\u176C\\u176E-\\u1770\\u1780-\\u17B3\\u17D7\\u17DC\\u1820-\\u1877\\u1880-\\u18A8\\u18AA\\u18B0-\\u18F5\\u1900-\\u191C\\u1950-\\u196D\\u1970-\\u1974\\u1980-\\u19AB\\u19C1-\\u19C7\\u1A00-\\u1A16\\u1A20-\\u1A54\\u1AA7\\u1B05-\\u1B33\\u1B45-\\u1B4B\\u1B83-\\u1BA0\\u1BAE\\u1BAF\\u1BBA-\\u1BE5\\u1C00-\\u1C23\\u1C4D-\\u1C4F\\u1C5A-\\u1C7D\\u1CE9-\\u1CEC\\u1CEE-\\u1CF1\\u1CF5\\u1CF6\\u1D00-\\u1DBF\\u1E00-\\u1F15\\u1F18-\\u1F1D\\u1F20-\\u1F45\\u1F48-\\u1F4D\\u1F50-\\u1F57\\u1F59\\u1F5B\\u1F5D\\u1F5F-\\u1F7D\\u1F80-\\u1FB4\\u1FB6-\\u1FBC\\u1FBE\\u1FC2-\\u1FC4\\u1FC6-\\u1FCC\\u1FD0-\\u1FD3\\u1FD6-\\u1FDB\\u1FE0-\\u1FEC\\u1FF2-\\u1FF4\\u1FF6-\\u1FFC\\u2071\\u207F\\u2090-\\u209C\\u2102\\u2107\\u210A-\\u2113\\u2115\\u2119-\\u211D\\u2124\\u2126\\u2128\\u212A-\\u212D\\u212F-\\u2139\\u213C-\\u213F\\u2145-\\u2149\\u214E\\u2183\\u2184\\u2C00-\\u2C2E\\u2C30-\\u2C5E\\u2C60-\\u2CE4\\u2CEB-\\u2CEE\\u2CF2\\u2CF3\\u2D00-\\u2D25\\u2D27\\u2D2D\\u2D30-\\u2D67\\u2D6F\\u2D80-\\u2D96\\u2DA0-\\u2DA6\\u2DA8-\\u2DAE\\u2DB0-\\u2DB6\\u2DB8-\\u2DBE\\u2DC0-\\u2DC6\\u2DC8-\\u2DCE\\u2DD0-\\u2DD6\\u2DD8-\\u2DDE\\u2E2F\\u3005\\u3006\\u3031-\\u3035\\u303B\\u303C\\u3041-\\u3096\\u309D-\\u309F\\u30A1-\\u30FA\\u30FC-\\u30FF\\u3105-\\u312D\\u3131-\\u318E\\u31A0-\\u31BA\\u31F0-\\u31FF\\u3400-\\u4DB5\\u4E00-\\u9FCC\\uA000-\\uA48C\\uA4D0-\\uA4FD\\uA500-\\uA60C\\uA610-\\uA61F\\uA62A\\uA62B\\uA640-\\uA66E\\uA67F-\\uA697\\uA6A0-\\uA6E5\\uA717-\\uA71F\\uA722-\\uA788\\uA78B-\\uA78E\\uA790-\\uA793\\uA7A0-\\uA7AA\\uA7F8-\\uA801\\uA803-\\uA805\\uA807-\\uA80A\\uA80C-\\uA822\\uA840-\\uA873\\uA882-\\uA8B3\\uA8F2-\\uA8F7\\uA8FB\\uA90A-\\uA925\\uA930-\\uA946\\uA960-\\uA97C\\uA984-\\uA9B2\\uA9CF\\uAA00-\\uAA28\\uAA40-\\uAA42\\uAA44-\\uAA4B\\uAA60-\\uAA76\\uAA7A\\uAA80-\\uAAAF\\uAAB1\\uAAB5\\uAAB6\\uAAB9-\\uAABD\\uAAC0\\uAAC2\\uAADB-\\uAADD\\uAAE0-\\uAAEA\\uAAF2-\\uAAF4\\uAB01-\\uAB06\\uAB09-\\uAB0E\\uAB11-\\uAB16\\uAB20-\\uAB26\\uAB28-\\uAB2E\\uABC0-\\uABE2\\uAC00-\\uD7A3\\uD7B0-\\uD7C6\\uD7CB-\\uD7FB\\uF900-\\uFA6D\\uFA70-\\uFAD9\\uFB00-\\uFB06\\uFB13-\\uFB17\\uFB1D\\uFB1F-\\uFB28\\uFB2A-\\uFB36\\uFB38-\\uFB3C\\uFB3E\\uFB40\\uFB41\\uFB43\\uFB44\\uFB46-\\uFBB1\\uFBD3-\\uFD3D\\uFD50-\\uFD8F\\uFD92-\\uFDC7\\uFDF0-\\uFDFB\\uFE70-\\uFE74\\uFE76-\\uFEFC\\uFF21-\\uFF3A\\uFF41-\\uFF5A\\uFF66-\\uFFBE\\uFFC2-\\uFFC7\\uFFCA-\\uFFCF\\uFFD2-\\uFFD7\\uFFDA-\\uFFDC]";
});
},
'epi-saas-base/widgets/_ActionButtonsMixin':function(){
define("epi-saas-base/widgets/_ActionButtonsMixin", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/dom-class",

    "dojox/mvc/equals"
], function (
    declare,
    lang,
    array,
    domClass,

    equals
    ) {
    return declare([], {
        // summary:
        //     Mixin to enable/disable buttons and show spinner while saving changes

        // actionButtons: Array
        //      The list of buttons that should indicate progress or be enabled/disabled depending on changes
        actionButtons: null,

        // enabledWhenNoChangesButtons: Array
        //      The list of buttons that should be enabled even if data is not changed
        enabledWhenNoChangesButtons: null,

        // hasChanges: Boolean
        //      Indicates whether data is changed and buttons should be enabled
        hasChanges: false,

        // disableWhenNoChanges: Boolean
        //      Indicates whether buttons should be disabled when there are no data changes
        disableWhenNoChanges: true,

        _defaultActionDataProperty: "value",

        _setHasChangesAttr: function(hasChanges) {
            if (this.disableWhenNoChanges) {
                this._toggleDisabled(!hasChanges);
            }
            this._set("hasChanges", hasChanges);
        },

        postMixInProperties: function () {
            this.inherited(arguments);

            this.actionButtons = this.actionButtons || [];
            if (!(this.actionButtons instanceof Array)) {
                this.actionButtons = [this.actionButtons];
            }

            this.enabledWhenNoChangesButtons = this.enabledWhenNoChangesButtons || [];
            if (!(this.enabledWhenNoChangesButtons instanceof Array)) {
                this.enabledWhenNoChangesButtons = [this.enabledWhenNoChangesButtons];
            }
        },

        postCreate: function () {
            this.inherited(arguments);
            if (this.disableWhenNoChanges) {
                this._toggleDisabled(!this.hasChanges);
            }
        },

        indicateInProgress: function (actionButton, buttonText) {
            // summary:
            //      Disables all buttons while the action is in progress
            //      and shows spinner on "submit" button that activated this action.
            // actionButton: Widget
            //		Button button that activated the action and should indicate the "in-progress" state.
            // buttonText: String
            //		Text that should be displayed on action button during the action, for example "Saving..."

            this._toggleDisabled(true, true);
            this._toggleInProgress(true, actionButton, buttonText);
        },

        indicateSuccess: function (actionButton, buttonText) {
            // summary:
            //      Disables all buttons (since the action is completed successfully) and removes the spinner.
            // actionButton: Widget
            //		Button button that indicates the "in-progress" state.
            // buttonText: String
            //		Text that should be displayed on action button after successful action, for example "Saved"

            this._toggleInProgress(false, actionButton, buttonText);
            this._toggleDisabled(this.disableWhenNoChanges);
        },

        indicateError: function (actionButton, buttonText) {
            // summary:
            //      Enables all buttons (to enable user to re-submit data) and removes the spinner.
            // actionButton: Widget
            //		Button button that indicates the "in-progress" state.
            // buttonText: String
            //		Text that should be displayed on action button when the action is failed, for example "Save - retry"

            this._toggleInProgress(false, actionButton, buttonText);
            this._toggleDisabled(false, true);
        },

        reset: function() {
            // summary:
            //      Resets data changes indication.

            this.inherited(arguments);
            this.set("hasChanges", false);
        },

        watchDataChanges: function (actionDataProperties) {
            // summary:
            //      Watches data changes based on specified actionDataProperty. This method can be overridden to implement custom changes monitoring.
            // actionDataProperties: String|Stateful|Array
            //		Names of the properties or objects that should be monitored, for example ["itemTitle", model.ctrl].
            // returns:
            //      The result of watching the specified properties, usually a handle(s) that should be disposed later.
            actionDataProperties = actionDataProperties || [this._defaultActionDataProperty];
            if (!(actionDataProperties instanceof Array)) {
                actionDataProperties = [actionDataProperties];
            }

            var watchResults = array.map(actionDataProperties, function (property) {
                return this._watchObjectProperty(property) || this._watchPropertyByName(property);
            }, this);

            var result = array.filter(watchResults, function (handle) {
                return handle;
            });

            return result;
        },

        _watchObjectProperty: function (property){
            // summary:
            //      Lets object to monitor own changes if it is possible
            if (property &&
                typeof property === "object" &&
                property.watch &&
                typeof property.watch === "function") {

                // let property value to monitor own changes
                return property.watch(lang.hitch(this, function() {
                    var hasChanges = true;
                    // if an EditModelRefController is observed
                    if (property.originalModel && property.sourceModel) {
                        // indicate changes only if there is any
                        hasChanges = !equals.equalsObject(property.originalModel, property.sourceModel);
                    }
                    this.set("hasChanges", hasChanges);
                }));
            }
            else {
                return null;
            }
        },

        _watchPropertyByName: function (propertyName){
            // summary:
            //      Monitor the property value for changes or delegates monitoring to property value if it is possible
            if (propertyName &&
                typeof propertyName === "string") {
                var propertyValue = this[propertyName];

                // try to let property value to monitor own changes
                var handle = this._watchObjectProperty(propertyValue);

                if (!handle &&
                    this.watch &&
                    typeof this.watch === "function") {

                    // monitor the property value for changes
                    handle = this.watch(propertyName, lang.hitch(this, function() {
                        this.set("hasChanges", true);
                    }));
                }
                return handle;
            }
            else {
                return null;
            }
        },

        _toggleDisabled: function (disabled, force) {
            array.forEach(this._getAllButtons(), function (button) {
                button.set("disabled", disabled &&
                    (force || array.indexOf(this.enabledWhenNoChangesButtons, button) === -1));
            }, this);
        },

        _toggleInProgress: function (inProgress, actionButton, buttonText) {
            array.forEach(this._getAllButtons(), function (button) {
                if (!inProgress || button === actionButton) {
                    domClass.toggle(button.domNode, "epi-btn--loading", inProgress);
                }
                if (button === actionButton && buttonText) {
                    button.set("label", buttonText);
                }
            });
        },

        _getAllButtons: function () {
            // put all buttons in one array without duplicates
            return this.actionButtons.concat(array.filter(this.enabledWhenNoChangesButtons, function (button) {
                return array.indexOf(this.actionButtons, button) === -1;
            }, this));
        }
    });
});
},
'dojox/mvc/equals':function(){
define("dojox/mvc/equals", [
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/Stateful",
	"./StatefulArray"
], function(array, lang, Stateful, StatefulArray){
	var equalsOptions = {
		// summary:
		//		Options used for dojox/mvc/equals().

		getType: function(/*Anything*/ v){
			// summary:
			//		Returns the type of the given value.
			// v: Anything
			//		The value.

			return lang.isArray(v) ? "array" : lang.isFunction((v || {}).getTime) ? "date" : v != null && ({}.toString.call(v) == "[object Object]" || lang.isFunction((v || {}).set) && lang.isFunction((v || {}).watch)) ? "object" : "value";
		},

		equalsArray: function(/*Anything[]*/ dst, /*Anything[]*/ src){
			// summary:
			//		Returns if the given two stateful arrays are equal.
			// dst: Anything[]
			//		The array to compare with.
			// src: Anything[]
			//		The array to compare with.

			for(var i = 0, l = Math.max(dst.length, src.length); i < l; i++){
				if(!equals(dst[i], src[i])){ return false; }
			}
			return true;
		},

		equalsDate: function(/*Date*/ dst, /*Date*/ src){
			return dst.getTime() == src.getTime();
		},

		equalsObject: function(/*Object*/ dst, /*Object*/ src){
			// summary:
			//		Returns if the given two stateful objects are equal.
			// dst: Object
			//		The object to compare with.
			// src: Object
			//		The object to compare with.

			var list = lang.mixin({}, dst, src);
			for(var s in list){
				if(!(s in Stateful.prototype) && s != "_watchCallbacks" && !equals(dst[s], src[s])){ return false; }
			}
			return true;
		},

		equalsValue: function(/*Anything*/ dst, /*Anything*/ src){
			// summary:
			//		Returns if the given two values are equal.

			return dst === src; // Boolean
		}
	};

	var equals = function(/*Anything*/ dst, /*Anything*/ src, /*dojox/mvc/equalsOptions*/ options){
		// summary:
		//		Compares two dojo/Stateful objects, by diving into the leaves.
		// description:
		//		Recursively iterates and compares stateful values.
		// dst: Anything
		//		The stateful value to compare with.
		// src: Anything
		//		The stateful value to compare with.
		// options: dojox/mvc/equalsOptions
		//		The object that defines how two stateful values are compared.
		// returns: Boolean
		//		True if dst equals to src, false otherwise.

		var opts = options || equals, types = [opts.getType(dst), opts.getType(src)];
		return types[0] != types[1] ? false : opts["equals" + types[0].replace(/^[a-z]/, function(c){ return c.toUpperCase(); })](dst, src); // Boolean
	};

	// lang.setObject() thing is for back-compat, remove it in 2.0
	return lang.setObject("dojox.mvc.equals", lang.mixin(equals, equalsOptions));
});

},
'epi-find/util/arrayx':function(){
define([
    "dojo/_base/array"
], function(
    array
) {
    // module:
    //		epi-find/util/arrayx

    var arrayExtensions = {
        // summary:
        //		This module provides utility methods to process and compare arrays.

        intersected: function(/*Array|Object*/ array1, /*Array|Object*/ array2, /*Boolean?*/ caseSensitive, /*Boolean?*/ honorWhitespaces) {
            // summary:
            //     Checks whether two specified arrays intersect.
            // array1:
            //		First array or object to compare.
            // array2:
            //		Second array or object to compare.
            // caseSensitive:
            //		Indicates whether comparison should be case sensitive.
            // honorWhitespaces:
            //		Indicates whether whitespaces should be honored and not ignored when comparing string entities.
            // tags:
            //		public
            // example:
            // |    var array1 = [1, 2], array2 = [2, false];
            // |	var intersected = arrayx.intersected(array1, array2); // should return true.
            // example:
            // |    var foo = " hello  ", array = ["Hello", 10, true];
            // |	var intersected = arrayx.intersected(foo, array); // should return true.

            if (!array1 || !array2) {
                return false;
            }

            var prepare = function (value) {
                if (value === null || value === undefined) {
                    return null;
                }
                if (typeof value === "string") {
                    if (!value) {
                        return null;
                    }
                    if (!caseSensitive) {
                        value = value.toLowerCase();
                    }
                    if (!honorWhitespaces) {
                        value = value.trim();
                    }
                }
                return value;
            };

            array1 = array.map(this.ensureArray(array1), prepare);
            array2 = array.map(this.ensureArray(array2), prepare);

            return array.some(array1, function (phraseItem) {
                if (phraseItem === null || phraseItem === undefined) {
                    return false;
                }
                return array.some(array2, function (synonymItem) {
                    if (synonymItem === null || synonymItem === undefined) {
                        return false;
                    }
                    return synonymItem === phraseItem;
                });
            });
        },

        parse: function (/*String*/ value, /*String*/ separator, /*Boolean?*/ keepWhitespaces, /*Boolean?*/ keepEmptyEntries) {
            // summary:
            //     Parses String value into array using specified separator.
            // value:
            //		Value to parse.
            // keepWhitespaces:
            //		Indicates whether whitespaces should not be trimmed.
            // keepEmptyEntries:
            //		Indicates whether null or undefined entities should be included in resulting array.
            // tags:
            //		public
            // example:
            // |    var value = "first,second   , ,,";
            // |	var parsedArray = arrayx.parse(value, ","); // should return ["first", "second"].
            // example:
            // |    var value = "first,second, whitespaces here ,,";
            // |	var parsedArray = arrayx.parse(value, ",", true); // should return ["first", "second", " whitespaces here "].
            // example:
            // |    var value = 100;
            // |	var parsedArray = arrayx.parse(value, ","); // should return [100].

            if (typeof value !== "string") {
                return this.ensureArray(value);
            }

            var result = value.split(separator);

            if (!keepWhitespaces) {
                result = array.map(result, function (item) {
                    return item.trim();
                });
            }

            if (!keepEmptyEntries) {
                result = array.filter(result, function (item) {
                    return !!item;
                });
            }

            return result;
        },

        ensureArray: function (value) {
            // summary:
            //     Returns original value if it is array or creates array with specified value as an entry.
            // value:
            //		Value that should be "represented" as an array.
            // tags:
            //		public
            // example:
            // |    var value = [1, 2];
            // |	var result = arrayx.ensureArray(value); // should return original [1, 2].
            // example:
            // |    var value = "some text";
            // |	var result = arrayx.ensureArray(value); // should return ["some text"].
            // example:
            // |    var value = null;
            // |	var result = arrayx.ensureArray(value); // should return empty array [].

            if (value === null || value === undefined) {
                return [];
            }
            return value instanceof Array ? value : [value];
        }
    };

    return arrayExtensions;
});
},
'url:epi-find/widget/templates/SynonymForm.html':"<form data-dojo-attach-point='containerNode' data-dojo-attach-event='onreset:_onReset,onsubmit:_onSubmit' class='epi-formsWidgetWrapper epi-vertical-spacer--double'>\r\n    <ol class='epi-formsWidgetWrapper'>\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\" data-dojo-attach-point=\"phraseFieldListItem\">\r\n            <input id=\"phraseField\" data-dojo-attach-point=\"phraseField\" data-dojo-type=\"epi-find/widget/PhraseValidationTextBox\"\r\n                   data-dojo-props=\"required:true, intermediateChanges: true, phraseDelimiter:',', invalidMessage: '${i18nPhraseHelp}'\" />\r\n        </li>\r\n\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\" data-dojo-attach-point=\"synonymFieldListItem\">\r\n            <input id=\"synonymField\" data-dojo-attach-point=\"synonymField\" data-dojo-type=\"epi-find/widget/PhraseValidationTextBox\"\r\n                   data-dojo-props=\"required:true, intermediateChanges: true, invalidMessage: '${i18nSynonymHelp}'\" />\r\n            <p class=\"epi-help\">${i18nSynonymHelp}</p>\r\n        </li>\r\n\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\" data-dojo-props=\"class: 'epi-formsRow--fullWidthInput'\" data-dojo-attach-point=\"bidirectionalCheckboxListItem\">\r\n            <input id=\"bidirectionalCheckbox\" data-dojo-attach-point=\"bidirectionalCheckbox\" data-dojo-type=\"dijit/form/CheckBox\" />\r\n            <p class=\"epi-help\">${i18n.help_bidirectional}</p>\r\n        </li>\r\n\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\" class=\"epi-formsRow--align-right\">\r\n            <input data-dojo-attach-point=\"submitButton\" type=\"submit\" data-dojo-type=\"dijit/form/Button\" class=\"epi-primary\" />\r\n            <input data-dojo-attach-point=\"resetButton\" type=\"reset\" data-dojo-type=\"dijit/form/Button\" />\r\n        </li>\r\n    </ol>\r\n</form>\r\n",
'epi/shell/dgrid/Formatter':function(){
﻿define("epi/shell/dgrid/Formatter", [
// dojo
    "dojo/_base/declare",
    "dojo/_base/array",
    "dojo/when",

    // DGrid
    "dgrid/Grid",

    // epi
    "epi/shell/dgrid/TemplateColumn"
], function (
// dojo
    declare,
    array,
    when,
    //
    Grid,
    // epi
    TemplateColumn
) {
    var module = declare(null, {
        // summary:
        //      Add formating capability for dgrid column cell.
        // tags:
        //      public

        _isGrid: false,

        configStructure: function () {
            // summary:
            //      Setup templated column(s)
            // tags:
            //      public

            this.inherited(arguments);

            this._isGrid = this instanceof Grid;

            if (this._isGrid) {
                this._setupTemplateColumns();
            } else {
                this._setupList();
            }
        },

        _setupTemplateColumns: function () {
            // summary:
            //      Loop all dgrid columns, check each is able to template column, convert each.
            // tags:
            //      private

            if (!this.subRows) {
                return;
            }

            var formatters;
            array.forEach(this.subRows, function (columns) {
                array.forEach(columns, function (column) {
                    formatters = [];

                    array.forEach(module.shorthands, function (shorthand) {
                        var formatterActive = column[shorthand],
                            formatter = module.formatters[shorthand];

                        if (formatterActive) {
                            if (formatter) {
                                // If not a bool value, execute formatterFunc with value that should be a factory method
                                formatters.push(formatter.isFactory ?
                                    formatter.handler(formatterActive) :
                                    formatter.handler);
                            } else {
                                throw new Error("Formatter with key " + shorthand + " not found");
                            }

                        }
                    });

                    // Add custom formatters
                    if (column.formatters) {
                        this._findFormatters(column.formatters, formatters);
                    }

                    if (formatters.length > 0) {
                        column = TemplateColumn(column, formatters);
                    }
                }, this);
            }, this);
        },

        _setupList: function () {
            // summary:
            //      Add configured formatters when used with dgrid list.
            // tags:
            //      private

            if (this.formatters) {
                this._resolvedFormatters = this._findFormatters(this.formatters);
            } else if (this.field) {
                this._resolvedFormatters = [function (data, object, row) {
                    return data;
                }];
            }
        },

        renderRow: function (object, options) {
            // summary:
            //      Override of default implementation to add formatter execution.
            // tags:
            //      public

            if (this._isGrid) {
                return this.inherited(arguments);
            }

            var row = this.inherited(arguments, ["", options]),
                data = this.field ? object[this.field] : object,
                formatters = this._resolvedFormatters,
                i = -1;

            function executeFormatter(value) {
                var formatter = formatters[i],
                    result;

                if (formatter) {
                    row.innerHTML = value;
                    data = value;
                }

                i++;

                if (formatters.length > i) {
                    formatter = formatters[i];
                    result = formatter(data, object, row);
                    when(result, executeFormatter);
                }
            }

            if (formatters.length > 0) {
                executeFormatter();
            }

            return row;

        },

        _findFormatters: function (formatterConfig, formatters) {
            // summary:
            //      Loops throgh an array that can consist of either formatter functions
            //      or formatter keys as strings. When a string is found it is replaced
            //      by a matching formatter function from the registered formatters.
            // tags:
            //      private

            formatters = formatters || [];

            array.forEach(formatterConfig, function (formatterKey) {
                var type = typeof formatterKey,
                    item, formatter;

                if (type === "string") {
                    item = module.formatters[formatterKey];

                    if (item) {
                        formatter = item.isFactory ? item.handler() : item.handler;
                        formatter.allParameters = item.allParameters;
                    }
                } else if (type === "function") {
                    formatter = formatterKey;
                }

                if (formatter) {
                    formatters.push(formatter);
                } else {
                    throw new Error("Formatter with key " + formatterKey + "not found");
                }
            });

            return formatters;
        }
    });

    module.formatters = {};
    module.shorthands = [];

    module.addFormatter = function (name, formatter, isFactory, addShortHand) {
        // summary:
        //      Adds formatter to the formatter registry.
        // tags:
        //      internal

        module.formatters[name] = { handler: formatter, isFactory: !!isFactory };
        if (addShortHand) {
            module.shorthands.push(name);
        }
    };

    return module;
});

},
'epi/shell/dgrid/TemplateColumn':function(){
﻿define("epi/shell/dgrid/TemplateColumn", [
// dojo
    "dojo/_base/array"
],
function (
// dojo
    array
) {
    return function (/*object*/column, /*array*/formatters) {
        // summary:
        //      Dgrid template column plugin
        // column: [object]
        //      Column definition
        // formatters: [array]
        //      Array of callback function(s)
        // tags:
        //      public

        var originalRenderCell = column.renderCell;

        // accept arguments as parameters to template function, or from column def,
        // but normalize to column def.
        column.formatters = formatters || column.formatters;

        if (column.formatters && column.formatters.length > 0) {
            column.renderCell = function (object, value, cell, options) {
                // Get each formatter and run it.
                array.forEach(column.formatters, function (formatter) {
                    value = formatter(value, object, cell, options);
                });

                if (value) {
                    cell.innerHTML = value;
                }
            };
        }

        // REMARK: This code will most likely fail in the case where renderCell is defined on the column,
        //         since the parameters that get passed through to the method are undefined here.
        return originalRenderCell ? originalRenderCell.call(column /* , object, value, cell, options */) : column;
    };
});

},
'epi-cms/dgrid/formatters':function(){
define("epi-cms/dgrid/formatters", [
// dojo
    "dojo/_base/lang",
    "dojo/dom-class",

    // epi
    "epi/datetime",
    "epi/username",
    "epi/string",

    "epi/shell/dgrid/Formatter",
    "epi/shell/dgrid/util/misc",
    "epi/shell/TypeDescriptorManager",

    "epi-cms/contentediting/ContentActionSupport",

    // Resources
    "dojo/i18n!epi/nls/episerver.cms.components.versions"
],
function (
// dojo
    lang,
    domClass,

    // epi
    epiDate,
    username,
    epiString,

    Formatter,
    misc,
    TypeDescriptorManager,

    ContentActionSupport,

    resources
) {
    var module = {
        // summary:
        //      Static module that defines miscellaneous formatter methods
        //      for use with the dgrid.
        // tags:
        //      public

        defaultIconClass: "epi-iconObjectUnknown",

        thumbnail: function (thumbnailUrl) {
            var template = "<image src=\"{0}?{1}\" class=\"epi-floatLeft epi-thumbnail\"/>";
            return thumbnailUrl ? lang.replace(template, [thumbnailUrl, new Date().getTime()]) : "";
        },

        contentIcon: function (typeIdentifier, additionalClass) {
            // summary:
            //      Adds an icon as specified for the type or a default object icon
            // typeIdentifier:
            //      key for the type
            // additionalClass: String?
            //      additional css class
            // tags:
            //      public

            return misc.icon(module.contentIconClass(typeIdentifier, additionalClass));
        },

        contentIconClass: function (typeIdentifier, additionalClass) {
            // summary:
            //      class name of an icon as specified for the type or a default object icon
            // typeIdentifier:
            //      key for the type
            // additionalClass: String?
            //      additional css class
            // tags:
            //      internal

            var iconNodeClass = TypeDescriptorManager.getValue(typeIdentifier, "iconClass");

            var input = iconNodeClass + " epi-objectIcon";
            if (additionalClass) {
                input += " " + additionalClass;
            }

            return input;
        },

        contentTypeDescription: function (typeIdentifier) {
            // summary:
            //      Return content type description
            // typeIdentifier:
            //      key for the type
            // tags:
            //      public
            return TypeDescriptorManager.getResourceValue(typeIdentifier, "name", false);
        },

        languageBranch: function (languageBranch) {
            // summary:
            //      Adds an indicator of the true language for the content object
            // languageBranch:
            //      Languagebranch object.
            // tags:
            //      public
            return languageBranch ? "<span class=\"epi-missingLanguageIndicator\">" + languageBranch.language + "</span>" : "";
        },

        menu: function (/*Object*/settings) {
            // summary:
            //      Adds a menu icon
            // settings:
            //      Settings information for menu
            // tags:
            //      public

            return lang.replace("<span class=\"dijitInline dijitIcon epi-iconContextMenu epi-floatRight\" title=\"{title}\">&nbsp;</span>", settings);
        },

        path: function (path) {
            // summary:
            //      Returns markup for the path to an item created from the names in the path array
            // path: String[]
            //      An array of with the path to render
            // tags:
            //      public

            if (path && path.length) {
                var title = misc.attributeEncode(path.join(" > "));
                path = epiString.truncateMiddle(path, 40, "\u2026");

                var text = path.map(misc.htmlEncode).join("<span class=\"epi-breadCrumbsSeparator\">&gt;</span>");

                return lang.replace("<span class=\"dgrid-column-treePath dojoxEllipsis\" title=\"{title}\">{text}</span>", {
                    title: title, text: text
                });
            }
            return "";
        },

        userName: function (userName) {
            // summary:
            //      Shim for the epi/username/toUserFriendlyHtml function
            // userName:
            //      The user name.
            // tags:
            //      public
            return username.toUserFriendlyHtml(userName);
        },

        localizedDate: function (date) {
            // summary:
            //      Shim for the epi/date/toUserFriendlyHtml function
            // date:
            //      The date to format.
            // tags:
            //      public
            return epiDate.toUserFriendlyHtml(date);
        },

        friendlyBoolean: function (value) {
            // summary:
            //      Creates a div with a "checked/not checked" css class.
            // tags:
            //      public
            var cssClass = value ? "epi-iconCheckmark" : "epi-iconStop";
            return "<div class=\"" + cssClass + "\" />";
        },

        versionStatus: function (versionId) {
            // summary:
            //      Shim for the epi-cms/contentediting/ContentActionSupport.getVersionStatus function
            // versionId:
            //      The value to format.
            // tags:
            //      public
            return ContentActionSupport.getVersionStatus(versionId);
        },

        commonDraft: function (item, value, node) {
            // summary:
            //      Adds an addition node to the inner HTML of the node if the item is a common draft.
            // tags:
            //      public
            if (item.isCommonDraft) {
                node.innerHTML = "<span class=\"dijitReset dijitInline dijitIcon epi-iconPrimary epi-floatRight\" title=\"" + resources.primarydraft + "\"></span>";
            }
            node.innerHTML += module.versionStatus(value);
        },

        visibleLink: function (value, valueIsHtml) {
            // summary:
            //      Wrappes the value with a span with the 'epi-visibleLink' css class
            // valueIsHtml:
            //      Pass true to skip html encoding of the value. The default is to encode the value.
            // tags:
            //      public

            if (!value) {
                return "";
            }

            var text = !valueIsHtml ? misc.htmlEncode(value) : value;

            return "<span class=\"epi-visibleLink\">" + text + "</span>";
        },

        secondaryText: function (value, valueIsHtml) {
            // summary:
            //      Wrappes the value with a span with the 'epi-secondaryText' css class
            // valueIsHtml:
            //      Pass true to skip html encoding of the value. The default is to encode the value.
            // tags:
            //      public

            if (!value) {
                return "";
            }

            var text = !valueIsHtml ? misc.htmlEncode(value) : value;

            return "<span class=\"epi-secondaryText\">" + text + "</span>";
        },

        contentItemFactory: function (textSelector, titleSelector, iconSelector, menuSettings, thumbnailSelector, languageSelector, isEncoded) {
            // summary:
            //      Factory function for configuring the contentItem formatter
            // textSelector:
            //      Selector function or property key of content object
            // titleSelector:
            //      Selector function or property key of content object
            // iconSelector:
            //      Selector function or property key of content object
            // menuSettings:
            //      Settings information for menu
            // isEncoded: Boolean
            //      Flags which is used to enabled/disable encode mode. (default true)
            // tags:
            //      public

            return function (value, object, node, options) {
                var returnValue,
                    typeIdentifier,
                    text,
                    title,
                    missingLanguageBranch,
                    valueIsString,
                    thumbnailUrl;

                function get(name) {
                    if (typeof object.get === "function") {
                        return object.get(name);
                    }
                    return object[name];
                }

                function selectorExecutor(selector) {
                    if (selector) {
                        if (typeof selector === "function") {
                            return selector(object);
                        } else {
                            return get(selector);
                        }
                    }
                    return value;
                }

                valueIsString = typeof value === "string";
                typeIdentifier = selectorExecutor(iconSelector || (valueIsString ? false : "typeIdentifier"));
                text = selectorExecutor(textSelector || (valueIsString ? false : "text"));
                title = selectorExecutor(titleSelector || (valueIsString ? false : "title"));
                missingLanguageBranch = (languageSelector && selectorExecutor(languageSelector)) || object.missingLanguageBranch;
                thumbnailUrl = thumbnailSelector && selectorExecutor(thumbnailSelector);

                returnValue = module.contentItem(typeIdentifier, missingLanguageBranch, text, title, menuSettings, thumbnailUrl, isEncoded);

                node.innerHTML = returnValue;

                //If in a list add css-class for the row as well
                if (node.nodeName !== "TD" && missingLanguageBranch) {
                    domClass.add(node, "epi-ct-missingLanguageRow");
                }

                return returnValue;
            };
        },

        contentItem: function (typeIdentifier, missingLanguageBranch, text, title, menuSettings, thumbnailUrl, isEncoded) {
            // summary:
            //      Formats a content object with icon missing language indicator and an optional menu
            // typeIdentifier:
            //      content type key for selecting the icon
            // missingLanguageBranch:
            //      Content items language information
            // text:
            //      The name or description of the content
            // title:
            //      The name or description of the content used for the title attribute.
            //      If not specified, the text value will be used
            // menuSettings:
            //      Settings information for menu
            // thumbnailUrl:
            //      The image public url of content
            // isEncoded: Boolean
            //      The flag which is used to enale/disable encode mode. (default true)
            // tags:
            //      public

            var menu = (menuSettings && menuSettings.hasMenu) ? module.menu(menuSettings.settings) : "",
                languageBranch = module.languageBranch(missingLanguageBranch);

            isEncoded = (isEncoded === null || isEncoded === undefined) ? true : isEncoded;
            title = isEncoded ? misc.attributeEncode(title || text) : (title || text);
            text = isEncoded ? misc.htmlEncode(text) : text;

            return (thumbnailUrl ? module.thumbnail(thumbnailUrl) : module.contentIcon(typeIdentifier)) + menu + languageBranch + misc.ellipsis(text, title);
        }
    };

    Formatter.addFormatter("contentIcon", module.contentIcon);
    Formatter.addFormatter("userName", module.userName);
    Formatter.addFormatter("versionStatus", module.versionStatus);
    Formatter.addFormatter("localizedDate", module.localizedDate);
    Formatter.addFormatter("contentItem", module.contentItemFactory, true);

    return module;
});

},
'epi/username':function(){
define("epi/username", [
    "dojox/html/entities",
    "epi/dependency",
    "dojo/i18n!epi/nls/episerver.shared.text"
],

function (
    entities,
    dependency,
    localizations
) {

    return {
        // tags:
        //      public

        toUserFriendlyString: function (username, currentUsername, capitalizeUsername, keepEmptyUsername, useSubjectPronoun) {
            // summary:
            //      Gets a user friendly username. If the username to be displayed is the same as
            //      the current username, then this will return "you".
            // username: String
            //      The username to be displayed.
            // currentUsername: String?
            //      The username to compare with to see if it's the current user.
            // capitalizeUsername: Boolean?
            //      If the first character should be in upper case.
            // keepEmptyUsername: Booelan?
            //      If the empty username should be kept or replaced by "Installer".
            // useSubjectPronoun: Booelan?
            //      True when the subject pronoun should be used to replace the username when it
            //      matches with the current username; otherwise false to use the object pronoun.
            // tags:
            //      public

            if (!currentUsername) {
                currentUsername = this._getCurrentUsername();
            }

            var emptyUsername = keepEmptyUsername ? "" : localizations.installer,
                youUsername = useSubjectPronoun ? localizations.yousubject : localizations.youobject,
                friendlyUsername = username && currentUsername.toLowerCase() === username.toLowerCase() ? youUsername : username || emptyUsername;

            if (capitalizeUsername) {
                friendlyUsername = friendlyUsername.charAt(0).toUpperCase() + friendlyUsername.slice(1);
            }
            return friendlyUsername;
        },

        toUserFriendlyHtml: function (username, currentUsername, capitalizeUsername, keepEmptyUsername, useSubjectPronoun) {
            // summary:
            //      Creates a user friendly version of the username and wraps it inside standardized
            //      HTML for styling purposes.
            // username: String
            //      The username to be displayed.
            // currentUsername: String?
            //      The username to compare with to see if it's the current user.
            // capitalizeUsername: Boolean?
            //      If the first character should be in upper case.
            // keepEmptyUsername: Booelan?
            //      If the empty username should be kept or replaced by "Installer".
            // useSubjectPronoun: Booelan?
            //      True when the subject pronoun should be used to replace the username when it
            //      matches with the current username; otherwise false to use the object pronoun.
            // tags:
            //      public

            var formattedUserName = this.toUserFriendlyString(username, currentUsername, capitalizeUsername, keepEmptyUsername, useSubjectPronoun);
            if (formattedUserName === "") {
                return formattedUserName;
            }
            return "<span class='epi-username'>" + entities.encode(formattedUserName) + "</span>";
        },

        _getCurrentUsername: function () {
            if (!this.currentUsername) {
                var profile = dependency.resolve("epi.shell.Profile");
                //TODO: Make this Async
                this.currentUsername = profile.get("userName");
            }
            return this.currentUsername;
        }
    };
});

},
'epi/string':function(){
﻿define("epi/string", [
// dojo
    "dojo/_base/kernel",
    "dojo/_base/lang",
    "dojox/html/entities"
],

function (
// dojo
    kernel,
    lang,
    entities
) {

    var nonEvaluatingBodyTag = document.implementation.createHTMLDocument("nonEvaluating").body;

    return {
        // tags:
        //      public

        pascalToCamel: function (/*String*/input) {
            // summary:
            //      Convert a pascal cased string into a camel cased one.
            // tags:
            //      public

            var parts = input.split(".");
            var output = this._toCamelCase(parts[0]);
            for (var i = 1; i < parts.length; i++) {
                output += "." + this._toCamelCase(parts[i]);
            }

            return output;
        },

        _toCamelCase: function (input) {

            if (!input) {
                return "";
            }

            if (input[0] === input[0].toLowerCase()) {
                return input;
            }

            var output = "";
            //Use the same algorithm as Newtonsoft Json.NET
            for (var i = 0; i < input.length; i++) {
                var hasNext = (i + 1 < input.length);
                if ((i === 0 || !hasNext) || input[i + 1] === input[i + 1].toUpperCase()) {
                    output += input[i].toLowerCase();
                } else {
                    output += input.substring(i);
                    break;
                }
            }
            return output;
        },

        stripHtmlTags: function (/*String*/input) {
            // summary:
            //      Get text data from the given HTML string
            //      Trim all spaces in the return value
            // tags:
            //      public

            if (this.isNullOrEmpty(input)) {
                return "";
            }

            nonEvaluatingBodyTag.innerHTML = input.replace(/&/gim, "&amp;");
            return this.removeAllCarriageReturns(nonEvaluatingBodyTag.textContent);
        },

        removeAllCarriageReturns: function (/*String*/input) {
            // summary:
            //      Remove all carriage returns from the input string
            // input: [String]
            //      Input string that wanted to remove all carriage returns
            // tags:
            //      public

            if (this.isNullOrEmpty(input)) {
                return "";
            }

            var output = input.replace(/\t|\r|\n/gim, "");

            // Best performance for each browser
            // Refer: http://jsperf.com/removing-multiple-spaces
            return output.replace(/ {2,}/gim, " ");
        },

        encodeForWebString: function (/*String*/input, /*Array*/uiSafeHtmlTags) {
            // summary:
            //      Encode text to display as Html.
            //      Not allowed html tags (not in uiSafeHtmlTags) will be rendered as text.
            // tags:
            //      public

            if (this.isNullOrEmpty(input)) {
                return "";
            }

            var safeString = entities.encode(input);
            if (!uiSafeHtmlTags || uiSafeHtmlTags.length === 0) {
                return safeString;
            }

            var pattern = new RegExp("&lt;(/?(" + uiSafeHtmlTags.join("|") + ")/?\\s*)&gt;", "gim");
            return safeString.replace(pattern, "<$1>");
        },

        isNullOrEmpty: function (/*String*/input) {
            // summary:
            //      Checks if the value is null, undefined, empty or whitespace.
            // tags:
            //      public

            return !input || (/^\s*$/).test(input);
        },

        toHTML: function (/*String*/input) {
            // summary:
            //      Replace line breaks with HTML breaks.
            // tags:
            //      public

            if (this.isNullOrEmpty(input)) {
                return "";
            }
            return input.replace(/\r\n|\n|\r|\\n/g, "<br/>");
        },

        toTooltipText: function (/* String */input) {
            // summary:
            //      Normalize text to display correctly as the value of HTML tooltip attribute.
            // tags:
            //      public

            if (this.isNullOrEmpty(input)) {
                return "";
            }

            return input.replace(/\\n/gim, "\n");
        },

        appendTrailingSlash: function (/*String*/url) {
            // summary:
            //      Appends a trailing / if it does not exists
            // tags:
            //      public

            return url.replace(/\/?$/, "/");
        },

        truncateMiddle: function (words, maxLength, ellipsisChar) {
            // summary:
            //      Truncates a list of words to the max length given
            // words: Array|String
            //      The words to truncate
            // maxLength: Number
            //      The max length (number of characters)
            // ellipsisChar: String?
            //      The characters to use as the ellipsis, default is '&hellip;'
            // returns:
            //      A truncated list of words with the ellipsis in the middle
            // tags:
            //      internal

            if (typeof words === "string") {
                words = words.split(/\s+/g);
            } else {
                //Clone the array
                words = words.concat();
            }

            //If it fits just return
            if (words.join("").length <= maxLength) {
                return words;
            }

            var splitOnWord = function (words, maxLength, fromLeft) {
                if (fromLeft) {
                    //Reverse the array to get the the words from the end
                    return splitOnWord(words.reverse(), maxLength).reverse();
                }
                var count = 0;
                return words.filter(function (word) {
                    count += word.length;
                    return count <= maxLength;
                });
            };

            var lengthFirstPart = Math.ceil(maxLength / 2);
            var lengthSecondPart = Math.floor(maxLength / 2);

            var firstPart = splitOnWord(words, lengthFirstPart);

            //Add the ellipsis character
            ellipsisChar = ellipsisChar || "&hellip;";
            firstPart.push(ellipsisChar);

            //Combine the first part and the second part
            return firstPart.concat(splitOnWord(words, lengthSecondPart, true));
        }

    };

});

},
'epi/shell/dgrid/util/misc':function(){
require({cache:{
'url:dojox/html/resources/ellipsis.css':".dojoxEllipsis,\n.dojoxEllipsisShown {\n\twhite-space: nowrap;\n\twidth: 100%;\n\toverflow: hidden;\n\ttext-overflow: ellipsis;\n\t-o-text-overflow: ellipsis;\n\t-webkit-text-overflow: ellipsis;\n}\n\n.dojoxEllipsis window {\n    width:100%;\n    -moz-user-focus:normal;\n    -moz-user-select:text;\n}\n.dojoxEllipsis description{\n    -moz-user-focus:normal;\n    -moz-user-select:text;\n}\n.dojoxEllipsisIFrame{\n\twhite-space: normal;\n\tborder: none;\n\twidth: 100%;\n\tdisplay: block;\n\theight: 1px;\n\tmargin-top: -1px;\n\tclear: both;\n}\n.dojoxEllipsisContainer{\n\twidth: 100%;\n}\n.dojoxEllipsisShown:after{\n\tcontent: \"\\2026\"\n}\n"}});
﻿define("epi/shell/dgrid/util/misc", [
// dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    // dojox
    "dojox/html/entities",

    // EPi
    "epi/shell/dgrid/Formatter",

    // resources
    "dojo/text!dojox/html/resources/ellipsis.css"
],
function (
// dojo
    declare,
    lang,
    // dojox
    htmlEntities,
    // EPi
    Formatter,
    // resources
    ellipsisCss
) {

    var module = {
        // summary:
        //      This static module defines miscellaneous utility methods.
        // tags:
        //      public

        _attributeEncodeMap: [["&","amp"], ["\"","quot"]],

        htmlEncode: function (text) {
            // summary:
            //      Make html encode for the given value
            // text:
            //      The text you want to apply html encode
            // tags:
            //      public

            return htmlEntities.encode(text);
        },

        attributeEncode: function (text) {
            return htmlEntities.encode(text, module._attributeEncodeMap);
        },

        ellipsis: function (text, title) {
            // summary:
            //      Make ellipsis for the given text
            // text:
            //      The text you want to apply html ellipsis
            // tags:
            //      public

            if (!title || !lang.isString(title)) {
                title = text;
            }

            return lang.replace("<div class=\"dojoxEllipsis\" title=\"{0}\">{1}</div>", [title, text]);
        },

        ellipsisNoTooltip: function (text) {
            // summary:
            //      Make ellipsis for the given text
            // text:
            //      The text you want to apply html ellipsis
            // tags:
            //      public

            return lang.replace("<div class=\"dojoxEllipsis\">{0}</div>", [text]);
        },

        icon: function (icon, text) {
            // summary:
            //      Add an icon span to the front of the given text
            // text:
            //      The text you want to add an icon in front
            // tags:
            //      public

            text = text || "";

            return lang.replace("<span class=\"dijitInline dijitIcon {0}\"></span>{1}", [icon, text]);
        }
    };

    Formatter.addFormatter("htmlEncode", module.htmlEncode, false, true);
    Formatter.addFormatter("icon", function (icon) {
        return function (value) {
            return module.icon(icon, value);
        };
    }, true, true);
    Formatter.addFormatter("ellipsis", module.ellipsis, false, true);
    Formatter.addFormatter("ellipsisNoTooltip", module.ellipsisNoTooltip, false, true);

    return module;

});

},
'url:dojox/html/resources/ellipsis.css':".dojoxEllipsis,\n.dojoxEllipsisShown {\n\twhite-space: nowrap;\n\twidth: 100%;\n\toverflow: hidden;\n\ttext-overflow: ellipsis;\n\t-o-text-overflow: ellipsis;\n\t-webkit-text-overflow: ellipsis;\n}\n\n.dojoxEllipsis window {\n    width:100%;\n    -moz-user-focus:normal;\n    -moz-user-select:text;\n}\n.dojoxEllipsis description{\n    -moz-user-focus:normal;\n    -moz-user-select:text;\n}\n.dojoxEllipsisIFrame{\n\twhite-space: normal;\n\tborder: none;\n\twidth: 100%;\n\tdisplay: block;\n\theight: 1px;\n\tmargin-top: -1px;\n\tclear: both;\n}\n.dojoxEllipsisContainer{\n\twidth: 100%;\n}\n.dojoxEllipsisShown:after{\n\tcontent: \"\\2026\"\n}\n",
'epi-cms/contentediting/ContentActionSupport':function(){
define("epi-cms/contentediting/ContentActionSupport", [
// resources
    "dojo/i18n!epi/nls/episerver.cms.versionstatus"
],

function (
// resources
    res
) {

    var versionLocalizations = {},
        actionSupport;

    if (res) {
        versionLocalizations = {
            0: res.notcreated,
            1: res.rejected,
            2: res.checkedout,
            3: res.checkedin,
            4: res.published,
            5: res.previouslypublished,
            6: res.delayedpublish,
            7: res.awaitingapproval,
            100: res.expired
        };
    }

    actionSupport = {
        // summary:
        //    This class helps dealing with the content data structure:
        //      * decode the access level mask provided by content data store
        //      * determine if an action is supported for a given version statuse
        // tags:
        //    public static

        // versionStatus: Object
        //      Version status codes used in contentContext.status
        versionStatus: {
            NotCreated: 0,
            Rejected: 1,
            CheckedOut: 2,
            CheckedIn: 3,
            Published: 4,
            PreviouslyPublished: 5,
            DelayedPublish: 6,
            AwaitingApproval: 7,
            Expired: 100
        },

        versionLocalizations: versionLocalizations,

        getVersionStatus: function (status) {
            return this.versionLocalizations[status];
        },

        // action: Object
        //      Known actions on a contentData object
        action: {
            Create: "Create",
            Edit: "Edit",
            Delete: "Delete",
            CheckIn: "CheckIn",
            Save: "Save",
            Reject: "Reject",
            Publish: "Publish",
            Administer: "Administer"
        },

        // accessLevel: Object
        //      Access level components of the contentContext.accessMask bitmask.
        accessLevel: {
            NoAccess: 0,
            Read: 1,
            Create: 2,
            Edit: 4,
            CheckIn: 4,
            Delete: 8,
            Publish: 16,
            Administer: 32,
            FullAccess: 63
        },

        // saveAction: Object
        //      Options available when saving content
        saveAction: {
            /// Do not save data.
            None: 0x0,
            /// Save a page, leaving it in a checked out state.
            Save: 0x1,
            /// Save and check in page, creating a new version only if necessary.
            CheckIn: 0x2,
            /// Publish page, creating a new version only if necessary.
            Publish: 0x3,
            /// Reject a checked-in page.
            Reject: 0x4,
            /// Sets a content instance to approval state.
            RequestApproval: 0x5,
            /// Sets a content instance in checked out/draft state.
            CheckOut: 0x6,
            /// Schedules a version for publishing.
            Schedule: 0x7,
            /// Flag that is used to force the creation of a new version.
            ForceNewVersion: 0x80,
            /// Save and check in page, always updating the current version
            ForceCurrentVersion: 0x100,
            /// Does not validate the data against <see cref:"EPiServer.Validation.IValidationService"/>
            SkipValidation: 0x200,
            /// Save and check in page, creating a new version only if necessary and sets the content as delayed publish.
            DelayedPublish: 0x400
        },

        // sortOrder: Object
        //      Sort order of contentData, need to verify when change
        sortOrder: {
            None: 0,
            CreatedDescending: 1,
            CreatedAscending: 2,
            Alphabetical: 3,
            Index: 4,
            ChangedDescending: 5,
            Rank: 6,
            PublishedAscending: 7,
            PublishedDescending: 8
        },

        // providerCapabilities: Object
        //      Capabilities bitmask of a content provider
        providerCapabilities: {
            None: 0,
            Create: 1,
            Edit: 2,
            Delete: 4,
            Move: 8,
            Copy: 16,
            MultiLanguage: 32,
            Security: 64,
            Search: 128,
            PageFolder: 256,
            Wastebasket: 512
        },


        isActionAvailable: function (contentData, actionType, providerCapabilityType, skipMissingLanguageCheck, skipLanguageAccess) {
            // summary:
            //    Returns true if the given action type is authorized and represents an allowed state transition for the given contentData context.
            //    This also checks for the capability of content provider whether is configured to allow the action.
            //    If the specified contentData has the missingLanguageBranch property set it will return false.
            // tags:
            //    public

            return actionSupport.hasAccessToAction(contentData, actionType, providerCapabilityType, skipMissingLanguageCheck, skipLanguageAccess) &&
                   actionSupport.canPerformAction(contentData, actionType);

        },

        hasAccessToAction: function (contentData, actionType, providerCapabilityType, skipMissingLanguageCheck, skipLanguageAccess) {
            // summary:
            //    Returns true if the given action type is authorized and the capability of content provider
            //    whether is configured to allow the action.
            //    If the specified contentData has the missingLanguageBranch property set it will return false.
            // tags:
            //    internal

            var access;

            if (!contentData) {
                return false;
            }

            if (!skipMissingLanguageCheck && contentData.capabilities && contentData.capabilities.language && contentData.missingLanguageBranch && contentData.missingLanguageBranch.isTranslationNeeded) {
                return false;
            }

            if (providerCapabilityType && !actionSupport.hasProviderCapability(contentData.providerCapabilityMask, providerCapabilityType)) {
                return false;
            }

            access = skipLanguageAccess ? contentData.accessRights : contentData.accessMask;

            return actionSupport.hasAccess(access, actionSupport.accessLevel[actionType]);
        },

        canPerformAction: function (contentContext, actionType) {
            // summary:
            //    Returns true if the action represents an allowed state transition.
            // tags:
            //    public

            var contentStatus = contentContext.status,
                versionStatus = actionSupport.versionStatus;

            // If the content is scheduled for published, we considered it locked for edit.
            if (contentStatus === versionStatus.DelayedPublish &&
               (actionType === actionSupport.action.Edit || actionType === actionSupport.action.Publish)) {
                return false;
            }

            if (actionType === actionSupport.action.Publish) {
                // we can publish everything that's not already published or expired
                return !(contentStatus === versionStatus.Published ||
                    contentStatus === versionStatus.Expired);
            }
            if (actionType === actionSupport.action.CheckIn) {
                // we can make "not ready" (checkedout) or rejected content "ready to publish" (checkedin)
                return contentStatus === versionStatus.CheckedOut ||
                    contentStatus === versionStatus.Rejected;
            }
            if (actionType === actionSupport.action.Edit) {
                var readonlyVersion =
                    (contentStatus === versionStatus.Published && !contentContext.isCommonDraft) ||
                    (contentStatus === versionStatus.Expired && !contentContext.isCommonDraft) ||
                    contentStatus === versionStatus.PreviouslyPublished ||
                    (contentStatus === versionStatus.CheckedIn && !contentContext.isPartOfAnotherProject) ||
                    contentStatus === versionStatus.DelayedPublish ||
                    contentStatus === versionStatus.AwaitingApproval;

                return !(readonlyVersion || contentContext.isDeleted || contentContext.isWastebasket);
            }
            return true;
        },

        hasAccess: function (accesLevelMask, requiredAccessLevel) {
            // summary:
            //    Returns true if the required access is part of the given access mask.
            //    The access mask can be accessed from a contentData context or lightweight contentData object.
            // tags:
            //    public
            return (accesLevelMask & requiredAccessLevel) === requiredAccessLevel;
        },

        hasLanguageAccess: function (item) {
            // summary:
            //      Verify the current content have access with the given language or not
            // context: Object
            //      A content data or context object
            // tags:
            //      public

            if (!item) {
                return false;
            }

            // Get the language information from either the context or the content data since we don't know what type we have.
            var language = item.languageContext || item.missingLanguageBranch || item;

            return !language || (language.hasTranslationAccess && language.isPreferredLanguageAvailable); // Boolean
        },

        getPermissionMap: function (accesLevelMask) {
            // summary:
            //    Maps the given access mask onto an object, for example:
            //    { Read:true, Edit:true, Delete:false, Publish:false, Administer:false }
            // tags:
            //    public

            var map = {};
            for (var al in actionSupport.accessLevel) {
                map[al] = actionSupport.hasAccess(accesLevelMask, actionSupport.accessLevel[al]);
            }
            return map;
        },

        hasProviderCapability: function (providerCapabilityMask, requiredCapability) {
            // summary:
            //     Returns true if the required capability is part of the given content provider capability mask.
            // tags:
            //    public

            return (providerCapabilityMask & requiredCapability) === requiredCapability;
        }
    };
    return actionSupport;
});

},
'url:epi-find/widget/templates/Synonyms.html':"<div>\r\n    <p class=\"epi-view--description\">${i18n.description}</p>\r\n    <form data-dojo-attach-point=\"synonymForm\"></form>\r\n    <p class=\"epi-help\">${i18n.help_table}</p>\r\n    <div class=\"epi-grid-title-search\" data-dojo-attach-point=\"synonymsHeader\">\r\n        <h1>${i18n.header}</h1>\r\n        <div class=\"epi-search-box\">\r\n            <input type=\"text\" name=\"query\"\r\n                   class=\"epi-search-input\"\r\n                   data-dojo-type=\"dijit/form/TextBox\"\r\n                   data-dojo-props=\"trim: true, intermediateChanges: true\"\r\n                   data-dojo-attach-point=\"searchNode\" placeholder=\"${i18n.search}\" />\r\n        </div>\r\n    </div>\r\n    <div id=\"synonymgrid\" data-dojo-attach-point=\"synonymGrid\"></div>\r\n</div>",
'epi-find/widget/Bestbets':function(){
﻿define([
    "dojo/_base/config",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-class",
    "dojo/Stateful",

    "dojo/store/Observable",

    "dojox/mvc/EditStoreRefController",

    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/_FocusMixin",
    "dijit/form/TextBox",

    "epi-saas-base/Sticker",
    "epi-saas-base/mvc/_CommitReturnMixin",
    "epi-saas-base/mvc/_EmptiableRefControllerMixin",
    "../ConfigModel",

    "./_SiblingFocusMixin",
    "./_ActionableMixin",
    "./_DisabledViewMixin",
    "./BestbetForm",
    "./BestbetGrid",


    "dojo/i18n!./nls/Bestbets",
    "dojo/text!./templates/Bestbets.html"
],
function(config,
         declare,
         lang,
         domClass,
         Stateful,

        Observable,

        EditStoreRefController,

        _WidgetBase,
        _TemplatedMixin,
        _WidgetsInTemplateMixin,
        _FocusMixin,
        TextBox,

        Sticker,
        _CommitReturnMixin,
        _EmptiableRefControllerMixin,
        ConfigModel,

        _SiblingFocusMixin,
        _ActionableMixin,
        _DisabledViewMixin,
        BestbetForm,
        BestbetGrid,
        i18n,
        template) {

    // module:
    //      epi-find/widget/Synonyms

    return declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, _FocusMixin, _SiblingFocusMixin, _ActionableMixin, _DisabledViewMixin], {
        // summary:
        //     Manage view.

        templateString: template,

        i18n: i18n,

        store: null,

        ctrl: null,

        _bestbetForm: null,
        _bestbetGrid: null,

        _searchText: null,

        _searchUpdateTimeout: 400,
        _searchUpdateTimer: null,

        constructor: function() {
            this.inherited(arguments);

            this.store = new Observable(config.dependencies["epi-find.BestBetStore"]);
        },

        takeAction: function(actions, params) {
            if (params && params.phrases) {
                this._bestbetForm.reset();
                this.ctrl.set("phrases", decodeURIComponent(params.phrases));
            }
            this._bestbetGrid.refresh();
        },

        postCreate: function() {
            this.inherited(arguments);
            var self = this;

            var EmptiableCommitController = declare([EditStoreRefController, _CommitReturnMixin, _EmptiableRefControllerMixin]);
            this.ctrl = new EmptiableCommitController({
                store: this.store,
                originalModel: new Stateful({ best_bet_has_own_style: true, target_type: "PageBestBetSelector" }),
                emptyModel: new Stateful({ best_bet_has_own_style: true, target_type: "PageBestBetSelector" })
            });

            this._bestbetForm = new BestbetForm({
                ctrl: this.ctrl
            }, this.bestbetForm);
            this.own(this._bestbetForm);

            this._bestbetGrid = new BestbetGrid({
                noDataMessage: i18n.noData,
                loadingMessage: i18n.dataLoading,
                showLoadingMessage: true,
                store: this.store,
                "class": "epi-plain-grid epi-plain-grid--no-border epi-plain-grid--no-header",
                onEdit: function(id) {
                    self._bestbetForm.edit(id);
					self._bestbetGrid.refresh();
                }
            }, this.bestbetGrid);

            this.own(
                this._bestbetGrid,
                this.searchNode.watch("value", function(name, oldValue, newValue) {
                    self._searchText = newValue;
                    self._delayedReload();
                })
            );

            domClass.toggle(this.searchNode.domNode, "epi-invisible", ConfigModel.getValue("api.best_bets.search") !== true);
        },

        refresh: function() {
            this._bestbetGrid.refresh();
            this._bestbetForm.reset();
            this.ctrl.empty();
        },

        startup: function(){
            this.inherited(arguments);
            var scrollNode = this.scrollNode || this.domNode.parentNode;
            this._bestbetForm.startup();
            this._bestbetGrid.set("scrollNode", scrollNode);
            this._bestbetGrid.startup();
            this.own(
                new Sticker({
                    domNode: this.bestBetsHeader,
                    offset: {top: 0},
                    scrollNode: scrollNode,
                    limitNode: this.bestbetGrid
                })
            );
        },

        _delayedReload: function() {
            //  summary:
            //      Updates grids query with text from the search textbox with delay.
            if (this._searchUpdateTimer) {
                clearTimeout(this._searchUpdateTimer);
            }
            this._searchUpdateTimer = setTimeout(lang.hitch(this, function () {
                this._searchUpdateTimer = null;
                this._bestbetGrid.set("query", this._searchText ? {q: this._searchText + "*"} : {});
            }), this._searchUpdateTimeout);
        }
    });
});

},
'epi-find/widget/BestbetForm':function(){
﻿define([
        "dojo/_base/array",
        "dojo/_base/declare",
        "dojo/_base/config",
        "dojo/_base/event",
        "dojo/_base/lang",
        "dojo/dom-construct",
        "dojo/topic",
        "dojo/when",

        "dojox/mvc/at",

        "dijit/form/Form",
        "dijit/_WidgetsInTemplateMixin",
        "dijit/registry",
        // widgets used in the template
        "dijit/form/Button",
        "dijit/form/ValidationTextBox",
        "dijit/form/RadioButton",
        "epi-saas-base/form/FormListItem",
        "./CmsContentSelector",
        "./ValidationTextAreaWithDefaultValue",
        "./TextBoxWithDefaultValue",
        "./TextAreaWithDefaultValue",
        "./PhraseValidationTextBox",

        "../ApplicationState",
        "epi-saas-base/widgets/_NotificationMixin",
        "epi-saas-base/UnicodeLetterRegexp",
        "epi-saas-base/UrlRegexp",
        "epi-saas-base/widgets/_ActionButtonsMixin",
        "../ConfigModel",

        "dojo/text!./templates/BestbetForm.html",
        "dojo/i18n!./nls/Bestbets"
    ],
    function(
        array,
        declare,
        config,
        event,
        lang,
        domConstruct,
        topic,
        when,

        at,

        Form,
        _WidgetsInTemplateMixin,
        registry,
        Button,
        ValidationTextBox,
        RadioButton,
        FormListItem,
        CmsContentSelector,
        Textarea,
        TextBoxWithDefaultValue,
        TextAreaWithDefaultValue,
        PhraseValidationTextBox,

        ApplicationState,
        _NotificationMixin,
        UnicodeLetterRegExp,
        UrlRegexp,
        _ActionButtonsMixin,
        ConfigModel,
        template,
        i18n
        ) {

        return declare([Form, _WidgetsInTemplateMixin, _NotificationMixin, _ActionButtonsMixin], {

            baseClass: "",
            ctrl: null,
            i18n: i18n,
            urlRegex: UrlRegexp,
            templateString: template,

            CMS_SELECTOR_TYPE: "PageBestBetSelector",
            COMMERCE_SELECTOR_TYPE: "CommerceBestBetSelector",
            EXTERNAL_SELECTOR_TYPE: "ExternalUrlBestBetSelector",

            postCreate: function() {
                this.inherited(arguments);
                this.bestBetPhraseListItem.set("label", i18n.fields_bestbetPhrase);
                this.bestBetPhraseListItem.set("description", i18n.separateMultiplePhrases);
                this.bestBetPhraseListItem.set("help", i18n.help_bestBetPhrase);
                this.targetContentListItem.set("help", i18n.help_targetField);
                this.targetContentListItem.set("label", i18n.fields_targetField);
                this.bestBetTitleListItem.set("label", i18n.fields_titleField);
                this.bestBetTitleListItem.set("help", i18n.help_titleField);
                this.inputIdListItem.set("label", i18n.fields_bestbetDescField);
                this.inputIdListItem.set("help", i18n.help_bestbetDescField);
                this.bestBetStyleListItem.set("help", i18n.help_bestbetStyleRadio);
                this.submitButton.set("label", i18n.addBestbet);
                this.resetButton.set("label", i18n.cancelBestbet);
                this.cmsSelector.set("title", i18n.contentType_select);
                this.commerceSelector.set("title", i18n.contentType_select);
            },

            startup:function() {
                this.inherited(arguments);

                var self = this, selectors = [{radio: this.targetContentRadio, selector: this.cmsSelector}];
                this.actionButtons = [this.submitButton, this.resetButton];
                this.enabledWhenNoChangesButtons = [this.resetButton];

                this.phraseField.set("value", at(this.ctrl, "phrases"));

                // Note: use double backslash \\ for one backslash \ in this regex (normal regex for a backslash is \\, but in this will need 4 times backslash \\\\)
                this.phraseField.set("pattern", "[^~`!@%^*()[\\]{}:;'|/<>?_-](?:"+UnicodeLetterRegExp+"|[0-9'&+#$_.,-\\s])*");
                this.phraseField.set("intermediateChanges", "true");

                this.cmsSelector.set("allowedTypes",  ["episerver.core.pagedata", "episerver.core.icontentmedia"]);
                this.cmsSelector.set("targetContent", at(this.ctrl, "target_key").transform({
                    format: function(value) {
                        var type = self.ctrl.get("target_type");
                        return self.isCMS(type) ? value : null;
                    },
                    parse: function(value) {
                        var type = self.ctrl.get("target_type");
                        return self.isCMS(type) ? value : self.ctrl.get("target_key"); // leave unchanged if external
                    }
                }));
                this.cmsSelector.set("rootPage", at(ApplicationState, "site")
                    .direction(at.from)
                    .transform({ format: function(site){
                        if (site) {
                            return site.rootPage;
                        }
                        return null;
                    }}));

                this.cmsSelector.set("selectedLang", at(ApplicationState, "language")
                    .direction(at.from)
                    .transform({ format: function(language){
                        if (language) {
                            return language.id;
                        }
                        return null;
                    }}));

                /*ApplicationState.set("language", language);*/
                if (ConfigModel.getValue("api.best_bets.external_content") === true) {
                    selectors.push({radio:this.externalLinkRadio, selector: this.externalUrl});
                    this.externalUrl.set("value",  at(this.ctrl, "target_key").transform({
                        format: function(value) {
                            var type = self.ctrl.get("target_type");
                            return self.isExternal(type) ? value : null;
                        },
                        parse: function(value) {
                            var type = self.ctrl.get("target_type");
                            return self.isExternal(type) ? value : self.ctrl.get("target_key"); // leave unchanged if not external
                        }
                    }));

                    if (ConfigModel.getValue("api.best_bets.commerce_content") === true &&
                        array.some(ConfigModel.getValue("api.best_bets.selector_types"), function (type) {
                            return self.COMMERCE_SELECTOR_TYPE === type;
                        })) {
                        selectors.push({ radio: this.commerceLinkRadio, selector: this.commerceSelector});

                        this.commerceSelector.set("allowedTypes",  ["episerver.commerce.catalog.contenttypes.entrycontentbase", "episerver.commerce.catalog.contenttypes.nodecontent"]);
                        this.commerceSelector.set("targetContent",  at(this.ctrl, "target_key").transform({
                            format: function(value) {
                                    var type = self.ctrl.get("target_type");
                                    return self.isCommerce(type) ? value : null;
                            },
                            parse: function(value) {
                                    var type = self.ctrl.get("target_type");
                                    return self.isCommerce(type) ? value : self.ctrl.get("target_key"); // leave unchanged if not external
                            }
                        }));


                        this.commerceSelector.set("selectedLang", at(ApplicationState, "language")
                            .direction(at.from)
                            .transform({ format: function(language){
                                if (language) {
                                    return language.id;
                                }
                                return null;
                            }}));
                    }
                    else {
                        this.commerceSelectorListItem.destroy();
                    }

                    this.own(
                        this.watch("value", function(name, oldValue, newValue) {
                            if (oldValue.contentSelection === newValue.contentSelection) {
                                return;
                            }
                            self.ctrl.set("target_type", newValue.contentSelection);
                        }),
                        this.ctrl.watch("target_type", function(name, oldValue, newValue) {
                            array.forEach(selectors, function(selector) {
                                if (selector.radio.value ==  newValue) {
                                    selector.radio.set("checked", true);
                                    selector.selector.set("disabled", false);
                                } else {
                                    selector.selector.reset();
                                    selector.selector.set("disabled", true);
                                }
                            });
                            self.titleField.set("required", self.isExternal( newValue));
                            self.bestbetDescField.set("required", self.isExternal( newValue));
                        })
                    );
                }
                else {
                    domConstruct.place(this.cmsSelector.domNode, this.targetContentListItem.wrapperNode, "first");
                    array.forEach(registry.findWidgets(this.targetContentInnerList), function(w) {
                        w.destroyRecursive();
                    });
                    domConstruct.destroy(this.targetContentInnerList);
                }

                if (ConfigModel.getValue("api.best_bets.get_indexed_data") === true) {
                    array.forEach(selectors, function(selector) {
                        if (!self.isExternal(selector.radio.value)) {
                            self.own(selector.selector.watch("targetContent", function(name, oldValue, newValue) {
                                if (newValue) {
                                    self.ctrl.store.query({targetKey:newValue}, {apiName:'GetIndexedData'}).forEach(lang.hitch(this, function(item) {
                                        if (newValue == selector.selector.get("targetContent")) { // to avoid slow updates from overwriting empty() during onReset()
                                            self.titleField.set("defaultValue", item.title || "");
                                            self.bestbetDescField.set("defaultValue", item.description || "");
                                        }
                                    }));
                                }
                                else {
                                    self.titleField.set("defaultValue", "");
                                    self.bestbetDescField.set("defaultValue", "");
                                }
                            }));
                        }
                    });
                }

                this.titleField.set("value", at(this.ctrl, "best_bet_target_title"));
                this.bestbetDescField.set("value", at(this.ctrl, "best_bet_target_description"));

                this.bestbetStyleRadio.set("name", "bestbetStyle");
                this.bestbetStyleRadio.set("checked", at(this.ctrl, "best_bet_has_own_style"));

                this.resultStyleRadio.set("name", "bestbetStyle");
                this.resultStyleRadio.set("checked", at(this.ctrl, "best_bet_has_own_style").transform({
                    format: function(value) {
                        return !value;
                    },
                    parse: function(value) {
                        return !value;
                    }
                }));

                this.reset();

                this.own(this.watchDataChanges(this.ctrl));
            },

            isExternal: function(targetType) {
                return targetType === this.EXTERNAL_SELECTOR_TYPE;
            },
            isCMS: function(targetType) {
                return targetType === this.CMS_SELECTOR_TYPE;
            },
            isCommerce: function(targetType) {
                return targetType === this.COMMERCE_SELECTOR_TYPE;
            },

            onSubmit: function (e) {
                var self = this;
                event.stop(e);
                if (!this.validate()) {
                    return ;
                }
                this.submitButton.focus();
                this.indicateInProgress(this.submitButton);
                when(this.ctrl.commit(),
                    function() {
                        self.reset();
                        self.showSuccess(i18n.savedMessage);
                        self.indicateSuccess();
                        topic.publish(config.intents.view.bestBets);
                        self.phraseField.focus();
                    },
                    function (error) {
                        self.showError(error, null, i18n.saveErrorTitle);
                        self.indicateError();
                    }
                );
            },

            onReset: function() {
                this.ctrl.reset();
                this.ctrl.empty();
                this.submitButton.set("label", i18n.addBestbet);
                return this.inherited(arguments);
            },

            edit: function(bestbetsID) {
                var self = this;
                if (this.editMode) {
                    this.reset();
                }
                when(this.ctrl.getStore(bestbetsID), function() {
                        self.phraseField.focus();
                        self.submitButton.set("label", i18n.saveBestbet);
                        self.set("hasChanges", false);
                    },
                    function (error) {
                        self.showError(error);
                    }
                );
            },

            submit: function() {
                // summary:
                //      pragmatically submit form if and only if the `onSubmit` returns true
                if (this.onSubmit() === false) {
                    return;
                }
                this.originalContainerNode.submit();
            }
        });

    });

},
'epi-find/widget/CmsContentSelector':function(){
﻿define([
    "dojo/_base/lang",
    "dojo/_base/declare",
    "dojo/when",
    "dojo/dom-class",

    "dijit/layout/ContentPane",

    "epi/shell/widget/dialog/Dialog",

    "epi-cms/widget/SearchBox",
    "epi-cms/widget/ContentSelector",
    "epi-cms/widget/ContentSelectorDialog",
    "./ContentStoreQueryLanguageHandler",

    "./_ContentStatusIndicatorMixin",

    "dojo/NodeList-manipulate" // workaround for the ContentSelectors query usage, bug #111507

],
function(
         lang,
         declare,
         when,
         domClass,
         ContentPane,
         Dialog,
         SearchBox,
         ContentSelector,
         ContentSelectorDialog,
         ContentStoreQueryLanguageHandler,
         _ContentStatusIndicatorMixin
    ) {


    return declare([ContentSelector, _ContentStatusIndicatorMixin], {
        model: null,
        title: "",
        targetContent: null,
        contentStatus: null,
        searchArea: null,
        selectButtonText: null,
        selectedLang: null,

        _setTargetContentAttr: function (value) {
            this.set("value", value);
            this._set("targetContent", value);

            if (!value && this.disabled) {
                this.set("state", ""); // reset invalid state
            } else {
                var content = this._getContentData(value);
                if (content) {
                    this._set("contentStatus", content.status);
                } else {
                    this._set("contentStatus", "");
                }
            }
        },

        rootPage: null,
        _setRootPageAttr: function (value) {
            this.rootPage = value;
            this.roots = [this.rootPage];
        },

        _setDisabledAttr: function (value) {
            this.inherited(arguments);
            domClass.toggle(this.displayNode, "dijitTextBoxDisabled", value); // grayout text node
            domClass.toggle(this.clearButton, "epi-hidden", value); // hide clear button
            this.set("state", ""); // reset invalid state

        },

        constructor: function() {
            this.showButtons = true;
            this.allowedTypes = [];
        },

        postMixInProperties: function() {
            try {
                this.inherited(arguments);
            }
            catch(e) { // if the base widget attempt to resolve roots results in exception
                this.roots = this.roots || [];
            }
        },

        startup: function() {
            this.inherited(arguments);
            this._started = true; // workaround for the bug #111543
        },

        onChange: function(value) {
            this.set("targetContent", value);
        },

        _getDialog: function () { // have to override this method in order to provide to the contentSelectorDialog
            // summary:
            //		Create page tree dialog
            // tags:
            //    protected

            // Verifies that the dialog instance and its dom node exists or not
            if (this.dialog && this.dialog.domNode) {
                return this.dialog;
            }
            var self = this;

            var model = new ContentStoreQueryLanguageHandler({roots: this.roots, lang: this.selectedLang});

            this.contentContainer = new ContentPane({
                "class": "epi-no-overflow-x"
            });

            this.searchBox = new SearchBox({
                baseClass: "epi-search-box",
                onItemAction: function(item) {
                    if (item && item.metadata) {
                        self.contentSelectorDialog.set("value", item.metadata.id);
                    }
                }
            });
            this.searchBox.set("area", this.searchArea);

            this.contentSelectorDialog = new ContentSelectorDialog({
                model: model,
                canSelectOwnerContent: this.canSelectOwnerContent,
                showButtons: false,
                roots: this.roots,
                allowedTypes: this.allowedTypes,
                showAllLanguages: this.showAllLanguages,
                layout: function() {} // prevent layoting
            });

            this.contentContainer.addChild(this.searchBox);
            this.contentContainer.addChild(this.contentSelectorDialog);

            this.dialog = new Dialog({
                title: this.title,
                dialogClass: "epi-dialog-portrait",
                content: this.contentContainer,
                destroyOnHide: true
            });
            this.dialog.confirmActionText = this.selectButtonText || this.dialog.confirmActionText;

            this.dialog.own(this.contentContainer);
            this.dialog.own(this.searchBox);
            this.dialog.own(this.contentSelectorDialog);
            this.dialog.own(model);

            this.connect(this.contentSelectorDialog, "onChange", "_setDialogButtonState");
            this.connect(this.dialog, "onExecute", "_onDialogExecute");
            this.connect(this.dialog, "onHide", "_onDialogHide");
            return this.dialog;
        },

        _setDialogButtonState: function(contentLink) {
            this.inherited(arguments);

            var self = this;
            when(this._store.get(contentLink), function (content) {
                self.dialog.definitionConsumer.setItemProperty(self.dialog._okButtonName, "disabled", !content.isVisibleOnSite);
            });
        },

      _getContentData: function (contentLink) {
            //summary:
            //    Loads the content data from the store
            // tags:
            //    private

            if (!contentLink) {
                return null;
            }
            var queryString = "?showAllLanguages=" + this.showAllLanguages;

            if(this.selectedLang)
            {
              queryString += "&lang=" + this.selectedLang;
            }
            return this._store.get(contentLink + queryString);
        },


        reset: function() {
            this.inherited(arguments);
            this.set("state",""); // reset validation state
        }
    });
});

},
'epi-cms/widget/SearchBox':function(){
﻿define("epi-cms/widget/SearchBox", [
// Dojo
    "dojo/_base/declare",
    "dojo/_base/connect",
    "dojo/_base/json",
    "dojo/_base/lang",
    "dojo/when",

    "dojo/dom-construct",

    // Dijit
    "dijit/_Widget",

    // EPi Framework
    "epi/shell/SearchManager",

    // EPi CMS
    "epi-cms/widget/ContentSearchBox"
],

function (
// Dojo
    declare,
    connect,
    json,
    lang,
    when,

    domConstruct,

    // Dijit
    _Widget,

    // EPi Framework
    SearchManager,

    // EPi CMS
    ContentSearchBox
) {

    return declare([_Widget], {
        // summary:
        //    A composite widget, which wrap the ContentSearchBox, for setting up the search box.
        //
        // tags:
        //    internal

        // area: [protected] String
        //      Search area to search.
        area: null,

        // searchRoots: [protected] String
        //      Optional comma separated list of roots to search.
        searchRoots: null,

        // filterOnCulture: [protected] Boolean
        //      Flag to set if search results should be filtered on culture. Default value is true.
        filterOnCulture: null,

        // triggerContextChange: [public] Boolean
        //      If [true] will cause a contextChange topic to be published.
        triggerContextChange: true,

        // _searchBox: [private] ContentSearchBox
        //      Content search box widget.
        _searchBox: null,

        // _searchManager: [private] SearchManager
        //      Content search manager.
        _searchManager: null,

        // innerSearchBoxClass: [public] String
        //      CSS class for the inner ContentSearchBox
        innerSearchBoxClass: "",

        parameters: null,

        postCreate: function () {
            // summary:
            //      Add and setup search box
            // tags:
            //      protected
            this.inherited(arguments);

            this._searchManager = new SearchManager();
            when(this._searchManager.getStore(), lang.hitch(this, function (store) {
                this._searchBox = new ContentSearchBox({
                    baseClass: this.innerSearchBoxClass,
                    searchAttr: "searchQuery",
                    store: store
                });

                var parameters = lang.mixin({ filterOnDeleted: true }, this.parameters);
                lang.mixin(this._searchBox.query, { parameters: json.toJson(parameters) });

                // Event onSelect of search box
                this._searchBox.on("select", lang.hitch(this, function (value) {
                    // Set blank value for textbox
                    this._searchBox.set("value", "");
                    this._searchBox.focus();
                    this._searchBox.toggleClearButton();
                    this.onItemAction(value);
                }));
                //search results show contents in all languages
                this.set("filterOnCulture", false);
                // add search box to DOM
                domConstruct.place(this._searchBox.domNode, this.domNode);
            }));
        },

        _setAreaAttr: function (value) {
            this._set("area", value);
            if (this._searchBox) {
                // get query for current search area
                when(this._searchManager.getProviderQuery(value), lang.hitch(this, function (query) {
                    lang.mixin(this._searchBox.query, query);
                }));
            }
        },

        _setSearchRootsAttr: function (value) {
            this._set("searchRoots", value);
            if (this._searchBox) {
                lang.mixin(this._searchBox.query, { searchRoots: value });
            }
        },

        _setFilterOnCultureAttr: function (value) {
            this._set("filterOnCulture", value);
            if (this._searchBox) {
                lang.mixin(this._searchBox.query, { filterOnCulture: value });
            }
        },

        onItemAction: function (item) {
            // summary:
            //      Trigger the context change event when an item has been selected in search box.
            // tags:
            //      callback

            if (this.triggerContextChange) {
                var contextParameters = { uri: "epi.cms.contentdata:///" + item.metadata.id };
                if (item.metadata.isUntranslatedContent === "false") {
                    contextParameters.epslanguage = item.metadata.languageBranch;
                }
                connect.publish("/epi/shell/context/request", [contextParameters, { sender: null }]);
            }
        }
    });
});

},
'epi/shell/SearchManager':function(){
﻿define("epi/shell/SearchManager", [
// dojo
    "dojo/_base/declare",
    "dojo/_base/Deferred",
    "dojo/_base/array",
    "dojo/_base/lang",
    "dojo/DeferredList",
    // epi
    "epi/dependency"
],
function (
// dojo
    declare,
    Deferred,
    array,
    lang,
    DeferredList,
    // epi
    dependency
) {
    return declare(null, {
        // summary:
        //      Search component that provides search provider stores and search results store
        //      to make searching by conditions and take searching results.
        // tags:
        //      public

        // _providersStore: [epi.shell.searchproviders] object
        _providersStore: null,
        // _resultsStore: [epi.shell.searchresults] object
        _resultsStore: null,

        constructor: function (params) {
            // summary:
            //      Setup search provider stores and search results store.
            // tags:
            //      public

            this._setupStores(params || {});
        },

        getProviderQuery: function (/*string*/area) {
            // summary:
            //      Return provider query parameter(s) that used as input parameter(s) for search results store.
            // area: string
            //      The area of the search provider(s).
            //      E.g: "cms/blocks" area, will return all search provider(s) that have area is "cms/blocks".
            //          Will return all search providers if passed blank area.
            // return:
            //      Deferred object that holds provider query parameters JSON object.
            // tags:
            //      public

            var deferred = new Deferred();

            Deferred.when(this.getProviders(area), lang.hitch(this, function (providers) {
                if (providers && providers.length > 0) {
                    deferred.resolve(this._getQuery(providers[0], {}));
                } else {
                    deferred.resolve();
                }
            }));

            return deferred;
        },

        getProviders: function (/*string*/area) {
            // summary:
            //      Return all search provider in the given area.
            //      If passed blank value will return all providers.
            // area: string
            //      The area that search provider(s) belong to.
            // return:
            //      Deferred object that holds an array of search provider objects.
            // tags:
            //      public

            var deferred = new Deferred();

            Deferred.when(this._providersStore.query({ searchArea: area }), function (providers) {
                if (providers && providers.length > 0) {
                    deferred.resolve(providers);
                } else {
                    deferred.resolve();
                }
            });

            return deferred;
        },

        getStore: function () {
            // summary:
            //      Return search results store object.
            // tags:
            //      public

            return this._resultsStore;
        },

        search: function (/*string*/area, /*object*/queryParams) {
            // summary:
            //      Get all results from search results by passing search provider area and query parameter(s).
            // area: string
            //      The area that search provider(s) belong to.
            // queryParams: JSON object
            //      Query that used to search.
            // return:
            //      Deferred object that contains an array of search result data items.
            // tags:
            //      public

            var deferred = new Deferred();

            // Get all search provider stores from search providers store
            Deferred.when(this.getProviders(area), lang.hitch(this, function (providers) {
                if (providers && providers.length > 0) {
                    var deferredArray = [];

                    // Query each search provider store to get its results
                    array.forEach(providers, function (provider, index) {
                        deferredArray.push(this._getData(provider, queryParams));
                    }, this);

                    var deferredList = new DeferredList(deferredArray),
                        data = [];

                    // Put all data together
                    deferredList.then(function (items) {
                        array.forEach(items, function (item) {
                            if (lang.isArray(item[1])) {
                                data = data.concat(item[1]);
                            }
                        });
                        deferred.resolve(data);
                    });
                }
            }));

            return deferred;
        },

        _setupStores: function (params) {
            // summary:
            //      Get search providers store and search results store from registry.
            // tags:
            //      private

            this._providersStore = params.providersStore || dependency.resolve("epi.storeregistry").get("epi.shell.searchproviders");
            this._resultsStore = params.resultsStore || dependency.resolve("epi.storeregistry").get("epi.shell.searchresults");
        },

        _getQuery: function (/*object*/provider, /*object*/queryParams) {
            // summary:
            //      Build search query.
            // provider: object
            //      Search provider store object.
            // queryParams: JSON object
            //      Query that used to search.
            // return:
            //      Query JSON object.
            // tags:
            //      private

            var query = { providerId: provider.id };

            lang.mixin(query, queryParams || {});

            return query;
        },

        _getData: function (/*object*/provider, /*object*/queryParams) {
            // summary:
            //      Get search results from search results store by the given search provider and query.
            // provider: object
            //      Search provider store object.
            // queryParams: JSON object
            //      Query that used to search.
            // return:
            //      Deferred object that contains an array of search result items.
            // tags:
            //      private

            var deferred = new Deferred();

            // Query by search results store
            Deferred.when(this._resultsStore.query(this._getQuery(provider, queryParams)), function (items) {
                if (items && items.length > 0) {
                    deferred.resolve(items);
                } else {
                    deferred.resolve();
                }
            });

            return deferred;
        }
    });
});

},
'dojo/DeferredList':function(){
define("dojo/DeferredList", ["./_base/kernel", "./_base/Deferred", "./_base/array"], function(dojo, Deferred, darray){
	// module:
	//		dojo/DeferredList


dojo.DeferredList = function(/*Array*/ list, /*Boolean?*/ fireOnOneCallback, /*Boolean?*/ fireOnOneErrback, /*Boolean?*/ consumeErrors, /*Function?*/ canceller){
	// summary:
	//		Deprecated, use dojo/promise/all instead.
	//		Provides event handling for a group of Deferred objects.
	// description:
	//		DeferredList takes an array of existing deferreds and returns a new deferred of its own
	//		this new deferred will typically have its callback fired when all of the deferreds in
	//		the given list have fired their own deferreds.  The parameters `fireOnOneCallback` and
	//		fireOnOneErrback, will fire before all the deferreds as appropriate
	// list:
	//		The list of deferreds to be synchronizied with this DeferredList
	// fireOnOneCallback:
	//		Will cause the DeferredLists callback to be fired as soon as any
	//		of the deferreds in its list have been fired instead of waiting until
	//		the entire list has finished
	// fireonOneErrback:
	//		Will cause the errback to fire upon any of the deferreds errback
	// canceller:
	//		A deferred canceller function, see dojo.Deferred
	var resultList = [];
	Deferred.call(this);
	var self = this;
	if(list.length === 0 && !fireOnOneCallback){
		this.resolve([0, []]);
	}
	var finished = 0;
	darray.forEach(list, function(item, i){
		item.then(function(result){
			if(fireOnOneCallback){
				self.resolve([i, result]);
			}else{
				addResult(true, result);
			}
		},function(error){
			if(fireOnOneErrback){
				self.reject(error);
			}else{
				addResult(false, error);
			}
			if(consumeErrors){
				return null;
			}
			throw error;
		});
		function addResult(succeeded, result){
			resultList[i] = [succeeded, result];
			finished++;
			if(finished === list.length){
				self.resolve(resultList);
			}

		}
	});
};
dojo.DeferredList.prototype = new Deferred();

dojo.DeferredList.prototype.gatherResults = function(deferredList){
	// summary:
	//		Gathers the results of the deferreds for packaging
	//		as the parameters to the Deferred Lists' callback
	// deferredList: dojo/DeferredList
	//		The deferred list from which this function gathers results.
	// returns: dojo/DeferredList
	//		The newly created deferred list which packs results as
	//		parameters to its callback.

	var d = new dojo.DeferredList(deferredList, false, true, false);
	d.addCallback(function(results){
		var ret = [];
		darray.forEach(results, function(result){
			ret.push(result[1]);
		});
		return ret;
	});
	return d;
};

return dojo.DeferredList;
});

},
'epi-cms/widget/ContentSearchBox':function(){
define("epi-cms/widget/ContentSearchBox", [
// Dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/event",

    "dojo/aspect",
    "dojo/Evented",
    "dojo/keys",

    "dojo/dom-attr",
    "dojo/dom-style",
    // EPi Framework
    "epi/shell/widget/SuggestionBox",
    "epi-cms/widget/SearchResultList",

    // Resources
    "dojo/i18n!epi/nls/episerver.cms.widget.contentlist"
],

function (
// Dojo
    declare,
    lang,
    event,

    aspect,
    Evented,
    keys,

    domAttr,
    domStyle,
    // EPi Framework
    SuggestionBox,
    SearchResultList,

    // Resources
    resources
) {

    return declare([SuggestionBox, Evented], {
        // summary:
        //    A Search box to search for content.
        //
        // tags:
        //    public

        // res: Object
        //      Resource object, for getting text.
        // tags:
        //      Protected
        res: resources,

        // autoComplete: Boolean
        //      If user types in a partial string, and then tab out of the `<input>` box,
        //      automatically copy the first entry displayed in the drop down list to
        //      the `<input>` field
        autoComplete: false,

        // dropDownClass: [protected extension] Function String
        //      Dropdown widget class, which would be used to create a widget, to show the search result.
        //      Subclasses should specify this.
        dropDownClass: SearchResultList,

        // queryExpr: String
        //      This specifies what query ComboBox/FilteringSelect sends to the data store,
        //      based on what the user has typed.  Changing this expression will modify
        //      whether the drop down shows only exact matches, a "starting with" match,
        //      etc.  Use it in conjunction with highlightMatch.
        //      dojo/data query expression pattern.
        //      `${0}` will be substituted for the user text.
        //      `*` is used for wildcards.
        //      `${0}*` means "starts with", `*${0}*` means "contains", `${0}` means "is"
        queryExpr: "${0}",

        // minimumNumberOfChar: Integer
        //      The minimum number of character that is valid for search.
        // tags:
        //      Protected
        minimumNumberOfChar: 1,

        postCreate: function () {
            // summary: Override to:
            //      - mixin additional query params into query before _startSearch
            //      - connect to onSelect event of dropDown, to forward it to ContentSearchBox after _startSearch
            // tags:
            //      protected
            this.inherited(arguments);

            // Delay in milliseconds between when user types something and we start
            // searching based on that value.
            this.searchDelay = this.defaultDelay;
            this.on("beforeSearch", lang.hitch(this, function () {
                // display dropdown with standby
                if (this.dropDown) {
                    this.dropDown.showStandby(true);
                }
            }));

            this.on("searchComplete", lang.hitch(this, function (results) {
                // hide dropdown's standby
                if (this.dropDown) {
                    this.dropDown.showStandby(false);
                }
            }));

            aspect.before(this, "_startSearch", function (method, args) {
                // doing something before the original call
                this.emit("beforeSearch", {});

                return [method, args];
            });
        },

        _openResultList: function (/*Object*/results, /*Object*/query, /*Object*/options) {
            // summary:
            //		Callback when a search completes.
            // description:
            //		Generates drop-down list and calls _showResultList() to display it

            this.emit("searchComplete", results);

            this._fetchHandle = null;

            this.dropDown.clearResultList();

            var dataAvailable = results.length > 0 || options.start !== 0;
            // display the empty text if data is not available
            this.dropDown.showErrorMessage(!dataAvailable, this.res.contentnotfound);
            this.dropDown.showGrid(dataAvailable);

            if (dataAvailable) {
                this.dropDown.createOptions(results, options, lang.hitch(this, "_getMenuLabelFromItem"));
            }

            // show our list
            this._showResultList();
        },

        _onDropDownMouseDown: function (/*Event*/e) {
            // summary:
            //		Callback when the user mousedown's on the arrow icon.
            // tags:
            //      Private override

            // Do nothing, to prevent start search on click to textbox

            // Enable mouse clickable when pinnable pane in un-pinned state.
            e.stopPropagation();
        },

        _onKey: function (/*Event*/evt) {
            // summary:
            //		Listen for Enter key, to start search request.
            // tags:
            //      private override

            switch (evt.keyCode) {
                case keys.ENTER:
                    if (this.dropDown && this.dropDown.selected) {
                        this.dropDown.handleKey(evt);
                        this.closeDropDown();
                    } else if (!this._fetchHandle) {
                        this._startSearchFromInput();
                        event.stop(evt);
                    }
                    break;
                case keys.PAGE_DOWN:
                case keys.DOWN_ARROW:
                case keys.PAGE_UP:
                case keys.UP_ARROW:
                    // let the dropdown handle these keystrokes for navigating between item
                    if (this.dropDown) {
                        this.dropDown.handleKey(evt);
                    }
                    event.stop(evt);
                    break;
                case keys.LEFT_ARROW:
                case keys.RIGHT_ARROW:
                    // do nothing when navigating, selecting text
                    break;
                default:
                    this.inherited(arguments);
            }
        },

        _startSearchFromInput: function () {
            // summary:
            //      Start the search action
            // tags:
            //      private, override

            var key = this.focusNode.value;

            // start the search action only if input text length is more than minimum number
            if (!key || key.length < this.minimumNumberOfChar) {
                this.closeDropDown();
                return;
            }

            if (!this.dropDown) {
                var popupId = this.id + "_popup",
                    dropDownConstructor = lang.isString(this.dropDownClass) ? lang.getObject(this.dropDownClass, false) : this.dropDownClass;

                this.dropDown = new dropDownConstructor({
                    id: popupId,
                    dir: this.dir,
                    textDir: this.textDir
                });

                domAttr.remove(this.textbox, "aria-activedescendant");
                domAttr.set(this.textbox, "area-owns", popupId); // associate popup with textbox

                this.dropDown.on("select", lang.hitch(this, function (value) {
                    this.closeDropDown();
                    this.emit("select", value);
                }));

                // Set dropdown width belong to input text width
                domStyle.set(this.dropDown.domNode, "width", this.textbox.clientWidth + "px");
            }
            this._showResultList();

            // We do not want to call the inherited method here since it will escape wild card characters (* and ?),
            // which will break any searches with wild card.
            this._startSearch(key);
        },

        _startSearchAll: function () {
            // disable search with empty string
        }
    });
});

},
'epi/shell/widget/SuggestionBox':function(){
﻿define("epi/shell/widget/SuggestionBox", [
// Dojo
    "dojo/_base/declare",
    "dojo/_base/lang",

    "dojo/dom-attr",

    // Dijit
    "dijit/_HasDropDown",
    "dijit/form/_ComboBoxMenu",
    "dijit/form/_AutoCompleterMixin",

    // EPi Framework
    "epi/shell/widget/SearchBox"
],

function (
// Dojo
    declare,
    lang,

    domAttr,

    // Dijit
    _HasDropDown,
    _ComboBoxMenu,
    _AutoCompleterMixin,

    // EPi Framework
    SearchBox
) {

    return declare([SearchBox, _HasDropDown, _AutoCompleterMixin], {
        // summary:
        //    A suggestion box, which will drop a list down when user input a string to a textbox to filter item in a pre-defined list.
        //
        // tags:
        //    public

        // dropDownClass: [protected extension] Function String
        //      Dropdown widget class, which would be used to create a widget, to show the search result.
        //      Subclasses should specify this.
        dropDownClass: _ComboBoxMenu,

        buildRendering: function () {
            this.inherited(arguments);

            domAttr.set(this.textbox, "autocomplete", "off");
            domAttr.set(this.textbox, "aria-haspopup", "true");
        },

        postCreate: function () {
            // summary:
            //      Post widget creation. Initialize event for buttons
            // tags:
            //      Protected

            this.inherited(arguments);

            // clear textbox value when clicking on clear button
            this.connect(this.clearButton, "onclick", lang.hitch(this, function () {
                this.closeDropDown();
            }));
        }
    });
});

},
'dijit/form/_ComboBoxMenu':function(){
define("dijit/form/_ComboBoxMenu", [
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add domClass.remove
	"dojo/dom-style", // domStyle.get
	"dojo/keys", // keys.DOWN_ARROW keys.PAGE_DOWN keys.PAGE_UP keys.UP_ARROW
	"../_WidgetBase",
	"../_TemplatedMixin",
	"./_ComboBoxMenuMixin",
	"./_ListMouseMixin"
], function(declare, domClass, domStyle, keys,
			_WidgetBase, _TemplatedMixin, _ComboBoxMenuMixin, _ListMouseMixin){


	// module:
	//		dijit/form/_ComboBoxMenu

	return declare("dijit.form._ComboBoxMenu",[_WidgetBase, _TemplatedMixin, _ListMouseMixin, _ComboBoxMenuMixin], {
		// summary:
		//		Focus-less menu for internal use in `dijit/form/ComboBox`
		//		Abstract methods that must be defined externally:
		//
		//		- onChange: item was explicitly chosen (mousedown somewhere on the menu and mouseup somewhere on the menu)
		//		- onPage: next(1) or previous(-1) button pressed
		// tags:
		//		private

		templateString: "<div class='dijitReset dijitMenu' data-dojo-attach-point='containerNode' style='overflow: auto; overflow-x: hidden;' role='listbox'>"
				+"<div class='dijitMenuItem dijitMenuPreviousButton' data-dojo-attach-point='previousButton' role='option'></div>"
				+"<div class='dijitMenuItem dijitMenuNextButton' data-dojo-attach-point='nextButton' role='option'></div>"
				+"</div>",

		baseClass: "dijitComboBoxMenu",

		postCreate: function(){
			this.inherited(arguments);
			if(!this.isLeftToRight()){
				domClass.add(this.previousButton, "dijitMenuItemRtl");
				domClass.add(this.nextButton, "dijitMenuItemRtl");
			}
			this.containerNode.setAttribute("role","listbox");
		},

		_createMenuItem: function(){
			// note: not using domConstruct.create() because need to specify document
			var item = this.ownerDocument.createElement("div");
			item.className = "dijitReset dijitMenuItem" +(this.isLeftToRight() ? "" : " dijitMenuItemRtl");
			item.setAttribute("role", "option");
			return item;
		},

		onHover: function(/*DomNode*/ node){
			// summary:
			//		Add hover CSS
			domClass.add(node, "dijitMenuItemHover");
		},

		onUnhover: function(/*DomNode*/ node){
			// summary:
			//		Remove hover CSS
			domClass.remove(node, "dijitMenuItemHover");
		},

		onSelect: function(/*DomNode*/ node){
			// summary:
			//		Add selected CSS
			domClass.add(node, "dijitMenuItemSelected");
		},

		onDeselect: function(/*DomNode*/ node){
			// summary:
			//		Remove selected CSS
			domClass.remove(node, "dijitMenuItemSelected");
		},

		_page: function(/*Boolean*/ up){
			// summary:
			//		Handles page-up and page-down keypresses

			var scrollamount = 0;
			var oldscroll = this.domNode.scrollTop;
			var height = domStyle.get(this.domNode, "height");
			// if no item is highlighted, highlight the first option
			if(!this.getHighlightedOption()){
				this.selectNextNode();
			}
			while(scrollamount<height){
				var highlighted_option = this.getHighlightedOption();
				if(up){
					// stop at option 1
					if(!highlighted_option.previousSibling ||
						highlighted_option.previousSibling.style.display == "none"){
						break;
					}
					this.selectPreviousNode();
				}else{
					// stop at last option
					if(!highlighted_option.nextSibling ||
						highlighted_option.nextSibling.style.display == "none"){
						break;
					}
					this.selectNextNode();
				}
				// going backwards
				var newscroll = this.domNode.scrollTop;
				scrollamount += (newscroll-oldscroll)*(up ? -1:1);
				oldscroll = newscroll;
			}
		},

		handleKey: function(evt){
			// summary:
			//		Handle keystroke event forwarded from ComboBox, returning false if it's
			//		a keystroke I recognize and process, true otherwise.
			switch(evt.keyCode){
				case keys.DOWN_ARROW:
					this.selectNextNode();
					return false;
				case keys.PAGE_DOWN:
					this._page(false);
					return false;
				case keys.UP_ARROW:
					this.selectPreviousNode();
					return false;
				case keys.PAGE_UP:
					this._page(true);
					return false;
				default:
					return true;
			}
		}
	});
});

},
'dijit/form/_ComboBoxMenuMixin':function(){
define("dijit/form/_ComboBoxMenuMixin", [
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/i18n", // i18n.getLocalization
	"dojo/i18n!./nls/ComboBox"
], function(array, declare, domAttr, i18n){

// module:
//		dijit/form/_ComboBoxMenuMixin

return declare( "dijit.form._ComboBoxMenuMixin", null, {
	// summary:
	//		Focus-less menu for internal use in `dijit/form/ComboBox`
	// tags:
	//		private

	// _messages: Object
	//		Holds "next" and "previous" text for paging buttons on drop down
	_messages: null,

	postMixInProperties: function(){
		this.inherited(arguments);
		this._messages = i18n.getLocalization("dijit.form", "ComboBox", this.lang);
	},

	buildRendering: function(){
		this.inherited(arguments);

		// fill in template with i18n messages
		this.previousButton.innerHTML = this._messages["previousMessage"];
		this.nextButton.innerHTML = this._messages["nextMessage"];
	},

	_setValueAttr: function(/*Object*/ value){
		this.value = value;
		this.onChange(value);
	},

	onClick: function(/*DomNode*/ node){
		if(node == this.previousButton){
			this._setSelectedAttr(null);
			this.onPage(-1);
		}else if(node == this.nextButton){
			this._setSelectedAttr(null);
			this.onPage(1);
		}else{
			this.onChange(node);
		}
	},

	// stubs
	onChange: function(/*Number*/ /*===== direction =====*/){
		// summary:
		//		Notifies ComboBox/FilteringSelect that user selected an option.
		// tags:
		//		callback
	},

	onPage: function(/*Number*/ /*===== direction =====*/){
		// summary:
		//		Notifies ComboBox/FilteringSelect that user clicked to advance to next/previous page.
		// tags:
		//		callback
	},

	onClose: function(){
		// summary:
		//		Callback from dijit.popup code to this widget, notifying it that it closed
		// tags:
		//		private
		this._setSelectedAttr(null);
	},

	_createOption: function(/*Object*/ item, labelFunc){
		// summary:
		//		Creates an option to appear on the popup menu subclassed by
		//		`dijit/form/FilteringSelect`.

		var menuitem = this._createMenuItem();
		var labelObject = labelFunc(item);
		if(labelObject.html){
			menuitem.innerHTML = labelObject.label;
		}else{
			menuitem.appendChild(
				menuitem.ownerDocument.createTextNode(labelObject.label)
			);
		}
		// #3250: in blank options, assign a normal height
		if(menuitem.innerHTML == ""){
			menuitem.innerHTML = "&#160;";	// &nbsp;
		}

		// update menuitem.dir if BidiSupport was required
		this.applyTextDir(menuitem, (menuitem.innerText || menuitem.textContent || ""));

		return menuitem;
	},

	createOptions: function(results, options, labelFunc){
		// summary:
		//		Fills in the items in the drop down list
		// results:
		//		Array of items
		// options:
		//		The options to the query function of the store
		//
		// labelFunc:
		//		Function to produce a label in the drop down list from a dojo.data item

		this.items = results;

		// display "Previous . . ." button
		this.previousButton.style.display = (options.start == 0) ? "none" : "";
		domAttr.set(this.previousButton, "id", this.id + "_prev");
		// create options using _createOption function defined by parent
		// ComboBox (or FilteringSelect) class
		// #2309:
		//		iterate over cache nondestructively
		array.forEach(results, function(item, i){
			var menuitem = this._createOption(item, labelFunc);
			menuitem.setAttribute("item", i);	// index to this.items; use indirection to avoid mem leak
			domAttr.set(menuitem, "id", this.id + i);
			this.nextButton.parentNode.insertBefore(menuitem, this.nextButton);
		}, this);
		// display "Next . . ." button
		var displayMore = false;
		// Try to determine if we should show 'more'...
		if(results.total && !results.total.then && results.total != -1){
			if((options.start + options.count) < results.total){
				displayMore = true;
			}else if((options.start + options.count) > results.total && options.count == results.length){
				// Weird return from a data store, where a start + count > maxOptions
				// implies maxOptions isn't really valid and we have to go into faking it.
				// And more or less assume more if count == results.length
				displayMore = true;
			}
		}else if(options.count == results.length){
			//Don't know the size, so we do the best we can based off count alone.
			//So, if we have an exact match to count, assume more.
			displayMore = true;
		}

		this.nextButton.style.display = displayMore ? "" : "none";
		domAttr.set(this.nextButton,"id", this.id + "_next");
	},

	clearResultList: function(){
		// summary:
		//		Clears the entries in the drop down list, but of course keeps the previous and next buttons.
		var container = this.containerNode;
		while(container.childNodes.length > 2){
			container.removeChild(container.childNodes[container.childNodes.length-2]);
		}
		this._setSelectedAttr(null);
	},

	highlightFirstOption: function(){
		// summary:
		//		Highlight the first real item in the list (not Previous Choices).
		this.selectFirstNode();
	},

	highlightLastOption: function(){
		// summary:
		//		Highlight the last real item in the list (not More Choices).
		this.selectLastNode();
	},

	selectFirstNode: function(){
		this.inherited(arguments);
		if(this.getHighlightedOption() == this.previousButton){
			this.selectNextNode();
		}
	},

	selectLastNode: function(){
		this.inherited(arguments);
		if(this.getHighlightedOption() == this.nextButton){
			this.selectPreviousNode();
		}
	},

	getHighlightedOption: function(){
		return this.selected;
	}
});

});

},
'dijit/form/_ListMouseMixin':function(){
define("dijit/form/_ListMouseMixin", [
	"dojo/_base/declare", // declare
	"dojo/mouse",
	"dojo/on",
	"dojo/touch",
	"./_ListBase"
], function(declare, mouse, on, touch, _ListBase){

// module:
//		dijit/form/_ListMouseMixin

return declare( "dijit.form._ListMouseMixin", _ListBase, {
	// summary:
	//		a Mixin to handle mouse or touch events for a focus-less menu
	//		Abstract methods that must be defined externally:
	//
	//		- onClick: item was chosen (mousedown somewhere on the menu and mouseup somewhere on the menu)
	// tags:
	//		private

	postCreate: function(){
		this.inherited(arguments);

		this._listConnect("click", "_onClick");
		this._listConnect("mousedown", "_onMouseDown");
		this._listConnect("mouseup", "_onMouseUp");
		this._listConnect("mouseover", "_onMouseOver");
		this._listConnect("mouseout", "_onMouseOut");
	},

	_onClick: function(/*Event*/ evt, /*DomNode*/ target){
		this._setSelectedAttr(target, false);
		if(this._deferredClick){
			this._deferredClick.remove();
		}
		this._deferredClick = this.defer(function(){
			this._deferredClick = null;
			this.onClick(target);
		});
	},

	_onMouseDown: function(/*Event*/ evt, /*DomNode*/ target){
		if(this._hoveredNode){
			this.onUnhover(this._hoveredNode);
			this._hoveredNode = null;
		}
		this._isDragging = true;
		this._setSelectedAttr(target, false);
	},

	_onMouseUp: function(/*Event*/ evt, /*DomNode*/ target){
		this._isDragging = false;
		var selectedNode = this.selected;
		var hoveredNode = this._hoveredNode;
		if(selectedNode && target == selectedNode){
			this.defer(function(){ this._onClick(evt, selectedNode); });
		}else if(hoveredNode){ // drag to select
			this.defer(function(){ this._onClick(evt, hoveredNode); });
		}
	},

	_onMouseOut: function(/*Event*/ evt, /*DomNode*/ target){
		if(this._hoveredNode){
			this.onUnhover(this._hoveredNode);
			this._hoveredNode = null;
		}
		if(this._isDragging){
			this._cancelDrag = (new Date()).getTime() + 1000; // cancel in 1 second if no _onMouseOver fires
		}
	},

	_onMouseOver: function(/*Event*/ evt, /*DomNode*/ target){
		if(this._cancelDrag){
			var time = (new Date()).getTime();
			if(time > this._cancelDrag){
				this._isDragging = false;
			}
			this._cancelDrag = null;
		}
		this._hoveredNode = target;
		this.onHover(target);
		if(this._isDragging){
			this._setSelectedAttr(target, false);
		}
	}
});

});

},
'dijit/form/_ListBase':function(){
define("dijit/form/_ListBase", [
	"dojo/_base/declare",	// declare
	"dojo/on",
	"dojo/window" // winUtils.scrollIntoView
], function(declare, on, winUtils){

// module:
//		dijit/form/_ListBase

return declare( "dijit.form._ListBase", null, {
	// summary:
	//		Focus-less menu to handle UI events consistently
	//		Abstract methods that must be defined externally:
	//
	//		- onSelect: item is active (mousedown but not yet mouseup, or keyboard arrow selected but no Enter)
	//		- onDeselect:  cancels onSelect
	// tags:
	//		private

	// selected: DOMNode
	//		currently selected node
	selected: null,

	_listConnect: function(/*String|Function*/ eventType, /*String*/ callbackFuncName){
		// summary:
		//		Connects 'containerNode' to specified method of this object
		//		and automatically registers for 'disconnect' on widget destroy.
		// description:
		//		Provide widget-specific analog to 'connect'.
		//		The callback function is called with the normal event object,
		//		but also a second parameter is passed that indicates which list item
		//		actually received the event.
		// returns:
		//		A handle that can be passed to `disconnect` in order to disconnect
		//		before the widget is destroyed.
		// tags:
		//		private

		var self = this;
		return self.own(on(self.containerNode,
			on.selector(
				function(eventTarget, selector, target){
					return eventTarget.parentNode == target;
				},
				eventType
			),
			function(evt){
				self[callbackFuncName](evt, this);
			}
		));
	},

	selectFirstNode: function(){
		// summary:
		//		Select the first displayed item in the list.
		var first = this.containerNode.firstChild;
		while(first && first.style.display == "none"){
			first = first.nextSibling;
		}
		this._setSelectedAttr(first, true);
	},

	selectLastNode: function(){
		// summary:
		//		Select the last displayed item in the list
		var last = this.containerNode.lastChild;
		while(last && last.style.display == "none"){
			last = last.previousSibling;
		}
		this._setSelectedAttr(last, true);
	},

	selectNextNode: function(){
		// summary:
		//		Select the item just below the current selection.
		//		If nothing selected, select first node.
		var selectedNode = this.selected;
		if(!selectedNode){
			this.selectFirstNode();
		}else{
			var next = selectedNode.nextSibling;
			while(next && next.style.display == "none"){
				next = next.nextSibling;
			}
			if(!next){
				this.selectFirstNode();
			}else{
				this._setSelectedAttr(next, true);
			}
		}
	},

	selectPreviousNode: function(){
		// summary:
		//		Select the item just above the current selection.
		//		If nothing selected, select last node (if
		//		you select Previous and try to keep scrolling up the list).
		var selectedNode = this.selected;
		if(!selectedNode){
			this.selectLastNode();
		}else{
			var prev = selectedNode.previousSibling;
			while(prev && prev.style.display == "none"){
				prev = prev.previousSibling;
			}
			if(!prev){
				this.selectLastNode();
			}else{
				this._setSelectedAttr(prev, true);
			}
		}
	},

	_setSelectedAttr: function(/*DomNode*/ node, /*Boolean*/ scroll){
		// summary:
		//		Does the actual select.
		// node:
		//		The option to select
		// scroll:
		//		If necessary, scroll node into view.  Set to false for mouse/touch to
		//		avoid jumping problems on mobile/RTL, see https://bugs.dojotoolkit.org/ticket/17739.
		if(this.selected != node){
			var selectedNode = this.selected;
			if(selectedNode){
				this.onDeselect(selectedNode);
				this.selected = null;
			}
			if(node){
				this.selected = node;
				if(scroll){
					winUtils.scrollIntoView(node);
				}
				this.onSelect(node);
			}
		}else if(node){
			this.onSelect(node);
		}
	}
});

});

},
'dijit/form/_AutoCompleterMixin':function(){
define("dijit/form/_AutoCompleterMixin", [
	"dojo/data/util/filter", // patternToRegExp
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.get
	"dojo/_base/event", // event.stop
	"dojo/keys",
	"dojo/_base/lang", // lang.clone lang.hitch
	"dojo/query", // query
	"dojo/regexp", // regexp.escapeString
	"dojo/sniff", // has("ie")
	"dojo/string", // string.substitute
	"./DataList",
	"../registry",	// registry.byId
	"./_TextBoxMixin",	// defines _TextBoxMixin.selectInputText
	"./_SearchMixin"
], function(filter, declare, domAttr, event, keys, lang, query, regexp, has, string,
			DataList, registry, _TextBoxMixin, SearchMixin){

	// module:
	//		dijit/form/_AutoCompleterMixin

	return declare("dijit.form._AutoCompleterMixin", SearchMixin, {
		// summary:
		//		A mixin that implements the base functionality for `dijit/form/ComboBox`/`dijit/form/FilteringSelect`
		// description:
		//		All widgets that mix in dijit/form/_AutoCompleterMixin must extend `dijit/form/_FormValueWidget`.
		// tags:
		//		protected

		// item: Object
		//		This is the item returned by the dojo/store/api/Store implementation that
		//		provides the data for this ComboBox, it's the currently selected item.
		item: null,

		// autoComplete: Boolean
		//		If user types in a partial string, and then tab out of the `<input>` box,
		//		automatically copy the first entry displayed in the drop down list to
		//		the `<input>` field
		autoComplete: true,

		// highlightMatch: String
		//		One of: "first", "all" or "none".
		//
		//		If the ComboBox/FilteringSelect opens with the search results and the searched
		//		string can be found, it will be highlighted.  If set to "all"
		//		then will probably want to change `queryExpr` parameter to '*${0}*'
		//
		//		Highlighting is only performed when `labelType` is "text", so as to not
		//		interfere with any HTML markup an HTML label might contain.
		highlightMatch: "first",

		// labelAttr: String?
		//		The entries in the drop down list come from this attribute in the
		//		dojo.data items.
		//		If not specified, the searchAttr attribute is used instead.
		labelAttr: "",

		// labelType: String
		//		Specifies how to interpret the labelAttr in the data store items.
		//		Can be "html" or "text".
		labelType: "text",

		// Flags to _HasDropDown to limit height of drop down to make it fit in viewport
		maxHeight: -1,

		// For backwards compatibility let onClick events propagate, even clicks on the down arrow button
		_stopClickEvents: false,

		_getCaretPos: function(/*DomNode*/ element){
			// khtml 3.5.2 has selection* methods as does webkit nightlies from 2005-06-22
			var pos = 0;
			if(typeof(element.selectionStart) == "number"){
				// FIXME: this is totally borked on Moz < 1.3. Any recourse?
				pos = element.selectionStart;
			}else if(has("ie")){
				// in the case of a mouse click in a popup being handled,
				// then the win.doc.selection is not the textarea, but the popup
				// var r = win.doc.selection.createRange();
				// hack to get IE 6 to play nice. What a POS browser.
				var tr = element.ownerDocument.selection.createRange().duplicate();
				var ntr = element.createTextRange();
				tr.move("character",0);
				ntr.move("character",0);
				try{
					// If control doesn't have focus, you get an exception.
					// Seems to happen on reverse-tab, but can also happen on tab (seems to be a race condition - only happens sometimes).
					// There appears to be no workaround for this - googled for quite a while.
					ntr.setEndPoint("EndToEnd", tr);
					pos = String(ntr.text).replace(/\r/g,"").length;
				}catch(e){
					// If focus has shifted, 0 is fine for caret pos.
				}
			}
			return pos;
		},

		_setCaretPos: function(/*DomNode*/ element, /*Number*/ location){
			location = parseInt(location);
			_TextBoxMixin.selectInputText(element, location, location);
		},

		_setDisabledAttr: function(/*Boolean*/ value){
			// Additional code to set disabled state of ComboBox node.
			// Overrides _FormValueWidget._setDisabledAttr() or ValidationTextBox._setDisabledAttr().
			this.inherited(arguments);
			this.domNode.setAttribute("aria-disabled", value ? "true" : "false");
		},

		_onKey: function(/*Event*/ evt){
			// summary:
			//		Handles keyboard events

			if(evt.charCode >= 32){ return; } // alphanumeric reserved for searching

			var key = evt.charCode || evt.keyCode;

			// except for cutting/pasting case - ctrl + x/v
			if(key == keys.ALT || key == keys.CTRL || key == keys.META || key == keys.SHIFT){
				return; // throw out spurious events
			}

			var pw = this.dropDown;
			var highlighted = null;
			this._abortQuery();

			// _HasDropDown will do some of the work:
			//
			//	1. when drop down is not yet shown:
			//		- if user presses the down arrow key, call loadDropDown()
			//	2. when drop down is already displayed:
			//		- on ESC key, call closeDropDown()
			//		- otherwise, call dropDown.handleKey() to process the keystroke
			this.inherited(arguments);

			if(evt.altKey || evt.ctrlKey || evt.metaKey){ return; } // don't process keys with modifiers  - but we want shift+TAB

			if(this._opened){
				highlighted = pw.getHighlightedOption();
			}
			switch(key){
				case keys.PAGE_DOWN:
				case keys.DOWN_ARROW:
				case keys.PAGE_UP:
				case keys.UP_ARROW:
					// Keystroke caused ComboBox_menu to move to a different item.
					// Copy new item to <input> box.
					if(this._opened){
						this._announceOption(highlighted);
					}
					event.stop(evt);
					break;

				case keys.ENTER:
					// prevent submitting form if user presses enter. Also
					// prevent accepting the value if either Next or Previous
					// are selected
					if(highlighted){
						// only stop event on prev/next
						if(highlighted == pw.nextButton){
							this._nextSearch(1);
							event.stop(evt); // prevent submit
							break;
						}else if(highlighted == pw.previousButton){
							this._nextSearch(-1);
							event.stop(evt); // prevent submit
							break;
						}
						event.stop(evt); // prevent submit if ENTER was to choose an item
					}else{
						// Update 'value' (ex: KY) according to currently displayed text
						this._setBlurValue(); // set value if needed
						this._setCaretPos(this.focusNode, this.focusNode.value.length); // move cursor to end and cancel highlighting
					}
					// fall through

				case keys.TAB:
					var newvalue = this.get('displayedValue');
					//	if the user had More Choices selected fall into the
					//	_onBlur handler
					if(pw && (
						newvalue == pw._messages["previousMessage"] ||
						newvalue == pw._messages["nextMessage"])
					){
						break;
					}
					if(highlighted){
						this._selectOption(highlighted);
					}
					// fall through

				case keys.ESCAPE:
					if(this._opened){
						this._lastQuery = null; // in case results come back later
						this.closeDropDown();
					}
					break;
			}
		},

		_autoCompleteText: function(/*String*/ text){
			// summary:
			//		Fill in the textbox with the first item from the drop down
			//		list, and highlight the characters that were
			//		auto-completed. For example, if user typed "CA" and the
			//		drop down list appeared, the textbox would be changed to
			//		"California" and "ifornia" would be highlighted.

			var fn = this.focusNode;

			// IE7: clear selection so next highlight works all the time
			_TextBoxMixin.selectInputText(fn, fn.value.length);
			// does text autoComplete the value in the textbox?
			var caseFilter = this.ignoreCase? 'toLowerCase' : 'substr';
			if(text[caseFilter](0).indexOf(this.focusNode.value[caseFilter](0)) == 0){
				var cpos = this.autoComplete ? this._getCaretPos(fn) : fn.value.length;
				// only try to extend if we added the last character at the end of the input
				if((cpos+1) > fn.value.length){
					// only add to input node as we would overwrite Capitalisation of chars
					// actually, that is ok
					fn.value = text;//.substr(cpos);
					// visually highlight the autocompleted characters
					_TextBoxMixin.selectInputText(fn, cpos);
				}
			}else{
				// text does not autoComplete; replace the whole value and highlight
				fn.value = text;
				_TextBoxMixin.selectInputText(fn);
			}
		},

		_openResultList: function(/*Object*/ results, /*Object*/ query, /*Object*/ options){
			// summary:
			//		Callback when a search completes.
			// description:
			//		1. generates drop-down list and calls _showResultList() to display it
			//		2. if this result list is from user pressing "more choices"/"previous choices"
			//			then tell screen reader to announce new option
			var wasSelected = this.dropDown.getHighlightedOption();
			this.dropDown.clearResultList();
			if(!results.length && options.start == 0){ // if no results and not just the previous choices button
				this.closeDropDown();
				return;
			}
			this._nextSearch = this.dropDown.onPage = lang.hitch(this, function(direction){
				results.nextPage(direction !== -1);
				this.focus();
			});

			// Fill in the textbox with the first item from the drop down list,
			// and highlight the characters that were auto-completed. For
			// example, if user typed "CA" and the drop down list appeared, the
			// textbox would be changed to "California" and "ifornia" would be
			// highlighted.

			this.dropDown.createOptions(
				results,
				options,
				lang.hitch(this, "_getMenuLabelFromItem")
			);

			// show our list (only if we have content, else nothing)
			this._showResultList();

			// #4091:
			//		tell the screen reader that the paging callback finished by
			//		shouting the next choice
			if("direction" in options){
				if(options.direction){
					this.dropDown.highlightFirstOption();
				}else if(!options.direction){
					this.dropDown.highlightLastOption();
				}
				if(wasSelected){
					this._announceOption(this.dropDown.getHighlightedOption());
				}
			}else if(this.autoComplete && !this._prev_key_backspace
				// when the user clicks the arrow button to show the full list,
				// startSearch looks for "*".
				// it does not make sense to autocomplete
				// if they are just previewing the options available.
				&& !/^[*]+$/.test(query[this.searchAttr].toString())){
					this._announceOption(this.dropDown.containerNode.firstChild.nextSibling); // 1st real item
			}
		},

		_showResultList: function(){
			// summary:
			//		Display the drop down if not already displayed, or if it is displayed, then
			//		reposition it if necessary (reposition may be necessary if drop down's height changed).
			this.closeDropDown(true);
			this.openDropDown();
			this.domNode.setAttribute("aria-expanded", "true");
		},

		loadDropDown: function(/*Function*/ /*===== callback =====*/){
			// Overrides _HasDropDown.loadDropDown().
			// This is called when user has pressed button icon or pressed the down arrow key
			// to open the drop down.
			this._startSearchAll();
		},

		isLoaded: function(){
			// signal to _HasDropDown that it needs to call loadDropDown() to load the
			// drop down asynchronously before displaying it
			return false;
		},

		closeDropDown: function(){
			// Overrides _HasDropDown.closeDropDown().  Closes the drop down (assuming that it's open).
			// This method is the callback when the user types ESC or clicking
			// the button icon while the drop down is open.  It's also called by other code.
			this._abortQuery();
			if(this._opened){
				this.inherited(arguments);
				this.domNode.setAttribute("aria-expanded", "false");
			}
		},

		_setBlurValue: function(){
			// if the user clicks away from the textbox OR tabs away, set the
			// value to the textbox value
			// #4617:
			//		if value is now more choices or previous choices, revert
			//		the value
			var newvalue = this.get('displayedValue');
			var pw = this.dropDown;
			if(pw && (
				newvalue == pw._messages["previousMessage"] ||
				newvalue == pw._messages["nextMessage"]
				)
			){
				this._setValueAttr(this._lastValueReported, true);
			}else if(typeof this.item == "undefined"){
				// Update 'value' (ex: KY) according to currently displayed text
				this.item = null;
				this.set('displayedValue', newvalue);
			}else{
				if(this.value != this._lastValueReported){
					this._handleOnChange(this.value, true);
				}
				this._refreshState();
			}
			// Remove aria-activedescendant since it may not be removed if they select with arrows then blur with mouse
			this.focusNode.removeAttribute("aria-activedescendant");
		},

		_setItemAttr: function(/*item*/ item, /*Boolean?*/ priorityChange, /*String?*/ displayedValue){
			// summary:
			//		Set the displayed valued in the input box, and the hidden value
			//		that gets submitted, based on a dojo.data store item.
			// description:
			//		Users shouldn't call this function; they should be calling
			//		set('item', value)
			// tags:
			//		private
			var value = '';
			if(item){
				if(!displayedValue){
					displayedValue = this.store._oldAPI ?	// remove getValue() for 2.0 (old dojo.data API)
						this.store.getValue(item, this.searchAttr) : item[this.searchAttr];
				}
				value = this._getValueField() != this.searchAttr ? this.store.getIdentity(item) : displayedValue;
			}
			this.set('value', value, priorityChange, displayedValue, item);
		},

		_announceOption: function(/*Node*/ node){
			// summary:
			//		a11y code that puts the highlighted option in the textbox.
			//		This way screen readers will know what is happening in the
			//		menu.

			if(!node){
				return;
			}
			// pull the text value from the item attached to the DOM node
			var newValue;
			if(node == this.dropDown.nextButton ||
				node == this.dropDown.previousButton){
				newValue = node.innerHTML;
				this.item = undefined;
				this.value = '';
			}else{
				var item = this.dropDown.items[node.getAttribute("item")];
				newValue = (this.store._oldAPI ?	// remove getValue() for 2.0 (old dojo.data API)
					this.store.getValue(item, this.searchAttr) : item[this.searchAttr]).toString();
				this.set('item', item, false, newValue);
			}
			// get the text that the user manually entered (cut off autocompleted text)
			this.focusNode.value = this.focusNode.value.substring(0, this._lastInput.length);
			// set up ARIA activedescendant
			this.focusNode.setAttribute("aria-activedescendant", domAttr.get(node, "id"));
			// autocomplete the rest of the option to announce change
			this._autoCompleteText(newValue);
		},

		_selectOption: function(/*DomNode*/ target){
			// summary:
			//		Menu callback function, called when an item in the menu is selected.
			this.closeDropDown();
			if(target){
				this._announceOption(target);
			}
			this._setCaretPos(this.focusNode, this.focusNode.value.length);
			this._handleOnChange(this.value, true);
			// Remove aria-activedescendant since the drop down is no loner visible
			// after closeDropDown() but _announceOption() adds it back in
			this.focusNode.removeAttribute("aria-activedescendant");
		},

		_startSearchAll: function(){
			this._startSearch('');
		},

		_startSearchFromInput: function(){
			this.item = undefined; // undefined means item needs to be set
			this.inherited(arguments);
		},

		_startSearch: function(/*String*/ key){
			// summary:
			//		Starts a search for elements matching key (key=="" means to return all items),
			//		and calls _openResultList() when the search completes, to display the results.
			if(!this.dropDown){
				var popupId = this.id + "_popup",
					dropDownConstructor = lang.isString(this.dropDownClass) ?
						lang.getObject(this.dropDownClass, false) : this.dropDownClass;
				this.dropDown = new dropDownConstructor({
					onChange: lang.hitch(this, this._selectOption),
					id: popupId,
					dir: this.dir,
					textDir: this.textDir
				});
			}
			this._lastInput = key; // Store exactly what was entered by the user.
			this.inherited(arguments);
		},

		_getValueField: function(){
			// summary:
			//		Helper for postMixInProperties() to set this.value based on data inlined into the markup.
			//		Returns the attribute name in the item (in dijit/form/_ComboBoxDataStore) to use as the value.
			return this.searchAttr;
		},

		//////////// INITIALIZATION METHODS ///////////////////////////////////////

		postMixInProperties: function(){
			this.inherited(arguments);
			if(!this.store){
				var srcNodeRef = this.srcNodeRef;
				// if user didn't specify store, then assume there are option tags
				this.store = new DataList({}, srcNodeRef);

				// if there is no value set and there is an option list, set
				// the value to the first value to be consistent with native Select
				// Firefox and Safari set value
				// IE6 and Opera set selectedIndex, which is automatically set
				// by the selected attribute of an option tag
				// IE6 does not set value, Opera sets value = selectedIndex
				if(!("value" in this.params)){
					var item = (this.item = this.store.fetchSelectedItem());
					if(item){
						var valueField = this._getValueField();
						// remove getValue() for 2.0 (old dojo.data API)
						this.value = this.store._oldAPI ? this.store.getValue(item, valueField) : item[valueField];
					}
				}
			}
		},

		postCreate: function(){
			// summary:
			//		Subclasses must call this method from their postCreate() methods
			// tags:
			//		protected

			// find any associated label element and add to ComboBox node.
			var label=query('label[for="'+this.id+'"]');
			if(label.length){
				if(!label[0].id){ label[0].id = this.id + "_label"; }
				this.domNode.setAttribute("aria-labelledby", label[0].id);

			}
			this.inherited(arguments);
			this.connect(this, "onSearch", "_openResultList");
		},

		_getMenuLabelFromItem: function(/*Item*/ item){
			var label = this.labelFunc(item, this.store),
				labelType = this.labelType;
			// If labelType is not "text" we don't want to screw any markup ot whatever.
			if(this.highlightMatch != "none" && this.labelType == "text" && this._lastInput){
				label = this.doHighlight(label, this._lastInput);
				labelType = "html";
			}
			return {html: labelType == "html", label: label};
		},

		doHighlight: function(/*String*/ label, /*String*/ find){
			// summary:
			//		Highlights the string entered by the user in the menu.  By default this
			//		highlights the first occurrence found. Override this method
			//		to implement your custom highlighting.
			// tags:
			//		protected

			var
				// Add (g)lobal modifier when this.highlightMatch == "all" and (i)gnorecase when this.ignoreCase == true
				modifiers = (this.ignoreCase ? "i" : "") + (this.highlightMatch == "all" ? "g" : ""),
				i = this.queryExpr.indexOf("${0}");
			find = regexp.escapeString(find); // escape regexp special chars
			//If < appears in label, and user presses t, we don't want to highlight the t in the escaped "&lt;"
			//first find out every occurences of "find", wrap each occurence in a pair of "\uFFFF" characters (which
			//should not appear in any string). then html escape the whole string, and replace '\uFFFF" with the
			//HTML highlight markup. 
			return this._escapeHtml(label.replace(
				new RegExp((i == 0 ? "^" : "") + "("+ find +")" + (i == (this.queryExpr.length - 4) ? "$" : ""), modifiers),
				'\uFFFF$1\uFFFF')).replace(
					/\uFFFF([^\uFFFF]+)\uFFFF/g, '<span class="dijitComboBoxHighlightMatch">$1</span>'
			); // returns String, (almost) valid HTML (entities encoded)
		},

		_escapeHtml: function(/*String*/ str){
			// TODO Should become dojo.html.entities(), when exists use instead
			// summary:
			//		Adds escape sequences for special characters in XML: `&<>"'`
			str = String(str).replace(/&/gm, "&amp;").replace(/</gm, "&lt;")
				.replace(/>/gm, "&gt;").replace(/"/gm, "&quot;"); //balance"
			return str; // string
		},

		reset: function(){
			// Overrides the _FormWidget.reset().
			// Additionally reset the .item (to clean up).
			this.item = null;
			this.inherited(arguments);
		},

		labelFunc: function(item, store){
			// summary:
			//		Computes the label to display based on the dojo.data store item.
			// item: Object
			//		The item from the store
			// store: dojo/store/api/Store
			//		The store.
			// returns:
			//		The label that the ComboBox should display
			// tags:
			//		private

			// Use toString() because XMLStore returns an XMLItem whereas this
			// method is expected to return a String (#9354).
			// Remove getValue() for 2.0 (old dojo.data API)
			return (store._oldAPI ? store.getValue(item, this.labelAttr || this.searchAttr) :
				item[this.labelAttr || this.searchAttr]).toString(); // String
		},

		_setValueAttr: function(/*String*/ value, /*Boolean?*/ priorityChange, /*String?*/ displayedValue, /*item?*/ item){
			// summary:
			//		Hook so set('value', value) works.
			// description:
			//		Sets the value of the select.
			this._set("item", item||null); // value not looked up in store
			if(value == null /* or undefined */){ value = ''; } // null translates to blank
			this.inherited(arguments);
		},
		_setTextDirAttr: function(/*String*/ textDir){
			// summary:
			//		Setter for textDir, needed for the dropDown's textDir update.
			// description:
			//		Users shouldn't call this function; they should be calling
			//		set('textDir', value)
			// tags:
			//		private
			this.inherited(arguments);
			// update the drop down also (_ComboBoxMenuMixin)
			if(this.dropDown){
				this.dropDown._set("textDir", textDir);
			}
		}
	});
});

},
'dojo/data/util/filter':function(){
define("dojo/data/util/filter", ["../../_base/lang"], function(lang){
	// module:
	//		dojo/data/util/filter
	// summary:
	//		TODOC

var filter = {};
lang.setObject("dojo.data.util.filter", filter);

filter.patternToRegExp = function(/*String*/pattern, /*boolean?*/ ignoreCase){
	// summary:
	//		Helper function to convert a simple pattern to a regular expression for matching.
	// description:
	//		Returns a regular expression object that conforms to the defined conversion rules.
	//		For example:
	//
	//		- ca*   -> /^ca.*$/
	//		- *ca*  -> /^.*ca.*$/
	//		- *c\*a*  -> /^.*c\*a.*$/
	//		- *c\*a?*  -> /^.*c\*a..*$/
	//
	//		and so on.
	// pattern: string
	//		A simple matching pattern to convert that follows basic rules:
	//
	//		- * Means match anything, so ca* means match anything starting with ca
	//		- ? Means match single character.  So, b?b will match to bob and bab, and so on.
	//		- \ is an escape character.  So for example, \* means do not treat * as a match, but literal character *.
	//
	//		To use a \ as a character in the string, it must be escaped.  So in the pattern it should be
	//		represented by \\ to be treated as an ordinary \ character instead of an escape.
	// ignoreCase:
	//		An optional flag to indicate if the pattern matching should be treated as case-sensitive or not when comparing
	//		By default, it is assumed case sensitive.

	var rxp = "^";
	var c = null;
	for(var i = 0; i < pattern.length; i++){
		c = pattern.charAt(i);
		switch(c){
			case '\\':
				rxp += c;
				i++;
				rxp += pattern.charAt(i);
				break;
			case '*':
				rxp += ".*"; break;
			case '?':
				rxp += "."; break;
			case '$':
			case '^':
			case '/':
			case '+':
			case '.':
			case '|':
			case '(':
			case ')':
			case '{':
			case '}':
			case '[':
			case ']':
				rxp += "\\"; //fallthrough
			default:
				rxp += c;
		}
	}
	rxp += "$";
	if(ignoreCase){
		return new RegExp(rxp,"mi"); //RegExp
	}else{
		return new RegExp(rxp,"m"); //RegExp
	}

};

return filter;
});

},
'dijit/form/DataList':function(){
define("dijit/form/DataList", [
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.byId
	"dojo/_base/lang", // lang.trim
	"dojo/query", // query
	"dojo/store/Memory",
	"../registry"	// registry.add registry.remove
], function(declare, dom, lang, query, MemoryStore, registry){

	// module:
	//		dijit/form/DataList

	function toItem(/*DOMNode*/ option){
		// summary:
		//		Convert `<option>` node to hash
		return {
			id: option.value,
			value: option.value,
			name: lang.trim(option.innerText || option.textContent || '')
		};
	}

	return declare("dijit.form.DataList", MemoryStore, {
		// summary:
		//		Inefficient but small data store specialized for inlined data via OPTION tags
		//
		// description:
		//		Provides a store for inlined data like:
		//
		//	|	<datalist>
		//	|		<option value="AL">Alabama</option>
		//	|		...

		constructor: function(params, srcNodeRef){
			// summary:
			//		Create the widget.
			// params: Object|null
			//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
			//		and functions, typically callbacks like onClick.
			//		The hash can contain any of the widget's properties, excluding read-only properties.
			// srcNodeRef: DOMNode|String
			//		Attach widget to this DOM node.

			// store pointer to original DOM tree
			this.domNode = dom.byId(srcNodeRef);

			lang.mixin(this, params);
			if(this.id){
				registry.add(this); // add to registry so it can be easily found by id
			}
			this.domNode.style.display = "none";

			this.inherited(arguments, [{
				data: query("option", this.domNode).map(toItem)
			}]);
		},

		destroy: function(){
			registry.remove(this.id);
		},

		fetchSelectedItem: function(){
			// summary:
			//		Get the option marked as selected, like `<option selected>`.
			//		Not part of dojo.data API.
			var option = query("> option[selected]", this.domNode)[0] || query("> option", this.domNode)[0];
			return option && toItem(option);
		}
	});
});

},
'dijit/form/_SearchMixin':function(){
define("dijit/form/_SearchMixin", [
	"dojo/data/util/filter", // patternToRegExp
	"dojo/_base/declare", // declare
	"dojo/_base/event", // event.stop
	"dojo/keys", // keys
	"dojo/_base/lang", // lang.clone lang.hitch
	"dojo/query", // query
	"dojo/sniff", // has("ie")
	"dojo/string", // string.substitute
	"dojo/when",
	"../registry"	// registry.byId
], function(filter, declare, event, keys, lang, query, has, string, when, registry){

	// module:
	//		dijit/form/_SearchMixin


	return declare("dijit.form._SearchMixin", null, {
		// summary:
		//		A mixin that implements the base functionality to search a store based upon user-entered text such as
		//		with `dijit/form/ComboBox` or `dijit/form/FilteringSelect`
		// tags:
		//		protected

		// pageSize: Integer
		//		Argument to data provider.
		//		Specifies maximum number of search results to return per query
		pageSize: Infinity,

		// store: [const] dojo/store/api/Store
		//		Reference to data provider object used by this ComboBox.
		//		The store must accept an object hash of properties for its query. See `query` and `queryExpr` for details.
		store: null,

		// fetchProperties: Object
		//		Mixin to the store's fetch.
		//		For example, to set the sort order of the ComboBox menu, pass:
		//	|	{ sort: [{attribute:"name",descending: true}] }
		//		To override the default queryOptions so that deep=false, do:
		//	|	{ queryOptions: {ignoreCase: true, deep: false} }
		fetchProperties:{},

		// query: Object
		//		A query that can be passed to `store` to initially filter the items.
		//		ComboBox overwrites any reference to the `searchAttr` and sets it to the `queryExpr` with the user's input substituted.
		query: {},

		// list: [const] String
		//		Alternate to specifying a store.  Id of a dijit/form/DataList widget.
		list: "",
		_setListAttr: function(list){
			// Avoid having list applied to the DOM node, since it has native meaning in modern browsers
			this._set("list", list);
		},

		// searchDelay: Integer
		//		Delay in milliseconds between when user types something and we start
		//		searching based on that value
		searchDelay: 200,

		// searchAttr: String
		//		Search for items in the data store where this attribute (in the item)
		//		matches what the user typed
		searchAttr: "name",

		// queryExpr: String
		//		This specifies what query is sent to the data store,
		//		based on what the user has typed.  Changing this expression will modify
		//		whether the results are only exact matches, a "starting with" match,
		//		etc.
		//		dojo.data query expression pattern.
		//		`${0}` will be substituted for the user text.
		//		`*` is used for wildcards.
		//		`${0}*` means "starts with", `*${0}*` means "contains", `${0}` means "is"
		queryExpr: "${0}*",

		// ignoreCase: Boolean
		//		Set true if the query should ignore case when matching possible items
		ignoreCase: true,

		_abortQuery: function(){
			// stop in-progress query
			if(this.searchTimer){
				this.searchTimer = this.searchTimer.remove();
			}
			if(this._queryDeferHandle){
				this._queryDeferHandle = this._queryDeferHandle.remove();
			}
			if(this._fetchHandle){
				if(this._fetchHandle.abort){
					this._cancelingQuery = true;
					this._fetchHandle.abort();
					this._cancelingQuery = false;
				}
				if(this._fetchHandle.cancel){
					this._cancelingQuery = true;
					this._fetchHandle.cancel();
					this._cancelingQuery = false;
				}
				this._fetchHandle = null;
			}
		},

		_processInput: function(/*Event*/ evt){
			// summary:
			//		Handles input (keyboard/paste) events
			if(this.disabled || this.readOnly){ return; }
			var key = evt.charOrCode;

			// except for cutting/pasting case - ctrl + x/v
			if("type" in evt && evt.type.substring(0,3) == "key" && (evt.altKey || ((evt.ctrlKey || evt.metaKey) && (key != 'x' && key != 'v')) || key == keys.SHIFT)){
				return; // throw out weird key combinations and spurious events
			}

			var doSearch = false;
			this._prev_key_backspace = false;

			switch(key){
				case keys.DELETE:
				case keys.BACKSPACE:
					this._prev_key_backspace = true;
					this._maskValidSubsetError = true;
					doSearch = true;
					break;

				default:
					// Non char keys (F1-F12 etc..) shouldn't start a search..
					// Ascii characters and IME input (Chinese, Japanese etc.) should.
					//IME input produces keycode == 229.
					doSearch = typeof key == 'string' || key == 229;
			}
			if(doSearch){
				// need to wait a tad before start search so that the event
				// bubbles through DOM and we have value visible
				if(!this.store){
					this.onSearch();
				}else{
					this.searchTimer = this.defer("_startSearchFromInput", 1);
				}
			}
		},

		onSearch: function(/*===== results, query, options =====*/){
			// summary:
			//		Callback when a search completes.
			//
			// results: Object
			//		An array of items from the originating _SearchMixin's store.
			//
			// query: Object
			//		A copy of the originating _SearchMixin's query property.
			//
			// options: Object
			//		The additional parameters sent to the originating _SearchMixin's store, including: start, count, queryOptions.
			//
			// tags:
			//		callback
		},

		_startSearchFromInput: function(){
			this._startSearch(this.focusNode.value);
		},

		_startSearch: function(/*String*/ text){
			// summary:
			//		Starts a search for elements matching text (text=="" means to return all items),
			//		and calls onSearch(...) when the search completes, to display the results.

			this._abortQuery();
			var
				_this = this,
				// Setup parameters to be passed to store.query().
				// Create a new query to prevent accidentally querying for a hidden
				// value from FilteringSelect's keyField
				query = lang.clone(this.query), // #5970
				options = {
					start: 0,
					count: this.pageSize,
					queryOptions: {		// remove for 2.0
						ignoreCase: this.ignoreCase,
						deep: true
					}
				},
				qs = string.substitute(this.queryExpr, [text.replace(/([\\\*\?])/g, "\\$1")]),
				q,
				startQuery = function(){
					var resPromise = _this._fetchHandle = _this.store.query(query, options);
					if(_this.disabled || _this.readOnly || (q !== _this._lastQuery)){
						return;
					} // avoid getting unwanted notify
					when(resPromise, function(res){
						_this._fetchHandle = null;
						if(!_this.disabled && !_this.readOnly && (q === _this._lastQuery)){ // avoid getting unwanted notify
							when(resPromise.total, function(total){
								res.total = total;
								var pageSize = _this.pageSize;
								if(isNaN(pageSize) || pageSize > res.total){ pageSize = res.total; }
								// Setup method to fetching the next page of results
								res.nextPage = function(direction){
									//	tell callback the direction of the paging so the screen
									//	reader knows which menu option to shout
									options.direction = direction = direction !== false;
									options.count = pageSize;
									if(direction){
										options.start += res.length;
										if(options.start >= res.total){
											options.count = 0;
										}
									}else{
										options.start -= pageSize;
										if(options.start < 0){
											options.count = Math.max(pageSize + options.start, 0);
											options.start = 0;
										}
									}
									if(options.count <= 0){
										res.length = 0;
										_this.onSearch(res, query, options);
									}else{
										startQuery();
									}
								};
								_this.onSearch(res, query, options);
							});
						}
					}, function(err){
						_this._fetchHandle = null;
						if(!_this._cancelingQuery){	// don't treat canceled query as an error
							console.error(_this.declaredClass + ' ' + err.toString());
						}
					});
				};

			lang.mixin(options, this.fetchProperties);

			// Generate query
			if(this.store._oldAPI){
				// remove this branch for 2.0
				q = qs;
			}else{
				// Query on searchAttr is a regex for benefit of dojo/store/Memory,
				// but with a toString() method to help dojo/store/JsonRest.
				// Search string like "Co*" converted to regex like /^Co.*$/i.
				q = filter.patternToRegExp(qs, this.ignoreCase);
				q.toString = function(){ return qs; };
			}

			// set _lastQuery, *then* start the timeout
			// otherwise, if the user types and the last query returns before the timeout,
			// _lastQuery won't be set and their input gets rewritten
			this._lastQuery = query[this.searchAttr] = q;
			this._queryDeferHandle = this.defer(startQuery, this.searchDelay);
		},

		//////////// INITIALIZATION METHODS ///////////////////////////////////////

		constructor: function(){
			this.query={};
			this.fetchProperties={};
		},

		postMixInProperties: function(){
			if(!this.store){
				var list = this.list;
				if(list){
					this.store = registry.byId(list);
				}
			}
			this.inherited(arguments);
		}
	});
});

},
'epi/shell/widget/SearchBox':function(){
require({cache:{
'url:epi/shell/widget/templates/SearchBox.html':"﻿<div class=\"dijit dijitReset dijitInline dijitLeft dijitTextBox epi-searchInput epi-contentSearchBox\" id=\"widget_${id}\">\n    <span data-dojo-attach-point=\"clearButton\" class=\"dijitReset dijitRight epi-clearButton\" role=\"presentation\"></span>\n    <div class=\"dijitReset dijitInputField dijitInputContainer\">\n        <input data-dojo-attach-point=\"textbox,focusNode\" type=\"text\" class=\"dijitReset dijitInputInner\" role=\"textbox\" />\n        <span class=\"dijitPlaceHolder dijitInputField\" data-dojo-attach-point=\"_phspan,placeHolderNode\"></span>\n    </div>\n</div>"}});
define("epi/shell/widget/SearchBox", [
// dojo
    "dojo/_base/declare",
    "dojo/_base/lang",

    "dojo/dom-attr",
    "dojo/dom-class",
    "dojo/dom-style",

    "dojo/keys",
    // dijit
    "dijit/_TemplatedMixin",
    "dijit/_Widget",
    "dijit/_WidgetsInTemplateMixin",

    "dijit/focus",
    "dijit/form/TextBox",
    // dojox
    "dojox/html/entities",
    // dgrid
    "dgrid/util/misc",
    // epi
    "epi",
    // Templates
    "dojo/text!./templates/SearchBox.html",
    // Resources
    "dojo/i18n!epi/nls/EPiServer.Shell.UI.Resources.SearchBox"
],
function (
// dojo
    declare,
    lang,

    domAttr,
    domClass,
    domStyle,

    keys,
    // dijit
    _TemplatedMixin,
    _Widget,
    _WidgetsInTemplateMixin,

    focusManager,
    TextBox,
    // dojox
    htmlEntities,
    // dgrid
    miscUtil,
    // epi
    epi,
    // Templates
    template,
    // Resources
    resources
) {
    return declare([TextBox], {
        // tags:
        //      internal

        templateString: template,

        // placeHolder: String
        //      Defines a hint to help users fill out the input field (as defined in HTML 5).
        //      This should only contain plain text (no html markup).
        placeHolder: epi.resources.action.search,

        // commonRes: [private] Object
        //    Common localization resources,
        commonRes: epi.resources,

        // defaultDelay: [Integer]
        //      Delay in milliseconds between when user types something and we start
        //      searching based on that value.
        // tags:
        //      protected
        defaultDelay: 500,

        // encodeSearchText: [public] Boolean
        //      Indicated that need to encode search text or not.
        encodeSearchText: false,

        // triggerChangeOnEnter: Boolean
        //      Indicates if ENTER should trigger the change event
        triggerChangeOnEnter: true,

        postCreate: function () {
            // summary:
            //		Post widget creation. Initialize event for buttons
            // tags:
            //      Protected

            this.inherited(arguments);

            domAttr.set(this.clearButton, "title", resources.clear);

            // clear textbox value when clicking on clear button
            this.connect(this.clearButton, "onclick", lang.hitch(this, this.clearValue));
            this.toggleClearButton();

            this.connect(this, "_onInput",
                miscUtil.debounce(lang.hitch(this, function (event) {
                    this.toggleClearButton();
                    this._valueChanged(event);
                }), null, this.defaultDelay));

            this.connect(this.domNode, "onclick", "_onClick");
        },

        clearValue: function () {
            // summary:
            //      Clears the value of the search box
            // tags:
            //      public

            this.set("value", "");
            this.focus();
            this.toggleClearButton();
            this._valueChanged();
        },

        toggleClearButton: function () {
            // summary:
            //      Set visibility for clear button
            // tags:
            //      private

            // show clear button only if textbox has value
            domStyle.set(this.clearButton, "visibility", this.textbox.value ? "visible" : "hidden");
        },

        focus: function () {
            // summary:
            //      Set focus for textbox
            // tags:
            //      public

            focusManager.focus(this.textbox);
        },

        _onClick: function () {
            // summary:
            //		Keep focus on textbox
            // tags:
            //		private

            this.focus();
        },

        _valueChanged: function (/* Event */event) {
            // summary:
            //      Trigger when textbox value has been changed
            // tags:
            //      private

            if (event && event.charOrCode) {
                switch (event.charOrCode) {
                    case keys.ESCAPE:
                        this.clearValue();
                        return;
                    case keys.PAGE_DOWN:
                    case keys.DOWN_ARROW:
                    case keys.PAGE_UP:
                    case keys.UP_ARROW:
                    case keys.LEFT_ARROW:
                    case keys.RIGHT_ARROW:
                    case keys.TAB:
                        // do nothing when navigating, selecting text
                        return;
                }

                if (!this.triggerChangeOnEnter && event.charOrCode === keys.ENTER) {
                    return;
                }
            }

            this.onSearchBoxChange(this.encodeSearchText ? htmlEntities.encode(this.textbox.value) : this.textbox.value);
        },
        onSearchBoxChange: function (queryText) {
            // summary:
            //      Callback for the search box change event
            // tags:
            //      public
        }
    });
});

},
'url:epi/shell/widget/templates/SearchBox.html':"﻿<div class=\"dijit dijitReset dijitInline dijitLeft dijitTextBox epi-searchInput epi-contentSearchBox\" id=\"widget_${id}\">\n    <span data-dojo-attach-point=\"clearButton\" class=\"dijitReset dijitRight epi-clearButton\" role=\"presentation\"></span>\n    <div class=\"dijitReset dijitInputField dijitInputContainer\">\n        <input data-dojo-attach-point=\"textbox,focusNode\" type=\"text\" class=\"dijitReset dijitInputInner\" role=\"textbox\" />\n        <span class=\"dijitPlaceHolder dijitInputField\" data-dojo-attach-point=\"_phspan,placeHolderNode\"></span>\n    </div>\n</div>",
'epi-cms/widget/SearchResultList':function(){
require({cache:{
'url:epi-cms/widget/templates/SearchResultList.html':"﻿<div class=\"epi-searchResultList\">\n    <div data-dojo-attach-point=\"containerNode\">\n        <div data-dojo-attach-point=\"errorMessage\" class=\"epi-searchResultListEmptyMessage\"></div>\n    </div>\n    <div class=\"dijitReset epi-searchResultListNavigation\">\n        <div data-dojo-attach-point=\"previousButton\" role=\"option\"></div>\n        <div data-dojo-attach-point=\"nextButton\" role=\"option\"></div>\n    </div>\n</div>"}});
﻿define("epi-cms/widget/SearchResultList", [
// Dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/Evented",
    "dojo/dom-construct",
    "dojo/dom-class",
    "dojo/dom-style",

    // Dojox
    "dojox/widget/Standby",

    // Dijit
    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dijit/form/_ListMouseMixin",
    "dijit/form/_ComboBoxMenuMixin",

    // EPi Framework
    "epi/dependency",
    "epi-cms/widget/_ContentListKeyMixin",
    "epi-cms/widget/_ContentListMouseMixin",
    "epi-cms/dgrid/formatters",
    "epi/shell/dgrid/Formatter",

    // DGrid
    "dgrid/OnDemandList",
    "dgrid/Selection",
    "dgrid/Keyboard",

    // Templates
    "dojo/text!./templates/SearchResultList.html"
],

function (
// Dojo
    declare,
    lang,
    Evented,
    domConstruct,
    domClass,
    domStyle,

    // Dojox
    Standby,

    // Dijit
    _WidgetBase,
    _TemplatedMixin,
    _ListMouseMixin,
    _ComboBoxMenuMixin,

    // EPi Framework
    dependency,
    _ContentListKeyMixin,
    _ContentListMouseMixin,
    formatters,
    Formatter,
    // DGrid
    OnDemandList,
    Selection,
    Keyboard,

    // Templates
    template
) {

    return declare([_WidgetBase, _TemplatedMixin, Evented, _ListMouseMixin, _ComboBoxMenuMixin, _ContentListKeyMixin, _ContentListMouseMixin], {
        // summary:
        //    A grid to display content list.
        //
        // tags:
        //    internal

        templateString: template,

        // _standby: Standby
        //      The standby object to indicate the loading process for grid.
        // tags:
        //      Protected
        _standby: null,

        // _messages: Object
        //		Holds "next" and "previous" text for paging buttons on drop down.
        _messages: { nextMessage: "", previousMessage: "" },

        postCreate: function () {
            // summary:
            //		Initialize grid for displaying content list.
            // tags:
            //      Protected
            this.inherited(arguments);

            var self = this,
                profile = dependency.resolve("epi.shell.Profile"),
                gridClass = declare([OnDemandList, Selection, Keyboard, Formatter]),
                // Get item's title
                titleSelector = function (item) {
                    return item.title;
                },
                // Get item's typeIdentifier
                iconSelector = function (item) {
                    return item.metadata.typeIdentifier;
                },
                // Get item's thumbnail url
                thumbnailSelector = function (item) {
                    var container = self.grid.domNode,
                        thumbnailClass = "epi-thumbnailContentList";

                    (item.metadata.thumbnailUrl && !domClass.contains(container, thumbnailClass)) && domClass.add(container, thumbnailClass);

                    return item.metadata.thumbnailUrl;
                },
                languageSelector = function (item) {
                    if (item.metadata && item.metadata.languageBranch !== profile.contentLanguage) {
                        return { language: item.metadata.languageBranch };
                    }
                };

            this.grid = new gridClass({
                formatters: [formatters.contentItemFactory("title", titleSelector, iconSelector, null, thumbnailSelector, languageSelector, false)],
                selectionMode: "single", // for Selection; only select a single row at a time
                cellNavigation: false // for Keyboard; allow only row-level keyboard navigation
            });

            // Hide header of the grid
            this.grid.set("showHeader", false);

            domConstruct.place(this.grid.domNode, this.containerNode);

            // trigger onSelect event when clicking on grid row
            this.grid.on(".dgrid-row:click", lang.hitch(this, function (event) {
                var row = this.grid.row(event);
                if (row) {
                    this._onSelect(row.data);
                }
            }));

            this._standby = new Standby({ target: this.domNode, color: "#fff" }).placeAt(document.body);
            this.own(this._standby);

            // assign the grid container as containerNode, so _ListMouseMixin and _ComboBoxMenuMixin can get the properly container
            this.containerNode = this._getGridContainer();
        },

        startup: function () {
            // summary:
            //		handle processing after any DOM fragments have been actually added to the document.
            // tags:
            //      Protected
            this.inherited(arguments);
            this._standby.startup();
        },

        showStandby: function (show) {
            // summary:
            //		Set standby visibility.
            // tags:
            //      Protected

            if (!this._standby) {
                return;
            }
            if (show) {
                if (!this._standby.isVisible()) {
                    this._standby.show();
                }
            } else {
                this._standby.hide();
            }
        },

        showGrid: function (show) {
            // summary:
            //		Set grid visibility.
            // tags:
            //      Protected
            domStyle.set(this.grid.domNode, "display", show ? "block" : "none");
        },

        _onSelect: function (value) {
            // summary:
            //		Emit onSelect event when an item has been selected.
            // tags:
            //      Private
            this.emit("select", value);
        },

        _getGridContainer: function () {
            // summary:
            //		Returns the grid container, which contains grid rows.
            // tags:
            //      Private
            return this.grid.contentNode;
        },

        createOptions: function (results, options, labelFunc) {
            // summary:
            //		Fills in the items in the list
            // results:
            //		Array of items
            // options:
            //		The options to the query function of the store
            //
            // labelFunc:
            //		Function to produce a label in the drop down list from a dojo/data item

            this.grid.renderArray(results);
            return this.containerNode.childNodes;
        },

        clearResultList: function () {
            // summary:
            //		Clears the entries in the list.

            this.grid.refresh();
        },

        getHighlightedOption: function () {
            // summary:
            //		This will be used to highlight the text in the textbox, to allow screen readers to know what is happening in the menu.
            //      Return null since we don't want this feature.

            return null;
        },

        showErrorMessage: function (show, message) {
            // summary:
            //		Set error message visibility.
            // tags:
            //      Protected

            this.clearResultList();
            this.showStandby(false);
            if (show && message) {
                this.errorMessage.innerHTML = message;
            }
            domStyle.set(this.errorMessage, "display", show ? "block" : "none");
        }
    });
});

},
'dojox/widget/Standby':function(){
define("dojox/widget/Standby", ["dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/event",
	"dojo/_base/sniff",
	"dojo/dom",
	"dojo/dom-attr",
	"dojo/dom-construct",
	"dojo/dom-geometry",
	"dojo/dom-style",
	"dojo/window",
	"dojo/_base/window",
	"dojo/_base/fx",
	"dojo/fx",
	"dijit/_Widget",
	"dijit/_TemplatedMixin",
	"dijit/registry"],

function(kernel,
		declare,
		array,
		event,
		has,
		dom,
		attr,
		construct,
		geometry,
		domStyle,
		window,
		baseWindow,
		baseFx,
		fx,
		_Widget,
		_TemplatedMixin,
		registry) {

kernel.experimental("dojox.widget.Standby");

return declare("dojox.widget.Standby", [_Widget, _TemplatedMixin],{
	// summary:
	//		A widget designed to act as a Standby/Busy/Disable/Blocking widget to indicate a
	//		particular DOM node is processing and cannot be clicked on at this time.
	//		This widget uses absolute positioning to apply the overlay and image.

	// image: String
	//		A URL to an image to center within the blocking overlay.
	//		The default is a basic spinner.
	image: require.toUrl("dojox/widget/Standby/images/loading.gif").toString(),

	// imageText: String
	//		Text to set on the ALT tag of the image.
	//		The default is 'Please wait...'
	imageText: "Please Wait...", // TODO: i18n

	// text: String
	//		Text/HTML to display in the center of the overlay
	//		This is used if image center is disabled.
	//		Defaults to 'Please Wait...'
	text: "Please wait...",

	// centerIndicator: String
	//		Property to define if the image and its alt text should be used, or
	//		a simple Text/HTML node should be used.  Allowable values are 'image'
	//		and 'text'.
	//		Default is 'image'.
	centerIndicator: "image",
	
	// target: DOMNode||DOMID(String)||WidgetID(String)
	//		The target to overlay when active.  Can be a widget id, a
	//		dom id, or a direct node reference.
	target: "",

	// color:	String
	//		The color to set the overlay.  Should be in #XXXXXX form.
	//		Default color for the translucent overlay is light gray.
	color: "#C0C0C0",

	// duration: Integer
	//		Integer defining how long the show and hide effects should take in milliseconds.
	//		Defaults to 500
	duration: 500,
	
	// zIndex: String
	//		Control that lets you specify if the zIndex for the overlay
	//		should be auto-computed based off parent zIndex, or should be set
	//		to a particular value.  This is useful when you want to overlay
	//		things in digit.Dialogs, you can specify a base zIndex to append from.
	zIndex: "auto",
	
	// opacity: float
	//		The opacity to make the overlay when it is displayed/faded in.
	//		The default is 0.75.  This does not affect the image opacity, only the
	//		overlay.
	opacity: 0.75,	
	
	// templateString: [protected] String
	//		The template string defining out the basics of the widget.  No need for an external
	//		file.
	templateString:
		"<div>" +
			"<div style=\"display: none; opacity: 0; z-index: 9999; " +
				"position: absolute; cursor:wait;\" dojoAttachPoint=\"_underlayNode\"></div>" +
			"<img src=\"${image}\" style=\"opacity: 0; display: none; z-index: -10000; " +
				"position: absolute; top: 0px; left: 0px; cursor:wait;\" "+
				"dojoAttachPoint=\"_imageNode\">" +
			"<div style=\"opacity: 0; display: none; z-index: -10000; position: absolute; " +
				"top: 0px;\" dojoAttachPoint=\"_textNode\"></div>" +
		"</div>",

	// _underlayNode: [private] DOMNode
	//		The node that is the translucent underlay for the
	//		image that blocks access to the target.
	_underlayNode: null,

	// _imageNode: [private] DOMNode
	//		The image node where we attach and define the image to display.
	_imageNode: null,

	// _textNode: [private] DOMNode
	//		The div to attach text/HTML in the overlay center item.
	_textNode: null,

	// _centerNode: [private] DOMNode
	//		Which node to use as the center node, the image or the text node.
	_centerNode: null,

	// _displayed: [private] Boolean
	//		Flag to indicate if the overlay is displayed or not.
	_displayed: false,

	// _resizeCheck: [private] Object
	//		Handle to interval function that checks the target for changes.
	_resizeCheck: null,

	// _started: [private] Boolean
	//		Trap flag to ensure startup only processes once.
	_started: false,

	// _parent: [private] DOMNode
	//		Wrapping div for the widget, also used for IE 7 in dealing with the
	//		zoom issue.
	_parent: null,

	startup: function(args){
		// summary:
		//		Over-ride of the basic widget startup function.
		//		Configures the target node and sets the image to use.
		if(!this._started){
			if(typeof this.target === "string"){
				var w = registry.byId(this.target);
				this.target = w ? w.domNode : dom.byId(this.target);
			}

			if(this.text){
				this._textNode.innerHTML = this.text;
			}
			if(this.centerIndicator === "image"){
				this._centerNode = this._imageNode;
				attr.set(this._imageNode, "src", this.image);
				attr.set(this._imageNode, "alt", this.imageText);
			}else{
				this._centerNode = this._textNode;
			}
			domStyle.set(this._underlayNode, {
				display: "none",
				backgroundColor: this.color
			});
			domStyle.set(this._centerNode, "display", "none");
			this.connect(this._underlayNode, "onclick", "_ignore");

			//Last thing to do is move the widgets parent, if any, to the current document body.
			//Avoids having to deal with parent relative/absolute mess.  Otherwise positioning
			//tends to go goofy.
			if(this.domNode.parentNode && this.domNode.parentNode != baseWindow.body()){
				baseWindow.body().appendChild(this.domNode);
			}

			//IE 7 has a horrible bug with zoom, so we have to create this node
			//to cross-check later.  Sigh.
			if(has("ie") == 7){
				this._ieFixNode = construct.create("div");
				domStyle.set(this._ieFixNode, {
					opacity: "0",
					zIndex: "-1000",
					position: "absolute",
					top: "-1000px"
				});
				baseWindow.body().appendChild(this._ieFixNode);
			}
			this.inherited(arguments);
		}		
	},

	show: function(){
		// summary:
		//		Function to display the blocking overlay and busy/status icon or text.
		if(!this._displayed){
			if(this._anim){
				this._anim.stop();
				delete this._anim;
			}
			this._displayed = true;
			this._size();
			this._disableOverflow();
			this._fadeIn();
		}
	},

	hide: function(){
		// summary:
		//		Function to hide the blocking overlay and status icon or text.
		if(this._displayed){
			if(this._anim){
				this._anim.stop();
				delete this._anim;
			}
			this._size();
			this._fadeOut();
			this._displayed = false;
			if(this._resizeCheck !== null){
				clearInterval(this._resizeCheck);
				this._resizeCheck = null;
			}
		}
	},

	isVisible: function(){
		// summary:
		//		Helper function so you can test if the widget is already visible or not.
		// returns:
		//		boolean indicating if the widget is in 'show' state or not.
		return this._displayed; // boolean
	},

	onShow: function(){
		// summary:
		//		Event that fires when the display of the Standby completes.
	},

	onHide: function(){
		// summary:
		//		Event that fires when the display of the Standby completes.
	},

	uninitialize: function(){
		// summary:
		//		Over-ride to hide the widget, which clears intervals, before cleanup.
		this._displayed = false;
		if(this._resizeCheck){
			clearInterval(this._resizeCheck);
		}
		domStyle.set(this._centerNode, "display", "none");
		domStyle.set(this._underlayNode, "display", "none");
		if(has("ie") == 7 && this._ieFixNode){
			baseWindow.body().removeChild(this._ieFixNode);
			delete this._ieFixNode;
		}
		if(this._anim){
			this._anim.stop();
			delete this._anim;
		}
		this.target = null;
		this._imageNode = null;
		this._textNode = null;
		this._centerNode = null;
		this.inherited(arguments);
	},

	_size: function(){
		// summary:
		//		Internal function that handles resizing the overlay and
		//		centering of the image on window resizing.
		// tags:
		//		private
		if(this._displayed){
			var dir = attr.get(baseWindow.body(), "dir");
			if(dir){dir = dir.toLowerCase();}
			var _ie7zoom;
			var scrollers = this._scrollerWidths();

			var target = this.target;

			//Show the image and make sure the zIndex is set high.
			var curStyle = domStyle.get(this._centerNode, "display");
			domStyle.set(this._centerNode, "display", "block");
			var box = geometry.position(target, true);
			if(target === baseWindow.body() || target === baseWindow.doc){
				// Target is the whole doc, so scale to viewport.
				box = window.getBox();
				box.x = box.l;
				box.y = box.t;
			}

			var cntrIndicator = geometry.getMarginBox(this._centerNode);
			domStyle.set(this._centerNode, "display", curStyle);

			//IE has a horrible zoom bug.  So, we have to try and account for
			//it and fix up the scaling.
			if(this._ieFixNode){
				_ie7zoom = -this._ieFixNode.offsetTop / 1000;
				box.x = Math.floor((box.x + 0.9) / _ie7zoom);
				box.y = Math.floor((box.y + 0.9) / _ie7zoom);
				box.w = Math.floor((box.w + 0.9) / _ie7zoom);
				box.h = Math.floor((box.h + 0.9) / _ie7zoom);
			}

			//Figure out how to zIndex this thing over the target.
			var zi = domStyle.get(target, "zIndex");
			var ziUl = zi;
			var ziIn = zi;

			if(this.zIndex === "auto"){
				if(zi != "auto"){
					ziUl = parseInt(ziUl, 10) + 1;
					ziIn = parseInt(ziIn, 10) + 2;
				}else{
					//We need to search up the chain to see if there
					//are any parent zIndexs to overlay.
					var cNode = target.parentNode;
					var oldZi = -100000;
					while(cNode && cNode !== baseWindow.body() && target !== baseWindow.body()){
						zi = domStyle.get(cNode, "zIndex");
						if(!zi || zi === "auto"){
							cNode = cNode.parentNode;
						}else{
							var newZi = parseInt(zi, 10);
							if(oldZi < newZi){
								oldZi = newZi;
								ziUl = newZi + 1;
								ziIn = newZi + 2;
							}
							// Keep looking until we run out, we want the highest zIndex.
							cNode = cNode.parentNode;
						}
					}
				}
			}else{
				ziUl = parseInt(this.zIndex, 10) + 1;
				ziIn = parseInt(this.zIndex, 10) + 2;
			}

			domStyle.set(this._centerNode, "zIndex", ziIn);
			domStyle.set(this._underlayNode, "zIndex", ziUl);


			var pn = target.parentNode;
			if(pn && pn !== baseWindow.body() &&
				target !== baseWindow.body() &&
				target !== baseWindow.doc){
				
				// If the parent is the body tag itself,
				// we can avoid all this, the body takes
				// care of overflow for me.  Besides, browser
				// weirdness with height and width on body causes
				// problems with this sort of intersect testing
				// anyway.
				var obh = box.h;
				var obw = box.w;
				var pnBox = geometry.position(pn, true);

				//More IE zoom corrections.  Grr.
				if(this._ieFixNode){
					_ie7zoom = -this._ieFixNode.offsetTop / 1000;
					pnBox.x = Math.floor((pnBox.x + 0.9) / _ie7zoom);
					pnBox.y = Math.floor((pnBox.y + 0.9) / _ie7zoom);
					pnBox.w = Math.floor((pnBox.w + 0.9) / _ie7zoom);
					pnBox.h = Math.floor((pnBox.h + 0.9) / _ie7zoom);
				}
				
				//Shift the parent width/height a bit if scollers are present.
				pnBox.w -= pn.scrollHeight > pn.clientHeight &&
					pn.clientHeight > 0 ? scrollers.v: 0;
				pnBox.h -= pn.scrollWidth > pn.clientWidth &&
					pn.clientWidth > 0 ? scrollers.h: 0;

				//RTL requires a bit of massaging in some cases
				//(and differently depending on browser, ugh!)
				//WebKit and others still need work.
				if(dir === "rtl"){
					if(has("opera")){
						box.x += pn.scrollHeight > pn.clientHeight &&
							pn.clientHeight > 0 ? scrollers.v: 0;
						pnBox.x += pn.scrollHeight > pn.clientHeight &&
							pn.clientHeight > 0 ? scrollers.v: 0;
					}else if(has("ie")){
						pnBox.x += pn.scrollHeight > pn.clientHeight &&
							pn.clientHeight > 0 ? scrollers.v: 0;
					}else if(has("webkit")){
						//TODO:  FIX THIS!
					}
				}

				//Figure out if we need to adjust the overlay to fit a viewable
				//area, then resize it, we saved the original height/width above.
				//This is causing issues on IE.  Argh!
				if(pnBox.w < box.w){
					//Scale down the width if necessary.
					box.w = box.w - pnBox.w;
				}
				if(pnBox.h < box.h){
					//Scale down the width if necessary.
					box.h = box.h - pnBox.h;
				}

				//Look at the y positions and see if we intersect with the
				//viewport borders.  Will have to do computations off it.
				var vpTop = pnBox.y;
				var vpBottom = pnBox.y + pnBox.h;
				var bTop = box.y;
				var bBottom = box.y + obh;
				var vpLeft = pnBox.x;
				var vpRight = pnBox.x + pnBox.w;
				var bLeft = box.x;
				var bRight = box.x + obw;
				var delta;
				//Adjust the height now
				if(bBottom > vpTop &&
					bTop < vpTop){
					box.y = pnBox.y;
					//intersecting top, need to do some shifting.
					delta = vpTop - bTop;
					var visHeight = obh - delta;
					//If the visible height < viewport height,
					//We need to shift it.
					if(visHeight < pnBox.h){
						box.h = visHeight;
					}else{
						//Deal with horizontal scrollbars if necessary.
						box.h -= 2*(pn.scrollWidth > pn.clientWidth &&
							pn.clientWidth > 0? scrollers.h: 0);
					}
				}else if(bTop < vpBottom && bBottom > vpBottom){
					//Intersecting bottom, just figure out how much
					//overlay to show.
					box.h = vpBottom - bTop;
				}else if(bBottom <= vpTop || bTop >= vpBottom){
					//Outside view, hide it.
					box.h = 0;
				}

				//adjust width
				if(bRight > vpLeft && bLeft < vpLeft){
					box.x = pnBox.x;
					//intersecting left, need to do some shifting.
					delta = vpLeft - bLeft;
					var visWidth = obw - delta;
					//If the visible width < viewport width,
					//We need to shift it.
					if(visWidth < pnBox.w){
						box.w = visWidth;
					}else{
						//Deal with horizontal scrollbars if necessary.
						box.w -= 2*(pn.scrollHeight > pn.clientHeight &&
							pn.clientHeight > 0? scrollers.w:0);
					}
				}else if(bLeft < vpRight && bRight > vpRight){
					//Intersecting right, just figure out how much
					//overlay to show.
					box.w = vpRight - bLeft;
				}else if(bRight <= vpLeft || bLeft >= vpRight){
					//Outside view, hide it.
					box.w = 0;
				}
			}

			if(box.h > 0 && box.w > 0){
				//Set position and size of the blocking div overlay.
				domStyle.set(this._underlayNode, {
					display: "block",
					width: box.w + "px",
					height: box.h + "px",
					top: box.y + "px",
					left: box.x + "px"
				});

				var styles = ["borderRadius", "borderTopLeftRadius",
					"borderTopRightRadius","borderBottomLeftRadius",
					"borderBottomRightRadius"];
				this._cloneStyles(styles);
				if(!has("ie")){
					//Browser specific styles to try and clone if non-IE.
					styles = ["MozBorderRadius", "MozBorderRadiusTopleft",
						"MozBorderRadiusTopright","MozBorderRadiusBottomleft",
						"MozBorderRadiusBottomright","WebkitBorderRadius",
						"WebkitBorderTopLeftRadius", "WebkitBorderTopRightRadius",
						"WebkitBorderBottomLeftRadius","WebkitBorderBottomRightRadius"
					];
					this._cloneStyles(styles, this);
				}
				var cntrIndicatorTop = (box.h/2) - (cntrIndicator.h/2);
				var cntrIndicatorLeft = (box.w/2) - (cntrIndicator.w/2);
				//Only show the image if there is height and width room.
				if(box.h >= cntrIndicator.h && box.w >= cntrIndicator.w){
					domStyle.set(this._centerNode, {
						top: (cntrIndicatorTop + box.y) + "px",
						left: (cntrIndicatorLeft + box.x) + "px",
						display: "block"
					});
				}else{
					domStyle.set(this._centerNode, "display", "none");
				}
			}else{
				//Target has no size, display nothing on it!
				domStyle.set(this._underlayNode, "display", "none");
				domStyle.set(this._centerNode, "display", "none");
			}
			if(this._resizeCheck === null){
				//Set an interval timer that checks the target size and scales as needed.
				//Checking every 10th of a second seems to generate a fairly smooth update.
				var self = this;
				this._resizeCheck = setInterval(function(){self._size();}, 100);
			}
		}
	},

	_cloneStyles: function(list){
		// summary:
		//		Internal function to clone a set of styles from the target to
		//		the underlay.
		// list: Array
		//		An array of style names to clone.
		//
		// tags:
		//		private
		array.forEach(list, function(s){
			domStyle.set(this._underlayNode, s, domStyle.get(this.target, s));
		}, this);
	},

	_fadeIn: function(){
		// summary:
		//		Internal function that does the opacity style fade in animation.
		// tags:
		//		private
		var self = this;
		var underlayNodeAnim = baseFx.animateProperty({
			duration: self.duration,
			node: self._underlayNode,
			properties: {opacity: {start: 0, end: self.opacity}}
		});
		var imageAnim = baseFx.animateProperty({
			duration: self.duration,
			node: self._centerNode,
			properties: {opacity: {start: 0, end: 1}},
			onEnd: function(){
				self.onShow();
				delete self._anim;
			}
		});
		this._anim = fx.combine([underlayNodeAnim,imageAnim]);
		this._anim.play();
	},

	_fadeOut: function(){
		// summary:
		//		Internal function that does the opacity style fade out animation.
		// tags:
		//		private
		var self = this;
		var underlayNodeAnim = baseFx.animateProperty({
			duration: self.duration,
			node: self._underlayNode,
			properties: {opacity: {start: self.opacity, end: 0}},
			onEnd: function(){
				domStyle.set(this.node,{"display":"none", "zIndex": "-1000"});
			}
		});
		var imageAnim = baseFx.animateProperty({
			duration: self.duration,
			node: self._centerNode,
			properties: {opacity: {start: 1, end: 0}},
			onEnd: function(){
				domStyle.set(this.node,{"display":"none", "zIndex": "-1000"});
				self.onHide();
				self._enableOverflow();
				delete self._anim;
			}
		});
		this._anim = fx.combine([underlayNodeAnim,imageAnim]);
		this._anim.play();
	},

	_ignore: function(e){
		// summary:
		//		Function to ignore events that occur on the overlay.
		// event: Event
		//		The event to halt
		// tags:
		//		private
		if(e){
			event.stop(e);
		}
	},

	_scrollerWidths: function(){
		// summary:
		//		This function will calculate the size of the vertical and
		//		horizontaol scrollbars.
		// returns:
		//		Object of form: {v: Number, h: Number} where v is vertical scrollbar width
		//		and h is horizontal scrollbar width.
		// tags:
		//		private
		var div = construct.create("div");
		domStyle.set(div, {
			position: "absolute",
			opacity: 0,
			overflow: "hidden",
			width: "50px",
			height: "50px",
			zIndex: "-100",
			top: "-200px",
			padding: "0px",
			margin: "0px"
		});
		var iDiv = construct.create("div");
		domStyle.set(iDiv, {
			width: "200px",
			height: "10px"
		});
		div.appendChild(iDiv);
		baseWindow.body().appendChild(div);

		//Figure out content size before and after
		//scrollbars are there, then just subtract to
		//get width.
		var b = geometry.getContentBox(div);
		domStyle.set(div, "overflow", "scroll");
		var a = geometry.getContentBox(div);
		baseWindow.body().removeChild(div);
		return { v: b.w - a.w, h: b.h - a.h };
	},

	/* The following are functions that tie into _Widget.attr() */

	_setTextAttr: function(text){
		// summary:
		//		Function to allow widget.attr to set the text displayed in center
		//		if using text display.
		// text: String
		//		The text to set.
		this._textNode.innerHTML = text;
		this.text = text;
	},

	_setColorAttr: function(c){
		// summary:
		//		Function to allow widget.attr to set the color used for the translucent
		//		div overlay.
		// c: String
		//		The color to set the background underlay to in #XXXXXX format..
		domStyle.set(this._underlayNode, "backgroundColor", c);
		this.color = c;
	},

	_setImageTextAttr: function(text){
		// summary:
		//		Function to allow widget.attr to set the ALT text text displayed for
		//		the image (if using image center display).
		// text: String
		//		The text to set.
		attr.set(this._imageNode, "alt", text);
		this.imageText = text;
	},

	_setImageAttr: function(url){
		// summary:
		//		Function to allow widget.attr to set the url source for the center image
		// text: String
		//		The url to set for the image.
		attr.set(this._imageNode, "src", url);
		this.image = url;
	},

	_setCenterIndicatorAttr: function(indicator){
		// summary:
		//		Function to allow widget.attr to set the node used for the center indicator,
		//		either the image or the text.
		// indicator: String
		//		The indicator to use, either 'image' or 'text'.
		this.centerIndicator = indicator;
		if(indicator === "image"){
			this._centerNode = this._imageNode;
			domStyle.set(this._textNode, "display", "none");
		}else{
			this._centerNode = this._textNode;
			domStyle.set(this._imageNode, "display", "none");
		}
	},

	_disableOverflow: function(){
		 // summary:
		 //		Function to disable scrollbars on the body.  Only used if the overlay
		 //		targets the body or the document.
		 if(this.target === baseWindow.body() || this.target === baseWindow.doc){
			 // Store the overflow state we have to restore later.
			 // IE had issues, so have to check that it's defined.  Ugh.
			 this._overflowDisabled = true;
			 var body = baseWindow.body();
			 if(body.style && body.style.overflow){
				 this._oldOverflow = domStyle.get(body, "overflow");
			 }else{
				 this._oldOverflow = "";
			 }
			 if(has("ie") && !has("quirks")){
				 // IE will put scrollbars in anyway, html (parent of body)
				 // also controls them in standards mode, so we have to
				 // remove them, argh.
				 if(body.parentNode &&
					body.parentNode.style &&
					body.parentNode.style.overflow){
					 this._oldBodyParentOverflow = body.parentNode.style.overflow;
				 }else{
					 try{
						this._oldBodyParentOverflow = domStyle.get(body.parentNode, "overflow");
					 }catch(e){
						 this._oldBodyParentOverflow = "scroll";
					 }
				 }
				 domStyle.set(body.parentNode, "overflow", "hidden");
			 }
			 domStyle.set(body, "overflow", "hidden");
		 }
	},

	_enableOverflow: function(){
		 // summary:
		 //		Function to restore scrollbars on the body.  Only used if the overlay
		 //		targets the body or the document.
		 if(this._overflowDisabled){
			delete this._overflowDisabled;
			var body = baseWindow.body();
			// Restore all the overflow.
			if(has("ie") && !has("quirks")){
				body.parentNode.style.overflow = this._oldBodyParentOverflow;
				delete this._oldBodyParentOverflow;
			}
			domStyle.set(body, "overflow", this._oldOverflow);
			if(has("webkit")){
				//Gotta poke WebKit, or scrollers don't come back. :-(
				var div = construct.create("div", { style: {
						height: "2px"
					}
				});
				body.appendChild(div);
				setTimeout(function(){
					body.removeChild(div);
				}, 0);
			}
			delete this._oldOverflow;
		}
	}
});

});

},
'epi-cms/widget/_ContentListKeyMixin':function(){
﻿define("epi-cms/widget/_ContentListKeyMixin", [
    // Dojo
    "dojo/_base/declare",
    "dojo/keys"
],

function (
// Dojo
    declare,
    keys
) {

    return declare(null, {
        // summary:
        //    A Mixin to handle key event for ContentList.
        //
        // tags:
        //    internal mixin

        handleKey: function (evt) {
            // summary:
            //		Handle keystroke event forwarded from ComboBox, returning false if it's
            //		a keystroke I recognize and process, true otherwise.
            // tags:
            //      Protected

            switch (evt.keyCode) {
                case keys.DOWN_ARROW:
                    this.selectNextNode();
                    return false;
                case keys.UP_ARROW:
                    this.selectPreviousNode();
                    return false;
                case keys.ENTER:
                    // select item on pressing Enter
                    if (this.selected && this.grid.row(this.selected)) {
                        this._onSelect(this.grid.row(this.selected).data);
                    }
                    return false;
                default:
                    return true;
            }
        }
    });
});

},
'epi-cms/widget/_ContentListMouseMixin':function(){
﻿define("epi-cms/widget/_ContentListMouseMixin", [
    // Dojo
    "dojo/_base/declare",
    "dojo/dom-class"
],

function (
// Dojo
    declare,
    domClass) {

    return declare(null, {
        // summary:
        //    A Mixin to handle mouse event for ContentList.
        //
        // tags:
        //    internal mixin

        onHover: function (/*DomNode*/node) {
            // summary:
            //		Add hover CSS
        },

        onUnhover: function (/*DomNode*/node) {
            // summary:
            //		Remove hover CSS
        },

        onSelect: function (/*DomNode*/node) {
            // summary:
            //		Add selected CSS

            domClass.add(node, "dgrid-selected ui-state-active dgrid-focus");
        },

        onDeselect: function (/*DomNode*/node) {
            // summary:
            //		Remove selected CSS
            domClass.remove(node, "dgrid-selected ui-state-active dgrid-focus");
        }
    });
});

},
'dgrid/Keyboard':function(){
define("dgrid/Keyboard", [
	"dojo/_base/declare",
	"dojo/aspect",
	"dojo/on",
	"dojo/_base/lang",
	"dojo/has",
	"put-selector/put",
	"./util/misc",
	"dojo/_base/Deferred",
	"dojo/_base/sniff"
], function(declare, aspect, on, lang, has, put, miscUtil, Deferred){

var delegatingInputTypes = {
		checkbox: 1,
		radio: 1,
		button: 1
	},
	hasGridCellClass = /\bdgrid-cell\b/,
	hasGridRowClass = /\bdgrid-row\b/;

var Keyboard = declare(null, {
	// summary:
	//		Adds keyboard navigation capability to a list or grid.
	
	// pageSkip: Number
	//		Number of rows to jump by when page up or page down is pressed.
	pageSkip: 10,
	
	tabIndex: 0,
	
	// keyMap: Object
	//		Hash which maps key codes to functions to be executed (in the context
	//		of the instance) for key events within the grid's body.
	keyMap: null,
	
	// headerKeyMap: Object
	//		Hash which maps key codes to functions to be executed (in the context
	//		of the instance) for key events within the grid's header row.
	headerKeyMap: null,
	
	postMixInProperties: function(){
		this.inherited(arguments);
		
		if(!this.keyMap){
			this.keyMap = lang.mixin({}, Keyboard.defaultKeyMap);
		}
		if(!this.headerKeyMap){
			this.headerKeyMap = lang.mixin({}, Keyboard.defaultHeaderKeyMap);
		}
	},
	
	postCreate: function(){
		this.inherited(arguments);
		var grid = this;
		
		function handledEvent(event){
			// text boxes and other inputs that can use direction keys should be ignored and not affect cell/row navigation
			var target = event.target;
			return target.type && (!delegatingInputTypes[target.type] || event.keyCode == 32);
		}
		
		function enableNavigation(areaNode){
			var cellNavigation = grid.cellNavigation,
				isFocusableClass = cellNavigation ? hasGridCellClass : hasGridRowClass,
				isHeader = areaNode === grid.headerNode,
				initialNode = areaNode;
			
			function initHeader(){
				if(grid._focusedHeaderNode){
					// Remove the tab index for the node that previously had it.
					grid._focusedHeaderNode.tabIndex = -1;
				}
				if(grid.showHeader){
					if(cellNavigation){
						// Get the focused element. Ensure that the focused element
						// is actually a grid cell, not a column-set-cell or some
						// other cell that should not be focused
						for(var i = 0, element, elements = grid.headerNode.getElementsByTagName("th"); (element = elements[i]); ++i){
							if(isFocusableClass.test(element.className)){
								grid._focusedHeaderNode = initialNode = element;
								break;
							}
						}
					}
					else{
						grid._focusedHeaderNode = initialNode = grid.headerNode;
					}

					// Set the tab index only if the header is visible.
					if(initialNode){
						initialNode.tabIndex = grid.tabIndex;
					}
				}
			}
			
			if(isHeader){
				// Initialize header now (since it's already been rendered),
				// and aspect after future renderHeader calls to reset focus.
				initHeader();
				aspect.after(grid, "renderHeader", initHeader, true);
			}else{
				aspect.after(grid, "renderArray", function(ret){
					// summary:
					//		Ensures the first element of a grid is always keyboard selectable after data has been
					//		retrieved if there is not already a valid focused element.
					
					return Deferred.when(ret, function(ret){
						var focusedNode = grid._focusedNode || initialNode;
						
						// do not update the focused element if we already have a valid one
						if(isFocusableClass.test(focusedNode.className) && miscUtil.contains(areaNode, focusedNode)){
							return ret;
						}
						
						// ensure that the focused element is actually a grid cell, not a
						// dgrid-preload or dgrid-content element, which should not be focusable,
						// even when data is loaded asynchronously
						for(var i = 0, elements = areaNode.getElementsByTagName("*"), element; (element = elements[i]); ++i){
							if(isFocusableClass.test(element.className)){
								focusedNode = grid._focusedNode = element;
								break;
							}
						}
						
						focusedNode.tabIndex = grid.tabIndex;
						return ret;
					});
				});
			}
			
			grid._listeners.push(on(areaNode, "mousedown", function(event){
				if(!handledEvent(event)){
					grid._focusOnNode(event.target, isHeader, event);
				}
			}));
			
			grid._listeners.push(on(areaNode, "keydown", function(event){
				// For now, don't squash browser-specific functionalities by letting
				// ALT and META function as they would natively
				if(event.metaKey || event.altKey) {
					return;
				}
				
				var handler = grid[isHeader ? "headerKeyMap" : "keyMap"][event.keyCode];
				
				// Text boxes and other inputs that can use direction keys should be ignored and not affect cell/row navigation
				if(handler && !handledEvent(event)){
					handler.call(grid, event);
				}
			}));
		}
		
		if(this.tabableHeader){
			enableNavigation(this.headerNode);
			on(this.headerNode, "dgrid-cellfocusin", function(){
				grid.scrollTo({ x: this.scrollLeft });
			});
		}
		enableNavigation(this.contentNode);
		
		this._debouncedEnsureRowScroll = miscUtil.debounce(this._ensureRowScroll, this);
	},
	
	removeRow: function(rowElement){
		if(!this._focusedNode){
			// Nothing special to do if we have no record of anything focused
			return this.inherited(arguments);
		}
		
		var self = this,
			isActive = document.activeElement === this._focusedNode,
			focusedTarget = this[this.cellNavigation ? "cell" : "row"](this._focusedNode),
			focusedRow = focusedTarget.row || focusedTarget,
			sibling;
		rowElement = rowElement.element || rowElement;
		
		// If removed row previously had focus, temporarily store information
		// to be handled in an immediately-following insertRow call, or next turn
		if(rowElement === focusedRow.element){
			sibling = this.down(focusedRow, true);
			
			// Check whether down call returned the same row, or failed to return
			// any (e.g. during a partial unrendering)
			if (!sibling || sibling.element === rowElement) {
				sibling = this.up(focusedRow, true);
			}
			
			this._removedFocus = {
				active: isActive,
				rowId: focusedRow.id,
				columnId: focusedTarget.column && focusedTarget.column.id,
				siblingId: !sibling || sibling.element === rowElement ? undefined : sibling.id
			};
			
			// Call _restoreFocus on next turn, to restore focus to sibling
			// if no replacement row was immediately inserted.
			// Pass original row's id in case it was re-inserted in a renderArray
			// call (and thus was found, but couldn't be focused immediately)
			setTimeout(function() {
				if(self._removedFocus){
					self._restoreFocus(focusedRow.id);
				}
			}, 0);
			
			// Clear _focusedNode until _restoreFocus is called, to avoid
			// needlessly re-running this logic
			this._focusedNode = null;
		}
		
		this.inherited(arguments);
	},
	
	insertRow: function(object){
		var rowElement = this.inherited(arguments);
		if(this._removedFocus && !this._removedFocus.wait){
			this._restoreFocus(rowElement);
		}
		return rowElement;
	},
	
	_restoreFocus: function(row) {
		// summary:
		//		Restores focus to the newly inserted row if it matches the
		//		previously removed row, or to the nearest sibling otherwise.
		
		var focusInfo = this._removedFocus,
			newTarget,
			cell;
		
		row = row && this.row(row);
		newTarget = row && row.element && row.id === focusInfo.rowId ? row :
			typeof focusInfo.siblingId !== "undefined" && this.row(focusInfo.siblingId);
		
		if(newTarget && newTarget.element){
			if(!newTarget.element.parentNode.parentNode){
				// This was called from renderArray, so the row hasn't
				// actually been placed in the DOM yet; handle it on the next
				// turn (called from removeRow).
				focusInfo.wait = true;
				return;
			}
			// Should focus be on a cell?
			if(typeof focusInfo.columnId !== "undefined"){
				cell = this.cell(newTarget, focusInfo.columnId);
				if(cell && cell.element){
					newTarget = cell;
				}
			}
			if(focusInfo.active && newTarget.element.offsetHeight !== 0){
				// Row/cell was previously focused and is visible, so focus the new one immediately
				this._focusOnNode(newTarget, false, null);
			}else{
				// Row/cell was not focused or is not visible, but we still need to
				// update _focusedNode and the element's tabIndex/class
				put(newTarget.element, ".dgrid-focus");
				newTarget.element.tabIndex = this.tabIndex;
				this._focusedNode = newTarget.element;
			}
		}
		
		delete this._removedFocus;
	},
	
	addKeyHandler: function(key, callback, isHeader){
		// summary:
		//		Adds a handler to the keyMap on the instance.
		//		Supports binding additional handlers to already-mapped keys.
		// key: Number
		//		Key code representing the key to be handled.
		// callback: Function
		//		Callback to be executed (in instance context) when the key is pressed.
		// isHeader: Boolean
		//		Whether the handler is to be added for the grid body (false, default)
		//		or the header (true).
		
		// Aspects may be about 10% slower than using an array-based appraoch,
		// but there is significantly less code involved (here and above).
		return aspect.after( // Handle
			this[isHeader ? "headerKeyMap" : "keyMap"], key, callback, true);
	},
	
	_ensureRowScroll: function(rowElement){
		// summary:
		//		Ensures that the entire row is visible within the viewport.
		//		Called for cell navigation in complex structures.

		var scrollY = this.getScrollPosition().y;
		if(scrollY > rowElement.offsetTop){
			// Row starts above the viewport
			this.scrollTo({ y: rowElement.offsetTop });
		}
		else if(scrollY + this.contentNode.offsetHeight < rowElement.offsetTop + rowElement.offsetHeight){
			// Row ends below the viewport
			this.scrollTo({ y: rowElement.offsetTop - this.contentNode.offsetHeight + rowElement.offsetHeight });
		}
	},
	
	_focusOnNode: function(element, isHeader, event){
		var focusedNodeProperty = "_focused" + (isHeader ? "Header" : "") + "Node",
			focusedNode = this[focusedNodeProperty],
			cellOrRowType = this.cellNavigation ? "cell" : "row",
			cell = this[cellOrRowType](element),
			inputs,
			input,
			numInputs,
			inputFocused,
			i;
		
		element = cell && cell.element;
		if(!element){ return; }
		
		if(this.cellNavigation){
			inputs = element.getElementsByTagName("input");
			for(i = 0, numInputs = inputs.length; i < numInputs; i++){
				input = inputs[i];
				if((input.tabIndex != -1 || "_dgridLastValue" in input) && !input.disabled){
					// Employ workaround for focus rectangle in IE < 8
					if(has("ie") < 8){ input.style.position = "relative"; }
					input.focus();
					if(has("ie") < 8){ input.style.position = ""; }
					inputFocused = true;
					break;
				}
			}
		}
		
		// Set up event information for dgrid-cellfocusout/in events.
		// Note that these events are not fired for _restoreFocus.
		if(event !== null){
			event = lang.mixin({ grid: this }, event);
			if(event.type){
				event.parentType = event.type;
			}
			if(!event.bubbles){
				// IE doesn't always have a bubbles property already true.
				// Opera throws if you try to set it to true if it is already true.
				event.bubbles = true;
			}
		}
		
		if(focusedNode){
			// Clean up previously-focused element
			// Remove the class name and the tabIndex attribute
			put(focusedNode, "!dgrid-focus[!tabIndex]");
			if(has("ie") < 8){
				// Clean up after workaround below (for non-input cases)
				focusedNode.style.position = "";
			}
			
			// Expose object representing focused cell or row losing focus, via
			// event.cell or event.row; which is set depends on cellNavigation.
			if(event){
				event[cellOrRowType] = this[cellOrRowType](focusedNode);
				on.emit(focusedNode, "dgrid-cellfocusout", event);
			}
		}
		focusedNode = this[focusedNodeProperty] = element;
		
		if(event){
			// Expose object representing focused cell or row gaining focus, via
			// event.cell or event.row; which is set depends on cellNavigation.
			// Note that yes, the same event object is being reused; on.emit
			// performs a shallow copy of properties into a new event object.
			event[cellOrRowType] = cell;
		}
		
		var isFocusableClass = this.cellNavigation ? hasGridCellClass : hasGridRowClass;
		if(!inputFocused && isFocusableClass.test(element.className)){
			if(has("ie") < 8){
				// setting the position to relative magically makes the outline
				// work properly for focusing later on with old IE.
				// (can't be done a priori with CSS or screws up the entire table)
				element.style.position = "relative";
			}
			element.tabIndex = this.tabIndex;
			element.focus();
		}
		put(element, ".dgrid-focus");
		
		if(event){
			on.emit(focusedNode, "dgrid-cellfocusin", event);
		}
		
		if(this.cellNavigation && (this.columnSets || this.subRows.length > 1) && !isHeader){
			this._debouncedEnsureRowScroll(cell.row.element);
		}
	},
	
	focusHeader: function(element){
		this._focusOnNode(element || this._focusedHeaderNode, true);
	},
	
	focus: function(element){
		var node = element || this._focusedNode;
		if(node){
			this._focusOnNode(node, false);
		}else{
			this.contentNode.focus();
		}
	}
});

// Common functions used in default keyMap (called in instance context)

var moveFocusVertical = Keyboard.moveFocusVertical = function(event, steps){
	var cellNavigation = this.cellNavigation,
		target = this[cellNavigation ? "cell" : "row"](event),
		columnId = cellNavigation && target.column.id,
		next = this.down(this._focusedNode, steps, true);
	
	// Navigate within same column if cell navigation is enabled
	if(cellNavigation){ next = this.cell(next, columnId); }
	this._focusOnNode(next, false, event);
	
	event.preventDefault();
};

var moveFocusUp = Keyboard.moveFocusUp = function(event){
	moveFocusVertical.call(this, event, -1);
};

var moveFocusDown = Keyboard.moveFocusDown = function(event){
	moveFocusVertical.call(this, event, 1);
};

var moveFocusPageUp = Keyboard.moveFocusPageUp = function(event){
	moveFocusVertical.call(this, event, -this.pageSkip);
};

var moveFocusPageDown = Keyboard.moveFocusPageDown = function(event){
	moveFocusVertical.call(this, event, this.pageSkip);
};

var moveFocusHorizontal = Keyboard.moveFocusHorizontal = function(event, steps){
	if(!this.cellNavigation){ return; }
	var isHeader = !this.row(event), // header reports row as undefined
		currentNode = this["_focused" + (isHeader ? "Header" : "") + "Node"];
	
	this._focusOnNode(this.right(currentNode, steps), isHeader, event);
	event.preventDefault();
};

var moveFocusLeft = Keyboard.moveFocusLeft = function(event){
	moveFocusHorizontal.call(this, event, -1);
};

var moveFocusRight = Keyboard.moveFocusRight = function(event){
	moveFocusHorizontal.call(this, event, 1);
};

var moveHeaderFocusEnd = Keyboard.moveHeaderFocusEnd = function(event, scrollToBeginning){
	// Header case is always simple, since all rows/cells are present
	var nodes;
	if(this.cellNavigation){
		nodes = this.headerNode.getElementsByTagName("th");
		this._focusOnNode(nodes[scrollToBeginning ? 0 : nodes.length - 1], true, event);
	}
	// In row-navigation mode, there's nothing to do - only one row in header
	
	// Prevent browser from scrolling entire page
	event.preventDefault();
};

var moveHeaderFocusHome = Keyboard.moveHeaderFocusHome = function(event){
	moveHeaderFocusEnd.call(this, event, true);
};

var moveFocusEnd = Keyboard.moveFocusEnd = function(event, scrollToTop){
	// summary:
	//		Handles requests to scroll to the beginning or end of the grid.
	
	// Assume scrolling to top unless event is specifically for End key
	var self = this,
		cellNavigation = this.cellNavigation,
		contentNode = this.contentNode,
		contentPos = scrollToTop ? 0 : contentNode.scrollHeight,
		scrollPos = contentNode.scrollTop + contentPos,
		endChild = contentNode[scrollToTop ? "firstChild" : "lastChild"],
		hasPreload = endChild.className.indexOf("dgrid-preload") > -1,
		endTarget = hasPreload ? endChild[(scrollToTop ? "next" : "previous") + "Sibling"] : endChild,
		endPos = endTarget.offsetTop + (scrollToTop ? 0 : endTarget.offsetHeight),
		handle;
	
	if(hasPreload){
		// Find the nearest dgrid-row to the relevant end of the grid
		while(endTarget && endTarget.className.indexOf("dgrid-row") < 0){
			endTarget = endTarget[(scrollToTop ? "next" : "previous") + "Sibling"];
		}
		// If none is found, there are no rows, and nothing to navigate
		if(!endTarget){ return; }
	}
	
	// Grid content may be lazy-loaded, so check if content needs to be
	// loaded first
	if(!hasPreload || endChild.offsetHeight < 1){
		// End row is loaded; focus the first/last row/cell now
		if(cellNavigation){
			// Preserve column that was currently focused
			endTarget = this.cell(endTarget, this.cell(event).column.id);
		}
		this._focusOnNode(endTarget, false, event);
	}else{
		// In IE < 9, the event member references will become invalid by the time
		// _focusOnNode is called, so make a (shallow) copy up-front
		if(!has("dom-addeventlistener")){
			event = lang.mixin({}, event);
		}
		
		// If the topmost/bottommost row rendered doesn't reach the top/bottom of
		// the contentNode, we are using OnDemandList and need to wait for more
		// data to render, then focus the first/last row in the new content.
		handle = aspect.after(this, "renderArray", function(rows){
			handle.remove();
			return Deferred.when(rows, function(rows){
				var target = rows[scrollToTop ? 0 : rows.length - 1];
				if(cellNavigation){
					// Preserve column that was currently focused
					target = self.cell(target, self.cell(event).column.id);
				}
				self._focusOnNode(target, false, event);
			});
		});
	}
	
	if(scrollPos === endPos){
		// Grid body is already scrolled to end; prevent browser from scrolling
		// entire page instead
		event.preventDefault();
	}
};

var moveFocusHome = Keyboard.moveFocusHome = function(event){
	moveFocusEnd.call(this, event, true);
};

function preventDefault(event){
	event.preventDefault();
}

Keyboard.defaultKeyMap = {
	32: preventDefault, // space
	33: moveFocusPageUp, // page up
	34: moveFocusPageDown, // page down
	35: moveFocusEnd, // end
	36: moveFocusHome, // home
	37: moveFocusLeft, // left
	38: moveFocusUp, // up
	39: moveFocusRight, // right
	40: moveFocusDown // down
};

// Header needs fewer default bindings (no vertical), so bind it separately
Keyboard.defaultHeaderKeyMap = {
	32: preventDefault, // space
	35: moveHeaderFocusEnd, // end
	36: moveHeaderFocusHome, // home
	37: moveFocusLeft, // left
	39: moveFocusRight // right
};

return Keyboard;
});
},
'url:epi-cms/widget/templates/SearchResultList.html':"﻿<div class=\"epi-searchResultList\">\n    <div data-dojo-attach-point=\"containerNode\">\n        <div data-dojo-attach-point=\"errorMessage\" class=\"epi-searchResultListEmptyMessage\"></div>\n    </div>\n    <div class=\"dijitReset epi-searchResultListNavigation\">\n        <div data-dojo-attach-point=\"previousButton\" role=\"option\"></div>\n        <div data-dojo-attach-point=\"nextButton\" role=\"option\"></div>\n    </div>\n</div>",
'epi-cms/widget/ContentSelector':function(){
define("epi-cms/widget/ContentSelector", [
// dojo
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-class",
    "dojo/dom-style",
    "dojo/dom-construct",
    "dojo/string",
    "dojo/when",

    // dijit
    "dijit/focus",
    "dijit/form/Button",

    // epi.shell
    "epi/dependency",
    "epi/shell/TypeDescriptorManager",
    "epi/shell/widget/dialog/Dialog",

    // epi.cms
    "epi-cms/core/ContentReference",
    "epi-cms/contentediting/ContentActionSupport",
    "epi-cms/widget/ContentSelectorDialog",
    "epi-cms/widget/_SelectorBase",

    // resources

    "dojo/i18n!epi/nls/episerver.cms.widget.contentselector"
],

function (
// dojo
    array,
    declare,
    lang,
    domClass,
    domStyle,
    domConstruct,
    stringUtil,
    when,

    focusManager,
    Button,

    // epi.shell
    dependency,
    TypeDescriptorManager,
    Dialog,

    // epi.cms
    ContentReference,
    ContentActionSupport,
    ContentSelectorDialog,
    _SelectorBase,

    // resources

    localization
) {

    return declare([_SelectorBase], {
        // summary:
        //    Represents the widget to select ContentReference.
        // tags:
        //      internal xproduct

        roots: null,

        value: null,

        // required: Boolean
        //		User is required to enter data into this field.
        required: false,

        // missingMessage: [public] String
        //    Message which is displayed when required is true and value is empty.
        missingMessage: localization.requiredmessage,

        // tooltipPosition: String[]
        //		See description of `dijit/Tooltip.defaultPosition` for details on this parameter.
        tooltipPosition: [],

        // contentLink: [public] String
        //      Content reference of the content being edited.
        contentLink: null,

        // previewUrl: [internal] String
        //      The preview url to the selected content
        previewUrl: null,

        // canSelectOwnerContent: [public] Boolean
        //      Indicates whether the owner content that is being edited can be selected
        canSelectOwnerContent: false,

        // accessLevel: [public] Enum
        //      The access right level for the content node can be selected
        accessLevel: ContentActionSupport.accessLevel.Read,

        // showAllLanguages: Boolean
        //      Flags to indicate that the content tree should show all content in multiple languages or not.
        showAllLanguages: true,

        // showSearchBox: Boolean
        //      If true - a search field is displayed that allows user to find content
        showSearchBox: true,

        // searchArea: String
        //      Search area for the search box.
        searchArea: null,

        _setDisabledAttr: function (value) {
            this.inherited(arguments);
            this.button.set("disabled", value);
            this._set("disabled", value);
        },

        buildRendering: function () {
            // summary:
            //		Creates and append the input hidden field to the dom to preserve the contentLink id
            // tags:
            //    protected

            var inputName = this.name ? (this.name.replace(/"/g, "&quot;")) : "";
            this.input = domConstruct.create("input", { type: "hidden", name: inputName });

            this.inherited(arguments);
            this.domNode.appendChild(this.input);
        },

        postMixInProperties: function () {
            // summary:
            //		Initialize properties
            // tags:
            //    protected

            this.inherited(arguments);

            var registry = dependency.resolve("epi.storeregistry");
            this._store = registry.get("epi.cms.content.light");

            var contentRepositoryDescriptors = this.contentRepositoryDescriptors || dependency.resolve("epi.cms.contentRepositoryDescriptors");
            var settings = contentRepositoryDescriptors.get(this.repositoryKey);

            if (settings &&  !this.roots) {
                this.roots = settings.roots;
            }

            if (settings && !this.searchArea) {
                this.searchArea = settings.searchArea;
            }
        },

        postCreate: function () {
            // summary:
            //      Overridden to initialize ui elements when no value is set.

            this.inherited(arguments);

            if (!this.value) {
                this._updateDisplayNode(null);
            }
        },

        _setReadOnlyAttr: function (value) {
            this._set("readOnly", value);

            domStyle.set(this.button.domNode, "display", value ? "none" : "");
            domStyle.set(this.clearButton, "display", value ? "none" : "");
            domClass.toggle(this.domNode, "dijitReadOnly", value);
        },

        _setValueAttr: function (value) {
            //summary:
            //    Value's setter.
            //
            // value: String
            //    Value to be set.
            //
            // tags:
            //    protected

            this._setValueAndFireOnChange(value);
        },

        _getDialog: function () {
            // summary:
            //		Create page tree dialog
            // tags:
            //    protected

            // Verifies that the dialog instance and its dom node existing or not
            if (this.dialog && this.dialog.domNode) {
                return this.dialog;
            }

            var title = localization.title;
            if (this.allowedTypes && this.allowedTypes.length === 1) {
                var name = TypeDescriptorManager.getResourceValue(this.allowedTypes[0], "name");
                if (name) {
                    title = stringUtil.substitute(localization.format, [name]);
                }
            }

            this.contentSelectorDialog = new ContentSelectorDialog({
                canSelectOwnerContent: this.canSelectOwnerContent,
                showButtons: false,
                roots: this.roots,
                allowedTypes: this.allowedTypes,
                restrictedTypes: this.restrictedTypes,
                showAllLanguages: this.showAllLanguages,
                showSearchBox: this.showSearchBox,
                searchArea: this.searchArea
            });

            this.dialog = new Dialog({
                title: title,
                dialogClass: "epi-dialog-portrait",
                content: this.contentSelectorDialog
            });

            this.dialog.own(this.contentSelectorDialog);

            this.connect(this.contentSelectorDialog, "onChange", "_setDialogButtonState");
            this.connect(this.dialog, "onExecute", "_onDialogExecute");
            this.connect(this.dialog, "onHide", "_onDialogHide");

            return this.dialog;
        },

        _isContentAllowed: function (contentTypeIdentifier) {
            // summary:
            //    Checks whether the given content type belongs to the any of the allowedTypes/parent type or not
            //
            // contentTypeIdentifier: String
            //    The content type identifier
            //
            // tags:
            //    Private

            var acceptedTypes = TypeDescriptorManager.getValidAcceptedTypes([contentTypeIdentifier], this.allowedTypes, this.restrictedTypes);

            return !!acceptedTypes.length;
        },

        _onButtonClick: function () {
            // summary:
            //    Handle pick button click
            // tags:
            //    private

            when(this._getContentData(this.contentLink), lang.hitch(this, function (content) {

                // if the current content is any of allowedTypes then allow to select it
                this.canSelectOwnerContent = this.canSelectOwnerContent && content && this._isContentAllowed(content.typeIdentifier);
                var dialog = this._getDialog();
                this.isShowingChildDialog = true;

                var valueAsContentReference;
                if (this.value) {
                    valueAsContentReference = this._convertValueToContentReference(this.value);
                }

                if (valueAsContentReference) {
                    valueAsContentReference = valueAsContentReference.createVersionUnspecificReference().toString();
                }

                this.contentSelectorDialog.set("value", valueAsContentReference);
                dialog.show();
            }));
        },

        _onDialogExecute: function () {
            //summary:
            //    Handle dialog close
            // tags:
            //    private

            var value = this.contentSelectorDialog.get("value");
            this._setValueAndFireOnChange(value);
        },

        _convertValueToContentReference: function (value) {
            //summary:
            //
            // tags:
            //    private
            return new ContentReference(value);
        },

        _setValueAndFireOnChange: function (value) {
            //summary:
            //    Gets the content data by selected value, updates the view and calls onChange if value was changed
            // tags:
            //    private

            var contentLink = value === "-" ? this.contentLink : value;

            this.set("permanentLink", null);
            this.set("previewUrl", null);

            this._valueChangedPromise = when(this._getContentData(contentLink), lang.hitch(this, function (content) {

                // Clear the pointer to the promise since it is resolved.
                this._valueChangedPromise = null;

                var hasChange = this.value !== contentLink;

                this.value = contentLink;
                this.input.value = this.value;
                this._started && this.validate();
                this._updateDisplayNode(content);

                if (hasChange) {
                    this.onChange(contentLink);
                }

                if (content) {
                    this.set("permanentLink", content.permanentLink);
                    this.set("previewUrl", content.previewUrl);
                }
            }));
        },

        _getContentData: function (contentLink) {
            //summary:
            //    Loads the content data from the store
            // tags:
            //    private

            if (!contentLink) {
                return null;
            }
            return this._store.get(contentLink);
        },

        _onFocus: function () {
            // summary:
            //		This is where widgets do processing for when they start being active,
            //		such as changing CSS classes.  See onFocus() for more details.
            // tags:
            //		protected

            if (this.get("disabled")) {
                return;
            }
            this.inherited(arguments);
            this.validate();
        },


        _onBlur: function () {
            // summary:
            //		This is where widgets do processing for when they stop being active,
            //		such as changing CSS classes.  See onBlur() for more details.
            // tags:
            //		protected

            if (this.get("disabled")) {
                return;
            }
            this.inherited(arguments);
            this.validate();
        },

        focus: function () {
            // summary:
            //       Put focus on this widget
            // tags:
            //      public

            if (!this.button.disabled && this.button.focusNode && this.button.focusNode.focus) {
                try {
                    focusManager.focus(this.button.focusNode);
                } catch (e) {
                    /*squelch errors from hidden nodes*/
                }
            }
        },

        _updateDisplayNode: function (content) {
            //summary:
            //    Update widget's display text
            // tags:
            //    protected

            this.inherited(arguments);

            // content selector also sets Link id
            if (content) {
                this.set("selectedContentLink", new ContentReference(content.contentLink).id);
            } else {
                this.set("selectedContentName", localization.helptext);
                this.set("selectedContentLink", "");
            }
        },

        reset: function () {
            this.set("value", null);
        },

        _setDialogButtonState: function (contentLink) {
            // summary:
            //    Set state of dialog button.
            //
            // tags:
            //    protected

            var self = this;

            if (!contentLink) {
                return;
            }

            when(self._store.get(contentLink), function (content) {
                self.dialog.definitionConsumer.setItemProperty(self.dialog._okButtonName, "disabled", !ContentActionSupport.hasAccess(content.accessMask, self.accessLevel));
            });
        }
    });
});

},
'epi-cms/widget/ContentSelectorDialog':function(){
define("epi-cms/widget/ContentSelectorDialog", [
// Dojo
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-geometry",
    "dojo/when",

    // Dijit
    "dijit/layout/_LayoutWidget",

    // EPi
    "epi",
    "epi/shell/TypeDescriptorManager",

    "epi-cms/ApplicationSettings",
    "epi-cms/widget/ContentTree",
    "epi-cms/widget/ContextualContentForestStoreModel",
    "epi-cms/widget/SearchBox",

    // EPi Framework
    "epi/shell/widget/_ActionProviderWidget",
    "epi/dependency",

    // Resources
    "dojo/i18n!epi/nls/episerver.cms.widget.contentselectordialog"
],

function (
// Dojo
    array,
    declare,
    lang,
    domGeometry,
    when,

    // Dijit
    _LayoutWidget,

    // EPi
    epi,
    TypeDescriptorManager,

    ApplicationSettings,
    ContentTree,
    ContextualContentForestStoreModel,
    SearchBox,

    // EPi Framework
    _ActionProviderWidget,
    dependency,

    // Resources
    localization
) {

    return declare([_LayoutWidget, _ActionProviderWidget], {
        // summary:
        //    Content selector widget.
        //
        // description:
        //    Used for editing PropertyPage properties in fly-out editor.
        //
        // tags:
        //    internal xproduct

        // _contentRef: [private] epi-cms/core/ContentReference
        //    Reference to the currently selected content.
        _contentRef: null,

        // res: [private] Object
        //    Resource bundle.
        res: localization,

        baseClass: "epi-content-selector-dialog",

        // roots: [public] Array
        //    A list of references to the root contents.
        roots: null,

        // typeIdentifiers: [public] Array | String
        //    Type identifiers to filter.
        typeIdentifiers: null,

        showButtons: true,

        model: null,

        // allowedTypes: String[]
        //      An array of types that should be selectable
        allowedTypes: null,

        // restrictedTypes: String[]
        //      An array of types that not should be selectable
        restrictedTypes: null,

        // canSelectOwnerContent: [private] Boolean
        //    Indicates whether select current owner content button should be available.
        canSelectOwnerContent: true,

        // multiRootsMode: [public] Boolean
        //      Indicate that multiple roots enabled in the tree or not
        multiRootsMode: false,

        _typesToDisplay: null,

        // showAllLanguages: Boolean
        //      Flags to indicate that the content tree should show all content in multiple languages or not.
        showAllLanguages: false,

        // selectedContentType: String
        //      The content type which is selected. (used when restore content for instance)
        selectedContentType: null,

        // showSearchBox: Boolean
        //      If true - a search field is displayed that allows user to find content
        showSearchBox: true,

        // searchArea: String
        //      Search area for the search box.
        searchArea: null,

        _searchBox: null,

        buildRendering: function () {
            this.inherited(arguments);

            var typeIdentifiers = this._getTypesToDisplay(),
                model = this.model,
                roots = this.roots || (model && model.roots);

            this.preventContextualContentFor = this.preventContextualContentFor || this._getSettingFromContentType();
            if (!model || this.showContextualContent) {
                model = new ContextualContentForestStoreModel({
                    roots: roots,
                    typeIdentifiers: typeIdentifiers,
                    notSupportContextualContents: this.preventContextualContentFor,
                    showAllLanguages: this.showAllLanguages
                });
            }

            model.set("view", "contentselector");

            if (this.showSearchBox && this.searchArea) {
                this._searchBox = new SearchBox({
                    innerSearchBoxClass: "epi-search--full-width",
                    triggerContextChange: false,
                    parameters: {
                        allowedTypes: this.allowedTypes,
                        restrictedTypes: this.restrictedTypes
                    },
                    onItemAction: lang.hitch(this, function (item) {
                        if (item && item.metadata && this._checkAcceptance(item.metadata.typeIdentifier)) {
                            this.set("value", item.metadata.id);
                        }
                    })
                });
                this._searchBox.set("area", this.searchArea);
                this._searchBox.set("searchRoots", this.roots);

                var searchBox = this._searchBox;
                model.getRoots(true).then(function (roots) {
                    searchBox.set("searchRoots", roots.join(","));
                });

                this.addChild(this._searchBox);
            }

            this.tree = new ContentTree({
                roots: roots,
                typeIdentifiers: typeIdentifiers,
                allowManipulation: false,
                model: model,
                allowedTypes: lang.clone(this.allowedTypes),
                restrictedTypes: lang.clone(this.restrictedTypes),
                disableRestrictedTypes: true,
                expandExtraNodeItems: ApplicationSettings.startPage // Set extra node items to expand when the given tree loaded
            });
            this.addChild(this.tree);

            this.connect(this.tree, "onClick", "_onTreeNodeClick");
        },

        layout: function () {
            this.inherited(arguments);

            var treeHeight = this._contentBox.h;

            //Subtract the height of the searchBox if it is created
            if (this._searchBox) {
                treeHeight -= domGeometry.getMarginBox(this._searchBox.domNode).h;
            }

            //Get the margin box for the tree and change the height
            var treeMarginBox = domGeometry.getMarginBox(this.tree.domNode);
            treeMarginBox.h = treeHeight;

            domGeometry.setMarginBox(this.tree.domNode, treeMarginBox);
        },

        _getSettingFromContentType: function () {
            // summary:
            //      Get preventContextualContentFor settings form selected content type
            // tags:
            //      private

            if (!this.selectedContentType) {
                return null;
            }
            var contentRepositoryDescriptors = this.contentRepositoryDescriptors || dependency.resolve("epi.cms.contentRepositoryDescriptors");

            var repositoryDescriptor,
                matchType = function (type) {
                    return TypeDescriptorManager.isBaseTypeIdentifier(this.selectedContentType, type);
                };
            for (var index in contentRepositoryDescriptors) {
                var descriptor = contentRepositoryDescriptors[index];
                if (array.some(descriptor.containedTypes, matchType, this)) {
                    repositoryDescriptor = descriptor;
                    break;
                }
            }

            return repositoryDescriptor.preventContextualContentFor;
        },

        _getTypesToDisplay: function () {
            if (!this._typesToDisplay) {

                var typesToDisplay = [];
                this._getContainerTypesRecursive(this.allowedTypes, typesToDisplay);
                this._typesToDisplay = typesToDisplay;
            }

            return this._typesToDisplay;
        },

        _getContainerTypesRecursive: function (types, results, checkedTypes) {
            if (!checkedTypes) {
                checkedTypes = [];
            }

            array.forEach(types, lang.hitch(this, function (type) {
                // To avoid infinite recursion, check if this type is already processed
                if (array.indexOf(checkedTypes, type) === -1) {
                    // Mark as checked
                    checkedTypes.push(type);

                    // Add the type itself if it isn't added already
                    if (array.indexOf(results, type) === -1) {
                        results.push(type);
                    }

                    // If there are any container types, process them as well
                    var containerTypesForType = TypeDescriptorManager.getValue(type, "containerTypes");

                    if (containerTypesForType) {
                        this._getContainerTypesRecursive(containerTypesForType, results, checkedTypes);
                    }
                }
            }));
        },

        _setValueAttr: function (value) {
            //summary:
            //    Value's setter.
            //
            // value: String
            //    Value to be set.
            //
            // tags:
            //    protected

            //construct ContentReference object
            if (value) {
                this._contentRef = value;
                var contentLinkToSelect = this._contentRef === "-" ? null : this._contentRef;

                if (this.multiRootsMode) {
                    // We need to wait until tree load top level children and then fire onLoad() event
                    when(this.tree.onLoadDeferred, lang.hitch(this, function () {
                        this.tree.selectContent(contentLinkToSelect, true);
                    }));
                } else {
                    if (!epi.isEmpty(contentLinkToSelect)) {
                        this.tree.selectContent(contentLinkToSelect, true);
                    }
                }
            } else {
                this._contentRef = null;
                this._clearSelectedContent();
            }
        },

        _getValueAttr: function () {
            //summary:
            //    Value's getter
            // tags:
            //    protected

            this.inherited(arguments);

            if (this._contentRef) {
                return this._contentRef.toString();
            } else {
                return "";
            }
        },

        _setAllowedTypesAttr: function (value) {
            this._typesToDisplay = null;
            this._set("allowedTypes", value);
        },

        _clearSelectedContent: function () {
            // summary:
            //      Clear selected node(s) in tree
            // tags:
            //      private

            this.tree.set("selectedNodes", []);
            this.tree.set("selectedItems", []);
        },

        getActions: function () {
            // summary:
            //      Overridden from _ActionProvider to get the select current owner content action added to the containing widget
            //
            // returns:
            //      An array containing a select page action definition, if it is not a shared block

            return this.canSelectOwnerContent ? [
                {
                    name: "selectpage",
                    label: localization.currentpage,
                    settings: { type: "submit" },
                    action: lang.hitch(this, function () {
                        this._setValueOwnerContent();
                    })
                }
            ]
                : [];
        },

        _onTreeNodeClick: function (content) {
            // summary:
            //    Handle the inner tree's content change event.
            //
            // content: Object
            //    The content object.
            //
            // tags:
            //    private

            var value = content.contentLink;

            if (!this._checkAcceptance(content.typeIdentifier)) {
                this.tree.lastFocused.setSelected(false);
                value = null;
            }

            this.set("value", value);
            this.onChange(this.get("value"));
        },

        _checkAcceptance: function (typeIdentifier) {
            // summary:
            //    Compares a type against arrays of allowed and restricted types
            //
            // typeIdentifier: String
            //    The type to check if it's accepted to use
            //
            // tags:
            //    protected

            var acceptedTypes = TypeDescriptorManager.getValidAcceptedTypes([typeIdentifier], this.allowedTypes, this.restrictedTypes);

            return !!acceptedTypes.length;

        },

        _setValueOwnerContent: function () {
            this.set("value", "-");
        },

        onChange: function (contentLink) {
            // summary:
            //    Fired when value is changed.
            //
            // contentLink:
            //    The content link
            // tags:
            //    public, callback
        },

        focus: function () {
            this.tree.focus();
        }

    });

});

},
'epi-cms/widget/ContentTree':function(){
define("epi-cms/widget/ContentTree", [
// dojo
    "dojo/_base/array",
    "dojo/_base/connect", // To use isCopyKey()
    "dojo/_base/declare",
    "dojo/_base/lang",

    "dojo/dom-class",
    "dojo/dom-style",

    "dojo/aspect",
    "dojo/Deferred",
    "dojo/Evented",

    "dojo/keys",
    "dojo/promise/all",
    "dojo/query",
    "dojo/sniff",
    "dojo/when",
    "dijit/registry",
    // epi
    "epi/dependency",
    "epi/shell/DestroyableByKey",
    "epi/shell/dnd/tree/dndSource",
    "epi/shell/TypeDescriptorManager",
    "epi/shell/widget/Tree",

    "epi-cms/core/ContentReference",

    "epi-cms/widget/_ContentTreeNode",
    "epi-cms/widget/ContentTreeModelConfirmation",
    "epi-cms/widget/ContentForestStoreModel",
    "dojo/i18n!epi/nls/episerver.shared.header"
],

function (
// dojo
    array,
    connect, // To use isCopyKey()
    declare,
    lang,

    domClass,
    domStyle,

    aspect,
    Deferred,
    Evented,

    keys,
    promiseAll,
    query,
    sniff,
    when,
    registry,
    // epi
    dependency,
    DestroyableByKey,
    dndSource,
    TypeDescriptorManager,
    Tree,

    ContentReference,

    _ContentTreeNode,
    ContentTreeModelConfirmation,
    ContentForestStoreModel,
    headingResources
) {

    return declare([Tree, Evented, DestroyableByKey], {
        // summary:
        //      The EPiServer content tree.
        // description:
        //      The page tree shows the EPiServer CMS page tree using a REST store to fetch the data.
        // tags:
        //      internal

        // roots: Array
        //      A list of references to the root nodes.
        roots: null,

        // expandExtraNodeItems: [public] Array|Integer
        //      A collection of content item id or one content item id need to expand by default.
        expandExtraNodeItems: null,

        // nodeConstructor: Function
        //      Tree node constructor. It should be a sub class of dijit/Tree/_TreeNode
        nodeConstructor: null,

        // typeIdentifiers: String | Array
        //      Type identifier  filter.
        typeIdentifiers: null,

        // allowManipulation: Boolean
        //      Denotes if node manipulation is allowed.
        allowManipulation: true,

        // persist: Boolean
        //      Enables/disables use of cookies for state saving.
        persist: false,

        // expandOnDnd: [public] Boolean
        //      Enables/disables expand on a tree node when Dnd over it.
        expandOnDnd: true,

        // showRoot: [public] Boolean
        //      Flag that indicates whether the the root node should be displayed.
        showRoot: false,

        // handleSelection: [public] Boolean
        //      Flag that indicates this tree need to reset selection each time it had refresh
        handleSelection: false,

        // disableRestrictedTypes: [public] Boolean
        //      Flag that indicates this tree must apply disable style for the node which are not selectable based on provided allowedTypes.
        disableRestrictedTypes: false,

        // dndSource: [public] dndSource
        //      The dnd source to use for the tree, if no one is specified the epi/shell/dnd/tree/dndSource will be used
        dndSource: null,

        postMixInProperties: function () {
            // summary:
            //      Post mixin initializations.
            // tags:
            //      protected

            this.inherited(arguments);

            if (!this.model) {
                this.model = this._createTreeModel();
            }

            if (this.allowManipulation) {
                this.dndController = this.dndSource || dndSource;

                //We allow dropping all types contained in the content repository, even if it's not displayed in the tree.
                this.accept = this.model.containedTypes || this.containedTypes;
            }

            // make sure these get called with this class as context
            // otherwise context will be the dndController for the tree
            this.checkItemAcceptance = lang.hitch(this, this.checkItemAcceptance);

            this.nodeConstructor = this.nodeConstructor || _ContentTreeNode;

            this.own(aspect.around(this.model, "pasteItems", this._aroundPasteItem.bind(this)));

            // we don't want to disable folders
            if (this.allowedTypes) {
                var contentFolderAdded = array.some(this.allowedTypes, function (item) {
                    return item === "episerver.core.contentfolder";
                });

                if (!contentFolderAdded) {
                    this.allowedTypes.push("episerver.core.contentfolder");
                }
            }
        },

        getItemType: function (item) {
            // summary:
            //      Resolve the type for each item. Called by the dnd source.

            return TypeDescriptorManager.getAndConcatenateValues(item.typeIdentifier, "dndTypes");
        },

        buildRendering: function () {
            // Wrap the model before build rendering
            this.model = this._wrapModel(this.model);

            this.inherited(arguments);
        },

        _wrapModel: function (model) {
            // summary:
            //      Wraps the model with a confirmation
            // tags:
            //      protected
            return ContentTreeModelConfirmation(this.model);
        },

        postCreate: function () {
            this.inherited(arguments);

            this.own(
                this.connect(this.model, "onSelect", lang.hitch(this, this._onSelect)),
                aspect.before(this.model, "onToggleContentDisplay", lang.hitch(this, function (/*String*/target, /*Boolean*/display) {
                    var targetContent = this.getNodesByItem(target)[0];
                    if (targetContent) {
                        domStyle.set(targetContent.domNode, {
                            display: display === true ? "" : "none"
                        });
                    }
                }))
            );
        },

        onLoad: function () {
            // summary:
            //      Called when tree has finished loading and expanding.
            // tags:
            //      protected

            this.inherited(arguments);

            return when(this._expandRootNodes(), lang.hitch(this, this._expandExtraNodes));
        },

        _expandRootNodes: function () {
            // summary:
            //      Expand the root nodes by default
            // tags:
            //      private

            // Expand the root nodes by default.
            var nodes = this.showRoot ? [this.rootNode] : this.rootNode.getChildren();
            return promiseAll(array.map(nodes, this.tree._expandNode, this));
        },

        _expandExtraNodes: function () {
            // summary:
            //      Expand the given extra nodes by default
            // tags:
            //      private

            if (!this.expandExtraNodeItems) {
                return;
            }

            var expandExtraNodeItems = this.expandExtraNodeItems instanceof Array ? this.expandExtraNodeItems : [this.expandExtraNodeItems];

            var extraNodes = array.map(expandExtraNodeItems, function (item) {
                return this.tree.getNodesByItem(item)[0];
            }, this);

            // Filter out empty node from the given array.
            extraNodes = array.filter(extraNodes, Boolean);

            return promiseAll(array.map(extraNodes, this.tree._expandNode, this));
        },

        _aroundPasteItem: function (originalPasteItems) {
            // summary:
            //      Paste item wrapper that marks the target node as processing while the paste is
            //      being executed on the server
            // tags:
            //      private

            return function (sourceItems, targetItem) {
                if (targetItem.isWastebasket) {
                    // wast basket is not in tree and it's not expandable
                    return originalPasteItems.apply(this.model, arguments);
                }

                var targetNode = this.get("selectedNode");
                // If we for some reason don't have a selected treeNode (UI)
                // we still want to try to paste the item
                if (!targetNode) {
                    return originalPasteItems.apply(this.model, arguments);
                }

                targetNode.markProcessing();
                return originalPasteItems.apply(this.model, arguments)
                    .then(function () {
                        if (!targetNode.isExpanded) {
                            targetNode.expand();
                        }
                    }).always(function () {
                        targetNode.unmarkProcessing();
                    });
            }.bind(this);
        },

        // gets propagated to dndSource
        checkItemAcceptance: function (targetNode, source, position) {
            // summary:
            //      Called from DnD to check if a specific target can accept the items in source

            var targetTreeNode = registry.getEnclosingWidget(targetNode),
                target = targetTreeNode && targetTreeNode.item;

            if (!target) {
                return false;
            }

            return this.model.checkItemAcceptance(target, source);
        },

        _setAllowManipulationAttr: function (value) {
            // summary:
            //      setter of allowManipulation property.
            // tags:
            //      private

            this._set("allowManipulation", value);

            if (value) {
                this.dndController = this.dndSource || dndSource;
            }
        },

        _createTreeModel: function () {
            // summary:
            //      Create content tree model
            // tags:
            //      protected

            return new ContentForestStoreModel({
                roots: this.roots,
                typeIdentifiers: this.typeIdentifiers
            });
        },

        _onItemDelete: function (/*Item*/item, /*boolean*/deleteChildren) {
            // summary:
            //      Override of dijit/Tree._onItemDelete to enable cleaning
            //      of the itemsNodeMap from children of the item being deleted.
            // tags:
            //      protected, override

            function flattenChildHierarchy(node, children) {
                array.forEach(node.getChildren(), function (child) {
                    children.push(child.item);
                    flattenChildHierarchy(child, children);
                });

                return children;
            }

            if (deleteChildren) {
                var treeNodes = this._itemNodesMap[this.model.getIdentity(item)];
                if (treeNodes) {
                    array.forEach(treeNodes, function (treeNode) {
                        array.forEach(flattenChildHierarchy(treeNode, []).reverse(), function (childItem) {
                            this._onItemDelete(childItem);
                        }, this);
                    }, this);
                }
            }

            this.inherited(arguments);
        },

        _setTypeIdentifiersAttr: function (value) {
            this._set("typeIdentifiers", value);

            if (this.model) {
                this.model.typeIdentifiers = value;
            }
        },

        getTooltip: function (item) {

            // summary:
            //		Overridden to select data for the tooltip
            // tags:
            //		public, extension
            var getter = this.model.getTooltip || this.model.getLabel;
            var reference = new ContentReference(item.contentLink),
                baseTooltip = getter ? getter(item) : this.inherited(arguments);

            return baseTooltip ? baseTooltip + ", " + headingResources.id + ": " + reference.id +
                " (" + headingResources.type + ": " + item.contentTypeName + ")" : baseTooltip;

        },

        buildNodeFromTemplate: function (args) {
            // summary:
            //      Build tree node from the given template.
            // args: Object
            //      Tree node creation arguments
            // tags:
            //      extension

            return new this.nodeConstructor(args);
        },

        _createTreeNode: function (/*Object*/args) {
            // summary:
            //      Overridable function to make tree node draggable after creation
            // args: Object
            //      Tree node creation arguments
            // tags:
            //      extension

            var params = lang.mixin({}, args, {
                dndData: args.item
            });

            return this.buildNodeFromTemplate(params);
        },

        getIconClass: function (/*dojo/data/Item*/item, /*Boolean*/opened) {
            // summary:
            //      Overridable function to return CSS class name to display icon
            // tags:
            //      extension

            if (!item) {
                return this.inherited(arguments);
            }

            var iconNodeClass = TypeDescriptorManager.getValue(item.typeIdentifier, "iconClass");
            if (this.model.isTypeOfRoot(item)) {
                return this.model.getObjectIconClass(item, iconNodeClass);
            }

            var supportsOpenCssClass = TypeDescriptorManager.getValue(item.typeIdentifier, "supportsOpenCssClass");

            if (opened && !item.isSubRoot && supportsOpenCssClass) {
                iconNodeClass = iconNodeClass + "Open";
            }

            return iconNodeClass + this._getItemDisableStyle(item);
        },

        getLabelClass: function (item, expanded) {
            // summary:
            //      Overridable function to return CSS class name to display node label
            // tags:
            //      extension

            return this._getItemDisableStyle(item);
        },

        focus: function () {
            // summary:
            //      Set focus to the widget.
            // tags:
            //      public

            this.focusNode(this.get("selectedNode") || this.rootNode);
        },

        _getItemDisableStyle: function (/* dojo/data/Item */item) {
            // summary:
            //      Returns the disabled style/css class for the given item only if disableRestrictedTypes is enabled and item is not part of allowedTypes
            // item:
            //      dojo/data/Item
            // tags:
            //      private

            if (this.disableRestrictedTypes) {
                return this._isItemRestricted(item) ? " is-disabled " : "";
            }

            return "";
        },

        _isItemRestricted: function (/* dojo/data/Item */item) {
            // summary:
            //      Checks whether the given item is restricted based on the allowedTypes
            // item:
            //      dojo/data/Item
            // tags:
            //      private

            var acceptedTypes = TypeDescriptorManager.getValidAcceptedTypes([item.typeIdentifier], this.allowedTypes, this.restrictedTypes);

            return !acceptedTypes.length;
        },

        _getTargetPath: function (/*String*/targetContent) {
            // summary:
            //      Get target content path
            // targetContent:
            //      Target content identity
            // tags:
            //      private

            var model = this.model,
                deferred = new Deferred();

            model.getAncestors(targetContent, function (ancestors) {
                // Path should include the target content, so push it onto the array for conversion.
                ancestors.push(targetContent);

                var targetPath = array.map(ancestors, function (item) {
                    return model.getIdentity(item).toString();
                });

                if (typeof model.filterAncestors === "function") {
                    targetPath = model.filterAncestors(targetPath);
                }

                deferred.resolve(targetPath);
            });

            return deferred.promise;
        },

        _isValidTargetContent: function (/*String*/content) {
            // summary:
            //      Validate target content before using
            // tags:
            //      private

            if (!content) {
                return false;
            }

            return !content.isDeleted && !content.isWastebasket;
        },

        _onSelect: function (id, setFocus, onComplete) {
            // summary:
            //      Raise event to set an item as selected
            // tags:
            //      private

            var selectedNode = this.get("selectedNode");
            this.tree.model.set("previousContextualSelection", {
                selectedContent: selectedNode
            });

            this.selectContent(id, setFocus, true, onComplete).then(lang.hitch(this, function () {
                selectedNode = this.get("selectedNode");
                if (selectedNode && selectedNode.item && selectedNode.item.contentLink) {
                    this.emit("select", selectedNode.item, selectedNode);
                }
            }));
        },

        selectContent: function (contentReferenceAsString, setFocus, needParentRefresh, onComplete) {
            // summary:
            //      Select a given page.
            // return: a <c href="dojo/Deferred" /> object which when it resolves will return pageName
            // tags:
            //      public

            var self = this,
                model = self.model,
                deferred = new Deferred(),
                setPaths = function (paths) {
                    self._preparePathNodes(paths);
                    self._setRecursivePaths(paths, deferred, setFocus, onComplete);
                };

            when(model.store.get(contentReferenceAsString), function (content) {
                if (!self._isValidTargetContent(content)) {
                    // Remark: Is this needed any more?
                    self.set("paths", []);
                    deferred.resolve(null);
                } else {
                    self._getTargetPath(content).then(function (targetPath) {
                        self._onSetTargetPath();
                        setPaths(targetPath);
                    });
                }
            });

            return deferred.promise;
        },

        _onSetTargetPath: function () {
            // summary:
            //      Handles selection when set target path for the tree.
            // tags:
            //      private

            this.model.set("handleSelection", this.handleSelection);
            if (!this.handleSelection) {
                return;
            }

            var selectedNode = this.get("selectedNode");
            if (selectedNode) {
                // Get selected node one more time by getNodesByItem() function
                // in order to ensures that the node is existing in the tree.
                selectedNode = this.getNodesByItem(selectedNode.item)[0];
                if (selectedNode) {
                    this.model.set("previousSelection", {
                        selectedContent: selectedNode,
                        selectedAncestors: selectedNode.getTreePath()
                    });
                }
            }
        },

        _preparePathNodes: function (paths) {
            // summary:
            //      Prepare steps before node selection in the tree
            // tags:
            //      protected, extension
        },

        _setRecursivePaths: function (paths, deferred, setFocus, onComplete) {
            if (!paths || paths.length === 0) {
                return;
            }

            this.set("paths", [paths]).then(lang.hitch(this, function () {
                var selectedNode = this.get("selectedNode");
                if (!selectedNode) {
                    // We didn't find a node to select. Remove the last node in the path and try again.
                    this._setRecursivePaths(paths.slice(0, -1), deferred, setFocus, onComplete);

                    return;
                }

                if (setFocus) {
                    this.focus();
                }

                if (onComplete) {
                    onComplete(selectedNode);
                }

                deferred.resolve(selectedNode);
            }, deferred.reject));
        },

        _onKeyDown: function (item, e) {
            // summary:
            //      Handles onKeyDown event to process cut, copy, paste, delete action (non-printable characters).
            // e:
            //      Event object
            // tags:
            //      private

            this.inherited(arguments);

            if (!this.allowManipulation) {
                return;
            }

            if (connect.isCopyKey(e)) {
                var character = e.keyCode ? String.fromCharCode(e.keyCode).toLowerCase() : "";
                switch (character) {
                    case "c":
                        this.emit("copyOrCut", true);
                        break;
                    case "x":
                        this.emit("copyOrCut", false);
                        break;
                    case "v":
                        this.emit("paste");
                        break;
                    default:
                        break;
                }
            }

            // Catch DELETE key
            if (e.altKey || e.ctrlKey || e.shiftKey || e.metaKey) {
                return;
            }
            if (e.keyCode === keys.DELETE) {
                this.emit("delete");
            }
        },

        _onNodeMouseLeave: function (/* Tree._TreeNode */node, evt) {
            // summary:
            //      Handles mouse leave event on a node
            // tags:
            //      private

            this.inherited(arguments);

            // Remove dijitTreeRowHover class for all tree nodes when mouse leave on a node (in IE browser, dijitTreeRowHover css class added by _CssStateMixin did not removed)
            if (sniff("ie")) {
                array.forEach(query(".dijitTreeRow.dijitTreeRowHover", this.tree.domNode), function (node) {
                    domClass.remove(node, "dijitTreeRowHover");
                });
            }
        }

    });

});

},
'epi/shell/dnd/tree/dndSource':function(){
﻿define("epi/shell/dnd/tree/dndSource", [
    "dojo/_base/declare", // declare

    "./multiDndSource"
], function (declare, multiDndSource) {

    return declare([multiDndSource], {
        // summary:
        //      dnd source that only supports single select
        // tags:
        //      internal

        //	singular: Boolean
        //      Allows selection of only one element, if true.
        //      Tree hasn't been tested in singular=true mode, unclear if it works.
        singular: true,

        userSelect: function (node) {
            // Override CTRL and SHIFT key press parameters since we don't support multiple selection.
            this.inherited(arguments, [node, false, false]);
        }
    });
});

},
'epi/shell/dnd/tree/multiDndSource':function(){
﻿define("epi/shell/dnd/tree/multiDndSource", [
    "dojo/_base/array", // array.forEach array.indexOf array.map
    "dojo/_base/connect", // isCopyKey
    "dojo/_base/declare", // declare
    "dojo/aspect",
    "dojo/dom-class", // domClass.add
    "dojo/dom-geometry", // domGeometry.position
    "dojo/_base/lang", // lang.mixin lang.hitch
    "dojo/on", // subscribe
    "dojo/touch",
    "dojo/topic",
    "dojo/dnd/Manager",
    "dojo/when",
    "dijit/tree/dndSource",
    "../_DndDataMixin",
    "epi/shell/TypeDescriptorManager"
], function (array, connect, declare, aspect, domClass, domGeometry, lang, on, touch, topic, dndManager, when, dndSource, _DndDataMixin, TypeDescriptorManager) {
    return declare([dndSource, _DndDataMixin], {
        // tags:
        //      internal xproduct

        generateText: false,

        //	singular: Boolean
        //      Allows selection of only one element, if true.
        //      Tree hasn't been tested in singular=true mode, unclear if it works.
        singular: false,

        constructor: function () {
            // If there are restricted types then add them to the accept array
            // with a value of zero to indicated that they are not accepted.
            var reject = this.reject;
            if (reject) {

                // we need to filter out restrictedDndTypes
                var allowedDnD = [];
                for (var key in this.accept) {
                    allowedDnD.push(key);
                }

                this.accept = {};
                allowedDnD = TypeDescriptorManager.removeIntersectingTypes(allowedDnD, this.reject);
                array.forEach(allowedDnD, function (allowed) {
                    this.accept[allowed] = 1;
                }, this);

                for (var i = 0; i < reject.length; ++i) {
                    this.accept[reject[i]] = 0;
                }
            }

            var manager = dndManager.manager();
            this._startDragHandle = aspect.before(manager, "startDrag", this._beforeStartDrag.bind(this));
        },

        destroy: function () {
            this.inherited(arguments);
            this._startDragHandle.remove();
            delete this._startDragHandle;
        },

        _beforeStartDrag: function (source, nodes, copy) {
            // summary:
            //      Replaces the nodes argument to include all selected nodes, not just parent nodes.
            //      dijit/tree/dndSource filters out nodes that are children to other nodes before calling startDrag.
            //      See: https://github.com/dojo/dijit/blob/1bd0548b63989418fc69a46976f5749651473ebb/tree/dndSource.js#L226
            // source: Object
            //      The dijit/tree/dndSource / dojo/dnd/Source which is providing the items
            // nodes: DomNode[]
            //      The list of transferred items, dndTreeNode nodes if dragging from a Tree
            // copy: Boolean
            //      Copy items, if true, move items otherwise
            // tags:
            //      private

            if (source !== this) {
                return arguments;
            }

            var allNodes = this.getSelectedTreeNodes();
            if (!allNodes.length) {
                return arguments;
            }

            allNodes = allNodes.map(function (treeNode) {
                return treeNode.domNode;
            });

            return [source, allNodes, copy];
        },

        deleteSelectedNodes: function () {
            // Method needed to interact with other sources but should not do anything
        },

        onDndDrop: function (source, nodes, copy, target) {
            // summary:
            //      Topic event processor for /dnd/drop, called to finish the DnD operation.
            // description:
            //      Updates data store items according to where node was dragged from and dropped
            //      to.   The tree will then respond to those data store updates and redraw itself.
            // source: Object
            //      The dijit.tree.dndSource / dojo.dnd.Source which is providing the items
            // nodes: DomNode[]
            //      The list of transferred items, dndTreeNode nodes if dragging from a Tree
            // copy: Boolean
            //      Copy items, if true, move items otherwise
            // tags:
            //      protected

            if (this.containerState === "Over") {
                var tree = this.tree,
                    model = tree.model,
                    targetAnchor = this.targetAnchor;

                this.isDragging = false;

                // Compute the new parent item
                var insertIndex;
                var newParentItem = (targetAnchor && targetAnchor.item) || tree.item;
                if (this.dropPosition === "Before" || this.dropPosition === "After") {
                    // Actually this should be checked during onMouseMove too, to make the drag icon red.
                    newParentItem = (targetAnchor.getParent() && targetAnchor.getParent().item) || tree.item;
                    // Compute the insert index for reordering
                    insertIndex = targetAnchor.getIndexInParent();
                    if (this.dropPosition === "After") {
                        insertIndex = targetAnchor.getIndexInParent() + 1;
                    }
                }

                // Get the hash to pass to model.newItem() / pasteItem().  A single call to
                // itemCreator() returns an array of hashes, one for each drag source node.
                var items = this.itemCreator(nodes, targetAnchor.rowNode, source, target);
                if (source === this) {
                    // This is a node from my own tree, and we are moving it, not copying.
                    // Remove item from old parent's children attribute.
                    // and this code should go there.

                    if (model.pasteItems) {
                        var pasteItems = items.map(function (item) {
                            if (item.dndData) {
                                return item.dndData.data;
                            }
                            return item;
                        });
                        model.pasteItems(pasteItems, newParentItem, copy, insertIndex);
                    } else {
                        array.forEach(items, function (item) {
                            if (item.dndData) {
                                var options = item.dndData.options;
                                var oldParentItem = options ? options.oldParentItem : null;
                                var indexInParent = options ? options.indexInParent : 0;

                                if (typeof insertIndex == "number") {
                                    if (newParentItem === oldParentItem && indexInParent < insertIndex) {
                                        insertIndex -= 1;
                                    }
                                }

                                model.pasteItem(item.dndData.data, oldParentItem, newParentItem, copy, insertIndex);
                            }
                        });
                    }
                } else {
                    // Create new items in the tree, based on the drag source.
                    if (model.newItems) {
                        model.newItems(items, newParentItem, insertIndex);
                    } else {
                        array.forEach(items, function (item) {
                            model.newItem(item, newParentItem, insertIndex);
                        });
                    }
                }

                // If both source and target have completed there event chains raise an onDndEnd.
                if (!source.isDragging && !target.isDragging) {
                    this.onDndEnd();
                }
            }

            //If the nodes has been dropped in another target and copy is set to false, raise the onDndItemRemoved on the source if it supports it
            if (this === target && source !== target && !copy && source.onDndItemRemoved) {
                var dndData = array.map(nodes, function (node) {
                    return this._getDndData(source.getItem(node.id), this.accept, false);
                }, this);
                source.onDndItemRemoved(dndData, source, nodes, copy, target);
            }

            this.onDndCancel();
        },

        onMouseOut: function () {
            // summary:
            //      Event processor for when mouse is moved away from a TreeNode
            // tags:
            //      private

            this.inherited(arguments);

            if (this._expandOnDndTimeout) {
                clearTimeout(this._expandOnDndTimeout);
            }
        },

        _onDragMouse: function (e, firstTime) {
            // summary:
            //      Helper method for processing onmousemove/onmouseover events while drag is in progress.
            //      Keeps track of current drop target.
            // e: Event
            //      The mousemove event.
            // firstTime: Boolean?
            //      If this flag is set, this is the first mouse move event of the drag, so call m.canDrop() etc.
            //      even if newTarget == null because the user quickly dragged a node in the Tree to a position
            //      over Tree.containerNode but not over any TreeNode (#7971)

            this.inherited(arguments);

            // Double check for droppable ability in case checkItemAcceptance() returns deferred,
            // because base _onDragMouse() only works correctly with checkItemAcceptance() that returns boolean.
            var m = dndManager.manager();
            if (m.canDropFlag) {
                when(this.checkItemAcceptance(this.current.rowNode, m.source, this.dropPosition.toLowerCase()), function (accepted) {
                    m.canDrop(accepted);
                });
            }

            if (this._expandOnDndTimeout) {
                clearTimeout(this._expandOnDndTimeout);
            }

            // Expand the targetAnchor node (if it's currently collapsed) so the user can see
            // where their node was dropped. In particular since that node is still selected.
            if (this.tree && this.tree.expandOnDnd === true && this.targetAnchor) {
                this._expandOnDndTimeout = setTimeout(function () {
                    // Only try to expand target node when still dragging
                    this.isDragging && this.tree._expandNode(this.targetAnchor);
                }.bind(this), 500);
            }
        },

        onDndEnd: function () {
            // summary:
            //      Event raised when both source and target have completed their drag events.
            // tags:
            //      callback
        },

        onDndItemRemoved: function (dndData, source, nodes, copy, target) {
            // summary:
            //      Called when the nodes has been removed from the source, i.e. dropped in another target
            //
            // dndData:
            //    Dnd data extracted from the dragging items which have the same data type to the current target
            //
            // source:
            //    The dnd source.
            //
            // nodes:
            //    The dragging nodes.
            //
            // copy:
            //    Denote that the drag is copy.
            //
            // tags:
            //    public, event
        },

        itemCreator: function (nodes, targetNode, source, target) {
            // summary:
            //      Returns objects passed to `Tree.model.newItem()` based on DnD nodes
            //      dropped onto the tree.
            // description:
            //      For each node in nodes[], which came from source, create a hash of name/value
            //      pairs to be passed to Tree.model.newItem().  Returns array of those hashes.
            // nodes: DomNode[]
            // targetNode: DomNode
            // source: dojo/dnd/Source
            // target: dojo/dnd/Source
            // returns: __Item[]
            //      Array of name/value hashes for each new item to be added to the Tree
            // tags:
            //      extension
            return array.map(nodes, function (node) {
                return {
                    id: node.id,
                    name: node.textContent || node.innerText || "",
                    dndData: this._getDndData(source.getItem(node.id), this.accept, this === source)
                };
            }, this);
        },

        getItem: function (/*String*/key) {
            // summary:
            //      Returns the dojo.dnd.Item (representing a dragged node) by it's key (id).
            //      Called by dojo.dnd.Source.checkAcceptance().
            // tags:
            //      protected
            var item = this.inherited(arguments);

            // change drag type for the tree
            if (this.tree.getItemType && item && item.data && item.data.item) {
                item.type = this.tree.getItemType(item.data.item);
            }

            // add other options
            item.options = this.getItemOptions(item);

            return item;
        },

        getItemOptions: function (item) {
            // summary:
            //      Add options for each item. Called by getItem.

            var opts = {};
            if (item.data) {
                if (item.data.getParent) {
                    var oldParent = item.data.getParent();
                    opts.oldParentItem = oldParent ? oldParent.item : null;
                }

                if (item.data.getIndexInParent) {
                    opts.indexInParent = item.data.getIndexInParent();
                }
            }
            return opts;
        },

        onMouseDown: function (e) {
            // summary:
            //      Event processor for onmousedown/ontouchstart.
            //      Override this to prevent dnd when user mouses down on scroll bar.
            // e: Event
            //      onmousedown/ontouchend event
            // tags:
            //      protected override

            // Prevent drag/drop from happening when user mouses down on scrollbar.
            var isContainer = domClass.contains(e.target, "dojoDndContainerOver");
            // Prevent drag/drop from happening when user mouses down between nodes.
            var isTreeContainer = domClass.contains(e.target, "dijitTreeContainer");
            if (isContainer || isTreeContainer) {
                return;
            }

            this.inherited(arguments);
        },

        onMouseUp: function (e) {

            // If multiple items are selected and the user clicks on the context menu
            // then it should not de-select the selected items
            if (this.current) {
                var id = this.current.id;

                if (!this.singular && !e.shiftKey && this.selection[id] && domClass.contains(e.target, "epi-iconContextMenu")) {
                    this._doDeselect = false;
                }
            }

            this.inherited(arguments);
        },

        checkAcceptance: function (source, nodes) {
            // summary:
            //		Checks if the target can accept nodes from this source.
            // source: Object
            //		The source which provides items.
            // nodes: Array
            //		The list of transferred items.

            if (this.readOnly) {
                return false;
            }

            var items = array.map(nodes, function (node) {
                return source.getItem(node.id);
            });

            return this._checkAcceptanceForItems(items, this.accept);
        }
    });
});

},
'dojo/dnd/Manager':function(){
define("dojo/dnd/Manager", [
	"../_base/array",  "../_base/declare", "../_base/event", "../_base/lang", "../_base/window",
	"../dom-class", "../Evented", "../has", "../keys", "../on", "../topic", "../touch",
	"./common", "./autoscroll", "./Avatar"
], function(array, declare, event, lang, win, domClass, Evented, has, keys, on, topic, touch,
	dnd, autoscroll, Avatar){

// module:
//		dojo/dnd/Manager

var Manager = declare("dojo.dnd.Manager", [Evented], {
	// summary:
	//		the manager of DnD operations (usually a singleton)
	constructor: function(){
		this.avatar  = null;
		this.source = null;
		this.nodes = [];
		this.copy  = true;
		this.target = null;
		this.canDropFlag = false;
		this.events = [];
	},

	// avatar's offset from the mouse
	OFFSET_X: has("touch") ? 0 : 16,
	OFFSET_Y: has("touch") ? -64 : 16,

	// methods
	overSource: function(source){
		// summary:
		//		called when a source detected a mouse-over condition
		// source: Object
		//		the reporter
		if(this.avatar){
			this.target = (source && source.targetState != "Disabled") ? source : null;
			this.canDropFlag = Boolean(this.target);
			this.avatar.update();
		}
		topic.publish("/dnd/source/over", source);
	},
	outSource: function(source){
		// summary:
		//		called when a source detected a mouse-out condition
		// source: Object
		//		the reporter
		if(this.avatar){
			if(this.target == source){
				this.target = null;
				this.canDropFlag = false;
				this.avatar.update();
				topic.publish("/dnd/source/over", null);
			}
		}else{
			topic.publish("/dnd/source/over", null);
		}
	},
	startDrag: function(source, nodes, copy){
		// summary:
		//		called to initiate the DnD operation
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise

		// Tell autoscroll that a drag is starting
		autoscroll.autoScrollStart(win.doc);

		this.source = source;
		this.nodes  = nodes;
		this.copy   = Boolean(copy); // normalizing to true boolean
		this.avatar = this.makeAvatar();
		win.body().appendChild(this.avatar.node);
		topic.publish("/dnd/start", source, nodes, this.copy);
		this.events = [
			on(win.doc, touch.move, lang.hitch(this, "onMouseMove")),
			on(win.doc, touch.release,   lang.hitch(this, "onMouseUp")),
			on(win.doc, "keydown",   lang.hitch(this, "onKeyDown")),
			on(win.doc, "keyup",     lang.hitch(this, "onKeyUp")),
			// cancel text selection and text dragging
			on(win.doc, "dragstart",   event.stop),
			on(win.body(), "selectstart", event.stop)
		];
		var c = "dojoDnd" + (copy ? "Copy" : "Move");
		domClass.add(win.body(), c);
	},
	canDrop: function(flag){
		// summary:
		//		called to notify if the current target can accept items
		var canDropFlag = Boolean(this.target && flag);
		if(this.canDropFlag != canDropFlag){
			this.canDropFlag = canDropFlag;
			this.avatar.update();
		}
	},
	stopDrag: function(){
		// summary:
		//		stop the DnD in progress
		domClass.remove(win.body(), ["dojoDndCopy", "dojoDndMove"]);
		array.forEach(this.events, function(handle){ handle.remove(); });
		this.events = [];
		this.avatar.destroy();
		this.avatar = null;
		this.source = this.target = null;
		this.nodes = [];
	},
	makeAvatar: function(){
		// summary:
		//		makes the avatar; it is separate to be overwritten dynamically, if needed
		return new Avatar(this);
	},
	updateAvatar: function(){
		// summary:
		//		updates the avatar; it is separate to be overwritten dynamically, if needed
		this.avatar.update();
	},

	// mouse event processors
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove
		// e: Event
		//		mouse event
		var a = this.avatar;
		if(a){
			autoscroll.autoScrollNodes(e);
			//autoscroll.autoScroll(e);
			var s = a.node.style;
			s.left = (e.pageX + this.OFFSET_X) + "px";
			s.top  = (e.pageY + this.OFFSET_Y) + "px";
			var copy = Boolean(this.source.copyState(dnd.getCopyKeyState(e)));
			if(this.copy != copy){
				this._setCopyStatus(copy);
			}
		}
		if(has("touch")){
			// Prevent page from scrolling so that user can drag instead.
			e.preventDefault();
		}
	},
	onMouseUp: function(e){
		// summary:
		//		event processor for onmouseup
		// e: Event
		//		mouse event
		if(this.avatar){
			if(this.target && this.canDropFlag){
				var copy = Boolean(this.source.copyState(dnd.getCopyKeyState(e)));
				topic.publish("/dnd/drop/before", this.source, this.nodes, copy, this.target, e);
				topic.publish("/dnd/drop", this.source, this.nodes, copy, this.target, e);
			}else{
				topic.publish("/dnd/cancel");
			}
			this.stopDrag();
		}
	},

	// keyboard event processors
	onKeyDown: function(e){
		// summary:
		//		event processor for onkeydown:
		//		watching for CTRL for copy/move status, watching for ESCAPE to cancel the drag
		// e: Event
		//		keyboard event
		if(this.avatar){
			switch(e.keyCode){
				case keys.CTRL:
					var copy = Boolean(this.source.copyState(true));
					if(this.copy != copy){
						this._setCopyStatus(copy);
					}
					break;
				case keys.ESCAPE:
					topic.publish("/dnd/cancel");
					this.stopDrag();
					break;
			}
		}
	},
	onKeyUp: function(e){
		// summary:
		//		event processor for onkeyup, watching for CTRL for copy/move status
		// e: Event
		//		keyboard event
		if(this.avatar && e.keyCode == keys.CTRL){
			var copy = Boolean(this.source.copyState(false));
			if(this.copy != copy){
				this._setCopyStatus(copy);
			}
		}
	},

	// utilities
	_setCopyStatus: function(copy){
		// summary:
		//		changes the copy status
		// copy: Boolean
		//		the copy status
		this.copy = copy;
		this.source._markDndStatus(this.copy);
		this.updateAvatar();
		domClass.replace(win.body(),
			"dojoDnd" + (this.copy ? "Copy" : "Move"),
			"dojoDnd" + (this.copy ? "Move" : "Copy"));
	}
});

// dnd._manager:
//		The manager singleton variable. Can be overwritten if needed.
dnd._manager = null;

Manager.manager = dnd.manager = function(){
	// summary:
	//		Returns the current DnD manager.  Creates one if it is not created yet.
	if(!dnd._manager){
		dnd._manager = new Manager();
	}
	return dnd._manager;	// Object
};

return Manager;
});

},
'dojo/dnd/Avatar':function(){
define("dojo/dnd/Avatar", [
	"../_base/declare",
	"../_base/window",
	"../dom",
	"../dom-attr",
	"../dom-class",
	"../dom-construct",
	"../hccss",
	"../query"
], function(declare, win, dom, domAttr, domClass, domConstruct, has, query){

// module:
//		dojo/dnd/Avatar

return declare("dojo.dnd.Avatar", null, {
	// summary:
	//		Object that represents transferred DnD items visually
	// manager: Object
	//		a DnD manager object

	constructor: function(manager){
		this.manager = manager;
		this.construct();
	},

	// methods
	construct: function(){
		// summary:
		//		constructor function;
		//		it is separate so it can be (dynamically) overwritten in case of need

		var a = domConstruct.create("table", {
				"class": "dojoDndAvatar",
				style: {
					position: "absolute",
					zIndex:   "1999",
					margin:   "0px"
				}
			}),
			source = this.manager.source, node,
			b = domConstruct.create("tbody", null, a),
			tr = domConstruct.create("tr", null, b),
			td = domConstruct.create("td", null, tr),
			k = Math.min(5, this.manager.nodes.length), i = 0;

		if(has("highcontrast")){
			domConstruct.create("span", {
				id : "a11yIcon",
				innerHTML : this.manager.copy ? '+' : "<"
			}, td)
		}
		domConstruct.create("span", {
			innerHTML: source.generateText ? this._generateText() : ""
		}, td);

		// we have to set the opacity on IE only after the node is live
		domAttr.set(tr, {
			"class": "dojoDndAvatarHeader",
			style: {opacity: 0.9}
		});
		for(; i < k; ++i){
			if(source.creator){
				// create an avatar representation of the node
				node = source._normalizedCreator(source.getItem(this.manager.nodes[i].id).data, "avatar").node;
			}else{
				// or just clone the node and hope it works
				node = this.manager.nodes[i].cloneNode(true);
				if(node.tagName.toLowerCase() == "tr"){
					// insert extra table nodes
					var table = domConstruct.create("table"),
						tbody = domConstruct.create("tbody", null, table);
					tbody.appendChild(node);
					node = table;
				}
			}
			node.id = "";
			tr = domConstruct.create("tr", null, b);
			td = domConstruct.create("td", null, tr);
			td.appendChild(node);
			domAttr.set(tr, {
				"class": "dojoDndAvatarItem",
				style: {opacity: (9 - i) / 10}
			});
		}
		this.node = a;
	},
	destroy: function(){
		// summary:
		//		destructor for the avatar; called to remove all references so it can be garbage-collected
		domConstruct.destroy(this.node);
		this.node = false;
	},
	update: function(){
		// summary:
		//		updates the avatar to reflect the current DnD state
		domClass.toggle(this.node, "dojoDndAvatarCanDrop", this.manager.canDropFlag);
		if(has("highcontrast")){
			var icon = dom.byId("a11yIcon");
			var text = '+';   // assume canDrop && copy
			if (this.manager.canDropFlag && !this.manager.copy){
				text = '< '; // canDrop && move
			}else if (!this.manager.canDropFlag && !this.manager.copy){
				text = "o"; //!canDrop && move
			}else if(!this.manager.canDropFlag){
				text = 'x';  // !canDrop && copy
			}
			icon.innerHTML=text;
		}
		// replace text
		query(("tr.dojoDndAvatarHeader td span" +(has("highcontrast") ? " span" : "")), this.node).forEach(
			function(node){
				node.innerHTML = this.manager.source.generateText ? this._generateText() : "";
			}, this);
	},
	_generateText: function(){
		// summary:
		//		generates a proper text to reflect copying or moving of items
		return this.manager.nodes.length.toString();
	}
});

});

},
'dijit/tree/dndSource':function(){
define("dijit/tree/dndSource", [
	"dojo/_base/array", // array.forEach array.indexOf array.map
	"dojo/_base/connect", // isCopyKey
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add
	"dojo/dom-geometry", // domGeometry.position
	"dojo/_base/lang", // lang.mixin lang.hitch
	"dojo/on", // subscribe
	"dojo/touch",
	"dojo/topic",
	"dojo/dnd/Manager", // DNDManager.manager
	"./_dndSelector"
], function(array, connect, declare, domClass, domGeometry, lang, on, touch, topic, DNDManager, _dndSelector){

// module:
//		dijit/tree/dndSource
// summary:
//		Handles drag and drop operations (as a source or a target) for `dijit.Tree`

/*=====
var __Item = {
	// summary:
	//		New item to be added to the Tree, like:
	// id: Anything
	id: "",
	// name: String
	name: ""
};
=====*/

var dndSource = declare("dijit.tree.dndSource", _dndSelector, {
	// summary:
	//		Handles drag and drop operations (as a source or a target) for `dijit.Tree`

	// isSource: Boolean
	//		Can be used as a DnD source.
	isSource: true,

	// accept: String[]
	//		List of accepted types (text strings) for the Tree; defaults to
	//		["text"]
	accept: ["text", "treeNode"],

	// copyOnly: [private] Boolean
	//		Copy items, if true, use a state of Ctrl key otherwise
	copyOnly: false,

	// dragThreshold: Number
	//		The move delay in pixels before detecting a drag; 5 by default
	dragThreshold: 5,

	// betweenThreshold: Integer
	//		Distance from upper/lower edge of node to allow drop to reorder nodes
	betweenThreshold: 0,

	// Flag used by Avatar.js to signal to generate text node when dragging
	generateText: true,

	constructor: function(/*dijit/Tree*/ tree, /*dijit/tree/dndSource*/ params){
		// summary:
		//		a constructor of the Tree DnD Source
		// tags:
		//		private
		if(!params){ params = {}; }
		lang.mixin(this, params);
		var type = params.accept instanceof Array ? params.accept : ["text", "treeNode"];
		this.accept = null;
		if(type.length){
			this.accept = {};
			for(var i = 0; i < type.length; ++i){
				this.accept[type[i]] = 1;
			}
		}

		// class-specific variables
		this.isDragging = false;
		this.mouseDown = false;
		this.targetAnchor = null;	// DOMNode corresponding to the currently moused over TreeNode
		this.targetBox = null;	// coordinates of this.targetAnchor
		this.dropPosition = "";	// whether mouse is over/after/before this.targetAnchor
		this._lastX = 0;
		this._lastY = 0;

		// states
		this.sourceState = "";
		if(this.isSource){
			domClass.add(this.node, "dojoDndSource");
		}
		this.targetState = "";
		if(this.accept){
			domClass.add(this.node, "dojoDndTarget");
		}

		// set up events
		this.topics = [
			topic.subscribe("/dnd/source/over", lang.hitch(this, "onDndSourceOver")),
			topic.subscribe("/dnd/start", lang.hitch(this, "onDndStart")),
			topic.subscribe("/dnd/drop", lang.hitch(this, "onDndDrop")),
			topic.subscribe("/dnd/cancel", lang.hitch(this, "onDndCancel"))
		];
	},

	// methods
	checkAcceptance: function(/*===== source, nodes =====*/){
		// summary:
		//		Checks if the target can accept nodes from this source
		// source: dijit/tree/dndSource
		//		The source which provides items
		// nodes: DOMNode[]
		//		Array of DOM nodes corresponding to nodes being dropped, dijitTreeRow nodes if
		//		source is a dijit/Tree.
		// tags:
		//		extension
		return true;	// Boolean
	},

	copyState: function(keyPressed){
		// summary:
		//		Returns true, if we need to copy items, false to move.
		//		It is separated to be overwritten dynamically, if needed.
		// keyPressed: Boolean
		//		The "copy" control key was pressed
		// tags:
		//		protected
		return this.copyOnly || keyPressed;	// Boolean
	},
	destroy: function(){
		// summary:
		//		Prepares the object to be garbage-collected.
		this.inherited(arguments);
		var h;
		while(h = this.topics.pop()){ h.remove(); }
		this.targetAnchor = null;
	},

	_onDragMouse: function(e, firstTime){
		// summary:
		//		Helper method for processing onmousemove/onmouseover events while drag is in progress.
		//		Keeps track of current drop target.
		// e: Event
		//		The mousemove event.
		// firstTime: Boolean?
		//		If this flag is set, this is the first mouse move event of the drag, so call m.canDrop() etc.
		//		even if newTarget == null because the user quickly dragged a node in the Tree to a position
		//		over Tree.containerNode but not over any TreeNode (#7971)

		var m = DNDManager.manager(),
			oldTarget = this.targetAnchor,			// the TreeNode corresponding to TreeNode mouse was previously over
			newTarget = this.current,				// TreeNode corresponding to TreeNode mouse is currently over
			oldDropPosition = this.dropPosition;	// the previous drop position (over/before/after)

		// calculate if user is indicating to drop the dragged node before, after, or over
		// (i.e., to become a child of) the target node
		var newDropPosition = "Over";
		if(newTarget && this.betweenThreshold > 0){
			// If mouse is over a new TreeNode, then get new TreeNode's position and size
			if(!this.targetBox || oldTarget != newTarget){
				this.targetBox = domGeometry.position(newTarget.rowNode, true);
			}
			if((e.pageY - this.targetBox.y) <= this.betweenThreshold){
				newDropPosition = "Before";
			}else if((e.pageY - this.targetBox.y) >= (this.targetBox.h - this.betweenThreshold)){
				newDropPosition = "After";
			}
		}

		if(firstTime || newTarget != oldTarget || newDropPosition != oldDropPosition){
			if(oldTarget){
				this._removeItemClass(oldTarget.rowNode, oldDropPosition);
			}
			if(newTarget){
				this._addItemClass(newTarget.rowNode, newDropPosition);
			}

			// Check if it's ok to drop the dragged node on/before/after the target node.
			if(!newTarget){
				m.canDrop(false);
			}else if(newTarget == this.tree.rootNode && newDropPosition != "Over"){
				// Can't drop before or after tree's root node; the dropped node would just disappear (at least visually)
				m.canDrop(false);
			}else{
				// Guard against dropping onto yourself (TODO: guard against dropping onto your descendant, #7140)
				var sameId = false;
				if(m.source == this){
					for(var dragId in this.selection){
						var dragNode = this.selection[dragId];
						if(dragNode.item === newTarget.item){
							sameId = true;
							break;
						}
					}
				}
				if(sameId){
					m.canDrop(false);
				}else if(this.checkItemAcceptance(newTarget.rowNode, m.source, newDropPosition.toLowerCase())
						&& !this._isParentChildDrop(m.source, newTarget.rowNode)){
					m.canDrop(true);
				}else{
					m.canDrop(false);
				}
			}

			this.targetAnchor = newTarget;
			this.dropPosition = newDropPosition;
		}
	},

	onMouseMove: function(e){
		// summary:
		//		Called for any onmousemove/ontouchmove events over the Tree
		// e: Event
		//		onmousemouse/ontouchmove event
		// tags:
		//		private
		if(this.isDragging && this.targetState == "Disabled"){ return; }
		this.inherited(arguments);
		var m = DNDManager.manager();
		if(this.isDragging){
			this._onDragMouse(e);
		}else{
			if(this.mouseDown && this.isSource &&
				 (Math.abs(e.pageX-this._lastX)>=this.dragThreshold || Math.abs(e.pageY-this._lastY)>=this.dragThreshold)){
				var nodes = this.getSelectedTreeNodes();
				if(nodes.length){
					if(nodes.length > 1){
						//filter out all selected items which has one of their ancestor selected as well
						var seen = this.selection, i = 0, r = [], n, p;
						nextitem: while((n = nodes[i++])){
							for(p = n.getParent(); p && p !== this.tree; p = p.getParent()){
								if(seen[p.id]){ //parent is already selected, skip this node
									continue nextitem;
								}
							}
							//this node does not have any ancestors selected, add it
							r.push(n);
						}
						nodes = r;
					}
					nodes = array.map(nodes, function(n){return n.domNode});
					m.startDrag(this, nodes, this.copyState(connect.isCopyKey(e)));
					this._onDragMouse(e, true);	// because this may be the only mousemove event we get before the drop
				}
			}
		}
	},

	onMouseDown: function(e){
		// summary:
		//		Event processor for onmousedown/ontouchstart
		// e: Event
		//		onmousedown/ontouchend event
		// tags:
		//		private
		this.mouseDown = true;
		this.mouseButton = e.button;
		this._lastX = e.pageX;
		this._lastY = e.pageY;
		this.inherited(arguments);
	},

	onMouseUp: function(e){
		// summary:
		//		Event processor for onmouseup/ontouchend
		// e: Event
		//		onmouseup/ontouchend event
		// tags:
		//		private
		if(this.mouseDown){
			this.mouseDown = false;
			this.inherited(arguments);
		}
	},

	onMouseOut: function(){
		// summary:
		//		Event processor for when mouse is moved away from a TreeNode
		// tags:
		//		private
		this.inherited(arguments);
		this._unmarkTargetAnchor();
	},

	checkItemAcceptance: function(/*===== target, source, position =====*/){
		// summary:
		//		Stub function to be overridden if one wants to check for the ability to drop at the node/item level
		// description:
		//		In the base case, this is called to check if target can become a child of source.
		//		When betweenThreshold is set, position="before" or "after" means that we
		//		are asking if the source node can be dropped before/after the target node.
		// target: DOMNode
		//		The dijitTreeRoot DOM node inside of the TreeNode that we are dropping on to
		//		Use dijit.getEnclosingWidget(target) to get the TreeNode.
		// source: dijit/tree/dndSource
		//		The (set of) nodes we are dropping
		// position: String
		//		"over", "before", or "after"
		// tags:
		//		extension
		return true;
	},

	// topic event processors
	onDndSourceOver: function(source){
		// summary:
		//		Topic event processor for /dnd/source/over, called when detected a current source.
		// source: Object
		//		The dijit/tree/dndSource / dojo/dnd/Source which has the mouse over it
		// tags:
		//		private
		if(this != source){
			this.mouseDown = false;
			this._unmarkTargetAnchor();
		}else if(this.isDragging){
			var m = DNDManager.manager();
			m.canDrop(false);
		}
	},
	onDndStart: function(source, nodes, copy){
		// summary:
		//		Topic event processor for /dnd/start, called to initiate the DnD operation
		// source: Object
		//		The dijit/tree/dndSource / dojo/dnd/Source which is providing the items
		// nodes: DomNode[]
		//		The list of transferred items, dndTreeNode nodes if dragging from a Tree
		// copy: Boolean
		//		Copy items, if true, move items otherwise
		// tags:
		//		private

		if(this.isSource){
			this._changeState("Source", this == source ? (copy ? "Copied" : "Moved") : "");
		}
		var accepted = this.checkAcceptance(source, nodes);

		this._changeState("Target", accepted ? "" : "Disabled");

		if(this == source){
			DNDManager.manager().overSource(this);
		}

		this.isDragging = true;
	},

	itemCreator: function(nodes /*===== , target, source =====*/){
		// summary:
		//		Returns objects passed to `Tree.model.newItem()` based on DnD nodes
		//		dropped onto the tree.   Developer must override this method to enable
		//		dropping from external sources onto this Tree, unless the Tree.model's items
		//		happen to look like {id: 123, name: "Apple" } with no other attributes.
		// description:
		//		For each node in nodes[], which came from source, create a hash of name/value
		//		pairs to be passed to Tree.model.newItem().  Returns array of those hashes.
		// nodes: DomNode[]
		// target: DomNode
		// source: dojo/dnd/Source
		// returns: __Item[]
		//		Array of name/value hashes for each new item to be added to the Tree
		// tags:
		//		extension

		// TODO: for 2.0 refactor so itemCreator() is called once per drag node, and
		// make signature itemCreator(sourceItem, node, target) (or similar).

		return array.map(nodes, function(node){
			return {
				"id": node.id,
				"name": node.textContent || node.innerText || ""
			};
		}); // Object[]
	},

	onDndDrop: function(source, nodes, copy){
		// summary:
		//		Topic event processor for /dnd/drop, called to finish the DnD operation.
		// description:
		//		Updates data store items according to where node was dragged from and dropped
		//		to.   The tree will then respond to those data store updates and redraw itself.
		// source: Object
		//		The dijit/tree/dndSource / dojo/dnd/Source which is providing the items
		// nodes: DomNode[]
		//		The list of transferred items, dndTreeNode nodes if dragging from a Tree
		// copy: Boolean
		//		Copy items, if true, move items otherwise
		// tags:
		//		protected
		if(this.containerState == "Over"){
			var tree = this.tree,
				model = tree.model,
				target = this.targetAnchor;

			this.isDragging = false;

			// Compute the new parent item
			var newParentItem;
			var insertIndex;
			var before;		// drop source before (aka previous sibling) of target
			newParentItem = (target && target.item) || tree.item;
			if(this.dropPosition == "Before" || this.dropPosition == "After"){
				// TODO: if there is no parent item then disallow the drop.
				// Actually this should be checked during onMouseMove too, to make the drag icon red.
				newParentItem = (target.getParent() && target.getParent().item) || tree.item;
				// Compute the insert index for reordering
				insertIndex = target.getIndexInParent();
				if(this.dropPosition == "After"){
					insertIndex = target.getIndexInParent() + 1;
					before = target.getNextSibling() && target.getNextSibling().item;
				}else{
					before = target.item;
				}
			}else{
				newParentItem = (target && target.item) || tree.item;
			}

			// If necessary, use this variable to hold array of hashes to pass to model.newItem()
			// (one entry in the array for each dragged node).
			var newItemsParams;

			array.forEach(nodes, function(node, idx){
				// dojo/dnd/Item representing the thing being dropped.
				// Don't confuse the use of item here (meaning a DnD item) with the
				// uses below where item means dojo.data item.
				var sourceItem = source.getItem(node.id);

				// Information that's available if the source is another Tree
				// (possibly but not necessarily this tree, possibly but not
				// necessarily the same model as this Tree)
				if(array.indexOf(sourceItem.type, "treeNode") != -1){
					var childTreeNode = sourceItem.data,
						childItem = childTreeNode.item,
						oldParentItem = childTreeNode.getParent().item;
				}

				if(source == this){
					// This is a node from my own tree, and we are moving it, not copying.
					// Remove item from old parent's children attribute.
					// TODO: dijit/tree/dndSelector should implement deleteSelectedNodes()
					// and this code should go there.

					if(typeof insertIndex == "number"){
						if(newParentItem == oldParentItem && childTreeNode.getIndexInParent() < insertIndex){
							insertIndex -= 1;
						}
					}
					model.pasteItem(childItem, oldParentItem, newParentItem, copy, insertIndex, before);
				}else if(model.isItem(childItem)){
					// Item from same model
					// (maybe we should only do this branch if the source is a tree?)
					model.pasteItem(childItem, oldParentItem, newParentItem, copy, insertIndex, before);
				}else{
					// Get the hash to pass to model.newItem().  A single call to
					// itemCreator() returns an array of hashes, one for each drag source node.
					if(!newItemsParams){
						newItemsParams = this.itemCreator(nodes, target.rowNode, source);
					}

					// Create new item in the tree, based on the drag source.
					model.newItem(newItemsParams[idx], newParentItem, insertIndex, before);
				}
			}, this);

			// Expand the target node (if it's currently collapsed) so the user can see
			// where their node was dropped.   In particular since that node is still selected.
			this.tree._expandNode(target);
		}
		this.onDndCancel();
	},

	onDndCancel: function(){
		// summary:
		//		Topic event processor for /dnd/cancel, called to cancel the DnD operation
		// tags:
		//		private
		this._unmarkTargetAnchor();
		this.isDragging = false;
		this.mouseDown = false;
		delete this.mouseButton;
		this._changeState("Source", "");
		this._changeState("Target", "");
	},

	// When focus moves in/out of the entire Tree
	onOverEvent: function(){
		// summary:
		//		This method is called when mouse is moved over our container (like onmouseenter)
		// tags:
		//		private
		this.inherited(arguments);
		DNDManager.manager().overSource(this);
	},
	onOutEvent: function(){
		// summary:
		//		This method is called when mouse is moved out of our container (like onmouseleave)
		// tags:
		//		private
		this._unmarkTargetAnchor();
		var m = DNDManager.manager();
		if(this.isDragging){
			m.canDrop(false);
		}
		m.outSource(this);

		this.inherited(arguments);
	},

	_isParentChildDrop: function(source, targetRow){
		// summary:
		//		Checks whether the dragged items are parent rows in the tree which are being
		//		dragged into their own children.
		//
		// source:
		//		The DragSource object.
		//
		// targetRow:
		//		The tree row onto which the dragged nodes are being dropped.
		//
		// tags:
		//		private

		// If the dragged object is not coming from the tree this widget belongs to,
		// it cannot be invalid.
		if(!source.tree || source.tree != this.tree){
			return false;
		}


		var root = source.tree.domNode;
		var ids = source.selection;

		var node = targetRow.parentNode;

		// Iterate up the DOM hierarchy from the target drop row,
		// checking of any of the dragged nodes have the same ID.
		while(node != root && !ids[node.id]){
			node = node.parentNode;
		}

		return node.id && ids[node.id];
	},

	_unmarkTargetAnchor: function(){
		// summary:
		//		Removes hover class of the current target anchor
		// tags:
		//		private
		if(!this.targetAnchor){ return; }
		this._removeItemClass(this.targetAnchor.rowNode, this.dropPosition);
		this.targetAnchor = null;
		this.targetBox = null;
		this.dropPosition = null;
	},

	_markDndStatus: function(copy){
		// summary:
		//		Changes source's state based on "copy" status
		this._changeState("Source", copy ? "Copied" : "Moved");
	}
});

/*=====
dndSource.__Item = __Item;
=====*/

return dndSource;
});

},
'dijit/tree/_dndSelector':function(){
define("dijit/tree/_dndSelector", [
	"dojo/_base/array", // array.filter array.forEach array.map
	"dojo/_base/connect", // connect.isCopyKey
	"dojo/_base/declare", // declare
	"dojo/_base/Deferred", // Deferred
	"dojo/_base/kernel",	// global
	"dojo/_base/lang", // lang.hitch
	"dojo/cookie", // cookie
	"dojo/mouse", // mouse.isLeft
	"dojo/on",
	"dojo/touch",
	"./_dndContainer"
], function(array, connect, declare, Deferred, kernel, lang, cookie, mouse, on, touch, _dndContainer){

	// module:
	//		dijit/tree/_dndSelector


	return declare("dijit.tree._dndSelector", _dndContainer, {
		// summary:
		//		This is a base class for `dijit/tree/dndSource` , and isn't meant to be used directly.
		//		It's based on `dojo/dnd/Selector`.
		// tags:
		//		protected

		/*=====
		// selection: Object
		//		(id to DomNode) map for every TreeNode that's currently selected.
		//		The DOMNode is the TreeNode.rowNode.
		selection: {},
		=====*/

		constructor: function(){
			// summary:
			//		Initialization
			// tags:
			//		private

			this.selection={};
			this.anchor = null;

			if(!this.cookieName && this.tree.id){
				this.cookieName = this.tree.id + "SaveSelectedCookie";
			}

			this.events.push(
				on(this.tree.domNode, touch.press, lang.hitch(this,"onMouseDown")),
				on(this.tree.domNode, touch.release, lang.hitch(this,"onMouseUp")),
				on(this.tree.domNode, touch.move, lang.hitch(this,"onMouseMove"))
			);
		},

		// singular: Boolean
		//		Allows selection of only one element, if true.
		//		Tree hasn't been tested in singular=true mode, unclear if it works.
		singular: false,

		// methods
		getSelectedTreeNodes: function(){
			// summary:
			//		Returns a list of selected node(s).
			//		Used by dndSource on the start of a drag.
			// tags:
			//		protected
			var nodes=[], sel = this.selection;
			for(var i in sel){
				nodes.push(sel[i]);
			}
			return nodes;
		},

		selectNone: function(){
			// summary:
			//		Unselects all items
			// tags:
			//		private

			this.setSelection([]);
			return this;	// self
		},

		destroy: function(){
			// summary:
			//		Prepares the object to be garbage-collected
			this.inherited(arguments);
			this.selection = this.anchor = null;
		},
		addTreeNode: function(/*dijit/Tree._TreeNode*/ node, /*Boolean?*/isAnchor){
			// summary:
			//		add node to current selection
			// node: Node
			//		node to add
			// isAnchor: Boolean
			//		Whether the node should become anchor.

			this.setSelection(this.getSelectedTreeNodes().concat( [node] ));
			if(isAnchor){ this.anchor = node; }
			return node;
		},
		removeTreeNode: function(/*dijit/Tree._TreeNode*/ node){
			// summary:
			//		remove node from current selection
			// node: Node
			//		node to remove
			this.setSelection(this._setDifference(this.getSelectedTreeNodes(), [node]));
			return node;
		},
		isTreeNodeSelected: function(/*dijit/Tree._TreeNode*/ node){
			// summary:
			//		return true if node is currently selected
			// node: Node
			//		the node to check whether it's in the current selection

			return node.id && !!this.selection[node.id];
		},
		setSelection: function(/*dijit/Tree._TreeNode[]*/ newSelection){
			// summary:
			//		set the list of selected nodes to be exactly newSelection. All changes to the
			//		selection should be passed through this function, which ensures that derived
			//		attributes are kept up to date. Anchor will be deleted if it has been removed
			//		from the selection, but no new anchor will be added by this function.
			// newSelection: Node[]
			//		list of tree nodes to make selected
			var oldSelection = this.getSelectedTreeNodes();
			array.forEach(this._setDifference(oldSelection, newSelection), lang.hitch(this, function(node){
				node.setSelected(false);
				if(this.anchor == node){
					delete this.anchor;
				}
				delete this.selection[node.id];
			}));
			array.forEach(this._setDifference(newSelection, oldSelection), lang.hitch(this, function(node){
				node.setSelected(true);
				this.selection[node.id] = node;
			}));
			this._updateSelectionProperties();
		},
		_setDifference: function(xs,ys){
			// summary:
			//		Returns a copy of xs which lacks any objects
			//		occurring in ys. Checks for membership by
			//		modifying and then reading the object, so it will
			//		not properly handle sets of numbers or strings.

			array.forEach(ys, function(y){ y.__exclude__ = true; });
			var ret = array.filter(xs, function(x){ return !x.__exclude__; });

			// clean up after ourselves.
			array.forEach(ys, function(y){ delete y['__exclude__'] });
			return ret;
		},
		_updateSelectionProperties: function(){
			// summary:
			//		Update the following tree properties from the current selection:
			//		path[s], selectedItem[s], selectedNode[s]

			var selected = this.getSelectedTreeNodes();
			var paths = [], nodes = [], selects = [];
			array.forEach(selected, function(node){
				var ary = node.getTreePath(), model = this.tree.model;
				nodes.push(node);
				paths.push(ary);
				ary = array.map(ary, function(item){
					return model.getIdentity(item);
				}, this);
				selects.push(ary.join("/"))
			}, this);
			var items = array.map(nodes,function(node){ return node.item; });
			this.tree._set("paths", paths);
			this.tree._set("path", paths[0] || []);
			this.tree._set("selectedNodes", nodes);
			this.tree._set("selectedNode", nodes[0] || null);
			this.tree._set("selectedItems", items);
			this.tree._set("selectedItem", items[0] || null);
            if (this.tree.persist && selects.length > 0) {
                cookie(this.cookieName, selects.join(","), {expires:365});
            }
		},
		_getSavedPaths: function(){
			// summary:
			//		Returns paths of nodes that were selected previously and saved in the cookie.

			var tree = this.tree;
			if(tree.persist && tree.dndController.cookieName){
				var oreo, paths = [];
				oreo = cookie(tree.dndController.cookieName);
				if(oreo){
					paths = array.map(oreo.split(","), function(path){
					   return path.split("/");
					})
				}
				return paths;
			}
		},
		// mouse events
		onMouseDown: function(e){
			// summary:
			//		Event processor for onmousedown/ontouchstart
			// e: Event
			//		onmousedown/ontouchstart event
			// tags:
			//		protected

			// ignore click on expando node
			if(!this.current || this.tree.isExpandoNode(e.target, this.current)){ return; }

			if(e.type == "mousedown" && mouse.isLeft(e)){
				// Prevent text selection while dragging on desktop, see #16328.   But don't call preventDefault()
				// for mobile because it will break things completely, see #15838.  Also, don't preventDefault() on
				// MSPointerDown or pointerdown events, because that stops the mousedown event from being generated,
				// see #17709.
				// TODO: remove this completely in 2.0.  It shouldn't be needed since dojo/dnd/Manager already
				// calls preventDefault() for the "selectstart" event.  It can also be achieved via CSS:
				// http://stackoverflow.com/questions/826782/css-rule-to-disable-text-selection-highlighting
				e.preventDefault();
			}else if(e.type != "touchstart"){
				// Ignore right click
				return;
			}

			var treeNode = this.current,
			  copy = connect.isCopyKey(e), id = treeNode.id;

			// if shift key is not pressed, and the node is already in the selection,
			// delay deselection until onmouseup so in the case of DND, deselection
			// will be canceled by onmousemove.
			if(!this.singular && !e.shiftKey && this.selection[id]){
				this._doDeselect = true;
				return;
			}else{
				this._doDeselect = false;
			}
			this.userSelect(treeNode, copy, e.shiftKey);
		},

		onMouseUp: function(e){
			// summary:
			//		Event processor for onmouseup/ontouchend
			// e: Event
			//		onmouseup/ontouchend event
			// tags:
			//		protected

			// _doDeselect is the flag to indicate that the user wants to either ctrl+click on
			// a already selected item (to deselect the item), or click on a not-yet selected item
			// (which should remove all current selection, and add the clicked item). This can not
			// be done in onMouseDown, because the user may start a drag after mousedown. By moving
			// the deselection logic here, the user can drags an already selected item.
			if(!this._doDeselect){ return; }
			this._doDeselect = false;
			this.userSelect(this.current, connect.isCopyKey(e), e.shiftKey);
		},
		onMouseMove: function(/*===== e =====*/){
			// summary:
			//		event processor for onmousemove/ontouchmove
			// e: Event
			//		onmousemove/ontouchmove event
			this._doDeselect = false;
		},

		_compareNodes: function(n1, n2){
			if(n1 === n2){
				return 0;
			}

			if('sourceIndex' in document.documentElement){ //IE
				//TODO: does not yet work if n1 and/or n2 is a text node
				return n1.sourceIndex - n2.sourceIndex;
			}else if('compareDocumentPosition' in document.documentElement){ //FF, Opera
				return n1.compareDocumentPosition(n2) & 2 ? 1: -1;
			}else if(document.createRange){ //Webkit
				var r1 = doc.createRange();
				r1.setStartBefore(n1);

				var r2 = doc.createRange();
				r2.setStartBefore(n2);

				return r1.compareBoundaryPoints(r1.END_TO_END, r2);
			}else{
				throw Error("dijit.tree._compareNodes don't know how to compare two different nodes in this browser");
			}
		},

		userSelect: function(node, multi, range){
			// summary:
			//		Add or remove the given node from selection, responding
			//		to a user action such as a click or keypress.
			// multi: Boolean
			//		Indicates whether this is meant to be a multi-select action (e.g. ctrl-click)
			// range: Boolean
			//		Indicates whether this is meant to be a ranged action (e.g. shift-click)
			// tags:
			//		protected

			if(this.singular){
				if(this.anchor == node && multi){
					this.selectNone();
				}else{
					this.setSelection([node]);
					this.anchor = node;
				}
			}else{
				if(range && this.anchor){
					var cr = this._compareNodes(this.anchor.rowNode, node.rowNode),
					begin, end, anchor = this.anchor;

					if(cr < 0){ //current is after anchor
						begin = anchor;
						end = node;
					}else{ //current is before anchor
						begin = node;
						end = anchor;
					}
					var nodes = [];
					//add everything betweeen begin and end inclusively
					while(begin != end){
						nodes.push(begin);
						begin = this.tree._getNextNode(begin);
					}
					nodes.push(end);

					this.setSelection(nodes);
				}else{
					if( this.selection[ node.id ] && multi ){
						this.removeTreeNode( node );
					}else if(multi){
						this.addTreeNode(node, true);
					}else{
						this.setSelection([node]);
						this.anchor = node;
					}
				}
			}
		},

		getItem: function(/*String*/ key){
			// summary:
			//		Returns the dojo/dnd/Container._Item (representing a dragged node) by it's key (id).
			//		Called by dojo/dnd/Source.checkAcceptance().
			// tags:
			//		protected

			var widget = this.selection[key];
			return {
				data: widget,
				type: ["treeNode"]
			}; // dojo/dnd/Container._Item
		},

		forInSelectedItems: function(/*Function*/ f, /*Object?*/ o){
			// summary:
			//		Iterates over selected items;
			//		see `dojo/dnd/Container.forInItems()` for details
			o = o || kernel.global;
			for(var id in this.selection){
				// console.log("selected item id: " + id);
				f.call(o, this.getItem(id), id, this);
			}
		}
	});
});

},
'dojo/cookie':function(){
define("dojo/cookie", ["./_base/kernel", "./regexp"], function(dojo, regexp){

// module:
//		dojo/cookie

/*=====
var __cookieProps = {
	// expires: Date|String|Number?
	//		If a number, the number of days from today at which the cookie
	//		will expire. If a date, the date past which the cookie will expire.
	//		If expires is in the past, the cookie will be deleted.
	//		If expires is omitted or is 0, the cookie will expire when the browser closes.
	// path: String?
	//		The path to use for the cookie.
	// domain: String?
	//		The domain to use for the cookie.
	// secure: Boolean?
	//		Whether to only send the cookie on secure connections
};
=====*/


dojo.cookie = function(/*String*/name, /*String?*/ value, /*__cookieProps?*/ props){
	// summary:
	//		Get or set a cookie.
	// description:
	//		If one argument is passed, returns the value of the cookie
	//		For two or more arguments, acts as a setter.
	// name:
	//		Name of the cookie
	// value:
	//		Value for the cookie
	// props:
	//		Properties for the cookie
	// example:
	//		set a cookie with the JSON-serialized contents of an object which
	//		will expire 5 days from now:
	//	|	require(["dojo/cookie", "dojo/json"], function(cookie, json){
	//	|		cookie("configObj", json.stringify(config, {expires: 5 }));
	//	|	});
	//
	// example:
	//		de-serialize a cookie back into a JavaScript object:
	//	|	require(["dojo/cookie", "dojo/json"], function(cookie, json){
	//	|		config = json.parse(cookie("configObj"));
	//	|	});
	//
	// example:
	//		delete a cookie:
	//	|	require(["dojo/cookie"], function(cookie){
	//	|		cookie("configObj", null, {expires: -1});
	//	|	});
	var c = document.cookie, ret;
	if(arguments.length == 1){
		var matches = c.match(new RegExp("(?:^|; )" + regexp.escapeString(name) + "=([^;]*)"));
		ret = matches ? decodeURIComponent(matches[1]) : undefined; 
	}else{
		props = props || {};
// FIXME: expires=0 seems to disappear right away, not on close? (FF3)  Change docs?
		var exp = props.expires;
		if(typeof exp == "number"){
			var d = new Date();
			d.setTime(d.getTime() + exp*24*60*60*1000);
			exp = props.expires = d;
		}
		if(exp && exp.toUTCString){ props.expires = exp.toUTCString(); }

		value = encodeURIComponent(value);
		var updatedCookie = name + "=" + value, propName;
		for(propName in props){
			updatedCookie += "; " + propName;
			var propValue = props[propName];
			if(propValue !== true){ updatedCookie += "=" + propValue; }
		}
		document.cookie = updatedCookie;
	}
	return ret; // String|undefined
};

dojo.cookie.isSupported = function(){
	// summary:
	//		Use to determine if the current browser supports cookies or not.
	//
	//		Returns true if user allows cookies.
	//		Returns false if user doesn't allow cookies.

	if(!("cookieEnabled" in navigator)){
		this("__djCookieTest__", "CookiesAllowed");
		navigator.cookieEnabled = this("__djCookieTest__") == "CookiesAllowed";
		if(navigator.cookieEnabled){
			this("__djCookieTest__", "", {expires: -1});
		}
	}
	return navigator.cookieEnabled;
};

return dojo.cookie;
});

},
'dijit/tree/_dndContainer':function(){
define("dijit/tree/_dndContainer", [
	"dojo/aspect",	// aspect.after
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add domClass.remove domClass.replace
	"dojo/_base/event",	// event.stop
	"dojo/_base/lang", // lang.mixin lang.hitch
	"dojo/on",
	"dojo/touch"
], function(aspect, declare,domClass, event, lang, on, touch){

	// module:
	//		dijit/tree/_dndContainer

	/*=====
	 var __Args = {
		 // summary:
		 //		A dict of parameters for Tree source configuration.
		 // isSource: Boolean?
		 //		Can be used as a DnD source. Defaults to true.
		 // accept: String[]
		 //		List of accepted types (text strings) for a target; defaults to
		 //		["text", "treeNode"]
		 // copyOnly: Boolean?
		 //		Copy items, if true, use a state of Ctrl key otherwise,
		 // dragThreshold: Number
		 //		The move delay in pixels before detecting a drag; 0 by default
		 // betweenThreshold: Integer
		 //		Distance from upper/lower edge of node to allow drop to reorder nodes
	 };
	 =====*/

	return declare("dijit.tree._dndContainer", null, {

		// summary:
		//		This is a base class for `dijit/tree/_dndSelector`, and isn't meant to be used directly.
		//		It's modeled after `dojo/dnd/Container`.
		// tags:
		//		protected

		/*=====
		// current: DomNode
		//		The currently hovered TreeNode.rowNode (which is the DOM node
		//		associated w/a given node in the tree, excluding it's descendants)
		current: null,
		=====*/

		constructor: function(tree, params){
			// summary:
			//		A constructor of the Container
			// tree: Node
			//		Node or node's id to build the container on
			// params: __Args
			//		A dict of parameters, which gets mixed into the object
			// tags:
			//		private
			this.tree = tree;
			this.node = tree.domNode;	// TODO: rename; it's not a TreeNode but the whole Tree
			lang.mixin(this, params);

			// class-specific variables
			this.current = null;	// current TreeNode's DOM node

			// states
			this.containerState = "";
			domClass.add(this.node, "dojoDndContainer");

			// set up events
			this.events = [
				// Mouse (or touch) enter/leave on Tree itself
				on(this.node, touch.enter, lang.hitch(this, "onOverEvent")),
				on(this.node, touch.leave,	lang.hitch(this, "onOutEvent")),

				// switching between TreeNodes
				aspect.after(this.tree, "_onNodeMouseEnter", lang.hitch(this, "onMouseOver"), true),
				aspect.after(this.tree, "_onNodeMouseLeave", lang.hitch(this, "onMouseOut"), true),

				// cancel text selection and text dragging
				on(this.node, "dragstart", lang.hitch(event, "stop")),
				on(this.node, "selectstart", lang.hitch(event, "stop"))
			];
		},

		destroy: function(){
			// summary:
			//		Prepares this object to be garbage-collected

			var h;
			while(h = this.events.pop()){ h.remove(); }

			// this.clearItems();
			this.node = this.parent = null;
		},

		// mouse events
		onMouseOver: function(widget /*===== , evt =====*/){
			// summary:
			//		Called when mouse is moved over a TreeNode
			// widget: TreeNode
			// evt: Event
			// tags:
			//		protected
			this.current = widget;
		},

		onMouseOut: function(/*===== widget, evt =====*/){
			// summary:
			//		Called when mouse is moved away from a TreeNode
			// widget: TreeNode
			// evt: Event
			// tags:
			//		protected
			this.current = null;
		},

		_changeState: function(type, newState){
			// summary:
			//		Changes a named state to new state value
			// type: String
			//		A name of the state to change
			// newState: String
			//		new state
			var prefix = "dojoDnd" + type;
			var state = type.toLowerCase() + "State";
			//domClass.replace(this.node, prefix + newState, prefix + this[state]);
			domClass.replace(this.node, prefix + newState, prefix + this[state]);
			this[state] = newState;
		},

		_addItemClass: function(node, type){
			// summary:
			//		Adds a class with prefix "dojoDndItem"
			// node: Node
			//		A node
			// type: String
			//		A variable suffix for a class name
			domClass.add(node, "dojoDndItem" + type);
		},

		_removeItemClass: function(node, type){
			// summary:
			//		Removes a class with prefix "dojoDndItem"
			// node: Node
			//		A node
			// type: String
			//		A variable suffix for a class name
			domClass.remove(node, "dojoDndItem" + type);
		},

		onOverEvent: function(){
			// summary:
			//		This function is called once, when mouse is over our container
			// tags:
			//		protected
			this._changeState("Container", "Over");
		},

		onOutEvent: function(){
			// summary:
			//		This function is called once, when mouse is out of our container
			// tags:
			//		protected
			this._changeState("Container", "");
		}
	});
});

},
'epi/shell/dnd/_DndDataMixin':function(){
﻿define("epi/shell/dnd/_DndDataMixin", [
    "dojo/_base/array",
    "dojo/_base/declare",

    //EPi Framework
    "epi/dependency",
    "epi/shell/conversion/ObjectConverterRegistry"

],
function (
    array,
    declare,

    dependency,
    ObjectConverterRegistry) {

    return declare(null, {
        // summary:
        //      Mixin used to extract dnd data from the dnd sources
        // tags:
        //      private

        // _converterRegistry: Object
        //      Registry of all available dnd converters
        // tags:
        //      protected

        _checkAcceptanceForItems: function (items, acceptedTypes) {
            // summary:
            //		Checks if the target can accept the items.
            // items: Array
            //		The list of items to check for acceptance.
            // acceptedTypes: object
            //		An object that contains properties matching the accepted types.

            return array.every(items, function (item) {
                return this._acceptsTypeWithoutConversion(item.type, acceptedTypes) ||
                    this._converterExistsForAnyType(item.type, acceptedTypes);
            }, this);
        },

        _acceptsTypeWithoutConversion: function (/*array*/types, acceptedTypes) {
            var flag = false;
            for (var j = 0; j < types.length; ++j) {
                if (types[j] in acceptedTypes) {
                    flag = !!acceptedTypes[types[j]];
                    break;
                }
            }
            return flag;
        },

        _converterExistsForAnyType: function (types, acceptedTypes) {
            for (var j = 0; j < types.length; ++j) {
                for (var type in acceptedTypes) {
                    if (ObjectConverterRegistry.getConverter(types[j], type)) {
                        return !!acceptedTypes[type];
                    }
                }
            }
            return false;
        },

        _getDndData: function (item, acceptedTypes, internalDrop) {
            // summary:
            //      Get the Dnd data for the node from the source
            // item: object
            //      The node to get the dnd data for
            // acceptedTypes: object
            //		An object that contains properties matching the accepted types.
            // internalDrop: boolean
            //      If this is an internal drop within the same widget or not.
            // tags:
            //      protected

            if (!item || !item.data) {
                return null;
            }

            var dndData = null;

            //If the data has a dndData property this is considered to be the data object since
            //the data object itself might be a draggable widget.
            var data = item.data.dndData || item.data;

            var normalizedData = null;

            if (this._acceptsTypeWithoutConversion(item.type, acceptedTypes)) {
                normalizedData = { type: item.type, data: data, options: item.options };
                dndData = normalizedData;
            } else {
                for (var type in acceptedTypes) {
                    var converter = ObjectConverterRegistry.getConverter(item.type, type);
                    if (converter) {
                        normalizedData = { type: [type] };
                        normalizedData.data = converter.convert(item.type, type, data);
                        normalizedData.options = item.options;
                        dndData = normalizedData;

                        break;
                    }
                }
            }

            // Don't assign converted data on internal move or copy
            if (!internalDrop) {
                //We assign the object containing the converted type and the converted data
                //to the original object since we need to access this in the original onDrop
                //and creator methods. We do not have access to the converted data there otherwise.
                //This might remove the need for a onDropData method.
                item.data.getNormalizedData = function () {
                    return normalizedData;
                };
            }
            return dndData;
        }
    });
});

},
'epi/shell/conversion/ObjectConverterRegistry':function(){
﻿define(
    "epi/shell/conversion/ObjectConverterRegistry", [
        "dojo/_base/array",
        "dojo/_base/lang"
    ],
    function (
        array,
        lang
    ) {

        return {
        // summary:
        //    A registry that keeps a list of converters that can convert data between different data types.
        // tags:
        //    public

        // _converterMappings: [private] Array
        //    They array with the registered converter mappings.
            _converterMappings: {},

            clear: function () {
            // summary:
            //    Clears the list of registered converters.
                this._converterMappings = {};
            },

            /*
        Custom functions
        */
            registerConverter: function (/*String*/sourceDataType, /*String*/targetDataType, converter) {
            // summary:
            //    Registers a converter between two given data types.
            //
            // sourceDataType: String
            //    The source data type.
            //
            // targetDataType: String
            //    The target data type.
            //
            // converter: Object
            //    The class that performs the actual conversion.
            // tags:
            //    public
                var existingConverterMappings = this._converterMappings[sourceDataType];
                if (!existingConverterMappings) {
                //First registration of the sourceType, register a mapping object
                    existingConverterMappings = {};
                    this._converterMappings[sourceDataType] = existingConverterMappings;
                }
                //Add the specific target type to the mapping
                existingConverterMappings[targetDataType] = converter;
            },

            getConverters: function (/*String*/sourceDataType) {
            // summary:
            //    Gets an object with properties for the different registered converters for the given type.
            //
            // sourceDataType: String | Array of strings
            //    The source data type.

                var converters;

                if (lang.isArray(sourceDataType)) {

                    converters = array.map(sourceDataType, function (name) {

                        if (name in this._converterMappings) {
                            return this._converterMappings[name];
                        }
                    }, this);

                    converters = array.filter(converters, function (item) {
                        return item !== undefined;
                    }, this);

                    return converters;
                } else {
                // only one, but return as array
                    if (this._converterMappings[sourceDataType] === undefined) {
                        return undefined;
                    }
                    return [this._converterMappings[sourceDataType]];
                }


            },

            getConverter: function (/*String*/sourceDataType, /*String*/targetDataType) {
            // summary:
            //    Gets an a converter for the given types or null if no converter exists.
            //
            // sourceDataType: String
            //    String | Array of strings.
            //
            // targetDataType: String
            //    The target data type.

                var converters = this.getConverters(sourceDataType);
                var converter;

                if (converters === undefined) {
                    return null;
                }

                array.some(converters, function (item, i) {
                    if ((item[targetDataType])) {
                        converter = item[targetDataType];
                        return true;
                    }
                });
                return converter;
            }
        };
    });

},
'epi/shell/widget/Tree':function(){
define("epi/shell/widget/Tree", [
// Dojo
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom",

    // Dijit
    "dijit/focus",
    "dijit/Tree"
],

function (
// Dojo
    array,
    declare,
    lang,
    dom,

    // Dijit
    focus,
    Tree
) {

    return declare(Tree, {
        // summary:
        //      Extends dijit/Tree with support for reloading expanded nodes
        //      by listening to the onItemChildrenReload event of the model.
        //      The subscriber needs to call getChildren to get the updated children collection
        // tags:
        //      public

        // dndParams: [internal] Array
        //      Parameters to pull off of the tree and pass on to the dndController as its params.
        dndParams: ["accept", "reject"].concat(Tree.prototype.dndParams),

        postCreate: function () {
            this.inherited(arguments);
            this.connect(this.model, "onItemChildrenReload", "_onChildrenReload");
        },

        _onChildrenReload: function (/*dojo/data/Item*/parent) {
            // summary:
            //      Processes notification of a change to an item's children
            // tags:
            //      private

            var identity = this.model.getIdentity(parent),
                parentNodes = this._itemNodesMap[identity];

            if (parentNodes) {
                if (array.some(parentNodes, function (node) {
                    return node.state === "LOADED";
                })) {
                    this.model.getChildren(parent, lang.hitch(this, function (newChildrenList) {
                        array.forEach(parentNodes, function (parentNode) {
                            parentNode.setChildItems(newChildrenList);
                        });

                        array.forEach(newChildrenList, function (child) {
                            // Update node's item
                            this.model.onChange(child);
                            // Recursive reload
                            this._onChildrenReload(child);
                        }, this);
                    }));
                } else {
                    // Maybe we process more than makeExpandable method
                    array.forEach(parentNodes, function (node) {
                        node.isExpandable = this.model.mayHaveChildren(node.item);
                        node._setExpando(false);
                    }, this);
                }
            }
        },

        _onItemChildrenChange: function () {
            // summary:
            //      Overridden in an effort to keep focus in the tree when the children changes.
            //      Since the tree detaches the nodes from the dom when refreshing children the
            //      focus will be lost

            var currentFocus = null;

            // Do we currently have focus
            if (focus.curNode && dom.isDescendant(focus.curNode, this.domNode)) {
                currentFocus = focus.curNode;
            }

            this.inherited(arguments);

            // Try to restore focus to the tree
            if (currentFocus) {
                if (dom.isDescendant(currentFocus, this.domNode)) {
                    currentFocus.focus();
                } else {
                    // We had focus but the previously focused node is not alive.
                    var node = this._getRootOrFirstNode();
                    if (node) {
                        this.focusNode(node);
                    }
                }
            }
        }
    });

});

},
'dijit/Tree':function(){
require({cache:{
'url:dijit/templates/TreeNode.html':"<div class=\"dijitTreeNode\" role=\"presentation\"\n\t><div data-dojo-attach-point=\"rowNode\" class=\"dijitTreeRow dijitInline\" role=\"presentation\"\n\t\t><div data-dojo-attach-point=\"indentNode\" class=\"dijitInline\"></div\n\t\t><img src=\"${_blankGif}\" alt=\"\" data-dojo-attach-point=\"expandoNode\" class=\"dijitTreeExpando\" role=\"presentation\"\n\t\t/><span data-dojo-attach-point=\"expandoNodeText\" class=\"dijitExpandoText\" role=\"presentation\"\n\t\t></span\n\t\t><span data-dojo-attach-point=\"contentNode\"\n\t\t\tclass=\"dijitTreeContent\" role=\"presentation\">\n\t\t\t<img src=\"${_blankGif}\" alt=\"\" data-dojo-attach-point=\"iconNode\" class=\"dijitIcon dijitTreeIcon\" role=\"presentation\"\n\t\t\t/><span data-dojo-attach-point=\"labelNode\" class=\"dijitTreeLabel\" role=\"treeitem\"\n\t\t\t\t\ttabindex=\"-1\" aria-selected=\"false\" id=\"${id}_label\"></span>\n\t\t</span\n\t></div>\n\t<div data-dojo-attach-point=\"containerNode\" class=\"dijitTreeContainer\" role=\"presentation\"\n\t\t style=\"display: none;\" aria-labelledby=\"${id}_label\"></div>\n</div>\n",
'url:dijit/templates/Tree.html':"<div class=\"dijitTree dijitTreeContainer\" role=\"tree\">\n\t<div class=\"dijitInline dijitTreeIndent\" style=\"position: absolute; top: -9999px\" data-dojo-attach-point=\"indentDetector\"></div>\n</div>\n"}});
define("dijit/Tree", [
	"dojo/_base/array", // array.filter array.forEach array.map
	"dojo/_base/connect",	// connect.isCopyKey()
	"dojo/cookie", // cookie
	"dojo/_base/declare", // declare
	"dojo/Deferred", // Deferred
	"dojo/DeferredList", // DeferredList
	"dojo/dom", // dom.isDescendant
	"dojo/dom-class", // domClass.add domClass.remove domClass.replace domClass.toggle
	"dojo/dom-geometry", // domGeometry.setMarginBox domGeometry.position
	"dojo/dom-style",// domStyle.set
	"dojo/_base/event", // event.stop
	"dojo/errors/create",	// createError
	"dojo/fx", // fxUtils.wipeIn fxUtils.wipeOut
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/keys",	// arrows etc.
	"dojo/_base/lang", // lang.getObject lang.mixin lang.hitch
	"dojo/on",		// on(), on.selector()
	"dojo/topic",
	"dojo/touch",
	"dojo/when",
	"./focus",
	"./registry",	// registry.byNode(), registry.getEnclosingWidget()
	"./_base/manager",	// manager.defaultDuration
	"./_Widget",
	"./_TemplatedMixin",
	"./_Container",
	"./_Contained",
	"./_CssStateMixin",
	"dojo/text!./templates/TreeNode.html",
	"dojo/text!./templates/Tree.html",
	"./tree/TreeStoreModel",
	"./tree/ForestStoreModel",
	"./tree/_dndSelector"
], function(array, connect, cookie, declare, Deferred, DeferredList,
			dom, domClass, domGeometry, domStyle, event, createError, fxUtils, kernel, keys, lang, on, topic, touch, when,
			focus, registry, manager, _Widget, _TemplatedMixin, _Container, _Contained, _CssStateMixin,
			treeNodeTemplate, treeTemplate, TreeStoreModel, ForestStoreModel, _dndSelector){

// module:
//		dijit/Tree

// Back-compat shim
Deferred = declare(Deferred, {
	addCallback: function(callback){ this.then(callback); },
	addErrback: function(errback){ this.then(null, errback); }
});

var TreeNode = declare(
	"dijit._TreeNode",
	[_Widget, _TemplatedMixin, _Container, _Contained, _CssStateMixin],
{
	// summary:
	//		Single node within a tree.   This class is used internally
	//		by Tree and should not be accessed directly.
	// tags:
	//		private

	// item: [const] Item
	//		the dojo.data entry this tree represents
	item: null,

	// isTreeNode: [protected] Boolean
	//		Indicates that this is a TreeNode.   Used by `dijit.Tree` only,
	//		should not be accessed directly.
	isTreeNode: true,

	// label: String
	//		Text of this tree node
	label: "",
	_setLabelAttr: {node: "labelNode", type: "innerText"},

	// isExpandable: [private] Boolean
	//		This node has children, so show the expando node (+ sign)
	isExpandable: null,

	// isExpanded: [readonly] Boolean
	//		This node is currently expanded (ie, opened)
	isExpanded: false,

	// state: [private] String
	//		Dynamic loading-related stuff.
	//		When an empty folder node appears, it is "UNCHECKED" first,
	//		then after dojo.data query it becomes "LOADING" and, finally "LOADED"
	state: "UNCHECKED",

	templateString: treeNodeTemplate,

	baseClass: "dijitTreeNode",

	// For hover effect for tree node, and focus effect for label
	cssStateNodes: {
		rowNode: "dijitTreeRow"
	},

	// Tooltip is defined in _WidgetBase but we need to handle the mapping to DOM here
	_setTooltipAttr: {node: "rowNode", type: "attribute", attribute: "title"},

	buildRendering: function(){
		this.inherited(arguments);

		// set expand icon for leaf
		this._setExpando();

		// set icon and label class based on item
		this._updateItemClasses(this.item);

		if(this.isExpandable){
			this.labelNode.setAttribute("aria-expanded", this.isExpanded);
		}

		//aria-selected should be false on all selectable elements.
		this.setSelected(false);
	},

	_setIndentAttr: function(indent){
		// summary:
		//		Tell this node how many levels it should be indented
		// description:
		//		0 for top level nodes, 1 for their children, 2 for their
		//		grandchildren, etc.

		// Math.max() is to prevent negative padding on hidden root node (when indent == -1)
		var pixels = (Math.max(indent, 0) * this.tree._nodePixelIndent) + "px";

		domStyle.set(this.domNode, "backgroundPosition", pixels + " 0px");	// TODOC: what is this for???
		domStyle.set(this.indentNode, this.isLeftToRight() ? "paddingLeft" : "paddingRight", pixels);

		array.forEach(this.getChildren(), function(child){
			child.set("indent", indent+1);
		});

		this._set("indent", indent);
	},

	markProcessing: function(){
		// summary:
		//		Visually denote that tree is loading data, etc.
		// tags:
		//		private
		this.state = "LOADING";
		this._setExpando(true);
	},

	unmarkProcessing: function(){
		// summary:
		//		Clear markup from markProcessing() call
		// tags:
		//		private
		this._setExpando(false);
	},

	_updateItemClasses: function(item){
		// summary:
		//		Set appropriate CSS classes for icon and label dom node
		//		(used to allow for item updates to change respective CSS)
		// tags:
		//		private
		var tree = this.tree, model = tree.model;
		if(tree._v10Compat && item === model.root){
			// For back-compat with 1.0, need to use null to specify root item (TODO: remove in 2.0)
			item = null;
		}
		this._applyClassAndStyle(item, "icon", "Icon");
		this._applyClassAndStyle(item, "label", "Label");
		this._applyClassAndStyle(item, "row", "Row");

		this.tree._startPaint(true);		// signifies paint started and finished (synchronously)
	},

	_applyClassAndStyle: function(item, lower, upper){
		// summary:
		//		Set the appropriate CSS classes and styles for labels, icons and rows.
		//
		// item:
		//		The data item.
		//
		// lower:
		//		The lower case attribute to use, e.g. 'icon', 'label' or 'row'.
		//
		// upper:
		//		The upper case attribute to use, e.g. 'Icon', 'Label' or 'Row'.
		//
		// tags:
		//		private

		var clsName = "_" + lower + "Class";
		var nodeName = lower + "Node";
		var oldCls = this[clsName];

		this[clsName] = this.tree["get" + upper + "Class"](item, this.isExpanded);
		domClass.replace(this[nodeName], this[clsName] || "", oldCls || "");

		domStyle.set(this[nodeName], this.tree["get" + upper + "Style"](item, this.isExpanded) || {});
	},

	_updateLayout: function(){
		// summary:
		//		Set appropriate CSS classes for this.domNode
		// tags:
		//		private
		var parent = this.getParent();
		if(!parent || !parent.rowNode || parent.rowNode.style.display == "none"){
			/* if we are hiding the root node then make every first level child look like a root node */
			domClass.add(this.domNode, "dijitTreeIsRoot");
		}else{
			domClass.toggle(this.domNode, "dijitTreeIsLast", !this.getNextSibling());
		}
	},

	_setExpando: function(/*Boolean*/ processing){
		// summary:
		//		Set the right image for the expando node
		// tags:
		//		private

		var styles = ["dijitTreeExpandoLoading", "dijitTreeExpandoOpened",
						"dijitTreeExpandoClosed", "dijitTreeExpandoLeaf"],
			_a11yStates = ["*","-","+","*"],
			idx = processing ? 0 : (this.isExpandable ?	(this.isExpanded ? 1 : 2) : 3);

		// apply the appropriate class to the expando node
		domClass.replace(this.expandoNode, styles[idx], styles);

		// provide a non-image based indicator for images-off mode
		this.expandoNodeText.innerHTML = _a11yStates[idx];

	},

	expand: function(){
		// summary:
		//		Show my children
		// returns:
		//		Deferred that fires when expansion is complete

		// If there's already an expand in progress or we are already expanded, just return
		if(this._expandDeferred){
			return this._expandDeferred;		// dojo/_base/Deferred
		}

		// cancel in progress collapse operation
		if(this._collapseDeferred){
			this._collapseDeferred.cancel();
			delete this._collapseDeferred;
		}

		// All the state information for when a node is expanded, maybe this should be
		// set when the animation completes instead
		this.isExpanded = true;
		this.labelNode.setAttribute("aria-expanded", "true");
		if(this.tree.showRoot || this !== this.tree.rootNode){
			this.containerNode.setAttribute("role", "group");
		}
		domClass.add(this.contentNode,'dijitTreeContentExpanded');
		this._setExpando();
		this._updateItemClasses(this.item);
		
		if(this == this.tree.rootNode && this.tree.showRoot){
			this.tree.domNode.setAttribute("aria-expanded", "true");
		}

		var def,
			wipeIn = fxUtils.wipeIn({
				node: this.containerNode,
				duration: manager.defaultDuration,
				onEnd: function(){
					def.resolve(true);
				}
			});

		// Deferred that fires when expand is complete
		def = (this._expandDeferred = new Deferred(function(){
			// Canceller
			wipeIn.stop();
		}));

		wipeIn.play();

		return def;		// dojo/_base/Deferred
	},

	collapse: function(){
		// summary:
		//		Collapse this node (if it's expanded)

		if(this._collapseDeferred){
			// Node is already collapsed, or there's a collapse in progress, just return that Deferred
			return this._collapseDeferred;
		}

		// cancel in progress expand operation
		if(this._expandDeferred){
			this._expandDeferred.cancel();
			delete this._expandDeferred;
		}

		this.isExpanded = false;
		this.labelNode.setAttribute("aria-expanded", "false");
		if(this == this.tree.rootNode && this.tree.showRoot){
			this.tree.domNode.setAttribute("aria-expanded", "false");
		}
		domClass.remove(this.contentNode,'dijitTreeContentExpanded');
		this._setExpando();
		this._updateItemClasses(this.item);

		var def,
			wipeOut = fxUtils.wipeOut({
				node: this.containerNode,
				duration: manager.defaultDuration,
				onEnd: function(){
					def.resolve(true);
				}
			});

		// Deferred that fires when expand is complete
		def = (this._collapseDeferred = new Deferred(function(){
			// Canceller
			wipeOut.stop();
		}));

		wipeOut.play();

		return def;		// dojo/_base/Deferred
	},

	// indent: Integer
	//		Levels from this node to the root node
	indent: 0,

	setChildItems: function(/* Object[] */ items){
		// summary:
		//		Sets the child items of this node, removing/adding nodes
		//		from current children to match specified items[] array.
		//		Also, if this.persist == true, expands any children that were previously
		//		opened.
		// returns:
		//		Deferred object that fires after all previously opened children
		//		have been expanded again (or fires instantly if there are no such children).

		var tree = this.tree,
			model = tree.model,
			defs = [];	// list of deferreds that need to fire before I am complete


		// Orphan all my existing children.
		// If items contains some of the same items as before then we will reattach them.
		// Don't call this.removeChild() because that will collapse the tree etc.
		var oldChildren = this.getChildren();
		array.forEach(oldChildren, function(child){
			_Container.prototype.removeChild.call(this, child);
		}, this);

		// All the old children of this TreeNode are subject for destruction if
		//		1) they aren't listed in the new children array (items)
		//		2) they aren't immediately adopted by another node (DnD)
		this.defer(function(){
			array.forEach(oldChildren, function(node){
				if(!node._destroyed && !node.getParent()){
					// If node is in selection then remove it.
					tree.dndController.removeTreeNode(node);

					// Deregister mapping from item id --> this node
					var id = model.getIdentity(node.item),
						ary = tree._itemNodesMap[id];
					if(ary.length == 1){
						delete tree._itemNodesMap[id];
					}else{
						var index = array.indexOf(ary, node);
						if(index != -1){
							ary.splice(index, 1);
						}
					}

					// And finally we can destroy the node
					node.destroyRecursive();
				}
			});
		});

		this.state = "LOADED";

		if(items && items.length > 0){
			this.isExpandable = true;

			// Create _TreeNode widget for each specified tree node, unless one already
			// exists and isn't being used (presumably it's from a DnD move and was recently
			// released
			array.forEach(items, function(item){	// MARKER: REUSE NODE
				var id = model.getIdentity(item),
					existingNodes = tree._itemNodesMap[id],
					node;
				if(existingNodes){
					for(var i=0;i<existingNodes.length;i++){
						if(existingNodes[i] && !existingNodes[i].getParent()){
							node = existingNodes[i];
							node.set('indent', this.indent+1);
							break;
						}
					}
				}
				if(!node){
					node = this.tree._createTreeNode({
						item: item,
						tree: tree,
						isExpandable: model.mayHaveChildren(item),
						label: tree.getLabel(item),
						tooltip: tree.getTooltip(item),
						ownerDocument: tree.ownerDocument,
						dir: tree.dir,
						lang: tree.lang,
						textDir: tree.textDir,
						indent: this.indent + 1
					});
					if(existingNodes){
						existingNodes.push(node);
					}else{
						tree._itemNodesMap[id] = [node];
					}
				}
				this.addChild(node);

				// If node was previously opened then open it again now (this may trigger
				// more data store accesses, recursively)
				if(this.tree.autoExpand || this.tree._state(node)){
					defs.push(tree._expandNode(node));
				}
			}, this);

			// note that updateLayout() needs to be called on each child after
			// _all_ the children exist
			array.forEach(this.getChildren(), function(child){
				child._updateLayout();
			});
		}else{
			this.isExpandable=false;
		}

		if(this._setExpando){
			// change expando to/from dot or + icon, as appropriate
			this._setExpando(false);
		}

		// Set leaf icon or folder icon, as appropriate
		this._updateItemClasses(this.item);

		// On initial tree show, make the selected TreeNode as either the root node of the tree,
		// or the first child, if the root node is hidden
		if(this == tree.rootNode){
			var fc = this.tree.showRoot ? this : this.getChildren()[0];
			if(fc){
				fc.setFocusable(true);
				tree.lastFocused = fc;
			}else{
				// fallback: no nodes in tree so focus on Tree <div> itself
				tree.domNode.setAttribute("tabIndex", "0");
			}
		}

		var def =  new DeferredList(defs);
		this.tree._startPaint(def);		// to reset TreeNode widths after an item is added/removed from the Tree
		return def;		// dojo/_base/Deferred
	},

	getTreePath: function(){
		var node = this;
		var path = [];
		while(node && node !== this.tree.rootNode){
				path.unshift(node.item);
				node = node.getParent();
		}
		path.unshift(this.tree.rootNode.item);

		return path;
	},

	getIdentity: function(){
		return this.tree.model.getIdentity(this.item);
	},

	removeChild: function(/* treeNode */ node){
		this.inherited(arguments);

		var children = this.getChildren();
		if(children.length == 0){
			this.isExpandable = false;
			this.collapse();
		}

		array.forEach(children, function(child){
				child._updateLayout();
		});
	},

	makeExpandable: function(){
		// summary:
		//		if this node wasn't already showing the expando node,
		//		turn it into one and call _setExpando()

		// TODO: hmm this isn't called from anywhere, maybe should remove it for 2.0

		this.isExpandable = true;
		this._setExpando(false);
	},

	setSelected: function(/*Boolean*/ selected){
		// summary:
		//		A Tree has a (single) currently selected node.
		//		Mark that this node is/isn't that currently selected node.
		// description:
		//		In particular, setting a node as selected involves setting tabIndex
		//		so that when user tabs to the tree, focus will go to that node (only).
		this.labelNode.setAttribute("aria-selected", selected ? "true" : "false");
		domClass.toggle(this.rowNode, "dijitTreeRowSelected", selected);
	},

	setFocusable: function(/*Boolean*/ selected){
		// summary:
		//		A Tree has a (single) node that's focusable.
		//		Mark that this node is/isn't that currently focsuable node.
		// description:
		//		In particular, setting a node as selected involves setting tabIndex
		//		so that when user tabs to the tree, focus will go to that node (only).

		this.labelNode.setAttribute("tabIndex", selected ? "0" : "-1");
	},


	_setTextDirAttr: function(textDir){
		if(textDir &&((this.textDir != textDir) || !this._created)){
			this._set("textDir", textDir);
			this.applyTextDir(this.labelNode, this.labelNode.innerText || this.labelNode.textContent || "");
			array.forEach(this.getChildren(), function(childNode){
				childNode.set("textDir", textDir);
			}, this);
		}
	}
});

var Tree = declare("dijit.Tree", [_Widget, _TemplatedMixin], {
	// summary:
	//		This widget displays hierarchical data from a store.

	// store: [deprecated] String|dojo/data/Store
	//		Deprecated.  Use "model" parameter instead.
	//		The store to get data to display in the tree.
	store: null,

	// model: dijit/tree/model
	//		Interface to read tree data, get notifications of changes to tree data,
	//		and for handling drop operations (i.e drag and drop onto the tree)
	model: null,

	// query: [deprecated] anything
	//		Deprecated.  User should specify query to the model directly instead.
	//		Specifies datastore query to return the root item or top items for the tree.
	query: null,

	// label: [deprecated] String
	//		Deprecated.  Use dijit/tree/ForestStoreModel directly instead.
	//		Used in conjunction with query parameter.
	//		If a query is specified (rather than a root node id), and a label is also specified,
	//		then a fake root node is created and displayed, with this label.
	label: "",

	// showRoot: [const] Boolean
	//		Should the root node be displayed, or hidden?
	showRoot: true,

	// childrenAttr: [deprecated] String[]
	//		Deprecated.   This information should be specified in the model.
	//		One ore more attributes that holds children of a tree node
	childrenAttr: ["children"],

	// paths: String[][] or Item[][]
	//		Full paths from rootNode to selected nodes expressed as array of items or array of ids.
	//		Since setting the paths may be asynchronous (because of waiting on dojo.data), set("paths", ...)
	//		returns a Deferred to indicate when the set is complete.
	paths: [],

	// path: String[] or Item[]
	//		Backward compatible singular variant of paths.
	path: [],

	// selectedItems: [readonly] Item[]
	//		The currently selected items in this tree.
	//		This property can only be set (via set('selectedItems', ...)) when that item is already
	//		visible in the tree.   (I.e. the tree has already been expanded to show that node.)
	//		Should generally use `paths` attribute to set the selected items instead.
	selectedItems: null,

	// selectedItem: [readonly] Item
	//		Backward compatible singular variant of selectedItems.
	selectedItem: null,

	// openOnClick: Boolean
	//		If true, clicking a folder node's label will open it, rather than calling onClick()
	openOnClick: false,

	// openOnDblClick: Boolean
	//		If true, double-clicking a folder node's label will open it, rather than calling onDblClick()
	openOnDblClick: false,

	templateString: treeTemplate,

	// persist: Boolean
	//		Enables/disables use of cookies for state saving.
	persist: true,

	// autoExpand: Boolean
	//		Fully expand the tree on load.   Overrides `persist`.
	autoExpand: false,

	// dndController: [protected] Function|String
	//		Class to use as as the dnd controller.  Specifying this class enables DnD.
	//		Generally you should specify this as dijit/tree/dndSource.
	//		Setting of dijit/tree/_dndSelector handles selection only (no actual DnD).
	dndController: _dndSelector,

	// parameters to pull off of the tree and pass on to the dndController as its params
	dndParams: ["onDndDrop","itemCreator","onDndCancel","checkAcceptance", "checkItemAcceptance", "dragThreshold", "betweenThreshold"],

	//declare the above items so they can be pulled from the tree's markup

	// onDndDrop: [protected] Function
	//		Parameter to dndController, see `dijit/tree/dndSource.onDndDrop()`.
	//		Generally this doesn't need to be set.
	onDndDrop: null,

	itemCreator: null,
	/*=====
	itemCreator: function(nodes, target, source){
		// summary:
		//		Returns objects passed to `Tree.model.newItem()` based on DnD nodes
		//		dropped onto the tree.   Developer must override this method to enable
		//		dropping from external sources onto this Tree, unless the Tree.model's items
		//		happen to look like {id: 123, name: "Apple" } with no other attributes.
		//
		//		For each node in nodes[], which came from source, create a hash of name/value
		//		pairs to be passed to Tree.model.newItem().  Returns array of those hashes.
		// nodes: DomNode[]
		//		The DOMNodes dragged from the source container
		// target: DomNode
		//		The target TreeNode.rowNode
		// source: dojo/dnd/Source
		//		The source container the nodes were dragged from, perhaps another Tree or a plain dojo/dnd/Source
		// returns: Object[]
		//		Array of name/value hashes for each new item to be added to the Tree, like:
		// |	[
		// |		{ id: 123, label: "apple", foo: "bar" },
		// |		{ id: 456, label: "pear", zaz: "bam" }
		// |	]
		// tags:
		//		extension
		return [{}];
	},
	=====*/

	// onDndCancel: [protected] Function
	//		Parameter to dndController, see `dijit/tree/dndSource.onDndCancel()`.
	//		Generally this doesn't need to be set.
	onDndCancel: null,

/*=====
	checkAcceptance: function(source, nodes){
		// summary:
		//		Checks if the Tree itself can accept nodes from this source
		// source: dijit/tree/dndSource
		//		The source which provides items
		// nodes: DOMNode[]
		//		Array of DOM nodes corresponding to nodes being dropped, dijitTreeRow nodes if
		//		source is a dijit/Tree.
		// tags:
		//		extension
		return true;	// Boolean
	},
=====*/
	checkAcceptance: null,

/*=====
	checkItemAcceptance: function(target, source, position){
		// summary:
		//		Stub function to be overridden if one wants to check for the ability to drop at the node/item level
		// description:
		//		In the base case, this is called to check if target can become a child of source.
		//		When betweenThreshold is set, position="before" or "after" means that we
		//		are asking if the source node can be dropped before/after the target node.
		// target: DOMNode
		//		The dijitTreeRoot DOM node inside of the TreeNode that we are dropping on to
		//		Use registry.getEnclosingWidget(target) to get the TreeNode.
		// source: dijit/tree/dndSource
		//		The (set of) nodes we are dropping
		// position: String
		//		"over", "before", or "after"
		// tags:
		//		extension
		return true;	// Boolean
	},
=====*/
	checkItemAcceptance: null,

	// dragThreshold: Integer
	//		Number of pixels mouse moves before it's considered the start of a drag operation
	dragThreshold: 5,

	// betweenThreshold: Integer
	//		Set to a positive value to allow drag and drop "between" nodes.
	//
	//		If during DnD mouse is over a (target) node but less than betweenThreshold
	//		pixels from the bottom edge, dropping the the dragged node will make it
	//		the next sibling of the target node, rather than the child.
	//
	//		Similarly, if mouse is over a target node but less that betweenThreshold
	//		pixels from the top edge, dropping the dragged node will make it
	//		the target node's previous sibling rather than the target node's child.
	betweenThreshold: 0,

	// _nodePixelIndent: Integer
	//		Number of pixels to indent tree nodes (relative to parent node).
	//		Default is 19 but can be overridden by setting CSS class dijitTreeIndent
	//		and calling resize() or startup() on tree after it's in the DOM.
	_nodePixelIndent: 19,

	_publish: function(/*String*/ topicName, /*Object*/ message){
		// summary:
		//		Publish a message for this widget/topic
		topic.publish(this.id, lang.mixin({tree: this, event: topicName}, message || {}));	// publish
	},

	postMixInProperties: function(){
		this.tree = this;

		if(this.autoExpand){
			// There's little point in saving opened/closed state of nodes for a Tree
			// that initially opens all it's nodes.
			this.persist = false;
		}

		this._itemNodesMap = {};

		if(!this.cookieName && this.id){
			this.cookieName = this.id + "SaveStateCookie";
		}

		// Deferred that fires when all the children have loaded.
		this.expandChildrenDeferred  = new Deferred();

		// Deferred that fires when all pending operations complete.
		this.pendingCommandsDeferred = this.expandChildrenDeferred;

		this.inherited(arguments);
	},

	postCreate: function(){
		this._initState();

		// Catch events on TreeNodes
		var self = this;
		this.own(
			on(this.domNode, on.selector(".dijitTreeNode", touch.enter), function(evt){
				self._onNodeMouseEnter(registry.byNode(this), evt);
			}),
			on(this.domNode, on.selector(".dijitTreeNode", touch.leave), function(evt){
				self._onNodeMouseLeave(registry.byNode(this), evt);
			}),
			on(this.domNode, on.selector(".dijitTreeRow", "click"), function(evt){
				self._onClick(registry.getEnclosingWidget(this), evt);
			}),
			on(this.domNode, on.selector(".dijitTreeRow", "dblclick"), function(evt){
				self._onDblClick(registry.getEnclosingWidget(this), evt);
			}),
			on(this.domNode, on.selector(".dijitTreeNode", "keypress"), function(evt){
				self._onKeyPress(registry.byNode(this), evt);
			}),
			on(this.domNode, on.selector(".dijitTreeNode", "keydown"), function(evt){
				self._onKeyDown(registry.byNode(this), evt);
			}),
			on(this.domNode, on.selector(".dijitTreeRow", "focusin"), function(evt){
				self._onNodeFocus(registry.getEnclosingWidget(this), evt);
			})
		);

		// Create glue between store and Tree, if not specified directly by user
		if(!this.model){
			this._store2model();
		}

		// monitor changes to items
		this.connect(this.model, "onChange", "_onItemChange");
		this.connect(this.model, "onChildrenChange", "_onItemChildrenChange");
		this.connect(this.model, "onDelete", "_onItemDelete");

		this.inherited(arguments);

		if(this.dndController){
			if(lang.isString(this.dndController)){
				this.dndController = lang.getObject(this.dndController);
			}
			var params={};
			for(var i=0; i<this.dndParams.length;i++){
				if(this[this.dndParams[i]]){
					params[this.dndParams[i]] = this[this.dndParams[i]];
				}
			}
			this.dndController = new this.dndController(this, params);
		}

		this._load();

		// If no path was specified to the constructor, use path saved in cookie
		if(!this.params.path && !this.params.paths && this.persist){
			this.set("paths", this.dndController._getSavedPaths());
		}

		// onLoadDeferred should fire when all commands that are part of initialization have completed.
		// It will include all the set("paths", ...) commands that happen during initialization.
		this.onLoadDeferred = this.pendingCommandsDeferred;
				
		this.onLoadDeferred.then(lang.hitch(this, "onLoad"));
	},

	_store2model: function(){
		// summary:
		//		User specified a store&query rather than model, so create model from store/query
		this._v10Compat = true;
		kernel.deprecated("Tree: from version 2.0, should specify a model object rather than a store/query");

		var modelParams = {
			id: this.id + "_ForestStoreModel",
			store: this.store,
			query: this.query,
			childrenAttrs: this.childrenAttr
		};

		// Only override the model's mayHaveChildren() method if the user has specified an override
		if(this.params.mayHaveChildren){
			modelParams.mayHaveChildren = lang.hitch(this, "mayHaveChildren");
		}

		if(this.params.getItemChildren){
			modelParams.getChildren = lang.hitch(this, function(item, onComplete, onError){
				this.getItemChildren((this._v10Compat && item === this.model.root) ? null : item, onComplete, onError);
			});
		}
		this.model = new ForestStoreModel(modelParams);

		// For backwards compatibility, the visibility of the root node is controlled by
		// whether or not the user has specified a label
		this.showRoot = Boolean(this.label);
	},

	onLoad: function(){
		// summary:
		//		Called when tree finishes loading and expanding.
		// description:
		//		If persist == true the loading may encompass many levels of fetches
		//		from the data store, each asynchronous.   Waits for all to finish.
		// tags:
		//		callback
	},

	_load: function(){
		// summary:
		//		Initial load of the tree.
		//		Load root node (possibly hidden) and it's children.
		this.model.getRoot(
			lang.hitch(this, function(item){
				var rn = (this.rootNode = this.tree._createTreeNode({
					item: item,
					tree: this,
					isExpandable: true,
					label: this.label || this.getLabel(item),
					textDir: this.textDir,
					indent: this.showRoot ? 0 : -1
				}));
				
				if(!this.showRoot){
					rn.rowNode.style.display="none";
					// if root is not visible, move tree role to the invisible
					// root node's containerNode, see #12135
					this.domNode.setAttribute("role", "presentation");
					this.domNode.removeAttribute("aria-expanded");
					this.domNode.removeAttribute("aria-multiselectable");

					// move the aria-label or aria-labelledby to the element with the role
					if(this["aria-label"]){
						rn.containerNode.setAttribute("aria-label", this["aria-label"]);
						this.domNode.removeAttribute("aria-label");
					}else if(this["aria-labelledby"]){
						rn.containerNode.setAttribute("aria-labelledby", this["aria-labelledby"]);
						this.domNode.removeAttribute("aria-labelledby");
					}
					rn.labelNode.setAttribute("role", "presentation");
					rn.containerNode.setAttribute("role", "tree");
					rn.containerNode.setAttribute("aria-expanded","true");
					rn.containerNode.setAttribute("aria-multiselectable", !this.dndController.singular);
				}else{
				  this.domNode.setAttribute("aria-multiselectable", !this.dndController.singular);
				}
				
				this.domNode.appendChild(rn.domNode);
				var identity = this.model.getIdentity(item);
				if(this._itemNodesMap[identity]){
					this._itemNodesMap[identity].push(rn);
				}else{
					this._itemNodesMap[identity] = [rn];
				}

				rn._updateLayout();		// sets "dijitTreeIsRoot" CSS classname

				// Load top level children, and if persist==true, all nodes that were previously opened
				this._expandNode(rn).then(lang.hitch(this, function(){
					// Then, select the nodes that were selected last time, or
					// the ones specified by params.paths[].

					this.expandChildrenDeferred.resolve(true);
				}));
			}),
			lang.hitch(this, function(err){
				console.error(this, ": error loading root: ", err);
			})
		);
	},

	getNodesByItem: function(/*Item or id*/ item){
		// summary:
		//		Returns all tree nodes that refer to an item
		// returns:
		//		Array of tree nodes that refer to passed item

		if(!item){ return []; }
		var identity = lang.isString(item) ? item : this.model.getIdentity(item);
		// return a copy so widget don't get messed up by changes to returned array
		return [].concat(this._itemNodesMap[identity]);
	},

	_setSelectedItemAttr: function(/*Item or id*/ item){
		this.set('selectedItems', [item]);
	},

	_setSelectedItemsAttr: function(/*Items or ids*/ items){
		// summary:
		//		Select tree nodes related to passed items.
		//		WARNING: if model use multi-parented items or desired tree node isn't already loaded
		//		behavior is undefined. Use set('paths', ...) instead.
		var tree = this;
		return this.pendingCommandsDeferred = this.pendingCommandsDeferred.then( lang.hitch(this, function(){
			var identities = array.map(items, function(item){
				return (!item || lang.isString(item)) ? item : tree.model.getIdentity(item);
			});
			var nodes = [];
			array.forEach(identities, function(id){
				nodes = nodes.concat(tree._itemNodesMap[id] || []);
			});
			this.set('selectedNodes', nodes);
		}));
	},

	_setPathAttr: function(/*Item[]|String[]*/ path){
		// summary:
		//		Singular variant of _setPathsAttr
		if(path.length){
			return this.set("paths", [path]);
		}else{
			// Empty list is interpreted as "select nothing"
			return this.set("paths", []);
		}
	},

	_setPathsAttr: function(/*Item[][]|String[][]*/ paths){
		// summary:
		//		Select the tree nodes identified by passed paths.
		// paths:
		//		Array of arrays of items or item id's
		// returns:
		//		Deferred to indicate when the set is complete

		var tree = this;

		// Let any previous set("path", ...) commands complete before this one starts.
		return this.pendingCommandsDeferred = this.pendingCommandsDeferred.then(function(){
			// We may need to wait for some nodes to expand, so setting
			// each path will involve a Deferred. We bring those deferreds
			// together with a DeferredList.
			return new DeferredList(array.map(paths, function(path){
				var d = new Deferred();

				// normalize path to use identity
				path = array.map(path, function(item){
					return lang.isString(item) ? item : tree.model.getIdentity(item);
				});

				if(path.length){
					// Wait for the tree to load, if it hasn't already.
					selectPath(path, [tree.rootNode], d);
				}else{
					d.reject(new Tree.PathError("Empty path"));
				}
				return d;
			}));
		}).then(setNodes);

		function selectPath(path, nodes, def){
			// Traverse path; the next path component should be among "nodes".
			var nextPath = path.shift();
			var nextNode = array.filter(nodes, function(node){
				return node.getIdentity() == nextPath;
			})[0];
			if(!!nextNode){
				if(path.length){
					tree._expandNode(nextNode).then(function(){ selectPath(path, nextNode.getChildren(), def); });
				}else{
					// Successfully reached the end of this path
					def.resolve(nextNode);
				}
			}else{
				def.reject(new Tree.PathError("Could not expand path at " + nextPath));
			}
		}

		function setNodes(newNodes){
			// After all expansion is finished, set the selection to
			// the set of nodes successfully found.
			tree.set("selectedNodes", array.map(
				array.filter(newNodes,function(x){return x[0];}),
				function(x){return x[1];}));
		}
	},

	_setSelectedNodeAttr: function(node){
		this.set('selectedNodes', [node]);
	},
	_setSelectedNodesAttr: function(nodes){
		// summary:
		//		Marks the specified TreeNodes as selected.
		// nodes: TreeNode[]
		//		TreeNodes to mark.
		this.dndController.setSelection(nodes);
	},


	expandAll: function(){
		// summary:
		//		Expand all nodes in the tree
		// returns:
		//		Deferred that fires when all nodes have expanded

		var _this = this;

		function expand(node){
			var def = new dojo.Deferred();

			// Expand the node
			_this._expandNode(node).then(function(){
				// When node has expanded, call expand() recursively on each non-leaf child
				var childBranches = array.filter(node.getChildren() || [], function(node){
						return node.isExpandable;
					}),
					defs = array.map(childBranches, expand);

				// And when all those recursive calls finish, signal that I'm finished
				new dojo.DeferredList(defs).then(function(){
					def.resolve(true);
				});
			});

			return def;
		}

		return expand(this.rootNode);
	},

	collapseAll: function(){
		// summary:
		//		Collapse all nodes in the tree
		// returns:
		//		Deferred that fires when all nodes have collapsed

		var _this = this;

		function collapse(node){
			var def = new dojo.Deferred();
			def.label = "collapseAllDeferred";

			// Collapse children first
			var childBranches = array.filter(node.getChildren() || [], function(node){
					return node.isExpandable;
				}),
				defs = array.map(childBranches, collapse);

			// And when all those recursive calls finish, collapse myself, unless I'm the invisible root node,
			// in which case collapseAll() is finished
			new dojo.DeferredList(defs).then(function(){
				if(!node.isExpanded || (node == _this.rootNode && !_this.showRoot)){
					def.resolve(true);
				}else{
					_this._collapseNode(node).then(function(){
						// When node has collapsed, signal that call is finished
						def.resolve(true);
					});
				}
			});


			return def;
		}

		return collapse(this.rootNode);
	},

	////////////// Data store related functions //////////////////////
	// These just get passed to the model; they are here for back-compat

	mayHaveChildren: function(/*dojo/data/Item*/ /*===== item =====*/){
		// summary:
		//		Deprecated.   This should be specified on the model itself.
		//
		//		Overridable function to tell if an item has or may have children.
		//		Controls whether or not +/- expando icon is shown.
		//		(For efficiency reasons we may not want to check if an element actually
		//		has children until user clicks the expando node)
		// tags:
		//		deprecated
	},

	getItemChildren: function(/*===== parentItem, onComplete =====*/){
		// summary:
		//		Deprecated.   This should be specified on the model itself.
		//
		//		Overridable function that return array of child items of given parent item,
		//		or if parentItem==null then return top items in tree
		// tags:
		//		deprecated
	},

	///////////////////////////////////////////////////////
	// Functions for converting an item to a TreeNode
	getLabel: function(/*dojo/data/Item*/ item){
		// summary:
		//		Overridable function to get the label for a tree node (given the item)
		// tags:
		//		extension
		return this.model.getLabel(item);	// String
	},

	getIconClass: function(/*dojo/data/Item*/ item, /*Boolean*/ opened){
		// summary:
		//		Overridable function to return CSS class name to display icon
		// tags:
		//		extension
		return (!item || this.model.mayHaveChildren(item)) ? (opened ? "dijitFolderOpened" : "dijitFolderClosed") : "dijitLeaf"
	},

	getLabelClass: function(/*===== item, opened =====*/){
		// summary:
		//		Overridable function to return CSS class name to display label
		// item: dojo/data/Item
		// opened: Boolean
		// returns: String
		//		CSS class name
		// tags:
		//		extension
	},

	getRowClass: function(/*===== item, opened =====*/){
		// summary:
		//		Overridable function to return CSS class name to display row
		// item: dojo/data/Item
		// opened: Boolean
		// returns: String
		//		CSS class name
		// tags:
		//		extension
	},

	getIconStyle: function(/*===== item, opened =====*/){
		// summary:
		//		Overridable function to return CSS styles to display icon
		// item: dojo/data/Item
		// opened: Boolean
		// returns: Object
		//		Object suitable for input to dojo.style() like {backgroundImage: "url(...)"}
		// tags:
		//		extension
	},

	getLabelStyle: function(/*===== item, opened =====*/){
		// summary:
		//		Overridable function to return CSS styles to display label
		// item: dojo/data/Item
		// opened: Boolean
		// returns:
		//		Object suitable for input to dojo.style() like {color: "red", background: "green"}
		// tags:
		//		extension
	},

	getRowStyle: function(/*===== item, opened =====*/){
		// summary:
		//		Overridable function to return CSS styles to display row
		// item: dojo/data/Item
		// opened: Boolean
		// returns:
		//		Object suitable for input to dojo.style() like {background-color: "#bbb"}
		// tags:
		//		extension
	},

	getTooltip: function(/*dojo/data/Item*/ /*===== item =====*/){
		// summary:
		//		Overridable function to get the tooltip for a tree node (given the item)
		// tags:
		//		extension
		return "";	// String
	},

	/////////// Keyboard and Mouse handlers ////////////////////

	_onKeyPress: function(/*TreeNode*/ treeNode, /*Event*/ e){
		// summary:
		//		Handles keystrokes for printable keys, doing search navigation

		if(e.charCode <= 32){
			// Avoid duplicate events on firefox (this is an arrow key that will be handled by keydown handler)
			return;
		}

		if(!e.altKey && !e.ctrlKey && !e.shiftKey && !e.metaKey){
			var c = String.fromCharCode(e.charCode);
			this._onLetterKeyNav( { node: treeNode, key: c.toLowerCase() } );
			event.stop(e);
		}
	},

	_onKeyDown: function(/*TreeNode*/ treeNode, /*Event*/ e){
		// summary:
		//		Handles arrow, space, and enter keys

		var key = e.keyCode;

		var map = this._keyHandlerMap;
		if(!map){
			// Setup table mapping keys to events.
			// On WebKit based browsers, the combination ctrl-enter does not get passed through. To allow accessible
			// multi-select on those browsers, the space key is also used for selection.
			// Therefore, also allow space key for keyboard "click" operation.
			map = {};
			map[keys.ENTER] = map[keys.SPACE] = map[" "] = "_onEnterKey";
			map[this.isLeftToRight() ? keys.LEFT_ARROW : keys.RIGHT_ARROW] = "_onLeftArrow";
			map[this.isLeftToRight() ? keys.RIGHT_ARROW : keys.LEFT_ARROW] = "_onRightArrow";
			map[keys.UP_ARROW] = "_onUpArrow";
			map[keys.DOWN_ARROW] = "_onDownArrow";
			map[keys.HOME] = "_onHomeKey";
			map[keys.END] = "_onEndKey";
			this._keyHandlerMap = map;
		}

		if(this._keyHandlerMap[key]){
			// clear record of recent printables (being saved for multi-char letter navigation),
			// because "a", down-arrow, "b" shouldn't search for "ab"
			if(this._curSearch){
				this._curSearch.timer.remove();
				delete this._curSearch;
			}

			this[this._keyHandlerMap[key]]( { node: treeNode, item: treeNode.item, evt: e } );
			event.stop(e);
		}
	},

	_onEnterKey: function(/*Object*/ message){
		this._publish("execute", { item: message.item, node: message.node } );
		this.dndController.userSelect(message.node, connect.isCopyKey( message.evt ), message.evt.shiftKey);
		this.onClick(message.item, message.node, message.evt);
	},

	_onDownArrow: function(/*Object*/ message){
		// summary:
		//		down arrow pressed; get next visible node, set focus there
		var node = this._getNextNode(message.node);
		if(node && node.isTreeNode){
			this.focusNode(node);
		}
	},

	_onUpArrow: function(/*Object*/ message){
		// summary:
		//		Up arrow pressed; move to previous visible node

		var node = message.node;

		// if younger siblings
		var previousSibling = node.getPreviousSibling();
		if(previousSibling){
			node = previousSibling;
			// if the previous node is expanded, dive in deep
			while(node.isExpandable && node.isExpanded && node.hasChildren()){
				// move to the last child
				var children = node.getChildren();
				node = children[children.length-1];
			}
		}else{
			// if this is the first child, return the parent
			// unless the parent is the root of a tree with a hidden root
			var parent = node.getParent();
			if(!(!this.showRoot && parent === this.rootNode)){
				node = parent;
			}
		}

		if(node && node.isTreeNode){
			this.focusNode(node);
		}
	},

	_onRightArrow: function(/*Object*/ message){
		// summary:
		//		Right arrow pressed; go to child node
		var node = message.node;

		// if not expanded, expand, else move to 1st child
		if(node.isExpandable && !node.isExpanded){
			this._expandNode(node);
		}else if(node.hasChildren()){
			node = node.getChildren()[0];
			if(node && node.isTreeNode){
				this.focusNode(node);
			}
		}
	},

	_onLeftArrow: function(/*Object*/ message){
		// summary:
		//		Left arrow pressed.
		//		If not collapsed, collapse, else move to parent.

		var node = message.node;

		if(node.isExpandable && node.isExpanded){
			this._collapseNode(node);
		}else{
			var parent = node.getParent();
			if(parent && parent.isTreeNode && !(!this.showRoot && parent === this.rootNode)){
				this.focusNode(parent);
			}
		}
	},

	_onHomeKey: function(){
		// summary:
		//		Home key pressed; get first visible node, and set focus there
		var node = this._getRootOrFirstNode();
		if(node){
			this.focusNode(node);
		}
	},

	_onEndKey: function(){
		// summary:
		//		End key pressed; go to last visible node.

		var node = this.rootNode;
		while(node.isExpanded){
			var c = node.getChildren();
			node = c[c.length - 1];
		}

		if(node && node.isTreeNode){
			this.focusNode(node);
		}
	},

	// multiCharSearchDuration: Number
	//		If multiple characters are typed where each keystroke happens within
	//		multiCharSearchDuration of the previous keystroke,
	//		search for nodes matching all the keystrokes.
	//
	//		For example, typing "ab" will search for entries starting with
	//		"ab" unless the delay between "a" and "b" is greater than multiCharSearchDuration.
	multiCharSearchDuration: 250,

	_onLetterKeyNav: function(message){
		// summary:
		//		Called when user presses a prinatable key; search for node starting with recently typed letters.
		// message: Object
		//		Like { node: TreeNode, key: 'a' } where key is the key the user pressed.

		// Branch depending on whether this key starts a new search, or modifies an existing search
		var cs = this._curSearch;
		if(cs){
			// We are continuing a search.  Ex: user has pressed 'a', and now has pressed
			// 'b', so we want to search for nodes starting w/"ab".
			cs.pattern = cs.pattern + message.key;
			cs.timer.remove();
		}else{
			// We are starting a new search
			cs = this._curSearch = {
					pattern: message.key,
					startNode: message.node
			};
		}

		// set/reset timer to forget recent keystrokes
		cs.timer = this.defer(function(){
			delete this._curSearch;
		}, this.multiCharSearchDuration);

		// Navigate to TreeNode matching keystrokes [entered so far].
		var node = cs.startNode;
		do{
			node = this._getNextNode(node);
			//check for last node, jump to first node if necessary
			if(!node){
				node = this._getRootOrFirstNode();
			}
		}while(node !== cs.startNode && (node.label.toLowerCase().substr(0, cs.pattern.length) != cs.pattern));
		if(node && node.isTreeNode){
			// no need to set focus if back where we started
			if(node !== cs.startNode){
				this.focusNode(node);
			}
		}
	},

	isExpandoNode: function(node, widget){
		// summary:
		//		check whether a dom node is the expandoNode for a particular TreeNode widget
		return dom.isDescendant(node, widget.expandoNode) || dom.isDescendant(node, widget.expandoNodeText);
	},

	_onClick: function(/*TreeNode*/ nodeWidget, /*Event*/ e){
		// summary:
		//		Translates click events into commands for the controller to process

		var domElement = e.target,
			isExpandoClick = this.isExpandoNode(domElement, nodeWidget);

		if( (this.openOnClick && nodeWidget.isExpandable) || isExpandoClick ){
			// expando node was clicked, or label of a folder node was clicked; open it
			if(nodeWidget.isExpandable){
				this._onExpandoClick({node:nodeWidget});
			}
		}else{
			this._publish("execute", { item: nodeWidget.item, node: nodeWidget, evt: e } );
			this.onClick(nodeWidget.item, nodeWidget, e);
			this.focusNode(nodeWidget);
		}
		event.stop(e);
	},
	_onDblClick: function(/*TreeNode*/ nodeWidget, /*Event*/ e){
		// summary:
		//		Translates double-click events into commands for the controller to process

		var domElement = e.target,
			isExpandoClick = (domElement == nodeWidget.expandoNode || domElement == nodeWidget.expandoNodeText);

		if( (this.openOnDblClick && nodeWidget.isExpandable) ||isExpandoClick ){
			// expando node was clicked, or label of a folder node was clicked; open it
			if(nodeWidget.isExpandable){
				this._onExpandoClick({node:nodeWidget});
			}
		}else{
			this._publish("execute", { item: nodeWidget.item, node: nodeWidget, evt: e } );
			this.onDblClick(nodeWidget.item, nodeWidget, e);
			this.focusNode(nodeWidget);
		}
		event.stop(e);
	},

	_onExpandoClick: function(/*Object*/ message){
		// summary:
		//		User clicked the +/- icon; expand or collapse my children.
		var node = message.node;

		// If we are collapsing, we might be hiding the currently focused node.
		// Also, clicking the expando node might have erased focus from the current node.
		// For simplicity's sake just focus on the node with the expando.
		this.focusNode(node);

		if(node.isExpanded){
			this._collapseNode(node);
		}else{
			this._expandNode(node);
		}
	},

	onClick: function(/*===== item, node, evt =====*/){
		// summary:
		//		Callback when a tree node is clicked
		// item: Object
		//		Object from the dojo/store corresponding to this TreeNode
		// node: TreeNode
		//		The TreeNode itself
		// evt: Event
		//		The event
		// tags:
		//		callback
	},
	onDblClick: function(/*===== item, node, evt =====*/){
		// summary:
		//		Callback when a tree node is double-clicked
		// item: Object
		//		Object from the dojo/store corresponding to this TreeNode
		// node: TreeNode
		//		The TreeNode itself
		// evt: Event
		//		The event
		// tags:
		//		callback
	},
	onOpen: function(/*===== item, node =====*/){
		// summary:
		//		Callback when a node is opened
		// item: dojo/data/Item
		// node: TreeNode
		// tags:
		//		callback
	},
	onClose: function(/*===== item, node =====*/){
		// summary:
		//		Callback when a node is closed
		// item: Object
		//		Object from the dojo/store corresponding to this TreeNode
		// node: TreeNode
		//		The TreeNode itself
		// tags:
		//		callback
	},

	_getNextNode: function(node){
		// summary:
		//		Get next visible node

		if(node.isExpandable && node.isExpanded && node.hasChildren()){
			// if this is an expanded node, get the first child
			return node.getChildren()[0];		// TreeNode
		}else{
			// find a parent node with a sibling
			while(node && node.isTreeNode){
				var returnNode = node.getNextSibling();
				if(returnNode){
					return returnNode;		// TreeNode
				}
				node = node.getParent();
			}
			return null;
		}
	},

	_getRootOrFirstNode: function(){
		// summary:
		//		Get first visible node
		return this.showRoot ? this.rootNode : this.rootNode.getChildren()[0];
	},

	_collapseNode: function(/*TreeNode*/ node){
		// summary:
		//		Called when the user has requested to collapse the node
		// returns:
		//		Deferred that fires when the node is closed

		if(node._expandNodeDeferred){
			delete node._expandNodeDeferred;
		}

		if(node.state == "LOADING"){
			// ignore clicks while we are in the process of loading data
			return;
		}

		if(node.isExpanded){
			var ret = node.collapse();

			this.onClose(node.item, node);
			this._state(node, false);

			this._startPaint(ret);	// after this finishes, need to reset widths of TreeNodes

			return ret;
		}
	},

	_expandNode: function(/*TreeNode*/ node){
		// summary:
		//		Called when the user has requested to expand the node
		// returns:
		//		Deferred that fires when the node is loaded and opened and (if persist=true) all it's descendants
		//		that were previously opened too

		// Signal that this call is complete
		var def = new Deferred();

		if(node._expandNodeDeferred){
			// there's already an expand in progress, or completed, so just return
			return node._expandNodeDeferred;	// dojo/_base/Deferred
		}

		var model = this.model,
			item = node.item,
			_this = this;

		// Load data if it's not already loaded
		if(!node._loadDeferred){
			// need to load all the children before expanding
			node.markProcessing();

			// Setup deferred to signal when the load and expand are finished.
			// Save that deferred in this._expandDeferred as a flag that operation is in progress.
			node._loadDeferred = new Deferred();

			// Get the children
			model.getChildren(
				item,
				function(items){
					node.unmarkProcessing();

					// Display the children and also start expanding any children that were previously expanded
					// (if this.persist == true).   The returned Deferred will fire when those expansions finish.
					node.setChildItems(items).then(function(){
						node._loadDeferred.resolve(items);
					});
				},
				function(err){
					console.error(_this, ": error loading " + node.label + " children: ", err);
					node._loadDeferred.reject(err);
				}
			);
		}

		// Expand the node after data has loaded
		node._loadDeferred.then(lang.hitch(this, function(){
			node.expand().then(function(){
				def.resolve(true);	// signal that this _expandNode() call is complete
			});

			// seems like these should be inside of then(), but left here for back-compat about
			// when this.isOpen flag gets set (ie, at the beginning of the animation)
			this.onOpen(node.item, node);
			this._state(node, true);
		}));

		this._startPaint(def);	// after this finishes, need to reset widths of TreeNodes

		return def;	// dojo/_base/Deferred
	},

	////////////////// Miscellaneous functions ////////////////

	focusNode: function(/* _tree.Node */ node){
		// summary:
		//		Focus on the specified node (which must be visible)
		// tags:
		//		protected

		// set focus so that the label will be voiced using screen readers
		focus.focus(node.labelNode);
	},

	_onNodeFocus: function(/*dijit/_WidgetBase*/ node){
		// summary:
		//		Called when a TreeNode gets focus, either by user clicking
		//		it, or programatically by arrow key handling code.
		// description:
		//		It marks that the current node is the selected one, and the previously
		//		selected node no longer is.

		if(node && node != this.lastFocused){
			if(this.lastFocused && !this.lastFocused._destroyed){
				// mark that the previously focsable node is no longer focusable
				this.lastFocused.setFocusable(false);
			}

			// mark that the new node is the currently selected one
			node.setFocusable(true);
			this.lastFocused = node;
		}
	},

	_onNodeMouseEnter: function(/*dijit/_WidgetBase*/ /*===== node =====*/){
		// summary:
		//		Called when mouse is over a node (onmouseenter event),
		//		this is monitored by the DND code
	},

	_onNodeMouseLeave: function(/*dijit/_WidgetBase*/ /*===== node =====*/){
		// summary:
		//		Called when mouse leaves a node (onmouseleave event),
		//		this is monitored by the DND code
	},

	//////////////// Events from the model //////////////////////////

	_onItemChange: function(/*Item*/ item){
		// summary:
		//		Processes notification of a change to an item's scalar values like label
		var model = this.model,
			identity = model.getIdentity(item),
			nodes = this._itemNodesMap[identity];

		if(nodes){
			var label = this.getLabel(item),
				tooltip = this.getTooltip(item);
			array.forEach(nodes, function(node){
				node.set({
					item: item,		// theoretically could be new JS Object representing same item
					label: label,
					tooltip: tooltip
				});
				node._updateItemClasses(item);
			});
		}
	},

	_onItemChildrenChange: function(/*dojo/data/Item*/ parent, /*dojo/data/Item[]*/ newChildrenList){
		// summary:
		//		Processes notification of a change to an item's children
		var model = this.model,
			identity = model.getIdentity(parent),
			parentNodes = this._itemNodesMap[identity];

		if(parentNodes){
			array.forEach(parentNodes,function(parentNode){
				parentNode.setChildItems(newChildrenList);
			});
		}
	},

	_onItemDelete: function(/*Item*/ item){
		// summary:
		//		Processes notification of a deletion of an item.
		//		Not called from new dojo.store interface but there's cleanup code in setChildItems() instead.

		var model = this.model,
			identity = model.getIdentity(item),
			nodes = this._itemNodesMap[identity];

		if(nodes){
			array.forEach(nodes,function(node){
				// Remove node from set of selected nodes (if it's selected)
				this.dndController.removeTreeNode(node);

				var parent = node.getParent();
				if(parent){
					// if node has not already been orphaned from a _onSetItem(parent, "children", ..) call...
					parent.removeChild(node);
				}
				node.destroyRecursive();
			}, this);
			delete this._itemNodesMap[identity];
		}
	},

	/////////////// Miscellaneous funcs

	_initState: function(){
		// summary:
		//		Load in which nodes should be opened automatically
		this._openedNodes = {};
		if(this.persist && this.cookieName){
			var oreo = cookie(this.cookieName);
			if(oreo){
				array.forEach(oreo.split(','), function(item){
					this._openedNodes[item] = true;
				}, this);
			}
		}
	},
	_state: function(node, expanded){
		// summary:
		//		Query or set expanded state for an node
		if(!this.persist){
			return false;
		}
		var path = array.map(node.getTreePath(), function(item){
				return this.model.getIdentity(item);
			}, this).join("/");
		if(arguments.length === 1){
			return this._openedNodes[path];
		}else{
			if(expanded){
				this._openedNodes[path] = true;
			}else{
				delete this._openedNodes[path];
			}
			if(this.persist && this.cookieName){
				var ary = [];
				for(var id in this._openedNodes){
					ary.push(id);
				}
				cookie(this.cookieName, ary.join(","), {expires:365});
			}
		}
	},

	destroy: function(){
		if(this._curSearch){
			this._curSearch.timer.remove();
			delete this._curSearch;
		}
		if(this.rootNode){
			this.rootNode.destroyRecursive();
		}
		if(this.dndController && !lang.isString(this.dndController)){
			this.dndController.destroy();
		}
		this.rootNode = null;
		this.inherited(arguments);
	},

	destroyRecursive: function(){
		// A tree is treated as a leaf, not as a node with children (like a grid),
		// but defining destroyRecursive for back-compat.
		this.destroy();
	},

	resize: function(changeSize){
		if(changeSize){
			domGeometry.setMarginBox(this.domNode, changeSize);
		}

		// The main JS sizing involved w/tree is the indentation, which is specified
		// in CSS and read in through this dummy indentDetector node (tree must be
		// visible and attached to the DOM to read this).
		// If the Tree is hidden domGeometry.position(this.tree.indentDetector).w will return 0, in which case just
		// keep the default value.
		this._nodePixelIndent = domGeometry.position(this.tree.indentDetector).w || this._nodePixelIndent;

		// resize() may be called before this.rootNode is created, so wait until it's available
		this.expandChildrenDeferred.then(lang.hitch(this, function(){
			// If tree has already loaded, then reset indent for all the nodes
			this.rootNode.set('indent', this.showRoot ? 0 : -1);

			// Also, adjust widths of all rows to match width of Tree
			this._adjustWidths();
		}));
	},

	_outstandingPaintOperations: 0,
	_startPaint: function(/*Promise|Boolean*/ p){
		// summary:
		//		Called at the start of an operation that will change what's displayed.
		// p:
		//		Promise that tells when the operation will complete.  Alternately, if it's just a Boolean, it signifies
		//		that the operation was synchronous, and already completed.

		this._outstandingPaintOperations++;
		if(this._adjustWidthsTimer){
			this._adjustWidthsTimer.remove();
			delete this._adjustWidthsTimer;
		}

		var oc = lang.hitch(this, function(){
			this._outstandingPaintOperations--;

			if(this._outstandingPaintOperations <= 0 && !this._adjustWidthsTimer && this._started){
				// Use defer() to avoid a width adjustment when another operation will immediately follow,
				// such as a sequence of opening a node, then it's children, then it's grandchildren, etc.
				this._adjustWidthsTimer = this.defer("_adjustWidths");
			}
		});
		when(p, oc, oc);
	},

	_adjustWidths: function(){
		// summary:
		//		Get width of widest TreeNode, or the width of the Tree itself, whichever is greater,
		//		and then set all TreeNodes to that width, so that selection/hover highlighting
		//		extends to the edge of the Tree (#13141)

		if(this._adjustWidthsTimer){
			this._adjustWidthsTimer.remove();
			delete this._adjustWidthsTimer;
		}

		var maxWidth = 0,
			nodes = [];
		function collect(/*TreeNode*/ parent){
			var node = parent.rowNode;
			node.style.width = "auto";		// erase setting from previous run
			maxWidth = Math.max(maxWidth, node.clientWidth);
			nodes.push(node);
			if(parent.isExpanded){
				array.forEach(parent.getChildren(), collect);
			}
		}
		collect(this.rootNode);
		maxWidth = Math.max(maxWidth, domGeometry.getContentBox(this.domNode).w);	// do after node.style.width="auto"
		array.forEach(nodes, function(node){
			node.style.width = maxWidth + "px";		// assumes no horizontal padding, border, or margin on rowNode
		});
	},

	_createTreeNode: function(/*Object*/ args){
		// summary:
		//		creates a TreeNode
		// description:
		//		Developers can override this method to define their own TreeNode class;
		//		However it will probably be removed in a future release in favor of a way
		//		of just specifying a widget for the label, rather than one that contains
		//		the children too.
		return new TreeNode(args);
	},

	_setTextDirAttr: function(textDir){
		if(textDir && this.textDir!= textDir){
			this._set("textDir",textDir);
			this.rootNode.set("textDir", textDir);
		}
	}
});

Tree.PathError = createError("TreePathError");
Tree._TreeNode = TreeNode;	// for monkey patching or creating subclasses of TreeNode

return Tree;
});

},
'url:dijit/templates/TreeNode.html':"<div class=\"dijitTreeNode\" role=\"presentation\"\n\t><div data-dojo-attach-point=\"rowNode\" class=\"dijitTreeRow dijitInline\" role=\"presentation\"\n\t\t><div data-dojo-attach-point=\"indentNode\" class=\"dijitInline\"></div\n\t\t><img src=\"${_blankGif}\" alt=\"\" data-dojo-attach-point=\"expandoNode\" class=\"dijitTreeExpando\" role=\"presentation\"\n\t\t/><span data-dojo-attach-point=\"expandoNodeText\" class=\"dijitExpandoText\" role=\"presentation\"\n\t\t></span\n\t\t><span data-dojo-attach-point=\"contentNode\"\n\t\t\tclass=\"dijitTreeContent\" role=\"presentation\">\n\t\t\t<img src=\"${_blankGif}\" alt=\"\" data-dojo-attach-point=\"iconNode\" class=\"dijitIcon dijitTreeIcon\" role=\"presentation\"\n\t\t\t/><span data-dojo-attach-point=\"labelNode\" class=\"dijitTreeLabel\" role=\"treeitem\"\n\t\t\t\t\ttabindex=\"-1\" aria-selected=\"false\" id=\"${id}_label\"></span>\n\t\t</span\n\t></div>\n\t<div data-dojo-attach-point=\"containerNode\" class=\"dijitTreeContainer\" role=\"presentation\"\n\t\t style=\"display: none;\" aria-labelledby=\"${id}_label\"></div>\n</div>\n",
'url:dijit/templates/Tree.html':"<div class=\"dijitTree dijitTreeContainer\" role=\"tree\">\n\t<div class=\"dijitInline dijitTreeIndent\" style=\"position: absolute; top: -9999px\" data-dojo-attach-point=\"indentDetector\"></div>\n</div>\n",
'dijit/tree/TreeStoreModel':function(){
define("dijit/tree/TreeStoreModel", [
	"dojo/_base/array", // array.filter array.forEach array.indexOf array.some
	"dojo/aspect", // aspect.after
	"dojo/_base/declare", // declare
	"dojo/_base/lang" // lang.hitch
], function(array, aspect, declare, lang){

	// module:
	//		dijit/tree/TreeStoreModel

	return declare("dijit.tree.TreeStoreModel", null, {
		// summary:
		//		Implements dijit/Tree/model connecting to a dojo.data store with a single
		//		root item.  Any methods passed into the constructor will override
		//		the ones defined here.

		// store: dojo/data/api/Read
		//		Underlying store
		store: null,

		// childrenAttrs: String[]
		//		One or more attribute names (attributes in the dojo.data item) that specify that item's children
		childrenAttrs: ["children"],

		// newItemIdAttr: String
		//		Name of attribute in the Object passed to newItem() that specifies the id.
		//
		//		If newItemIdAttr is set then it's used when newItem() is called to see if an
		//		item with the same id already exists, and if so just links to the old item
		//		(so that the old item ends up with two parents).
		//
		//		Setting this to null or "" will make every drop create a new item.
		newItemIdAttr: "id",

		// labelAttr: String
		//		If specified, get label for tree node from this attribute, rather
		//		than by calling store.getLabel()
		labelAttr: "",

		// root: [readonly] dojo/data/Item
		//		Pointer to the root item (read only, not a parameter)
		root: null,

		// query: anything
		//		Specifies datastore query to return the root item for the tree.
		//		Must only return a single item.   Alternately can just pass in pointer
		//		to root item.
		// example:
		//	|	{id:'ROOT'}
		query: null,

		// deferItemLoadingUntilExpand: Boolean
		//		Setting this to true will cause the TreeStoreModel to defer calling loadItem on nodes
		//		until they are expanded. This allows for lazying loading where only one
		//		loadItem (and generally one network call, consequently) per expansion
		//		(rather than one for each child).
		//		This relies on partial loading of the children items; each children item of a
		//		fully loaded item should contain the label and info about having children.
		deferItemLoadingUntilExpand: false,

		constructor: function(/* Object */ args){
			// summary:
			//		Passed the arguments listed above (store, etc)
			// tags:
			//		private

			lang.mixin(this, args);

			this.connects = [];

			var store = this.store;
			if(!store.getFeatures()['dojo.data.api.Identity']){
				throw new Error("dijit.tree.TreeStoreModel: store must support dojo.data.Identity");
			}

			// if the store supports Notification, subscribe to the notification events
			if(store.getFeatures()['dojo.data.api.Notification']){
				this.connects = this.connects.concat([
					aspect.after(store, "onNew", lang.hitch(this, "onNewItem"), true),
					aspect.after(store, "onDelete", lang.hitch(this, "onDeleteItem"), true),
					aspect.after(store, "onSet", lang.hitch(this, "onSetItem"), true)
				]);
			}
		},

		destroy: function(){
			var h;
			while(h = this.connects.pop()){ h.remove(); }
			// TODO: should cancel any in-progress processing of getRoot(), getChildren()
		},

		// =======================================================================
		// Methods for traversing hierarchy

		getRoot: function(onItem, onError){
			// summary:
			//		Calls onItem with the root item for the tree, possibly a fabricated item.
			//		Calls onError on error.
			if(this.root){
				onItem(this.root);
			}else{
				this.store.fetch({
					query: this.query,
					onComplete: lang.hitch(this, function(items){
						if(items.length != 1){
							throw new Error("dijit.tree.TreeStoreModel: root query returned " + items.length +
								" items, but must return exactly one");
						}
						this.root = items[0];
						onItem(this.root);
					}),
					onError: onError
				});
			}
		},

		mayHaveChildren: function(/*dojo/data/Item*/ item){
			// summary:
			//		Tells if an item has or may have children.  Implementing logic here
			//		avoids showing +/- expando icon for nodes that we know don't have children.
			//		(For efficiency reasons we may not want to check if an element actually
			//		has children until user clicks the expando node)
			return array.some(this.childrenAttrs, function(attr){
				return this.store.hasAttribute(item, attr);
			}, this);
		},

		getChildren: function(/*dojo/data/Item*/ parentItem, /*function(items)*/ onComplete, /*function*/ onError){
			// summary:
			//		Calls onComplete() with array of child items of given parent item, all loaded.

			var store = this.store;
			if(!store.isItemLoaded(parentItem)){
				// The parent is not loaded yet, we must be in deferItemLoadingUntilExpand
				// mode, so we will load it and just return the children (without loading each
				// child item)
				var getChildren = lang.hitch(this, arguments.callee);
				store.loadItem({
					item: parentItem,
					onItem: function(parentItem){
						getChildren(parentItem, onComplete, onError);
					},
					onError: onError
				});
				return;
			}
			// get children of specified item
			var childItems = [];
			for(var i=0; i<this.childrenAttrs.length; i++){
				var vals = store.getValues(parentItem, this.childrenAttrs[i]);
				childItems = childItems.concat(vals);
			}

			// count how many items need to be loaded
			var _waitCount = 0;
			if(!this.deferItemLoadingUntilExpand){
				array.forEach(childItems, function(item){ if(!store.isItemLoaded(item)){ _waitCount++; } });
			}

			if(_waitCount == 0){
				// all items are already loaded (or we aren't loading them).  proceed...
				onComplete(childItems);
			}else{
				// still waiting for some or all of the items to load
				array.forEach(childItems, function(item, idx){
					if(!store.isItemLoaded(item)){
						store.loadItem({
							item: item,
							onItem: function(item){
								childItems[idx] = item;
								if(--_waitCount == 0){
									// all nodes have been loaded, send them to the tree
									onComplete(childItems);
								}
							},
							onError: onError
						});
					}
				});
			}
		},

		// =======================================================================
		// Inspecting items

		isItem: function(/* anything */ something){
			return this.store.isItem(something);	// Boolean
		},

		fetchItemByIdentity: function(/* object */ keywordArgs){
			this.store.fetchItemByIdentity(keywordArgs);
		},

		getIdentity: function(/* item */ item){
			return this.store.getIdentity(item);	// Object
		},

		getLabel: function(/*dojo/data/Item*/ item){
			// summary:
			//		Get the label for an item
			if(this.labelAttr){
				return this.store.getValue(item,this.labelAttr);	// String
			}else{
				return this.store.getLabel(item);	// String
			}
		},

		// =======================================================================
		// Write interface

		newItem: function(/* dijit/tree/dndSource.__Item */ args, /*dojo/data/api/Item*/ parent, /*int?*/ insertIndex){
			// summary:
			//		Creates a new item.   See `dojo/data/api/Write` for details on args.
			//		Used in drag & drop when item from external source dropped onto tree.
			// description:
			//		Developers will need to override this method if new items get added
			//		to parents with multiple children attributes, in order to define which
			//		children attribute points to the new item.

			var pInfo = {parent: parent, attribute: this.childrenAttrs[0]}, LnewItem;

			if(this.newItemIdAttr && args[this.newItemIdAttr]){
				// Maybe there's already a corresponding item in the store; if so, reuse it.
				this.fetchItemByIdentity({identity: args[this.newItemIdAttr], scope: this, onItem: function(item){
					if(item){
						// There's already a matching item in store, use it
						this.pasteItem(item, null, parent, true, insertIndex);
					}else{
						// Create new item in the tree, based on the drag source.
						LnewItem=this.store.newItem(args, pInfo);
						if(LnewItem && (insertIndex!=undefined)){
							// Move new item to desired position
							this.pasteItem(LnewItem, parent, parent, false, insertIndex);
						}
					}
				}});
			}else{
				// [as far as we know] there is no id so we must assume this is a new item
				LnewItem=this.store.newItem(args, pInfo);
				if(LnewItem && (insertIndex!=undefined)){
					// Move new item to desired position
					this.pasteItem(LnewItem, parent, parent, false, insertIndex);
				}
			}
		},

		pasteItem: function(/*Item*/ childItem, /*Item*/ oldParentItem, /*Item*/ newParentItem, /*Boolean*/ bCopy, /*int?*/ insertIndex){
			// summary:
			//		Move or copy an item from one parent item to another.
			//		Used in drag & drop
			var store = this.store,
				parentAttr = this.childrenAttrs[0];	// name of "children" attr in parent item

			// remove child from source item, and record the attribute that child occurred in
			if(oldParentItem){
				array.forEach(this.childrenAttrs, function(attr){
					if(store.containsValue(oldParentItem, attr, childItem)){
						if(!bCopy){
							var values = array.filter(store.getValues(oldParentItem, attr), function(x){
								return x != childItem;
							});
							store.setValues(oldParentItem, attr, values);
						}
						parentAttr = attr;
					}
				});
			}

			// modify target item's children attribute to include this item
			if(newParentItem){
				if(typeof insertIndex == "number"){
					// call slice() to avoid modifying the original array, confusing the data store
					var childItems = store.getValues(newParentItem, parentAttr).slice();
					childItems.splice(insertIndex, 0, childItem);
					store.setValues(newParentItem, parentAttr, childItems);
				}else{
					store.setValues(newParentItem, parentAttr,
						store.getValues(newParentItem, parentAttr).concat(childItem));
				}
			}
		},

		// =======================================================================
		// Callbacks

		onChange: function(/*dojo/data/Item*/ /*===== item =====*/){
			// summary:
			//		Callback whenever an item has changed, so that Tree
			//		can update the label, icon, etc.   Note that changes
			//		to an item's children or parent(s) will trigger an
			//		onChildrenChange() so you can ignore those changes here.
			// tags:
			//		callback
		},

		onChildrenChange: function(/*===== parent, newChildrenList =====*/){
			// summary:
			//		Callback to do notifications about new, updated, or deleted items.
			// parent: dojo/data/Item
			// newChildrenList: dojo/data/Item[]
			// tags:
			//		callback
		},

		onDelete: function(/*dojo/data/Item*/ /*===== item =====*/){
			// summary:
			//		Callback when an item has been deleted.
			// description:
			//		Note that there will also be an onChildrenChange() callback for the parent
			//		of this item.
			// tags:
			//		callback
		},

		// =======================================================================
		// Events from data store

		onNewItem: function(/* dojo/data/Item */ item, /* Object */ parentInfo){
			// summary:
			//		Handler for when new items appear in the store, either from a drop operation
			//		or some other way.   Updates the tree view (if necessary).
			// description:
			//		If the new item is a child of an existing item,
			//		calls onChildrenChange() with the new list of children
			//		for that existing item.
			//
			// tags:
			//		extension

			// We only care about the new item if it has a parent that corresponds to a TreeNode
			// we are currently displaying
			if(!parentInfo){
				return;
			}

			// Call onChildrenChange() on parent (ie, existing) item with new list of children
			// In the common case, the new list of children is simply parentInfo.newValue or
			// [ parentInfo.newValue ], although if items in the store has multiple
			// child attributes (see `childrenAttr`), then it's a superset of parentInfo.newValue,
			// so call getChildren() to be sure to get right answer.
			this.getChildren(parentInfo.item, lang.hitch(this, function(children){
				this.onChildrenChange(parentInfo.item, children);
			}));
		},

		onDeleteItem: function(/*Object*/ item){
			// summary:
			//		Handler for delete notifications from underlying store
			this.onDelete(item);
		},

		onSetItem: function(item, attribute /*===== , oldValue, newValue =====*/){
			// summary:
			//		Updates the tree view according to changes in the data store.
			// description:
			//		Handles updates to an item's children by calling onChildrenChange(), and
			//		other updates to an item by calling onChange().
			//
			//		See `onNewItem` for more details on handling updates to an item's children.
			// item: Item
			// attribute: attribute-name-string
			// oldValue: Object|Array
			// newValue: Object|Array
			// tags:
			//		extension

			if(array.indexOf(this.childrenAttrs, attribute) != -1){
				// item's children list changed
				this.getChildren(item, lang.hitch(this, function(children){
					// See comments in onNewItem() about calling getChildren()
					this.onChildrenChange(item, children);
				}));
			}else{
				// item's label/icon/etc. changed.
				this.onChange(item);
			}
		}
	});
});

},
'dijit/tree/ForestStoreModel':function(){
define("dijit/tree/ForestStoreModel", [
	"dojo/_base/array", // array.indexOf array.some
	"dojo/_base/declare", // declare
	"dojo/_base/kernel", // global
	"dojo/_base/lang", // lang.hitch
	"./TreeStoreModel"
], function(array, declare, kernel, lang, TreeStoreModel){

// module:
//		dijit/tree/ForestStoreModel

return declare("dijit.tree.ForestStoreModel", TreeStoreModel, {
	// summary:
	//		Interface between a dijit.Tree and a dojo.data store that doesn't have a root item,
	//		a.k.a. a store that has multiple "top level" items.
	//
	// description:
	//		Use this class to wrap a dojo.data store, making all the items matching the specified query
	//		appear as children of a fabricated "root item".  If no query is specified then all the
	//		items returned by fetch() on the underlying store become children of the root item.
	//		This class allows dijit.Tree to assume a single root item, even if the store doesn't have one.
	//
	//		When using this class the developer must override a number of methods according to their app and
	//		data, including:
	//
	//		- onNewRootItem
	//		- onAddToRoot
	//		- onLeaveRoot
	//		- onNewItem
	//		- onSetItem

	// Parameters to constructor

	// rootId: String
	//		ID of fabricated root item
	rootId: "$root$",

	// rootLabel: String
	//		Label of fabricated root item
	rootLabel: "ROOT",

	// query: String
	//		Specifies the set of children of the root item.
	// example:
	//	|	{type:'continent'}
	query: null,

	// End of parameters to constructor

	constructor: function(params){
		// summary:
		//		Sets up variables, etc.
		// tags:
		//		private

		// Make dummy root item
		this.root = {
			store: this,
			root: true,
			id: params.rootId,
			label: params.rootLabel,
			children: params.rootChildren	// optional param
		};
	},

	// =======================================================================
	// Methods for traversing hierarchy

	mayHaveChildren: function(/*dojo/data/Item*/ item){
		// summary:
		//		Tells if an item has or may have children.  Implementing logic here
		//		avoids showing +/- expando icon for nodes that we know don't have children.
		//		(For efficiency reasons we may not want to check if an element actually
		//		has children until user clicks the expando node)
		// tags:
		//		extension
		return item === this.root || this.inherited(arguments);
	},

	getChildren: function(/*dojo/data/Item*/ parentItem, /*function(items)*/ callback, /*function*/ onError){
		// summary:
		//		Calls onComplete() with array of child items of given parent item, all loaded.
		if(parentItem === this.root){
			if(this.root.children){
				// already loaded, just return
				callback(this.root.children);
			}else{
				this.store.fetch({
					query: this.query,
					onComplete: lang.hitch(this, function(items){
						this.root.children = items;
						callback(items);
					}),
					onError: onError
				});
			}
		}else{
			this.inherited(arguments);
		}
	},

	// =======================================================================
	// Inspecting items

	isItem: function(/* anything */ something){
		return (something === this.root) ? true : this.inherited(arguments);
	},

	fetchItemByIdentity: function(/* object */ keywordArgs){
		if(keywordArgs.identity == this.root.id){
			var scope = keywordArgs.scope || kernel.global;
			if(keywordArgs.onItem){
				keywordArgs.onItem.call(scope, this.root);
			}
		}else{
			this.inherited(arguments);
		}
	},

	getIdentity: function(/* item */ item){
		return (item === this.root) ? this.root.id : this.inherited(arguments);
	},

	getLabel: function(/* item */ item){
		return	(item === this.root) ? this.root.label : this.inherited(arguments);
	},

	// =======================================================================
	// Write interface

	newItem: function(/* dijit/tree/dndSource.__Item */ args, /*Item*/ parent, /*int?*/ insertIndex){
		// summary:
		//		Creates a new item.   See dojo/data/api/Write for details on args.
		//		Used in drag & drop when item from external source dropped onto tree.
		if(parent === this.root){
			this.onNewRootItem(args);
			return this.store.newItem(args);
		}else{
			return this.inherited(arguments);
		}
	},

	onNewRootItem: function(/* dijit/tree/dndSource.__Item */ /*===== args =====*/){
		// summary:
		//		User can override this method to modify a new element that's being
		//		added to the root of the tree, for example to add a flag like root=true
	},

	pasteItem: function(/*Item*/ childItem, /*Item*/ oldParentItem, /*Item*/ newParentItem, /*Boolean*/ bCopy, /*int?*/ insertIndex){
		// summary:
		//		Move or copy an item from one parent item to another.
		//		Used in drag & drop
		if(oldParentItem === this.root){
			if(!bCopy){
				// It's onLeaveRoot()'s responsibility to modify the item so it no longer matches
				// this.query... thus triggering an onChildrenChange() event to notify the Tree
				// that this element is no longer a child of the root node
				this.onLeaveRoot(childItem);
			}
		}
		this.inherited(arguments, [childItem,
			oldParentItem === this.root ? null : oldParentItem,
			newParentItem === this.root ? null : newParentItem,
			bCopy,
			insertIndex
		]);
		if(newParentItem === this.root){
			// It's onAddToRoot()'s responsibility to modify the item so it matches
			// this.query... thus triggering an onChildrenChange() event to notify the Tree
			// that this element is now a child of the root node
			this.onAddToRoot(childItem);
		}
	},

	// =======================================================================
	// Handling for top level children

	onAddToRoot: function(/* item */ item){
		// summary:
		//		Called when item added to root of tree; user must override this method
		//		to modify the item so that it matches the query for top level items
		// example:
		//	|	store.setValue(item, "root", true);
		// tags:
		//		extension
		console.log(this, ": item ", item, " added to root");
	},

	onLeaveRoot: function(/* item */ item){
		// summary:
		//		Called when item removed from root of tree; user must override this method
		//		to modify the item so it doesn't match the query for top level items
		// example:
		//	|	store.unsetAttribute(item, "root");
		// tags:
		//		extension
		console.log(this, ": item ", item, " removed from root");
	},

	// =======================================================================
	// Events from data store

	_requeryTop: function(){
		// reruns the query for the children of the root node,
		// sending out an onSet notification if those children have changed
		var oldChildren = this.root.children || [];
		this.store.fetch({
			query: this.query,
			onComplete: lang.hitch(this, function(newChildren){
				this.root.children = newChildren;

				// If the list of children or the order of children has changed...
				if(oldChildren.length != newChildren.length ||
					array.some(oldChildren, function(item, idx){ return newChildren[idx] != item;})){
					this.onChildrenChange(this.root, newChildren);
				}
			})
		});
	},

	onNewItem: function(/* dojo/data/api/Item */ item, /* Object */ parentInfo){
		// summary:
		//		Handler for when new items appear in the store.  Developers should override this
		//		method to be more efficient based on their app/data.
		// description:
		//		Note that the default implementation requeries the top level items every time
		//		a new item is created, since any new item could be a top level item (even in
		//		addition to being a child of another item, since items can have multiple parents).
		//
		//		If developers can detect which items are possible top level items (based on the item and the
		//		parentInfo parameters), they should override this method to only call _requeryTop() for top
		//		level items.  Often all top level items have parentInfo==null, but
		//		that will depend on which store you use and what your data is like.
		// tags:
		//		extension
		this._requeryTop();

		this.inherited(arguments);
	},

	onDeleteItem: function(/*Object*/ item){
		// summary:
		//		Handler for delete notifications from underlying store

		// check if this was a child of root, and if so send notification that root's children
		// have changed
		if(array.indexOf(this.root.children, item) != -1){
			this._requeryTop();
		}

		this.inherited(arguments);
	},

	onSetItem: function(/* item */ item,
					/* attribute-name-string */ attribute,
					/* Object|Array */ oldValue,
					/* Object|Array */ newValue){
		// summary:
		//		Updates the tree view according to changes to an item in the data store.
		//		Developers should override this method to be more efficient based on their app/data.
		// description:
		//		Handles updates to an item's children by calling onChildrenChange(), and
		//		other updates to an item by calling onChange().
		//
		//		Also, any change to any item re-executes the query for the tree's top-level items,
		//		since this modified item may have started/stopped matching the query for top level items.
		//
		//		If possible, developers should override this function to only call _requeryTop() when
		//		the change to the item has caused it to stop/start being a top level item in the tree.
		// tags:
		//		extension

		this._requeryTop();
		this.inherited(arguments);
	}

});

});

},
'epi-cms/widget/_ContentTreeNode':function(){
require({cache:{
'url:epi-cms/widget/templates/_ContentTreeNode.html':"﻿<div class=\"dijitTreeNode\" role=\"presentation\">\n    <a href=\"#\" tabindex=\"-1\" data-dojo-attach-point=\"rowNode\" class=\"dijitTreeRow\" role=\"presentation\">\n        <div data-dojo-attach-point=\"indentNode\" class=\"dijitInline\"></div>\n        <span data-dojo-attach-point=\"contentNode\" class=\"dijitTreeContent\" role=\"presentation\">\n            <span data-dojo-attach-point=\"expandoNode\" class=\"dijitTreeExpando\" role=\"presentation\"></span>\n            <span data-dojo-attach-point=\"expandoNodeText\" class=\"dijitExpandoText\" role=\"presentation\"></span>\n            <span data-dojo-attach-point=\"iconNode\" class=\"dijitIcon dijitTreeIcon\" role=\"presentation\"></span>            \n            <span data-dojo-attach-point=\"thumbnailNode\" class=\"epi-thumbnailIcon\" role=\"presentation\"></span>\n            <span data-dojo-attach-point=\"extraIconsContainer\" class=\"epi-extraIconsContainer\" role=\"presentation\">\n                <span data-dojo-attach-point=\"iconNodeLanguage\" class=\"epi-extraIcon epi-pt-currentLanguageMissing\" role=\"presentation\">&nbsp;</span>\n            </span>\n            <span data-dojo-attach-point=\"labelNode\" class=\"dijitTreeLabel\" role=\"treeitem\" tabindex=\"-1\" aria-selected=\"false\"></span>\n        </span>\n    </a>\n    <div data-dojo-attach-point=\"containerNode\" class=\"dijitTreeContainer\" role=\"presentation\" style=\"display: none;\"></div>\n</div>"}});
﻿define("epi-cms/widget/_ContentTreeNode", [
// dojo
    "dojo/_base/declare",

    "dojo/dom-class",
    "dojo/dom-construct",
    "dojo/string",

    // dijit
    "dijit/Tree",
    "dijit/_TemplatedMixin",
    // dojox
    "dojox/html/ellipsis",
    // epi
    "epi-cms/widget/_ContentTreeNodeMixin",
    // template
    "dojo/text!./templates/_ContentTreeNode.html"
],

function (
// dojo
    declare,
    domClass,
    domConstruct,
    string,

    // dijit
    Tree,
    _TemplatedMixin,
    // dojox
    htmlEllipsis,
    // epi
    _ContentTreeNodeMixin,
    // template
    templateString
) {

    return declare([Tree._TreeNode, _TemplatedMixin, _ContentTreeNodeMixin], {
        // summary:
        //      A customized treenode for page tree
        // description:
        //      Redefine template string. Use <a> tag to be able to drag in IE
        // tags:
        //      internal

        templateString: templateString,

        buildRendering: function () {
            this.inherited(arguments);

            domClass.add(this.labelNode, "dojoxEllipsis");
        },

        _setItemAttr: function (item) {

            this._set("item", item);
            this.set("dndData", item);

            if (item && item.thumbnailUrl) {
                var image = domConstruct.create("img", {
                    src: item.thumbnailUrl + string.substitute("?${0}", [new Date().getTime()]),
                    "class": "epi-thumbnail"
                });
                this.thumbnailNode.appendChild(image);
                domClass.add(this.contentNode, "epi-mediaContent");
            }

            this.inherited(arguments);
        }

    });

});

},
'dojox/html/ellipsis':function(){
define("dojox/html/ellipsis",["dojo/_base/kernel", "dojo/_base/lang", "dojo/_base/array", "dojo/_base/Color", "dojo/colors"], function(d){
	/*=====
	return {
		// summary:
		//		offers cross-browser support for text-overflow: ellipsis
		// description:
		//		Add "dojoxEllipsis" on any node that you want to ellipsis-ize. In order to function properly,
		//		the node with the dojoxEllipsis class set on it should be a child of a node with a defined width.
		//		It should also be a block-level element (i.e. `<div>`) - it will not work on td elements.
		//		NOTE: When using the dojoxEllipsis class within tables, the table needs to have the table-layout: fixed style
	};
	=====*/
	
	if(d.isFF < 7){ //TODO: feature detect text-overflow in computed style?
		// The delay (in ms) to wait so that we don't keep querying when many
		// changes happen at once - set config "dojoxFFEllipsisDelay" if you
		// want a different value
		var delay = 1;
		if("dojoxFFEllipsisDelay" in d.config){
			delay = Number(d.config.dojoxFFEllipsisDelay);
			if(isNaN(delay)){
				delay = 1;
			}
		}
		try{
			var createXULEllipsis = (function(){
				// Create our stub XUL elements for cloning later
				// NOTE: this no longer works as of FF 4.0:
				// https://developer.mozilla.org/En/Firefox_4_for_developers#Remote_XUL_support_removed
				var sNS = 'http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul';
				var xml = document.createElementNS(sNS, 'window');
				var label = document.createElementNS(sNS, 'description');
				label.setAttribute('crop', 'end');
				xml.appendChild(label);

				return function(/* Node */ n){
					// Summary:
					//		Given a node, it creates the XUL and sets its
					//		content so that it will have an ellipsis
					var x = xml.cloneNode(true);
					x.firstChild.setAttribute('value', n.textContent);
					n.innerHTML = '';
					n.appendChild(x);
				};
			})();
		}catch(e){}
		
		// Create our iframe elements for cloning later
		var create = d.create;
		var dd = d.doc;
		var dp = d.place;
		var iFrame = create("iframe", {className: "dojoxEllipsisIFrame",
					src: "javascript:'<html><head><script>if(\"loadFirebugConsole\" in window){window.loadFirebugConsole();}</script></head><body></body></html>'", style: {display: "none"}});
		var rollRange = function(/* W3C Range */ r, /* int? */ cnt){
			// summary:
			//		Rolls the given range back one character from the end
			// r: W3C Range
			//		The range to roll back
			// cnt: int?
			//		An optional number of times to roll back (defaults 1)
			if(r.collapsed){
				// Do nothing - we are already collapsed
				return;
			}
			if(cnt > 0){
				do{
					rollRange(r);
					cnt--;
				}while(cnt);
				return;
			}
			if(r.endContainer.nodeType == 3 && r.endOffset > 0){
				r.setEnd(r.endContainer, r.endOffset - 1);
			}else if(r.endContainer.nodeType == 3){
				r.setEndBefore(r.endContainer);
				rollRange(r);
				return;
			}else if(r.endOffset && r.endContainer.childNodes.length >= r.endOffset){
				var nCont = r.endContainer.childNodes[r.endOffset - 1];
				if(nCont.nodeType == 3){
					r.setEnd(nCont, nCont.length - 1);
				}else if(nCont.childNodes.length){
					r.setEnd(nCont, nCont.childNodes.length);
					rollRange(r);
					return;
				}else{
					r.setEndBefore(nCont);
					rollRange(r);
					return;
				}
			}else{
				r.setEndBefore(r.endContainer);
				rollRange(r);
				return;
			}
		};
		var createIFrameEllipsis = function(/* Node */ n){
			// summary:
			//		Given a node, it creates an iframe and and ellipsis div and
			//		sets up the connections so that they will work correctly.
			//		This function is used when createXULEllipsis is not able
			//		to be used (because there is markup within the node) - it's
			//		a bit slower, but does the trick
			var c = create("div", {className: "dojoxEllipsisContainer"});
			var e = create("div", {className: "dojoxEllipsisShown", style: {display: "none"}});
			n.parentNode.replaceChild(c, n);
			c.appendChild(n);
			c.appendChild(e);
			var i = iFrame.cloneNode(true);
			var ns = n.style;
			var es = e.style;
			var ranges;
			var resizeNode = function(){
				ns.display = "";
				es.display = "none";
				if(n.scrollWidth <= n.offsetWidth){ return; }
				var r = dd.createRange();
				r.selectNodeContents(n);
				ns.display = "none";
				es.display = "";
				var done = false;
				do{
					var numRolls = 1;
					dp(r.cloneContents(), e, "only");
					var sw = e.scrollWidth, ow = e.offsetWidth;
					done = (sw <= ow);
					var pct = (1 - ((ow * 1) / sw));
					if(pct > 0){
						numRolls = Math.max(Math.round(e.textContent.length * pct) - 1, 1);
					}
					rollRange(r, numRolls);
				}while(!r.collapsed && !done);
			};
			i.onload = function(){
				i.contentWindow.onresize = resizeNode;
				resizeNode();
			};
			c.appendChild(i);
		};

		// Function for updating the ellipsis
		var hc = d.hasClass;
		var doc = d.doc;
		var s, fn, opt;
		if(doc.querySelectorAll){
			s = doc;
			fn = "querySelectorAll";
			opt = ".dojoxEllipsis";
		}else if(doc.getElementsByClassName){
			s = doc;
			fn = "getElementsByClassName";
			opt = "dojoxEllipsis";
		}else{
			s = d;
			fn = "query";
			opt = ".dojoxEllipsis";
		}
		fx = function(){
			d.forEach(s[fn].apply(s, [opt]), function(n){
				if(!n || n._djx_ellipsis_done){ return; }
				n._djx_ellipsis_done = true;
				if(createXULEllipsis && n.textContent == n.innerHTML && !hc(n, "dojoxEllipsisSelectable")){
					// We can do the faster XUL version, instead of calculating
					createXULEllipsis(n);
				}else{
					createIFrameEllipsis(n);
				}
			});
		};
		
		d.addOnLoad(function(){
			// Apply our initial stuff
			var t = null;
			var c = null;
			var connFx = function(){
				if(c){
					// disconnect us - so we don't fire anymore
					d.disconnect(c);
					c = null;
				}
				if(t){ clearTimeout(t); }
				t = setTimeout(function(){
					t = null;
					fx();
					// Connect to the modified function so that we can catch
					// our next change
					c = d.connect(d.body(), "DOMSubtreeModified", connFx);
				}, delay);
			};
			connFx();
		});
	}
});

},
'epi-cms/widget/_ContentTreeNodeMixin':function(){
﻿define("epi-cms/widget/_ContentTreeNodeMixin", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/when",

    "dojo/dom-attr",
    "dojo/dom-class",
    "dojo/dom-construct",

    "epi-cms/ContentLanguageHelper"
],

function (
    declare,
    lang,
    when,

    domAttr,
    domClass,
    domConstruct,

    ContentLanguageHelper
) {

    return declare(null, {
        // summary:
        //      Stubs to handler common functions for tree
        // tags:
        //      internal mixin

        _missingLanguageCssClass: "epi-ct-missingLanguageRow",

        // =======================================================================
        // Protected overridden functions

        _updateItemClasses: function (/*Item*/item) {
            // summary:
            //      Override dijit/_TreeNode._updateItemClasses function
            //
            //      Set appropriate CSS classes for icon and label dom node
            //      (used to allow for item updates to change respective CSS)
            // tags:
            //      protected, override

            this.inherited(arguments);

            this._updateLayout();
        },

        _updateLayout: function () {
            // summary:
            //      Override dijit/_TreeNode._updateLayout function
            //
            //      Set appropriate CSS classes for this.domNode
            // tags:
            //      protected, override

            this.inherited(arguments);

            this._updateLanguageIndicator();
            this._updateIndividualLayout();
        },

        // =======================================================================
        // Protected functions

        _updateIndividualLayout: function () {
            // summary:
            //      Update tree node layout for individual case
            // tags:
            //      protected, extension
        },

        _addLanguageIndicator: function () {
            // summary:
            //      Setup missing language branch indicator UI
            // tags:
            //      protected

            var missingLanguageBranch = this.item.missingLanguageBranch;

            if (this.iconNodeLanguage) {
                this.iconNodeLanguage.innerHTML = missingLanguageBranch.language || "";
                when(ContentLanguageHelper.getMissingLanguageMessage(this.item), lang.hitch(this, function (message) {
                    domAttr.set(this.iconNodeLanguage, "title", message || "");
                }));
            }

            domClass.add(this.rowNode, this._missingLanguageCssClass);
        },

        _removeLanguageIndicator: function () {
            // summary:
            //      Remove all settings for language indicator
            // tags:
            //      protected

            if (this.iconNodeLanguage) {
                domConstruct.empty(this.iconNodeLanguage);
                domAttr.remove(this.iconNodeLanguage, "title");
            }
            domClass.remove(this.rowNode, this._missingLanguageCssClass);
        },

        _updateLanguageIndicator: function () {
            // summary:
            //      Set language indicator for the tree node
            // tags:
            //      protected

            if (!this._isTypeOfRoot() && this.item.missingLanguageBranch) {
                this._addLanguageIndicator();
            } else {
                this._removeLanguageIndicator();
            }

        },

        _isTypeOfRoot: function () {
            // summary:
            //      Verifies the current tree node item data object is type of root or not
            // returns: [Boolean]
            // tags:
            //      protected

            if (!this.tree.model || typeof this.tree.model.isTypeOfRoot !== "function") {
                return false;
            }

            return this.tree.model.isTypeOfRoot(this.item);
        }

    });

});

},
'epi-cms/ContentLanguageHelper':function(){
define("epi-cms/ContentLanguageHelper", [
    "dojo/_base/lang",
    "dojo/_base/declare",
    "dojo/_base/Deferred",
    "dojo/promise/all",

    "epi/dependency",
    "epi/shell/TypeDescriptorManager"
], function (lang, declare, Deferred, all, dependency, TypeDescriptorManager) {

    return {
        // summary:
        //      Helper methods for working with content languages
        //
        // tags:
        //      internal

        getMissingLanguageMessage: function (contentItem) {
            // summary:
            //      Gets a user friendly message explaining why the language is missing
            // contentItem: Object
            //      The content item to read the missing language information from
            // tags:
            //      public

            if (!contentItem || !contentItem.missingLanguageBranch) {
                return "";
            }

            this._languageStore = this._languageStore || dependency.resolve("epi.storeregistry").get("epi.cms.language");

            var def = new Deferred();

            var missingLanguageBranch = contentItem.missingLanguageBranch;

            all([this._languageStore.get(missingLanguageBranch.preferredLanguage),
                this._languageStore.get(missingLanguageBranch.language)
            ]).then(lang.hitch(this, function (languages) {

                var reason = missingLanguageBranch.reason;
                var resourceKey = "languagenotifications.";
                if (reason === 6) {
                    // None
                    resourceKey += "missing";
                } else if (reason === 3 || reason === 5) {
                    //Replacement or ReplacementFallback
                    resourceKey += "replacement";
                } else if (reason === 4) {
                    //Fallback
                    resourceKey += "fallback";
                }

                var resourceValue = TypeDescriptorManager.getResourceValue(contentItem.typeIdentifier, resourceKey);
                resourceValue = lang.replace(resourceValue, [languages[0].name, languages[1].name]);

                def.resolve(resourceValue);

            }), function () {
                def.resolve("");
            });
            return def;
        }
    };
});

},
'url:epi-cms/widget/templates/_ContentTreeNode.html':"﻿<div class=\"dijitTreeNode\" role=\"presentation\">\n    <a href=\"#\" tabindex=\"-1\" data-dojo-attach-point=\"rowNode\" class=\"dijitTreeRow\" role=\"presentation\">\n        <div data-dojo-attach-point=\"indentNode\" class=\"dijitInline\"></div>\n        <span data-dojo-attach-point=\"contentNode\" class=\"dijitTreeContent\" role=\"presentation\">\n            <span data-dojo-attach-point=\"expandoNode\" class=\"dijitTreeExpando\" role=\"presentation\"></span>\n            <span data-dojo-attach-point=\"expandoNodeText\" class=\"dijitExpandoText\" role=\"presentation\"></span>\n            <span data-dojo-attach-point=\"iconNode\" class=\"dijitIcon dijitTreeIcon\" role=\"presentation\"></span>            \n            <span data-dojo-attach-point=\"thumbnailNode\" class=\"epi-thumbnailIcon\" role=\"presentation\"></span>\n            <span data-dojo-attach-point=\"extraIconsContainer\" class=\"epi-extraIconsContainer\" role=\"presentation\">\n                <span data-dojo-attach-point=\"iconNodeLanguage\" class=\"epi-extraIcon epi-pt-currentLanguageMissing\" role=\"presentation\">&nbsp;</span>\n            </span>\n            <span data-dojo-attach-point=\"labelNode\" class=\"dijitTreeLabel\" role=\"treeitem\" tabindex=\"-1\" aria-selected=\"false\"></span>\n        </span>\n    </a>\n    <div data-dojo-attach-point=\"containerNode\" class=\"dijitTreeContainer\" role=\"presentation\" style=\"display: none;\"></div>\n</div>",
'epi-cms/widget/ContentTreeModelConfirmation':function(){
define("epi-cms/widget/ContentTreeModelConfirmation", [
// dojo
    "dojo/_base/declare",
    "dojo/_base/lang",

    "dojo/aspect",
    "dojo/Deferred",
    "dojo/string",
    "dojo/when",
    "dojo/_base/json",

    // dojox
    "dojox/html/entities",

    // epi
    "epi/dependency",
    "epi/string",

    "epi/shell/TypeDescriptorManager",
    "epi/shell/widget/dialog/Alert",
    "epi/shell/widget/dialog/Confirmation",

    "epi-cms/contentediting/ContentActionSupport",
    "epi-cms/widget/sharedContentDialogHandler",
    // resources
    "dojo/i18n!epi/nls/episerver.cms.components.pagetree",
    "dojo/i18n!epi/nls/episerver.cms.components.project"
],

function (
// dojo
    declare,
    lang,

    aspect,
    Deferred,
    string,
    when,
    json,

    // dojox
    htmlEntities,

    // epi
    dependency,
    epistring,

    TypeDescriptorManager,
    Alert,
    Confirmation,

    ContentActionSupport,
    sharedContentDialogHandler,
    res,
    projectResources
) {

    return function (model) {
        // summary:
        //      Attaches confirmation to the pasteItem method of a dijit/tree.model instance.
        //      Looks at the paste parameters to show a confirmation before delting or moving items.
        //
        // tags:
        //      internal xproduct

        // NOTE: This pattern isn't really that nice, since it alters the aspect.around actually alters the underlying store.
        //      It's probably a better idea to use delegation, but that requires us to -implement set() and _set() to get
        //      model property changes updated on the underlying model and not the delegating facade.

        var _isFunction = function (object) {
            return object && typeof object == "function";
        };

        var _showNotification = function (message, callback) {
            // summary:
            //      Uses the alert dialog implementation to notify with the supplied message
            var dialog = new Alert({
                description: epistring.toHTML(message),
                onAction: function () {
                    if (_isFunction(callback)) {
                        callback();
                    }
                }
            });

            dialog.show();
        };

        var _showConfirmation = function (title, confirmationMessage) {
            // summary:
            //      Wrap epi/shell/widget/dialog/Confirmation for short type
            //      and return deferred object
            // confirmQuestion:
            //      String / Deferred : Text to display on dialog
            // tags:
            //		private
            var deferred = new Deferred();

            when(confirmationMessage, function (message) {
                var dialog = new Confirmation({
                    description: epistring.toHTML(message),
                    title: title,
                    onAction: function (confirmed) {
                        confirmed ? deferred.resolve() : deferred.cancel();
                    }
                });
                dialog.show();
            });

            return deferred;
        };

        var onResponse = function (result, callback) {
            // Checks the result of a call to pasteItem and shows error messages if result indicates a failure
            when(result,
                function (serverResponse) {
                    if (serverResponse && (serverResponse.message || serverResponse.statusDescription)) {
                        var message = serverResponse.message || serverResponse.statusDescription;
                        _showNotification(message, callback);
                    } else {
                        if (_isFunction(callback)) {
                            callback();
                        }
                    }
                },
                function (e) {
                    _showNotification(htmlEntities.encode(json.fromJson(e.xhr.responseText)));
                });
            return result;
        };

        var pasteItems = function (sourceItems, newParent, copy, sortIndex, originalMethod, originalArguments) {
            var source = sourceItems[0],
                localizationPrefix = sourceItems.length > 1 ? "move.multiple." : "move.single.";

            var moveTitle = TypeDescriptorManager.getResourceValue(source.typeIdentifier, localizationPrefix + "title");
            // Defaulting confirmation to an affirmative response in case we don't need a confirmation
            var confirmation = null;

            if (newParent.isWastebasket) {
                confirmation = sharedContentDialogHandler({ contentItems: sourceItems });
            } else if (sortIndex !== undefined && (newParent.properties.pageChildOrderRule !== ContentActionSupport.sortOrder.Index)) {

                // Page is copied/moved and sorted and the new parent sorting rule must be changed
                var message, title;
                if (copy) {
                    localizationPrefix = sourceItems.length > 1 ? "copy.multiple." : "copy.single.";
                    title = TypeDescriptorManager.getResourceValue(source.typeIdentifier, localizationPrefix + "title");
                    message = string.substitute(res.changepeerorderconfirmationcopy, [model.getLabel(newParent)]);
                } else {
                    // move
                    title = moveTitle;
                    message = string.substitute(res.changepeerorderconfirmation, [model.getLabel(newParent)]);
                }
                confirmation = _showConfirmation(title, message);
            } else if (!copy) {
                if (sourceItems.some(function (item) {
                    return !model.canCut(item);
                })) {
                    _showNotification(TypeDescriptorManager.getResourceValue(source.typeIdentifier, "moverequiresdeleteaccess"), function () {
                        return false;
                    });
                    return false;
                }

                var formatTemplate = "{0}<br/><br/>{1}",
                    confirmationText = lang.replace(TypeDescriptorManager.getResourceValue(source.typeIdentifier, localizationPrefix + "confirmation"), [sourceItems.length]),
                    description = lang.replace(formatTemplate, [
                        confirmationText,
                        TypeDescriptorManager.getResourceValue(source.typeIdentifier, "movedescription")
                    ]);

                var projectService = dependency.resolve("epi.cms.ProjectService");
                if (projectService.isProjectModeEnabled) {
                    description = lang.replace(formatTemplate, [description, projectResources.moveconfirmation]);
                }

                // Plain move
                confirmation = _showConfirmation(moveTitle, description);

            } else if (!model.canCopy(source)) {
                _showNotification(TypeDescriptorManager.getResourceValue(source.typeIdentifier, "copynotsupported"));
                return false;
            }

            return when(confirmation, function () {
                // Call pasteItem of the model
                return originalMethod.apply(model, originalArguments);
            }, function () {
                return false;
            });
        };

        aspect.around(model, "pasteItem", function (originalMethod) {
            return function (source, oldParent, newParent, copy, sortIndex) {
                return pasteItems([source], newParent, copy, sortIndex, originalMethod, arguments);
            };
        });

        aspect.around(model, "pasteItems", function (originalMethod) {
            return function (sourceItems, newParent, copy, sortIndex) {
                return pasteItems(sourceItems, newParent, copy, sortIndex, originalMethod, arguments);
            };
        });

        aspect.after(model, "pasteItem", function (result) {
            return result && onResponse(result);
        });

        aspect.after(model, "remove", function (result) {
            return result && onResponse(result);
        });

        return model;
    };

});

},
'epi-cms/widget/sharedContentDialogHandler':function(){
define("epi-cms/widget/sharedContentDialogHandler", [
    "dojo/_base/lang",
    "dojo/Deferred",
    "dojox/html/entities",
    "epi",
    "epi/shell/TypeDescriptorManager",
    "epi/shell/widget/dialog/Dialog",
    "epi/shell/widget/DelayableStandby",
    "epi-cms/widget/ContentReferences",
    // Resources
    "dojo/i18n!epi/nls/episerver.cms.widget.contentreferences"
], function (
    lang,
    Deferred,
    entities,
    epi,
    TypeDescriptorManager,
    Dialog,
    Standby,
    ContentReferences,
    // Resources
    resources
) {

    var handler = function (settings) {
        // tags:
        //      internal

        var deferred = new Deferred(),
            contentItems = settings.contentItems,
            contentData = contentItems[0],
            standby = new Standby({ target: document.body, zIndex: 999 }).placeAt(document.body);

        // Display a the standby widget in case the content references takes a long time to query the store.
        standby.show(50, 0);

        var mode = settings.mode || handler.mode.confirm,
            isConfirm = mode === handler.mode.confirm,
            content = new ContentReferences({
                contentItems: contentItems,
                mode: mode
            });

        var watch = content.watch("numberOfReferences", function (propertyName, oldValue, newValue) {

            // Remove the watch so we don't leak memory or create additional dialogs on refresh.
            watch.remove();

            var isSingleItem = contentItems.length === 1;
            var description;
            var descriptionResourceKey = isSingleItem ? "movetotrashsingleitemdescription" : "movetotrashdescription";

            if (isConfirm) {
                // The description is determined by whether the content being deleted has children.
                description = TypeDescriptorManager.getResourceValue(contentData.typeIdentifier, descriptionResourceKey);
                description = lang.replace(description, [isSingleItem ? entities.encode(contentData.name) : contentItems.length]);

            } else {
                description = TypeDescriptorManager.getResourceValue(contentData.typeIdentifier, "referencesdescription");
                description = lang.replace(description, [entities.encode(contentData.name)]);
            }

            // If not passed with settings, then the heading is determined by the dialog mode.
            var title = settings.title;
            if (!title) {
                var translationKey = isConfirm ? (isSingleItem ? "movetotrashsingleitem" : "movetotrash") : "references";
                title = TypeDescriptorManager.getResourceValue(contentData.typeIdentifier, translationKey);
            }

            var dialog = new Dialog({
                focusActionsOnLoad: true,
                defaultActionsVisible: isConfirm,
                dialogClass: "epi-dialog-contentReferences",
                title: title,
                description: description,
                cancelActionText: epi.resources.action.cancel,
                confirmActionText: newValue === 0 ? resources.buttons.movetotrash : resources.buttons.movetotrashanyway,
                setFocusOnConfirmButton: newValue === 0
            });

            if (!isConfirm) {
                dialog.definitionConsumer.add({
                    name: "close",
                    label: epi.resources.action.close,
                    action: dialog.onCancel
                });
            }

            dialog.connect(dialog, "onExecute", deferred.resolve);
            dialog.connect(dialog, "onCancel", deferred.cancel);

            dialog.set("content", content);

            // Hide the dialog when a reference is viewed.
            var handle = content.on("viewReference", function () {
                dialog.hide();
                handle.remove();
            });

            standby.hide();

            // Show the dialog when the content is ready
            var refreshHandle = content.grid.on("dgrid-refresh-complete", function () {
                refreshHandle.remove();
                dialog.show();
            }.bind(this));
        });

        content.startup();

        return deferred.promise;
    };

    handler.mode = {
        confirm: "confirm",
        show: "show"
    };

    return handler;
});

},
'epi/shell/widget/DelayableStandby':function(){
﻿define("epi/shell/widget/DelayableStandby", [
// Dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojox/widget/Standby"
],

function (
// Dojo
    declare,
    lang,
    Standby
) {

    return declare([Standby], {
        // summary:
        //    Represent a Standby widget, which would be display in a delay time.
        //
        // tags:
        //    internal

        // delayTime: [public] Integer
        //    Delay time before displaying standby, in millisecond.
        delayTime: 300,


        // showTimeOut: Integer
        //      Timeout in milliseconds when to hide standby if it's shown. Set to zero or negative
        //      to show standby until closed
        showTimeOut: 30e3,

        // color:	String
        //		The color to set the overlay.  Should be in #XXXXXX form.
        //		Default color is set to white.
        color: "#FFFFFF",

        // Id of settimeout when show loading indicator
        _timeoutId: -1,

        // _isShowing: Boolean
        //      Indicates if the standby is showing or is about to show
        _isShowing: false,

        destroy: function () {
            this._clearTimeout();

            this.inherited(arguments);
        },

        show: function (delayTime, showTimeOut) {
            // summary:
            //		Show the standby, after a delayTime
            // tags:
            //		public

            var params = arguments;
            this._clearTimeout();

            this._isShowing = true;

            if (typeof delayTime !== undefined && delayTime === 0) {
                this.inherited(params);
            } else {
                this._timeoutId = setTimeout(lang.hitch(this, function () {

                    var alsoHideAfter = showTimeOut || this.showTimeOut;

                    // create another timeout to hide the standby, so we don't get stuck with it forever
                    if (alsoHideAfter >= 0) {
                        this._timeoutId = setTimeout(lang.hitch(this, function () {
                            this.hide();
                        }), alsoHideAfter);
                    }

                    // call inherited show method, this has to be the original arguments, otherwise we loose context
                    this.inherited(params);

                }), delayTime || this.delayTime);
            }
        },

        hide: function () {
            // summary:
            //		Hide the standby
            // tags:
            //		public

            this._isShowing = false;
            this._clearTimeout();
            this.inherited(arguments);
        },

        _clearTimeout: function () {
            // summary:
            //		Clears any existing timeout process.
            // tags:
            //		private

            if (this._timeoutId >= 0) {
                clearTimeout(this._timeoutId);
            }
        },

        _getIsShowingAttr: function () {
            // summary:
            //      Indicated if the standby is showing or is about to show
            return this._isShowing;
        }
    });
});

},
'epi-cms/widget/ContentReferences':function(){
require({cache:{
'url:epi-cms/widget/templates/ContentReferences.html':"﻿<div class=\"epi-contentReferences\">\n    <div data-dojo-attach-point=\"contentReferencesNotificationBar\" class=\"epi-notificationBar epi-notificationBarWithBorders epi-notificationBarItem\">\n        <div data-dojo-attach-point=\"contentReferencesNotificationNode\" class=\"epi-notificationBarText\" ></div>\n    </div>\n    <div class=\"epi-captionPanel\" data-dojo-attach-point=\"captionPanelNode\">\n        <strong data-dojo-attach-point=\"totalLinksNode\"></strong>\n        <div class=\"epi-floatRight\">\n            <button class=\"epi-chromelessButton\"\n                    data-dojo-type=\"dijit/form/Button\"\n                    data-dojo-attach-event=\"onClick:fetchData\"\n                    data-dojo-props=\"iconClass:'epi-iconReload'\">${res.buttons.refresh}</button>\n        </div>\n    </div>\n    <div data-dojo-attach-point=\"gridNode\"></div>\n</div>\n"}});
define("epi-cms/widget/ContentReferences", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-class",
    "dojo/dom-construct",
    "dojo/dom-style",
    "dojo/keys",
    "dojo/when",
    //epi
    "epi-cms/dgrid/formatters",
    "epi/dependency",
    // dgrid
    "dgrid/tree",
    // Base class and mixins
    "./_GridWidgetBase",
    "./viewmodel/ContentReferencesViewModel",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dojo/Evented",
    "epi/shell/widget/_ModelBindingMixin",
    // Resources
    "dojo/text!./templates/ContentReferences.html",
    "dojo/i18n!epi/nls/episerver.cms.widget.contentreferences",
    // Widgets used in the template
    "dijit/form/Button"
], function (
    declare,
    lang,
    domClass,
    domConstruct,
    domStyle,
    keys,
    when,
    formatters,
    dependency,
    // dgrid
    tree,
    // Base class and mixins
    _GridWidgetBase,
    ContentReferencesViewModel,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    Evented,
    _ModelBindingMixin,
    // Resources
    template,
    resources
) {

    return declare([_GridWidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, Evented, _ModelBindingMixin], {
        // tags:
        //      internal

        // widget resources
        res: resources,

        // templateString: String
        //  Template for the widget
        templateString: template,

        storeKeyName: "epi.cms.referenced-content",

        contextChangeEvent: "dblclick",

        // trackActiveItem: [public] Boolean
        //      A flag indicating whether the item matching the current context should be highlight. The default value is false.
        trackActiveItem: false,

        // ignoreVersionWhenComparingLinks: [public] Boolean
        //      A flag indicating whether the version should be ignored when comparing content references.
        ignoreVersionWhenComparingLinks: false,

        // modelBindingMap: [protected] Object
        //      A map contain bindings from the model to properties on this object.
        modelBindingMap: {
            notification: ["notification"],
            notificationBarStyle: ["notificationBarStyle"],
            numberOfReferences: ["numberOfReferences"],
            showToolbar: ["showToolbar"],
            showGrid: ["showGrid"],
            totalLinks: ["totalLinks"]
        },

        constructor: function (params) {
            this.model = lang.mixin(new ContentReferencesViewModel(), params);
            this.own(this.model);
        },

        _setTotalLinksAttr: { node: "totalLinksNode", type: "innerText" },

        _setNotificationAttr: { node: "contentReferencesNotificationNode", type: "innerText" },

        _setNotificationBarStyleAttr: function (value) {
            // summary:
            //      Sets the color for notification bar. 1 for green, 2 for orange and yellow for everything else
            // tags:
            //      protected
            switch (value) {
                case 1:          // green
                    domClass.add(this.contentReferencesNotificationBar, "epi-statusIndicator epi-statusIndicatorOk");
                    break;
                case 2:      // orange
                    domClass.add(this.contentReferencesNotificationBar, "epi-statusIndicator epi-statusIndicatorWarning");
                    break;
            }
        },

        _setShowToolbarAttr: function (value) {
            domStyle.set(this.captionPanelNode, "display", value ? "" : "none");
        },

        _setShowGridAttr: function (value) {
            domStyle.set(this.grid.domNode, "visibility", value ? "visible" : "hidden");
        },

        postMixInProperties: function () {
            this.inherited(arguments);

            this.store = this.model.getUpdatedStore(this.store);
        },

        buildRendering: function () {
            // summary:
            //      Construct the UI for this widget. this.gridNode is initialized as a dGrid.
            // tags:
            //      protected
            var linkTemplate = this._getLinkTemplate();

            var gridSettings = lang.mixin({
                columns: {
                    name: tree({
                        renderCell: lang.hitch(this, "_renderContentItem"),
                        shouldExpand: function () {
                            return true;
                        }
                    }),
                    language: {
                        className: "epi-width10",
                        renderCell: function (obj, language, node) {
                            // If the language hasn't been specified there is no need to try to compare
                            // it with the current edit language
                            if (!language) {
                                return;
                            }
                            when(dependency.resolve("epi.shell.Profile").getContentLanguage(), function (profileContentLanguage) {
                                if (language !== profileContentLanguage) {
                                    node.textContent = language;
                                }
                            });
                        }
                    },
                    treePath: {
                        className: "epi-width50",
                        get: function (item) {
                            var treePath = item.treePath || [];
                            return treePath.join("<span class=\"epi-breadCrumbsSeparator\">&gt;</span>");
                        },
                        ellipsisNoTooltip: true
                    },
                    uri: {
                        className: "epi-width10",
                        label: " ",
                        formatter: function (value) {
                            return value ? linkTemplate : "";
                        },
                        sortable: false
                    }
                },
                store: this.store,
                query: {
                    ids: this.model.contentItems.map(function (content) {
                        return content.contentLink;
                    })
                },
                showHeader: false
            }, this.defaultGridMixin);

            this.inherited(arguments);

            this.grid = new this._gridClass(gridSettings, this.gridNode);

            // Hide the notification box if in show mode.
            domClass.toggle(this.contentReferencesNotificationBar, "dijitHidden", this.model.mode === "show");
        },

        startup: function () {
            this.inherited(arguments);

            // Setup event handling for click on the view link.
            this.grid.on(".dgrid-column-uri a:click", lang.hitch(this, "_onChangeContext"));
        },

        fetchData: function () {
            // summary:
            //      Fetches data by setting a query on the grid. A get children query will be performed on the store.
            // tags:
            //      protected

            this.grid.refresh();
        },

        _onChangeContext: function (e) {
            // summary:
            //      Change the context to the content associated with the given row.
            // tags:
            //      private
            var row = this.grid.row(e),
                newContext = { uri: row.data.uri };

            if (newContext.uri) {
                this._requestNewContext(newContext, { sender: this });

                this.emit("viewReference");
            }
        },

        _getLinkTemplate: function () {
            // summary:
            //      Returns an HTML string representing the view link.
            // tags:
            //      private
            var node = domConstruct.create("a", {
                "class": "epi-visibleLink",
                innerHTML: resources.view.label,
                title: resources.view.tooltip
            });

            return node.outerHTML;
        }
    });
});

},
'dgrid/tree':function(){
define("dgrid/tree", [
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/Deferred",
	"dojo/query",
	"dojo/on",
	"dojo/aspect",
	"./util/has-css3",
	"./Grid",
	"dojo/has!touch?./util/touch",
	"put-selector/put"
], function(declare, arrayUtil, Deferred, querySelector, on, aspect, has, Grid, touchUtil, put){

function defaultRenderExpando(level, hasChildren, expanded, object){
	// summary:
	//		Provides default implementation for column.renderExpando.
	
	var dir = this.grid.isRTL ? "right" : "left",
		cls = ".dgrid-expando-icon",
		node;
	if(hasChildren){
		cls += ".ui-icon.ui-icon-triangle-1-" + (expanded ? "se" : "e");
	}
	node = put("div" + cls + "[style=margin-" + dir + ": " +
		(level * (this.indentWidth || 9)) + "px; float: " + dir + "]");
	node.innerHTML = "&nbsp;"; // for opera to space things properly
	return node;
}

function ontransitionend(event){
	var container = this,
		height = this.style.height;
	if(height){
		// After expansion, ensure display is correct;
		// after collapse, set display to none to improve performance
		this.style.display = height == "0px" ? "none" : "block";
	}
	
	// Reset height to be auto, so future height changes (from children
	// expansions, for example), will expand to the right height.
	if(event){
		// For browsers with CSS transition support, setting the height to
		// auto or "" will cause an animation to zero height for some
		// reason, so temporarily set the transition to be zero duration
		put(this, ".dgrid-tree-resetting");
		setTimeout(function(){
			// Turn off the zero duration transition after we have let it render
			put(container, "!dgrid-tree-resetting");
		});
	}
	// Now set the height to auto
	this.style.height = "";
}

function tree(column){
	// summary:
	//		Adds tree navigation capability to a column.
	
	var originalRenderCell = column.renderCell || Grid.defaultRenderCell;
	
	var currentLevel, // tracks last rendered item level (for aspected insertRow)
		clicked; // tracks row that was clicked (for expand dblclick event handling)
		
	if(!column){ column = {}; }
	
	column.shouldExpand = column.shouldExpand || function(row, level, previouslyExpanded){
		// summary:
		//		Function called after each row is inserted to determine whether
		//		expand(rowElement, true) should be automatically called.
		//		The default implementation re-expands any rows that were expanded
		//		the last time they were rendered (if applicable).
		
		return previouslyExpanded;
	};
	
	aspect.after(column, "init", function(){
		var grid = column.grid,
			colSelector = ".dgrid-content .dgrid-column-" + column.id,
			listeners = []; // to be removed when this column is destroyed

		// Turn off automatic cleanup of empty observers, to prevent confusion
		// due to observers operating at multiple hierarchy levels.
		grid.cleanEmptyObservers = false;
		
		if(!grid.store){
			throw new Error("dgrid tree column plugin requires a store to operate.");
		}
		
		if (!column.renderExpando){
			column.renderExpando = defaultRenderExpando;
		}
		
		// Set up the event listener once and use event delegation for better memory use.
		listeners.push(grid.on(
			column.expandOn || ".dgrid-expando-icon:click," + colSelector + ":dblclick," + colSelector + ":keydown",
			function(event){
				var row = grid.row(event);	
				if((!grid.store.mayHaveChildren || grid.store.mayHaveChildren(row.data)) &&
						(event.type != "keydown" || event.keyCode == 32) &&
						!(event.type == "dblclick" && clicked && clicked.count > 1 &&
							row.id == clicked.id && event.target.className.indexOf("dgrid-expando-icon") > -1)){
					grid.expand(row);
				}
				
				// If the expando icon was clicked, update clicked object to prevent
				// potential over-triggering on dblclick (all tested browsers but IE < 9).
				if(event.target.className.indexOf("dgrid-expando-icon") > -1){
					if(clicked && clicked.id == grid.row(event).id){
						clicked.count++;
					}else{
						clicked = {
							id: grid.row(event).id,
							count: 1
						};
					}
				}
			})
		);
		
		if(has("touch")){
			// Also listen on double-taps of the cell.
			listeners.push(grid.on(touchUtil.selector(colSelector, touchUtil.dbltap),
				function(){ grid.expand(this); }));
		}
		
		// Set up hash to store IDs of expanded rows
		if(!grid._expanded){ grid._expanded = {}; }
		
		listeners.push(aspect.after(grid, "insertRow", function(rowElement){
			// Auto-expand (shouldExpand) considerations
			var row = this.row(rowElement),
				expanded = column.shouldExpand(row, currentLevel, this._expanded[row.id]);
			
			if(expanded){ this.expand(rowElement, true, true); }
			return rowElement; // pass return value through
		}));
		
		listeners.push(aspect.before(grid, "removeRow", function(rowElement, justCleanup){
			var connected = rowElement.connected;
			if(connected){
				// if it has a connected expando node, we process the children
				querySelector(">.dgrid-row", connected).forEach(function(element){
					grid.removeRow(element, true);
				});
				// now remove the connected container node
				if(!justCleanup){
					put(connected, "!");
				}
			}
		}));
		
		if(column.collapseOnRefresh){
			// Clear out the _expanded hash on each call to cleanup
			// (which generally coincides with refreshes, as well as destroy).
			listeners.push(aspect.after(grid, "cleanup", function(){
				this._expanded = {};
			}));
		}
		
		grid._calcRowHeight = function(rowElement){
			// we override this method so we can provide row height measurements that
			// include the children of a row
			var connected = rowElement.connected;
			// if connected, need to consider this in the total row height
			return rowElement.offsetHeight + (connected ? connected.offsetHeight : 0); 
		};
		
		grid.expand = function(target, expand, noTransition){
			// summary:
			//		Expands the row corresponding to the given target.
			// target: Object
			//		Row object (or something resolvable to one) to expand/collapse.
			// expand: Boolean?
			//		If specified, designates whether to expand or collapse the row;
			//		if unspecified, toggles the current state.
			
			var row = target.element ? target : grid.row(target),
				hasTransitionend = has("transitionend"),
				dfd = new Deferred(),
				promise = dfd.promise;
			
			// Resolve initial promise immediately;
			// promise will be reassigned later if necessary to only resolve
			// after data is retrieved
			dfd.resolve();
			
			target = row.element;
			target = target.className.indexOf("dgrid-expando-icon") > -1 ? target :
				querySelector(".dgrid-expando-icon", target)[0];
			
			noTransition = noTransition || column.enableTransitions === false;
			
			if(target && target.mayHaveChildren &&
					(noTransition || expand !== !!this._expanded[row.id])){
				// toggle or set expand/collapsed state based on optional 2nd argument
				var expanded = expand === undefined ? !this._expanded[row.id] : expand;
				
				// update the expando display
				put(target, ".ui-icon-triangle-1-" + (expanded ? "se" : "e") +
					"!ui-icon-triangle-1-" + (expanded ? "e" : "se"));
				
				var rowElement = row.element,
					container = rowElement.connected,
					containerStyle,
					scrollHeight,
					options = {
						originalQuery: this.query
					};
				
				if(!container){
					// if the children have not been created, create a container, a preload node and do the 
					// query for the children
					container = options.container = rowElement.connected =
						put(rowElement, '+div.dgrid-tree-container');
					var query = function(options){
						return grid.store.getChildren(row.data, options);
					};
					if(column.allowDuplicates){
						// If allowDuplicates is specified, include parentId in options
						// in order to facilitate unique IDs for each occurrence of the
						// same item under multiple different parents.
						options.parentId = row.id;
					}
					// Include level information on query for renderQuery case
					if("level" in target){
						query.level = target.level;
					}
					// Add the query to the promise chain.
					promise = promise.then(function(){
						if(grid.renderQuery){
							return grid.renderQuery(query, options);
						}
						
						// If not using OnDemandList, we don't need preload nodes,
						// but we still need a beforeNode to pass to renderArray,
						// so create a temporary one
						var firstChild = put(container, "div");
						var rows = grid.renderArray(query(options), firstChild,
							"level" in query ? { queryLevel: query.level } : {});
						Deferred.when(rows, function(){
							put(firstChild, "!");
						});
						return rows;
					});
					
					if(hasTransitionend){
						on(container, hasTransitionend, ontransitionend);
					}else{
						ontransitionend.call(container);
					}
				}
				
				// Show or hide all the children.
				
				container.hidden = !expanded;
				containerStyle = container.style;
				
				// make sure it is visible so we can measure it
				if(!hasTransitionend || noTransition){
					containerStyle.display = expanded ? "block" : "none";
					containerStyle.height = "";
				}else{
					if(expanded){
						containerStyle.display = "block";
						scrollHeight = container.scrollHeight;
						containerStyle.height = "0px";
					}
					else{
						// if it will be hidden we need to be able to give a full height
						// without animating it, so it has the right starting point to animate to zero
						put(container, ".dgrid-tree-resetting");
						containerStyle.height = container.scrollHeight + "px";
					}
					// Perform a transition for the expand or collapse.
					setTimeout(function(){
						put(container, "!dgrid-tree-resetting");
						containerStyle.height =
							expanded ? (scrollHeight ? scrollHeight + "px" : "auto") : "0px";
					});
				}
				
				// Update _expanded map.
				if(expanded){
					this._expanded[row.id] = true;
				}else{
					delete this._expanded[row.id];
				}
			}
			return promise;
		}; // end function grid.expand
		
		// Set up a destroy function on column to tear down the listeners/aspects
		// established above if the grid's columns are redefined later.
		aspect.after(column, "destroy", function(){
			arrayUtil.forEach(listeners, function(l){ l.remove(); });
			// Delete methods we added/overrode on the instance.
			delete grid.expand;
			delete grid._calcRowHeight;
		});
	});
	
	column.renderCell = function(object, value, td, options){
		// summary:
		//		Renders a cell that can be expanded, creating more rows
		
		var grid = column.grid,
			level = Number(options && options.queryLevel) + 1,
			mayHaveChildren = !grid.store.mayHaveChildren || grid.store.mayHaveChildren(object),
			parentId = options.parentId,
			expando, node;
		
		level = currentLevel = isNaN(level) ? 0 : level;
		expando = column.renderExpando(level, mayHaveChildren,
			grid._expanded[(parentId ? parentId + "-" : "") + grid.store.getIdentity(object)], object);
		expando.level = level;
		expando.mayHaveChildren = mayHaveChildren;
		
		node = originalRenderCell.call(column, object, value, td, options);
		if(node && node.nodeType){
			put(td, expando);
			put(td, node);
		}else{
			td.insertBefore(expando, td.firstChild);
		}
	};
	return column;
}

tree.defaultRenderExpando = defaultRenderExpando;
return tree;
});

},
'dgrid/util/has-css3':function(){
define("dgrid/util/has-css3", ["dojo/has"],
function(has){
	// This module defines feature tests for CSS3 features such as transitions.
	// The css-transitions, css-transforms, and css-transforms3d has-features
	// can report either boolean or string:
	// * false indicates no support
	// * true indicates prefix-less support
	// * string indicates the vendor prefix under which the feature is supported

	var cssPrefixes = ["ms", "O", "Moz", "Webkit"];
	
	function testStyle(element, property) {
		var style = element.style,
			i;
		
		if (property in style) {
			// Standard, no prefix
			return true;
		}
		property = property.slice(0, 1).toUpperCase() + property.slice(1);
		for (i = cssPrefixes.length; i--;) {
			if ((cssPrefixes[i] + property) in style) {
				// Vendor-specific css property prefix
				return cssPrefixes[i];
			}
		}
		
		// Otherwise, not supported
		return false;
	}
	
	has.add("css-transitions", function(global, doc, element){
		return testStyle(element, "transitionProperty");
	});
	
	has.add("css-transforms", function(global, doc, element){
		return testStyle(element, "transform");
	});
	
	has.add("css-transforms3d", function(global, doc, element){
		return testStyle(element, "perspective");
	});
	
	has.add("transitionend", function(){
		// Infer transitionend event name based on CSS transitions has-feature.
		var tpfx = has("css-transitions");
		if(!tpfx){ return false; }
		if(tpfx === true){ return "transitionend"; }
		return {
			ms: "MSTransitionEnd",
			O: "oTransitionEnd",
			Moz: "transitionend",
			Webkit: "webkitTransitionEnd"
		}[tpfx];
	});
	
	return has;
});

},
'epi-cms/widget/_GridWidgetBase':function(){
﻿define("epi-cms/widget/_GridWidgetBase", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/aspect",
    "dojo/dom-construct",
    "dojo/keys",
    "dojo/topic",
    "dojo/when",

    "dgrid/Keyboard",
    "dgrid/OnDemandGrid",
    "dgrid/Selection",

    "dijit/layout/_LayoutWidget",

    "epi",
    "epi/datetime",
    "epi/UriParser",
    "epi/username",

    "epi/shell/dgrid/Formatter",
    "epi/shell/TypeDescriptorManager",

    "epi-cms/_ContentContextMixin",
    "epi-cms/core/ContentReference",
    "epi-cms/dgrid/DnD",
    "epi-cms/dgrid/formatters",
    "epi-cms/dgrid/WithContextMenu",
    "epi/dependency"
], function (
    declare,
    lang,
    aspect,
    domConstruct,
    keys,
    topic,
    when,

    Keyboard,
    OnDemandGrid,
    Selection,

    _LayoutWidget,

    epi,
    epiDate,
    UriParser,
    username,

    Formatter,
    TypeDescriptorManager,

    _ContentContextMixin,
    ContentReference,
    DnD,
    formatters,
    WithContextMenu,
    dependency
) {

    return declare([_LayoutWidget, _ContentContextMixin], {
        // tags:
        //      internal

        // _gridClass: [protected] Object
        //		The class to construct the grid with.
        _gridClass: declare([OnDemandGrid, Formatter, Selection, DnD, Keyboard, WithContextMenu]),

        // storeKeyName: [public] string
        //		The key of the store used to retrieve the store from the store registry.
        storeKeyName: null,

        // grid: [public] dgrid
        //		The grid that is used to display information.
        grid: null,

        // store: [public] dojo/store
        //		The store that contains the data.
        store: null,

        // ignoreVersionWhenComparingLinks: [public] boolean
        //		If item comparison should consider versions or not.
        ignoreVersionWhenComparingLinks: true,

        // trackActiveItem: [public] boolean
        //		If comparison should be done to highlight the currently active context item.
        trackActiveItem: true,

        // defaultGridMixin: [protected] object
        //		An object with default settings for dnd to mixin to the grid class.
        defaultGridMixin: null,

        currentItem: null,

        // contextChangeEvent: [public] String
        //      Set the event that will cause a context change (i.e. click or dblclick). Default value is "click".
        contextChangeEvent: "click",

        // forceContextReload: [public] boolean
        //      If context should be forced to reloaded when changing context
        forceContextReload: false,

        // selectionMode: [public] String
        //      The selection mode of the grid. See documentation on dgrid/Selection for valid values.
        selectionMode: "single",

        // selection: [public] Array
        //      An array containing the selected items in the grid.
        selection: null,

        postMixInProperties: function () {
            // summary:
            //		Called after constructor parameters have been mixed-in; sets default values for parameters that have not been initialized.
            // tags:
            //		protected

            this.inherited(arguments);

            this.defaultGridMixin = {
                selectionMode: this.get("selectionMode"),
                dndParams: {
                    creator: lang.hitch(this, this._dndNodeCreator),
                    copyOnly: true,
                    selfAccept: false
                }
            };

            if (!this.store && this.storeKeyName) {
                var registry = dependency.resolve("epi.storeregistry");
                this.store = registry.get(this.storeKeyName);
            }
        },

        startup: function () {
            // summary:
            //		Connects event handlers to common grid events.
            // tags:
            //		protected

            this.inherited(arguments);

            if (this.contextChangeEvent) {
                var callback = lang.hitch(this, "_onChangeContext");
                this.grid.on(".dgrid-row:" + this.contextChangeEvent, callback);

                // Setup the default keyboard handling for changing context if the Keyboard mixin is supported.
                if (typeof this.grid.addKeyHandler == "function") {
                    this.grid.addKeyHandler(keys.ENTER, callback);
                    this.grid.addKeyHandler(keys.SPACE, callback);
                }
            }

            this.grid.on("dgrid-select, dgrid-deselect", lang.hitch(this, "_onSelectionChanged"));
            this.grid.on("dgrid-error", lang.hitch(this, "_onError"));

            if (this.trackActiveItem) {
                this.own(aspect.after(this.grid, "insertRow", lang.hitch(this, this._selectActiveItem)));
            }

            this.own(this.connect(this, "layout", lang.hitch(this, function () {
                this.grid.resize();
            })));

            this.grid.startup();
        },

        destroy: function () {
            if (this.grid) {
                this.grid.destroy();
            }

            this.inherited(arguments);
        },

        select: function (/*Object*/item) {
            // summary:
            //      Select a grid row from the given item
            // item: [Object]
            // tags:
            //      public

            this.grid.clearSelection();
            if (item) {
                // using .toString() instead of item.id to get the full content link, which contains provider name
                var contentLink = (item instanceof ContentReference ? item : new ContentReference(item.id)).createVersionUnspecificReference().toString();
                this.grid.select(contentLink);
            }
        },

        _forceKeepContext: function (/*String*/uri) {
            // summary:
            //
            // uri: [String]
            //
            // tags:
            //      protected, extension

            return this._isSameAsCurrentContext(uri);
        },

        _onChangeContext: function (e) {
            // summary:
            //      Publish a context request when a version is selected in the list.
            // tags:
            //      protected, extension
            this._onChangeContextInternal(e);
        },

        _onChangeContextInternal: function (e) {
            // summary:
            //      Publish a context request when a version is selected in the list.
            // tags:
            //      internal

            var row = this.grid.row(e),
                uri = row.data.uri,
                newContext = { uri: uri };

            if (this._forceKeepContext(uri)) {
                return;
            }

            this._requestNewContext(newContext, { sender: this, forceReload: this.forceContextReload });
        },

        _onSelectionChanged: function (e) {
            // summary:
            //      Emits "itemsSelected" as an array of the items selected.
            // tags:
            //      private

            var selection = [];
            for (var id in this.grid.selection) {
                if (this.grid.selection[id]) {
                    var row = this.grid.row(id);
                    selection.push(row.data);
                }
            }

            this.set("selection", selection);
            this.emit("selectionChanged", { selection: selection });
            this._onSelect(e);
        },

        _onSelect: function (e) {
            // summary:
            //		Publish a context request when a version is selected in the list.
            // tags:
            //		protected, extension
        },

        _onError: function (e) {
            // summary:
            //		Shows an error message to the user when failing to load data.
            // tags:
            //		protected, extension
            var errorMessage = this.grid.errorMessage;
            var error = e.error;

            if (error.status && error.status === 403) {
                errorMessage = epi.resources.messages.nopermissiontoviewdata;
            }
            this._showErrorMessage(errorMessage);
        },

        _requestNewContext: function (contextParameters, callerData) {
            topic.publish("/epi/shell/context/request", contextParameters, callerData);
        },

        restore: function () {
            // summary: Restore component visuals
            this.resize();
        },

        _isActiveItem: function (/*dgrid row*/row) {
            // summary:
            //      Compare uri current dgrid row with current context
            // tags:
            //      private

            return when(this.getCurrentContext(), lang.hitch(this, function (context) {
                var rowUri = row.data.uri;
                return context && this._compareUris(rowUri, context.uri);
            }));
        },

        _selectActiveItem: function (node) {
            // summary:
            //		Called 'after' the insertRow method in order to select the active version.
            // tags:
            //		private

            var row = this.grid.row(node);

            if (!row) {
                //Note: We don't know why row is null in some cases. Is something wrong?
                return node;
            }

            when(this._isActiveItem(row), lang.hitch(this, function (isActive) {
                if (isActive) {
                    if (this.grid.selectionMode === "single") {
                        this.grid.clearSelection();
                    }
                    this.grid.select(row, null);
                }
            }));

            return node;
        },

        contextChanged: function (context, callerData) {
            this.inherited(arguments);

            if (context.type !== "epi.cms.contentdata") {
                return;
            }

            if (!callerData || callerData.sender !== this) {
                // the context changed, probably because we navigated or published something
                this.onContextChanged(context);
            }
        },

        onContextChanged: function (context) {
            this.fetchData();
        },

        fetchData: function () {
            // summary:
            //      Override this method to implement your logic to bind data to the grid.
            //
            // tags:
            //    protected
        },

        _showErrorMessage: function (errorMessage) {
            this.grid.cleanup();

            // There is no public method for setting messages, so use the same style as dgrid.
            this.grid.contentNode.innerHTML = errorMessage  || "";
        },

        contextChangeFailed: function (oldContext, requestContextParams, callerData) {
            // summary:
            //    Subcriptions to /epi/shell/context/requestfailed topic to sync page tree selection
            //
            // tags:
            //    private

            this.inherited(arguments);

            // select old page link
            if (callerData && callerData.sender === this) {
                this._selectActiveItem();
            }
        },

        _renderContentItem: function (object, value, node, options) {
            node.innerHTML = formatters.contentItem(object.typeIdentifier, object.missingLanguageBranch, value);
        },

        _localizeDate: function (value) {
            return formatters.localizedDate(value);
        },

        _createUserFriendlyUsername: function (name) {
            return formatters.userName(name);
        },

        _dndNodeCreator: function (item, hint) {
            // summary:
            //      Custom DnD avatar creator method

            var dndTypes = TypeDescriptorManager.getAndConcatenateValues(item.typeIdentifier, "dndTypes");

            if (!dndTypes && this.dndTypes) {
                dndTypes = this.dndTypes;
            }

            var node = domConstruct.create("div").appendChild(document.createTextNode(item.name));
            return {
                node: node,
                type: dndTypes,
                data: item
            };
        },

        _aroundInsertRow: function (original) {
            // summary:
            //      Called 'around' the insertRow method to fix the grids less than perfect selection.
            // tags:
            //      protected

            return lang.hitch(this, function (object, parent, beforeNode, i, options) {

                // Call original method
                var row = original.apply(this.grid, arguments);

                var currentItem = this.get("currentItem");
                if (currentItem) {
                    // using .toString() instead of currentItem.id to get the full content link, which contains provider name
                    var contentLink = (currentItem instanceof ContentReference ? currentItem : new ContentReference(currentItem.id)).createVersionUnspecificReference().toString();
                    if (contentLink === object.contentLink) {
                        this.select(currentItem);
                    }
                }

                return row;
            });
        },

        _getCurrentItem: function () {
            // summary:
            //      Get current context row based on current context
            // returns: [Promise]
            //      An instance of dgrid row
            // tags:
            //      protected

            return when(this.getCurrentContext(), function (context) {
                return {
                    id: ContentReference.tryConvert(context.id, context).id
                };
            });
        },

        _setSelectionModeAttr: function (/*String*/selectionMode) {
            // summary:
            //      Set the selection mode for the grid.
            // selectionMode: String
            //      The selection mode.
            // tags:
            //      protected

            if (this.grid) {
                this.grid.set("selectionMode", selectionMode);
            }
            this._set("selectionMode", selectionMode);
        }
    });

});

},
'epi/UriParser':function(){
﻿define("epi/UriParser", ["dojo", "epi"], function (dojo, epi) {
    return dojo.declare(null, {
        // tags:
        //      public

        constructor: function (uri) {
            var ore = new RegExp("^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\\?([^#]*))?(#(.*))?$");

            var r = uri.match(ore);

            if (!this.scheme) {
                this.scheme = r[2] || (r[1] ? "" : null);
            }

            if (!this.authority) {
                this.authority = r[4] || (r[3] ? "" : null);
            }

            if (!this.path) {
                this.path = r[5]; // can never be undefined
            }
        },

        getModuleArea: function () {
            return this.scheme.split(".")[1];
        },

        getStoreName: function () {
            return this.scheme.split(".")[2];
        },

        getType: function () {
            return this.scheme;
        },

        getId: function () {
            return this.path.substr(1);
        },

        toString: function () {
            return this.scheme + "://" + this.authority + this.path;
        }
    });
});

},
'epi-cms/_ContentContextMixin':function(){
define("epi-cms/_ContentContextMixin", [
// dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/aspect",

    "dojo/Deferred",
    "dojo/when",

    // epi
    "epi/dependency",
    "epi/UriParser",

    "epi/shell/_ContextMixin",

    "epi-cms/core/ContentReference"
],

function (
// dojo
    declare,
    lang,
    aspect,

    Deferred,
    when,

    // epi
    dependency,
    UriParser,

    _ContextMixin,

    ContentReference
) {

    return declare([_ContextMixin], {
        // summary:
        //      Helps keeping up to date with the current context. Mixing this into a class gives access to current context and other useful features.
        // tags:
        //      public abstract

        ignoreVersionWhenComparingLinks: false,

        contentDataStore: null,

        _currentContentContext: null,

        postscript: function () {
            this.inherited(arguments);

            this.own(aspect.after(this.getContentDataStore(), "notify", function (item, id) {
                this.itemChanged(id, item);
            }.bind(this), true));
        },

        getContentDataStore: function () {
            // summary:
            //      Gets the content data store from store registry if it's not already cached

            if (!this.contentDataStore) {
                var registry = dependency.resolve("epi.storeregistry");
                this.contentDataStore = registry.get("epi.cms.contentdata");
            }
            return this.contentDataStore;
        },

        getCurrentContent: function () {
            // summary:
            //      Returns either the current content or a deferred which will resolve a content data as soon as it becomes available.
            // example:
            //      dojo.when(this.getCurrentContent(), function(content) {
            //          console.log("now we know we've got ", content);
            //      });
            // tags:
            //      protected

            var deferred = new Deferred();

            when(this.getCurrentContext(), lang.hitch(this, function (ctx) {
                if (!this._isContentContext(ctx)) {
                    deferred.reject({ message: "Not a content data", context: ctx });
                    return;
                }

                when(this.getContentDataStore().get(ctx.id), function (contentData) {
                    deferred.resolve(contentData);
                });
            }));

            return deferred;
        },

        contextChanged: function (ctx, callerData) {
            this.inherited(arguments);

            if (!this._isContentContext(ctx)) {
                this._currentContentContext = null;
                return;
            }

            var forceChange = (callerData && callerData.forceContextChange ? callerData.forceContextChange : false);

            if (!forceChange && this._isSameAsCurrentContentContext(ctx)) {
                // already loaded
                return;
            }

            this._currentContentContext = ctx;

            this.contentContextChanged(ctx, callerData);
        },

        contextUpdated: function (ctx, callerData) {

            if (!this._isContentContext(ctx)) {
                this._currentContentContext = null;
                return;
            }

            this._currentContentContext = ctx;

            this.contentContextUpdated(ctx, callerData);
        },

        contentContextChanged: function (ctx, callerData) {
            // summary:
            //      Called when the currently loaded content changes. I.e. a new content data object is loaded into the preview area.
            // tags:
            //      protected
        },

        contentContextUpdated: function (ctx, callerData) {
            // summary:
            //      Called when the currently loaded content updated.
            // tags:
            //      protected
        },

        itemChanged: function (id, item) {
            // summary:
            //      Notification event when an item has been patched or reloaded.
            // object: id
            //      The id of the updated item.
            // object: item
            //      The updated item.
            // tags:
            //      public deprecated
        },

        _isContentContext: function (ctx) {
            // summary:
            //      Verifies that the context has content data
            // ctx: context
            //      The context to check
            // tags:
            //      protected

            return ctx && ctx.type === "epi.cms.contentdata";
        },

        _isSameAsCurrentContentContext: function (ctx) {
            // summary:
            //      Checks if the content context has Id and PreferredLanguage the same as the already loaded current content context.
            // ctx: Object
            //      The content context to compare with current content context.
            // tags:
            //      protected

            if (this._currentContentContext && ctx) {
                return this._currentContentContext.id === ctx.id
                        && (this._currentContentContext.languageContext && ctx.languageContext
                            && this._currentContentContext.languageContext.preferredLanguage === ctx.languageContext.preferredLanguage);
            } else {
                return false;
            }
        },

        _isSameAsCurrentContext: function (uri) {
            // summary:
            //      Checks if the URI is the same as the already loaded context.
            // object: uri
            //      The uri of the context to compare with current.
            // tags:
            //      protected

            if (this._currentContext) {
                return this._compareUris(uri, this._currentContext.uri);
            } else {
                return false;
            }
        },

        _compareUris: function (obj1, obj2) {
            // summary:
            //      Checks if the URI:s are equal
            // object: obj1
            //      First object to compare.
            // object: obj2
            //      Second object to compare.
            // tags:
            //      protected

            var uri1 = new UriParser(obj1),
                uri2 = new UriParser(obj2);

            if (this.ignoreVersionWhenComparingLinks &&
                uri1.getType() === "epi.cms.contentdata" &&
                uri2.getType() === "epi.cms.contentdata" &&
                ContentReference.compareIgnoreVersion(uri1.getId(), uri2.getId())) {
                return true;
            } else if (obj1 === obj2) {
                return true;
            }

            return false;
        }

    });

});

},
'epi/shell/_ContextMixin':function(){
﻿define("epi/shell/_ContextMixin", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/Deferred",
    "dojo/topic",
    "dijit/Destroyable"
], function (
    declare,
    lang,
    Deferred,
    topic,
    Destroyable
) {

    return declare([Destroyable], {
        // summary:
        //    Helps keeping up to date with the current context. Mixing this into a class gives access to current context and other useful features.
        // tags:
        //    public abstract

        // _currentContext: [protected] Object
        //      The currently loaded context
        _currentContext: null,

        // _waitingForContext: [private] Promise
        //      Set to a promise when waiting for initial context
        _waitingForContext: null,

        constructor: function (options) {
            var initHandle;

            function setupCurrentContext(ctx, callerData) {
                if (this._currentContext || !callerData || callerData.sender !== this) {
                    return;
                }

                initHandle.remove();
                initHandle = null;

                // we have context, make sure everyone interested knows
                this._initialContextChanged(ctx);
            }

            this.own(
                topic.subscribe("/epi/shell/context/request", lang.hitch(this, this._contextRequest)),
                topic.subscribe("/epi/shell/context/changed", lang.hitch(this, this._contextChanged)),
                topic.subscribe("/epi/shell/context/updated", lang.hitch(this, this._contextUpdated)),
                topic.subscribe("/epi/shell/context/requestfailed", lang.hitch(this, this._contextFailed)),
                initHandle = topic.subscribe("/epi/shell/context/current", lang.hitch(this, setupCurrentContext))
            );

            topic.publish("/epi/shell/context/requestcurrent", { sender: this });
        },

        getCurrentContext: function () {
            // summary:
            //      Returns either the current context or a deferred which will resolve context as soon as it becomes available.
            //
            // example:
            //      when(this.getCurrentContext(), function(ctx) {
            //          console.log("now we know we've got ", ctx);
            //      });
            //
            // tags:
            //      protected

            if (this._currentContext !== null) {
                // we have context, resolve it immediately
                return this._currentContext;
            }

            if (!this._waitingForContext) {
                // we've never requested context before, store deferred
                this._waitingForContext = new Deferred();
            }

            return this._waitingForContext.promise;
        },

        contextChanged: function (ctx, callerData) {
            // summary:
            //    Called when the currently loaded page changes. I.e. a new page is loaded into the preview area.
            // tags:
            //    protected
        },

        contextUpdated: function (ctx, callerData) {
            // summary:
            //    Called when the currently loaded page is updated. For instance when the language settings are changed.
            // tags:
            //    protected
        },

        contextChangeFailed: function (ctx, callerData) {
            // summary:
            //    Called when loading the context fails
            // tags:
            //    protected
        },

        _contextChanged: function (ctx, callerData) {
            // tags:
            //    private

            if (!this._currentContext) {
                this._initialContextChanged(ctx);
            }

            this._currentContext = ctx;
            this.contextChanged(ctx, callerData);
        },

        _initialContextChanged: function (context, callerData) {
            // tags:
            //    private

            this._currentContext = context;

            if (this._waitingForContext) {
                this._waitingForContext.resolve(context);
                this._waitingForContext = null;
            }
        },

        _contextRequest: function () {
            // tags:
            //    private

            // Invalidate the current context when someone requests a new context
            this._currentContext = null;
        },

        _contextUpdated: function (ctx, callerData) {
            // tags:
            //    private

            this._currentContext = ctx;
            this.contextUpdated(ctx, callerData);
        },

        _contextFailed: function (ctx, callerData) {
            // tags:
            //    private

            if (!this._currentContext) {
                this._initialContextChanged(ctx);
            }

            this._currentContext = ctx;
            this.contextChangeFailed(ctx, callerData);
        }
    });
});

},
'epi-cms/dgrid/DnD':function(){
﻿define("epi-cms/dgrid/DnD", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/NodeList",
    "epi/shell/dnd/Source",
    "dojo/dom-construct",

    "epi/dependency",
    "epi/shell/TypeDescriptorManager",

    "put-selector/put"

], function (
    declare,
    lang,
    NodeList,
    DnDSource,
    domConstruct,

    dependency,
    TypeDescriptorManager,

    put) {

    var GridSource = declare(DnDSource, {
        _legalMouseDown: function (evt) {
            // Fix _legalMouseDown to only allow starting drag from an item
            // (not from bodyNode outside contentNode).
            var legal = this.inherited(arguments);
            return legal && evt.target !== this.grid.bodyNode;
        },

        getSelectedNodes: function () {
            var grid = this.grid;

            // If dgrid's Selection mixin is in use, synchronize with it, using a
            // map of node references (updated on dgrid-[de]select events).
            var nodes = this.inherited(arguments);
            if (!grid.selection || Object.keys(grid.selection).length === 0) {
                // Only select if there are nodes available.
                if (nodes.length) {
                    grid.select(nodes[0]);
                }
                return nodes;
            }

            var isGridSelection = false;
            var gridSelectionNodes = new NodeList();

            //isGridSelection when exist any selection in nodes
            for (var id in grid.selection) {
                if (grid.row(id).element !== nodes[0]) {
                    gridSelectionNodes.push(grid.row(id).element);
                } else {
                    isGridSelection = true;
                    gridSelectionNodes.push(nodes[0]);
                }
            }
            if (isGridSelection) {
                return gridSelectionNodes;
            }

            grid.clearSelection();
            // Only select if there are nodes available.
            if (nodes.length) {
                grid.select(nodes[0]);
            }
            return nodes;
        },

        destroy: function () {

            this.grid = null;

            this.inherited(arguments);
        }
    });

    return declare([], {
        // tags:
        //      public

        // dndSourceTypes: Array
        //      Specifies the types which will be set for DnD items in the grid.
        dndSourceTypes: null,

        // dndParams: Object
        //      Object containing params to be passed to the DnD Source constructor.
        dndParams: null,

        // dndConstructor: Function
        //      Constructor from which to instantiate the DnD Source.
        dndConstructor: GridSource,

        // dndDisabled: Booleaen
        //      Set to true to disable the dnd support.
        dndDisabled: false,

        // dndFormatSuffix: String
        //      The format suffix to append to the dndTypes for items contained in the list
        //      Set to allow dragging the item to targets that allow for the type with the specified suffix or have a converter
        dndFormatSuffix: null,

        postCreate: function () {
            this.inherited(arguments);

            if (!this.dndDisabled) {
                this._setupDnD();
            }
        },

        destroy: function () {

            this.inherited(arguments);

            this.dndSource && this.dndSource.destroy();
            this.dndParams = this.dndSource = null;

        },

        insertRow: function (object) {
            // override to add dojoDndItem class to make the rows draggable
            var row = this.inherited(arguments);

            if (this.dndDisabled) {
                return row;
            }
            put(row, ".dojoDndItem");

            // setup the source if it hasn't been done yet
            this._setupDnD();

            var opts = {};
            if (this.query) {
                opts.oldParentItem = { contentLink: this.query.referenceId };
            }

            this.dndSource.setItem(row.id, { data: object, type: this._getDndType(object), options: opts });
            return row;
        },

        _getDndType: function (object) {
            // Get a custom dnd type array from the specific object or a general type array from the source.
            var dndTypes = TypeDescriptorManager.getAndConcatenateValues(object.typeIdentifier, "dndTypes");
            if ((!dndTypes || dndTypes.length <= 0) && this.dndSourceTypes) {
                dndTypes = this.dndSourceTypes;
            }
            if (this.dndFormatSuffix) {
                dndTypes = dndTypes.map(lang.hitch(this, function (type) {
                    return type + "." + this.dndFormatSuffix;
                }));
            }
            return dndTypes;
        },

        _setupDnD: function () {

            if (this.dndSource) {
                return;
            }

            if (!this.dndParams.creator) {
                this.dndParams.creator = lang.hitch(this, this._dndNodeCreator);
            }

            this.dndSource = new this.dndConstructor(
                this.bodyNode,
                lang.mixin(this.dndParams, {
                    // add cross-reference to grid for potential use in inter-grid drop logic
                    grid: this,
                    dropParent: this.contentNode
                })
            );
            // Allows selection of only one element
            this.dndSource.singular = true;
        },

        _dndNodeCreator: function (item, hint) {
            // summary:
            //      Custom DnD avatar creator method

            var dndTypes, node;

            dndTypes = this._getDndType(item);

            if (!dndTypes && this.dndTypes) {
                dndTypes = this.dndTypes;
            }

            node = domConstruct.create("div").appendChild(document.createTextNode(item.name));
            return {
                node: node,
                type: dndTypes,
                data: item
            };
        }
    });
});

},
'epi/shell/dnd/Source':function(){
﻿define("epi/shell/dnd/Source", [

//Dojo
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/topic",
    "dojo/dom-construct",
    "dojo/dnd/Source",
    //EPi Framework
    "./_DndDataMixin",
    "epi/shell/TypeDescriptorManager"],

function (
    //Dojo
    array,
    declare,
    lang,
    topic,
    domConstruct,
    Source,
    //EPi Framework
    _DndDataMixin,
    TypeDescriptorManager) {

    return declare([Source, _DndDataMixin], {
        // tags:
        //      private

        alwaysCopy: true,

        createItemOnDrop: true,

        //	singular: Boolean
        //		Allows selection of only one element, if true.
        //		Tree hasn't been tested in singular=true mode, unclear if it works.
        singular: true,

        generateText: false,

        _dummyDropItemContainer: null,

        readOnly: null,

        postscript: function () {

            this.inherited(arguments);

            if (!this.createItemOnDrop) {
                this.parent = this._dummyDropItemContainer = domConstruct.create("div");
            }

            // If there are restricted types then add them to the accept array
            // with a value of zero to indicated that they are not accepted.
            var reject = this.reject;
            if (reject) {

                // we need to filter out restrictedDndTypes
                var allowedDnD = [];
                for (var key in this.accept) {
                    allowedDnD.push(key);
                }

                this.accept = {};
                allowedDnD = TypeDescriptorManager.removeIntersectingTypes(allowedDnD, this.reject);
                array.forEach(allowedDnD, function (allowed) {
                    this.accept[allowed] = 1;
                }, this);

                for (var i = 0; i < reject.length; ++i) {
                    this.accept[reject[i]] = 0;
                }
            }
        },

        destroy: function () {

            if (this._dummyDropItemContainer) {
                domConstruct.destroy(this._dummyDropItemContainer);
                this._dummyDropItemContainer = null;
            }

            this.inherited(arguments);
        },

        checkAcceptance: function (source, nodes) {
            // summary:
            //		Checks if the target can accept nodes from this source.
            // source: Object
            //		The source which provides items.
            // nodes: Array
            //		The list of transferred items.

            if (this.readOnly) {
                return false;
            }

            var items = array.map(nodes, function (node) {
                return source.getItem(node.id);
            });

            return this._checkAcceptanceForItems(items, this.accept);
        },

        getOrderedItems: function () {
            return array.map(this.getAllNodes(), function (node) {
                return this.getItem(node.id);
            }, this);
        },

        onDndItemRemoved: function (dndData, source, nodes, copy, target) {
            // summary:
            //      Called when the nodes has been removed from the source, i.e. droped in another target
            //
            // dndData:
            //    Dnd data extracted from the dragging items which have the same data type to the current target
            //
            // source:
            //    The dnd source.
            //
            // nodes:
            //    The dragging nodes.
            //
            // copy:
            //    Denote that the drag is copy.
            //
            // tags:
            //    public, event
        },

        onDropData: function (dndData, source, nodes, copy) {
            // summary:
            //    Drop data event.
            //
            // dndData:
            //    Dnd data extracted from the dragging items which have the same data type to the current target
            //
            // source:
            //    The dnd source.
            //
            // nodes:
            //    The dragging nodes.
            //
            // copy:
            //    Denote that the drag is copy.
            //
            // tags:
            //    public, event
        },

        onDrop: function (source, nodes, copy) {
            // summary:
            //		Called only on the current target, when drop is performed
            // source: Object
            //		The source which provides items
            // nodes: Array
            //		The list of transferred items
            // copy: Boolean
            //		Copy items, if true, move items otherwise

            // always copy when dropping on this target type

            var dndData = array.map(nodes, function (node) {
                return this._getDndData(source.getItem(node.id), this.accept, this === source);
            }, this);

            this.inherited(arguments, [source, nodes, this.alwaysCopy || copy]);

            if (!this.createItemOnDrop) {
                domConstruct.empty(this._dummyDropItemContainer);
            }

            this.onDropData(dndData, source, nodes, this.alwaysCopy || copy);

            if (!copy && this !== source && source.onDndItemRemoved) {
                source.onDndItemRemoved(dndData, source, nodes, copy, this);
            }

            topic.publish("/epi/dnd/dropdata", dndData, source, nodes, this.alwaysCopy || copy);
        }
    });
}
);

},
'dojo/dnd/Source':function(){
define("dojo/dnd/Source", [
	"../_base/array", "../_base/connect", "../_base/declare", "../_base/kernel", "../_base/lang",
	"../dom-class", "../dom-geometry", "../mouse", "../ready", "../topic",
	"./common", "./Selector", "./Manager"
], function(array, connect, declare, kernel, lang, domClass, domGeom, mouse, ready, topic,
			dnd, Selector, Manager){

// module:
//		dojo/dnd/Source

/*
	Container property:
		"Horizontal"- if this is the horizontal container
	Source states:
		""			- normal state
		"Moved"		- this source is being moved
		"Copied"	- this source is being copied
	Target states:
		""			- normal state
		"Disabled"	- the target cannot accept an avatar
	Target anchor state:
		""			- item is not selected
		"Before"	- insert point is before the anchor
		"After"		- insert point is after the anchor
*/

/*=====
var __SourceArgs = {
	// summary:
	//		a dict of parameters for DnD Source configuration. Note that any
	//		property on Source elements may be configured, but this is the
	//		short-list
	// isSource: Boolean?
	//		can be used as a DnD source. Defaults to true.
	// accept: Array?
	//		list of accepted types (text strings) for a target; defaults to
	//		["text"]
	// autoSync: Boolean
	//		if true refreshes the node list on every operation; false by default
	// copyOnly: Boolean?
	//		copy items, if true, use a state of Ctrl key otherwise,
	//		see selfCopy and selfAccept for more details
	// delay: Number
	//		the move delay in pixels before detecting a drag; 0 by default
	// horizontal: Boolean?
	//		a horizontal container, if true, vertical otherwise or when omitted
	// selfCopy: Boolean?
	//		copy items by default when dropping on itself,
	//		false by default, works only if copyOnly is true
	// selfAccept: Boolean?
	//		accept its own items when copyOnly is true,
	//		true by default, works only if copyOnly is true
	// withHandles: Boolean?
	//		allows dragging only by handles, false by default
	// generateText: Boolean?
	//		generate text node for drag and drop, true by default
};
=====*/

// For back-compat, remove in 2.0.
if(!kernel.isAsync){
	ready(0, function(){
		var requires = ["dojo/dnd/AutoSource", "dojo/dnd/Target"];
		require(requires);	// use indirection so modules not rolled into a build
	});
}

var Source = declare("dojo.dnd.Source", Selector, {
	// summary:
	//		a Source object, which can be used as a DnD source, or a DnD target

	// object attributes (for markup)
	isSource: true,
	horizontal: false,
	copyOnly: false,
	selfCopy: false,
	selfAccept: true,
	skipForm: false,
	withHandles: false,
	autoSync: false,
	delay: 0, // pixels
	accept: ["text"],
	generateText: true,

	constructor: function(/*DOMNode|String*/ node, /*__SourceArgs?*/ params){
		// summary:
		//		a constructor of the Source
		// node:
		//		node or node's id to build the source on
		// params:
		//		any property of this class may be configured via the params
		//		object which is mixed-in to the `dojo/dnd/Source` instance
		lang.mixin(this, lang.mixin({}, params));
		var type = this.accept;
		if(type.length){
			this.accept = {};
			for(var i = 0; i < type.length; ++i){
				this.accept[type[i]] = 1;
			}
		}
		// class-specific variables
		this.isDragging = false;
		this.mouseDown = false;
		this.targetAnchor = null;
		this.targetBox = null;
		this.before = true;
		this._lastX = 0;
		this._lastY = 0;
		// states
		this.sourceState  = "";
		if(this.isSource){
			domClass.add(this.node, "dojoDndSource");
		}
		this.targetState  = "";
		if(this.accept){
			domClass.add(this.node, "dojoDndTarget");
		}
		if(this.horizontal){
			domClass.add(this.node, "dojoDndHorizontal");
		}
		// set up events
		this.topics = [
			topic.subscribe("/dnd/source/over", lang.hitch(this, "onDndSourceOver")),
			topic.subscribe("/dnd/start",  lang.hitch(this, "onDndStart")),
			topic.subscribe("/dnd/drop",   lang.hitch(this, "onDndDrop")),
			topic.subscribe("/dnd/cancel", lang.hitch(this, "onDndCancel"))
		];
	},

	// methods
	checkAcceptance: function(source, nodes){
		// summary:
		//		checks if the target can accept nodes from this source
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		if(this == source){
			return !this.copyOnly || this.selfAccept;
		}
		for(var i = 0; i < nodes.length; ++i){
			var type = source.getItem(nodes[i].id).type;
			// type instanceof Array
			var flag = false;
			for(var j = 0; j < type.length; ++j){
				if(type[j] in this.accept){
					flag = true;
					break;
				}
			}
			if(!flag){
				return false;	// Boolean
			}
		}
		return true;	// Boolean
	},
	copyState: function(keyPressed, self){
		// summary:
		//		Returns true if we need to copy items, false to move.
		//		It is separated to be overwritten dynamically, if needed.
		// keyPressed: Boolean
		//		the "copy" key was pressed
		// self: Boolean?
		//		optional flag that means that we are about to drop on itself

		if(keyPressed){ return true; }
		if(arguments.length < 2){
			self = this == Manager.manager().target;
		}
		if(self){
			if(this.copyOnly){
				return this.selfCopy;
			}
		}else{
			return this.copyOnly;
		}
		return false;	// Boolean
	},
	destroy: function(){
		// summary:
		//		prepares the object to be garbage-collected
		Source.superclass.destroy.call(this);
		array.forEach(this.topics, function(t){t.remove();});
		this.targetAnchor = null;
	},

	// mouse event processors
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove
		// e: Event
		//		mouse event
		if(this.isDragging && this.targetState == "Disabled"){ return; }
		Source.superclass.onMouseMove.call(this, e);
		var m = Manager.manager();
		if(!this.isDragging){
			if(this.mouseDown && this.isSource &&
					(Math.abs(e.pageX - this._lastX) > this.delay || Math.abs(e.pageY - this._lastY) > this.delay)){
				var nodes = this.getSelectedNodes();
				if(nodes.length){
					m.startDrag(this, nodes, this.copyState(dnd.getCopyKeyState(e), true));
				}
			}
		}
		if(this.isDragging){
			// calculate before/after
			var before = false;
			if(this.current){
				if(!this.targetBox || this.targetAnchor != this.current){
					this.targetBox = domGeom.position(this.current, true);
				}
				if(this.horizontal){
					// In LTR mode, the left part of the object means "before", but in RTL mode it means "after".
					before = (e.pageX - this.targetBox.x < this.targetBox.w / 2) == domGeom.isBodyLtr(this.current.ownerDocument);
				}else{
					before = (e.pageY - this.targetBox.y) < (this.targetBox.h / 2);
				}
			}
			if(this.current != this.targetAnchor || before != this.before){
				this._markTargetAnchor(before);
				m.canDrop(!this.current || m.source != this || !(this.current.id in this.selection));
			}
		}
	},
	onMouseDown: function(e){
		// summary:
		//		event processor for onmousedown
		// e: Event
		//		mouse event
		if(!this.mouseDown && this._legalMouseDown(e) && (!this.skipForm || !dnd.isFormElement(e))){
			this.mouseDown = true;
			this._lastX = e.pageX;
			this._lastY = e.pageY;
			Source.superclass.onMouseDown.call(this, e);
		}
	},
	onMouseUp: function(e){
		// summary:
		//		event processor for onmouseup
		// e: Event
		//		mouse event
		if(this.mouseDown){
			this.mouseDown = false;
			Source.superclass.onMouseUp.call(this, e);
		}
	},

	// topic event processors
	onDndSourceOver: function(source){
		// summary:
		//		topic event processor for /dnd/source/over, called when detected a current source
		// source: Object
		//		the source which has the mouse over it
		if(this !== source){
			this.mouseDown = false;
			if(this.targetAnchor){
				this._unmarkTargetAnchor();
			}
		}else if(this.isDragging){
			var m = Manager.manager();
			m.canDrop(this.targetState != "Disabled" && (!this.current || m.source != this || !(this.current.id in this.selection)));
		}
	},
	onDndStart: function(source, nodes, copy){
		// summary:
		//		topic event processor for /dnd/start, called to initiate the DnD operation
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise
		if(this.autoSync){ this.sync(); }
		if(this.isSource){
			this._changeState("Source", this == source ? (copy ? "Copied" : "Moved") : "");
		}
		var accepted = this.accept && this.checkAcceptance(source, nodes);
		this._changeState("Target", accepted ? "" : "Disabled");
		if(this == source){
			Manager.manager().overSource(this);
		}
		this.isDragging = true;
	},
	onDndDrop: function(source, nodes, copy, target){
		// summary:
		//		topic event processor for /dnd/drop, called to finish the DnD operation
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise
		// target: Object
		//		the target which accepts items
		if(this == target){
			// this one is for us => move nodes!
			this.onDrop(source, nodes, copy);
		}
		this.onDndCancel();
	},
	onDndCancel: function(){
		// summary:
		//		topic event processor for /dnd/cancel, called to cancel the DnD operation
		if(this.targetAnchor){
			this._unmarkTargetAnchor();
			this.targetAnchor = null;
		}
		this.before = true;
		this.isDragging = false;
		this.mouseDown = false;
		this._changeState("Source", "");
		this._changeState("Target", "");
	},

	// local events
	onDrop: function(source, nodes, copy){
		// summary:
		//		called only on the current target, when drop is performed
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise

		if(this != source){
			this.onDropExternal(source, nodes, copy);
		}else{
			this.onDropInternal(nodes, copy);
		}
	},
	onDropExternal: function(source, nodes, copy){
		// summary:
		//		called only on the current target, when drop is performed
		//		from an external source
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise

		var oldCreator = this._normalizedCreator;
		// transferring nodes from the source to the target
		if(this.creator){
			// use defined creator
			this._normalizedCreator = function(node, hint){
				return oldCreator.call(this, source.getItem(node.id).data, hint);
			};
		}else{
			// we have no creator defined => move/clone nodes
			if(copy){
				// clone nodes
				this._normalizedCreator = function(node /*=====, hint =====*/){
					var t = source.getItem(node.id);
					var n = node.cloneNode(true);
					n.id = dnd.getUniqueId();
					return {node: n, data: t.data, type: t.type};
				};
			}else{
				// move nodes
				this._normalizedCreator = function(node /*=====, hint =====*/){
					var t = source.getItem(node.id);
					source.delItem(node.id);
					return {node: node, data: t.data, type: t.type};
				};
			}
		}
		this.selectNone();
		if(!copy && !this.creator){
			source.selectNone();
		}
		this.insertNodes(true, nodes, this.before, this.current);
		if(!copy && this.creator){
			source.deleteSelectedNodes();
		}
		this._normalizedCreator = oldCreator;
	},
	onDropInternal: function(nodes, copy){
		// summary:
		//		called only on the current target, when drop is performed
		//		from the same target/source
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise

		var oldCreator = this._normalizedCreator;
		// transferring nodes within the single source
		if(this.current && this.current.id in this.selection){
			// do nothing
			return;
		}
		if(copy){
			if(this.creator){
				// create new copies of data items
				this._normalizedCreator = function(node, hint){
					return oldCreator.call(this, this.getItem(node.id).data, hint);
				};
			}else{
				// clone nodes
				this._normalizedCreator = function(node/*=====, hint =====*/){
					var t = this.getItem(node.id);
					var n = node.cloneNode(true);
					n.id = dnd.getUniqueId();
					return {node: n, data: t.data, type: t.type};
				};
			}
		}else{
			// move nodes
			if(!this.current){
				// do nothing
				return;
			}
			this._normalizedCreator = function(node /*=====, hint =====*/){
				var t = this.getItem(node.id);
				return {node: node, data: t.data, type: t.type};
			};
		}
		this._removeSelection();
		this.insertNodes(true, nodes, this.before, this.current);
		this._normalizedCreator = oldCreator;
	},
	onDraggingOver: function(){
		// summary:
		//		called during the active DnD operation, when items
		//		are dragged over this target, and it is not disabled
	},
	onDraggingOut: function(){
		// summary:
		//		called during the active DnD operation, when items
		//		are dragged away from this target, and it is not disabled
	},

	// utilities
	onOverEvent: function(){
		// summary:
		//		this function is called once, when mouse is over our container
		Source.superclass.onOverEvent.call(this);
		Manager.manager().overSource(this);
		if(this.isDragging && this.targetState != "Disabled"){
			this.onDraggingOver();
		}
	},
	onOutEvent: function(){
		// summary:
		//		this function is called once, when mouse is out of our container
		Source.superclass.onOutEvent.call(this);
		Manager.manager().outSource(this);
		if(this.isDragging && this.targetState != "Disabled"){
			this.onDraggingOut();
		}
	},
	_markTargetAnchor: function(before){
		// summary:
		//		assigns a class to the current target anchor based on "before" status
		// before: Boolean
		//		insert before, if true, after otherwise
		if(this.current == this.targetAnchor && this.before == before){ return; }
		if(this.targetAnchor){
			this._removeItemClass(this.targetAnchor, this.before ? "Before" : "After");
		}
		this.targetAnchor = this.current;
		this.targetBox = null;
		this.before = before;
		if(this.targetAnchor){
			this._addItemClass(this.targetAnchor, this.before ? "Before" : "After");
		}
	},
	_unmarkTargetAnchor: function(){
		// summary:
		//		removes a class of the current target anchor based on "before" status
		if(!this.targetAnchor){ return; }
		this._removeItemClass(this.targetAnchor, this.before ? "Before" : "After");
		this.targetAnchor = null;
		this.targetBox = null;
		this.before = true;
	},
	_markDndStatus: function(copy){
		// summary:
		//		changes source's state based on "copy" status
		this._changeState("Source", copy ? "Copied" : "Moved");
	},
	_legalMouseDown: function(e){
		// summary:
		//		checks if user clicked on "approved" items
		// e: Event
		//		mouse event

		// accept only the left mouse button, or the left finger
		if(e.type != "touchstart" && !mouse.isLeft(e)){ return false; }

		if(!this.withHandles){ return true; }

		// check for handles
		for(var node = e.target; node && node !== this.node; node = node.parentNode){
			if(domClass.contains(node, "dojoDndHandle")){ return true; }
			if(domClass.contains(node, "dojoDndItem") || domClass.contains(node, "dojoDndIgnore")){ break; }
		}
		return false;	// Boolean
	}
});

return Source;

});

},
'dojo/dnd/Selector':function(){
define("dojo/dnd/Selector", [
	"../_base/array", "../_base/declare", "../_base/event", "../_base/kernel", "../_base/lang",
	"../dom", "../dom-construct", "../mouse", "../_base/NodeList", "../on", "../touch", "./common", "./Container"
], function(array, declare, event, kernel, lang, dom, domConstruct, mouse, NodeList, on, touch, dnd, Container){

// module:
//		dojo/dnd/Selector

/*
	Container item states:
		""			- an item is not selected
		"Selected"	- an item is selected
		"Anchor"	- an item is selected, and is an anchor for a "shift" selection
*/

/*=====
var __SelectorArgs = declare([Container.__ContainerArgs], {
	// singular: Boolean
	//		allows selection of only one element, if true
	singular: false,

	// autoSync: Boolean
	//		autosynchronizes the source with its list of DnD nodes,
	autoSync: false
});
=====*/

var Selector = declare("dojo.dnd.Selector", Container, {
	// summary:
	//		a Selector object, which knows how to select its children

	/*=====
	// selection: Set<String>
	//		The set of id's that are currently selected, such that this.selection[id] == 1
	//		if the node w/that id is selected.  Can iterate over selected node's id's like:
	//	|		for(var id in this.selection)
	selection: {},
	=====*/

	constructor: function(node, params){
		// summary:
		//		constructor of the Selector
		// node: Node||String
		//		node or node's id to build the selector on
		// params: __SelectorArgs?
		//		a dictionary of parameters
		if(!params){ params = {}; }
		this.singular = params.singular;
		this.autoSync = params.autoSync;
		// class-specific variables
		this.selection = {};
		this.anchor = null;
		this.simpleSelection = false;
		// set up events
		this.events.push(
			on(this.node, touch.press, lang.hitch(this, "onMouseDown")),
			on(this.node, touch.release, lang.hitch(this, "onMouseUp"))
		);
	},

	// object attributes (for markup)
	singular: false,	// is singular property

	// methods
	getSelectedNodes: function(){
		// summary:
		//		returns a list (an array) of selected nodes
		var t = new NodeList();
		var e = dnd._empty;
		for(var i in this.selection){
			if(i in e){ continue; }
			t.push(dom.byId(i));
		}
		return t;	// NodeList
	},
	selectNone: function(){
		// summary:
		//		unselects all items
		return this._removeSelection()._removeAnchor();	// self
	},
	selectAll: function(){
		// summary:
		//		selects all items
		this.forInItems(function(data, id){
			this._addItemClass(dom.byId(id), "Selected");
			this.selection[id] = 1;
		}, this);
		return this._removeAnchor();	// self
	},
	deleteSelectedNodes: function(){
		// summary:
		//		deletes all selected items
		var e = dnd._empty;
		for(var i in this.selection){
			if(i in e){ continue; }
			var n = dom.byId(i);
			this.delItem(i);
			domConstruct.destroy(n);
		}
		this.anchor = null;
		this.selection = {};
		return this;	// self
	},
	forInSelectedItems: function(/*Function*/ f, /*Object?*/ o){
		// summary:
		//		iterates over selected items;
		//		see `dojo/dnd/Container.forInItems()` for details
		o = o || kernel.global;
		var s = this.selection, e = dnd._empty;
		for(var i in s){
			if(i in e){ continue; }
			f.call(o, this.getItem(i), i, this);
		}
	},
	sync: function(){
		// summary:
		//		sync up the node list with the data map

		Selector.superclass.sync.call(this);

		// fix the anchor
		if(this.anchor){
			if(!this.getItem(this.anchor.id)){
				this.anchor = null;
			}
		}

		// fix the selection
		var t = [], e = dnd._empty;
		for(var i in this.selection){
			if(i in e){ continue; }
			if(!this.getItem(i)){
				t.push(i);
			}
		}
		array.forEach(t, function(i){
			delete this.selection[i];
		}, this);

		return this;	// self
	},
	insertNodes: function(addSelected, data, before, anchor){
		// summary:
		//		inserts new data items (see `dojo/dnd/Container.insertNodes()` method for details)
		// addSelected: Boolean
		//		all new nodes will be added to selected items, if true, no selection change otherwise
		// data: Array
		//		a list of data items, which should be processed by the creator function
		// before: Boolean
		//		insert before the anchor, if true, and after the anchor otherwise
		// anchor: Node
		//		the anchor node to be used as a point of insertion
		var oldCreator = this._normalizedCreator;
		this._normalizedCreator = function(item, hint){
			var t = oldCreator.call(this, item, hint);
			if(addSelected){
				if(!this.anchor){
					this.anchor = t.node;
					this._removeItemClass(t.node, "Selected");
					this._addItemClass(this.anchor, "Anchor");
				}else if(this.anchor != t.node){
					this._removeItemClass(t.node, "Anchor");
					this._addItemClass(t.node, "Selected");
				}
				this.selection[t.node.id] = 1;
			}else{
				this._removeItemClass(t.node, "Selected");
				this._removeItemClass(t.node, "Anchor");
			}
			return t;
		};
		Selector.superclass.insertNodes.call(this, data, before, anchor);
		this._normalizedCreator = oldCreator;
		return this;	// self
	},
	destroy: function(){
		// summary:
		//		prepares the object to be garbage-collected
		Selector.superclass.destroy.call(this);
		this.selection = this.anchor = null;
	},

	// mouse events
	onMouseDown: function(e){
		// summary:
		//		event processor for onmousedown
		// e: Event
		//		mouse event
		if(this.autoSync){ this.sync(); }
		if(!this.current){ return; }
		if(!this.singular && !dnd.getCopyKeyState(e) && !e.shiftKey && (this.current.id in this.selection)){
			this.simpleSelection = true;
			if(mouse.isLeft(e)){
				// Accept the left button and stop the event.   Stopping the event prevents text selection while
				// dragging.   However, don't stop the event on mobile because that prevents a click event,
				// and also prevents scroll (see #15838).
				// For IE we don't stop event when multiple buttons are pressed.
				event.stop(e);
			}
			return;
		}
		if(!this.singular && e.shiftKey){
			if(!dnd.getCopyKeyState(e)){
				this._removeSelection();
			}
			var c = this.getAllNodes();
			if(c.length){
				if(!this.anchor){
					this.anchor = c[0];
					this._addItemClass(this.anchor, "Anchor");
				}
				this.selection[this.anchor.id] = 1;
				if(this.anchor != this.current){
					var i = 0, node;
					for(; i < c.length; ++i){
						node = c[i];
						if(node == this.anchor || node == this.current){ break; }
					}
					for(++i; i < c.length; ++i){
						node = c[i];
						if(node == this.anchor || node == this.current){ break; }
						this._addItemClass(node, "Selected");
						this.selection[node.id] = 1;
					}
					this._addItemClass(this.current, "Selected");
					this.selection[this.current.id] = 1;
				}
			}
		}else{
			if(this.singular){
				if(this.anchor == this.current){
					if(dnd.getCopyKeyState(e)){
						this.selectNone();
					}
				}else{
					this.selectNone();
					this.anchor = this.current;
					this._addItemClass(this.anchor, "Anchor");
					this.selection[this.current.id] = 1;
				}
			}else{
				if(dnd.getCopyKeyState(e)){
					if(this.anchor == this.current){
						delete this.selection[this.anchor.id];
						this._removeAnchor();
					}else{
						if(this.current.id in this.selection){
							this._removeItemClass(this.current, "Selected");
							delete this.selection[this.current.id];
						}else{
							if(this.anchor){
								this._removeItemClass(this.anchor, "Anchor");
								this._addItemClass(this.anchor, "Selected");
							}
							this.anchor = this.current;
							this._addItemClass(this.current, "Anchor");
							this.selection[this.current.id] = 1;
						}
					}
				}else{
					if(!(this.current.id in this.selection)){
						this.selectNone();
						this.anchor = this.current;
						this._addItemClass(this.current, "Anchor");
						this.selection[this.current.id] = 1;
					}
				}
			}
		}
		event.stop(e);
	},
	onMouseUp: function(/*===== e =====*/){
		// summary:
		//		event processor for onmouseup
		// e: Event
		//		mouse event
		if(!this.simpleSelection){ return; }
		this.simpleSelection = false;
		this.selectNone();
		if(this.current){
			this.anchor = this.current;
			this._addItemClass(this.anchor, "Anchor");
			this.selection[this.current.id] = 1;
		}
	},
	onMouseMove: function(/*===== e =====*/){
		// summary:
		//		event processor for onmousemove
		// e: Event
		//		mouse event
		this.simpleSelection = false;
	},

	// utilities
	onOverEvent: function(){
		// summary:
		//		this function is called once, when mouse is over our container
		this.onmousemoveEvent = on(this.node, touch.move, lang.hitch(this, "onMouseMove"));
	},
	onOutEvent: function(){
		// summary:
		//		this function is called once, when mouse is out of our container
		if(this.onmousemoveEvent){
			this.onmousemoveEvent.remove();
			delete this.onmousemoveEvent;
		}
	},
	_removeSelection: function(){
		// summary:
		//		unselects all items
		var e = dnd._empty;
		for(var i in this.selection){
			if(i in e){ continue; }
			var node = dom.byId(i);
			if(node){ this._removeItemClass(node, "Selected"); }
		}
		this.selection = {};
		return this;	// self
	},
	_removeAnchor: function(){
		if(this.anchor){
			this._removeItemClass(this.anchor, "Anchor");
			this.anchor = null;
		}
		return this;	// self
	}
});

return Selector;

});

},
'dojo/dnd/Container':function(){
define("dojo/dnd/Container", [
	"../_base/array",
	"../_base/declare",
	"../_base/event",
	"../_base/kernel",
	"../_base/lang",
	"../_base/window",
	"../dom",
	"../dom-class",
	"../dom-construct",
	"../Evented",
	"../has",
	"../on",
	"../query",
	"../ready",
	"../touch",
	"./common"
], function(
	array, declare, event, kernel, lang, win,
	dom, domClass, domConstruct, Evented, has, on, query, ready, touch, dnd){

// module:
//		dojo/dnd/Container

/*
	Container states:
		""		- normal state
		"Over"	- mouse over a container
	Container item states:
		""		- normal state
		"Over"	- mouse over a container item
*/



var Container = declare("dojo.dnd.Container", Evented, {
	// summary:
	//		a Container object, which knows when mouse hovers over it,
	//		and over which element it hovers

	// object attributes (for markup)
	skipForm: false,
	// allowNested: Boolean
	//		Indicates whether to allow dnd item nodes to be nested within other elements.
	//		By default this is false, indicating that only direct children of the container can
	//		be draggable dnd item nodes
	allowNested: false,
	/*=====
	// current: DomNode
	//		The DOM node the mouse is currently hovered over
	current: null,

	// map: Hash<String, Container.Item>
	//		Map from an item's id (which is also the DOMNode's id) to
	//		the dojo/dnd/Container.Item itself.
	map: {},
	=====*/

	constructor: function(node, params){
		// summary:
		//		a constructor of the Container
		// node: Node
		//		node or node's id to build the container on
		// params: Container.__ContainerArgs
		//		a dictionary of parameters
		this.node = dom.byId(node);
		if(!params){ params = {}; }
		this.creator = params.creator || null;
		this.skipForm = params.skipForm;
		this.parent = params.dropParent && dom.byId(params.dropParent);

		// class-specific variables
		this.map = {};
		this.current = null;

		// states
		this.containerState = "";
		domClass.add(this.node, "dojoDndContainer");

		// mark up children
		if(!(params && params._skipStartup)){
			this.startup();
		}

		// set up events
		this.events = [
			on(this.node, touch.over, lang.hitch(this, "onMouseOver")),
			on(this.node, touch.out,  lang.hitch(this, "onMouseOut")),
			// cancel text selection and text dragging
			on(this.node, "dragstart",   lang.hitch(this, "onSelectStart")),
			on(this.node, "selectstart", lang.hitch(this, "onSelectStart"))
		];
	},

	// object attributes (for markup)
	creator: function(){
		// summary:
		//		creator function, dummy at the moment
	},

	// abstract access to the map
	getItem: function(/*String*/ key){
		// summary:
		//		returns a data item by its key (id)
		return this.map[key];	// Container.Item
	},
	setItem: function(/*String*/ key, /*Container.Item*/ data){
		// summary:
		//		associates a data item with its key (id)
		this.map[key] = data;
	},
	delItem: function(/*String*/ key){
		// summary:
		//		removes a data item from the map by its key (id)
		delete this.map[key];
	},
	forInItems: function(/*Function*/ f, /*Object?*/ o){
		// summary:
		//		iterates over a data map skipping members that
		//		are present in the empty object (IE and/or 3rd-party libraries).
		o = o || kernel.global;
		var m = this.map, e = dnd._empty;
		for(var i in m){
			if(i in e){ continue; }
			f.call(o, m[i], i, this);
		}
		return o;	// Object
	},
	clearItems: function(){
		// summary:
		//		removes all data items from the map
		this.map = {};
	},

	// methods
	getAllNodes: function(){
		// summary:
		//		returns a list (an array) of all valid child nodes
		return query((this.allowNested ? "" : "> ") + ".dojoDndItem", this.parent);	// NodeList
	},
	sync: function(){
		// summary:
		//		sync up the node list with the data map
		var map = {};
		this.getAllNodes().forEach(function(node){
			if(node.id){
				var item = this.getItem(node.id);
				if(item){
					map[node.id] = item;
					return;
				}
			}else{
				node.id = dnd.getUniqueId();
			}
			var type = node.getAttribute("dndType"),
				data = node.getAttribute("dndData");
			map[node.id] = {
				data: data || node.innerHTML,
				type: type ? type.split(/\s*,\s*/) : ["text"]
			};
		}, this);
		this.map = map;
		return this;	// self
	},
	insertNodes: function(data, before, anchor){
		// summary:
		//		inserts an array of new nodes before/after an anchor node
		// data: Array
		//		a list of data items, which should be processed by the creator function
		// before: Boolean
		//		insert before the anchor, if true, and after the anchor otherwise
		// anchor: Node
		//		the anchor node to be used as a point of insertion
		if(!this.parent.firstChild){
			anchor = null;
		}else if(before){
			if(!anchor){
				anchor = this.parent.firstChild;
			}
		}else{
			if(anchor){
				anchor = anchor.nextSibling;
			}
		}
		var i, t;
		if(anchor){
			for(i = 0; i < data.length; ++i){
				t = this._normalizedCreator(data[i]);
				this.setItem(t.node.id, {data: t.data, type: t.type});
				anchor.parentNode.insertBefore(t.node, anchor);
			}
		}else{
			for(i = 0; i < data.length; ++i){
				t = this._normalizedCreator(data[i]);
				this.setItem(t.node.id, {data: t.data, type: t.type});
				this.parent.appendChild(t.node);
			}
		}
		return this;	// self
	},
	destroy: function(){
		// summary:
		//		prepares this object to be garbage-collected
		array.forEach(this.events, function(handle){ handle.remove(); });
		this.clearItems();
		this.node = this.parent = this.current = null;
	},

	// markup methods
	markupFactory: function(params, node, Ctor){
		params._skipStartup = true;
		return new Ctor(node, params);
	},
	startup: function(){
		// summary:
		//		collects valid child items and populate the map

		// set up the real parent node
		if(!this.parent){
			// use the standard algorithm, if not assigned
			this.parent = this.node;
			if(this.parent.tagName.toLowerCase() == "table"){
				var c = this.parent.getElementsByTagName("tbody");
				if(c && c.length){ this.parent = c[0]; }
			}
		}
		this.defaultCreator = dnd._defaultCreator(this.parent);

		// process specially marked children
		this.sync();
	},

	// mouse events
	onMouseOver: function(e){
		// summary:
		//		event processor for onmouseover or touch, to mark that element as the current element
		// e: Event
		//		mouse event
		var n = e.relatedTarget;
		while(n){
			if(n == this.node){ break; }
			try{
				n = n.parentNode;
			}catch(x){
				n = null;
			}
		}
		if(!n){
			this._changeState("Container", "Over");
			this.onOverEvent();
		}
		n = this._getChildByEvent(e);
		if(this.current == n){ return; }
		if(this.current){ this._removeItemClass(this.current, "Over"); }
		if(n){ this._addItemClass(n, "Over"); }
		this.current = n;
	},
	onMouseOut: function(e){
		// summary:
		//		event processor for onmouseout
		// e: Event
		//		mouse event
		for(var n = e.relatedTarget; n;){
			if(n == this.node){ return; }
			try{
				n = n.parentNode;
			}catch(x){
				n = null;
			}
		}
		if(this.current){
			this._removeItemClass(this.current, "Over");
			this.current = null;
		}
		this._changeState("Container", "");
		this.onOutEvent();
	},
	onSelectStart: function(e){
		// summary:
		//		event processor for onselectevent and ondragevent
		// e: Event
		//		mouse event
		if(!this.skipForm || !dnd.isFormElement(e)){
			event.stop(e);
		}
	},

	// utilities
	onOverEvent: function(){
		// summary:
		//		this function is called once, when mouse is over our container
	},
	onOutEvent: function(){
		// summary:
		//		this function is called once, when mouse is out of our container
	},
	_changeState: function(type, newState){
		// summary:
		//		changes a named state to new state value
		// type: String
		//		a name of the state to change
		// newState: String
		//		new state
		var prefix = "dojoDnd" + type;
		var state  = type.toLowerCase() + "State";
		//domClass.replace(this.node, prefix + newState, prefix + this[state]);
		domClass.replace(this.node, prefix + newState, prefix + this[state]);
		this[state] = newState;
	},
	_addItemClass: function(node, type){
		// summary:
		//		adds a class with prefix "dojoDndItem"
		// node: Node
		//		a node
		// type: String
		//		a variable suffix for a class name
		domClass.add(node, "dojoDndItem" + type);
	},
	_removeItemClass: function(node, type){
		// summary:
		//		removes a class with prefix "dojoDndItem"
		// node: Node
		//		a node
		// type: String
		//		a variable suffix for a class name
		domClass.remove(node, "dojoDndItem" + type);
	},
	_getChildByEvent: function(e){
		// summary:
		//		gets a child, which is under the mouse at the moment, or null
		// e: Event
		//		a mouse event
		var node = e.target;
		if(node){
			for(var parent = node.parentNode; parent; node = parent, parent = node.parentNode){
				if((parent == this.parent || this.allowNested) && domClass.contains(node, "dojoDndItem")){ return node; }
			}
		}
		return null;
	},
	_normalizedCreator: function(/*Container.Item*/ item, /*String*/ hint){
		// summary:
		//		adds all necessary data to the output of the user-supplied creator function
		var t = (this.creator || this.defaultCreator).call(this, item, hint);
		if(!lang.isArray(t.type)){ t.type = ["text"]; }
		if(!t.node.id){ t.node.id = dnd.getUniqueId(); }
		domClass.add(t.node, "dojoDndItem");
		return t;
	}
});

dnd._createNode = function(tag){
	// summary:
	//		returns a function, which creates an element of given tag
	//		(SPAN by default) and sets its innerHTML to given text
	// tag: String
	//		a tag name or empty for SPAN
	if(!tag){ return dnd._createSpan; }
	return function(text){	// Function
		return domConstruct.create(tag, {innerHTML: text});	// Node
	};
};

dnd._createTrTd = function(text){
	// summary:
	//		creates a TR/TD structure with given text as an innerHTML of TD
	// text: String
	//		a text for TD
	var tr = domConstruct.create("tr");
	domConstruct.create("td", {innerHTML: text}, tr);
	return tr;	// Node
};

dnd._createSpan = function(text){
	// summary:
	//		creates a SPAN element with given text as its innerHTML
	// text: String
	//		a text for SPAN
	return domConstruct.create("span", {innerHTML: text});	// Node
};

// dnd._defaultCreatorNodes: Object
//		a dictionary that maps container tag names to child tag names
dnd._defaultCreatorNodes = {ul: "li", ol: "li", div: "div", p: "div"};

dnd._defaultCreator = function(node){
	// summary:
	//		takes a parent node, and returns an appropriate creator function
	// node: Node
	//		a container node
	var tag = node.tagName.toLowerCase();
	var c = tag == "tbody" || tag == "thead" ? dnd._createTrTd :
			dnd._createNode(dnd._defaultCreatorNodes[tag]);
	return function(item, hint){	// Function
		var isObj = item && lang.isObject(item), data, type, n;
		if(isObj && item.tagName && item.nodeType && item.getAttribute){
			// process a DOM node
			data = item.getAttribute("dndData") || item.innerHTML;
			type = item.getAttribute("dndType");
			type = type ? type.split(/\s*,\s*/) : ["text"];
			n = item;	// this node is going to be moved rather than copied
		}else{
			// process a DnD item object or a string
			data = (isObj && item.data) ? item.data : item;
			type = (isObj && item.type) ? item.type : ["text"];
			n = (hint == "avatar" ? dnd._createSpan : c)(String(data));
		}
		if(!n.id){
			n.id = dnd.getUniqueId();
		}
		return {node: n, data: data, type: type};
	};
};

/*=====
Container.__ContainerArgs = declare([], {
	creator: function(){
		// summary:
		//		a creator function, which takes a data item, and returns an object like that:
		//		{node: newNode, data: usedData, type: arrayOfStrings}
	},

	// skipForm: Boolean
	//		don't start the drag operation, if clicked on form elements
	skipForm: false,

	// dropParent: Node||String
	//		node or node's id to use as the parent node for dropped items
	//		(must be underneath the 'node' parameter in the DOM)
	dropParent: null,

	// _skipStartup: Boolean
	//		skip startup(), which collects children, for deferred initialization
	//		(this is used in the markup mode)
	_skipStartup: false
});

Container.Item = function(){
	// summary:
	//		Represents (one of) the source node(s) being dragged.
	//		Contains (at least) the "type" and "data" attributes.
	// type: String[]
	//		Type(s) of this item, by default this is ["text"]
	// data: Object
	//		Logical representation of the object being dragged.
	//		If the drag object's type is "text" then data is a String,
	//		if it's another type then data could be a different Object,
	//		perhaps a name/value hash.

	this.type = type;
	this.data = data;
};
=====*/

return Container;
});

},
'epi-cms/widget/viewmodel/ContentReferencesViewModel':function(){
define("epi-cms/widget/viewmodel/ContentReferencesViewModel", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/aspect",
    "dojo/store/util/QueryResults",
    "dojo/when",
    "epi/shell/TypeDescriptorManager",
    // Parent class and mixins
    "dojo/Stateful",
    "dijit/Destroyable",
    // Resources
    "dojo/i18n!epi/nls/episerver.cms.widget.contentreferences"
], function (
    declare,
    lang,
    aspect,
    QueryResults,
    when,
    TypeDescriptorManager,
    // Parent class and mixins
    Stateful,
    Destroyable,
    // Resources
    resources
) {
    return declare([Stateful, Destroyable], {
        // tags:
        //      internal

        // contentItems: [public] Content[]
        //      The content items whose references the widget should show.
        contentItems: null,

        // mode: [public] String
        //      What mode the content references widget should be in.
        mode: null,

        // notification: [public] String
        //      The notification text.
        notification: null,

        // notificationBarStyle: [public] Number
        //      The notification text style. 1 for green, 2 for orange and yellow for everything else.
        notificationBarStyle: null,

        // numberOfReferences: [public] Number
        //      How many references the content items have.
        numberOfReferences: null,

        // showToolbar: [public] Boolean
        //      Flag to indicate if the tool bar should show.
        showToolbar: null,

        // showGrid: [public] Boolean
        //      Flag to indicate if the content reference grid should show.
        showGrid: null,

        // totalLinks: [public] String
        //      The text for the number of references.
        totalLinks: null,

        getUpdatedStore: function (store) {
            // summary:
            //      Updates the store with functionality to support the tree structure of the data.
            // store: dojo/store
            //      The store to update.
            // tags:
            //      public

            var updatedStore = lang.delegate(store, {
                getChildren: function (object) {
                    return new QueryResults(object.references);
                },

                mayHaveChildren: function (object) {
                    return !!object.references && object.references.length > 0;
                }
            });

            this.own(aspect.after(updatedStore, "query", function (results) {
                when(results).then(this._afterStoreQuery.bind(this));
                return results;
            }.bind(this)));

            return updatedStore;
        },

        _afterStoreQuery: function (results) {
            // summary:
            //      Sets the values of properties that are dependent on the results from the store.
            // results: ReferenceItem[]
            //      An array of referenced items that will be used when getting the number of references.
            // tags:
            //      private

            var total = this._getReferencesCount(results);
            this._updateReferenceProperties(total);
            this.set("showToolbar", total > 0);
            this.set("showGrid", total > 0);
        },

        _getReferencesCount: function (results) {
            // summary:
            //      Counts the number of references in the results.
            // results: ReferenceItem[]
            //      An array of referenced items that will be used when getting the number of references.
            // tags:
            //      private

            return results.reduce(function (previousValue, referenceItem) {
                return previousValue + referenceItem.references.length;
            }, 0);
        },

        _updateReferenceProperties: function (number) {
            // summary:
            //      sets the number of references, and widget notification, styles based on the number.
            // tags:
            //      private

            var contentItems = this.contentItems,
                contentData = contentItems[0],
                hasReferences = number > 0,
                isSingleItem = contentItems.length === 1,
                // Determine if any item has children
                hasChildren = contentItems.some(function (item) {
                    return item.hasChildren;
                }),
                hasPublicUrls = contentItems.some(function (item) {
                    return !!item.publicUrl;
                });

            var setNotification = function (resourceKey, styleType) {
                resourceKey = "reference." + (isSingleItem ? "single" : "multiple") + "." + resourceKey;

                // Remarks: the typeIdentifier might be wrong if the user has selected more than one item
                // but most of the time it will display the correct resource value
                var text = TypeDescriptorManager.getResourceValue(contentData.typeIdentifier, resourceKey);

                this.set("notification", text);
                this.set("notificationBarStyle", styleType);
            }.bind(this);

            if (hasReferences) {
                setNotification(hasChildren ? "usedwithchildren" : "used", 2);
            } else if (hasPublicUrls) {
                setNotification(hasChildren ? "publicwithchildren" : "public", 2);
            } else {
                setNotification(hasChildren ? "notusedwithchildren" : "notused", hasChildren ? 2 : 1);
            }

            // build number of links text
            if (hasReferences) {
                var resourceText = number === 1 ? resources.totallink : resources.totallinks;
                var numberOfLinks = lang.replace(resourceText, [number]);
                this.set("totalLinks", numberOfLinks);
            }

            this.set("numberOfReferences", number);
        }

    });
});

},
'url:epi-cms/widget/templates/ContentReferences.html':"﻿<div class=\"epi-contentReferences\">\n    <div data-dojo-attach-point=\"contentReferencesNotificationBar\" class=\"epi-notificationBar epi-notificationBarWithBorders epi-notificationBarItem\">\n        <div data-dojo-attach-point=\"contentReferencesNotificationNode\" class=\"epi-notificationBarText\" ></div>\n    </div>\n    <div class=\"epi-captionPanel\" data-dojo-attach-point=\"captionPanelNode\">\n        <strong data-dojo-attach-point=\"totalLinksNode\"></strong>\n        <div class=\"epi-floatRight\">\n            <button class=\"epi-chromelessButton\"\n                    data-dojo-type=\"dijit/form/Button\"\n                    data-dojo-attach-event=\"onClick:fetchData\"\n                    data-dojo-props=\"iconClass:'epi-iconReload'\">${res.buttons.refresh}</button>\n        </div>\n    </div>\n    <div data-dojo-attach-point=\"gridNode\"></div>\n</div>\n",
'epi-cms/widget/ContentForestStoreModel':function(){
﻿define("epi-cms/widget/ContentForestStoreModel", [
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/promise/all",
    "dojo/when",
    "epi/epi",
    "epi-cms/widget/ContentTreeStoreModel"
], function (
    array,
    declare,
    lang,
    promiseAll,
    when,
    epi,
    ContentTreeStoreModel
) {

    return declare([ContentTreeStoreModel], {
        // summary:
        //      A store model for tree widgets that must support content with multiple roots.
        // description:
        //      The model creates a fabricated root item in order to support a store that has
        //      content with multiple roots. It is expected that the root will never be displayed
        //      in the tree.
        //
        // tags:
        //      internal xproduct

        // root: [readonly] Object
        //      The fabricated root item for the model.
        root: null,

        constructor: function () {
            this.root = {
                id: "0",
                name: "ROOT"
            };
        },

        // =======================================================================
        // Methods for traversing hierarchy

        isRoot: function (/*dojo/data/Item*/contentItem) {
            // summary:
            //		Checks to see if the item is a root node in the content forest
            // contentItem: [dojo/data/Item]
            //      The given data item object that wanted to verify
            // returns: [Boolean]
            // tags:
            //      public

            var inherited = this.inherited(arguments);
            return inherited || !!(contentItem && this.roots && (array.indexOf(this.roots, this.getIdentity(contentItem)) >= 0));
        },

        getRoot: function (/*Function*/ onItem) {
            // summary:
            //      Calls onItem with the fake root item for the tree.
            onItem(this.root);
        },

        mayHaveChildren: function (item) {
            // summary:
            //      Tells if an item has or may have children.
            // item: Object
            //      Item from the dojo/store
            return item === this.root || this.inherited(arguments); // Boolean
        },

        getChildren: function (parentItem, /*Function*/ onComplete, /*Function*/ onError) {
            // summary:
            //      Calls onComplete() with an array of child items of given parent item.
            // parentItem: Object
            //      Item from the dojo/store
            if (parentItem === this.root) {
                var deferreds = array.map(this.roots, function (item) {
                    return this.store.get(item);
                }, this);

                when(promiseAll(deferreds), onComplete, onError);
            } else {
                this.inherited(arguments);
            }
        },

        getAncestors: function (item, /*Function*/ onComplete) {
            // summary:
            //      Calls onComplete() with an array of ancestors of the given item.
            // item: Object
            //      Item from the dojo/store

            // Return root if item is subroot
            if (this.roots.some(function (subRoot) {
                return this.getIdentity(item) === subRoot;
            }, this)) {
                onComplete([this.root]);
                return;
            }

            var root = this.root,
                callback = lang.hitch(this, function (results) {
                    if (array.some(this.roots, function (subRoot) {
                        return this.getIdentity(results[0]) === subRoot;
                    }, this)) {
                        results.unshift(root);
                    }
                    onComplete(results);
                });

            this.inherited(arguments, [item, callback]);
        },

        // =======================================================================
        // Inspecting items

        getIdentity: function (item) {
            // summary:
            //      Returns identity for an item.
            // item: Object
            //      Item from the dojo/store
            return epi.areEqual(item, this.root) ? this.root.id : this.inherited(arguments); // String
        },

        canCopy: function (item) {
            // summary:
            //      Determine whether the copy action is supported for an item.
            // item: Object
            //      Item from the dojo/store
            return !this.isRoot(item) && this.inherited(arguments);
        },

        canCut: function (item) {
            // summary:
            //      Determine whether the cut action is supported for an item.
            // item: Object
            //      Item from the dojo/store
            return !this.isRoot(item) && this.inherited(arguments);
        },

        canDelete: function (item) {
            // summary:
            //      Determine whether the delete action is supported for an item.
            // item: Object
            //      Item from the dojo/store
            return !this.isTypeOfRoot(item) && this.inherited(arguments);
        },

        canEdit: function (item) {
            // summary:
            //      Determine whether the edit action is supported for an item.
            // item: Object
            //      Item from the dojo/store
            return !this.isRoot(item) && this.inherited(arguments);
        }
    });
});

},
'epi-cms/widget/ContentTreeStoreModel':function(){
define("epi-cms/widget/ContentTreeStoreModel", [
// dojo
    "dojo/_base/array",
    "dojo/_base/connect",
    "dojo/_base/declare",
    "dojo/_base/lang",

    "dojo/aspect",
    "dojo/Deferred",
    "dojo/promise/all",
    "dojo/when",
    "dojo/topic",
    // epi
    "epi",
    "epi/dependency",
    "epi/shell/_StatefulGetterSetterMixin",
    "epi/shell/TypeDescriptorManager",
    "epi/shell/_ContextMixin",
    "epi/shell/ViewSettings",

    // EPi CMS
    "epi-cms/ApplicationSettings",
    "epi-cms/contentediting/ContentActionSupport",
    "epi-cms/core/ContentReference",
    "epi-cms/widget/_HierarchicalModelMixin",
    "epi-cms/widget/viewmodel/_UpdateableStoreModelMixin"
],

function (
// dojo
    array,
    connect,
    declare,
    lang,

    aspect,
    Deferred,
    promiseAll,
    when,
    topic,
    // epi
    epi,
    dependency,
    _StatefulGetterSetterMixin,
    TypeDescriptorManager,
    _ContextMixin,
    ViewSettings,
    // EPi CMS
    ApplicationSettings,
    ContentActionSupport,
    ContentReference,
    _HierarchicalModelMixin,
    _UpdateableStoreModelMixin
) {

    return declare([_StatefulGetterSetterMixin, _HierarchicalModelMixin, _UpdateableStoreModelMixin, _ContextMixin], {
        // summary:
        //      A store model for tree widgets that only support content with a single root.
        //
        // tags:
        //      internal xproduct

        // store: [protected] dojo/store/api/Store
        //		Underlying store that will be queried for page tree items.
        store: null,

        // service: [protected] epi-cms/contentediting/ContentHierarchyService
        //		Service to use when updating data.
        service: null,

        // root: [public] ContentReference|String
        //		Id of the root content
        root: null,

        // notAllowToDelete: [public] List of item that we don't allow to delete. For example: Waste basket, start, root folder,...
        //		Id of the root content
        notAllowToDelete: null,

        // notAllowToCopy: [public] List of item that we don't allow to copy. For example: Waste basket, root folder,...
        //		Id of the root content
        notAllowToCopy: null,

        // typeIdentifiers: [protected] Array | String
        //		The set of type identifiers to be taken into account. Can either be an array or a comma separated string.
        typeIdentifiers: null,

        showAllLanguages: true,

        getChildrenQuery: "getchildren",

        getRootChildrenQuery: null,

        // _observers: [protected] Array
        //		Collection to store all the listeners from observed queries.
        _observers: null,

        // _defaultDataStoreName: [protected] String
        //		Default data store name to get from registry, if store is null.
        _defaultDataStoreName: "epi.cms.content.light",

        // additionalQueryOptions: [protected] Object
        //		Additional query options added when getting children. Set by inherited model.
        additionalQueryOptions: {},

        // createAsLocalAsset: [public] Boolean
        //      Indicate if the content should be created as local asset of its parent.
        createAsLocalAsset: false,

        // autoSelectPastedItem: Boolean
        //      Set to true if the pasted item automatically should be selected
        autoSelectPastedItem: true,

        constructor: function (args) {
            // summary:
            //		Construct the ContentTreeStoreModel object.

            declare.safeMixin(this, args);

            this._observers = {};
            this.store = this.store || dependency.resolve("epi.storeregistry").get(this._defaultDataStoreName);
            this.service = this.service || dependency.resolve("epi.cms.ContentHierarchyService");

            this._queryOptions = lang.mixin({
                ignore: ["query"],
                comparers: {
                    typeIdentifiers: lang.hitch(this, function (queryValue, instance) {
                        return array.some(queryValue, lang.hitch(this, function (item) {
                            return item === instance.typeIdentifier || TypeDescriptorManager.isBaseTypeIdentifier(instance.typeIdentifier, item);
                        }));
                    }),
                    referenceId: function (queryValue, instance) {
                        return ContentReference.compareIgnoreVersion(queryValue, instance.parentLink);
                    },
                    allLanguages: function (queryValue, instance) {
                        if (queryValue || !instance.currentLanguageBranch) {
                            return true;
                        }

                        return epi.areEqual(ApplicationSettings.currentContentLanguage, instance.currentLanguageBranch.languageId);
                    },
                    references: function (queryValue, instance) {
                        return queryValue.some(function (referenceId) {
                            return ContentReference.compareIgnoreVersion(referenceId, instance.parentLink);
                        });
                    }
                },
                sort: this._getSortSettings(this.typeIdentifiers)
            }, this.additionalQueryOptions);

            var updateHandle = connect.subscribe("/epi/cms/contentdata/updated", this, function (updatedContent) {
                this.store.refresh(updatedContent.contentLink).then(lang.hitch(this, function () {
                    if (updatedContent.recursive) {
                        this.onItemChildrenReload(updatedContent);
                    }
                }));
            });

            //publish command has been executed: update children
            var childrenChangeHandle = connect.subscribe("/epi/cms/contentdata/childrenchanged", lang.hitch(this, this._childrenChanged));

            //when the store is refreshed, update the item
            var itemNotifyHandle = aspect.after(this.store, "notify", lang.hitch(this, function (item, id) {
                this.onChange(item);
            }), true);

            this._handles = [updateHandle, childrenChangeHandle, itemNotifyHandle];
        },

        destroy: function () {
            // summary:
            //		Destroy the object.

            for (var id in this._observers) {
                this._observers[id].remove();

                this._observers[id] = null;
            }
            if (this._handles) {
                array.forEach(this._handles, function (handle) {
                    handle.remove();
                });

                this._handles = null;
            }
        },

        isSupportedType: function (dataType) {
            // summary:
            //      Check the supported type of data type selected.
            // dataType: [String]
            //      The type of data selected
            // tags:
            //      public

            return array.some(this.typeIdentifiers, function (type) {
                return type === dataType || TypeDescriptorManager.isBaseTypeIdentifier(dataType, type);
            });
        },

        _setShowAllLanguagesAttr: function (value) {
            this._set("showAllLanguages", value);
            this.onItemChildrenReload(this.root);
        },

        // =======================================================================
        // Methods for traversing hierarchy

        getRoot: function (onItem, onError) {
            // summary:
            //		Calls onItem with the root item for the tree, possibly a fabricated item.

            if (lang.isArray(this.root)) {
                onItem({});
            } else {
                when(this.store.get(this.root), onItem, onError);
            }
        },

        mayHaveChildren: function (item) {
            // summary:
            //		Tells if an item has or may have children.  Implementing logic here
            //		avoids showing +/- expando icon for nodes that we know don't have children.
            //		(For efficiency reasons we may not want to check if an element actually
            //		has children until user clicks the expando node)

            return item.hasChildren;
        },

        getChildren: function (parentItem, onComplete) {
            // summary:
            //		Calls onComplete() with array of child items of given parent item, all loaded.

            var id = this.getIdentity(parentItem),
                observer = this._observers[id],
                isRoot = id === this.root,
                queryType = isRoot && this.getRootChildrenQuery ? this.getRootChildrenQuery : this.getChildrenQuery,
                query = this._createQuery({ referenceId: id, query: queryType }),
                results = this.store.query(query, this._queryOptions);

            if (observer) {
                observer.remove();
                delete this._observers[id];
            }

            // Hook up an observer to the results to re-query if the collection changes.
            // We cannot reuse the results directly since the query-options are crucial for the data
            // if the items to be correct(i.e. hasChildren depending on contentTypes and language)
            this._observers[id] = results.observe(function (item, removedFrom, insertedInto) {
                when(this.store.query(query, this._queryOptions), this.onChildrenChange.bind(this, parentItem));
            }.bind(this), true);

            when(results, onComplete);
        },

        _getSortSettings: function (/*String || Array*/typeIdentifiers) {
            // summary:
            //      Get sort settings by typeIdentifiers.
            // tags:
            //      protected virtual

            if (!typeIdentifiers) {
                return {};
            }

            if (typeIdentifiers instanceof String) {
                typeIdentifiers = [typeIdentifiers];
            }

            var sortColumn,
                sortSettings = [];

            typeIdentifiers.map(function (type) {
                sortColumn = TypeDescriptorManager.getValue(type, "sortKey");
                if (sortColumn) {
                    sortSettings.push(sortColumn.sortDescending ?
                        { attribute: sortColumn.columnName, descending: true } :
                        { attribute: sortColumn.columnName });
                }
            });

            return sortSettings;
        },

        _createQuery: function (queryBase, excludeType) {
            var typeIdentifiers = this.typeIdentifiers &&
                    (lang.isArray(this.typeIdentifiers) ? this.typeIdentifiers : this.typeIdentifiers.split(","));

            var language = this.showAllLanguages ? { allLanguages: true } : {},
                requestType = excludeType ? {} : { typeIdentifiers: typeIdentifiers };

            var query = lang.mixin(queryBase, requestType, language);

            return query;
        },

        _createChildrenQuery: function (queryBase, excludeType) {
            var childrenQuery = lang.mixin(queryBase, { query: "getchildren" });
            return this._createQuery(childrenQuery, excludeType);
        },

        move: function (sourceItems, target) {
            // summary:
            //      Move the source item to target.

            var contentItems = sourceItems.map(function (item) {
                return item.data;
            });

            return this.pasteItems(contentItems, target, false);
        },

        remove: function (items) {
            // summary
            //      Removes (deletes) the source

            items = items.map(function (item) {
                return item.data;
            });

            var deferreds = items.map(function (item) {
                return this.store.remove(item.contentLink);
            }, this);

            return promiseAll(deferreds).then(lang.hitch(this, function () {
                this.onDeleted(items);
            }));
        },

        copy: function (sourceItems, target) {
            // summary:
            //      Copy the source items to target.

            var contentItems = sourceItems.map(function (item) {
                return item.data;
            });

            return this.pasteItems(contentItems, target, true);
        },

        pasteItems: function (sourceItems, targetItem, copy, sortIndex) {
            // summary:
            //		Move or copy several items from one parent item to another.
            // sourceItems: [Item]
            //      The items that should be copied or moved
            // targetItem: Item
            //      The target item for the operation
            // copy: Boolean
            //      If true the childItem will be copied; otherwise it will be moved.
            // sortIndex: integer
            //      Optional sortIndex in the child collection of the new parent
            // returns: object
            //      An object indicating the result of the operation
            //
            // tags: public

            var method = copy ? "copy" : "move",
                ids,
                ignoredItems = [],
                deletingCurrentContext;

            ids = sourceItems.map(this.getIdentity, this);

            // check if the current context is being deleted before doing the operation
            // afterwards the structure has changed which giver the wrong answer
            return when(this.getCurrentContext()).then(function (ctx) {
                return this._isNodeAContextAncestor(ids, ctx)
                    .then(function (isNodeAContextAncestor) {
                        deletingCurrentContext = isNodeAContextAncestor;
                        return isNodeAContextAncestor;
                    });
            }.bind(this)).then(function () {
                return this.service[method](ids, this.getIdentity(targetItem), this.createAsLocalAsset, sortIndex);
            }.bind(this)).then(function (response) {

                // Get all items that are part of the paste operation
                return promiseAll(sourceItems.map(function (item) {
                    var itemId = response.extraInformation[item.contentLink].extraInformation || item.contentLink;

                    var contentLink = ContentReference.toContentReference(itemId).createVersionUnspecificReference().toString();

                    return this.store.get(contentLink);
                }, this));
            }.bind(this)).then(function (items) {
                if (targetItem.isWastebasket) {
                    this._handleSelect(sourceItems, deletingCurrentContext);
                } else {
                    // Publish a children changed event for the new parent to ensure that the new
                    // parent node is updated if it exists in other tree structures.
                    topic.publish("/epi/cms/contentdata/childrenchanged", targetItem.contentLink);

                    // Publish a children changed event also for the sourceItems parents in order to
                    // refresh it's tree structure
                    var refreshedItems = [];
                    sourceItems.forEach(function (sourceItem) {
                        if (refreshedItems.indexOf(sourceItem.parentLink) === -1) {
                            topic.publish("/epi/cms/contentdata/childrenchanged", sourceItem.parentLink);
                            refreshedItems.push(sourceItem.parentLink);
                        }
                    });
                }

                this._handleDelete(items);

                items.forEach(function (item) {
                    this._onPasteComplete(item, item.isDeleted, null, targetItem);
                }, this);

                // Get all items that were already there in the target folder, if the file is pasted successfully then
                // the parentLink will equal the target folder. Otherwise it will remain the same.
                ignoredItems = items.filter(function (item) {
                    return sourceItems.some(function (sourceItem) {
                        return (sourceItem.contentLink === item.contentLink) && (sourceItem.parentLink === item.parentLink);
                    });
                });

                if (this.createAsLocalAsset === true) {
                    topic.publish("/epi/cms/action/createlocalasset");
                }

            }.bind(this)).then(function () {
                if (ignoredItems.length > 0) {
                    this.onPasteItemsFailed(ignoredItems);
                }
            }.bind(this));
        },

        pasteItem: function (childItem, oldParentItem, newParentItem, copy, sortIndex) {
            // summary:
            //		Move or copy an item from one parent item to another.
            // childItem: Item
            //      A reference to the item that should be moved or copied
            // oldParentItem: Item
            //      The current parent item
            // newParentItem: Item
            //      The target item for the operation
            // copy: Boolean
            //      If true the childItem will be copied; otherwise it will be moved.
            // sortIndex: integer
            //      Optional sortIndex in the child collection of the new parent
            // returns: object
            //      An object indicating the result of the operation
            //
            // tags: public
            //

            // Having copy/paste/delete call pasteItem may seem a bit backwards, but since the drag & drop
            // is calling pasteItem() with all required arguments it becomes a bit messy to drop some arguments
            // here (i.e. old/new parent) and then do the lookup again.
            // Those methods also have to duplicate the notification logic

            return this.pasteItems([childItem], newParentItem, copy, sortIndex)
                .then(function (response) {
                    // Return the response from the first extraInformation
                    return response.extraInformation[childItem.contentLink];
                });
        },

        onPasteComplete: function () {
            // summary:
            //      Stub to do somethings when paste process complete
            // tags:
            //      public, callback
        },

        _onPasteComplete: function (item, isDeleted, oldParentItem, newParentItem) {
            // summary:
            //		Executed after a paste operation has completed
            // item: Item
            //      The target item of the operation
            // isDeleted: boolean
            //      If the operation resulted in the item being deleted, i.e. moved to the trash
            // oldParentItem: Item
            //      The current parent item
            // newParentItem: Item
            //      The target item for the operation
            // tags: protected

            this.onPasteComplete();

            this._selectItemOnPasteComplete(item, isDeleted, oldParentItem, newParentItem);
        },

        _selectItemOnPasteComplete: function (item, isDeleted, oldParentItem, newParentItem) {
            // summary:
            //		Select an item after a paste operation has completed
            // item: Item
            //      The target item of the operation
            // isDeleted: boolean
            //      If the operation resulted in the item being deleted, i.e. moved to the trash
            // oldParentItem: Item
            //      The current parent item
            // newParentItem: Item
            //      The target item for the operation
            // tags: protected

            if (this.autoSelectPastedItem && item && item.contentLink && !isDeleted) {
                this.onSelect(item.contentLink, true);
            }
        },

        newItem: function (item, newParentItem) {
            // summary:
            //		Called when an item that's accepted in the tree is added by an external source, for instance by dragging an item from a listing to the tree.

            this.newItems([item], newParentItem);
        },

        newItems: function (items, newParentItem) {
            var newItems = items.map(function (item) {
                return item.dndData.data;
            });

            this._pasteNewItems(newItems, newParentItem);
        },

        _pasteNewItems: function (newItems, newParentItem) {
            //Right now we only support move actions when moving items from an external source to the tree. Might be changed in the future.
            this.pasteItems(newItems, newParentItem, false);
        },

        _updateItemChanged: function (parentContentLink) {
            // update the parent cause of in some case its properties changed
            when(this.store.refresh(parentContentLink), lang.hitch(this, function (updatedItem) {
                // then update its children
                this._childrenChanged(updatedItem);
            }));
        },

        _childrenChanged: function (parent) {
            var dfd = new Deferred();

            this.getChildren(parent, lang.hitch(this, function (children) {
                this.onChildrenChange(parent, children);
                dfd.resolve(children);
            }));

            return dfd;
        },

        _isNodeAContextAncestor: function (nodeContentLinks, context) {
            var dfd = new Deferred(),
                directMatch = nodeContentLinks.some(function (contentId) {
                    return ContentReference.compareIgnoreVersion(contentId, context.id);
                });

            if (directMatch) {
                dfd.resolve(true);
            } else {
                this.getAncestors(context, function (ancestors) {
                    var result = ancestors.some(function (ancestor) {
                        return nodeContentLinks.indexOf(ancestor.contentLink) > -1;
                    });

                    dfd.resolve(result);
                });
            }

            return dfd.promise;
        },

        _handleSelect: function (deletedItems, deletingCurrentContext) {
            // summary:
            //      Makes sure selection and context is updated based on what has been deleted.
            // tags:
            //      private

            var firstDeletedItem, singleItem;

            if (deletingCurrentContext) {
                firstDeletedItem = deletedItems[0];
                if (firstDeletedItem.capabilities.isPage) {
                    singleItem = deletedItems.length === 1 && firstDeletedItem;
                    this.onSelect(singleItem ? singleItem.parentLink : ApplicationSettings.startPage, true);
                    return;
                }

                topic.publish("/epi/shell/context/request", { uri: ViewSettings.settings.defaultContext });
            }

            var folders = deletedItems.filter(function (deletedItem) {
                return this.isSupportedType(deletedItem.typeIdentifier);
            }.bind(this));

            /*
                Deleting from asset pane:
                1) Single folder - look for parent
                2) Many folders - select GlobalAssetsFolder
                3) Item or many items - the selection should not change
            */

            if (folders.length === 1) {
                this.onSelect(folders[0].parentLink, true);
            } else if (folders.length > 1) {
                this.onSelect(ApplicationSettings.globalAssetsFolder, true);
            }
        },

        checkItemAcceptance: function (target, source) {
            // summary:
            //      Called from DnD to check if a specific target can accept the items in source
            //
            // tags: public

            // list source uses getSelectedNodes, tree source uses getSelectedTreeNodes
            var selectedNodes = source.getSelectedNodes ? source.getSelectedNodes() : source.getSelectedTreeNodes();
            var canPastePromises = selectedNodes.map(function (node) {
                var item = source.getItem(node.id);
                var data = item.data;
                if (data.dndData) {
                    data = data.dndData;
                }
                // canPaste can return a promise in classes that inherit from this one.
                return this.canPaste(data, target);
            }, this);

            return promiseAll(canPastePromises).then(function (results) {
                return results.every(function (canPaste) {
                    return canPaste;
                });
            });
        },

        _handleDelete: function (items) {

            var deletedItems = items.filter(function (item) {
                return item.isDeleted;
            });

            if (deletedItems.length > 0) {
                deletedItems.forEach(function (item) {
                    var id = this.getIdentity(item),
                        observer = this._observers[id];
                    if (observer) {
                        observer.remove();
                        delete this._observers[id];
                    }
                    this.onDelete(item, true);
                }, this);

                this.onDeleted(deletedItems);
            }
        },

        // =======================================================================
        // Inspecting items

        getIdentity: function (item) {
            // summary:
            //		Returns identity for an item

            if (lang.isObject(item)) {
                return this.store.getIdentity(item);
            }
            return item;
        },

        getObjectIconClass: function (/*Object*/item, /*String*/fallbackIconClass) {
            // summary:
            //      Get icon class for content based on its content link
            // item: [Object]
            //      Content item data
            // fallbackIconClass: [String]
            //      Default icon class in case nothing returned
            // tags:
            //      public, extension

            var defaultIconClass = TypeDescriptorManager.getValue(item.typeIdentifier, "iconClass");
            if (!defaultIconClass) {
                return fallbackIconClass;
            }

            var suffix = "";
            switch (parseInt(item.contentLink, 10)) {
                case ApplicationSettings.globalAssetsFolder:
                    suffix = "AllSites";
                    break;

                case ApplicationSettings.siteAssetsFolder:
                    suffix = "ThisSite";
                    break;

                default:
                    break;
            }

            return (suffix === "" && defaultIconClass === fallbackIconClass) ? defaultIconClass : (fallbackIconClass + " " + defaultIconClass + suffix);
        },

        getLabel: function (item) {
            // summary:
            //		Get the label for an item

            return item.name;
        },

        canExpandTo: function (item) {
            // summary:
            //		Test if the tree can expand to an item
            // item: Object
            //      The item
            // returns:
            //       A promise which resolves to true if the item can be reached, otherwise false.

            var deferred = new Deferred();

            this.getAncestors(item, lang.hitch(this, function (ancestors) {
                deferred.resolve(this.getIdentity(ancestors[0]) === this.root.id);
            }));

            return deferred.promise;
        },

        canCopy: function (item) {
            // summary:
            //		Determine whether the given item is able to be copied by the current user.
            // remarks:
            //      Override to implement additional check.
            // tags:
            //		public

            if (!item) {
                return false;
            }

            var reference = item.contentLink,
                isSystemPage = reference === this.root || (array.indexOf(this.notAllowToCopy, reference) >= 0);

            return !isSystemPage &&
                ContentActionSupport.hasAccess(item.accessMask, ContentActionSupport.accessLevel.Read) &&
                ContentActionSupport.hasProviderCapability(item.providerCapabilityMask, ContentActionSupport.providerCapabilities.Copy);
        },

        canCut: function (item) {
            // summary:
            //		Determine whether the given item is able to be cut by the current user.
            // tags:
            //		public

            if (!item) {
                return false;
            }

            var reference = item.contentLink,
                isSystemPage = reference === this.root || (array.indexOf(this.notAllowToDelete, reference) >= 0);

            return !isSystemPage &&
                ContentActionSupport.isActionAvailable(item, ContentActionSupport.action.Delete, ContentActionSupport.providerCapabilities.Move, true);
        },

        canDelete: function (item) {
            // summary:
            //		Determine whether the given item is able to be deleted by the current user.
            // tags:
            //		public

            if (!item) {
                return false;
            }

            var reference = item.contentLink,
                isSystemPage = reference === this.root || (array.indexOf(this.notAllowToDelete, reference) >= 0);

            return !isSystemPage &&
                ContentActionSupport.isActionAvailable(item, ContentActionSupport.action.Delete, ContentActionSupport.providerCapabilities.Delete, true);
        },

        canPaste: function (item, target, isCopy) {
            // summary:
            //      Determines whether the given item can be moved or copied from its current parent to a new location
            //      Primarily used when validating drag & drop operations.
            // tags:
            //      public

            if (!item || !target) {
                return false;
            }

            var isCopyToWastebasket = isCopy && target.isWastebasket,
                isCutToTheSameItem = !isCopy && (this.getIdentity(item) === this.getIdentity(target)),
                isCutToTheSameParent = !isCopy && (item.parentLink === this.getIdentity(target)),
                hasAccess = ContentActionSupport.isActionAvailable(target, ContentActionSupport.action.Create, ContentActionSupport.providerCapabilities.Create, true);

            return !target.isDeleted && !isCopyToWastebasket && !isCutToTheSameItem && !isCutToTheSameParent && hasAccess;
        },

        canEdit: function (item) {
            // summary:
            //		Determine whether the given item is able to be edit by the current user.
            // tags:
            //		public

            if (!item) {
                return false;
            }

            var reference = item.contentLink,
                isSystemPage = reference === this.root;

            return !isSystemPage && ContentActionSupport.hasAccess(item.accessMask, ContentActionSupport.accessLevel.Edit);
        },

        // =======================================================================
        // Callbacks

        onItemChildrenReload: function (/*Object*/parent) {
            // summary:
            //  Raised when the children of an item must be reloaded.
            //  The subscriber needs to call getChildren to get the updated children collection
        },

        onChange: function (/*dojo/data/Item*/ /*===== item =====*/) {
            // summary:
            //		Callback whenever an item has changed, so that Tree
            //		can update the label, icon, etc.   Note that changes
            //		to an item's children or parent(s) will trigger an
            //		onChildrenChange() so you can ignore those changes here.
            // tags:
            //		callback
        },

        onChildrenChange: function (/*===== parent, newChildrenList =====*/) {
            // summary:
            //		Callback to do notifications about new, updated, or deleted items.
            // parent: dojo/data/Item
            // newChildrenList: dojo/data/Item[]
            // tags:
            //		callback
        },

        onDelete: function (/*dojo/data/Item*/ /*===== item =====*/) {
            // summary:
            //		Callback when an item has been deleted.
            // description:
            //		Note that there will also be an onChildrenChange() callback for the parent
            //		of this item.
            // tags:
            //		callback
        },

        onDeleted: function (deletedItems) {
            // summary:
            //      Callback when an item has been deleted to set the parent folder as selected
            // deletedItems: dojo/data/Item[]
            //      items deleted
            // tags:
            //      public callback
        },

        onSelect: function (ItemId, setFocus, onComplete) {
            // summary:
            //      Raise event to set an item as selected
            // tags:
            //      callback
        },

        onPasteItemsFailed: function (ignoredItems) {
            // summary:
            //      Raise event to select the items that were not pasted successfully.
            // tags:
            //      public callback
        }
    });
});

},
'epi-cms/widget/_HierarchicalModelMixin':function(){
﻿define("epi-cms/widget/_HierarchicalModelMixin", [
// dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    // epi
    "epi/shell/_HierarchicalModelMixin"
],

function (
// dojo
    declare,
    lang,
    // epi
    _HierarchicalModelMixin
) {

    return declare([_HierarchicalModelMixin], {
        // summary:
        //      Decorate model for epi/shell/_HierarchicalModelMixin
        // tags:
        //      public

        isTypeOfRoot: function (/*dojo/data/Item*/contentItem) {
            // summary:
            //      Checks to see if the item is type of root node such as: root, sub root
            // contentItem: [dojo/data/Item]
            //      The given data item object that wanted to verify
            // returns: [Boolean]
            // tags:
            //      public, extension

            return !!(contentItem && (this.isRoot(contentItem) || contentItem.isSubRoot));
        },

        isRoot: function (/*dojo/data/Item*/contentItem) {
            // summary:
            //      Checks to see if the item is a root node in the content tree
            // contentItem: [dojo/data/Item]
            //      The given data item object that wanted to verify
            // returns: [Boolean]
            // tags:
            //      public

            return !!(contentItem && this.root && contentItem === this.root);
        }

    });

});

},
'epi/shell/_HierarchicalModelMixin':function(){
// Disable eqeqeq for this file
/*eslint eqeqeq:0*/

define("epi/shell/_HierarchicalModelMixin", [
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/Deferred",
    "dojo/promise/all",
    "dojo/when"
], function (
    array,
    declare,
    lang,
    Deferred,
    all,
    when
) {

    // summary:
    //		A mixin that adds Hierarchical support for a model
    return declare([], {
        // tags:
        //    public

        // parentProperty: [public] string
        //      The name of the property that indentifies the parent (e.g. parentLink)
        parentProperty: "parentLink",

        // store: [public] dojo/store/JsonRest
        //      The store that is used to get the ancestors
        store: null,

        constructor: function (options) {
            declare.safeMixin(this, options);
        },

        getParentIdentity: function (item) {
            // summary:
            //		Get the identity of the parent
            //
            // item:
            //      The item to get the parent identity for
            //
            // tags:
            //      public

            return lang.getObject(this.parentProperty, false, item);
        },

        isAncestor: function (source, target) {
            // summary:
            //		Checks to see if the source is an ancestor of the target.
            //
            // source:
            //      The item to check if ancestor
            //
            // target:
            //      The item to check the source against
            //
            // tags:
            //      public

            var sourceId = this.store.getIdentity(source),
                targetId = this.store.getIdentity(target),
                targetParentId = this.getParentIdentity(target);

            if (sourceId == targetId || (target.parentLink && source.contentLink == target.parentLink)) {
                return true;
            }

            if (targetParentId) {
                return false;
            }

            return when(this.store.get(targetParentId), lang.hitch(this, function (parent) {
                return this.isAncestor(source, parent);
            }));
        },

        getAncestors: function (item, onComplete) {
            // summary:
            //		Get the ancestors for the item
            //
            // item: Object
            //      The item to get the ancestors for, or the identity of the item
            //
            // onComplete: callback function
            //      Function that will be invoked when the getAncestors has completed
            //
            // tags:
            //      public

            if (!this.store) {
                onComplete([]);
                return;
            }

            var obj = typeof item === "object" ? item : this.store.get(item);
            when(obj, lang.hitch(this, function (storeItem) {
                when(this._getAncestors([storeItem]), onComplete);
            }));
        },

        _getAncestors: function (items) {
            // If the first item is a root then stop the recursion.
            if (this._hasRoot(items)) {
                return items.slice(0, -1);
            }

            var parentId = this.getParentIdentity(items[0]);
            if (!parentId) {
                return items.slice(0, -1);
            }

            return when(this.store.get(parentId), lang.hitch(this, function (parent) {
                if (parent) {
                    items.unshift(parent);
                    return this._getAncestors(items);
                }
                return items.slice(0, -1);
            }));
        },

        _hasRoot: function (items) {
            // summary:
            //      Checks to see if the first item is a root item.
            // tags:
            //      private
            var roots = this.roots || [this.root];

            if (!items.length || !roots.length || typeof this.getIdentity !== "function") {
                return false;
            }

            var identity = this.getIdentity(items[0]);

            return array.some(roots, function (root) {
                return root == identity;
            }, this);
        },

        refreshAncestors: function (item, filter) {
            // summary:
            //      Refreshes the data for each ancester. If a filter function is specified
            //      only ancestors matching the filter will be refreshed.
            //
            //  item: Object
            //      The item or the identity to refresh the ancestors for
            //
            //  filter: function
            //      Filter function used to select which ancestors to refresh. Optional
            //
            // tags:
            //      public

            var store = this.store,
                deferred = new Deferred(),
                refreshDeferreds = [],
                refreshed = [];

            this.getAncestors(item, function (ancestors) {

                array.forEach(ancestors, function (ancestor) {
                    if (!filter || filter(ancestor)) {
                        refreshDeferreds.push(when(store.refresh(store.getIdentity(ancestor)), function (item) {
                            refreshed.push(item);
                        }));
                    }
                });

                all(refreshDeferreds).then(function () {
                    deferred.resolve(refreshed);
                });

            });

            return deferred.promise;
        }
    });
});

},
'epi-cms/widget/viewmodel/_UpdateableStoreModelMixin':function(){
define("epi-cms/widget/viewmodel/_UpdateableStoreModelMixin", [
// dojo
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",

    "dojo/Deferred",
    "dojo/Evented",
    "dojo/json",
    "dojo/topic",
    "dojo/when",
    // epi
    "epi/dependency",

    "epi-cms/ApplicationSettings",
    "epi-cms/_ContentContextMixin",
    "epi-cms/core/ContentReference"
],

function (
// dojo
    array,
    declare,
    lang,

    Deferred,
    Evented,
    json,
    topic,
    when,
    // epi
    dependency,

    ApplicationSettings,
    _ContentContextMixin,
    ContentReference
) {

    return declare(Evented, {
        // tags:
        //      internal mixin

        // updateableStore: [private] dojo/store
        //      Underlying store that will be post data to server for items.
        updateableStore: null,

        // updateableStoreKey: [public] string
        //      Dependency key to resolve the updateableStore
        updateableStoreKey: "epi.cms.contentdata",

        // propertyNames: [public] Object
        //      Maps between client side property names and their server side counterparts
        propertyNames: {
            name: "icontent_name",
            nameInUrl: "iroutable_routesegment"
        },

        // refreshTranslatedAncestors: [public] boolean
        //      If true ancestors of the changed node will be refreshed
        refreshTranslatedAncestors: true,

        // onAddDelegate: [public] function
        //      Will be executed after add
        onAddDelegate: null,

        postscript: function () {
            this.inherited(arguments);
            this._setupStore();
        },

        _setupStore: function () {
            if (!this.updateableStore && this.updateableStoreKey) {
                this.updateableStore = dependency.resolve("epi.storeregistry").get(this.updateableStoreKey);
            }
        },

        rename: function (id, name) {
            // summary:
            //      Renames an item
            // tags:
            //      public

            var self = this,
                deferred = new Deferred();

            when(self.patch(id, { name: name, nameInUrl: null }), function (item) {
                when(self.refreshTranslated(id), function () {
                    deferred.resolve(item);
                }, function (e) {
                    deferred.reject(e);
                });
            }, function (e) {
                deferred.reject(e);
            });

            return deferred;
        },

        add: function (/*Object*/item, /*Function?*/onComplete) {
            // summary:
            //      Add new item
            // item: [Object]
            //      New item data
            // onComplete: [Function?]
            //      Callback function to do something when new content created
            // tags:
            //      Public

            var self = this;

            var def = new Deferred();

            when(self.updateableStore.add(item), function (response) {
                var id = self.getContentId(response),
                    parentId = self.getParentIdentity(item),
                    refreshDeferred = null;

                when(self.updateableStore.get(id), function (/*dojo/data/Item*/addedItem) {
                    if (self.getParentIdentity(addedItem) !== parentId) {
                        refreshDeferred = typeof onComplete == "function" && onComplete();
                    }
                    when(refreshDeferred)
                        .then(self.updateableStore.updateDependentStores.bind(this, addedItem))
                        .then(function () {
                            when(self._childrenChanged(parentId), function () {
                                when(self.refreshTranslated(id), function () {
                                    self.onSelect(id, true, self.onAddDelegate);
                                    def.resolve(addedItem);
                                });
                            });
                        });
                });
            });

            return def.promise;
        },

        patch: function (id, properties) {
            // summary:
            //      Patches the item with the submitted id and updates cache on success
            // id:
            //      Identity of the item to patch
            // properties:
            //      Properties to patch for the item
            // tags:
            //      public

            var self = this,
                deferred = new Deferred(),
                patchData = { id: id, properties: {} };

            //Fix the properties and change the names if needed
            for (var key in properties) {
                if (properties.hasOwnProperty(key)) {
                    patchData.properties[this._convertPropertyName(key)] = json.stringify(properties[key]);
                }
            }

            when(self.updateableStore.patch(patchData, { id: patchData.id }), function (item) {
                if (item && item.successful) {
                    var cacheData = {
                        contentLink: id,
                        properties: patchData.properties,
                        capabilities: { supportsVersions: false } // TODO: This is temporary until we have implemented dependant store capabilities checks
                    };
                    lang.mixin(cacheData, properties);

                    when(self.updateableStore.patchCache(cacheData), deferred.resolve, deferred.reject);
                } else {
                    return deferred.reject(item && item.validationErrors);
                }
            });

            return deferred;
        },

        _convertPropertyName: function (name) {
            return this.propertyNames[name] || name;
        },

        refreshTranslated: function (/*String*/contentId) {
            // summary:
            //      Refresh translated ancestor contents
            // tags:
            //      public

            var self = this,
                deferred = new Deferred(),
                failed;

            // Used to filter which ancestors to refresh
            function filter(item) {
                var id = self.getIdentity(item),
                    parentId = self.getParentIdentity(item);

                if (parentId === self.root || // is subroot
                    id === self.root || // is root
                    !item.missingLanguageBranch) { // already translated
                    return false;
                }

                return true;
            }

            if (!self.refreshTranslatedAncestors) {
                return deferred.resolve();
            }

            // Then refresh the ancestors
            when(self.refreshAncestors(contentId, filter), function (refreshedItems) {
                if (!(refreshedItems instanceof Array) || refreshedItems.length === 0) {
                    deferred.resolve();
                    return;
                }

                // Reverse array in order to get nearest ancestor
                refreshedItems.reverse();

                var hadAnyUntranslatedAncestor = array.some(refreshedItems, function (item) {
                    if (item.missingLanguageBranch) {
                        failed = self.getLabel(item);
                        return true;
                    }

                    return false;
                });

                // Check if any ancestors were not translated, in that case show notification of this
                when(self.store.get(contentId), function (contentItem) {
                    if (hadAnyUntranslatedAncestor && contentItem && !contentItem.missingLanguageBranch) {
                        self.emit("translationFailed", failed);
                    }

                    deferred.resolve();
                });
            });

            return deferred;
        },

        getContentId: function (/*String*/contentLink) {
            // summary:
            //      Get content link without its work version
            // tags:
            //      private

            return ContentReference.toContentReference(contentLink).id.toString();
        }

    });

});

},
'epi-cms/widget/ContextualContentForestStoreModel':function(){
﻿define("epi-cms/widget/ContextualContentForestStoreModel", [
// dojo
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",

    "dojo/aspect",
    "dojo/Deferred",
    "dojo/promise/all",
    "dojo/topic",
    "dojo/when",
    // epi
    "epi/shell/TypeDescriptorManager",
    "epi-cms/contentediting/_ContextualContentContextMixin",
    "epi-cms/core/ContentReference",
    "epi-cms/widget/ContentForestStoreModel"
],

function (
// dojo
    array,
    declare,
    lang,

    aspect,
    Deferred,
    promiseAll,
    topic,
    when,
    // epi
    TypeDescriptorManager,
    _ContextualContentContextMixin,
    ContentReference,
    ContentForestStoreModel
) {

    return declare([ContentForestStoreModel, _ContextualContentContextMixin], {
        // description:
        //      Adds a contextual root node to the tree.
        // tags:
        //      internal

        // handleSelection: [public] Boolean
        //      Flag indicates that we should handle selection on the given tree when it had refresh
        handleSelection: null,

        // previousSelection: [public] Object
        //      Holds information about previous selection in the tree
        //      Object's properties:
        //          selectedContent: [public] Object
        //              An instance of the "dijit/_TreeNode"
        //              Holds the previous selected content of the given tree
        //          selectedAncestors: [public] Array
        //              Holds the ancestors of the previous selected content of the given tree
        previousSelection: null,

        // =======================================================================
        // Public overrided functions

        postscript: function () {

            this.inherited(arguments);

            this._handles.push(topic.subscribe("/epi/cms/action/createlocalasset", lang.hitch(this, this.refreshRoots)));
            this._handles.push(topic.subscribe("/epi/cms/action/togglecreatemode", lang.hitch(this, this._toggleContextualContent)));

            this._handles.push(aspect.around(this, "pasteItem", lang.hitch(this, this._aroundPasteItem)));
            this._handles.push(aspect.around(this, "pasteItems", lang.hitch(this, this._aroundPasteItems)));
            this._handles.push(aspect.after(this.store, "get", lang.hitch(this, this._afterStoreGet)));
        },

        contextChanged: function (context, callerData) {

            // summary:
            //    Called when the currently loaded context changes
            // tags:
            //    protected

            this.inherited(arguments);

            this.refreshRoots();

        },

        getChildren: function (parentItem, onComplete, onError) {
            // summary:
            //      Calls onComplete() with array of child items of given parent item.
            // parentItem: Object
            //      Item from the dojo/store
            // tags:
            //      public

            if (parentItem === this.root) {
                when(this._getRootItems(), onComplete, onError);
            } else {
                // Overrides onComplete callback function in order to return an empty array of child content for pseudo contextual root
                if (this.isPseudoContextualRoot(parentItem)) {
                    onComplete([]);
                    return;
                }

                this.inherited(arguments);
            }
        },

        // TODO: The getAncestors method should be generalized to support multiple roots and moved to the _HierarchicalModelMixin. (bemc)
        getAncestors: function (item, /*Function*/ onComplete) {
            // summary:
            //      Calls onComplete() with an array of ancestors of the given item.
            // item: Object
            //      Item from the dojo/store

            var inherited = this.getInherited(arguments),
                onCompleteCallback = onComplete;

            // Get the contextual root ID and concatenate it with the roots array.
            when(this.getCurrentContent()).then(lang.hitch(this, function (currentContent) {
                var roots = this.roots.concat(currentContent.assetsFolderLink),
                    callback = lang.hitch(this, function (results) {
                        // Loop through the results from the end to find the first item that is a
                        // root. If a root is found then append the virtual root as the first item and resolve.
                        var ancestor;
                        for (var i = results.length - 1; i >= 0; i--) {
                            ancestor = results[i];

                            // Update as contextual root
                            if (this.isContextualRoot(ancestor)) {
                                results[i] = this.getContextualRoot(ancestor);
                            }

                            if (array.indexOf(roots, this.getIdentity(ancestor)) >= 0) {
                                results = results.slice(i);
                                results.unshift(this.root);
                                onComplete(results);
                                return;
                            }
                        }

                        // Otherwise if no root item is found resolve with only the virtual root.
                        onComplete([this.root]);
                    });

                // If the given item is a root node then resolve with only the virtual root.
                if (array.indexOf(roots, this.getIdentity(item)) >= 0) {
                    onComplete([this.root]);
                }

                //Replace the default callback
                onCompleteCallback = callback;

            })).always(lang.hitch(this, function () {
                inherited.apply(this, [item, onCompleteCallback]);
            }));
        },

        getObjectIconClass: function (/*dojo/data/Item*/contentItem, /*String*/fallbackIconClass) {
            // summary:
            //      Get icon class for content based on its content link
            // contentItem: [dojo/data/Item]
            //      Content item data
            // fallbackIconClass: [String]
            //      Default icon class in case nothing returned
            // tags:
            //      public, extension

            var dataType = this._currentContext && this._currentContext.dataType;

            if (!dataType) {
                return this.inherited(arguments);
            }

            var contentIconClass = TypeDescriptorManager.getValue(dataType, "iconClass");

            return this.isContextualContent(contentItem)
                ? (contentIconClass
                    ? fallbackIconClass + " " + contentIconClass + "Contextual"
                    : fallbackIconClass
                )
                : this.inherited(arguments);
        },

        // =======================================================================
        // Public functions

        onToggleContentDisplay: function (/*String*/target, /*Boolean*/display) {
            // summary:
            //      Callback whenever widget switcher changed
            // target: [String]
            //      Assets folder content item that want to show/hide
            // display: [Boolean]
            // tags:
            //      public, callback
        },

        onRefreshRoots: function (/*dojo/data/Item*/refreshItem) {
            // summary:
            //      Callback whenever context changed
            // refreshItem: [dojo/data/Item]
            //      Assets folder content item that want to refresh
            // tags:
            //      public, callback
        },

        refreshRoots: function (sender) {
            // summary:
            //      Refresh root contents
            // sender: Object
            //      The instance that published the topic
            // tags:
            //      public

            //If the topic was published from this model do not react on it.
            if (sender === this) {
                return true;
            }

            var self = this;
            return when(self._getRootItems(), function (children) {
                self.onChildrenChange(self.root, children);
                if (self.get("handleSelection") === true) {
                    return self._setSelection(lang.clone(children).pop());
                }
            });
        },

        canCopy: function (item) {
            // summary:
            //      Determine whether the copy action is supported for an item.
            // item: Object
            //      Item from the dojo/store
            // tags:
            //      public, override

            // Do not allow to copy the current contextual folder
            return this.isContextualContent(item) ? false : this.inherited(arguments);
        },

        canCut: function (/*dojo/data/Item*/item) {
            // summary:
            //      Determine whether the given item is able to be cut by the current user.
            // item: [dojo/data/Item]
            //      The source content item data that is being copied/cut
            // tags:
            //      public, override

            // Do not allow to cut the current contextual folder
            return this.isContextualContent(item) ? false : this.inherited(arguments);
        },

        canPaste: function (item, target, isCopy) {
            // summary:
            //      Determines whether the given item can be moved or copied from its current parent to a new location
            //      Primarily used when validating drag & drop operations.
            // item: [dojo/data/Item]
            //      The source content item data that is being copied/cut
            // target: [dojo/data/Item]
            //      The destination content item data
            // isCopy: Boolean
            //      If true, action is copy, otherwise is cut
            // tags:
            //      public override

            var self = this,
                canPaste = this.inherited(arguments),
                deferred = new Deferred();

            if (isCopy) {
                deferred.resolve(canPaste);
            } else {
                self.getAncestors(target, function (ancestors) {

                    // If we paste into current contextual folder or one of its children
                    var isPastingToContextualContent = self.isContextualContent(target) || self.hasContextual(ancestors);
                    if (isPastingToContextualContent) {
                        when(self.getCurrentContent()).then(function (currentContent) {

                            // and cut content is the current context
                            if (ContentReference.compareIgnoreVersion(item.contentLink, currentContent.contentLink)) {
                                deferred.resolve(false);
                            } else {
                                // or cut content is an ancestor of the current context
                                self.getAncestors(currentContent.contentLink, function (ancestors) {

                                    var itemIsParent = array.some(ancestors, function (ancestor) {
                                        return item.contentLink === ancestor.contentLink;
                                    }, this);

                                    deferred.resolve(itemIsParent ? false : canPaste);
                                });
                            }
                        }).otherwise(function () {
                            deferred.resolve(false);
                        });
                    } else {
                        deferred.resolve(canPaste);
                    }
                });
            }

            return deferred.promise;
        },

        filterAncestors: function (ancestors) {
            // summary:
            //      Filter out the pseudo contextual content from the given ancestor list incase pseudo content is not the deepest ancestor.
            // ancestors: [Array]
            //      Collection of ancestor content
            // tags:
            //      public

            if (!(ancestors instanceof Array)) {
                return ancestors;
            }

            var pseudoIndex = ancestors.indexOf(this._getPseudoContextualContent().toString());
            if (pseudoIndex !== -1 && pseudoIndex < ancestors.length - 1) {
                ancestors.splice(pseudoIndex, 1);
            }

            return ancestors;
        },

        isContextualContent: function (item) {
            // summary:
            //    Overrides _ContextualContentContextMixin.isContextualContent to only report items included in the tree as contextual content.
            // tags:
            //     public, override

            return !!item && this.isSupportedType(item.typeIdentifier) && this.inherited(arguments);
        },

        // =======================================================================
        // Private functions

        _toggleContextualContent: function (/*Boolean*/createMode) {
            // summary:
            //      Show/Hide contextual content (local asset folder)
            // createMode: [Boolean]
            //      Flag indicates that the current mode is create new content or not
            // tags:
            //      private

            when(this.getCurrentContent(), lang.hitch(this, function (currentContent) {
                this.onToggleContentDisplay(currentContent.assetsFolderLink, !createMode);
                this._setSelection(null); // Pass 'null' value in order to set selection to the default target on the tree
            }));
        },

        _setSelection: function (/*Object*/targetItem) {
            // summary:
            //      Set selection tree node by the given content link string
            // targetItem: [Object]
            //      An instance of "dojo/data/Item"
            // tags:
            //      private

            var defaultTarget = this.roots.length > 0 ? this.roots[0].toString() : (!targetItem ? null : targetItem.assetsFolderLink),
                target = targetItem != null ? targetItem.contentLink : defaultTarget,
                previousSelection = this.get("previousSelection"),
                previousContextualSelection = this.get("previousContextualSelection"),
                previousSelectedContextualContent = previousContextualSelection && previousContextualSelection.selectedContent,
                selectDeferred = new Deferred();

            //Return if there is no new target to select or any previous selection
            if (!(target && previousSelection)) {
                selectDeferred.resolve();
                return selectDeferred.promise;
            }

            var focusTarget = false, // Set focus to the target is FALSE to prevent steal focus of the others
                previousSelectedContent = previousSelection.selectedContent,
                previousSelectedAncestors = previousSelection.selectedAncestors,
                previousIsVirtualRoot = previousSelectedContent.item && !previousSelectedContent.item.contentLink;

            var onComplete = function () {
                selectDeferred.resolve();
            };

            if (previousIsVirtualRoot || this.hasContextual(previousSelectedAncestors)) {
                this.onSelect(target, focusTarget, onComplete);
            } else if (previousSelectedContent.item.contentLink === defaultTarget
                    && previousSelectedContextualContent
                    && previousSelectedContextualContent.item
                    && previousSelectedContextualContent.item.contentLink) {
                this.onSelect(previousSelectedContextualContent.item.contentLink, focusTarget, onComplete);
            } else {
                selectDeferred.resolve();
            }

            return selectDeferred.promise;
        },

        getRoots: function (filterContentAssetsFolder) {
            // summary:
            //      Returns the available roots for the current context
            //
            // filterContentAssetsFolder: Boolean?
            //      Set to true if you want to remove the content assets folder from the available roots
            //
            // returns: Promise
            //      A promise that will be resolved with the current roots for the context
            // tags:
            //      internal

            var def = new Deferred();

            var contentAssetRoot = this._getPseudoContextualContent().toString();
            var filterContentAssetsRootFolder = function (rootItem) {
                if (!filterContentAssetsFolder) {
                    return true;
                }
                return rootItem !== contentAssetRoot;
            };

            var mapContentLink = function (rootItem) {
                return rootItem.contentLink;
            };

            when(this._getRootItems(), function (rootItems) {
                def.resolve(rootItems.map(mapContentLink).filter(filterContentAssetsRootFolder));
            }, def.reject);

            return def.promise;
        },

        _getRootItems: function () {
            // summary:
            //      Resolves the root items, including the contextual root.
            // tags:
            //      private

            var self = this,
                currentStore = self.store,
                // Load all ordinary roots
                rootLoads = array.map(self.roots, function (item) {
                    return currentStore.get(item);
                });

            var contextRootLoad =
                // REMARK: This is not a longterm solution. We should push the currentContent to the model instead of getting it from the ContentContextMixin.
                // this model could be use when we do not have a editing context. E.g. an addon that just want's to pick a page/image etc.
                when(promiseAll([self.getCurrentContent(), self.getCurrentContext()]), function (result) {

                    var currentContent = result[0],
                        currentContext = result.length > 1 ? result[1] : null;

                    var currentContentWithoutVersion = new ContentReference(currentContent.contentLink).createVersionUnspecificReference().toString();
                    return when(currentStore.get(currentContentWithoutVersion), function (content) {
                        var inCreateMode = currentContext && currentContext.currentMode === "create";
                        if ((inCreateMode && self.get("view") === "contentselector") || !self.canHaveContextualContent(content)) {
                            return null;
                        }

                        // If the current context provided a context root, load it
                        var assetsFolderLink = content.assetsFolderLink;
                        if (assetsFolderLink) {
                            var getContextualContent = function (localFolderLink) {

                                // We need to call refresh local folder here in order to set its selection on the folder tree and refresh its icon and label.
                                return when(currentStore.get(localFolderLink), function (contextualContent) {
                                    // Take query in order to filters properly children that are allowed.
                                    // So that, we will show the expanded/collapsed icon for the root node correctly.
                                    var query = {
                                        id: contextualContent.contentLink,
                                        typeIdentifiers: self.typeIdentifiers,
                                        allLanguages: self.showAllLanguages
                                    };

                                    return when(currentStore.query(query), function (refreshedContent) {
                                        // Only show contextual content in the container that supports its type identifier
                                        if (typeof self.isSupportedType === "function" && !self.isSupportedType(refreshedContent.typeIdentifier)) {
                                            return null;
                                        }

                                        if (!self.isPseudoContextualRoot(refreshedContent)) {
                                            return self.getContextualRoot(refreshedContent);
                                        } else {
                                            return when(self.onRefreshRoots(refreshedContent), function () {
                                                return self._modifyContentAccess(refreshedContent, content);
                                            });
                                        }
                                    });
                                });
                            };

                            if (assetsFolderLink === self._getPseudoContextualContent().toString()) {
                                return when(currentStore.refresh(currentContentWithoutVersion), function (refreshedContent) {
                                    return getContextualContent(refreshedContent.assetsFolderLink);
                                });
                            } else {
                                return getContextualContent(assetsFolderLink);
                            }
                        }

                        return null;
                    });
                },
                //We want a return even the context is not a content item
                function () {
                    return null;
                });

            // Add context root load to the ordinary root loads
            rootLoads.push(contextRootLoad);

            return when(promiseAll(rootLoads), function (rootItems) {
                // When loads are done, filter the ones that resolve null
                return array.filter(rootItems, Boolean);
            });
        },

        _aroundPasteItem: function (originalFunction) {
            // summary:
            //      Stubs to do somethings before pasteItem processing
            // tags:
            //      private

            var self = this;

            return function (/*dojo/data/Item*/childItem, /*dojo/data/Item*/oldParentItem, /*dojo/data/Item*/newParentItem, /*Boolean*/copy, /*Integer?*/sortIndex) {
                var targetItem = newParentItem,
                    createAsLocalAsset = self.get("createAsLocalAsset");

                // If the new parent item is pseudo contextual content, get current content context item insteads
                // In additional, change the value of the "createAsLocalAsset" to TRUE if its value is FALSE, otherwise do nothing
                return when(self.getCurrentContent()).then(function (contextContentItem) {

                    if (!createAsLocalAsset) {
                        createAsLocalAsset = self.isPseudoContextualRoot(targetItem);
                        self.set("createAsLocalAsset", createAsLocalAsset);
                    }

                    createAsLocalAsset && (targetItem = contextContentItem);
                }).always(function () {
                    return when(originalFunction.apply(self, [childItem, oldParentItem, targetItem, copy, sortIndex]), function (response) {
                        if (createAsLocalAsset) {
                            self.set("createAsLocalAsset", false);
                            return when(self.refreshRoots(), function () {

                                // REMARK: This is not a very good place to do this, but when we can't rely on the
                                // store notfications for the contextual content we need to do this
                                // If the item got a new parent, publish the local asset created topic
                                topic.publish("/epi/cms/action/createlocalasset", self);

                                return response;
                            });
                        } else {
                            return response;
                        }
                    });
                });
            };
        },

        _aroundPasteItems: function (originalFunction) {
            // summary:
            //      Aspect around PasteItems to handle pasting content to the local asset folder.
            //      In this case the targetItem has to be replaced with the contentOwner contentLink
            // tags:
            //      private

            var self = this;
            return function (/*Array*/sourceItems, /*dojo/data/Item*/targetItem, /*Boolean*/copy, /*Integer?*/sortIndex) {
                var createAsLocalAsset = self.get("createAsLocalAsset");
                // If the new parent item is pseudo contextual content, get current content context item instead
                // In additional, change the value of the "createAsLocalAsset" to TRUE if its value is FALSE, otherwise do nothing
                return when(self.getCurrentContent()).then(function (contextContentItem) {
                    if (!createAsLocalAsset) {
                        createAsLocalAsset = self.isPseudoContextualRoot(targetItem);
                        self.set("createAsLocalAsset", createAsLocalAsset);
                    }

                    createAsLocalAsset && (targetItem = contextContentItem);
                }).always(function () {
                    return when(originalFunction.apply(self, [sourceItems, targetItem, copy, sortIndex]), function (response) {
                        if (createAsLocalAsset) {
                            self.set("createAsLocalAsset", false);
                        } else {
                            return response;
                        }
                    });
                });
            };
        },

        _afterStoreGet: function (/*Object*/promise) {
            // summary:
            //      Decorates result before send it back
            // promise: [Object]
            //      Store get() result
            // tags:
            //      private

            var self = this;
            return when(promise, function (result) {
                return when(self.getCurrentContent(), function (currentContent) {
                    return self._modifyContentAccess(result, currentContent);
                }, function (error) {
                    //If we fail to load the current content return the value from the store
                    return result;
                });
            });
        },

        _modifyContentAccess: function (/*dojo/data/Item*/targetContent, /*dojo/data/Item*/sourceContent) {
            // summary:
            //      Modify access marks for the given pseudo contextual content
            // targetContent: [dojo/data/Item]
            //      Pseudo contextual content asset folder that needed to overrides access marks
            // sourceContent: [dojo/data/Item]
            //      Source content that used to copy access marks from
            // tags:
            //      private

            // Only modify the content item name if it is a contextual content
            if (this.isContextualContent(targetContent)) {
                var contextualRoot = this.getContextualRoot(targetContent);
                // Only modify the access maske if its the fake folder
                return this.isPseudoContextualRoot(targetContent)
                    ? lang.mixin(contextualRoot, {
                        isPreferredLanguageAvailable: sourceContent.isPreferredLanguageAvailable,
                        hasTranslationAccess: sourceContent.hasTranslationAccess,
                        accessMask: sourceContent.accessRights, // Mix on the access rights that does not include language access rights
                        ownerContentLink: sourceContent.contentLink
                    })
                    : contextualRoot;
            }

            return targetContent;
        }

    });

});

},
'epi-cms/contentediting/_ContextualContentContextMixin':function(){
﻿// We need to disable eqeqeq in this file because we need to do == comparisons because some components are sending the ids as int and some as strings
/*eslint eqeqeq:0*/

define("epi-cms/contentediting/_ContextualContentContextMixin", [
// dojo
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",

    "dojo/when",
    // epi
    "epi-cms/_ContentContextMixin",
    "epi/shell/TypeDescriptorManager",

    "epi-cms/ApplicationSettings",
    "epi-cms/contentediting/PseudoContextualCommandDecorator",
    "epi-cms/core/ContentReference"
],

function (
// dojo
    array,
    declare,
    lang,

    when,
    // epi
    _ContentContextMixin,
    TypeDescriptorManager,

    // EPi CMS
    ApplicationSettings,
    PseudoContextualCommandDecorator,
    ContentReference) {

    return declare([_ContentContextMixin], {
        // summary:
        //      Provides stubs to interacts with contextual content of the current context.
        // tags:
        //      public

        // _commandDecorator: [private] "epi-cms/contentediting/PseudoContextualCommandDecorator"
        //      Decorates commands for a pseudo contextual content
        _commandDecorator: null,

        // =======================================================================
        // Public overrided functions

        postscript: function () {

            this.inherited(arguments);

            this._commandDecorator = this._commandDecorator || new PseudoContextualCommandDecorator();
        },

        destroy: function () {

            this._commandDecorator && this._commandDecorator.destroy();

            this.inherited(arguments);
        },

        // =======================================================================
        // Public functions

        isTypeOfRoot: function (/*dojo/data/Item*/contentItem) {
            // summary:
            //      Checks to see if the item is type of root node such as: root, sub root, context root
            // contentItem: [dojo/data/Item]
            //      The given data item object that wanted to verify
            // returns: [Boolean]
            // tags:
            //      public, extension

            var inherited = this.inherited(arguments),
                isContextualRoot = this.isContextualContent(contentItem);

            return inherited != null ? !!(inherited || isContextualRoot) : !!(contentItem && isContextualRoot);
        },

        isContextualContent: function (/*dojo/data/Item*/contentItem) {
            // summary:
            //      Verifies the given content item data is contextual or not
            // contentItem: [dojo/data/Item]
            //      The given data item object that wanted to verify
            // returns: [Boolean]
            // tags:
            //      public

            return this.isContextualRoot(contentItem) || this.isPseudoContextualRoot(contentItem);
        },

        isContextualRoot: function (/*dojo/data/Item*/contentItem) {
            // summary:
            //      Checks to see the given data item object is contextual root or not
            // contentItem: [dojo/data/Item]
            //      The given data item object that wanted to verify
            // returns: [Boolean]
            // tags:
            //      public

            return contentItem.parentLink == this._getPseudoContextualContent();
        },

        isPseudoContextualRoot: function (/*dojo/data/Item*/contentItem) {
            // summary:
            //      Checks to see if the item is a pseudo contextual content
            // contentItem: [dojo/data/Item]
            //      The given data item object that wanted to verify
            // returns: [Boolean]
            // tags:
            //      public

            var contentReference = ContentReference.toContentReference(contentItem.contentLink);
            return !contentReference.providerName && contentReference.id == this._getPseudoContextualContent();
        },

        hasContextual: function (/*Array*/contentAncestors) {
            // summary:
            //      Verifies the given tree node has a ancestor is contextual or not
            // contentAncestors: [Array]
            //      A collection of "dijit/_TreeNode" item object (dojo/data/Item) that are ancestors of the given selected content
            // returns: [Boolean]
            // tags:
            //      public

            if (!(contentAncestors instanceof Array) || contentAncestors.length <= 0) {
                return false;
            }

            return array.some(contentAncestors, this.isContextualContent, this);
        },

        canHaveContextualContent: function (/*dojo/data/Item*/contentItem) {
            // summary:
            //      Verifies the given object allowed to show contextual content or not
            // contentItem: [dojo/data/Item]
            //      Content item data object
            // returns: [Boolean]
            // tags:
            //      public

            var contentReference = ContentReference.toContentReference(contentItem.contentLink);
            return !array.some(this.notSupportContextualContents, function (id) {
                return !contentReference.providerName && contentReference.id == id;
            });
        },

        getContextualRoot: function (/*dojo/data/Item*/contentItem) {
            // summary:
            //      Get contextual content from the given content item data
            // contentItem: [dojo/data/Item]
            //      Content item data object
            // returns: [dojo/data/Item]
            //      A clone object that acts as a contextual content (local content assets folder for the current content context)
            // tags:
            //      public

            if (typeof contentItem !== "object") {
                return null;
            }

            var modifiedItem = lang.clone(contentItem);

            // Change the original name of the content (in this case is "Content Assets" root folder) based on the current content context's type identifier
            // Will be (core content data types):
            //      "For This Page" for the "episerver.core.pagedata" type identifier
            //      "For This Block" for the "episerver.core.blockdata" type identifier
            //      "For This {0}" if the given type identifier not found
            modifiedItem.name = this.getContextualRootName(modifiedItem);

            // Set "hasChildren" property to FALSE in order to hides collapsed icon for pseudo contextual content.
            if (this.isPseudoContextualRoot(modifiedItem)) {
                modifiedItem.hasChildren = false;
            }

            return modifiedItem;
        },

        getContextualRootName: function (/*dojo/data/Item?*/contentItem) {
            // summary:
            //      Get contextual root name based on the current content context data type.
            // contentItem: [dojo/data/Item?]
            //      Content item data object
            // returns: [String]
            //      "For This Page" for the "episerver.core.pagedata" type
            //      "For This Block" for the "episerver.core.blockdata" type
            //      "For This {0}" if the given type identifier not found
            // tags:
            //      public

            var ownerTypeIdentifier = contentItem && contentItem.ownerTypeIdentifier,
                typeIdentifier = ownerTypeIdentifier || (this._currentContext && this._currentContext.dataType) || "",
                typeName = TypeDescriptorManager.getResourceValue(typeIdentifier, "name", true);

            var label =  TypeDescriptorManager.getResourceValue(typeIdentifier, "forthis", false);

            return typeName ? lang.replace(label, [typeName]) : label;
        },

        decoratePseudoContextualCommands: function (/*Arrays*/commands) {
            // summary:
            //      Converts the given commands to work in case pseudo contextual content
            // commands: [Array]
            //      A collection of object that is instance of "epi/shell/command/_Command" class
            // tags:
            //      public

            when(this.getCurrentContent()).then(lang.hitch(this, function (contentItem) {
                this._commandDecorator.decorateCommands(commands, contentItem, /*pseudoContextualRootFilter*/lang.hitch(this, this.isPseudoContextualRoot));
            })).otherwise(function () {});
        },

        // =======================================================================
        // Protected functions

        _getPseudoContextualContent: function () {
            // summary:
            //      Get pseudo contextual content
            // returns: [Integer]
            //      The content link
            // tags:
            //      protected

            return ApplicationSettings.contentAssetsFolder;
        }

    });

});

},
'epi-cms/contentediting/PseudoContextualCommandDecorator':function(){
﻿define("epi-cms/contentediting/PseudoContextualCommandDecorator", [
// dojo
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",

    "dojo/aspect",
    // epi
    "epi/shell/command/_Command",
    "epi/shell/command/_ClipboardCommandMixin",
    "epi/shell/command/_SelectionCommandMixin"
],

function (
// dojo
    array,
    declare,
    lang,

    aspect,
    // epi
    _Command,
    _ClipboardCommandMixin,
    _SelectionCommandMixin
) {

    return declare(null, {
        // summary:
        //      Decorates commands for a pseudo contextual content
        // tags:
        //      internal

        // =======================================================================
        // Public functions

        destroy: function () {

            this._clearContextualCommandHandlers();
        },

        decorateCommands: function (/*Arrays*/commands, /*dojo/data/Item*/newModel, /*Function?*/pseudoContextualRootFilter) {
            // summary:
            //      Converts the given commands to work in case pseudo contextual content
            // commands: [Array]
            //      A collection of object that is instance of "epi/shell/command/_Command" class
            // newModel: [dojo/data/Item]
            // pseudoContextualRootFilter: [Function?]
            //      Command filter function
            // tags:
            //      public

            this._clearContextualCommandHandlers();

            if (!(commands instanceof Array) || commands.length <= 0 || typeof newModel !== "object") {
                return;
            }

            this._decorateCommands(commands, newModel, pseudoContextualRootFilter);
        },

        // =======================================================================
        // Private functions

        _decorateCommands: function (/*Arrays*/commands, /*dojo/data/Item*/newModel, /*Function?*/pseudoContextualRootFilter) {
            // summary:
            //      Converts the given commands to work in case pseudo contextual content
            // commands: [Array]
            //      A collection of object that is instance of "epi/shell/command/_Command" class
            // newModel: [dojo/data/Item]
            // pseudoContextualRootFilter: [Function?]
            //      Command filter function
            // tags:
            //      public

            array.forEach(commands, function (command) {
                if (!this._isValidCommand(command)) {
                    return;
                }

                // Do nothing if the current selected item is a real contextual content
                if (typeof pseudoContextualRootFilter === "function" && !pseudoContextualRootFilter(this._getCommandModel(command))) {
                    return;
                }

                this._onAroundCommandExecute(command, newModel);
            }, this);
        },

        _getCommandModel: function (command) {
            // summary:
            //      Returns the commands model.
            // returns:
            //      The first item if the model is an array; otherwise just the model.
            // tags:
            //      private

            if (command.model && command.model instanceof Array) {
                if (command.model.length === 1) {
                    return command.model[0];
                }
                return null;
            }
            return command.isInstanceOf(_SelectionCommandMixin) ? command.get("selectionData") : command.model;
        },

        _onAroundCommandExecute: function (/*Object*/command, /*Object*/newModel) {
            // summary:
            //      Stub to do something before and after the execution of the given command object
            // command: [Object]
            //      An instance of "epi/shell/command/_Command"
            // newModel: [Object]
            //      An instance of "dojo/data/Item"
            // tags:
            //      private

            var self = this;
            self._contextualCommandHandlers.push(aspect.around(command, "execute", function (originalFunction) {
                return function () {
                    var originalData = self._decorateCommandBeforeExecute(command, newModel);

                    originalFunction.apply(command, arguments);

                    self._restoreCommandAfterExecute(command, originalData);
                };
            }));
        },

        _decorateCommandBeforeExecute: function (/*Object*/command, /*Object*/newModel) {
            // summary:
            //      Decorates the given command: marks it always TRUE for "canExecute", "isAvailable" and "createAsLocalAsset" flags
            // command: [Object]
            //      An instance of "epi/shell/command/_Command"
            // newModel: [Object]
            //      An instance of "dojo/data/Item"
            // returns: [Object] original command data
            //      Default data:
            //          createAsLocalAsset: [Boolean]
            //      Optional data:
            //          model: [Object]
            //          data: [Object]
            // tags:
            //      private

            var originalData = this._decorateCommand(command);

            if (this._isNormalCommand(command)) {
                lang.mixin(originalData, this._decorateNormalCommand(command, newModel));
            } else if (command.isInstanceOf(_SelectionCommandMixin)) {
                lang.mixin(originalData, this._decorateSelectionCommand(command, newModel));
            }

            command.set({
                canExecute: true,
                isAvailable: true,
                createAsLocalAsset: true
            });

            return originalData;
        },

        _decorateCommand: function (/*Object*/command) {
            // summary:
            //      Returns the original value for the createAsLocalAsset flag and unwatches the commands model watches
            // command: [Object]
            //      An instance of "epi/shell/command/_Command"
            // tags:
            //      private

            var originalData = {
                createAsLocalAsset: command.get("createAsLocalAsset")
            };

            command.isInstanceOf(_Command) && command.unwatchModelChange();
            command.isInstanceOf(_ClipboardCommandMixin) && command.unwatchClipboardChange();
            command.isInstanceOf(_SelectionCommandMixin) && command.unwatchSelectionChange();

            return originalData;
        },

        _decorateNormalCommand: function (/*Object*/command, /*Object*/newModel) {
            // summary:
            //      Replaces the model in the command with the specified newModel and returns an object with the original model
            // command: [Object]
            //      An instance of "epi/shell/command/_Command"
            // newModel: [Object]
            //      An instance of "dojo/data/Item"
            // returns: [Object] original command data
            //      model: [Object]
            // tags:
            //      private

            var originalData = {
                model: command.get("model")
            };

            command.set("model", newModel);

            return originalData;
        },

        _decorateSelectionCommand: function (/*Object*/command, /*Object*/newModel) {
            // summary:
            //      Replaces the selection data in the command with the specified newModel and returns an object with the original selection data.
            // command: [Object]
            //      An instance of "epi/shell/command/_SelectionCommandMixin"
            // newModel: [Object]
            //      An instance of "dojo/data/Item"
            // returns: [Object] original command data
            //      data: [Object]
            // tags:
            //      private

            var originalData = {
                data: command.get("selectionData")
            };

            // Selection command case
            // If the given command object is instance of "epi/shell/command/_Command" and had mixed-in "epi/shell/command/_SelectionCommandMixin"
            var model = command.get("model");
            if (model && typeof model.refreshRoots == "function") {
                command.onRefreshSelection = function () {
                    return model.refreshRoots();
                };
            }

            command.selection.set("data", [{
                data: newModel,
                type: command.get("selectionType")
            }]);

            return originalData;
        },

        _restoreCommandAfterExecute: function (/*Object*/command, /*Object*/originalData) {
            // summary:
            //      Stub to do something after execution of the given command object
            // command: [Object]
            //      An instance of "epi/shell/command/_Command"
            // originalData: [Object]
            // tags:
            //      private

            this._restoreCommand(command, originalData);

            if (this._isNormalCommand(command)) {
                this._restoreNormalCommand(command, originalData);

                return;
            }

            // _SelectionCommandMixin
            if (command.isInstanceOf(_SelectionCommandMixin)) {
                this._restoreSelectionCommand(command, originalData);
            }
        },

        _restoreCommand: function (/*Object*/command, originalData) {
            // summary:
            //      Restores the given command to its original states
            // command: [Object]
            //      An instance of "epi/shell/command/_Command"
            // tags:
            //      private

            command.set("createAsLocalAsset", originalData.createAsLocalAsset);

            command.isInstanceOf(_Command) && command.watchModelChange();
            command.isInstanceOf(_ClipboardCommandMixin) && command.watchClipboardChange();
            command.isInstanceOf(_SelectionCommandMixin) && command.watchSelectionChange();
        },

        _restoreNormalCommand: function (/*Object*/command, /*Object*/originalData) {
            // summary:
            //      Restores the given normal command to its original states
            // command: [Object]
            //      An instance of "epi/shell/command/_Command"
            // originalData: [Object]
            // tags:
            //      private

            command.set("model", originalData.model);
        },

        _restoreSelectionCommand: function (/*Object*/command, /*Object*/originalData) {
            // summary:
            //      Restores the given selection command to its original states
            // command: [Object]
            //      An instance of "epi/shell/command/_SelectionCommandMixin"
            // originalData: [Object]
            // tags:
            //      private

            command.selection.set("data", [{
                data: originalData.data,
                type: command.get("selectionType")
            }]);
        },

        _clearContextualCommandHandlers: function () {
            // summary:
            //      Clear all handlers for pseudo contextual content commands
            // tags:
            //      private

            if (this._contextualCommandHandlers instanceof Array && this._contextualCommandHandlers.length > 0) {
                array.forEach(this._contextualCommandHandlers, function (handler) {
                    typeof handler.remove === "function" && handler.remove();
                });
            }

            this._contextualCommandHandlers = [];
        },

        _isNormalCommand: function (/*Object*/command) {
            // summary:
            //      Verifies the given command object is normal command or not
            //      The given command must be an instance of "epi/shell/command/_Command" only
            // command: [Object]
            //      The given object to verifies
            // returns: [Boolean]
            // tags:
            //      private

            return !(command.isInstanceOf(_SelectionCommandMixin) || command.isInstanceOf(_ClipboardCommandMixin));
        },

        _isValidCommand: function (/*Object*/command) {
            // summary:
            //      Verifies the given object is instance of "epi/shell/command/_Command" class or not.
            //      In additional, the given command must have its model.
            // command: [Object]
            //      The given object to verifies
            // returns: [Boolean]
            // tags:
            //      private

            return !!(command && command.isInstanceOf(_Command) && this._getCommandModel(command));
        }

    });

});

},
'epi/shell/command/_Command':function(){
﻿define("epi/shell/command/_Command", [
// dojo
    "dojo/_base/declare",
    "dojo/Stateful"
],

function (
// dojo
    declare,
    Stateful
) {

    return declare([Stateful], {
        // summary:
        //      A base class implementation for commands.
        // tags:
        //      public abstract

        // label: [public] String
        //      The action text of the command to be used in visual elements.
        label: null,

        // tooltip: [public] String
        //      The description text of the command to be used in visual elements.
        tooltip: null,

        // iconClass: [public] String
        //      The icon class of the command to be used in visual elements.
        iconClass: null,

        // category: [readonly] String
        //      A category which provides a hint about how the command could be displayed, e.g. "setting".
        category: null,

        // order: [readonly] Integer
        //      An ordering indication used when generating a ui for this command.
        //      Commands with order indication will be placed before commands with no order indication.
        order: null,

        // model: [public] Object
        //      The model which this command will use to determine if it can execute; this may
        //      also be the subject of the execute action.
        model: null,

        // canExecute: [readonly] Boolean
        //      Flag which indicates whether this command is able to be executed.
        canExecute: false,

        // isAvailable: [readonly] Boolean
        //      Flag which indicates whether this command is available in the current context.
        isAvailable: true,

        // _callOnModelChange: [private] Boolean
        //      Flag indicating if _onModelChange should be called when the model changes.
        _callOnModelChange: false,

        // =======================================================================
        // Public overridden functions

        postscript: function ()  {
            // summary:
            //      Watch model and initialize model dependent properties.
            // tags:
            //      public

            this.inherited(arguments);

            // Make initial onModelChange call if a model was injected in the constructor.
            this.model && this._onModelChange();
            this._callOnModelChange = true;
        },

        // =======================================================================
        // Public functions

        destroy: function () {
            // summary:
            //      Subclasses needing to release resources should override this method
            // tags:
            //      public
        },

        execute: function () {
            // summary:
            //      Executes this command if canExecute is true; otherwise do nothing.
            // tags:
            //      public

            if (this.isAvailable && this.canExecute) {
                return this._execute();
            }
        },

        watchModelChange: function () {
            // summary:
            //      Register a watch handler when the command change its model
            // tags:
            //      internal

            this.set("_callOnModelChange", true);
        },

        unwatchModelChange: function () {
            // summary:
            //      Clear the registered watch handler for command model change
            // tags:
            //      internal

            this.set("_callOnModelChange", false);
        },

        // =======================================================================
        // Protected functions

        _execute: function () {
            // summary:
            //      Executes this command assuming canExecute has been checked. Subclasses should override this method.
            // tags:
            //      protected
        },

        _onModelChange: function () {
            // summary:
            //      Updates canExecute after the model has been updated. Subclasses should override this method.
            // tags:
            //      protected
        },

        _modelSetter: function (model) {
            // summary:
            //
            // tags:
            //      protected

            this.model = model;
            this._callOnModelChange && this._onModelChange();
        }
    });
});

},
'epi/shell/command/_ClipboardCommandMixin':function(){
﻿define("epi/shell/command/_ClipboardCommandMixin", [
// dojo
    "dojo/_base/declare",
    "dojo/_base/lang"
],

function (
// dojo
    declare,
    lang
) {

    return declare(null, {
        // summary:
        //      Stub to handles content copy/paste for an instance of "epi/shell/command/_Command" object
        // tags:
        //      internal, mixin


        // clipboard: [public] Object
        //      Hold information about copy/paste content
        clipboard: null,

        // =======================================================================
        // Public overrided functions

        postscript: function () {
            // summary:
            //      Register a watch hanlder when the command change its model
            //      Watch model and initialize model dependent properties.
            // tags:
            //      public, overrides

            this.inherited(arguments);

            this.watchClipboardChange();
        },

        // =======================================================================
        // Public functions

        onRefreshClipboard: function () {
            // summary:
            //      Stub to do somethings when clipboard refreshed
            // tags:
            //      public, callback
        },

        watchClipboardChange: function () {
            // summary:
            //      Register a watch hanlder when the command change its model
            // tags:
            //      public

            this._clipboardChangeWatcher = this.clipboard.watch("data", lang.hitch(this, this._onModelChange));
        },

        unwatchClipboardChange: function () {
            // summary:
            //      Clear the registered watch handler for command model change
            // tags:
            //      public

            this._clipboardChangeWatcher && this._clipboardChangeWatcher.unwatch();
        },

        // =======================================================================
        // Protected functions

        _clipboardDataGetter: function () {
            // summary:
            //      Return clipboard content item data
            // tags:
            //      protected

            return this._getClipboardData();
        },

        _clipboardTypeGetter: function () {
            // summary:
            //      Return clipboard content type
            // tags:
            //      protected

            return this._getClipboardData().type;
        },

        // =======================================================================
        // Private functions

        _getClipboardData: function () {
            // summary:
            //      Return clipboard data
            // tags:
            //      private

            if (!this.clipboard || !(this.clipboard.data instanceof Array)) {
                return null;
            }

            return this.clipboard.data;
        }

    });

});

},
'epi/shell/command/_SelectionCommandMixin':function(){
﻿define("epi/shell/command/_SelectionCommandMixin", [
// dojo
    "dojo/_base/declare",
    "dojo/_base/lang"
],

function (
// dojo
    declare,
    lang
) {

    return declare(null, {
        // summary:
        //      Stub to handle selection content for an instance of "epi/shell/command/_Command" object
        // tags:
        //      internal xproduct mixin

        // selection: [public] Object
        //      Hold information about selected content
        selection: null,

        // =======================================================================
        // Public overridden functions

        postscript: function () {
            // summary:
            //      Register a watch handler when the command change its model
            //      Watch model and initialize model dependent properties.
            // tags:
            //      public

            this.inherited(arguments);

            this.watchSelectionChange();
        },

        // =======================================================================
        // Public functions

        onRefreshSelection: function () {
            // summary:
            //      Stub to do somethings when selection refreshed
            // tags:
            //      public callback
        },

        watchSelectionChange: function () {
            // summary:
            //      Register a watch handler when the command change its model
            // tags:
            //      public

            this._selectionChangeWatcher = this.selection.watch("data", lang.hitch(this, this._onModelChange));
        },

        unwatchSelectionChange: function () {
            // summary:
            //      Clear the registered watch handler for command model change
            // tags:
            //      public

            this._selectionChangeWatcher && this._selectionChangeWatcher.unwatch();
        },

        // =======================================================================
        // Protected functions

        _getSingleSelectionData: function () {
            // summary:
            //      Returns the selected item if only one is selected otherwise null;
            // tags:
            //      protected

            var target = this._getSelectionData();

            return target && (this.selection.data.length === 1) ? target.data : null;
        },

        _selectionDataGetter: function () {
            // summary:
            //      Return selection content item data
            // tags:
            //      protected

            var selectionData = this._getSelectionData();
            return selectionData && selectionData.type === "epi.cms.contentdata" ? selectionData.data : null;
        },

        _selectionTypeGetter: function () {
            // summary:
            //      Return selection type
            // tags:
            //      protected

            return this._getSelectionData().type;
        },

        // =======================================================================
        // Private functions

        _getSelectionData: function () {
            // summary:
            //      Return the selection data for the first item selected
            // tags:
            //      private

            if (!this.selection || !(this.selection.data instanceof Array)) {
                return null;
            }

            return this.selection.data[0];
        }
    });

});

},
'epi/shell/widget/_ActionProviderWidget':function(){
﻿define("epi/shell/widget/_ActionProviderWidget", [
    "epi",
    "dojo",
    "dijit",
    "epi/shell/widget/_ActionConsumer",
    "epi/shell/widget/_ActionProvider"
],

function (epi, dojo, dijit, _ActionConsumer, _ActionProvider) {

    return dojo.declare([_ActionProvider], {
        // summary:
        //    Mixin for widgets that can provide the list of action and handlers to execute.
        //    Registers this provider widget in the closest enclosing action consumer widget on startup.
        //
        // tags:
        //    deprecated

        // _actions: [private] Object
        //    The enclosing action conumer widget for this provider.
        _enclosingConsumer: null,

        startup: function () {
            // summary:
            //    Widget startup method, executes self registration of this provider.
            //
            // tags:
            //      public
            this.inherited(arguments);
            this._registerSelf();
        },

        destroy: function () {
            // summary:
            //    Tear-down method.
            //
            // tags:
            //    public
            this._unregisterSelf();
            this.inherited(arguments);
        },

        _registerSelf: function () {
            // summary:
            //    Registers this provider in closest found action consumer.
            //
            // tags:
            //    private
            this._enclosingConsumer = this._getEnclosingActionConsumer();
            if (this._enclosingConsumer !== null) {
                this._enclosingConsumer.addProvider(this);
            }
        },

        _unregisterSelf: function () {
            // summary:
            //    Unregisters this provider in closest found action consumer provider list.
            //
            // tags:
            //    private
            if (this._enclosingConsumer !== null) {
                this._enclosingConsumer.removeProvider(this);
            }
        },

        _getEnclosingActionConsumer: function () {
            // summary:
            //    Gets the closest parent widget that is action aonsumer. Returns null if enclosing action consumer widget is not found.
            //
            // tags:
            //    private
            var node = this.domNode.parentNode;
            while (node !== null) {
                var parentWidget = dijit.getEnclosingWidget(node);

                if (!parentWidget) {
                    return null;
                }

                if (parentWidget.isInstanceOf(_ActionConsumer)) {
                    return parentWidget;
                }
                node = parentWidget.domNode.parentNode;
            }
            return null;
        }

    });
});

},
'epi/shell/widget/_ActionProvider':function(){
﻿define("epi/shell/widget/_ActionProvider", [
    "dojo",
    "dojo/_base/array",
    "dojo/_base/kernel",
    "epi/shell/widget/_ActionProviderBase"],

function (dojo, array, kernel, _ActionProviderBase) {

    return dojo.declare([_ActionProviderBase], {
        // summary:
        //    Mixin for components that can provide the list of action and handlers to execute
        //
        // tags:
        //    deprecated

        // _actions: [private] Array
        //    The list of actions provided by this instance.
        _actions: null,

        constructor: function () {
            kernel.deprecated("epi/shell/widget/_ActionProvider", "use the command pattern with epi/shell/command/_CommandProviderMixin instead");

            this._actions = [];
        },

        getActions: function () {
            // summary:
            //    Provides the list of actions.
            //
            // tags:
            //      public

            // Hand out a copy of the array instead of a reference
            return this._actions.concat();
        },

        hasAction: function (/*String*/actionName) {
            // summary:
            //      Check whether an action with the specified name exists in this provider.
            // actionName:
            //      Name of the action to check for
            // returns:
            //      true if an action with the specified name exists; otherwise false.

            return array.some(this._actions, function (action) {
                return action.name === actionName;
            });
        },

        setActionProperty: function (/*Object|String*/ actionOrName, /*String*/ property, /*Object*/ value) {
            // summary:
            //      Changes the property on a registered action
            //
            // actionOrName:
            //      The action object or the name of the action to change
            //
            // property:
            //      name of the property to alter.
            //
            // value:
            //      The new value of the action property
            //
            // tags:
            //      public

            var actionName = typeof actionOrName == "string" ? actionOrName : actionOrName.name;

            for (var i = 0; i < this._actions.length; i++) {
                var action = this._actions[i];
                if (action.name === actionName) {
                    action[property] = value;
                    this.onActionPropertyChanged(action, property, value);
                    return;
                }
            }
            throw new Error("No action registered with the name: " + actionName);
        },

        addActions: function (/* Object|Array */actions) {
            // summary:
            //    Adds an action or an array of actions to the action list.
            //
            // actions:
            //      A toolbar action definition, or an array of toolbar action definitions to add in the list.
            //
            //      Example of adding two actions:
            //      | toolbar.addActions([
            //      |     {
            //      |         name: "myAction1",
            //      |         label: "Do my action",
            //      |         action: console.log
            //      |     },
            //      |     {
            //      |         name: "myAction2",
            //      |         label: "Say hello",
            //      |         action: alert("Hello!")
            //      |     }]);
            //
            // tags:
            //      public

            if (dojo.isArray(actions)) {
                dojo.forEach(actions, this._addAction, this);
            } else {
                this._addAction(actions);
            }
        },

        removeActions: function (/* Object|Array */actions) {
            // summary:
            //      Remove a single action or an array of actions
            //
            // actions:
            //      The action(s) to remove
            //
            // tags: public

            if (dojo.isArray(actions)) {
                dojo.forEach(actions, this._removeAction, this);
            } else {
                this._removeAction(actions);
            }
        },

        _removeAction: function (action) {
            // summary:
            //      Removes a single action and raises the onActionRemoved event
            // tags: private
            for (var i = 0; i < this._actions.length; i++) {
                if (this._actions[i].name === action.name) {
                    this.onActionRemoved(this._actions.splice(i, 1)[0]);
                }
            }
        },

        _addAction: function (action) {
            // summary:
            //      Adds a single action to the action  collection and calls the onActionAdded event
            // tags:
            //      private
            if (dojo.some(this._actions, function (existingAction) {
                return existingAction.name === action.name;
            })) {
                throw new Error("An action named '" + action.name + "' is already registered");
            }

            this._actions.push(action);
            this.onActionAdded(action);
        }
    });
});

},
'epi-cms/widget/_SelectorBase':function(){
require({cache:{
'url:epi-cms/widget/templates/_SelectorBase.html':"﻿<div data-dojo-attach-point=\"inputContainer\" class=\"dijitReset dijitInputField dijitInputContainer dijitInline epi-resourceInputContainer\" id=\"widget_${id}\">\n    <div data-dojo-attach-point=\"displayNode, dropAreaNode, stateNode\" class=\"dijit dijitReset dijitInline dijitInlineTable dijitLeft dijitTextBox displayNode\">\n        <div class=\"dijitReset dijitLeft dijitInputField dijitInputContainer\">\n            <div data-dojo-attach-point=\"clearButton\" class=\"epi-clearButton\">&nbsp;</div>\n            <div data-dojo-attach-point=\"resourceName\" class=\"dijitInline epi-resourceName dojoxEllipsis\" >\n                <span data-dojo-attach-point=\"selectedContentNameNode\"></span> <span data-dojo-attach-point=\"selectedContentLinkNode\"></span>\n            </div>\n        </div>\n    </div>\n    <div data-dojo-type=\"dijit/form/Button\" data-dojo-attach-event=\"onClick: _onButtonClick\" data-dojo-attach-point=\"button\" data-dojo-props=\"label:'...'\"></div>\n</div>"}});
﻿define("epi-cms/widget/_SelectorBase", [
// dojo
    "dojo/_base/declare",
    "dojo/_base/lang",

    "dojo/aspect",
    "dojo/Deferred",
    "dojo/dom-class",
    "dojo/dom-style",
    "dojo/dom-prop",

    "dojo/query",
    "dojo/when",

    // dijit,
    "dijit/_CssStateMixin",
    "dijit/_TemplatedMixin",
    "dijit/_Widget",
    "dijit/_WidgetsInTemplateMixin",

    // epi.shell
    "epi/dependency",
    "epi/i18n",
    "epi/shell/widget/_ValueRequiredMixin",

    // epi.cms
    "epi-cms/widget/_Droppable",
    "epi-cms/widget/_HasClearButton",
    "epi-cms/widget/_HasChildDialogMixin",

    // resources
    "dojo/text!./templates/_SelectorBase.html"
],

function (
// dojo
    declare,
    lang,

    aspect,
    Deferred,
    domClass,
    domStyle,
    domProp,

    query,
    when,

    // dijit
    _CssStateMixin,
    _TemplatedMixin,
    _Widget,
    _WidgetsInTemplateMixin,

    // epi.shell
    dependency,
    i18n,
    _ValueRequiredMixin,

    // epi.cms
    _Droppable,
    _HasClearButton,
    _HasChildDialogMixin,

    // resources
    template
) {
    return declare([_Widget, _TemplatedMixin, _WidgetsInTemplateMixin, _CssStateMixin, _HasClearButton, _HasChildDialogMixin, _Droppable, _ValueRequiredMixin], {
        // summary:
        //    Represents base class for item selector widget (a textbox and a browse button).
        // tags:
        //    public

        templateString: template,

        roots: null,

        value: null,

        // required: Boolean
        //		User is required to enter data into this field.
        required: false,

        // missingMessage: [public] String
        //    Message which is displayed when required is true and value is empty.
        missingMessage: "",

        // tooltipPosition: String[]
        //		See description of `dijit/Tooltip.defaultPosition` for details on this parameter.
        tooltipPosition: [],

        // canSelectOwnerContent: [public] Boolean
        //      Indicates whether the owner content that is being edited can be selected
        canSelectOwnerContent: false,

        contentRepositoryDescriptors: null,

        allowedDndTypes: [],

        // _valueChangedPromise: [protected] Promise
        //      Promise which is set while the value in being changed within an asynchronous call.
        _valueChangedPromise: null,

        _setSelectedContentNameAttr: function (value) {
            // summary:
            //		Sets the display name for selected content
            // tags:
            //    protected

            // set the text
            domProp.set(this.selectedContentNameNode, "textContent", value);

            // set the title
            this._updateDisplayNodeTitle();
        },

        _setSelectedContentLinkAttr: function (value) {
            domProp.set(this.selectedContentLinkNode, "innerHtml", value);
            this._updateDisplayNodeTitle();
        },

        _updateDisplayNodeTitle: function () {
            // summary:
            //		updates the tooltip with the current text of displayNode
            // tags:
            //    protected

            this.resourceName.title = this.resourceName.textContent.trim();
        },

        postCreate: function () {
            // summary:
            //      Post create initialization.
            // description:
            //      Setup related components.
            // tags:
            //      protected

            this.inherited(arguments);

            // After dnd into widget
            this.own(aspect.after(this, "onDrop", lang.hitch(this, this.focus)));
        },

        postMixInProperties: function () {
            // summary:
            //		Initialize properties
            // tags:
            //    protected

            this.inherited(arguments);

            if (!this.contentRepositoryDescriptors) {
                this.contentRepositoryDescriptors = dependency.resolve("epi.cms.contentRepositoryDescriptors");
            }

            if (!this.roots && this.repositoryKey) {
                var settings = this.contentRepositoryDescriptors.get(this.repositoryKey);
                this.roots = settings.roots;
            }
        },

        destroy: function (/*Boolean*/ preserveDom) {
            // summary:
            //      Destroy this class, cancelling any promises that are in progress.
            // tags:
            //      public
            this.inherited(arguments);

            // If there is a value change promise in progress then cancel it.
            if (this._valueChangedPromise) {
                this._valueChangedPromise.cancel();
                this._valueChangedPromise = null;
            }
        },

        _setReadOnlyAttr: function (value) {
            this._set("readOnly", value);

            domStyle.set(this.button.domNode, "display", value ? "none" : "");
            domStyle.set(this.clearButton, "display", value ? "none" : "");
            domClass.toggle(this.domNode, "dijitReadOnly", value);
        },

        _getDialog: function () {
            // summary:
            //		Create page tree dialog. Need override
            // tags:
            //    protected
        },

        _onButtonClick: function () {
            //summary:
            //    Handle pick button click. Need override
            // tags:
            //    protected

            if (!this.dialog) {
                this.dialog = this._getDialog();
                this.dialog.startup();

                this.own(aspect.after(this.dialog, "onExecute", lang.hitch(this, this._onDialogExecute)));
                this.own(aspect.after(this.dialog, "onShow", lang.hitch(this, this._onDialogShow)));
                this.own(aspect.after(this.dialog, "onHide", lang.hitch(this, this._onDialogHide)));
                this.own(this.dialog);
            }

            this.dialog.show();
        },

        _onDialogShow: function () {
            //summary:
            //    Handle onShow dialog event.
            // tags:
            //    protected

            this.isShowingChildDialog = true;
        },

        _onDialogExecute: function () {
            //summary:
            //    Handle dialog close. Need override
            // tags:
            //    protected
        },

        _onDialogHide: function () {
            //summary:
            //    Handle dialog hide. Need override
            // tags:
            //    protected

            this.isShowingChildDialog = false;

            // Return focus to the editor when the dialog closes.
            this.focus();
        },

        _onBlur: function () {
            // summary:
            //		This is where widgets do processing for when they stop being active,
            //		such as changing CSS classes.  See onBlur() for more details.
            // tags:
            //		protected

            this.inherited(arguments);

            this.validate();
        },

        focus: function () {
            this.button.focus();
        },

        onChange: function (value) {
            // summary:
            //    Fired when value is changed.
            //
            // value:
            //    The value
            // tags:
            //    public, callback
        },

        _updateDisplayNode: function (content) {
            //summary:
            //    Update widget's display text and title.
            // tags:
            //    protected

            if (content) {
                domClass.remove(this.domNode, "epi-noValue");
                domClass.remove(this.resourceName, "dijitPlaceHolder");
                this.set("selectedContentName", content.name);

            } else {
                domClass.add(this.domNode, "epi-noValue");
                domClass.add(this.resourceName, "dijitPlaceHolder");
                this.set("selectedContentName", "");
            }
        },

        _setDialogButtonState: function (contentLink) {
            // summary:
            //    Set state of dialog button.
            //
            // tags:
            //    protected
        }
    });
});

},
'epi/shell/widget/_ValueRequiredMixin':function(){
﻿define("epi/shell/widget/_ValueRequiredMixin", [
    "dojo/_base/declare",
    "dojo/_base/lang",

    "dijit/_CssStateMixin",
    "dijit/Tooltip"],

function (declare, lang, _CssStateMixin, Tooltip) {
    // summary:
    //		Mixin to support value required widgets which don't inherit ValidationTextBox.
    //
    // tags:
    //      public

    return declare([_CssStateMixin], {

        // required: Boolean
        //		User is required to enter data into this field.
        required: null,

        // missingMessage: [public] String
        //    Message which is displayed when required is true and value is empty.
        missingMessage: "",

        destroy: function () {
            this.displayMessage(null);
            this.inherited(arguments);
        },

        _onFocus: function () {
            // summary:
            //		This is where widgets do processing for when they start being active,
            //		such as changing CSS classes.  See onFocus() for more details.
            // tags:
            //		protected

            this.inherited(arguments);

            this.validate(true);
        },

        _onBlur: function () {
            // summary:
            //		This is where widgets do processing for when they stop being active,
            //		such as changing CSS classes.  See onBlur() for more details.
            // tags:
            //		protected

            this.inherited(arguments);

            this.validate();
        },

        validate: function (/*Boolean*/ isFocused) {
            // summary:
            //    Show missing or invalid messages if appropriate, and highlight textbox field.
            // tags:
            //    protected

            var isValid = this.isValid(isFocused);

            // Behave like dijit form widgets:
            // only show error state when focused or has been previously focused. I.e. a newly loaded form should not be full of errors.
            var state = isValid ? "" : (isFocused || this._hasBeenBlurred) ? "Error" : "Incomplete";
            this.set("state", state);

            this.set("message", this.getErrorMessage(isFocused));

            this.displayMessage(isValid ? "" : this.getErrorMessage());

            return isValid;
        },

        isValid: function (/*Boolean*/ isFocused) {
            // summary:
            //    Check if widget's value is valid.
            // isFocused:
            //    Indicate that the widget is being focused.
            // tags:
            //    protected
            return (!this.required || this.value || this.value === 0 || (lang.isArray(this.value) && this.value.length === 0)); // Not required or have some value.
        },

        displayMessage: function (/*String*/message) {
            // summary:
            //    Overridable method to display validation errors/hints.
            //    By default uses a tooltip.
            // tags:
            //    extension

            var tooltipNode = this.tooltipNode || this.domNode;

            if (message && this.focused && !this.isShowingChildDialog) {
                Tooltip.show(message, tooltipNode, this.tooltipPosition, !this.isLeftToRight());
            } else {
                Tooltip.hide(tooltipNode);
            }
        },

        getErrorMessage: function (/*Boolean*/ isFocused) {
            // summary:
            //      Return an error message to show if appropriate
            // tags:
            //      public

            var isValid = this.get("state") !== "Error";
            return isValid ? "" : this.missingMessage;
        }
    });
});

},
'epi-cms/widget/_Droppable':function(){
﻿define("epi-cms/widget/_Droppable", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/when",
    "epi/shell/dnd/Target"
], function (declare, lang, when, Target) {
    return declare(null, {
        // summary:
        //    Mixin for widgets that are droppable.
        // tags:
        //    public

        // dropAreaNode: [protected] DomNode
        //      The dom node where user can drop
        dropAreaNode: null,

        // allowedDndTypes: [protected] Array
        //      List of data types that this widget can handle
        allowedDndTypes: null,

        postCreate: function () {
            // summary:
            //		Initialize child widgets
            // tags:
            //    protected

            this.inherited(arguments);

            //create drop target
            var dropTarget = this.own(new Target(this.dropAreaNode, {
                accept: this.allowedDndTypes,
                reject: this.restrictedDndTypes,
                createItemOnDrop: false,
                readOnly: this.readOnly,
                skipForm: true
            }))[0];

            this.connect(dropTarget, "onDropData", "_onDropData");
        },

        onDropping: function () {
            // summary:
            //    Triggered when something is dropping onto the widget.
            //
            // tags:
            //    public callback
        },

        onDrop: function () {
            // summary:
            //    Triggered when something has been dropped onto the widget.
            //
            // tags:
            //    public callback
        },

        _onDropData: function (dndData, source, nodes, copy) {
            //summary:
            //    Handle drop data event.
            //
            // dndData:
            //    Dnd data extracted from the dragging items which have the same data type to the current target
            //
            // source:
            //    The dnd source.
            //
            // nodes:
            //    The dragging nodes.
            //
            // copy:
            //    Denote that the drag is copy.
            //
            // tags:
            //    private

            var dropItem = dndData ? (dndData.length ? dndData[0] : dndData) : null;

            if (dropItem) {
                this.onDropping();

                when(dropItem.data, lang.hitch(this, function (resolvedValue) {
                    var value;
                    if (this.dndSourcePropertyName) {
                        value = resolvedValue[this.dndSourcePropertyName];
                    } else {
                        value = resolvedValue;
                    }
                    this.set("value", value);
                    this.onDrop();
                }));
            }
        }
    });
});

},
'epi/shell/dnd/Target':function(){
﻿define("epi/shell/dnd/Target", [
    "dojo/_base/declare",
    "dojo/dom-class",
    "./Source"
], function (
    declare,
    domClass,
    Source
) {
    return declare([Source], {
        // tags:
        //      internal

        // allowMultipleItems: [public] Boolean
        //      Indicates if the editor allows multiple items
        allowMultipleItems: false,

        checkAcceptance: function (source, nodes) {
            // summary:
            //		Checks if the target can accept nodes from this source.
            // source: Object
            //		The source which provides items.
            // nodes: Array
            //		The list of transferred items.

            if (!this.allowMultipleItems && nodes.length > 1) {
                return false;
            } else {
                return this.inherited(arguments);
            }
        },

        constructor: function (node, params) {
            // summary:
            //		a constructor of the Target --- see the `dojo/dnd/Source.constructor` for details
            this.isSource = false;
            domClass.remove(this.node, "dojoDndSource");
        }
    });

});

},
'epi-cms/widget/_HasClearButton':function(){
﻿define("epi-cms/widget/_HasClearButton", [
    "epi",
    "dojo",
    "dijit"
], function (epi, dojo, dijit) {
    return dojo.declare(null, {
        // summary:
        //    Mixin for widgets that need child dialog ability.
        // tags:
        //    internal

        postCreate: function () {
            // summary:
            //		Initialize child widgets
            // tags:
            //    protected

            this.inherited(arguments);

            var eventName = this.clearButton.onClick ? "onClick" : "onclick";

            dojo.connect(this.clearButton, eventName, dojo.hitch(this, this.clearValue));
        },

        clearValue: function () {
            // summary:
            //		Do clear value
            // tags:
            //    protected
            if (this.readOnly) {
                return;
            }
            var value = this.getEmptyValue();
            this.set("value", value);
        },

        getEmptyValue: function () {
            // summary:
            //		Gets widget's empty value.
            // description:
            //      Returns null by default. Override this method to change widget's empty value.
            // tags:
            //    protected

            return null;
        }
    });
});

},
'epi-cms/widget/_HasChildDialogMixin':function(){
﻿define("epi-cms/widget/_HasChildDialogMixin", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/aspect",
    "dojo/dom",
    "dijit/_FocusMixin",
    "dijit/popup"
], function (
    declare,
    lang,
    aspect,
    dom,
    _FocusMixin,
    popup
) {

    return declare([_FocusMixin], {
        // summary:
        //    Base class for all EPiServer CMS property editor controls.
        // tags:
        //    internal

        // isShowingChildDialog: [public] Boolean
        //    Indicates whether a child dialog window is currently being displayed.
        isShowingChildDialog: false,

        // _openedChildDialog: [private] Widget
        //      A reference to the currently open child dialog
        _openedChildDialog: null,

        // _waitingForFocus: [private] Boolean
        //      A flag indicating that the child dialog has been closed and we're waiting for a focus on this widget
        _waitingForFocus: false,

        startup: function () {
            if (!this._started) {
                this.own(
                    aspect.before(popup, "open", lang.hitch(this, "_onPopupOpen"), true),
                    aspect.after(popup, "close", lang.hitch(this, "_onPopupClosed"), true)
                );
            }

            this.inherited(arguments);
        },

        _onPopupOpen: function (openArgs) {
            // summary:
            //      Called before a popup is opened.
            //      Checks if the popup is opened for a descendant, and if so it's assumed to be a child dialog.
            var popup = openArgs.popup;

            // If the popup is a context menu with a parent node set then the dijit focus handling
            // will keep this widget in the stack and not blur the owner widget.
            // The WithContextMenu mixin sets popupParent for grid context menus.
            if (popup && popup.popupParent) {
                return;
            }

            // currentTarget is the node clicked to open a pop-up menu
            var target = popup && popup.currentTarget;

            // if the popup is a opened for a descendant node, then it's opened for this widget
            // and we're entering "prevent blur" mode.
            if (target && dom.isDescendant(target, this.domNode)) {
                this._openedChildDialog = popup;
                this.isShowingChildDialog = true;
            }
        },

        _onPopupClosed: function (popup) {
            // summary:
            //      Called when a popup dialog has been closed

            if (this._openedChildDialog === popup) {
                this._openedChildDialog = null;
                this.isShowingChildDialog = false;

                // If the popup is closed by a click on this widget we get a focus call,
                // but when focus is lost to a widget which isn't a child of ours we need to explicitly
                // call _onBlur since we've prevented the original blur call when the pop-up was opened.
                // If we're still waiting for a focus call when _waitForFocus executes we've lost focus
                // to an element which is not a descendant.
                this._waitingForFocus = true;
                setTimeout(lang.hitch(this, "_waitForFocus"), 0);
            }
        },

        _waitForFocus: function () {
            // Pop-up was closed and we should have received focus by now if we are the new focus target.

            if (this._waitingForFocus) {
                this._waitingForFocus = false;
                this._onBlur();
            }
        },

        _onBlur: function () {
            // summary:
            //		This is where widgets do processing for when they stop being active,
            //		such as changing CSS classes.  See onBlur() for more details.
            // tags:
            //		protected

            if (this.isShowingChildDialog) {
                return;
            }

            this.inherited(arguments);
        },

        _onFocus: function () {
            // summary:
            //		This is where widgets do processing for when they are active,
            //		such as changing CSS classes.  See onFocus() for more details.
            // tags:
            //		protected

            this._waitingForFocus = false;

            if (this.isShowingChildDialog) {
                return;
            }

            this.inherited(arguments);
        }
    });
});

},
'url:epi-cms/widget/templates/_SelectorBase.html':"﻿<div data-dojo-attach-point=\"inputContainer\" class=\"dijitReset dijitInputField dijitInputContainer dijitInline epi-resourceInputContainer\" id=\"widget_${id}\">\n    <div data-dojo-attach-point=\"displayNode, dropAreaNode, stateNode\" class=\"dijit dijitReset dijitInline dijitInlineTable dijitLeft dijitTextBox displayNode\">\n        <div class=\"dijitReset dijitLeft dijitInputField dijitInputContainer\">\n            <div data-dojo-attach-point=\"clearButton\" class=\"epi-clearButton\">&nbsp;</div>\n            <div data-dojo-attach-point=\"resourceName\" class=\"dijitInline epi-resourceName dojoxEllipsis\" >\n                <span data-dojo-attach-point=\"selectedContentNameNode\"></span> <span data-dojo-attach-point=\"selectedContentLinkNode\"></span>\n            </div>\n        </div>\n    </div>\n    <div data-dojo-type=\"dijit/form/Button\" data-dojo-attach-event=\"onClick: _onButtonClick\" data-dojo-attach-point=\"button\" data-dojo-props=\"label:'...'\"></div>\n</div>",
'epi-find/widget/ContentStoreQueryLanguageHandler':function(){
define([
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/promise/all",
    "dojo/when",
    "epi-cms/widget/ContentForestStoreModel"

], function (
    array,
    declare,
    lang,
    promiseAll,
    when,
    ContentForestStoreModel
) {

  return declare([ContentForestStoreModel], {

    _createQuery: function (queryBase, excludeType) {
        var query = this.inherited(arguments);

        if (this.lang) {
            delete query.allLanguages;
            query.lang = this.lang;
        }
        else {
            query.allLanguages = true;
        }
        return query;
    }
  });
});

},
'epi-find/widget/_ContentStatusIndicatorMixin':function(){
define([
        "dojo/_base/declare",
        "dojo/_base/lang",
        "dojo/dom-attr",
        "dojo/dom-class",
        "dojo/dom-construct",

        "epi-cms/widget/viewmodel/ContentStatusViewModel"

    ],
    function(
        declare,
        lang,
        domAttr,
        domClass,
        domConstruct,
        ContentStatusViewModel

    ) {
        //  summary:
        //      Mixin for ContentSelectors that adds selected content status indicaiton
        // module:
        //      epi-find/widget/_ContentStatusIndicatorMixin

        return declare([], {

            showStatusIcon: false,
            statusIconNode: null,
            statusViewModel: null,

            startup: function() {
                this.inherited(arguments);
                this.watch("contentStatus", lang.hitch(this, function(name, oldValue, value) {
                    if (this.showStatusIcon) {
                        if (value) {
                            this.statusViewModel.set("contentStatus", value);
                        }
                        else {
                            this._resetStatusIcon();
                        }
                    }
                }));
            },

            buildRendering: function() {
                this.inherited(arguments);
                if (this.showStatusIcon) {
                    this.statusIconNode = domConstruct.create("span");
                    domConstruct.place(this.statusIconNode, this.inputContainer, "first");
                    this._resetStatusIcon();
                    this.statusViewModel = new ContentStatusViewModel({autoLoadStatus: false, getCurrentContext: function() {return null;}});

                    this.own(this.statusViewModel.watch("statusIcon", lang.hitch(this, function(name, oldValue, newValue){
                        domClass.remove(this.statusIconNode);
                        domClass.add(this.statusIconNode, newValue);
                    })));
                    this.own(this.statusViewModel.watch("statusMessage", lang.hitch(this, function(name, oldValue, newValue){
                        domAttr.set(this.statusIconNode, "title",  newValue || "");
                    })));
                }
            },

            _resetStatusIcon: function() {
                domClass.remove(this.statusIconNode);
                domClass.add(this.statusIconNode, ["epi-statusIndicatorIcon", "epi-no-background"]);
            }
        });
    });
},
'epi-cms/widget/viewmodel/ContentStatusViewModel':function(){
define("epi-cms/widget/viewmodel/ContentStatusViewModel", [
// dojo
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/Deferred",
    "dojo/_base/lang",
    "dojo/when",
    "dojo/promise/all",
    "dojo/Stateful",

    // epi
    "epi/dependency",
    "epi/shell/_StatefulGetterSetterMixin",
    "epi-cms/contentediting/ContentActionSupport",
    "epi/shell/_ContextMixin",
    "epi-cms/ContentLanguageHelper",

    // resources
    "dojo/i18n!epi/nls/episerver.cms.contentediting.versionstatus"
],
function (
// dojo
    array,
    declare,
    Deferred,
    lang,
    when,
    all,
    Stateful,

    // epi
    dependency,
    _StatefulGetterSetterMixin,
    ContentActionSupport,
    _ContextMixin,
    ContentLanguageHelper,

    // resources
    versionStatusResources
) {

    return declare([Stateful, _ContextMixin], {
        // tags:
        //      internal

        _contentStructureStore: null,

        // autoLoadStatus: [Boolean]
        //      Set it to true of the status information should be automatically loaded when the contentLink is set
        // tags:
        //      public
        autoLoadStatus: false,

        // statusIcon: [String]
        //      Store content status icon classes
        // tags:
        //      public
        statusIcon: null,

        // statusMessage: [Object]
        //      Store content status message.
        //      Can be [String] or [Array] data type.
        // tags:
        //      public
        statusMessage: null,

        // contentLink: [String]
        //      The content link.
        // tags:
        //      public
        contentLink: null,

        constructor: function () {
            this._statusIconMap = {
                // Not created icon CSS classes
                0: ["epi-statusIndicatorIcon", "epi-statusIndicator0"],
                // Rejected icon CSS classes
                1: ["epi-statusIndicatorIcon", "epi-statusIndicator1"],
                // Checked-out icon CSS classes
                2: ["epi-statusIndicatorIcon", "epi-statusIndicator2"],
                // Checked-in icon CSS classes
                3: ["epi-statusIndicatorIcon", "epi-statusIndicator3"],
                // Published icon CSS classes
                4: ["epi-statusIndicatorIcon", "epi-statusIndicator4"],
                // Previously published icon CSS classes
                5: ["epi-statusIndicatorIcon", "epi-statusIndicator5"],
                // Delayed publish icon CSS classes
                6: ["epi-statusIndicatorIcon", "epi-statusIndicator6"],
                // Awaiting approval icon CSS classes
                7: ["epi-statusIndicatorIcon", "epi-statusIndicator7"],
                // Expired icon CSS classes
                100: ["epi-statusIndicatorIcon", "epi-statusIndicator100"]
            };

            this._statusMessageMap = {
                // Not created message
                0: versionStatusResources.notcreated,
                // Rejected message
                1: versionStatusResources.rejected,
                // Checked-out message
                2: versionStatusResources.checkedout,
                // Checked-in message
                3: versionStatusResources.checkedin,
                // Published message
                4: versionStatusResources.published,
                // Previously published message
                5: versionStatusResources.previouslypublished,
                // Delayed publish message
                6: versionStatusResources.delayedpublish,
                // Awaiting approval
                7: versionStatusResources.awaitingapproval,
                // Expired message
                100: versionStatusResources.expired
            };
        },

        postscript: function () {
            this.inherited(arguments);

            this._contentStructureStore = this._contentStructureStore || dependency.resolve("epi.storeregistry").get("epi.cms.content.light");
        },

        loadStatus: function () {
            // summary:
            //      Loads the status for the current contentLink
            // tags:
            //      public
            if (!this.contentLink) {
                return;
            }
            when(this.getCurrentContext(), lang.hitch(this, function (ctx) {
                //- Using refresh instead get function to load content link
                //- Do not want use cache in this case
                when(this._contentStructureStore.refresh(this.contentLink), lang.hitch(this, function (contentInfo) {
                    if (contentInfo && contentInfo.status !== undefined) {
                        this.set("content", contentInfo);
                    }
                }));
            }));
        },

        _contentLinkSetter: function (/* String */contentLink) {
            // summary:
            //      Content Link setter
            // contentLink: [String]

            if (!contentLink) {
                return;
            }

            this.contentLink = contentLink;

            if (this.autoLoadStatus) {
                this.loadStatus();
            }
        },

        _isDeletedGetter: function () {
            return this.get("content") && this.get("content").isDeleted;
        },

        _isTranslationNeededGetter: function () {
            return this.get("content") && this.get("content").missingLanguageBranch && this.get("content").missingLanguageBranch.isTranslationNeeded;
        },

        _missingLanguageBranchGetter: function () {
            return this.get("content") && this.get("content").missingLanguageBranch;
        },

        _contentSetter: function (content) {
            this.content = content;

            this.set("contentStatus", content.status);
        },

        _contentStatusSetter: function (status) {
            this.contentStatus = status;

            if (!status) {
                return;
            }

            this.set("statusIcon", this._statusIconMap[status]);

            when(this._getStatusMessage(), lang.hitch(this, function (message) {
                this.set("statusMessage", message);
            }));
        },

        _getStatusMessage: function () {
            // summary:
            //      Get content version status message.
            //      Override this function to customize content status message.
            // tags:
            //      protected

            //If the content is deleted return version not found
            if (this.get("isDeleted")) {
                return versionStatusResources.versionnotfound;
            }
            if (this.get("isTranslationNeeded")) {
                return ContentLanguageHelper.getMissingLanguageMessage(this.content);
            }

            if (this.get("isVisibleOnSite")) {
                return null;
            }
            return this._statusMessageMap[this.get("contentStatus")];
        },

        _isVisibleOnSiteGetter: function () {
            // summary:
            //      Allow create additional information area or not.
            //      Based on content version status.
            // status: [Integer]
            //      Content status
            // tags:
            //      public

            var missingLanguageBranch = this.get("missingLanguageBranch");
            if (this.get("isDeleted") || missingLanguageBranch && missingLanguageBranch.reason === 6) {
                return false;
            }

            return this.get("contentStatus") === ContentActionSupport.versionStatus.Published;
        }
    });
});

},
'dojo/NodeList-manipulate':function(){
define("dojo/NodeList-manipulate", ["./query", "./_base/lang", "./_base/array", "./dom-construct", "./NodeList-dom"], function(dquery, lang, array, construct){
	// module:
	//		dojo/NodeList-manipulate

	/*=====
	return function(){
		// summary:
		//		Adds chainable methods to dojo.query() / NodeList instances for manipulating HTML
		//		and DOM nodes and their properties.
	};
	=====*/

	var NodeList = dquery.NodeList;

	//TODO: add a way to parse for widgets in the injected markup?

	function getText(/*DOMNode*/node){
		// summary:
		//		recursion method for text() to use. Gets text value for a node.
		// description:
		//		Juse uses nodedValue so things like <br/> tags do not end up in
		//		the text as any sort of line return.
		var text = "", ch = node.childNodes;
		for(var i = 0, n; n = ch[i]; i++){
			//Skip comments.
			if(n.nodeType != 8){
				if(n.nodeType == 1){
					text += getText(n);
				}else{
					text += n.nodeValue;
				}
			}
		}
		return text;
	}

	function getWrapInsertion(/*DOMNode*/node){
		// summary:
		//		finds the innermost element to use for wrap insertion.

		//Make it easy, assume single nesting, no siblings.
		while(node.childNodes[0] && node.childNodes[0].nodeType == 1){
			node = node.childNodes[0];
		}
		return node; //DOMNode
	}

	function makeWrapNode(/*DOMNode||String*/html, /*DOMNode*/refNode){
		// summary:
		//		convert HTML into nodes if it is not already a node.
		if(typeof html == "string"){
			html = construct.toDom(html, (refNode && refNode.ownerDocument));
			if(html.nodeType == 11){
				//DocumentFragment cannot handle cloneNode, so choose first child.
				html = html.childNodes[0];
			}
		}else if(html.nodeType == 1 && html.parentNode){
			//This element is already in the DOM clone it, but not its children.
			html = html.cloneNode(false);
		}
		return html; /*DOMNode*/
	}

	lang.extend(NodeList, {
		_placeMultiple: function(/*String||Node||NodeList*/query, /*String*/position){
			// summary:
			//		private method for inserting queried nodes into all nodes in this NodeList
			//		at different positions. Differs from NodeList.place because it will clone
			//		the nodes in this NodeList if the query matches more than one element.
			var nl2 = typeof query == "string" || query.nodeType ? dquery(query) : query;
			var toAdd = [];
			for(var i = 0; i < nl2.length; i++){
				//Go backwards in DOM to make dom insertions easier via insertBefore
				var refNode = nl2[i];
				var length = this.length;
				for(var j = length - 1, item; item = this[j]; j--){
					if(i > 0){
						//Need to clone the item. This also means
						//it needs to be added to the current NodeList
						//so it can also be the target of other chaining operations.
						item = this._cloneNode(item);
						toAdd.unshift(item);
					}
					if(j == length - 1){
						construct.place(item, refNode, position);
					}else{
						refNode.parentNode.insertBefore(item, refNode);
					}
					refNode = item;
				}
			}

			if(toAdd.length){
				//Add the toAdd items to the current NodeList. Build up list of args
				//to pass to splice.
				toAdd.unshift(0);
				toAdd.unshift(this.length - 1);
				Array.prototype.splice.apply(this, toAdd);
			}

			return this; // dojo/NodeList
		},

		innerHTML: function(/*String|DOMNode|NodeList?*/ value){
			// summary:
			//		allows setting the innerHTML of each node in the NodeList,
			//		if there is a value passed in, otherwise, reads the innerHTML value of the first node.
			// description:
			//		This method is simpler than the dojo/NodeList.html() method provided by
			//		`dojo/NodeList-html`. This method just does proper innerHTML insertion of HTML fragments,
			//		and it allows for the innerHTML to be read for the first node in the node list.
			//		Since dojo/NodeList-html already took the "html" name, this method is called
			//		"innerHTML". However, if dojo/NodeList-html has not been loaded yet, this
			//		module will define an "html" method that can be used instead. Be careful if you
			//		are working in an environment where it is possible that dojo/NodeList-html could
			//		have been loaded, since its definition of "html" will take precedence.
			//		The nodes represented by the value argument will be cloned if more than one
			//		node is in this NodeList. The nodes in this NodeList are returned in the "set"
			//		usage of this method, not the HTML that was inserted.
			// returns:
			//		if no value is passed, the result is String, the innerHTML of the first node.
			//		If a value is passed, the return is this dojo/NodeList
			// example:
			//		assume a DOM created by this markup:
			//	|	<div id="foo"></div>
			//	|	<div id="bar"></div>
			//		This code inserts `<p>Hello World</p>` into both divs:
			//	|	dojo.query("div").innerHTML("<p>Hello World</p>");
			// example:
			//		assume a DOM created by this markup:
			//	|	<div id="foo"><p>Hello Mars</p></div>
			//	|	<div id="bar"><p>Hello World</p></div>
			//		This code returns `<p>Hello Mars</p>`:
			//	|	var message = dojo.query("div").innerHTML();
			if(arguments.length){
				return this.addContent(value, "only"); // dojo/NodeList
			}else{
				return this[0].innerHTML; //String
			}
		},

		/*=====
		html: function(value){
			// summary:
			//		see the information for "innerHTML". "html" is an alias for "innerHTML", but is
			//		only defined if dojo/NodeList-html has not been loaded.
			// description:
			//		An alias for the "innerHTML" method, but only defined if there is not an existing
			//		"html" method on dojo/NodeList. Be careful if you are working in an environment
			//		where it is possible that dojo/NodeList-html could have been loaded, since its
			//		definition of "html" will take precedence. If you are not sure if dojo/NodeList-html
			//		could be loaded, use the "innerHTML" method.
			// value: String|DOMNode|NodeList?
			//		The HTML fragment to use as innerHTML. If value is not passed, then the innerHTML
			//		of the first element in this NodeList is returned.
			// returns:
			//		if no value is passed, the result is String, the innerHTML of the first node.
			//		If a value is passed, the return is this dojo/NodeList
			return; // dojo/NodeList|String
		},
		=====*/

		text: function(/*String*/value){
			// summary:
			//		allows setting the text value of each node in the NodeList,
			//		if there is a value passed in, otherwise, returns the text value for all the
			//		nodes in the NodeList in one string.
			// example:
			//		assume a DOM created by this markup:
			//	|	<div id="foo"></div>
			//	|	<div id="bar"></div>
			//		This code inserts "Hello World" into both divs:
			//	|	dojo.query("div").text("Hello World");
			// example:
			//		assume a DOM created by this markup:
			//	|	<div id="foo"><p>Hello Mars <span>today</span></p></div>
			//	|	<div id="bar"><p>Hello World</p></div>
			//		This code returns "Hello Mars today":
			//	|	var message = dojo.query("div").text();
			// returns:
			//		if no value is passed, the result is String, the text value of the first node.
			//		If a value is passed, the return is this dojo/NodeList
			if(arguments.length){
				for(var i = 0, node; node = this[i]; i++){
					if(node.nodeType == 1){
						construct.empty(node);
						node.appendChild(node.ownerDocument.createTextNode(value));
					}
				}
				return this; // dojo/NodeList
			}else{
				var result = "";
				for(i = 0; node = this[i]; i++){
					result += getText(node);
				}
				return result; //String
			}
		},

		val: function(/*String||Array*/value){
			// summary:
			//		If a value is passed, allows seting the value property of form elements in this
			//		NodeList, or properly selecting/checking the right value for radio/checkbox/select
			//		elements. If no value is passed, the value of the first node in this NodeList
			//		is returned.
			// returns:
			//		if no value is passed, the result is String or an Array, for the value of the
			//		first node.
			//		If a value is passed, the return is this dojo/NodeList
			// example:
			//		assume a DOM created by this markup:
			//	|	<input type="text" value="foo">
			//	|	<select multiple>
			//	|		<option value="red" selected>Red</option>
			//	|		<option value="blue">Blue</option>
			//	|		<option value="yellow" selected>Yellow</option>
			//	|	</select>
			//		This code gets and sets the values for the form fields above:
			//	|	dojo.query('[type="text"]').val(); //gets value foo
			//	|	dojo.query('[type="text"]').val("bar"); //sets the input's value to "bar"
			// 	|	dojo.query("select").val() //gets array value ["red", "yellow"]
			// 	|	dojo.query("select").val(["blue", "yellow"]) //Sets the blue and yellow options to selected.

			//Special work for input elements.
			if(arguments.length){
				var isArray = lang.isArray(value);
				for(var index = 0, node; node = this[index]; index++){
					var name = node.nodeName.toUpperCase();
					var type = node.type;
					var newValue = isArray ? value[index] : value;

					if(name == "SELECT"){
						var opts = node.options;
						for(var i = 0; i < opts.length; i++){
							var opt = opts[i];
							if(node.multiple){
								opt.selected = (array.indexOf(value, opt.value) != -1);
							}else{
								opt.selected = (opt.value == newValue);
							}
						}
					}else if(type == "checkbox" || type == "radio"){
						node.checked = (node.value == newValue);
					}else{
						node.value = newValue;
					}
				}
				return this; // dojo/NodeList
			}else{
				//node already declared above.
				node = this[0];
				if(!node || node.nodeType != 1){
					return undefined;
				}
				value = node.value || "";
				if(node.nodeName.toUpperCase() == "SELECT" && node.multiple){
					//A multivalued selectbox. Do the pain.
					value = [];
					//opts declared above in if block.
					opts = node.options;
					//i declared above in if block;
					for(i = 0; i < opts.length; i++){
						//opt declared above in if block
						opt = opts[i];
						if(opt.selected){
							value.push(opt.value);
						}
					}
					if(!value.length){
						value = null;
					}
				}
				return value; //String||Array
			}
		},

		append: function(/*String||DOMNode||NodeList*/content){
			// summary:
			//		appends the content to every node in the NodeList.
			// description:
			//		The content will be cloned if the length of NodeList
			//		is greater than 1. Only the DOM nodes are cloned, not
			//		any attached event handlers.
			// returns:
			//		dojo/NodeList, the nodes currently in this NodeList will be returned,
			//		not the appended content.
			// example:
			//		assume a DOM created by this markup:
			//	|	<div id="foo"><p>Hello Mars</p></div>
			//	|	<div id="bar"><p>Hello World</p></div>
			//		Running this code:
			//	|	dojo.query("div").append("<span>append</span>");
			//		Results in this DOM structure:
			//	|	<div id="foo"><p>Hello Mars</p><span>append</span></div>
			//	|	<div id="bar"><p>Hello World</p><span>append</span></div>
			return this.addContent(content, "last"); // dojo/NodeList
		},

		appendTo: function(/*String*/query){
			// summary:
			//		appends nodes in this NodeList to the nodes matched by
			//		the query passed to appendTo.
			// description:
			//		The nodes in this NodeList will be cloned if the query
			//		matches more than one element. Only the DOM nodes are cloned, not
			//		any attached event handlers.
			// returns:
			//		dojo/NodeList, the nodes currently in this NodeList will be returned,
			//		not the matched nodes from the query.
			// example:
			//		assume a DOM created by this markup:
			//	|	<span>append</span>
			//	|	<p>Hello Mars</p>
			//	|	<p>Hello World</p>
			//		Running this code:
			//	|	dojo.query("span").appendTo("p");
			//		Results in this DOM structure:
			//	|	<p>Hello Mars<span>append</span></p>
			//	|	<p>Hello World<span>append</span></p>
			return this._placeMultiple(query, "last"); // dojo/NodeList
		},

		prepend: function(/*String||DOMNode||NodeList*/content){
			// summary:
			//		prepends the content to every node in the NodeList.
			// description:
			//		The content will be cloned if the length of NodeList
			//		is greater than 1. Only the DOM nodes are cloned, not
			//		any attached event handlers.
			// returns:
			//		dojo/NodeList, the nodes currently in this NodeList will be returned,
			//		not the appended content.
			//		assume a DOM created by this markup:
			//	|	<div id="foo"><p>Hello Mars</p></div>
			//	|	<div id="bar"><p>Hello World</p></div>
			//		Running this code:
			//	|	dojo.query("div").prepend("<span>prepend</span>");
			//		Results in this DOM structure:
			//	|	<div id="foo"><span>prepend</span><p>Hello Mars</p></div>
			//	|	<div id="bar"><span>prepend</span><p>Hello World</p></div>
			return this.addContent(content, "first"); // dojo/NodeList
		},

		prependTo: function(/*String*/query){
			// summary:
			//		prepends nodes in this NodeList to the nodes matched by
			//		the query passed to prependTo.
			// description:
			//		The nodes in this NodeList will be cloned if the query
			//		matches more than one element. Only the DOM nodes are cloned, not
			//		any attached event handlers.
			// returns:
			//		dojo/NodeList, the nodes currently in this NodeList will be returned,
			//		not the matched nodes from the query.
			// example:
			//		assume a DOM created by this markup:
			//	|	<span>prepend</span>
			//	|	<p>Hello Mars</p>
			//	|	<p>Hello World</p>
			//		Running this code:
			//	|	dojo.query("span").prependTo("p");
			//		Results in this DOM structure:
			//	|	<p><span>prepend</span>Hello Mars</p>
			//	|	<p><span>prepend</span>Hello World</p>
			return this._placeMultiple(query, "first"); // dojo/NodeList
		},

		after: function(/*String||Element||NodeList*/content){
			// summary:
			//		Places the content after every node in the NodeList.
			// description:
			//		The content will be cloned if the length of NodeList
			//		is greater than 1. Only the DOM nodes are cloned, not
			//		any attached event handlers.
			// returns:
			//		dojo/NodeList, the nodes currently in this NodeList will be returned,
			//		not the appended content.
			// example:
			//		assume a DOM created by this markup:
			//	|	<div id="foo"><p>Hello Mars</p></div>
			//	|	<div id="bar"><p>Hello World</p></div>
			//		Running this code:
			//	|	dojo.query("div").after("<span>after</span>");
			//		Results in this DOM structure:
			//	|	<div id="foo"><p>Hello Mars</p></div><span>after</span>
			//	|	<div id="bar"><p>Hello World</p></div><span>after</span>
			return this.addContent(content, "after"); // dojo/NodeList
		},

		insertAfter: function(/*String*/query){
			// summary:
			//		The nodes in this NodeList will be placed after the nodes
			//		matched by the query passed to insertAfter.
			// description:
			//		The nodes in this NodeList will be cloned if the query
			//		matches more than one element. Only the DOM nodes are cloned, not
			//		any attached event handlers.
			// returns:
			//		dojo/NodeList, the nodes currently in this NodeList will be returned,
			//		not the matched nodes from the query.
			// example:
			//		assume a DOM created by this markup:
			//	|	<span>after</span>
			//	|	<p>Hello Mars</p>
			//	|	<p>Hello World</p>
			//		Running this code:
			//	|	dojo.query("span").insertAfter("p");
			//		Results in this DOM structure:
			//	|	<p>Hello Mars</p><span>after</span>
			//	|	<p>Hello World</p><span>after</span>
			return this._placeMultiple(query, "after"); // dojo/NodeList
		},

		before: function(/*String||DOMNode||NodeList*/content){
			// summary:
			//		Places the content before every node in the NodeList.
			// description:
			//		The content will be cloned if the length of NodeList
			//		is greater than 1. Only the DOM nodes are cloned, not
			//		any attached event handlers.
			// returns:
			//		dojo/NodeList, the nodes currently in this NodeList will be returned,
			//		not the appended content.
			// example:
			//		assume a DOM created by this markup:
			//	|	<div id="foo"><p>Hello Mars</p></div>
			//	|	<div id="bar"><p>Hello World</p></div>
			//		Running this code:
			//	|	dojo.query("div").before("<span>before</span>");
			//		Results in this DOM structure:
			//	|	<span>before</span><div id="foo"><p>Hello Mars</p></div>
			//	|	<span>before</span><div id="bar"><p>Hello World</p></div>
			return this.addContent(content, "before"); // dojo/NodeList
		},

		insertBefore: function(/*String*/query){
			// summary:
			//		The nodes in this NodeList will be placed after the nodes
			//		matched by the query passed to insertAfter.
			// description:
			//		The nodes in this NodeList will be cloned if the query
			//		matches more than one element. Only the DOM nodes are cloned, not
			//		any attached event handlers.
			// returns:
			//		dojo/NodeList, the nodes currently in this NodeList will be returned,
			//		not the matched nodes from the query.
			// example:
			//		assume a DOM created by this markup:
			//	|	<span>before</span>
			//	|	<p>Hello Mars</p>
			//	|	<p>Hello World</p>
			//		Running this code:
			//	|	dojo.query("span").insertBefore("p");
			//		Results in this DOM structure:
			//	|	<span>before</span><p>Hello Mars</p>
			//	|	<span>before</span><p>Hello World</p>
			return this._placeMultiple(query, "before"); // dojo/NodeList
		},

		/*=====
		remove: function(simpleFilter){
			// summary:
			//		alias for dojo/NodeList's orphan method. Removes elements
			//		in this list that match the simple filter from their parents
			//		and returns them as a new NodeList.
			// simpleFilter: String
			//		single-expression CSS rule. For example, ".thinger" or
			//		"#someId[attrName='value']" but not "div > span". In short,
			//		anything which does not invoke a descent to evaluate but
			//		can instead be used to test a single node is acceptable.

			return; // dojo/NodeList
		},
		=====*/
		remove: NodeList.prototype.orphan,

		wrap: function(/*String||DOMNode*/html){
			// summary:
			//		Wrap each node in the NodeList with html passed to wrap.
			// description:
			//		html will be cloned if the NodeList has more than one
			//		element. Only DOM nodes are cloned, not any attached
			//		event handlers.
			// returns:
			//		the nodes in the current NodeList will be returned,
			//		not the nodes from html argument.
			// example:
			//		assume a DOM created by this markup:
			//	|	<b>one</b>
			//	|	<b>two</b>
			//		Running this code:
			//	|	dojo.query("b").wrap("<div><span></span></div>");
			//		Results in this DOM structure:
			//	|	<div><span><b>one</b></span></div>
			//	|	<div><span><b>two</b></span></div>
			if(this[0]){
				html = makeWrapNode(html, this[0]);

				//Now cycle through the elements and do the insertion.
				for(var i = 0, node; node = this[i]; i++){
					//Always clone because if html is used to hold one of
					//the "this" nodes, then on the clone of html it will contain
					//that "this" node, and that would be bad.
					var clone = this._cloneNode(html);
					if(node.parentNode){
						node.parentNode.replaceChild(clone, node);
					}
					//Find deepest element and insert old node in it.
					var insertion = getWrapInsertion(clone);
					insertion.appendChild(node);
				}
			}
			return this; // dojo/NodeList
		},

		wrapAll: function(/*String||DOMNode*/html){
			// summary:
			//		Insert html where the first node in this NodeList lives, then place all
			//		nodes in this NodeList as the child of the html.
			// returns:
			//		the nodes in the current NodeList will be returned,
			//		not the nodes from html argument.
			// example:
			//		assume a DOM created by this markup:
			//	|	<div class="container">
			// 	|		<div class="red">Red One</div>
			// 	|		<div class="blue">Blue One</div>
			// 	|		<div class="red">Red Two</div>
			// 	|		<div class="blue">Blue Two</div>
			//	|	</div>
			//		Running this code:
			//	|	dojo.query(".red").wrapAll('<div class="allRed"></div>');
			//		Results in this DOM structure:
			//	|	<div class="container">
			// 	|		<div class="allRed">
			// 	|			<div class="red">Red One</div>
			// 	|			<div class="red">Red Two</div>
			// 	|		</div>
			// 	|		<div class="blue">Blue One</div>
			// 	|		<div class="blue">Blue Two</div>
			//	|	</div>
			if(this[0]){
				html = makeWrapNode(html, this[0]);

				//Place the wrap HTML in place of the first node.
				this[0].parentNode.replaceChild(html, this[0]);

				//Now cycle through the elements and move them inside
				//the wrap.
				var insertion = getWrapInsertion(html);
				for(var i = 0, node; node = this[i]; i++){
					insertion.appendChild(node);
				}
			}
			return this; // dojo/NodeList
		},

		wrapInner: function(/*String||DOMNode*/html){
			// summary:
			//		For each node in the NodeList, wrap all its children with the passed in html.
			// description:
			//		html will be cloned if the NodeList has more than one
			//		element. Only DOM nodes are cloned, not any attached
			//		event handlers.
			// returns:
			//		the nodes in the current NodeList will be returned,
			//		not the nodes from html argument.
			// example:
			//		assume a DOM created by this markup:
			//	|	<div class="container">
			// 	|		<div class="red">Red One</div>
			// 	|		<div class="blue">Blue One</div>
			// 	|		<div class="red">Red Two</div>
			// 	|		<div class="blue">Blue Two</div>
			//	|	</div>
			//		Running this code:
			//	|	dojo.query(".red").wrapInner('<span class="special"></span>');
			//		Results in this DOM structure:
			//	|	<div class="container">
			// 	|		<div class="red"><span class="special">Red One</span></div>
			// 	|		<div class="blue">Blue One</div>
			// 	|		<div class="red"><span class="special">Red Two</span></div>
			// 	|		<div class="blue">Blue Two</div>
			//	|	</div>
			if(this[0]){
				html = makeWrapNode(html, this[0]);
				for(var i = 0; i < this.length; i++){
					//Always clone because if html is used to hold one of
					//the "this" nodes, then on the clone of html it will contain
					//that "this" node, and that would be bad.
					var clone = this._cloneNode(html);

					//Need to convert the childNodes to an array since wrapAll modifies the
					//DOM and can change the live childNodes NodeList.
					this._wrap(lang._toArray(this[i].childNodes), null, this._NodeListCtor).wrapAll(clone);
				}
			}
			return this; // dojo/NodeList
		},

		replaceWith: function(/*String||DOMNode||NodeList*/content){
			// summary:
			//		Replaces each node in ths NodeList with the content passed to replaceWith.
			// description:
			//		The content will be cloned if the length of NodeList
			//		is greater than 1. Only the DOM nodes are cloned, not
			//		any attached event handlers.
			// returns:
			//		The nodes currently in this NodeList will be returned, not the replacing content.
			//		Note that the returned nodes have been removed from the DOM.
			// example:
			//		assume a DOM created by this markup:
			//	|	<div class="container">
			// 	|		<div class="red">Red One</div>
			// 	|		<div class="blue">Blue One</div>
			// 	|		<div class="red">Red Two</div>
			// 	|		<div class="blue">Blue Two</div>
			//	|	</div>
			//		Running this code:
			//	|	dojo.query(".red").replaceWith('<div class="green">Green</div>');
			//		Results in this DOM structure:
			//	|	<div class="container">
			// 	|		<div class="green">Green</div>
			// 	|		<div class="blue">Blue One</div>
			// 	|		<div class="green">Green</div>
			// 	|		<div class="blue">Blue Two</div>
			//	|	</div>
			content = this._normalize(content, this[0]);
			for(var i = 0, node; node = this[i]; i++){
				this._place(content, node, "before", i > 0);
				node.parentNode.removeChild(node);
			}
			return this; // dojo/NodeList
		},

		replaceAll: function(/*String*/query){
			// summary:
			//		replaces nodes matched by the query passed to replaceAll with the nodes
			//		in this NodeList.
			// description:
			//		The nodes in this NodeList will be cloned if the query
			//		matches more than one element. Only the DOM nodes are cloned, not
			//		any attached event handlers.
			// returns:
			//		The nodes currently in this NodeList will be returned, not the matched nodes
			//		from the query. The nodes currently in this NodeLIst could have
			//		been cloned, so the returned NodeList will include the cloned nodes.
			// example:
			//		assume a DOM created by this markup:
			//	|	<div class="container">
			// 	|		<div class="spacer">___</div>
			// 	|		<div class="red">Red One</div>
			// 	|		<div class="spacer">___</div>
			// 	|		<div class="blue">Blue One</div>
			// 	|		<div class="spacer">___</div>
			// 	|		<div class="red">Red Two</div>
			// 	|		<div class="spacer">___</div>
			// 	|		<div class="blue">Blue Two</div>
			//	|	</div>
			//		Running this code:
			//	|	dojo.query(".red").replaceAll(".blue");
			//		Results in this DOM structure:
			//	|	<div class="container">
			// 	|		<div class="spacer">___</div>
			// 	|		<div class="spacer">___</div>
			// 	|		<div class="red">Red One</div>
			// 	|		<div class="red">Red Two</div>
			// 	|		<div class="spacer">___</div>
			// 	|		<div class="spacer">___</div>
			// 	|		<div class="red">Red One</div>
			// 	|		<div class="red">Red Two</div>
			//	|	</div>
			var nl = dquery(query);
			var content = this._normalize(this, this[0]);
			for(var i = 0, node; node = nl[i]; i++){
				this._place(content, node, "before", i > 0);
				node.parentNode.removeChild(node);
			}
			return this; // dojo/NodeList
		},

		clone: function(){
			// summary:
			//		Clones all the nodes in this NodeList and returns them as a new NodeList.
			// description:
			//		Only the DOM nodes are cloned, not any attached event handlers.
			// returns:
			//		a cloned set of the original nodes.
			// example:
			//		assume a DOM created by this markup:
			//	|	<div class="container">
			// 	|		<div class="red">Red One</div>
			// 	|		<div class="blue">Blue One</div>
			// 	|		<div class="red">Red Two</div>
			// 	|		<div class="blue">Blue Two</div>
			//	|	</div>
			//		Running this code:
			//	|	dojo.query(".red").clone().appendTo(".container");
			//		Results in this DOM structure:
			//	|	<div class="container">
			// 	|		<div class="red">Red One</div>
			// 	|		<div class="blue">Blue One</div>
			// 	|		<div class="red">Red Two</div>
			// 	|		<div class="blue">Blue Two</div>
			// 	|		<div class="red">Red One</div>
			// 	|		<div class="red">Red Two</div>
			//	|	</div>

			//TODO: need option to clone events?
			var ary = [];
			for(var i = 0; i < this.length; i++){
				ary.push(this._cloneNode(this[i]));
			}
			return this._wrap(ary, this, this._NodeListCtor); // dojo/NodeList
		}
	});

	//set up html method if one does not exist
	if(!NodeList.prototype.html){
		NodeList.prototype.html = NodeList.prototype.innerHTML;
	}

	return NodeList;
});

},
'epi-find/widget/ValidationTextAreaWithDefaultValue':function(){
define([
        "dojo/_base/declare",
        "epi-saas-base/widgets/ValidationTextarea",
        "epi-find/widget/_DefaultValueHintMixin"

    ],
    function(declare, ValidationTextarea, _DefaultValueHintMixin) {
        //  summary:
        //      A mixin for TextBox/TextArea based widgets that accepts a defaultValue that is used as a placeholder.
        //      If text in the widget equals to the defaultValue, then widgets value attribute returns empty string.

        // module:
        //      epi-find/widget/_DefaultValueHintMixin

        return declare([ValidationTextarea, _DefaultValueHintMixin], {});
    });





},
'epi-saas-base/widgets/ValidationTextarea':function(){
define("epi-saas-base/widgets/ValidationTextarea", [
    "dojo/_base/declare", // declare
    "dojo/i18n", // i18n.getLocalization
    "dijit/form/Textarea",
    "dijit/Tooltip"
], function(declare, i18n, Textarea, Tooltip){

    // module:
    //		epi-saas-base/widgets/ValidationTextarea

    return declare(Textarea, {
        // summary:
        //		Base class for textarea widgets with the ability to validate content of various types and provide user feedback.
        //      This is pretty much exact copy of dijit/form/ValidationTextBox except the base class.

        // required: Boolean
        //		User is required to enter data into this field.
        required: false,

        // promptMessage: String
        //		If defined, display this hint string immediately on focus to the textarea, if empty.
        //		Also displays if the textarea value is Incomplete (not yet valid but will be with additional input).
        //		Think of this like a tooltip that tells the user what to do, not an error message
        //		that tells the user what they've done wrong.
        //
        //		Message disappears when user starts typing.
        promptMessage: "",

        // invalidMessage: String
        //		The message to display if value is invalid.
        //		The translated string value is read from the message file by default.
        //		Set to "" to use the promptMessage instead.
        invalidMessage: "$_unset_$",

        // missingMessage: String
        //		The message to display if value is empty and the field is required.
        //		The translated string value is read from the message file by default.
        //		Set to "" to use the invalidMessage instead.
        missingMessage: "$_unset_$",

        // message: String
        //		Currently error/prompt message.
        //		When using the default tooltip implementation, this will only be
        //		displayed when the field is focused.
        message: "",

        // constraints: __Constraints
        //		user-defined object needed to pass parameters to the validator functions
        constraints: {},

        // pattern: [extension protected] String|Function(constraints) returning a string.
        //		This defines the regular expression used to validate the input.
        //		Do not add leading ^ or $ characters since the widget adds these.
        //		A function may be used to generate a valid pattern when dependent on constraints or other runtime factors.
        //		set("pattern", String|Function).
        pattern: ".*",

        // state: [readonly] String
        //		Shows current state (ie, validation result) of input (""=Normal, Incomplete, or Error)
        state: "",

        // tooltipPosition: String[]
        //		See description of `dijit/Tooltip.defaultPosition` for details on this parameter.
        tooltipPosition: [],

        _setValueAttr: function(){
            // summary:
            //		Hook so set("value", ...) works.
            this.inherited(arguments);
            this.validate(this.focused);
        },

        validator: function(/*anything*/ value, /*__Constraints*/ constraints){
            // summary:
            //		Overridable function used to validate the text input against the regular expression.
            // tags:
            //		protected
            return (new RegExp("^(?:" + this._getPatternAttr(constraints) + ")"+(this.required?"":"?")+"$","m")).test(value) &&
                (!this.required || !this._isEmpty(value)) &&
                (this._isEmpty(value) || this.parse(value, constraints) !== undefined); // Boolean
        },

        _isValidSubset: function(){
            // summary:
            //		Returns true if the value is either already valid or could be made valid by appending characters.
            //		This is used for validation while the user [may be] still typing.
            return this.textbox.value.search(this._partialre) === 0;
        },

        isValid: function(/*Boolean*/ /*===== isFocused =====*/){
            // summary:
            //		Tests if value is valid.
            //		Can override with your own routine in a subclass.
            // tags:
            //		protected
            return this.validator(this.textbox.value, this.constraints);
        },

        _isEmpty: function(value){
            // summary:
            //		Checks for whitespace
            return (this.trim ? /^\s*$/ : /^$/).test(value); // Boolean
        },

        getErrorMessage: function(/*Boolean*/ /*===== isFocused =====*/){
            // summary:
            //		Return an error message to show if appropriate
            // tags:
            //		protected
            var invalid = this.invalidMessage === "$_unset_$" ? this.messages.invalidMessage :
                !this.invalidMessage ? this.promptMessage : this.invalidMessage;
            var missing = this.missingMessage === "$_unset_$" ? this.messages.missingMessage :
                !this.missingMessage ? invalid : this.missingMessage;
            return (this.required && this._isEmpty(this.textbox.value)) ? missing : invalid; // String
        },

        getPromptMessage: function(/*Boolean*/ /*===== isFocused =====*/){
            // summary:
            //		Return a hint message to show when widget is first focused
            // tags:
            //		protected
            return this.promptMessage; // String
        },

        _maskValidSubsetError: true,
        validate: function(/*Boolean*/ isFocused){
            // summary:
            //		Called by oninit, onblur, and onkeypress.
            // description:
            //		Show missing or invalid messages if appropriate, and highlight textarea field.
            // tags:
            //		protected
            var message = "";
            var isValid = this.disabled || this.isValid(isFocused);
            if(isValid){ this._maskValidSubsetError = true; }
            var isEmpty = this._isEmpty(this.textbox.value);
            var isValidSubset = !isValid && isFocused && this._isValidSubset();
            this._set("state", isValid ? "" : (((((!this._hasBeenBlurred || isFocused) && isEmpty) || isValidSubset) && (this._maskValidSubsetError || (isValidSubset && !this._hasBeenBlurred && isFocused))) ? "Incomplete" : "Error"));
            this.focusNode.setAttribute("aria-invalid", isValid ? "false" : "true");

            if(this.state === "Error"){
                this._maskValidSubsetError = isFocused && isValidSubset; // we want the error to show up after a blur and refocus
                message = this.getErrorMessage(isFocused);
            }else if(this.state === "Incomplete"){
                message = this.getPromptMessage(isFocused); // show the prompt whenever the value is not yet complete
                this._maskValidSubsetError = !this._hasBeenBlurred || isFocused; // no Incomplete warnings while focused
            }else if(isEmpty){
                message = this.getPromptMessage(isFocused); // show the prompt whenever there's no error and no text
            }
            this.set("message", message);

            return isValid;
        },

        displayMessage: function(/*String*/ message){
            // summary:
            //		Overridable method to display validation errors/hints.
            //		By default uses a tooltip.
            // tags:
            //		extension
            if(message && this.focused){
                Tooltip.show(message, this.domNode, this.tooltipPosition, !this.isLeftToRight());
            }else{
                Tooltip.hide(this.domNode);
            }
        },

        _refreshState: function(){
            // Overrides TextBox._refreshState()
            if(this._created){
                this.validate(this.focused);
            }
            this.inherited(arguments);
        },

        //////////// INITIALIZATION METHODS ///////////////////////////////////////

        constructor: function(params /*===== , srcNodeRef =====*/){
            // summary:
            //		Create the widget.
            // params: Object|null
            //		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
            //		and functions, typically callbacks like onClick.
            //		The hash can contain any of the widget's properties, excluding read-only properties.
            // srcNodeRef: DOMNode|String?
            //		If a srcNodeRef (DOM node) is specified, replace srcNodeRef with my generated DOM tree.

            this.constraints = {};
            this.baseClass += " dijitValidationTextBox";
        },

        startup: function(){
            this.inherited(arguments);
            this._refreshState(); // after all _set* methods have run
        },

        _setConstraintsAttr: function(/*__Constraints*/ constraints){
            if(!constraints.locale && this.lang){
                constraints.locale = this.lang;
            }
            this._set("constraints", constraints);
            this._refreshState();
        },

        _setPatternAttr: function(/*String|Function*/ pattern){
            this._set("pattern", pattern); // don't set on INPUT to avoid native HTML5 validation
        },


        postMixInProperties: function(){
            this.inherited(arguments);
            this.messages = i18n.getLocalization("dijit.form", "validate", this.lang);
            this._setConstraintsAttr(this.constraints); // this needs to happen now (and later) due to codependency on _set*Attr calls attachPoints
        },

        _getPatternAttr: function(/*__Constraints*/ constraints){
            // summary:
            //		Hook to get the current regExp and to compute the partial validation RE.
            var p = this.pattern;
            var type = (typeof p).toLowerCase();
            if(type === "function"){
                p = this.pattern(constraints || this.constraints);
            }
            if(p !== this._lastRegExp){
                var partialre = "";
                this._lastRegExp = p;
                // parse the regexp and produce a new regexp that matches valid subsets
                // if the regexp is .* then there's no use in matching subsets since everything is valid
                if(p !== ".*"){
                    p.replace(/\\.|\[\]|\[.*?[^\\]{1}\]|\{.*?\}|\(\?[=:!]|./g,
                        function(re){
                            switch(re.charAt(0)){
                                case "{":
                                case "+":
                                case "?":
                                case "*":
                                case "^":
                                case "$":
                                case "|":
                                case "(":
                                    partialre += re;
                                    break;
                                case ")":
                                    partialre += "|$)";
                                    break;
                                default:
                                    partialre += "(?:"+re+"|$)";
                                    break;
                            }
                        });
                }
                try{ // this is needed for now since the above regexp parsing needs more test verification
                    "".search(partialre);
                }catch(e){ // should never be here unless the original RE is bad or the parsing is bad
                    partialre = this.pattern;
                    console.warn("RegExp error in " + this.declaredClass + ": " + this.pattern);
                } // should never be here unless the original RE is bad or the parsing is bad
                this._partialre = "^(?:" + partialre + ")$";
            }
            return p;
        },
        
        _setDisabledAttr: function(/*Boolean*/ value){
            this.inherited(arguments);	// call FormValueWidget._setDisabledAttr()
            this._refreshState();
        },

        _setRequiredAttr: function(/*Boolean*/ value){
            this._set("required", value);
            this.focusNode.setAttribute("aria-required", value);
            this._refreshState();
        },

        _setMessageAttr: function(/*String*/ message){
            this._set("message", message);
            this.displayMessage(message);
        },

        reset:function(){
            // Overrides dijit/form/Textarea.reset() by also
            // hiding errors about partial matches
            this._maskValidSubsetError = true;
            this.inherited(arguments);
        },

        _onBlur: function(){
            // the message still exists but for back-compat, and to erase the tooltip
            // (if the message is being displayed as a tooltip), call displayMessage('')
            this.displayMessage("");

            this.inherited(arguments);
        }
    });
});

},
'dijit/form/Textarea':function(){
define("dijit/form/Textarea", [
	"dojo/_base/declare", // declare
	"dojo/dom-style", // domStyle.set
	"./_ExpandingTextAreaMixin",
	"./SimpleTextarea"
], function(declare, domStyle, _ExpandingTextAreaMixin, SimpleTextarea){

// module:
//		dijit/form/Textarea


return declare("dijit.form.Textarea", [SimpleTextarea, _ExpandingTextAreaMixin], {
	// summary:
	//		A textarea widget that adjusts it's height according to the amount of data.
	//
	// description:
	//		A textarea that dynamically expands/contracts (changing it's height) as
	//		the user types, to display all the text without requiring a scroll bar.
	//
	//		Takes nearly all the parameters (name, value, etc.) that a vanilla textarea takes.
	//		Rows is not supported since this widget adjusts the height.
	//
	// example:
	// |	<textarea data-dojo-type="dijit/form/TextArea">...</textarea>


	// TODO: for 2.0, rename this to ExpandingTextArea, and rename SimpleTextarea to TextArea

	baseClass: "dijitTextBox dijitTextArea dijitExpandingTextArea",

	// Override SimpleTextArea.cols to default to width:100%, for backward compatibility
	cols: "",

	buildRendering: function(){
		this.inherited(arguments);

		// tweak textarea style to reduce browser differences
		domStyle.set(this.textbox, { overflowY: 'hidden', overflowX: 'auto', boxSizing: 'border-box', MsBoxSizing: 'border-box', WebkitBoxSizing: 'border-box', MozBoxSizing: 'border-box' });
	}
});

});

},
'dijit/form/_ExpandingTextAreaMixin':function(){
define("dijit/form/_ExpandingTextAreaMixin", [
	"dojo/_base/declare", // declare
	"dojo/dom-construct", // domConstruct.create
	"dojo/has",
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"dojo/_base/window", // win.body
	"../Viewport"
], function(declare, domConstruct, has, lang, on, win, Viewport){

	// module:
	//		dijit/form/_ExpandingTextAreaMixin

	// feature detection, true for mozilla and webkit
	has.add("textarea-needs-help-shrinking", function(){
		var body = win.body(),	// note: if multiple documents exist, doesn't matter which one we use
			te = domConstruct.create('textarea', {
			rows:"5",
			cols:"20",
			value: ' ',
			style: {zoom:1, fontSize:"12px", height:"96px", overflow:'hidden', visibility:'hidden', position:'absolute', border:"5px solid white", margin:"0", padding:"0", boxSizing: 'border-box', MsBoxSizing: 'border-box', WebkitBoxSizing: 'border-box', MozBoxSizing: 'border-box' }
		}, body, "last");
		var needsHelpShrinking = te.scrollHeight >= te.clientHeight;
		body.removeChild(te);
		return needsHelpShrinking;
	});

	return declare("dijit.form._ExpandingTextAreaMixin", null, {
		// summary:
		//		Mixin for textarea widgets to add auto-expanding capability

		_setValueAttr: function(){
			this.inherited(arguments);
			this.resize();
		},

		postCreate: function(){
			this.inherited(arguments);
			var textarea = this.textbox;
			textarea.style.overflowY = "hidden";
			this.own(on(textarea, "focus, resize", lang.hitch(this, "_resizeLater")));
		},

		startup: function(){ 
			this.inherited(arguments);
			this.own(Viewport.on("resize", lang.hitch(this, "_resizeLater")));
			this._resizeLater();
		},

		_onInput: function(e){
			this.inherited(arguments);
			this.resize();
		},

		_estimateHeight: function(){
			// summary:
			//		Approximate the height when the textarea is invisible with the number of lines in the text.
			//		Fails when someone calls setValue with a long wrapping line, but the layout fixes itself when the user clicks inside so . . .
			//		In IE, the resize event is supposed to fire when the textarea becomes visible again and that will correct the size automatically.
			//
			var textarea = this.textbox;
			// #rows = #newlines+1
			textarea.rows = (textarea.value.match(/\n/g) || []).length + 1;
		},

		_resizeLater: function(){
			this.defer("resize");
		},

		resize: function(){
			// summary:
			//		Resizes the textarea vertically (should be called after a style/value change)

			var textarea = this.textbox;

			function textareaScrollHeight(){
				var empty = false;
				if(textarea.value === ''){
					textarea.value = ' ';
					empty = true;
				}
				var sh = textarea.scrollHeight;
				if(empty){ textarea.value = ''; }
				return sh;
			}

			if(textarea.style.overflowY == "hidden"){ textarea.scrollTop = 0; }
			if(this.busyResizing){ return; }
			this.busyResizing = true;
			if(textareaScrollHeight() || textarea.offsetHeight){
				var newH = textareaScrollHeight() + Math.max(textarea.offsetHeight - textarea.clientHeight, 0);
				var newHpx = newH + "px";
				if(newHpx != textarea.style.height){
					textarea.style.height = newHpx;
					textarea.rows = 1; // rows can act like a minHeight if not cleared
				}
				if(has("textarea-needs-help-shrinking")){
					var	origScrollHeight = textareaScrollHeight(),
						newScrollHeight = origScrollHeight,
						origMinHeight = textarea.style.minHeight,
						decrement = 4, // not too fast, not too slow
						thisScrollHeight,
						origScrollTop = textarea.scrollTop;
					textarea.style.minHeight = newHpx; // maintain current height
					textarea.style.height = "auto"; // allow scrollHeight to change
					while(newH > 0){
						textarea.style.minHeight = Math.max(newH - decrement, 4) + "px";
						thisScrollHeight = textareaScrollHeight();
						var change = newScrollHeight - thisScrollHeight;
						newH -= change;
						if(change < decrement){
							break; // scrollHeight didn't shrink
						}
						newScrollHeight = thisScrollHeight;
						decrement <<= 1;
					}
					textarea.style.height = newH + "px";
					textarea.style.minHeight = origMinHeight;
					textarea.scrollTop = origScrollTop;
				}
				textarea.style.overflowY = textareaScrollHeight() > textarea.clientHeight ? "auto" : "hidden";
				if(textarea.style.overflowY == "hidden"){ textarea.scrollTop = 0; }
			}else{
				// hidden content of unknown size
				this._estimateHeight();
			}
			this.busyResizing = false;
		}
	});
});

},
'dijit/form/SimpleTextarea':function(){
define("dijit/form/SimpleTextarea", [
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add
	"dojo/sniff", // has("ie") has("opera")
	"./TextBox"
], function(declare, domClass, has, TextBox){

// module:
//		dijit/form/SimpleTextarea


return declare("dijit.form.SimpleTextarea", TextBox, {
	// summary:
	//		A simple textarea that degrades, and responds to
	//		minimal LayoutContainer usage, and works with dijit/form/Form.
	//		Doesn't automatically size according to input, like Textarea.
	//
	// example:
	//	|	<textarea data-dojo-type="dijit/form/SimpleTextarea" name="foo" value="bar" rows=30 cols=40></textarea>
	//
	// example:
	//	|	new SimpleTextarea({ rows:20, cols:30 }, "foo");

	baseClass: "dijitTextBox dijitTextArea",

	// rows: Number
	//		The number of rows of text.
	rows: "3",

	// rows: Number
	//		The number of characters per line.
	cols: "20",

	templateString: "<textarea ${!nameAttrSetting} data-dojo-attach-point='focusNode,containerNode,textbox' autocomplete='off'></textarea>",

	postMixInProperties: function(){
		// Copy value from srcNodeRef, unless user specified a value explicitly (or there is no srcNodeRef)
		// TODO: parser will handle this in 2.0
		if(!this.value && this.srcNodeRef){
			this.value = this.srcNodeRef.value;
		}
		this.inherited(arguments);
	},

	buildRendering: function(){
		this.inherited(arguments);
		if(has("ie") && this.cols){ // attribute selectors is not supported in IE6
			domClass.add(this.textbox, "dijitTextAreaCols");
		}
	},

	filter: function(/*String*/ value){
		// Override TextBox.filter to deal with newlines... specifically (IIRC) this is for IE which writes newlines
		// as \r\n instead of just \n
		if(value){
			value = value.replace(/\r/g,"");
		}
		return this.inherited(arguments);
	},

	_onInput: function(/*Event?*/ e){
		// Override TextBox._onInput() to enforce maxLength restriction
		if(this.maxLength){
			var maxLength = parseInt(this.maxLength);
			var value = this.textbox.value.replace(/\r/g,'');
			var overflow = value.length - maxLength;
			if(overflow > 0){
				var textarea = this.textbox;
				if(textarea.selectionStart){
					var pos = textarea.selectionStart;
					var cr = 0;
					if(has("opera")){
						cr = (this.textbox.value.substring(0,pos).match(/\r/g) || []).length;
					}
					this.textbox.value = value.substring(0,pos-overflow-cr)+value.substring(pos-cr);
					textarea.setSelectionRange(pos-overflow, pos-overflow);
				}else if(this.ownerDocument.selection){ //IE
					textarea.focus();
					var range = this.ownerDocument.selection.createRange();
					// delete overflow characters
					range.moveStart("character", -overflow);
					range.text = '';
					// show cursor
					range.select();
				}
			}
		}
		this.inherited(arguments);
	}
});

});

},
'epi-find/widget/_DefaultValueHintMixin':function(){
define([
    "dojo/_base/declare"
],
function(declare) {
    //  summary:
    //      A mixin for TextBox/TextArea based widgets that accepts a defaultValue that is used as a placeholder.
    //      If text in the widget equals to the defaultValue, then widgets value attribute returns empty string.

    // module:
    //      epi-find/widget/_DefaultValueHintMixin

    return declare(null, {
        defaultValue: null,
        _setDefaultValueAttr: function(value) {
            // save original value of the widgets "value" attribute
            var oldValue = this.get("value"),
            // A bit invasive: save current watch callbacks and nullify them in order to
            // prevent "watches" from firing while we changing the "value" property
                watchCallbacks = this._watchCallbacks;
            this._watchCallbacks = null;

            // Run the default value via widgets "value" attribute setter
            this.set("value", value);
            // use "value" after possible transformations in the setter
            this._set("defaultValue", this.value);
            // show default value as a placeholder
            this.set("placeholder", this.value);
            // restore original attribute value
            this.set("value", oldValue);

            // Restore watch callbacks
            this._watchCallbacks = watchCallbacks;
        },

        onFocus: function() {
            this.inherited(arguments);
            if (!this.value && this.defaultValue) {
                this.set("value", this.defaultValue);
            }
        },

        onBlur: function() {
            this.inherited(arguments);
            if (this.value && this.value === this.defaultValue) {
                this.set("value", "");
            }
        },

        reset: function() {
            this.inherited(arguments);
            this.set("defaultValue", "");
        }
    });
});

},
'epi-find/widget/TextBoxWithDefaultValue':function(){
define([
    "dojo/_base/declare",
    "dijit/form/ValidationTextBox",
    "./_DefaultValueHintMixin"
],
function(declare,
         ValidationTextBox,
         _DefaultValueHintMixin
    ) {

    return declare([ValidationTextBox, _DefaultValueHintMixin], {});
});
},
'epi-find/widget/TextAreaWithDefaultValue':function(){
define([
    "dojo/_base/declare",
    "dijit/form/Textarea",
    "./_DefaultValueHintMixin"
],
function(declare,
         Textarea,
         _DefaultValueHintMixin
    ) {

    return declare([Textarea, _DefaultValueHintMixin], {});
});
},
'epi-find/widget/PhraseValidationTextBox':function(){
define([
    "dojo/_base/array",
    "dojo/_base/declare",

    "dijit/form/ValidationTextBox",
    "dojo/i18n!./nls/PhraseValidation"
], function(
    array,
    declare,
    ValidationTextBox,
    i18n
){

    // module:
    //      epi-find/widget/PhraseValidationTextBox

    return declare([ValidationTextBox], {
        // summary:
        //      A validation text box adapted to properly validate phrases.
        validationMessage: "",
        phraseDelimiter: "",

        validator: function(/*anything*/ value){
            // summary:
            //      Overridden function to add validation regarding double spaces within phrases and
            //      empty phrases or duplicates on multiple comma separated phrases
            // tags:
            //      protected
            this.validationMessage = "";
            if(this.inherited(arguments) && value && value.split) {
                // Split value on
                var values = this.phraseDelimiter !== "" ? array.map(value.split(this.phraseDelimiter), function(item) { return item.trim();}) : [value.trim()];
                return !array.some(values, function(item, i){
                    // Empty phrases or double spaces (within phrases) or duplicates are not allowed
                    if(item.length === 0) {
                        this.validationMessage = i18n.emptyPhrases;
                        return true;
                    }
                    if(item.match(/\s{2}/)) {
                        this.validationMessage = i18n.doubleSpaces;
                        return true;
                    }
                    if (values.indexOf(item.toLowerCase(), i+1) !== -1) {
                        this.validationMessage = i18n.duplicatePhrases;
                        return true;
                    }
                }, this);
            }
            return false;
        },

        getErrorMessage: function(/*Boolean*/ /*===== isFocused =====*/){
            // summary:
            //        Return an error message to show if appropriate
            // tags:
            //        protected
            return this.validationMessage || this.inherited(arguments);
        }
    });
});

},
'epi-saas-base/UrlRegexp':function(){
define("epi-saas-base/UrlRegexp", [], function() {
    // summary:
    //      A string containing a regexp matching a valid URL

    // Until https://bugs.dojotoolkit.org/ticket/11212 has been resolved we can use this instead of dojox.validate.isUrl
    return "(([Hh][tT][tT]|[fF][tT])[pP][sS]{0,1})\\://(((?:(?:[\\da-zA-Z](?:[-\\da-zA-Z]{0,61}[\\da-zA-Z])?)\\.)+(?:[a-zA-Z](?:[-\\da-zA-Z]{0,61}[\\da-zA-Z])?)\\.?)|(((\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])\\.){3}(\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])|(0[xX]0*[\\da-fA-F]?[\\da-fA-F]\\.){3}0[xX]0*[\\da-fA-F]?[\\da-fA-F]|(0+[0-3][0-7][0-7]\\.){3}0+[0-3][0-7][0-7]|(0|[1-9]\\d{0,8}|[1-3]\\d{9}|4[01]\\d{8}|42[0-8]\\d{7}|429[0-3]\\d{6}|4294[0-8]\\d{5}|42949[0-5]\\d{4}|429496[0-6]\\d{3}|4294967[01]\\d{2}|42949672[0-8]\\d|429496729[0-5])|0[xX]0*[\\da-fA-F]{1,8}|([\\da-fA-F]{1,4}\\:){7}[\\da-fA-F]{1,4}|([\\da-fA-F]{1,4}\\:){6}((\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])\\.){3}(\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])))(\\:\\d+)?(/(?:[^?#\\s/]+/)*(?:[^?#\\s/]*(?:\\?[^?#\\s/]*)?(?:#[A-Za-z][\\w.:-]*)?)?)?";
});
},
'url:epi-find/widget/templates/BestbetForm.html':"<form data-dojo-attach-point='containerNode' data-dojo-attach-event='onreset:_onReset,onsubmit:_onSubmit' class='epi-formsWidgetWrapper'>\r\n    <ol class='epi-formsWidgetWrapper'>\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\" data-dojo-attach-point=\"bestBetPhraseListItem\">\r\n            <input id=\"bestbetPhrase\" name=\"phraseField\" data-dojo-attach-point=\"phraseField\" data-dojo-type=\"epi-find/widget/PhraseValidationTextBox\" data-dojo-props=\"required:true, phraseDelimiter:','\"/>\r\n        </li>\r\n\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\"  data-dojo-attach-point=\"targetContentListItem\">\r\n            <ol data-dojo-attach-point=\"targetContentInnerList\">\r\n                <li data-dojo-type=\"epi-saas-base/form/FormListItem\" class=\"epi-formsRow--inlineInputs--radio\" data-dojo-props=\"inputStyleType: 'inlineInputs'\">\r\n                    <input id=\"targetContentRadio\" data-dojo-attach-point=\"targetContentRadio\" data-dojo-type=\"dijit/form/RadioButton\" data-dojo-props=\"checked: true, value: '${CMS_SELECTOR_TYPE}'\" name=\"contentSelection\"/>\r\n                    <label for=\"targetContentRadio\" data-dojo-attach-point=\"targetContentRadioLabel\">${i18n.contentType_local}</label>\r\n                    <span id=\"bestbetTarget\" data-dojo-attach-point=\"cmsSelector\" data-dojo-type=\"epi-find/widget/CmsContentSelector\" data-dojo-props=\"required:true, repositoryKey: 'pages', searchArea: 'CMS/pages&files', allowedTypes: ['episerver.core.pagedata', 'episerver.core.icontentmedia'], showStatusIcon: true\"></span>\r\n                </li>\r\n                <li data-dojo-type=\"epi-saas-base/form/FormListItem\" data-dojo-attach-point=\"commerceSelectorListItem\" class=\"epi-formsRow--inlineInputs--radio\" data-dojo-props=\"inputStyleType: 'inlineInputs'\">\r\n                    <input id=\"commerceLinkRadio\" data-dojo-attach-point=\"commerceLinkRadio\" data-dojo-type=\"dijit/form/RadioButton\" data-dojo-props=\"value: '${COMMERCE_SELECTOR_TYPE}'\" name=\"contentSelection\"/>\r\n                    <label for=\"commerceLinkRadio\" data-dojo-attach-point=\"commerceLinkRadioLabel\" >${i18n.contentType_commerce}</label>\r\n                    <span id=\"bestbetCommerceTarget\" data-dojo-attach-point=\"commerceSelector\" data-dojo-type=\"epi-find/widget/CmsContentSelector\" data-dojo-props=\"required:true, repositoryKey: 'catalog', searchArea: 'Commerce/Catalog', allowedTypes: ['episerver.commerce.catalog.contenttypes.entrycontentbase', 'episerver.commerce.catalog.contenttypes.nodecontent'], showStatusIcon: true\"></span>\r\n                </li>\r\n                <li data-dojo-type=\"epi-saas-base/form/FormListItem\" class=\"epi-formsRow--inlineInputs--radio\" data-dojo-props=\"inputStyleType: 'inlineInputs'\">\r\n                    <input id=\"externalLinkRadio\" data-dojo-attach-point=\"externalLinkRadio\" data-dojo-type=\"dijit/form/RadioButton\" data-dojo-props=\"value: '${EXTERNAL_SELECTOR_TYPE}'\" name=\"contentSelection\"/>\r\n                    <label for = \"externalLinkRadio\" data-dojo-attach-point=\"externalLinkRadioLabel\" >${i18n.contentType_external}</label>\r\n                    <span class=\"epi-statusIndicatorIcon epi-no-background\"></span>\r\n                    <span class=\"epi-resourceInputContainer\">\r\n                       <input id=\"externalUrlTarget\" data-dojo-attach-point=\"externalUrl\" data-dojo-type=\"dijit/form/ValidationTextBox\" data-dojo-props=\"required:true, trim:true, pattern: this.urlRegex, intermediateChanges:true\" />\r\n                    </span>\r\n                </li>\r\n            </ol>\r\n        </li>\r\n\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\" data-dojo-attach-point=\"bestBetTitleListItem\">\r\n            <input id=\"bestbetTitle\" name=\"title\" data-dojo-attach-point=\"titleField\" data-dojo-type=\"epi-find/widget/TextBoxWithDefaultValue\" data-dojo-props=\"required:false, intermediateChanges:true\" />\r\n        </li>\r\n\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\" data-dojo-attach-point=\"inputIdListItem\">\r\n            <input id=\"bestbetDescription\" name=\"description\" data-dojo-attach-point=\"bestbetDescField\" data-dojo-type=\"epi-find/widget/ValidationTextAreaWithDefaultValue\" data-dojo-props=\"required:false, intermediateChanges:true\"/>\r\n        </li>\r\n\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\"  data-dojo-props=\"inputStyleType: ''\" data-dojo-attach-point=\"bestBetStyleListItem\" class=\"epi-formsRow--stacked-inputs--radio\">\r\n            <input id=\"bestbetStyleRadio\" data-dojo-attach-point=\"bestbetStyleRadio\" data-dojo-type=\"dijit/form/RadioButton\" data-dojo-props=\"checked: true \" name=\"bestbetStyle\"/>\r\n            <label for = \"bestbetStyleRadio\">${i18n.fields_bestbetStyleRadio1}</label><br />\r\n            <input id=\"resultStyleRadio2\" data-dojo-attach-point=\"resultStyleRadio\" data-dojo-type=\"dijit/form/RadioButton\" name=\"bestbetStyle\"/>\r\n            <label for = \"resultStyleRadio2\">${i18n.fields_bestbetStyleRadio2}</label>\r\n        </li>\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\" class=\"epi-formsRow--align-right\">\r\n            <input data-dojo-attach-point=\"submitButton\" type=\"submit\" data-dojo-type=\"dijit/form/Button\" class=\"epi-primary\" />\r\n            <input data-dojo-attach-point=\"resetButton\" type=\"reset\" data-dojo-type=\"dijit/form/Button\" />\r\n        </li>\r\n    </ol>\r\n</form>\r\n",
'epi-find/widget/BestbetGrid':function(){
﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/date",
    "dojo/dom-class",
    "dojo/dom-construct",
    "dojo/when",

    "dojox/html/entities",

    "dijit/MenuItem",

    "dgrid/Grid",
    "dgrid/extensions/DijitRegistry",

    "put-selector/put",

    "epi/shell/widget/ContextMenu",
    "epi/shell/dgrid/Formatter",
    "epi-cms/dgrid/formatters",

    "epi-saas-base/widgets/_GridMoreMixin",
    "epi-saas-base/widgets/_AutoLoadScrollMixin",
    "epi-saas-base/widgets/_WidgetsInListMixin",
    "epi-saas-base/widgets/_NotificationMixin",
    "epi-saas-base/widgets/_GridContextMenuMixin",

    "./BestbetGridRow",

    "dojo/i18n!./nls/Bestbets"
],
    function(
             declare,
             lang,
             date,
             domClass,
             domConstruct,
             when,

             entities,

             MenuItem,

             Grid,
             DijitRegistry,

             put,

             ContextMenu,
             Formatter,
             formatters,

             _GridMoreMixin,
             _AutoLoadScrollMixin,
             _WidgetsInListMixin,
             _NotificationMixin,
             _GridContextMenuMixin,

             BestbetGridRow,
             i18n
             ) {

        // module:
        //      epi-find/widget/BestbetGrid

        return declare([Grid, _GridMoreMixin, _AutoLoadScrollMixin, Formatter, _GridContextMenuMixin, DijitRegistry, _WidgetsInListMixin, _NotificationMixin], {
            // summary:
            //     A grid designed to display Best Bets
            constructor: function() {
                // Sets the loader (#loadNextPage, available in _GridMoreMixn) to be used by the _AutoLoadScrollMixin
                this.loader = lang.hitch(this, this.loadNextPage);
            },
            contextMenu: new ContextMenu(),
            i18n: i18n,
            showHeader: false,
            margin: 40,
            columns: {
                phrases: {
                    renderCell: function(object, value, node, options) {
                        var item = new BestbetGridRow({
                            title: entities.encode(object.best_bet_target_title || object.best_bet_target_document_title || ""),
                            description: entities.encode(object.best_bet_target_description || object.best_bet_target_document_description || ""),
                            url: object.best_bet_target_url,
                            phrases: entities.encode(object.phrases)
                        }, node);
                        this.grid.own(item);
                        domConstruct.place(item.domNode, node);
                    }
                },
                expiration: {
                    className: "epi-grid--20 epi-alignMiddle",
                    renderCell: function(object, value, node, options) {
                        var status = object.best_bet_target_document_status || 0;
                        var div = put(node, "div.epi-expired-container");
                        if (status !== 0 && status !== 4 ) { // not published
                            put(div, "span.dijitIcon.dijitInline.dijitReset.epi-statusIndicatorIcon.epi-statusIndicator"+status);
                            put(div, "div", i18n["status_"+status]);
                        }
                    }
                },
                context: {
                    label: " ",
                    className: "epi-grid-column--icon24x24",
                    formatter: function() {
                        return formatters.menu({title: i18n.contextMenuTitle});
                    }
                }
            },

            postCreate: function() {
                this.inherited(arguments);

                var self = this,
                    getCurrentItem = function(context) {
                        var row = self.getRow(context);
                        return encodeURIComponent(row.id);
                    },
                    editMenuItem = new MenuItem({
                        label: i18n.actions_edit,
                        onClick: function() {
                            self.onEdit(getCurrentItem(this));
                        }
                    }),
                    deleteMenuItem = new MenuItem({
                        label: i18n.actions_delete,
                        onClick: function() {
                            var id = getCurrentItem(this);
                            self.showConfirmation({
                                title: i18n.deleteBestbet,
                                message: i18n.areYouSure,
                                confirmActionText: i18n.buttons_confirmDelete,
                                cancelActionText: i18n.buttons_cancelDelete,
                                action: function() {
                                    when(self.store.remove(id),
                                        function() {
                                            self.showSuccess(i18n.deletedMessage);
                                        },
                                        function(error) {
                                            self.showError(error);
                                            self.refresh();
                                        }
                                    );
                                }
                            });
                        }
                    });

                this.own(editMenuItem, deleteMenuItem,
                    this.on("dgrid-error", function(error) {
                        self.showErrorInGrid(error);
                    })
                );
                this.contextMenu.addChild(editMenuItem);
                this.contextMenu.addChild(deleteMenuItem);
            },

            onEdit: function(id) {
                // summary:
                //      A callback for Edit action
            }

        });
    });

},
'epi-saas-base/widgets/_WidgetsInListMixin':function(){
define("epi-saas-base/widgets/_WidgetsInListMixin", [
    "dojo/_base/declare",
    "dojo/_base/array",

    "dijit/registry"
],
    function(declare,
             array,
             registry) {

        return declare([], {
            // summary:
            //      This class should be mixed to dgrid lists or grids in order to find and destroy all nested widgets when the row is removed.

            removeRow: function (rowElement, justCleanup) {
                // summary:
                //		Finds and destroys widgets under rowElement node.
                // rowElement: Object|DOMNode
                //		Object or element representing the row to be removed.
                // justCleanup: Boolean
                //		If true, the row element will not be removed from the DOM; this can
                //		be used by extensions/plugins in cases where the DOM will be
                //		massively cleaned up at a later point in time.

                var rowNode = rowElement.element || rowElement;
                var nestedWidgets = registry.findWidgets(rowNode);
                var rowWidget = registry.byNode(rowNode);
                if (rowWidget) {
                    nestedWidgets.push(rowWidget);
                }
                array.forEach(nestedWidgets, function (widget) {
                    if (widget) {
                        widget.destroyRecursive();
                    }
                });

                this.inherited(arguments);
            }
        });

    });
},
'epi-find/widget/BestbetGridRow':function(){
define([
        "dojo/_base/declare",
        "dojo/_base/array",
        "dojo/dom-attr",
        "dojo/dom-class",

        "dijit/_WidgetBase",
        "dijit/_TemplatedMixin",

        "dojo/text!./templates/BestbetGridRow.html",
        "dojo/i18n!./nls/Bestbets",
        "dojo/on"
    ],
    function(declare,
             array,
             domAttr,
             domClass,
             _WidgetBase,
             _TemplatedMixin,
             template,
             i18n,
             on
        ) {
        return declare([_WidgetBase, _TemplatedMixin], {
            i18n: i18n,

            templateString: template,

            _setTitleAttr: {node: "titleNode", type: "innerHTML"},

            _setDescriptionAttr: {node: "descriptionNode", type: "innerHTML" },

            _setUrlAttr: function(url) {
                this.urlNode.innerHTML = url || "";
                var rxA = /\/(global|content|site)assets\//;
                var rxIsHtm = /\.(html?)$/i;

                //set no href if the url points to an asset, if it is of htm(l) extension
                if( !rxA.test(url) || !rxIsHtm.test(url) ) {
                  domAttr.set(this.urlNode, "href", url || "#");
                }
            },

            _setPhrasesAttr: function(phrases) {
                domClass.toggle(this.phrasesNode, "epi-hidden", !phrases);
                this.phrasesNode.innerHTML = phrases ? array.map(phrases.split(","), function(phrase) {
                    return "<span class=\"epi-tag epi-lozenge\">" + phrase + "</span>";
                }).join("") : "";
            },

            postCreate: function(){
              var url = this.urlNode.innerText;
              var rxA = /\/(global|content|site)assets\//;
              var rxIsHtm = /\.(html?)$/i;

              //attach onclick if url points to an asset, with htm(l) extension
              if( rxA.test(url) && rxIsHtm.test(url) ) {
                on(this.urlNode, "click", function(event) {
                  var w = window.open();
                  var b = w.document.body;
                  var xhrArgs = {
                    url: url,
                    handleAs: "text",
                    load: function(data) {
                      b.innerHTML = data;
                    },
                    error: function(error) {
                      b.innerHTML = "An unexpected error occurred: " + error;
                    }
                  };
                  var deferred = dojo.xhrGet(xhrArgs);
                });
              }
          }
        });
    });

},
'url:epi-find/widget/templates/BestbetGridRow.html':"<div class=\"epi-bestbet-grid-row\">\r\n    <h3 data-dojo-attach-point=\"titleNode\"></h3>\r\n    <span data-dojo-attach-point=\"descriptionNode\"></span>\r\n    <p><a data-dojo-attach-point=\"urlNode\" class=\"epi-visibleLink\"></a></p>\r\n    <div class=\"epi-vertical-spacer\" data-dojo-attach-point=\"phrasesNode\" title=\"${i18n.fields_bestbetPhrase}\"></div>\r\n</div>",
'url:epi-find/widget/templates/Bestbets.html':"<div>\r\n    <div class=\"epi-vertical-spacer--double\">\r\n        <p class=\"epi-view--description\">${i18n.description}</p>\r\n        <form data-dojo-attach-point=\"bestbetForm\"></form>\r\n        <p class=\"epi-help\">${i18n.help_table}</p>\r\n    </div>\r\n\r\n    <div class=\"epi-grid-title-search\" data-dojo-attach-point=\"bestBetsHeader\">\r\n        <h1>${i18n.header}</h1>\r\n        <div class=\"epi-search-box\">\r\n            <input type=\"text\" name=\"query\"\r\n                   class=\"epi-search-input\"\r\n                   data-dojo-type=\"dijit/form/TextBox\"\r\n                   data-dojo-props=\"trim: true, intermediateChanges: true\"\r\n                   data-dojo-attach-point=\"searchNode\" placeholder=\"${i18n.search}\" />\r\n        </div>\r\n    </div>\r\n    <div id=\"bestbetGrid\" data-dojo-attach-point=\"bestbetGrid\"></div>\r\n</div>",
'epi-find/widget/RelatedQueries':function(){
define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/when",
    "dojo/on",

    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/_FocusMixin",
    "dijit/form/TextBox",

    "dojox/mvc/at",

    "epi-saas-base/Sticker",
    "epi-saas-base/StickyContainer",

    "./_SiblingFocusMixin",
    "./_ActionableMixin",
    "./_DisabledViewMixin",
    "./RelatedQueriesForm",
    "./RelatedQueriesGrid",

    "dojo/i18n!./nls/RelatedQueries",
    "dojo/text!./templates/RelatedQueries.html"

],
    function (
        declare,
        lang,
        when,
        on,
        _WidgetBase,
        _TemplatedMixin,
        _WidgetsInTemplateMixin,
        _FocusMixin,
        TextBox,
        at,
        Sticker,
        StickyContainer,
        _SiblingFocusMixin,
        _ActionableMixin,
        _DisabledViewMixin,
        RelatedQueriesForm,
        RelatedQueriesGrid,
        i18n,
        template) {

        // module:
        //      epi-find/widget/RelatedQueries

        return declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, _FocusMixin,
                       _SiblingFocusMixin, _ActionableMixin, _DisabledViewMixin], {
            // summary:
            //     Related Queries view.

            templateString: template,

            i18n: i18n,

            model: null,

            _form: null,
            _grid: null,

            _searchText: null,

            _searchUpdateTimeout: 400,
            _searchUpdateTimer: null,

            takeAction: function (actions, params) {
                if (params && params.phrases) {
                    this._form.reset();
                    this.model.ctrl.set("query", decodeURIComponent(params.phrases));
                }
                this._grid.refresh();
            },

            postCreate: function () {
                this.inherited(arguments);
                var self = this;

                var form = this._form = new RelatedQueriesForm({
                    model: this.model,
                    afterSubmit: function() {
                        self._grid.refresh();
                    }
                }, this.relatedQueriesForm);
                this.own(this._form);

                this._grid = new RelatedQueriesGrid({
                    "class": "epi-plain-grid epi-plain-grid--no-border",
                    showHeader: true,
                    store: this.model.store,
                    query: this.model.gridQuery,
                    model: this.model,
                    loadingMessage: i18n.dataLoading,
                    showLoadingMessage: true,
                    onEdit: function (args) {
                        form.edit(args.id, args.suggestion);
                    }
                }, this.relatedQueriesGrid);

                this.own(
                    this._grid,
                    this.model.watch("gridQuery", function(name, oldValue, newValue) {
                        self._grid.set("query", newValue);
                    }),
                    this.searchNode.watch("value", function(name, oldValue, newValue) {
                        self._searchText = newValue;
                        self._delayedReload();
                    })
                );
            },

            _delayedReload: function() {
                //  summary:
                //      Updates grids query with text from the search textbox with delay.
                if (this._searchUpdateTimer) {
                    clearTimeout(this._searchUpdateTimer);
                }
                this._searchUpdateTimer = setTimeout(lang.hitch(this, function () {
                    this._searchUpdateTimer = null;
                    this.model.set("searchText", this._searchText ? "(@ |\"\")" + this._searchText.toLowerCase() + "." : "");
                }), this._searchUpdateTimeout);
            },

            startup: function () {
                this.inherited(arguments);
                var scrollNode = this.scrollNode || this.domNode.parentNode;
                this._form.startup();
                this._grid.set("scrollNode", scrollNode);
                this._grid.startup();

                this.own(
                    new StickyContainer({
                        scrollNode: scrollNode,
                        stickers: [
                            new Sticker({ domNode: this.relatedQueriesHeader, offset: {top: 0}, limitNode: this._grid.domNode, subscribe: false}),
                            new Sticker({ domNode: this._grid.headerNode, offset: {top: 0}, limitNode: this._grid.domNode, subscribe: false})
                        ]
                    })
                );
            },

            refresh: function () {
                this._grid.refresh();
                this._form.reset();
            }
        });
    });

},
'epi-find/widget/RelatedQueriesForm':function(){
﻿define([
    "dojo/_base/declare",
    "dojo/_base/config",
    "dojo/_base/event",
    "dojo/_base/lang",
    "dojo/topic",
    "dojo/when",

    "dojox/mvc/at",
    "dijit/form/Form",
    "dijit/form/_TextBoxMixin",
    "dijit/_WidgetsInTemplateMixin",
    // widgets used in the template
    "dijit/form/Button",
    "dijit/form/ValidationTextBox",
    "./PhraseValidationTextBox",
    "epi-saas-base/form/FormListItem",
    "epi-saas-base/widgets/_NotificationMixin",
    "epi-saas-base/UnicodeLetterRegexp",
    "epi-saas-base/widgets/_ActionButtonsMixin",

    "epi/shell/widget/dialog/Dialog",
    "dojo/text!./templates/RelatedQueriesForm.html",
    "dojo/i18n!./nls/RelatedQueries"

],
    function(declare,
             config,
             event,
             lang,
             topic,
             when,

             at,

             Form,
             _TextBoxMixin,
             _WidgetsInTemplateMixin,
             Button,
             ValidationTextBox,
             PhraseValidationTextBox,
             FormListItem,
             _NotificationMixin,
             UnicodeLetterRegExp,
             _ActionButtonsMixin,

             Dialog,
             template,
             i18n
        ) {
        // module:
        //      epi-find/widget/RelatedQueriesForm
        return declare([Form, _WidgetsInTemplateMixin, _NotificationMixin, _ActionButtonsMixin], {

            baseClass: "",
            model: null,
            i18n: i18n,
            templateString: template,

            i18nPhraseHelp: i18n.help_phrase,
            i18nSuggestionHelp: i18n.help_suggestions,

            // phraseValidationRegExp: string
            //      A string with regular expression defining a phrase in unicode
            phraseValidationRegExp: "[^~`!@%^*()\\[\\]\\{\\}:;'|/<>?_-]("+UnicodeLetterRegExp+"|[0-9&'+#$_.\\-\\s])*",

            // suggestionValidationRegExp: string
            //      A string with regular expression defining a comma-separated list of phrases in unicode
            suggestionValidationRegExp: "[^~`!@%^*()\\[\\]\\{\\}:;'|/<>?_-]("+UnicodeLetterRegExp+"|[0-9&'+#$_.\\-\\s,])*",

            phraseField: null,
            suggestionsField: null,

            postCreate: function() {
                this.inherited(arguments);
                this.phraseListItem.set("label", i18n.fields_phrase);
                this.phraseListItem.set("help", i18n.help_phrase);
                this.suggestionsListItem.set("label", i18n.fields_suggestions);
                this.suggestionsListItem.set("description", i18n.separateMultipleSuggestions);
                this.suggestionsListItem.set("help", i18n.help_suggestions);
                this.submitButton.set("label", i18n.buttons_add);
                this.resetButton.set("label", i18n.buttons_cancel);
            },

            startup: function () {
                this.inherited(arguments);

                var self = this;

                this.actionButtons = [this.submitButton, this.resetButton];
                this.enabledWhenNoChangesButtons = [this.resetButton];

                this.phraseField.set("value", at(this.model.ctrl, "query"));
                this.phraseField.set("pattern", this.phraseValidationRegExp);
                this.suggestionsField.set("value", at(this.model.ctrl, "suggestion"));
                this.suggestionsField.set("pattern", this.suggestionValidationRegExp);
                this.reset();


                this.own(this.watchDataChanges(this.model.ctrl));
            },

            onSubmit: function (e) {
                var self = this;
                event.stop(e);
                if (!this.validate()) {
                    return ;
                }
                this.submitButton.focus();
                this.indicateInProgress(this.submitButton);
                when(this.model.saveChanges(),
                    function (results) {
                        self.reset();
                        self.showSuccess(i18n.savedMessage);
                        topic.publish(config.intents.view.relatedQueries);
                        self.afterSubmit({items: results});
                        self.indicateSuccess();
                        self.phraseField.focus();
                    },
                    function (error) {
                        self.showError(error);
                        self.indicateError();
                        self.afterSubmit({ items: [] });
                    }
                );
            },

            onReset: function() {
                this.model.ctrl.reset();
                this.model.ctrl.empty();
                this.submitButton.set("label", i18n.buttons_add);
                return this.inherited(arguments);
            },

            edit: function(id, suggestion) {
                if (this.editMode) {
                    this.reset();
                }

                when(this.model.ctrl.getStore(id),
                    lang.hitch(this, function() {
                        this.suggestionsField.focus();
                        var selectionStart = this.suggestionsField.value.indexOf(suggestion);
                        _TextBoxMixin.selectInputText(this.suggestionsField.textbox, selectionStart, selectionStart + suggestion.length);
                        this.submitButton.set("label", i18n.buttons_save);
                        this.set("hasChanges", false);
                    }),
                    lang.hitch(this, function(error) {
                        this.showError(error);
                    })
                );

            },

            submit: function() {
                // summary:
                //      pragmatically submit form if and only if the `onSubmit` returns true
                if (this.onSubmit() === false) {
                    return;
                }
                this.originalContainerNode.submit();
            },

            afterSubmit: function(/*Object*/ e) {
                // summary:
                //      Refresh callback
                // e: Object
                //      Callback arguments, e.items contains an array of submitted entities
            }

        });

    });

},
'url:epi-find/widget/templates/RelatedQueriesForm.html':"<form data-dojo-attach-point='containerNode' data-dojo-attach-event='onreset:_onReset,onsubmit:_onSubmit' class='epi-formsWidgetWrapper'>\r\n    <ol class='epi-formsWidgetWrapper'>\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\" data-dojo-attach-point=\"phraseListItem\">\r\n            <input id=\"phrase\" data-dojo-attach-point=\"phraseField\" data-dojo-type=\"epi-find/widget/PhraseValidationTextBox\" data-dojo-props=\"required:true, intermediateChanges: true, invalidMessage: '${i18nPhraseHelp}'\"/>\r\n        </li>\r\n\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\" data-dojo-attach-point=\"suggestionsListItem\">\r\n            <input id=\"suggestions\" data-dojo-attach-point=\"suggestionsField\" data-dojo-type=\"epi-find/widget/PhraseValidationTextBox\" data-dojo-props=\"required:true, intermediateChanges: true, phraseDelimiter:',', invalidMessage: '${i18nSuggestionHelp}'\" />\r\n        </li>\r\n\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\" class=\"epi-formsRow--align-right\">\r\n            <input data-dojo-attach-point=\"submitButton\" type=\"submit\" data-dojo-type=\"dijit/form/Button\" class=\"epi-primary\" />\r\n            <input data-dojo-attach-point=\"resetButton\" type=\"reset\" data-dojo-type=\"dijit/form/Button\" />\r\n        </li>\r\n    </ol>\r\n</form>\r\n",
'epi-find/widget/RelatedQueriesGrid':function(){
define([
    "dojo/_base/declare",
    "dojo/_base/array",
    "dojo/_base/lang",
    "dojo/when",
    "dojo/promise/all",

    "dijit/MenuItem",
    "dijit/MenuSeparator",

    "dgrid/Grid",
    "dgrid/extensions/DijitRegistry",

    "epi/shell/widget/ContextMenu",
    "epi/shell/dgrid/Formatter",
    "epi-cms/dgrid/formatters",

    "epi-saas-base/widgets/_GridMoreMixin",
    "epi-saas-base/widgets/_AutoLoadScrollMixin",
    "epi-saas-base/widgets/_NotificationMixin",
    "epi-saas-base/widgets/_GridContextMenuMixin",

    "dojo/i18n!./nls/RelatedQueries"
],
    function(declare,
             array,
             lang,
             when,
             all,
             MenuItem,
             MenuSeparator,
             Grid,
             DijitRegistry,
             ContextMenu,
             Formatter,
             formatters,
             _GridMoreMixin,
             _AutoLoadScrollMixin,
             _NotificationMixin,
             _GridContextMenuMixin,
             i18n
        ) {
        // module:
        //      epi-find/widget/RelatedQueriesGrid
        return declare([Grid, _GridMoreMixin, _AutoLoadScrollMixin, Formatter, _GridContextMenuMixin, _NotificationMixin], {
            // summary:
            //     Related Queries grid.
            constructor: function() {
                // Sets the loader (#loadNextPage, available in _GridMoreMixn) to be used by the _AutoLoadScrollMixin
                this.loader = lang.hitch(this, this.loadNextPage);
            },
            i18n: i18n,
            margin: 140,
            noDataMessage: i18n.noData,
            keepScrollPosition: true,
            columns: {
                query: {
                    label: i18n.fields_phrase,
                    sortable: true,
                    className:"epi-grid--30",
                    renderCell: function(object, value, node) {
                        // render "query" value only for the first row, so that suggestions
                        // that belong to the same query will look somewhat grouped
                        node.innerHTML = this.grid.model.canMoveUp(object) ? "" : value;
                    }
                },
                suggestion: {
                    label: i18n.fields_suggestions,
                    sortable: false
                },

                context: {
                    label: " ",
                    sortable: false,
                    className: "epi-grid--10  epi-inner-related-queries-grid",
                    formatter: function() {
                        return formatters.menu({title: i18n.contextMenuTitle});
                    }
                }
            },

            postCreate: function() {
                var self = this, menuSeparator1 = new MenuSeparator(), menuSeparator2 = new MenuSeparator(),
                    getCurrentItem = function(context) {
                        var row = self.getRow(context);
                        return encodeURIComponent(row.id);
                    },
                    // This sorting should be in sync with the backend other_suggestions sorting.
                    defaultSort = [
                        {attribute: "priority", descending: true},
                        {attribute: "timestamp", descending: true}
                    ];

                this.on("dgrid-sort", function(event) {
                    event.preventDefault();
                    event.stopPropagation();
                    self.set("sort", event.sort.concat(defaultSort));
                });
                this._sort = [{attribute:"query", descending: false}].concat(defaultSort);

                this.inherited(arguments);

                this.editMenuItem = new MenuItem({
                    label: i18n.actions_edit,
                    onClick: function () {
                        var id = getCurrentItem(this);
                        self.onEdit({grid: self, id: encodeURIComponent(id), suggestion: self.row(id).data.suggestion});
                    }
                });
                this.moveUpMenuItem = new MenuItem({
                    label: i18n.actions_moveUp,
                    onClick: function () {
                        var id = getCurrentItem(this);
                        when(self.model.moveUp(self.row(id).data), function() {
                                self.showSuccess(i18n.savedMessage);
                                self.refresh();
                            },
                            function(error) {
                                self.showError(error);
                                self.refresh();
                            }
                        );
                    }
                });

                this.moveDownMenuItem = new MenuItem({
                    label: i18n.actions_moveDown,
                    onClick: function () {
                        var id = getCurrentItem(this);
                        when(self.model.moveDown(self.row(id).data), function() {
                                self.showSuccess(i18n.savedMessage);
                                self.refresh();
                            },
                            function(error) {
                                self.showError(error);
                                self.refresh();
                            }
                        );
                    }
                });

                this.deleteMenuItem = new MenuItem({
                    label: i18n.actions_deleteSuggestion,
                    onClick: function () {
                        var id = getCurrentItem(this);
                        self.showConfirmation({
                            title: i18n.deleteSuggestionTitle,
                            message: i18n.areYouSureSuggestion,
                            confirmActionText: i18n.buttons_confirmDelete,
                            cancelActionText: i18n.buttons_cancelDelete,
                            action: function() {
                                when(self.model.remove(id), function(){
                                        self.showSuccess(i18n.suggestionDeletedMessage);
                                        self.refresh();
                                    },
                                    function(error){
                                        self.showError(error);
                                        self.refresh();
                                    }
                                );
                            }
                        });
                    }
                });

                this.deleteAllMenuItem = new MenuItem({
                    label: i18n.actions_deleteAll,
                    onClick: function () {
                        var id = getCurrentItem(this);
                        self.showConfirmation({
                            title: i18n.deleteTitle,
                            message: i18n.areYouSurePhrase,
                            confirmActionText: i18n.buttons_confirmDelete,
                            cancelActionText: i18n.buttons_cancelDelete,
                            action: function() {
                                var removals = array.map(self.row(id).data.other_suggestions, function(item) {
                                    return self.model.remove(item);
                                });
                                all(removals).then(function(){
                                        self.showSuccess(i18n.phraseDeletedMessage);
                                        self.refresh();
                                    },
                                    function(error) {
                                        self.showError(error);
                                        self.refresh();
                                    }
                                );
                            }
                        });
                    }
                });

                this.contextMenu = new ContextMenu({
                    onOpen: function() {
                        var data = self.row(this.currentTarget).data;
                        self.moveUpMenuItem.set("disabled", !self.model.canMoveUp(data));
                        self.moveDownMenuItem.set("disabled", !self.model.canMoveDown(data));

                        // Show deleteAllMenuItem only for the first suggestion
                        // and if there are other suggestions
                        if (!self.model.canMoveUp(data) && data.other_suggestions && data.other_suggestions.length > 1) {
                            if (self.contextMenu.getIndexOfChild(self.deleteAllMenuItem) === -1) {
                                // place deleteAllMenuItem before deleteMenuItem
                                self.contextMenu.addChild(self.deleteAllMenuItem,
                                    self.contextMenu.getIndexOfChild(self.deleteMenuItem));
                            }
                        }
                        else {
                            self.contextMenu.removeChild(self.deleteAllMenuItem);
                        }
                    }
                });

                // Creating menu structure
                this.contextMenu.addChild(this.editMenuItem);
                this.contextMenu.addChild(menuSeparator1);
                this.contextMenu.addChild(this.moveUpMenuItem);
                this.contextMenu.addChild(this.moveDownMenuItem);
                this.contextMenu.addChild(menuSeparator2);
                this.contextMenu.addChild(this.deleteMenuItem);

                this.own( this.editMenuItem,
                    menuSeparator1,
                    this.moveUpMenuItem,
                    this.moveDownMenuItem,
                    menuSeparator2,
                    this.deleteMenuItem,
                    this.deleteAllMenuItem,
                    this.on("dgrid-error", function(event) {
                        self.showErrorInGrid(event.error);
                    })
                );

            },

            onEdit: function(args) {
                // summary:
                //      Callback for edit action
                // args: Object
                //      Callback arguments: args.grid - reference to the grid, args.id - selected item ID
            }
        });
    });
},
'url:epi-find/widget/templates/RelatedQueries.html':"<div>\r\n    <p class=\"epi-view--description\">${i18n.description}</p>\r\n    <div class=\"error\" data-dojo-attach-point=\"errorNode\"></div>\r\n    <form data-dojo-attach-point=\"relatedQueriesForm\"></form>\r\n    <p class=\"epi-help\">${i18n.help_grid}</p>\r\n    <div class=\"epi-grid-title-search\" data-dojo-attach-point=\"relatedQueriesHeader\">\r\n        <h1>${i18n.header}</h1>\r\n        <div class=\"epi-search-box\">\r\n            <input type=\"text\" name=\"query\"\r\n                   class=\"epi-search-input\"\r\n                   data-dojo-type=\"dijit/form/TextBox\"\r\n                   data-dojo-props=\"trim: true, intermediateChanges: true\"\r\n                   data-dojo-attach-point=\"searchNode\" placeholder=\"${i18n.search}\" />\r\n        </div>\r\n    </div>\r\n    <div data-dojo-attach-point=\"relatedQueriesGrid\" class=\"epi-related-query-grid epi-plain-grid  epi-plain-grid--no-border  epi-grid-max-height--700\"></div>\r\n</div>",
'epi-find/widget/Autocomplete':function(){
define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/when",

    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/_FocusMixin",
    "dijit/form/TextBox",

    "epi-saas-base/Sticker",
    "epi-saas-base/StickyContainer",

    "./_SiblingFocusMixin",
    "./_ActionableMixin",
    "./_DisabledViewMixin",
    "./AutocompleteForm",
    "./AutocompleteGrid",

    "dojo/i18n!./nls/Autocomplete",
    "dojo/text!./templates/Autocomplete.html"

],
    function (
        declare,
        lang,
        when,

        _WidgetBase,
        _TemplatedMixin,
        _WidgetsInTemplateMixin,
        _FocusMixin,
        TextBox,

        Sticker,
        StickyContainer,

        _SiblingFocusMixin,
        _ActionableMixin,
        _DisabledViewMixin,
        AutocompleteForm,
        AutocompleteGrid,
        i18n,
        template) {

        // module:
        //      epi-find/widget/Autocomplete

        return declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, _FocusMixin,
                       _SiblingFocusMixin, _ActionableMixin, _DisabledViewMixin], {
            // summary:
            //     Autocomplete view.

            templateString: template,

            i18n: i18n,

            model: null,

            _form: null,
            _grid: null,

            _searchText: null,

            _searchUpdateTimeout: 400,
            _searchUpdateTimer: null,

            takeAction: function (actions, params) {
                if (params && params.phrases) {
                    this._form.reset();
                    this.model.ctrl.set("query", decodeURIComponent(params.phrases));
                }
                this._grid.refresh();
            },

            postCreate: function () {
                this.inherited(arguments);
                var self = this;

                var form = this._form = new AutocompleteForm({
                    model: this.model,
                    afterSubmit: function(e) {
                        when(self._grid.refresh(), function() {
                            if (e && e.item) {
                                self._grid.select(e.item.id);
                            }
                        });
                    }
                }, this.autocompleteForm);
                this.own(this._form);

                this._grid = new AutocompleteGrid({
                    "class": "epi-plain-grid epi-plain-grid--no-border",
                    store: this.model.store,
                    query: this.model.gridQuery,
                    model: this.model,
                    loadingMessage: i18n.dataLoading,
                    showLoadingMessage: true,
                    onEdit: function (args) {
                        form.edit(args.id);
                    }
                }, this.autocompleteGrid);

                this.own(
                    this._grid,
                    this.model.watch("gridQuery", function(name, oldValue, newValue) {
                        self._grid.set("query", newValue);
                    }),
                    this.searchNode.watch("value", function(name, oldValue, newValue) {
                        self._searchText = newValue;
                        self._delayedReload();
                    })
                );
            },

            _delayedReload: function() {
                //  summary:
                //      Updates grids query with text from the search textbox with delay.
                if (this._searchUpdateTimer) {
                    clearTimeout(this._searchUpdateTimer);
                }
                this._searchUpdateTimer = setTimeout(lang.hitch(this, function () {
                    this._searchUpdateTimer = null;
                    this.model.set("searchText", this._searchText ? "(@ |\"\")" + this._searchText.toLowerCase() + "." : "");
                }), this._searchUpdateTimeout);
            },

            startup: function () {
                this.inherited(arguments);
                var scrollNode = this.scrollNode || this.domNode.parentNode;
                this._form.startup();
                this._grid.set("scrollNode", scrollNode);
                this._grid.startup();

                this.own(
                    new StickyContainer({
                        scrollNode: scrollNode,
                        stickers: [
                            new Sticker({ domNode: this.autocompleteHeader, offset: {top: 0}, limitNode: this._grid.domNode, subscribe: false}),
                            new Sticker({ domNode: this._grid.headerNode, offset: {top: 0}, limitNode: this._grid.domNode, subscribe: false})
                        ]
                    })
                );
            },

            refresh: function () {
                this._grid.refresh();
                this._form.reset();
            }
        });
    });

},
'epi-find/widget/AutocompleteForm':function(){
define([
    "dojo/_base/declare",
    "dojo/_base/config",
    "dojo/_base/event",
    "dojo/_base/lang",
    "dojo/topic",
    "dojo/when",

    "dojox/mvc/at",
    "dijit/form/Form",
    "dijit/_WidgetsInTemplateMixin",
    // widgets used in the template
    "dijit/form/Button",
    "dijit/form/ValidationTextBox",

    "epi-find/widget/PhraseValidationTextBox",
    "epi-saas-base/UnicodeLetterRegexp",
    "epi-saas-base/form/FormListItem",
    "epi-saas-base/widgets/_ActionButtonsMixin",
    "epi-saas-base/widgets/_NotificationMixin",

    "dojo/text!./templates/AutocompleteForm.html",
    "dojo/i18n!./nls/Autocomplete"

],
    function(declare,
             config,
             event,
             lang,
             topic,
             when,

             at,

             Form,
             _WidgetsInTemplateMixin,
             Button,
             ValidationTextBox,
             PhraseValidationTextBox,
             UnicodeLetterRegExp,
             FormListItem,
             _ActionButtonsMixin,
             _NotificationMixin,

             template,
             i18n
        ) {
        // module:
        //      epi-find/widget/AutocompleteForm
        return declare([Form, _WidgetsInTemplateMixin, _NotificationMixin, _ActionButtonsMixin], {


            baseClass: "",
            model: null,
            i18n: i18n,
            templateString: template,

            i18nPhraseHelp: i18n.help_phrase,

            postCreate: function() {
                this.inherited(arguments);
                this.autocompletePhraseListItem.set("label", i18n.fields_phrase);
                this.autocompletePhraseListItem.set("help", i18n.help_phrase);
                this.resetButton.set("label", i18n.buttons_cancel);
            },

            startup:function() {
                this.inherited(arguments);

                this.actionButtons = [this.submitButton, this.resetButton];
                this.enabledWhenNoChangesButtons = [this.resetButton];

                this.phraseField.set("value", at(this.model.ctrl, "query"));
                this.phraseField.set("regExp", "[^~`!@%^*()\\[\\]\\{\\}:;'|/<>?_-](?:"+UnicodeLetterRegExp+"|[0-9&'+#$_.\\-\\s])*");

                this.submitButton.set("label", i18n.buttons_add);
                this.reset();
                this.own(this.watchDataChanges(this.model.ctrl));

            },

            onSubmit: function (e) {
                var self = this;
                event.stop(e);
                if (!this.validate()) {
                    return ;
                }
                this.submitButton.focus();
                this.indicateInProgress(this.submitButton);
                when(this.model.save(),
                    function (result) {
                        self.reset();
                        self.showSuccess(i18n.savedMessage);
                        topic.publish(config.intents.view.autocomplete);
                        self.afterSubmit({item: result});
                        self.indicateSuccess();
                        self.phraseField.focus();
                    },
                    function (error) {
                        self.showError(error);
                        self.indicateError();
                    }
                );
            },

            onReset: function() {
                this.model.ctrl.reset();
                this.model.ctrl.empty();
                this.submitButton.set("label", i18n.buttons_add);
                return this.inherited(arguments);
            },

            edit: function(id) {
                if (this.editMode) {
                    this.reset();
                }

                when(this.model.ctrl.getStore(id),
                    lang.hitch(this, function() {
                        this.phraseField.focus();
                        this.submitButton.set("label", i18n.buttons_save);
                        this.set("hasChanges", false);
                    }),
                    lang.hitch(this, function(error) {
                        this.showError(error);
                    })
                );

            },

            submit: function() {
                // summary:
                //      pragmatically submit form if and only if the `onSubmit` returns true
                if (this.onSubmit() === false) {
                    return;
                }
                this.originalContainerNode.submit();
            },

            afterSubmit: function(/*Object*/ e) {
                // summary:
                //      Refresh callback
                // e: Object
                //      Callback arguments, e.items contains an array of submitted entities
            }

        });

    });

},
'url:epi-find/widget/templates/AutocompleteForm.html':"<form data-dojo-attach-point='containerNode' data-dojo-attach-event='onreset:_onReset,onsubmit:_onSubmit' class='epi-formsWidgetWrapper'>\r\n    <ol class='epi-formsWidgetWrapper'>\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\" data-dojo-attach-point=\"autocompletePhraseListItem\">\r\n            <input id=\"autocompletePhrase\" data-dojo-attach-point=\"phraseField\" data-dojo-type=\"epi-find/widget/PhraseValidationTextBox\" data-dojo-props=\"required:true, intermediateChanges: true, invalidMessage: '${i18nPhraseHelp}'\"/>\r\n        </li>\r\n\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\" class=\"epi-formsRow--align-right\">\r\n            <input data-dojo-attach-point=\"submitButton\" type=\"submit\" data-dojo-type=\"dijit/form/Button\" class=\"epi-primary\" />\r\n            <input data-dojo-attach-point=\"resetButton\" type=\"reset\" data-dojo-type=\"dijit/form/Button\" />\r\n        </li>\r\n    </ol>\r\n</form>\r\n",
'epi-find/widget/AutocompleteGrid':function(){
define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/Evented",
    "dojo/on",
    "dojo/when",
    "dojo/dom-style",

    "dijit/MenuItem",

    "dgrid/Grid",
    "dgrid/extensions/DnD",
    "dgrid/extensions/DijitRegistry",

    "epi/shell/widget/ContextMenu",
    "epi/shell/dgrid/Formatter",
    "epi-cms/dgrid/formatters",

    "epi-saas-base/widgets/_NotificationMixin",
    "epi-saas-base/widgets/_GridContextMenuMixin",
    "epi-saas-base/widgets/_GridMoreMixin",
    "epi-saas-base/widgets/_AutoLoadScrollMixin",

    "dojo/i18n!./nls/Autocomplete"
],
    function(
        declare,
        lang,
        Evented,
        on,
        when,
        domStyle,
        MenuItem,
        Grid,
        DnD,
        DijitRegistry,
        ContextMenu,
        Formatter,
        formatters,
        _NotificationMixin,
        _GridContextMenuMixin,
        _GridMoreMixin,
        _AutoLoadScrollMixin,
        i18n
        ) {
        // module:
        //      epi-find/widget/AutocompleteGrid

       return declare([Grid, _GridMoreMixin, _AutoLoadScrollMixin, DnD, Formatter, _GridContextMenuMixin, _NotificationMixin], {
            // summary:
            //    Autocomplete grid.
            constructor: function() {
                // Sets the loader (#loadNextPage, available in _GridMoreMixn) to be used by the _AutoLoadScrollMixin
                this.loader = lang.hitch(this, this.loadNextPage);
            },
            i18n: i18n,
            showHeader: true,
            margin: 140,
            noDataMessage: i18n.noData,
            keepScrollPosition: true,
            columns: {
                query: {
                    label: i18n.fields_phrase,
                    sortable: false,
                    className:"epi-grid--30"
                },

                context: {
                    label: " ",
                    sortable: false,
                    className: "epi-grid--10",
                    formatter: function() {
                        return formatters.menu({title: i18n.contextMenuTitle});
                    }
                }
            },

            postCreate: function() {
                var self = this,
                    getCurrentItem = function(context) {
                        var row = self.getRow(context);
                        return encodeURIComponent(row.id);
                    };

                this.inherited(arguments);
                this._sort = [
                    {attribute:"priority", descending: true},
                    {attribute:"timestamp", descending: true}
                ];

                this.editMenuItem = new MenuItem({
                    label: i18n.actions_edit,
                    onClick: function () {
                        var id = getCurrentItem(this);
                        self.onEdit({grid: self, id: encodeURIComponent(id)});
                    }
                });

                this.deleteMenuItem = new MenuItem({
                    label: i18n.actions_delete,
                    onClick: function () {
                        var id = getCurrentItem(this);
                        self.showConfirmation({
                            title: i18n.deleteTitle,
                            message: i18n.areYouSureDelete,
                            confirmActionText: i18n.buttons_confirmDelete,
                            cancelActionText: i18n.buttons_cancelDelete,
                            action: function() {
                                when(self.model.remove(id), function(){
                                        self.showSuccess(i18n.deletedMessage);
                                        self.refresh();
                                    },
                                    function(error){
                                        self.showError(error);
                                        self.refresh();
                                    }
                                );
                            }
                        });
                    }
                });

                this.contextMenu = new ContextMenu({});

                // Creating menu structure
                this.contextMenu.addChild(this.editMenuItem);
                this.contextMenu.addChild(this.deleteMenuItem);

                this.own(
                    this.editMenuItem,
                    this.deleteMenuItem,
                    this.on("dgrid-error", function(event) {
                        self.showErrorInGrid(event.error);
                    })
                );
                if (this.store.isInstanceOf(Evented)) {
                    this.own(
                        on(this.store, "refresh", function() {
                            self.refresh();
                        }),
                        on(this.store, "dndComplete", function() {
                            self.showSuccess(i18n.priorityChangedMessage);
                        }),
                        on(this.store, "dndError", function(error) {
                            self.showError(error);
                        })
                    );
                }

            },

            onEdit: function(args) {
                // summary:
                //      Callback for edit action
                // args: Object
                //      Callback arguments: args.grid - reference to the grid, args.id - selected item ID
            }
        });
    });

},
'dgrid/extensions/DnD':function(){
define("dgrid/extensions/DnD", [
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/_base/Deferred",
	"dojo/aspect",
	"dojo/on",
	"dojo/topic",
	"dojo/has",
	"dojo/dnd/Source",
	"dojo/dnd/Manager",
	"dojo/_base/NodeList",
	"put-selector/put",
	"../Selection",
	"dojo/has!touch?../util/touch",
	"dojo/has!touch?./_DnD-touch-autoscroll",
	"xstyle/css!dojo/resources/dnd.css"
], function(declare, lang, arrayUtil, Deferred, aspect, on, topic, has, DnDSource, DnDManager, NodeList, put, Selection, touchUtil){
	// Requirements
	// * requires a store (sounds obvious, but not all Lists/Grids have stores...)
	// * must support options.before in put calls
	//   (if undefined, put at end)
	// * should support copy
	//   (copy should also support options.before as above)
	
	// TODOs
	// * consider sending items rather than nodes to onDropExternal/Internal
	// * consider emitting store errors via OnDemandList._trackError
	
	var GridDnDSource = declare(DnDSource, {
		grid: null,
		
		getObject: function(node){
			// summary:
			//		getObject is a method which should be defined on any source intending
			//		on interfacing with dgrid DnD.
			
			var grid = this.grid;
			// Extract item id from row node id (gridID-row-*).
			return grid.store.get(node.id.slice(grid.id.length + 5));
		},
		_legalMouseDown: function(evt){
			// Fix _legalMouseDown to only allow starting drag from an item
			// (not from bodyNode outside contentNode).
			var legal = this.inherited(arguments);
			return legal && evt.target != this.grid.bodyNode;
		},

		// DnD method overrides
		onDrop: function(sourceSource, nodes, copy){
			var targetSource = this,
				targetRow = this._targetAnchor = this.targetAnchor, // save for Internal
				grid = this.grid,
				store = grid.store;
			
			if(!this.before && targetRow){
				// target before next node if dropped within bottom half of this node
				// (unless there's no node to target at all)
				targetRow = targetRow.nextSibling;
			}
			targetRow = targetRow && grid.row(targetRow);
			
			Deferred.when(targetRow && store.get(targetRow.id), function(target){
				// Note: if dropping after the last row, or into an empty grid,
				// target will be undefined.  Thus, it is important for store to place
				// item last in order if options.before is undefined.
				
				// Delegate to onDropInternal or onDropExternal for rest of logic.
				// These are passed the target item as an additional argument.
				if(targetSource != sourceSource){
					targetSource.onDropExternal(sourceSource, nodes, copy, target);
				}else{
					targetSource.onDropInternal(nodes, copy, target);
				}
			});
		},
		onDropInternal: function(nodes, copy, targetItem){
			var grid = this.grid,
				store = grid.store,
				targetSource = this,
				anchor = targetSource._targetAnchor,
				targetRow,
				nodeRow;
			
			if(anchor){ // (falsy if drop occurred in empty space after rows)
				targetRow = this.before ? anchor.previousSibling : anchor.nextSibling;
			}
			
			// Don't bother continuing if the drop is really not moving anything.
			// (Don't need to worry about edge first/last cases since dropping
			// directly on self doesn't fire onDrop, but we do have to worry about
			// dropping last node into empty space beyond rendered rows.)
			nodeRow = grid.row(nodes[0]);
			if(!copy && (targetRow === nodes[0] ||
					(!targetItem && nodeRow && grid.down(nodeRow).element == nodes[0]))){
				return;
			}
			
			nodes.forEach(function(node){
				Deferred.when(targetSource.getObject(node), function(object){
					var id = store.getIdentity(object);
					
					// For copy DnD operations, copy object, if supported by store;
					// otherwise settle for put anyway.
					// (put will relocate an existing item with the same id, i.e. move).
					store[copy && store.copy ? "copy" : "put"](object, {
						before: targetItem
					});
					
					// Self-drops won't cause the dgrid-select handler to re-fire,
					// so update the cached node manually
					if(targetSource._selectedNodes[id]){
						targetSource._selectedNodes[id] = grid.row(id).element;
					}
				});
			});
		},
		onDropExternal: function(sourceSource, nodes, copy, targetItem){
			// Note: this default implementation expects that two grids do not
			// share the same store.  There may be more ideal implementations in the
			// case of two grids using the same store (perhaps differentiated by
			// query), dragging to each other.
			var store = this.grid.store,
				sourceGrid = sourceSource.grid;
			
			// TODO: bail out if sourceSource.getObject isn't defined?
			nodes.forEach(function(node, i){
				Deferred.when(sourceSource.getObject(node), function(object){
					if(!copy){
						if(sourceGrid){
							// Remove original in the case of inter-grid move.
							// (Also ensure dnd source is cleaned up properly)
							Deferred.when(sourceGrid.store.getIdentity(object), function(id){
								!i && sourceSource.selectNone(); // deselect all, one time
								sourceSource.delItem(node.id);
								sourceGrid.store.remove(id);
							});
						}else{
							sourceSource.deleteSelectedNodes();
						}
					}
					// Copy object, if supported by store; otherwise settle for put
					// (put will relocate an existing item with the same id).
					// Note that we use store.copy if available even for non-copy dnd:
					// since this coming from another dnd source, always behave as if
					// it is a new store item if possible, rather than replacing existing.
					store[store.copy ? "copy" : "put"](object, {
						before: targetItem
					});
				});
			});
		},
		
		onDndStart: function(source, nodes, copy){
			// Listen for start events to apply style change to avatar.
			
			this.inherited(arguments); // DnDSource.prototype.onDndStart.apply(this, arguments);
			if(source == this){
				// If TouchScroll is in use, cancel any pending scroll operation.
				if(this.grid.cancelTouchScroll){ this.grid.cancelTouchScroll(); }
				
				// Set avatar width to half the grid's width.
				// Kind of a naive default, but prevents ridiculously wide avatars.
				DnDManager.manager().avatar.node.style.width =
					this.grid.domNode.offsetWidth / 2 + "px";
			}
		},
		
		onMouseDown: function(evt){
			// Cancel the drag operation on presence of more than one contact point.
			// (This check will evaluate to false under non-touch circumstances.)
			if(has("touch") && this.isDragging &&
					touchUtil.countCurrentTouches(evt, this.grid.touchNode) > 1){
				topic.publish("/dnd/cancel");
				DnDManager.manager().stopDrag();
			}else{
				this.inherited(arguments);
			}
		},
		
		onMouseMove: function(evt){
			// If we're handling touchmove, only respond to single-contact events.
			if(!has("touch") || touchUtil.countCurrentTouches(evt, this.grid.touchNode) <= 1){
				this.inherited(arguments);
			}
		},
		
		checkAcceptance: function(source, nodes){
			// Augment checkAcceptance to block drops from sources without getObject.
			return source.getObject &&
				DnDSource.prototype.checkAcceptance.apply(this, arguments);
		},
		getSelectedNodes: function(){
			// If dgrid's Selection mixin is in use, synchronize with it, using a
			// map of node references (updated on dgrid-[de]select events).
			
			if(!this.grid.selection){
				return this.inherited(arguments);
			}
			var t = new NodeList(),
				id;
			for(id in this.grid.selection){
				t.push(this._selectedNodes[id]);
			}
			return t;	// NodeList
		}
		// TODO: could potentially also implement copyState to jive with default
		// onDrop* implementations (checking whether store.copy is available);
		// not doing that just yet until we're sure about default impl.
	});
	
	// Mix in Selection for more resilient dnd handling, particularly when part
	// of the selection is scrolled out of view and unrendered (which we
	// handle below).
	var DnD = declare(Selection, {
		// dndSourceType: String
		//		Specifies the type which will be set for DnD items in the grid,
		//		as well as what will be accepted by it by default.
		dndSourceType: "dgrid-row",
		
		// dndParams: Object
		//		Object containing params to be passed to the DnD Source constructor.
		dndParams: null,
		
		// dndConstructor: Function
		//		Constructor from which to instantiate the DnD Source.
		//		Defaults to the GridSource constructor defined/exposed by this module.
		dndConstructor: GridDnDSource,
		
		postMixInProperties: function(){
			this.inherited(arguments);
			// ensure dndParams is initialized
			this.dndParams = lang.mixin({ accept: [this.dndSourceType] }, this.dndParams);
		},
		
		postCreate: function(){
			this.inherited(arguments);
			
			// Make the grid's content a DnD source/target.
			this.dndSource = new (this.dndConstructor || GridDnDSource)(
				this.bodyNode,
				lang.mixin(this.dndParams, {
					// add cross-reference to grid for potential use in inter-grid drop logic
					grid: this,
					dropParent: this.contentNode
				})
			);
			
			// Set up select/deselect handlers to maintain references, in case selected
			// rows are scrolled out of view and unrendered, but then dragged.
			var selectedNodes = this.dndSource._selectedNodes = {};
			
			function selectRow(row){
				selectedNodes[row.id] = row.element;
			}
			function deselectRow(row){
				delete selectedNodes[row.id];
				// Re-sync dojo/dnd UI classes based on deselection
				// (unfortunately there is no good programmatic hook for this)
				put(row.element, '!dojoDndItemSelected!dojoDndItemAnchor');
			}
			
			this.on("dgrid-select", function(event){
				arrayUtil.forEach(event.rows, selectRow);
			});
			this.on("dgrid-deselect", function(event){
				arrayUtil.forEach(event.rows, deselectRow);
			});
			
			aspect.after(this, "destroy", function(){
				delete this.dndSource._selectedNodes;
				selectedNodes = null;
				this.dndSource.destroy();
			}, true);
		},
		
		insertRow: function(object){
			// override to add dojoDndItem class to make the rows draggable
			var row = this.inherited(arguments),
				type = typeof this.getObjectDndType == "function" ?
					this.getObjectDndType(object) : [this.dndSourceType];
			
			put(row, ".dojoDndItem");
			this.dndSource.setItem(row.id, {
				data: object,
				type: type instanceof Array ? type : [type]
			});
			return row;
		},
		
		removeRow: function (rowElement) {
			this.dndSource.delItem(this.row(rowElement));
			this.inherited(arguments);
		}
	});
	DnD.GridSource = GridDnDSource;
	
	return DnD;
});

},
'url:epi-find/widget/templates/Autocomplete.html':"<div>\r\n    <p class=\"epi-view--description\">${i18n.description}</p>\r\n    <div class=\"error\" data-dojo-attach-point=\"errorNode\"></div>\r\n    <form data-dojo-attach-point=\"autocompleteForm\"></form>\r\n    <p class=\"epi-help\">${i18n.help_grid}</p>\r\n    <div class=\"epi-grid-title-search\" data-dojo-attach-point=\"autocompleteHeader\">\r\n        <h1>${i18n.header}</h1>\r\n        <div class=\"epi-search-box\">\r\n            <input type=\"text\" name=\"query\"\r\n                   class=\"epi-search-input\"\r\n                   data-dojo-type=\"dijit/form/TextBox\"\r\n                   data-dojo-props=\"trim: true, intermediateChanges: true\"\r\n                   data-dojo-attach-point=\"searchNode\" placeholder=\"${i18n.search}\" />\r\n        </div>\r\n    </div>\r\n    <div id=\"autocompleteGrid\" data-dojo-attach-point=\"autocompleteGrid\" class=\"epi-plain-grid  epi-plain-grid--no-border  epi-grid-max-height--700\"></div>\r\n</div>",
'url:epi-find/widget/templates/Optimization.html':"<div class=\"epi-pane  epi-pane--tools is-active epi-focused\">\r\n    <div class=\"epi-pane__content-wrapper\">\r\n        <div class=\"epi-pane__header\">\r\n            <h1>${i18n.header}</h1>\r\n            <p class=\"epi-pane__description\">${i18n.description}</p>\r\n        </div>\r\n        <div data-dojo-attach-point=\"contentNode\" class=\"epi-pane__content\">\r\n            <p class=\"epi-pane__description\">${i18n.description}</p>\r\n            <ul class=\"epi-pane__content__nav clearfix\" data-dojo-attach-point=\"links\">\r\n                <li><a href=\"#\" title=\"${i18n.nav_bestBets}\" class=\"epi-pane__content__header-link--best-bets\" data-dojo-attach-point=\"bestBetsLink\">${i18n.nav_bestBets}</a></li>\r\n                <li><a href=\"#\" title=\"${i18n.nav_relatedQueries}\" class=\"epi-pane__content__header-link--related-queries\" data-dojo-attach-point=\"relatedQueriesLink\">${i18n.nav_relatedQueries}</a></li>\r\n                <li><a href=\"#\" title=\"${i18n.nav_synonyms}\" class=\"epi-pane__content__header-link--synonyms\" data-dojo-attach-point=\"synonymsLink\">${i18n.nav_synonyms}</a></li>\r\n                <li><a href=\"#\" title=\"${i18n.nav_autocomplete}\" class=\"epi-pane__content__header-link--autocomplete\" data-dojo-attach-point=\"autocompleteLink\">${i18n.nav_autocomplete}</a></li>\r\n            </ul>\r\n            <div data-dojo-attach-point=\"containerNode\" class=\"epi-pane__content__content-container\"></div>\r\n        </div>\r\n    </div>\r\n</div>",
'epi-find/configure/ConfigureController':function(){
﻿define([
    "dojo/_base/declare",

    "../_ControllerBase",
    "./ConfigureModel",
    "../widget/Configure",
    "../widget/_ActionableMixin"
],
function(declare,
    _ControllerBase,
    ConfigureModel, Configure, _ActionableMixin
) {
    return declare([_ControllerBase, _ActionableMixin], {
        // summary:
        //      Controller for Manage views.

        model: null,

        widget: null,

        startup: function() {
            if (this.isStarted()) {
                return;
            }
            this.inherited(arguments);
            this.model = new ConfigureModel();
            this.model.init();

            this.widget = new Configure({
                model: this.model
            });

            this.own(this.widget);
        },

        show: function() {
            this._setupView([this.widget]);
        },

        takeAction: function(actions, params) {
            this.widget.takeAction(actions, params);
            this.currentAction = this.widget.currentAction;
            return this.widget;
        }
    });
});
},
'epi-find/configure/ConfigureModel':function(){
﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/Stateful",
    "dojo/_base/config",

    "dijit/Destroyable",

    "dojox/mvc/getPlainValue",

    "../ConfigModel",
    "./ConnectorModel",
    "./BoostingEditorModel",
    "./BoostingPreviewModel",
    "./IndexModel"
],
function (declare, 
          lang, 
          Stateful, 
          config, 
          Destroyable, 
          getPlainValue, 
          ConfigModel, 
          ConnectorModel,
          BoostingEditorModel, 
          BoostingPreviewModel,
          IndexModel) {

    return declare([Stateful, Destroyable], {
        // summary:
        //      Configure model.

        // connectorModel: epi-find/configure/ConnectorModel
        //      Model for the connectors view
        connectorModel: null,

        indexModel: null,

        boostingEditorModel: null,

        boostingPreviewModel: null,

        _boostingStore: null,
        _boostingPreviewStore: null,

        constructor: function () {
            this._boostingStore = config.dependencies["epi-find.BoostingStore"];
            this._boostingPreviewStore = config.dependencies["epi-find.BoostingPreviewStore"];
        },

        init: function(){
            //  summary:
            //      Sets the model properties to initial state.
            //      This method should be called when widgets are instantiated and subscribed on model updates,
            //      so that when this method is called dependent widgets will get initial values from the model.
            this.connectorModel = new ConnectorModel();

            var self = this;

            this.boostingEditorModel = new BoostingEditorModel({
                store: this._boostingStore,
                invalidateCacheUrl: config.find.siteServiceApiBaseUrl + "editorialboosting/invalidatecache/"
            });

            this.boostingPreviewModel = new BoostingPreviewModel({
                store: this._boostingPreviewStore,
                languages: ConfigModel.config.languages
            });

            this.boostingEditorModel.weightsModel.watch(function() {
                self.boostingPreviewModel.set("weights", getPlainValue(this.model));
            });

            this.indexModel = new IndexModel({indexingPluginUrl: config.find.indexingPluginUrl});
        }
    });
});

},
'epi-find/configure/ConnectorModel':function(){
define([
        "dojo/_base/array",
        "dojo/_base/declare",
        "dojo/_base/config",
        "dojo/_base/lang",
        "dojo/Deferred",
        "dojo/when",
        "dojo/Evented",
        "dojo/Stateful",

        "dojo/store/Memory",
        "dojo/store/Observable",
        "dojo/store/Cache",

        "dijit/Destroyable",

        "dojox/mvc/at",
        "dojox/mvc/EditModelRefController",
        "dojox/mvc/EditStoreRefController",

        "./ScheduleModel",

        "epi-saas-base/mvc/_CommitReturnMixin",
        "epi-saas-base/mvc/_EmptiableRefControllerMixin"
],
    function(array,
             declare,
             config,
             lang,
             Deferred,
             when,
             Evented,
             Stateful,

             MemoryStore,
             Observable,
             Cache,

             Destroyable,

             at,
             EditModelRefController,
             EditStoreRefController,

             ScheduleModel,

             _CommitReturnMixin,
             _EmptiableRefControllerMixin) {

        // module:
        //      epi-find/configure/ConnectorModel

        return declare([Stateful, Destroyable, Evented], {
            // summary:
            //     Model for the site menu for site and language.

            // store: dojo/store/JsonRest
            //      A store that is used for connector data
            store: null,

            // store: dojo/store/JsonRest
            //      A store that is used for connector jobs
            jobStore: null,

            // connectorJobStore: dojo/store/JsonRest
            //      A store that is used for connector jobs
            connectorJobStore: null,

            // connectorJobCachingStore: dojo/store/Cache
            //      A caching store for connector jobs
            connectorJobCachingStore: null,

            // ctrl: dojox/mvc/EditStoreRefController
            //      A controller for binding
            ctrl: null,

            // configController: dojox/mvc/EditModelRefController
            //      A controller for binding "configuration" property of connector
            configController: null,

            // scheduleModel: epi-find/configure/ScheduleModel
            //      Model for the connectors schedule widget
            scheduleModel: null,

            // types: Array
            //      Available connector types
            types: [
                { name: "crawler"},
                { name: "rss_atom"}
            ],

            // channels: Object
            //      Available connector channels
            channels: {
              "crawler" : "system!web",
              "rss_atom": "system!rss_atom"
            },


            // refreshesLeft : Number
            //      Amount of refreshes left for _enqueueRefresh function
            refreshesLeft: 0,

            // refreshQueue: Array
            //      Queue of refreshes for _enqueueRefresh function
            refreshQueue: null,

            // jobStartMaxUpdates: Number
            //      Maximal amount of status update requests when waiting for job to start
            jobStartMaxUpdates: 10,

            // jobStopMaxUpdates: Number
            //      Maximal amount of status update requests when waiting for job to stop
            jobStopMaxUpdates:20,

            postscript: function() {
                this.inherited(arguments);
                this.refreshQueue = [];

                this.store = new Observable(this.store || config.dependencies["epi-find.ConnectorStore"]);
                this.jobStore = this.jobStore || config.dependencies["epi-find.ConnectorJobStore"];

                this.connectorJobCachingStore = new MemoryStore({idProperty: this.jobStore.idProperty});
                this.connectorJobStore = Cache(this.jobStore, this.connectorJobCachingStore);

                var BindingController = declare([EditStoreRefController, _CommitReturnMixin, _EmptiableRefControllerMixin]);
                this.ctrl = new BindingController({
                    store: this.store,
                    emptyModel: new Stateful({
                        type: this.types[0].name,
                        configuration: new Stateful({})
                    })
                });

                this.configController = new EditModelRefController({
                    model: at(this.ctrl, "configuration")
                });

                this.scheduleModel = new ScheduleModel();
            },

            refreshJobCache: function() {
                // summary:
                //      Refreshes connectors job cache.
                //  description:
                //      All connector jobs are stored in cache in order to prevent multiple individual requests
                //      in order to find out job status for each connector.
                return this.connectorJobStore.query();
            },

            getJob: function(connector) {
                // summary:
                //      Returns corresponding connector job for a specified connector
                return this.connectorJobCachingStore.get(this.store.getIdentity(connector));
            },

            startJob: function(connector) {
                // summary:
                //      Starts corresponding connector job for a specified connector
                return when(this.connectorJobStore.start(this.store.getIdentity(connector)), lang.hitch(this, function() {
                    this._enqueueRefresh(connector, "running", this.jobStartMaxUpdates);
                }));
            },

            stopJob: function(connector) {
                // summary:
                //      Stops corresponding connector job for a specified connector

                return when(this.connectorJobStore.stop(this.store.getIdentity(connector)), lang.hitch(this, function() {
                    // Add status refresh to refreshes queue.
                    // Stopping the job usually takes more time then starting,
                    // so we put max 20 attempts to get expected status
                    this._enqueueRefresh(connector, "stopped", this.jobStopMaxUpdates);
                }));
            },

            isActionDisabled: function(connector) {
                // summary:
                //      Returns true if an action was recently triggered, but job status didn't changed accordingly yet.

                var self = this, job = this.getJob(connector),
                    currentStatus = (job && job.mode) ? job.mode : "";
                return array.some(this.refreshQueue, function(item) {
                    return self.store.getIdentity(item.connector) === self.store.getIdentity(connector) && item.status !== currentStatus;
                });
            },

            updateChannel: function() {
              this.ctrl.set("channel", this.channels[this.ctrl.get("type")]);
            },

            _enqueueRefresh: function(/*Object*/ connector, /*String*/ expectedStatus, /*Number*/ maxTries) {

                this.refreshQueue.push({connector: connector, status: expectedStatus});
                var isStarted = this.refreshesLeft > 0;
                this.refreshesLeft = Math.max(this.refreshesLeft, maxTries);
                if (!isStarted) {
                    this._updateStatuses();
                }
            },

            _updateStatuses: function() {

                var nextRefresh = this.refreshQueue.length ? this.refreshQueue[0] : null, // peek top item in the queue
                    self = this;
                if (nextRefresh) {
                    when(this._updateStatus(nextRefresh), function(newStatus) {
                        self.refreshQueue.shift(); // remove top item from queue
                        self.emit("statusChanged", { connector: nextRefresh.connector, status: newStatus });
                        self.store.notify(nextRefresh.connector, self.store.getIdentity(nextRefresh.connector)); // enable button
                        self._updateStatuses();
                    });
                }
                else {
                    this.refreshesLeft = 0;
                }
            },

            _updateStatus : function(data) {
                var self = this, result = new Deferred(), getNewStatus = function () {
                    when(self.refreshJobCache(), function(jobs) {
                        jobs.forEach(function(item) {
                            for (var i=0; i< self.refreshQueue.length; i++) {
                                if (self.store.getIdentity(self.refreshQueue[i].connector) === self.jobStore.getIdentity(item)) {
                                    self.store.notify(self.refreshQueue[i].connector,  self.store.getIdentity(self.refreshQueue[i].connector)); // notify about new status
                                }
                            }
                        });

                        var job = self.getJob(data.connector);
                        var newStatus = job ? job.mode : null;
                        if (newStatus !== data.status && self.refreshesLeft > 0) {
                            self.refreshesLeft--;
                            setTimeout(getNewStatus, 5000); // refresh statuses every 5 seconds
                        }
                        else {
                            result.resolve(newStatus);
                        }
                    });
                };
                getNewStatus();
                return result;
            }

        });
    });

},
'epi-find/configure/ScheduleModel':function(){
define([
        "dojo/_base/declare",
        "dojo/_base/lang",
        "dojo/date/locale",

        "dojox/lang/functional",

        "dojo/i18n!./../widget/nls/Schedule"
    ],
function(
        declare,
        lang,
        locale,

        functional,

        i18n
    ) {
    return declare([], {
        i18n: i18n,
        offset: null,
        constructor: function() {
          this.offset = new Date().getTimezoneOffset();
        },
        parseCronSchedule: function(/*String*/ value){
            // summary:
            //      Parses string in cron format (http://en.wikipedia.org/wiki/Cron#Predefined_scheduling_definitions)
            //      into a schedule object with type and parameters. For now it support only those formats that we allow to edit in UI
            if (!value) {
                throw new Error("No schedule value specified");
            }
            if (typeof value !== "string") {
                throw new Error("Schedule value should be a string");
            }

            var weeklyMatch = /(\d+) (\d+) \* \* (\d+)( \*)?/.exec(value);
            if (weeklyMatch) { // weekly
                return {type: "weekly", dayOfTheWeek: parseInt(weeklyMatch[3], 10), time: new Date(0, 0, 0, weeklyMatch[2], weeklyMatch[1])};
            }
            var dailyMatch = /(\d+) (\d+) \* \* \*( \*)?/.exec(value);
            if (dailyMatch) {  // daily
                return {type: "daily", time: new Date(0, 0, 0, dailyMatch[2], dailyMatch[1])};
            }
            return {type: "custom", cronSchedule: value};
        },

        serialiseToCronSchedule: function (/*Object*/ schedule) {
            // summary:
            //      Serialises schedule object as a string in cron format
            if (!schedule) {
                return {};
            }
            if (schedule.type === "weekly") {
                return {cron: "" + schedule.time.getMinutes() + " " + schedule.time.getHours() + " * * " + schedule.dayOfTheWeek};
            }
            else if (schedule.type === "daily") {
                return {cron: "" + schedule.time.getMinutes() + " " + schedule.time.getHours() + " * * *"};
            }
            else if (schedule.type === "custom") {
                return  {cron: schedule.cronSchedule};
            }
        },

        changeScheduleTZ: function(/*Object*/ schedule, /*Int*/ timezoneOffset) {
            //  summary:
            //       Takes schedule object and a timezone offset in minutes and returns a new schedule object without offset.
            if (!schedule || !schedule.time) {
              return {};
            }
            var day;
            var minute = timezoneOffset % 60 + schedule.time.getMinutes();
            var hour = Math.floor(Math.abs(timezoneOffset/60))*this.sign(timezoneOffset) + Math.floor(Math.abs(minute/60))*this.sign(minute) + schedule.time.getHours();
            minute = minute % 60;
            if (typeof(schedule.dayOfTheWeek) !== "undefined") {
              day = parseInt(schedule.dayOfTheWeek, 10);
            }
            if (minute < 0) {
              minute += 60;
              hour --;
            }
            if (hour < 0) {
              hour += 24;
              if (schedule.type === "weekly") {
                day = (day+6) % 7;
              }
            } else if (hour > 24) {
              hour -= 24;
              if (schedule.type === "weekly") {
                day = (day+1) % 7;
              }
            }
            if (schedule.type === "weekly") {
              return {type: schedule.type, dayOfTheWeek: day, time: new Date(0,0,0,hour,minute)};
            } else {
              return {type: schedule.type, time: new Date(0,0,0,hour,minute)};
            }
        },

        sign: function(/*int*/ nbr) {
          return nbr ? nbr<0 ? -1 : 1 : 0;
        },

        getTextDescription: function (value) {
            // summary:
            //      A "static" function that transforms a schedule value in a user friendly string
            if (!value || !value.cron) {
                return this.i18n.noSchedule;
            }
            //Uses timezone offset to change date/time to UTC.
            var schedule = this.parseCronSchedule(value.cron);
            if (schedule.type === "daily") {
                schedule = this.changeScheduleTZ(schedule, -this.offset);
                return lang.replace(this.i18n.dailySchedule, {time: locale.format(schedule.time, { selector: "time"})});
            }
            else if (schedule.type === "weekly") {
                schedule = this.changeScheduleTZ(schedule, -this.offset);
                var weekdaysArray = [i18n.weekdays_monday, i18n.weekdays_tuesday, i18n.weekdays_wednesday, i18n.weekdays_thursday, i18n.weekdays_friday, i18n.weekdays_saturday, i18n.weekdays_sunday];
                return lang.replace(this.i18n.weeklySchedule, {
                    time: locale.format(schedule.time, { selector: "time"}),
                    dayOfTheWeek: weekdaysArray[schedule.dayOfTheWeek - 1]
                });
            }
            return lang.replace(this.i18n.customSchedule, schedule);
        }

    });
});

},
'epi-find/configure/BoostingEditorModel':function(){
﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/when",
    "dojo/Deferred",
    "dojo/Stateful",

    "dojox/mvc/ModelRefController",
    "dojox/mvc/EditStoreRefController",
    "dojox/mvc/getPlainValue",

    "dojox/lang/functional",

    "epi-saas-base/mvc/_CommitReturnMixin",
    "epi-saas-base/mvc/_EmptiableRefControllerMixin",
    "epi-saas-base/RetryableXhrWrapper"
],
    function (declare,
              lang,
              array,
              when,
              Deferred,
              Stateful,
              ModelRefController,
              EditStoreRefController,
              getPlainValue,
              functional,
              _CommitReturnMixin,
              _EmptiableRefControllerMixin,
              RetryableXhrWrapper) {

        return declare([EditStoreRefController, _EmptiableRefControllerMixin, _CommitReturnMixin], {
            // summary:
            //      View model for Boosting Weights Editor widget.

            // weightsModel: ModelRefController
            //      Model controller for boosting weight properties
            weightsModel: null,

            // propertyNames: Array
            //      Names of the unified search object properties that can have a weight
            propertyNames: null,

            // defaultWeight: Integer
            //      Default weight value
            defaultWeight: 1,

            // invalidateCacheUrl: String
            //      URL of the endpoint to invalidate unified search weights cache.
            invalidateCacheUrl: null,

            set: function(/*String*/ name, /*Anything*/ value){
                // summary:
                //		Set a property to this. Overridden in order to reload weights from the new model.
                // name: String
                //		The property to set.
                // value: Anything
                //		The value to set in the property.

                this.inherited(arguments);
                if(name === this._refEditModelProp || name === this._refSourceModelProp){
                    this._refreshWeights(value);
                }
            },

            postscript: function() {
                this.inherited(arguments);
                this.weightsModel = new ModelRefController();
                this.set("emptyModel", new Stateful({
                    "name": "default",
                    "weights": {
                        "search_title": this.defaultWeight,
                        "search_text": this.defaultWeight,
                        "search_summary": this.defaultWeight,
                        "search_attachment": this.defaultWeight
                    },
                    tags: [ "searchtype:default" ]
                }));
                this.xhrHandler = new RetryableXhrWrapper();
            },

            refresh: function() {
                // summary:
                //      Loads first found unified search object. This method will be obsolete when objects are loaded by id.

                // TODO: temporary solution to load one item by id; it should be removed when user can select which search object is being edited
                var deferred = new Deferred();
                when(this.store.query({ tags: "searchtype:default" }), lang.hitch(this, function(results) {
                    var firstResult = results.length > 0 ? results[0] : null;
                    // To remove multiple boosting settings from the index, if there are such
                    if (results.length > 1) {
                        for(var i = 1; i < results.length; i++) {
                            this.store.remove(results[i].id);
                        }
                    }
                    if (!firstResult) {
                        // UnifiedSearch items are not found. Create and use new object with default weights.
                        this.empty();
                        deferred.resolve(this.get("model"));
                        return;
                    }
                    if (!firstResult.id) {
                        deferred.reject(new Error("Unable to load unified search object by id."));
                    }
                    when(this.getStore(firstResult.id), function(unifiedSearch) {
                        deferred.resolve(unifiedSearch);
                    }, function(error) {
                        deferred.reject(error);
                    });
                }), function (error) {
                    deferred.reject(error);
                });
                return deferred.promise;
            },

            commit: function () {
                return when(this.inherited(arguments), lang.hitch(this, function(weight) {
                    this._set("id", weight.id);
                    return this.invalidateCache();
                }));
            },

            setDefault: function () {
                // summary:
                //      Sets default values for weight properties.

                array.forEach(this.propertyNames, function(property){
                    this.weightsModel.set(property, this.defaultWeight);
                }, this);
            },

            hasControllerProperty: function(/*String*/ name){
                // summary:
                //		Returns true if this controller itself owns the given property.
                // name: String
                //		The property name.
                return this.inherited(arguments) || name === "weightsModel" || name === "propertyNames" || name === "defaultWeight";
            },

            invalidateCache: function () {
                return this.xhrHandler.xhr("POST", {
                    url: this.invalidateCacheUrl,
                    handleAs: "json"
                }).promise;
            },

            _refreshWeights: function(unifiedSearch) {
                if (!unifiedSearch) {
                    return;
                }
                this.propertyNames = functional.map(getPlainValue(unifiedSearch).weights, function (value, property) {
                    return property;
                });
                this.weightsModel.set("model", unifiedSearch.weights);
            }
        });
    });

},
'epi-find/configure/BoostingPreviewModel':function(){
﻿define([
        "dojo/_base/declare",
        "dojo/_base/lang",
        "dojo/_base/config",
        "dojo/Stateful",

        "../util/search"
    ],
    function (declare,
              lang,
              config,
              Stateful,
              search) {

        return declare([Stateful], {
            // summary:
            //      View model for Boosting Search Preview widget.

            // searchText: String
            //      Search query text
            searchText: null,

            _searchTextSetter: function (value) {
                if (value !== this.searchText) {
                    this.searchText = value;
                    this.set("query", this._buildQuery());
                }
            },

            // language: String
            //      Language to filter search hits
            language: null,

            _languageSetter: function (value) {
                if (value !== this.language) {
                    this.language = value;
                    this.set("query", this._buildQuery());
                }
            },

            // bestBets: Boolean
            //      Indicates whether best bets should be used when searching
            bestBets: false,

            _bestBetsSetter: function (value) {
                value = !!value;
                if (value !== this.bestBets) {
                    this.bestBets = value;
                    this.set("query", this._buildQuery());
                }
            },

            // synonyms: Boolean
            //      Indicates whether synonyms should be used when searching
            synonyms: false,

            _synonymsSetter: function (value) {
                value = !!value;
                if (value !== this.synonyms) {
                    this.synonyms = value;
                    this.set("query", this._buildQuery());
                }
            },

            // weights: Object
            //      Unified search properties weights
            weights: null,

            _weightsSetter: function (value) {
                this.weights = value;
                this.set("query", this._buildQuery());
            },

            // query: Object
            //      Query that is used to run search
            query: null,

            _queryGetter: function () {
                if (!this.query) {
                    this.query = this._buildQuery();
                }
                return this.query;
            },

            // store: Store
            //      Elastic search store
            store: null,

            // languages: Array
            //      Array of available languages
            languages: null,

            _queryTemplate: {
                "query": null,
                "language": null,
                "apply_best_bets": false,
                "weights": { }
            },

            postscript: function() {
                this.inherited(arguments);
                if (!this.languages) {
                    this.languages = [];
                }
            },

            _buildQuery: function() {
                // summary:
                //      Builds ElasticSearch query based on model parameters.
                // tags:
                //      Private
                var query = lang.clone(this._queryTemplate);

                if (this.searchText) {
                    query.query = this.searchText;
                }

                if (this.language) {
                    query.language = this.language;
                }

                if (this.weights) {
                    query.weights = this.weights;
                }

                query.apply_best_bets = this.bestBets;
                query.apply_synonyms = this.synonyms;

                return query;
            }
        });
    });
},
'epi-find/util/search':function(){
define([], function() {
    // module:
    //		epi-find/util/search

    var search = {
        // summary:
        //		This module utility methods that can be used when executing Elastic Search queries.

        escapeQueryStringQuery: function(value, skipEscapingWildcard) {
            // summary:
            //     Escapes characters that have a special meaning in a querystring query.


            // Also see QueryEscaping in the EPiServer.Find .NET API

            // From http://www.elasticsearch.org/guide/en/elasticsearch/reference/current/query-dsl-query-string-query.html#_reserved_characters:
            // The reserved characters are: + - && || ! ( ) { } [ ] ^ " ~ * ? : \ /
            //
            // We already had the semicolon in our list and it is unclear if escaping it actually serves a purpose.
            var specialCharactersSkipWildcardRegex = /(\+|\-|\&\&|\|\||\!|\(|\)|\{|\}|\[|\]|\^|\~|\?|\:|\\|\/|\;)/g;
            var specialCharactersRegex             = /(\+|\-|\&\&|\|\||\!|\(|\)|\{|\}|\[|\]|\^|\~|\*|\?|\:|\\|\/|\;)/g;
            var quotesRegex = /(")/g;
            var evenNumberOfQuotes = /^(([^"]*"){2})*[^"]*$/;

            var escaped = value.replace(skipEscapingWildcard ? specialCharactersSkipWildcardRegex : specialCharactersRegex, "\\$1");
            if(!evenNumberOfQuotes.test(escaped))
            {
                escaped = escaped.replace(quotesRegex, "\\$1");
            }

            return escaped;
        }

    };

    return search;
});

},
'epi-find/configure/IndexModel':function(){
define([
    "dojo/_base/declare",
    "dojo/_base/config",
    "dojo/Deferred",
    "dojo/Stateful",
    "epi-saas-base/RetryableXhrWrapper"
],
    function(declare,
        config,
        Deferred,
        Stateful,
        RetryableXhrWrapper) {
        return declare([Stateful], {
            // summary:
            //     Model for Index configuration view.

            // xhrHandler: object
            //      An XHR implementation to use when reading configuration
            xhrHandler: null,

            constructor: function(options) {
                // summary:
                //          Constructs an instance of IndexModel

                declare.safeMixin(this, options);
                this.xhrHandler = this.xhrHandler || new RetryableXhrWrapper();
            },

            clearContentIndex: function () {
                // summary:
                //          Clears content index.
                // returns: dojo/promise/Promise

                return this._clearIndex(true, false);
            },

            clearStats: function () {
                // summary:
                //          Clears statistics.
                // returns: dojo/promise/Promise

                return this._clearIndex(false, true);
            },

            // indexingPluginUrl: String
            //      Url of the EPiServer Find indexing job plugin page
            indexingPluginUrl: "",

            _getTargetUrl: function(/*Boolean*/ clearIndex, /*Boolean*/ clearStats){
                //summary:
                //          Gets url for clear index endpoint
                // clearIndex: Boolean
                //          Flag to indicate whether content index should be cleared
                // clearStats: Boolean
                //          Flag to indicate whether statistics should be cleared
                // returns: String

                return config.find.serviceApiBaseUrl + "_clear?clear_index=" + clearIndex + "&clear_stats=" + clearStats;
            },

            _clearIndex: function(/*Boolean*/ clearIndex, /*Boolean*/ clearStats){
                // summary:
                //          Connects to the endpoint and returns a promise
                // clearIndex: Boolean
                //          Flag to indicate whether content index should be cleared
                // clearStats: Boolean
                //          Flag to indicate whether statistics should be cleared
                // returns: dojo/promise/Promise
                return this.xhrHandler.xhrPost({
                        url: this._getTargetUrl(clearIndex, clearStats),
                        handleAs: "json",
                        xsrfProtection: true
                    }).promise;
            }
        });
    });
},
'epi-find/widget/Configure':function(){
﻿define([
        "dojo/_base/config",
        "dojo/_base/declare",
        "dojo/on",
        "dojo/topic",
        "dojo/dom-class",
        "dojo/query",

        "dijit/_WidgetBase",
        "dijit/_WidgetsInTemplateMixin",
        "dijit/_TemplatedMixin",
        "dijit/layout/ContentPane",
        "dijit/layout/StackContainer",

        "./_ActionableMixin",
        "./Connectors",
        "./Boosting",
        "./Index",

        "../AccessModel",

        "dojo/i18n!./nls/Configure",
        "dojo/text!./templates/Configure.html"
],
    function(config,
             declare,
             on,
             topic,
             domClass,
             query,
             _WidgetBase,
             _WidgetsInTemplateMixin,
             _TemplatedMixin,
             ContentPane,
             StackContainer,
             _ActionableMixin,
             Connectors,
             Boosting,
             Index,
             AccessModel,
             i18n,
             template) {

        // module:
        //      epi-find/widget/Connectors

        return declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, _ActionableMixin], {
            // summary:
            //     Manage view.

            templateString: template,

            i18n: i18n,

            model: null,

            constructor: function() {
                this.inherited(arguments);
            },

            postCreate: function() {
                this.inherited(arguments);

                this.boostingWidget = new Boosting({
                    model: this.model,
                    disabledView: !AccessModel.isEnabled(config.intents.view.boosting),
                    scrollNode: this.contentNode
                });

                this.widgetContainer.addChild(this.boostingWidget);
                this.own(this.boostingWidget);

                this.indexWidget = new Index({
                    model: this.model.indexModel
                });

                this.widgetContainer.addChild(this.indexWidget);
                this.own(this.indexWidget);

                this.connectorsWidget = new Connectors({
                    model: this.model.connectorModel,
                    disabledView: !AccessModel.isEnabled(config.intents.view.connectors),
                    scrollNode: this.contentNode
                });

                this.widgetContainer.addChild(this.connectorsWidget);
                this.own(this.connectorsWidget);

                var self = this,
                   boostingAction = {
                        currentAction: "boosting",
                        takeAction: function(actions, params) {
                            self.widgetContainer.selectChild(self.boostingWidget);
                            self.boostingWidget.takeAction(actions, params);
                            self._switchSelection(self.boostingLink.parentNode);
                            return self.boostingWidget;
                        }
                    },
                    indexAction = {
                        currentAction: "index",
                        takeAction: function(actions, params) {
                            self.widgetContainer.selectChild(self.indexWidget);
                            self.indexWidget.takeAction(actions, params);
                            self._switchSelection(self.indexLink.parentNode);
                            return self.indexWidget;
                        }
                    },
                    connectorsAction = {
                        currentAction: "connectors",
                        takeAction: function(actions, params) {
                            self.widgetContainer.selectChild(self.connectorsWidget);
                            self.connectorsWidget.takeAction(actions, params);
                            self._switchSelection(self.connectorsLink.parentNode);
                            return self.connectorsWidget;
                        }
                    };

                this.currentAction = boostingAction.currentAction;
                this.registerAction(boostingAction.currentAction, boostingAction);
                this.registerAction(indexAction.currentAction, indexAction);
                this.registerAction(connectorsAction.currentAction, connectorsAction);

                this.own(
                    on(this.boostingLink, "click", function(event) {
                        topic.publish(config.intents.view.boosting);
                        event.preventDefault();
                    }),
                    on(this.indexLink, "click", function(event) {
                        topic.publish(config.intents.view.index);
                        event.preventDefault();
                    }),
                    on(this.connectorsLink, "click", function(event) {
                        topic.publish(config.intents.view.connectors);
                        event.preventDefault();

                    })
                );

                this.previousActionFallback = true;
            },

            startup: function() {
                this.inherited(arguments);
                this.takeAction([this.currentAction]);
            },

            _switchSelection: function(/*domNode*/ node) {
                // summary:
                //      Removes current selection indication and puts selection indicator to a new node
                query(".epi-pane__content__nav li.is-active", this.domNode).removeClass("is-active");
                domClass.add(node, "is-active");
            }
        });
    });

},
'dijit/layout/StackContainer':function(){
define("dijit/layout/StackContainer", [
	"dojo/_base/array", // array.forEach array.indexOf array.some
	"dojo/cookie", // cookie
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add domClass.replace
	"dojo/has",	// has("dijit-legacy-requires")
	"dojo/_base/lang",	// lang.extend
	"dojo/ready",
	"dojo/topic", // publish
	"../registry",	// registry.byId
	"../_WidgetBase",
	"./_LayoutWidget",
	"dojo/i18n!../nls/common"
], function(array, cookie, declare, domClass, has, lang, ready, topic,
			registry, _WidgetBase, _LayoutWidget){

// module:
//		dijit/layout/StackContainer

// Back compat w/1.6, remove for 2.0
if(has("dijit-legacy-requires")){
	ready(0, function(){
		var requires = ["dijit/layout/StackController"];
		require(requires);	// use indirection so modules not rolled into a build
	});
}

var StackContainer = declare("dijit.layout.StackContainer", _LayoutWidget, {
	// summary:
	//		A container that has multiple children, but shows only
	//		one child at a time
	//
	// description:
	//		A container for widgets (ContentPanes, for example) That displays
	//		only one Widget at a time.
	//
	//		Publishes topics [widgetId]-addChild, [widgetId]-removeChild, and [widgetId]-selectChild
	//
	//		Can be base class for container, Wizard, Show, etc.
	//
	//		See `StackContainer.ChildWidgetProperties` for details on the properties that can be set on
	//		children of a `StackContainer`.

	// doLayout: Boolean
	//		If true, change the size of my currently displayed child to match my size
	doLayout: true,

	// persist: Boolean
	//		Remembers the selected child across sessions
	persist: false,

	baseClass: "dijitStackContainer",

/*=====
	// selectedChildWidget: [readonly] dijit._Widget
	//		References the currently selected child widget, if any.
	//		Adjust selected child with selectChild() method.
	selectedChildWidget: null,
=====*/

	buildRendering: function(){
		this.inherited(arguments);
		domClass.add(this.domNode, "dijitLayoutContainer");
	},

	postCreate: function(){
		this.inherited(arguments);
		this.connect(this.domNode, "onkeypress", this._onKeyPress);
	},

	startup: function(){
		if(this._started){ return; }

		var children = this.getChildren();

		// Setup each page panel to be initially hidden
		array.forEach(children, this._setupChild, this);

		// Figure out which child to initially display, defaulting to first one
		if(this.persist){
			this.selectedChildWidget = registry.byId(cookie(this.id + "_selectedChild"));
		}else{
			array.some(children, function(child){
				if(child.selected){
					this.selectedChildWidget = child;
				}
				return child.selected;
			}, this);
		}
		var selected = this.selectedChildWidget;
		if(!selected && children[0]){
			selected = this.selectedChildWidget = children[0];
			selected.selected = true;
		}

		// Publish information about myself so any StackControllers can initialize.
		// This needs to happen before this.inherited(arguments) so that for
		// TabContainer, this._contentBox doesn't include the space for the tab labels.
		topic.publish(this.id+"-startup", {children: children, selected: selected});

		// Startup each child widget, and do initial layout like setting this._contentBox,
		// then calls this.resize() which does the initial sizing on the selected child.
		this.inherited(arguments);
	},

	resize: function(){
		// Overrides _LayoutWidget.resize()
		// Resize is called when we are first made visible (it's called from startup()
		// if we are initially visible). If this is the first time we've been made
		// visible then show our first child.
		if(!this._hasBeenShown){
			this._hasBeenShown = true;
			var selected = this.selectedChildWidget;
			if(selected){
				this._showChild(selected);
			}
		}
		this.inherited(arguments);
	},

	_setupChild: function(/*dijit/_WidgetBase*/ child){
		// Overrides _LayoutWidget._setupChild()

		this.inherited(arguments);

		domClass.replace(child.domNode, "dijitHidden", "dijitVisible");

		// remove the title attribute so it doesn't show up when i hover
		// over a node
		child.domNode.title = "";
	},

	addChild: function(/*dijit/_WidgetBase*/ child, /*Integer?*/ insertIndex){
		// Overrides _Container.addChild() to do layout and publish events

		this.inherited(arguments);

		if(this._started){
			topic.publish(this.id+"-addChild", child, insertIndex);	// publish

			// in case the tab titles have overflowed from one line to two lines
			// (or, if this if first child, from zero lines to one line)
			// TODO: w/ScrollingTabController this is no longer necessary, although
			// ScrollTabController.resize() does need to get called to show/hide
			// the navigation buttons as appropriate, but that's handled in ScrollingTabController.onAddChild().
			// If this is updated to not layout [except for initial child added / last child removed], update
			// "childless startup" test in StackContainer.html to check for no resize event after second addChild()
			this.layout();

			// if this is the first child, then select it
			if(!this.selectedChildWidget){
				this.selectChild(child);
			}
		}
	},

	removeChild: function(/*dijit/_WidgetBase*/ page){
		// Overrides _Container.removeChild() to do layout and publish events

		this.inherited(arguments);

		if(this._started){
			// this will notify any tablists to remove a button; do this first because it may affect sizing
			topic.publish(this.id + "-removeChild", page);	// publish
		}

		// If all our children are being destroyed than don't run the code below (to select another page),
		// because we are deleting every page one by one
		if(this._descendantsBeingDestroyed){ return; }

		// Select new page to display, also updating TabController to show the respective tab.
		// Do this before layout call because it can affect the height of the TabController.
		if(this.selectedChildWidget === page){
			this.selectedChildWidget = undefined;
			if(this._started){
				var children = this.getChildren();
				if(children.length){
					this.selectChild(children[0]);
				}
			}
		}

		if(this._started){
			// In case the tab titles now take up one line instead of two lines
			// (note though that ScrollingTabController never overflows to multiple lines),
			// or the height has changed slightly because of addition/removal of tab which close icon
			this.layout();
		}
	},

	selectChild: function(/*dijit/_WidgetBase|String*/ page, /*Boolean*/ animate){
		// summary:
		//		Show the given widget (which must be one of my children)
		// page:
		//		Reference to child widget or id of child widget

		page = registry.byId(page);

		if(this.selectedChildWidget != page){
			// Deselect old page and select new one
			var d = this._transition(page, this.selectedChildWidget, animate);
			this._set("selectedChildWidget", page);
			topic.publish(this.id+"-selectChild", page);	// publish

			if(this.persist){
				cookie(this.id + "_selectedChild", this.selectedChildWidget.id);
			}
		}

		return d;		// If child has an href, promise that fires when the child's href finishes loading
	},

	_transition: function(newWidget, oldWidget /*===== ,  animate =====*/){
		// summary:
		//		Hide the old widget and display the new widget.
		//		Subclasses should override this.
		// newWidget: dijit/_WidgetBase
		//		The newly selected widget.
		// oldWidget: dijit/_WidgetBase
		//		The previously selected widget.
		// animate: Boolean
		//		Used by AccordionContainer to turn on/off slide effect.
		// tags:
		//		protected extension
		if(oldWidget){
			this._hideChild(oldWidget);
		}
		var d = this._showChild(newWidget);

		// Size the new widget, in case this is the first time it's being shown,
		// or I have been resized since the last time it was shown.
		// Note that page must be visible for resizing to work.
		if(newWidget.resize){
			if(this.doLayout){
				newWidget.resize(this._containerContentBox || this._contentBox);
			}else{
				// the child should pick it's own size but we still need to call resize()
				// (with no arguments) to let the widget lay itself out
				newWidget.resize();
			}
		}

		return d;	// If child has an href, promise that fires when the child's href finishes loading
	},

	_adjacent: function(/*Boolean*/ forward){
		// summary:
		//		Gets the next/previous child widget in this container from the current selection.

		// TODO: remove for 2.0 if this isn't being used.   Otherwise, fix to skip disabled tabs.

		var children = this.getChildren();
		var index = array.indexOf(children, this.selectedChildWidget);
		index += forward ? 1 : children.length - 1;
		return children[ index % children.length ]; // dijit/_WidgetBase
	},

	forward: function(){
		// summary:
		//		Advance to next page.
		return this.selectChild(this._adjacent(true), true);
	},

	back: function(){
		// summary:
		//		Go back to previous page.
		return this.selectChild(this._adjacent(false), true);
	},

	_onKeyPress: function(e){
		topic.publish(this.id+"-containerKeyPress", { e: e, page: this});	// publish
	},

	layout: function(){
		// Implement _LayoutWidget.layout() virtual method.
		var child = this.selectedChildWidget;
		if(child && child.resize){
			if(this.doLayout){
				child.resize(this._containerContentBox || this._contentBox);
			}else{
				child.resize();
			}
		}
	},

	_showChild: function(/*dijit/_WidgetBase*/ page){
		// summary:
		//		Show the specified child by changing it's CSS, and call _onShow()/onShow() so
		//		it can do any updates it needs regarding loading href's etc.
		// returns:
		//		Promise that fires when page has finished showing, or true if there's no href
		var children = this.getChildren();
		page.isFirstChild = (page == children[0]);
		page.isLastChild = (page == children[children.length-1]);
		page._set("selected", true);

		domClass.replace(page.domNode, "dijitVisible", "dijitHidden");

		return (page._onShow && page._onShow()) || true;
	},

	_hideChild: function(/*dijit/_WidgetBase*/ page){
		// summary:
		//		Hide the specified child by changing it's CSS, and call _onHide() so
		//		it's notified.
		page._set("selected", false);
		domClass.replace(page.domNode, "dijitHidden", "dijitVisible");

		page.onHide && page.onHide();
	},

	closeChild: function(/*dijit/_WidgetBase*/ page){
		// summary:
		//		Callback when user clicks the [X] to remove a page.
		//		If onClose() returns true then remove and destroy the child.
		// tags:
		//		private
		var remove = page.onClose(this, page);
		if(remove){
			this.removeChild(page);
			// makes sure we can clean up executeScripts in ContentPane onUnLoad
			page.destroyRecursive();
		}
	},

	destroyDescendants: function(/*Boolean*/ preserveDom){
		this._descendantsBeingDestroyed = true;
		this.selectedChildWidget = undefined;
		array.forEach(this.getChildren(), function(child){
			if(!preserveDom){
				this.removeChild(child);
			}
			child.destroyRecursive(preserveDom);
		}, this);
		this._descendantsBeingDestroyed = false;
	}
});

StackContainer.ChildWidgetProperties = {
	// summary:
	//		These properties can be specified for the children of a StackContainer.

	// selected: Boolean
	//		Specifies that this widget should be the initially displayed pane.
	//		Note: to change the selected child use `dijit/layout/StackContainer.selectChild`
	selected: false,

	// disabled: Boolean
	//		Specifies that the button to select this pane should be disabled.
	//		Doesn't affect programmatic selection of the pane, nor does it deselect the pane if it is currently selected.
	disabled: false,

	// closable: Boolean
	//		True if user can close (destroy) this child, such as (for example) clicking the X on the tab.
	closable: false,

	// iconClass: String
	//		CSS Class specifying icon to use in label associated with this pane.
	iconClass: "dijitNoIcon",

	// showTitle: Boolean
	//		When true, display title of this widget as tab label etc., rather than just using
	//		icon specified in iconClass
	showTitle: true
};

// Since any widget can be specified as a StackContainer child, mix them
// into the base widget class.  (This is a hack, but it's effective.)
// This is for the benefit of the parser.   Remove for 2.0.  Also, hide from doc viewer.
lang.extend(_WidgetBase, /*===== {} || =====*/ StackContainer.ChildWidgetProperties);

return StackContainer;
});

},
'epi-find/widget/Connectors':function(){
﻿define([
        "dojo/_base/declare",

        "dijit/_WidgetBase",
        "dijit/_TemplatedMixin",
        "dijit/_FocusMixin",
        "dijit/layout/_LayoutWidget",

        "epi-saas-base/Sticker",

        "./_ActionableMixin",
        "./_SiblingFocusMixin",
        "./_DisabledViewMixin",
        "./ConnectorForm",
        "./ConnectorGrid",

        "dojo/i18n!./nls/Connectors",
        "dojo/text!./templates/Connectors.html"
],
    function(declare,
             _WidgetBase,
             _TemplatedMixin,
             _FocusMixin,
             _LayoutWidget,

             Sticker,

             _ActionableMixin,
             _SiblingFocusMixin,
             _DisabledViewMixin,
             ConnectorForm,
             ConnectorGrid,

             i18n,
             template) {

        // module:
        //      epi-find/widget/Connectors

        return declare([_WidgetBase, _LayoutWidget, _TemplatedMixin, _ActionableMixin, _FocusMixin, _SiblingFocusMixin, _DisabledViewMixin], {
            // summary:
            //     Manage view.

            templateString: template,

            i18n: i18n,

            // model: epi-find/configure/ConnectorModel
            //      Model for the connectors view
            model: null,

            connectorForm: null,
            connectorGrid: null,

            postCreate: function() {
                this.inherited(arguments);

                var connectorForm = this.connectorForm = new ConnectorForm({
                    model: this.model,
                    onSubmitted: function() {
                        // HACK: Workaround for delayed update
                        setTimeout(function() {
                            grid.refresh();
                        }, 5000);
                    }
                }, this.connectorsFormNode);
                connectorForm.reset();
                this.own(connectorForm);

                var grid = this.connectorGrid = new ConnectorGrid({
                    "class": "epi-plain-grid epi-plain-grid--no-border",
                    margin: 140,
                    model: this.model,
                    loadingMessage: i18n.dataLoading,
                    showLoadingMessage: true,
                    onEdit: function(id) {
                        connectorForm.edit(id);
                    }
                }, this.connectorsGridNode);
                this.own(grid);
            },

            startup: function() {
                var scrollNode = this.scrollNode || this.domNode.parentNode;
                this.connectorGrid.set("scrollNode", scrollNode);
                this.inherited(arguments);
                this.connectorForm.startup();
                this.connectorGrid.startup();
                this.own(
                    new Sticker({
                        domNode: this.connectorGrid.headerNode,
                        offset: {top: 0},
                        scrollNode: scrollNode,
                        limitNode: this.connectorGrid.domNode
                    })
                );
            },

            layout: function() {
                // We resize the grid in order to make sure that sizes of header, scroller, etc
                // are recalculated when it's needed, for instance, when grid becomes visible.
                this.connectorGrid.resize();
            },

            takeAction: function() {
                this.connectorGrid.refresh();
                return this;
            }
        });
    });

},
'epi-find/widget/ConnectorForm':function(){
﻿define([
    "dojo/_base/declare",
    "dojo/_base/event",
    "dojo/_base/array",
    "dojo/_base/lang",
    "dojo/dom-style",
    "dojo/on",
    "dojo/when",

    "dojox/mvc/at",
    "dojox/mvc/sync",

    "dijit/_WidgetsInTemplateMixin",
    "dijit/form/Form",
    "dijit/form/CheckBox",
    "dijit/form/Button",
    "dijit/form/ValidationTextBox",
    "dijit/form/Select",
    "dijit/form/RadioButton",

    "epi-find/widget/Schedule",
    "epi-saas-base/UrlRegexp",
    "epi-saas-base/form/FormListItem",
    "epi-saas-base/widgets/_NotificationMixin",
    "epi-saas-base/widgets/TagEditor",
    "epi-saas-base/widgets/_ActionButtonsMixin",
    "./InvertedValidationGrid",

    "dojo/text!./templates/ConnectorForm.html",
    "dojo/i18n!./nls/Connectors",
    "epi-saas-base/UnicodeLetterRegexp"
],
    function(declare,
             event,
             array,
             lang,
             domStyle,
             on,
             when,

             at,
             sync,

             _WidgetsInTemplateMixin,
             Form,
             CheckBox,
             Button,
             ValidationTextBox,
             Select,
             RadioButton,
             Schedule,
             UrlRegexp,
             FormListItem,
             _NotificationMixin,
             TagEditor,
             _ActionButtonsMixin,
             InvertedValidationGrid,

             template,
             i18n,
             UnicodeLetterRegExp

             ) {

        // module:
        //      epi-find/widget/ConnectorForm

        var ViewModes = {
            simple: {
                name: "simple",
                value: "simple"
            },
            advanced: {
                name: "advanced",
                value: "advanced"
            }
        };

        return declare([Form, _WidgetsInTemplateMixin, _NotificationMixin, _ActionButtonsMixin], {
            // summary:
            //     Connector form that allows to create or edit settings of connectors

            i18n: i18n,

            model: null,
            ctrl: null,
            configController: null,

            nameField: null,
            submitButton: null,
            addExcludeUrlFieldButton: null,

            excludeUrlsGrid: null,

            viewMode: ViewModes.simple,

            // typeWidgets: Array
            //      Each array element holds set of widgets related to a specific connector type
            typeWidgets: null,
            advancedWidgets: null,

            baseClass: "",

            templateString: template,

            urlRegex: UrlRegexp,

            globRegex: ".*\\{\\}.*|.*\\[\\].*|.*\\{[^\\}]*|.*\\[[^\\]]*|^(?:\\{[^\\{\\}]\\})*[^\\{]*\\}|^(?:\\[[^\\[\\]]\\])*[^\\[]*\\]|.*\\[[^\\]]*-\\]",

            queryArgsRegex: ".*\\&.*|.*\\=.*",

            nameRegex: "("+UnicodeLetterRegExp+"|[0-9\\-\\&'\\s])*",

            mimeValidationPattern: "[a-zA-Z0-9!#$%^&\\*_\\-\\+\\\\{}\\?\\|'.`~,\\!\\[\\]]+/[a-zA-Z0-9!#$%^&\\*_\\-\\+\\\\{}\\?\\|'.`~,\\!\\[\\]]+", // http://tools.ietf.org/html/rfc2045, http://tools.ietf.org/html/rfc2046

            postCreate: function() {
                this.nameFieldListItem.set("label", i18n.fields_name);
                this.typeSelectListItem.set("label", i18n.fields_type);
                this.startUrlListItem.set("label", i18n.fields_startUrl);
                this.excludeMimeTypesListItem.set("label", i18n.fields_excludeMimeTypes);
                this.includedCrawlPatternListItem.set("label", i18n.fields_includedCrawlPattern);
                this.excludedCrawlPatternListItem.set("label", i18n.fields_excludedCrawlPattern);
                this.excludedIndexPatternListItem.set("label", i18n.fields_excludedIndexPattern);
                this.removeQueryArgsListItem.set("label", i18n.fields_removeQueryArgs);
                this.feedUrlsListItem.set("label", i18n.fields_feedUrls);
                this.scheduleListItem.set("label", i18n.fields_schedule);
                this.submitButton.set("label", i18n.buttons_add);
                this.resetButton.set("label", i18n.buttons_cancel);

                var self = this;

                this.actionButtons = [this.submitButton, this.resetButton];
                this.enabledWhenNoChangesButtons = [this.resetButton];

                this.inherited(arguments);

                this.ctrl = this.model.ctrl;
                this.configController = this.model.configController;

                this.nameField.set("value", at(this.ctrl, "name"));

                var typeOptions = array.map(this.model.types, function(t) {
                    return {label: i18n["types_"+t.name], value: t.name };
                });

                this.typeSelect.set("options", typeOptions);
                this.typeSelect.set("value", at(this.ctrl, "type"));

                this.advancedButton.set("label", at(this, "viewMode").transform({
                    format: function(value) {
                        return i18n[value.name];
                    }
                }).direction(at.from));

                this.own(
                    this.advancedButton.on("click", function(event) {
                        self.set("viewMode", self.get("viewMode") === ViewModes.simple ? ViewModes.advanced : ViewModes.simple);
                        event.preventDefault();
                    }),
                    this.watch("viewMode", function(name, oldValue, newValue) {
                        self.layoutForm(self.advancedWidgets, newValue);
                    }),
                    this.typeSelect.on("change", function() {
                        self.set("viewMode", ViewModes.simple); // Hides advanced widgets on "type" change
                        self.layoutForm(self.typeWidgets, self.typeSelect);
                    })
                );

                this.startUrl.set("value", at(this.configController, "starting_urls").transform({
                    format: function(value) {
                        if (!value) {
                            return "";
                        }
                        return value.join(",");
                    },
                    parse: function(value){
                        return value.split(",");
                    }
                }));

                this.excludedCrawlPatternGrid.set("value", at(this.configController, "excluded_crawl_patterns"));

                this.excludedIndexPatternGrid.set("value", at(this.configController, "excluded_index_patterns"));

                this.removeQueryArgsGrid.set("value", at(this.configController, "remove_query_args"));

                this.includedCrawlPatternGrid.set("value", at(this.configController, "included_crawl_patterns"));
                this.excludeMimeTypes.set("value", at(this.configController, "excluded_mime_type_patterns"));

                this.feedUrls.set("value", at(this.configController, "feed_urls").transform({
                    format: function(value) {
                        return value.join(",");
                    },
                    parse: function(value){
                        return value.split(",");
                    }
                }));

                this.schedule.set("schedule", at(this.ctrl, "schedule"));

                // Configuring sets of widgets that corresponds to different connector types.
                // These sets are used to show/hide, enable/disable widgets depending on the selected connector type
                this.typeWidgets = array.map(this.model.types, lang.hitch(this, function(t) {
                    if (t.name === "crawler") {
                        // In this implementation, only crawler can have advanced form items. :(
                        return {name: t.name, widgets: [this.startUrl, this.excludeMimeTypes, this.advancedButton]};
                    }
                    if (t.name === "rss_atom") {
                        return {name: t.name, widgets:  [this.feedUrls]};
                    }
                }));

                this.advancedWidgets = [{
                    name: ViewModes.simple.name, widgets: []
                }, {
                    name: ViewModes.advanced.name, widgets: [this.includedCrawlPatternGrid, this.excludedCrawlPatternGrid, this.excludedIndexPatternGrid, this.removeQueryArgsGrid]
                }];
                this.layoutForm(this.typeWidgets, this.typeSelect);
                this.layoutForm(this.advancedWidgets, this.get("viewMode"));
                this.reset();

                this.own(this.watchDataChanges([this.ctrl, this.configController]));
            },

            layoutForm: function(/*Object[]*/ domList, /*Object*/ type) {
                array.forEach(domList, function(entry) {
                    var disable = entry.name !== type.value;
                    array.forEach(entry.widgets, function(widget) {
                        if (widget.domNode) {
                            domStyle.set(widget.domNode.parentNode.parentNode, "display", disable ? "none" : "");
                            widget.set("disabled", disable);
                        } else {
                            domStyle.set(widget.parentNode.parentNode, "display", disable ? "none" : "");
                        }
                    });
                });
            },

            edit: function(connectorId) {
                if (this.editMode) {
                    this.reset();
                }
                when(this.ctrl.getStore(connectorId), lang.hitch(this, function() {
                        this.submitButton.set("label", i18n.buttons_save);
                        this.nameField.focus();
                        this.set("hasChanges", false);
                    }),
                    lang.hitch(this, function(error) {
                        this.showError(error);
                    })
                );
            },

            onSubmit: function(e) {
                event.stop(e);

                if (!this.validate()) {
                    return;
                }

                this.model.updateChannel();
                this.submitButton.focus();
                this.indicateInProgress(this.submitButton);
                when(this.ctrl.commit(), lang.hitch(this, function() {
                        this.showSuccess(i18n.savedMessage);
                        this.reset();
                        this.onSubmitted();
                        this.indicateSuccess();
                        this.nameField.focus();
                    }),
                    lang.hitch(this, function(error) {
                        this.showError(error);
                        this.indicateError();
                    })
                );
            },

            onSubmitted: function() {
                // summary:
                //      A callback for the client hooks
            },

            onReset: function() {
                this.model.ctrl.reset();
                this.model.ctrl.empty();
                this.submitButton.set("label", i18n.buttons_add);
                return this.inherited(arguments);
            }

        });
    });

},
'dijit/form/Select':function(){
require({cache:{
'url:dijit/form/templates/Select.html':"<table class=\"dijit dijitReset dijitInline dijitLeft\"\n\tdata-dojo-attach-point=\"_buttonNode,tableNode,focusNode,_popupStateNode\" cellspacing='0' cellpadding='0'\n\trole=\"listbox\" aria-haspopup=\"true\"\n\t><tbody role=\"presentation\"><tr role=\"presentation\"\n\t\t><td class=\"dijitReset dijitStretch dijitButtonContents\" role=\"presentation\"\n\t\t\t><div class=\"dijitReset dijitInputField dijitButtonText\"  data-dojo-attach-point=\"containerNode\" role=\"presentation\"></div\n\t\t\t><div class=\"dijitReset dijitValidationContainer\"\n\t\t\t\t><input class=\"dijitReset dijitInputField dijitValidationIcon dijitValidationInner\" value=\"&#935; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t\t\t/></div\n\t\t\t><input type=\"hidden\" ${!nameAttrSetting} data-dojo-attach-point=\"valueNode\" value=\"${value}\" aria-hidden=\"true\"\n\t\t/></td\n\t\t><td class=\"dijitReset dijitRight dijitButtonNode dijitArrowButton dijitDownArrowButton dijitArrowButtonContainer\"\n\t\t\tdata-dojo-attach-point=\"titleNode\" role=\"presentation\"\n\t\t\t><input class=\"dijitReset dijitInputField dijitArrowButtonInner\" value=\"&#9660; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t\t\t\t${_buttonInputDisabled}\n\t\t/></td\n\t></tr></tbody\n></table>\n"}});
define("dijit/form/Select", [
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/dom-class", // domClass.add domClass.remove domClass.toggle
	"dojo/dom-geometry", // domGeometry.setMarginBox
	"dojo/_base/event", // event.stop
	"dojo/i18n", // i18n.getLocalization
	"dojo/_base/lang", // lang.hitch
	"dojo/sniff", // has("ie")
	"./_FormSelectWidget",
	"../_HasDropDown",
	"../Menu",
	"../MenuItem",
	"../MenuSeparator",
	"../Tooltip",
	"dojo/text!./templates/Select.html",
	"dojo/i18n!./nls/validate"
], function(array, declare, domAttr, domClass, domGeometry, event, i18n, lang, has,
			_FormSelectWidget, _HasDropDown, Menu, MenuItem, MenuSeparator, Tooltip, template){

// module:
//		dijit/form/Select


var _SelectMenu = declare("dijit.form._SelectMenu", Menu, {
	// summary:
	//		An internally-used menu for dropdown that allows us a vertical scrollbar

	// Override Menu.autoFocus setting so that opening a Select highlights the current value.
	autoFocus: true,

	buildRendering: function(){
		// summary:
		//		Stub in our own changes, so that our domNode is not a table
		//		otherwise, we won't respond correctly to heights/overflows
		this.inherited(arguments);
		var o = (this.menuTableNode = this.domNode);
		var n = (this.domNode = this.ownerDocument.createElement("div"));
		n.style.cssText = "overflow-x: hidden; overflow-y: scroll";
		if(o.parentNode){
			o.parentNode.replaceChild(n, o);
		}
		domClass.remove(o, "dijitMenuTable");
		n.className = o.className + " dijitSelectMenu";
		o.className = "dijitReset dijitMenuTable";
		n.setAttribute("role", "listbox");
		o.setAttribute("role", "presentation");
		n.appendChild(o);
	},

	postCreate: function(){
		// summary:
		//		stop mousemove from selecting text on IE to be consistent with other browsers

		this.inherited(arguments);

		this.connect(this.domNode, "onselectstart", event.stop);
	},


	focus: function(){
		// summary:
		//		Overridden so that the previously selected value will be focused instead of only the first item
		var	found = false,
			val = this.parentWidget.value;
		if(lang.isArray(val)){
			val = val[val.length-1];
		}
		if(val){ // if focus selected
			array.forEach(this.parentWidget._getChildren(), function(child){
				if(child.option && (val === child.option.value)){ // find menu item widget with this value
					found = true;
					this.focusChild(child, false); // focus previous selection
				}
			}, this);
		}
		if(!found){
			this.inherited(arguments); // focus first item by default
		}
	},

	resize: function(/*Object*/ mb){
		// summary:
		//		Overridden so that we are able to handle resizing our
		//		internal widget.  Note that this is not a "full" resize
		//		implementation - it only works correctly if you pass it a
		//		marginBox.
		//
		// mb: Object
		//		The margin box to set this dropdown to.
		if(mb){
			domGeometry.setMarginBox(this.domNode, mb);
			if("w" in mb){
				// We've explicitly set the wrapper <div>'s width, so set <table> width to match.
				// 100% is safer than a pixel value because there may be a scroll bar with
				// browser/OS specific width.
				this.menuTableNode.style.width = "100%";
			}
		}
	}
});

var Select = declare("dijit.form.Select", [_FormSelectWidget, _HasDropDown], {
	// summary:
	//		This is a "styleable" select box - it is basically a DropDownButton which
	//		can take a `<select>` as its input.

	baseClass: "dijitSelect dijitValidationTextBox",

	templateString: template,

	_buttonInputDisabled: has("ie") ? "disabled" : "", // allows IE to disallow focus, but Firefox cannot be disabled for mousedown events

	// required: Boolean
	//		Can be true or false, default is false.
	required: false,

	// state: [readonly] String
	//		"Incomplete" if this select is required but unset (i.e. blank value), "" otherwise
	state: "",

	// message: String
	//		Currently displayed error/prompt message
	message: "",

	// tooltipPosition: String[]
	//		See description of `dijit/Tooltip.defaultPosition` for details on this parameter.
	tooltipPosition: [],

	// emptyLabel: string
	//		What to display in an "empty" dropdown
	emptyLabel: "&#160;",	// &nbsp;

	// _isLoaded: Boolean
	//		Whether or not we have been loaded
	_isLoaded: false,

	// _childrenLoaded: Boolean
	//		Whether or not our children have been loaded
	_childrenLoaded: false,

	_fillContent: function(){
		// summary:
		//		Set the value to be the first, or the selected index
		this.inherited(arguments);
		// set value from selected option
		if(this.options.length && !this.value && this.srcNodeRef){
			var si = this.srcNodeRef.selectedIndex || 0; // || 0 needed for when srcNodeRef is not a SELECT
			this.value = this.options[si >= 0 ? si : 0].value;
		}
		// Create the dropDown widget
		this.dropDown = new _SelectMenu({ id: this.id + "_menu", parentWidget: this });
		domClass.add(this.dropDown.domNode, this.baseClass.replace(/\s+|$/g, "Menu "));
	},

	_getMenuItemForOption: function(/*_FormSelectWidget.__SelectOption*/ option){
		// summary:
		//		For the given option, return the menu item that should be
		//		used to display it.  This can be overridden as needed
		if(!option.value && !option.label){
			// We are a separator (no label set for it)
			return new MenuSeparator({ownerDocument: this.ownerDocument});
		}else{
			// Just a regular menu option
			var click = lang.hitch(this, "_setValueAttr", option);
			var item = new MenuItem({
				option: option,
				label: option.label || this.emptyLabel,
				onClick: click,
				ownerDocument: this.ownerDocument,
				dir: this.dir,
				disabled: option.disabled || false
			});
			item.focusNode.setAttribute("role", "option");
			return item;
		}
	},

	_addOptionItem: function(/*_FormSelectWidget.__SelectOption*/ option){
		// summary:
		//		For the given option, add an option to our dropdown.
		//		If the option doesn't have a value, then a separator is added
		//		in that place.
		if(this.dropDown){
			this.dropDown.addChild(this._getMenuItemForOption(option));
		}
	},

	_getChildren: function(){
		if(!this.dropDown){
			return [];
		}
		return this.dropDown.getChildren();
	},

	_loadChildren: function(/*Boolean*/ loadMenuItems){
		// summary:
		//		Resets the menu and the length attribute of the button - and
		//		ensures that the label is appropriately set.
		// loadMenuItems: Boolean
		//		actually loads the child menu items - we only do this when we are
		//		populating for showing the dropdown.

		if(loadMenuItems === true){
			// this.inherited destroys this.dropDown's child widgets (MenuItems).
			// Avoid this.dropDown (Menu widget) having a pointer to a destroyed widget (which will cause
			// issues later in _setSelected). (see #10296)
			if(this.dropDown){
				delete this.dropDown.focusedChild;
			}
			if(this.options.length){
				this.inherited(arguments);
			}else{
				// Drop down menu is blank but add one blank entry just so something appears on the screen
				// to let users know that they are no choices (mimicing native select behavior)
				array.forEach(this._getChildren(), function(child){ child.destroyRecursive(); });
				var item = new MenuItem({
					ownerDocument: this.ownerDocument,
					label: this.emptyLabel
				});
				this.dropDown.addChild(item);
			}
		}else{
			this._updateSelection();
		}

		this._isLoaded = false;
		this._childrenLoaded = true;

		if(!this._loadingStore){
			// Don't call this if we are loading - since we will handle it later
			this._setValueAttr(this.value, false);
		}
	},

	_refreshState: function(){
		if(this._started){
			this.validate(this.focused);
		}
	},

	startup: function(){
		this.inherited(arguments);
		this._refreshState(); // after all _set* methods have run
	},

	_setValueAttr: function(value){
		this.inherited(arguments);
		domAttr.set(this.valueNode, "value", this.get("value"));
		this._refreshState();	// to update this.state
	},

	_setDisabledAttr: function(/*Boolean*/ value){
		this.inherited(arguments);
		this._refreshState();	// to update this.state
	},

	_setRequiredAttr: function(/*Boolean*/ value){
		this._set("required", value);
		this.focusNode.setAttribute("aria-required", value);
		this._refreshState();	// to update this.state
	},

	_setOptionsAttr: function(/*Array*/ options){
		this._isLoaded = false;
		this._set('options', options);
	},

	_setDisplay: function(/*String*/ newDisplay){
		// summary:
		//		sets the display for the given value (or values)
		var lbl = newDisplay || this.emptyLabel;
		this.containerNode.innerHTML = '<span role="option" class="dijitReset dijitInline ' + this.baseClass.replace(/\s+|$/g, "Label ")+'">' + lbl + '</span>';
	},

	validate: function(/*Boolean*/ isFocused){
		// summary:
		//		Called by oninit, onblur, and onkeypress, and whenever required/disabled state changes
		// description:
		//		Show missing or invalid messages if appropriate, and highlight textbox field.
		//		Used when a select is initially set to no value and the user is required to
		//		set the value.

		var isValid = this.disabled || this.isValid(isFocused);
		this._set("state", isValid ? "" : (this._hasBeenBlurred ? "Error" : "Incomplete"));
		this.focusNode.setAttribute("aria-invalid", isValid ? "false" : "true");
		var message = isValid ? "" : this._missingMsg;
		if(message && this.focused && this._hasBeenBlurred){
			Tooltip.show(message, this.domNode, this.tooltipPosition, !this.isLeftToRight());
		}else{
			Tooltip.hide(this.domNode);
		}
		this._set("message", message);
		return isValid;
	},

	isValid: function(/*Boolean*/ /*===== isFocused =====*/){
		// summary:
		//		Whether or not this is a valid value.  The only way a Select
		//		can be invalid is when it's required but nothing is selected.
		return (!this.required || this.value === 0 || !(/^\s*$/.test(this.value || ""))); // handle value is null or undefined
	},

	reset: function(){
		// summary:
		//		Overridden so that the state will be cleared.
		this.inherited(arguments);
		Tooltip.hide(this.domNode);
		this._refreshState();	// to update this.state
	},

	postMixInProperties: function(){
		// summary:
		//		set the missing message
		this.inherited(arguments);
		this._missingMsg = i18n.getLocalization("dijit.form", "validate", this.lang).missingMessage;
	},

	postCreate: function(){
		// summary:
		//		stop mousemove from selecting text on IE to be consistent with other browsers

		this.inherited(arguments);
 
		this.connect(this.domNode, "onselectstart", event.stop);
		this.domNode.setAttribute("aria-expanded", "false");
	},

	_setStyleAttr: function(/*String||Object*/ value){
		this.inherited(arguments);
		domClass.toggle(this.domNode, this.baseClass.replace(/\s+|$/g, "FixedWidth "), !!this.domNode.style.width);
	},

	isLoaded: function(){
		return this._isLoaded;
	},

	loadDropDown: function(/*Function*/ loadCallback){
		// summary:
		//		populates the menu
		this._loadChildren(true);
		this._isLoaded = true;
		loadCallback();
	},

	closeDropDown: function(){
		// overriding _HasDropDown.closeDropDown()
		this.inherited(arguments);

		if(this.dropDown && this.dropDown.menuTableNode){
			// Erase possible width: 100% setting from _SelectMenu.resize().
			// Leaving it would interfere with the next openDropDown() call, which
			// queries the natural size of the drop down.
			this.dropDown.menuTableNode.style.width = "";
		}
	},

	destroy: function(preserveDom){
		if(this.dropDown && !this.dropDown._destroyed){
			this.dropDown.destroyRecursive(preserveDom);
			delete this.dropDown;
		}
		this.inherited(arguments);
	},

	_onFocus: function(){
		this.validate(true);	// show tooltip if second focus of required tooltip, but no selection
		this.inherited(arguments);
	},

	_onBlur: function(){
		Tooltip.hide(this.domNode);
		this.inherited(arguments);
		this.validate(false);
	}
});

Select._Menu = _SelectMenu;	// for monkey patching

return Select;
});

},
'dijit/form/_FormSelectWidget':function(){
define("dijit/form/_FormSelectWidget", [
	"dojo/_base/array", // array.filter array.forEach array.map array.some
	"dojo/_base/Deferred",
	"dojo/aspect", // aspect.after
	"dojo/data/util/sorter", // util.sorter.createSortFunction
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.setSelectable
	"dojo/dom-class", // domClass.toggle
	"dojo/_base/kernel",	// _scopeName
	"dojo/_base/lang", // lang.delegate lang.isArray lang.isObject lang.hitch
	"dojo/query", // query
	"dojo/when",
	"dojo/store/util/QueryResults",
	"./_FormValueWidget"
], function(array, Deferred, aspect, sorter, declare, dom, domClass, kernel, lang, query, when,
			QueryResults, _FormValueWidget){

// module:
//		dijit/form/_FormSelectWidget

/*=====
var __SelectOption = {
	// value: String
	//		The value of the option.  Setting to empty (or missing) will
	//		place a separator at that location
	// label: String
	//		The label for our option.  It can contain html tags.
	// selected: Boolean
	//		Whether or not we are a selected option
	// disabled: Boolean
	//		Whether or not this specific option is disabled
};
=====*/

var _FormSelectWidget = declare("dijit.form._FormSelectWidget", _FormValueWidget, {
	// summary:
	//		Extends _FormValueWidget in order to provide "select-specific"
	//		values - i.e., those values that are unique to `<select>` elements.
	//		This also provides the mechanism for reading the elements from
	//		a store, if desired.

	// multiple: [const] Boolean
	//		Whether or not we are multi-valued
	multiple: false,

	// options: __SelectOption[]
	//		The set of options for our select item.  Roughly corresponds to
	//		the html `<option>` tag.
	options: null,

	// store: dojo/store/api/Store
	//		A store to use for getting our list of options - rather than reading them
	//		from the `<option>` html tags.   Should support getIdentity().
	//		For back-compat store can also be a dojo/data/api/Identity.
	store: null,

	// query: object
	//		A query to use when fetching items from our store
	query: null,

	// queryOptions: object
	//		Query options to use when fetching from the store
	queryOptions: null,

	// labelAttr: String?
	//		The entries in the drop down list come from this attribute in the dojo.store items.
	//		If ``store`` is set, labelAttr must be set too, unless store is an old-style
	//		dojo.data store rather than a new dojo/store.
	labelAttr: "",

	// onFetch: Function
	//		A callback to do with an onFetch - but before any items are actually
	//		iterated over (i.e. to filter even further what you want to add)
	onFetch: null,

	// sortByLabel: Boolean
	//		Flag to sort the options returned from a store by the label of
	//		the store.
	sortByLabel: true,


	// loadChildrenOnOpen: Boolean
	//		By default loadChildren is called when the items are fetched from the
	//		store.  This property allows delaying loadChildren (and the creation
	//		of the options/menuitems) until the user clicks the button to open the
	//		dropdown.
	loadChildrenOnOpen: false,

	// onLoadDeferred: [readonly] dojo.Deferred
	//		This is the `dojo.Deferred` returned by setStore().
	//		Calling onLoadDeferred.then() registers your
	//		callback to be called only once, when the prior setStore completes.
	onLoadDeferred: null,

	getOptions: function(/*anything*/ valueOrIdx){
		// summary:
		//		Returns a given option (or options).
		// valueOrIdx:
		//		If passed in as a string, that string is used to look up the option
		//		in the array of options - based on the value property.
		//		(See dijit/form/_FormSelectWidget.__SelectOption).
		//
		//		If passed in a number, then the option with the given index (0-based)
		//		within this select will be returned.
		//
		//		If passed in a dijit/form/_FormSelectWidget.__SelectOption, the same option will be
		//		returned if and only if it exists within this select.
		//
		//		If passed an array, then an array will be returned with each element
		//		in the array being looked up.
		//
		//		If not passed a value, then all options will be returned
		//
		// returns:
		//		The option corresponding with the given value or index.
		//		null is returned if any of the following are true:
		//
		//		- A string value is passed in which doesn't exist
		//		- An index is passed in which is outside the bounds of the array of options
		//		- A dijit/form/_FormSelectWidget.__SelectOption is passed in which is not a part of the select

		// NOTE: the compare for passing in a dijit/form/_FormSelectWidget.__SelectOption checks
		//		if the value property matches - NOT if the exact option exists
		// NOTE: if passing in an array, null elements will be placed in the returned
		//		array when a value is not found.
		var opts = this.options || [];

		if(valueOrIdx == null){
			return opts; // __SelectOption[]
		}
		if(lang.isArray(valueOrIdx)){
			return array.map(valueOrIdx, "return this.getOptions(item);", this); // __SelectOption[]
		}
		if(lang.isString(valueOrIdx)){
			valueOrIdx = { value: valueOrIdx };
		}
		if(lang.isObject(valueOrIdx)){
			// We were passed an option - so see if it's in our array (directly),
			// and if it's not, try and find it by value.

			if(!array.some(opts, function(option, idx){
				for(var a in valueOrIdx){
					if(!(a in option) || option[a] != valueOrIdx[a]){ // == and not === so that 100 matches '100'
						return false;
					}
				}
				valueOrIdx = idx;
				return true; // stops iteration through opts
			})){
				valueOrIdx = -1;
			}
		}
		if(valueOrIdx >= 0 && valueOrIdx < opts.length){
			return opts[valueOrIdx]; // __SelectOption
		}
		return null; // null
	},

	addOption: function(/*__SelectOption|__SelectOption[]*/ option){
		// summary:
		//		Adds an option or options to the end of the select.  If value
		//		of the option is empty or missing, a separator is created instead.
		//		Passing in an array of options will yield slightly better performance
		//		since the children are only loaded once.
		array.forEach(lang.isArray(option) ? option : [option], function(i){
			if(i && lang.isObject(i)){
				this.options.push(i);
			}
		}, this);
		this._loadChildren();
	},

	removeOption: function(/*String|__SelectOption|Number|Array*/ valueOrIdx){
		// summary:
		//		Removes the given option or options.  You can remove by string
		//		(in which case the value is removed), number (in which case the
		//		index in the options array is removed), or select option (in
		//		which case, the select option with a matching value is removed).
		//		You can also pass in an array of those values for a slightly
		//		better performance since the children are only loaded once.
		//		For numeric option values, specify {value: number} as the argument.
		var oldOpts = this.getOptions(lang.isArray(valueOrIdx) ? valueOrIdx : [valueOrIdx]);
		array.forEach(oldOpts, function(option){
			// We can get null back in our array - if our option was not found.  In
			// that case, we don't want to blow up...
			if(option){
				this.options = array.filter(this.options, function(node){
					return (node.value !== option.value || node.label !== option.label);
				});
				this._removeOptionItem(option);
			}
		}, this);
		this._loadChildren();
	},

	updateOption: function(/*__SelectOption|__SelectOption[]*/ newOption){
		// summary:
		//		Updates the values of the given option.  The option to update
		//		is matched based on the value of the entered option.  Passing
		//		in an array of new options will yield better performance since
		//		the children will only be loaded once.
		array.forEach(lang.isArray(newOption) ? newOption : [newOption], function(i){
			var oldOpt = this.getOptions({ value: i.value }), k;
			if(oldOpt){
				for(k in i){ oldOpt[k] = i[k]; }
			}
		}, this);
		this._loadChildren();
	},

	setStore: function(store, selectedValue, fetchArgs){
		// summary:
		//		Sets the store you would like to use with this select widget.
		//		The selected value is the value of the new store to set.  This
		//		function returns the original store, in case you want to reuse
		//		it or something.
		// store: dojo/store/api/Store
		//		The dojo.store you would like to use - it MUST implement getIdentity()
		//		and MAY implement observe().
		//		For backwards-compatibility this can also be a data.data store, in which case
		//		it MUST implement dojo/data/api/Identity,
		//		and MAY implement dojo/data/api/Notification.
		// selectedValue: anything?
		//		The value that this widget should set itself to *after* the store
		//		has been loaded
		// fetchArgs: Object?
		//		Hash of parameters to set filter on store, etc.
		//
		//		- query: new value for Select.query,
		//		- queryOptions: new value for Select.queryOptions,
		//		- onFetch: callback function for each item in data (Deprecated)
		var oStore = this.store;
		fetchArgs = fetchArgs || {};

		if(oStore !== store){
			// Our store has changed, so cancel any listeners on old store (remove for 2.0)
			var h;
			while((h = this._notifyConnections.pop())){ h.remove(); }

			// For backwards-compatibility, accept dojo.data store in addition to dojo.store.store.  Remove in 2.0.
			if(!store.get){
				lang.mixin(store, {
					_oldAPI: true,
					get: function(id){
						// summary:
						//		Retrieves an object by it's identity. This will trigger a fetchItemByIdentity.
						//		Like dojo.store.DataStore.get() except returns native item.
						var deferred = new Deferred();
						this.fetchItemByIdentity({
							identity: id,
							onItem: function(object){
								deferred.resolve(object);
							},
							onError: function(error){
								deferred.reject(error);
							}
						});
						return deferred.promise;
					},
					query: function(query, options){
						// summary:
						//		Queries the store for objects.   Like dojo/store/DataStore.query()
						//		except returned Deferred contains array of native items.
						var deferred = new Deferred(function(){ if(fetchHandle.abort){ fetchHandle.abort(); } } );
						deferred.total = new Deferred();
						var fetchHandle = this.fetch(lang.mixin({
							query: query,
							onBegin: function(count){
								deferred.total.resolve(count);
							},
							onComplete: function(results){
								deferred.resolve(results);
							},
							onError: function(error){
								deferred.reject(error);
							}
						}, options));
						return new QueryResults(deferred);
					}
				});

				if(store.getFeatures()["dojo.data.api.Notification"]){
					this._notifyConnections = [
						aspect.after(store, "onNew", lang.hitch(this, "_onNewItem"), true),
						aspect.after(store, "onDelete", lang.hitch(this, "_onDeleteItem"), true),
						aspect.after(store, "onSet", lang.hitch(this, "_onSetItem"), true)
					];
				}
			}
			this._set("store", store);			// Our store has changed, so update our notifications
		}

		// Remove existing options (if there are any)
		if(this.options && this.options.length){
			this.removeOption(this.options);
		}

			// Cancel listener for updates to old (dojo.data) store
		if(this._queryRes && this._queryRes.close){
			this._queryRes.close();
		}
			
			// Cancel listener for updates to new (dojo.store) store
			if(this._observeHandle && this._observeHandle.remove){
				this._observeHandle.remove();
				this._observeHandle = null;
			}

		// If user has specified new query and query options along with this new store, then use them.
		if(fetchArgs.query){
			this._set("query", fetchArgs.query);
			this._set("queryOptions", fetchArgs.queryOptions);
		}

		// Add our new options
		if(store){
			this._loadingStore = true;
			this.onLoadDeferred = new Deferred();

			// Run query
			// Save result in this._queryRes so we can cancel the listeners we register below
			this._queryRes = store.query(this.query, this.queryOptions);
			when(this._queryRes, lang.hitch(this, function(items){

				if(this.sortByLabel && !fetchArgs.sort && items.length){
					if(store.getValue){
						// Old dojo.data API to access items, remove for 2.0
						items.sort(sorter.createSortFunction([{
							attribute: store.getLabelAttributes(items[0])[0]
						}], store));
					}else{
						var labelAttr = this.labelAttr;
						items.sort(function(a, b){
							return a[labelAttr] > b[labelAttr] ? 1 :  b[labelAttr] > a[labelAttr] ? -1 : 0;
						});
					}
				}

				if(fetchArgs.onFetch){
						items = fetchArgs.onFetch.call(this, items, fetchArgs);
				}

				// TODO: Add these guys as a batch, instead of separately
				array.forEach(items, function(i){
					this._addOptionForItem(i);
				}, this);

				// Register listener for store updates
				if(this._queryRes.observe){
						// observe returns yet another handle that needs its own explicit gc
						this._observeHandle = this._queryRes.observe(lang.hitch(this, function(object, deletedFrom, insertedInto){
						if(deletedFrom == insertedInto){
							this._onSetItem(object);
						}else{
							if(deletedFrom != -1){
								this._onDeleteItem(object);
							}
							if(insertedInto != -1){
								this._onNewItem(object);
							}
						}
					}), true);
				}

				// Set our value (which might be undefined), and then tweak
				// it to send a change event with the real value
				this._loadingStore = false;
				this.set("value", "_pendingValue" in this ? this._pendingValue : selectedValue);
				delete this._pendingValue;

				if(!this.loadChildrenOnOpen){
					this._loadChildren();
				}else{
					this._pseudoLoadChildren(items);
				}
				this.onLoadDeferred.resolve(true);
				this.onSetStore();
			}), function(err){
					console.error('dijit.form.Select: ' + err.toString());
					this.onLoadDeferred.reject(err);
			});
		}
		return oStore;	// dojo/data/api/Identity
	},

	// TODO: implement set() and watch() for store and query, although not sure how to handle
	// setting them individually rather than together (as in setStore() above)

	_setValueAttr: function(/*anything*/ newValue, /*Boolean?*/ priorityChange){
		// summary:
		//		set the value of the widget.
		//		If a string is passed, then we set our value from looking it up.
		if(!this._onChangeActive){ priorityChange = null; }
		if(this._loadingStore){
			// Our store is loading - so save our value, and we'll set it when
			// we're done
			this._pendingValue = newValue;
			return;
		}
		if(newValue == null){
			return;
		}
		if(lang.isArray(newValue)){
			newValue = array.map(newValue, function(value){ return lang.isObject(value) ? value : { value: value }; }); // __SelectOption[]
		}else if(lang.isObject(newValue)){
			newValue = [newValue];
		}else{
			newValue = [{ value: newValue }];
		}
		newValue = array.filter(this.getOptions(newValue), function(i){ return i && i.value; });
		var opts = this.getOptions() || [];
		if(!this.multiple && (!newValue[0] || !newValue[0].value) && !!opts.length){
			newValue[0] = opts[0];
		}
		array.forEach(opts, function(opt){
			opt.selected = array.some(newValue, function(v){ return v.value === opt.value; });
		});
		var val = array.map(newValue, function(opt){ return opt.value; });

		if(typeof val == "undefined" || typeof val[0] == "undefined"){ return; } // not fully initialized yet or a failed value lookup
		var disp = array.map(newValue, function(opt){ return opt.label; });
		this._setDisplay(this.multiple ? disp : disp[0]);
		this.inherited(arguments, [ this.multiple ? val : val[0], priorityChange ]);
		this._updateSelection();
	},

	_getDisplayedValueAttr: function(){
		// summary:
		//		returns the displayed value of the widget
		var ret = array.map([].concat(this.get('selectedOptions')), function(v){
			if(v && "label" in v){
				return v.label;
			}else if(v){
				return v.value;
			}
			return null;
		}, this);
		return this.multiple ? ret : ret[0];
	},

	_setDisplayedValueAttr: function(label){
		// summary:
		//		Sets the displayed value of the widget
		this.set('value', this.getOptions(typeof label == "string" ? { label: label } : label));
	},

	_loadChildren: function(){
		// summary:
		//		Loads the children represented by this widget's options.
		//		reset the menu to make it populatable on the next click
		if(this._loadingStore){ return; }
		array.forEach(this._getChildren(), function(child){
			child.destroyRecursive();
		});
		// Add each menu item
		array.forEach(this.options, this._addOptionItem, this);

		// Update states
		this._updateSelection();
	},

	_updateSelection: function(){
		// summary:
		//		Sets the "selected" class on the item for styling purposes
		this._set("value", this._getValueFromOpts());
		var val = [].concat(this.value);
		if(val && val[0]){
			array.forEach(this._getChildren(), function(child){
				var isSelected = array.some(val, function(v){
					return child.option && (v === child.option.value);
				});
				domClass.toggle(child.domNode, this.baseClass.replace(/\s+|$/g, "SelectedOption "), isSelected);
				child.domNode.setAttribute("aria-selected", isSelected ? "true" : "false");
			}, this);
		}
	},

	_getValueFromOpts: function(){
		// summary:
		//		Returns the value of the widget by reading the options for
		//		the selected flag
		var opts = this.getOptions() || [];
		if(!this.multiple && opts.length){
			// Mirror what a select does - choose the first one
			var opt = array.filter(opts, function(i){
				return i.selected;
			})[0];
			if(opt && opt.value){
				return opt.value;
			}else{
				opts[0].selected = true;
				return opts[0].value;
			}
		}else if(this.multiple){
			// Set value to be the sum of all selected
			return array.map(array.filter(opts, function(i){
				return i.selected;
			}), function(i){
				return i.value;
			}) || [];
		}
		return "";
	},

	// Internal functions to call when we have store notifications come in
	_onNewItem: function(/*item*/ item, /*Object?*/ parentInfo){
		if(!parentInfo || !parentInfo.parent){
			// Only add it if we are top-level
			this._addOptionForItem(item);
		}
	},
	_onDeleteItem: function(/*item*/ item){
		var store = this.store;
		this.removeOption({value: store.getIdentity(item) });
	},
	_onSetItem: function(/*item*/ item){
		this.updateOption(this._getOptionObjForItem(item));
	},

	_getOptionObjForItem: function(item){
		// summary:
		//		Returns an option object based off the given item.  The "value"
		//		of the option item will be the identity of the item, the "label"
		//		of the option will be the label of the item.

		// remove getLabel() call for 2.0 (it's to support the old dojo.data API)
		var store = this.store,
			label = (this.labelAttr && this.labelAttr in item) ? item[this.labelAttr] : store.getLabel(item),
			value = (label ? store.getIdentity(item) : null);
		return {value: value, label: label, item: item}; // __SelectOption
	},

	_addOptionForItem: function(/*item*/ item){
		// summary:
		//		Creates (and adds) the option for the given item
		var store = this.store;
		if(store.isItemLoaded && !store.isItemLoaded(item)){
			// We are not loaded - so let's load it and add later.
			// Remove for 2.0 (it's the old dojo.data API)
			store.loadItem({item: item, onItem: function(i){
				this._addOptionForItem(i);
			},
			scope: this});
			return;
		}
		var newOpt = this._getOptionObjForItem(item);
		this.addOption(newOpt);
	},

	constructor: function(params /*===== , srcNodeRef =====*/){
		// summary:
		//		Create the widget.
		// params: Object|null
		//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
		//		and functions, typically callbacks like onClick.
		//		The hash can contain any of the widget's properties, excluding read-only properties.
		// srcNodeRef: DOMNode|String?
		//		If a srcNodeRef (DOM node) is specified, replace srcNodeRef with my generated DOM tree

		//		Saves off our value, if we have an initial one set so we
		//		can use it if we have a store as well (see startup())
		this._oValue = (params || {}).value || null;
		this._notifyConnections = [];	// remove for 2.0
	},

	buildRendering: function(){
		this.inherited(arguments);
		dom.setSelectable(this.focusNode, false);
	},

	_fillContent: function(){
		// summary:
		//		Loads our options and sets up our dropdown correctly.  We
		//		don't want any content, so we don't call any inherit chain
		//		function.
		if(!this.options){
			this.options =
				this.srcNodeRef
				? query("> *", this.srcNodeRef).map(
					function(node){
						if(node.getAttribute("type") === "separator"){
							return { value: "", label: "", selected: false, disabled: false };
						}
						return {
							value: (node.getAttribute("data-" + kernel._scopeName + "-value") || node.getAttribute("value")),
							label: String(node.innerHTML),
							// FIXME: disabled and selected are not valid on complex markup children (which is why we're
							// looking for data-dojo-value above.  perhaps we should data-dojo-props="" this whole thing?)
							// decide before 1.6
							selected: node.getAttribute("selected") || false,
							disabled: node.getAttribute("disabled") || false
						};
					},
					this)
				: [];
		}
		if(!this.value){
			this._set("value", this._getValueFromOpts());
		}else if(this.multiple && typeof this.value == "string"){
			this._set("value", this.value.split(","));
		}
	},

	postCreate: function(){
		// summary:
		//		sets up our event handling that we need for functioning
		//		as a select
		this.inherited(arguments);

		// Make our event connections for updating state
		aspect.after(this, "onChange", lang.hitch(this, "_updateSelection"));

		// moved from startup
		//		Connects in our store, if we have one defined
		var store = this.store;
		if(store && (store.getIdentity || store.getFeatures()["dojo.data.api.Identity"])){
			// Temporarily set our store to null so that it will get set
			// and connected appropriately
			this.store = null;
			this.setStore(store, this._oValue);
		}
	},

	startup: function(){
		// summary:
		this._loadChildren();
		this.inherited(arguments);
	},

	destroy: function(){
		// summary:
		//		Clean up our connections

		var h;
		while((h = this._notifyConnections.pop())){ h.remove(); }

		// Cancel listener for store updates
		if(this._queryRes && this._queryRes.close){
			this._queryRes.close();
			}

			// Cancel listener for updates to new (dojo.store) store
			if(this._observeHandle && this._observeHandle.remove){
				this._observeHandle.remove();
				this._observeHandle = null;
		}

		this.inherited(arguments);
	},

	_addOptionItem: function(/*__SelectOption*/ /*===== option =====*/){
		// summary:
		//		User-overridable function which, for the given option, adds an
		//		item to the select.  If the option doesn't have a value, then a
		//		separator is added in that place.  Make sure to store the option
		//		in the created option widget.
	},

	_removeOptionItem: function(/*__SelectOption*/ /*===== option =====*/){
		// summary:
		//		User-overridable function which, for the given option, removes
		//		its item from the select.
	},

	_setDisplay: function(/*String or String[]*/ /*===== newDisplay =====*/){
		// summary:
		//		Overridable function which will set the display for the
		//		widget.  newDisplay is either a string (in the case of
		//		single selects) or array of strings (in the case of multi-selects)
	},

	_getChildren: function(){
		// summary:
		//		Overridable function to return the children that this widget contains.
		return [];
	},

	_getSelectedOptionsAttr: function(){
		// summary:
		//		hooks into this.attr to provide a mechanism for getting the
		//		option items for the current value of the widget.
		return this.getOptions({ selected: true });
	},

	_pseudoLoadChildren: function(/*item[]*/ /*===== items =====*/){
		// summary:
		//		a function that will "fake" loading children, if needed, and
		//		if we have set to not load children until the widget opens.
		// items:
		//		An array of items that will be loaded, when needed
	},

	onSetStore: function(){
		// summary:
		//		a function that can be connected to in order to receive a
		//		notification that the store has finished loading and all options
		//		from that store are available
	}
});

/*=====
_FormSelectWidget.__SelectOption = __SelectOption;
=====*/

return _FormSelectWidget;

});

},
'dojo/data/util/sorter':function(){
define("dojo/data/util/sorter", ["../../_base/lang"], function(lang){
	// module:
	//		dojo/data/util/sorter
	// summary:
	//		TODOC

var sorter = {};
lang.setObject("dojo.data.util.sorter", sorter);

sorter.basicComparator = function(	/*anything*/ a,
													/*anything*/ b){
	// summary:
	//		Basic comparison function that compares if an item is greater or less than another item
	// description:
	//		returns 1 if a > b, -1 if a < b, 0 if equal.
	//		'null' values (null, undefined) are treated as larger values so that they're pushed to the end of the list.
	//		And compared to each other, null is equivalent to undefined.

	//null is a problematic compare, so if null, we set to undefined.
	//Makes the check logic simple, compact, and consistent
	//And (null == undefined) === true, so the check later against null
	//works for undefined and is less bytes.
	var r = -1;
	if(a === null){
		a = undefined;
	}
	if(b === null){
		b = undefined;
	}
	if(a == b){
		r = 0;
	}else if(a > b || a == null){
		r = 1;
	}
	return r; //int {-1,0,1}
};

sorter.createSortFunction = function(	/* attributes[] */sortSpec, /*dojo/data/api/Read*/ store){
	// summary:
	//		Helper function to generate the sorting function based off the list of sort attributes.
	// description:
	//		The sort function creation will look for a property on the store called 'comparatorMap'.  If it exists
	//		it will look in the mapping for comparisons function for the attributes.  If one is found, it will
	//		use it instead of the basic comparator, which is typically used for strings, ints, booleans, and dates.
	//		Returns the sorting function for this particular list of attributes and sorting directions.
	// sortSpec:
	//		A JS object that array that defines out what attribute names to sort on and whether it should be descenting or asending.
	//		The objects should be formatted as follows:
	// |	{
	// |		attribute: "attributeName-string" || attribute,
	// |		descending: true|false;   // Default is false.
	// |	}
	// store:
	//		The datastore object to look up item values from.

	var sortFunctions=[];

	function createSortFunction(attr, dir, comp, s){
		//Passing in comp and s (comparator and store), makes this
		//function much faster.
		return function(itemA, itemB){
			var a = s.getValue(itemA, attr);
			var b = s.getValue(itemB, attr);
			return dir * comp(a,b); //int
		};
	}
	var sortAttribute;
	var map = store.comparatorMap;
	var bc = sorter.basicComparator;
	for(var i = 0; i < sortSpec.length; i++){
		sortAttribute = sortSpec[i];
		var attr = sortAttribute.attribute;
		if(attr){
			var dir = (sortAttribute.descending) ? -1 : 1;
			var comp = bc;
			if(map){
				if(typeof attr !== "string" && ("toString" in attr)){
					 attr = attr.toString();
				}
				comp = map[attr] || bc;
			}
			sortFunctions.push(createSortFunction(attr,
				dir, comp, store));
		}
	}
	return function(rowA, rowB){
		var i=0;
		while(i < sortFunctions.length){
			var ret = sortFunctions[i++](rowA, rowB);
			if(ret !== 0){
				return ret;//int
			}
		}
		return 0; //int
	}; // Function
};

return sorter;
});

},
'url:dijit/form/templates/Select.html':"<table class=\"dijit dijitReset dijitInline dijitLeft\"\n\tdata-dojo-attach-point=\"_buttonNode,tableNode,focusNode,_popupStateNode\" cellspacing='0' cellpadding='0'\n\trole=\"listbox\" aria-haspopup=\"true\"\n\t><tbody role=\"presentation\"><tr role=\"presentation\"\n\t\t><td class=\"dijitReset dijitStretch dijitButtonContents\" role=\"presentation\"\n\t\t\t><div class=\"dijitReset dijitInputField dijitButtonText\"  data-dojo-attach-point=\"containerNode\" role=\"presentation\"></div\n\t\t\t><div class=\"dijitReset dijitValidationContainer\"\n\t\t\t\t><input class=\"dijitReset dijitInputField dijitValidationIcon dijitValidationInner\" value=\"&#935; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t\t\t/></div\n\t\t\t><input type=\"hidden\" ${!nameAttrSetting} data-dojo-attach-point=\"valueNode\" value=\"${value}\" aria-hidden=\"true\"\n\t\t/></td\n\t\t><td class=\"dijitReset dijitRight dijitButtonNode dijitArrowButton dijitDownArrowButton dijitArrowButtonContainer\"\n\t\t\tdata-dojo-attach-point=\"titleNode\" role=\"presentation\"\n\t\t\t><input class=\"dijitReset dijitInputField dijitArrowButtonInner\" value=\"&#9660; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t\t\t\t${_buttonInputDisabled}\n\t\t/></td\n\t></tr></tbody\n></table>\n",
'epi-find/widget/Schedule':function(){
define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/date/locale",
    "dojo/_base/event",

    "dojo/dom-attr",
    "dojo/dom-class",

    "dojo/store/Memory",

    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",

    "dijit/form/Select",
    "dijit/form/ComboBox",
    "dijit/form/RadioButton",
    "dijit/form/TimeTextBox",
    "dijit/form/ValidationTextBox",
    "dojox/lang/functional",
    "dojox/validate",

    "dojo/text!./templates/Schedule.html",
    "dojo/i18n!./nls/Schedule"
],
    function( declare, lang, array, locale, event,
             domAttr, domClass,
             Memory,
             _WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin,

             Select, ComboBox, RadioButton, TimeTextBox, ValidationTextBox,
             functional, validate,
             template, i18n
        ) {

        var DailyLabel = declare([_WidgetBase, _TemplatedMixin], {
            templateString: "<label data-dojo-attach-point=\"label\">"+ i18n.dailySchedule
                .replace("{time}", "<span data-dojo-attach-point=\"timeNode\"></span> ")+"</label>",
            time: null,
            inputId: null,
            schedule: null,
            _setScheduleAttr: function(value) {
                this.time.set("value", value ? value.time : "");
                this._set("schedule", value);
            },
            disabled: false,
            _setDisabledAttr: function(value) {
                if (this.time) {
                    this.time.set("disabled", value);
                }
                this._set("disabled", value);
            },
            postCreate: function() {
                this.inherited(arguments);
                this.containerNode = this.domNode;
                var self = this;
                this.time = new TimeTextBox({
                    "class": "epi-chromeless epi-chromeless--with-arrow epi-flat",
                    style: { width: "100px" },
                    required: true,
                    disabled: this.disabled
                }, this.timeNode);
                this.own(
                    this.time,
                    this.time.watch("value", function() {
                        if (self.time.isValid()) {
                            self._set("schedule", {type: "daily", time: self.time.get("value")});
                        }
                    })
                );
                domAttr.set(this.label, "for", this.inputId);
            }
        });

        var WeeklyLabel = declare([_WidgetBase, _TemplatedMixin], {
            templateString: "<label data-dojo-attach-point=\"label\">"+ i18n.weeklySchedule
                .replace("{time}", "<span data-dojo-attach-point=\"timeNode\"></span>")
                .replace("{dayOfTheWeek}", "<span data-dojo-attach-point=\"dayOfTheWeekNode\"></span>")+"</label>",
            time: null,
            dayOfTheWeek: null,
            inputId: null,
            schedule: null,
            _setScheduleAttr: function(value) {
                this.time.set("value", value ? value.time : "");
                this.dayOfTheWeek.set("value", value ? value.dayOfTheWeek : "");
                this._set("schedule", value);
            },
            disabled: false,
            _setDisabledAttr: function(value) {
                if (this.time) {
                    this.time.set("disabled", value);
                }
                if (this.dayOfTheWeek) {
                    this.dayOfTheWeek.set("disabled", value);
                }
                this._set("disabled", value);
            },
            updateSchedule: function() {
                if (this.time.isValid() && this.dayOfTheWeek.isValid()){
                    this._set("schedule", {type: "weekly", time: this.time.get("value"), dayOfTheWeek: this.dayOfTheWeek.get("value")});
                }
            },
            postCreate: function() {
                this.inherited(arguments);
                this.containerNode = this.domNode;
                this.time = new TimeTextBox({
                    "class": "epi-chromeless epi-chromeless--with-arrow epi-flat",
                    style: { width: "100px" },
                    required: true,
                    disabled: this.disabled
                }, this.timeNode);
                this.own(
                    this.time,
                    this.time.watch("value", lang.hitch(this, this.updateSchedule))
                );
                var weekdaysSelectOptions = array.map([i18n.weekdays_monday, i18n.weekdays_tuesday, i18n.weekdays_wednesday, i18n.weekdays_thursday, i18n.weekdays_friday, i18n.weekdays_saturday, i18n.weekdays_sunday], function(item, index) {
                    return { label: item, value: (index + 1).toString() };
                });
                this.dayOfTheWeek = new Select({
                    "class": "epi-chromeless epi-chromeless--with-arrow epi-flat",
                    options: weekdaysSelectOptions,
                    disabled: this.disabled
                }, this.dayOfTheWeekNode);
                this.own(
                    this.dayOfTheWeek,
                    this.dayOfTheWeek.watch("value", lang.hitch(this, this.updateSchedule))
                );
                domAttr.set(this.label, "for", this.inputId);
            }
        });

        var CustomLabel = declare([_WidgetBase, _TemplatedMixin], {
            templateString: "<label data-dojo-attach-point=\"label\" class=\"epi-hidden\">"+ i18n.customSchedule
                .replace("{cronSchedule}", "<span data-dojo-attach-point=\"valueNode\"></span> ")+"</label>",
            cronSchedule: null,
            inputId: null,
            schedule: null,
            _setScheduleAttr: function(value) {
                this.cronSchedule.set("value", value ? value.cronSchedule : "");
                this._set("schedule", value);
            },
            disabled: false,
            _setDisabledAttr: function(value) {
                if (this.cronSchedule) {
                    this.cronSchedule.set("disabled", value);
                }
                this._set("disabled", value);
            },

            postCreate: function() {
                this.inherited(arguments);
                this.containerNode = this.domNode;
                var self = this;

                this.cronSchedule = new ValidationTextBox({
                    "class": "epi-chromeless epi-flat",
                    required: true,
                    disabled: this.disabled
                }, this.valueNode);
                this.own(
                    this.cronSchedule,
                    this.cronSchedule.watch("value", function() {
                        if (self.cronSchedule.isValid()) {
                            self._set("schedule", {type: "custom", cronSchedule: self.cronSchedule.get("value")});
                        }
                    })
                );
                domAttr.set(this.label, "for", this.inputId);
            }
        });

        // module:
        //      epi-find/widget/Schedule

        return declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin], {
            // summary:
            //     Connector settings schedule

            i18n: i18n,

            model: null,

            daily: null,
            weekly: null,
            custom: null,

            schedule: null,
            // summary:
            //      Schedule as an object with a "cron" property containing a string in cron format (http://en.wikipedia.org/wiki/Cron#Predefined_scheduling_definitions)
            _setScheduleAttr: function(value) {

                if (!value || !value.cron) {
                    this.noSchedule.set("checked", true);
                    this._hideCustomField();
                    this._set("schedule", value);
                }
                else {
                    if (this.schedule && value.cron === this.schedule.cron) {
                        return;
                    }
                    this._set("schedule", value);
                    this._hideCustomField();

                    var schedule = this.model.changeScheduleTZ(this.model.parseCronSchedule(value.cron), -this.model.offset);
                    if (schedule.type === "daily") {
                        this.dailySchedule.set("checked", true);
                        this.daily.set("schedule", schedule);
                    }
                    else if (schedule.type === "weekly") {
                        this.weeklySchedule.set("checked", true);
                        this.weekly.set("schedule", schedule);
                    }
                    else if (schedule.type === "custom") {
                        this._showCustomField();
                        this.customSchedule.set("checked", true);
                        this.custom.set("schedule", schedule);
                    }
                }
            },

            templateString: template,

            postCreate: function() {
                this.inherited(arguments);
                var self = this;
                this.containerNode = this.domNode;
                this.daily = new DailyLabel({inputId: "dailySchedule", disabled: true}, this.dailyNode);
                this.weekly = new WeeklyLabel({inputId: "weeklySchedule", disabled: true}, this.weeklyNode);
                this.custom = new CustomLabel({inputId: "customSchedule", disabled: true}, this.customNode);

                this.own(
                    this.daily,
                    this.weekly,
                    this.custom,
                    this.daily.watch("schedule", lang.hitch(this, function() { this._updateSchedule(this.dailySchedule); })),
                    this.weekly.watch("schedule", lang.hitch(this, function() { this._updateSchedule(this.weeklySchedule); })),
                    this.custom.watch("schedule", lang.hitch(this, function() { this._updateSchedule(this.customSchedule); })),
                    this.noSchedule.watch("checked", lang.hitch(this, function(prop, oldValue, newValue) {
                        if (newValue) {
                           this._updateSchedule();
                        }
                    })),
                    this.dailySchedule.watch("checked", function(prop, oldValue, newValue) {
                        self.daily.set("disabled", !newValue);
                    }),
                    this.weeklySchedule.watch("checked", function(prop, oldValue, newValue) {
                        self.weekly.set("disabled", !newValue);
                    }),
                    this.customSchedule.watch("checked", function(prop, oldValue, newValue) {
                        self.custom.set("disabled", !newValue);
                    })
                );
                this._hideCustomField();
            },

            _updateSchedule: function(checkbox) {
                var result;
                if (checkbox) {
                    checkbox.set("checked", true);
                }

                if (this.noSchedule.get("checked")) {
                    result = null;
                }
                else if (this.dailySchedule.get("checked")) {
                    result = this.daily.get("schedule");
                }
                else if (this.weeklySchedule.get("checked")) {
                    result = this.weekly.get("schedule");
                }
                else if (this.customSchedule.get("checked")) {
                    result = this.custom.get("schedule");
                }
                this.set("schedule", this.model.serialiseToCronSchedule(this.model.changeScheduleTZ(result, this.model.offset)));
            },

            _showCustomField: function() {
                this.customSchedule.set("disabled", false);
                this.custom.set("disabled", false);

                domClass.remove(this.customSchedule.domNode, "epi-hidden");
                domClass.remove(this.custom.domNode, "epi-hidden");
            },

            _hideCustomField: function() {
                this.customSchedule.set("disabled", true);
                this.custom.set("disabled", true);

                domClass.add(this.customSchedule.domNode, "epi-hidden");
                domClass.add(this.custom.domNode, "epi-hidden");
            }

        });

    });

},
'dijit/form/ComboBox':function(){
define("dijit/form/ComboBox", [
	"dojo/_base/declare", // declare
	"./ValidationTextBox",
	"./ComboBoxMixin"
], function(declare, ValidationTextBox, ComboBoxMixin){

	// module:
	//		dijit/form/ComboBox

	return declare("dijit.form.ComboBox", [ValidationTextBox, ComboBoxMixin], {
		// summary:
		//		Auto-completing text box
		//
		// description:
		//		The drop down box's values are populated from an class called
		//		a data provider, which returns a list of values based on the characters
		//		that the user has typed into the input box.
		//		If OPTION tags are used as the data provider via markup,
		//		then the OPTION tag's child text node is used as the widget value
		//		when selected.  The OPTION tag's value attribute is ignored.
		//		To set the default value when using OPTION tags, specify the selected
		//		attribute on 1 of the child OPTION tags.
		//
		//		Some of the options to the ComboBox are actually arguments to the data
		//		provider.
	});
});

},
'dijit/form/ComboBoxMixin':function(){
require({cache:{
'url:dijit/form/templates/DropDownBox.html':"<div class=\"dijit dijitReset dijitInline dijitLeft\"\n\tid=\"widget_${id}\"\n\trole=\"combobox\"\n\taria-haspopup=\"true\"\n\tdata-dojo-attach-point=\"_popupStateNode\"\n\t><div class='dijitReset dijitRight dijitButtonNode dijitArrowButton dijitDownArrowButton dijitArrowButtonContainer'\n\t\tdata-dojo-attach-point=\"_buttonNode\" role=\"presentation\"\n\t\t><input class=\"dijitReset dijitInputField dijitArrowButtonInner\" value=\"&#9660; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"button presentation\" aria-hidden=\"true\"\n\t\t\t${_buttonInputDisabled}\n\t/></div\n\t><div class='dijitReset dijitValidationContainer'\n\t\t><input class=\"dijitReset dijitInputField dijitValidationIcon dijitValidationInner\" value=\"&#935; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t/></div\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\n\t\t><input class='dijitReset dijitInputInner' ${!nameAttrSetting} type=\"text\" autocomplete=\"off\"\n\t\t\tdata-dojo-attach-point=\"textbox,focusNode\" role=\"textbox\"\n\t/></div\n></div>\n"}});
define("dijit/form/ComboBoxMixin", [
	"dojo/_base/declare", // declare
	"dojo/_base/Deferred",
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.mixin
	"dojo/store/util/QueryResults",
	"./_AutoCompleterMixin",
	"./_ComboBoxMenu",
	"../_HasDropDown",
	"dojo/text!./templates/DropDownBox.html"
], function(declare, Deferred, kernel, lang, QueryResults, _AutoCompleterMixin, _ComboBoxMenu, _HasDropDown, template){


	// module:
	//		dijit/form/ComboBoxMixin

	return declare("dijit.form.ComboBoxMixin", [_HasDropDown, _AutoCompleterMixin], {
		// summary:
		//		Provides main functionality of ComboBox widget

		// dropDownClass: [protected extension] Function String
		//		Dropdown widget class used to select a date/time.
		//		Subclasses should specify this.
		dropDownClass: _ComboBoxMenu,

		// hasDownArrow: Boolean
		//		Set this textbox to have a down arrow button, to display the drop down list.
		//		Defaults to true.
		hasDownArrow: true,

		templateString: template,

		baseClass: "dijitTextBox dijitComboBox",

		/*=====
		// store: [const] dojo/store/api/Store|dojo/data/api/Read
		//		Reference to data provider object used by this ComboBox.
		//
		//		Should be dojo/store/api/Store, but dojo/data/api/Read supported
		//		for backwards compatibility.
		store: null,
		=====*/

		// Set classes like dijitDownArrowButtonHover depending on
		// mouse action over button node
		cssStateNodes: {
			"_buttonNode": "dijitDownArrowButton"
		},

		_setHasDownArrowAttr: function(/*Boolean*/ val){
			this._set("hasDownArrow", val);
			this._buttonNode.style.display = val ? "" : "none";
		},

		_showResultList: function(){
			// hide the tooltip
			this.displayMessage("");
			this.inherited(arguments);
		},

		_setStoreAttr: function(store){
			// For backwards-compatibility, accept dojo.data store in addition to dojo/store/api/Store.  Remove in 2.0.
			if(!store.get){
				lang.mixin(store, {
					_oldAPI: true,
					get: function(id){
						// summary:
						//		Retrieves an object by it's identity. This will trigger a fetchItemByIdentity.
						//		Like dojo/store/DataStore.get() except returns native item.
						var deferred = new Deferred();
						this.fetchItemByIdentity({
							identity: id,
							onItem: function(object){
								deferred.resolve(object);
							},
							onError: function(error){
								deferred.reject(error);
							}
						});
						return deferred.promise;
					},
					query: function(query, options){
						// summary:
						//		Queries the store for objects.   Like dojo/store/DataStore.query()
						//		except returned Deferred contains array of native items.
						var deferred = new Deferred(function(){ fetchHandle.abort && fetchHandle.abort(); });
						deferred.total = new Deferred();
						var fetchHandle = this.fetch(lang.mixin({
							query: query,
							onBegin: function(count){
								deferred.total.resolve(count);
							},
							onComplete: function(results){
								deferred.resolve(results);
							},
							onError: function(error){
								deferred.reject(error);
							}
						}, options));
						return QueryResults(deferred);
					}
				});
			}
			this._set("store", store);
		},

		postMixInProperties: function(){
			// Since _setValueAttr() depends on this.store, _setStoreAttr() needs to execute first.
			// Unfortunately, without special code, it ends up executing second.
			var store = this.params.store || this.store;
			if(store){
				this._setStoreAttr(store);
			}

			this.inherited(arguments);

			// User may try to access this.store.getValue() etc.  in a custom labelFunc() function.
			// It's not available with the new data store for handling inline <option> tags, so add it.
			if(!this.params.store && !this.store._oldAPI){
				var clazz = this.declaredClass;
				lang.mixin(this.store, {
					getValue: function(item, attr){
						kernel.deprecated(clazz + ".store.getValue(item, attr) is deprecated for builtin store.  Use item.attr directly", "", "2.0");
						return item[attr];
					},
					getLabel: function(item){
						kernel.deprecated(clazz + ".store.getLabel(item) is deprecated for builtin store.  Use item.label directly", "", "2.0");
						return item.name;
					},
					fetch: function(args){
						kernel.deprecated(clazz + ".store.fetch() is deprecated for builtin store.", "Use store.query()", "2.0");
						var shim = ["dojo/data/ObjectStore"];	// indirection so it doesn't get rolled into a build
						require(shim, lang.hitch(this, function(ObjectStore){
							new ObjectStore({objectStore: this}).fetch(args);
						}));
					}
				});
			}
		}
	});
});

},
'dijit/form/TimeTextBox':function(){
define("dijit/form/TimeTextBox", [
	"dojo/_base/declare", // declare
	"dojo/keys", // keys.DOWN_ARROW keys.ENTER keys.ESCAPE keys.TAB keys.UP_ARROW
	"dojo/_base/lang", // lang.hitch
	"../_TimePicker",
	"./_DateTimeTextBox"
], function(declare, keys, lang, _TimePicker, _DateTimeTextBox){

	// module:
	//		dijit/form/TimeTextBox


	/*=====
	var __Constraints = declare([_DateTimeTextBox.__Constraints, _TimePicker.__Constraints], {
	});
	=====*/

	return declare("dijit.form.TimeTextBox", _DateTimeTextBox, {
		// summary:
		//		A validating, serializable, range-bound time text box with a drop down time picker

		baseClass: "dijitTextBox dijitComboBox dijitTimeTextBox",
		popupClass: _TimePicker,
		_selector: "time",

/*=====
		// constraints: __Constraints
		constraints:{},
=====*/

		// value: Date
		//		The value of this widget as a JavaScript Date object.  Note that the date portion implies time zone and daylight savings rules.
		//
		//		Example:
		// |	new dijit/form/TimeTextBox({value: stamp.fromISOString("T12:59:59", new Date())})
		//
		//		When passed to the parser in markup, must be specified according to locale-independent
		//		`stamp.fromISOString` format.
		//
		//		Example:
		// |	<input data-dojo-type='dijit/form/TimeTextBox' value='T12:34:00'>
		value: new Date(""),		// value.toString()="NaN"
		//FIXME: in markup, you have no control over daylight savings

		_onKey: function(evt){
			if(this.disabled || this.readOnly){ return; }
			this.inherited(arguments);

			// If the user has backspaced or typed some numbers, then filter the result list
			// by what they typed.  Maybe there's a better way to detect this, like _handleOnChange()?
			switch(evt.keyCode){
				case keys.ENTER:
				case keys.TAB:
				case keys.ESCAPE:
				case keys.DOWN_ARROW:
				case keys.UP_ARROW:
					// these keys have special meaning
					break;
				default:
					// defer() because the keystroke hasn't yet appeared in the <input>,
					// so the get('displayedValue') call below won't give the result we want.
					this.defer(function(){
						// set this.filterString to the filter to apply to the drop down list;
						// it will be used in openDropDown()
						var val = this.get('displayedValue');
						this.filterString = (val && !this.parse(val, this.constraints)) ? val.toLowerCase() : "";

						// close the drop down and reopen it, in order to filter the items shown in the list
						// and also since the drop down may need to be repositioned if the number of list items has changed
						// and it's being displayed above the <input>
						if(this._opened){
							this.closeDropDown();
						}
						this.openDropDown();
					});
			}
		}
	});
});

},
'dijit/_TimePicker':function(){
require({cache:{
'url:dijit/templates/TimePicker.html':"<div id=\"widget_${id}\" class=\"dijitMenu\"\n    ><div data-dojo-attach-point=\"upArrow\" class=\"dijitButtonNode dijitUpArrowButton\" data-dojo-attach-event=\"onmouseenter:_buttonMouse,onmouseleave:_buttonMouse\"\n\t\t><div class=\"dijitReset dijitInline dijitArrowButtonInner\" role=\"presentation\">&#160;</div\n\t\t><div class=\"dijitArrowButtonChar\">&#9650;</div></div\n    ><div data-dojo-attach-point=\"timeMenu,focusNode\" data-dojo-attach-event=\"onclick:_onOptionSelected,onmouseover,onmouseout\"></div\n    ><div data-dojo-attach-point=\"downArrow\" class=\"dijitButtonNode dijitDownArrowButton\" data-dojo-attach-event=\"onmouseenter:_buttonMouse,onmouseleave:_buttonMouse\"\n\t\t><div class=\"dijitReset dijitInline dijitArrowButtonInner\" role=\"presentation\">&#160;</div\n\t\t><div class=\"dijitArrowButtonChar\">&#9660;</div></div\n></div>\n"}});
define("dijit/_TimePicker", [
	"dojo/_base/array", // array.forEach
	"dojo/date", // date.compare
	"dojo/date/locale", // locale.format
	"dojo/date/stamp", // stamp.fromISOString stamp.toISOString
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add domClass.contains domClass.toggle
	"dojo/dom-construct", // domConstruct.create
	"dojo/_base/event", // event.stop
	"dojo/_base/kernel", // deprecated
	"dojo/keys", // keys
	"dojo/_base/lang", // lang.mixin
	"dojo/sniff", // has(...)
	"dojo/query", // query
	"dojo/mouse", // mouse.wheel
	"./typematic",
	"./_Widget",
	"./_TemplatedMixin",
	"./form/_FormValueWidget",
	"dojo/text!./templates/TimePicker.html"
], function(array, ddate, locale, stamp, declare, domClass, domConstruct, event, kernel, keys, lang, has, query, mouse,
			typematic, _Widget, _TemplatedMixin, _FormValueWidget, template){

	// module:
	//		dijit/_TimePicker


	var TimePicker = declare("dijit._TimePicker", [_Widget, _TemplatedMixin], {
		// summary:
		//		A graphical time picker.
		//		This widget is used internally by other widgets and is not available
		//		as a standalone widget due to lack of accessibility support.

		templateString: template,

		// baseClass: [protected] String
		//		The root className to use for the various states of this widget
		baseClass: "dijitTimePicker",

		// clickableIncrement: String
		//		ISO-8601 string representing the amount by which
		//		every clickable element in the time picker increases.
		//		Set in local time, without a time zone.
		//		Example: `T00:15:00` creates 15 minute increments
		//		Must divide dijit/_TimePicker.visibleIncrement evenly
		clickableIncrement: "T00:15:00",

		// visibleIncrement: String
		//		ISO-8601 string representing the amount by which
		//		every element with a visible time in the time picker increases.
		//		Set in local time, without a time zone.
		//		Example: `T01:00:00` creates text in every 1 hour increment
		visibleIncrement: "T01:00:00",

		// visibleRange: String
		//		ISO-8601 string representing the range of this TimePicker.
		//		The TimePicker will only display times in this range.
		//		Example: `T05:00:00` displays 5 hours of options
		visibleRange: "T05:00:00",

		// value: String
		//		Date to display.
		//		Defaults to current time and date.
		//		Can be a Date object or an ISO-8601 string.
		//		If you specify the GMT time zone (`-01:00`),
		//		the time will be converted to the local time in the local time zone.
		//		Otherwise, the time is considered to be in the local time zone.
		//		If you specify the date and isDate is true, the date is used.
		//		Example: if your local time zone is `GMT -05:00`,
		//		`T10:00:00` becomes `T10:00:00-05:00` (considered to be local time),
		//		`T10:00:00-01:00` becomes `T06:00:00-05:00` (4 hour difference),
		//		`T10:00:00Z` becomes `T05:00:00-05:00` (5 hour difference between Zulu and local time)
		//		`yyyy-mm-ddThh:mm:ss` is the format to set the date and time
		//		Example: `2007-06-01T09:00:00`
		value: new Date(),

		_visibleIncrement:2,
		_clickableIncrement:1,
		_totalIncrements:10,

		// constraints: TimePicker.__Constraints
		//		Specifies valid range of times (start time, end time)
		constraints:{},

/*=====
		serialize: function(val, options){
			// summary:
			//		User overridable function used to convert the attr('value') result to a String
			// val: Date
			//		The current value
			// options: Object?
			// tags:
			//		protected
		},
=====*/
		serialize: stamp.toISOString,

/*=====
		// filterString: string
		//		The string to filter by
		filterString: "",
=====*/

		setValue: function(/*Date*/ value){
			// summary:
			//		Deprecated.  Used set('value') instead.
			// tags:
			//		deprecated
			kernel.deprecated("dijit._TimePicker:setValue() is deprecated.  Use set('value', ...) instead.", "", "2.0");
			this.set('value', value);
		},

		_setValueAttr: function(/*Date*/ date){
			// summary:
			//		Hook so set('value', ...) works.
			// description:
			//		Set the value of the TimePicker.
			//		Redraws the TimePicker around the new date.
			// tags:
			//		protected
			this._set("value", date);
			this._showText();
		},

		_setFilterStringAttr: function(val){
			// summary:
			//		Called by TimeTextBox to filter the values shown in my list
			this._set("filterString", val);
			this._showText();
		},

		isDisabledDate: function(/*===== dateObject, locale =====*/){
			// summary:
			//		May be overridden to disable certain dates in the TimePicker e.g. `isDisabledDate=locale.isWeekend`
			// dateObject: Date
			// locale: String?
			// type:
			//		extension
			return false; // Boolean
		},

		_getFilteredNodes: function(/*number*/ start, /*number*/ maxNum, /*Boolean*/ before, /*DOMnode*/ lastNode){
			// summary:
			//		Returns an array of nodes with the filter applied.  At most maxNum nodes
			//		will be returned - but fewer may be returned as well.  If the
			//		before parameter is set to true, then it will return the elements
			//		before the given index
			// tags:
			//		private
			var
				nodes = [],
				lastValue = lastNode ? lastNode.date : this._refDate,
				n,
				i = start,
				max = this._maxIncrement + Math.abs(i),
				chk = before ? -1 : 1,
				dec = before ? 1 : 0,
				inc = 1 - dec;
			do{
				i -= dec;
				n = this._createOption(i);
				if(n){
					if((before && n.date > lastValue) || (!before && n.date < lastValue)){
						break; // don't wrap
					}
					nodes[before ? "unshift" : "push"](n);
					lastValue = n.date;
				}
				i += inc;
			}while(nodes.length < maxNum && (i*chk) < max);
			return nodes;
		},

		_showText: function(){
			// summary:
			//		Displays the relevant choices in the drop down list
			// tags:
			//		private
			var fromIso = stamp.fromISOString;
			this.timeMenu.innerHTML = "";
			this._clickableIncrementDate=fromIso(this.clickableIncrement);
			this._visibleIncrementDate=fromIso(this.visibleIncrement);
			this._visibleRangeDate=fromIso(this.visibleRange);
			// get the value of the increments and the range in seconds (since 00:00:00) to find out how many divs to create
			var
				sinceMidnight = function(/*Date*/ date){
					return date.getHours() * 60 * 60 + date.getMinutes() * 60 + date.getSeconds();
				},
				clickableIncrementSeconds = sinceMidnight(this._clickableIncrementDate),
				visibleIncrementSeconds = sinceMidnight(this._visibleIncrementDate),
				visibleRangeSeconds = sinceMidnight(this._visibleRangeDate),
				// round reference date to previous visible increment
				time = (this.value || this.currentFocus).getTime();

			this._refDate = new Date(time - time % (clickableIncrementSeconds*1000));
			this._refDate.setFullYear(1970,0,1); // match parse defaults

			// assume clickable increment is the smallest unit
			this._clickableIncrement = 1;
			// divide the visible range by the clickable increment to get the number of divs to create
			// example: 10:00:00/00:15:00 -> display 40 divs
			this._totalIncrements = visibleRangeSeconds / clickableIncrementSeconds;
			// divide the visible increments by the clickable increments to get how often to display the time inline
			// example: 01:00:00/00:15:00 -> display the time every 4 divs
			this._visibleIncrement = visibleIncrementSeconds / clickableIncrementSeconds;
			// divide the number of seconds in a day by the clickable increment in seconds to get the
			// absolute max number of increments.
			this._maxIncrement = (60 * 60 * 24) / clickableIncrementSeconds;

			var
				// Find the nodes we should display based on our filter.
				// Limit to 10 nodes displayed as a half-hearted attempt to stop drop down from overlapping <input>.
				count = Math.min(this._totalIncrements, 10),
				after = this._getFilteredNodes(0, (count >> 1) + 1, false),
				moreAfter = [],
				estBeforeLength = count - after.length,
				before = this._getFilteredNodes(0, estBeforeLength, true, after[0]);
			if(before.length < estBeforeLength && after.length > 0){
				moreAfter = this._getFilteredNodes(after[after.length-1].idx + 1, estBeforeLength - before.length, false, after[after.length-1]);
			}
			array.forEach(before.concat(after, moreAfter), function(n){ this.timeMenu.appendChild(n); }, this);
			// never show empty due to a bad filter
			if(!before.length && !after.length && !moreAfter.length && this.filterString){
				this.filterString = '';
				this._showText();
			}
		},

		constructor: function(/*===== params, srcNodeRef =====*/){
			// summary:
			//		Create the widget.
			// params: Object|null
			//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
			//		and functions, typically callbacks like onClick.
			//		The hash can contain any of the widget's properties, excluding read-only properties.
			// srcNodeRef: DOMNode|String?
			//		If a srcNodeRef (DOM node) is specified, replace srcNodeRef with my generated DOM tree

			this.constraints = {};
		},

		postMixInProperties: function(){
			this.inherited(arguments);
			this._setConstraintsAttr(this.constraints); // this needs to happen now (and later) due to codependency on _set*Attr calls
		},

		_setConstraintsAttr: function(/* Object */ constraints){
			// brings in visibleRange, increments, etc.
			for (var key in { clickableIncrement: 1, visibleIncrement: 1, visibleRange: 1 }) {
				if (key in constraints) {
					this[key] = constraints[key];
				}
			}

			// locale needs the lang in the constraints as locale
			if(!constraints.locale){
				constraints.locale = this.lang;
			}
		},

		postCreate: function(){
			// assign typematic mouse listeners to the arrow buttons
			this.connect(this.timeMenu, mouse.wheel, "_mouseWheeled");
			this.own(
				typematic.addMouseListener(this.upArrow, this, "_onArrowUp", 33, 250),
				typematic.addMouseListener(this.downArrow, this, "_onArrowDown", 33, 250)
			);

			this.inherited(arguments);
		},

		_buttonMouse: function(/*Event*/ e){
			// summary:
			//		Handler for hover (and unhover) on up/down arrows
			// tags:
			//		private

			// in non-IE browser the "mouseenter" event will become "mouseover",
			// but in IE it's still "mouseenter"
			domClass.toggle(e.currentTarget, e.currentTarget == this.upArrow ? "dijitUpArrowHover" : "dijitDownArrowHover",
				e.type == "mouseenter" || e.type == "mouseover");
		},

		_createOption: function(/*Number*/ index){
			// summary:
			//		Creates a clickable time option
			// tags:
			//		private
			var date = new Date(this._refDate);
			var incrementDate = this._clickableIncrementDate;
			date.setTime(date.getTime()
				+ incrementDate.getHours() * index * 3600000
				+ incrementDate.getMinutes() * index * 60000
				+ incrementDate.getSeconds() * index * 1000);
			if(this.constraints.selector == "time"){
				date.setFullYear(1970,0,1); // make sure each time is for the same date
			}
			var dateString = locale.format(date, this.constraints);
			if(this.filterString && dateString.toLowerCase().indexOf(this.filterString) !== 0){
				// Doesn't match the filter - return null
				return null;
			}

			var div = this.ownerDocument.createElement("div");
			div.className = this.baseClass+"Item";
			div.date = date;
			div.idx = index;
			domConstruct.create('div',{
				"class": this.baseClass + "ItemInner",
				innerHTML: dateString
			}, div);

			if(index%this._visibleIncrement<1 && index%this._visibleIncrement>-1){
				domClass.add(div, this.baseClass+"Marker");
			}else if(!(index%this._clickableIncrement)){
				domClass.add(div, this.baseClass+"Tick");
			}

			if(this.isDisabledDate(date)){
				// set disabled
				domClass.add(div, this.baseClass+"ItemDisabled");
			}
			if(this.value && !ddate.compare(this.value, date, this.constraints.selector)){
				div.selected = true;
				domClass.add(div, this.baseClass+"ItemSelected");
				if(domClass.contains(div, this.baseClass+"Marker")){
					domClass.add(div, this.baseClass+"MarkerSelected");
				}else{
					domClass.add(div, this.baseClass+"TickSelected");
				}

				// Initially highlight the current value.   User can change highlight by up/down arrow keys
				// or mouse movement.
				this._highlightOption(div, true);
			}
			return div;
		},

		_onOptionSelected: function(/*Object*/ tgt){
			// summary:
			//		Called when user clicks an option in the drop down list
			// tags:
			//		private
			var tdate = tgt.target.date || tgt.target.parentNode.date;
			if(!tdate || this.isDisabledDate(tdate)){ return; }
			this._highlighted_option = null;
			this.set('value', tdate);
			this.onChange(tdate);
		},

		onChange: function(/*Date*/ /*===== time =====*/){
			// summary:
			//		Notification that a time was selected.  It may be the same as the previous value.
			// tags:
			//		public
		},

		_highlightOption: function(/*node*/ node, /*Boolean*/ highlight){
			// summary:
			//		Turns on/off highlight effect on a node based on mouse out/over event
			// tags:
			//		private
			if(!node){return;}
			if(highlight){
				if(this._highlighted_option){
					this._highlightOption(this._highlighted_option, false);
				}
				this._highlighted_option = node;
			}else if(this._highlighted_option !== node){
				return;
			}else{
				this._highlighted_option = null;
			}
			domClass.toggle(node, this.baseClass+"ItemHover", highlight);
			if(domClass.contains(node, this.baseClass+"Marker")){
				domClass.toggle(node, this.baseClass+"MarkerHover", highlight);
			}else{
				domClass.toggle(node, this.baseClass+"TickHover", highlight);
			}
		},

		onmouseover: function(/*Event*/ e){
			// summary:
			//		Handler for onmouseover event
			// tags:
			//		private
			this._keyboardSelected = null;
			var tgr = (e.target.parentNode === this.timeMenu) ? e.target : e.target.parentNode;
			// if we aren't targeting an item, then we return
			if(!domClass.contains(tgr, this.baseClass+"Item")){return;}
			this._highlightOption(tgr, true);
		},

		onmouseout: function(/*Event*/ e){
			// summary:
			//		Handler for onmouseout event
			// tags:
			//		private
			this._keyboardSelected = null;
			var tgr = (e.target.parentNode === this.timeMenu) ? e.target : e.target.parentNode;
			this._highlightOption(tgr, false);
		},

		_mouseWheeled: function(/*Event*/ e){
			// summary:
			//		Handle the mouse wheel events
			// tags:
			//		private
			this._keyboardSelected = null;
			event.stop(e);
			// we're not _measuring_ the scroll amount, just direction
			this[(e.wheelDelta>0 ? "_onArrowUp" : "_onArrowDown")](); // yes, we're making a new dom node every time you mousewheel, or click
		},

		_onArrowUp: function(count){
			// summary:
			//		Handler for up arrow key.
			// description:
			//		Removes the bottom time and add one to the top
			// tags:
			//		private
			if(count === -1){
				domClass.remove(this.upArrow, "dijitUpArrowActive");
				return;
			}else if(count === 0){
				domClass.add(this.upArrow, "dijitUpArrowActive");

			} // typematic end
			if(!this.timeMenu.childNodes.length){ return; }
			var index = this.timeMenu.childNodes[0].idx;
			var divs = this._getFilteredNodes(index, 1, true, this.timeMenu.childNodes[0]);
			if(divs.length){
				this.timeMenu.removeChild(this.timeMenu.childNodes[this.timeMenu.childNodes.length - 1]);
				this.timeMenu.insertBefore(divs[0], this.timeMenu.childNodes[0]);
			}
		},

		_onArrowDown: function(count){
			// summary:
			//		Handler for up arrow key.
			// description:
			//		Remove the top time and add one to the bottom
			// tags:
			//		private
			if(count === -1){
				domClass.remove(this.downArrow, "dijitDownArrowActive");
				return;
			}else if(count === 0){
				domClass.add(this.downArrow, "dijitDownArrowActive");
			} // typematic end
			if(!this.timeMenu.childNodes.length){ return; }
			var index = this.timeMenu.childNodes[this.timeMenu.childNodes.length - 1].idx + 1;
			var divs = this._getFilteredNodes(index, 1, false, this.timeMenu.childNodes[this.timeMenu.childNodes.length - 1]);
			if(divs.length){
				this.timeMenu.removeChild(this.timeMenu.childNodes[0]);
				this.timeMenu.appendChild(divs[0]);
			}
		},

		handleKey: function(/*Event*/ e){
			// summary:
			//		Called from `dijit/form/_DateTimeTextBox` to pass a keypress event
			//		from the `dijit/form/TimeTextBox` to be handled in this widget
			// tags:
			//		protected
			if(e.keyCode == keys.DOWN_ARROW || e.keyCode == keys.UP_ARROW){
				event.stop(e);
				// Figure out which option to highlight now and then highlight it
				if(this._highlighted_option && !this._highlighted_option.parentNode){
					this._highlighted_option = null;
				}
				var timeMenu = this.timeMenu,
					tgt = this._highlighted_option || query("." + this.baseClass + "ItemSelected", timeMenu)[0];
				if(!tgt){
					tgt = timeMenu.childNodes[0];
				}else if(timeMenu.childNodes.length){
					if(e.keyCode == keys.DOWN_ARROW && !tgt.nextSibling){
						this._onArrowDown();
					}else if(e.keyCode == keys.UP_ARROW && !tgt.previousSibling){
						this._onArrowUp();
					}
					if(e.keyCode == keys.DOWN_ARROW){
						tgt = tgt.nextSibling;
					}else{
						tgt = tgt.previousSibling;
					}
				}
				this._highlightOption(tgt, true);
				this._keyboardSelected = tgt;
				return false;
			}else if(e.keyCode == keys.ENTER || e.keyCode === keys.TAB){
				// mouse hover followed by TAB is NO selection
				if(!this._keyboardSelected && e.keyCode === keys.TAB){
					return true;	// true means don't call stopEvent()
				}

				// Accept the currently-highlighted option as the value
				if(this._highlighted_option){
					this._onOptionSelected({target: this._highlighted_option});
				}

				// Call stopEvent() for ENTER key so that form doesn't submit,
				// but not for TAB, so that TAB does switch focus
				return e.keyCode === keys.TAB;
			}
			return undefined;
		}
	});

	/*=====
	 TimePicker.__Constraints = declare(locale.__FormatOptions, {
		 // clickableIncrement: String
		 //		See `dijit/_TimePicker.clickableIncrement`
		 clickableIncrement: "T00:15:00",

		 // visibleIncrement: String
		 //		See `dijit/_TimePicker.visibleIncrement`
		 visibleIncrement: "T01:00:00",

		 // visibleRange: String
		 //		See `dijit/_TimePicker.visibleRange`
		 visibleRange: "T05:00:00"
	 });
	 =====*/

	return TimePicker;
});

},
'url:dijit/templates/TimePicker.html':"<div id=\"widget_${id}\" class=\"dijitMenu\"\n    ><div data-dojo-attach-point=\"upArrow\" class=\"dijitButtonNode dijitUpArrowButton\" data-dojo-attach-event=\"onmouseenter:_buttonMouse,onmouseleave:_buttonMouse\"\n\t\t><div class=\"dijitReset dijitInline dijitArrowButtonInner\" role=\"presentation\">&#160;</div\n\t\t><div class=\"dijitArrowButtonChar\">&#9650;</div></div\n    ><div data-dojo-attach-point=\"timeMenu,focusNode\" data-dojo-attach-event=\"onclick:_onOptionSelected,onmouseover,onmouseout\"></div\n    ><div data-dojo-attach-point=\"downArrow\" class=\"dijitButtonNode dijitDownArrowButton\" data-dojo-attach-event=\"onmouseenter:_buttonMouse,onmouseleave:_buttonMouse\"\n\t\t><div class=\"dijitReset dijitInline dijitArrowButtonInner\" role=\"presentation\">&#160;</div\n\t\t><div class=\"dijitArrowButtonChar\">&#9660;</div></div\n></div>\n",
'dojox/validate':function(){
define("dojox/validate", ["./validate/_base"], function(validate){

	/*=====
	 return {
	 // summary:
	 //		Deprecated.  Should require dojox/validate modules directly rather than trying to access them through
	 //		this module.
	 };
	 =====*/

	return validate;
});

},
'dojox/validate/_base':function(){
define("dojox/validate/_base", [
	"dojo/_base/lang",
	"dojo/regexp", // dojo core expressions
	"dojo/number", // dojo number expressions
	"./regexp" // additional expressions
], function(lang, regexp, number, xregexp) {

var validate = lang.getObject("dojox.validate", true);

validate.isText = function(value, flags){
	// summary:
	//		Checks if a string has non whitespace characters.
	//		Parameters allow you to constrain the length.
	// value: String
	// flags: Object?
	//		{length: Number, minlength: Number, maxlength: Number}
	//
	//		- flags.length  If set, checks if there are exactly flags.length number of characters.
	//		- flags.minlength  If set, checks if there are at least flags.minlength number of characters.
	//		- flags.maxlength  If set, checks if there are at most flags.maxlength number of characters.
	
	flags = (typeof flags == "object") ? flags : {};
	
	// test for text
	if(/^\s*$/.test(value)){ return false; } // Boolean
	
	// length tests
	if(typeof flags.length == "number" && flags.length != value.length){ return false; } // Boolean
	if(typeof flags.minlength == "number" && flags.minlength > value.length){ return false; } // Boolean
	if(typeof flags.maxlength == "number" && flags.maxlength < value.length){ return false; } // Boolean
	
	return true; // Boolean

};

validate._isInRangeCache = {};
validate.isInRange = function(value, flags){
	// summary:
	//		Validates whether a string denoting a number
	//		is between a max and min.
	// value: String
	// flags: Object?
	//		{max:Number, min:Number, decimal:String}
	//
	//		- flags.max  A number, which the value must be less than or equal to for the validation to be true.
	//		- flags.min  A number, which the value must be greater than or equal to for the validation to be true.
	//		- flags.decimal  The character used for the decimal point.  Default is ".".
	
	value = number.parse(value, flags);
	if(isNaN(value)){
		return false; // Boolean
	}
    
	// assign default values to missing paramters
	flags = (typeof flags == "object") ? flags : {};
	var max = (typeof flags.max == "number") ? flags.max : Infinity,
		min = (typeof flags.min == "number") ? flags.min : -Infinity,
		dec = (typeof flags.decimal == "string") ? flags.decimal : ".",
	
		cache = validate._isInRangeCache,
		cacheIdx = value + "max" + max + "min" + min + "dec" + dec
	;
	if(typeof cache[cacheIdx] != "undefined"){
		return cache[cacheIdx];
	}

	cache[cacheIdx] = !(value < min || value > max);
	return cache[cacheIdx]; // Boolean

};

validate.isNumberFormat = function(value, flags){
	// summary:
	//		Validates any sort of number based format
	// description:
	//		Validates any sort of number based format. Use it for phone numbers,
	//		social security numbers, zip-codes, etc. The value can be validated
	//		against one format or one of multiple formats.
	//
	//		Format Definition
	//		|   #        Stands for a digit, 0-9.
	//		|   ?        Stands for an optional digit, 0-9 or nothing.
	//		All other characters must appear literally in the expression.
	// example:
	// |  "(###) ###-####"       ->   (510) 542-9742
	// |  "(###) ###-#### x#???" ->   (510) 542-9742 x153
	// |  "###-##-####"          ->   506-82-1089       i.e. social security number
	// |  "#####-####"           ->   98225-1649        i.e. zip code
	// value: String
	// flags: Object?
	//		- flags.format  A string or an Array of strings for multiple formats.
	// example:
	// |	require(["dojox/validate/_base"], function(validate){
	// |		// returns true:
	// |		validate.isNumberFormat("123-45", { format:"###-##" });
	// |	});		
	// example:
	//		Check Multiple formats:
	// |	require(["dojox/validate/_base"], function(validate){
	// |		validate.isNumberFormat("123-45", {
	// |			format:["### ##","###-##","## ###"]
	// |	});
	//

	var re = new RegExp("^" + xregexp.numberFormat(flags) + "$", "i");
	return re.test(value); // Boolean
};

validate.isValidLuhn = function(/* String */value){
	// summary:
	//		Validate a String value against the Luhn algorithm.
	// description:
	//		Validate a String value against the Luhn algorithm to verify
	//		its integrity.
	
	var sum = 0, parity, curDigit;
	if(!lang.isString(value)){
		value = String(value);
	}
	value = value.replace(/[- ]/g,''); //ignore dashes and whitespaces
	parity = value.length % 2;

	for(var i = 0; i < value.length; i++){
		curDigit = parseInt(value.charAt(i));
		if(i % 2 == parity){
			curDigit *= 2;
		}
		if(curDigit > 9){
			curDigit -= 9;
		}
		sum += curDigit;
	}
	return !(sum % 10); // Boolean
};

return validate;

});

},
'dojox/validate/regexp':function(){
define("dojox/validate/regexp", ["dojo/_base/lang", "dojo/regexp", "dojox/main"], 
  function(lang, regexp, dojox){

var dxregexp = lang.getObject("validate.regexp", true, dojox);
dxregexp = dojox.validate.regexp = {
	
	ipAddress: function(flags){
		// summary:
		//		Builds a RE that matches an IP Address
		// description:
		//		Supports 5 formats for IPv4: dotted decimal, dotted hex, dotted octal, decimal and hexadecimal.
		//		Supports 2 formats for Ipv6.
		// flags: Object?
		//		All flags are boolean with default = true.
		//
		//		- flags.allowDottedDecimal  Example, 207.142.131.235.  No zero padding.
		//		- flags.allowDottedHex  Example, 0x18.0x11.0x9b.0x28.  Case insensitive.  Zero padding allowed.
		//		- flags.allowDottedOctal  Example, 0030.0021.0233.0050.  Zero padding allowed.
		//		- flags.allowDecimal  Example, 3482223595.  A decimal number between 0-4294967295.
		//		- flags.allowHex  Example, 0xCF8E83EB.  Hexadecimal number between 0x0-0xFFFFFFFF.
		//		  Case insensitive.  Zero padding allowed.
		//		- flags.allowIPv6   IPv6 address written as eight groups of four hexadecimal digits.
		
		//	FIXME: ipv6 can be written multiple ways IIRC
		//		- flags.allowHybrid   IPv6 address written as six groups of four hexadecimal digits
		//		-   followed by the usual 4 dotted decimal digit notation of IPv4. x:x:x:x:x:x:d.d.d.d

		// assign default values to missing paramters
		flags = (typeof flags == "object") ? flags : {};
		if(typeof flags.allowDottedDecimal != "boolean"){ flags.allowDottedDecimal = true; }
		if(typeof flags.allowDottedHex != "boolean"){ flags.allowDottedHex = true; }
		if(typeof flags.allowDottedOctal != "boolean"){ flags.allowDottedOctal = true; }
		if(typeof flags.allowDecimal != "boolean"){ flags.allowDecimal = true; }
		if(typeof flags.allowHex != "boolean"){ flags.allowHex = true; }
		if(typeof flags.allowIPv6 != "boolean"){ flags.allowIPv6 = true; }
		if(typeof flags.allowHybrid != "boolean"){ flags.allowHybrid = true; }

		// decimal-dotted IP address RE.
		var dottedDecimalRE =
			// Each number is between 0-255.  Zero padding is not allowed.
			"((\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])\\.){3}(\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])";

		// dotted hex IP address RE.  Each number is between 0x0-0xff.  Zero padding is allowed, e.g. 0x00.
		var dottedHexRE = "(0[xX]0*[\\da-fA-F]?[\\da-fA-F]\\.){3}0[xX]0*[\\da-fA-F]?[\\da-fA-F]";

		// dotted octal IP address RE.  Each number is between 0000-0377.
		// Zero padding is allowed, but each number must have at least 4 characters.
		var dottedOctalRE = "(0+[0-3][0-7][0-7]\\.){3}0+[0-3][0-7][0-7]";

		// decimal IP address RE.  A decimal number between 0-4294967295.
		var decimalRE =  "(0|[1-9]\\d{0,8}|[1-3]\\d{9}|4[01]\\d{8}|42[0-8]\\d{7}|429[0-3]\\d{6}|" +
			"4294[0-8]\\d{5}|42949[0-5]\\d{4}|429496[0-6]\\d{3}|4294967[01]\\d{2}|42949672[0-8]\\d|429496729[0-5])";

		// hexadecimal IP address RE.
		// A hexadecimal number between 0x0-0xFFFFFFFF. Case insensitive.  Zero padding is allowed.
		var hexRE = "0[xX]0*[\\da-fA-F]{1,8}";

		// IPv6 address RE.
		// The format is written as eight groups of four hexadecimal digits, x:x:x:x:x:x:x:x,
		// where x is between 0000-ffff. Zero padding is optional. Case insensitive.
		var ipv6RE = "([\\da-fA-F]{1,4}\\:){7}[\\da-fA-F]{1,4}";

		// IPv6/IPv4 Hybrid address RE.
		// The format is written as six groups of four hexadecimal digits,
		// followed by the 4 dotted decimal IPv4 format. x:x:x:x:x:x:d.d.d.d
		var hybridRE = "([\\da-fA-F]{1,4}\\:){6}" +
			"((\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])\\.){3}(\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])";

		// Build IP Address RE
		var a = [];
		if(flags.allowDottedDecimal){ a.push(dottedDecimalRE); }
		if(flags.allowDottedHex){ a.push(dottedHexRE); }
		if(flags.allowDottedOctal){ a.push(dottedOctalRE); }
		if(flags.allowDecimal){ a.push(decimalRE); }
		if(flags.allowHex){ a.push(hexRE); }
		if(flags.allowIPv6){ a.push(ipv6RE); }
		if(flags.allowHybrid){ a.push(hybridRE); }

		var ipAddressRE = "";
		if(a.length > 0){
			ipAddressRE = "(" + a.join("|") + ")";
		}
		return ipAddressRE; // String
	},

	host: function(flags){
		// summary:
		//		Builds a RE that matches a host
		// description:
		//		A host is a named host (A-z0-9_- but not starting with -), a domain name or an IP address, possibly followed by a port number.
		// flags: Object?
		//		- flags.allowNamed Allow a named host for local networks. Default is false.
		//		- flags.allowIP  Allow an IP address for hostname.  Default is true.
		//		- flags.allowLocal  Allow the host to be "localhost".  Default is false.
		//		- flags.allowPort  Allow a port number to be present.  Default is true.
		//		- flags in regexp.ipAddress can be applied.

		// assign default values to missing paramters
		flags = (typeof flags == "object") ? flags : {};

		if(typeof flags.allowIP != "boolean"){ flags.allowIP = true; }
		if(typeof flags.allowLocal != "boolean"){ flags.allowLocal = false; }
		if(typeof flags.allowPort != "boolean"){ flags.allowPort = true; }
		if(typeof flags.allowNamed != "boolean"){ flags.allowNamed = false; }

		//TODO: support unicode hostnames?
		// Domain name labels can not end with a dash.
		var domainLabelRE = "(?:[\\da-zA-Z](?:[-\\da-zA-Z]{0,61}[\\da-zA-Z])?)";
		var domainNameRE = "(?:[a-zA-Z](?:[-\\da-zA-Z]{0,6}[\\da-zA-Z])?)"; // restricted version to allow backwards compatibility with allowLocal, allowIP

		// port number RE
		var portRE = flags.allowPort ? "(\\:\\d+)?" : "";

		// build host RE
		var hostNameRE = "((?:" + domainLabelRE + "\\.)+" + domainNameRE + "\\.?)";
		if(flags.allowIP){ hostNameRE += "|" +  dxregexp.ipAddress(flags); }
		if(flags.allowLocal){ hostNameRE += "|localhost"; }
		if(flags.allowNamed){ hostNameRE += "|^[^-][a-zA-Z0-9_-]*"; }
		return "(" + hostNameRE + ")" + portRE; // String

	},

	url: function(flags){
		// summary:
		//		Builds a regular expression that matches a URL
		// flags: Object?
		//		- flags.scheme  Can be true, false, or [true, false].
		//		-   This means: required, not allowed, or match either one.
		//		- flags in regexp.host can be applied.
		//		- flags in regexp.ipAddress can be applied.

		// assign default values to missing paramters
		flags = (typeof flags == "object") ? flags : {};
		if(!("scheme" in flags)){ flags.scheme = [true, false]; }

		// Scheme RE
		var protocolRE = regexp.buildGroupRE(flags.scheme,
			function(q){ if(q){ return "(https?|ftps?)\\://"; } return ""; }
		);

		// Path and query and anchor RE
		var pathRE = "(/(?:[^?#\\s/]+/)*(?:[^?#\\s/]+(?:\\?[^?#\\s/]*)?(?:#[A-Za-z][\\w.:-]*)?)?)?";

		return protocolRE + dxregexp.host(flags) + pathRE;
	},

	emailAddress: function(flags){
		// summary:
		//		Builds a regular expression that matches an email address
		// flags: Object?
		//		- flags.allowCruft  Allow address like `<mailto:foo@yahoo.com>`.  Default is false.
		//		- flags in regexp.host can be applied.
		//		- flags in regexp.ipAddress can be applied.

		// assign default values to missing paramters
		flags = (typeof flags == "object") ? flags : {};
		if (typeof flags.allowCruft != "boolean") { flags.allowCruft = false; }
		flags.allowPort = false; // invalid in email addresses

		// user name RE per rfc5322
		var usernameRE = "([!#-'*+\\-\\/-9=?A-Z^-~]+[.])*[!#-'*+\\-\\/-9=?A-Z^-~]+";

		// build emailAddress RE
		var emailAddressRE = usernameRE + "@" + dxregexp.host(flags);

		// Allow email addresses with cruft
		if ( flags.allowCruft ) {
			emailAddressRE = "<?(mailto\\:)?" + emailAddressRE + ">?";
		}

		return emailAddressRE; // String
	},

	emailAddressList: function(flags){
		// summary:
		//		Builds a regular expression that matches a list of email addresses.
		// flags: Object?
		//		- flags.listSeparator  The character used to separate email addresses.  Default is ";", ",", "\n" or " ".
		//		- flags in regexp.emailAddress can be applied.
		//		- flags in regexp.host can be applied.
		//		- flags in regexp.ipAddress can be applied.

		// assign default values to missing paramters
		flags = (typeof flags == "object") ? flags : {};
		if(typeof flags.listSeparator != "string"){ flags.listSeparator = "\\s;,"; }

		// build a RE for an Email Address List
		var emailAddressRE = dxregexp.emailAddress(flags);
		var emailAddressListRE = "(" + emailAddressRE + "\\s*[" + flags.listSeparator + "]\\s*)*" +
			emailAddressRE + "\\s*[" + flags.listSeparator + "]?\\s*";

		return emailAddressListRE; // String
	},
	
	numberFormat: function(flags){
		// summary:
		//		Builds a regular expression to match any sort of number based format
		// description:
		//		Use this method for phone numbers, social security numbers, zip-codes, etc.
		//		The RE can match one format or one of multiple formats.
		//
		//		Format:
		//
		//		- #        Stands for a digit, 0-9.
		//		- ?        Stands for an optional digit, 0-9 or nothing.
		//		- All other characters must appear literally in the expression.
		//
		// example:
		//		- "(###) ###-####"		-    ->   (510) 542-9742
		//		- "(###) ###-#### x#???" ->   (510) 542-9742 x153
		//		- "###-##-####"		- 		-   ->   506-82-1089		-    i.e. social security number
		//		- "#####-####"		- 		-    ->   98225-1649		- 		- i.e. zip code
		//
		// flags:  Object?
		//		- flags.format  A string or an Array of strings for multiple formats.

		// assign default values to missing paramters
		flags = (typeof flags == "object") ? flags : {};
		if(typeof flags.format == "undefined"){ flags.format = "###-###-####"; }

		// Converts a number format to RE.
		var digitRE = function(format){
			// escape all special characters, except '?'
			return regexp.escapeString(format, "?")
				// Now replace '?' with Regular Expression
				.replace(/\?/g, "\\d?")
				// replace # with Regular Expression
				.replace(/#/g, "\\d")
			;
		};

		// build RE for multiple number formats
		return regexp.buildGroupRE(flags.format, digitRE); //String
	},
	
	ca: {

		postalCode: function(){
			// summary:
			//		String regular Express to match Canadain Postal Codes
			return "([A-Z][0-9][A-Z] [0-9][A-Z][0-9])";
		},

		province: function(){
			// summary:
			//		a regular expression to match Canadian Province Abbreviations
			return "(AB|BC|MB|NB|NL|NS|NT|NU|ON|PE|QC|SK|YT)";
		}

	},
	
	us:{
		state: function(flags){
			// summary:
			//		A regular expression to match US state and territory abbreviations
			// flags: Object?
			//		- flags.allowTerritories  Allow Guam, Puerto Rico, etc.  Default is true.
			//		- flags.allowMilitary  Allow military 'states', e.g. Armed Forces Europe (AE).  Default is true.

			// assign default values to missing parameters
			flags = (typeof flags == "object") ? flags : {};
			if(typeof flags.allowTerritories != "boolean"){ flags.allowTerritories = true; }
			if(typeof flags.allowMilitary != "boolean"){ flags.allowMilitary = true; }

			// state RE
			var statesRE =
				"AL|AK|AZ|AR|CA|CO|CT|DE|DC|FL|GA|HI|ID|IL|IN|IA|KS|KY|LA|ME|MD|MA|MI|MN|MS|MO|MT|" +
				"NE|NV|NH|NJ|NM|NY|NC|ND|OH|OK|OR|PA|RI|SC|SD|TN|TX|UT|VT|VA|WA|WV|WI|WY";

			// territories RE
			var territoriesRE = "AS|FM|GU|MH|MP|PW|PR|VI";

			// military states RE
			var militaryRE = "AA|AE|AP";

			// Build states and territories RE
			if(flags.allowTerritories){ statesRE += "|" + territoriesRE; }
			if(flags.allowMilitary){ statesRE += "|" + militaryRE; }

			return "(" + statesRE + ")"; // String
		}

	}
	
};

return dxregexp;

});

},
'url:epi-find/widget/templates/Schedule.html':"<div>\r\n    <div>\r\n        <input data-dojo-type=\"dijit/form/RadioButton\" data-dojo-attach-point=\"noSchedule\" data-dojo-props=\"checked: true\" id=\"noSchedule\" name=\"schedule\"  /><label for=\"noSchedule\" >${i18n.noSchedule}</label>\r\n    </div>\r\n    <div>\r\n        <span class=\"epi-formsRow__input-wrapper--inline\">\r\n            <input data-dojo-type=\"dijit/form/RadioButton\" data-dojo-attach-point=\"dailySchedule\" id=\"dailySchedule\" name=\"schedule\" /><label data-dojo-attach-point=\"dailyNode\"></label>\r\n        </span>\r\n        <span class=\"epi-formsRow__input-wrapper--inline\">\r\n            <input data-dojo-type=\"dijit/form/RadioButton\" data-dojo-attach-point=\"weeklySchedule\" id=\"weeklySchedule\" name=\"schedule\" /><label data-dojo-attach-point=\"weeklyNode\"></label>\r\n        </span>\r\n        <span class=\"epi-formsRow__input-wrapper--inline\">\r\n            <input data-dojo-type=\"dijit/form/RadioButton\" data-dojo-attach-point=\"customSchedule\" id=\"customSchedule\" name=\"schedule\"/><label data-dojo-attach-point=\"customNode\"></label>\r\n        </span>\r\n        <p class=\"epi-secondaryText\">${i18n.daylightSavings}</p>\r\n    </div>\r\n</div>\r\n",
'epi-saas-base/widgets/TagEditor':function(){
require({cache:{
'url:epi-saas-base/widgets/templates/TagEditor.html':"<div>\r\n    <div class=\"epi-vertical-spacer\">\r\n        <input data-dojo-type=\"epi-saas-base/widgets/ValidationTextarea\" class=\"epi-textArea-growing\" data-dojo-attach-point=\"tagEditTextBox\" data-dojo-props=\"trim: true, pattern: this.tagValidationPattern\" />\r\n    </div>\r\n    <div class=\"epi-vertical-spacer epi-tag--no-selection\" data-dojo-attach-point=\"containerNode\" >\r\n    </div>\r\n</div>"}});
define("epi-saas-base/widgets/TagEditor", [
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/on",
    "dojo/keys",

    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/_Container",

    "./ValidationTextarea",
    "./Tag",

    "dojo/text!./templates/TagEditor.html"
], function (
    array,
    declare,
    lang,
    on,
    keys,
    WidgetBase,
    _TemplatedMixin,
    WidgetsInTemplateMixin,
    _Container,
    ValidationTextarea,
    Tag,
    template
    ) {

    // module:
    //   epi-saas-base/widgets/TagEditor

    return declare([WidgetBase, _TemplatedMixin, WidgetsInTemplateMixin, _Container], {

        templateString: template,
        tagValidationPattern: ".*",
        allowDuplicateTags: false,
        _currentIndex: null,

        // value: string[]
        //   An array of strings representing the value of the tags.
        value: null,
        _setValueAttr: function( /*String|Array*/ value) {
            if (!this._started) {
                return;
            }
            if (typeof value === "undefined" || value === null) {
                value = [];
            }
            if (!lang.isArray(value)) {
                value = [value];
            }
            this._currentIndex = null;
            this.tagEditTextBox.reset();

            array.forEach(this.getChildren(), function(tag) {
                tag.destroy();
            });

            array.forEach(value, this._renderTag, this);

            this._set("value", value);
        },

        startup: function () {
            this.inherited(arguments);
            var self = this, baseValidator = this.tagEditTextBox.validator;
            this.tagEditTextBox.validator = function(value, constraints){
                var values = value.split("\n"),
                    baseIsValid = array.every(values, lang.hitch(this, function(item) {
                        return baseValidator.apply(this, [item, constraints]);
                    })),
                    duplicateIsValid = self.allowDuplicateTags ||
                        array.every(values, function(item, index) {
                            return (!self.value || self.value.indexOf(item) < 0 // no values or item is not in values
                                || (self._currentIndex !== null && self._currentIndex.indexOf(self.value.indexOf(item)) !== -1)) // item is one of the currently edited
                                && values.indexOf(item) === index; // there are no duplicates in the currently edited items
                        });
                return baseIsValid && duplicateIsValid;
            };
            this.own(
                on(this.tagEditTextBox, "change", lang.hitch(this, function(value) {
                    var newValue = lang.clone(this.value || []), values;
                    if (this.setting) {
                        this.setting = false;
                        return;
                    }
                    if (!this.tagEditTextBox.isValid()) {
                        return;
                    }
                    if (!value ) {
                        this._currentIndex = null;
                        this._removeSelection();
                        return;
                    }
                    values = array.filter(value.split("\n"), function(item) {
                        return item.trim().length > 0;
                    });
                    if (this._currentIndex === null) {
                        newValue = newValue.concat(values);
                        this.tagEditTextBox.reset();
                        this.set("value", newValue);
                    } else if (lang.isArray(this._currentIndex)) {
                        array.forEach(this._currentIndex, function(itemIndex) {
                            newValue[itemIndex] = values.shift();
                        });
                        newValue = newValue.concat(values);
                        newValue = array.filter(newValue, function(item) {
                            return !!item;
                        });
                        this.tagEditTextBox.reset();
                        this.set("value", newValue);
                    }
                })),
                on(this.tagEditTextBox, "keypress", lang.hitch(this, function(event) {
                    if (event.keyCode  === keys.ENTER) {
                        if (this.tagEditTextBox.get("value").indexOf("\n") === -1 && !event.shiftKey) {
                            this.tagEditTextBox.set("value", this.tagEditTextBox.get("value"));
                            event.preventDefault();
                        }
                    }
                    if (event.keyCode === keys.ESCAPE) {
                        this.tagEditTextBox.reset();
                    }
                }))
            );
        },

        _removeSelection: function() {
            array.forEach(this.getChildren(), function(tag) {
                tag.deSelect();
            });
        },

        _selectAll: function() {
            array.forEach(this.getChildren(), function(tag) {
                tag.select();
            });
        },

        _renderTag: function(item, index) {
            var tag = new Tag();
            this.addChild(tag);

            tag.set("value", item);
            tag.set("index", index);

            tag.on("tagclick", lang.hitch(this, this._tagClickHandler));
            tag.on("tagdblclick", lang.hitch(this, this._tagDblClickHandler));
            tag.on("remove", lang.hitch(this, this._tagRemoveHandler));
        },

        _tagClickHandler: function(event) {
            var tag = event.tag;
            if (tag.isSelected()) {
                this._removeSelection();
                this._currentIndex = null;
                this.tagEditTextBox.reset();
            } else {
                this._currentIndex = [tag.get("index")];
                this.setting = true;
                this.tagEditTextBox.set("value", tag.get("value"));
                this.tagEditTextBox.focus();
                this._removeSelection();
                tag.select();
            }
        },

        _tagDblClickHandler: function(event) {
            var items = this.value.slice(0, this.value.length);
            this._currentIndex = [];
            for (var i=0; i < this.value.length; i++) {
                this._currentIndex.push(i);
            }
            this._selectAll();
            this.setting = true;
            this.tagEditTextBox.set("value", items.join("\r\n"));
        },

        _tagRemoveHandler: function(event) {
            var tag = event.tag;
            var newValue = lang.clone(this.value);
            newValue.splice(tag.get("index"), 1);
            this.set("value", newValue);
        },

        validate: function(value, constraints) {
            return this.tagEditTextBox.validate();
        },

        focus: function(){
            return this.tagEditTextBox.focus();
        }
    });
});

},
'epi-saas-base/widgets/Tag':function(){
require({cache:{
'url:epi-saas-base/widgets/templates/Tag.html':"<span class=\"epi-tag epi-tag--margin-bottom epi-lozenge\">\r\n    <span class=\"epi-clickable epi-tag__text\" data-dojo-attach-point=\"textNode\"></span>\r\n    <span class=\"dijitInline dijitReset dijiIcon epi-closeIcon epi-clickable\" data-dojo-attach-point=\"removeButtonNode\"></span>\r\n</span>"}});
define("epi-saas-base/widgets/Tag", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-class",
    "dojo/on",

    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",

    "dojo/text!./templates/Tag.html"
], function (
    declare,
    lang,
    domClass,
    on,

    WidgetBase,
    _TemplatedMixin,
    WidgetsInTemplateMixin,
    template
    ) {

    // module:
    //   epi-saas-base/widgets/Tag

    return declare([WidgetBase, _TemplatedMixin, WidgetsInTemplateMixin], {

        templateString: template,

        index: null,
        value: null,
        _setValueAttr: { node: "textNode", type: "innerHTML" },

        startup: function () {
            this.inherited(arguments);

            this.own(
                on(this.textNode, "click", lang.hitch(this, function(event) {
                    this.emit("tagclick", {event: event, tag: this});
                })),
                on(this.textNode, "dblclick", lang.hitch(this, function(event) {
                    this.emit("tagdblclick", {event: event, tag: this});
                })),
                on(this.removeButtonNode, "click", lang.hitch(this, function(event) {
                    this.emit("remove", {event: event, tag: this});
                }))
            );
        },

        select: function() {
            domClass.add(this.domNode, "epi-tag--selected");
        },

        deSelect: function() {
            domClass.remove(this.domNode, "epi-tag--selected");
        },

        isSelected: function() {
            return domClass.contains(this.domNode, "epi-tag--selected");
        }
    });
});
},
'url:epi-saas-base/widgets/templates/Tag.html':"<span class=\"epi-tag epi-tag--margin-bottom epi-lozenge\">\r\n    <span class=\"epi-clickable epi-tag__text\" data-dojo-attach-point=\"textNode\"></span>\r\n    <span class=\"dijitInline dijitReset dijiIcon epi-closeIcon epi-clickable\" data-dojo-attach-point=\"removeButtonNode\"></span>\r\n</span>",
'url:epi-saas-base/widgets/templates/TagEditor.html':"<div>\r\n    <div class=\"epi-vertical-spacer\">\r\n        <input data-dojo-type=\"epi-saas-base/widgets/ValidationTextarea\" class=\"epi-textArea-growing\" data-dojo-attach-point=\"tagEditTextBox\" data-dojo-props=\"trim: true, pattern: this.tagValidationPattern\" />\r\n    </div>\r\n    <div class=\"epi-vertical-spacer epi-tag--no-selection\" data-dojo-attach-point=\"containerNode\" >\r\n    </div>\r\n</div>",
'epi-find/widget/InvertedValidationGrid':function(){
﻿define([
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/aspect",
    "dojo/dom",
    "dojo/keys",
    "dojo/store/Memory",

    "dijit/_WidgetBase",
    "dijit/_Container",
    "dijit/focus",
    "dijit/MenuItem",

    "./InvertedValidationTextbox",

    "dgrid/OnDemandGrid",
    "dgrid/Keyboard",

    "epi/shell/dgrid/Formatter",
    "epi-cms/dgrid/formatters",

    "dojo/i18n!./nls/Connectors",

    "epi-saas-base/widgets/_GridContextMenuMixin"
],

function (
    array,
    declare,
    lang,
    aspect,
    dom,
    keys,
    Memory,
    _WidgetBase,
    _Container,
    focusUtil,
    MenuItem,
    InvertedValidationTextBox,
    Grid,
    Keyboard,
    Formatter,
    formatters,
    i18n,
    _GridContextMenuMixin
    ) {

    return declare([_WidgetBase, _Container], {

        textBox: null,
        grid: null,
        store: null,
        value: null,
        validationPattern: null,
        label: null,
        editMode: false,

        _setValueAttr: function(value) {
            this._set("value", value);
            var a = array.map(value, function(item) {
                return { value: item, id: Math.random() };
            });
            this.store.setData(a);
            this.grid.refresh();
        },

        postMixInProperties: function() {
            this.inherited(arguments);
            this.store = new Memory({ data: this.value });
        },

        buildRendering: function() {
            this.inherited(arguments);

            this.textBox = new InvertedValidationTextBox({
                required:false,
                pattern: this.validationPattern,
                intermediateChanges: false
            });

            this.addChild(this.textBox);
            this.own(
                this.textBox,
                this.textBox.watch("value", lang.hitch(this, function(name, oldValue, newValue) {
                    if (!this.editMode && newValue !== "" && this.textBox.isValid()) {
                        this.updateOnEvent();
                    }
                    this.editMode = false;
                })),
                this.textBox.on("keydown", lang.hitch(this, function(event) {
                    if (event.keyCode === keys.ENTER) {
                        if(this.textBox.isValid()) {
                            this.updateOnEvent();
                            focusUtil.focus(this.textBox.focusNode);
                        } else {
                            focusUtil.curNode.blur();
                        }
                        event.preventDefault();
                    } else if (event.keyCode === keys.ESCAPE) {
                        this.editMode = false;
                        this.editId = null;
                        this.textBox.set("value","");
                    }
                }))
            );

            var self = this;

            this.grid = new declare([Grid, Formatter, Keyboard, _GridContextMenuMixin])({
                "class": "dgrid dgrid-grid ui-widget epi-plain-grid epi-grid-height--auto",
                showHeader: false,
                store: this.store,
                selectionMode: "single",
                noDataMessage: i18n.noPatternsYet,
                sort: {
                    name: "id", descending: false
                },
                columns: [
                    {
                        autoSave: true,
                        label: this.label,
                        field: "value",
                        sortable: false
                    },
                    {
                        sortable: false,
                        label: " ",
                        className: "epi-grid--10",
                        formatter: function() {
                            return formatters.menu({title: ""});
                        }
                    }
                ]
            });

            this.grid.editMenuItem = new MenuItem({
                label: i18n.actions_edit,
                onClick: function (e) {
                    self.editMode = true;
                    self.editId = self.grid.getRow(this).id;
                    self.textBox.set("value", self.grid.getRow(this).data.value);
                    focusUtil.focus(self.textBox.domNode);
                }
            });
            this.grid.deleteMenuItem = new MenuItem({
                label: i18n.actions_delete,
                onClick: function (e) {
                    self.store.remove(self.grid.getRow(this).id);
                    self.grid.refresh();
                    self.grid.resize();
                    self.textBox.set("value","");
                    e.preventDefault();
                    e.stopPropagation();
                }
            });

            this.grid.contextMenu.addChild(this.grid.editMenuItem);
            this.grid.contextMenu.addChild(this.grid.deleteMenuItem);

            this.addChild(this.grid);
            this.own(this.grid);
        },

        updateOnEvent: function() {
            if (this.editId) {
                if (this.textBox.get("value") !== "") {
                    this.store.put({ value: this.textBox.get("value"), id: this.editId});
                } else {
                    this.store.remove(this.editId);
                }
                this.editId = null;
            } else {
                this.store.add({ value: this.textBox.get("value") });
            }
            this.textBox.set("value", "");
            this.grid.refresh();
            this.grid.resize();
        },

        postCreate: function() {
            this.inherited(arguments);
            aspect.after(this.store, "put", lang.hitch(this, function(response){
                this._set("value", array.filter(array.map(this.store.query(), function(item) {
                    return item.value;
                }), function(value) {
                    return value !== "";
                }));
                this.grid.refresh();
            }));
            aspect.after(this.store, "remove", lang.hitch(this, function(response){
                this._set("value", array.filter(array.map(this.store.query(), function(item) {
                    return item.value;
                }), function(value) {
                    return value !== "";
                }));
                this.grid.refresh();
            }));
        },

        startup: function() {
            this.inherited(arguments);
            this.grid.startup();
            this.set("value", []);
            this.own(this.grid.on("dgrid-refresh-complete", lang.hitch(this, function () {
                this.grid.resize();
            })));
        }
    });
});

},
'epi-find/widget/InvertedValidationTextbox':function(){
define([
    "dojo/_base/declare",
    "dijit/form/ValidationTextBox"
], function(declare,ValidationTextBox) {
    return declare(ValidationTextBox, {
        // summary:
        //      Textarea that validates if the pattern gets no hits.
        validator: function(value, constraints) {
            var superMethod = this.getInherited(arguments);
            if (value === "") {
                return !this.required;
            }
            return !superMethod.apply(this, [value, constraints]);
        }
    });
});

},
'url:epi-find/widget/templates/ConnectorForm.html':"﻿<form data-dojo-attach-point='containerNode' data-dojo-attach-event='onreset:_onReset,onsubmit:_onSubmit' class='epi-formsWidgetWrapper epi-vertical-spacer--double'>\r\n    <ol class='epi-formsWidgetWrapper'>\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\" data-dojo-attach-point=\"nameFieldListItem\">\r\n            <input id=\"nameField\" data-dojo-attach-point=\"nameField\" data-dojo-type=\"epi-find/widget/PhraseValidationTextBox\" data-dojo-props=\"required:true, pattern: this.nameRegex, intermediateChanges: true\" />\r\n            <p class=\"epi-help\">${i18n.help_name}</p>\r\n        </li>\r\n\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\" data-dojo-attach-point=\"typeSelectListItem\">\r\n            <input id=\"typeSelect\" data-dojo-attach-point=\"typeSelect\" data-dojo-type=\"dijit/form/Select\" data-dojo-props=\"required:true, trim:true\" />\r\n            <p class=\"epi-help\">${i18n.help_typeAndChannels}</p>\r\n        </li>\r\n\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\" data-dojo-attach-point=\"startUrlListItem\">\r\n            <input id=\"startUrl\" data-dojo-attach-point=\"startUrl\" data-dojo-type=\"epi-find/widget/PhraseValidationTextBox\" data-dojo-props=\"required:true, pattern: this.urlRegex, intermediateChanges: true\" />\r\n            <p class=\"epi-help\">${i18n.help_rootUrl}</p>\r\n        </li>\r\n\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\" data-dojo-attach-point=\"feedUrlsListItem\">\r\n            <input id=\"feedUrls\" data-dojo-attach-point=\"feedUrls\" data-dojo-type=\"epi-find/widget/PhraseValidationTextBox\" data-dojo-props=\"required:true, pattern: this.urlRegex, intermediateChanges: true\" />\r\n            <p class=\"epi-help\">${i18n.help_feedUrls}</p>\r\n        </li>\r\n\r\n\t\t<li data-dojo-type=\"epi-saas-base/form/FormListItem\" data-dojo-attach-point=\"excludeMimeTypesListItem\">\r\n            <div id=\"excludeMimeTypes\" data-dojo-attach-point=\"excludeMimeTypes\" data-dojo-type=\"epi-saas-base/widgets/TagEditor\" data-dojo-props=\"tagValidationPattern: this.mimeValidationPattern\"></div>\r\n            <p class=\"epi-help\">${i18n.help_excludeMimeTypes}</p>\r\n        </li>\r\n\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\" data-dojo-attach-point=\"advancedListItem\">\r\n            <input class=\"epi-chromeless epi-visibleLink\" data-dojo-attach-point=\"advancedButton\" data-dojo-type=\"dijit/form/Button\" />\r\n            <p data-dojo-attach-point=\"patternHelpText\" class=\"epi-help\">${i18n.help_patterns}</p>\r\n        </li>\r\n\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\" data-dojo-attach-point=\"excludedCrawlPatternListItem\">\r\n            <div data-dojo-attach-point=\"excludedCrawlPatternGrid\" data-dojo-type=\"epi-find/widget/InvertedValidationGrid\" data-dojo-props=\"validationPattern: this.globRegex\">\r\n            </div>\r\n            <p data-dojo-attach-point=\"excludedCrawlPatternHelpText\" class=\"epi-help\">${i18n.help_excludedCrawlPattern}</p>\r\n        </li>\r\n\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\" data-dojo-attach-point=\"excludedIndexPatternListItem\">\r\n            <div data-dojo-attach-point=\"excludedIndexPatternGrid\" data-dojo-type=\"epi-find/widget/InvertedValidationGrid\" data-dojo-props=\"validationPattern: this.globRegex\">\r\n            </div>\r\n            <p data-dojo-attach-point=\"excludedIndexPatternHelpText\" class=\"epi-help\">${i18n.help_excludedIndexPattern}</p>\r\n        </li>\r\n\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\" data-dojo-attach-point=\"removeQueryArgsListItem\">\r\n            <div data-dojo-attach-point=\"removeQueryArgsGrid\" data-dojo-type=\"epi-find/widget/InvertedValidationGrid\" data-dojo-props=\"validationPattern: this.queryArgsRegex\">\r\n            </div>\r\n            <p data-dojo-attach-point=\"removeQueryArgsHelpText\" class=\"epi-help\">${i18n.help_removeQueryArgs}</p>\r\n        </li>\r\n\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\" data-dojo-attach-point=\"includedCrawlPatternListItem\">\r\n            <div data-dojo-attach-point=\"includedCrawlPatternGrid\" data-dojo-type=\"epi-find/widget/InvertedValidationGrid\" data-dojo-props=\"validationPattern: this.globRegex\">\r\n            </div>\r\n            <p data-dojo-attach-point=\"includedCrawlPatternHelpText\" class=\"epi-help\">${i18n.help_includedCrawlPattern}</p>\r\n        </li>\r\n\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\" data-dojo-props=\"inputStyleType: 'inlineInputs'\" data-dojo-attach-point=\"scheduleListItem\">\r\n            <div class=\"epi-formsRow--inline-elements\" data-dojo-attach-point=\"schedule\" data-dojo-type=\"epi-find/widget/Schedule\" data-dojo-props=\"model: this.model.scheduleModel\"></div>\r\n            <p class=\"epi-help\">${i18n.help_schedule}</p>\r\n        </li>\r\n\r\n        <li data-dojo-type=\"epi-saas-base/form/FormListItem\" class=\"epi-formsRow--align-right\">\r\n            <input data-dojo-attach-point=\"submitButton\" type=\"submit\" data-dojo-type=\"dijit/form/Button\" class=\"epi-primary\" />\r\n            <input data-dojo-attach-point=\"resetButton\" type=\"reset\" data-dojo-type=\"dijit/form/Button\" />\r\n        </li>\r\n    </ol>\r\n</form>\r\n",
'epi-find/widget/ConnectorGrid':function(){
define([
        "dojo/_base/declare",
        "dojo/_base/lang",
        "dojo/dom-construct",
        "dojo/dom-class",
        "dojo/when",
        "dojo/Deferred",
        "dojo/on",
        "dojo/date/locale",

        "dojox/html/entities",

        "dijit/form/Button",
        "dijit/MenuItem",

        "dgrid/Grid",
        "dgrid/extensions/DijitRegistry",

        "./Schedule",

        "epi/shell/dgrid/Formatter",
        "epi/datetime",
        "epi-cms/dgrid/formatters",

        "epi-saas-base/widgets/_NotificationMixin",
        "epi-saas-base/widgets/_GridContextMenuMixin",
        "epi-saas-base/widgets/_GridMoreMixin",
        "epi-saas-base/widgets/_AutoLoadScrollMixin",

        "dojo/i18n!./nls/Connectors"
    ],
    function (declare,
            lang,
            domConstruct,
            domClass,
            when,
            Deferred,
            on,
            locale,

            entities,

            Button,
            MenuItem,

            Grid,
            DijitRegistry,

            Schedule,

            Formatter,
            datetime,
            formatters,

            _NotificationMixin,
            _GridContextMenuMixin,
            _GridMoreMixin,
            _AutoLoadScrollMixin,

            i18n
        ) {

        // module:
        //      epi-find/widget/ConnectorGrid

        return declare([Grid, _GridMoreMixin, _AutoLoadScrollMixin, Formatter, _GridContextMenuMixin, DijitRegistry, _NotificationMixin], {

            noDataMessage: i18n.noConnectors,
            className: "dgrid-autoheight",

            // model: epi-find/configure/ConnectorsModel
            //      Model for the connectors view
            model: null,

            selectionMode: "single",

            i18n: i18n,

            // editMenuItem: dijit/MenuItem
            //      Context menu item for Edit action
            editMenuItem: null,

            // deleteMenuItem: dijit/MenuItem
            //      Context menu item for Delete action
            deleteMenuItem: null,

            constructor: function(options) {
                declare.safeMixin(this, options);

                // Sets the loader (#loadNextPage, available in _GridMoreMixn) to be used by the _AutoLoadScrollMixin
                this.loader = lang.hitch(this, this.loadNextPage);

                var self = this;
                this.store = this.model.store;
                this.columns = {
                    name: {
                        label: i18n.fields_name,
                        sortable: false
                    },
                    type: {
                        label: i18n.fields_type,
                        sortable: false,
                        formatter: function(value) {
                            return i18n["types_"+value] || value;
                        }
                    },
                    schedule: {
                        label: i18n.fields_schedule,
                        sortable: false,
                        formatter: function(value) {
                            return self.model.scheduleModel.getTextDescription(value);
                        }
                    },
                    status: {
                        sortable: false,
                        renderHeaderCell: function(node) {
                            var grid = this.grid,
                                refreshAnchor = new Button({
                                    "class": "epi-chromeless epi-visibleLink",
                                    iconClass: "epi-iconReload",
                                    label: i18n.fields_status
                                });
                            domConstruct.place(refreshAnchor.domNode, node);
                            self.own(refreshAnchor,
                                on(refreshAnchor, "click", function(e) {
                                    e.preventDefault();
                                    e.stopPropagation();
                                    grid.refresh();
                                }));
                        },
                        renderCell: function(object, value, node, options) {
                            if (object) {
                                when(self.model.getJob(object), function(job) {
                                    var status = job ? job.mode : "";
                                    if (status) {
                                        var statusDetails = {
                                            stopTime: locale.format(new Date(job.stop_time), {formatLength: "short"})
                                        };
                                        var statusNode = domConstruct.create("span", {innerHTML:  lang.replace(i18n["statuses_" + status] || status, statusDetails)});
                                        if (status === "stopped") {
                                            domClass.add(statusNode, "epi-lozenge epi-background--success");
                                        }
                                        domConstruct.place(statusNode, node);

                                    }
                                });
                            }
                        }
                    },
                    date: {
                        sortable: false,
                        label: i18n.fields_date,
                        renderCell: function(object, value, node, options) {
                            if (object) {
                                when(self.model.getJob(object), function(job) {
                                    var status = job ? job.mode : "";
                                    if (status === "stopped") {
                                        node.innerHTML = datetime.toUserFriendlyString(new Date(job.stop_time));
                                    }
                                });
                            }
                        }
                    },

                    documents: {
                        sortable: false,
                        label: i18n.fields_documents,
                        className: "epi-grid-column--centered",
                        renderCell: function(object, value, node, options) {
                            if (object) {
                                when(self.model.getJob(object), function(job) {
                                    if (job && job.details ) {
                                        node.innerHTML = job.details.indexed_document_count || 0;
                                    }
                                });
                            }
                        }

                    },

                    actions: {
                        sortable: false,
                        label: "",

                        renderCell: function(object, value, node, options) {
                            var grid = this.grid, clickAction = null, anchor = new Button({
                                    "class": "epi-chromeless epi-visibleLink",
                                    disabled: self.model.isActionDisabled(object)
                                });
                            domConstruct.place(anchor.domNode, node);
                            self.own( // own is available because epi-cms/dgrid/WithContextMenu is based on Destroyable
                                anchor,
                                on(anchor, "click", function(e) {
                                    e.preventDefault();
                                    e.stopPropagation();
                                    anchor.set("disabled", true);// disable anchor until the cell is refreshed
                                    if (clickAction === null) {
                                        return;
                                    }
                                    when(clickAction(object), function() {
                                            anchor.set("disabled", self.model.isActionDisabled(object));
                                        },
                                        function(error) {
                                            self.showError(error);
                                        }
                                    );
                                })
                            );
                            when(self.model.getJob(object), function(job) {
                                var status = job ? job.mode : "";
                                if (status === "running" || status === "starting" || status === "stopping") {
                                    anchor.set("iconClass", "epi-gridIconStop");
                                    anchor.set("label", i18n.actions_stop);
                                    clickAction = function(connector) {
                                        return self.model.stopJob(connector);
                                    };
                                }
                                else {
                                    anchor.set("iconClass", "epi-gridIconStart");
                                    anchor.set("label", i18n.actions_start);
                                    clickAction = function(connector) {
                                        return self.model.startJob(connector);
                                    };
                                }

                            });
                        }
                    },
                    context: {
                        sortable: false,
                        label: " ",
                        className: "epi-grid--10",
                        formatter: function() {
                            return formatters.menu({title: i18n.contextMenuTitle});
                        }
                    }
                };
            },

            postCreate: function() {
                var self = this,
                    getCurrentItem = function(context) {
                        var row = self.getRow(context);
                        return encodeURIComponent(row.id);
                    };
                this.inherited(arguments);

                this.editMenuItem = new MenuItem({
                    label: i18n.actions_edit,
                    onClick: function () {
                        self.onEdit(getCurrentItem(this));
                    }
                });
                this.deleteMenuItem = new MenuItem({
                    label: i18n.actions_delete,
                    onClick: function () {
                        var id = getCurrentItem(this);
                        self.showConfirmation({
                            title: i18n.deleteConnector,
                            message: i18n.areYouSure,
                            confirmActionText: i18n.buttons_confirmDelete,
                            cancelActionText: i18n.buttons_cancelDelete,
                            action: function() {
                                when(self.store.remove(id),
                                    function() {
                                        self.showSuccess(i18n.deletedMessage);
                                    },
                                    function(error) {
                                        self.showError(error);
                                        self.refresh();
                                    }
                                );
                            }
                        });
                    }
                });

                this.contextMenu.addChild(this.editMenuItem);
                this.contextMenu.addChild(this.deleteMenuItem);

                this.contextMenu.onOpen = function() {
                    var data, job, mode, row = self.row(this.currentTarget);
                    if (!row) {
                        return;
                    }
                    data = row.data;
                    job = self.model.getJob(data);
                    mode = job && job.mode ? job.mode : null;
                    self.editMenuItem.set("disabled", (mode && mode !== "stopped"));
                };
                this.own(this.editMenuItem, this.editMenuItem,
                    this.on("dgrid-error", function(event) {
                        self.showErrorInGrid(event.error);
                    }),
                    this.model.on("statusChanged", function (e) {
                        var status = e && e.status ? e.status : null;
                        var message = status ? i18n[status + "StatusMessage"] : null;
                        if (!message) {
                            return;
                        }
                        var name = e.connector && e.connector.name ? "\"" + entities.encode(e.connector.name) + "\"" : "";
                        message = lang.replace(message, [name]);
                        self.showInfo(message);
                    })
                );
            },

            refresh: function() {
                // we need to refresh cached connector jobs store before refreshing the grid
                var baseRefresh = lang.hitch(this, this.getInherited(arguments)), self = this;
                return when(this.model.refreshJobCache(), function() {
                        return baseRefresh();
                    },
                    function (error) {
                        self.showErrorInGrid(error);
                    }
                );
            },


            onEdit: function(id) {
                // summary:
                //      A callback for edit context menu

            }
        });
    });

},
'url:epi-find/widget/templates/Connectors.html':"﻿<div>\r\n    <p class=\"epi-view--description\">${i18n.description}</p>\r\n    <form data-dojo-attach-point=\"connectorsFormNode\"></form>\r\n    <p class=\"epi-help\">${i18n.help_grid}</p>\r\n    <div id=\"connectorsgrid\" data-dojo-attach-point=\"connectorsGridNode\"></div>\r\n</div>",
'epi-find/widget/Boosting':function(){
﻿define([
    "dojo/_base/declare",

    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dijit/layout/_LayoutWidget",

    "./_ActionableMixin",
    "./_DisabledViewMixin",
    "./BoostingEditor",
    "./BoostingPreview",

    "dojo/i18n!./nls/Boosting",
    "dojo/text!./templates/Boosting.html"
],
    function(declare,
        _WidgetBase,
        _TemplatedMixin,

        _LayoutWidget,

        _ActionableMixin,
        _DisabledViewMixin,
        BoostingEditor,
        BoostingPreview,
        i18n,
        template) {

        // module:
        //      epi-find/widget/Connectors

        return declare([_WidgetBase, _LayoutWidget, _TemplatedMixin, _ActionableMixin, _DisabledViewMixin], {
            // summary:
            //     Manage view.

            templateString: template,

            i18n: i18n,

            model: null,

            boostingEditor: null,

            boostingPreview: null,

            postCreate: function() {
                this.inherited(arguments);
                if (!this.model) {
                    throw "No model specified";
                }

                this.boostingEditor = new BoostingEditor({
                    model: this.model.boostingEditorModel
                }, this.boostingEditorNode);
                this.own(this.boostingEditor);
                this.addChild(this.boostingEditor);

                this.boostingPreview = new BoostingPreview({
                    model: this.model.boostingPreviewModel,
                    scrollNode: this.scrollNode
                }, this.boostingPreviewNode);
                this.own(this.boostingPreview);
                this.addChild(this.boostingPreview);
            },

            layout: function() {
                this.boostingEditor.resize();
            },

            takeAction: function() {
                this.boostingEditor.refresh();
                return this;
            }
        });
    });
},
'epi-find/widget/BoostingEditor':function(){
﻿define([
        "dojo/_base/declare",
        "dojo/_base/lang",
        "dojo/dom-construct",
        "dojo/on",
        "dojo/dom-class",
        "dojo/when",

        "dijit/_WidgetBase",
        "dijit/_TemplatedMixin",
        "dijit/_WidgetsInTemplateMixin",

        "dijit/form/Button",
        "dijit/form/CheckBox",
        "dijit/form/HorizontalSlider",

        "dgrid/Grid",

        "put-selector/put",

        "dojox/mvc/at",

        "./WeightSelector",

        "epi-saas-base/widgets/_NotificationMixin",
        "epi-saas-base/widgets/_ActionButtonsMixin",

        "dojo/i18n!./nls/BoostingEditor",
        "dojo/text!./templates/BoostingEditor.html"
],
    function(declare,
            lang,
            domConstruct,
            on,
            domClass,
            when,
            _WidgetBase,
            _TemplatedMixin,
            _WidgetsInTemplateMixin,
            Button,
            CheckBox,
            HorizontalSlider,
            Grid,
            put,
            at,
            WeightSelector,
            _NotificationMixin,
            _ActionButtonsMixin,
            i18n,
            template) {

        // module:
        //      epi-find/widget/BoostingEditor

        return declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, _NotificationMixin, _ActionButtonsMixin], {
            // summary:
            //     Boosting weight editor.

            templateString: template,

            i18n: i18n,

            model: null,

            weightValues: null,

            postCreate: function() {
                this.inherited(arguments);

                this.submitButton.set("label", i18n.buttons_save);
                this.resetButton.set("label", i18n.buttons_cancel);
                this.resetToDefaultButton.set("label", i18n.buttons_resetToDefault);

                if (!this.model) {
                    throw "No model specified";
                }

                this.actionButtons = [
                    this.submitButton,
                    this.resetButton,
                    this.resetToDefaultButton
                ];

                this.enabledWhenNoChangesButtons = [
                    this.resetToDefaultButton
                ];

                if (!this.weightValues) {
                    this.weightValues = [0, 0.15, 0.5, 1, 1.5, 2];
                }

                var self = this, EditorGrid = declare([Grid, _NotificationMixin]);

                this.propertiesGrid = new EditorGrid({
                        i18n: i18n,
                        columns: [
                            {
                                label: " ",
                                sortable: false,
                                className: "epi-grid-column--checkbox",
                                renderCell: function(property, value, node) {
                                    var includedBox = new CheckBox({
                                        propertyWeight: at(self.model.weightsModel, property).direction(at.from).transform({
                                            format: function (weight) {
                                                includedBox.propertyPreviousWeight = includedBox.propertyWeight;
                                                return weight;
                                            }
                                        }),
                                        checked: at(self.model.weightsModel, property).transform({
                                            format: function(weight){
                                                return weight > 0;
                                            },
                                            parse: function(checked){
                                                return checked ?
                                                    includedBox.propertyPreviousWeight || includedBox.propertyWeight || 1 :
                                                    0;
                                            }
                                        })
                                    });
                                    self.own(includedBox);
                                    domConstruct.place(includedBox.domNode, node);
                                    includedBox.startup();
                                }
                            },
                            {
                                label: i18n.fields_name,
                                className: "epi-grid-header--h3",
                                sortable: false,
                                formatter: function (propertyName) {
                                    return i18n["properties_" + propertyName] || propertyName;
                                }
                            },
                            {
                                label: i18n.fields_weight,
                                sortable: false,
                                className: "epi-grid-header--h3",
                                renderCell: function(property, value, node) {
                                    var selector = new WeightSelector({
                                            value: at(self.model.weightsModel, property),
                                            values: self.weightValues
                                        });
                                    domConstruct.place(selector.domNode, node);
                                    selector.startup();
                                }
                            }
                        ]
                    }, this.propertiesGridNode);

                this.own(
                    this.propertiesGrid,
                    on(this.submitButton, "click", function(e) {
                        e.preventDefault();
                        e.stopPropagation();
                        self.indicateInProgress(self.submitButton);
                        when(self.model.commit(), function () {
                            self.showSuccess(i18n.savedMessage);
                            self.indicateSuccess();
                            self.reset();
                        }, function (error) {
                            self.showError(error, i18n.errorMessage);
                            self.indicateError();
                        });
                    }),
                    on(this.resetButton, "click", function(e) {
                        e.preventDefault();
                        e.stopPropagation();
                        self.model.reset();
                        self.reset();
                        self.refresh();
                    }),
                    on(this.resetToDefaultButton, "click", function(e) {
                        e.preventDefault();
                        e.stopPropagation();
                        self.model.setDefault();
                    }),
                    this.propertiesGrid.on("dgrid-error", function(error) {
                        self.propertiesGrid.showErrorInGrid(error);
                    }),
                    this.watchDataChanges([
                        this.model,
                        this.model.weightsModel])
                );
            },

            startup: function() {
                this.inherited(arguments);

                this.propertiesGrid.startup();
                this.refresh();
            },

            refresh: function(forceRefresh) {
                if (this.hasChanges && !forceRefresh) {
                    return;
                }
                when(this.model.refresh(), lang.hitch(this, function() {
                        this.propertiesGrid.refresh();
                        this.propertiesGrid.renderArray(this.model.propertyNames);

                        this.set("hasChanges", false);
                    }),
                    lang.hitch(this, function(error) {
                        this.propertiesGrid.showErrorInGrid(error);
                    })
                );
            },

            resize: function () {
                // We resize the grid in order to make sure that sizes of header, scroller, etc
                // are recalculated when it's needed, for instance, when grid becomes visible.
                this.propertiesGrid.resize();
            }
        });
    });

},
'dijit/form/HorizontalSlider':function(){
require({cache:{
'url:dijit/form/templates/HorizontalSlider.html':"<table class=\"dijit dijitReset dijitSlider dijitSliderH\" cellspacing=\"0\" cellpadding=\"0\" border=\"0\" rules=\"none\" data-dojo-attach-event=\"onkeypress:_onKeyPress,onkeyup:_onKeyUp\"\n\trole=\"presentation\"\n\t><tr class=\"dijitReset\"\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\n\t\t><td data-dojo-attach-point=\"topDecoration\" class=\"dijitReset dijitSliderDecoration dijitSliderDecorationT dijitSliderDecorationH\"></td\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\n\t></tr\n\t><tr class=\"dijitReset\"\n\t\t><td class=\"dijitReset dijitSliderButtonContainer dijitSliderButtonContainerH\"\n\t\t\t><div class=\"dijitSliderDecrementIconH\" style=\"display:none\" data-dojo-attach-point=\"decrementButton\"><span class=\"dijitSliderButtonInner\">-</span></div\n\t\t></td\n\t\t><td class=\"dijitReset\"\n\t\t\t><div class=\"dijitSliderBar dijitSliderBumper dijitSliderBumperH dijitSliderLeftBumper\" data-dojo-attach-event=\"press:_onClkDecBumper\"></div\n\t\t></td\n\t\t><td class=\"dijitReset\"\n\t\t\t><input data-dojo-attach-point=\"valueNode\" type=\"hidden\" ${!nameAttrSetting}\n\t\t\t/><div class=\"dijitReset dijitSliderBarContainerH\" role=\"presentation\" data-dojo-attach-point=\"sliderBarContainer\"\n\t\t\t\t><div role=\"presentation\" data-dojo-attach-point=\"progressBar\" class=\"dijitSliderBar dijitSliderBarH dijitSliderProgressBar dijitSliderProgressBarH\" data-dojo-attach-event=\"press:_onBarClick\"\n\t\t\t\t\t><div class=\"dijitSliderMoveable dijitSliderMoveableH\"\n\t\t\t\t\t\t><div data-dojo-attach-point=\"sliderHandle,focusNode\" class=\"dijitSliderImageHandle dijitSliderImageHandleH\" data-dojo-attach-event=\"press:_onHandleClick\" role=\"slider\"></div\n\t\t\t\t\t></div\n\t\t\t\t></div\n\t\t\t\t><div role=\"presentation\" data-dojo-attach-point=\"remainingBar\" class=\"dijitSliderBar dijitSliderBarH dijitSliderRemainingBar dijitSliderRemainingBarH\" data-dojo-attach-event=\"press:_onBarClick\"></div\n\t\t\t></div\n\t\t></td\n\t\t><td class=\"dijitReset\"\n\t\t\t><div class=\"dijitSliderBar dijitSliderBumper dijitSliderBumperH dijitSliderRightBumper\" data-dojo-attach-event=\"press:_onClkIncBumper\"></div\n\t\t></td\n\t\t><td class=\"dijitReset dijitSliderButtonContainer dijitSliderButtonContainerH\"\n\t\t\t><div class=\"dijitSliderIncrementIconH\" style=\"display:none\" data-dojo-attach-point=\"incrementButton\"><span class=\"dijitSliderButtonInner\">+</span></div\n\t\t></td\n\t></tr\n\t><tr class=\"dijitReset\"\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\n\t\t><td data-dojo-attach-point=\"containerNode,bottomDecoration\" class=\"dijitReset dijitSliderDecoration dijitSliderDecorationB dijitSliderDecorationH\"></td\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\n\t></tr\n></table>\n"}});
define("dijit/form/HorizontalSlider", [
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dnd/move",
	"dojo/_base/event", // event.stop
	"dojo/_base/fx", // fx.animateProperty
	"dojo/dom-geometry", // domGeometry.position
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/keys", // keys.DOWN_ARROW keys.END keys.HOME keys.LEFT_ARROW keys.PAGE_DOWN keys.PAGE_UP keys.RIGHT_ARROW keys.UP_ARROW
	"dojo/_base/lang", // lang.hitch
	"dojo/sniff", // has("ie") has("mozilla")
	"dojo/dnd/Moveable", // Moveable
	"dojo/dnd/Mover", // Mover Mover.prototype.destroy.apply
	"dojo/query", // query
	"dojo/mouse", // mouse.wheel
	"../registry", // registry.findWidgets
	"../focus",		// focus.focus()
	"../typematic",
	"./Button",
	"./_FormValueWidget",
	"../_Container",
	"dojo/text!./templates/HorizontalSlider.html"
], function(array, declare, move, event, fx, domGeometry, domStyle, keys, lang, has, Moveable, Mover, query, mouse,
			registry, focus, typematic, Button, _FormValueWidget, _Container, template){

// module:
//		dijit/form/HorizontalSlider


var _SliderMover = declare("dijit.form._SliderMover", Mover, {
	onMouseMove: function(e){
		var widget = this.widget;
		var abspos = widget._abspos;
		if(!abspos){
			abspos = widget._abspos = domGeometry.position(widget.sliderBarContainer, true);
			widget._setPixelValue_ = lang.hitch(widget, "_setPixelValue");
			widget._isReversed_ = widget._isReversed();
		}
		var pixelValue = e[widget._mousePixelCoord] - abspos[widget._startingPixelCoord];
		widget._setPixelValue_(widget._isReversed_ ? (abspos[widget._pixelCount]-pixelValue) : pixelValue, abspos[widget._pixelCount], false);
	},

	destroy: function(e){
		Mover.prototype.destroy.apply(this, arguments);
		var widget = this.widget;
		widget._abspos = null;
		widget._setValueAttr(widget.value, true);
	}
});

var HorizontalSlider = declare("dijit.form.HorizontalSlider", [_FormValueWidget, _Container], {
	// summary:
	//		A form widget that allows one to select a value with a horizontally draggable handle

	templateString: template,

	// Overrides FormValueWidget.value to indicate numeric value
	value: 0,

	// showButtons: [const] Boolean
	//		Show increment/decrement buttons at the ends of the slider?
	showButtons: true,

	// minimum: [const] Integer
	//		The minimum value the slider can be set to.
	minimum: 0,

	// maximum: [const] Integer
	//		The maximum value the slider can be set to.
	maximum: 100,

	// discreteValues: Integer
	//		If specified, indicates that the slider handle has only 'discreteValues' possible positions,
	//		and that after dragging the handle, it will snap to the nearest possible position.
	//		Thus, the slider has only 'discreteValues' possible values.
	//
	//		For example, if minimum=10, maxiumum=30, and discreteValues=3, then the slider handle has
	//		three possible positions, representing values 10, 20, or 30.
	//
	//		If discreteValues is not specified or if it's value is higher than the number of pixels
	//		in the slider bar, then the slider handle can be moved freely, and the slider's value will be
	//		computed/reported based on pixel position (in this case it will likely be fractional,
	//		such as 123.456789).
	discreteValues: Infinity,

	// pageIncrement: Integer
	//		If discreteValues is also specified, this indicates the amount of clicks (ie, snap positions)
	//		that the slider handle is moved via pageup/pagedown keys.
	//		If discreteValues is not specified, it indicates the number of pixels.
	pageIncrement: 2,

	// clickSelect: Boolean
	//		If clicking the slider bar changes the value or not
	clickSelect: true,

	// slideDuration: Number
	//		The time in ms to take to animate the slider handle from 0% to 100%,
	//		when clicking the slider bar to make the handle move.
	slideDuration: registry.defaultDuration,

	// Map widget attributes to DOMNode attributes.
	_setIdAttr: "",		// Override _FormWidget which sends id to focusNode

	baseClass: "dijitSlider",

	// Apply CSS classes to up/down arrows and handle per mouse state
	cssStateNodes: {
		incrementButton: "dijitSliderIncrementButton",
		decrementButton: "dijitSliderDecrementButton",
		focusNode: "dijitSliderThumb"
	},

	_mousePixelCoord: "pageX",
	_pixelCount: "w",
	_startingPixelCoord: "x",
	_handleOffsetCoord: "left",
	_progressPixelSize: "width",

	_onKeyUp: function(/*Event*/ e){
		if(this.disabled || this.readOnly || e.altKey || e.ctrlKey || e.metaKey){ return; }
		this._setValueAttr(this.value, true);
	},

	_onKeyPress: function(/*Event*/ e){
		if(this.disabled || this.readOnly || e.altKey || e.ctrlKey || e.metaKey){ return; }
		switch(e.charOrCode){
			case keys.HOME:
				this._setValueAttr(this.minimum, false);
				break;
			case keys.END:
				this._setValueAttr(this.maximum, false);
				break;
			// this._descending === false: if ascending vertical (min on top)
			// (this._descending || this.isLeftToRight()): if left-to-right horizontal or descending vertical
			case ((this._descending || this.isLeftToRight()) ? keys.RIGHT_ARROW : keys.LEFT_ARROW):
			case (this._descending === false ? keys.DOWN_ARROW : keys.UP_ARROW):
			case (this._descending === false ? keys.PAGE_DOWN : keys.PAGE_UP):
				this.increment(e);
				break;
			case ((this._descending || this.isLeftToRight()) ? keys.LEFT_ARROW : keys.RIGHT_ARROW):
			case (this._descending === false ? keys.UP_ARROW : keys.DOWN_ARROW):
			case (this._descending === false ? keys.PAGE_UP : keys.PAGE_DOWN):
				this.decrement(e);
				break;
			default:
				return;
		}
		event.stop(e);
	},

	_onHandleClick: function(e){
		if(this.disabled || this.readOnly){ return; }
		if(!has("ie")){
			// make sure you get focus when dragging the handle
			// (but don't do on IE because it causes a flicker on mouse up (due to blur then focus)
			focus.focus(this.sliderHandle);
		}
		event.stop(e);
	},

	_isReversed: function(){
		// summary:
		//		Returns true if direction is from right to left
		// tags:
		//		protected extension
		return !this.isLeftToRight();
	},

	_onBarClick: function(e){
		if(this.disabled || this.readOnly || !this.clickSelect){ return; }
		focus.focus(this.sliderHandle);
		event.stop(e);
		var abspos = domGeometry.position(this.sliderBarContainer, true);
		var pixelValue = e[this._mousePixelCoord] - abspos[this._startingPixelCoord];
		this._setPixelValue(this._isReversed() ? (abspos[this._pixelCount] - pixelValue) : pixelValue, abspos[this._pixelCount], true);
		this._movable.onMouseDown(e);
	},

	_setPixelValue: function(/*Number*/ pixelValue, /*Number*/ maxPixels, /*Boolean?*/ priorityChange){
		if(this.disabled || this.readOnly){ return; }
		var count = this.discreteValues;
		if(count <= 1 || count == Infinity){ count = maxPixels; }
		count--;
		var pixelsPerValue = maxPixels / count;
		var wholeIncrements = Math.round(pixelValue / pixelsPerValue);
		this._setValueAttr(Math.max(Math.min((this.maximum-this.minimum)*wholeIncrements/count + this.minimum, this.maximum), this.minimum), priorityChange);
	},

	_setValueAttr: function(/*Number*/ value, /*Boolean?*/ priorityChange){
		// summary:
		//		Hook so set('value', value) works.
		this._set("value", value);
		this.valueNode.value = value;
		this.focusNode.setAttribute("aria-valuenow", value);
		this.inherited(arguments);
		var percent = (value - this.minimum) / (this.maximum - this.minimum);
		var progressBar = (this._descending === false) ? this.remainingBar : this.progressBar;
		var remainingBar = (this._descending === false) ? this.progressBar : this.remainingBar;
		if(this._inProgressAnim && this._inProgressAnim.status != "stopped"){
			this._inProgressAnim.stop(true);
		}
		if(priorityChange && this.slideDuration > 0 && progressBar.style[this._progressPixelSize]){
			// animate the slider
			var _this = this;
			var props = {};
			var start = parseFloat(progressBar.style[this._progressPixelSize]);
			var duration = this.slideDuration * (percent-start/100);
			if(duration == 0){ return; }
			if(duration < 0){ duration = 0 - duration; }
			props[this._progressPixelSize] = { start: start, end: percent*100, units:"%" };
			this._inProgressAnim = fx.animateProperty({ node: progressBar, duration: duration,
				onAnimate: function(v){
					remainingBar.style[_this._progressPixelSize] = (100 - parseFloat(v[_this._progressPixelSize])) + "%";
				},
				onEnd: function(){
					delete _this._inProgressAnim;
				},
				properties: props
			});
			this._inProgressAnim.play();
		}else{
			progressBar.style[this._progressPixelSize] = (percent*100) + "%";
			remainingBar.style[this._progressPixelSize] = ((1-percent)*100) + "%";
		}
	},

	_bumpValue: function(signedChange, /*Boolean?*/ priorityChange){
		if(this.disabled || this.readOnly){ return; }
		var s = domStyle.getComputedStyle(this.sliderBarContainer);
		var c = domGeometry.getContentBox(this.sliderBarContainer, s);
		var count = this.discreteValues;
		if(count <= 1 || count == Infinity){ count = c[this._pixelCount]; }
		count--;
		var value = (this.value - this.minimum) * count / (this.maximum - this.minimum) + signedChange;
		if(value < 0){ value = 0; }
		if(value > count){ value = count; }
		value = value * (this.maximum - this.minimum) / count + this.minimum;
		this._setValueAttr(value, priorityChange);
	},

	_onClkBumper: function(val){
		if(this.disabled || this.readOnly || !this.clickSelect){ return; }
		this._setValueAttr(val, true);
	},

	_onClkIncBumper: function(){
		this._onClkBumper(this._descending === false ? this.minimum : this.maximum);
	},

	_onClkDecBumper: function(){
		this._onClkBumper(this._descending === false ? this.maximum : this.minimum);
	},

	decrement: function(/*Event*/ e){
		// summary:
		//		Decrement slider
		// tags:
		//		private
		this._bumpValue(e.charOrCode == keys.PAGE_DOWN ? -this.pageIncrement : -1);
	},

	increment: function(/*Event*/ e){
		// summary:
		//		Increment slider
		// tags:
		//		private
		this._bumpValue(e.charOrCode == keys.PAGE_UP ? this.pageIncrement : 1);
	},

	_mouseWheeled: function(/*Event*/ evt){
		// summary:
		//		Event handler for mousewheel where supported
		event.stop(evt);
		this._bumpValue(evt.wheelDelta < 0 ? -1 : 1, true); // negative scroll acts like a decrement
	},

	startup: function(){
		if(this._started){ return; }

		array.forEach(this.getChildren(), function(child){
			if(this[child.container] != this.containerNode){
				this[child.container].appendChild(child.domNode);
			}
		}, this);

		this.inherited(arguments);
	},

	_typematicCallback: function(/*Number*/ count, /*Object*/ button, /*Event*/ e){
		if(count == -1){
			this._setValueAttr(this.value, true);
		}else{
			this[(button == (this._descending? this.incrementButton : this.decrementButton)) ? "decrement" : "increment"](e);
		}
	},

	buildRendering: function(){
		this.inherited(arguments);
		if(this.showButtons){
			this.incrementButton.style.display="";
			this.decrementButton.style.display="";
		}

		// find any associated label element and add to slider focusnode.
		var label = query('label[for="'+this.id+'"]');
		if(label.length){
			if(!label[0].id){ label[0].id = this.id + "_label"; }
			this.focusNode.setAttribute("aria-labelledby", label[0].id);
		}

		this.focusNode.setAttribute("aria-valuemin", this.minimum);
		this.focusNode.setAttribute("aria-valuemax", this.maximum);
	},

	postCreate: function(){
		this.inherited(arguments);

		if(this.showButtons){
			this.own(
				typematic.addMouseListener(this.decrementButton, this, "_typematicCallback", 25, 500),
				typematic.addMouseListener(this.incrementButton, this, "_typematicCallback", 25, 500)
			);
		}
		this.connect(this.domNode, mouse.wheel, "_mouseWheeled");

		// define a custom constructor for a SliderMover that points back to me
		var mover = declare(_SliderMover, {
			widget: this
		});
		this._movable = new Moveable(this.sliderHandle, {mover: mover});

		this._layoutHackIE7();
	},

	destroy: function(){
		this._movable.destroy();
		if(this._inProgressAnim && this._inProgressAnim.status != "stopped"){
			this._inProgressAnim.stop(true);
		}
		this.inherited(arguments);
	}
});

HorizontalSlider._Mover = _SliderMover;	// for monkey patching

return HorizontalSlider;
});

},
'dojo/dnd/move':function(){
define("dojo/dnd/move", [
	"../_base/declare",
	"../dom-geometry", "../dom-style",
	"./common", "./Mover", "./Moveable"
], function(declare, domGeom, domStyle, dnd, Mover, Moveable){

// module:
//		dojo/dnd/move

/*=====
var __constrainedMoveableArgs = declare([Moveable.__MoveableArgs], {
	// constraints: Function
	//		Calculates a constraint box.
	//		It is called in a context of the moveable object.
	constraints: function(){},

	// within: Boolean
	//		restrict move within boundaries.
	within: false
});
=====*/

var constrainedMoveable = declare("dojo.dnd.move.constrainedMoveable", Moveable, {
	// object attributes (for markup)
	constraints: function(){},
	within: false,

	constructor: function(node, params){
		// summary:
		//		an object that makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: __constrainedMoveableArgs?
		//		an optional object with additional parameters;
		//		the rest is passed to the base class
		if(!params){ params = {}; }
		this.constraints = params.constraints;
		this.within = params.within;
	},
	onFirstMove: function(/*Mover*/ mover){
		// summary:
		//		called during the very first move notification;
		//		can be used to initialize coordinates, can be overwritten.
		var c = this.constraintBox = this.constraints.call(this, mover);
		c.r = c.l + c.w;
		c.b = c.t + c.h;
		if(this.within){
			var mb = domGeom.getMarginSize(mover.node);
			c.r -= mb.w;
			c.b -= mb.h;
		}
	},
	onMove: function(/*Mover*/ mover, /*Object*/ leftTop){
		// summary:
		//		called during every move notification;
		//		should actually move the node; can be overwritten.
		var c = this.constraintBox, s = mover.node.style;
		this.onMoving(mover, leftTop);
		leftTop.l = leftTop.l < c.l ? c.l : c.r < leftTop.l ? c.r : leftTop.l;
		leftTop.t = leftTop.t < c.t ? c.t : c.b < leftTop.t ? c.b : leftTop.t;
		s.left = leftTop.l + "px";
		s.top  = leftTop.t + "px";
		this.onMoved(mover, leftTop);
	}
});

/*=====
var __boxConstrainedMoveableArgs = declare([__constrainedMoveableArgs], {
	// box: Object
	//		a constraint box
	box: {}
});
=====*/

var boxConstrainedMoveable = declare("dojo.dnd.move.boxConstrainedMoveable", constrainedMoveable, {
	// box:
	//		object attributes (for markup)
	box: {},

	constructor: function(node, params){
		// summary:
		//		an object, which makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: __boxConstrainedMoveableArgs?
		//		an optional object with parameters
		var box = params && params.box;
		this.constraints = function(){ return box; };
	}
});

/*=====
var __parentConstrainedMoveableArgs = declare( [__constrainedMoveableArgs], {
	// area: String
	//		A parent's area to restrict the move.
	//		Can be "margin", "border", "padding", or "content".
	area: ""
});
=====*/

var parentConstrainedMoveable = declare("dojo.dnd.move.parentConstrainedMoveable", constrainedMoveable, {
	// area:
	//		object attributes (for markup)
	area: "content",

	constructor: function(node, params){
		// summary:
		//		an object, which makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: __parentConstrainedMoveableArgs?
		//		an optional object with parameters
		var area = params && params.area;
		this.constraints = function(){
			var n = this.node.parentNode,
				s = domStyle.getComputedStyle(n),
				mb = domGeom.getMarginBox(n, s);
			if(area == "margin"){
				return mb;	// Object
			}
			var t = domGeom.getMarginExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			if(area == "border"){
				return mb;	// Object
			}
			t = domGeom.getBorderExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			if(area == "padding"){
				return mb;	// Object
			}
			t = domGeom.getPadExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			return mb;	// Object
		};
	}
});


return {
	// summary:
	//		TODOC
	constrainedMoveable: constrainedMoveable,
	boxConstrainedMoveable: boxConstrainedMoveable,
	parentConstrainedMoveable: parentConstrainedMoveable
};

});

},
'url:dijit/form/templates/HorizontalSlider.html':"<table class=\"dijit dijitReset dijitSlider dijitSliderH\" cellspacing=\"0\" cellpadding=\"0\" border=\"0\" rules=\"none\" data-dojo-attach-event=\"onkeypress:_onKeyPress,onkeyup:_onKeyUp\"\n\trole=\"presentation\"\n\t><tr class=\"dijitReset\"\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\n\t\t><td data-dojo-attach-point=\"topDecoration\" class=\"dijitReset dijitSliderDecoration dijitSliderDecorationT dijitSliderDecorationH\"></td\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\n\t></tr\n\t><tr class=\"dijitReset\"\n\t\t><td class=\"dijitReset dijitSliderButtonContainer dijitSliderButtonContainerH\"\n\t\t\t><div class=\"dijitSliderDecrementIconH\" style=\"display:none\" data-dojo-attach-point=\"decrementButton\"><span class=\"dijitSliderButtonInner\">-</span></div\n\t\t></td\n\t\t><td class=\"dijitReset\"\n\t\t\t><div class=\"dijitSliderBar dijitSliderBumper dijitSliderBumperH dijitSliderLeftBumper\" data-dojo-attach-event=\"press:_onClkDecBumper\"></div\n\t\t></td\n\t\t><td class=\"dijitReset\"\n\t\t\t><input data-dojo-attach-point=\"valueNode\" type=\"hidden\" ${!nameAttrSetting}\n\t\t\t/><div class=\"dijitReset dijitSliderBarContainerH\" role=\"presentation\" data-dojo-attach-point=\"sliderBarContainer\"\n\t\t\t\t><div role=\"presentation\" data-dojo-attach-point=\"progressBar\" class=\"dijitSliderBar dijitSliderBarH dijitSliderProgressBar dijitSliderProgressBarH\" data-dojo-attach-event=\"press:_onBarClick\"\n\t\t\t\t\t><div class=\"dijitSliderMoveable dijitSliderMoveableH\"\n\t\t\t\t\t\t><div data-dojo-attach-point=\"sliderHandle,focusNode\" class=\"dijitSliderImageHandle dijitSliderImageHandleH\" data-dojo-attach-event=\"press:_onHandleClick\" role=\"slider\"></div\n\t\t\t\t\t></div\n\t\t\t\t></div\n\t\t\t\t><div role=\"presentation\" data-dojo-attach-point=\"remainingBar\" class=\"dijitSliderBar dijitSliderBarH dijitSliderRemainingBar dijitSliderRemainingBarH\" data-dojo-attach-event=\"press:_onBarClick\"></div\n\t\t\t></div\n\t\t></td\n\t\t><td class=\"dijitReset\"\n\t\t\t><div class=\"dijitSliderBar dijitSliderBumper dijitSliderBumperH dijitSliderRightBumper\" data-dojo-attach-event=\"press:_onClkIncBumper\"></div\n\t\t></td\n\t\t><td class=\"dijitReset dijitSliderButtonContainer dijitSliderButtonContainerH\"\n\t\t\t><div class=\"dijitSliderIncrementIconH\" style=\"display:none\" data-dojo-attach-point=\"incrementButton\"><span class=\"dijitSliderButtonInner\">+</span></div\n\t\t></td\n\t></tr\n\t><tr class=\"dijitReset\"\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\n\t\t><td data-dojo-attach-point=\"containerNode,bottomDecoration\" class=\"dijitReset dijitSliderDecoration dijitSliderDecorationB dijitSliderDecorationH\"></td\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\n\t></tr\n></table>\n",
'epi-find/widget/WeightSelector':function(){
﻿define([
    "dojo/_base/declare",
    "dojo/_base/array",
    "dojo/_base/lang",
    "dojo/query",
    "dojo/dom-construct",
    "dojo/dom-class",
    "dijit/_Container",
    "dijit/form/RadioButton",
    "dijit/_WidgetBase",
    "epi-saas-base/form/RadioButtonList",
    "dojo/i18n!./nls/WeightSelector"
], function (declare, array, lang, query, domConstruct, domClass, _Container, RadioButton, _WidgetBase, RadioButtonList, i18n) {

    // module:
    //      epi-find/widget/WeightSelector

    return declare([_WidgetBase, _Container], {
        // summary:
        //              Weight selector for picking a weight.
        // description:
        //              The selector is configurable and can take a custom array of values and/or labels.
        //              The first item is the "none-selected" value and label that is returned if no selection was made
        //              Default value for the weight selector is 0 (None selected)
        // example:
        //              In the example below, -2 is the none-selected value, and "None" is the none-selected label
        //              Watches on the selector's value and label writes the user selections to the console
        //
        // |            <script>
        // |            require(["dojo/dom", "epi-find/widget/WeightSelector", "dojo/domReady!"],
        // |                function(dom, WeightSelector) {
        // |                    var weightContainer = dom.byId("widgetNode"),
        // |                        selector = new WeightSelector({
        // |                            value: 1,
        // |                            values: [{value: -2, label: "None"},
        // |                                     {value: -1, label: "Cold"},
        // |                                     {value: 0, label: "Mild"},
        // |                                     {value: 1, label:"Hot"}]
        // |                        });
        // |                    selector.placeAt(weightContainer);
        // |                    selector.startup();
        // |                    selector.watch("value", function (attr, oldVal, newVal) {
        // |                        console.log("Changed value from " + oldVal + " to " + newVal);
        // |                    });
        // |                    selector.watch("label", function (attr, oldVal, newVal) {
        // |                        console.log("Changed level from " + oldVal + " to " + newVal);
        // |                    });
        // |                });
        // |            </script>
        // |            <div id="widgetNode"></div>

        // value: Number
        //      Value of selected weight
        value: 0,

        _setValueAttr: function (/*Number*/ value) {
            // summary:
            //          Setter for the value attribute
            // value: Number
            //          The new value to assign to the weight selector

            //set new value and label
            this._set("value", value);
            this.set("label", this._getLabelKeyForValue(value));

            if(value == this.values[0].value){ //non-selected value
                //set checked state of the radio buttons to false
                array.forEach(this._radioButtons, function(radioButton){
                    radioButton.set("checked",false);
                    query(".is-filled", this.domNode).forEach(function(node){
                        domClass.remove(node, "is-filled");
                    });
                }, this);
            }
            else{
                //find the radio button for the specified value and set its checked state
                var radioButton = array.filter(this._radioButtons, function (item) {
                    return item.value == value;
                });
                if (radioButton.length) {
                    radioButton[0].set("checked", true);
                }
            }
        },

        // label: String
        //      Description of checked weight. Updated when value changes
        label: "",

        // Setter for the label attribute
        // Maps the attribute to _labelDomNode
        _setLabelAttr: {node: "_labelDomNode", type: "innerHTML"},

        // disabled: Boolean
        //      Disable the weight selector by setting disabled to true
        disabled: false,

        _setDisabledAttr: function(/*Boolean*/ disabled){
            //summary:
            //          Setter for the disabled attribute
            // disabled: Boolean
            //          true if weight selector should be disabled, false if enabled

            this._set("disabled", disabled);
            this._toggleState(disabled);
        },

        // values: Object[] of {value: Number, label: String}
        //      Values to use for radio buttons
        //      First value is the none-selected value
        values: null,

        // baseClass: String
        //      Css class to use for the weight selector
        baseClass: "epi-weight-selector",

        _toggleState: function(/*Boolean*/ disabled){
            // summary:
            //          toggles the radio buttons' disabled property based on disabled
            // disabled: Boolean
            //          true if radio buttons should be disabled, false if enabled

            array.forEach(this._radioButtons, function(item) {
                item.set("disabled", disabled);
            });
        },

        // _labelDomNode: DomNode
        //      Reference to the label domNode for displaying label text
        _labelDomNode: null,

        // _radioButtons: dijit/form/RadioButton[]
        //      Reference to all radio buttons for managing checked state upon value changes
        _radioButtons: null,

        constructor: function () {
            // summary:
            //          Constructs an instance of WeightSelector widget
            // options: Object[] of { value: Number, label: String }

            //default values and labels
            //keep reference in _defaultValues for postMixInProperties
            this.values = this._defaultValues = [
                { value: 0, label: i18n.none },
                { value: 1, label: i18n.veryLow },
                { value: 2, label: i18n.low },
                { value: 3, label: i18n.medium },
                { value: 4, label: i18n.high },
                { value: 5, label: i18n.veryHigh }
            ];

            this._radioButtons = [];
        },

        postMixInProperties: function(){
            // Merge value objects with default value objects
            array.forEach(this.values, function (value, idx) {
                this.values[idx] = lang.mixin({}, this._defaultValues[idx] || {}, value instanceof Object ? value : { value: value });
            }, this);
        },

        _getLabelKeyForValue: function (/*Number*/ value) {
            // summary:
            //              Get the label for the specified value
            // description:
            //              If fewer labels than values specified, and no matching label found, returns value
            //              If value does not match any in values array, return localized message "value out of range"
            // value: Number
            //              Value for which to return a label
            // returns: String

            var matches = array.filter(this.values, function(item){
                return item.value === value;
            });
            return matches.length ? matches[0].label || value : i18n.outOfRange;
        },

        _createRadioButtons: function() {
            // summary:
            //          Create and return an array of radio buttons from the values array,
            //              except for the first item (non-selected value)
            // returns: dijit/form/RadioButton[]

            var self = this,

                //create a radio button with the specified value
                createRadioBtn = function(decValue) {
                    var radBtn = new RadioButton({
                        value: decValue,
                        name: "Weight " + self.id //widget's id, thus same for all radio buttons (group)
                    });
                    return radBtn;
                },

                //generate configurable number of radio buttons, except first item (exclusion value)
                radioButtons = array.map(this.values.slice(1), function(item) {
                    return createRadioBtn(item.value);
                });
            return radioButtons;
        },

        buildRendering: function() {
            // summary:
            //          Build and render the weight selector widget

            this.inherited(arguments);
            var self = this,
                //inherit from RadioButtonList and override createListItem to manage is-filled css class
                SelectorList = declare([RadioButtonList], {
                    createListItem: function(radioButton, options) {
                        var listItem = this.inherited(arguments),
                            list = this,
                            checkedBtnIndex = this.getIndexOfChild(listItem),
                            setDomClasses = function(){
                                var children = list.getChildren();
                                //set and remove the is-filled css class depending on before or after checked radiobutton
                                array.forEach(children, function(sibling, siblingIndex){
                                    if (siblingIndex <= checkedBtnIndex) {
                                        domClass.add(sibling.domNode, "is-filled");
                                    } else {
                                        domClass.remove(sibling.domNode, "is-filled");
                                    }
                                });
                            };

                        self.own(radioButton.watch("checked", function(name, oldValue, newValue) {
                            if (newValue) {
                                //set value of widget to selected radio button
                                self.set("value", radioButton.value);
                                setDomClasses();
                            }
                        }));
                    }
                }),
                weightList = new SelectorList();

            // create radio buttons and add to container widget weightList
            this.own(weightList);
            this.addChild(weightList);
            this._radioButtons = this._createRadioButtons();
            array.forEach(this._radioButtons, function (item, radioButtonIndex) {
                self.own(item);
                weightList.createListItem(item, { blockClass: "epi-weight-selector__block", inputStyleType: radioButtonIndex+1 });
            });

            //Add label to the dom
            this._labelDomNode = domConstruct.toDom("<span id=label" + this.id + "></span>");
            domConstruct.place(this._labelDomNode, this.domNode, "last");
        }
    });
});
},
'epi-saas-base/form/RadioButtonList':function(){
define("epi-saas-base/form/RadioButtonList", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-class",

    "./FormList",
    "./FormListItem"
], function (declare, lang, domClass,
    FormList, FormListItem) {
    
    // module:
    //      epi-find/widget/RadioButtonList
    
    return declare([FormList], {

        // summary:
        //     Radio button list based on FormList to provide the proper markup for styling
        
        createListItem: function(radioButton, options) {
            options = lang.mixin({ blockClass: "" }, options);
            var listItem = new FormListItem(options);
            listItem.addChild(radioButton);
            this.addChild(listItem);
            this.own(
                radioButton.watch("checked", function() {
                    domClass.toggle(listItem.domNode, "is-active");
                })
            );
            return listItem;
        }
    });
});
},
'url:epi-find/widget/templates/BoostingEditor.html':"﻿<div class=\"epi-boosting__sections  epi-boosting__editor-section\">\r\n    <div data-dojo-attach-point=\"propertiesGridNode\" class=\"epi-chromeless\"></div>\r\n    <div>\r\n        <input data-dojo-attach-point=\"submitButton\" type=\"submit\" data-dojo-type=\"dijit/form/Button\"\r\n               class=\"epi-primary\" />\r\n        <input data-dojo-attach-point=\"resetButton\" type=\"reset\" data-dojo-type=\"dijit/form/Button\" />\r\n        <input data-dojo-attach-point=\"resetToDefaultButton\" data-dojo-type=\"dijit/form/Button\"\r\n               class=\"epi-chromeless epi-visibleLink\" />\r\n    </div>\r\n    <p class=\"epi-help\">${i18n.help_weights}</p>\r\n</div>\r\n",
'epi-find/widget/BoostingPreview':function(){
﻿    define([
        "dojo/_base/declare",
        "dojo/_base/lang",
        "dojo/_base/array",
        "dojo/dom-construct",
        "dojo/json",

        "dijit/_WidgetBase",
        "dijit/_TemplatedMixin",
        "dijit/_WidgetsInTemplateMixin",
        "dijit/form/CheckBox",
        "dijit/form/TextBox",
        "dijit/form/Select",

        "dojox/html/entities",
        "dojox/mvc/at",

        "dgrid/List",
        "dgrid/extensions/DijitRegistry",

        "epi-saas-base/widgets/_WidgetsInListMixin",
        "./BestbetGridRow",

        "epi-saas-base/widgets/_NotificationMixin",
        "epi-saas-base/widgets/_GridMoreMixin",
        "epi-saas-base/widgets/_AutoLoadScrollMixin",

        "dojo/i18n!epi-saas-base/nls/Languages",
        "dojo/i18n!./nls/BoostingPreview",
        "dojo/text!./templates/BoostingPreview.html"
],
    function(declare,
            lang,
            array,
            domConstruct,
            JSON,

            _WidgetBase,
            _TemplatedMixin,
            _WidgetsInTemplateMixin,
            CheckBox,
            TextBox,
            Select,

            entities,
            at,

            List,
            DijitRegistry,
            _WidgetsInListMixin,
            BestbetGridRow,
            _NotificationMixin,
            _GridMoreMixin,
            _AutoLoadScrollMixin,

            i18nLanguages,
            i18n,
            template) {

        // module:
        //      epi-find/widget/BoostingPreview

        return declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin], {
            // summary:
            //     Boosting preview view.

            templateString: template,

            i18n: i18n,

            model: null,

            resultsList: null,

            _searchTimeout: 500,
            _searchTimer: null,

            _allLanguagesValue: "allLanguages",

            postCreate: function() {
                this.inherited(arguments);

                this.searchText.set("placeHolder", i18n.search);

                if (!this.model) {
                    throw "No model specified";
                }

                var self = this;

                this.searchText.set("value", at(this.model, "searchText"));

                var languageSelectOptions = array.map(this.model.languages, function (item) {
                    return { value: item, label: i18nLanguages[item] || (i18nLanguages[item.substring(0, 2)] ? i18nLanguages[item.substring(0, 2)] + " (" + item + ")" : null) || item };
                });
                languageSelectOptions.unshift({ value: this._allLanguagesValue, label: i18n.allLanguages });
                this.languageSelect.set("options", languageSelectOptions);

                this.languageSelect.set("value", at(this.model, "language").transform({
                    format: function(language){
                        return language || self._allLanguagesValue;
                    },
                    parse: function(value){
                        return value === self._allLanguagesValue ? null : value;
                    }}));

                this.bestBetsCheckbox.set("checked", at(this.model, "bestBets"));
                this.synonymsCheckbox.set("checked", at(this.model, "synonyms"));

                var ResultList = declare([List, _GridMoreMixin, _AutoLoadScrollMixin, DijitRegistry, _WidgetsInListMixin, _NotificationMixin], {
                    constructor: function() {
                        // Sets the loader (#loadNextPage, available in _GridMoreMixn) to be used by the _AutoLoadScrollMixin
                        this.loader = lang.hitch(this, this.loadNextPage);
                    },
                    margin: 140
                });

                this.resultsList = new ResultList({
                    i18n: i18n,
                    store: this.model.store,
                    query: this.model.get("query"),
                    loadingMessage: i18n.dataLoading,
                    showLoadingMessage: true,
                    "class": "epi-plain-grid epi-plain-grid--no-border",
                    useTouchScroll: false,
                    renderRow: function (object, options) {
                        var row = new BestbetGridRow({
                            title: object.Title || object.Url,
                            url: object.Url,
                            description: object.Excerpt,
                            phrases: entities.encode((object.MetaData && object.MetaData.BestBetPhrase) ? object.MetaData.BestBetPhrase.StringValue : null)
                        });
                        self.own(row);
                        return row.domNode;
                    }
                }, this.searchResultsNode);

                this.own(
                    this.resultsList.on("dgrid-error", function(event) {
                        self.resultsList.showErrorInGrid(event.error);
                    }),
                    this.resultsList,
                    this.model.watch("query", lang.hitch(this, function () {
                        this._triggerDelayedSearch();
                    })));
            },

            startup: function() {
                this.inherited(arguments);
                this.resultsList.set("scrollNode", this.scrollNode || this.domNode.parentNode);
                this.resultsList.startup();
            },

            _triggerDelayedSearch: function() {
                //  summary:
                //      Runs search query after delay in order to avoid extra requests.
                if (this._searchTimer) {
                    clearTimeout(this._searchTimer);
                }
                this._searchTimer = setTimeout(lang.hitch(this, function () {
                    this._searchTimer = null;
                    this.resultsList.set("query", this.model.get("query"));
                }), this._searchTimeout);
            }
        });
    });

},
'url:epi-find/widget/templates/BoostingPreview.html':"﻿<div class=\"epi-boosting__sections  epi-boosting__preview-section\">\r\n    <h3>${i18n.preview}</h3>\r\n    <div>\r\n        <div class=\"epi-boosting__search-wrapper\">\r\n            <p class=\"epi-secondaryText\">${i18n.description}</p>\r\n            <div class=\"epi-vertical-spacer\">\r\n              <input id=\"searchText\"\r\n                     type=\"text\"\r\n                     data-dojo-attach-point=\"searchText\"\r\n                     data-dojo-type=\"dijit/form/TextBox\"\r\n                     data-dojo-props=\"intermediateChanges: true\" />\r\n              <input id=\"languageSelect\"\r\n                     data-dojo-attach-point=\"languageSelect\"\r\n                     data-dojo-type=\"dijit/form/Select\"\r\n                     data-dojo-props=\"trim:true\" />\r\n            </div>\r\n            <input id=\"bestBetsCheckbox\"\r\n                   type=\"checkbox\"\r\n                   data-dojo-attach-point=\"bestBetsCheckbox\"\r\n                   data-dojo-type=\"dijit/form/CheckBox\">\r\n            <label for=\"bestBetsCheckbox\">${i18n.bestBets}</label>\r\n            <input id=\"synonymsCheckbox\"\r\n                   type=\"checkbox\"\r\n                   data-dojo-attach-point=\"synonymsCheckbox\"\r\n                   data-dojo-type=\"dijit/form/CheckBox\">\r\n            <label for=\"synonymsCheckbox\">${i18n.synonyms}</label>\r\n\r\n        </div>\r\n    </div>\r\n    <div>\r\n        <div data-dojo-attach-point=\"searchResultsNode\"></div>\r\n    </div>\r\n</div>\r\n",
'url:epi-find/widget/templates/Boosting.html':"<div>\r\n    <p class=\"epi-view--description\">${i18n.description}</p>\r\n    <div data-dojo-attach-point=\"boostingEditorNode\"></div>\r\n    <div data-dojo-attach-point=\"boostingPreviewNode\"></div>\r\n</div>",
'epi-find/widget/Index':function(){
﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/on",
    "dojo/when",
    "dojo/dom-class",

    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/layout/_LayoutWidget",
    "dijit/form/Button",

    "epi-saas-base/widgets/_NotificationMixin",
    "epi-saas-base/widgets/_ActionButtonsMixin",

    "./_ActionableMixin",
    "./_DisabledViewMixin",

    "dojo/i18n!./nls/Index",
    "dojo/text!./templates/Index.html"
],
    function(declare,
             lang,
             on,
             when,
             domClass,
            _WidgetBase,
            _TemplatedMixin,
            _WidgetsInTemplateMixin,
            _LayoutWidget,
            Button,
            _NotificationMixin,
            _ActionButtonsMixin,
            _ActionableMixin,
			_DisabledViewMixin,
            i18n,
            template) {
        return declare([_WidgetBase, _LayoutWidget, _TemplatedMixin, _WidgetsInTemplateMixin, _ActionableMixin, _DisabledViewMixin, _NotificationMixin, _ActionButtonsMixin], {
            // summary:
            //     Clear content index and stats view.

            templateString: template,

            i18n: i18n,

            model: null,

            // reindexNote: String
            //      The note about possibly required reindexing after clearing operation.
            reindexNote: null,

            postMixInProperties: function(){
                this.inherited(arguments);
                this.reindexNote = this.model.indexingPluginUrl ?
                    lang.replace(i18n.reindexNote,
                        [ lang.replace("<a href='{0}' class='epi-visibleLink'>{1}</a>",
                            [this.model.indexingPluginUrl, i18n.indexingJobPageTitle])
                        ]) : "";
                this.clearIndexDescription = this.model.indexingPluginUrl ?
                    lang.replace(i18n.clearIndexDescription, [this.model.indexingPluginUrl]) : "";
            },

            postCreate: function() {
                this.inherited(arguments);

                this.clearIndexButton.set("label", i18n.buttons_clearIndex);
                this.clearStatsButton.set("label", i18n.buttons_clearStats);

                if (!this.model) {
                    throw "No model specified";
                }

                this.actionButtons = [
                    this.clearIndexButton,
                    this.clearStatsButton
                ];

                this.disableWhenNoChanges = false;

                this.own(
                    on(this.clearIndexButton, "click", lang.hitch(this, function (e) {
                        e.preventDefault();
                        e.stopPropagation();

                        this.showConfirmation({
                            title: i18n.notification_clearIndexTitle,
                            message: i18n.notification_clearIndexDescription,
                            action: lang.hitch(this, this.clearIndex),
                            confirmActionText: i18n.buttons_confirmClearContent,
                            cancelActionText: i18n.buttons_confirmCancel
                        });
                    })),

                    on(this.clearStatsButton, "click", lang.hitch(this, function (e) {
                        e.preventDefault();
                        e.stopPropagation();

                        this.showConfirmation({
                            title: i18n.notification_clearStatsTitle,
                            message: i18n.notification_clearStatsDescription,
                            action: lang.hitch(this, this.clearStats),
                            confirmActionText: i18n.buttons_confirmClearStatistics,
                            cancelActionText: i18n.buttons_confirmCancel
                        });
                    })));
            },


            clearIndex: function(){
                // summary:
                //          Clears the content index and shows results
                this.indicateInProgress(this.clearIndexButton, i18n.buttons_clearingIndex);

                return when(this.model.clearContentIndex(), lang.hitch(this, function(success) {
                    this.indicateSuccess(this.clearIndexButton, i18n.buttons_clearIndex);
                    this.showSuccess(i18n.notification_successClearIndex + "<br/>" + this.reindexNote,
                        i18n.notification_clearIndexTitle);
                }), lang.hitch(this,function(error){
                    this.indicateError(this.clearIndexButton, i18n.buttons_clearIndex);
                    var errorText = (error || {}).responseText ? error.responseText : i18n.notification_errorClearIndex;
                    this.showError(error, errorText +
                        (error ? " " + i18n.notification_errorDetails : ""),
                        i18n.notification_clearIndexTitle);
                }));
            },

            clearStats: function(){
                // summary:
                //          Clears the statistics and shows results

                this.indicateInProgress(this.clearStatsButton, i18n.buttons_clearingStats);

                return when(this.model.clearStats(), lang.hitch(this, function(success){
                    this.indicateSuccess(this.clearStatsButton, i18n.buttons_clearStats);
                    this.showSuccess(i18n.notification_successClearStats, i18n.notification_clearStatsTitle);
                }), lang.hitch(this, function(error){
                    this.indicateError(this.clearStatsButton, i18n.buttons_clearStats);
                    var errorText = (error || {}).responseText ? error.responseText : i18n.notification_errorClearStats;
                    this.showError(error, errorText +
                        (error ? " " + i18n.notification_errorDetails : ""),
                        i18n.notification_clearStatsTitle);
                }));
            }
        });
    });

},
'url:epi-find/widget/templates/Index.html':"﻿<form data-dojo-attach-point='containerNode'>\r\n    <p class=\"epi-clear-index--description\">${i18n.description}</p>\r\n    <div class=\"epi-clear-index-wrapper\">\r\n      <div class=\"epi-clear-index-column\">\r\n        <h1>${i18n.labels_clearIndex}</h1>\r\n        <p>${clearIndexDescription}</p>\r\n        <span class=\"epi-sprite epi-sprite--clear-content\"></span>\r\n\r\n        <input data-dojo-attach-point=\"clearIndexButton\"\r\n                     type=\"button\"\r\n                     data-dojo-type=\"dijit/form/Button\"\r\n                     data-dojo-props=\"class: 'epi-danger'\" />\r\n      </div>\r\n\r\n      <div class=\"epi-clear-index-column\">\r\n        <h1>${i18n.labels_clearStats}</h1>\r\n        <p>${i18n.clearStatsDescription}</p>\r\n        <span class=\"epi-sprite epi-sprite--clear-stats\"></span>\r\n\r\n        <input data-dojo-attach-point=\"clearStatsButton\"\r\n                   type=\"button\"\r\n                   data-dojo-type=\"dijit/form/Button\"\r\n                   data-dojo-props=\"class: 'epi-danger'\" />\r\n      </div>\r\n    </div>\r\n</form>\r\n",
'url:epi-find/widget/templates/Configure.html':"﻿<div class=\"epi-pane  epi-pane--tools is-active epi-focused\">\r\n    <div class=\"epi-pane__content-wrapper\">\r\n        <div class=\"epi-pane__header\">\r\n            <h1>${i18n.header}</h1>\r\n            <p class=\"epi-pane__description\">${i18n.description}</p>\r\n        </div>\r\n        <div data-dojo-attach-point=\"contentNode\" class=\"epi-pane__content\">\r\n            <p class=\"epi-pane__description\">${i18n.description}</p>\r\n                <ul class=\"epi-pane__content__nav clearfix\">\r\n                    <li><a href=\"#\" title=\"${i18n.nav_boosting}\" class=\"epi-pane__content__header-link--boosting\" data-dojo-attach-point=\"boostingLink\">${i18n.nav_boosting}</a></li>\r\n                    <li><a href=\"#\" title=\"${i18n.nav_index}\" class=\"epi-pane__content__header-link--index\" data-dojo-attach-point=\"indexLink\">${i18n.nav_index}</a></li>\r\n                    <li><a href=\"#\" title=\"${i18n.nav_connectors}\" class=\"epi-pane__content__header-link--connectors\" data-dojo-attach-point=\"connectorsLink\">${i18n.nav_connectors}</a></li>\r\n                </ul>\r\n                <div data-dojo-type=\"dijit/layout/StackContainer\" data-dojo-attach-point=\"widgetContainer\" data-dojo-props=\"doLayout: false, isLayoutContainer: true\" class=\"epi-pane__content__content-container\"></div>\r\n        </div>\r\n    </div>\r\n</div>",
'epi-find/servicehealth/ServiceHealthController':function(){
﻿define([
    "dojo/_base/declare",
    "../_ControllerBase",
    "./ServiceHealthModel",
    "../widget/ServiceHealth",
    "../widget/_ActionableMixin"
],
function(declare,
    _ControllerBase,
    ServiceHealthModel,
    ServiceHealth,
    _ActionableMixin
) {
    return declare([_ControllerBase, _ActionableMixin], {
        // summary:
        //      Controller for Service Health views.

        model: null,

        widget: null,

        startup: function() {
            if (this.isStarted()) {
                return;
            }
            this.inherited(arguments);

            this.model = new ServiceHealthModel();
            this.model.init();

            this.widget = new ServiceHealth({
                model: this.model
            });
            this.own(this.widget);
        },

        show: function() {
            this._setupView([this.widget]);
        }
    });
});

},
'epi-find/servicehealth/ServiceHealthModel':function(){
﻿define([
    "dojo/_base/declare",
    "dojo/_base/config",
    "dojo/_base/lang",
    "dojo/when",
    "dojo/Stateful",
    "dijit/Destroyable"
],
function (
    declare,
    config,
    lang,
    when,
    Stateful,
    Destroyable
    ) {

    return declare([Stateful, Destroyable], {
        // summary:
        //      Service Health model.
        store: null,
        url: null,

        postscript: function() {
            this.inherited(arguments);
            if (!this.store) {
                this.store = config.dependencies["epi-find.ServiceHealthStore"];
            }
        },

        init: function () {
        },

        load: function () {
            var servicehealthQuery = this.store.query();

            when(servicehealthQuery.data, lang.hitch(this, function (data) {
                this.set("url", data.baseUrl + "?token=" + data.token);
            }), lang.hitch(this, function (error) {
                this.set("url", "");
            }));
        }
    });
});

},
'epi-find/widget/ServiceHealth':function(){
﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",

    "./_ActionableMixin",
    "./_DisabledViewMixin",

    "dojo/i18n!./nls/ServiceHealth",
    "dojo/text!./templates/ServiceHealth.html"
], function(declare,
    lang,
    array,
    _WidgetBase,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,

    _ActionableMixin,
    _DisabledViewMixin,

    i18n,
    template
) {
    var iFrame = declare([_WidgetBase], {
        // template: String
        //      A string which contains template markers; for example: "{0} message".
        //      {0} would then be replaced by the first element in descriptionVariables.
        template: null,

        // descriptionVariables: Mixed[]
        //  The variables to use when building the description string from the template.
        descriptionVariables: null,

        // formatter: function(key, value)
        //      A function which runs for every description value when the text is rebuilt. Should be overidden.
        formatter: function(key, value){},

        constructor: function () {
            this.descriptionVariables = [];
        },

        _setDescriptionVariablesAttr: function(variables) {
            this._set("descriptionVariables", variables);
            var description = this.template;

            array.forEach(variables, function(item, index) {
                if (this.formatter) {
                    item = this.formatter(index, item);
                }
                description = description.replace("{" + index + "}", item);
            }, this);
            this.set("description", description);
        },

        _setDescriptionAttr: { node: "domNode", type: "innerHTML" }
    });

    return declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, _ActionableMixin, _DisabledViewMixin], {
        // summary:
        //    Service Health widget

        templateString: template,

        i18n: i18n,

        model: null,
        _dashboard: null,

        buildRendering: function () {
            this.inherited(arguments);
            this._dashboard = new iFrame({
                template: "<iframe src='{0}' frameborder='0' style='width:100%; height:100%; min-height:800px; border:0;'></iframe>",
                formatter: function (key, value) {
                    return value !== null ? value : "?";
                },
                descriptionVariables: [null]
            }, this.dashboardFrame);
        },

        postCreate: function() {
            this.inherited(arguments);

            if (!this.model) {
                throw new Error("'model' is not defined");
            }

            this.bindDashboard();
            this.model.load();
        },

        startup: function () {
            if (this._started) {
                return;
            }
            this.inherited(arguments);
        },

        bindDashboard: function () {
            this.own(
                this.model.watch("url", lang.hitch(this, function () {
                    this._dashboard.set("descriptionVariables", [this.model.url]);
                }))
            );
        }
    });
});
},
'url:epi-find/widget/templates/ServiceHealth.html':"﻿<div class=\"epi-pane epi-pane--tools is-active epi-focused\">\r\n    <div class=\"epi-pane__content-wrapper\">\r\n        <div class=\"epi-pane__header\">\r\n            <h1>${i18n.header}</h1>\r\n            <p class=\"epi-pane__description\">${i18n.description}</p>\r\n        </div>\r\n        <div data-dojo-attach-point=\"contentNode\" class=\"epi-pane__content\">\r\n            <div data-dojo-type=\"dijit/layout/StackContainer\">\r\n                <div class=\"epi-pane__content__section\">\r\n                    <div data-dojo-attach-point=\"dashboardFrame\"></div>\r\n                </div>\r\n            </div>\r\n        </div>\r\n    </div>\r\n</div>",
'epi-find/content/ContentController':function(){
define([
    "dojo/_base/declare",

    "../_ControllerBase",
    "./OverviewModel",
    "./ExploreModel",
    "../widget/Contents",
    "../widget/_ActionableMixin"
],
function(declare,
    _ControllerBase,
    OverviewModel, ExploreModel, ContentsWidget, _ActionableMixin
) {
    return declare([_ControllerBase, _ActionableMixin], {
        // summary:
        //      Controller for Overview views.

        overviewModel: null,
        exploreModel: null,
        widget: null,

        startup: function() {
            if (this.isStarted()) {
                return;
            }
            this.inherited(arguments);

            this.overviewModel = new OverviewModel();
            this.overviewModel.init();

            this.exploreModel = new ExploreModel();
            this.exploreModel.init();

            this.widget = new ContentsWidget({
                overviewModel: this.overviewModel,
                exploreModel: this.exploreModel
            });

            this.own(this.widget);
        },

        takeAction: function(actions, params) {
            this.widget.takeAction(actions, params);
            this.currentAction = this.widget.currentAction;
            return this.widget;
        },

        show: function() {
            this._setupView([this.widget]);
        }
    });
});
},
'epi-find/content/OverviewModel':function(){
﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/when",
    "dojo/Stateful",
    "dojo/_base/config",

    "dijit/Destroyable",

    "../ConfigModel",
    "../ApplicationState"
],
function (declare,
          lang,
          array,
          when,
          Stateful,
          config,
          Destroyable,
          ConfigModel,
          ApplicationState) {

    return declare([Stateful, Destroyable], {
        // summary:
        //      Overview model.

        store: null,

        totalDocuments: null,
        totalDocumentTypes: null,
        documentTypes: null,

        contentTypes: null,
        totalContentTypes: null,
        totalContentItems: null,
        totalMediaItems: null,

        totalLanguageBranches: null,
        languageBranches: null,

        documentInfoError: null,
        cmsInfoError: null,

        indexName: null,
        findApiVersion: null,

        facetSize: 1000,

        // config: object
        //      Configuration parameters
        config: null,
        _configSetter: function(value) {
            if (!value) {
                return;
            }
            this.set("indexName", value.index_name);
            this.set("findApiVersion", value.find_api_version);
        },

        site: null,
        _siteSetter: function (site) {
            this.site = site;
            this._updateSiteFilter(site);
        },

        documentsInfoQuery: null,

        contentInfoQuery: null,

        mediaInfoQuery: null,

        _siteFilter: null,

        postscript: function() {
            this.inherited(arguments);
            if (!this.store) {
                this.store = config.dependencies["epi-find.ElasticsearchStore"];
            }

            this._siteFilter = {};

            this.documentsInfoQuery = {
                size: 0,
                query: {
                    "filtered": {
                        "query": {
                            "match_all": {}
                        },
                        "filter": this._siteFilter
                    }
                },
                facets: {
                    type: {
                        terms: {
                            field: "$type",
                            size: this.facetSize
                        }
                    }
                }
            };

            this.contentInfoQuery = {
                query: {
                    filtered: {
                        query: {
                            constant_score: {
                                filter: {
                                    not: {
                                        filter: {
                                            or: [
                                                {
                                                    term: {
                                                        ___types: "EPiServer.Core.IContentMedia"
                                                    }
                                                },
                                                {
                                                    term: {
                                                        ___types: "EPiServer.Core.ContentFolder"
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                }
                            }
                        },
                        filter: {
                            "and": [
                                {
                                    "term": {
                                        "___types": "EPiServer.Core.IContent"
                                    }
                                },
                                this._siteFilter
                            ]
                        }
                    }
                },
                facets: {
                    contentType: {
                        terms: {
                            // TODO: Use _TypeShortName instead, when that has been merged into Owl
                            field: "ContentTypeName$$string",
                            size: this.facetSize
                        }
                    },
                    language: {
                        terms: {
                            field: "Language.Name$$string",
                            size: this.facetSize
                        }
                    }
                }
            };

            this.mediaInfoQuery = {
                "size": 0,
                "query": {
                    "filtered": {
                        "query": {
                            "match_all": {}
                        },
                        "filter": {
                            "and": [
                                {
                                    "term": {
                                        "___types": "EPiServer.Core.IContentMedia"
                                    }
                                },
                                this._siteFilter
                            ]
                        }
                    }
                }
            };
        },

        init: function () {
            //  summary:
            //      Sets the model properties to initial state.
            //      This method should be called when widgets are instantiated and subscribed on model updates,
            //      so that when this method is called dependent widgets will get initial values from the model.
            var self = this;

            this.documentTypes = [];
            this.contentTypes = [];
            this.languageBranches = [];

            this.set("site", ApplicationState.site || null);
            this.own(ApplicationState.watch("site", function (name, oldValue, site) {
                self.set("site", site);
                self.load();
            }));
        },

        load: function() {
            this.loadDocumentsInfo();
            this.loadContentInfo();
            this.loadMediaInfo();
            this.loadConfigInfo();
        },

        loadConfigInfo: function () {
            var self = this;
            // read current configuration
            this.set("config", ConfigModel.config);
            // and watch for configuration changes
            this.own(ConfigModel.watch("config", function(name, oldValue, value) {
                self.set("config", value);
            }));
        },

        loadDocumentsInfo: function () {
            this.set("documentInfoError", null);
            var documentQuery = this.store.query(this.documentsInfoQuery);

            when(documentQuery.facets, lang.hitch(this, function (facets) {
                this.set("totalDocuments", facets.type.total);
                this.set("totalDocumentTypes", facets.type.terms.length);
                this.set("documentTypes", facets.type.terms);
            }), lang.hitch(this, function (error) {
                this.set("documentInfoError", error);
            }));
        },

        loadContentInfo: function () {
            this.set("cmsInfoError", null);
            var cmsContentQuery = this.store.query(this.contentInfoQuery);

            when(cmsContentQuery.total, lang.hitch(this, function (totalContentItems) {
                this.set("totalContentItems", totalContentItems);
            }), lang.hitch(this, function (error) {
                this.set("cmsInfoError", error);
            }));

            when(cmsContentQuery.facets, lang.hitch(this, function (facets) {
                this.set("contentTypes", facets.contentType.terms);
                this.set("totalContentTypes", facets.contentType.terms.length);

                this.set("languageBranches", facets.language.terms);
                this.set("totalLanguageBranches", facets.language.terms.length);
            }), lang.hitch(this, function (error) {
                this.set("cmsInfoError", error);
            }));
        },

        loadMediaInfo: function () {
            this.set("cmsInfoError", null);
            var cmsMediaCountQuery = this.store.query(this.mediaInfoQuery);

            when(cmsMediaCountQuery.total, lang.hitch(this, function (totalMediaItems) {
                this.set("totalMediaItems", totalMediaItems);
            }), lang.hitch(this, function (error) {
                this.set("cmsInfoError", error);
            }));
        },

        _updateSiteFilter: function (site) {
            var siteId = site ? site.id : null,
                siteName = site ? site.name : null;

            if (siteId) {
                this._addSiteFilter(siteId, siteName);
            }
            else {
                this._removeSiteFilter();
            }
        },

        _addSiteFilter: function (siteId, siteName) {
            if (!siteId) {
                return;
            }

            this._siteFilter.or = [
                {
                    "term": {
                        "SiteId$$string": siteId
                    }

                }
            ];

            if (siteName) {
                this._siteFilter.or.push({
                    "term": {
                        "SiteId$$string": siteName
                    }
                });
            }
        },

        _removeSiteFilter: function () {
            delete this._siteFilter.or;
        }
    });
});

},
'epi-find/content/ExploreModel':function(){
﻿define([
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/Deferred",
    "dojo/when",
    "dojo/aspect",
    "dojo/_base/config",
    "dojo/Stateful",
    "dojo/store/util/QueryResults",
    "dojo/store/Memory",

    "../store/StoreFactory",

    "../util/search"

],
function (array,
		  declare,
          lang,
          Deferred,
          when,
          aspect,
          config,
          Stateful,
          QueryResults,
          Memory,
          StoreFactory,
          search) {

    return declare([Stateful], {
        // summary:
        //      Explore view model.

        store: null,

        queryInProgress: false,

        query: null,

        facetSize: 100,

        _queryGetter: function () {
            if (!this.query) {
                this._buildQuery();
            }
            return this.query;
        },

        _querySetter: function (value) {
            this.query = value;
        },

        searchText: null,

        _searchTextGetter: function () {
            return this.searchText;
        },

        _searchTextSetter: function (value) {
            if (value !== this.searchText) {
                this.searchText = value;
                if (value) {
                    // reset sort if there is a search text in order to sort by score
                    this.set("sort", []);
                }
                this._buildQuery();
            }
        },

        typeFilter: null,

        _typeFilterGetter: function () {
            return this.typeFilter;
        },

        _typeFilterSetter: function (value) {
            if (value !== this.typeFilter) {
                this.typeFilter = value;
                this._buildQuery();
            }
        },

        querySort: null,
        sort: null,
        _sortSetter: function(value) {
            var direction;
            if (!value || !value.length) {
                this.querySort = [];
            }
            else if (value[0].attribute === "name") {
                direction = value[0].descending ? "desc": "asc";
                this.querySort = [					
					{
                        "SearchTitle$$string":{
                            "order": direction,
                            "ignore_unmapped" : true
                        },
						"Name$$string":{
							"order": direction,
							"ignore_unmapped" : true
						}
                    }
                ];
            }
            else if (value[0].attribute === "type") {
                direction = value[0].descending ? "desc" : "asc";
                this.querySort = [
                    {
                        "_TypeShortName":{
                            "order": direction,
                            "ignore_unmapped" : true
                        },
                        "$type":{
                            "order": direction,
                            "ignore_unmapped" : true
                        }
                    }
                ];
            }
            this._buildQuery();
            this.sort = value;
        },

        totalDocuments: null,

        constructor: function() {
            var self = this;
            // Have to clone in order to use aspect
            this.store = StoreFactory.createElasticSearchStore();
            this.sort = [];
            this.typeFacetStore = new Memory({ idProperty: "term" });

            // Updating model when query is completed
            aspect.around(this.store, "query", function (originalQuery) {

                return function(query, options) {

                    var deferred = new Deferred();

                    if (!self._isLastQuery(query)) {
                        deferred.cancel();
                        return new QueryResults(deferred.promise);
                    }

                    self.set("queryInProgress", true);

                    var originalResult = originalQuery.apply(self.store, arguments);
                    var result = new QueryResults(originalResult);

                    when (originalResult, function (data) {
                        if (!self._isLastQuery(query)) {
                            deferred.cancel();
                            return;
                        }

                        // Updating total number of documents
                        when(originalResult.total, function (total) {
                            self.set("totalDocuments", total);
                        });
                        // Updating available document types
                        when(originalResult.facets, function (facets) {
                            var terms = facets.type.terms;
                            if (self.typeFilter && self.typeFilter.length) { // if the filter by type is set ...
                                array.forEach(self.typeFilter, function (filter) {
                                    // ... and a type in the filter is not in the list of types returned by facets
                                    if (!array.some(terms, function (item) {
                                            return item.term === filter;
                                        })) {
                                        terms.push({term: filter, count: 0}); // ... add a type from the filter to the types list
                                    }
                                });
                            }

                            self.typeFacetStore.setData(terms);
                            self.set("queryInProgress", false);
                        });

                        deferred.resolve(data);

                    }, function(err){
                        console.error(err);
                        // set queryInProgress to false to make search event continue working when rejected.
                        self.set("queryInProgress", false);
                    });

                    return result;

                };
            });
        },

        init: function () {
            //  summary:
            //      Sets the model properties to initial state.
            //      This method should be called when widgets are instantiated and subscribed on model updates,
            //      so that when this method is called dependent widgets will get initial values from the model.
            this.searchText = null;
            this.typeFilter = null;
            this._buildQuery();
        },

        getShortTypeName: function(typeName) {
            if (typeName && typeName.split) {
                var fullTypeName = typeName.split(",")[0], // take the full type name without assembly name
                    typeParts = fullTypeName.split("."),
                    shortType = typeParts[typeParts.length-1]; // take class name from full type name
                return shortType;
            }
            else {
                return typeName;
            }
        },

        _isLastQuery: function(query) {
            var q1 = JSON.stringify(this.get("query"));
            var q2 = JSON.stringify(query);
            return q1 === q2;
        },

        _buildQuery: function() {
            //  summary:
            //      Builds ElasticSearch query based on model parameters.
            var query;
            if (this.searchText && this.searchText !== "") {
                query = { query: { query_string: { query: search.escapeQueryStringQuery(this.searchText, true) } }, partial_fields: { partial_source: { include: "*", exclude: "*$$attachment" } } };
            } else {
                query = { query: { query_string: { query: "*:*" } }, partial_fields: { partial_source: { include: "*", exclude: "*$$attachment" } } };
            }
            query.facets = { type: { terms: { "field": "$type", size: this.facetSize } } };

            if (this.typeFilter && this.typeFilter.length) {
                query.filter = { terms: { "$type": this.typeFilter } };
            }

            if (this.querySort && this.querySort.length) {
                query.sort = this.querySort;
            }

            this.query = query;
        },

        resetPaging: function () {
            if (this.query) {
                if (this.query.from) {
                    this.query.from = null;
                }
                if (this.query.size) {
                    this.query.size = null;
                }
            }
        }
    });
});

},
'epi-find/widget/Contents':function(){
define([
    "dojo/_base/config",
    "dojo/_base/declare",
    "dojo/on",
    "dojo/topic",
    "dojo/dom-class",
    "dojo/query",

    "dijit/_WidgetBase",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/_TemplatedMixin",

    "./_ActionableMixin",
    "./Overview",
    "./Explore",

    "../AccessModel",

    "dojo/i18n!./nls/Contents",
    "dojo/text!./templates/Contents.html"
],
function(config,
    declare,
    on,
    topic,
    domClass,
    query,
    _WidgetBase,
    _WidgetsInTemplateMixin,
    _TemplatedMixin,
    _ActionableMixin,
    Overview,
    Explore,
    AccessModel,
    i18n,
    template) {

    return declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, _ActionableMixin], {
        // summary:
        //     Overview view.

        templateString: template,

        i18n: i18n,

        overviewModel: null,
        exploreModel: null,

        constructor: function() {
            this.inherited(arguments);
        },

        postCreate: function() {
            this.inherited(arguments);

            this.overviewWidget = new Overview({
                model: this.overviewModel,
                disabledView: !AccessModel.isEnabled(config.intents.view.overview),
                scrollNode: this.contentNode
            });
            this.widgetContainer.addChild(this.overviewWidget);
            this.own(this.overviewWidget);

            this.exploreWidget = new Explore({
                model: this.exploreModel,
                disabledView: !AccessModel.isEnabled(config.intents.view.explore),
                scrollNode: this.contentNode
            });
            this.widgetContainer.addChild(this.exploreWidget);
            this.own(this.exploreWidget);

            var self = this,
                overviewAction = {
                    currentAction: "overview",
                    takeAction: function(actions, params) {
                        self.widgetContainer.selectChild(self.overviewWidget);
                        self.overviewWidget.takeAction(actions, params);
                        self._switchSelection(self.overviewLink.parentNode);
                        return self.overviewWidget;
                    }
                },
                exploreAction = {
                    currentAction: "explore",
                    takeAction: function(actions, params) {
                        self.widgetContainer.selectChild(self.exploreWidget);
                        self.exploreWidget.takeAction(actions, params);
                        self._switchSelection(self.exploreLink.parentNode);
                        return self.exploreWidget;
                    }
                };

            this.currentAction = overviewAction.currentAction;
            this.registerAction(overviewAction.currentAction, overviewAction);
            this.registerAction(exploreAction.currentAction, exploreAction);

            this.own(
                on(this.overviewLink, "click", function(event) {
                    topic.publish(config.intents.view.overview);
                    event.preventDefault();
                }),
                on(this.exploreLink, "click", function(event) {
                    topic.publish(config.intents.view.explore);
                    event.preventDefault();

                })
            );

            this.previousActionFallback = true;
        },

        startup: function() {
            this.inherited(arguments);
            this.takeAction([this.currentAction]);
        },

        _switchSelection: function(/*domNode*/ node) {
            // summary:
            //      Removes current selection indication and puts selection indicator to a new node
            query(".epi-pane__content__nav li.is-active", this.domNode).removeClass("is-active");
            domClass.add(node, "is-active");
        }
    });
});

},
'epi-find/widget/Overview':function(){
﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/dom-prop",
    "dojo/dom-class",
    "dojo/store/Memory",

    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",

    "dgrid/Grid",

    "put-selector/put",

    "epi-saas-base/Sticker",
    "epi-saas-base/StickyContainer",
    "epi-saas-base/widgets/_GridMoreMixin",
    "epi-saas-base/widgets/_AutoLoadScrollMixin",

    "./_ActionableMixin",
    "./_DisabledViewMixin",

    "dojo/i18n!./nls/Overview",
    "dojo/text!./templates/Overview.html"
],
function(declare,
    lang,
    array,
    domProp,
    domClass,
    Memory,
    _WidgetBase,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    Grid,
    put,
    Sticker,
    StickyContainer,
    _GridMoreMixin,
    _AutoLoadScrollMixin,
    _ActionableMixin,
    _DisabledViewMixin,
    i18n,
    template) {

    // module:
    //      epi-find/widget/Overview

    var DescriptionText = declare([_WidgetBase], {
        // template: String
        //      A string which contains template markers; for example: "{0} message".
        //      {0} would then be replaced by the first element in descriptionVariables.
        template: null,

        // descriptionVariables: Mixed[]
        //  The variables to use when building the description string from the template.
        descriptionVariables: null,

        // formatter: function(key, value)
        //      A function which runs for every description value when the text is rebuilt. Should be overidden.
        formatter: function(key, value){},

        constructor: function() {
            this.descriptionVariables = [];
        },

        _setDescriptionVariablesAttr: function(variables) {
            this._set("descriptionVariables", variables);
            var description = this.template;

            array.forEach(variables, function(item, index) {
                if (this.formatter) {
                    item = this.formatter(index, item);
                }
                description = description.replace("{" + index + "}", item);
            }, this);
            this.set("description", description);
        },

        _setDescriptionAttr: { node: "domNode", type: "innerHTML" }
    });

    var TermFacetGrid = declare([Grid, _GridMoreMixin, _AutoLoadScrollMixin], {
        constructor: function() {
            // Sets the loader (#loadNextPage, available in _GridMoreMixn) to be used by the _AutoLoadScrollMixin
            this.loader = lang.hitch(this, this.loadNextPage);
        },
        postMixInProperties: function() {
            if (! this.columns.term) {
                this.columns.term = {};
            }
            this.columns.term.sortable = false;

            if (! this.columns.count) {
                this.columns.count = {
                    className: "epi-grid-column--right epi-grid--15",
                    label: i18n.count,
                    sortable: false,
                    renderCell: function (object, value, node, options) {
                        domClass.add(node, "epi-grid-column--bold");
                        domProp.set(node, "innerHTML", value);
                    }
                };
            }
            this.inherited(arguments);
        }
    });

    return declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, _ActionableMixin, _DisabledViewMixin], {
        // summary:
        //     Overview view.

        templateString: template,

        i18n: i18n,

        model: null,

        _indexNameDescription: null,

        _findApiVersionDescription: null,

        _docDescription: null,

        _docTypeGrid: null,

        _cmsDescription: null,

        _cmsTypeGrid: null,

        _cmsLanguageGrid: null,

        buildRendering: function() {
            this.inherited(arguments);

            this._indexNameDescription = new DescriptionText({
                template: i18n.indexName,
                formatter: function(key, value) {
                    return value !== null ? value : "?";
                },
                descriptionVariables: [null]
            }, this.indexNameText),

            this._findApiVersionDescription = new DescriptionText({
                template: i18n.findApiVersion,
                formatter: function (key, value) {
                    return value !== null ? value : "?";
                },
                descriptionVariables: [null]
            }, this.findApiVersionText),

            this._docDescription = new DescriptionText({
                template: i18n.documentTypes_description,
                formatter: function(key, value) {
                    return value !== null ? value : "?";
                },
                descriptionVariables: [null, null]
            }, this.documentTypesDescriptionText);

            this._docTypeGrid = new TermFacetGrid({
                "class": "epi-plain-grid epi-plain-grid--no-border",
                margin: 140,
                columns: {
                    term: {
                        label: i18n.documentTypes_column
                    }
                },
                store: new Memory({
                    idProperty: "term"
                }),
                noDataMessage: i18n.noDocumentTypes,
                loadingMessage: i18n.dataLoading,
                showLoadingMessage: true
            }, this.documentTypesGrid);

            this._cmsDescription = new DescriptionText({
                template: i18n.cmsContent_description,
                formatter: function(key, value) {
                    return value !== null ? value : "?";
                },
                descriptionVariables: [null, null, null, null]
            }, this.cmsContentDescriptionText);

            this._cmsTypeGrid = new TermFacetGrid({
                "class": "epi-plain-grid epi-plain-grid--no-border epi-vertical-spacer",
                margin: 140,
                columns: {
                    term: {
                        label: i18n.cmsContent_column
                    }
                },
                store: new Memory({
                    idProperty: "term"
                }),
                noDataMessage: i18n.noPageTypes,
                loadingMessage: i18n.dataLoading
            }, this.cmsContentGrid);

            this._cmsLanguageGrid = new TermFacetGrid({
                "class": "epi-plain-grid epi-plain-grid--no-border",
                margin: 140,
                columns: {
                    term: {
                        label: i18n.language_column
                    }
                },
                store: new Memory({
                    idProperty: "term"
                }),
                noDataMessage: i18n.noLanguageBranches,
                loadingMessage: i18n.dataLoading
            }, this.languageContentGrid);
        },

        postCreate: function() {
            this.inherited(arguments);

            if (!this.model) {
                throw new Error("'model' is not defined");
            }

            this.handleModelErrors();
            this.bindDocumentTypes();
            this.bindCmsContent();
            this.bindConfigInfo();

            this.model.load();
        },

        startup: function() {
            if (this._started) {
                return;
            }
            this.inherited(arguments);
            var scrollNode = this.scrollNode || this.domNode.parentNode;
            this.own(
                new StickyContainer({
                    scrollNode: scrollNode,
                    stickers: [
                        new Sticker({ domNode: this.documentTypesHeader, offset: {top: 0}, limitNode: this._docTypeGrid.domNode, subscribe: false}),
                        new Sticker({ domNode: this._docTypeGrid.headerNode, offset: {top: 0}, limitNode: this._docTypeGrid.domNode, subscribe: false})
                    ]
                }),
                new StickyContainer({
                    scrollNode: scrollNode,
                    stickers: [
                        new Sticker({ domNode: this.cmsContentHeader, offset: {top: 0}, limitNode: this._cmsTypeGrid.domNode, subscribe: false}),
                        new Sticker({ domNode: this._cmsTypeGrid.headerNode, offset: {top: 0}, limitNode: this._cmsTypeGrid.domNode, subscribe: false})
                    ]
                })
            );

            this._docTypeGrid.set("scrollNode", scrollNode);
            this._cmsTypeGrid.set("scrollNode", scrollNode);
            this._cmsLanguageGrid.set("scrollNode", scrollNode);
        },

        bindConfigInfo: function () {
            this.own(
                this.model.watch("indexName", lang.hitch(this, function (name, oldValue, newValue) {
                    this._indexNameDescription.set("descriptionVariables", [this.model.indexName]);
                })),
                this.model.watch("findApiVersion", lang.hitch(this, function (name, oldValue, newValue) {
                    this._findApiVersionDescription.set("descriptionVariables", [this.model.findApiVersion]);
                }))
            );
        },

        bindDocumentTypes: function() {
            var updateTypesDescription = lang.hitch(this, function() {
                this._docDescription.set("descriptionVariables", [this.model.totalDocuments, this.model.totalDocumentTypes]);
            });

            this.own(
                this.model.watch("documentTypes", lang.hitch(this, function (name, oldValue, newValue) {
                    this._reloadGrid(this._docTypeGrid, newValue);
                })),
                this.model.watch("totalDocuments", lang.hitch(this, function (name, oldValue, newValue) {
                    updateTypesDescription();
                })),
                this.model.watch("totalDocumentTypes", lang.hitch(this, function (name, oldValue, newValue) {
                    updateTypesDescription();
                }))
            );
        },

        bindCmsContent: function() {
            var updateCmsDescription = lang.hitch(this, function() {
                this._cmsDescription.set("descriptionVariables",
                    [
                        this.model.totalContentItems, this.model.totalMediaItems,
                        this.model.totalContentTypes, this.model.totalLanguageBranches
                    ]);
            });

            this.own(
                this.model.watch("contentTypes", lang.hitch(this, function (name, oldValue, newValue) {
                    this._reloadGrid(this._cmsTypeGrid, newValue);
                })),
                this.model.watch("languageBranches", lang.hitch(this, function (name, oldValue, newValue) {
                    this._reloadGrid(this._cmsLanguageGrid, newValue);
                })),
                this.model.watch("totalMediaItems", updateCmsDescription),
                this.model.watch("totalContentItems", updateCmsDescription),
                this.model.watch("totalContentTypes", updateCmsDescription),
                this.model.watch("totalLanguageBranches", updateCmsDescription)
            );
        },

        handleModelErrors: function () {
            var gridErrorHandler = function(grid) {
                // removing existing noData node and creating new one with error message
                if(grid.noDataNode){
                    put(grid.noDataNode, "!");
                    delete grid.noDataNode;
                }
                grid.noDataNode = put(grid.contentNode, "div.dgrid-no-data");
                grid.noDataNode.innerHTML = i18n.dataLoadError;
            };

            this.own(
                this.model.watch("documentInfoError", lang.hitch(this, function (name, oldValue, newValue) {
                    if (newValue) {
                        gridErrorHandler(this._docTypeGrid);
                        this._docDescription.set("descriptionVariables", [null, null]);
                    }
                })),
                this.model.watch("cmsInfoError", lang.hitch(this, function (name, oldValue, newValue) {
                    if (newValue) {
                        gridErrorHandler(this._cmsTypeGrid);
                        gridErrorHandler(this._cmsLanguageGrid);
                        this._cmsDescription.set("descriptionVariables", [null, null, null, null]);
                    }
                }))
            );
        },

        takeAction: function() {
            this.refresh();
            return this;
        },

        refresh: function () {
            this.model.load();
        },

        _reloadGrid: function(grid, data) {
            if (grid && data) {
                if (!(data instanceof Array)) {
                    data = [data];
                }
                grid.store.setData(data);
                grid.refresh();
            }
        }
    });
});

},
'url:epi-find/widget/templates/Overview.html':"﻿<div>\r\n    <p class=\"epi-view--description\">${i18n.description}</p>\r\n    <div class=\"epi-pane__content__content-container\">\r\n        <div class=\"epi-pane__content__section\">\r\n            <h1 class=\"epi-color-signal  epi-typo-serif\">Index</h1>\r\n            <p data-dojo-attach-point=\"indexNameText\"></p>\r\n            <p data-dojo-attach-point=\"findApiVersionText\"></p>\r\n        </div>\r\n\r\n        <div class=\"epi-pane__content__section\" data-dojo-attach-point=\"documentTypesNode\">\r\n            <div data-dojo-attach-point=\"documentTypesHeader\">\r\n                <h1 class=\"epi-color-signal  epi-typo-serif\">${i18n.documentTypes_header}</h1>\r\n\r\n                <p data-dojo-attach-point=\"documentTypesDescriptionText\"></p>\r\n                <p class=\"epi-help\">${i18n.help_documentTypes}</p>\r\n            </div>\r\n            <div data-dojo-attach-point=\"documentTypesGrid\"></div>\r\n        </div>\r\n\r\n        <div class=\"epi-pane__content__section\" data-dojo-attach-point=\"cmsContantNode\">\r\n            <div data-dojo-attach-point=\"cmsContentHeader\">\r\n                <h1 class=\"epi-color-signal  epi-typo-serif\" data-dojo-attach-point=\"cmsContentTitle\">${i18n.cmsContent_header}</h1>\r\n\r\n                <p data-dojo-attach-point=\"cmsContentDescriptionText\"></p>\r\n\r\n                <p class=\"epi-help\">${i18n.help_cmsContent}</p>\r\n            </div>\r\n            <div data-dojo-attach-point=\"cmsContentGrid\"></div>\r\n\r\n            <h2 class=\"epi-color-signal  epi-typo-serif\">${i18n.language_header}</h2>\r\n\r\n            <p class=\"epi-help\">${i18n.help_cmsLanguages}</p>\r\n            <div data-dojo-attach-point=\"languageContentGrid\"></div>\r\n        </div>\r\n    </div>\r\n</div>",
'epi-find/widget/Explore':function(){
define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/json",
    "dojo/on",
    "dojo/query",
    "dojo/dom-class",
    "dojo/when",

    "dojox/html/entities",
    "dojox/mvc/at",
    "dojox/mvc/Output",

    "dijit/layout/ContentPane",
    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/layout/_LayoutWidget",
    "dijit/form/TextBox",

    "dgrid/Grid",
    "dgrid/Selection",
    "dgrid/OnDemandGrid",

    "put-selector/put",

    "epi-saas-base/Sticker",
    "epi-saas-base/StickyContainer",
    "epi-saas-base/widgets/_GridMoreMixin",
    "epi-saas-base/widgets/_AutoLoadScrollMixin",

    "./_ActionableMixin",
    "./_DisabledViewMixin",
    "./ContentWrapperWithSidebar",
    "./HelpText",

    "dojo/i18n!./nls/Explore",
    "dojo/text!./templates/Explore.html"
],
function (declare,
    lang,
    JSON,
    on,
    query,
    domClass,
    when,
    entities,
    at,
    Output,
    ContentPane,
    _WidgetBase,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    _LayoutWidget,
    TextBox,
    Grid,
    Selection,
    OnDemandGrid,
    put,
    Sticker,
    StickyContainer,
    _GridMoreMixin,
    _AutoLoadScrollMixin,
    _ActionableMixin,
    _DisabledViewMixin,
    ContentWrapperWithSidebar,
    HelpText,
    i18n,
    template) {

    // module:
    //      epi-find/widget/Explore

    return declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, _ActionableMixin, _DisabledViewMixin], {
        // summary:
        //     Explore view.

        templateString: template,

        i18n: i18n,

        model: null,

        resultsGrid: null,
        typeFacetGrid: null,

        expandedNode: null,

        _searchText: null,
        _typeFilter: null,

        _modelUpdateTimeout: 400,
        _modelUpdateTimer: null,

        postCreate: function () {
            this.inherited(arguments);

            if (!this.model) {
                throw new Error("'model' is not defined");
            }

            var model = this.model,
                AutoLoadScrollGrid = declare([Grid, _GridMoreMixin, _AutoLoadScrollMixin], {
                    constructor: function() {
                        this.loader = lang.hitch(this, this.loadNextPage);
                    }
                });

            this._typeFilter = this.model.typeFilter;
            this._searchText = this.model.searchText;

            this.contentWrapper.addSidebar(this.filterPane);

            // Main document explore list
            this.resultsGrid = new AutoLoadScrollGrid({
                className: "epi-plain-grid epi-stack-list-wrapper",
                showFooter: false,
                useTouchScroll: false,
                store: this.model.store,
                query: this.model.query,
                pagingDelay: 200,
                margin: 140,
                columns: [
                    {
                        className: "epi-grid-column--icon24x24",
                        renderCell: function () {
                            return put("div.epi-gridIconCollapsed");
                        }
                    },
                    {
                        label: i18n.fields_name,
                        sortable: true,
                        className: "epi-grid-column--sortable--tight-arrow",
                        field: "name",
                        renderHeaderCell: function() {
                            return put("span.epi-grid-column-header__string-wrapper", i18n.fields_name);
                        },
                        get: function (object) {
                            return object.fields.partial_source.SearchTitle$$string ||
                                object.fields.partial_source.PageName$$string ||
                                object.fields.partial_source.Name$$string ||
                                object.fields.partial_source.title || // for RSS/ATOM feeds
                                object._id;
                        },
                        renderCell: function (object, value, node) {
                            node.title = object._id;
                            node.innerHTML = entities.encode(value || "");
                        }
                    },
                    {
                        label: i18n.fields_type,
                        sortable: true,
                        field: "type",
                        className: "epi-grid-column--right epi-grid--30 epi-color-secondary epi-columnPadding--right",
                        get: function (object) {
                            return model.getShortTypeName(object.fields.partial_source.$type);
                        },
                        renderCell: function (object, value, node) {
                            node.title = object.fields.partial_source.$type;
                            node.innerHTML = value;
                        }
                    }
                ],
                noDataMessage: i18n.noResults,
                loadingMessage: i18n.dataLoading,
                showLoadingMessage: true,
                renderRow: function (object) {
                    var div = put("div.collapsed", Grid.prototype.renderRow.apply(this, arguments)),
                        pre = put("pre", JSON.stringify(object.fields.partial_source, null, 4));
                        put(div, "div.expando", pre);
                    return div;
                }
            }, this.resultsGridNode);

            var SelectGrid = declare([OnDemandGrid, Selection], {});
            this.typeFacetGrid = new SelectGrid({
                columns: [
                    {
                        get: function (object) {
                            return object.term;
                        },
                        renderCell: function (object, value, node) {
                            node.title = value;
                            node.innerHTML = model.getShortTypeName(value);
                        }
                    },
                    {
                        className: "epi-grid-column--right  epi-grid-column--bold  epi-grid--30",
                        get: function (object) {
                            return object.count;
                        },
                        renderCell: function (object, value, node) {
                            node.title = value;
                            node.innerHTML = value;
                        }
                    }
                ],
                useTouchScroll: false,
                showHeader: false,
                noDataMessage: i18n.noResults,  // Data is provided via #renderArray, so message will never be shown.
                selectionMode: "toggle",
                deselectOnRefresh: false,
                store: this.model.typeFacetStore
            }, this.typeFacetGridNode);

            // Displaying total number of found documents
            var totalDocumentsWidget = new Output({
                value: at(this.model, "totalDocuments")
                    .direction(at.from)
                    .transform({
                        format: function(total){
                            return "<p class=\"epi-floatRight\">" + i18n.results.replace("{0}", total ? total : 0) + "</p>";
                        }
                    })
            }, this.totalNode);
            this.own(totalDocumentsWidget);

            this.own(
                // Update query parameters when type filter is selected
                on(this.typeFacetGrid, "dgrid-select", lang.hitch(this, function () {
                    this._updateTypeFilter();
                    this._delayedReload();
                })),
                on(this.typeFacetGrid, "dgrid-deselect", lang.hitch(this, function () {
                    this._updateTypeFilter();
                    this._delayedReload();
                })),

                // Update query with new search text value
                on(this.queryNode, "change", lang.hitch(this, function () {
                    var searchText = lang.trim(this.queryNode.get("value").toLowerCase() || "");
                    this.resultsGrid.updateSortArrow([], true);
                    this._searchText = searchText;
                    this._delayedReload();
                })),

                // Expand/collapse document data rows (on cell only, not for expanded)
                on.pausable(this.resultsGrid, ".dgrid-row .dgrid-cell:click", lang.hitch(this, function (event) {
                    var node = this.resultsGrid.row(event).element,
                        collapsed = node.className.indexOf("collapsed") >= 0;

                    // toggle state of node which was clicked
                    put(node, (collapsed ? "!" : ".") + "collapsed");
                    this._toggleResultRowIcon(node);

                    // if clicked row was not expanded, collapse any previously-expanded row
                    if(collapsed && this.expandedNode) {
                        put(this.expandedNode, ".collapsed");
                        this._toggleResultRowIcon(this.expandedNode);
                    }
                    // if the row clicked was previously expanded, nothing is expanded now
                    this.expandedNode = collapsed ? node : null;
                })),
                on(this.resultsGrid, "dgrid-error", lang.hitch(this, function () {
                    // removing existing noData node and creating new one with error message
                    if(this.resultsGrid.noDataNode){
                        put(this.resultsGrid.noDataNode, "!");
                        delete this.resultsGrid.noDataNode;
                    }
                    this.resultsGrid.noDataNode = put(this.resultsGrid.contentNode, "div.dgrid-no-data");
                    this.resultsGrid.noDataNode.innerHTML = i18n.dataLoadError;
                })),
                on(this.resultsGrid, "dgrid-sort", lang.hitch(this, function(event) {
                    event.preventDefault();
                    this._sort = event.sort;
                    this._delayedReload();
                }))
            );

            this.own(
                this.model.watch("queryInProgress", lang.hitch(this, function (name, oldValue, queryInProgress) {
                    if (!queryInProgress) {
                        this.typeFacetGrid.refresh();
                    }
                }))
            );

            this.queryNode.set("value", this.model.searchText);
        },

        takeAction: function() {
            this.refresh();
            return this;
        },

        refresh: function() {
            this.model.set("searchText", this._searchText || null);
            this.model.set("typeFilter", this._typeFilter || null);
            this.model.set("sort", this._sort || []);
            this.model.resetPaging();
            this.resultsGrid.set("query", this.model.query);
            this.resultsGrid.updateSortArrow(this.model.sort, true);
        },

        _updateTypeFilter: function() {
            this._typeFilter = [];
            for(var id in this.typeFacetGrid.selection){
                if(this.typeFacetGrid.selection[id]){
                    this._typeFilter.push(id);
                }
            }
        },

        _delayedReload: function() {
            //  summary:
            //      Updates model parameters and reloads data after delay in order to avoid extra requests.
            if (this._modelUpdateTimer) {
                clearTimeout(this._modelUpdateTimer);
            }
            this._modelUpdateTimer = setTimeout(lang.hitch(this, function () {
                if (this.model.get("queryInProgress")) {
                    this._delayedReload();
                    return;
                }

                clearTimeout(this._modelUpdateTimer);
                this._modelUpdateTimer = null;
                this.refresh();
            }), this._modelUpdateTimeout);
        },

        _toggleResultRowIcon: function (rowNode) {
            query(".epi-grid-column--icon24x24 div", rowNode).forEach(function(iconNode){
                domClass.toggle(iconNode, "epi-gridIconCollapsed");
                domClass.toggle(iconNode, "epi-gridIconExpanded");
            });
        },

        resize: function() {
            this.inherited(arguments);
            this.resultsGrid.resize();
        },

        startup: function() {
            this.inherited(arguments);
            var grid = this.resultsGrid;
            this.resultsGrid.startup();
            grid.set("scrollNode", this.domNode.parentNode);
            grid.on("dgrid-refresh-complete", function() {
                grid.load();
            });

            this.own(
                new StickyContainer({
                    scrollNode: this.domNode.parentNode,
                    stickers: [
                        new Sticker({ domNode: this.containerNode, offset: {top: 0}, limitNode: this.resultsGridNode, subscribe: false}),
                        new Sticker({ domNode: this.resultsGrid.headerNode, offset: {top: 0}, limitNode: this.resultsGridNode, subscribe: false})
                    ]
                }),
                new Sticker({ domNode: this.filterSticker, offset: {top: 0}, scrollNode: this.domNode.parentNode})
            );
        }

    });
});

},
'dojox/mvc/Output':function(){
define("dojox/mvc/Output", [
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/dom",
	"dijit/_WidgetBase",
	"dojo/regexp"
], function(declare, lang, dom, _WidgetBase, regexp){

	return declare("dojox.mvc.Output", _WidgetBase, {
		// summary:
		//		A simple widget that displays templated output, parts of which may
		//		be data-bound.
		//
		// description:
		//		Simple output example:
		//
		//		|  <span data-dojo-type="dojox/mvc/Output" data-dojo-props="value: at(model, 'balance')"></span>
		//
		//		Another simple output example:
		//
		//		|  <span data-dojo-type="dojox/mvc/Output" data-dojo-props="value: at(model, 'balance')">
		//		|    Your balance is: ${this.value}
		//		|  </span>
		//
		//
		//		The output widget being data-bound, if the balance changes in the
		//		model, the content within the `<span>` will be
		//		updated accordingly.

		// exprchar:  Character
		//		Character to use for a substitution expression, for a substitution string like ${this.value}
		exprchar: '$',
	
		// templateString: [private] String
		//		The template or data-bound output content.
		templateString : "",
	
		postscript: function(params, srcNodeRef){
			// summary:
			//		Override and save template from body.
			this.srcNodeRef = dom.byId(srcNodeRef);
			if(this.srcNodeRef){
				this.templateString = this.srcNodeRef.innerHTML;
				this.srcNodeRef.innerHTML = "";
			}
			this.inherited(arguments);
		},
	
		set: function(name, value){
			// summary:
			//		Override and refresh output on value change.
			// name:
			//		The property to set.
			// value:
			//		The value to set in the property.
			this.inherited(arguments);
			if(name === "value"){
				this._output();
			}
		},
	
		////////////////////// PRIVATE METHODS ////////////////////////
	
		_updateBinding: function(name, old, current){
			// summary:
			//		Rebuild output UI if data binding changes.
			// tags:
			//		private
			this.inherited(arguments);
			this._output();
		},
	
		_output: function(){
			// summary:
			//		Produce the data-bound output.
			// tags:
			//		private
			var outputNode = this.srcNodeRef || this.domNode;
			outputNode.innerHTML = this.templateString ? this._exprRepl(this.templateString) : this.value;
		},
	
		_exprRepl: function(tmpl){
			// summary:
			//		Does substitution of ${foo+bar} type expressions in template string.
			// tags:
			//		private
			var pThis = this, transform = function(value, key){
				if(!value){return "";}
				var exp = value.substr(2);
				exp = exp.substr(0, exp.length - 1);
				with(pThis){
					var val = eval(exp);
					return (val || val == 0 ? val : "");
				}
			};
			transform = lang.hitch(this, transform);
			return tmpl.replace(new RegExp(regexp.escapeString(this.exprchar)+"(\{.*?\})","g"),
				function(match, key, format){
					return transform(match, key).toString();
				});
		}
	});
});

},
'url:epi-find/widget/templates/Explore.html':"﻿<div>\r\n    <p class=\"epi-view--description\">${i18n.description}</p>\r\n    <div data-dojo-type=\"epi-find/widget/ContentWrapperWithSidebar\" data-dojo-attach-point=\"contentWrapper\" class=\"epi-explore__content_container\">\r\n        <div class=\"epi-clear-both epi-no-overflow-x\" data-dojo-type=\"dijit/layout/ContentPane\" data-dojo-attach-point=\"mainPane\">\r\n            <p class=\"epi-help\">${i18n.help_search}</p>\r\n            <div data-dojo-attach-point=\"containerNode\">\r\n                <div data-dojo-attach-point=\"mainContentNode\">\r\n                    <div class=\"epi-box epi-background--dark-grey epi-search-box\">\r\n                        <input id=\"queryNode\" type=\"text\" name=\"query\"\r\n                            class=\"epi-search-input\"\r\n                            data-dojo-type=\"dijit/form/TextBox\"\r\n                            data-dojo-props=\"trim: true, intermediateChanges: true\"\r\n                            data-dojo-attach-point=\"queryNode\" placeholder=\"${i18n.search}\" />\r\n                    </div>\r\n                </div>\r\n                <div class=\"epi-clear-both\" data-dojo-attach-point=\"totalNode\"></div>\r\n                <p class=\"epi-clear-both epi-help\">${i18n.help_documents}</p>\r\n            </div>\r\n            <div class=\"epi-plain-grid  epi-plain-grid--no-border  epi-grid-max-height--700\" data-dojo-attach-point=\"resultsGridNode\"></div>\r\n        </div>\r\n        <div data-dojo-type=\"dijit/layout/ContentPane\" data-dojo-attach-point=\"filterPane\">\r\n            <p class=\"epi-help\">${i18n.help_documentTypes}</p>\r\n            <div data-dojo-attach-point=\"filterSticker\">\r\n                <h2 class=\"epi-explore__filter-header\">${i18n.filter}</h2>\r\n                <div class=\"epi-chromeless  epi-chromeless--with-pointer  epi-chromeless--no-header  epi-explore__filter-grid\" data-dojo-attach-point=\"typeFacetGridNode\"></div>\r\n            </div>\r\n        </div>\r\n    </div>\r\n</div>\r\n",
'url:epi-find/widget/templates/Contents.html':"<div class=\"epi-pane epi-pane--tools is-active epi-focused\">\r\n    <div class=\"epi-pane__content-wrapper\">\r\n        <div class=\"epi-pane__header\">\r\n            <h1>${i18n.header}</h1>\r\n            <p class=\"epi-pane__description\">${i18n.description}</p>\r\n        </div>\r\n        <div data-dojo-attach-point=\"contentNode\" class=\"epi-pane__content\">\r\n            <p class=\"epi-pane__description\">${i18n.description}</p>\r\n                <ul class=\"epi-pane__content__nav clearfix\">\r\n                    <li><a href=\"#\" title=\"${i18n.nav_overview}\" class=\"epi-pane__content__header-link--overview\" data-dojo-attach-point=\"overviewLink\">${i18n.nav_overview}</a></li>\r\n                    <li><a href=\"#\" title=\"${i18n.nav_explore}\" class=\"epi-pane__content__header-link--explore\" data-dojo-attach-point=\"exploreLink\">${i18n.nav_explore}</a></li>\r\n                </ul>\r\n                <div data-dojo-type=\"dijit/layout/StackContainer\" data-dojo-attach-point=\"widgetContainer\" data-dojo-props=\"doLayout: false, isLayoutContainer: true\" class=\"epi-pane__content__content-container\"></div>\r\n        </div>\r\n    </div>\r\n</div>",
'dojox/gfx/path':function(){
define("dojox/gfx/path", ["./_base", "dojo/_base/lang","dojo/_base/declare", "./matrix", "./shape"], 
	function(g, lang, declare, matrix, shapeLib){
// summary:
//		This module contains the core graphics Path API.
//		Path command format follows the W3C SVG 1.0 Path api.

	var Path = declare("dojox.gfx.path.Path", shapeLib.Shape, {
		// summary:
		//		a generalized path shape

		constructor: function(rawNode){
			// summary:
			//		a path constructor
			// rawNode: Node
			//		a DOM node to be used by this path object
			this.shape = lang.clone(g.defaultPath);
			this.segments = [];
			this.tbbox = null;
			this.absolute = true;
			this.last = {};
			this.rawNode = rawNode;
			this.segmented = false;
		},

		// mode manipulations
		setAbsoluteMode: function(mode){
			// summary:
			//		sets an absolute or relative mode for path points
			// mode: Boolean
			//		true/false or "absolute"/"relative" to specify the mode
			this._confirmSegmented();
			this.absolute = typeof mode == "string" ? (mode == "absolute") : mode;
			return this; // self
		},
		getAbsoluteMode: function(){
			// summary:
			//		returns a current value of the absolute mode
			this._confirmSegmented();
			return this.absolute; // Boolean
		},

		getBoundingBox: function(){
			// summary:
			//		returns the bounding box {x, y, width, height} or null
			this._confirmSegmented();
			return (this.bbox && ("l" in this.bbox)) ? {x: this.bbox.l, y: this.bbox.t, width: this.bbox.r - this.bbox.l, height: this.bbox.b - this.bbox.t} : null; // dojox/gfx.Rectangle
		},

		_getRealBBox: function(){
			// summary:
			//		returns an array of four points or null
			//		four points represent four corners of the untransformed bounding box
			this._confirmSegmented();
			if(this.tbbox){
				return this.tbbox;	// Array
			}
			var bbox = this.bbox, matrix = this._getRealMatrix();
			this.bbox = null;
			for(var i = 0, len = this.segments.length; i < len; ++i){
				this._updateWithSegment(this.segments[i], matrix);
			}
			var t = this.bbox;
			this.bbox = bbox;
			this.tbbox = t ? [
				{x: t.l, y: t.t},
				{x: t.r, y: t.t},
				{x: t.r, y: t.b},
				{x: t.l, y: t.b}
			] : null;
			return this.tbbox;	// Array
		},

		getLastPosition: function(){
			// summary:
			//		returns the last point in the path, or null
			this._confirmSegmented();
			return "x" in this.last ? this.last : null; // Object
		},

		_applyTransform: function(){
			this.tbbox = null;
			return this.inherited(arguments);
		},

		// segment interpretation
		_updateBBox: function(x, y, m){
			// summary:
			//		updates the bounding box of path with new point
			// x: Number
			//		an x coordinate
			// y: Number
			//		a y coordinate

			if(m){
				var t = matrix.multiplyPoint(m, x, y);
				x = t.x;
				y = t.y;
			}

			// we use {l, b, r, t} representation of a bbox
			if(this.bbox && ("l" in this.bbox)){
				if(this.bbox.l > x) this.bbox.l = x;
				if(this.bbox.r < x) this.bbox.r = x;
				if(this.bbox.t > y) this.bbox.t = y;
				if(this.bbox.b < y) this.bbox.b = y;
			}else{
				this.bbox = {l: x, b: y, r: x, t: y};
			}
		},
		_updateWithSegment: function(segment, matrix){
			// summary:
			//		updates the bounding box of path with new segment
			// segment: Object
			//		a segment
			var n = segment.args, l = n.length, i;
			// update internal variables: bbox, absolute, last
			switch(segment.action){
				case "M":
				case "L":
				case "C":
				case "S":
				case "Q":
				case "T":
					for(i = 0; i < l; i += 2){
						this._updateBBox(n[i], n[i + 1], matrix);
					}
					this.last.x = n[l - 2];
					this.last.y = n[l - 1];
					this.absolute = true;
					break;
				case "H":
					for(i = 0; i < l; ++i){
						this._updateBBox(n[i], this.last.y, matrix);
					}
					this.last.x = n[l - 1];
					this.absolute = true;
					break;
				case "V":
					for(i = 0; i < l; ++i){
						this._updateBBox(this.last.x, n[i], matrix);
					}
					this.last.y = n[l - 1];
					this.absolute = true;
					break;
				case "m":
					var start = 0;
					if(!("x" in this.last)){
						this._updateBBox(this.last.x = n[0], this.last.y = n[1], matrix);
						start = 2;
					}
					for(i = start; i < l; i += 2){
						this._updateBBox(this.last.x += n[i], this.last.y += n[i + 1], matrix);
					}
					this.absolute = false;
					break;
				case "l":
				case "t":
					for(i = 0; i < l; i += 2){
						this._updateBBox(this.last.x += n[i], this.last.y += n[i + 1], matrix);
					}
					this.absolute = false;
					break;
				case "h":
					for(i = 0; i < l; ++i){
						this._updateBBox(this.last.x += n[i], this.last.y, matrix);
					}
					this.absolute = false;
					break;
				case "v":
					for(i = 0; i < l; ++i){
						this._updateBBox(this.last.x, this.last.y += n[i], matrix);
					}
					this.absolute = false;
					break;
				case "c":
					for(i = 0; i < l; i += 6){
						this._updateBBox(this.last.x + n[i], this.last.y + n[i + 1], matrix);
						this._updateBBox(this.last.x + n[i + 2], this.last.y + n[i + 3], matrix);
						this._updateBBox(this.last.x += n[i + 4], this.last.y += n[i + 5], matrix);
					}
					this.absolute = false;
					break;
				case "s":
				case "q":
					for(i = 0; i < l; i += 4){
						this._updateBBox(this.last.x + n[i], this.last.y + n[i + 1], matrix);
						this._updateBBox(this.last.x += n[i + 2], this.last.y += n[i + 3], matrix);
					}
					this.absolute = false;
					break;
				case "A":
					for(i = 0; i < l; i += 7){
						this._updateBBox(n[i + 5], n[i + 6], matrix);
					}
					this.last.x = n[l - 2];
					this.last.y = n[l - 1];
					this.absolute = true;
					break;
				case "a":
					for(i = 0; i < l; i += 7){
						this._updateBBox(this.last.x += n[i + 5], this.last.y += n[i + 6], matrix);
					}
					this.absolute = false;
					break;
			}
			// add an SVG path segment
			var path = [segment.action];
			for(i = 0; i < l; ++i){
				path.push(g.formatNumber(n[i], true));
			}
			if(typeof this.shape.path == "string"){
				this.shape.path += path.join("");
			}else{
				for(i = 0, l = path.length; i < l; ++i){
					this.shape.path.push(path[i]);
				}
			}
		},

		// a dictionary, which maps segment type codes to a number of their arguments
		_validSegments: {m: 2, l: 2, h: 1, v: 1, c: 6, s: 4, q: 4, t: 2, a: 7, z: 0},

		_pushSegment: function(action, args){
			// summary:
			//		adds a segment
			// action: String
			//		valid SVG code for a segment's type
			// args: Array
			//		a list of parameters for this segment
			this.tbbox = null;
			var group = this._validSegments[action.toLowerCase()], segment;
			if(typeof group == "number"){
				if(group){
					if(args.length >= group){
						segment = {action: action, args: args.slice(0, args.length - args.length % group)};
						this.segments.push(segment);
						this._updateWithSegment(segment);
					}
				}else{
					segment = {action: action, args: []};
					this.segments.push(segment);
					this._updateWithSegment(segment);
				}
			}
		},

		_collectArgs: function(array, args){
			// summary:
			//		converts an array of arguments to plain numeric values
			// array: Array
			//		an output argument (array of numbers)
			// args: Array
			//		an input argument (can be values of Boolean, Number, dojox/gfx.Point, or an embedded array of them)
			for(var i = 0; i < args.length; ++i){
				var t = args[i];
				if(typeof t == "boolean"){
					array.push(t ? 1 : 0);
				}else if(typeof t == "number"){
					array.push(t);
				}else if(t instanceof Array){
					this._collectArgs(array, t);
				}else if("x" in t && "y" in t){
					array.push(t.x, t.y);
				}
			}
		},

		// segments
		moveTo: function(){
			// summary:
			//		forms a move segment
			this._confirmSegmented();
			var args = [];
			this._collectArgs(args, arguments);
			this._pushSegment(this.absolute ? "M" : "m", args);
			return this; // self
		},
		lineTo: function(){
			// summary:
			//		forms a line segment
			this._confirmSegmented();
			var args = [];
			this._collectArgs(args, arguments);
			this._pushSegment(this.absolute ? "L" : "l", args);
			return this; // self
		},
		hLineTo: function(){
			// summary:
			//		forms a horizontal line segment
			this._confirmSegmented();
			var args = [];
			this._collectArgs(args, arguments);
			this._pushSegment(this.absolute ? "H" : "h", args);
			return this; // self
		},
		vLineTo: function(){
			// summary:
			//		forms a vertical line segment
			this._confirmSegmented();
			var args = [];
			this._collectArgs(args, arguments);
			this._pushSegment(this.absolute ? "V" : "v", args);
			return this; // self
		},
		curveTo: function(){
			// summary:
			//		forms a curve segment
			this._confirmSegmented();
			var args = [];
			this._collectArgs(args, arguments);
			this._pushSegment(this.absolute ? "C" : "c", args);
			return this; // self
		},
		smoothCurveTo: function(){
			// summary:
			//		forms a smooth curve segment
			this._confirmSegmented();
			var args = [];
			this._collectArgs(args, arguments);
			this._pushSegment(this.absolute ? "S" : "s", args);
			return this; // self
		},
		qCurveTo: function(){
			// summary:
			//		forms a quadratic curve segment
			this._confirmSegmented();
			var args = [];
			this._collectArgs(args, arguments);
			this._pushSegment(this.absolute ? "Q" : "q", args);
			return this; // self
		},
		qSmoothCurveTo: function(){
			// summary:
			//		forms a quadratic smooth curve segment
			this._confirmSegmented();
			var args = [];
			this._collectArgs(args, arguments);
			this._pushSegment(this.absolute ? "T" : "t", args);
			return this; // self
		},
		arcTo: function(){
			// summary:
			//		forms an elliptic arc segment
			this._confirmSegmented();
			var args = [];
			this._collectArgs(args, arguments);
			this._pushSegment(this.absolute ? "A" : "a", args);
			return this; // self
		},
		closePath: function(){
			// summary:
			//		closes a path
			this._confirmSegmented();
			this._pushSegment("Z", []);
			return this; // self
		},

		_confirmSegmented: function() {
			if (!this.segmented) {
				var path = this.shape.path;
				// switch to non-updating version of path building
				this.shape.path = [];
				this._setPath(path);
				// switch back to the string path
				this.shape.path = this.shape.path.join("");
				// become segmented
				this.segmented = true;
			}
		},

		// setShape
		_setPath: function(path){
			// summary:
			//		forms a path using an SVG path string
			// path: String
			//		an SVG path string
			var p = lang.isArray(path) ? path : path.match(g.pathSvgRegExp);
			this.segments = [];
			this.absolute = true;
			this.bbox = {};
			this.last = {};
			if(!p) return;
			// create segments
			var action = "",	// current action
				args = [],		// current arguments
				l = p.length;
			for(var i = 0; i < l; ++i){
				var t = p[i], x = parseFloat(t);
				if(isNaN(x)){
					if(action){
						this._pushSegment(action, args);
					}
					args = [];
					action = t;
				}else{
					args.push(x);
				}
			}
			this._pushSegment(action, args);
		},
		setShape: function(newShape){
			// summary:
			//		forms a path using a shape
			// newShape: Object
			//		an SVG path string or a path object (see dojox/gfx.defaultPath)
			this.inherited(arguments, [typeof newShape == "string" ? {path: newShape} : newShape]);

			this.segmented = false;
			this.segments = [];
			if(!g.lazyPathSegmentation){
				this._confirmSegmented();
			}
			return this; // self
		},

		// useful constant for descendants
		_2PI: Math.PI * 2
	});

	var TextPath = declare("dojox.gfx.path.TextPath", Path, {
		// summary:
		//		a generalized TextPath shape

		constructor: function(rawNode){
			// summary:
			//		a TextPath shape constructor
			// rawNode: Node
			//		a DOM node to be used by this TextPath object
			if(!("text" in this)){
				this.text = lang.clone(g.defaultTextPath);
			}
			if(!("fontStyle" in this)){
				this.fontStyle = lang.clone(g.defaultFont);
			}
		},
		getText: function(){
			// summary:
			//		returns the current text object or null
			return this.text;	// Object
		},
		setText: function(newText){
			// summary:
			//		sets a text to be drawn along the path
			this.text = g.makeParameters(this.text,
				typeof newText == "string" ? {text: newText} : newText);
			this._setText();
			return this;	// self
		},
		getFont: function(){
			// summary:
			//		returns the current font object or null
			return this.fontStyle;	// Object
		},
		setFont: function(newFont){
			// summary:
			//		sets a font for text
			this.fontStyle = typeof newFont == "string" ?
				g.splitFontString(newFont) :
				g.makeParameters(g.defaultFont, newFont);
			this._setFont();
			return this;	// self
		}
	});

	return g.path = { // our hash of newly defined objects
		Path: Path,
		TextPath: TextPath
	};
});

},
'dojox/gfx/svg':function(){
define("dojox/gfx/svg", ["dojo/_base/lang", "dojo/_base/sniff", "dojo/_base/window", "dojo/dom", "dojo/_base/declare", "dojo/_base/array",
  "dojo/dom-geometry", "dojo/dom-attr", "dojo/_base/Color", "./_base", "./shape", "./path"],
function(lang, has, win, dom, declare, arr, domGeom, domAttr, Color, g, gs, pathLib){

	var svg = g.svg = {
		// summary:
		//		This the graphics rendering bridge for browsers compliant with W3C SVG1.0.
		//		This is the preferred renderer to use for interactive and accessible graphics.
	};
	svg.useSvgWeb = (typeof window.svgweb != "undefined");

	// Need to detect iOS in order to workaround bug when
	// touching nodes with text
	var uagent = navigator.userAgent,
		safMobile = has("ios"),
		android = has("android"),
		textRenderingFix = has("chrome") || (android && android>=4) ? "auto" : "optimizeLegibility";// #16099, #16461

	function _createElementNS(ns, nodeType){
		// summary:
		//		Internal helper to deal with creating elements that
		//		are namespaced.  Mainly to get SVG markup output
		//		working on IE.
		if(win.doc.createElementNS){
			return win.doc.createElementNS(ns,nodeType);
		}else{
			return win.doc.createElement(nodeType);
		}
	}
	
	function _setAttributeNS(node, ns, attr, value){
		if(node.setAttributeNS){
			return node.setAttributeNS(ns, attr, value);
		}else{
			return node.setAttribute(attr, value);
		}
	}

	function _createTextNode(text){
		if(svg.useSvgWeb){
			return win.doc.createTextNode(text, true);
		}else{
			return win.doc.createTextNode(text);
		}
	}

	function _createFragment(){
		if(svg.useSvgWeb){
			return win.doc.createDocumentFragment(true);
		}else{
			return win.doc.createDocumentFragment();
		}
	}

	svg.xmlns = {
		xlink: "http://www.w3.org/1999/xlink",
		svg:   "http://www.w3.org/2000/svg"
	};

	svg.getRef = function(name){
		// summary:
		//		looks up a node by its external name
		// name: String
		//		an SVG external reference
		// returns:
		//      returns a DOM Node specified by the name argument or null
		if(!name || name == "none") return null;
		if(name.match(/^url\(#.+\)$/)){
			return dom.byId(name.slice(5, -1));	// Node
		}
		// alternative representation of a reference
		if(name.match(/^#dojoUnique\d+$/)){
			// we assume here that a reference was generated by dojox/gfx
			return dom.byId(name.slice(1));	// Node
		}
		return null;	// Node
	};

	svg.dasharray = {
		solid:				"none",
		shortdash:			[4, 1],
		shortdot:			[1, 1],
		shortdashdot:		[4, 1, 1, 1],
		shortdashdotdot:	[4, 1, 1, 1, 1, 1],
		dot:				[1, 3],
		dash:				[4, 3],
		longdash:			[8, 3],
		dashdot:			[4, 3, 1, 3],
		longdashdot:		[8, 3, 1, 3],
		longdashdotdot:		[8, 3, 1, 3, 1, 3]
	};

	var clipCount = 0;

	svg.Shape = declare("dojox.gfx.svg.Shape", gs.Shape, {
		// summary:
		//		SVG-specific implementation of dojox/gfx/shape.Shape methods

		destroy: function(){
			if(this.fillStyle && "type" in this.fillStyle){
				var fill = this.rawNode.getAttribute("fill"),
					ref  = svg.getRef(fill);
				if(ref){
					ref.parentNode.removeChild(ref);
				}
			}
			if(this.clip){
				var clipPathProp = this.rawNode.getAttribute("clip-path");
				if(clipPathProp){
					var clipNode = dom.byId(clipPathProp.match(/gfx_clip[\d]+/)[0]);
					clipNode && clipNode.parentNode.removeChild(clipNode);
				}
			}
			this.rawNode = null;
			gs.Shape.prototype.destroy.apply(this, arguments);
		},

		setFill: function(fill){
			// summary:
			//		sets a fill object (SVG)
			// fill: Object
			//		a fill object
			//		(see dojox/gfx.defaultLinearGradient,
			//		dojox/gfx.defaultRadialGradient,
			//		dojox/gfx.defaultPattern,
			//		or dojo/_base/Color)

			if(!fill){
				// don't fill
				this.fillStyle = null;
				this.rawNode.setAttribute("fill", "none");
				this.rawNode.setAttribute("fill-opacity", 0);
				return this;
			}
			var f;
			// FIXME: slightly magical. We're using the outer scope's "f", but setting it later
			var setter = function(x){
					// we assume that we're executing in the scope of the node to mutate
					this.setAttribute(x, f[x].toFixed(8));
				};
			if(typeof(fill) == "object" && "type" in fill){
				// gradient
				switch(fill.type){
					case "linear":
						f = g.makeParameters(g.defaultLinearGradient, fill);
						var gradient = this._setFillObject(f, "linearGradient");
						arr.forEach(["x1", "y1", "x2", "y2"], setter, gradient);
						break;
					case "radial":
						f = g.makeParameters(g.defaultRadialGradient, fill);
						var grad = this._setFillObject(f, "radialGradient");
						arr.forEach(["cx", "cy", "r"], setter, grad);
						break;
					case "pattern":
						f = g.makeParameters(g.defaultPattern, fill);
						var pattern = this._setFillObject(f, "pattern");
						arr.forEach(["x", "y", "width", "height"], setter, pattern);
						break;
				}
				this.fillStyle = f;
				return this;
			}
			// color object
			f = g.normalizeColor(fill);
			this.fillStyle = f;
			this.rawNode.setAttribute("fill", f.toCss());
			this.rawNode.setAttribute("fill-opacity", f.a);
			this.rawNode.setAttribute("fill-rule", "evenodd");
			return this;	// self
		},

		setStroke: function(stroke){
			// summary:
			//		sets a stroke object (SVG)
			// stroke: Object
			//		a stroke object (see dojox/gfx.defaultStroke)

			var rn = this.rawNode;
			if(!stroke){
				// don't stroke
				this.strokeStyle = null;
				rn.setAttribute("stroke", "none");
				rn.setAttribute("stroke-opacity", 0);
				return this;
			}
			// normalize the stroke
			if(typeof stroke == "string" || lang.isArray(stroke) || stroke instanceof Color){
				stroke = { color: stroke };
			}
			var s = this.strokeStyle = g.makeParameters(g.defaultStroke, stroke);
			s.color = g.normalizeColor(s.color);
			// generate attributes
			if(s){
				rn.setAttribute("stroke", s.color.toCss());
				rn.setAttribute("stroke-opacity", s.color.a);
				rn.setAttribute("stroke-width",   s.width);
				rn.setAttribute("stroke-linecap", s.cap);
				if(typeof s.join == "number"){
					rn.setAttribute("stroke-linejoin",   "miter");
					rn.setAttribute("stroke-miterlimit", s.join);
				}else{
					rn.setAttribute("stroke-linejoin",   s.join);
				}
				var da = s.style.toLowerCase();
				if(da in svg.dasharray){
					da = svg.dasharray[da];
				}
				if(da instanceof Array){
					da = lang._toArray(da);
					for(var i = 0; i < da.length; ++i){
						da[i] *= s.width;
					}
					if(s.cap != "butt"){
						for(var i = 0; i < da.length; i += 2){
							da[i] -= s.width;
							if(da[i] < 1){ da[i] = 1; }
						}
						for(var i = 1; i < da.length; i += 2){
							da[i] += s.width;
						}
					}
					da = da.join(",");
				}
				rn.setAttribute("stroke-dasharray", da);
				rn.setAttribute("dojoGfxStrokeStyle", s.style);
			}
			return this;	// self
		},

		_getParentSurface: function(){
			var surface = this.parent;
			for(; surface && !(surface instanceof g.Surface); surface = surface.parent);
			return surface;
		},

		_setFillObject: function(f, nodeType){
			var svgns = svg.xmlns.svg;
			this.fillStyle = f;
			var surface = this._getParentSurface(),
				defs = surface.defNode,
				fill = this.rawNode.getAttribute("fill"),
				ref  = svg.getRef(fill);
			if(ref){
				fill = ref;
				if(fill.tagName.toLowerCase() != nodeType.toLowerCase()){
					var id = fill.id;
					fill.parentNode.removeChild(fill);
					fill = _createElementNS(svgns, nodeType);
					fill.setAttribute("id", id);
					defs.appendChild(fill);
				}else{
					while(fill.childNodes.length){
						fill.removeChild(fill.lastChild);
					}
				}
			}else{
				fill = _createElementNS(svgns, nodeType);
				fill.setAttribute("id", g._base._getUniqueId());
				defs.appendChild(fill);
			}
			if(nodeType == "pattern"){
				fill.setAttribute("patternUnits", "userSpaceOnUse");
				var img = _createElementNS(svgns, "image");
				img.setAttribute("x", 0);
				img.setAttribute("y", 0);
				img.setAttribute("width",  f.width .toFixed(8));
				img.setAttribute("height", f.height.toFixed(8));
				_setAttributeNS(img, svg.xmlns.xlink, "xlink:href", f.src);
				fill.appendChild(img);
			}else{
				fill.setAttribute("gradientUnits", "userSpaceOnUse");
				for(var i = 0; i < f.colors.length; ++i){
					var c = f.colors[i], t = _createElementNS(svgns, "stop"),
						cc = c.color = g.normalizeColor(c.color);
					t.setAttribute("offset",       c.offset.toFixed(8));
					t.setAttribute("stop-color",   cc.toCss());
					t.setAttribute("stop-opacity", cc.a);
					fill.appendChild(t);
				}
			}
			this.rawNode.setAttribute("fill", "url(#" + fill.getAttribute("id") +")");
			this.rawNode.removeAttribute("fill-opacity");
			this.rawNode.setAttribute("fill-rule", "evenodd");
			return fill;
		},

		_applyTransform: function() {
			var matrix = this.matrix;
			if(matrix){
				var tm = this.matrix;
				this.rawNode.setAttribute("transform", "matrix(" +
					tm.xx.toFixed(8) + "," + tm.yx.toFixed(8) + "," +
					tm.xy.toFixed(8) + "," + tm.yy.toFixed(8) + "," +
					tm.dx.toFixed(8) + "," + tm.dy.toFixed(8) + ")");
			}else{
				this.rawNode.removeAttribute("transform");
			}
			return this;
		},

		setRawNode: function(rawNode){
			// summary:
			//		assigns and clears the underlying node that will represent this
			//		shape. Once set, transforms, gradients, etc, can be applied.
			//		(no fill & stroke by default)
			var r = this.rawNode = rawNode;
			if(this.shape.type!="image"){
				r.setAttribute("fill", "none");
			}
			r.setAttribute("fill-opacity", 0);
			r.setAttribute("stroke", "none");
			r.setAttribute("stroke-opacity", 0);
			r.setAttribute("stroke-width", 1);
			r.setAttribute("stroke-linecap", "butt");
			r.setAttribute("stroke-linejoin", "miter");
			r.setAttribute("stroke-miterlimit", 4);
			// Bind GFX object with SVG node for ease of retrieval - that is to
			// save code/performance to keep this association elsewhere
			r.__gfxObject__ = this.getUID();
		},

		setShape: function(newShape){
			// summary:
			//		sets a shape object (SVG)
			// newShape: Object
			//		a shape object
			//		(see dojox/gfx.defaultPath,
			//		dojox/gfx.defaultPolyline,
			//		dojox/gfx.defaultRect,
			//		dojox/gfx.defaultEllipse,
			//		dojox/gfx.defaultCircle,
			//		dojox/gfx.defaultLine,
			//		or dojox/gfx.defaultImage)
			this.shape = g.makeParameters(this.shape, newShape);
			for(var i in this.shape){
				if(i != "type"){
					this.rawNode.setAttribute(i, this.shape[i]);
				}
			}
			this.bbox = null;
			return this;	// self
		},

		// move family

		_moveToFront: function(){
			// summary:
			//		moves a shape to front of its parent's list of shapes (SVG)
			this.rawNode.parentNode.appendChild(this.rawNode);
			return this;	// self
		},
		_moveToBack: function(){
			// summary:
			//		moves a shape to back of its parent's list of shapes (SVG)
			this.rawNode.parentNode.insertBefore(this.rawNode, this.rawNode.parentNode.firstChild);
			return this;	// self
		},
		setClip: function(clip){
			// summary:
			//		sets the clipping area of this shape.
			// description:
			//		This method overrides the dojox/gfx/shape.Shape.setClip() method.
			// clip: Object
			//		an object that defines the clipping geometry, or null to remove clip.
			this.inherited(arguments);
			var clipType = clip ? "width" in clip ? "rect" : 
							"cx" in clip ? "ellipse" : 
							"points" in clip ? "polyline" : "d" in clip ? "path" : null : null;
			if(clip && !clipType){
				return this;
			}
			if(clipType === "polyline"){
				clip = lang.clone(clip);
				clip.points = clip.points.join(",");
			}
			var clipNode, clipShape,
				clipPathProp = domAttr.get(this.rawNode, "clip-path");
			if(clipPathProp){
				clipNode = dom.byId(clipPathProp.match(/gfx_clip[\d]+/)[0]);
				if(clipNode){ // may be null if not in the DOM anymore
					clipNode.removeChild(clipNode.childNodes[0]);
				}
			}
			if(clip){
				if(clipNode){
					clipShape = _createElementNS(svg.xmlns.svg, clipType);
					clipNode.appendChild(clipShape);
				}else{
					var idIndex = ++clipCount;
					var clipId = "gfx_clip" + idIndex;
					var clipUrl = "url(#" + clipId + ")";
					this.rawNode.setAttribute("clip-path", clipUrl);
					clipNode = _createElementNS(svg.xmlns.svg, "clipPath");
					clipShape = _createElementNS(svg.xmlns.svg, clipType);
					clipNode.appendChild(clipShape);
					this.rawNode.parentNode.appendChild(clipNode);
					domAttr.set(clipNode, "id", clipId);
				}
				domAttr.set(clipShape, clip);
			}else{
				//remove clip-path
				this.rawNode.removeAttribute("clip-path");
				if(clipNode){
					clipNode.parentNode.removeChild(clipNode);
				}
			}
			return this;
		},
		_removeClipNode: function(){
			var clipNode, clipPathProp = domAttr.get(this.rawNode, "clip-path");
			if(clipPathProp){
				clipNode = dom.byId(clipPathProp.match(/gfx_clip[\d]+/)[0]);
				if(clipNode){
					clipNode.parentNode.removeChild(clipNode);
				}
			}
			return clipNode;
		}
	});


	svg.Group = declare("dojox.gfx.svg.Group", svg.Shape, {
		// summary:
		//		a group shape (SVG), which can be used
		//		to logically group shapes (e.g, to propagate matricies)
		constructor: function(){
			gs.Container._init.call(this);
		},
		setRawNode: function(rawNode){
			// summary:
			//		sets a raw SVG node to be used by this shape
			// rawNode: Node
			//		an SVG node
			this.rawNode = rawNode;
			// Bind GFX object with SVG node for ease of retrieval - that is to
			// save code/performance to keep this association elsewhere
			this.rawNode.__gfxObject__ = this.getUID();
		},
		destroy: function(){
			// summary:
			//		Releases all internal resources owned by this shape. Once this method has been called,
			//		the instance is considered disposed and should not be used anymore.
			this.clear(true);
			// avoid this.inherited
			svg.Shape.prototype.destroy.apply(this, arguments);
		}
	});
	svg.Group.nodeType = "g";

	svg.Rect = declare("dojox.gfx.svg.Rect", [svg.Shape, gs.Rect], {
		// summary:
		//		a rectangle shape (SVG)
		setShape: function(newShape){
			// summary:
			//		sets a rectangle shape object (SVG)
			// newShape: Object
			//		a rectangle shape object
			this.shape = g.makeParameters(this.shape, newShape);
			this.bbox = null;
			for(var i in this.shape){
				if(i != "type" && i != "r"){
					this.rawNode.setAttribute(i, this.shape[i]);
				}
			}
			if(this.shape.r != null){
				this.rawNode.setAttribute("ry", this.shape.r);
				this.rawNode.setAttribute("rx", this.shape.r);
			}
			return this;	// self
		}
	});
	svg.Rect.nodeType = "rect";

	svg.Ellipse = declare("dojox.gfx.svg.Ellipse", [svg.Shape, gs.Ellipse], {});
	svg.Ellipse.nodeType = "ellipse";

	svg.Circle = declare("dojox.gfx.svg.Circle", [svg.Shape, gs.Circle], {});
	svg.Circle.nodeType = "circle";

	svg.Line = declare("dojox.gfx.svg.Line", [svg.Shape, gs.Line], {});
	svg.Line.nodeType = "line";

	svg.Polyline = declare("dojox.gfx.svg.Polyline", [svg.Shape, gs.Polyline], {
		// summary:
		//		a polyline/polygon shape (SVG)
		setShape: function(points, closed){
			// summary:
			//		sets a polyline/polygon shape object (SVG)
			// points: Object|Array
			//		a polyline/polygon shape object, or an array of points
			if(points && points instanceof Array){
				this.shape = g.makeParameters(this.shape, { points: points });
				if(closed && this.shape.points.length){
					this.shape.points.push(this.shape.points[0]);
				}
			}else{
				this.shape = g.makeParameters(this.shape, points);
			}
			this.bbox = null;
			this._normalizePoints();
			var attr = [], p = this.shape.points;
			for(var i = 0; i < p.length; ++i){
				attr.push(p[i].x.toFixed(8), p[i].y.toFixed(8));
			}
			this.rawNode.setAttribute("points", attr.join(" "));
			return this;	// self
		}
	});
	svg.Polyline.nodeType = "polyline";

	svg.Image = declare("dojox.gfx.svg.Image", [svg.Shape, gs.Image], {
		// summary:
		//		an image (SVG)
		setShape: function(newShape){
			// summary:
			//		sets an image shape object (SVG)
			// newShape: Object
			//		an image shape object
			this.shape = g.makeParameters(this.shape, newShape);
			this.bbox = null;
			var rawNode = this.rawNode;
			for(var i in this.shape){
				if(i != "type" && i != "src"){
					rawNode.setAttribute(i, this.shape[i]);
				}
			}
			rawNode.setAttribute("preserveAspectRatio", "none");
			_setAttributeNS(rawNode, svg.xmlns.xlink, "xlink:href", this.shape.src);
			// Bind GFX object with SVG node for ease of retrieval - that is to
			// save code/performance to keep this association elsewhere
			rawNode.__gfxObject__ = this.getUID();
			return this;	// self
		}
	});
	svg.Image.nodeType = "image";

	svg.Text = declare("dojox.gfx.svg.Text", [svg.Shape, gs.Text], {
		// summary:
		//		an anchored text (SVG)
		setShape: function(newShape){
			// summary:
			//		sets a text shape object (SVG)
			// newShape: Object
			//		a text shape object
			this.shape = g.makeParameters(this.shape, newShape);
			this.bbox = null;
			var r = this.rawNode, s = this.shape;
			r.setAttribute("x", s.x);
			r.setAttribute("y", s.y);
			r.setAttribute("text-anchor", s.align);
			r.setAttribute("text-decoration", s.decoration);
			r.setAttribute("rotate", s.rotated ? 90 : 0);
			r.setAttribute("kerning", s.kerning ? "auto" : 0);
			r.setAttribute("text-rendering", textRenderingFix);

			// update the text content
			if(r.firstChild){
				r.firstChild.nodeValue = s.text;
			}else{
				r.appendChild(_createTextNode(s.text));
			}
			return this;	// self
		},
		getTextWidth: function(){
			// summary:
			//		get the text width in pixels
			var rawNode = this.rawNode,
				oldParent = rawNode.parentNode,
				_measurementNode = rawNode.cloneNode(true);
			_measurementNode.style.visibility = "hidden";

			// solution to the "orphan issue" in FF
			var _width = 0, _text = _measurementNode.firstChild.nodeValue;
			oldParent.appendChild(_measurementNode);

			// solution to the "orphan issue" in Opera
			// (nodeValue == "" hangs firefox)
			if(_text!=""){
				while(!_width){
//Yang: work around svgweb bug 417 -- http://code.google.com/p/svgweb/issues/detail?id=417
if (_measurementNode.getBBox)
					_width = parseInt(_measurementNode.getBBox().width);
else
	_width = 68;
				}
			}
			oldParent.removeChild(_measurementNode);
			return _width;
		}
	});
	svg.Text.nodeType = "text";

	svg.Path = declare("dojox.gfx.svg.Path", [svg.Shape, pathLib.Path], {
		// summary:
		//		a path shape (SVG)
		_updateWithSegment: function(segment){
			// summary:
			//		updates the bounding box of path with new segment
			// segment: Object
			//		a segment
			this.inherited(arguments);
			if(typeof(this.shape.path) == "string"){
				this.rawNode.setAttribute("d", this.shape.path);
			}
		},
		setShape: function(newShape){
			// summary:
			//		forms a path using a shape (SVG)
			// newShape: Object
			//		an SVG path string or a path object (see dojox/gfx.defaultPath)
			this.inherited(arguments);
			if(this.shape.path){
				this.rawNode.setAttribute("d", this.shape.path);
			}else{
				this.rawNode.removeAttribute("d");
			}
			return this;	// self
		}
	});
	svg.Path.nodeType = "path";

	svg.TextPath = declare("dojox.gfx.svg.TextPath", [svg.Shape, pathLib.TextPath], {
		// summary:
		//		a textpath shape (SVG)
		_updateWithSegment: function(segment){
			// summary:
			//		updates the bounding box of path with new segment
			// segment: Object
			//		a segment
			this.inherited(arguments);
			this._setTextPath();
		},
		setShape: function(newShape){
			// summary:
			//		forms a path using a shape (SVG)
			// newShape: Object
			//		an SVG path string or a path object (see dojox/gfx.defaultPath)
			this.inherited(arguments);
			this._setTextPath();
			return this;	// self
		},
		_setTextPath: function(){
			if(typeof this.shape.path != "string"){ return; }
			var r = this.rawNode;
			if(!r.firstChild){
				var tp = _createElementNS(svg.xmlns.svg, "textPath"),
					tx = _createTextNode("");
				tp.appendChild(tx);
				r.appendChild(tp);
			}
			var ref  = r.firstChild.getAttributeNS(svg.xmlns.xlink, "href"),
				path = ref && svg.getRef(ref);
			if(!path){
				var surface = this._getParentSurface();
				if(surface){
					var defs = surface.defNode;
					path = _createElementNS(svg.xmlns.svg, "path");
					var id = g._base._getUniqueId();
					path.setAttribute("id", id);
					defs.appendChild(path);
					_setAttributeNS(r.firstChild, svg.xmlns.xlink, "xlink:href", "#" + id);
				}
			}
			if(path){
				path.setAttribute("d", this.shape.path);
			}
		},
		_setText: function(){
			var r = this.rawNode;
			if(!r.firstChild){
				var tp = _createElementNS(svg.xmlns.svg, "textPath"),
					tx = _createTextNode("");
				tp.appendChild(tx);
				r.appendChild(tp);
			}
			r = r.firstChild;
			var t = this.text;
			r.setAttribute("alignment-baseline", "middle");
			switch(t.align){
				case "middle":
					r.setAttribute("text-anchor", "middle");
					r.setAttribute("startOffset", "50%");
					break;
				case "end":
					r.setAttribute("text-anchor", "end");
					r.setAttribute("startOffset", "100%");
					break;
				default:
					r.setAttribute("text-anchor", "start");
					r.setAttribute("startOffset", "0%");
					break;
			}
			//r.parentNode.setAttribute("alignment-baseline", "central");
			//r.setAttribute("dominant-baseline", "central");
			r.setAttribute("baseline-shift", "0.5ex");
			r.setAttribute("text-decoration", t.decoration);
			r.setAttribute("rotate", t.rotated ? 90 : 0);
			r.setAttribute("kerning", t.kerning ? "auto" : 0);
			r.firstChild.data = t.text;
		}
	});
	svg.TextPath.nodeType = "text";

	// Fix for setDimension bug:
	// http://bugs.dojotoolkit.org/ticket/16100
	// (https://code.google.com/p/chromium/issues/detail?id=162628)
	var hasSvgSetAttributeBug = (function(){ var matches = /WebKit\/(\d*)/.exec(uagent); return matches ? matches[1] : 0})() > 534;

	svg.Surface = declare("dojox.gfx.svg.Surface", gs.Surface, {
		// summary:
		//		a surface object to be used for drawings (SVG)
		constructor: function(){
			gs.Container._init.call(this);
		},
		destroy: function(){
			// no need to call svg.Container.clear to remove the children raw
			// nodes since the surface raw node will be removed. So, only dispose at gfx level	
			gs.Container.clear.call(this, true); 
			this.defNode = null;	// release the external reference
			this.inherited(arguments);
		},
		setDimensions: function(width, height){
			// summary:
			//		sets the width and height of the rawNode
			// width: String
			//		width of surface, e.g., "100px"
			// height: String
			//		height of surface, e.g., "100px"
			if(!this.rawNode){ return this; }
			this.rawNode.setAttribute("width",  width);
			this.rawNode.setAttribute("height", height);
			if(hasSvgSetAttributeBug){
				this.rawNode.style.width =  width;
				this.rawNode.style.height =  height;
			}
			return this;	// self
		},
		getDimensions: function(){
			// summary:
			//		returns an object with properties "width" and "height"
			var t = this.rawNode ? {
				width:  g.normalizedLength(this.rawNode.getAttribute("width")),
				height: g.normalizedLength(this.rawNode.getAttribute("height"))} : null;
			return t;	// Object
		}
	});

	svg.createSurface = function(parentNode, width, height){
		// summary:
		//		creates a surface (SVG)
		// parentNode: Node
		//		a parent node
		// width: String|Number
		//		width of surface, e.g., "100px" or 100
		// height: String|Number
		//		height of surface, e.g., "100px" or 100
		// returns: dojox/gfx/shape.Surface
		//     newly created surface

		var s = new svg.Surface();
		s.rawNode = _createElementNS(svg.xmlns.svg, "svg");
		s.rawNode.setAttribute("overflow", "hidden");
		if(width){
			s.rawNode.setAttribute("width",  width);
		}
		if(height){
			s.rawNode.setAttribute("height", height);
		}

		var defNode = _createElementNS(svg.xmlns.svg, "defs");
		s.rawNode.appendChild(defNode);
		s.defNode = defNode;

		s._parent = dom.byId(parentNode);
		s._parent.appendChild(s.rawNode);

		return s;	// dojox/gfx.Surface
	};

	// Extenders

	var Font = {
		_setFont: function(){
			// summary:
			//		sets a font object (SVG)
			var f = this.fontStyle;
			// next line doesn't work in Firefox 2 or Opera 9
			//this.rawNode.setAttribute("font", dojox.gfx.makeFontString(this.fontStyle));
			this.rawNode.setAttribute("font-style", f.style);
			this.rawNode.setAttribute("font-variant", f.variant);
			this.rawNode.setAttribute("font-weight", f.weight);
			this.rawNode.setAttribute("font-size", f.size);
			this.rawNode.setAttribute("font-family", f.family);
		}
	};

	var C = gs.Container, Container = {
		openBatch: function() {
			// summary:
			//		starts a new batch, subsequent new child shapes will be held in
			//		the batch instead of appending to the container directly
			this.fragment = _createFragment();
		},
		closeBatch: function() {
			// summary:
			//		submits the current batch, append all pending child shapes to DOM
			if (this.fragment) {
				this.rawNode.appendChild(this.fragment);
				delete this.fragment;
			}
		},
		add: function(shape){
			// summary:
			//		adds a shape to a group/surface
			// shape: dojox/gfx/shape.Shape
			//		an VML shape object
			if(this != shape.getParent()){
				if (this.fragment) {
					this.fragment.appendChild(shape.rawNode);
				} else {
					this.rawNode.appendChild(shape.rawNode);
				}
				C.add.apply(this, arguments);
				// update clipnode with new parent
				shape.setClip(shape.clip);
			}
			return this;	// self
		},
		remove: function(shape, silently){
			// summary:
			//		remove a shape from a group/surface
			// shape: dojox/gfx/shape.Shape
			//		an VML shape object
			// silently: Boolean?
			//		if true, regenerate a picture
			if(this == shape.getParent()){
				if(this.rawNode == shape.rawNode.parentNode){
					this.rawNode.removeChild(shape.rawNode);
				}
				if(this.fragment && this.fragment == shape.rawNode.parentNode){
					this.fragment.removeChild(shape.rawNode);
				}
				// remove clip node from parent 
				shape._removeClipNode();
				C.remove.apply(this, arguments);
			}
			return this;	// self
		},
		clear: function(){
			// summary:
			//		removes all shapes from a group/surface
			var r = this.rawNode;
			while(r.lastChild){
				r.removeChild(r.lastChild);
			}
			var defNode = this.defNode;
			if(defNode){
				while(defNode.lastChild){
					defNode.removeChild(defNode.lastChild);
				}
				r.appendChild(defNode);
			}
			return C.clear.apply(this, arguments);
		},
		getBoundingBox: C.getBoundingBox,
		_moveChildToFront: C._moveChildToFront,
		_moveChildToBack:  C._moveChildToBack
	};

	var Creator = {
		// summary:
		//		SVG shape creators
		createObject: function(shapeType, rawShape){
			// summary:
			//		creates an instance of the passed shapeType class
			// shapeType: Function
			//		a class constructor to create an instance of
			// rawShape: Object
			//		properties to be passed in to the classes "setShape" method
			if(!this.rawNode){ return null; }
			var shape = new shapeType(),
				node = _createElementNS(svg.xmlns.svg, shapeType.nodeType);

			shape.setRawNode(node);
			shape.setShape(rawShape);
			// rawNode.appendChild() will be done inside this.add(shape) below
			this.add(shape);
			return shape;	// dojox/gfx/shape.Shape
		}
	};

	lang.extend(svg.Text, Font);
	lang.extend(svg.TextPath, Font);

	lang.extend(svg.Group, Container);
	lang.extend(svg.Group, gs.Creator);
	lang.extend(svg.Group, Creator);

	lang.extend(svg.Surface, Container);
	lang.extend(svg.Surface, gs.Creator);
	lang.extend(svg.Surface, Creator);

	// Mouse/Touch event
	svg.fixTarget = function(event, gfxElement) {
		// summary:
		//		Adds the gfxElement to event.gfxTarget if none exists. This new
		//		property will carry the GFX element associated with this event.
		// event: Object
		//		The current input event (MouseEvent or TouchEvent)
		// gfxElement: Object
		//		The GFX target element
		if (!event.gfxTarget) {
			if (safMobile && event.target.wholeText) {
				// Workaround iOS bug when touching text nodes
				event.gfxTarget = gs.byId(event.target.parentElement.__gfxObject__);
			} else {
				event.gfxTarget = gs.byId(event.target.__gfxObject__);
			}
		}
		return true;
	};

	// some specific override for svgweb + flash
	if(svg.useSvgWeb){
		// override createSurface()
		svg.createSurface = function(parentNode, width, height){
			var s = new svg.Surface();

			// ensure width / height
			if(!width || !height){
				var pos = domGeom.position(parentNode);
				width  = width  || pos.w;
				height = height || pos.h;
			}

			// ensure id
			parentNode = dom.byId(parentNode);
			var id = parentNode.id ? parentNode.id+'_svgweb' : g._base._getUniqueId();

			// create dynamic svg root
			var mockSvg = _createElementNS(svg.xmlns.svg, 'svg');
			mockSvg.id = id;
			mockSvg.setAttribute('width', width);
			mockSvg.setAttribute('height', height);
			svgweb.appendChild(mockSvg, parentNode);

			// notice: any call to the raw node before flash init will fail.
			mockSvg.addEventListener('SVGLoad', function(){
				// become loaded
				s.rawNode = this;
				s.isLoaded = true;

				// init defs
				var defNode = _createElementNS(svg.xmlns.svg, "defs");
				s.rawNode.appendChild(defNode);
				s.defNode = defNode;

				// notify application
				if (s.onLoad)
					s.onLoad(s);
			}, false);

			// flash not loaded yet
			s.isLoaded = false;
			return s;
		};

		// override Surface.destroy()
		svg.Surface.extend({
			destroy: function(){
				var mockSvg = this.rawNode;
				svgweb.removeChild(mockSvg, mockSvg.parentNode);
			}
		});

		// override connect() & disconnect() for Shape & Surface event processing
		var _eventsProcessing = {
			connect: function(name, object, method){
				// connect events using the mock addEventListener() provided by svgweb
				if (name.substring(0, 2)==='on') { name = name.substring(2); }
				if (arguments.length == 2) {
					method = object;
				} else {
					method = lang.hitch(object, method);
				}
				this.getEventSource().addEventListener(name, method, false);
				return [this, name, method];
			},
			disconnect: function(token){
				// disconnect events using the mock removeEventListener() provided by svgweb
				this.getEventSource().removeEventListener(token[1], token[2], false);
				delete token[0];
			}
		};

		lang.extend(svg.Shape, _eventsProcessing);
		lang.extend(svg.Surface, _eventsProcessing);
	}

	return svg;
});

},
'xstyle/core/load-css':function(){
define("xstyle/core/load-css", [], function(){
	'use strict';
	// this module is responsible for doing the loading/insertion
	// of stylesheets to get CSS loaded.

	var cache = typeof _css_cache == 'undefined' ? {} : _css_cache;
	var doc = document;

	function has(){
		return !doc.createStyleSheet;
	}
	var head = doc.head;
	function insertCss(css){
		if(has("dom-create-style-element")){
			// we can use standard <style> element creation
			styleSheet = doc.createElement("style");
			styleSheet.setAttribute("type", "text/css");
			styleSheet.appendChild(doc.createTextNode(css));
			head.insertBefore(styleSheet, head.firstChild);
			return styleSheet;
		}
		else{
			// IE's stylesheet insertion
			var styleSheet = doc.createStyleSheet();
			styleSheet.cssText = css;
			return styleSheet.owningElement;
		}
	}

	function load(resourceDef, callback, options){
		var cached = cache[resourceDef];
		if(cached){
			// if it is cached (from a build), we directly insert
			link = insertCss(cached);
			return callback(link);
		}
		// create a link element to load the stylesheet
		var link = doc.createElement('link');
		link.type = 'text/css';
		link.rel = 'stylesheet';
		link.href = resourceDef;
		var wait = !options || options.wait !== false;
		// old webkit's would claim to have onload, but didn't really support it
		var webkitVersion = navigator.userAgent.match(/AppleWebKit\/(\d+\.?\d*)/);
		webkitVersion = webkitVersion && +webkitVersion[1];
		if(link.onload === null && !(webkitVersion < 536)){
			// most browsers support this onload function now
			link.onload = function(){
				// cleanup
				link.onload = null;
				link.onerror = null;
				wait && callback(link);
			};
			// always add the error handler, so we can notify of any errors
			link.onerror = function(){
				// there isn't really any recourse in AMD for errors, so
				// we just output the error and continue on
				console.error('Error loading stylesheet ' + resourceDef);
				wait && callback(link);
			};
		}else if(wait){
			var interval = setInterval(function(){
				if(link.style){
					// loaded
					clearInterval(interval);
					callback(link);
				}
			}, 15);
		}
		// add it to the head to trigger loading
		(head || doc.getElementsByTagName('head')[0]).appendChild(link);
		if(!wait){
			// don't wait for the stylesheet to load, proceed
			callback(link);
		}
	}
	load.insertCss = insertCss;
	return load;
});

},
'dijit/TooltipDialog':function(){
require({cache:{
'url:dijit/templates/TooltipDialog.html':"<div role=\"alertdialog\" tabIndex=\"-1\">\n\t<div class=\"dijitTooltipContainer\" role=\"presentation\">\n\t\t<div class=\"dijitTooltipContents dijitTooltipFocusNode\" data-dojo-attach-point=\"containerNode\"></div>\n\t</div>\n\t<div class=\"dijitTooltipConnector\" role=\"presentation\" data-dojo-attach-point=\"connectorNode\"></div>\n</div>\n"}});
define("dijit/TooltipDialog", [
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.replace
	"dojo/_base/event", // event.stop
	"dojo/keys", // keys
	"dojo/_base/lang", // lang.hitch
	"./focus",
	"./layout/ContentPane",
	"./_DialogMixin",
	"./form/_FormMixin",
	"./_TemplatedMixin",
	"dojo/text!./templates/TooltipDialog.html",
	"./main"		// exports methods to dijit global
], function(declare, domClass, event, keys, lang,
			focus, ContentPane, _DialogMixin, _FormMixin, _TemplatedMixin, template, dijit){

	// module:
	//		dijit/TooltipDialog


	return declare("dijit.TooltipDialog",
		[ContentPane, _TemplatedMixin, _FormMixin, _DialogMixin], {
		// summary:
		//		Pops up a dialog that appears like a Tooltip

		// title: String
		//		Description of tooltip dialog (required for a11y)
		title: "",

		// doLayout: [protected] Boolean
		//		Don't change this parameter from the default value.
		//		This ContentPane parameter doesn't make sense for TooltipDialog, since TooltipDialog
		//		is never a child of a layout container, nor can you specify the size of
		//		TooltipDialog in order to control the size of an inner widget.
		doLayout: false,

		// autofocus: Boolean
		//		A Toggle to modify the default focus behavior of a Dialog, which
		//		is to focus on the first dialog element after opening the dialog.
		//		False will disable autofocusing.  Default: true.
		autofocus: true,

		// baseClass: [protected] String
		//		The root className to use for the various states of this widget
		baseClass: "dijitTooltipDialog",

		// _firstFocusItem: [private readonly] DomNode
		//		The pointer to the first focusable node in the dialog.
		//		Set by `dijit/_DialogMixin._getFocusItems()`.
		_firstFocusItem: null,

		// _lastFocusItem: [private readonly] DomNode
		//		The pointer to which node has focus prior to our dialog.
		//		Set by `dijit/_DialogMixin._getFocusItems()`.
		_lastFocusItem: null,

		templateString: template,

		_setTitleAttr: function(/*String*/ title){
			this.containerNode.title = title;
			this._set("title", title);
		},

		postCreate: function(){
			this.inherited(arguments);
			this.connect(this.containerNode, "onkeypress", "_onKey");
		},

		orient: function(/*DomNode*/ node, /*String*/ aroundCorner, /*String*/ tooltipCorner){
			// summary:
			//		Configure widget to be displayed in given position relative to the button.
			//		This is called from the dijit.popup code, and should not be called
			//		directly.
			// tags:
			//		protected

			// Note: intentionally not using dijitTooltip class since that sets position:absolute, which
			// confuses dijit/popup trying to get the size of the tooltip.
			var newC = {
				"MR-ML": "dijitTooltipRight",
				"ML-MR": "dijitTooltipLeft",
				"TM-BM": "dijitTooltipAbove",
				"BM-TM": "dijitTooltipBelow",
				"BL-TL": "dijitTooltipBelow dijitTooltipABLeft",
				"TL-BL": "dijitTooltipAbove dijitTooltipABLeft",
				"BR-TR": "dijitTooltipBelow dijitTooltipABRight",
				"TR-BR": "dijitTooltipAbove dijitTooltipABRight",
				"BR-BL": "dijitTooltipRight",
				"BL-BR": "dijitTooltipLeft"
			}[aroundCorner + "-" + tooltipCorner];

			domClass.replace(this.domNode, newC, this._currentOrientClass || "");
			this._currentOrientClass = newC;

			// Tooltip.orient() has code to reposition connector for when Tooltip is before/after anchor.
			// Not putting here to avoid code bloat, and since TooltipDialogs are generally above/below.
			// Should combine code from Tooltip and TooltipDialog.
		},

		focus: function(){
			// summary:
			//		Focus on first field
			this._getFocusItems(this.containerNode);
			focus.focus(this._firstFocusItem);
		},

		onOpen: function(/*Object*/ pos){
			// summary:
			//		Called when dialog is displayed.
			//		This is called from the dijit.popup code, and should not be called directly.
			// tags:
			//		protected

			this.orient(this.domNode,pos.aroundCorner, pos.corner);

			// Position the tooltip connector for middle alignment.
			// This could not have been done in orient() since the tooltip wasn't positioned at that time.
			var aroundNodeCoords = pos.aroundNodePos;
			if(pos.corner.charAt(0) == 'M' && pos.aroundCorner.charAt(0) == 'M'){
				this.connectorNode.style.top = aroundNodeCoords.y + ((aroundNodeCoords.h - this.connectorNode.offsetHeight) >> 1) - pos.y + "px";
				this.connectorNode.style.left = "";
			}else if(pos.corner.charAt(1) == 'M' && pos.aroundCorner.charAt(1) == 'M'){
				this.connectorNode.style.left = aroundNodeCoords.x + ((aroundNodeCoords.w - this.connectorNode.offsetWidth) >> 1) - pos.x + "px";
			}

			this._onShow(); // lazy load trigger  (TODO: shouldn't we load before positioning?)
		},

		onClose: function(){
			// summary:
			//		Called when dialog is hidden.
			//		This is called from the dijit.popup code, and should not be called directly.
			// tags:
			//		protected
			this.onHide();
		},

		_onKey: function(/*Event*/ evt){
			// summary:
			//		Handler for keyboard events
			// description:
			//		Keep keyboard focus in dialog; close dialog on escape key
			// tags:
			//		private

			var node = evt.target;
			if(evt.charOrCode === keys.TAB){
				this._getFocusItems(this.containerNode);
			}
			var singleFocusItem = (this._firstFocusItem == this._lastFocusItem);
			if(evt.charOrCode == keys.ESCAPE){
				// Use defer to avoid crash on IE, see #10396.
				this.defer("onCancel");
				event.stop(evt);
			}else if(node == this._firstFocusItem && evt.shiftKey && evt.charOrCode === keys.TAB){
				if(!singleFocusItem){
					focus.focus(this._lastFocusItem); // send focus to last item in dialog
				}
				event.stop(evt);
			}else if(node == this._lastFocusItem && evt.charOrCode === keys.TAB && !evt.shiftKey){
				if(!singleFocusItem){
					focus.focus(this._firstFocusItem); // send focus to first item in dialog
				}
				event.stop(evt);
			}else if(evt.charOrCode === keys.TAB){
				// we want the browser's default tab handling to move focus
				// but we don't want the tab to propagate upwards
				evt.stopPropagation();
			}
		}
	});
});

},
'url:dijit/templates/TooltipDialog.html':"<div role=\"alertdialog\" tabIndex=\"-1\">\n\t<div class=\"dijitTooltipContainer\" role=\"presentation\">\n\t\t<div class=\"dijitTooltipContents dijitTooltipFocusNode\" data-dojo-attach-point=\"containerNode\"></div>\n\t</div>\n\t<div class=\"dijitTooltipConnector\" role=\"presentation\" data-dojo-attach-point=\"connectorNode\"></div>\n</div>\n",
'dojo/fx/Toggler':function(){
define("dojo/fx/Toggler", ["../_base/lang","../_base/declare","../_base/fx", "../_base/connect"], 
  function(lang, declare, baseFx, connectUtil){
	// module:
	//		dojo/fx/Toggler

return declare("dojo.fx.Toggler", null, {
	// summary:
	//		A simple `dojo.Animation` toggler API.
	// description:
	//		class constructor for an animation toggler. It accepts a packed
	//		set of arguments about what type of animation to use in each
	//		direction, duration, etc. All available members are mixed into
	//		these animations from the constructor (for example, `node`,
	//		`showDuration`, `hideDuration`).
	// example:
	//	|	var t = new dojo/fx/Toggler({
	//	|		node: "nodeId",
	//	|		showDuration: 500,
	//	|		// hideDuration will default to "200"
	//	|		showFunc: dojo/fx/wipeIn,
	//	|		// hideFunc will default to "fadeOut"
	//	|	});
	//	|	t.show(100); // delay showing for 100ms
	//	|	// ...time passes...
	//	|	t.hide();

	// node: DomNode
	//		the node to target for the showing and hiding animations
	node: null,

	// showFunc: Function
	//		The function that returns the `dojo.Animation` to show the node
	showFunc: baseFx.fadeIn,

	// hideFunc: Function
	//		The function that returns the `dojo.Animation` to hide the node
	hideFunc: baseFx.fadeOut,

	// showDuration:
	//		Time in milliseconds to run the show Animation
	showDuration: 200,

	// hideDuration:
	//		Time in milliseconds to run the hide Animation
	hideDuration: 200,

	// FIXME: need a policy for where the toggler should "be" the next
	// time show/hide are called if we're stopped somewhere in the
	// middle.
	// FIXME: also would be nice to specify individual showArgs/hideArgs mixed into
	// each animation individually.
	// FIXME: also would be nice to have events from the animations exposed/bridged

	/*=====
	_showArgs: null,
	_showAnim: null,

	_hideArgs: null,
	_hideAnim: null,

	_isShowing: false,
	_isHiding: false,
	=====*/

	constructor: function(args){
		var _t = this;

		lang.mixin(_t, args);
		_t.node = args.node;
		_t._showArgs = lang.mixin({}, args);
		_t._showArgs.node = _t.node;
		_t._showArgs.duration = _t.showDuration;
		_t.showAnim = _t.showFunc(_t._showArgs);

		_t._hideArgs = lang.mixin({}, args);
		_t._hideArgs.node = _t.node;
		_t._hideArgs.duration = _t.hideDuration;
		_t.hideAnim = _t.hideFunc(_t._hideArgs);

		connectUtil.connect(_t.showAnim, "beforeBegin", lang.hitch(_t.hideAnim, "stop", true));
		connectUtil.connect(_t.hideAnim, "beforeBegin", lang.hitch(_t.showAnim, "stop", true));
	},

	show: function(delay){
		// summary:
		//		Toggle the node to showing
		// delay: Integer?
		//		Amount of time to stall playing the show animation
		return this.showAnim.play(delay || 0);
	},

	hide: function(delay){
		// summary:
		//		Toggle the node to hidden
		// delay: Integer?
		//		Amount of time to stall playing the hide animation
		return this.hideAnim.play(delay || 0);
	}
});

});

},
'dojo/dnd/AutoSource':function(){
define("dojo/dnd/AutoSource", ["../_base/declare", "./Source"], function(declare, Source){
	return declare("dojo.dnd.AutoSource", Source, {
		// summary:
		//		a source that syncs its DnD nodes by default

		constructor: function(/*===== node, params =====*/){
			// summary:
			//		constructor of the AutoSource --- see the Source constructor for details
			this.autoSync = true;
		}
	});
});

},
'dojo/dnd/Target':function(){
define("dojo/dnd/Target", [ "../_base/declare", "../dom-class", "./Source" ], function(declare, domClass, Source){
	return declare("dojo.dnd.Target", Source, {
		// summary:
		//		a Target object, which can be used as a DnD target

		constructor: function(/*===== node, params =====*/){
			// summary:
			//		a constructor of the Target --- see the `dojo/dnd/Source` constructor for details
			this.isSource = false;
			domClass.remove(this.node, "dojoDndSource");
		}
	});
});

},
'dijit/layout/StackController':function(){
define("dijit/layout/StackController", [
	"dojo/_base/array", // array.forEach array.indexOf array.map
	"dojo/_base/declare", // declare
	"dojo/dom-class",
	"dojo/_base/event", // event.stop
	"dojo/keys", // keys
	"dojo/_base/lang", // lang.getObject
	"dojo/on",
	"../focus",		// focus.focus()
	"../registry",	// registry.byId
	"../_Widget",
	"../_TemplatedMixin",
	"../_Container",
	"../form/ToggleButton",
	"dojo/i18n!../nls/common"
], function(array, declare, domClass, event, keys, lang, on,
			focus, registry, _Widget, _TemplatedMixin, _Container, ToggleButton){

	// module:
	//		dijit/layout/StackController

	var StackButton = declare("dijit.layout._StackButton", ToggleButton, {
		// summary:
		//		Internal widget used by StackContainer.
		// description:
		//		The button-like or tab-like object you click to select or delete a page
		// tags:
		//		private

		// Override _FormWidget.tabIndex.
		// StackContainer buttons are not in the tab order by default.
		// Probably we should be calling this.startupKeyNavChildren() instead.
		tabIndex: "-1",

		// closeButton: Boolean
		//		When true, display close button for this tab
		closeButton: false,
		
		_aria_attr: "aria-selected",

		buildRendering: function(/*Event*/ evt){
			this.inherited(arguments);
			(this.focusNode || this.domNode).setAttribute("role", "tab");
		}
	});


	var StackController = declare("dijit.layout.StackController", [_Widget, _TemplatedMixin, _Container], {
		// summary:
		//		Set of buttons to select a page in a `dijit/layout/StackContainer`
		// description:
		//		Monitors the specified StackContainer, and whenever a page is
		//		added, deleted, or selected, updates itself accordingly.

		baseClass: "dijitStackController",

		templateString: "<span role='tablist' data-dojo-attach-event='onkeypress'></span>",

		// containerId: [const] String
		//		The id of the page container that I point to
		containerId: "",

		// buttonWidget: [const] Constructor
		//		The button widget to create to correspond to each page
		buttonWidget: StackButton,

		// buttonWidgetCloseClass: String
		//		CSS class of [x] close icon, used by event delegation code to tell when close button was clicked
		buttonWidgetCloseClass: "dijitStackCloseButton",

		constructor: function(params /*===== , srcNodeRef =====*/){
			// summary:
			//		Create the widget.
			// params: Object|null
			//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
			//		and functions, typically callbacks like onClick.
			//		The hash can contain any of the widget's properties, excluding read-only properties.
			// srcNodeRef: DOMNode|String?
			//		If a srcNodeRef (DOM node) is specified, replace srcNodeRef with my generated DOM tree

			this.pane2button = {};		// mapping from pane id to buttons
		},

		postCreate: function(){
			this.inherited(arguments);

			// Listen to notifications from StackContainer.
			// TODO: do this through bubbled events instead of topics
			this.subscribe(this.containerId+"-startup", "onStartup");
			this.subscribe(this.containerId+"-addChild", "onAddChild");
			this.subscribe(this.containerId+"-removeChild", "onRemoveChild");
			this.subscribe(this.containerId+"-selectChild", "onSelectChild");
			this.subscribe(this.containerId+"-containerKeyPress", "onContainerKeyPress");

			// Listen for click events to select or close tabs.
			// No need to worry about ENTER/SPACE key handling: tabs are selected via left/right arrow keys,
			// and closed via shift-F10 (to show the close menu).
			this.connect(this.containerNode, 'click', function(evt){
				var button = registry.getEnclosingWidget(evt.target);
				if(button != this.containerNode && !button.disabled && button.page){
					for(var target = evt.target; target !== this.containerNode; target = target.parentNode){
						if(domClass.contains(target, this.buttonWidgetCloseClass)){
							this.onCloseButtonClick(button.page);
							break;
						}else if(target == button.domNode){
							this.onButtonClick(button.page);
							break;
						}
					}
				}
			});
		},

		onStartup: function(/*Object*/ info){
			// summary:
			//		Called after StackContainer has finished initializing
			// tags:
			//		private
			array.forEach(info.children, this.onAddChild, this);
			if(info.selected){
				// Show button corresponding to selected pane (unless selected
				// is null because there are no panes)
				this.onSelectChild(info.selected);
			}

			// Reflect events like page title changes to tab buttons
			var containerNode = registry.byId(this.containerId).containerNode,
				pane2button = this.pane2button,
				paneToButtonAttr = {
					"title": "label",
					"showtitle": "showLabel",
					"iconclass": "iconClass",
					"closable": "closeButton",
					"tooltip": "title",
					"disabled": "disabled"
				},
				connectFunc = function(attr, buttonAttr){
					return on(containerNode, "attrmodified-" + attr, function(evt){
						var button = pane2button[evt.detail && evt.detail.widget && evt.detail.widget.id];
						if(button){
							button.set(buttonAttr, evt.detail.newValue);
						}
					});
				};
			for(var attr in paneToButtonAttr){
				this.own(connectFunc(attr, paneToButtonAttr[attr]));
			}
		},

		destroy: function(){
			// Since the buttons are internal to the StackController widget, destroy() should remove them, which is
			// done by calling onRemoveChild().
			for(var pane in this.pane2button){
				this.onRemoveChild(registry.byId(pane));
			}

			// TODO: destroyRecursive() will call destroy() on each child button twice.   Once from the above code,
			// and once because _WidgetBase.destroyDescendants() deletes anything inside of this.containerNode.
			// Probably shouldn't attach that DOMNode as this.containerNode.

			this.inherited(arguments);
		},

		onAddChild: function(/*dijit/_WidgetBase*/ page, /*Integer?*/ insertIndex){
			// summary:
			//		Called whenever a page is added to the container.
			//		Create button corresponding to the page.
			// tags:
			//		private

			// create an instance of the button widget
			// (remove typeof buttonWidget == string support in 2.0)
			var Cls = lang.isString(this.buttonWidget) ? lang.getObject(this.buttonWidget) : this.buttonWidget;
			var button = new Cls({
				id: this.id + "_" + page.id,
				name: this.id + "_" + page.id,
				label: page.title,
				disabled: page.disabled,
				ownerDocument: this.ownerDocument,
				dir: page.dir,
				lang: page.lang,
				textDir: page.textDir,
				showLabel: page.showTitle,
				iconClass: page.iconClass,
				closeButton: page.closable,
				title: page.tooltip,
				page: page
			});

			this.addChild(button, insertIndex);
			this.pane2button[page.id] = button;
			page.controlButton = button;	// this value might be overwritten if two tabs point to same container
			if(!this._currentChild){
				// If this is the first child then StackContainer will soon publish that it's selected,
				// but before that StackContainer calls layout(), and before layout() is called the
				// StackController needs to have the proper height... which means that the button needs
				// to be marked as selected now.   See test_TabContainer_CSS.html for test.
				this.onSelectChild(page);
			}
		},

		onRemoveChild: function(/*dijit/_WidgetBase*/ page){
			// summary:
			//		Called whenever a page is removed from the container.
			//		Remove the button corresponding to the page.
			// tags:
			//		private

			if(this._currentChild === page){ this._currentChild = null; }

			var button = this.pane2button[page.id];
			if(button){
				this.removeChild(button);
				delete this.pane2button[page.id];
				button.destroy();
			}
			delete page.controlButton;
		},

		onSelectChild: function(/*dijit/_WidgetBase*/ page){
			// summary:
			//		Called when a page has been selected in the StackContainer, either by me or by another StackController
			// tags:
			//		private

			if(!page){ return; }

			if(this._currentChild){
				var oldButton=this.pane2button[this._currentChild.id];
				oldButton.set('checked', false);
				oldButton.focusNode.setAttribute("tabIndex", "-1");
			}

			var newButton=this.pane2button[page.id];
			newButton.set('checked', true);
			this._currentChild = page;
			newButton.focusNode.setAttribute("tabIndex", "0");
			var container = registry.byId(this.containerId);
		},

		onButtonClick: function(/*dijit/_WidgetBase*/ page){
			// summary:
			//		Called whenever one of my child buttons is pressed in an attempt to select a page
			// tags:
			//		private

			var button = this.pane2button[page.id];

			// For TabContainer where the tabs are <span>, need to set focus explicitly when left/right arrow
			focus.focus(button.focusNode);

			if(this._currentChild && this._currentChild.id === page.id) {
				//In case the user clicked the checked button, keep it in the checked state because it remains to be the selected stack page.
				button.set('checked', true);
			}
			var container = registry.byId(this.containerId);
			container.selectChild(page);
		},

		onCloseButtonClick: function(/*dijit/_WidgetBase*/ page){
			// summary:
			//		Called whenever one of my child buttons [X] is pressed in an attempt to close a page
			// tags:
			//		private

			var container = registry.byId(this.containerId);
			container.closeChild(page);
			if(this._currentChild){
				var b = this.pane2button[this._currentChild.id];
				if(b){
					focus.focus(b.focusNode || b.domNode);
				}
			}
		},

		// TODO: this is a bit redundant with forward, back api in StackContainer
		adjacent: function(/*Boolean*/ forward){
			// summary:
			//		Helper for onkeypress to find next/previous button
			// tags:
			//		private

			if(!this.isLeftToRight() && (!this.tabPosition || /top|bottom/.test(this.tabPosition))){ forward = !forward; }
			// find currently focused button in children array
			var children = this.getChildren();
			var idx = array.indexOf(children, this.pane2button[this._currentChild.id]),
				current = children[idx];

			// Pick next/previous non-disabled button to focus on.   If we get back to the original button it means
			// that all buttons must be disabled, so return current child to avoid an infinite loop.
			var child;
			do{
				idx = (idx + (forward ? 1 : children.length - 1)) % children.length;
				child = children[idx];
			}while(child.disabled && child != current);

			return child; // dijit/_WidgetBase
		},

		onkeypress: function(/*Event*/ e){
			// summary:
			//		Handle keystrokes on the page list, for advancing to next/previous button
			//		and closing the current page if the page is closable.
			// tags:
			//		private

			if(this.disabled || e.altKey ){ return; }
			var forward = null;
			if(e.ctrlKey || !e._djpage){
				switch(e.charOrCode){
					case keys.LEFT_ARROW:
					case keys.UP_ARROW:
						if(!e._djpage){ forward = false; }
						break;
					case keys.PAGE_UP:
						if(e.ctrlKey){ forward = false; }
						break;
					case keys.RIGHT_ARROW:
					case keys.DOWN_ARROW:
						if(!e._djpage){ forward = true; }
						break;
					case keys.PAGE_DOWN:
						if(e.ctrlKey){ forward = true; }
						break;
					case keys.HOME:
						// Navigate to first non-disabled child
						var children = this.getChildren();
						for(var idx = 0; idx < children.length; idx++){
							var child = children[idx];
							if(!child.disabled){
								this.onButtonClick(child.page);
								break;
							}
						}
						event.stop(e);
						break;
					case keys.END:
						// Navigate to last non-disabled child
						var children = this.getChildren();
						for(var idx = children.length-1; idx >= 0; idx--){
							var child = children[idx];
							if(!child.disabled){
								this.onButtonClick(child.page);
								break;
							}
						}
						event.stop(e);
						break;
					case keys.DELETE:
						if(this._currentChild.closable){
							this.onCloseButtonClick(this._currentChild);
						}
						event.stop(e);
						break;
					default:
						if(e.ctrlKey){
							if(e.charOrCode === keys.TAB){
								this.onButtonClick(this.adjacent(!e.shiftKey).page);
								event.stop(e);
							}else if(e.charOrCode == "w"){
								if(this._currentChild.closable){
									this.onCloseButtonClick(this._currentChild);
								}
								event.stop(e); // avoid browser tab closing.
							}
						}
				}
				// handle next/previous page navigation (left/right arrow, etc.)
				if(forward !== null){
					this.onButtonClick(this.adjacent(forward).page);
					event.stop(e);
				}
			}
		},

		onContainerKeyPress: function(/*Object*/ info){
			// summary:
			//		Called when there was a keypress on the container
			// tags:
			//		private
			info.e._djpage = info.page;
			this.onkeypress(info.e);
		}
	});

	StackController.StackButton = StackButton;	// for monkey patching

	return StackController;
});

},
'*now':function(r){r(['dojo/i18n!*preload*epi-find/nls/run*["ar","ca","cs","da","de","el","en-gb","en-us","en-ca","en-au","es-es","fi-fi","fr-fr","he-il","hu","it-it","ja-jp","ko-kr","nl-nl","nb","pl","pt-br","pt-pt","ru","sk","sl","sv","th","tr","zh-tw","zh-cn","ROOT"]']);}
}});
require({
    cache:{}, // consume the cached dojo layer
    packages: [
        "dojo",
        "dijit",
        "dojox",
        "dgrid",
        "put-selector",
        "xstyle",
        { name:"fastclick", main: "lib/fastclick" },
        "epi",
        "epi-cms",
        "epi-saas-base"
    ]
}, ["epi-find/Configuration!", "epi-find/Application"]);